Package: meta
Title: General Package for Meta-Analysis
Version: 8.5-0
Date: 2026-05-25
Depends: R (>= 4.0.0), metabook (>= 0.2-0)
Imports: metafor (>= 3.0-0), grid, ggplot2, lme4, CompQuadForm, xml2,
        methods, readr, dplyr, magrittr, purrr, stringr, rlang, tibble,
        scales, cli
Suggests: netmeta, BiasedUrn, pimeta, estmeansd, robvis, brglm2,
        writexl, rmarkdown, knitr, ggpubr, gridExtra, gemtc, ragg,
        metadat, svglite
Authors@R: person("Guido", "Schwarzer",
                  role = c("cre", "aut"),
                  email = "guido.schwarzer@uniklinik-freiburg.de",
                  comment = c(ORCID = "0000-0001-6214-9087"))
Author: Guido Schwarzer [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-6214-9087>)
Maintainer: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
URL: https://github.com/guido-s/meta/
        https://link.springer.com/book/10.1007/978-3-319-21416-0
Description: User-friendly general package providing standard methods for meta-analysis and supporting Schwarzer, Carpenter, and Rücker <DOI:10.1007/978-3-319-21416-0>, "Meta-Analysis with R" (2015):
 - common effect and random effects meta-analysis;
 - several plots (forest, funnel, Galbraith / radial, L'Abbe, Baujat, bubble);
 - three-level meta-analysis model;
 - generalised linear mixed model;
 - logistic regression with penalised likelihood for rare events;
 - Hartung-Knapp method for random effects model;
 - Kenward-Roger method for random effects model;
 - prediction interval and density of the prediction distribution;
 - expected proportion of comparable studies with clinically important benefit or harm;
 - statistical tests  for funnel plot asymmetry;
 - trim-and-fill method to evaluate bias in meta-analysis;
 - meta-regression;
 - cumulative meta-analysis and leave-one-out meta-analysis;
 - import data from 'RevMan 5';
 - produce forest plot summarising several (subgroup) meta-analyses.
License: GPL (>= 2)
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 22:18:27 UTC; sc
Repository: CRAN
Date/Publication: 2026-05-25 05:10:02 UTC
Built: R 4.6.0; ; 2026-05-25 08:32:34 UTC; unix

Package: meta
Title: General Package for Meta-Analysis
Version: 8.5-0
Date: 2026-05-25
Depends: R (>= 4.0.0), metabook (>= 0.2-0)
Imports: metafor (>= 3.0-0), grid, ggplot2, lme4, CompQuadForm, xml2,
        methods, readr, dplyr, magrittr, purrr, stringr, rlang, tibble,
        scales, cli
Suggests: netmeta, BiasedUrn, pimeta, estmeansd, robvis, brglm2,
        writexl, rmarkdown, knitr, ggpubr, gridExtra, gemtc, ragg,
        metadat, svglite
Authors@R: person("Guido", "Schwarzer",
                  role = c("cre", "aut"),
                  email = "guido.schwarzer@uniklinik-freiburg.de",
                  comment = c(ORCID = "0000-0001-6214-9087"))
Author: Guido Schwarzer [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-6214-9087>)
Maintainer: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
URL: https://github.com/guido-s/meta/
        https://link.springer.com/book/10.1007/978-3-319-21416-0
Description: User-friendly general package providing standard methods for meta-analysis and supporting Schwarzer, Carpenter, and Rücker <DOI:10.1007/978-3-319-21416-0>, "Meta-Analysis with R" (2015):
 - common effect and random effects meta-analysis;
 - several plots (forest, funnel, Galbraith / radial, L'Abbe, Baujat, bubble);
 - three-level meta-analysis model;
 - generalised linear mixed model;
 - logistic regression with penalised likelihood for rare events;
 - Hartung-Knapp method for random effects model;
 - Kenward-Roger method for random effects model;
 - prediction interval and density of the prediction distribution;
 - expected proportion of comparable studies with clinically important benefit or harm;
 - statistical tests  for funnel plot asymmetry;
 - trim-and-fill method to evaluate bias in meta-analysis;
 - meta-regression;
 - cumulative meta-analysis and leave-one-out meta-analysis;
 - import data from 'RevMan 5';
 - produce forest plot summarising several (subgroup) meta-analyses.
License: GPL (>= 2)
Encoding: UTF-8
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 22:18:27 UTC; sc
Repository: CRAN
Date/Publication: 2026-05-25 05:10:02 UTC
Built: R 4.6.0; ; 2026-05-25 07:34:34 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method("[",longarm)
S3method("[",pairwise)
S3method(as.data.frame,meta)
S3method(barplot,rob)
S3method(baujat,meta)
S3method(blup,meta)
S3method(bubble,metareg)
S3method(cidprop,meta)
S3method(estimates,blup.meta)
S3method(estimates,meta)
S3method(forest,meta)
S3method(forest,metabind)
S3method(forest,metacum)
S3method(forest,metainf)
S3method(funnel,meta)
S3method(labbe,default)
S3method(labbe,metabin)
S3method(labels,meta)
S3method(metabias,cdir)
S3method(metabias,default)
S3method(metabias,meta)
S3method(metabias,rm5)
S3method(metacum,default)
S3method(metacum,meta)
S3method(metainf,default)
S3method(metainf,meta)
S3method(metareg,default)
S3method(metareg,meta)
S3method(nnt,default)
S3method(nnt,meta)
S3method(plot,cidprop)
S3method(plot,meta)
S3method(print,blup.meta)
S3method(print,cdir)
S3method(print,cidprop)
S3method(print,estimates.blup.meta)
S3method(print,estimates.meta)
S3method(print,meta)
S3method(print,metabias)
S3method(print,metacum)
S3method(print,metainf)
S3method(print,nnt.default)
S3method(print,nnt.meta)
S3method(print,rd.default)
S3method(print,rd.meta)
S3method(print,rm5)
S3method(print,rob)
S3method(print,summary.cdir)
S3method(print,summary.meta)
S3method(print,summary.rm5)
S3method(print,trimfill.cdir)
S3method(print,trimfill.rm5)
S3method(radial,default)
S3method(radial,meta)
S3method(rd,default)
S3method(rd,meta)
S3method(subset,longarm)
S3method(subset,pairwise)
S3method(summary,cdir)
S3method(summary,meta)
S3method(summary,metacum)
S3method(summary,metainf)
S3method(summary,rm5)
S3method(trimfill,cdir)
S3method(trimfill,default)
S3method(trimfill,meta)
S3method(trimfill,rm5)
S3method(update,meta)
S3method(weights,meta)
export(.forestArgs)
export(JAMAlabels)
export(VE2logVR)
export(asin2ir)
export(asin2p)
export(backtransf)
export(baujat)
export(blup)
export(bubble)
export(ci)
export(cidprop)
export(cilayout)
export(cor2z)
export(drapery)
export(estimates)
export(forest)
export(forest_dims)
export(funnel)
export(gs)
export(inches2units)
export(labbe)
export(logVR2VE)
export(logit2p)
export(longarm)
export(metaadd)
export(metabias)
export(metabin)
export(metabind)
export(metacont)
export(metacor)
export(metacr)
export(metacum)
export(metagen)
export(metainc)
export(metainf)
export(metamean)
export(metamerge)
export(metaprop)
export(metarate)
export(metareg)
export(nnt)
export(or2smd)
export(p2asin)
export(p2logit)
export(pairwise)
export(radial)
export(rd)
export(read.cdir)
export(read.mtv)
export(read.rm5)
export(rob)
export(settings.meta)
export(setvals)
export(smd2or)
export(traffic_light)
export(transf)
export(trimfill)
export(units2inches)
export(z2cor)
import(metabook)
importFrom(CompQuadForm,farebrother)
importFrom(cli,cli_abort)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_warn)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,group_map)
importFrom(dplyr,if_else)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,rename_with)
importFrom(dplyr,select)
importFrom(dplyr,tibble)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(grDevices,bmp)
importFrom(grDevices,cairo_pdf)
importFrom(grDevices,cairo_ps)
importFrom(grDevices,dev.cur)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.set)
importFrom(grDevices,dev.size)
importFrom(grDevices,gray)
importFrom(grDevices,gray.colors)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,jpeg)
importFrom(grDevices,pdf)
importFrom(grDevices,pictex)
importFrom(grDevices,png)
importFrom(grDevices,postscript)
importFrom(grDevices,svg)
importFrom(grDevices,tiff)
importFrom(graphics,abline)
importFrom(graphics,axis)
importFrom(graphics,barplot)
importFrom(graphics,box)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(grid,arrow)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,convertX)
importFrom(grid,gpar)
importFrom(grid,grid.circle)
importFrom(grid,grid.draw)
importFrom(grid,grid.get)
importFrom(grid,grid.gget)
importFrom(grid,grid.grabExpr)
importFrom(grid,grid.layout)
importFrom(grid,grid.lines)
importFrom(grid,grid.newpage)
importFrom(grid,grid.polygon)
importFrom(grid,grid.rect)
importFrom(grid,grid.text)
importFrom(grid,grid.xaxis)
importFrom(grid,grobTree)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,rectGrob)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(grid,unit.c)
importFrom(grid,viewport)
importFrom(lme4,glmer)
importFrom(magrittr,"%<>%")
importFrom(metafor,baujat)
importFrom(metafor,blup)
importFrom(metafor,confint.rma.mv)
importFrom(metafor,confint.rma.uni)
importFrom(metafor,escalc)
importFrom(metafor,forest)
importFrom(metafor,funnel)
importFrom(metafor,funnel.default)
importFrom(metafor,labbe)
importFrom(metafor,predict.rma)
importFrom(metafor,radial)
importFrom(metafor,regtest)
importFrom(metafor,rma.glmm)
importFrom(metafor,rma.mv)
importFrom(metafor,rma.uni)
importFrom(metafor,to.long)
importFrom(metafor,trimfill)
importFrom(metafor,vcalc)
importFrom(methods,formalArgs)
importFrom(purrr,compact)
importFrom(readr,cols)
importFrom(readr,read_csv)
importFrom(rlang,":=")
importFrom(rlang,enexprs)
importFrom(rlang,expr)
importFrom(rlang,is_bare_list)
importFrom(scales,number_format)
importFrom(stats,as.formula)
importFrom(stats,binom.test)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,dt)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,poisson.test)
importFrom(stats,pt)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,residuals)
importFrom(stats,runif)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,weighted.mean)
importFrom(stats,weights)
importFrom(stringr,str_pad)
importFrom(tibble,column_to_rownames)
importFrom(tools,file_ext)
importFrom(utils,assignInNamespace)
importFrom(utils,combn)
importFrom(utils,count.fields)
importFrom(utils,find)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(utils,unzip)
importFrom(xml2,as_xml_document)
importFrom(xml2,xml_attr)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_text)

NAME

    meta - meta-programming API

SYNOPSIS

       use v5.14;
       use meta;
    
       my $metapkg = meta::get_package( "MyApp::Some::Package" );
    
       $metapkg->add_symbol(
          '&a_function' => sub { say "New function was created" }
       );
    
       MyApp::Some::Package::a_function();

DESCRIPTION

    This package provides an API for metaprogramming; that is, allowing
    code to inspect or manipulate parts of its own program structure. Parts
    of the perl interpreter itself can be accessed by means of
    "meta"-objects provided by this package. Methods on these objects allow
    inspection of details, as well as creating new items or removing
    existing ones.

    The intention of this API is to provide a nicer replacement for
    existing tricks such as no strict 'refs' and using globrefs, and also
    to be a more consistent place to add new abilities, such as more APIs
    for inspection and alteration of internal structures, metaprogramming
    around the new 'class' feature, and other such uses.

    This module should be considered experimental; no API stability
    guarantees are made at this time. Behaviour may be added, altered, or
    removed in later versions. Once a workable API shape has been found, it
    is hoped that this module will eventually become dual-life and shipped
    as part of Perl core, as the implementation for PPC 0022. See the link
    in the "SEE ALSO" section.

    This module attempts to find a balance between accurately representing
    low-level concepts within the current implementation of the Perl
    interpreter, while also providing higher-level abstractions that
    provide useful behaviour for code that uses it. One place this can be
    seen is the lower-level "list_globs" method, which directly maps to the
    way that GVs are stored in symbol table stashes but requires the user
    to be aware of the GV-less optimisised storage of CVs, as compared to
    the higher-level "list_symbols" method which provides an abstraction
    over this complication and presents the more useful but less accurate
    impression of separately named symbols that neatly map to their values.

    Since version 0.003_002 all the entry-point functions and constructors
    in this module will provoke warnings in the meta::experimental
    category. They can be silenced by

       use meta;
       no warnings 'meta::experimental';

    Since version 0.005 the various can_...-prefixed variant accessor
    methods print deprecation warnings. They are likely to be removed soon.

FUNCTIONS

 get_package

       $metapkg = meta::get_package( $pkgname );

    Returns a metapackage reference representing the given package name,
    creating it if it did not previously exist.

    An alternative to meta::package->get in a plain function style.

 get_this_package

       $metapkg = meta::get_this_package;

    Since version 0.002.

    Returns a metapackage reference representing the package of the code
    that called the function.

    Useful for performing meta-programming on the contents of a module
    during its BEGIN or loading time. Equivalent to but more efficient than
    the following:

       meta::get_package(__PACKAGE__)

 for_reference

       $metasym = meta::for_reference( $ref );

    Since version 0.007.

    Returns a metasymbol reference representing the glob, variable or
    subroutine that is pointed to by the given reference.

    Note that passing in a reference to a symbol table hash ("stash") does
    not result in a metapackage. For that you will have to call
    "get_package" or similar.

METHODS ON meta::package

 get

       $metapkg = meta::package->get( $pkgname );

    Since version 0.003_001.

    Returns a metapackage reference representing the given package name,
    creating it if it did not previously exist.

    An alternative to meta::get_package in an object constructor style.

 is_class

       $bool = $metapkg->is_class;

    Since version 0.009.

    Returns true if on a version of Perl that supports use feature 'class',
    and the package being represented is a real class created by that
    feature. False for regular packages, and always false on earlier
    versions of Perl before that feature was introduced.

 name

       $name = $metapkg->name;

    Returns the name of the package being represented.

 get_glob

       $metaglob = $metapkg->get_glob( $name );

    Returns a metaglob reference representing the given symbol name within
    the package, if it exists. Throws an exception if not.

 try_get_glob, can_glob

       $metaglob = $metapkg->try_get_glob( $name );
       $metaglob = $metapkg->can_glob( $name );

    Similar to "get_glob" but returns undef if the glob does not exist.

 list_globs

       @metaglobs = $metapkg->list_globs;

    Since version 0.005.

    Returns a list of all the globs in the package that may refer to
    symbols (i.e. not subpackages). They are returned in no particular
    order.

    For a more convenient return value form, see also "list_symbols".

 list_subpackage_globs

 list_all_globs

       @metaglobs = $metapkg->list_subpackage_globs;
       @metaglobs = $metapkg->list_all_globs;

    Since version 0.005.

    Returns a list of all the globs in the package that refer to
    subpackages, or all globs, including subpackages. They are returned in
    no particular order.

    For a more convenient return value form, see also "list_subpackages".

 get_symbol

       $metasym = $metapkg->get_symbol( $name );

    Returns a metasymbol reference representing the given symbol name
    within the package. The symbol name should include the leading sigil;
    one of the characters *, $, @, % or &. Throws an exception if the
    symbol does not exist.

 try_get_symbol, can_symbol

       $metasym = $metapkg->try_get_symbol( $name );
       $metasym = $metapkg->can_symbol( $name );

    Similar to "get_symbol" but returns undef if the symbol does not exist.

 add_symbol

       $metasym = $metapkg->add_symbol( $name, $valueref );

    Creates a new symbol of the given name in the given package. The new
    symbol will refer to the item given by reference, whose type must match
    the sigil of the symbol name. Returns a metasymbol reference as per
    "get_symbol". If a symbol already existed of the given name then an
    exception is thrown.

    Note that this does not create a copy of a variable, but stores an
    alias to the referred item itself within the symbol table.

       $metapkg->add_symbol( '@things', \my @array );
    
       push @array, "more", "values";
       # these values are now visible in the @things array

    If adding a scalar, array or hash variable, the $valueref argument is
    optional. If not provided then a new, blank variable of the correct
    type will be created.

 get_or_add_symbol

       $metasym = $metapkg->get_or_add_symbol( $name, $valueref );

    Since version 0.003_003.

    Similar to "get_symbol" but creates a new symbol if it didn't already
    exist as per "add_symbol".

    Note that if the symbol did already exist it is returned and $valueref
    will be ignored. The symbol will not be modified in that case to point
    to the value referred to instead.

 remove_symbol

       $metapkg->remove_symbol( $name );

    Removes a symbol of the given name from the given package. If the
    symbol was the last item in the glob then the glob too is removed from
    the package. If the named symbol did not previously exist then an
    exception is thrown.

    To only conditionally remove a symbol if it already exists, test for it
    first by using "try_get_symbol":

       $metapkg->try_get_symbol( '$variable' ) and
          $metapkg->remove_symbol( '$variable' );

 list_symbols

       %sub_metasyms = $metapkg->list_symbols;
       %sub_metasyms = $metapkg->list_symbols( sigils => $filter );

    Since version 0.006.

    Returns an even-sized key/value list containing the symbols within the
    given package instance. Each symbol is returned as a pair, with its
    sigil-prefixed basename first, followed by a metasymbol instance
    representing it. Since the sigil-prefixed names must be unique, it is
    convenient to assign this list into a hash. The symbols are returned in
    no particular order.

    If the optional sigils named parameter is given, it should be a string
    of possible symbol sigils (the characters $, @, % or &). In this case,
    only symbols whose sigil is present in this string will be returned.

 list_subpackages

       %sub_metapkgs = $metapkg->list_subpackages;

    Since version 0.006.

    Returns an even-sized key/value list containing the immediate
    sub-packages of the given package instance. Each sub-package is
    returned as a pair, with its basename first (minus the "::" suffix),
    followed by a metapackage instance representing it. Since the names of
    each sub-package must be unique, it is convenient to assign this list
    into a hash. The sub-packages are returned in no particular order.

 add_named_sub

       $metasub = $metapkg->add_named_sub( $name, $code );

    Since version 0.008.

    A convenient shortcut for adding a subroutine symbol and setting the
    subname of the newly-added sub. Equivalent to calling "add_symbol" and
    then "set_subname" on its result, but more efficient as it does not
    have to create a separate fake GV to store the subname in.

    Note that $name should be given as a barename, without the leading &
    sigil.

METHODS ON METASYMBOLS

 is_glob, is_scalar, ...

       $bool = $metasym->is_glob;
       $bool = $metasym->is_scalar;
       $bool = $metasym->is_array;
       $bool = $metasym->is_hash;
       $bool = $metasym->is_subroutine;

    Returns true if the symbol being referred to is of the given type, or
    false if not.

 reference

       $ref = $metasym->reference;

    Returns a regular Perl reference to the symbol being represented.

METHODS ON meta::glob

 get

       $metaglob = meta::glob->get( $globname );

    Since version 0.003_001.

    Returns a metaglob reference representing the given symbol from the
    symbol table from a fully-qualified name, if it exists. Throws an
    exception if not.

 try_get

       $metaglob = meta::glob->try_get( $globname );

    Since version 0.003_003.

    Similar to "get" but returns undef if the given symbol does not exist.

 get_or_add

       $metaglob = meta::glob->get_or_add( $globname );

    Since version 0.003_003.

    Similar to "get" but creates the symbol if it didn't already exist.

 basename

       $name = $metaglob->basename;

    Returns the name of the glob within its package.

 get_scalar, get_array, ...

       $metasym = $metaglob->get_scalar;
       $metasym = $metaglob->get_array;
       $metasym = $metaglob->get_hash;
       $metasym = $metaglob->get_code;

    Returns a metasymbol reference representing the symbol in the given
    slot of the glob, if it exists. Throws an exception if not.

 try_get_scalar, try_get_array, ...

    Similar to "get_scalar", "get_array", etc... but returns undef if the
    given slot does not exist.

METHODS ON METAVARIABLES

 value

       $scalar = $metavar->value;
       @array  = $metavar->value;
       %hash   = $metavar->value;
    
       $count = scalar $metavar->value;

    Returns the current value of the variable, as if it appeared in regular
    Perl code.

METHODS ON METASUBROUTINES

 is_method

       $bool = $metasub->is_method;

    Since version 0.009.

    Returns true if on a version of Perl that supports use feature 'class',
    and the subroutine being represented is a real method created by that
    feature. False for regular sub-based subroutines, and always false on
    earlier versions of Perl before that feature was introduced.

 subname

       $name = $metasub->subname;

    Returns the (fully-qualified) name of the subroutine.

 set_subname

       $metasub = $metasub->set_subname( $name );

    Since version 0.007.

    Sets a new name for the subroutine.

    If $name is not fully-qualified (i.e. does not contain a :: sequence),
    then the package name of the caller is used to create the
    fully-qualified name to be stored.

 prototype

       $proto = $metasub->prototype;

    Returns the prototype of the subroutine.

 set_prototype

       $metasub = $metasub->set_prototype( $proto );

    Since version 0.007.

    Sets a new prototype for the subroutine.

    Returns the $metasub instance itself to allow for easy chaining.

 signature

       $metasig = $metasub->signature;

    Since version 0.010.

    If on Perl version 5.26 or above and the subroutine has a signature,
    returns an object reference representing details about the signature.
    This can be queried using the methods below. If the subroutine does not
    use a signature (or on Perl versions before 5.26) returns undef.

METHODS ON SUBROUTINE METASIGNATURES

 mandatory_params

       $n = $metasig->mandatory_params;

    Returns the number of parameters that are mandatory (i.e. do not have a
    defaulting expression). This is the minimum number of argument values
    that must be passed to any call of this function and does not count a
    final slurpy parameter.

    Note that the implicit $self parameter to a method subroutine is
    included in this count. This count will always be at least 1 on such a
    method.

 optional_params

       $n = $metasig->optional_params;

    Returns the number of parameters that are optional (i.e. have a
    defaulting expression).

 slurpy

       $slurpy = $metasig->slurpy;

    Returns the sigil character associated with the final slurpy parameter
    if it exists (i.e. % or @), or undef if no slurpy parameter is defined.

 min_args

 max_args

       $n = $metasig->min_args;
    
       $n = $metasig->max_args;

    Returns the minimum or maximum number of argument values that can be
    passed to a call to this function. min_args is the same as
    mandatory_params but is offered as an alias in case the data model ever
    changes. max_args will be undef if the function uses a slurpy final
    parameter.

TODO

      * Access to the new parts of API introduced by Perl 5.38 to deal with
      classes, methods, fields.

SEE ALSO

    PPC 0022 "metaprogramming"
    <https://github.com/Perl/PPCs/blob/main/ppcs/ppc0022-metaprogramming.md>

AUTHOR

    Paul Evans <leonerd@leonerd.org.uk>

# |OpenViBE meta repository| |README|

[![Website](https://img.shields.io/badge/Web-Website-informational)](http://openvibe.inria.fr/)
[![Doxygen Documentation](https://img.shields.io/badge/Doc-Doxygen%20Documentation-informational)](http://openvibe.inria.fr/documentation/latest/)
[![License: AGPL v3](https://img.shields.io/badge/License-AGPL%20v3-blue.svg)](https://www.gnu.org/licenses/agpl-3.0)

OpenViBE project is now divided into 3 parts :

- SDK, that contains the certifiable core and plugins of OpenViBE
- Designer, the graphical interface for OpenViBE
- Extras, for community plugins and contributions

The current repository, OpenViBE-meta, exists to bring the three repositories together and build the project.

# User Install

Here are the installation steps For **Using Openvibe** . If you want to develop in Openvibe, please refer to the section *Developer Install*: 

1. install miniconda: https://docs.conda.io/projects/miniconda/en/latest/
2. create an empty conda environment and activate it: `conda create -n openvibe` and `conda activate openvibe`
3. install openvibe package: `conda install -c openvibe -c conda-forge openvibe`
4. Launch: `openvibe-designer`


# Developer Install
Compilers will come with the conda environment for **Linux** (*gcc*) and **Macos** (*clang*). For **Windows**, you should install [*Visual Studio 2019* ](https://my.visualstudio.com/Downloads?q=visual%20studio%202019&wt.mc_id=o~msft~vscom~older-downloads)

1. install miniconda: https://docs.conda.io/projects/miniconda/en/latest/
(for **Windows**, use a powershell for the following steps)
2. clone OpenVibe: `git clone --recurse-submodules git@gitlab.inria.fr:openvibe/meta.git`
3. install openvibe dependencies: `conda env update -f conda/env_{linux|osx|windows}.yaml`
4. activate the environment: `conda activate openvibe`
5. make build dir: `mkdir build ; cd build`
  - 5.1 **Windows Only** set up the shell:
```
$vsPath = &"${env:ProgramFiles(x86)}\Microsoft Visual Studio\Installer\vswhere.exe" -version '[16.0,17.0)' -products * -requires Microsoft.VisualStudio.Component.VC.Tools.x86.x64 -property installationpath
Import-Module (Get-ChildItem $vsPath -Recurse -File -Filter Microsoft.VisualStudio.DevShell.dll).FullName
Enter-VsDevShell -VsInstallPath $vsPath -SkipAutomaticLocation -DevCmdArguments '-arch=x64'
```
6. configure and build:
  - 6.1 **Linux and Mac**: `cmake .. && make -j4`
  - 6.2 **Windows**: `cmake .. -G Ninja ; ninja`

7. Launch:
  - 7.1 **Linux and Mac** : `./bin/openvibe-designer`
  - 7.2 **Windows** `.\bin\openvibe-designer`

8. **Optional**:  Launch the tests (from the build dir)
  - 8.1 Extras `cd extras; cTest -T Test --output-on-failure ; cd ..`
  - 8.2 unit-test: `cd sdk/unit-test ; cTest -T Test --output-on-failure ; cd ../..`
  - 8.2 validation-test: `cd sdk/validation-test ; cTest -T Test --output-on-failure ; cd ../..`


## Updating the repository

You can update the whole directory (including submodules) with :

```bash
git pull
git submodule sync --recursive
git submodule update --init --recursive
```

Aliases can be created to ease the global update process :
`git config --global alias.spull '!git pull && git submodule sync --recursive && git submodule update --init --recursive'`

# |OpenViBE conda repository| |README|

## HowTo

This conda package in made in order to be able to build and execute OpenVibes on the following plateforms
* Win64
* Mac OS
* Linux 64

It is compose of the corresponding setup files
In order to be able to compile and execute, you have first to get *conda* on your machine 

## Get miniconda

Please refer to the [![Website](https://docs.conda.io/en/latest/miniconda.html#installing)](miniconda web site)

## Setup

Create your environment from config file :
`$ conda env create -n openvibe --file ENV.yml` 

if it failed, try with 
`mamba env update --file ENV.yaml `

Activate it
`$ conda activate openvibe`

## Install

Create a build folder
`$ cd BUILD_FOLDER`

Configure CMake
`$ cmake OPENVIBE_META_FOLDER`

Make with all your processors
`$ make -j` 

Install OpenVibe
`$ make install`

## Tests

`$ cd sdk; ./ctest-launcher.sh `
`$ cd extra; ctest`

#' Cochrane review: summary of meta-analyses
#' 
#' @description
#' Calculate and print a summary of all meta-analyses in a
#' Cochrane review.
#' 
#' @param x An object of class \code{rm5}.
#' @param comp.no Comparison number.
#' @param outcome.no Outcome number.
#' @param ... Additional arguments (passed on to \code{metacr}).
#' 
#' @details
#' This function can be used to redo all or selected meta-analyses of
#' a Cochrane Review of interventions (Higgins et al., 2023).
#' 
#' Review Manager 5 (RevMan 5) was the software used for preparing and
#' maintaining Cochrane Reviews. In RevMan 5, subgroup analyses can be defined
#' and data from a Cochrane review can be imported to R using the function
#' \code{read.rm5}.
#' 
#' The R function \code{\link{metacr}} is called internally.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{summary.meta}}, \code{\link{metacr}},
#'   \code{\link{read.rm5}}, \code{\link{metabias.rm5}}
#' 
#' @references
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
#' VA (editors) (2023):
#' \emph{Cochrane Handbook for Systematic Reviews of Interventions
#'   Version 6.4 (updated August 2023)}.
#' Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}
#' 
#' @examples
#' # Locate export data file "Fleiss1993_CR.csv"
#' # in sub-directory of package "meta"
#' #
#' filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
#' Fleiss1993_CR <- read.rm5(filename)
#' 
#' # Print results for all meta-analysis
#' #
#' Fleiss1993_CR
#' 
#' # Print results only for second outcome of first comparison
#' #
#' print(Fleiss1993_CR, comp.no = 1, outcome.no = 2)
#' 
#' @method print rm5
#' @export

print.rm5 <- function(x, comp.no, outcome.no, ...) {
  
  
  #
  #
  # (1) Check for rm5 object
  #
  #
  chkclass(x, "rm5")
  
  
  if (missing(comp.no))
    comp.no <- unique(x$comp.no)
  #
  res <- list()
  #
  n <- 1
  #
  for (i in comp.no) {
    if (missing(outcome.no))
      jj <- unique(x$outcome.no[x$comp.no == i])
    else
      jj <- outcome.no
    for (j in jj) {
      #
      res[[n]] <- metacr(x, i, j, ...)
      #
      n <- n + 1
    }
  }
  #
  n <- 1
  #
  for (i in seq_len(length(res))) {
    if (n > 1)
      cat("\n*****\n\n")
    print(res[[n]])
    n <- n + 1
  }
  
  
  invisible(NULL)
}

gm <- function(x, digits = 4, debug = FALSE) {
  
  if (inherits(x, c("metabin", "metainc", "metaprop", "metarate"))) {
    incr_not_0 <- sort(unique(x$incr[x$incr != 0]))
    #
    if (length(incr_not_0) == 1)
      incr <- incr_not_0
    else if (length(incr_not_0) == 0)
      incr <- 0
    else
      incr <- paste0("{", paste(incr_not_0, collapse = ", "), "}")
  }
  
  func <- if (debug) list else data.frame
  # Get rid of warning 'Undefined global functions or variables'
  model <- NULL
  #
  if (inherits(x, "metabind")) {
    res <-
      with(
        x$data,
        list(
          meth =
            func(
              model, method, three.level,
              k, k.all,
              k.MH = replaceNULL(x$k.MH, NA),
              k.study, k.TE,
              k0 = replaceNULL(x$k0, NA),
              method.tau, method.tau.ci,
              tau = round(tau, digits), tau.preset,
              method.I2,
              method.common.ci,
              method.random.ci,
              df.random = replaceNULL(df.random),
              adhoc.hakn.ci,
              rho),
          pred =
            func(
              model, method.predict,
              df.predict = replaceNULL(df.predict), adhoc.hakn.pi)
        )
      )
    #
    res$meth <- subset(res$meth, model %in% c("common", "random"))
    res$pred <- subset(res$pred, model == "predict")
    #
    return(res)
  }
  #
  if (inherits(x, c("metacum", "metainf"))) {
    if (x$pooled == "random") {
      df.randoms <-
        replaceNULL(sort(unique(c(x$df.random, x$df.random.pooled))))
      #
      df.randoms <- df.randoms[is.finite(df.randoms)]
      #
      if (length(df.randoms) == 0)
        label.df.random <- ""
      else if (length(df.randoms) == 1)
        label.df.random <- df.randoms
      else {
        if (all(diff(df.randoms) == 1) & length(df.randoms) > 5)
          label.df.random <-
            paste0("{", min(df.randoms), ", ..., ", max(df.randoms), "}")
        else
          label.df.random <-
            paste0("{", paste(df.randoms, collapse = ", "), "}")
      }
      #
      if (x$prediction) {
        df.predicts <-
          replaceNULL(sort(unique(c(x$df.predict, x$df.predict.pooled))))
        #
        df.predicts <- df.predicts[is.finite(df.predicts)]
        #
        if (length(df.predicts) == 0)
          label.df.predict <- ""
        else if (length(df.predicts) == 1)
          label.df.predict <- df.predicts
        else {
          if (all(diff(df.predicts) == 1))
            label.df.predict <-
              paste0("{", min(df.predicts), ", ..., ", max(df.predicts), "}")
          else
            label.df.predict <-
              paste0("{", paste(df.predicts, collapse = ", "), "}")
        }
      }
      else
        label.df.predict <- ""
    }
    else {
      label.df.random <- ""
      label.df.predict <- ""
    }
    #
    res <-
      with(
        x,
        list(
          meth =
            func(
              model = pooled,
              method = method, three.level = FALSE,
              k = k.pooled, k.all = k.all.pooled,
              k.MH = replaceNULL(x$k.MH.pooled),
              k.study = k.study.pooled, k.TE = k.TE.pooled,
              k0 = NA,
              method.tau = method.tau,
              method.tau.ci = replaceNULL(method.tau.ci, ""),
              tau = round(tau.pooled, digits),
              tau.preset = replaceNULL(tau.preset),
              method.I2 = method.I2,
              method.common.ci = method.common.ci,
              method.random.ci = method.random.ci,
              df.random = label.df.random,
              Q.Cochrane = replaceNULL(x$Q.Cochrane, FALSE),
              adhoc.hakn.ci = adhoc.hakn.ci,
              rho = 0),
          pred =
            func(
              model = if (pooled == "random") "predict" else pooled,
              method.predict,
              df.predict = label.df.predict,
              adhoc.hakn.pi)
        )
      )
    #
    if (debug)
      print(res)
    #
    res$meth <- subset(res$meth, model %in% c("common", "random"))
    res$pred <- subset(res$pred, model == "predict")
    #
    return(res)
  }
  else {
    #
    n.com <- length(x$lower.common)
    n.ran <- length(x$lower.random)
    n.prd <- length(x$lower.predict)
    #
    x$method <- expandvar(x$method, n.com, 1)
    #
    x$method.common.ci <- expandvar(x$method.common.ci, n.com, 1)
    #
    x$method.random <- expandvar(x$method.random, n.ran, 1)
    x$method.random.ci <- expandvar(x$method.random.ci, n.ran, 1)
    x$method.tau <- expandvar(x$method.tau, n.ran, 1)
    x$method.tau.ci <- expandvar(x$method.tau.ci, n.ran, 1)
    if (length(x$three.level) == 1 && x$three.level)
      x$tau <- sum(x$tau)
    x$three.level <- expandvar(x$three.level, n.ran, 1)
    x$tau <- expandvar(x$tau, n.ran, 1)
    x$adhoc.hakn.ci <- expandvar(x$adhoc.hakn.ci, n.ran, 1)
    x$rho <- expandvar(x$rho, n.ran, 1)
    #
    x$method.predict <- expandvar(x$method.predict, n.prd, 1)
    #
    res <-
      list(meth = 
             with(x,
                  func(
                    model = c(rep("common", n.com), rep("random", n.ran)),
                    method = c(method, method.random),
                    three.level = c(rep(FALSE, n.com), x$three.level),
                    #
                    k = k, k.all = k.all, k.MH = NA, k.study = k.study,
                    k.TE = k.TE, k0 = NA,
                    #
                    method.tau = c(rep("", n.com), x$method.tau),
                    method.tau.ci = c(rep("", n.com), x$method.tau.ci),
                    tau = round(c(rep(NA, n.com), x$tau), digits),
                    tau.preset = NA,
                    #
                    method.I2 = x$method.I2,
                    #
                    method.common.ci = c(method.common.ci, rep("", n.ran)),
                    method.random.ci = c(rep("", n.com), method.random.ci),
                    df.random = c(rep(NA, n.com), unlist(df.random)),
                    adhoc.hakn.ci = c(rep("", n.com), adhoc.hakn.ci),
                    #
                    rho = c(rep(0, n.com), x$rho))),
           pred =
             with(x,
                  data.frame(method.predict = method.predict))
           )
  }
  #
  if (!is.null(x$tau.preset))
    res$meth$tau.preset <- round(c(rep(NA, n.com), x$tau.preset), digits)
  #
  if (inherits(x, "metabin")) {
    res$meth$k.MH <- x$k.MH
    res$meth$incr <- incr
    res$meth$method.incr <- x$method.incr
    res$meth$sparse <- x$sparse
    res$meth$allstudies <- x$allstudies
    res$meth$doublezeros <- x$doublezeros
    res$meth$MH.exact <- x$MH.exact
    res$meth$RR.Cochrane <- x$RR.Cochrane
    if (length(x$Q.Cochrane) == 1)
      x$Q.Cochrane <- rep(x$Q.Cochrane, n.ran)
    res$meth$Q.Cochrane <- c(rep(FALSE, n.com), x$Q.Cochrane)
  }
  #
  if (inherits(x, "metacont")) {
    res$meth$pooledvar <- x$pooledvar
    res$meth$method.smd <- x$method.smd
    res$meth$sd.glass <- x$sd.glass
    res$meth$exact.smd <- x$exact.smd
    res$meth$method.mean <- x$method.mean
    res$meth$method.sd <- x$method.sd
  }
  #
  if (inherits(x, "metainc"))
    res$meth$k.MH <- x$k.MH
  #
  if (inherits(x, c("metainc", "metaprop", "metarate"))) {
    res$meth$incr <- incr
    res$meth$method.incr <- x$method.incr
    res$meth$sparse <- x$sparse
  }
  if (!is.null(x$model.glmm))
    res$meth$model.glmm <- x$model.glmm
  #
  if (!is.null(x$phi))
    res$meth$phi <- x$phi
  #
  if (inherits(x, "trimfill")) {
    res$meth$k0 <- x$k0
    res$meth$left <- x$left
    res$meth$ma.common <- x$ma.common
    res$meth$type <- x$type
    res$meth$n.iter.max <- x$n.iter.max
    res$meth$n.iter <- x$n.iter
  }
  if (allNA(res$meth$k.MH))
    res$meth$k.MH <- NULL
  if (allNA(res$meth$k0))
    res$meth$k0 <- NULL
  #
  if (!is.null(x$df.predict))
    res$pred$df.predict <- unlist(x$df.predict)
  if (!is.null(x$adhoc.hakn.pi))
    res$pred$adhoc.hakn.pi <- x$adhoc.hakn.pi
  #
  res
}

#' Description of summary measures available in R package \bold{meta}
#' 
#' @description
#' Description of summary measures available in R package \bold{meta}
#' 
#' @details
#' The following summary measures (argument \code{sm}) are recognised
#' in R package \bold{meta}.
#'
#' \subsection{Meta-analysis of binary outcome data (\code{\link{metabin})}}{
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "OR"} \tab Odds ratio (Fleiss, 1993)\cr
#' \code{sm = "RR"} \tab Risk ratio (Fleiss, 1993) \cr
#' \code{sm = "RD"} \tab Risk difference (Fleiss, 1993) \cr
#' \code{sm = "ASD"} \tab Arcsine difference (Rücker et al., 2009) \cr
#' \code{sm = "DOR"} \tab Diagnostic odds ratio (Moses et al., 1993)
#'   \cr
#' \code{sm = "VE"} \tab Vaccine efficacy or vaccine effectiveness
#' }
#'
#' Note, mathematically, odds ratios and diagnostic odds ratios are
#' identical, however, the labels in printouts and figures
#' differ. Furthermore, log risk ratio (logRR) and log vaccine ratio
#' (logVR) are mathematical identical, however, back-transformed
#' results differ as vaccine efficacy or effectiveness is defined as
#' \code{VE = 100 * (1 - RR)}.
#'
#' A continuity correction is used for some summary measures in the
#' case of a zero cell count (see \code{\link{metabin}}). 
#'
#' List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
#' contain transformed values, e.g., log odds ratios, log risk ratios
#' or log vaccine ratios.  In printouts and plots transformed values
#' are back transformed if argument \code{backtransf = TRUE}
#' (default), with exception of the arcsine difference where no
#' back-transformation exists. Auxiliary function
#' \code{\link{logVR2VE}} is used to back-transform log vaccine ratios
#' to vaccine efficacy or effectiveness while \code{\link[base]{exp}}
#' is used to back-transform log odds or risk ratios.
#' }
#'
#' \subsection{Meta-analysis of continuous outcome data (\code{\link{metacont})}}{
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "MD"} \tab Mean difference \cr
#' \code{sm = "SMD"} \tab Standardised mean difference \cr
#' \code{sm = "ROM"} \tab Ratio of means
#' }
#'
#' Three variants to calculate the standardised mean difference are
#' available (see \code{\link{metacont}}).
#' 
#' For the ratio of means, list elements \code{TE}, \code{TE.common},
#' \code{TE.random}, etc., contain the log transformed ratio of
#' means. In printouts and plots these values are back transformed
#' using \code{\link[base]{exp}} if argument \code{backtransf = TRUE}
#' (default).
#' }
#'
#' \subsection{Meta-analysis of correlations (\code{\link{metacor})}}{
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "ZCOR"} \tab Fisher's z transformed correlation \cr
#' \code{sm = "COR"} \tab Untransformed correlations
#' }
#'
#' For Fisher's z transformed correlations, list elements \code{TE},
#' \code{TE.common}, \code{TE.random}, etc., contain the transformed
#' correlations. In printouts and plots these values are back
#' transformed using auxiliary function \code{\link{z2cor}} if
#' argument \code{backtransf = TRUE} (default).
#' }
#'
#' \subsection{Meta-analysis of incidence rates (\code{\link{metainc})}}{
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "IRR"} \tab Incidence rate ratio \cr
#' \code{sm = "IRD"} \tab Incidence rate difference \cr
#' \code{sm = "IRSD"} \tab Square root transformed incidence rate
#'   difference \cr
#' \code{sm = "VE"} \tab Vaccine efficacy or vaccine effectiveness
#' }
#'
#' Note, log incidence rate ratio (logIRR) and log vaccine ratio
#' (logVR) are mathematical identical, however, back-transformed
#' results differ as vaccine efficacy or effectiveness is defined as
#' \code{VE = 100 * (1 - IRR)}.
#'
#' List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
#' contain the transformed incidence rates. In printouts and plots
#' these values are back transformed if argument \code{backtransf =
#' TRUE} (default). For back-transformation, \code{\link[base]{exp}}
#' is used for the incidence rate ratio, power of 2 is used for square
#' root transformed rates and \code{\link{logVR2VE}} is used for
#' vaccine efficacy / effectiveness.
#' }
#'
#' \subsection{Meta-analysis of single means (\code{\link{metamean})}}{
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "MRAW"} \tab Raw, i.e. untransformed, means \cr
#' \code{sm = "MLN"} \tab Log transformed means
#' }
#' 
#' Calculations are conducted on the log scale if \code{sm =
#' "MLN"}. Accordingly, list elements \code{TE}, \code{TE.common}, and
#' \code{TE.random} contain the logarithm of means. In printouts and
#' plots these values are back transformed using
#' \code{\link[base]{exp}} if argument \code{backtransf = TRUE}.
#' }
#'
#' \subsection{Meta-analysis of single proportions (\code{\link{metaprop})}}{
#' 
#' The following transformations of proportions are
#' implemented to calculate an overall proportion:
#' 
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "PLOGIT"} \tab Logit transformation \cr
#' \code{sm = "PAS"} \tab Arcsine transformation \cr
#' \code{sm = "PFT"} \tab Freeman-Tukey Double arcsine transformation
#'   \cr
#' \code{sm = "PLN"} \tab Log transformation \cr
#' \code{sm = "PRAW"} \tab No transformation
#' }
#'
#' List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
#' contain the transformed proportions. In printouts and plots these
#' values are back transformed if argument \code{backtransf = TRUE}
#' (default). For back-transformation, \code{\link{logit2p}} is used
#' for logit transformed proportions, \code{\link{asin2p}} is used for
#' (Freeman-Tukey) arcsine transformed proportions and
#' \code{\link[base]{exp}} is used for log transformed proportions.
#' }
#'
#' \subsection{Meta-analysis of single rates (\code{\link{metarate})}}{
#' 
#' The following transformations of incidence rates are implemented to
#' calculate an overall rate:
#' 
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "IRLN"} \tab Log transformation \cr
#' \code{sm = "IRS"} \tab Square root transformation \cr
#' \code{sm = "IRFT"} \tab Freeman-Tukey Double arcsine transformation
#'   \cr
#' \code{sm = "IR"} \tab No transformation
#' }
#'
#' List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
#' contain the transformed incidence rates. In printouts and plots
#' these values are back transformed if argument \code{backtransf =
#' TRUE} (default). For back-transformation, \code{\link[base]{exp}}
#' is used for log transformed rates, power of 2 is used for square
#' root transformed rates and \code{\link{asin2ir}} is used for
#' Freeman-Tukey arcsine transformed rates.
#' }
#'
#' \subsection{Generic inverse variance method (\code{\link{metagen})}}{
#' 
#' The following summary measures are recognised in addition to the
#' above mentioned summary measures:
#' 
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Summary measure} \cr
#' \code{sm = "HR"} \tab Hazard ratio \cr
#' \code{sm = "VE"} \tab Vaccine efficacy or vaccine effectiveness
#' }
#'
#' List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
#' contain transformed values, i.e., log hazard ratios and log vaccine
#' ratios. In printouts and plots these values are back transformed if
#' argument \code{backtransf = TRUE} (default); see also
#' \link{meta-transf}.
#' }
#'
#' @name meta-sm
#' 
#' @aliases meta-sm
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{meta-transf}},
#'   \code{\link{meta-object}}, \code{\link{print.meta}},
#'   \code{\link{summary.meta}}, \code{\link{forest.meta}}
#' 
#' @references
#' Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
#' A basic introduction to fixed-effect and random-effects models for
#' meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{1}, 97--111
#' 
#' Fleiss JL (1993):
#' The statistical basis of meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{2}, 121--45
#'
#' Moses LE, Shapiro D, Littenberg B (1993):
#' Combining Independent Studies of a Diagnostic Test into a Summary
#' Roc Curve: Data-Analytic Approaches and Some Additional
#' Considerations.
#' \emph{Statistics in Medicine},
#' \bold{12}, 1293--1316
#' 
#' Rücker G, Schwarzer G, Carpenter J, Olkin I (2009):
#' Why add anything to nothing? The arcsine difference as a measure of
#' treatment effect in meta-analysis with zero cells.
#' \emph{Statistics in Medicine},
#' \bold{28}, 721--38
#' 
#' Stijnen T, Hamza TH, Ozdemir P (2010):
#' Random effects meta-analysis of event outcome in the framework of
#' the generalized linear mixed model with applications in sparse
#' data.
#' \emph{Statistics in Medicine},
#' \bold{29}, 3046--67


NULL

#' Meta-analysis of continuous outcome data
#' 
#' @description
#' Calculation of common and random effects estimates for meta-analyses
#' with continuous outcome data; inverse variance weighting is used
#' for pooling.
#' 
#' @param n.e Number of observations in experimental group or an R object
#'   created with \code{\link{pairwise}}.
#' @param mean.e Estimated mean in experimental group.
#' @param sd.e Standard deviation in experimental group.
#' @param n.c Number of observations in control group.
#' @param mean.c Estimated mean in control group.
#' @param sd.c Standard deviation in control group.
#' @param studlab An optional vector with study labels.
#' @param data An optional data frame containing the study
#'   information.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots.
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param median.e Median in experimental group (used to estimate the
#'   mean and standard deviation).
#' @param q1.e First quartile in experimental group (used to estimate
#'   the mean and standard deviation).
#' @param q3.e Third quartile in experimental group (used to estimate
#'   the mean and standard deviation).
#' @param min.e Minimum in experimental group (used to estimate the
#'   mean and standard deviation).
#' @param max.e Maximum in experimental group (used to estimate the
#'   mean and standard deviation).
#' @param median.c Median in control group (used to estimate the mean
#'   and standard deviation).
#' @param q1.c First quartile in control group (used to estimate the
#'   mean and standard deviation).
#' @param q3.c Third quartile in control group (used to estimate the
#'   mean and standard deviation).
#' @param min.c Minimum in control group (used to estimate the mean
#'   and standard deviation).
#' @param max.c Maximum in control group (used to estimate the mean
#'   and standard deviation).
#' @param method.mean A character string indicating which method to
#'   use to approximate the mean from the median and other statistics
#'   (see Details).
#' @param method.sd A character string indicating which method to use
#'   to approximate the standard deviation from sample size, median,
#'   interquartile range and range (see Details).
#' @param approx.mean.e Approximation method to estimate means in
#'   experimental group (see Details).
#' @param approx.mean.c Approximation method to estimate means in
#'   control group (see Details).
#' @param approx.sd.e Approximation method to estimate standard
#'   deviations in experimental group (see Details).
#' @param approx.sd.c Approximation method to estimate standard
#'   deviations in control group (see Details).
#' @param method.ci A character string indicating which method is used
#'   to calculate confidence intervals for individual studies (see
#'   Details).
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param method.bias A character string indicating which test is to
#'   be used. Either \code{"Begg"}, \code{"Egger"}, \code{"Thompson"},
#'   or \code{"Pustejovsky"} (see \code{\link{metabias}}), can be
#'   abbreviated.
#' @param backtransf A logical indicating whether results for ratio of
#'   means (\code{sm="ROM"}) should be back transformed in printouts
#'   and plots. If TRUE (default), results will be presented as ratio
#'   of means; otherwise log ratio of means will be shown.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.e Label for experimental group.
#' @param label.c Label for control group.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param sm A character string indicating which summary measure
#'   (\code{"MD"}, \code{"SMD"}, or \code{"ROM"}) is to be used for
#'   pooling of studies.
#' @param pooledvar A logical indicating if a pooled variance should
#'   be used for the mean difference (\code{sm="MD"}) or ratio of
#'   means (\code{sm="ROM"}).
#' @param method.smd A character string indicating which method is
#'   used to estimate the standardised mean difference
#'   (\code{sm="SMD"}). Either \code{"Hedges"} for Hedges' g
#'   (default), \code{"Cohen"} for Cohen's d, or \code{"Glass"} for
#'   Glass' delta, can be abbreviated.
#' @param sd.glass A character string indicating which standard
#'   deviation is used in the denominator for Glass' method to
#'   estimate the standardised mean difference. Either
#'   \code{"control"} using the standard deviation in the control
#'   group (\code{sd.c}) or \code{"experimental"} using the standard
#'   deviation in the experimental group (\code{sd.e}), can be
#'   abbreviated.
#' @param exact.smd A logical indicating whether exact formulae should
#'   be used in estimation of the standardised mean difference and its
#'   standard error (see Details).
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param id Deprecated argument (replaced by 'cluster').
#' @param adhoc.hakn Deprecated argument (replaced by
#'   'adhoc.hakn.ci').
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if studies are excluded from meta-analysis due to zero
#'   standard deviations).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}}.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' Calculation of common and random effects estimates for meta-analyses
#' with continuous outcome data; inverse variance weighting is used
#' for pooling.
#' 
#' A three-level random effects meta-analysis model (Van den Noortgate
#' et al., 2013) is utilised if argument \code{cluster} is used and at
#' least one cluster provides more than one estimate. Internally,
#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
#' "rowsum"} is used to calculate random effects weights.
#' 
#' Default settings are utilised for several arguments (assignments
#' using \code{\link{gs}} function). These defaults can be changed for
#' the current R session using the \code{\link{settings.meta}}
#' function.
#' 
#' Furthermore, R function \code{\link{update.meta}} can be used to
#' rerun a meta-analysis with different settings.
#' 
#' Three different types of summary measures are available for continuous
#' outcomes:
#' \itemize{
#' \item mean difference (argument \code{sm = "MD"})
#' \item standardised mean difference (\code{sm = "SMD"})
#' \item ratio of means (\code{sm = "ROM"})
#' }
#' 
#' \subsection{Standardised mean difference}{
#' 
#' For the standardised mean difference three methods are implemented:
#' \itemize{
#' \item Hedges' g (default, \code{method.smd = "Hedges"}) - see
#'   Hedges (1981)
#' \item Cohen's d (\code{method.smd = "Cohen"}) - see Cohen (1988)
#' \item Glass' delta (\code{method.smd = "Glass"}) - see Glass (1976)
#' }
#'
#' Hedges (1981) calculated the exact bias in Cohen's d which is a
#' ratio of gamma distributions with the degrees of freedom,
#' i.e. total sample size minus two, as parameter. In the 1980s and 1990s, the
#' gamma distribution was not easily accessible in statistical software.
#' Accordingly, an accurate approximation of the exact bias, also provided in
#' Hedges (1981), was utilised for Hedges' g and its standard error in software
#' like RevMan 5. White and Thomas (2005) argued that approximations are
#' unnecessary with modern software and accordingly promoted to use the exact
#' formulae which is the default in R function metacont
#' (argument \code{exact.smd = TRUE}). For Hedges' g, the exact formulae are
#' used to calculate the standardised mean difference as well as the standard
#' error; for Cohen's d the exact formula is only used to calculate the standard
#' error. In typical applications (with sample sizes above 10), the differences
#' between using the exact formulae (\code{exact.smd = TRUE}) and the
#' approximation (\code{exact.smd = FALSE}) will be minimal.
#' 
#' For Glass' delta, by default (argument \code{sd.glass =
#' "control"}), the standard deviation in the control group
#' (\code{sd.c}) is used in the denominator of the standard mean
#' difference. The standard deviation in the experimental group
#' (\code{sd.e}) can be used by specifying \code{sd.glass =
#' "experimental"}.
#' }
#' 
#' \subsection{Ratio of means}{
#' 
#' Meta-analysis of ratio of means -- also called response ratios --
#' is described in Hedges et al. (1999) and Friedrich et al. (2008).
#' Calculations are conducted on the log scale and list elements
#' \code{TE}, \code{TE.common}, and \code{TE.random} contain the
#' logarithm of the ratio of means. In printouts and plots these
#' values are back transformed if argument \code{backtransf = TRUE}.
#' }
#' 
#' \subsection{Approximate means from sample sizes, medians and other statistics}{
#' 
#' Missing means in the experimental group (analogously for the
#' control group) can be derived from
#' \enumerate{
#' \item sample size, median, interquartile range and range (arguments
#'   \code{n.e}, \code{median.e}, \code{q1.e}, \code{q3.e},
#'   \code{min.e}, and \code{max.e}),
#' \item sample size, median and interquartile range (arguments
#'   \code{n.e}, \code{median.e}, \code{q1.e}, and \code{q3.e}), or
#' \item sample size, median and range (arguments \code{n.e},
#'   \code{median.e}, \code{min.e}, and \code{max.e}).
#' }
#' 
#' By default, methods described in Luo et al. (2018) are utilised
#' (argument \code{method.mean = "Luo"}):
#' \itemize{
#' \item equation (15) if sample size, median, interquartile range and 
#'   range are available,
#' \item equation (11) if sample size, median and interquartile range
#'   are available,
#' \item equation (7) if sample size, median and range are available.
#' }
#' 
#' Instead the methods described in Wan et al. (2014) are used if
#' argument \code{method.mean = "Wan"}:
#' \itemize{
#' \item equation (10) if sample size, median, interquartile range and 
#'   range are available,
#' \item equation (14) if sample size, median and interquartile range
#'   are available,
#' \item equation (2) if sample size, median and range are available.
#' }
#'
#' The following methods are also available to estimate means from
#' quantiles or ranges if R package \bold{estmeansd} is installed:
#' \itemize{
#' \item Method for Unknown Non-Normal Distributions (MLN) approach
#'   (Cai et al. (2021), argument \code{method.mean = "Cai"}),
#' \item Quantile Estimation (QE) method (McGrath et al. (2020),
#'   argument \code{method.mean = "QE-McGrath"})),
#' \item Box-Cox (BC) method (McGrath et al. (2020),
#'   argument \code{method.mean = "BC-McGrath"})).
#' }
#'
#' By default, missing means are replaced successively using
#' interquartile ranges and ranges (if available), interquartile
#' ranges (if available) and finally ranges. Arguments
#' \code{approx.mean.e} and \code{approx.mean.c} can be used to
#' overwrite this behaviour for each individual study and treatment
#' arm:
#' \itemize{
#' \item use means directly (entry \code{""} in argument
#'   \code{approx.mean.e} or \code{approx.mean.c});
#' \item median, interquartile range and range (\code{"iqr.range"});
#' \item median and interquartile range (\code{"iqr"});
#' \item median and range (\code{"range"}).
#' }
#' }
#'
#' \subsection{Approximate standard deviations from sample sizes,
#' medians and other statistics}{
#' 
#' Missing standard deviations in the experimental group (analogously
#' for the control group) can be derived from
#' \enumerate{
#' \item sample size, median, interquartile range and range (arguments
#'   \code{n.e}, \code{median.e}, \code{q1.e}, \code{q3.e},
#'   \code{min.e}, and \code{max.e}),
#' \item sample size, median and interquartile range (arguments
#'   \code{n.e}, \code{median.e}, \code{q1.e} and \code{q3.e}), or
#' \item sample size, median and range (arguments \code{n.e},
#'   \code{median.e}, \code{min.e} and \code{max.e}).
#' }
#' 
#' Wan et al. (2014) describe methods to estimate the standard
#' deviation from the sample size, median and additional
#' statistics. Shi et al. (2020) provide an improved estimate of the
#' standard deviation if the interquartile range and range are
#' available in addition to the sample size and median. Accordingly,
#' equation (11) in Shi et al. (2020) is the default (argument
#' \code{method.sd = "Shi"}), if the median, interquartile range and
#' range are provided. The method by Wan et al. (2014) is used if
#' argument \code{method.sd = "Wan"} and, depending on the sample
#' size, either equation (12) or (13) is used. If only the
#' interquartile range or range is available, equations (15) / (16)
#' and (7) / (9) in Wan et al. (2014) are used, respectively.
#'
#' The following methods are also available to estimate standard
#' deviations from quantiles or ranges if R package \bold{estmeansd}
#' is installed:
#' \itemize{
#' \item Method for Unknown Non-Normal Distributions (MLN) approach
#'   (Cai et al. (2021), argument \code{method.mean = "Cai"}),
#' \item Quantile Estimation (QE) method (McGrath et al. (2020),
#'   argument \code{method.mean = "QE-McGrath"})),
#' \item Box-Cox (BC) method (McGrath et al. (2020),
#'   argument \code{method.mean = "BC-McGrath"})).
#' }
#'
#' By default, missing standard deviations are replaced successively
#' using these method, i.e., interquartile ranges and ranges are used
#' before interquartile ranges before ranges. Arguments
#' \code{approx.sd.e} and \code{approx.sd.c} can be used to overwrite
#' this default for each individual study and treatment arms:
#' \itemize{
#' \item sample size, median, interquartile range and range
#'   (\code{"iqr.range"});
#' \item sample size, median and interquartile range (\code{"iqr"});
#' \item sample size, median and range (\code{"range"}).
#' }
#' }
#' 
#' \subsection{Confidence intervals for individual studies}{
#' 
#' For the mean difference (argument \code{sm = "MD"}), the confidence
#' interval for individual studies can be based on the
#' \itemize{
#' \item standard normal distribution (\code{method.ci = "z"}, default), or
#' \item t-distribution (\code{method.ci = "t"}).
#' }
#' 
#' Note, this choice does not affect the results of the common effect
#' and random effects meta-analysis.
#' }
#'
#' \subsection{Subgroup analysis}{
#' 
#' Argument \code{subgroup} can be used to conduct subgroup analysis for
#' a categorical covariate. The \code{\link{metareg}} function can be
#' used instead for more than one categorical covariate or continuous
#' covariates.
#' }
#' 
#' \subsection{Exclusion of studies from meta-analysis}{
#'
#' Arguments \code{subset} and \code{exclude} can be used to exclude
#' studies from the meta-analysis. Studies are removed completely from
#' the meta-analysis using argument \code{subset}, while excluded
#' studies are shown in printouts and forest plots using argument
#' \code{exclude} (see Examples in \code{\link{metagen}}).
#' Meta-analysis results are the same for both arguments.
#' }
#' 
#' \subsection{Presentation of meta-analysis results}{
#' 
#' Internally, both common effect and random effects models are
#' calculated regardless of values choosen for arguments \code{common}
#' and \code{random}. Accordingly, the estimate for the random effects
#' model can be extracted from component \code{TE.random} of an object
#' of class \code{"meta"} even if argument \code{random =
#' FALSE}. However, all functions in R package \bold{meta} will
#' adequately consider the values for \code{common} and
#' \code{random}. E.g. function \code{\link{print.meta}} will not
#' print results for the random effects model if \code{random =
#' FALSE}.
#'
#' A prediction interval will only be shown if \code{prediction =
#' TRUE}.
#' }
#' 
#' @note
#' The function \code{\link{metagen}} is called internally to
#' calculate individual and overall treatment estimates and standard
#' errors.
#' 
#' @return
#' An object of class \code{c("metacont", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{update.meta}},
#'   \code{\link{metabin}}, \code{\link{metagen}}, \code{\link{pairwise}}
#' 
#' @references
#' Borenstein M, Hedges LV, Higgins JPT, Rothstein HR (2009):
#' \emph{Introduction to Meta-Analysis}.
#' Chichester: Wiley
#'
#' Cai S, Zhou J, Pan J (2021):
#' Estimating the sample mean and standard deviation from order
#' statistics and sample size in meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{30}, 2701--2719
#' 
#' Cohen J (1988):
#' \emph{Statistical Power Analysis for the Behavioral Sciences
#'   (second ed.)}.
#' Lawrence Erlbaum Associates
#' 
#' Friedrich JO, Adhikari NK, Beyene J (2008):
#' The ratio of means method as an alternative to mean differences for
#' analyzing continuous outcome variables in meta-analysis: A
#' simulation study.
#' \emph{BMC Medical Research Methodology},
#' \bold{8}, 32
#' 
#' Glass G (1976):
#' Primary, secondary, and meta-analysis of research.
#' \emph{Educational Researcher},
#' \bold{5}, 3--8
#' 
#' Hedges LV (1981):
#' Distribution theory for Glass's estimator of effect size and
#' related estimators.
#' \emph{Journal of Educational and Behavioral Statistics},
#' \bold{6}, 107--28
#' 
#' Hedges LV, Gurevitch J, Curtis PS (1999):
#' The meta-analysis of response ratios in experimental ecology.
#' \emph{Ecology},
#' \bold{80}, 1150--6
#'
#' Luo D, Wan X, Liu J, Tong T (2018):
#' Optimally estimating the sample mean from the sample size, median,
#' mid-range, and/or mid-quartile range.
#' \emph{Statistical Methods in Medical Research},
#' \bold{27}, 1785--805
#'
#' McGrath S, Zhao X, Steele R, et al. and the DEPRESsion Screening
#' Data (DEPRESSD) Collaboration (2020):
#' Estimating the sample mean and standard deviation from commonly
#' reported quantiles in meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{29}, 2520--2537
#' 
#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
#' The Cochrane Collaboration, 2020
#' 
#' Shi J, Luo D, Weng H, Zeng XT, Lin L, Chu H, Tong T (2020):
#' Optimally estimating the sample standard deviation from the
#' five-number summary.
#' \emph{Research Synthesis Methods},
#' \bold{11}, 641--54
#'
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#'
#' Wan X, Wang W, Liu J, Tong T (2014):
#' Estimating the sample mean and standard deviation from the sample
#' size, median, range and/or interquartile range.
#' \emph{BMC Medical Research Methodology},
#' \bold{14}, 135
#' 
#' White IR, Thomas J (2005):
#' Standardized mean differences in individually-randomized and
#' cluster-randomized trials, with applications to meta-analysis.
#' \emph{Clinical Trials},
#' \bold{2}, 141--51
#' 
#' @examples
#' data(Fleiss1993cont)
#'
#' # Meta-analysis with Hedges' g as effect measure
#' #
#' ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD")
#' ma1
#' forest(ma1)
#' 
#' # Use Cohen's d instead of Hedges' g as effect measure
#' #
#' update(ma1, method.smd = "Cohen")
#' 
#' # Use Glass' delta instead of Hedges' g as effect measure
#' #
#' update(ma1, method.smd = "Glass")
#' 
#' # Use Glass' delta based on the standard deviation in the experimental group
#' #
#' update(ma1, method.smd = "Glass", sd.glass = "experimental")
#' 
#' # Calculate Hedges' g based on exact formulae
#' #
#' update(ma1, exact.smd = TRUE)
#' 
#' data(amlodipine)
#' ma2 <- metacont(n.amlo, mean.amlo, sqrt(var.amlo),
#'   n.plac, mean.plac, sqrt(var.plac),
#'   data = amlodipine, studlab = study)
#' ma2
#' 
#' # Use pooled variance
#' #
#' update(ma2, pooledvar = TRUE)
#' 
#' # Meta-analysis of response ratios (Hedges et al., 1999)
#' #
#' data(woodyplants)
#' ma3 <- metacont(n.elev, mean.elev, sd.elev, n.amb, mean.amb, sd.amb,
#'   data = woodyplants, sm = "ROM")
#' ma3
#' print(ma3, backtransf = FALSE)
#' 
#' @export metacont

metacont <- function(n.e, mean.e, sd.e, n.c, mean.c, sd.c, studlab,
                     #
                     data = NULL, subset = NULL, exclude = NULL,
                     cluster = NULL, rho = 0,
                     #
                     weights = NULL,
                     weights.common = weights, weights.random = weights,
                     #
                     median.e, q1.e, q3.e, min.e, max.e,
                     median.c, q1.c, q3.c, min.c, max.c,
                     method.mean = "Luo", method.sd = "Shi",
                     approx.mean.e, approx.mean.c = approx.mean.e,
                     approx.sd.e, approx.sd.c = approx.sd.e,
                     #
                     sm = gs("smcont"),
                     method.ci = gs("method.ci.cont"),
                     level = gs("level"),
                     #
                     pooledvar = gs("pooledvar"),
                     method.smd = gs("method.smd"),
                     sd.glass = gs("sd.glass"),
                     exact.smd = gs("exact.smd"),
                     #
                     common = gs("common"),
                     random = gs("random") | !is.null(tau.preset),
                     overall = common | random,
                     overall.hetstat =
                       if (is.null(gs("overall.hetstat")))
                         common | random
                       else
                         gs("overall.hetstat"),   
                     prediction = gs("prediction") | !missing(method.predict),
                     #
                     method.tau = gs("method.tau"),
                     method.tau.ci = gs("method.tau.ci"),
                     level.hetstat = gs("level.hetstat"),
                     tau.preset = NULL, TE.tau = NULL,
                     tau.common = gs("tau.common"),
                     detail.tau = NULL,
                     #
                     method.I2 = gs("method.I2"),
                     #
                     level.ma = gs("level.ma"),
                     method.common.ci = gs("method.common.ci"),
                     method.random.ci = gs("method.random.ci"),
                     adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                     #
                     level.predict = gs("level.predict"),
                     method.predict = gs("method.predict"),
                     adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                     seed.predict = NULL,
                     #
                     method.bias = gs("method.bias"),
                     #
                     backtransf = gs("backtransf"),
                     #
                     text.common = gs("text.common"),
                     text.random = gs("text.random"),
                     text.predict = gs("text.predict"),
                     text.w.common = gs("text.w.common"),
                     text.w.random = gs("text.w.random"),
                     #
                     title = gs("title"), complab = gs("complab"),
                     outclab = "",
                     #
                     label.e = gs("label.e"), label.c = gs("label.c"),
                     label.left = gs("label.left"),
                     label.right = gs("label.right"),
                     col.label.left = gs("col.label.left"),
                     col.label.right = gs("col.label.right"),
                     #
                     subgroup, subgroup.name = NULL,
                     print.subgroup.name = gs("print.subgroup.name"),
                     sep.subgroup = gs("sep.subgroup"),
                     test.subgroup = gs("test.subgroup"),
                     prediction.subgroup = gs("prediction.subgroup"),
                     seed.predict.subgroup = NULL,
                     #
                     byvar, id, adhoc.hakn,
                     keepdata = gs("keepdata"),
                     warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
                     #
                     control = NULL,
                     ...) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  
  missing.sm <- missing(sm)
  missing.subgroup <- missing(subgroup)
  missing.byvar <- missing(byvar)
  missing.overall <- missing(overall)
  missing.overall.hetstat <- missing(overall.hetstat)
  missing.test.subgroup <- missing(test.subgroup)
  #
  missing.n.c <- missing(n.c)
  missing.mean.e <- missing(mean.e)
  missing.mean.c <- missing(mean.c)
  missing.sd.e <- missing(sd.e)
  missing.sd.c <- missing(sd.c)
  #
  missing.studlab <- missing(studlab)
  #
  missing.method.tau <- missing(method.tau)
  missing.tau.common <- missing(tau.common)
  missing.method.predict <- missing(method.predict)
  missing.level.ma <- missing(level.ma)
  missing.common <- missing(common)
  missing.random <- missing(random)
  missing.method.random.ci <- missing(method.random.ci)
  missing.adhoc.hakn.ci <- missing(adhoc.hakn.ci)
  missing.adhoc.hakn <- missing(adhoc.hakn)
  missing.subgroup.name <- missing(subgroup.name)
  missing.print.subgroup.name <- missing(print.subgroup.name)
  missing.sep.subgroup <- missing(sep.subgroup)
  #
  missing.label.e <- missing(label.e)
  missing.label.c <- missing(label.c)
  missing.complab <- missing(complab)
  #
  missing.cluster <- missing(cluster)
  missing.id <- missing(id)
  #
  missing.median.e <- missing(median.e)
  missing.q1.e <- missing(q1.e)
  missing.q3.e <- missing(q3.e)
  missing.min.e <- missing(min.e)
  missing.max.e <- missing(max.e)
  #
  missing.median.c <- missing(median.c)
  missing.q1.c <- missing(q1.c)
  missing.q3.c <- missing(q3.c)
  missing.min.c <- missing(min.c)
  missing.max.c <- missing(max.c)
  #
  missing.approx.mean.e <- missing(approx.mean.e)
  #
  if (missing.approx.mean.e)
    missing.approx.mean.c <- missing(approx.mean.c)
  else
    missing.approx.mean.c <- FALSE
  #
  missing.approx.sd.e <- missing(approx.sd.e)
  if (missing.approx.sd.e)
    missing.approx.sd.c <- missing(approx.sd.c)
  else
    missing.approx.sd.c <- FALSE
  #
  chknumeric(rho, min = -1, max = 1)
  #
  chknull(sm)
  sm <- setchar(sm, gs("sm4cont"))
  chklevel(level)
  #
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  method.tau <- setchar(method.tau, gs("meth4tau"))
  #
  tau.common <- replaceNULL(tau.common, FALSE)
  chklogical(tau.common)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  chklogical(prediction)
  chklevel(level.predict)
  #
  method.tau <-
    set_method_tau(method.tau, missing.method.tau,
                   method.predict, missing.method.predict)
  method.predict <-
    set_method_predict(method.predict, missing.method.predict,
                       method.tau, missing.method.tau)
  #
  if (any(method.predict == "NNF"))
    is_installed_package("pimeta", argument = "method.predict", value = "NNF")
  #
  adhoc.hakn.pi <- setchar(replaceNA(adhoc.hakn.pi, ""), gs("adhoc4hakn.pi"))
  #
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  if (!is.null(text.random))
    chkchar(text.random)
  if (!is.null(text.predict))
    chkchar(text.predict)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chklogical(keepdata)
  #
  # Additional arguments / checks for metacont objects
  #
  fun <- "metacont"
  if (sm != "MD")
    method.ci <- "z"
  method.ci <- setchar(method.ci, gs("ci4cont"))
  #
  method.mean <-
    setchar(method.mean, c("Luo", "Wan", "Cai", "QE-McGrath", "BC-McGrath"))
  method.sd <-
    setchar(method.sd, c("Shi", "Wan", "Cai", "QE-McGrath", "BC-McGrath"))
  #
  if (method.mean %in% c("Cai", "QE-McGrath", "BC-McGrath"))
    is_installed_package("estmeansd", argument = "method.mean",
                         value = method.mean)
  if (method.sd %in% c("Cai", "QE-McGrath", "BC-McGrath"))
    is_installed_package("estmeansd", argument = "method.sd",
                         value = method.sd)
  #
  chklogical(pooledvar)
  method.smd <- setchar(method.smd, c("Hedges", "Cohen", "Glass"))
  sd.glass <- setchar(sd.glass, c("control", "experimental"))
  chklogical(warn)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing.level.ma, args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing.random, args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.random.ci <-
    deprecated(method.random.ci, missing.method.random.ci,
               args, "hakn", warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  adhoc.hakn.ci <-
    deprecated2(adhoc.hakn.ci, missing.adhoc.hakn.ci,
                adhoc.hakn, missing.adhoc.hakn, warn.deprecated)
  adhoc.hakn.ci <- setchar(replaceNA(adhoc.hakn.ci, ""), gs("adhoc4hakn.ci"))
  #
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing.print.subgroup.name,
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing.sep.subgroup, args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  # Some more checks
  #
  chklogical(overall)
  chklogical(overall.hetstat)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata) {
    data <- sfsp
    data.pairwise <- FALSE
  }
  else
    data.pairwise <- inherits(data, "pairwise")
  #
  # Catch 'n.e', 'mean.e', 'sd.e', 'n.c', 'mean.c', 'sd.c', 'studlab',
  # and 'subgroup' from data:
  #
  n.e <- catch("n.e", mc, data, sfsp)
  chknull(n.e)
  #
  if (inherits(n.e, "pairwise")) {
    is.pairwise <- TRUE
    #
    type <- attr(n.e, "type")
    #
    if (type != "continuous")
      stop("Wrong type for pairwise() object: '", type, "'.", call. = FALSE)
    #
    txt.ignore <- "as first argument is a pairwise object"
    #
    warn_ignore_input(n.c, !missing.n.c, txt.ignore)
    warn_ignore_input(mean.e, !missing.mean.e, txt.ignore)
    warn_ignore_input(mean.c, !missing.mean.c, txt.ignore)
    warn_ignore_input(sd.e, !missing.sd.e, txt.ignore)
    warn_ignore_input(sd.c, !missing.sd.c, txt.ignore)
    warn_ignore_input(studlab, !missing.studlab, txt.ignore)
    #
    missing.n.c <- FALSE
    missing.mean.e <- FALSE
    missing.mean.c <- FALSE
    missing.sd.e <- FALSE
    missing.sd.c <- FALSE
    #
    if (missing.sm)
      sm <- attr(n.e, "sm")
    #
    reference.group <- attr(n.e, "reference.group")
    #
    studlab <- n.e$studlab
    #
    treat1 <- n.e$treat1
    treat2 <- n.e$treat2
    #
    n.c <- n.e$n2
    mean.e <- n.e$mean1
    mean.c <- n.e$mean2
    sd.e <- n.e$sd1
    sd.c <- n.e$sd2
    #
    pairdata <- n.e
    data <- n.e
    nulldata <- FALSE
    #
    n.e <- n.e$n1
    #
    wo <- treat1 == reference.group
    #
    if (any(wo)) {
      ttreat1 <- treat1
      treat1[wo] <- treat2[wo]
      treat2[wo] <- ttreat1[wo]
      #
      tn.e <- n.e
      n.e[wo] <- n.c[wo]
      n.c[wo] <- tn.e[wo]
      #
      tmean.e <- mean.e
      mean.e[wo] <- mean.c[wo]
      mean.c[wo] <- tmean.e[wo]
      #
      tsd.e <- sd.e
      sd.e[wo] <- sd.c[wo]
      sd.c[wo] <- tsd.e[wo]
    }
    #
    if (missing.subgroup) {
      subgroup <- paste(treat1, treat2, sep = " vs ")
      #
      if (length(unique(subgroup)) == 1) {
        if (missing.complab)
          complab <- unique(subgroup)
        #
        if (missing.label.e)
          label.e <- unique(treat1)
        if (missing.label.c)
          label.c <- unique(treat2)
        #
        subgroup <- NULL
      }
      else {
        if (missing.overall)
          overall <- FALSE
        if (missing.overall.hetstat)
          overall.hetstat <- FALSE
        if (missing.test.subgroup)
          test.subgroup <- FALSE
      }
    }
    else
      subgroup <- catch("subgroup", mc, data, sfsp)
  }
  else {
    is.pairwise <- FALSE
    #
    if (missing.sm && !is.null(data) && !is.null(attr(data, "sm")))
      sm <- attr(data, "sm")
    #
    mean.e <- catch("mean.e", mc, data, sfsp)
    sd.e <- catch("sd.e", mc, data, sfsp)
    #
    n.c <- catch("n.c", mc, data, sfsp)
    chknull(n.c)
    mean.c <- catch("mean.c", mc, data, sfsp)
    sd.c <- catch("sd.c", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    subgroup <- catch("subgroup", mc, data, sfsp)
    byvar <- catch("byvar", mc, data, sfsp)
    #
    subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                            warn.deprecated)
  }
  #
  # Classic tests + Pustejovsky
  #
  method.bias <-
    setmethodbias(method.bias, c(1:3, if (sm == "SMD") 8))
  #
  by <- !is.null(subgroup)
  #
  k.All <- length(n.e)
  #
  avail.mean.e <- !(missing.mean.e || is.null(mean.e))
  if (avail.mean.e)
    chknull(mean.e)
  else
    mean.e <- rep(NA, k.All)
  #
  avail.sd.e <- !(missing.sd.e || is.null(sd.e))
  if (avail.sd.e)
    chknull(sd.e)
  else
    sd.e <- rep(NA, k.All)
  #
  avail.mean.c <- !(missing.mean.c || is.null(mean.c))
  if (avail.mean.c)
    chknull(mean.c)
  else
    mean.c <- rep(NA, k.All)
  avail.sd.c <- !(missing.sd.c || is.null(sd.c))
  if (avail.sd.c)
    chknull(sd.c)
  else
    sd.c <- rep(NA, k.All)
  #
  studlab <- setstudlab(studlab, k.All)
  #
  # Catch 'subset', 'exclude', and 'cluster' from data:
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  exclude <- catch("exclude", mc, data, sfsp)
  missing.exclude <- is.null(exclude)
  #
  cluster <- catch("cluster", mc, data, sfsp)
  id <- catch("id", mc, data, sfsp)
  #
  cluster <- deprecated2(cluster, missing.cluster, id, missing.id,
                         warn.deprecated)
  with.cluster <- !is.null(cluster)
  #
  if (is.null(method.tau.ci))
    if (with.cluster)
      method.tau.ci <- "PL"
    else if (method.tau == "DL")
      method.tau.ci <- "J"
     else
      method.tau.ci <- "QP"
  #
  method.tau.ci <- setchar(method.tau.ci, gs("meth4tau.ci"))
  #
  # Catch 'weights', 'weights.common', and 'weights.random' from data:
  #
  if (!missing(weights))
    weights <- catch("weights", mc, data, sfsp)
  if (!missing(weights.common))
    weights.common <- catch("weights.common", mc, data, sfsp)
  if (!missing(weights.random))
    weights.random <- catch("weights.random", mc, data, sfsp)
  #
  if (!is.null(weights) & is.null(weights.common))
    weights.common <- weights
  #
  if (!is.null(weights) & is.null(weights.random))
    weights.random <- weights
  #
  usw.common <- !is.null(weights.common)
  usw.random <- !is.null(weights.random)
  #
  if (usw.common)
    chknumeric(weights.common, min = 0)
  #
  if (usw.random)
    chknumeric(weights.random, min = 0)
  
  
  #
  # Catch 'median.e', 'q1.e', 'q3.e', 'min.e', 'max.e', 'median.c',
  # 'q1.c', 'q3.c', 'min.c', 'max.c', 'approx.mean.e', 'approx.sd.e',
  # 'approx.mean.c', and 'approx.sd.c', from data:
  #
  median.e <- catch("median.e", mc, data, sfsp)
  #
  q1.e <- catch("q1.e", mc, data, sfsp)
  #
  q3.e <- catch("q3.e", mc, data, sfsp)
  #
  min.e <- catch("min.e", mc, data, sfsp)
  #
  max.e <- catch("max.e", mc, data, sfsp)
  #
  median.c <- catch("median.c", mc, data, sfsp)
  #
  q1.c <- catch("q1.c", mc, data, sfsp)
  #
  q3.c <- catch("q3.c", mc, data, sfsp)
  #
  min.c <- catch("min.c", mc, data, sfsp)
  #
  max.c <- catch("max.c", mc, data, sfsp)
  #
  avail.median.e <- !(missing.median.e || is.null(median.e))
  avail.q1.e <- !(missing.q1.e || is.null(q1.e))
  avail.q3.e <- !(missing.q3.e || is.null(q3.e))
  avail.min.e <- !(missing.min.e || is.null(min.e))
  avail.max.e <- !(missing.max.e || is.null(max.e))
  #
  avail.median.c <- !(missing.median.c || is.null(median.c))
  avail.q1.c <- !(missing.q1.c || is.null(q1.c))
  avail.q3.c <- !(missing.q3.c || is.null(q3.c))
  avail.min.c <- !(missing.min.c || is.null(min.c))
  avail.max.c <- !(missing.max.c || is.null(max.c))
  #
  approx.mean.e <- catch("approx.mean.e", mc, data, sfsp)
  approx.mean.c <- catch("approx.mean.c", mc, data, sfsp)
  approx.sd.e <- catch("approx.sd.e", mc, data, sfsp)
  approx.sd.c <- catch("approx.sd.c", mc, data, sfsp)
  #
  avail.approx.mean.e <-
    !(missing.approx.mean.e || is.null(approx.mean.e)) &&
    any(approx.mean.e != "")
  avail.approx.mean.c <-
    !(missing.approx.mean.c || is.null(approx.mean.c)) &&
    any(approx.mean.c != "")
  avail.approx.sd.e <-
    !(missing.approx.sd.e || is.null(approx.sd.e)) &&
    any(approx.sd.e != "")
  avail.approx.sd.c <-
    !(missing.approx.sd.c || is.null(approx.sd.c)) &&
    any(approx.sd.c != "")
  
  
  #
  # Additional checks
  #
  if (!avail.mean.e & !avail.median.e)
    stop("Provide either argument 'mean.e' or 'median.e'.",
         call. = FALSE)
  #
  if (!avail.mean.c & !avail.median.c)
    stop("Provide either argument 'mean.c' or 'median.c'.",
         call. = FALSE)
  #
  if (!avail.sd.e &
      !((avail.q1.e & avail.q3.e) | (avail.min.e & avail.max.e)))
    stop("Provide either argument 'sd.e' and ",
         "arguments 'q1.e' & 'q3.e' or 'min.e & 'max.e'.",
         call. = FALSE)
  #
  if (!avail.sd.c &
      !((avail.q1.c & avail.q3.c) | (avail.min.c & avail.max.c)))
    stop("Provide either argument 'sd.c' and ",
         "arguments 'q1.c' & 'q3.c' or 'min.c & 'max.c'.",
         call. = FALSE)
  #
  if (!by & tau.common) {
    warning("Value for argument 'tau.common' set to FALSE as ",
            "argument 'subgroup' is missing.")
    tau.common <- FALSE
  }
  if (by & !tau.common & !is.null(tau.preset)) {
    warning("Argument 'tau.common' set to TRUE as ",
            "argument tau.preset is not NULL.")
    tau.common <- TRUE
  }
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  arg <- "n.e"
  chklength(mean.e, k.All, arg)
  chklength(sd.e, k.All, arg)
  chklength(n.c, k.All, arg)
  chklength(mean.c, k.All, arg)
  chklength(sd.c, k.All, arg)
  chklength(studlab, k.All, arg)
  #
  if (with.cluster)
    chklength(cluster, k.All, arg)
  #
  if (usw.common) {
    if (length(weights.common) == 1)
      weights.common <- rep(weights.common, k.All)
    else
      chklength(weights.common, k.All, arg)
  }
  #
  if (usw.random) {
    if (length(weights.random) == 1)
      weights.random <- rep(weights.random, k.All)
    else
      chklength(weights.random, k.All, arg)
  }
  #
  if (avail.median.e)
    chklength(median.e, k.All, arg)
  if (avail.q1.e)
    chklength(q1.e, k.All, arg)
  if (avail.q3.e)
    chklength(q3.e, k.All, arg)
  if (avail.min.e)
    chklength(min.e, k.All, arg)
  if (avail.max.e)
    chklength(max.e, k.All, arg)
  if (avail.median.c)
    chklength(median.c, k.All, arg)
  if (avail.q1.c)
    chklength(q1.c, k.All, arg)
  if (avail.q3.c)
    chklength(q3.c, k.All, arg)
  if (avail.min.c)
    chklength(min.c, k.All, arg)
  if (avail.max.c)
    chklength(max.c, k.All, arg)
  #
  if (avail.approx.mean.e) {
    if (length(approx.mean.e) == 1)
      rep_len(approx.mean.e, k.All)
    else
      chklength(approx.mean.e, k.All, arg)
    #
    approx.mean.e <- setchar(approx.mean.e, c("", "iqr.range", "iqr", "range"))
  }
  else
    approx.mean.e <- rep_len("", k.All)
  #
  if (avail.approx.mean.c) {
    if (length(approx.mean.c) == 1)
      rep_len(approx.mean.c, k.All)
    else
      chklength(approx.mean.c, k.All, arg)
    #
    approx.mean.c <- setchar(approx.mean.c, c("", "iqr.range", "iqr", "range"))
  }
  else
    approx.mean.c <- rep_len("", k.All)
  #
  if (avail.approx.sd.e) {
    if (length(approx.sd.e) == 1)
      rep_len(approx.sd.e, k.All)
    else
      chklength(approx.sd.e, k.All, arg)
    #
    approx.sd.e <- setchar(approx.sd.e, c("", "iqr.range", "iqr", "range"))
  }
  else
    approx.sd.e <- rep_len("", k.All)
  #
  if (avail.approx.sd.c) {
    if (length(approx.sd.c) == 1)
      rep_len(approx.sd.c, k.All)
    else
      chklength(approx.sd.c, k.All, arg)
    #
    approx.sd.c <- setchar(approx.sd.c, c("", "iqr.range", "iqr", "range"))
  }
  else
    approx.sd.c <- rep_len("", k.All)
  #
  if (by) {
    chklength(subgroup, k.All, arg)
    chklogical(test.subgroup)
    chklogical(prediction.subgroup)
  }
  
  
  #
  #
  # (4) Subset, exclude studies, and subgroups
  #
  #
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of subset is larger than number of studies.")
  #
  if (!missing.exclude) {
    if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
        (length(exclude) > k.All))
      stop("Length of argument 'exclude' is larger than number of studies.")
    #
    exclude2 <- rep(FALSE, k.All)
    exclude2[exclude] <- TRUE
    exclude <- exclude2
  }
  else
    exclude <- rep(FALSE, k.All)
  
  
  #
  #
  # (5) Store complete dataset in list object data
  #     (if argument keepdata is TRUE)
  #
  #
  if (keepdata) {
    if (nulldata)
      data <- data.frame(.n.e = n.e)
    else
      data$.n.e <- n.e
    #
    data$.mean.e <- mean.e
    data$.sd.e <- sd.e
    data$.n.c <- n.c
    data$.mean.c <- mean.c
    data$.sd.c <- sd.c
    data$.studlab <- studlab
    #
    if (avail.median.e)
      data$.median.e <- median.e
    if (avail.q1.e)
      data$.q1.e <- q1.e
    if (avail.q3.e)
      data$.q3.e <- q3.e
    if (avail.min.e)
      data$.min.e <- min.e
    if (avail.max.e)
      data$.max.e <- max.e
    if (avail.median.c)
      data$.median.c <- median.c
    if (avail.q1.c)
      data$.q1.c <- q1.c
    if (avail.q3.c)
      data$.q3.c <- q3.c
    if (avail.min.c)
      data$.min.c <- min.c
    if (avail.max.c)
      data$.max.c <- max.c
    #
    data$.approx.mean.e <- approx.mean.e
    data$.approx.mean.c <- approx.mean.c
    data$.approx.sd.e <- approx.sd.e
    data$.approx.sd.c <- approx.sd.c
    #
    if (by)
      data$.subgroup <- subgroup
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
    #
    if (!missing.exclude)
      data$.exclude <- exclude
    #
    if (with.cluster)
      data$.id <- data$.cluster <- cluster
    #
    if (usw.common)
      data$.weights.common <- weights.common
    #
    if (usw.random)
      data$.weights.random <- weights.random
  }
  
  
  #
  #
  # (6) Use subset for analysis
  #
  #
  if (!missing.subset) {
    n.e <- n.e[subset]
    mean.e <- mean.e[subset]
    sd.e <- sd.e[subset]
    n.c <- n.c[subset]
    mean.c <- mean.c[subset]
    sd.c <- sd.c[subset]
    studlab <- studlab[subset]
    #
    cluster <- cluster[subset]
    exclude <- exclude[subset]
    #
    weights.common <- weights.common[subset]
    weights.random <- weights.random[subset]
    #
    if (avail.median.e)
      median.e <- median.e[subset]
    if (avail.q1.e)
      q1.e <- q1.e[subset]
    if (avail.q3.e)
      q3.e <- q3.e[subset]
    if (avail.min.e)
      min.e <- min.e[subset]
    if (avail.max.e)
      max.e <- max.e[subset]
    if (avail.median.c)
      median.c <- median.c[subset]
    if (avail.q1.c)
      q1.c <- q1.c[subset]
    if (avail.q3.c)
      q3.c <- q3.c[subset]
    if (avail.min.c)
      min.c <- min.c[subset]
    if (avail.max.c)
      max.c <- max.c[subset]
    #
    approx.mean.e <- approx.mean.e[subset]
    approx.mean.c <- approx.mean.c[subset]
    approx.sd.e <- approx.sd.e[subset]
    approx.sd.c <- approx.sd.c[subset]
    #
    if (by)
      subgroup <- subgroup[subset]
    #
    if (missing.subgroup & is.pairwise & by) {
      if (length(unique(subgroup)) == 1) {
        by <- FALSE
        #
        if (missing.complab)
          complab <- unique(subgroup)
        #
        subgroup <- NULL
        #
        if (keepdata)
          data$.subgroup <- NULL
        #
        if (missing.overall)
          overall <- TRUE
        if (missing.overall.hetstat)
          overall.hetstat <- TRUE
      }
    }
  }
  #
  # Determine total number of studies
  #
  k.all <- length(n.e)
  #
  if (k.all == 0)
    stop("No studies to combine in meta-analysis.")
  #
  # No meta-analysis for a single study
  #
  if (k.all == 1) {
    common <- FALSE
    random <- FALSE
    prediction <- FALSE
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  #
  # Check variable values
  #
  chknumeric(n.e)
  chknumeric(mean.e)
  chknumeric(sd.e)
  chknumeric(n.c)
  chknumeric(mean.c)
  chknumeric(sd.c)
  #
  if (avail.median.e)
    chknumeric(median.e)
  if (avail.q1.e)
    chknumeric(q1.e)
  if (avail.q3.e)
    chknumeric(q3.e)
  if (avail.min.e)
    chknumeric(min.e)
  if (avail.max.e)
    chknumeric(max.e)
  if (avail.median.c)
    chknumeric(median.c)
  if (avail.q1.c)
    chknumeric(q1.c)
  if (avail.q3.c)
    chknumeric(q3.c)
  if (avail.min.c)
    chknumeric(min.c)
  if (avail.max.c)
    chknumeric(max.c)
  #
  # Recode integer as numeric:
  #
  n.e    <- int2num(n.e)
  mean.e <- int2num(mean.e)
  sd.e   <- int2num(sd.e)
  n.c    <- int2num(n.c)
  mean.c <- int2num(mean.c)
  sd.c   <- int2num(sd.c)
  #
  if (avail.median.e)
    median.e <- int2num(median.e)
  if (avail.q1.e)
    q1.e <- int2num(q1.e)
  if (avail.q3.e)
    q3.e <- int2num(q3.e)
  if (avail.min.e)
    min.e <- int2num(min.e)
  if (avail.max.e)
    max.e <- int2num(max.e)
  if (avail.median.c)
    median.c <- int2num(median.c)
  if (avail.q1.c)
    q1.c <- int2num(q1.c)
  if (avail.q3.c)
    q3.c <- int2num(q3.c)
  if (avail.min.c)
    min.c <- int2num(min.c)
  if (avail.max.c)
    max.c <- int2num(max.c)
  #
  if (by) {
    chkmiss(subgroup)
    #
    if (missing.subgroup.name & is.null(subgroup.name)) {
      if (!missing.subgroup)
        subgroup.name <- byvarname("subgroup", mc)
      else if (!missing.byvar)
        subgroup.name <- byvarname("byvar", mc)
    }
  }
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  
  
  #
  #
  # (7) Calculate means from other information
  #
  #
  
  if (!avail.approx.mean.e) {
    #
    # (a) Use IQR and range
    #
    sel.NA.e <- is.na(mean.e)
    if (any(sel.NA.e) &
        avail.median.e & avail.q1.e & avail.q3.e & avail.min.e & avail.max.e) {
      #
      j <- sel.NA.e & !is.na(median.e) & !is.na(q1.e) & !is.na(q3.e) &
        !is.na(min.e) & !is.na(max.e)
      approx.mean.e[j] <- "iqr.range"
      #
      mean.e[j] <- mean_sd_iqr_range(n.e[j], median.e[j], q1.e[j], q3.e[j],
                                     min.e[j], max.e[j], method.mean)$mean
    }
    #
    # (b) Use IQR
    #
    sel.NA.e <- is.na(mean.e)
    if (any(sel.NA.e) &
        avail.median.e & avail.q1.e & avail.q3.e) {
      #
      j <- sel.NA.e & !is.na(median.e) & !is.na(q1.e) & !is.na(q3.e)
      approx.mean.e[j] <- "iqr"
      mean.e[j] <- mean_sd_iqr(n.e[j], median.e[j], q1.e[j], q3.e[j],
                               method.mean)$mean
    }
    #
    # (c) Use range
    #
    sel.NA.e <- is.na(mean.e)
    if (any(sel.NA.e) &
        avail.median.e & avail.min.e & avail.max.e) {
      #
      j <- sel.NA.e & !is.na(median.e) & !is.na(min.e) & !is.na(max.e)
      approx.mean.e[j] <- "range"
      mean.e[j] <- mean_sd_range(n.e[j], median.e[j], min.e[j], max.e[j],
                                 method.mean)$mean
    }
  }
  else {
    j <- 0
    for (i in approx.mean.e) {
      j <- j + 1
      #
      if (i == "iqr.range") {
        mean.e[j] <- mean_sd_iqr_range(n.e[j], median.e[j], q1.e[j], q3.e[j],
                                       min.e[j], max.e[j], method.mean)$mean
      }
      else if (i == "iqr") {
        mean.e[j] <- mean_sd_iqr(n.e[j], median.e[j], q1.e[j], q3.e[j],
                                 method.mean)$mean
      }
      else if (i == "range") {
        mean.e[j] <- mean_sd_range(n.e[j], median.e[j], min.e[j], max.e[j],
                                   method.mean)$mean
      }
    }
  }
  #
  if (!avail.approx.mean.c) {
    #
    # (a) Use IQR and range
    #
    sel.NA.c <- is.na(mean.c)
    if (any(sel.NA.c) &
        avail.median.c & avail.q1.c & avail.q3.c & avail.min.c & avail.max.c) {
      #
      j <- sel.NA.c & !is.na(median.c) & !is.na(q1.c) & !is.na(q3.c) &
        !is.na(min.c) & !is.na(max.c)
      approx.mean.c[j] <- "iqr.range"
      #
      mean.c[j] <- mean_sd_iqr_range(n.c[j], median.c[j], q1.c[j], q3.c[j],
                                     min.c[j], max.c[j], method.mean)$mean
    }
    #
    # (b) Use IQR
    #
    sel.NA.c <- is.na(mean.c)
    if (any(sel.NA.c) &
        avail.median.c & avail.q1.c & avail.q3.c) {
      #
      j <- sel.NA.c & !is.na(median.c) & !is.na(q1.c) & !is.na(q3.c)
      approx.mean.c[j] <- "iqr"
      mean.c[j] <- mean_sd_iqr(n.c[j], median.c[j], q1.c[j], q3.c[j],
                               method.mean)$mean
    }
    #
    # (c) Use range
    #
    sel.NA.c <- is.na(mean.c)
    if (any(sel.NA.c) &
        avail.median.c & avail.min.c & avail.max.c) {
      #
      j <- sel.NA.c & !is.na(median.c) & !is.na(min.c) & !is.na(max.c)
      approx.mean.c[j] <- "range"
      mean.c[j] <- mean_sd_range(n.c[j], median.c[j], min.c[j], max.c[j],
                                 method.mean)$mean
    }
  }
  else {
    j <- 0
    for (i in approx.mean.c) {
      j <- j + 1
      #
      if (i == "iqr.range") {
        mean.c[j] <- mean_sd_iqr_range(n.c[j], median.c[j], q1.c[j], q3.c[j],
                                       min.c[j], max.c[j], method.mean)$mean
      }
      else if (i == "iqr") {
        mean.c[j] <- mean_sd_iqr(n.c[j], median.c[j], q1.c[j], q3.c[j],
                                 method.mean)$mean
      }
      else if (i == "range") {
        mean.c[j] <- mean_sd_range(n.c[j], median.c[j], min.c[j], max.c[j],
                                   method.mean)$mean
      }
    }
  }
  
  
  #
  #
  # (8) Calculate standard deviation from other information
  #
  #
  
  if (!avail.median.e) {
    median.e.sd <- mean.e
    avail.median.e <- TRUE
    export.median.e <- FALSE
  }
  else {
    median.e.sd <- median.e
    median.e.sd[is.na(median.e.sd)] <- mean.e[is.na(median.e.sd)]
    export.median.e <- TRUE
  }
  #
  if (!avail.approx.sd.e) {
    #
    # (a) Use IQR and range
    #
    sel.NA.e <- is.na(sd.e)
    if (any(sel.NA.e) &
        avail.median.e & avail.q1.e & avail.q3.e & avail.min.e & avail.max.e) {
      #
      j <- sel.NA.e & !is.na(median.e.sd) & !is.na(q1.e) & !is.na(q3.e) &
        !is.na(min.e) & !is.na(max.e)
      approx.sd.e[j] <- "iqr.range"
      #
      sd.e[j] <- mean_sd_iqr_range(n.e[j], median.e.sd[j], q1.e[j], q3.e[j],
                                   min.e[j], max.e[j],
                                   method.sd = method.sd)$sd
    }
    #
    # (b) Use IQR
    #
    sel.NA.e <- is.na(sd.e)
    if (any(sel.NA.e) &
        avail.median.e & avail.q1.e & avail.q3.e) {
      #
      j <- sel.NA.e & !is.na(median.e.sd) & !is.na(q1.e) & !is.na(q3.e)
      approx.sd.e[j] <- "iqr"
      sd.e[j] <- mean_sd_iqr(n.e[j], median.e.sd[j], q1.e[j], q3.e[j])$sd
    }
    #
    # (c) Use range
    #
    sel.NA.e <- is.na(sd.e)
    if (any(sel.NA.e) &
        avail.median.e & avail.min.e & avail.max.e) {
      #
      j <- sel.NA.e & !is.na(median.e.sd) & !is.na(min.e) & !is.na(max.e)
      approx.sd.e[j] <- "range"
      sd.e[j] <- mean_sd_range(n.e[j], median.e.sd[j], min.e[j], max.e[j])$sd
    }
  }
  else {
    j <- 0
    for (i in approx.sd.e) {
      j <- j + 1
      #
      if (i == "iqr.range") {
        sd.e[j] <- mean_sd_iqr_range(n.e[j], median.e.sd[j], q1.e[j], q3.e[j],
                                     min.e[j], max.e[j],
                                     method.sd = method.sd)$sd
      }
      else if (i == "iqr") {
        sd.e[j] <- mean_sd_iqr(n.e[j], median.e.sd[j], q1.e[j], q3.e[j])$sd
      }
      else if (i == "range") {
        sd.e[j] <- mean_sd_range(n.e[j], median.e.sd[j], min.e[j], max.e[j])$sd
      }
    }
  }
  #
  if (!avail.median.c) {
    median.c.sd <- mean.c
    avail.median.c <- TRUE
    export.median.c <- FALSE
  }
  else {
    median.c.sd <- median.c
    median.c.sd[is.na(median.c.sd)] <- mean.c[is.na(median.c.sd)]
    export.median.c <- TRUE
  }
  #
  if (!avail.approx.sd.c) {
    #
    # (a) Use IQR and range
    #
    sel.NA.c <- is.na(sd.c)
    if (any(sel.NA.c) &
        avail.median.c & avail.q1.c & avail.q3.c & avail.min.c & avail.max.c) {
      #
      j <- sel.NA.c & !is.na(median.c.sd) & !is.na(q1.c) & !is.na(q3.c) &
        !is.na(min.c) & !is.na(max.c)
      approx.sd.c[j] <- "iqr.range"
      #
      sd.c[j] <- mean_sd_iqr_range(n.c[j], median.c.sd[j], q1.c[j], q3.c[j],
                                   min.c[j], max.c[j],
                                   method.sd = method.sd)$sd
    }
    #
    # (b) Use IQR
    #
    sel.NA.c <- is.na(sd.c)
    if (any(sel.NA.c) &
        avail.median.c & avail.q1.c & avail.q3.c) {
      #
      j <- sel.NA.c & !is.na(median.c.sd) & !is.na(q1.c) & !is.na(q3.c)
      approx.sd.c[j] <- "iqr"
      sd.c[j] <- mean_sd_iqr(n.c[j], median.c.sd[j], q1.c[j], q3.c[j])$sd
    }
    #
    # (c) Use range
    #
    sel.NA.c <- is.na(sd.c)
    if (any(sel.NA.c) &
        avail.median.c & avail.min.c & avail.max.c) {
      #
      j <- sel.NA.c & !is.na(median.c.sd) & !is.na(min.c) & !is.na(max.c)
      approx.sd.c[j] <- "range"
      sd.c[j] <- mean_sd_range(n.c[j], median.c.sd[j], min.c[j], max.c[j])$sd
    }
  }
  else {
    j <- 0
    for (i in approx.sd.c) {
      j <- j + 1
      #
      if (i == "iqr.range") {
        sd.c[j] <- mean_sd_iqr_range(n.c[j], median.c[j], q1.c[j], q3.c[j],
                                     min.c[j], max.c[j],
                                     method.sd = method.sd)$sd
      }
      else if (i == "iqr") {
        sd.c[j] <- mean_sd_iqr(n.c[j], median.c.sd[j], q1.c[j], q3.c[j])$sd
      }
      else if (i == "range") {
        sd.c[j] <- mean_sd_range(n.c[j], median.c.sd[j], min.c[j], max.c[j])$sd
      }
    }
  }
  #
  if (keepdata) {
    if (!isCol(data, ".subset")) {
      data$.sd.e <- sd.e
      data$.mean.e <- mean.e
      data$.sd.c <- sd.c
      data$.mean.c <- mean.c
      #
      data$.approx.mean.e <- approx.mean.e
      data$.approx.mean.c <- approx.mean.c
      data$.approx.sd.e <- approx.sd.e
      data$.approx.sd.c <- approx.sd.c
    }
    else {
      data$.sd.e[data$.subset] <- sd.e
      data$.mean.e[data$.subset] <- mean.e
      data$.sd.c[data$.subset] <- sd.c
      data$.mean.c[data$.subset] <- mean.c
      #
      data$.approx.mean.e[data$.subset] <- approx.mean.e
      data$.approx.mean.c[data$.subset] <- approx.mean.c
      data$.approx.sd.e[data$.subset] <- approx.sd.e
      data$.approx.sd.c[data$.subset] <- approx.sd.c
    }
  }
  
  
  #
  #
  # (9) Calculate results for individual studies
  #
  #
  npn.n <- npn(n.e) | npn(n.c)
  #
  N <- n.e + n.c
  if (sm == "MD" | sm == "ROM")
    var.pooled <- ((n.e - 1) * sd.e^2 + (n.c - 1) * sd.c^2) / (N - 2)
  #
  if (any(npn.n) & warn)
    warning("Note, studies with non-positive values for n.e and / or n.c get no weight in meta-analysis.")
  #
  if (sm == "MD") {
    TE <- ifelse(npn.n, NA, mean.e - mean.c)
    #
    if (pooledvar)
      seTE <-
        ifelse(npn.n, NA, sqrt(var.pooled * (1 / n.e + 1 / n.c)))
    else
      seTE <-
        ifelse(npn.n, NA, sqrt(sd.e^2 / n.e + sd.c^2 / n.c))
    #
    seTE[is.na(TE)] <- NA
    #
    if (method.ci == "t")
      ci.study <- ci(TE, seTE, level = level, df = n.e + n.c - 2)
  }
  else if (sm == "SMD") {
    J <- function(x)
      exp(lgamma(x / 2) - log(sqrt(x / 2)) - lgamma((x - 1) / 2))
    #
    K <- function(x) 1 - (x - 2) / (x * J(x)^2)
    #
    if (method.smd %in% c("Hedges", "Cohen"))
      S.within <- sqrt(((n.e - 1) * sd.e^2 + (n.c - 1) * sd.c^2) / (N - 2))
    else
      S.within <- if (sd.glass == "control") sd.c else sd.e
    #
    smd <- ifelse(npn.n, NA, (mean.e - mean.c) / S.within)
    #
    if (method.smd == "Cohen") {
      #
      # Borenstein et al. (2009), p. 26-27;
      # White and Thomas (2005), p. 143
      #
      TE <- smd
      if (exact.smd) {
        J <- function(x)
          exp(lgamma(x / 2) - log(sqrt(x / 2)) - lgamma((x - 1) / 2))
        #
        K <- function(x) 1 - (x - 2) / (x * J(x)^2)
        #
        seTE <-
          ifelse(npn.n, NA,
                 sqrt(1 / n.e + 1 / n.c + (J(N - 2) * smd)^2 * K(N - 2)))
      }
      else
        seTE <-
          ifelse(npn.n, NA, sqrt(1 / n.e + 1 / n.c + TE^2 / (2 * N)))
    }
    else if (method.smd == "Hedges") {
      #
      # Hedges and Olkin (1985); White and Thomas (2005), p. 143;
      # formulae used in RevMan 5 (exact.smd = FALSE)
      #
      if (exact.smd) {
        J <- function(x)
          exp(lgamma(x / 2) - log(sqrt(x / 2)) - lgamma((x - 1) / 2))
        #
        K <- function(x) 1 - (x - 2) / (x * J(x)^2)
      }
      else {
        J <- function(x) 1 - 3 / (4 * x - 1)
        K <- function(x) 1 / (2 * (x - 1.94))
      }
      #
      TE   <- J(N - 2) * smd
      seTE <-
        ifelse(npn.n, NA, sqrt(1 / n.e + 1 / n.c + TE^2 * K(N - 2)))
    }
    else if (method.smd == "Glass") {
      #
      # see Cooper & Hedges (1994), p. 238
      #
      n.g  <- if (sd.glass == "control") n.c else n.e
      #
      TE <- smd
      seTE <-
        ifelse(npn.n, NA, sqrt(1 / n.e + 1 / n.c + TE^2 / (2 * (n.g - 1))))
    }
    #
    seTE[is.na(TE)] <- NA
  }
  #
  else if (sm == "ROM") {
    npn.mean <- npn(mean.e) | npn(mean.c)
    #
    if (any(npn.mean) & warn)
      warning("Note, studies with negative or zero means get no weight in meta-analysis.")
    
    TE <- ifelse(npn.n | npn.mean, NA, log(mean.e / mean.c))
    #
    if (pooledvar)
      seTE <- ifelse(npn.n, NA,
                     sqrt(var.pooled * (1 / (n.e * mean.e^2) + 1 / (n.c * mean.c^2))))
    else
      seTE <- ifelse(npn.n | npn.mean, NA,
                     sqrt(sd.e^2 / (n.e * mean.e^2) + sd.c^2 / (n.c * mean.c^2)))
    #
    seTE[is.na(TE)] <- NA
  }
  #
  # Studies with non-positive variance get zero weight in meta-analysis
  #
  sel <- sd.e <= 0 | sd.c <= 0
  #
  if (any(sel, na.rm = TRUE) & warn)
    warning("Note, studies with non-positive values for sd.e or sd.c get no weight in meta-analysis.")
  #
  seTE[sel] <- NA
  #
  if (sm == "SMD")
    TE[sel] <- NA
  
  
  #
  #
  # (10) Additional checks for three-level model
  #
  #
  three.level <- FALSE
  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (with.cluster &&
      length(unique(cluster[sel.ni])) != length(cluster[sel.ni]))
    three.level <- TRUE
  #
  if (three.level) {
    chkmlm(method.tau, missing.method.tau, method.predict)
    #
    common <- FALSE
    #
    if (!(method.tau %in% c("REML", "ML")))
      method.tau <- "REML"
  }
  
  
  #
  #
  # (11) Do meta-analysis
  #
  #
  m <- metagen(TE, seTE, studlab,
               exclude = if (missing.exclude) NULL else exclude,
               cluster = cluster, rho = rho,
               #
               weights.common = weights.common,
               weights.random = weights.random,
               #
               sm = sm,
               level = level,
               #
               common = common,
               random = random,
               overall = overall,
               overall.hetstat = overall.hetstat,
               prediction = prediction,
               #
               method.tau = method.tau, method.tau.ci = method.tau.ci,
               level.hetstat = level.hetstat,
               tau.preset = tau.preset,
               TE.tau = TE.tau,
               tau.common = FALSE,
               detail.tau = detail.tau,
               #
               method.I2 = method.I2,
               #
               level.ma = level.ma,
               method.common.ci = method.common.ci,
               method.random.ci = method.random.ci,
               adhoc.hakn.ci = adhoc.hakn.ci,
               #
               level.predict = level.predict,
               method.predict = method.predict,
               adhoc.hakn.pi = adhoc.hakn.pi,
               seed.predict = seed.predict,
               #
               method.bias = method.bias,
               #
               backtransf = backtransf,
               #
               text.common = text.common, text.random = text.random,
               text.predict = text.predict,
               text.w.common = text.w.common, text.w.random = text.w.random,
               #
               title = title, complab = complab, outclab = outclab,
               #
               label.e = label.e, label.c = label.c,
               label.left = label.left, label.right = label.right,
               col.label.left = col.label.left,
               col.label.right = col.label.right,
               #
               keepdata = FALSE,
               warn = warn,
               #
               control = control)
  #
  # Estimate common tau-squared across subgroups
  #
  if (by & tau.common)
    hcc <- hetcalc(TE, seTE, method.tau, "", TE.tau,
                   method.I2, level.hetstat, subgroup, control)
  
  
  #
  #
  # (11) Generate R object
  #
  #
  res <- list(n.e = n.e, mean.e = mean.e, sd.e = sd.e,
              n.c = n.c, mean.c = mean.c, sd.c = sd.c,
              pooledvar = pooledvar,
              method.smd = method.smd, sd.glass = sd.glass,
              exact.smd = exact.smd,
              method.ci = method.ci,
              method.mean = method.mean, method.sd = method.sd,
              n.e.pooled = sum(n.e, na.rm = TRUE),
              n.c.pooled = sum(n.c, na.rm = TRUE))
  #
  if (export.median.e)
    res$median.e <- median.e
  if (avail.q1.e)
    res$q1.e <- q1.e
  if (avail.q3.e)
    res$q3.e <- q3.e
  if (avail.min.e)
    res$min.e <- min.e
  if (avail.max.e)
    res$max.e <- max.e
  #
  if (export.median.c)
    res$median.c <- median.c
  if (avail.q1.c)
    res$q1.c <- q1.c
  if (avail.q3.c)
    res$q3.c <- q3.c
  if (avail.min.c)
    res$min.c <- min.c
  if (avail.max.c)
    res$max.c <- max.c
  #
  res$approx.sd.e <- approx.sd.e
  res$approx.sd.c <- approx.sd.c
  res$approx.mean.e <- approx.mean.e
  res$approx.mean.c <- approx.mean.c
  #
  # Add meta-analysis results
  # (after removing unneeded list elements)
  #
  m$n.e <- NULL
  m$n.c <- NULL
  m$pscale <- NULL
  m$irscale <- NULL
  m$irunit <- NULL
  m$method.ci <- NULL
  m$method.mean <- NULL
  m$approx.TE <- NULL
  m$approx.seTE <- NULL
  #
  res <- c(res, m)
  #
  # Add data
  #
  if (is.pairwise | data.pairwise) {
    res$pairwise <- TRUE
    res$k.study <- length(unique(res$studlab[!is.na(res$TE)]))
  }
  #
  res$call <- match.call()
  #
  if (keepdata) {
    res$data <- data
    if (!missing.subset)
      res$subset <- subset
  }
  #
  if (method.ci == "t") {
    res$lower <- ci.study$lower
    res$upper <- ci.study$upper
    res$statistic <- ci.study$statistic
    res$pval <- ci.study$p
    res$df <- ci.study$df
  }
  else if (!is.null(res$df) && all(is.na(res$df)))
    res$df <- NULL
  #
  if (all(res$approx.mean.e == "")) {
    res$approx.mean.e <- NULL
    res$data$.approx.mean.e <- NULL
  }
  if (all(res$approx.sd.e == "")) {
    res$approx.sd.e <- NULL
    res$data$.approx.sd.e <- NULL
  }
  if (all(res$approx.mean.c == "")) {
    res$approx.mean.c <- NULL
    res$data$.approx.mean.c <- NULL
  }
  if (all(res$approx.sd.c == "")) {
    res$approx.sd.c <- NULL
    res$data$.approx.sd.c <- NULL
  }
  #
  class(res) <- c(fun, "meta")
  #
  # Add results from subgroup analysis
  #
  if (by) {
    res$subgroup <- subgroup
    res$subgroup.name <- subgroup.name
    res$print.subgroup.name <- print.subgroup.name
    res$sep.subgroup <- sep.subgroup
    res$test.subgroup <- test.subgroup
    res$prediction.subgroup <- prediction.subgroup
    res$tau.common <- tau.common
    #
    if (!tau.common) {
      res <- c(res, subgroup(res, seed = seed.predict.subgroup))
      if (res$three.level)
        res <- setNA3(res)
    }
    else if (!is.null(tau.preset))
      res <-
        c(res, subgroup(res, tau.preset, seed = seed.predict.subgroup))
    else {
      if (res$three.level)
        res <- c(res, subgroup(res, NULL,
                               factor(res$subgroup, bylevs(res$subgroup))))
      else
        res <-
          c(res, subgroup(res, hcc$tau.resid, seed = seed.predict.subgroup))
    }
    #
    if (tau.common && is.null(tau.preset))
      res <- addHet(res, hcc)
    #
    res$n.w <- NULL
    res$event.w <- NULL
    #
    res$n.harmonic.mean.w <- NULL
    #
    res$event.e.w <- NULL
    res$event.c.w <- NULL
    #
    res$time.e.w <- NULL
    res$time.c.w <- NULL
    res$t.harmonic.mean.w <- NULL
    #
    res <- setNAwithin(res, res$three.level)
  }
  #
  # Unset some variables
  #
  if (sm == "SMD") {
    res$pooledvar <- NA
    #
    if (method.smd != "Glass")
      res$sd.glass <- ""
    else
      res$exact.smd <- NA
  }
  else {
    res$method.smd <- ""
    res$sd.glass <- ""
    res$exact.smd <- NA
  }
  #
  if (is.null(res$approx.mean.e) & is.null(res$approx.mean.c))
    res$method.mean <- ""
  #
  if (is.null(res$approx.sd.e) & is.null(res$approx.sd.c))
    res$method.sd <- ""
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  class(res) <- c(fun, "meta")
  
  
  res
}

#' Cochrane review: detailed summary of meta-analyses
#' 
#' @description
#' Calculate and print a detailed summary of all meta-analyses in a
#' Cochrane review.
#' 
#' @aliases summary.rm5 summary.cdir 
#' 
#' @param object An object of class \code{rm5} or \code{cdir}.
#' @param x An object of class \code{summary.rm5} or
#'   \code{summary.cdir}.
#' @param comp.no Comparison number.
#' @param outcome.no Outcome number.
#' @param ... Additional arguments (passed on to \code{metacr}).
#' 
#' @details
#' This function can be used to rerun all or selected meta-analyses of
#' a Cochrane Review of interventions (Higgins et al., 2023).
#' 
#' The R function \code{\link{metacr}} is called internally.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{summary.meta}}, \code{\link{metacr}},
#'   \code{\link{read.rm5}}, \code{\link{read.cdir}},
#'   \code{\link{metabias.rm5}}, \code{\link{metabias.cdir}}
#' 
#' @references
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
#' VA (editors) (2023):
#' \emph{Cochrane Handbook for Systematic Reviews of Interventions
#'   Version 6.4 (updated August 2023)}.
#' Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}
#' 
#' @examples
#' # Locate export data file "Fleiss1993_CR.csv"
#' # in sub-directory of package "meta"
#' #
#' filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
#' Fleiss1993_CR <- read.rm5(filename)
#' 
#' # Print summary results for all meta-analysis
#' #
#' summary(Fleiss1993_CR)
#' 
#' # Print summary results only for second outcome of first comparison
#' #
#' summary(Fleiss1993_CR, comp.no = 1, outcome.no = 2)
#' 
#' @method summary rm5
#' @export

summary.rm5 <- function(object, comp.no, outcome.no, ...) {
  
  chkclass(object, "rm5")
  #
  if (missing(comp.no))
    comp.no <- unique(object$comp.no)
  #
  res <- list()
  #
  n <- 1
  #
  for (i in comp.no) {
    if (missing(outcome.no))
      jj <- unique(object$outcome.no[object$comp.no == i])
    else
      jj <- outcome.no
    #
    for (j in jj) {
      res[[n]] <- summary(metacr(object, i, j, ...))
      #
      n <- n + 1
    }
  }
  #
  class(res) <- "summary.rm5"
  
  res
}





#' @rdname summary.rm5
#' @method summary cdir
#' @export

summary.cdir <- function(object, comp.no, outcome.no, ...) {
  
  chkclass(object, "cdir")
  #
  if (missing(comp.no))
    comp.no <- unique(object$data$comp.no)
  #
  res <- list()
  #
  n <- 1
  #
  for (i in comp.no) {
    if (missing(outcome.no))
      jj <- unique(object$data$outcome.no[object$data$comp.no == i])
    else
      jj <- outcome.no
    #
    for (j in jj) {
      res[[n]] <- summary(metacr(object, i, j, ...))
      #
      n <- n + 1
    }
  }
  #
  class(res) <- "summary.cdir"
  
  res
}





#' @rdname summary.rm5
#' @method print summary.rm5
#' @export

print.summary.rm5 <- function(x, ...) {
  
  chkclass(x, "summary.rm5")
  #
  n <- 1
  #
  for (i in 1:length(x)) {
    if (n > 1)
      cat("\n*****\n\n")
    #
    print(x[[i]])
    #
    n <- n + 1
  }
  
  invisible(NULL)
}





#' @rdname summary.rm5
#' @method print summary.cdir
#' @export

print.summary.cdir <- function(x, ...) {
  
  chkclass(x, "summary.cdir")
  #
  n <- 1
  #
  for (i in 1:length(x)) {
    if (n > 1)
      cat("\n*****\n\n")
    #
    print(x[[i]])
    #
    n <- n + 1
  }
  
  invisible(NULL)
}

# Auxiliary functions
#
# Package: meta
# Author: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
# License: GPL (>= 2)
#

allNA <- function(x)
  all(is.na(x))

bylevs <- function(x) {
  if (is.factor(x))
    res <- levels(factor(x))
  else
    res <- unique(x)
  res
}

byvarname <- function(argname, matchcall) {
  #
  # Determine name of subgroup variable
  #
  res <- as.character(matchcall[[match(argname, names(matchcall))]])
  #
  if (length(res) > 1 & res[1] == "$")
    res <- res[length(res)]
  #
  if (length(res) == 0 || length(res) > 1)
    res <- "subgroup"
  #
  res
}

catch <- function(argname, matchcall, data, encl)
  eval(matchcall[[match(argname, names(matchcall))]], data, enclos = encl)

catch2 <- function(x, varname, return = NULL, fromobject = FALSE) {
  if (!is.null(x[[varname]]) & fromobject)
    return(x[[varname]])
  else if (isCol(x$data, paste0(".", varname)))
    return(x$data[[paste0(".", varname)]])
  else
    return(return)
}

int2num <- function(x) {
  #
  # Convert integer to numeric
  #
  if (is.integer(x))
    res <- as.numeric(x)
  else
    res <- x
  #
  res
}

npn <- function(x) {
  #
  # Check for non-positive values in vector
  #
  selNA <- is.na(x)
  res <- selNA
  if (sum(!selNA) > 0)
    x[!selNA] <- x[!selNA] <= 0
  #
  res
}

replaceNULL <- function(x, replace = NA) {
  if (is.null(x))
    return(replace)
  x
}

replaceNA <- function(x, replace = NA) {
  if (is.null(x))
    return(x)
  else
    x[is.na(x)] <- replace
  x
}

replaceVal <- function(x, old, new) {
  if (is.null(x))
    return(x)
  else
    x[x == old] <- new
  x
}

warnarg <- function(x, y, fun, cl, otherarg) {
  if (x %in% y)
    if (!missing(cl))
      warning("Argument '", x, "' has been removed from R function ", fun,
              ".\nThis argument can be used in R function ", cl, ".",
              call. = FALSE)
    else if (!missing(otherarg))
      warning("Argument '", x, "' has been replaced by argument '", otherarg,
              "' in R function ", fun, ".\nSee help page of R function ",
              fun, " for information on the use of the new argument.",
              call. = FALSE)
  #
  invisible(NULL)
}

catchvar <- function(varname, x, mf) {
  res <- NULL
  error <-
    try(res <- eval(mf[[match(varname, names(mf))]],
                    x,
                    enclos = sys.frame(sys.parent())),
        silent = TRUE)
  #
  if (inherits(error, "try-error")) {
    res <- eval(mf[[match(varname, names(mf))]],
                x$data, enclos = NULL)
  }
  #
  res
}

augment <- function(x, len, fun) {
  if (length(x) > 1)
    chklength(x, len, fun)
  else
    x <- rep(x, len)
  x
}

stoponly <- function(arg, val, func)
  stop("Argument ", arg, " =\"", val, "\"",
       " only defined for meta-analysis conducted with ",
       func, ".",
       call. = FALSE)

deprecated <- function(newvar, newmiss, args, old, warn = TRUE) {
  #
  new <- deparse(substitute(newvar))
  #
  if (length(args) == 0)
    return(newvar)
  #
  if (is.list(args[[1]]))
    args <- args[[1]]
  #
  additional.arguments <- names(args)
  #
  if (!is.na(charmatch(old, additional.arguments)))
    if (!newmiss) {
      if (warn)
        warning("Deprecated argument '", old, "' ignored as ",
                "'", new, "' is also provided.",
                call. = FALSE)
      return(newvar)
    }
    else {
      if (warn)
        warning("Use argument '", new, "' instead of '",
                old, "' (deprecated).",
                call. = FALSE)
      return(args[[charmatch(old, additional.arguments)]])
    }
  else
    return(newvar)
}

deprecated2 <- function(newvar, newmiss, oldvar, oldmiss, warn = TRUE,
                        oldtxt = NULL) {
  #
  new <- deparse(substitute(newvar))
  if (is.null(oldtxt))
    oldtxt <- deparse(substitute(oldvar))
  #
  if (newmiss & oldmiss)
    return(newvar)
  else if (!newmiss & oldmiss)
    return(newvar)
  else if (!newmiss & !oldmiss) {
    if (warn)
      warning("Deprecated argument '", oldtxt, "' ignored as ",
              "'", new, "' is also provided.",
              call. = FALSE)
    return(newvar)
  }
  else if (newmiss & !oldmiss) {
    if (warn)
      warning("Use argument '", new, "' instead of '",
              oldtxt, "' (deprecated).",
              call. = FALSE)
    return(oldvar)
  }
}

runNN <- function(func, args, warn = TRUE) {
  args <- args[!sapply(args, is.null)]
  if (warn)
    do.call(func, args)
  else
    suppressWarnings(do.call(func, args))
}

setNA3 <- function(x) {
  res <- x
  #
  res$Q.b.random <- NA
  res$df.Q.b <- NA
  res$df.Q.b.random <- NA
  res$pval.Q.b.random <- NA
  #
  res
}

mismatch <- function(x, y, var) {
  x <- x$data[[var]]
  y <- y$data[[var]]
  #
  bothnull <- is.null(x) & is.null(y)
  #
  if (bothnull)
    return(FALSE)
  else {
    if (!is.null(x) & !is.null(y))
      return(any(x != y, na.rm = TRUE))
    else
      return(TRUE)
  }
}

addHet <- function(x, hcc, within = TRUE) {
  
  res <- x
  #
  if (within) {
    res$Q.w.random <- hcc$Q.resid
    res$df.Q.w.random <- hcc$df.Q.resid
    res$pval.Q.w.random <- hcc$pval.Q.resid
  }
  #
  res$tau2.resid <- hcc$tau2.resid
  res$lower.tau2.resid <- hcc$lower.tau2.resid
  res$upper.tau2.resid <- hcc$upper.tau2.resid
  #
  res$tau.resid <- hcc$tau.resid
  res$lower.tau.resid <- hcc$lower.tau.resid
  res$upper.tau.resid <- hcc$upper.tau.resid
  res$sign.lower.tau.resid <- hcc$sign.lower.tau.resid
  res$sign.upper.tau.resid <- hcc$sign.upper.tau.resid
  #
  res$Q.resid <- hcc$Q.resid
  res$df.Q.resid <- hcc$df.Q.resid
  res$pval.Q.resid <- hcc$pval.Q.resid
  #
  res$H.resid <- hcc$H.resid
  res$lower.H.resid <- hcc$lower.H.resid
  res$upper.H.resid <- hcc$upper.H.resid
  #
  res$I2.resid <- hcc$I2.resid
  res$lower.I2.resid <- hcc$lower.I2.resid
  res$upper.I2.resid <- hcc$upper.I2.resid
  #
  # No confidence interval for tau2.w for three-level model
  #
  if (x$tau.common & x$method.tau.ci %in% c("BJ", "J", "QP")) {
    res$lower.tau2.w[is.na(res$lower.tau2.w)] <- res$lower.tau2.resid
    res$upper.tau2.w[is.na(res$upper.tau2.w)] <- res$upper.tau2.resid
  }
  #
  res
}

backward <- function(x) {
  res <- x
  #
  res$w.fixed <- res$w.common
  res$TE.fixed <- res$TE.common
  res$seTE.fixed <- res$seTE.common
  res$lower.fixed <- res$lower.common
  res$upper.fixed <- res$upper.common
  res$statistic.fixed <- res$statistic.common
  res$pval.fixed <- res$pval.common
  res$zval.fixed <- res$zval.common
  #
  res$text.fixed <- res$text.common
  res$text.w.fixed <- res$text.w.common
  #
  res$df.hakn <- res$df.hakn.ci
  res$seTE.hakn <- res$seTE.hakn.ci
  res$seTE.hakn.adhoc <- res$seTE.hakn.adhoc.ci
  #
  if (!is.null(res$subgroup)) {
    res$byvar <- res$subgroup
    res$bylab <- res$subgroup.name
    res$print.byvar <- res$print.subgroup.name
    res$byseparator <- res$sep.subgroup
    #
    res$TE.fixed.w <- res$TE.common.w
    res$seTE.fixed.w <- res$seTE.common.w
    res$lower.fixed.w <- res$lower.common.w
    res$upper.fixed.w <- res$upper.common.w
    res$statistic.fixed.w <- res$statistic.common.w
    res$pval.fixed.w <- res$pval.common.w
    res$zval.fixed.w <- res$zval.common.w
    res$w.fixed.w <- res$w.common.w
    #
    res$Q.w.fixed <- res$Q.w.common
    res$pval.Q.w.fixed <- res$pval.Q.w.common
    res$Q.b.fixed <- res$Q.b.common
    res$pval.Q.b.fixed <- res$pval.Q.b.common
  }
  #
  if (!is.null(res$three.level) && any(res$three.level))
    res$id <- res$cluster
  #
  res
}

ci2meta <- function(x, ci.c = NULL, ci.r = NULL) {
  res <- x
  #
  if (!is.null(ci.c)) {
    res$TE.common <- ci.c$TE
    res$seTE.common <- ci.c$seTE
    res$lower.common <- ci.c$lower
    res$upper.common <- ci.c$upper
    res$statistic.common <- ci.c$statistic
    res$pval.common <- ci.c$p      
    res$zval.common <- ci.c$statistic
  }
  #
  if (!is.null(ci.r)) {
    res$TE.random <- ci.r$TE
    res$seTE.random <- ci.r$seTE
    res$lower.random <- ci.r$lower
    res$upper.random <- ci.r$upper
    res$statistic.random <- ci.r$statistic
    res$pval.random <- ci.r$p      
    res$zval.random <- ci.r$statistic
  }
  #
  res
}

setNAwithin <- function(x, condition) {
  res <- x
  #
  if (condition) {
    res$Q.w.common <- NA
    res$Q.w.random <- NA
    res$df.Q.w <- NA
    res$pval.Q.w.common <- NA
    res$pval.Q.w.random <- NA
  }
  #
  res
}

extrVar <- function(x, name)
  x[[name]]

# Function only used with MLM, GLMM, LRP, or USW
#
calcPI <- function(x) {
  
  res <- x
  #
  k <- x$k
  TE.random <- x$TE.random[1]
  seTE.random <- x$seTE.random[1]
  method.predict <- x$method.predict
  level.predict <- x$level.predict
  #
  res$df.predict <- res$upper.predict <- res$lower.predict <-
    res$seTE.predict <- NA
  #
  tau2.calc <- if (is.na(sum(res$tau2))) 0 else sum(res$tau2)
  seTE.predict <- sqrt(seTE.random^2 + tau2.calc)
  #
  df.predict <- set_df_predict(method.predict, k)
  #
  ci.p <- ci(TE.random, seTE.predict, level.predict, df = df.predict)
  #
  res$seTE.predict <- rep_len(seTE.predict, length(df.predict))
  res$lower.predict <- ci.p$lower
  res$upper.predict <- ci.p$upper
  res$df.predict <- ci.p$df
  #
  if (length(method.predict) > 1)
    names(res$seTE.predict) <-
      names(res$lower.predict) <- names(res$upper.predict) <-
      names(res$df.predict) <- method.predict
  #
  res
}

runMLM <- function(x, method.tau, method.random.ci, level,
                   warn = TRUE, ...) {
  
  n.methci <- length(method.random.ci)
  #
  list.b <- list(level = 100 * level, method = method.tau,
                 random = as.call(~ 1 | cluster / idx), ...)
  #
  res <- list.r <- vector("list", n.methci)
  #
  for (i in seq_len(n.methci))
    list.r[[i]] <- c(x, list.b,
                     test = ifelse(method.random.ci[i] == "HK", "t", "z"))
  #
  for (i in seq_len(n.methci))
    res[[i]] <- runNN(rma.mv, list.r[[i]], warn = warn)
  #
  res
}

extrMLM <- function(x, k, len, sel,
                    method.random.ci, method.predict,
                    level.ma, level.predict, null.effect) {
  
  #
  # Random effects model(s)
  #
  res <- list(tau2 = NA, tau = NA, se.tau2 = NA)
  #
  res$k <- k
  res$method.predict <- method.predict
  res$level.predict <- level.predict
  #
  if (k > 1) {
    res$tau2 <- x[[1]]$sigma2
    res$tau <- sqrt(res$tau2)
  }
  #
  res$TE.random   <- as.numeric(x[[1]]$b)
  res$seTE.random <- sapply(x, extrVar, "se")
  #
  res$w.random <- rep_len(NA, len)
  res$w.random[sel] <- weights(x[[1]], type = "rowsum")
  res$w.random[is.na(res$w.random)] <- 0
  #
  res$lower.random <- sapply(x, extrVar, "ci.lb")
  res$upper.random <- sapply(x, extrVar, "ci.ub")
  res$statistic.random <- sapply(x, extrVar, "zval")
  res$pval.random <-
    ci(res$TE.random, res$seTE.random,
       level = level.ma,
       null.effect = null.effect,
       df = ifelse(method.random.ci == "HK", k - 1, Inf))$p
  res$zval.random <- res$statistic.random
  #
  if (length(method.random.ci) > 1)
    names(res$seTE.random) <-
      names(res$lower.random) <- names(res$upper.random) <-
      names(res$statistic.random) <- names(res$pval.random) <-
      names(res$zval.random) <- method.random.ci
  #
  # Prediction interval
  #
  res <- calcPI(res)
  #
  res$k <- res$method.predict <- res$level.predict <- NULL
  #
  res
}

runUSW <- function(x, method.tau, method.random.ci, level,
                   warn = TRUE, ...) {
  
  if (method.tau %in% c("FE", "CE")) {
    n.methci <- 1
    #
    list.b <- list(level = 100 * level, method = method.tau, ...)
    #
    res <- list.c <- vector("list", n.methci)
    #
    for (i in seq_len(n.methci)) {
      list.c[[i]] <- c(x, list.b, test = "z")
      #
      res[[i]] <- runNN(rma.uni, list.c[[i]], warn = warn)
    }
  }
  else {
    n.methci <- length(method.random.ci)
    #
    list.b <- list(level = 100 * level, method = method.tau, ...)
    #
    res <- list.r <- vector("list", n.methci)
    #
    for (i in seq_len(n.methci)) {
      list.r[[i]] <- c(x, list.b,
                       test = ifelse(method.random.ci[i] == "HK", "t", "z"))
      #  
      res[[i]] <- runNN(rma.uni, list.r[[i]], warn = warn)
    }
  }
  #
  res
}

extrUSW <- function(x, method.tau, null.effect,
                    k, len, sel,
                    method.random.ci, method.predict,
                    level.ma, level.predict) {
  if (method.tau %in% c("FE", "CE")) {
    res <- list(
      w.common = x[[1]]$weights,
      TE.common = as.numeric(x[[1]]$b),
      seTE.common = as.numeric(x[[1]]$se),
      lower.common = x[[1]]$ci.lb,
      upper.common = x[[1]]$ci.ub)
    #
    ci.c <- ci(res$TE.common - null.effect, res$seTE.common,
         null.effect = null.effect)
    #
    res$statistic.common <- ci.c$statistic
    res$pval.common <- ci.c$p
    res$zval.common <- res$statistic.common
  }
  else {
    #
    # Random effects model(s)
    #
    res <- list(tau2 = NA, tau = NA, se.tau2 = NA)
    #
    res$k <- k
    res$method.predict <- method.predict
    res$level.predict <- level.predict
    #
    if (k > 1) {
      res$tau2 <- x[[1]]$tau2
      res$tau <- sqrt(res$tau2)
    }
    #
    res$w.random <- x[[1]]$weights
    #
    res$TE.random   <- as.numeric(x[[1]]$b)
    res$seTE.random <- sapply(x, extrVar, "se")
    #
    res$lower.random <- sapply(x, extrVar, "ci.lb")
    res$upper.random <- sapply(x, extrVar, "ci.ub")
    #
    ci.r <- ci(res$TE.random, res$seTE.random,
               level = level.ma,
               null.effect = null.effect,
               df = ifelse(method.random.ci == "HK", k - 1, Inf))
    #
    res$statistic.random <- ci.r$statistic
    res$pval.random <- ci.r$p
    res$zval.random <- res$statistic.random
    #
    if (length(method.random.ci) > 1)
      names(res$seTE.random) <-
      names(res$lower.random) <- names(res$upper.random) <-
      names(res$statistic.random) <- names(res$pval.random) <-
      names(res$zval.random) <- method.random.ci
    #
    # Prediction interval
    #
    res <- calcPI(res)
  }
  #
  res$k <- res$method.predict <- res$level.predict <- NULL
  #
  res
}

runGLMM <- function(x, method.tau, method.random.ci, level,
                    use.random = TRUE, warn = TRUE, ...) {
  
  n.methci <- length(method.random.ci)
  #
  list.b <- list(level = 100 * level, ...)
  list.c <- c(x, list.b, method = "FE", test = "z")
  #
  res.r <- list.r <- vector("list", n.methci)
  #
  for (i in seq_len(n.methci))
    list.r[[i]] <- c(x, list.b, method = method.tau,
                     test = ifelse(method.random.ci[i] == "HK", "t", "z"))
  #
  res.c <- runNN(rma.glmm, list.c, warn = FALSE)
  #
  for (i in seq_len(n.methci)) {
    if (use.random) {
      res.r[[i]] <-
        try(runNN(rma.glmm, list.r[[i]], warn = warn), silent = TRUE)
      #
      if ("try-error" %in% class(res.r[[i]])) {
        if (grepl(paste0("Number of parameters to be estimated is ",
                         "larger than the number of observations"),
                  res.r[[i]]) |
            grepl("Cannot fit ML model.", res.r[[i]])) {
          warning(paste(res.r[[i]],
                        " Using result of common effect model as fallback."))
          #
          res.r[[i]] <- res.c
        }
        else
          stop(res.r[[i]])
      }
    }
    else {
      # Fallback to common effect model due to small number of
      # studies or zero or all events
      res.r[[i]] <- res.c
    }
  }
  
  res <- list(glmm.common = res.c, glmm.random = res.r)
  #
  res
}

addGLMM <- function(x, glmm, method.I2, null.effect = x$null.effect) {
  
  res <- x
  #
  k <- x$k
  len <- length(x$TE)
  level.ma <- x$level.ma
  level.hetstat <- x$level.hetstat
  level.predict <- x$level.predict
  method.random.ci <- x$method.random.ci
  method.predict <- x$method.predict
  null.effect <- replaceNULL(null.effect, 0)
  #
  # Common effect model
  #
  glmm.common <- glmm$glmm.common
  #
  res$TE.common   <- as.numeric(glmm.common$b)
  res$seTE.common <- as.numeric(glmm.common$se)
  res$w.common <- rep(NA, len)
  res$lower.common <- glmm.common$ci.lb
  res$upper.common <- glmm.common$ci.ub
  #
  ci.c <- ci(res$TE.common, res$seTE.common, level = level.ma,
             null.effect = null.effect)
  res$statistic.common <- res$zval.common <- ci.c$statistic
  res$pval.common <- ci.c$p
  #
  # Random effects model(s)
  #
  glmm.random <- glmm$glmm.random
  #
  res$TE.random   <- as.numeric(glmm.random[[1]]$b)
  res$seTE.random <- sapply(glmm.random, extrVar, "se")
  res$w.random <- rep(NA, len)
  res$lower.random <- sapply(glmm.random, extrVar, "ci.lb")
  res$upper.random <- sapply(glmm.random, extrVar, "ci.ub")
  #
  ci.r <-
    ci(res$TE.random, res$seTE.random,
       level = level.ma,
       null.effect = null.effect,
       df = ifelse(method.random.ci == "HK", k - 1, Inf))
  res$statistic.random <- res$zval.random <- ci.r$statistic
  res$pval.random <- ci.r$p
  #
  if (length(method.random.ci) > 1)
    names(res$seTE.random) <-
      names(res$lower.random) <- names(res$upper.random) <-
      names(res$statistic.random) <- names(res$pval.random) <-
      names(res$zval.random) <- method.random.ci
  #
  res$se.tau2 <- res$tau <- res$tau2 <- NA
  #
  if (k > 1) {
    res$tau2 <- glmm.random[[1]]$tau2
    res$tau <- sqrt(res$tau2)
    res$se.tau2 <- glmm.random[[1]]$se.tau2
  }
  #
  # Prediction interval
  #
  res <- calcPI(res)
  #
  # Heterogeneity measures
  #
  gr1 <- glmm.random[[1]]
  #
  Q <- if (gr1$k > 1) gr1$QE.Wld else 0
  df.Q <- gr1$QE.df
  pval.Q <- pvalQ(Q, df.Q)
  #
  Q.LRT <- if (gr1$k > 1) gr1$QE.LRT else 0
  pval.Q.LRT <- pvalQ(Q.LRT, df.Q)
  #
  res$Q <- c(Q, Q.LRT)
  res$df.Q <- c(df.Q, df.Q)
  res$pval.Q <- c(pval.Q, pval.Q.LRT)
  names(res$Q) <- c("Wald", "LRT")
  #
  res$upper.tau <- res$lower.tau <- res$upper.tau2 <- res$lower.tau2 <- NA
  #
  res$sign.upper.tau <- res$sign.lower.tau <- res$method.tau.ci <- ""
  #
  if (method.I2 == "Q") {
    H <- calcH(Q, df.Q, level.hetstat)
    res$H <- H$TE
    res$lower.H <- H$lower
    res$upper.H <- H$upper
    #
    I2 <- isquared(Q, df.Q, level.hetstat)
    res$I2 <- I2$TE
    res$lower.I2 <- I2$lower
    res$upper.I2 <- I2$upper
  }
  else {
    res$H <- sqrt(gr1$H2)
    res$lower.H <- NA
    res$upper.H <- NA
    #
    res$I2 <- gr1$I2 / 100
    res$lower.I2 <- NA
    res$upper.I2 <- NA
  }
  #
  res$upper.Rb <- res$lower.Rb <- res$Rb <- NA
  #
  res$.glmm.common <- glmm.common
  #
  if (length(glmm.random) == 1)
    res$.glmm.random <- glmm.random[[1]]
  else
    res$.glmm.random <- glmm.random
  #
  res$version.metafor <- packageDescription("metafor")$Version
  #
  res
}

hccGLMM <- function(x, glmm, method.I2) {
  Q.r <- glmm$QE.Wld
  df.Q.r <- glmm$k - glmm$p
  #
  if (method.I2 == "Q") {
    H.r  <- calcH(Q.r, df.Q.r, x$level.hetstat)
    I2.r <- isquared(Q.r, df.Q.r, x$level.hetstat)
  }
  else {
    H.r  <- list(TE = sqrt(glmm$H2), lower = NA, upper = NA)
    I2.r <- list(TE = glmm$I2 / 100, lower = NA, upper = NA)
  }
  #
  list(tau2.resid = glmm$tau2,
       lower.tau2.resid = NA,
       upper.tau2.resid = NA,
       #
       tau.resid = sqrt(glmm$tau2),
       lower.tau.resid = NA,
       upper.tau.resid = NA,
       sign.lower.tau.resid = "",
       sign.upper.tau.resid = "",
       #
       Q.resid = Q.r,
       df.Q.resid = df.Q.r,
       pval.Q.resid = pvalQ(Q.r, df.Q.r),
       #
       H.resid = H.r$TE,
       lower.H.resid = H.r$lower,
       upper.H.resid = H.r$upper,
       #
       I2.resid = I2.r$TE,
       lower.I2.resid = I2.r$lower,
       upper.I2.resid = I2.r$upper
       )
}

runLRP <- function(event1, n1, event2, n2, warn = TRUE, ...) {
  
  dat.bin <- data.frame(event1, n1, event2, n2)
  #
  long.bin <-
    longarm("B", "A", event1 = event1, n1 = n1, event2 = event2, n2 = n2,
            data = dat.bin)
  #
  text.formula <- "cbind(events, nonevents) ~ as.factor(treat)"
  #
  if (length(unique(long.bin$studlab)) > 1)
    text.formula <- paste(text.formula, "+ as.factor(studlab)")
  #
  res.lrp <- glm(as.formula(text.formula), data = long.bin,
                 family = binomial(link = "logit"),
                 method = brglm2::brglmFit, type = "MPL_Jeffreys",
                 ...)
  #
  phi <- 1
  if (length(unique(long.bin$studlab)) > 1)
    phi <- phi(res.lrp)
  #
  sel.trt <- grepl("treat", names(coef(res.lrp)))
  #
  TE.common   <- as.numeric(coef(res.lrp)[sel.trt])
  seTE.common <- as.numeric(sqrt(diag(vcov(res.lrp)))[sel.trt])
  #
  res <- list(TE.common = TE.common, seTE.common = seTE.common,
              TE.random = TE.common, seTE.random = seTE.common * phi,
              phi = phi, fit = res.lrp)
  #
  res
}

# Estimate the heterogeneity parameter phi using the
# modified version of Pearson's statistic.
#
phi <- function(x) {
  # Extract number of trials
  n.trials <- x$prior.weights
  #
  if (identical(unique(n.trials), 1))
    stop("The number of successes must be summarized for valid computation of ",
         "c-hat.")
  
  # Pearson chi-square
  chisq <- sum(residuals(x, type = "pearson")^2)
  
  # Extract raw residuals
  raw.res <- residuals(x, type = "response")
  
  # Extract fitted values
  fit.vals <- fitted(x)
  
  # Estimate s.bar
  s.bar <- mean((1 - 2 * fit.vals) / ((n.trials * fit.vals) * (1 - fit.vals)))
  
  # Calculate estimate based on Fletcher estimator
  phi <- (chisq / x$df.residual) / (1 + s.bar)
  
  # Set phi = 1 if phi < 1 to remain consistent with common effect model
  phi <- max(phi, 1)
  
  phi
}


calcPercent <- function(x)
  100 * x / sum(x, na.rm = TRUE)

cond <- function(x, only.finite = TRUE, digits = 2, big.mark = "") {
  if (is.null(x))
    return(x)
  #
  if (is.character(x))
    return(x)
  #
  if (only.finite)
    x <- x[is.finite(x)]
  #
  paste(formatN(unique(round(x, digits = digits)), digits = digits,
                big.mark = big.mark), collapse = ", ")
}

setNA_ifnot <- function(x, y, unequal) {
  if (is.null(x) | is.null(y))
    return(x)
  else
    return(ifelse(y != unequal, NA, x))
}

list2mat <- function(x, y = NULL) {
  if (is.list(x)) {
    #
    if (!is.null(y)) {
      for (i in seq_len(length(x)))
        x[[i]] <- matrix(x[[i]], nrow = length(x[[i]]),
                         ncol = ncol(y) / length(x))
    }
    #
    lnames <- names(x)
    #    
    if (!is.null(dimnames(x[[1]])[[2]])) {
      if (!is.null(y)) {
        for (i in seq_along(lnames)) {
          dimnames(x[[i]]) <- dimnames(y)
        }
      }
      else {
        for (i in seq_along(lnames)) {
          dimnames(x[[i]])[[2]] <-
            paste(lnames[i], dimnames(x[[i]])[[2]], sep = "-")
          #
          dimnames(x[[i]])[[2]] <-
            gsub("-classic$", "", dimnames(x[[i]])[[2]])
        }
      }
    }
    #
    return(do.call("cbind", x))
  }
  else if (is.vector(x))
    return(matrix(x, ncol = 1))
  else
    return(x)
}

list2vec <- function(x) {
  if (is.list(x))
    return(do.call("c", x))
  else
    return(x)
}

em2sm <- function(x, type = NULL) {
  x[x == "Odds Ratio"] <- "OR"
  x[x == "Odds Ratio (Non-event)"] <- "OR"
  x[x == "Peto Odds Ratio"] <- "OR"
  x[x == "Risk Ratio"] <- "RR"
  x[x == "Risk Difference"] <- "RD"
  x[x == "Mean Difference"] <- "MD"
  x[x == "Standardized Mean Difference"] <- "SMD"
  x[x == "Std. Mean Difference"] <- "SMD"
  x[x == "Hazard Ratio"] <- "HR"
  #
  if (!is.null(type))
    x <- ifelse(is.na(x) & type == "I", "", x)
  #
  x
}

sm2meth <- function(x) {
  x[x == "IV"] <- "Inverse"
  x[x == "EXP_O_E_VAR"] <- "Peto"
  #
  x
}

setsv <- function(x) {
  if (is.null(x))
    res <- "desirable"
  else {
    res <- setchar(x, c("good", "bad"), stop.at.error = FALSE)
    #
    if (!is.null(res))
      res <- switch(res, good = "desirable", bad = "undesirable")
    else
      res <- x
  }
  #
  setchar(res, c("desirable", "undesirable"))
}

expand <- function(x, y) {
  if (length(x) == 1 & length(y) > 1)
    return(rep_len(x, length(y)))
  else
    return(x)  
}

expandvar <- function(x, n, length = NULL) {
  res <- x
  if (!is.null(length))
    lenOK <- length(x) == length
  else
    lenOK <- TRUE
  #
  if (lenOK & length(x) != n)
    res <- rep(x, rep_len(n, length(x)))
  #
  res
}

warn_ignore_input <- function(x, cond = TRUE, text = "") {
  if (cond)
    warning("Argument '", deparse(substitute(x)), "' ignored",
            if (text != "") " ", text, ".",
            call. = FALSE)
  #
  invisible(NULL)
}

warn_set_input <- function(x, cond = TRUE, set, text = "") {
  if (cond)
    warning("Argument '", deparse(substitute(x)), " = ", set, "'",
            if (text != "") " ", text, ".",
            call. = FALSE)
  #
  invisible(NULL)
}

drop_from_dots <- function(x, old, new) {
  for (i in seq_along(old)) {
    if (!is.null(x[[old[i]]])) {
      if (new[i] != "")
        warning("Argument '", old[i],
                "' ignored; please use argument '",
                new[i], "' instead.",
                call. = FALSE)
      else
        warning("Argument '", old[i],
                "' ignored as it is used internally.",
                call. = FALSE)
      #
      x[[old[i]]] <- NULL
    }
  }
  x
}

extrVec <- function(x, varname, levs, first = FALSE) {
  res <- sapply(x, "[", varname)
  #
  if (max(sapply(res, length)) == 0)
    return(NULL)
  #
  if (first)
    res <- lapply(res, first)
  #
  if (length(res[[1]]) == 1) {
    res <- unlist(res)
    names(res) <- levs
    return(res)
  }
  #
  print(res)
  stop("List element '", varname, "' does not contain single values.",
         call. = FALSE)
}

extrMat <- function(x, varname, clab, rlab) {
  res <- sapply(x, "[", varname)
  #
  if (max(sapply(res, length)) == 0)
    return(NULL)
  #
  maxlen <- max(sapply(res, length))
  res <- lapply(res, addNAs, max = maxlen)
  #
  if (length(res[[1]]) == 1) {
    res <- unlist(res)
    names(res) <- clab
    return(res)
  }
  else if (length(res[[1]]) == length(rlab)) {
    res <- matrix(unlist(res), nrow = length(rlab))
    rownames(res) <- rlab
    colnames(res) <- clab
    return(t(res))
  }
  #
  print(res)
  stop("Wrong number of labels for list element '", varname, "'.",
       call. = FALSE)
}

addNAs <- function(x, max) {
  if (length(x) == 0)
    return(rep(NA, max))
  else if (length(x) == 1 && length(x) < max)
    return(c(x, rep(NA, max - 1)))
  else
    return(x)
}

first <- function(x)
  x[1]

subsetVar <- function(x, subset) {
  if (is.null(x))
    return(x)
  else if (length(x) == 1)
    return(x)
  else
    return(x[subset])
}

#' Print detailed meta-analysis results
#' 
#' @description
#' Print method for objects of class \code{summary.meta}.
#' 
#' @aliases print.summary.meta
#' 
#' @param x An object of class \code{summary.meta}
#' @param sortvar An optional vector used to sort the individual
#'   studies (must be of same length as \code{x$TE}).
#' @param common A logical indicating whether results of common effect
#'   meta-analysis should be printed.
#' @param random A logical indicating whether results of random
#'   effects meta-analysis should be printed.
#' @param details A logical indicating whether further details of
#'   individual studies should be printed.
#' @param ma A logical indicating whether the summary results of the
#'   meta-analysis should be printed.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf = TRUE}, results
#'   for \code{sm = "OR"} are printed as odds ratios rather than log
#'   odds ratios and results for \code{sm = "ZCOR"} are printed as
#'   correlations rather than Fisher's z transformed correlations, for
#'   example.
#' @param pscale A numeric giving scaling factor for printing of
#'   single event probabilities or risk differences, i.e. if argument
#'   \code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
#'   \code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.
#' @param irscale A numeric defining a scaling factor for printing of
#'   single incidence rates or incidence rate differences, i.e. if
#'   argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
#'   \code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.
#' @param irunit A character specifying the time unit used to
#'   calculate rates, e.g. person-years.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   deviations and standard errors, see \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value of test for effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of test of treatment effect, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistic Q, see \code{print.default}.
#' @param digits.df Minimal number of significant digits for degrees
#'   of freedom.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity test, see \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   and Rb statistic, see \code{print.default}.
#' @param digits.H Minimal number of significant digits for H
#'   statistic, see \code{print.default}.
#' @param digits.prop Minimal number of significant digits for
#'   proportions, see \code{print.default}.
#' @param digits.weight Minimal number of significant digits for
#'   weights, see \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param big.mark A character used as thousands separator.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau2.ci A logical value indicating whether to print
#'   the confidence interval of \eqn{\tau^2}.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.tau.ci A logical value indicating whether to print the
#'   confidence interval of \eqn{\tau}.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param print.H A logical specifying whether heterogeneity statistic
#'   H should be printed.
#' @param print.Rb A logical specifying whether heterogeneity
#'   statistic R\eqn{_b} should be printed.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param text.I2 Text printed to identify heterogeneity statistic
#'   I\eqn{^2}.
#' @param text.Rb Text printed to identify heterogeneity statistic
#'   R\eqn{_b}.
#' @param truncate An optional vector used to truncate the printout of
#'   results for individual studies (must be a logical vector of same
#'   length as \code{x$TE} or contain numerical values).
#' @param text.truncate A character string printed if study results
#'   were truncated from the printout.
#' @param details.methods A logical specifying whether details on
#'   statistical methods should be printed.
#' @param warn.backtransf Deprecated argument (ignored).
#' @param \dots Additional arguments (passed on to
#'   \code{\link{print.meta}} called internally).
#' 
#' @details
#' Print method for objects of class \code{summary.meta} giving
#' detailed information on the meta-analysis.
#' 
#' Argument \code{pscale} can be used to rescale single proportions or
#' risk differences, e.g. \code{pscale = 1000} means that proportions
#' are expressed as events per 1000 observations. This is useful in
#' situations with (very) low event probabilities.
#' 
#' Argument \code{irscale} can be used to rescale single rates or rate
#' differences, e.g. \code{irscale = 1000} means that rates are
#' expressed as events per 1000 time units, e.g. person-years. This is
#' useful in situations with (very) low rates. Argument \code{irunit}
#' can be used to specify the time unit used in individual studies
#' (default: "person-years"). This information is printed in summaries
#' and forest plots if argument \code{irscale} is not equal to 1.
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{summary.meta}}, \code{\link{update.meta}},
#'   \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}
#' 
#' @references Cooper H & Hedges LV (1994), \emph{The Handbook of
#'   Research Synthesis}.  Newbury Park, CA: Russell Sage Foundation.
#' 
#' Crippa A, Khudyakov P, Wang M, Orsini N, Spiegelman D (2016), A new measure
#' of between-studies heterogeneity in meta-analysis.  \emph{Statistics in
#' Medicine}, \bold{35}, 3661--75.
#' 
#' Higgins JPT & Thompson SG (2002), Quantifying heterogeneity in a
#' meta-analysis.  \emph{Statistics in Medicine}, \bold{21}, 1539--58.
#' 
#' @keywords print
#' 
#' @examples
#' data(Fleiss1993cont)
#' ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD", studlab = paste(study, year))
#' sma1 <- summary(ma1)
#' sma1
#' 
#' print(sma1, digits = 2)
#' 
#' \dontrun{
#' # Use unicode characters to print tau^2, tau, and I^2 
#' print(sma1,
#'   text.tau2 = "\u03c4\u00b2",
#'   text.tau = "\u03c4", text.I2 = "I\u00b2")
#' }
#' 
#' @method print summary.meta
#' @export

print.summary.meta <- function(x,
                               sortvar,
                               common = x$x$common,
                               random = x$x$random,
                               details = FALSE, ma = TRUE,
                               overall = x$overall & ma,
                               #
                               backtransf = x$backtransf,
                               pscale = x$pscale,
                               irscale = x$irscale,
                               irunit = x$irunit,
                               #
                               digits = gs("digits"),
                               digits.se = gs("digits.se"),
                               digits.stat = gs("digits.stat"),
                               digits.pval = max(gs("digits.pval"), 2),
                               #
                               digits.tau2 = gs("digits.tau2"),
                               digits.tau = gs("digits.tau"),
                               #
                               digits.Q = gs("digits.Q"),
                               digits.df = gs("digits.df"),
                               digits.pval.Q = max(gs("digits.pval.Q"), 2),
                               #
                               digits.H = gs("digits.H"),
                               digits.I2 = gs("digits.I2"),
                               #
                               digits.prop = gs("digits.prop"),
                               digits.weight = gs("digits.weight"),
                               #
                               scientific.pval = gs("scientific.pval"),
                               zero.pval = gs("zero.pval"),
                               JAMA.pval = gs("JAMA.pval"),
                               #
                               big.mark = gs("big.mark"),
                               #
                               print.tau2 = gs("print.tau2"),
                               print.tau2.ci = gs("print.tau2.ci"),
                               print.tau = gs("print.tau"),
                               print.tau.ci = gs("print.tau.ci"),
                               #
                               print.Q = gs("print.Q"),
                               print.I2 = gs("print.I2"),
                               print.I2.ci = gs("print.I2.ci"),
                               print.H = gs("print.H"),
                               print.Rb = gs("print.Rb"),
                               #
                               text.tau2 = gs("text.tau2"),
                               text.tau = gs("text.tau"),
                               text.I2 = gs("text.I2"),
                               text.Rb = gs("text.Rb"),
                               #
                               truncate,
                               text.truncate = "*** Output truncated ***",
                               #
                               details.methods = TRUE,
                               #
                               warn.backtransf = FALSE,
                               ...
                               ) {
  
  
  #
  #
  # (1) Check for summary.meta object
  #
  #
  chkclass(x, "summary.meta")
  #
  k.all <- length(x$TE)
  #
  x.meta <- updateversion(x$x)
  #
  fbt <- x$x$func.backtransf
  abt <- x$x$args.backtransf
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  error <-
    try(sortvar <-
          catch("sortvar", mc, x.meta, sfsp),
        silent = TRUE)
  if (inherits(error, "try-error")) {
    sortvar <- catch("sortvar", mc, x$data,  NULL)
    if (isCol(x$data, ".subset"))
      sortvar <- sortvar[x$data$.subset]
  }
  sort <- !is.null(sortvar)
  if (sort && (length(sortvar) != k.all))
    stop("Number of studies in object 'x' and ",
         "argument 'sortvar' have different length.")
  if (!sort)
    sortvar <- 1:k.all
  #
  chklogical(details)
  chklogical(ma)
  overall <- replaceNULL(overall, TRUE)
  chklogical(overall)
  #
  if (is_untransformed(x$sm))
    backtransf <- TRUE
  chklogical(backtransf)
  #
  chklogical(details.methods)
  #
  if (!is.null(pscale))
    chknumeric(pscale, length = 1)
  else
    pscale <- 1
  if (!backtransf & pscale != 1 & !is_untransformed(x$sm)) {
    warning("Argument 'pscale' set to 1 as argument 'backtransf' is FALSE.")
    pscale <- 1
  }
  #
  if (!is.null(irscale))
    chknumeric(irscale, length = 1)
  else
    irscale <- 1
  if (!backtransf & irscale != 1 & !is_untransformed(x$sm)) {
    warning("Argument 'irscale' set to 1 as argument 'backtransf' is FALSE.")
    irscale <- 1
  }
  #
  scale <- 1
  if (pscale != 1 || irscale != 1) {
    if (pscale != 1 && irscale != 1)
      stop("Provide either arguments 'pscale' or 'irscale' but not ",
           "both arguments.",
           call. = FALSE)
    if (pscale != 1)
      scale <- pscale
    else
      scale <- irscale
  }
  #
  if (!is.null(irunit) && !is.na(irunit))
    chkchar(irunit)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  chknumeric(digits.weight, min = 0, length = 1)
  #
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  metainf.metacum <- inherits(x, "metainf") | inherits(x, "metacum")
  #
  chklogical(print.tau2)
  chklogical(print.tau2.ci)
  chklogical(print.tau)
  chklogical(print.tau.ci)
  chklogical(print.I2)
  chklogical(print.I2.ci)
  chklogical(print.H)
  chklogical(print.Rb)
  chklogical(print.Q)
  #
  chkchar(text.tau2, length = 1)
  chkchar(text.tau, length = 1)
  chkchar(text.I2, length = 1)
  chkchar(text.Rb, length = 1)
  #
  # Catch 'truncate' from meta-analysis object:
  #
  missing.truncate <- missing(truncate)
  if (!missing.truncate) {
    truncate <- catch("truncate", mc, x.meta, sfsp)
    #
    if (is.null(truncate))
      truncate <- catch("truncate", mc, x$data, sfsp)
    #
    if (length(truncate) > k.all)
      stop("Length of argument 'truncate' is too long.",
           call. = FALSE)
    else if (length(truncate) < k.all) {
      if (is.numeric(truncate)) {
        if (any(is.na(truncate)) | max(truncate) > k.all | min(truncate) < 0)
          stop("Numeric values in argument 'truncate' must be between 1 and ",
               k.all, ".",
               call. = FALSE)
        truncate2 <- rep(FALSE, k.all)
        truncate2[truncate] <- TRUE
        truncate <- truncate2
      }
      else if (is.character(truncate)) {
        if (any(!(truncate %in% x$studlab)))
          stop("At least one value of argument 'truncate' does not ",
               "match a study label.",
               call. = FALSE)
        truncate2 <- rep(FALSE, k.all)
        truncate2[x$studlab %in% truncate] <- TRUE
        truncate <- truncate2
      }
      else
        stop("Argument 'truncate' must contain integers or study labels if ",
             "length differs from number of treatment effects.",
             call. = FALSE)
    }
  }
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  #
  missing.common <- missing(common)
  common <- replaceNULL(common, x$comb.common)
  common <- deprecated(common, missing.common, args, "comb.fixed", FALSE)
  common <- deprecated(common, missing.common, args, "fixed", FALSE)
  chklogical(common)
  #
  random <- replaceNULL(random, x$comb.random)
  random <- deprecated(random, missing(random), args, "comb.random", FALSE)
  chklogical(random)
  #
  # More checks ...
  #
  cl <- paste0("update.meta() or ", class(x)[1], "()")
  addargs <- names(list(...))
  #
  level <- x$level
  level.ma <- replaceNULL(x$level.ma, x$level.comb)
  level.predict <- x$level.predict
  
  
  #
  #
  # (3) Some additional settings
  #
  #
  ci.lab <- paste0(round(100 * level, 1), "%-CI")
  #
  sm <- x$sm
  #
  sm.lab <- smlab(sm, backtransf, pscale, irscale)
  #
  if (is.null(x$text.w.common))
    text.w.common <- paste0("%W(", gs("text.w.common"), ")")
  else
    text.w.common <- paste0("%W(", x$text.w.common, ")")
  #
  if (is.null(x$text.w.random))
    text.w.random <- paste0("%W(", gs("text.w.random"), ")")
  else
    text.w.random <- paste0("%W(", x$text.w.random, ")")
  #
  subgroup <- replaceNULL(x$subgroup, x$byvar)
  subgroup.name <- replaceNULL(x$subgroup.name, x$bylab)
  #
  by <- !is.null(subgroup)
  three.level <- !is.null(x$three.level) && any(x$three.level)
  n_of_1 <- !is.null(x$cycles)
  
  
  #
  #
  # (4) Print title and details
  #
  #
  is.metamiss <- inherits(x, "metamiss")
  show.imor <- is.metamiss &
    !is.null(x$IMOR.e) & !is.null(x$IMOR.c) &&
    (length(unique(x$IMOR.e)) != 1 | length(unique(x$IMOR.c)) != 1)
  #
  if (is.metamiss)
    cat("Sensitivity analysis for missing binary data\n\n")
  #
  crtitle(x)
  #
  if (details) {
    if (is.metamiss) {
      res <- cbind(event.e = formatN(x$event.e, digits = 0,
                                     "NA", big.mark = big.mark),
                   noevent.e = formatN(x$n.e - x$event.e - x$miss.e,
                                       digits = 0,
                                       "NA", big.mark = big.mark),
                   miss.e = formatN(x$miss.e, digits = 0,
                                    "NA", big.mark = big.mark),
                   event.c = formatN(x$event.c, digits = 0,
                                     "NA", big.mark = big.mark),
                   noevent.c = formatN(x$n.c - x$event.c - x$miss.c,
                                       digits = 0,
                                       "NA", big.mark = big.mark),
                   miss.c = formatN(x$miss.c, digits = 0,
                                    "NA", big.mark = big.mark))
    }
    else if (inherits(x, "metabin")) {
      res <- cbind(event.e = formatN(x$event.e, digits = 0,
                                     "NA", big.mark = big.mark),
                   n.e = formatN(x$n.e, digits = 0,
                                 "NA", big.mark = big.mark),
                   event.c = formatN(x$event.c, digits = 0,
                                     "NA", big.mark = big.mark),
                   n.c = formatN(x$n.c, digits = 0,
                                 "NA", big.mark = big.mark))
      #
      if (pscale == 1) {
        res <- cbind(res,
                     p.e = formatN(round(x$event.e / x$n.e, digits.prop),
                                   digits.prop, big.mark = big.mark))
        res <- cbind(res,
                     p.c = formatN(round(x$event.c / x$n.c, digits.prop),
                                   digits.prop, big.mark = big.mark))
      }
      else {
        res <- cbind(res,
                     events.e = formatN(round(pscale * x$event.e / x$n.e,
                                              digits),
                                        digits, "NA", big.mark = big.mark))
        res <- cbind(res,
                     events.c = formatN(round(pscale * x$event.c / x$n.c,
                                              digits),
                                        digits, "NA", big.mark = big.mark))
      }
    }
    else if (inherits(x, "metacont")) {
      res <- cbind(n.e = formatN(x$n.e, digits = 0,
                                 "NA", big.mark = big.mark),
                   mean.e = formatN(round(x$mean.e, digits), digits,
                                    "NA", big.mark = big.mark),
                   sd.e = formatN(round(x$sd.e, digits.se), digits.se,
                                  "NA", big.mark = big.mark),
                   n.c = formatN(x$n.c, digits = 0,
                                 "NA", big.mark = big.mark),
                   mean.c = formatN(round(x$mean.c, digits), digits,
                                    "NA", big.mark = big.mark),
                   sd.c = formatN(round(x$sd.c, digits.se), digits.se,
                                  "NA", big.mark = big.mark))
    }
    else if (inherits(x, "metacor")) {
      res <- cbind(cor = x$cor,
                   n = formatN(x$n, digits = 0,
                               "NA", big.mark = big.mark))
    }
    else if (inherits(x, "metagen")) {
      res <- cbind(TE = formatN(round(x$TE, digits), digits,
                                "NA", big.mark = big.mark),
                   seTE = formatN(round(x$seTE, digits.se), digits.se,
                                  "NA", big.mark = big.mark))
    }
    else if (inherits(x, "metainc")) {
      res <- cbind(event.e = formatN(x$event.e, digits = 0,
                                     "NA", big.mark = big.mark),
                   time.e = formatN(round(x$time.e, digits), digits,
                                    "NA", big.mark = big.mark),
                   event.c = formatN(x$event.c, digits = 0,
                                     "NA", big.mark = big.mark),
                   time.c = formatN(round(x$time.c, digits), digits,
                                    "NA", big.mark = big.mark))
      #
      if (irscale == 1) {
        res <- cbind(res,
                     rate.e = formatN(round(x$event.e / x$time.e,
                                            digits.prop),
                                      big.mark = big.mark))
        res <- cbind(res,
                     rate.c = formatN(round(x$event.c / x$time.c,
                                            digits.prop),
                                      big.mark = big.mark))
      }
      else {
        res <- cbind(res,
                     events.e = formatN(round(irscale * x$event.e / x$n.e,
                                              digits),
                                        digits, "NA", big.mark = big.mark))
        res <- cbind(res,
                     events.c = formatN(round(irscale * x$event.c / x$n.c,
                                              digits),
                                        digits, "NA", big.mark = big.mark))
      }
    }
    else if (inherits(x, "metaprop")) {
      res <- cbind(event = formatN(x$event, digits = 0,
                                   "NA", big.mark = big.mark),
                   n = formatN(x$n, digits = 0,
                               "NA", big.mark = big.mark))
      if (pscale == 1)
        res <- cbind(res,
                     p = formatN(round(x$event / x$n, digits.prop),
                                 digits.prop, "NA", big.mark = big.mark))
      else
        res <- cbind(res,
                     events = formatN(round(pscale * x$event / x$n, digits),
                                      digits, "NA", big.mark = big.mark))
    }
    else if (inherits(x, "metarate")) {
      res <- cbind(event = formatN(x$event, digits = 0,
                                   "NA", big.mark = big.mark),
                   time = formatN(x$time, digits = digits,
                                  "NA", big.mark = big.mark))
      if (irscale == 1)
        res <- cbind(res,
                     rate = formatN(round(x$event / x$time, digits.prop),
                                    digits.prop, "NA", big.mark = big.mark))
      else
        res <- cbind(res,
                     events = formatN(round(irscale * x$event / x$time,
                                            digits),
                                      digits, "NA", big.mark = big.mark))
      if (!is.null(x$n))
        res <- cbind(res,
                     n = formatN(x$n, digits = 0,
                                 "NA", big.mark = big.mark))
    }
    else {
      res <- cbind(TE = formatN(round(x$TE, digits), digits,
                                "NA", big.mark = big.mark),
                   seTE = formatN(round(x$seTE, digits), digits,
                                  "NA", big.mark = big.mark))
    }
    #
    if (three.level)
      res <- cbind(res, cluster = as.character(x$cluster))
    #
    if (n_of_1)
      res <- cbind(res, cycles = x$cycles)
    #
    if (by)
      res <- cbind(res, subgroup = as.character(subgroup))
    #
    if (common & !is.null(x$weights.common))
      res <- cbind(res, weights.common = x$weights.common)
    #
    if (random & !is.null(x$weights.random))
      res <- cbind(res, weights.random = x$weights.random)
    #
    if ("weights.common" %in% colnames(res)) {
      if (!random)
        colnames(res)[colnames(res) == "weights.common"] <- "W"
      else
        colnames(res)[colnames(res) == "weights.common"] <- "W(common)"
    }
    #
    if ("weights.random" %in% colnames(res)) {
      if (!common)
        colnames(res)[colnames(res) == "weights.random"] <- "W"
      else
        colnames(res)[colnames(res) == "weights.random"] <- "W(random)"
    }
    #
    dimnames(res)[[1]] <- x$studlab
    #
    if (!missing.truncate) {
      sortvar <- sortvar[truncate]
      res <- res[truncate, , drop = FALSE]
    }
    #
    prmatrix(res[order(sortvar), , drop = FALSE],
             quote = FALSE, right = TRUE)
    if (!missing.truncate)
      cat(text.truncate, "\n")
    cat("\n")
  }
  
  
  #
  #
  # (5) Print results for individual studies
  #
  #
  if (k.all == 1 &&
      !(inherits(x, c("metaprop", "metarate")) |
        (inherits(x, "metabin") && x$sm == "RR" && !x$RR.Cochrane &&
         !is_zero(x$TE - x$TE.common)) |
        (inherits(x, c("metacont", "metamean")) && x$method.ci == "t"))) {
    print.meta(x.meta,
               header = FALSE,
               digits = digits,
               backtransf = backtransf, pscale = pscale,
               irscale = irscale, irunit = irunit,
               #
               text.tau2 = text.tau2, text.tau = text.tau, text.I2 = text.I2,
               #
               scientific.pval = scientific.pval, big.mark = big.mark,
               zero.pval = zero.pval, JAMA.pval = JAMA.pval,
               #
               details.methods = details.methods,
               ...)
  }
  else {
    TE <- x$TE
    seTE <- x$seTE
    lowTE <- x$lower
    uppTE <- x$upper
    #
    if (k.all == 1 &&
        inherits(x, "metabin") && x$sm == "RR" && !x$RR.Cochrane &&
        !is_zero(x$TE - x$TE.common))
      x$method.ci <- "!RR.Cochrane"
    #
    if (backtransf) {
      # Freeman-Tukey Arcsin transformation
      if (metainf.metacum | inherits(x, "metabind")) {
        if (sm == "IRFT")
          harmonic.mean <- x$t.harmonic.mean
        else
          harmonic.mean <- x$n.harmonic.mean
      }
      else {
        if (sm == "IRFT")
          harmonic.mean <- x$time
        else
          harmonic.mean <- x$n
      }
      #
      if (inherits(x, "metaprop"))
        TE <- x$event / x$n
      #
      else if (inherits(x, "metarate"))
        TE <- x$event / x$time
      else
        TE <- backtransf(TE, sm, harmonic.mean, harmonic.mean, fbt, abt)
      #
      lowTE <- backtransf(lowTE, sm, harmonic.mean, harmonic.mean, fbt, abt)
      uppTE <- backtransf(uppTE, sm, harmonic.mean, harmonic.mean, fbt, abt)
      #
      TE <- scale * TE
      lowTE <- scale * lowTE
      uppTE <- scale * uppTE
      #
      if (sm == "VE") {
        tmp.l <- lowTE
        lowTE <- uppTE
        uppTE <- tmp.l
      }
    }
    #
    TE <- round(TE, digits)
    lowTE <- round(lowTE, digits)
    uppTE <- round(uppTE, digits)
    #
    if (!metainf.metacum) {
      if (common) {
        w.common <- x$w.common
        #
        if (is.matrix(w.common)) {
          keep <- !apply(w.common, 2, allNA)
          w.common <- w.common[, keep]
          text.w.common <- text.w.common[keep]
        }
        #
        if (!is.null(w.common) & !all(is.na(w.common)) &&
            sum(w.common) > 0) {
          if (is.matrix(w.common))
            w.common.p <-
              round(apply(w.common, 2, calcPercent), digits.weight)
          else
            w.common.p <-
              round(calcPercent(w.common), digits.weight)
        }
        else
          w.common.p <- w.common
        #
        if (!allNA(w.common.p) && all(w.common.p == 0, na.rm = TRUE))
          w.common.p[!is.na(w.common.p)] <- NA
      }
      #
      if (random) {
        w.random <- x$w.random
        #
        if (is.matrix(w.random)) {
          keep <- !apply(w.random, 2, allNA)
          w.random <- w.random[, keep]
          text.w.random <- text.w.random[keep]
        }
        #
        if (!is.null(w.random) && !all(is.na(w.random)) &&
            sum(w.random) > 0) {
          if (is.matrix(w.random))
            w.random.p <-
              round(apply(w.random, 2, calcPercent), digits.weight)
          else
            w.random.p <-
              round(calcPercent(w.random), digits.weight)
        }
        else
          w.random.p <- w.random
        #
        if (!allNA(w.random.p) && all(w.random.p == 0, na.rm = TRUE))
          w.random.p[!is.na(w.random.p)] <- NA
      }
    }
    #
    if (metainf.metacum) {
      if (is.null(x$text.common))
        text.common <- gs("text.common")
      else
        text.common <- x$text.common
      #
      if (is.null(x$text.random))
        text.random <- gs("text.random")
      else
        text.random <- x$text.random
      #
      if (any(substring(text.common, 1, 5) %in% c("Fixed", "Commo"))) {
        text.common <- gsub("Fixed", "fixed", text.common)
        text.common <- gsub("Common", "common", text.common)
        text.common <- gsub("Effect", "effect", text.common)
      }
      #
      if (any(substring(text.random, 1, 5) %in% c("Rando"))) {
        text.random <- gsub("Random", "random", text.random)
        text.random <- gsub("Effect", "effect", text.random)
      }
      #
      is.random <- x$pooled == "random"
      #
      I2 <- formatN(round(100 * x$I2, digits.I2), digits.I2, "")
      #
      pval <- formatPT(x$pval, digits = digits.pval,
                       scientific = scientific.pval,
                       zero = zero.pval, JAMA = JAMA.pval,
                       lab.NA = "")
      #
      tau2 <- formatN(round(x$tau2, digits.tau2), digits.tau2, "",
                      big.mark = big.mark)
      tau <- formatN(round(x$tau, digits.tau), digits.tau, "",
                     big.mark = big.mark)
      #
      res <- cbind(formatN(round(TE, digits), digits, "",
                           big.mark = big.mark),
                   formatCI(formatN(round(lowTE, digits), digits, "NA",
                                    big.mark = big.mark),
                            formatN(round(uppTE, digits), digits, "NA",
                                    big.mark = big.mark)),
                   pval,
                   if (print.tau2) paste0(" ", tau2),
                   if (print.tau) paste0(" ", tau),
                   if (print.I2) paste0(" ", I2, ifelse(I2 == "", "", "%")))
      dimnames(res) <- list(x$studlab,
                            c(sm.lab, ci.lab, "p-value",
                              if (print.tau2) text.tau2,
                              if (print.tau) text.tau,
                              if (print.I2) text.I2))
      #
      if (inherits(x, "metainf")) {
        if (!is.random)
          cat(paste0("Influential analysis (", text.common, ")\n"))
        else
          cat(paste0("Influential analysis (", text.random, ")\n"))
      }
      else if (inherits(x, "metacum")) {
        if (!is.random)
          cat(paste0("Cumulative meta-analysis (", text.common, ")\n"))
        else
          cat(paste0("Cumulative meta-analysis (", text.random, ")\n"))
      }
      cat("\n")
      prmatrix(res, quote = FALSE, right = TRUE, na.print = "--")
      #
      # Print information on summary method:
      #
      if (details.methods)
        catmeth(x,
                common, random, x$prediction, overall, x$overall.hetstat,
                #
                func.transf = x$func.transf,
                backtransf = backtransf, func.backtransf = x$func.backtransf,
                #
                big.mark = big.mark, digits = digits,
                digits.tau = digits.tau,
                text.tau = text.tau, text.tau2 = text.tau2, text.I2 = text.I2,
                #
                print.tau2 = print.tau2, print.tau = print.tau,
                print.I2 = print.I2)
    }
    else if (!(inherits(x, "metabind") && !x$show.studies)) {
      show.w.common  <-
        (overall | by) &
        (common && !all(is.na(w.common.p)))
      show.w.random <-
        (overall | by) &
        (random && !all(is.na(w.random.p)))
      #
      res <- cbind(formatN(round(TE, digits), digits, "NA",
                           big.mark = big.mark),
                   formatCI(formatN(round(lowTE, digits), digits, "NA",
                                    big.mark = big.mark),
                            formatN(round(uppTE, digits), digits, "NA",
                                    big.mark = big.mark)),
                   if (show.w.common)
                     formatN(w.common.p, digits.weight,
                             big.mark = big.mark),
                   if (show.w.random)
                     formatN(w.random.p, digits.weight,
                             big.mark = big.mark),
                   if (three.level) as.character(x$cluster),
                   if (n_of_1) x$cycles,
                   if (by) as.character(subgroup),
                   if (show.imor) round(x$IMOR.e, 4),
                   if (show.imor) round(x$IMOR.c, 4),
                   if (!is.null(x$exclude))
                     ifelse(is.na(x$exclude), "",
                     ifelse(x$exclude, "*", "")))
      # Printout for a single proportion, mean difference or mean:
      if (k.all == 1) {
        #
        print.stat <- FALSE
        print.pval <- FALSE
        #
        if (!is.null(x$method.ci)) {
          if (x$method.ci == "t") {
            details.ci <-
              "Confidence interval based on t-distribution:\n\n"
            #
            # Add test statistic and p-value
            #
            if (inherits(x, c("metacont", "metamean"))) {
              if (any(!is.na(x$statistic))) {
                res <- cbind(res,
                             formatN(x$statistic,
                                     digits = digits.stat,
                                     big.mark = big.mark))
                #
                print.stat <- TRUE
              }
              #
              if (any(!is.na(x$pval))) {
                res <- cbind(res,
                             formatPT(x$pval, digits = digits.pval,
                                      scientific = scientific.pval,
                                      zero = zero.pval, JAMA = JAMA.pval,
                                      lab.NA = ""))
                #
                print.pval <- TRUE
              }
            }
          }            
          else if (x$method.ci == "CP") {
            details.ci <-
              "Clopper-Pearson confidence interval:\n\n"
            #
            # Add p-value of binomial test
            #
            if (any(!is.na(x$pval))) {
              res <- cbind(res,
                           formatPT(x$pval, digits = digits.pval,
                                    scientific = scientific.pval,
                                    zero = zero.pval, JAMA = JAMA.pval,
                                    lab.NA = ""))
              #
              print.pval <- TRUE
            }
          }
          else if (x$method.ci == "WS")
            details.ci <-
              "Wilson Score confidence interval:\n\n"
          else if (x$method.ci == "WSCC")
            details.ci <-
              "Wilson Score confidence interval with continuity correction:\n\n"
          else if (x$method.ci == "AC")
            details.ci <-
              "Agresti-Coull confidence interval:\n\n"
          else if (x$method.ci == "SA")
            details.ci <-
              "Simple approximation confidence interval:\n\n"
          else if (x$method.ci == "SACC")
            details.ci <-
              paste0("Simple approximation confidence interval with ",
                     "continuity correction:\n\n")
          else if (x$method.ci == "Poisson")
            details.ci <-
              "Exact Poisson confidence interval for individual studies:\n\n"
          else if (x$method.ci == "!RR.Cochrane")
            details.ci <-
              paste0("Continuity correction of 1*incr for sample sizes\n",
                     "(Hartung & Knapp, 2001, Stat Med, equation (18)):\n\n")
          #
          if (x$method.ci != "NAsm") {
            if (inherits(x, "metacont")) {
              catobsev(x$n.e + x$n.c, type = "n", addrow = TRUE)
              x.meta$n.e <- x.meta$n.c <- NA
            }
            else if (inherits(x, "metamean")) {
              catobsev(x$n, type = "n", addrow = TRUE)
              x.meta$n <- NA
            }
            else if (x$method.ci == "!RR.Cochrane") {
              catobsev(x$n.e + x$n.c, type = "n")
              catobsev(x$event.e + x$event.c, type = "e", addrow = TRUE)
              x.meta$n.e <- x.meta$event.e <-
                x.meta$n.c <- x.meta$event.c <- NA
            }
            else {
              catobsev(x$n, type = "n")
              catobsev(x$event, type = "e", addrow = TRUE)
              x.meta$n <- x.meta$event <- NA
            }
            #
            dimnames(res) <-
              list(x$studlab,
                   c(sm.lab, ci.lab,
                     if (show.w.common) text.w.common,
                     if (show.w.random) text.w.random,
                     if (three.level) "cluster",
                     if (n_of_1) "cycles",
                     if (by) subgroup.name,
                     if (!is.null(x$exclude)) "exclude",
                     if (print.stat) "t",
                     if (print.pval) "p-value")
                   )
            prmatrix(res, quote = FALSE, right = TRUE)
            cat("\n")
          }
        }
        if (ma) {
          if (inherits(x, c("metaprop", "metarate", "metacont", "metamean")))
            cat("Normal approximation confidence interval:")
          else if (!is.null(x$method.ci) && x$method.ci == "!RR.Cochrane")
            cat("Mantel-Haenszel method:")
        }
        else {
          if (!(x$method.ci %in% c("t", "NAsm"))) {
            if (pscale != 1)
              sm.details <- paste0("\n- Events per ", pscale, " observations")
            else
              sm.details <- ""
            #
            if (x$method.ci == "CP" & any(!is.na(x$pval))) {
              if (pscale != 1)
                sm.details <-
                  paste0(sm.details,
                         "\n- Null hypothesis: effect is equal to ",
                         format(round(x$null.effect * pscale, digits),
                                scientific = FALSE, big.mark = big.mark),
                         " events per ",
                         format(pscale, scientific = FALSE,
                                big.mark = big.mark),
                         " observations")
              else
                sm.details <-
                  paste0(sm.details,
                         "\n- Null hypothesis: effect is equal to ",
                         format(x$null.effect, scientific = FALSE,
                                big.mark = big.mark))
            }
            #
            if (sm.details != "")
              cat(paste0("Details:", sm.details, "\n"))
          }
        }
      }
      else {
        dimnames(res) <-
          list(x$studlab,
               c(sm.lab, ci.lab,
                 if (show.w.common) text.w.common,
                 if (show.w.random) text.w.random,
                 if (three.level) "cluster",
                 if (n_of_1) "cycles",
                 if (by) subgroup.name,
                 if (show.imor) "IMOR.e",
                 if (show.imor) "IMOR.c",
                 if (!is.null(x$exclude)) "exclude"))
        #
        if (!missing.truncate) {
          sortvar <- sortvar[truncate]
          res <- res[truncate, , drop = FALSE]
        }
        #
        prmatrix(res[order(sortvar), , drop = FALSE],
                 quote = FALSE, right = TRUE)
        if (!missing.truncate)
          cat(text.truncate, "\n")
      }
    }
    
    
    #
    #
    # (6) Print result for meta-analysis
    #
    #
    if (ma & !metainf.metacum) {
      if (!all(is.na(x$k)))
        cat("\n")
      #
      attr(x.meta, ".print.study.results.") <- max(k.all, na.rm = TRUE) > 1
      print.meta(x.meta,
                 header = FALSE,
                 digits = digits,
                 common = common, random = random,
                 overall = overall,
                 backtransf = backtransf, pscale = pscale,
                 irscale = irscale, irunit = irunit,
                 #
                 print.tau2 = print.tau2, print.tau2.ci = print.tau2.ci,
                 digits.tau2 = digits.tau2,
                 #
                 print.tau = print.tau, print.tau.ci = print.tau.ci,
                 digits.tau = digits.tau,
                 #
                 print.Q = print.Q, digits.Q = digits.Q,
                 print.I2 = print.I2, digits.I2 = digits.I2,
                 print.I2.ci = print.I2.ci,
                 print.H = print.H, digits.H = digits.H,
                 print.Rb = print.Rb,
                 #
                 scientific.pval = scientific.pval, big.mark = big.mark,
                 zero.pval = zero.pval, JAMA.pval = JAMA.pval,
                 #
                 text.tau2 = text.tau2, text.tau = text.tau,
                 text.I2 = text.I2, text.Rb = text.Rb,
                 #
                 details.methods = details.methods,
                 warn.deprecated = FALSE,
                 ...)
    }
  }
  
  
  invisible(NULL)
}

#' Calculate absolute and percentage weights for meta-analysis
#' 
#' @description
#' This function returns a data frame containing information on
#' absolute and percentage weights of individual studies contributing
#' to common effect and random effects meta-analysis.
#' 
#' @param object An object of class \code{meta}.
#' @param common A logical indicating whether absolute and
#'   percentage weights from the common effect model should be
#'   calculated.
#' @param random A logical indicating whether absolute and
#'   percentage weights from the random effects model should be
#'   calculated.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @return
#' A data frame with the following variables is returned:
#' \tabular{lll}{
#' \bold{Variable} \tab \bold{Definition} \tab \bold{Condition} \cr
#' w.common \tab absolute weights in common effect model \tab (if
#'   \code{common = TRUE}) \cr
#' p.common \tab percentage weights in common effect model \tab (if
#'   \code{common = TRUE}) \cr
#' w.random \tab absolute weights in random effects model \tab (if
#'   \code{random = TRUE}) \cr
#' p.random \tab percentage weights in random effects model \tab (if
#'   \code{random = TRUE})
#' }
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}
#' 
#' @examples
#' data(Fleiss1993cont)
#' # Do meta-analysis (common effect and random effects model)
#' #
#' meta1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, studlab = paste(study, year), sm = "SMD")
#' 
#' # Print weights for common effect and random effects meta-analysis
#' #
#' weights(meta1)
#' 
#' # Do meta-analysis (only random effects model)
#' #
#' meta2 <- update(meta1, common = FALSE)
#' 
#' # Print weights for random effects meta-analysis
#' #
#' weights(meta2)
#' 
#' # Print weights for common effect and random effects meta-analysis
#' #
#' weights(meta2, common = TRUE)
#' 
#' @method weights meta
#' @export

weights.meta <- function(object,
                         common = object$common,
                         random = object$random,
                         warn.deprecated = gs("warn.deprecated"),
                         ...) {
  
  #
  #
  # (1) Check for meta object
  #
  #
  chkclass(object, "meta")
  object <- update(object)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  missing.common <- missing(common)
  common <-
    deprecated(common, missing.common, args, "comb.fixed",
               warn.deprecated)
  common <-
    deprecated(common, missing.common, args, "fixed",
               warn.deprecated)
  #
  random <-
    deprecated(random, missing(random), args, "comb.random",
               warn.deprecated)
  
  
  if (!(common | random)) {
    warning("Information missing which weights should be ",
            "calculated (see arguments 'common' and 'random')")
    return(NULL)
  }

  w.common  <- object$w.common
  w.random <- object$w.random
  #
  p.common <- 100 * w.common / sum(w.common, na.rm = TRUE)
  p.random <- 100 * w.random / sum(w.random, na.rm = TRUE)

  res <- data.frame(w.common, p.common, w.random, p.random)
  #
  rownames(res) <- object$studlab
  #
  if (!common) {
    res$w.common <- NULL
    res$p.common <- NULL
  }
  #
  if (!random) {
    res$w.random <- NULL
    res$p.random <- NULL
  }
  
  res
}

#' Forest plot to display the result of a meta-analysis
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid
#' graphics system).
#' 
#' @aliases forest.metabind
#' 
#' @param x An object of class \code{\link{metabind}}.
#' @param leftcols A character vector specifying (additional) columns
#'   to be plotted on the left side of the forest plot or a logical
#'   value (see Details).
#' @param leftlabs A character vector specifying labels for
#'   (additional) columns on left side of the forest plot (see
#'   Details).
#' @param rightcols A character vector specifying (additional) columns
#'   to be plotted on the right side of the forest plot or a logical
#'   value (see Details).
#' @param rightlabs A character vector specifying labels for
#'   (additional) columns on right side of the forest plot (see
#'   Details).
#' @param common A logical indicating whether common effect estimates
#'   should be plotted.
#' @param random A logical indicating whether random effects estimates
#'   should be plotted.
#' @param overall A logical indicating whether overall summaries
#'   should be plotted. This argument is useful in a meta-analysis
#'   with subgroups if summaries should only be plotted on group
#'   level.
#' @param subgroup A logical indicating whether subgroup results
#'   should be shown in forest plot. This argument is useful in a
#'   meta-analysis with subgroups if summaries should not be plotted
#'   on group level.
#' @param hetstat Either a logical value indicating whether to print
#'   results for heterogeneity measures at all or a character string
#'   (see Details).
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether prediction
#'   interval(s) should be printed.
#' @param type A character string or vector specifying how to
#'   plot estimates.
#' @param type.common A single character string specifying how to
#'   plot common effect estimates.
#' @param type.random A single character string specifying how to
#'   plot random effects estimates.
#' @param type.predict A single character string specifying how to
#'   plot prediction intervals.
#' @param lab.NA A character string to label missing values.
#' @param col.square The colour for squares reflecting study's weight
#'   in the meta-analysis.
#' @param col.square.lines The colour for the outer lines of squares
#'   reflecting study's weight in the meta-analysis.
#' @param col.circle The colour for circles reflecting study weights
#'   in the meta-analysis.
#' @param col.circle.lines The colour for the outer lines of circles
#'   reflecting study's weight in the meta-analysis.
#' @param col.diamond The colour of diamonds representing the results
#'   for common effect and random effects models.
#' @param col.diamond.common The colour of diamonds for common effect
#'   estimates.
#' @param col.diamond.random The colour of diamonds for random effects
#'   estimates.
#' @param col.diamond.lines The colour of the outer lines of diamonds
#'   representing the results for common effect and random effects
#'   models.
#' @param col.diamond.lines.common The colour of the outer lines of
#'   diamond for common effect estimates.
#' @param col.diamond.lines.random The colour of the outer lines of
#'   diamond for random effects estimates.
#' @param col.predict Background colour of prediction intervals.
#' @param col.predict.lines Colour of outer lines of prediction
#'   intervals.
#' @param digits Minimal number of significant digits for treatment
#'   effects, see \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   errors, see \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-statistic for test of overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity test, see \code{print.default}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistic Q, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance, see \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic, see \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param big.mark A character used as thousands separator.
#' @param print.subgroup.labels A logical indicating whether subgroup
#'   label should be printed.
#' @param addrow.subgroups A logical value indicating whether an empty
#'   row is printed between results for subgroups.
#' @param smlab A label for the summary measurex (printed at top of
#'   figure).
#' @param calcwidth.pooled A logical indicating whether text for
#'   common effect and random effects model should be considered to
#'   calculate width of the column with study labels.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional graphical arguments (passed on to
#'   \code{\link{forest.meta}}).
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window. The forest functions in R package
#' \bold{meta} are based on the grid graphics system. In order to
#' print the forest plot, resize the graphics window and either use
#' \code{\link{dev.copy2eps}} or \code{\link{dev.copy2pdf}}. Another
#' possibility is to create a file using \code{\link{pdf}},
#' \code{\link{png}}, or \code{\link{svg}} and to specify the width
#' and height of the graphic (see \code{\link{forest.meta}} examples).
#' 
#' The arguments \code{leftcols} and \code{rightcols} can be used to
#' specify columns which are plotted on the left and right side of the
#' forest plot, respectively.
#' 
#' The arguments \code{leftlabs} and \code{rightlabs} can be used to
#' specify column headings which are plotted on left and right side of
#' the forest plot, respectively. For certain columns predefined
#' labels exist. For other columns, the column name will be used as a
#' label. It is possible to only provide labels for new columns (see
#' \code{\link{forest.meta}} examples). Otherwise the length of
#' \code{leftlabs} and \code{rightlabs} must be the same as the number
#' of printed columns, respectively. The value \code{NA} can be used
#' to specify columns which should use default labels.
#' 
#' Argument \code{hetstat} can be a character string to specify where
#' to print heterogeneity information:
#' \itemize{
#' \item row with results for common effect model (\code{hetstat =
#' "common"}),
#' \item row with results for random effects model (\code{hetstat =
#' "random"}),
#' \item rows with 'study' information (\code{hetstat = "study"}).
#' }
#' Otherwise, information on heterogeneity is printed in dedicated rows.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.meta}}, \code{\link{metabin}},
#'   \code{\link{metacont}}, \code{\link{metagen}},
#'   \code{\link{metabind}}, \code{\link{settings.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Fleiss1993cont)
#' 
#' # Add some (fictitious) grouping variables:
#' #
#' Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
#' Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")
#' 
#' ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD")
#'
#' # Conduct two subgroup analyses
#' #
#' ma1 <- update(ma, subgroup = age, subgroup.name = "Age group")
#' ma2 <- update(ma, subgroup = region, subgroup.name = "Region")
#'
#' # Combine subgroup meta-analyses and show forest plot with subgroup
#' # results
#' #
#' mb <- metabind(ma1, ma2)
#' mb
#' forest(mb)
#'
#' @method forest metabind
#' @export

forest.metabind <- function(x,
                            leftcols, leftlabs,
                            rightcols = c("effect", "ci"), rightlabs,
                            #
                            common = x$common,
                            random = x$random,
                            overall = x$overall,
                            subgroup = FALSE,
                            hetstat = FALSE,
                            overall.hetstat = x$overall.hetstat,
                            prediction = x$prediction,
                            #
                            lab.NA = "",
                            #
                            col.square = gs("col.square"),
                            col.square.lines = col.square,
                            col.circle = gs("col.circle"),
                            col.circle.lines = col.circle,
                            #
                            col.diamond = gs("col.diamond"),
                            col.diamond.common = col.diamond,
                            col.diamond.random = col.diamond,
                            col.diamond.lines = gs("col.diamond.lines"),
                            col.diamond.lines.common = col.diamond.lines,
                            col.diamond.lines.random = col.diamond.lines,
                            #
                            col.predict = gs("col.predict"),
                            col.predict.lines = gs("col.predict.lines"),
                            #
                            type = NULL,
                            type.common = NULL,
                            type.random = NULL,
                            type.predict = NULL,
                            #
                            digits = gs("digits.forest"),
                            digits.se = gs("digits.se"),
                            digits.stat = gs("digits.stat"),
                            digits.pval = max(gs("digits.pval") - 2, 2),
                            digits.pval.Q = max(gs("digits.pval.Q") - 2, 2),
                            digits.Q = gs("digits.Q"),
                            digits.tau2 = gs("digits.tau2"),
                            digits.tau = gs("digits.tau"),
                            digits.I2 = max(gs("digits.I2") - 1, 0),
                            #
                            scientific.pval = gs("scientific.pval"),
                            big.mark = gs("big.mark"),
                            #
                            print.subgroup.labels = x$with.subgroups,
                            addrow.subgroups = print.subgroup.labels,
                            #
                            smlab,
                            calcwidth.pooled = overall,
                            #
                            warn.deprecated = gs("warn.deprecated"),
                            #
                            ...) {
  
  
  #
  #
  # (1) Check and set arguments
  #
  #
  chkclass(x, "metabind")
  x <- updateversion(x)
  #
  missing.overall <- missing(overall)
  missing.overall.hetstat <- missing(overall.hetstat)
  #
  overall <- replaceNULL(overall, FALSE)
  overall.hetstat <- replaceNULL(overall.hetstat, FALSE)
  #
  chklogical(common)
  chklogical(random)
  chklogical(overall)
  chklogical(subgroup)
  chklogical(overall.hetstat)
  #
  chklogical(print.subgroup.labels)
  chklogical(addrow.subgroups)
  #
  chkchar(lab.NA)
  #
  if (missing(type))
    type <- x$data$type
  else {
    if (length(type) == 1 & nrow(x$data) > 1)
      type <- rep(type, nrow(x$data))
    else if (length(type) != nrow(x$data))
      stop("Argument 'type' must be a character vector of length 1 or ",
           nrow(x$data), ".",
           call. = FALSE)
  }
  #
  if (!is.null(type.common)) {
    if (length(type.common) != 1)
      stop("Argument 'type.common' must be of length 1.",
           call. = FALSE)
    #
    type.common <-
      setchar(type.common, c("square", "diamond", "predict", "circle"))
  }
  #
  if (!is.null(type.random)) {
    if (length(type.random) != 1)
      stop("Argument 'type.random' must be of length 1.",
           call. = FALSE)
    #
    type.random <-
      setchar(type.random, c("square", "diamond", "predict", "circle"))
  }
  #
  if (!is.null(type.predict)) {
    if (length(type.predict) != 1)
      stop("Argument 'type.predict' must be of length 1.",
           call. = FALSE)
    #
    type.predict <-
      setchar(type.predict, c("square", "diamond", "predict", "circle"))
  }
  #
  chkcolor(col.square, length = 1)
  chkcolor(col.square.lines, length = 1)
  chkcolor(col.circle, length = 1)
  chkcolor(col.circle.lines, length = 1)
  #
  chkcolor(col.diamond, length = 1)
  chkcolor(col.diamond.common, length = 1)
  chkcolor(col.diamond.random, length = 1)
  chkcolor(col.diamond.lines, length = 1)
  chkcolor(col.diamond.lines.common, length = 1)
  chkcolor(col.diamond.lines.random, length = 1)
  #
  chkcolor(col.predict, length = 1)
  chkcolor(col.predict.lines, length = 1)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.pval.Q, min = 1, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  #
  chklogical(scientific.pval)
  chklogical(calcwidth.pooled)
  #
  addargs <- names(list(...))
  #
  idx <- charmatch(tolower(addargs), "hetstat", nomatch = NA)
  if (any(!is.na(idx)) && length(idx) > 0)
    if (list(...)[["hetstat"]])
      stop("Argument 'hetstat' must be FALSE for metabind objects.",
           call. = TRUE)
  #
  for (i in addargs) {
    if (!is.null(setchar(i, "type.study", stop.at.error = FALSE)))
      stop("Use argument 'type' as argument 'type.study' is set internally.",
           call. = TRUE)
    if (!is.null(setchar(i, "fixed", stop.at.error = FALSE)))
      stop("Argument 'fixed' cannot be used with metabind objects.",
           call. = TRUE)
  }
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  digits.stat <-
    deprecated(digits.stat, missing(digits.stat), args, "digits.zval",
               warn.deprecated)
  digits.stat <- replaceNULL(digits.stat, gs("digits.stat"))
  chknumeric(digits.stat, min = 0, length = 1)
  
  
  x$k.w.orig <- x$k.w
  
  x$k.w <- x$k.study.w <- x$k.all.w <- x$k.TE.w <-
    as.vector(table(x$data$subgroup)[unique(x$data$subgroup)])
  #as.vector(table(x$data$name)[unique(x$data$name)])
  
  
  missing.leftcols <- missing(leftcols)
  #
  if (missing.leftcols) {
    leftcols <- c("studlab", "k")
    if (x$with.subgroups)
      leftcols <- c(leftcols, "pval.Q.b")
    else {
      leftcols <- c(leftcols, "tau2")
    }
  }
  #
  if (!is.logical(rightcols)) {
    if ("pval.Q.b" %in% rightcols & "pval.Q.b" %in% leftcols)
      leftcols <- leftcols[leftcols != "pval.Q.b"]
    #
    if ("k" %in% rightcols & "k" %in% leftcols)
      leftcols <- leftcols[leftcols != "k"]
    #
    if ("tau2" %in% rightcols & "tau2" %in% leftcols)
      leftcols <- leftcols[leftcols != "tau2"]
    #
    if ("Q" %in% rightcols & "Q" %in% leftcols)
      leftcols <- leftcols[leftcols != "Q"]
    #
    if ("pval.Q" %in% rightcols & "pval.Q" %in% leftcols)
      leftcols <- leftcols[leftcols != "pval.Q"]
  }
  #
  label.studlab <- ifelse(x$with.subgroups, "Subgroup", "Method")
  #
  newlab <- function(col, lab, varname, value) {
    sel <- col == varname
    if (!any(sel) || !is.na(lab[sel]))
      return(lab)
    else
      lab[sel] <- value
    lab
  }
  #
  if (missing(leftlabs))
    leftlabs <- rep(NA, length(leftcols))
  if (any(is.na(leftlabs))) {
    leftlabs <-
      newlab(leftcols, leftlabs, "studlab", label.studlab)
    leftlabs <-
      newlab(leftcols, leftlabs, "k", "Number of\nStudies")
    leftlabs <-
      newlab(leftcols, leftlabs, "tau2", "Between-study\nvariance")
    leftlabs <-
      newlab(leftcols, leftlabs, "tau", "Between-study\nSD")
    leftlabs <-
      newlab(leftcols, leftlabs, "pval.Q.b", "Interaction\nP-value")
    leftlabs <-
      newlab(leftcols, leftlabs, "pval.Q", "P-value")
  }
  #
  if (missing(rightlabs))
    rightlabs <- rep(NA, length(rightcols))
  if (any(is.na(rightlabs))) {
    rightlabs <-
      newlab(rightcols, rightlabs, "studlab", label.studlab)
    rightlabs <-
      newlab(rightcols, rightlabs, "k", "Number of\nStudies")
    rightlabs <-
      newlab(rightcols, rightlabs, "tau2", "Between-study\nvariance")
    rightlabs <-
      newlab(rightcols, rightlabs, "tau", "Between-study\nSD")
    rightlabs <-
      newlab(rightcols, rightlabs, "pval.Q.b", "Interaction\nP-value")
    rightlabs <-
      newlab(rightcols, rightlabs, "pval.Q", "P-value")
  }
  
  
  #
  # Round and round ...
  #
  if (x$with.subgroups) {
    x$data$Q.b <- round(x$data$Q.b, digits.Q)
    x$data$pval.Q.b <- formatPT(x$data$pval.Q.b,
                                lab = FALSE,
                                digits = digits.pval.Q,
                                scientific = scientific.pval,
                                lab.NA = lab.NA)
    x$data$pval.Q.b <- rmSpace(x$data$pval.Q.b)
  }
  else {
    x$data$Q <- round(x$data$Q, digits.Q)
    x$Q <- round(x$Q, digits.Q)
    x$data$pval.Q <-
      rmSpace(formatPT(x$data$pval.Q,
                       lab = FALSE,
                       digits = digits.pval.Q,
                       scientific = scientific.pval,
                       lab.NA = lab.NA))
  }
  #
  x$data$tau2 <-
    rmSpace(formatPT(x$data$tau2, digits = digits.tau2,
                     big.mark = big.mark,
                     lab = FALSE, lab.NA = lab.NA))
  #
  x$data$tau <-
    rmSpace(formatPT(x$data$tau, digits = digits.tau,
                     big.mark = big.mark,
                     lab = FALSE, lab.NA = lab.NA))
  #
  I2.na <- is.na(x$data$I2)
  x$data$I2 <-
    formatN(round(100 * x$data$I2, digits.I2), digits.I2, "")
  x$data$lower.I2 <-
    formatN(round(100 * x$data$lower.I2, digits.I2), digits.I2, "")
  x$data$upper.I2 <-
    formatN(round(100 * x$data$upper.I2, digits.I2), digits.I2, "")
  #
  x$data$I2 <-
    rmSpace(paste0(x$data$I2, ifelse(I2.na, "", "%")))
  x$data$lower.I2 <-
    rmSpace(paste0(x$data$lower.I2, ifelse(I2.na, "", "%")))
  x$data$upper.I2 <-
    rmSpace(paste0(x$data$upper.I2, ifelse(I2.na, "", "%")))
  #
  x$data$k <- as.character(x$data$k)
  #
  model <- x$data$model
  #
  if (!is.null(type.common))
    type[model == "common"] <- type.common
  #
  if (!is.null(type.random))
    type[model == "random"] <- type.random
  #
  if (!is.null(type.predict))
    type[model == "predict"] <- type.predict
  #
  cols.square <- vector("character", length(type))
  cols.square.lines <- vector("character", length(type))
  #
  for (i in seq_along(type)) {
    if (type[i] == "diamond") {
      if (model[i] == "common") {
        cols.square[i] <- col.diamond.common
        cols.square.lines[i] <- col.diamond.lines.common
      }
      else if (model[i] == "random") {
        cols.square[i] <- col.diamond.random
        cols.square.lines[i] <- col.diamond.lines.random
      }
      else {
        col.predict[i] <- col.diamond.random
        cols.square.lines[i] <- col.diamond.lines.random
      }
    }
    else if (type[i] == "predict") {
      cols.square[i] <- col.predict
      cols.square.lines[i] <- col.predict.lines
    }
    else if (type[i] == "circle") {
      cols.square[i] <- col.circle
      cols.square.lines[i] <- col.circle.lines
    }
    else {
      cols.square[i] <- col.square
      cols.square.lines[i] <- col.square.lines
    }
  }
  #
  if (missing(smlab))
    smlab <- xlab_meta(x$sm, x$backtransf)
  #
  if (!x$samedata) {
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  
  
  x.forest <- x
  #
  if (x$with.subgroups) {
    m.forest <- metagen(x.forest$TE,
                        lower = x.forest$lower,
                        upper = x.forest$upper,
                        subgroup = x.forest$subgroup,
                        studlab = x.forest$studlab,
                        data = x.forest,
                        common = common,
                        random = random,
                        prediction = prediction,
                        overall = overall,
                        print.subgroup.name = FALSE)
    #
    class(m.forest) <- c(class(m.forest), "is.metabind")
    m.forest$sm <- x.forest$sm
    #
    m.forest$TE[m.forest$data$type == "predict"] <- NA
    m.forest$statistic <- x.forest$statistic
    m.forest$pval <- x.forest$pval
    #
    m.forest$n.harmonic.mean <- x.forest$n.harmonic.mean
    m.forest$t.harmonic.mean <- x.forest$t.harmonic.mean
    m.forest$n.harmonic.mean.ma <- x.forest$n.harmonic.mean.ma
    m.forest$t.harmonic.mean.ma <- x.forest$t.harmonic.mean.ma
    m.forest$n.harmonic.mean.w <- x.forest$n.harmonic.mean.w
    m.forest$t.harmonic.mean.w <- x.forest$t.harmonic.mean.w
    #
    if (x$samedata) {
      m.forest$method <- x.forest$method
      m.forest$method.random <- x.forest$method.random
      m.forest$method.predict <- x.forest$method.predict
      m.forest$method.tau <- x.forest$method.tau
      #
      m.forest$TE.common <- x.forest$TE.common
      m.forest$seTE.common <- x.forest$seTE.common
      m.forest$lower.common <- x.forest$lower.common
      m.forest$upper.common <- x.forest$upper.common
      m.forest$statistic.common <- x.forest$statistic.common
      m.forest$pval.common <- x.forest$pval.common
      m.forest$zval.common <- x.forest$zval.common
      m.forest$text.common <- x.forest$text.common
      #
      m.forest$w.common <-
        ifelse(is.na(m.forest$w.common), m.forest$w.common, NA)
      #
      m.forest$TE.random <- x.forest$TE.random
      m.forest$seTE.random <- x.forest$seTE.random
      m.forest$lower.random <- x.forest$lower.random
      m.forest$upper.random <- x.forest$upper.random
      m.forest$statistic.random <- x.forest$statistic.random
      m.forest$pval.random <- x.forest$pval.random
      m.forest$zval.random <- x.forest$zval.random
      m.forest$df.random <- x.forest$df.random
      m.forest$text.random <- x.forest$text.random
      #
      m.forest$w.random <-
        ifelse(is.na(m.forest$w.random), m.forest$w.random, NA)
      #
      m.forest$lower.predict <- x.forest$lower.predict
      m.forest$upper.predict <- x.forest$upper.predict
      m.forest$text.predict <-
        replaceNULL(x.forest$text.predict, gs("text.predict"))
    }
    #
    res <-
      forest(m.forest,
             leftcols = leftcols, leftlabs = leftlabs,
             rightcols = rightcols, rightlabs = rightlabs,
             print.subgroup.labels = print.subgroup.labels,
             addrow.subgroups = addrow.subgroups,
             #
             hetstat = hetstat, overall.hetstat = overall.hetstat,
             calcwidth.pooled = calcwidth.pooled,
             lab.NA = lab.NA, smlab = smlab,
             #
             type.study = type, col.square = cols.square,
             col.square.lines = cols.square.lines,
             #
             pooled.totals = FALSE,
             common.subgroup = FALSE, random.subgroup = FALSE,
             prediction.subgroup = FALSE,
             #
             col.predict = col.predict,
             #
             weight.study = "same",
             #
             digits = digits,
             digits.se = digits.se,
             digits.stat = digits.stat,
             digits.pval = digits.pval,
             digits.pval.Q = digits.pval.Q,
             digits.Q = digits.Q,
             digits.tau2 = digits.tau2,
             digits.tau = digits.tau,
             digits.I2 = digits.I2,
             #
             scientific.pval = scientific.pval,
             big.mark = big.mark,
             #
             test.subgroup = FALSE,
             details = FALSE,
             ...)
  }
  else {
    if (isCol(x.forest$data, "subgroup"))
      m.forest <- metagen(x.forest$data$TE,
                          lower = x.forest$data$lower,
                          upper = x.forest$data$upper,
                          studlab = x.forest$data$studlab,
                          subgroup = x.forest$data$subgroup,
                          print.subgroup.name = FALSE,
                          data = x.forest,
                          common = FALSE,
                          random = FALSE,
                          prediction = FALSE,
                          overall = FALSE,
                          method.tau = "DL", method.tau.ci = "")
    else
      m.forest <- metagen(x.forest$data$TE,
                          lower = x.forest$data$lower,
                          upper = x.forest$data$upper,
                          studlab = x.forest$data$studlab,
                          data = x.forest,
                          common = common,
                          random = random,
                          prediction = prediction,
                          overall = overall)
    #
    class(m.forest) <- c(class(m.forest), "is.metabind")
    m.forest$sm <- x.forest$sm
    m.forest$with.subgroups <- TRUE
    #
    m.forest$n.harmonic.mean <- x.forest$n.harmonic.mean
    m.forest$t.harmonic.mean <- x.forest$t.harmonic.mean
    #
    m.forest$TE[m.forest$data$type == "predict"] <- NA
    #
    res <-
      forest(m.forest,
             leftcols = leftcols, leftlabs = leftlabs,
             rightcols = rightcols, rightlabs = rightlabs,
             #
             hetstat = hetstat, overall.hetstat = overall.hetstat,
             calcwidth.pooled = calcwidth.pooled,
             lab.NA = lab.NA, smlab = smlab,
             #
             type.study = type, col.square = cols.square,
             col.square.lines = cols.square.lines,
             #
             pooled.totals = FALSE,
             #
             col.predict = col.predict,
             #
             weight.study = "same",
             #
             digits = digits,
             digits.se = digits.se,
             digits.stat = digits.stat,
             digits.pval = digits.pval,
             digits.pval.Q = digits.pval.Q,
             digits.Q = digits.Q,
             digits.tau2 = digits.tau2,
             digits.tau = digits.tau,
             digits.I2 = digits.I2,
             #
             scientific.pval = scientific.pval,
             big.mark = big.mark,
             #
             test.subgroup = FALSE,
             details = FALSE,
             ...)
  }
  #
  return(invisible(res))
}

#' Conversion from standardised mean difference to log odds ratio
#' 
#' @description
#' Conversion from standardised mean difference to log odds ratio
#' using method by Hasselblad & Hedges (1995) or Cox (1970).
#' 
#' @param smd Standardised mean difference(s) (SMD) or meta-analysis
#'   object.
#' @param se.smd Standard error(s) of SMD (ignored if argument
#'   \code{smd} is a meta-analysis object).
#' @param studlab An optional vector with study labels (ignored if
#'   argument \code{smd} is a meta-analysis object).
#' @param data An optional data frame containing the study information
#'   (ignored if argument \code{smd} is a meta-analysis object).
#' @param subset An optional vector specifying a subset of studies to
#'   be used (ignored if argument \code{smd} is a meta-analysis
#'   object).
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots (ignored if argument \code{smd} is a meta-analysis object).
#' @param method A character string indicating which method is used to
#'   convert SMDs to log odds ratios. Either \code{"HH"} or
#'   \code{"CS"}, can be abbreviated.
#' @param backtransf A logical indicating whether odds ratios (if
#'   TRUE) or log odds ratios (if FALSE) should be shown in printouts
#'   and plots.
#' @param \dots Additional arguments passed on to
#'   \code{\link{metagen}} (ignored if argument \code{smd} is a
#'   meta-analysis object).
#' 
#' @details
#' This function implements the following methods for the conversion
#' from standardised mean difference to log odds ratio:
#' \itemize{
#' \item Hasselblad & Hedges (1995) assuming logistic distributions
#'   (\code{method == "HH"})
#' \item Cox (1970) and Cox & Snell (1989) assuming normal
#'   distributions (\code{method == "CS"})
#' }
#' Internally, \code{\link{metagen}} is used to conduct a
#' meta-analysis with the odds ratio as summary measure.
#' 
#' Argument \code{smd} can be either a vector of standardised mean
#' differences or a meta-analysis object created with
#' \code{\link{metacont}} or \code{\link{metagen}} and the
#' standardised mean difference as summary measure.
#'
#' Argument \code{se.smd} is mandatory if argument \code{smd} is a
#' vector and ignored otherwise. Additional arguments in \code{\dots}
#' are only passed on to \code{\link{metagen}} if argument \code{smd}
#' is a vector.
#' 
#' @return
#' An object of class \code{c("metagen", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{or2smd}}, \code{\link{metacont}},
#'   \code{\link{metagen}}, \code{\link{metabin}}
#' 
#' @references
#' Borenstein M, Hedges LV, Higgins JPT, Rothstein HR (2009):
#' \emph{Introduction to Meta-Analysis}.
#' Chichester: Wiley
#'
#' Cox DR (1970):
#' \emph{Analysis of Binary Data}.
#' London: Chapman and Hall / CRC
#'
#' Cox DR, Snell EJ (1989):
#' \emph{Analysis of Binary Data} (2nd edition).
#' London: Chapman and Hall / CRC
#'
#' Hasselblad V, Hedges LV (1995):
#' Meta-analysis of screening and diagnostic tests.
#' \emph{Psychological Bulletin},
#' \bold{117}, 167--78
#' 
#' @examples
#' # Example from Borenstein et al. (2009), Chapter 7
#' #
#' ma0 <- smd2or(0.5, sqrt(0.0205), backtransf = FALSE)
#' # TE = log odds ratio; seTE = standard error of log odds ratio
#' data.frame(lnOR = round(ma0$TE, 4), varlnOR = round(ma0$seTE^2, 4))
#'
#' # Use dataset from Fleiss (1993)
#' #
#' data(Fleiss1993cont)
#' ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD",
#'   studlab = paste(study, year))
#' smd2or(ma1)
#' 
#' @export smd2or

smd2or <- function(smd, se.smd, studlab,
                   data = NULL, subset = NULL, exclude = NULL,
                   method = "HH", backtransf = gs("backtransf"), ...) {
  
  
  is.meta <- inherits(smd, "meta")
  #
  if (is.meta) {
    smd <- updateversion(smd)
    if (smd$sm != "SMD")
      stop("Effect measure must be equal to 'SMD'.", call. = FALSE)
    else {
      mdat <- smd
      smd <- mdat$TE
      se.smd <- mdat$seTE
    }
  }
  else {
    #
    # Read data
    #
    nulldata <- is.null(data)
    sfsp <- sys.frame(sys.parent())
    mc <- match.call()
    #
    if (nulldata)
      data <- sfsp
    #
    # Catch 'smd' and 'se.smd' from data:
    #
    smd <- catch("smd", mc, data, sfsp)
    #
    se.smd <- catch("se.smd", mc, data, sfsp)
    #
    k.All <- length(smd)
    chknull(smd)
    #
    # Catch 'studlab', 'subset', and 'exclude' from data:
    #
    studlab <- catch("studlab", mc, data, sfsp)
    studlab <- setstudlab(studlab, k.All)
    #
    missing.subset <- missing(subset)
    subset <- catch("subset", mc, data, sfsp)
    #
    missing.exclude <- missing(exclude)
    exclude <- catch("exclude", mc, data, sfsp)
    #
    # Check length of essential variables
    #
    arg <- "smd"
    #
    chklength(se.smd, k.All, arg)
    chklength(studlab, k.All, arg)
    #
    # Subset and exclude studies
    #
    if (!missing.subset)
      if ((is.logical(subset) & (sum(subset) > k.All)) ||
          (length(subset) > k.All))
        stop("Length of argument 'subset' is larger than number of studies.")
    #
    if (!missing.exclude)
      if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
          (length(exclude) > k.All))
        stop("Length of argument 'exclude' is larger than number of studies.")
  }
  
  
  method <- setchar(method, c("HH", "CS"))
  #
  if (method == "HH") {
    lnOR <- smd * pi / sqrt(3)
    selnOR <- se.smd * pi / sqrt(3)
  }
  else if (method == "CS") {
    lnOR <- smd * 1.65
    selnOR <- se.smd * 1.65
  }
  #
  chklogical(backtransf)
  
  
  if (is.meta) {
    if (is.null(mdat$subgroup))
      res <- metagen(lnOR, selnOR, studlab = mdat$studlab,
                     data = mdat,
                     subset = mdat$subset, exclude = mdat$exclude,
                     cluster = mdat$cluster,
                     #
                     sm = "OR",
                     #
                     level = mdat$level, level.ma = mdat$level.ma,
                     common = mdat$common,
                     random = mdat$random,
                     overall = mdat$overall,
                     overall.hetstat = mdat$overall.hetstat,
                     #
                     method.random.ci = mdat$method.random.ci,
                     adhoc.hakn.ci = mdat$adhoc.hakn.ci,
                     #
                     prediction = mdat$prediction,
                     method.predict = mdat$method.predict,
                     adhoc.hakn.pi = mdat$adhoc.hakn.pi,
                     level.predict = mdat$level.predict,
                     #
                     method.tau = mdat$method.tau,
                     method.tau.ci = mdat$method.tau.ci,
                     level.hetstat = mdat$level.hetstat,
                     tau.common = mdat$tau.common,
                     detail.tau = mdat$detail.tau,
                     #
                     null.effect = 0,
                     #
                     method.bias = mdat$method.bias,
                     #
                     backtransf = backtransf,
                     #
                     text.common = mdat$text.common,
                     text.random = mdat$text.random,
                     text.predict = mdat$text.predict,
                     text.w.common = mdat$text.w.common,
                     text.w.random = mdat$text.w.random,
                     #
                     title = mdat$title, complab = mdat$complab,
                     outclab = mdat$outclab,
                     #
                     label.e = mdat$label.e, label.c = mdat$label.c,
                     label.left = mdat$label.left,
                     label.right = mdat$label.right,
                     col.label.left = mdat$col.label.left,
                     col.label.right = mdat$col.label.right,
                     #
                     control = mdat$control)
    else
      res <- metagen(lnOR, selnOR, studlab = mdat$studlab,
                     data = mdat,
                     subset = mdat$subset, exclude = mdat$exclude,
                     cluster = mdat$cluster,
                     #
                     sm = "OR",
                     #
                     level = mdat$level, level.ma = mdat$level.ma,
                     common = mdat$common,
                     random = mdat$random,
                     overall = mdat$overall,
                     overall.hetstat = mdat$overall.hetstat,
                     #
                     method.random.ci = mdat$method.random.ci,
                     adhoc.hakn.ci = mdat$adhoc.hakn.ci,
                     #
                     prediction = mdat$prediction,
                     method.predict = mdat$method.predict,
                     adhoc.hakn.pi = mdat$adhoc.hakn.pi,
                     level.predict = mdat$level.predict,
                     #
                     method.tau = mdat$method.tau,
                     method.tau.ci = mdat$method.tau.ci,
                     level.hetstat = mdat$level.hetstat,
                     tau.common = mdat$tau.common,
                     detail.tau = mdat$detail.tau,
                     #
                     null.effect = 0,
                     #
                     method.bias = mdat$method.bias,
                     #
                     backtransf = backtransf,
                     #
                     text.common = mdat$text.common,
                     text.random = mdat$text.random,
                     text.predict = mdat$text.predict,
                     text.w.common = mdat$text.w.common,
                     text.w.random = mdat$text.w.random,
                     #
                     title = mdat$title, complab = mdat$complab,
                     outclab = mdat$outclab,
                     #
                     label.e = mdat$label.e, label.c = mdat$label.c,
                     label.left = mdat$label.left,
                     label.right = mdat$label.right,
                     col.label.left = mdat$col.label.left,
                     col.label.right = mdat$col.label.right,
                     #
                     subgroup = mdat$subgroup,
                     subgroup.name = mdat$subgroup.name,
                     print.subgroup.name = mdat$print.subgroup.name,
                     sep.subgroup = mdat$sep.subgroup,
                     test.subgroup = mdat$test.subgroup,
                     prediction.subgroup = mdat$prediction.subgroup,
                     #
                     control = mdat$control)
  }
  else {
    dat <- data.frame(smd, se.smd, lnOR, selnOR, OR = exp(lnOR), studlab)
    dat$subset <- subset
    dat$exclude <- exclude
    #
    res <- metagen(lnOR, selnOR, studlab = studlab,
                   data = dat, subset = subset, exclude = exclude,
                   sm = "OR", backtransf = backtransf, ...)
  }
  #
  res$method.smd2or <- method
  
  res
}

#' Cochrane review: Test for funnel plot asymmetry
#' 
#' @description
#' Conduct a test for funnel plot asymmetry for all outcomes in a
#' Cochrane review of intervention studies
#' 
#' @aliases metabias.rm5 metabias.cdir 
#' 
#' @param x An object of class \code{rm5} or \code{cdir}.
#' @param comp.no Comparison number.
#' @param outcome.no Outcome number.
#' @param method.bias A character string indicating which test for
#'   small-study effects is to be used for all outcomes. Either
#'   \code{"rank"}, \code{"linreg"}, or \code{"mm"}, can be
#'   abbreviated. See function \code{\link{metabias}}
#' @param method.bias.binary A character string indicating which test
#'   is to be used for binary outcomes. Either \code{"rank"},
#'   \code{"linreg"}, \code{"mm"}, \code{"count"}, \code{"score"}, or
#'   \code{"peters"}, can be abbreviated. See function
#'   \code{\link{metabias}}
#' @param method.bias.or A character string indicating which test is
#'   to be used for binary outcomes with odds ratio as summary
#'   measure. Either \code{"rank"}, \code{"linreg"}, \code{"mm"},
#'   \code{"count"}, \code{"score"}, or \code{"peters"}, can be
#'   abbreviated. See function \code{\link{metabias}}
#' @param k.min Minimum number of studies to perform test for
#'   small-study effects.
#' @param ... Additional arguments (ignored at the moment)
#' 
#' @details
#' This function can be used to conduct a test for funnel plot
#' asymmetry for all or selected meta-analyses in a Cochrane review of
#' intervention studies (Higgins et al, 2023).
#' 
#' The R function \code{\link{metacr}} is called internally.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabias}}, \code{\link{metacr}},
#'   \code{\link{read.rm5}}, \code{\link{read.cdir}},
#'   \code{\link{summary.rm5}}, \code{\link{summary.cdir}}
#' 
#' @references
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
#' VA (editors) (2023):
#' \emph{Cochrane Handbook for Systematic Reviews of Interventions
#'   Version 6.4 (updated August 2023)}.
#' Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}
#' 
#' @keywords htest
#' 
#' @examples
#' # Locate export data file "Fleiss1993_CR.csv" in sub-directory of
#' # package "meta"
#' #
#' filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
#' Fleiss1993_CR <- read.rm5(filename)
#' 
#' # Print results for all tests of small-study effects
#' #
#' metabias(Fleiss1993_CR, k.min = 5)
#' 
#' # Print result of test of small-study effects for second outcome in
#' # first comparison
#' #
#' metabias(Fleiss1993_CR, comp.no = 1, outcome.no = 2, k.min = 5)
#' 
#' @method metabias rm5
#' @export

metabias.rm5 <- function(x, comp.no, outcome.no,
                         method.bias = "linreg",
                         method.bias.binary = method.bias,
                         method.bias.or = "score",
                         k.min = 10, ...) {
  
  
  #
  #
  # (1) Check for rm5 object
  #
  #
  chkclass(x, "rm5")
  
  
  if (missing(comp.no))
    comp.no <- unique(x$comp.no)
  
  
  n <- 1
  #
  for (i in comp.no) {
    if (missing(outcome.no))
      jj <- unique(x$outcome.no[x$comp.no == i])
    else
      jj <- outcome.no
    for (j in jj) {
      #
      m1 <- metacr(x, i, j)
      #
      if (inherits(m1, "metabin")) {
        if (m1$sm == "OR")
          mb1 <- metabias(m1, k.min = k.min, method.bias = method.bias.or)
        else 
          mb1 <- metabias(m1, k.min = k.min, method.bias = method.bias.binary)
      }
      else
        mb1 <- metabias(m1, k.min = k.min, method.bias = method.bias)
      #
      if (!is.null(mb1$estimate)) {
        if (n > 1)
          cat("\n*****\n\n")
        print(mb1)
        #
        n <- n + 1
      }
    }
  }
  
  invisible(NULL)
}





#' @rdname metabias.rm5
#' @method metabias cdir
#' @export

metabias.cdir <- function(x, comp.no, outcome.no,
                          method.bias = "linreg",
                          method.bias.binary = method.bias,
                          method.bias.or = "score",
                          k.min = 10, ...) {
  
  
  #
  #
  # (1) Check for cdir object
  #
  #
  chkclass(x, "cdir")
  
  
  if (missing(comp.no))
    comp.no <- unique(x$data$comp.no)
  
  
  n <- 1
  #
  for (i in comp.no) {
    if (missing(outcome.no))
      jj <- unique(x$data$outcome.no[x$data$comp.no == i])
    else
      jj <- outcome.no
    for (j in jj) {
      #
      m1 <- metacr(x, i, j)
      #
      if (inherits(m1, "metabin")) {
        if (m1$sm == "OR")
          mb1 <- metabias(m1, k.min = k.min, method.bias = method.bias.or)
        else 
          mb1 <- metabias(m1, k.min = k.min, method.bias = method.bias.binary)
      }
      else
        mb1 <- metabias(m1, k.min = k.min, method.bias = method.bias)
      #
      if (!is.null(mb1$estimate)) {
        if (n > 1)
          cat("\n*****\n\n")
        print(mb1)
        #
        n <- n + 1
      }
    }
  }
  
  invisible(NULL)
}

#' Return subset of pairwise object
#' 
#' @description
#' The \code{subset} method returns a subset of a pairwise object.
#' 
#' @param x An object of class \code{pairwise}.
#' @param subset A logical expression indicating rows to keep; missing values
#'   are taken as false.
#' @param \dots Additional arguments.
#' @return A pairwise object is returned.
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' @seealso \code{\link{pairwise}}, \code{\link[metabook]{Franchini2012}}
#' 
#' @examples
#' # Transform data from arm-based format to contrast-based format
#' pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
#'   n = list(n1, n2, n3),
#'   mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
#'   data = Franchini2012, studlab = Study)
#' # Subset without Lieberman studies
#' subset(pw1, !grepl("Lieberman", studlab))[, 1:5]
#'
#' @method subset pairwise
#' @export

subset.pairwise <- function(x, subset, ...){
  
  chkclass(x, "pairwise")
  #
  attribs <- attributes(x)
  #
  attribs$names <- attribs$row.names <- NULL
  
  
  # Catch 'subset'
  #
  if (missing(subset))
    return(x)
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  subset <- catch("subset", mc, x, sfsp)
  if (is.null(subset))
    subset <- catch("subset", mc, x$data, sfsp)
  
  
  # Select subset
  #
  x.df <- x
  class(x.df) <- "data.frame"
  #
  res <- subset(x.df, subset, ...)
  
  # Check whether 'reference.group' is available in the subset
  #
  reference.group <- attr(x, "reference.group")
  #
  trts <- sort(unique(c(res$treat1, res$treat2)))
  if (!(reference.group %in% trts))
    reference.group <- trts[1]
  
  
  # Return subset
  #
  for (i in names(attribs))
    attr(res, i) <- attr(x, i)
  #
  class(res) <- unique(c("pairwise", class(res)))
  #
  res
}

#' Create study labels for forest plot
#' 
#' @description
#' Create study labels for forest plot.
#' 
#' @details
#' This auxiliary function can be used to create study labels in JAMA
#' or Lancet layout which can be added to a forest plot using argument
#' 'studlab'.
#' 
#' @param object An object of class \code{meta}.
#' @param author An optional vector providing study authors.
#' @param year An optional vector providing year of publication.
#' @param citation An optional vector providing citation numbers.
#' @param layout A character string specifying layout. Either "JAMA"
#'   or "Lancet".
#' @param data An optional data frame containing the study
#'   information.
#' @param \dots Additional arguments (ignored at the moment).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.meta}}
#' 
#' @examples
#' data(Fleiss1993bin)
#' 
#' refs <- 20 + 1:7
#'
#' m <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
#'  studlab = study, sm = "OR", random = FALSE)
#' 
#' forest(m,
#'   studlab = labels(m, year = year, citation = refs, layout = "JAMA"),
#'   layout = "JAMA", fontfamily = "Times", fontsize = 10)
#' 
#' forest(m,
#'   studlab = labels(m, year = year, citation = refs, layout = "Lancet"))
#' 
#' @method labels meta
#' @export

labels.meta <- function(object,
                        author = object$studlab, year = "", citation = NULL,
                        layout = "JAMA",
                        data = object$data,
                        ...) {
  
  chkclass(object, "meta")
  layout <- setchar(layout, c("JAMA", "Lancet"))
  
  
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  # Catch 'author', 'year', and 'citation' from data:
  #
  if (!missing(author))
    author <- catch("author", mc, data, sfsp)
  #
  if (!missing(year))
    year <- catch("year", mc, data, sfsp)
  else
    year <- rep_len("", length(author))
  #
  if (!missing(citation))
    citation <- catch("citation", mc, data, sfsp)
  else
    citation <- seq_along(author)
  #
  if (length(author) != length(year))
    stop("Arguments 'author' and 'year' must be of same length.",
         call. = FALSE)
  #
  if (length(author) != length(citation))
    stop("Arguments 'author' and 'citation' must be of same length.",
         call. = FALSE)
  
  
  res <- rep_len(NA, length(author))
  #
  if (layout == "JAMA") {
    for (i in seq(along = author)) {
      res[i] <-
        if (year[i] == "" | is.na(year[i]))
          as.expression(substitute(study^ref,
                                   list(study = author[i],
                                        ref = citation[i],
                                        year = year[i])))
        else
          as.expression(substitute(study^ref~(year),
                                   list(study = author[i],
                                        ref = citation[i],
                                        year = year[i])))
    }
  }
  else if (layout == "Lancet") {
    for (i in seq(along = author)) {
      res[i] <-
        if (year[i] == "" | is.na(year[i]))
          as.expression(substitute(study^ref,
                                   list(study = author[i],
                                        ref = citation[i],
                                        year = year[i])))
        else
          as.expression(substitute(study~(year)^ref,
                                   list(study = author[i],
                                        ref = citation[i],
                                        year = year[i])))
    }
  }
  #
  res
}

catarg <- function(x, newline = TRUE, end = "") {
  xname <- x
  x <- gsub(" ", "", x)
  #
  if (newline)
    cat("- ")
  #
  if (is.null(.settings[[x]]))
    cat(paste0(xname, ' = NULL', end, '\n'))
  else if (is.character(.settings[[x]]))
    cat(paste0(xname, ' = "', .settings[[x]], '"', end, '\n'))
  else
    cat(paste0(xname, ' = ', .settings[[x]], end, "\n"))
  #
  invisible(NULL)
}


specificSettings <- function(args, new, setting, quietly = FALSE) {
  isnull.old <- as.vector(unlist(lapply(.settings[args], is.null)))
  ischar.old <- as.vector(unlist(lapply(.settings[args], is.character)))
  old <- rep("character", length(isnull.old))
  old[!isnull.old] <- as.vector(unlist(.settings[args]))
  #
  isnull.new <- as.vector(unlist(lapply(new, is.null)))
  ischar.new <- as.vector(unlist(lapply(new, is.character)))
  new <- as.vector(unlist(new))
  #
  label.old <- ifelse(isnull.old, "NULL",
               ifelse(ischar.old, paste0("\"", old, "\""), old))
  label.new <- ifelse(ischar.new, paste0("\"", new, "\""), new)
  #
  sel <- new != old
  if (any(sel)) {
    tdata <- data.frame(argument = c("Argument",
                                     "--------",
                                     args[sel]),
                        space1 = rep("  ", along = c(1:2, sel)),
                        new = c("New value",
                                "---------",
                                label.new[sel]),
                        space2 = rep("  ", along = c(1:2, sel)),
                        previous = c("Previous value",
                                     "--------------",
                                     label.old[sel]))
    
    names(tdata) <- c("--------", "", "---------",
                      "", "--------------")
    #
    if (!quietly) {
      cat(paste0("\n** Use ", setting, " (R package meta) **\n\n"))
      prmatrix(tdata, quote = FALSE, right = FALSE,
               rowlab = rep_len("", 2 + sum(sel)))
    }
    #
    for (i in seq(along = args)) {
      new.i <- new[i]
      if (!ischar.new[i]) {
        if (new.i %in% c("TRUE", "FALSE"))
          new.i <- as.logical(new.i)
        else
          new.i <- as.numeric(new.i)
      }
      setOption(args[i], new.i)
    }
  }
  else {
    if (!quietly) {
      if (substring(setting, 1, 1) == "s")
        setting <- paste0("S", substring(setting, 2))
      cat(paste0("\n** ", setting, " already in use (R package meta). **\n\n"))
    } 
  }
}


setcharacter <- function(argname, args, set = NULL, length = 1,
                         NULL.ok = FALSE, ignore.other = FALSE,
                         logical.ok = FALSE) {
  id <- argid(names(args), argname)
  #
  if (!is.na(id)) {
    val <- args[[id]]
    #
    if (NULL.ok & is.null(val)) {
      setOption(argname, val)
      return(invisible(NULL))
    }
    #
    if (logical.ok & is.logical(val)) {
      setOption(argname, val)
      return(invisible(NULL))
    }
    #
    if (!is.character(val) & ignore.other)
      return(invisible(id))
    #
    if (!is.null(set))
      val <- setchar(val, set, name = argname)
    else
      chkchar(val, length = length, name = argname)
    #
    setOption(argname, val)
  }
  #
  invisible(id)
}


setcolor <- function(argname, args) {
  id <- argid(names(args), argname)
  #
  if (!is.na(id)) {
    val <- args[[id]]
    chkcolor(val, name = argname)
    setOption(argname, val)
  }
  #
  invisible(id)
}


setlevel <- function(argname, args) {
  id <- argid(names(args), argname)
  #
  if (!is.na(id)) {
    val <- args[[id]]
    chklevel(val, name = argname)
    setOption(argname, val)
  }
  #
  invisible(id)
}


setlogical <- function(argname, args, NULL.ok = FALSE,
                       ignore.other = FALSE) {
  id <- argid(names(args), argname)
  #
  if (!is.na(id)) {
    val <- args[[id]]
    #
    if (NULL.ok & is.null(val)) {
      setOption(argname, val)
      return(invisible(NULL))
    }
    #
    if (!is.logical(val) & ignore.other)
      return(invisible(id))
    #
    chklogical(val, name = argname)
    setOption(argname, val)
  }
  #
  invisible(id)
}


setnumeric <- function(argname, args, NULL.ok = FALSE) {
  id <- argid(names(args), argname)
  #
  if (!is.na(id)) {
    val <- args[[id]]
    #
    if (NULL.ok & is.null(val)) {
      setOption(argname, val)
      return(invisible(NULL))
    }
    #
    chknumeric(val, min = 0, length = 1, name = argname)
    setOption(argname, val)
  }
  #
  invisible(id)
}

TE.seTE.ci <- function(lower, upper, level = 0.95,
                       df = rep_len(NA, length(lower))) {
  #
  # Check arguments
  #
  if (missing(lower))
    stop("Mandatory argument 'lower' missing.", call. = FALSE)
  if (missing(upper))
    stop("Mandatory argument 'upper' missing.", call. = FALSE)
  #
  k <- length(lower)
  arg <- "lower"
  chklength(upper, k, arg)
  chklength(df, k, arg)
  #
  if (any(lower == upper, na.rm = TRUE)) {
    sel <- lower == upper
    lower[sel] <- NA
    upper[sel] <- NA
  }
  #
  if (any(lower > upper, na.rm = TRUE))
    stop("Lower limit must be smaller than upper limit.", call. = FALSE)
  #
  chklevel(level, length = 0)
  #
  # Parmar et al. (1998), Stat Med
  #
  # Section 4.1 Indirect variance estimation
  #
  # Equation (7)
  #
  varTE <- ifelse(is.na(df),
  ((upper - lower) /
   (2 * qnorm((1 - level) / 2, lower.tail = FALSE)))^2,
  ((upper - lower) /
   (2 * qt((1 - level) / 2, df = df, lower.tail = FALSE)))^2)
  #
  seTE <- sqrt(varTE)
  #
  res <- list(TE = lower + (upper - lower) / 2, seTE = seTE,
              lower = lower, upper = upper,
              level = level, df = df)
  #
  res
}
seTE.pval <- function(TE, pval, df = rep_len(NA, length(TE))) {
  #
  # Check arguments
  #
  if (missing(TE))
    stop("Mandatory argument 'TE' missing.", call. = FALSE)
  if (missing(pval))
    stop("Mandatory argument 'pval' missing", call. = FALSE)
  #
  k <- length(TE)
  arg <- "TE"
  chklength(pval, k, arg)
  chklength(df, k, arg)
  #
  if (any(pval <= 0, na.rm = TRUE) | any(pval >= 1, na.rm = TRUE))
    stop("No valid value for p-value", call. = FALSE)
  #
  # Parmar et al. (1998), Stat Med
  #
  # Equation (7)
  #
  varTE <- ifelse(is.na(df),
  (TE / qnorm(pval / 2, lower.tail = FALSE))^2,
  (TE / qt(pval / 2, df = df, lower.tail = FALSE))^2)
  seTE <- sqrt(varTE)
  #
  res <- list(TE = TE, seTE = seTE, pval = pval)
  #
  res
}


NULL

meta2meth <- function(x, outclab = NULL) {
  list(sm = x$sm,
       method = x$method,
       method.random = x$method.random,
       three.level = replaceNULL(x$three.level, NA),
       #
       k = x$k,
       k.study = x$k.study,
       k.all = x$k.all,
       k.TE = x$k.TE,
       #
       level = x$level.ma,
       level.ma = x$level.ma,
       level.predict = x$level.predict,
       #
       common = x$common,
       random = x$random,
       overall = x$overall,
       overall.hetstat = x$overall.hetstat,
       #
       method.common.ci = x$method.common.ci,
       method.random.ci = x$method.random.ci,
       adhoc.hakn.ci = x$adhoc.hakn.ci,
       method.tau = x$method.tau,
       tau.preset = replaceNULL(x$tau.preset),
       TE.tau = replaceNULL(x$TE.tau),
       tau.common = replaceNULL(x$tau.common, FALSE),
       #
       method.I2 = replaceNULL(x$method.I2, "Q"),
       #
       prediction = x$prediction,
       prediction.subgroup =
         replaceNULL(x$prediction.subgroup, FALSE),
       method.predict = x$method.predict,
       adhoc.hakn.pi = x$adhoc.hakn.pi,
       #
       method.bias = "",
       null.effect = x$null.effect,
       #
       title = x$title,
       complab = x$complab,
       outclab = replaceNULL(outclab, x$outclab),
       #
       label.e = x$label.e,
       label.c = x$label.c,
       label.left = x$label.left,
       label.right = x$label.right,
       #
       print.subgroup.name = FALSE,
       sep.subgroup = "",
       warn = replaceNULL(x$warn, FALSE),
       #
       backtransf = x$backtransf,
       pscale = replaceNULL(x$pscale, 1),
       irscale = replaceNULL(x$irscale, 1),
       irunit = replaceNULL(x$ir.unit))
}


overall2meta <- function(x, common, random, prediction, name) {
  res <-
    list(
      TE.common = if (common) x$TE.common else NULL,
      seTE.common = if (common) x$seTE.common else NULL,
      lower.common = if (common) x$lower.common else NULL,
      upper.common = if (common) x$upper.common else NULL,
      statistic.common = if (common) x$statistic.common else NULL,
      pval.common = if (common) x$pval.common else NULL,
      text.common = if (common) x$text.common else NULL,
      #
      TE.random = if (random) x$TE.random else NULL,
      seTE.random = if (random) x$seTE.random else NULL,
      lower.random = if (random) x$lower.random else NULL,
      upper.random = if (random) x$upper.random else NULL,
      statistic.random = if (random) x$statistic.random else NULL,
      pval.random = if (random) x$pval.random else NULL,
      text.random = if (random) x$text.random else NULL,
      #
      df.random = if (random) replaceNULL(x$df.random) else NULL,
      df.hakn = if (random) replaceNULL(x$df.hakn) else NULL,
      df.kero = if (random) replaceNULL(x$df.kero) else NULL,
      #
      n.harmonic.mean.ma = 1 / mean(1 / replaceNULL(x$n)),
      t.harmonic.mean.ma = 1 / mean(1 / replaceNULL(x$time)),
      #
      seTE.predict = if (prediction) x$seTE.predict else NULL,
      df.predict = if (prediction) x$df.predict else NULL,
      lower.predict = if (prediction) x$lower.predict else NULL,
      upper.predict = if (prediction) x$upper.predict else NULL,
      #
      n.e = sum(replaceNULL(x$n.e)),
      n.c = sum(replaceNULL(x$n.c)),
      #
      Q = x$Q[1],
      df.Q = x$df.Q[1],
      pval.Q = x$pval.Q[1],
      #
      tau2 = x$tau2,
      lower.tau2 = x$lower.tau2,
      upper.tau2 = x$upper.tau2,
      se.tau2 = replaceNULL(x$se.tau2),
      tau = x$tau,
      lower.tau = x$lower.tau,
      upper.tau = x$upper.tau,
      #
      H = x$H,
      lower.H = x$lower.H,
      upper.H = x$upper.H,
      I2 = x$I2,
      lower.I2 = x$lower.I2,
      upper.I2 = x$upper.I2,
      Rb = x$Rb,
      lower.Rb = x$lower.Rb,
      upper.Rb = x$upper.Rb,
      #
      Q.b.common = NA,
      Q.b.random = NA,
      df.Q.b = NA,
      pval.Q.b.common = NA,
      pval.Q.b.random = NA
      )
  #
  res
}


subgr2meta <- function(x, common, random, prediction, name) {
  n.subgr <- length(x$subgroup.levels)
  #
  res <- list(
    studlab =
      c(if (common)
          paste0(x$subgroup.levels,
                if (random | prediction) " (common effect)"),
        if (random)
          paste0(x$subgroup.levels,
                if (common | prediction) " (random effects)"),
        if (prediction)
          paste0(x$subgroup.levels,
                if (common | random) " (prediction)")
        ),
    #
    subgroup = NA,
    #
    subgroup.levels = name,
    #
    TE =
      c(if (common) x$TE.common.w,
        if (random) x$TE.random.w,
        if (prediction) rep(NA, n.subgr)),
    #
    seTE =
      c(if (common) x$seTE.common.w,
        if (random) x$seTE.random.w,
        if (prediction) rep(NA, n.subgr)),
    lower =
      c(if (common) x$lower.common.w,
        if (random) x$lower.random.w,
        if (prediction) x$lower.predict.w),
    upper =
      c(if (common) x$upper.common.w,
        if (random) x$upper.random.w,
        if (prediction) x$upper.predict.w),
    statistic =
      c(if (common) x$statistic.common.w,
        if (random) x$statistic.random.w,
        if (prediction) rep(NA, n.subgr)),
    pval =
      c(if (common) x$pval.common.w,
        if (random) x$pval.random.w,
        if (prediction) rep(NA, n.subgr)),
    #
    TE.common = x$TE.common,
    seTE.common = x$seTE.common,
    lower.common = x$lower.common,
    upper.common = x$upper.common,
    statistic.common = x$statistic.common,
    pval.common = x$pval.common,
    text.common = if (common) x$text.common else NULL,
    #
    TE.random = x$TE.random,
    seTE.random = x$seTE.random,
    lower.random = x$lower.random,
    upper.random = x$upper.random,
    statistic.random = x$statistic.random,
    pval.random = x$pval.random,
    text.random = if (random) x$text.random else NULL,
    #
    lower.predict = x$lower.predict,
    upper.predict = x$upper.predict,
    text.predict = if (prediction) x$text.predict else NULL,
    #
    df.random = replaceNULL(x$df.random),
    df.hakn = replaceNULL(x$df.hakn),
    df.kero = replaceNULL(x$df.kero),
    #
    n.harmonic.mean = replaceNULL(x$n.harmonic.mean.w),
    t.harmonic.mean = replaceNULL(x$t.harmonic.mean.w),
    #
    n.e = replaceNULL(x$n.e),
    n.c = replaceNULL(x$n.c),
    #
    Q = x$Q[1],
    df.Q = x$df.Q[1],
    pval.Q = x$pval.Q[1],
    #
    tau2 = x$tau2,
    tau = x$tau,
    H = x$H,
    lower.H = x$lower.H,
    upper.H = x$upper.H,
    I2 = x$I2,
    lower.I2 = x$lower.I2,
    upper.I2 = x$upper.I2,
    Rb = x$Rb,
    lower.Rb = x$lower.Rb,
    upper.Rb = x$upper.Rb,
    #
    Q.b.common = NA,
    Q.b.random = NA,
    df.Q.b = NA,
    pval.Q.b.common = NA,
    pval.Q.b.random = NA)
  #
  res$subgroup <- rep_len(name, length(res$TE))
  #
  res
}


overall2subgr <- function(x) {
  res <- list()
  #
  vars.c <- c("TE.common", "seTE.common", "lower.common", "upper.common",
              "statistic.common", "pval.common")
  #
  for (i in vars.c)
    res[[paste0(i, ".w")]] <- replaceNULL(x[[i]])
  #
  res[["w.common.w"]] <- rep(0, length(res[["lower.common.w"]]))
  #
  vars.r <- c("TE.random", "seTE.random", "lower.random", "upper.random",
              "statistic.random", "pval.random",
              "df.random", "df.hakn", "df.kero")
  #
  for (i in vars.r)
    res[[paste0(i, ".w")]] <- replaceNULL(x[[i]])
  #
  res[["w.random.w"]] <- rep(0, length(res[["lower.random.w"]]))
  #
  res[["n.harmonic.mean.ma"]] <- 1 / mean(1 / replaceNULL(x$n))
  res[["t.harmonic.mean.ma"]] <- 1 / mean(1 / replaceNULL(x$time))
  #
  res[["n.harmonic.mean.w"]] <- 1 / mean(1 / replaceNULL(x$n))
  res[["t.harmonic.mean.w"]] <- 1 / mean(1 / replaceNULL(x$time))
  #
  res[["n.e.w"]] <- sum(replaceNULL(x$n.e))
  res[["n.c.w"]] <- sum(replaceNULL(x$n.c))
  #
  vars <- c("k", "k.study", "k.all", "k.TE",
            "Q", "df.Q", "pval.Q",
            "tau2", "tau",
            "H", "lower.H", "upper.H",
            "I2", "lower.I2", "upper.I2",
            "Rb", "lower.Rb", "upper.Rb")
  #
  for (i in vars)
    res[[paste0(i, ".w")]] <- replaceNULL(x[[i]])
  #
  res$Q.w <- res$Q.w[1]
  res$df.Q.w <- res$df.Q.w[1]
  res$pval.Q.w <- res$pval.Q.w[1]
  #
  res
}


subgr2data <- function(x, common, random, prediction, name, debug = FALSE) {
  n.subgr <- length(x$subgroup.levels)
  #
  NAs <- rep_len(NA, n.subgr - 1)
  fillNAs <- rep_len(NA, n.subgr * (common + random + prediction - 1))
  #
  res <- list()
  #
  res$name <-
    c(if (common)
        paste(rep(name, n.subgr),
              if (random | prediction)
                "(common effect)"),
      if (random)
        paste(rep(name, n.subgr),
              if (common | prediction)
                "(random effects)", n.subgr),
      if (prediction)
        paste(rep(name, n.subgr),
              if (common | random)
                "(prediction)"))
  #
  res$studlab <-
      c(if (common)
          paste0(x$subgroup.levels,
                if (random | prediction) " (common effect)"),
        if (random)
          paste0(x$subgroup.levels,
                if (common | prediction) " (random effects)"),
        if (prediction)
          paste0(x$subgroup.levels,
                if (common | random) " (prediction)"))
  #
  res$subgroup <-
    c(if (common) rep(name, n.subgr),
      if (random) rep(name, n.subgr),
      if (prediction) rep(name, n.subgr))
  #
  res$type <-
    c(if (common) rep("square", n.subgr),
      if (random) rep("square", n.subgr),
      if (prediction) rep("predict", n.subgr))
  #
  res$model <-
    c(if (common) rep("common", n.subgr),
      if (random) rep("random", n.subgr),
      if (prediction) rep("predict", n.subgr))
  #
  res$method <-
    c(if (common) rep(x$method, n.subgr),
      if (random) rep(x$method.random, n.subgr),
      if (prediction) rep(x$method.predict, n.subgr))
  #
  res$method.tau <-
    c(if (common) rep("", n.subgr),
      if (random) rep(x$method.tau, n.subgr),
      if (prediction) rep("", n.subgr))
  #
  res$three.level <-
    c(if (common) rep(FALSE, n.subgr),
      if (random) rep(x$three.level, n.subgr),
      if (prediction) rep(FALSE, n.subgr))
  #
  res$method.tau.ci <-
    c(if (common) rep("", n.subgr),
      if (random) rep(x$method.tau.ci, n.subgr),
      if (prediction) rep("", n.subgr))
  #
  res$tau.preset <-
    c(if (common) rep(NA, n.subgr),
      if (random) rep(replaceNULL(x$tau.preset, NA), n.subgr),
      if (prediction) rep(NA, n.subgr))
  #
  res$method.I2 <-
    c(if (common) rep(x$method.I2, n.subgr),
      if (random) rep(x$method.I2, n.subgr),
      if (prediction) rep("", n.subgr))
  #
  res$method.common.ci <-
    c(if (common) rep(x$method.common.ci, n.subgr),
      if (random) rep("", n.subgr),
      if (prediction) rep("", n.subgr))
  #
  res$method.random.ci <-
    c(if (common) rep("", n.subgr),
      if (random) rep(x$method.random.ci, n.subgr),
      if (prediction) rep("", n.subgr))
  #
  res$df.random <-
    c(if (common) rep(NA, n.subgr),
      if (random) rep(x$df.random, n.subgr),
      if (prediction) rep(NA, n.subgr))
  #
  res$adhoc.hakn.ci <-
    c(if (common) rep("", n.subgr),
      if (random) rep(x$adhoc.hakn.ci, n.subgr),
      if (prediction) rep("", n.subgr))
  #
  res$method.predict <-
    c(if (common) rep("", n.subgr),
      if (random) rep("", n.subgr),
      if (prediction) rep(x$method.predict, n.subgr))
  #
  res$df.predict <-
    c(if (common) rep(NA, n.subgr),
      if (random) rep(NA, n.subgr),
      if (prediction) rep(x$df.predict, n.subgr))
  #
  res$adhoc.hakn.pi <-
    c(if (common) rep("", n.subgr),
      if (random) rep("", n.subgr),
      if (prediction) rep(x$adhoc.hakn.pi, n.subgr))
  #
  res$rho <-
    c(if (common) rep(NA, n.subgr),
      if (random) rep(replaceNULL(x$rho), n.subgr),
      if (prediction) rep(NA, n.subgr))
  #
  res$n.e <-
    c(if (common) replaceNULL(x$n.e.w, rep(NA, n.subgr)),
      if (random) replaceNULL(x$n.e.w, rep(NA, n.subgr)),
      if (prediction) replaceNULL(x$n.e.w, rep(NA, n.subgr)))
  #
  res$n.c <-
    c(if (common) replaceNULL(x$n.c.w, rep(NA, n.subgr)),
      if (random) replaceNULL(x$n.c.w, rep(NA, n.subgr)),
      if (prediction) replaceNULL(x$n.c.w, rep(NA, n.subgr)))
  #
  res$TE <-
    c(if (common) x$TE.common.w,
      if (random) x$TE.random.w,
      if (prediction) rep(NA, n.subgr))
  #
  res$seTE <-
    c(if (common) x$seTE.common.w,
      if (random) x$seTE.random.w,
      if (prediction) rep(NA, n.subgr))
  #
  res$lower <-
    c(if (common) x$lower.common.w,
      if (random) x$lower.random.w,
      if (prediction) x$lower.predict.w)
  #
  res$upper <-
    c(if (common) x$upper.common.w,
      if (random) x$upper.random.w,
      if (prediction) x$upper.predict.w)
  #
  res$statistic <-
    c(if (common) x$statistic.common.w,
      if (random) x$statistic.random.w,
      if (prediction) rep(NA, n.subgr))
  #
  res$pval <-
      c(if (common) x$pval.common.w,
        if (random) x$pval.random.w,
        if (prediction) rep(NA, n.subgr))
  #
  res$w.random <- res$w.common <-
    c(if (common) x$w.common.w,
      if (random) x$w.random.w,
      if (prediction) rep(NA, n.subgr))
  #
  if (!is.null(names(res$TE)))
    names(res$seTE) <- names(res$lower) <- names(res$upper) <-
      names(res$statistic) <- names(res$pval) <- names(res$TE)
  #
  res$k <- crp(x$k.w, common, random, prediction)
  res$k.study <- crp(x$k.study.w, common, random, prediction)
  res$k.all <- crp(x$k.all.w, common, random, prediction)
  res$k.TE <- crp(x$k.TE.w, common, random, prediction)
  #
  res$Q <- c(x$Q.w, fillNAs)
  res$df.Q <- c(x$k.w - 1, fillNAs)
  res$pval.Q <- c(x$pval.Q.w, fillNAs)
  #
  res$tau2 <- c(x$tau2.w, fillNAs)
  res$tau <- c(x$tau.w, fillNAs)
  #
  res$H <- c(x$H.w, fillNAs)
  res$lower.H <- c(x$lower.H.w, fillNAs)
  res$upper.H <- c(x$upper.H.w, fillNAs)
  #
  res$I2 <- c(x$I2.w, fillNAs)
  res$lower.I2 <- c(x$lower.I2.w, fillNAs)
  res$upper.I2 <- c(x$upper.I2.w, fillNAs)
  #
  res$Rb <- c(x$Rb.w, fillNAs)
  res$lower.Rb <- c(x$lower.Rb.w, fillNAs)
  res$upper.Rb <- c(x$upper.Rb.w, fillNAs)
  #
  res$Q.b <-
    c(if (common) addNAs2var(x$Q.b.common, n.subgr),
      if (random) addNAs2var(x$Q.b.random, n.subgr),
      if (prediction) rep(NA, n.subgr))
  #
  res$df.Q.b <-
    c(if (common)
        addNAs2var(unlist(lapply(x$df.Q.b.common, compr)), n.subgr),
      if (random)
        addNAs2var(unlist(lapply(x$df.Q.b.random, compr)), n.subgr),
      if (prediction)
        rep(NA, n.subgr))
  #
  res$pval.Q.b <-
    c(if (common) addNAs2var(x$pval.Q.b.common, n.subgr),
      if (random) addNAs2var(x$pval.Q.b.random, n.subgr),
      if (prediction) rep(NA, n.subgr))
  #
  if (debug)
    print(res)
  #
  res <- as.data.frame(res, row.names = seq_len(length(res$TE)))
  #
  res
}


overall2data <- function(x, common, random, prediction, name, subgroup,
                         debug = FALSE) {
  res <- list()
  #
  res$studlab <-
    c(if (common)
        paste0(name,
              if (random | prediction) " (common effect)"),
      if (random)
        paste0(name,
              if (common | prediction) " (random effects)"),
      if (prediction)
        paste0(name,
              if (common | random) " (prediction)")
      )
  #
  res$type <-
    c(if (common) "square",
      if (random) "square",
      if (prediction) "predict")
  #
  res$model <-
    c(if (common) "common",
      if (random) "random",
      if (prediction) "predict")
  #
  res$method <-
    c(if (common) x$method,
      if (random) x$method.random,
      if (prediction) x$method.predict)
  #
  res$method.tau <-
    c(if (common) "",
      if (random) x$method.tau,
      if (prediction) "")
  #
  res$three.level <-
    c(if (common) FALSE,
      if (random) x$three.level,
      if (prediction) FALSE)
  #
  res$method.tau.ci <-
    c(if (common) "",
      if (random) x$method.tau.ci,
      if (prediction) "")
  #
  res$tau.preset <-
    c(if (common) NA,
      if (random) replaceNULL(x$tau.preset, NA),
      if (prediction) NA)
  #
  res$method.I2 <-
    c(if (common) x$method.I2,
      if (random) x$method.I2,
      if (prediction) "")
  #
  res$method.common.ci <-
    c(if (common) x$method.common.ci,
      if (random) "",
      if (prediction) "")
  #
  res$method.random.ci <-
    c(if (common) "",
      if (random) x$method.random.ci,
      if (prediction) "")
  #
  res$df.random <-
    c(if (common) NA,
      if (random) x$df.random,
      if (prediction) NA)
  #
  res$adhoc.hakn.ci <-
    c(if (common) "",
      if (random) x$adhoc.hakn.ci,
      if (prediction) "")
  #
  res$method.predict <-
    c(if (common) "",
      if (random) "",
      if (prediction) x$method.predict)
  #
  res$df.predict <-
    c(if (common) NA,
      if (random) NA,
      if (prediction) x$df.predict)
  #
  res$adhoc.hakn.pi <-
    c(if (common) "",
      if (random) "",
      if (prediction) x$adhoc.hakn.pi)
  #
  res$rho <-
    c(if (common) NA,
      if (random) replaceNULL(x$rho, NA),
      if (prediction) NA)
  #
  res$n.e <- sum(x$n.e, na.rm = TRUE)
  #
  res$n.c <- sum(x$n.c, na.rm = TRUE)
  #
  res$TE <-
    c(if (common) x$TE.common,
      if (random) x$TE.random,
      if (prediction) NA)
  #
  res$seTE <-
    c(if (common) x$seTE.common,
      if (random) x$seTE.random,
      if (prediction) NA)
  #
  res$lower <-
    c(if (common) x$lower.common,
      if (random) x$lower.random,
      if (prediction) x$lower.predict)
  #
  res$upper <-
    c(if (common) x$upper.common,
      if (random) x$upper.random,
      if (prediction) x$upper.predict)
  #
  res$statistic <-
    c(if (common) x$statistic.common,
      if (random) x$statistic.random,
      if (prediction) NA)
  #
  res$pval <-
    c(if (common) x$pval.common,
      if (random) x$pval.random,
      if (prediction) NA)
  #
  res$k <- crp(x$k, common, random, prediction)
  res$k.study <- crp(x$k.study, common, random, prediction)
  res$k.all <- crp(x$k.all, common, random, prediction)
  res$k.TE <- crp(x$k.TE, common, random, prediction)
  res$k.MH <- crp(x$k.MH, common, random, prediction)
  res$k0 <- crp(x$k0, common, random, prediction)
  #
  sumcrp <- common + random + prediction
  #
  res$Q <- c(x$Q[1], rep(NA, sumcrp - 1))
  res$df.Q <- c(x$df.Q[1], rep(NA, sumcrp - 1))
  res$pval.Q <- c(x$pval.Q[1], rep(NA, sumcrp - 1))
  #
  res$tau2 <- c(x$tau2, rep(NA, sumcrp - 1))
  res$tau <- c(x$tau, rep(NA, sumcrp - 1))
  #
  res$H <- c(x$H, rep(NA, sumcrp - 1))
  res$lower.H <- c(x$lower.H, rep(NA, sumcrp - 1))
  res$upper.H <- c(x$upper.H, rep(NA, sumcrp - 1))
  #
  res$I2 <- c(x$I2, rep(NA, sumcrp - 1))
  res$lower.I2 <- c(x$lower.I2, rep(NA, sumcrp - 1))
  res$upper.I2 <- c(x$upper.I2, rep(NA, sumcrp - 1))
  #
  res$Rb <- c(x$Rb, rep(NA, sumcrp - 1))
  res$lower.Rb <- c(x$lower.Rb, rep(NA, sumcrp - 1))
  res$upper.Rb <- c(x$upper.Rb, rep(NA, sumcrp - 1))
  #
  res$subgroup <- subgroup
  #
  if (debug)
    print(res)
  #
  res <- as.data.frame(res)
  #
  res
}


makeunique <- function(x, val = NA) {
  res <- unique(x)
  #
  if (length(res) != 1)
    res <- val
  #
  res
}


addNAs2var <- function(x, n) {
  if (n == 1)
    res <- x
  else {
    res <- vector("numeric", 0)
    #
    for (i in seq_along(x))
      res <- c(res, c(x[i], rep(NA, n - 1)))
  }
  #
  res
}


crp <- function(x, common, random, prediction, replace = NA) {
  x <- replaceNULL(x, replace)
  c(if (common) x, if (random) x, if (prediction) x)
}


compr <- function(x) {
  res <- paste(x, collapse = ", ")
  #
  if (all(!grepl(",", res)))
    res <- as.numeric(res)
  #
  res
}

#' Auxiliary functions for (back) transformations
#' 
#' @description
#' Auxiliary functions to (back) transform effect estimates or
#' confidence / prediction interval limit(s).
#' 
#' @details
#' Often in a meta-analysis, effect estimates are transformed before
#' calculating a weighted average. For example, the log odds ratio and
#' its standard error is used instead of the odds ratio in R function
#' \code{\link{metagen}}. To report the results of a meta-analysis,
#' effect estimates are typically back transformed to the original
#' scale. R package \bold{meta} provides some auxiliary functions for
#' (back) transformations.
#'
#' \subsection{Transformations}{
#' The following auxiliary functions are provided by R package
#' \bold{meta} to transform effect estimates or confidence /
#' prediction interval limits.
#'
#' \tabular{ll}{
#' \bold{Function} \tab \bold{Transformation} \cr
#' \code{cor2z} \tab Correlations to Fisher's Z transformed correlations \cr
#' \code{p2logit} \tab Proportions to logit transformed proportions \cr
#' \code{p2asin} \tab Proportions to arcsine transformed proportions \cr
#' \code{VE2logVR} \tab Vaccine efficacy / effectiveness to log vaccine ratio
#' }
#'
#' Note, no function for the Freeman-Tukey arcsine transformation is
#' provided as this transformation is based on the number of events
#' and sample sizes instead of the effect estimates.
#' 
#' R function \code{transf} is a wrapper function for the above and
#' additional transformations, e.g., the log transformation using
#' \code{\link[base]{log}} for odds or risk ratios. Argument \code{sm}
#' is mandatory to specify the requested transformation. It is also
#' possible to specify a different function with arguments \code{func}
#' and \code{args}.
#' }
#'
#' \subsection{Back transformations}{
#' The following auxiliary functions are available to back transform
#' effect estimates or confidence / prediction interval limits.
#'
#' \tabular{ll}{
#' \bold{Function} \tab \bold{Transformation} \cr
#' \code{asin2ir} \tab Freeman-Tukey arcsine transformed rates to rates \cr
#' \code{asin2p} \tab (Freeman-Tukey) arcsine transformed proportions to proportions \cr
#' \code{logit2p} \tab Logit transformed proportions to proportions \cr
#' \code{logVR2VE} \tab Log vaccine ratio to vaccine efficacy / effectiveness \cr
#' \code{z2cor} \tab Fisher's Z transformed correlations  to correlations
#' }
#' 
#' Argument \code{time} is mandatory in R function \code{asin2ir}.
#'
#' If argument \code{n} is provided in R function \code{asin2p},
#' Freeman-Tukey arcsine transformed proportions are
#' back transformed. Otherwise, arcsine transformed proportions are
#' back transformed.
#'
#' R function \code{backtransf} is a wrapper function for the above
#' and additional transformations, e.g., the exponential
#' transformation using \code{\link[base]{exp}} for log odds or log
#' risk ratios. Argument \code{sm} is mandatory to specify the
#' requested transformation. For the Freeman-Tukey transformations,
#' argument \code{n} or \code{time} is mandatory.
#'
#' It is also possible to specify a different function with arguments
#' \code{func} and \code{args}.
#' }
#' 
#' @param x Numerical vector with effect estimates, lower or upper
#'   confidence / prediction interval limit(s).
#' @param sm Summary measure.
#' @param func User-specified function for (back) transformation.
#' @param args Function arguments for user-specified function.
#' @param n Sample size(s) to back transform Freeman-Tukey transformed
#'   proportions.
#' @param time Time(s) to back transform Freeman-Tukey transformed
#'   incidence rates.
#'
#' @name meta-transf
#' 
#' @aliases meta-transf
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-sm}}
#' 
#' @examples
#' logit2p(p2logit(0.5))
#'
#' @rdname meta-transf
#' @export transf

transf <- function(x, sm, func = NULL, args = NULL) {
  
  #
  # Do nothing if all values are NA
  #
  if (all(is.na(x)))
    return(x)

  if (!is.null(func))
    res <- do.call(func, c(list(x), args))
  #
  else if (is_relative_effect(sm) | is_log_effect(sm))
    res <- log(x)
  #
  else if (sm == "ZCOR")
    res <- cor2z(x)
  #
  else if (sm == "PLOGIT")
    res <- p2logit(x)
  #
  else if (sm == "PAS")
    res <- p2asin(x)
  #
  else if (sm == "IRS")
    res <- sqrt(x)
  #
  else if (sm == "VE")
    res <- VE2logVR(x)
  #
  else
    res <- x
  
  res
}





#' @rdname meta-transf
#' @export cor2z

cor2z <- function(x)
  0.5 * log((1 + x) / (1 - x))





#' @rdname meta-transf
#' @export p2asin

p2asin <- function(x)
  asin(sqrt(x))





#' @rdname meta-transf
#' @export p2logit

p2logit <- function(x)
  qlogis(x)





#' @rdname meta-transf
#' @export VE2logVR

VE2logVR <- function(x)
  log(1 - x / 100)





#' @rdname meta-transf
#' @export backtransf

backtransf <- function(x, sm, n, time, func = NULL, args = NULL) {
  
  chkchar(sm, length = 1)
  
  #
  # Do nothing if all values are NA
  #
  if (all(is.na(x)))
    return(x)
  
  if (!is.null(func))
    res <- do.call(func, c(list(x), args))
  #
  else if (is_relative_effect(sm) | is_log_effect(sm))
    res <- exp(x)
  #
  else if (sm == "ZCOR")
    res <- z2cor(x)
  #
  else if (sm == "PLOGIT")
    res <- logit2p(x)
  #
  else if (sm == "PAS")
    res <- asin2p(x)
  #
  else if (sm == "PFT") {
    if (missing(n))
      stop("Argument 'n' must be provided to back transform ",
           "Freeman-Tukey transformed proportions.")
    res <- asin2p(x, n)
  }
  #
  else if (sm == "IRS")
    res <- x^2
  #
  else if (sm == "IRFT") {
    if (missing(time))
      stop("Argument 'time' must be provided to back transform ",
           "Freeman-Tukey transformed incidence rates.")
    res <- asin2ir(x, time)
  }
  #
  else if (sm == "VE")
    res <- logVR2VE(x)
  #
  else
    res <- x

  if (sm == "PRAW") {
    sel0 <- res[!is.na(res)] < 0
    sel1 <- res[!is.na(res)] > 1
    #
    if (any(sel0, na.rm = TRUE))
      res[sel0] <- 0
    if (any(sel1, na.rm = TRUE))
      res[sel1] <- 1
  }
  #
  if (sm == "PLN") {
    sel0 <- res[!is.na(res)] < 0
    sel1 <- res[!is.na(res)] > 1
    #
    if (any(sel0, na.rm = TRUE))
      res[sel0] <- 0
    else if (any(sel1, na.rm = TRUE))
      res[sel1] <- 1
  }
  
  if (sm == "IR") {
    sel0 <- res[!is.na(res)] < 0
    #
    if (any(sel0, na.rm = TRUE))
      res[sel0] <- 0
  }

  res
}





#' @rdname meta-transf
#' @export asin2ir

asin2ir <- function(x, time = NULL) {
  
  #
  # Do nothing if all values are NA
  #
  if (all(is.na(x)))
    return(x)
  
  
  #
  # Calculate possible minimum for each transformation
  #
  minimum <- 0.5 * (sqrt(0 / time) + sqrt((0 + 1) / time))
  #
  sel0 <- x < minimum
  
  
  res <- rep(NA, length(x))
  #
  sel <- !sel0
  sel <- !is.na(sel) & sel
  #
  res[sel0] <- 0
  #
  # Back transformation of Freeman-Tukey double arcsine transformation:
  #
  res[sel] <- (1 / time[sel] - 8 * x[sel]^2 + 16 * time[sel] * x[sel]^4) /
    (16 * x[sel]^2 * time[sel])
  #
  res[res < 0] <- 0
  
  
  res
}





#' @rdname meta-transf
#' @export asin2p

asin2p <- function(x, n = NULL) {
  
  #
  # Do nothing if all values are NA
  #
  if (all(is.na(x)))
    return(x)
  
  
  #
  # Calculate possible minimum and maximum
  # for each transformation
  #
  if (is.null(n)) {
    minimum <- asin(sqrt(0))
    maximum <- asin(sqrt(1))
  }
  else {
    minimum <- 0.5 * (asin(sqrt(0 / (n + 1))) + asin(sqrt((0 + 1) / (n + 1))))
    maximum <- 0.5 * (asin(sqrt(n / (n + 1))) + asin(sqrt((n + 1) / (n + 1))))
  }
  #
  sel0 <- x < minimum
  sel1 <- x > maximum
  
  
  res <- rep(NA, length(x))
  #
  sel <- !(sel0 | sel1)
  sel <- !is.na(sel) & sel
  #
  res[sel0] <- 0
  res[sel1] <- 1
  #
  if (is.null(n)) {
    #
    # Back transformation of arcsine transformation:
    #
    res[sel] <- sin(x[sel])^2
  }
  else {
    #
    # Back transformation of Freeman-Tukey double arcsine transformation:
    #
    res[sel] <- 0.5 * (1 - sign(cos(2 * x[sel])) *
                       sqrt(1 - (sin(2 * x[sel]) +
                                 (sin(2 * x[sel]) -
                                  1 / sin(2 * x[sel])) / n[sel])^2))
  }
  res
}





#' @rdname meta-transf
#' @export logit2p

logit2p <- function(x)
  1 / (1 + exp(-x))





#' @rdname meta-transf
#' @export logVR2VE

logVR2VE <- function(x)
  100 * (1 - exp(x))





#' @rdname meta-transf
#' @export z2cor

z2cor <- function(x)
  tanh(x)

# Auxiliary functions to check function arguments
#
# Package: meta
# Author: Guido Schwarzer <guido.schwarzer@uniklinik-freiburg.de>
# License: GPL (>= 2)
#

chkchar <- function(x, length = 0, name = NULL, nchar = NULL, single = FALSE,
                    NULL.ok = FALSE) {
  if (is.null(x) & NULL.ok)
    return(invisible(NULL))
  #
  if (!missing(single) && single)
    length <- 1
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (length && length(x) != length) {
    if (length == 1)
      stop("Argument '", name, "' must be a character string.",
           call. = FALSE)
    else
      stop("Argument '", name, "' must be a character vector of length ",
           length, ".",
           call. = FALSE)
  }
  #
  if (length == 1) {
    if (!is.null(nchar) && !(nchar(x) %in% nchar))
      if (length(nchar) == 1 && nchar == 1)
        stop("Argument '", name, "' must be a single character.",
             call. = FALSE)
      else
        stop("Argument '", name, "' must be a character string of length ",
             if (length(nchar) == 2)
               paste0(nchar, collapse = " or ")
             else
               paste0(nchar, collapse = ", "),
             ".",
             call. = FALSE)
  }
  #
  if (!is.character(x) & !is.numeric(x))
    stop("Argument '", name, "' must be a character vector.")
  else {
    if (!is.null(nchar) & any(!(nchar(x) %in% nchar)))
      if (length(nchar) == 1 && nchar == 1)
        stop("Argument '", name, "' must be a vector of single characters.",
             call. = FALSE)
      else
        stop("Argument '", name, "' must be a character vector where ",
             "each element has ",
             if (length(nchar) == 2)
               paste0(nchar, collapse = " or ")
             else
               paste0(nchar, collapse = ", "),
             " characters.",
             call. = FALSE)
  }
  #
  invisible(NULL)
}

chkclass <- function(x, class, name = NULL) {
  #
  # Check class of R object
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  n.class <- length(class)
  if (n.class == 1)
    text.class <- paste0('"', class, '"')
  else if (n.class == 2)
    text.class <- paste0('"', class, '"', collapse = " or ")
  else
    text.class <- paste0(paste0('"', class[-n.class], '"', collapse = ", "),
                    ', or ', '"', class[n.class], '"')
  #
  if (!inherits(x, class))
    stop("Argument '", name, "' must be an object of class ", text.class, ".",
         call. = FALSE)
  #
  invisible(NULL)
}

chkcolor <- function(x, length = 0, name = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (length && length(x) != length)
    stop("Argument '", name, "' must must be a character or ",
         "numeric vector of length ", length, ".",
         call. = FALSE)
  else if (!(is.character(x) || is.numeric(x)))
    stop("Argument '", name, "' must be a character or numeric vector.",
         call. = FALSE)
}

chkfunc <- function(x, name = NULL) {
  #
  # Check whether argument is a function
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (!is.null(x)) {
    if (is.character(x)) {
      if (length(find(x)) == 0)
        stop("Function '", x, "' not available.", call. = FALSE)
    }
    else if (!is.function(x))
      stop("Argument '", name, "' must be a function.", call. = FALSE)
  }
  #
  invisible(NULL)
}

chklength <- function(x, k.all, fun = "", text, name = NULL) {
  #
  # Check length of vector
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (length(x) != k.all) {
    funcs <- c("metabin", "metacont", "metacor",
               "metagen", "metainc", "metamean",
               "metaprop", "metarate",
               "funnel", "forest.meta")
    args <- c("event.e", "n.e", "cor",
              "TE", "event.e", "n",
              "event", "event",
              "TE", "TE")
    #
    idx <- charmatch(fun, funcs, nomatch = NA)
    if (!is.na(idx))
      argname <- args[idx]
    else
      argname <- fun
    #
    if (missing(text))
      stop("Arguments '", argname, "' and '", name,
           "' must have the same length.",
           call. = FALSE)
    else
      stop(text, call. = FALSE)
  }
  #
  invisible(NULL)
}

chklevel <- function(x, length = 0, ci = TRUE, name = NULL, single = FALSE) {
  if (!missing(single) && single)
    length <- 1
  #
  # Check for levels of confidence interval / contour level
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  if (ci)
    "level for confidence interval (range: 0-1)"
  else
    "contour levels (range: 0-1)"
  #
  if (!is.numeric(x))
    if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
    else
      stop("Argument '", name, "' must be numeric.",
           call. = FALSE)
  #
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
  #
  if (any(x <= 0, na.rm = TRUE) | any(x >= 1, na.rm = TRUE))
    stop("Argument '", name, "' must be a numeric between 0 and 1.",
         call. = FALSE)
  #
  invisible(NULL)
}

chklist <- function(x, name = NULL) {
  #
  # Check whether argument is a list
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (!is.null(x) && !is.list(x))
    stop("Argument '", name, "' must be a list.", call. = FALSE)
  #
  invisible(NULL)
}

chklogical <- function(x, name = NULL, text = "") {
  #
  # Check whether argument is logical
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (is.numeric(x))
    x <- as.logical(x)
  #
  if (length(x) !=  1 || !is.logical(x) || is.na(x))
    stop("Argument '", name, "' must be a logical",
         if (text != "") " ", text, ".", call. = FALSE)
  #
  invisible(NULL)
}

chkmiss <- function(x, name = NULL) {
  #
  # Check for missing values
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (anyNA(x))
    stop("Missing values in argument '", name, "'.",
         call. = FALSE)
  #
  invisible(NULL)
}

chknull <- function(x, name = NULL) {
  #
  # Check whether argument is NULL
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (is.null(x))
    stop("Argument '", name, "' is NULL.", call. = FALSE)
  #
  invisible(NULL)
}

chknumeric <- function(x, min, max, zero = FALSE, length = 0,
                       name = NULL, single = FALSE, integer = FALSE,
                       NA.ok = TRUE) {
  if (!missing(single) && single)
    length <- 1
  #
  # Check numeric variable
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  #
  if (NA.ok)
    x <- x[!is.na(x)]
  else if (anyNA(x))
    stop("Missing values not allowed in argument '", name, "'.",
         call. = FALSE)
  #
  if (length(x) == 0)
    return(NULL)
  #
  if (!is.numeric(x))
    stop("Non-numeric value for argument '", name, "'.",
         call. = FALSE)
  #
  if (length && length(x) != length)
    stop("Argument '", name, "' must be a numeric of length ", length, ".",
         call. = FALSE)
  #
  if (!missing(min) & missing(max)) {
    if (zero & min == 0 & any(x <= min, na.rm = TRUE))
      stop("Argument '", name, "' must be positive.",
           call. = FALSE)
    else if (any(x < min, na.rm = TRUE))
      stop("Argument '", name, "' must be larger equal ",
           min, ".", call. = FALSE)
  }
  #
  if (missing(min) & !missing(max)) {
    if (zero & max == 0 & any(x >= max, na.rm = TRUE))
      stop("Argument '", name, "' must be negative.",
           call. = FALSE)
    else if (any(x > max, na.rm = TRUE))
      stop("Argument '", name, "' must be smaller equal ",
           min, ".", call. = FALSE)
  }
  #
  if ((!missing(min) & !missing(max)) &&
      (any(x < min, na.rm = TRUE) | any(x > max, na.rm = TRUE)))
    stop("Argument '", name, "' must be between ",
         min, " and ", max, ".", call. = FALSE)
  #
  if (integer && any(!is_wholenumber(x))) {
    if (length(x) == 1)
      stop("Argument '", name, "' must be an integer.",
           call. = FALSE)
    else
      stop("Argument '", name, "' may only contain integers.",
           call. = FALSE)
  }
  #
  invisible(NULL)
}

argid <- function(x, value) {
  if (any(x == value))
    res <- seq(along = x)[x == value]
  else
    res <- NA
  res
}

chkdeprecated <- function(x, new, old, warn = TRUE) {
  depr <- !is.na(argid(x, old))
  new.given <- !is.na(argid(x, new))
  #
  if (depr & warn) {
    if (new.given)
      warning("Deprecated argument '", old, "' ignored as ",
              "'", new, "' is also provided, see help(meta).",
              call. = FALSE)
    else
      warning("Use argument '", new, "' instead of '",
              old, "' (deprecated), see help(meta).",
              call. = FALSE)
  }
  #
  invisible(depr)
}

chkglmm <- function(sm, method.tau, method.random.ci, method.predict,
                    adhoc.hakn.ci, adhoc.hakn.pi,
                    sm.allowed) {
  
  if (!(sm %in% sm.allowed))
    stop("Generalised linear mixed models only possible with ",
         "argument 'sm = \"", paste0(sm.allowed, collapse = ", "), "\"'.",
         call. = FALSE)
  #
  if (method.tau != "ML")
    stop("Generalised linear mixed models only possible with ",
         "argument 'method.tau = \"ML\"'.",
         call. = FALSE)
  #
  if (any(method.random.ci == "KR"))
    stop("Kenward-Roger method for random effects meta-analysis not ",
         "available for GLMMs.",
         call. = FALSE)
  #
  if (any(method.random.ci == "HK" & adhoc.hakn.ci != ""))
    stop("Hartung-Knapp method with ad hoc correction not ",
         "available for GLMMs.",
         call. = FALSE)
  #
  if (any(method.predict == "KR"))
    stop("Kenward-Roger method for prediction interval not ",
         "available for GLMMs.",
         call. = FALSE)
  #
  if (any(method.predict == "NNF"))
    stop("Bootstrap method for prediction interval not ",
         "available for GLMMs.",
         call. = FALSE)
  #
  if (any(method.predict == "HK")) 
    stop("Hartung-Knapp method for prediction interval not ",
         "available for GLMMs; use 'method.predict = \"V\".",
         call. = FALSE)
  
  return(invisible(NULL))
}

chkmlm <- function(method.tau, missing.method.tau,
                   method.predict,
                   method = "Inverse", missing.method = FALSE) {
  
  if (method != "Inverse" & !missing.method)
    warning("Inverse variance method used in three-level model.",
            call. = FALSE)
  #
  if (!(method.tau %in% c("REML", "ML")) & !missing.method.tau)
    warning("For three-level model, argument 'method.tau' set to ",
            "\"REML\".",
            call. = FALSE)
  #
  if (any(!(method.predict %in% c("V", "HTS", "S"))))
    stop("Available prediction interval methods for ",
         "three-level models: \"V\", \"HTS\", \"S\".",
         call. = FALSE)
  #
  return(invisible(NULL))
}

chklrp <- function(sm, method.tau, method.random.ci, method.predict,
                   adhoc.hakn.ci, adhoc.hakn.pi,
                   sm.allowed) {
  
  if (!(sm %in% sm.allowed))
    stop("Penalised logistic regression only possible with ",
         "argument 'sm = \"", paste0(sm.allowed, collapse = ", "), "\"'.",
         call. = FALSE)
  #
  if (any(method.random.ci == "KR"))
    stop("Kenward-Roger method for random effects meta-analysis not ",
         "available for penalised logistic regression.",
         call. = FALSE)
  #
  if (any(method.random.ci == "HK" & adhoc.hakn.ci != ""))
    stop("Hartung-Knapp method with ad hoc correction not ",
         "available for penalised logistic regression.",
         call. = FALSE)
  #
  if (any(method.predict == "KR"))
    stop("Kenward-Roger method for prediction interval not ",
         "available for penalised logistic regression.",
         call. = FALSE)
  #
  if (any(method.predict == "NNF"))
    stop("Bootstrap method for prediction interval not ",
         "available for penalised logistic regression.",
         call. = FALSE)
  #
  if (any(method.predict == "HK")) 
    stop("Hartung-Knapp method for prediction interval not ",
         "available for penalised logistic regression; use ",
         "'method.predict = \"V\".",
         call. = FALSE)
  
  return(invisible(NULL))
}

chksuitable <- function(x, method,
                        classes =
                          c("metacum", "metainf",
                            "netpairwise"),
                        addtext = NULL,
                        check.mlm = TRUE,
                        stop = TRUE,
                        status = "suitable") {
  if (missing(addtext)) {
    addtext <- rep("", length(classes))
    addtext[classes == "netpairwise"] <-
      " without argument 'separate = TRUE'"
  }
  #
  func <- if (stop) base::stop else base::warning
  #
  for (i in seq_along(classes)) {
    if (inherits(x, classes[i])) {
      do.call(func,
              list(paste0(method, " not ", status, " for an object of class \"",
                          classes[i], "\"", addtext[i], "."),
                   call. = FALSE))
      #
      return(FALSE)
    }
  }
  #
  if (check.mlm) {
    if (!is.null(x$three.level) && any(x$three.level)) {
      do.call(func,
              list(paste(method, "not implemented for three-level model."),
                   call. = FALSE))
      #
      return(FALSE)
    }
  }
  #
  return(TRUE)
}

text_meth <- function(x, i, random, method) {
  #
  if (x$method[i] == "MH" & x$model[i] == "common")
    txt <- text_MH(x, i, random)
  else if (x$method[i] == "Peto")
    txt <- text_Peto(x, i, random)
  else if (x$method[i] == "Inverse" |
           (x$method[i] == "MH" & x$model[i] == "random"))
    txt <- text_Inverse(x, i, random, method)
  else if (x$method[i] == "GLMM")
    txt <- text_GLMM(x, i, random, method)
  else if (x$method[i] == "LRP")
    txt <- text_LRP(x, i, random, method)
  else if (x$method[i] == "Cochran")
    txt <- text_Cochran(x, i, random)
  else if (x$method[i] == "SSW")
    txt <- text_SSW(x, i, random)
  else if (x$method[i] != "") 
    txt <- paste("\n-", x$method[i])
  else
    txt <- ""
  #
  txt
}


text_MH <- function(x, i, random) {
  meth.i <- x[i, , drop = FALSE]
  #
  txt <- paste0("\n- ",
                if (!is.null(meth.i$MH.exact) && meth.i$MH.exact)
                  "Exact ",
                "Mantel-Haenszel method")
  #
  if (random)
    txt <- paste0(txt, " (", gs("text.w.common"), " effect model)")
  #
  txt
}


text_Peto <- function(x, i, random) {
  meth.i <- x[i, , drop = FALSE]
  #
  txt <- "\n- Peto method"
  #
  if (meth.i$model == "common" &
      !any(x$method[x$model == "random"] == "Peto"))
    txt <- paste0(txt, " (", gs("text.w.common"), " effect model)")
  else if (meth.i$model == "random" &
      !any(x$method[x$model == "common"] == "Peto"))
    txt <- paste0(txt, " (", gs("text.w.random"), " effects model)")
  #
  txt
}


text_Inverse <- function(x, i, random, method) {
  meth.i <- x[i, , drop = FALSE]
  #
  txt <- "\n- Inverse variance method"
  #
  if (meth.i$three.level)
    txt <-
      paste0(txt,
             " (three-level model",
             if (meth.i$rho != 0)
               paste0(", rho = ", meth.i$rho, ")")
             else
               ")")
  else {
    if (meth.i$model == "common" && any(x$model == "random") &&
        !any(x$method[x$model == "random"] == "Inverse"))
      txt <- paste0(txt, " (", gs("text.w.common"), " effect model)")
    else if (meth.i$model == "random" && any(x$model == "common") &&
             !any(x$method[x$model == "common"] == "Inverse"))
      txt <- paste0(txt, " (", gs("text.w.random"), " effects model)")
  }
  #
  txt <-
    paste0(txt,
           if (method == "metacont" &&
               !is.null(meth.i$pooledvar) &&
               !is.na(meth.i$pooledvar) && meth.i$pooledvar)
             " (with pooled variance for individual studies)"
           else
             "")
  #
  txt
}


text_GLMM <- function(x, i, random, method) {
  meth.i <- x[i, , drop = FALSE]
  #
  if ("metabin" %in% method) {
    txt <-
      if (meth.i$model.glmm == "UM.FS")
        "\n- Logistic regression model (fixed study effects)"
    else if (meth.i$model.glmm == "UM.RS")
      paste0(
        "\n- Mixed-effects logistic regression model ",
        "(random study effects)")
    else if (meth.i$model.glmm == "CM.EL")
      paste0(
        "\n- Generalised linear mixed model ",
        "(conditional Hypergeometric-Normal)")
    else if (meth.i$model.glmm == "CM.AL")
      "\n- Generalised linear mixed model (conditional Binomial-Normal)"
  }
  else if ("metainc" %in% method) {
    txt <-
      if (meth.i$model.glmm == "UM.FS")
        "\n- Poisson regression model (fixed study effects)"
    else if (meth.i$model.glmm == "UM.RS")
      paste0(
        "\n- Mixed-effects Poisson regression model ",
        "(random study effects)")
    else if (meth.i$model.glmm == "CM.EL")
      paste0(
        "\n- Generalised linear mixed model ",
        "(conditional Poisson-Normal)")
  }
  else if ("metaprop" %in% method)
    txt <-
      "\n- Random intercept logistic regression model"
  else if ("metarate" %in% method)
    txt <-
      "\n- Random intercept Poisson regression model"
  #
  if (meth.i$model == "common" && any(x$model == "random") &&
      !any(x$method[x$model == "random"] == "GLMM"))
    txt <- paste0(txt, " (", gs("text.w.common"), " effect model)")
  else  if (meth.i$model == "random" && any(x$model == "common") &&
            !any(x$method[x$model == "common"] == "GLMM"))
    txt <- paste0(txt, " (", gs("text.w.random"), " effects model)")
  #
  txt
}


text_LRP <- function(x, i, random, method) {
  meth.i <- x[i, , drop = FALSE]
  #
  txt <- "\n- Penalised logistic regression model"
  #
  if (any(x$method == "LRP" & x$model == "random"))
    txt <- paste0(txt, " (sqrt(phi) = ", round(sqrt(x$phi[i]), 4), ")")
  #
  txt
}


text_Cochran <- function(x, i, random) {
  meth.i <- x[i, , drop = FALSE]
  #
  txt <- "\n- Cochran method"
  #
  if (random)
    txt <- paste0(txt, " (", gs("text.w.common"), " effect model)")
  #
  txt
}


text_SSW <- function(x, i, random) {
  meth.i <- x[i, , drop = FALSE]
  #
  txt <- "\n- Sample size method"
  #
  if (meth.i$model == "common" &
      !any(x$method[x$model == "random"] == "SSW"))
    txt <- paste0(txt, " (", gs("text.w.common"), " effect model)")
  else if (meth.i$model == "random" &
      !any(x$method[x$model == "common"] != "SSW"))
    txt <- paste0(txt, " (", gs("text.w.random"), " effects model)")
  #
  txt
}

#' Baujat plot to explore heterogeneity in meta-analysis
#' 
#' @description
#' Draw a Baujat plot to explore heterogeneity in meta-analysis.
#' 
#' @aliases baujat baujat.meta
#' 
#' @param x An object of class \code{meta}.
#' @param yscale Scaling factor for values on y-axis.
#' @param xlim The x limits (min,max) of the plot.
#' @param ylim The y limits (min,max) of the plot.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param pch The plotting symbol used for individual studies.
#' @param cex The magnification to be used for plotting symbol.
#' @param col A vector with colour of plotting symbols.
#' @param bg A vector with background colour of plotting symbols (only
#'   used if \code{pch} in \code{21:25}).
#' @param studlab A logical indicating whether study labels should be
#'   printed in the graph. A vector with study labels can also be
#'   provided (must be of same length as \code{x$TE} then).
#' @param cex.studlab The magnification for study labels.
#' @param pos.studlab Position of study labels, see argument
#'   \code{pos} in \code{\link{text}}.
#' @param offset Offset for study labels (see \code{\link{text}}).
#' @param xmin A numeric specifying minimal value to print study
#'   labels (on x-axis).
#' @param ymin A numeric specifying minimal value to print study
#'   labels (on y-axis).
#' @param grid A logical indicating whether a grid is printed in the
#'   plot.
#' @param col.grid Colour for grid lines.
#' @param lty.grid The line type for grid lines.
#' @param lwd.grid The line width for grid lines.
#' @param pty A character specifying type of plot region (see
#'   \code{\link{par}}).
#' @param pooled A character string indicating whether a common effect
#'   or random effects model is used for pooling. Either missing (see
#'   Details), \code{"common"} or \code{"random"}, can be abbreviated.
#' @param \dots Graphical arguments as in \code{par} may also be
#'   passed as arguments.
#'
#' @details
#' Baujat et al. (2002) introduced a scatter plot to explore
#' heterogeneity in meta-analysis. On the x-axis the contribution of
#' each study to the overall heterogeneity statistic (see list object
#' \code{Q} of the meta-analysis object \code{x}) is plotted. On the
#' y-axis the standardised difference of the overall treatment effect
#' with and without each study is plotted; this quantity describes the
#' influence of each study on the overal treatment effect.
#' 
#' Information from object \code{x} is utilised if argument
#' \code{pooled} is missing. A common effect model is assumed
#' (\code{pooled="common"}) if argument \code{x$common} is
#' \code{TRUE}; a random effects model is assumed
#' (\code{pooled="random"}) if argument \code{x$random} is
#' \code{TRUE} and \code{x$common} is \code{FALSE}.
#' 
#' Internally, the \code{\link{metainf}} function is used to calculate
#' the values on the y-axis.
#' 
#' @return
#' A data.frame with the following variables:
#' \item{x}{Coordinate on x-axis (contribution to heterogeneity
#'   statistic)}
#' \item{y}{Coordinate on y-axis (influence on overall treatment
#'   effect)}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metagen}}, \code{\link{metainf}}
#' 
#' @references
#' Baujat B, Mahé C, Pignon JP, Hill C (2002):
#' A graphical method for exploring heterogeneity in meta-analyses:
#' Application to a meta-analysis of 65 trials.
#' \emph{Statistics in Medicine},
#' \bold{30}, 2641--52
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Olkin1995)
#'
#' # Only consider first ten studies
#' ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, sm = "OR", method = "I", studlab = paste(author, year),
#'   subset = 1:10)
#' 
#' # Generate Baujat plot
#' baujat(ma1)
#'
#' \dontrun{
#' ma2 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, sm = "OR", method = "I", studlab = paste(author, year))
#' 
#' # Do not print study labels if the x-value is smaller than 4 and
#' # the y-value is smaller than 1
#' baujat(ma2, yscale = 10, xmin = 4, ymin = 1)
#' 
#' # Change position of study labels
#' baujat(ma2, yscale = 10, xmin = 4, ymin = 1,
#'   pos = 1, xlim = c(0, 6.5))
#' 
#' # Generate Baujat plot and assign x- and y- coordinates to R object
#' # ba2
#' ba2 <- baujat(ma2)
#' 
#' # Calculate overall heterogeneity statistic
#' sum(ba2$x)
#' ma2$Q
#' }
#' 
#' @method baujat meta
#' @export

baujat.meta <- function(x,
                        yscale = 1,
                        xlim, ylim,
                        xlab = "Contribution to overall heterogeneity",
                        ylab = "Influence on overall result",
                        pch = 21, cex = 1, col = "black", bg = "darkgray",
                        studlab = TRUE, cex.studlab = 0.8,
                        pos.studlab, offset = 0.5,
                        xmin = 0, ymin = 0,
                        grid = TRUE, col.grid = "lightgray",
                        lty.grid = "dotted", lwd.grid = par("lwd"),
                        pty = "s",
                        pooled,
                        ...) {
  
  
  #
  #
  # (1) Check for meta object and upgrade older meta objects
  #
  #
  chkclass(x, "meta")
  chksuitable(x, "Baujat plot",
              c("trimfill", "metacum", "metainf", "metamerge", "netpairwise"))
  #
  x <- updateversion(x)
  
  
  #
  #
  # (2) Check arguments
  #
  #
  chknumeric(yscale)
  chknumeric(cex)
  chknumeric(cex.studlab)
  missing.pos.studlab <- missing(pos.studlab)
  if (!missing.pos.studlab) {
    pos.studlab <- as.numeric(setchar(pos.studlab, as.character(1:4)))
    chknumeric(pos.studlab, min = 1, max = 4)
  }
  #
  if (!missing(pooled)) {
    pooled <- setchar(pooled, c("common", "random", "fixed"))
    pooled[pooled == "fixed"] <- "common"
  }
  else
    if (!x$common & x$random)
      pooled <- "random"
    else
      pooled <- "common"
  chknumeric(offset)
  chknumeric(xmin)
  chknumeric(ymin)
  chklogical(grid)
  chknumeric(lwd.grid)
  
  
  oldpar <- par(pty = pty)
  on.exit(par(oldpar))
  
  
  TE <- x$TE
  seTE <- x$seTE
  #
  if (pooled == "random")
    TE.s <- x$TE.random
  else
    TE.s <- x$TE.common
  #
  k <- x$k
  #
  if (is.logical(studlab)) {
    if (studlab)
      labels.studlab <- x$studlab
    else
      labels.studlab <- rep("", length(TE))
  }
  else if (is.numeric(studlab)) {
    labels.studlab <- x$studlab
  }
  else {
    labels.studlab <- as.character(studlab)
    if (length(labels.studlab) != length(TE))
      stop("Length of argument 'studlab' must be the same as ",
           "number of studies in meta-analysis.")
  }
  #
  if (!missing.pos.studlab) {
    if (length(pos.studlab) > 1)
      chklength(pos.studlab, TE,
                text = paste("Length of argument 'pos.studlab' must be",
                             "the same as number of studies",
                             "in meta-analysis."))
    else
      pos.studlab <- rep(pos.studlab, length(TE))
  }
  
  
  m.inf <- metainf(x, pooled = pooled)
  TE.inf <- m.inf$TE
  seTE.inf <- m.inf$seTE
  #
  ys <- (TE.inf - TE.s)^2 / seTE.inf^2
  ys <- ys * yscale
  #  
  xs <- (TE - TE.s)^2 / seTE^2
  #
  if (!is.null(x$exclude))
    xs[x$exclude] <- NA
  
  
  if (missing(xlim))
    xlim <- c(0, max(xs, na.rm = TRUE))
  #
  if (missing(ylim))
    ylim <- c(0, max(ys, na.rm = TRUE))
  
  
  #
  # Do not print labels for studies with x and/or y values below
  # limits
  #
  if (!missing(xmin) & !missing(ymin))
    labels.studlab[xs < xmin & ys < ymin] <- ""
  else if (!missing(xmin) & missing(ymin))
    labels.studlab[xs < xmin] <- ""
  else if (missing(xmin) & !missing(ymin))
    labels.studlab[ys < ymin] <- ""
  #
  if (is.numeric(studlab)) {
    if (length(studlab) == 1)
      labels.studlab[xs < studlab] <- ""
    else if (length(studlab) == 2) {
      labels.studlab[xs < studlab[1]] <- ""
      labels.studlab[ys < studlab[2]] <- ""
    }
  }
  
  
  plot(xs, ys,
       xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab,
       type = "n")
  #
  if (grid)
    grid(col = col.grid, lty = lty.grid, lwd = lwd.grid)
  #
  points(xs, ys, pch = pch, cex = cex, col = col, bg = bg)
  #
  if (missing.pos.studlab)
    pos.studlab <- ifelse(xs > mean(xlim), 2, 4)
  text(xs, ys, labels = labels.studlab, cex = cex.studlab,
       pos = pos.studlab, offset = offset)
  
  
  res <- data.frame(studlab = x$studlab, x = xs, y = ys)
  attr(res, "pooled") <- pooled
  
  
  invisible(res)
}

#' Meta-analysis of binary outcome data
#' 
#' @description
#' Calculation of common effect and random effects estimates (risk
#' ratio, odds ratio, risk difference, arcsine difference, or
#' diagnostic odds ratio) for meta-analyses with binary outcome
#' data. Mantel-Haenszel, inverse variance, Peto method, generalised
#' linear mixed model (GLMM), logistic regression with penalised likelihood
#' and sample size method are available for pooling. For GLMMs,
#' the \code{\link[metafor]{rma.glmm}} function from R package \bold{metafor}
#' (Viechtbauer, 2010) is called internally. For penalised logistic regression,
#' R package \bold{brglma2} must be available.
#' 
#' @param event.e Number of events in experimental group, or true
#'   positives in diagnostic study, or an R object
#'   created with \code{\link{pairwise}}.
#' @param n.e Number of observations in experimental group or number
#'   of ill participants in diagnostic study.
#' @param event.c Number of events in control group or false positives
#'   in diagnostic study.
#' @param n.c Number of observations in control group or number of
#'   healthy participants in diagnostic study.
#' @param studlab An optional vector with study labels.
#' @param data An optional data frame containing the study
#'   information, i.e., event.e, n.e, event.c, and n.c.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots.
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param method A character string indicating which method is to be
#'   used for pooling of studies. One of \code{"Inverse"},
#'   \code{"MH"}, \code{"Peto"}, \code{"GLMM"}, \code{"LRP"}, or \code{"SSW"},
#'   can be abbreviated.
#' @param sm A character string indicating which summary measure
#'   (\code{"RR"}, \code{"OR"}, \code{"RD"}, \code{"ASD"},
#'   \code{"DOR"}, or \code{"VE"}) is to be used for pooling of
#'   studies, see Details.
#' @param incr Could be either a numerical value which is added to
#'   cell frequencies for studies with a zero cell count, the
#'   character string \code{"TACC"} which stands for treatment arm
#'   continuity correction, or a numeric vector with the continuity
#'   correction for each study, see Details.
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (\code{"only0"},
#'   \code{"if0all"}, \code{"all"}, or \code{"user"}), see Details.
#' @param allstudies A logical indicating if studies with zero or all
#'   events in both groups are to be included in the meta-analysis
#'   (applies only if \code{sm} is equal to \code{"RR"}, \code{"OR"},
#'   or \code{"DOR"}).
#' @param incr.e Continuity correction in experimental group, see Details.
#' @param incr.c Continuity correction in control group, see Details.
#' @param MH.exact A logical indicating if \code{incr} is not to be
#'   added to cell frequencies for studies with a zero cell count to
#'   calculate the pooled estimate based on the Mantel-Haenszel
#'   method.
#' @param RR.Cochrane A logical indicating if 2*\code{incr} instead of
#'   1*\code{incr} is to be added to \code{n.e} and \code{n.c} in the
#'   calculation of the risk ratio (i.e., \code{sm="RR"}) for studies
#'   with a zero cell. This is used in RevMan 5, the program for
#'   preparing and maintaining Cochrane reviews.
#' @param Q.Cochrane A logical indicating if the Mantel-Haenszel
#'   estimate is used in the calculation of the heterogeneity
#'   statistic Q which is implemented in RevMan 5, the program for
#'   preparing and maintaining Cochrane reviews.
#' @param model.glmm A character string indicating which GLMM should
#'   be used.  One of \code{"UM.FS"}, \code{"UM.RS"}, \code{"CM.EL"},
#'   and \code{"CM.AL"}, see Details.
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param method.bias A character string indicating which test for
#'   funnel plot asymmetry is to be used. Either \code{"Begg"},
#'   \code{"Egger"}, \code{"Thompson"}, \code{"Schwarzer"},
#'   \code{"Harbord"}, \code{"Peters"}, or \code{"Deeks"}, can be
#'   abbreviated. See function \code{\link{metabias}.}
#' @param backtransf A logical indicating whether results for odds
#'   ratio (\code{sm="OR"}), risk ratio (\code{sm="RR"}), or
#'   diagnostic odds ratio (\code{sm="DOR"}) should be back
#'   transformed in printouts and plots. If TRUE (default), results
#'   will be presented as odds ratios and risk ratios; otherwise log
#'   odds ratios and log risk ratios will be shown.
#' @param pscale A numeric defining a scaling factor for printing of
#'   risk differences.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.e Label for experimental group.
#' @param label.c Label for control group.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param hakn Deprecated argument (replaced by 'method.random.ci').
#' @param adhoc.hakn Deprecated argument (replaced by
#'   'adhoc.hakn.ci').
#' @param print.CMH A logical indicating whether result of the
#'   Cochran-Mantel-Haenszel test for overall effect should be
#'   printed.
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if \code{incr} is added to studies with zero cell
#'   frequencies or if estimation problems exist in fitting a GLMM).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}} or
#'   \code{\link[metafor]{rma.glmm}}.
#' @param \dots Additional arguments passed on to
#'   \code{\link[metafor]{rma.glmm}} function and to catch deprecated
#'   arguments.
#' 
#' @details
#' Calculation of common and random effects estimates for meta-analyses
#' with binary outcome data.
#' 
#' The following measures of treatment effect are available (Rücker et
#' al., 2009):
#' 
#' \itemize{
#' \item Risk ratio (\code{sm = "RR"})
#' \item Odds ratio (\code{sm = "OR"})
#' \item Risk difference (\code{sm = "RD"})
#' \item Arcsine difference (\code{sm = "ASD"})
#' \item Diagnostic Odds ratio (\code{sm = "DOR"})
#' \item Vaccine efficacy or vaccine effectiveness (\code{sm = "VE"})
#' }
#'
#' Note, mathematically, odds ratios and diagnostic odds ratios are
#' identical, however, the labels in printouts and figures
#' differ. Furthermore, log risk ratio (logRR) and log vaccine ratio
#' (logVR) are mathematical identical, however, back-transformed
#' results differ as vaccine efficacy or effectiveness is defined as
#' \code{VE = 100 * (1 - RR)}.
#' 
#' A three-level random effects meta-analysis model (Van den Noortgate
#' et al., 2013) is utilised if argument \code{cluster} is used and at
#' least one cluster provides more than one estimate. Internally,
#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
#' "rowsum"} is used to calculate random effects weights.
#' 
#' Default settings are utilised for several arguments (assignments
#' using \code{\link{gs}} function). These defaults can be changed for
#' the current R session using the \code{\link{settings.meta}}
#' function.
#' 
#' Furthermore, R function \code{\link{update.meta}} can be used to
#' rerun a meta-analysis with different settings.
#' 
#' \subsection{Meta-analysis method}{
#' 
#' By default, both common effect (also called common effect) and
#' random effects models are considered (see arguments \code{common}
#' and \code{random}). If \code{method} is \code{"MH"} (default), the
#' Mantel-Haenszel method (Greenland & Robins, 1985; Robins et al.,
#' 1986) is used to calculate the common effect estimate; if
#' \code{method} is \code{"Inverse"}, inverse variance weighting is
#' used for pooling (Fleiss, 1993); if \code{method} is \code{"Peto"},
#' the Peto method is used for pooling (Yusuf et al., 1985); if
#' \code{method} is \code{"SSW"}, the sample size method is used for
#' pooling (Bakbergenuly et al., 2020).
#'
#' While the Mantel-Haenszel and Peto method are defined under the
#' common effect model, random effects variants based on these methods
#' are also implemented in \code{metabin}. Following RevMan 5, the
#' Mantel-Haenszel estimator is used in the calculation of the
#' between-study heterogeneity statistic Q which is used in the
#' DerSimonian-Laird estimator (DerSimonian and Laird,
#' 1986). Accordingly, the results for the random effects
#' meta-analysis using the Mantel-Haenszel or inverse variance method
#' are typically very similar. For the Peto method, Peto's log odds
#' ratio, i.e. \code{(O-E) / V} and its standard error \code{sqrt(1 /
#' V)} with \code{O-E} and \code{V} denoting "Observed minus Expected"
#' and its variance, are utilised in the random effects
#' model. Accordingly, results of a random effects model using
#' \code{sm = "Peto"} can be different to results from a random
#' effects model using \code{sm = "MH"} or \code{sm =
#' "Inverse"}. Note, the random effects estimate is based on the
#' inverse variance method for all methods discussed so far.
#' 
#' A distinctive and frequently overlooked advantage of binary
#' endpoints is that individual patient data (IPD) can be extracted
#' from a two-by-two table.  Accordingly, statistical methods for IPD,
#' i.e., logistic regression and generalised linear mixed models, can
#' be utilised in a meta-analysis of binary outcomes (Stijnen et al.,
#' 2010; Simmonds et al., 2016).
#' 
#' R package \bold{brglma2} must be available to fit a one-stage logistic
#' regression model with penalised likelihood (Evrenoglou et al., 2022).
#' The estimation of the summary odds ratio relies on the maximisation of the 
#' likelihood function, penalised using a Firth-type correction. This
#' penalisation aims to reduce bias in cases with rare events and a small
#' number of available studies. However, this method is not restricted 
#' to only such cases and can be applied more generally to binary data. Note,
#' with this type of penalisation, all studies can be included in the analysis,
#' regardless of the total number of observed events. This allows both single
#' and double zero studies to be included without any continuity correction.
#' The random effects model uses a multiplicative heterogeneity parameter
#' \eqn{\phi}, added to the model as an \emph{ad hoc} term. The estimation of
#' this parameter relies on a modified expression of Pearson's statistic, which
#' accounts for sparse data. An estimate of \eqn{\phi} equal to 1 indicates the
#' absence of heterogeneity.
#' 
#' Generalised linear mixed models are available
#' (argument \code{method = "GLMM"}) for the odds ratio as summary measure for
#' the common effect and random effects model by calling the
#' \code{\link[metafor]{rma.glmm}} function from R package
#' \bold{metafor} internally. 
#'
#' Four different GLMMs are available for
#' meta-analysis with binary outcomes using argument \code{model.glmm}
#' (which corresponds to argument \code{model} in the
#' \code{\link[metafor]{rma.glmm}} function):
#' \tabular{cl}{
#' 1. \tab Logistic regression model with common study effects
#'  (default) \cr
#'  \tab (\code{model.glmm = "UM.FS"}, i.e., \bold{U}nconditional
#'  \bold{M}odel - \bold{F}ixed \bold{S}tudy effects) \cr
#' 2. \tab Mixed-effects logistic regression model with random study
#'  effects \cr
#'  \tab (\code{model.glmm = "UM.RS"}, i.e., \bold{U}nconditional
#'  \bold{M}odel - \bold{R}andom \bold{S}tudy effects) \cr
#' 3. \tab Generalised linear mixed model (conditional
#'  Hypergeometric-Normal) \cr
#'  \tab (\code{model.glmm = "CM.EL"}, i.e., \bold{C}onditional
#'  \bold{M}odel - \bold{E}xact \bold{L}ikelihood) \cr
#' 4. \tab Generalised linear mixed model (conditional
#'   Binomial-Normal) \cr
#'  \tab (\code{model.glmm = "CM.AL"}, i.e., \bold{C}onditional
#'   \bold{M}odel - \bold{A}pproximate \bold{L}ikelihood)
#' }
#'
#' Details on these four GLMMs as well as additional arguments which
#' can be provided using argument '\code{\dots}' in \code{metabin} are
#' described in \code{\link[metafor]{rma.glmm}} where you can also
#' find information on the iterative algorithms used for estimation.
#' Note, regardless of which value is used for argument
#' \code{model.glmm}, results for two different GLMMs are calculated:
#' common effect model (with fixed treatment effect) and random
#' effects model (with random treatment effects).
#' }
#' 
#' \subsection{Continuity correction}{
#'
#' Four approaches are available to apply a continuity correction:
#' \itemize{
#' \item Only studies with a zero cell count (\code{method.incr =
#'   "only0"})
#' \item All studies if at least one study has a zero cell count
#'   (\code{method.incr = "if0all"})
#' \item All studies irrespective of zero cell counts
#'   (\code{method.incr = "all"})
#' \item Use values provided in arguments \code{incr.e} and \code{incr.c}
#'   (\code{method.incr = "user"})
#' }
#'
#' By default, a continuity correction is only applied to studies with
#' a zero cell count (\code{method.incr = "only0"}). This method
#' showed the best performance for the odds ratio in a simulation
#' study under the random effects model (Weber et al., 2020).
#'
#' The continuity correction method is used both to calculate
#' individual study results with confidence limits and to conduct
#' meta-analysis based on the inverse variance method. For the risk
#' difference, the method is only considered to calculate standard
#' errors and confidence limits.  For Peto method and GLMMs no
#' continuity correction is used in the meta-analysis. Furthermore,
#' the continuity correction is ignored for individual studies for the
#' Peto method.
#'
#' For studies with a zero cell count, by default, 0.5 (argument
#' \code{incr}) is added to all cell frequencies for the odds ratio or
#' only the number of events for the risk ratio (argument
#' \code{RR.Cochrane = FALSE}, default). The increment is added to all
#' cell frequencies for the risk ratio if argument \code{RR.Cochrane =
#' TRUE}. For the risk difference, \code{incr} is only added to all
#' cell frequencies to calculate the standard error. Finally, a
#' treatment arm continuity correction is used if \code{incr = "TACC"}
#' (Sweeting et al., 2004; Diamond et al., 2007).
#'
#' For odds ratio and risk ratio, treatment estimates and standard
#' errors are only calculated for studies with zero or all events in
#' both groups if \code{allstudies = TRUE}.
#'
#' For the Mantel-Haenszel method, by default (if \code{MH.exact} is
#' FALSE), \code{incr} is added to cell frequencies of a study with a
#' zero cell count in the calculation of the pooled risk ratio or odds
#' ratio as well as the estimation of the variance of the pooled risk
#' difference, risk ratio or odds ratio. This approach is also used in
#' other software, e.g. RevMan 5 and the Stata procedure
#' metan. According to Fleiss (in Cooper & Hedges, 1994), there is no
#' need to add 0.5 to a cell frequency of zero to calculate the
#' Mantel-Haenszel estimate and he advocates the exact method
#' (\code{MH.exact} = TRUE). Note, estimates based on exact
#' Mantel-Haenszel method or GLMM are not defined if the number of
#' events is zero in all studies either in the experimental or control
#' group.
#' }
#'
#' \subsection{Subgroup analysis}{
#' 
#' Argument \code{subgroup} can be used to conduct subgroup analysis for
#' a categorical covariate. The \code{\link{metareg}} function can be
#' used instead for more than one categorical covariate or continuous
#' covariates.
#' }
#' 
#' \subsection{Exclusion of studies from meta-analysis}{
#'
#' Arguments \code{subset} and \code{exclude} can be used to exclude
#' studies from the meta-analysis. Studies are removed completely from
#' the meta-analysis using argument \code{subset}, while excluded
#' studies are shown in printouts and forest plots using argument
#' \code{exclude} (see Examples in \code{\link{metagen}}).
#' Meta-analysis results are the same for both arguments.
#' }
#' 
#' \subsection{Presentation of meta-analysis results}{
#' 
#' Internally, both common effect and random effects models are
#' calculated regardless of values choosen for arguments
#' \code{common} and \code{random}. Accordingly, the estimate
#' for the random effects model can be extracted from component
#' \code{TE.random} of an object of class \code{"meta"} even if
#' argument \code{random = FALSE}. However, all functions in R
#' package \bold{meta} will adequately consider the values for
#' \code{common} and \code{random}. E.g. function
#' \code{\link{print.meta}} will not print results for the random
#' effects model if \code{random = FALSE}.
#'
#' A prediction interval will only be shown if \code{prediction =
#' TRUE}.
#' }
#' 
#' @return
#' An object of class \code{c("metabin", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{update.meta}},
#'   \code{\link{forest}}, \code{\link{funnel}},
#'   \code{\link{metabias}}, \code{\link{metacont}},
#'   \code{\link{metagen}}, \code{\link{metareg}},
#'   \code{\link{print.meta}}
#' 
#' @references
#' Bakbergenuly I, Hoaglin DC, Kulinskaya E (2020):
#' Methods for estimating between-study variance and overall
#' effect in meta-analysis of odds-ratios.
#' \emph{Research Synthesis Methods},
#' \bold{11}, 426--42
#' 
#' Cooper H & Hedges LV (1994):
#' \emph{The Handbook of Research Synthesis}.
#' Newbury Park, CA: Russell Sage Foundation
#' 
#' Diamond GA, Bax L, Kaul S (2007):
#' Uncertain Effects of Rosiglitazone on the Risk for Myocardial
#' Infarction and Cardiovascular Death.
#' \emph{Annals of Internal Medicine},
#' \bold{147}, 578--81
#' 
#' DerSimonian R & Laird N (1986):
#' Meta-analysis in clinical trials.
#' \emph{Controlled Clinical Trials},
#' \bold{7}, 177--88
#' 
#' Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A. (2022):
#' Network meta-analysis of rare events using penalized likelihood regression.
#' \emph{Statistics in Medicine},
#' \bold{41}, 5203--19
#' 
#' Fleiss JL (1993):
#' The statistical basis of meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{2}, 121--45
#' 
#' Greenland S & Robins JM (1985):
#' Estimation of a common effect parameter from sparse follow-up data.
#' \emph{Biometrics},
#' \bold{41}, 55--68
#' 
#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
#' The Cochrane Collaboration, 2020
#'
#' Robins J, Breslow N, Greenland S (1986):
#' Estimators of the Mantel-Haenszel Variance Consistent in Both
#' Sparse Data and Large-Strata Limiting Models.
#' \emph{Biometrics},
#' \bold{42}, 311--23
#' 
#' Rücker G, Schwarzer G, Carpenter J, Olkin I (2009):
#' Why add anything to nothing? The arcsine difference as a measure of
#' treatment effect in meta-analysis with zero cells.
#' \emph{Statistics in Medicine},
#' \bold{28}, 721--38
#' 
#' Simmonds MC, Higgins JP (2016):
#' A general framework for the use of logistic regression models in
#' meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{25}, 2858--77
#' 
#' StataCorp. 2011.
#' \emph{Stata Statistical Software: Release 12}.
#' College Station, TX: StataCorp LP.
#' 
#' Stijnen T, Hamza TH, Ozdemir P (2010):
#' Random effects meta-analysis of event outcome in the framework of
#' the generalized linear mixed model with applications in sparse
#' data.
#' \emph{Statistics in Medicine},
#' \bold{29}, 3046--67
#' 
#' Sweeting MJ, Sutton AJ, Lambert PC (2004):
#' What to add to nothing? Use and avoidance of continuity corrections
#' in meta-analysis of sparse data.
#'\emph{Statistics in Medicine},
#' \bold{23}, 1351--75
#'
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#' 
#' Viechtbauer W (2010):
#' Conducting meta-analyses in R with the metafor package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#' 
#' Weber F, Knapp G, Ickstadt K, Kundt G, Glass Ä (2020):
#' Zero-cell corrections in random-effects meta-analyses.
#' \emph{Research Synthesis Methods},
#' \bold{11}, 913--9
#'
#' Yusuf S, Peto R, Lewis J, Collins R, Sleight P (1985):
#' Beta blockade during and after myocardial infarction: An overview
#' of the randomized trials.
#' \emph{Progress in Cardiovascular Diseases},
#' \bold{27}, 335--71
#' 
#' @examples
#' # Calculate odds ratio and confidence interval for a single study
#' #
#' metabin(10, 20, 15, 20, sm = "OR")
#' 
#' # Different results (due to handling of studies with double zeros)
#' #
#' metabin(0, 10, 0, 10, sm = "OR")
#' metabin(0, 10, 0, 10, sm = "OR", allstudies = TRUE)
#' 
#' # Use subset of Olkin (1995) to conduct meta-analysis based on
#' # inverse variance method (with risk ratio as summary measure)
#' #
#' data(Olkin1995)
#' ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, subset = c(41, 47, 51, 59),
#'   studlab = paste(author, year),
#'   method = "Inverse")
#' ma1
#' # Show results for individual studies
#' summary(ma1)
#' 
#' # Use different subset of Olkin (1995)
#' #
#' ma2 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, subset = year < 1970,
#'   studlab = paste(author, year),
#'   method = "Inverse")
#' ma2
#' forest(ma2)
#' 
#' # Meta-analysis with odds ratio as summary measure
#' #
#' ma3 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, subset = year < 1970,
#'   studlab = paste(author, year),
#'   sm = "OR", method = "Inverse")
#' # Same meta-analysis result using 'update.meta' function
#' ma3 <- update(ma2, sm = "OR")
#' ma3
#' 
#' # Meta-analysis based on Mantel-Haenszel method (with odds ratio as
#' # summary measure)
#' #
#' ma4 <- update(ma3, method = "MH")
#' ma4
#' 
#' # Meta-analysis based on Peto method (only available for odds ratio
#' # as summary measure)
#' #
#' ma5 <- update(ma3, method = "Peto")
#' ma5
#' 
#' \dontrun{
#' # Meta-analyses using generalised linear mixed models (GLMM)
#' 
#' # Logistic regression model with (k = 4) fixed study effects
#' # (default: model.glmm = "UM.FS")
#' ma6 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   studlab = paste(author, year),
#'   data = Olkin1995, subset = year < 1970, method = "GLMM")
#' # Same results:
#' ma6 <- update(ma2, method = "GLMM")
#' ma6
#' 
#' # Mixed-effects logistic regression model with random study effects
#' ma7 <- update(ma6, model.glmm = "UM.RS")
#' #
#' # Use additional argument 'nAGQ' for internal call of 'rma.glmm'
#' # function
#' #
#' ma7 <- update(ma6, model.glmm = "UM.RS", nAGQ = 1)
#' ma7
#' 
#' # Generalised linear mixed model (conditional Hypergeometric-Normal)
#' # (R package 'BiasedUrn' must be available)
#' if (requireNamespace("BiasedUrn", quietly = TRUE)) {
#'  ma8 <- update(ma6, model.glmm = "CM.EL")
#'  ma8
#' }
#' 
#' # Generalised linear mixed model (conditional Binomial-Normal)
#' ma9 <- update(ma6, model.glmm = "CM.AL")
#' ma9
#' 
#' # Logistic regression model with (k = 70) fixed study effects
#' ma10 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   studlab = paste(author, year),
#'   data = Olkin1995, method = "GLMM")
#' ma10
#' 
#' # Mixed-effects logistic regression model with random study effects
#' update(ma10, model.glmm = "UM.RS")
#' 
#' # Conditional Hypergeometric-Normal GLMM (with long computation time)
#' system.time(ma11 <- update(ma10, model.glmm = "CM.EL"))
#' ma11
#' 
#' # Generalised linear mixed model (conditional Binomial-Normal)
#' update(ma10, model.glmm = "CM.AL")
#' }
#' 
#' @export metabin

metabin <- function(event.e, n.e, event.c, n.c, studlab,
                    #
                    data = NULL, subset = NULL, exclude = NULL,
                    cluster = NULL, rho = 0,
                    #
                    weights = NULL,
                    weights.common = weights, weights.random = weights,
                    #
                    method = ifelse(tau.common, "Inverse", gs("method")),
                    sm =
                      ifelse(!is.na(charmatch(tolower(method),
                                              c("peto", "glmm", "lrp", "ssw"),
                                              nomatch = NA)),
                             "OR", gs("smbin")),
                    incr = gs("incr"), method.incr = gs("method.incr"),
                    allstudies = gs("allstudies"),
                    incr.e = if (length(incr) > 1) incr else NULL,
                    incr.c = if (length(incr) > 1) incr else NULL,
                    #
                    level = gs("level"),
                    #
                    MH.exact = gs("MH.exact"), RR.Cochrane = gs("RR.Cochrane"),
                    Q.Cochrane =
                      gs("Q.Cochrane") & method == "MH" & method.tau == "DL",
                    model.glmm = gs("model.glmm"),
                    #
                    common = gs("common"),
                    random = gs("random") | !is.null(tau.preset),
                    overall = common | random,
                    overall.hetstat =
                      if (is.null(gs("overall.hetstat")))
                        common | random
                      else
                        gs("overall.hetstat"),
                    prediction = gs("prediction") | !missing(method.predict),
                    #
                    method.tau,
                    method.tau.ci = gs("method.tau.ci"),
                    level.hetstat = gs("level.hetstat"),
                    tau.preset = NULL, TE.tau = NULL,
                    tau.common = gs("tau.common"),
                    detail.tau = NULL,
                    #
                    method.I2 = gs("method.I2"),
                    #
                    level.ma = gs("level.ma"),
                    method.common.ci = gs("method.common.ci"),
                    method.random.ci = gs("method.random.ci"),
                    adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                    #
                    level.predict = gs("level.predict"),
                    method.predict = gs("method.predict"),
                    adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                    seed.predict = NULL,
                    #
                    method.bias = ifelse(sm == "OR", "Harbord",
                                  ifelse(sm == "DOR", "Deeks",
                                         gs("method.bias"))),
                    #
                    backtransf = gs("backtransf"),
                    pscale = 1,
                    #
                    text.common = gs("text.common"),
                    text.random = gs("text.random"),
                    text.predict = gs("text.predict"),
                    text.w.common = gs("text.w.common"),
                    text.w.random = gs("text.w.random"),
                    #
                    title = gs("title"), complab = gs("complab"),
                    outclab = "",
                    #
                    label.e = gs("label.e"), label.c = gs("label.c"),
                    label.left = gs("label.left"),
                    label.right = gs("label.right"),
                    col.label.left = gs("col.label.left"),
                    col.label.right = gs("col.label.right"),
                    #
                    subgroup, subgroup.name = NULL,
                    print.subgroup.name = gs("print.subgroup.name"),
                    sep.subgroup = gs("sep.subgroup"),
                    test.subgroup = gs("test.subgroup"),
                    prediction.subgroup = gs("prediction.subgroup"),
                    seed.predict.subgroup = NULL,
                    #
                    byvar, hakn, adhoc.hakn,
                    #
                    print.CMH = gs("print.CMH"),
                    #
                    keepdata = gs("keepdata"),
                    warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
                    #
                    control = NULL,
                    ...
                    ) {


  #
  #
  # (1) Check arguments
  #
  #
  
  args <- list(...)
  nam.args <- names(args)
  #
  missing.sm <- missing(sm)
  missing.subgroup <- missing(subgroup)
  missing.byvar <- missing(byvar)
  missing.overall <- missing(overall)
  missing.overall.hetstat <- missing(overall.hetstat)
  missing.test.subgroup <- missing(test.subgroup)
  #
  missing.event.c <- missing(event.c)
  missing.n.e <- missing(n.e)
  missing.n.c <- missing(n.c)
  #
  missing.studlab <- missing(studlab)
  #
  missing.incr <- missing(incr)
  avail.incr <- !missing.incr && !is.null(incr)
  #
  missing.method.incr <- missing(method.incr)
  avail.method.incr <- !missing.method.incr && !is.null(method.incr)
  #
  missing.allstudies <- missing(allstudies)
  #
  missing.incr.e <- missing(incr.e)
  missing.incr.c <- missing(incr.c)
  #
  missing.method.tau <- missing(method.tau)
  missing.tau.common <- missing(tau.common)
  missing.method.predict <- missing(method.predict)
  missing.method <- missing(method)
  missing.Q.Cochrane <- missing(Q.Cochrane)
  missing.level.ma <- missing(level.ma)
  missing.common <- missing(common)
  missing.random <- missing(random)
  missing.method.common.ci <- missing(method.common.ci)
  missing.method.random.ci <- missing(method.random.ci)
  missing.method.I2 <- missing(method.I2)
  #
  missing.hakn <- missing(hakn)
  missing.adhoc.hakn.ci <- missing(adhoc.hakn.ci)
  missing.adhoc.hakn <- missing(adhoc.hakn)
  missing.RR.Cochrane <- missing(RR.Cochrane)
  #
  missing.subgroup.name <- missing(subgroup.name)
  missing.print.subgroup.name <- missing(print.subgroup.name)
  missing.sep.subgroup <- missing(sep.subgroup)
  #
  missing.label.e <- missing(label.e)
  missing.label.c <- missing(label.c)
  missing.complab <- missing(complab)
  #
  missing.cluster <- missing(cluster)
  #
  chknumeric(rho, min = -1, max = 1)
  #
  chknull(sm)
  sm <- setchar(sm, gs("sm4bin"))
  #
  chklevel(level)
  #
  method <- setchar(method, gs("meth4bin"))
  #
  is.glmm <- method == "GLMM"
  is.lrp <- method == "LRP"
  #
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  if (method != "Inverse" & method.common.ci == "IVhet") {
    if (!missing.method.common.ci)
      warning("Argument 'method.common.ci = \"IVhet\"' only available ",
              "if 'method = \"Inverse\".",
              call. = FALSE)
    method.common.ci <- "classic"
  }
  #
  if (missing.method.tau) {
    if (is.lrp)
      method.tau <- "DL"
    else if (is.glmm)
      method.tau <- "ML"
    else
      method.tau <- gs("method.tau")
  }
  #
  method.tau <- setchar(method.tau, c(gs("meth4tau"), "KD"))
  #
  tau.common <- replaceNULL(tau.common, FALSE)
  chklogical(tau.common)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  chklogical(prediction)
  chklevel(level.predict)
  #
  method.predict <- setchar(method.predict, gs("meth4pi"))
  #
  method.tau <-
    set_method_tau(method.tau, missing.method.tau,
                   method.predict, missing.method.predict)
  method.predict <-
    set_method_predict(method.predict, missing.method.predict,
                       method.tau, missing.method.tau)
  #
  if (any(method.predict == "NNF"))
    is_installed_package("pimeta", argument = "method.predict", value = "NNF")
  #
  adhoc.hakn.pi <- setchar(replaceNA(adhoc.hakn.pi, ""), gs("adhoc4hakn.pi"))
  #
  chklogical(backtransf)
  #
  chknumeric(pscale, length = 1)
  #
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  if (!is.null(text.random))
    chkchar(text.random)
  if (!is.null(text.predict))
    chkchar(text.predict)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chklogical(keepdata)
  #
  # Additional arguments / checks for metabin objects
  #
  fun <- "metabin"
  #
  chklogical(warn)
  if (sm != "RD" & pscale != 1) {
    if (warn)
      warning("Argument 'pscale' only considered for risk differences.",
              call. = FALSE)
    pscale <- 1
  }
  #
  method.incr <- setchar(method.incr, gs("meth4incr"))
  #
  chklogical(allstudies)
  #
  chklogical(MH.exact)
  chklogical(Q.Cochrane)
  if (Q.Cochrane & (method != "MH" | method.tau != "DL")) {
    warning("Argument 'Q.Cochrane' only considered for ",
            "Mantel-Haenszel method in combination with ",
            "DerSimonian-Laird estimator.",
            call. = FALSE)
    Q.Cochrane <- FALSE
  }
  #
  if (length(model.glmm) == 0)
    model.glmm <- gs("model.glmm")
  model.glmm <- setchar(model.glmm, c("UM.FS", "UM.RS", "CM.EL", "CM.AL", ""))
  #
  chklogical(print.CMH)
  #
  if (sm == "ASD") {
    method <- "Inverse"
    if (!missing.Q.Cochrane && Q.Cochrane)
      warning("Argument 'Q.Cochrane' only considered for ",
              "Mantel-Haenszel method in combination with ",
              "DerSimonian-Laird estimator.",
              call. = FALSE)
    Q.Cochrane <- FALSE
  }
  #
  # Check for deprecated arguments in '...'
  #
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing.level.ma, args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing.random, args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.random.ci <-
    deprecated2(method.random.ci, missing.method.random.ci,
                hakn, missing.hakn,
                warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  adhoc.hakn.ci <-
    deprecated2(adhoc.hakn.ci, missing.adhoc.hakn.ci,
                adhoc.hakn, missing.adhoc.hakn, warn.deprecated)
  adhoc.hakn.ci <- setchar(replaceNA(adhoc.hakn.ci, ""), gs("adhoc4hakn.ci"))
  #
  missing.subgroup.name <- missing.subgroup.name
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing.print.subgroup.name,
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing.sep.subgroup, args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  RR.Cochrane <-
    deprecated(RR.Cochrane, missing.RR.Cochrane, args, "RR.cochrane",
               warn.deprecated)
  chklogical(RR.Cochrane)
  #
  # Some more checks
  #
  chklogical(overall)
  chklogical(overall.hetstat)
  #
  # Ignore deprecated arguments 'addincr' and 'allincr'
  #
  txt.ignore <- "(deprecated); use argument 'method.incr'"
  #
  if (!is.na(charmatch("addincr", nam.args)))
    warn_ignore_input(addincr, TRUE, txt.ignore)
  if (!is.na(charmatch("allincr", nam.args)))
    warn_ignore_input(allincr, TRUE, txt.ignore)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata) {
    data <- sfsp
    data.pairwise <- FALSE
  }
  else
    data.pairwise <- inherits(data, "pairwise")
  #
  # Catch 'event.e', 'n.e', 'event.c', 'n.c', 'studlab', 'subgroup', 'incr',
  # 'incr.e' and 'incr.c' from data:
  #
  event.e <- catch("event.e", mc, data, sfsp)
  chknull(event.e)
  #
  if (inherits(event.e, "pairwise")) {
    is.pairwise <- TRUE
    #
    type <- attr(event.e, "type")
    if (type != "binary")
      stop("Wrong type for pairwise() object: '", type, "'.", call. = FALSE)
    #
    txt.ignore <- "as first argument is a pairwise object"
    #
    warn_ignore_input(event.c, !missing.event.c, txt.ignore)
    warn_ignore_input(n.e, !missing.n.e, txt.ignore)
    warn_ignore_input(n.c, !missing.n.c, txt.ignore)
    warn_ignore_input(incr.e, !missing.incr.e, txt.ignore)
    warn_ignore_input(incr.c, !missing.incr.c, txt.ignore)
    #
    warn_ignore_input(studlab, !missing.studlab, txt.ignore)
    #
    if (missing.sm)
      sm <- attr(event.e, "sm")
    #
    if (missing.method)
      method <- attr(event.e, "method")
    #
    if (!avail.method.incr & !avail.incr)
      method.incr <- "user"
    #
    missing.sm <- FALSE
    missing.method <- FALSE
    #
    avail.method.incr <- TRUE
    missing.incr <- FALSE
    missing.method.incr <- FALSE
    #
    reference.group <- attr(event.e, "reference.group")
    #
    studlab <- event.e$studlab
    #
    treat1 <- event.e$treat1
    treat2 <- event.e$treat2
    #
    event.c <- event.e$event2
    #
    n.e <- event.e$n1
    n.c <- event.e$n2
    #
    incr.e <- event.e$incr1
    incr.c <- event.e$incr2
    #
    if (avail.incr | method.incr != "user") {
      incr.e <- NULL
      incr.c <- NULL
    }
    #
    if (!avail.incr)
      incr <- attr(event.e, "incr")
    #
    pairdata <- event.e
    data <- event.e
    nulldata <- FALSE
    #
    event.e <- event.e$event1
    #
    wo <- treat1 == reference.group
    #
    if (any(wo)) {
      ttreat1 <- treat1
      treat1[wo] <- treat2[wo]
      treat2[wo] <- ttreat1[wo]
      #
      tevent.e <- event.e
      event.e[wo] <- event.c[wo]
      event.c[wo] <- tevent.e[wo]
      #
      tn.e <- n.e
      n.e[wo] <- n.c[wo]
      n.c[wo] <- tn.e[wo]
      #
      if (!(is.null(incr.e) | is.null(incr.c))) {
        tincr.e <- incr.e
        incr.e[wo] <- incr.c[wo]
        incr.c[wo] <- tincr.e[wo]
      }
    }
    #
    if (missing.subgroup & missing.byvar) {
      subgroup <- paste(treat1, treat2, sep = " vs ")
      #
      if (length(unique(subgroup)) == 1) {
        if (missing.complab)
          complab <- unique(subgroup)
        #
        if (missing.label.e)
          label.e <- unique(treat1)
        if (missing.label.c)
          label.c <- unique(treat2)
        #
        subgroup <- NULL
      }
      else {
        if (missing.overall)
          overall <- FALSE
        if (missing.overall.hetstat)
          overall.hetstat <- FALSE
        if (missing.test.subgroup)
          test.subgroup <- FALSE
      }
    }
    else {
      subgroup <- catch("subgroup", mc, data, sfsp)
      byvar <- catch("byvar", mc, data, sfsp)
      #
      subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                              warn.deprecated)
    }
  }
  else {
    is.pairwise <- FALSE
    #
    if (missing.sm && !is.null(data) && !is.null(attr(data, "sm")))
      sm <- attr(data, "sm")
    #
    n.e <- catch("n.e", mc, data, sfsp)
    #
    event.c <- catch("event.c", mc, data, sfsp)
    n.c <- catch("n.c", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    subgroup <- catch("subgroup", mc, data, sfsp)
    byvar <- catch("byvar", mc, data, sfsp)
    #
    subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                            warn.deprecated)
    #
    if (!missing.incr)
      incr <- catch("incr", mc, data, sfsp)
    #
    if (!missing.incr.e)
      incr.e <- catch("incr.e", mc, data, sfsp)
    if (!missing.incr.c)
      incr.c <- catch("incr.c", mc, data, sfsp)
  }
  #
  method.bias <- setmethodbias(method.bias)
  #
  is.tacc <- FALSE
  #
  if (is.numeric(incr))
    chknumeric(incr, min = 0)
  else {
    incr <- setchar(incr, "TACC",
                    "should be numeric or the character string \"TACC\"")
    is.tacc <- TRUE
  }
  #
  avail.incr.e <- !is.null(incr.e)
  avail.incr.c <- !is.null(incr.c)
  avail.incr.both <- avail.incr.e & avail.incr.c
  #
  if (avail.incr.e + avail.incr.c == 1)
    stop("Arguments 'incr.e' and 'incr.c' are required together.",
         call. = FALSE)
  #
  if (avail.incr.e)
    chknumeric(incr.e, min = 0)
  #
  if (avail.incr.c)
    chknumeric(incr.c, min = 0)
  #
  if (avail.incr.both) {
    if (!avail.method.incr)
      method.incr <- "user"
    #
    txt.ignore <- "as arguments 'incr.e' and 'incr.c' are provided"
    #
    warn_set_input(method.incr, method.incr != "user", '"user"', txt.ignore)
    #
    if (length(incr) != length(incr.e) | any(incr != incr.e))
      warn_ignore_input(incr, avail.incr, txt.ignore)
  }
  else {
    addincr <- allincr <- FALSE
    #
    if (method.incr == "all")
      addincr <- TRUE
    else if (method.incr == "if0all")
      allincr <- TRUE
    addincr <- allincr <- FALSE
    #
    if (!(sm == "ASD" | method %in% c("Peto", "GLMM"))) {
      if (method.incr == "all")
        addincr <- TRUE
      else if (method.incr == "if0all")
        allincr <- TRUE
    }
  }
  #
  k.All <- length(event.e)
  #
  chknull(n.e)
  chknull(event.c)
  chknull(n.c)
  #
  studlab <- setstudlab(studlab, k.All)
  #
  by <- !is.null(subgroup)
  #
  # Catch 'subset', 'exclude' and 'cluster' from data:
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  exclude <- catch("exclude", mc, data, sfsp)
  missing.exclude <- is.null(exclude)
  #
  cluster <- catch("cluster", mc, data, sfsp)
  with.cluster <- !is.null(cluster)
  #
  # Catch 'weights', 'weights.common', and 'weights.random' from data:
  #
  if (!missing(weights))
    weights <- catch("weights", mc, data, sfsp)
  if (!missing(weights.common))
    weights.common <- catch("weights.common", mc, data, sfsp)
  if (!missing(weights.random))
    weights.random <- catch("weights.random", mc, data, sfsp)
  #
  if (!is.null(weights) & is.null(weights.common))
    weights.common <- weights
  #
  if (!is.null(weights) & is.null(weights.random))
    weights.random <- weights
  #
  usw.common <- !is.null(weights.common)
  usw.random <- !is.null(weights.random)
  #
  if (usw.common)
    chknumeric(weights.common, min = 0)
  #
  if (usw.random)
    chknumeric(weights.random, min = 0)
  #
  if (usw.common & method != "Inverse")
    stop("User-specified weights for the common effect model only implemented ",
         "for the inverse variance method (method = \"Inverse\").",
         call. = FALSE)
  #
  if (usw.random & method %in% c("GLMM", "LRP", "SSW"))
    stop("User-specified weights for the random effects model not implemented ",
         "for method = \"GLMM\",  \"LRP\" or  \"SSW\".",
         call. = FALSE)
  #
  # Check variable values
  #
  chknumeric(event.e)
  chknumeric(n.e)
  chknumeric(event.c)
  chknumeric(n.c)
  #
  # Recode integer as numeric:
  #
  event.e <- int2num(event.e)
  n.e     <- int2num(n.e)
  event.c <- int2num(event.c)
  n.c     <- int2num(n.c)
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  
  chklength(n.e, k.All, fun)
  chklength(event.c, k.All, fun)
  chklength(n.c, k.All, fun)
  chklength(studlab, k.All, fun)
  #
  if (with.cluster)
    chklength(cluster, k.All, fun)
  #
  if (usw.common) {
    if (length(weights.common) == 1)
      weights.common <- rep(weights.common, k.All)
    else
      chklength(weights.common, k.All, fun)
  }
  #
  if (usw.random) {
    if (length(weights.random) == 1)
      weights.random <- rep(weights.random, k.All)
    else
      chklength(weights.random, k.All, fun)
  }
  #
  if (length(incr) > 1)
    chklength(incr, k.All, fun)
  #
  if (avail.incr.e) {
    if (length(incr.e) == 1)  
      incr.e <- rep_len(incr.e, k.All)
    else
      chklength(incr.e, k.All, fun)
  }
  #
  if (avail.incr.c) {
    if (length(incr.c) == 1)  
      incr.c <- rep_len(incr.c, k.All)
    else
      chklength(incr.c, k.All, fun)
  }
  #
  if (by) {
    chklength(subgroup, k.All, fun)
    chklogical(test.subgroup)
    chklogical(prediction.subgroup)
  }
  #
  # Additional checks
  #
  if (!by & tau.common) {
    if (warn)
      warning("Value for argument 'tau.common' set to FALSE as ",
              "argument 'subgroup' is missing.",
              call. = FALSE)
    tau.common <- FALSE
  }
  #
  if (by & !tau.common & !is.null(tau.preset)) {
    if (warn)
      warning("Argument 'tau.common' set to TRUE as ",
              "argument tau.preset is not NULL.",
              call. = FALSE)
    tau.common <- TRUE
  }
  #
  if (method.incr == "user" & !avail.incr.both) {
    if (is.null(incr.e) | is.null(incr.c))
      stop("Non-null input for arguments 'incr.e' and 'incr.c' required if ",
           "'method.incr = \"user\"'.",
           call. = FALSE)
    else
      stop("Arguments 'incr.e' and 'incr.c' are required if ",
           "'method.incr = \"user\"'.",
           call. = FALSE)
  }
  
  
  #
  #
  # (4) Subset, exclude studies, and subgroups
  #
  #
  
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of subset is larger than number of studies.")
  #
  if (!missing.exclude) {
    if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
        (length(exclude) > k.All))
      stop("Length of argument 'exclude' is larger than number of studies.")
    #
    exclude2 <- rep(FALSE, k.All)
    exclude2[exclude] <- TRUE
    exclude <- exclude2
  }
  else
    exclude <- rep(FALSE, k.All)
  
  
  #
  #
  # (5) Store complete dataset in list object data
  #     (if argument keepdata is TRUE)
  #
  #
  
  if (keepdata) {
    if (nulldata)
      data <- data.frame(.studlab = studlab)
    else
      data$.studlab <- studlab
    #
    data$.event.e <- event.e
    data$.n.e <- n.e
    data$.event.c <- event.c
    data$.n.c <- n.c
    #
    data$.incr.e <- NA
    data$.incr.c <- NA
    #
    if (by)
      data$.subgroup <- subgroup
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
    #
    if (!missing.exclude)
      data$.exclude <- exclude
    #
    if (with.cluster)
      data$.id <- data$.cluster <- cluster
    #
    if (usw.common)
      data$.weights.common <- weights.common
    #
    if (usw.random)
      data$.weights.random <- weights.random
  }
  
  
  #
  #
  # (6) Use subset for analysis
  #
  #
  
  if (!missing.subset) {
    event.e <- event.e[subset]
    n.e <- n.e[subset]
    event.c <- event.c[subset]
    n.c <- n.c[subset]
    studlab <- studlab[subset]
    #
    cluster <- cluster[subset]
    exclude <- exclude[subset]
    #
    weights.common <- weights.common[subset]
    weights.random <- weights.random[subset]
    #
    incr.e <- incr.e[subset]
    incr.c <- incr.c[subset]
    #
    if (length(incr) > 1)
      incr <- incr[subset]
    #
    if (by)
      subgroup <- subgroup[subset]
  }
  #
  if (missing.subgroup & is.pairwise & by) {
    if (length(unique(subgroup)) == 1) {
      by <- FALSE
      #
      if (missing.complab)
        complab <- unique(subgroup)
      #
      subgroup <- NULL
      #
      if (keepdata)
        data$.subgroup <- NULL
      #
      if (missing.overall)
        overall <- TRUE
      if (missing.overall.hetstat)
        overall.hetstat <- TRUE
    }
  }
  #
  # Determine total number of studies
  #
  k.all <- length(event.e)
  #
  if (k.all == 0)
    stop("No studies to combine in meta-analysis.")
  #
  # No meta-analysis for a single study
  #
  if (k.all == 1) {
    common <- FALSE
    random <- FALSE
    prediction <- FALSE
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  #
  if (by) {
    chkmiss(subgroup)
    #
    if (missing.subgroup.name & is.null(subgroup.name)) {
      if (!missing.subgroup)
        subgroup.name <- byvarname("subgroup", mc)
      else if (!missing.byvar)
        subgroup.name <- byvarname("byvar", mc)
    }
  }
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  
  
  #
  #
  # (7) Continuity correction
  #
  #
  
  #
  # Include non-informative studies?
  # (i.e. studies with either zero or all events in both groups)
  #
  allevents <- event.c == n.c & event.e == n.e
  #
  if (sm == "RD" | sm == "ASD")
    incl <- rep(1, k.all)
  else {
    if (allstudies)
      incl <- rep(1, k.all)
    else {
      if (sm %in% c("OR", "DOR"))
        incl <- ifelse((event.c == 0   & event.e == 0) |
                         (event.c == n.c & event.e == n.e), NA, 1)
      if (sm %in% c("RR", "VE"))
        incl <- ifelse((event.c == 0 & event.e == 0), NA, 1)
    }
  }
  #
  # Exclude studies from meta-analysis:
  #
  sel1 <- event.e > n.e
  sel2 <- event.c > n.c
  if ((any(sel1, na.rm = TRUE)) & warn)
    warning("Studies with event.e > n.e get no weight in meta-analysis.",
            call. = FALSE)
  if ((any(sel2, na.rm = TRUE)) & warn)
    warning("Studies with event.c > n.c get no weight in meta-analysis.",
            call. = FALSE)
  incl[sel1 | sel2] <- NA
  #
  sel3 <- n.e <= 0 | n.c <= 0
  if ((any(sel3, na.rm = TRUE)) & warn)
    warning("Studies with non-positive values for n.e and / or n.c ",
            "get no weight in meta-analysis.",
            call. = FALSE)
  incl[sel3] <- NA
  #
  sel4 <- event.e < 0 | event.c < 0
  if ((any(sel4, na.rm = TRUE)) & warn)
    warning("Studies with negative values for event.e and / or event.c ",
            "get no weight in meta-analysis.",
            call. = FALSE)
  incl[sel4] <- NA
  #
  # Sparse computation
  #
  sel <- switch(sm,
                OR = ((n.e - event.e) == 0 | event.e == 0 |
                        (n.c - event.c) == 0 | event.c == 0),
                RD = ((n.e - event.e) == 0 | event.e == 0 |
                        (n.c - event.c) == 0 | event.c == 0),
                RR = ((n.e - event.e) == 0 | event.e == 0 |
                        (n.c - event.c) == 0 | event.c == 0),
                VE = ((n.e - event.e) == 0 | event.e == 0 |
                        (n.c - event.c) == 0 | event.c == 0),
                ASD = rep(FALSE, length(event.e)),
                DOR = ((n.e - event.e) == 0 | event.e == 0 |
                         (n.c - event.c) == 0 | event.c == 0))
  #
  sel[is.na(incl)] <- FALSE
  incl[is.na(incl)] <- 0
  #
  sparse <- any(sel, na.rm = TRUE)
  #
  # Check for pairwise comparisons with zero cell frequencies in both groups
  #
  doublezeros <- FALSE
  if (sparse & sm %in% c("RR", "OR") & !(method %in% c("Peto", "GLMM"))) {
    sel.doublezeros <- switch(sm,
                              OR = (event.e == 0   & event.c ==   0) |
                                (event.c == n.c & event.e == n.e),
                              RR = (event.c == 0 & event.e == 0))
    if (any(sel.doublezeros, na.rm = TRUE))
      doublezeros <- TRUE
  }
  #
  # Define continuity correction
  #
  #
  if (avail.incr.both) {
    chknumeric(incr.e, min = 0, NA.ok = FALSE)
    chknumeric(incr.c, min = 0, NA.ok = FALSE)
  }
  else {
    if (addincr) {
      #
      if (is.numeric(incr)) {
        if (is.null(incr.e))
          incr.e <- if (length(incr) == 1) rep(incr, k.all) else incr
        if (is.null(incr.c))
          incr.c <- if (length(incr) == 1) rep(incr, k.all) else incr
      }
      else {
        if (is.tacc) {
          #
          # Treatment arm continuity correction:
          #
          incr.e <- n.e / (n.e + n.c)
          incr.c <- n.c / (n.e + n.c)
        }
      }
    }
    else {
      if (sparse) {
        if (allincr) {
          #
          if (is.numeric(incr)) {
            incr.e <- if (length(incr) == 1) rep(incr, k.all) else incr
            incr.c <- if (length(incr) == 1) rep(incr, k.all) else incr
          }
          else {
            if (is.tacc) {
              #
              # Treatment arm continuity correction:
              #
              incr.e <- n.e / (n.e + n.c)
              incr.c <- n.c / (n.e + n.c)
            }
          }
        }
        else {
          #
          # Bradburn, Deeks, Altman, Stata-procedure "metan":
          # & SAS PROC FREQ (for method = "Inverse")
          #
          if (is.numeric(incr)) {
            incr.e <- incr * sel
            incr.c <- incr * sel
          }
          else {
            if (is.tacc) {
              #
              # Treatment arm continuity correction:
              #
              incr.e <- n.e / (n.e + n.c) * sel
              incr.c <- n.c / (n.e + n.c) * sel
            }
          }
        }
      }
      else {
        incr.e <- rep(0, k.all)
        incr.c <- rep(0, k.all)
      }
    }
    #
    # No continuity correction for Peto method
    #
    if (method == "Peto") {
      incr <- 0
      incr.e <- rep(0, k.all)
      incr.c <- rep(0, k.all)
    }
  }
  #
  incr.e <- incr.e * incl
  incr.c <- incr.c * incl
  #
  if (keepdata) {
    if (missing.subset) {
      data$.incr.e <- incr.e
      data$.incr.c <- incr.c
    }
    else {
      data$.incr.e <- NA
      data$.incr.c <- NA
      #
      data$.incr.e[subset] <- incr.e
      data$.incr.c[subset] <- incr.c
    }
  }
  
  
  #
  #
  # (8) Calculate results for individual studies
  #
  #
  
  n11 <- event.e * incl
  n21 <- event.c * incl
  n1. <- n.e * incl
  n2. <- n.c * incl
  #
  n.. <- n1. + n2.
  n12 <- n1. - n11
  n22 <- n2. - n21
  n.1 <- n11 + n21
  n.2 <- n12 + n22
  #
  Q.CMH <- (sum((n11 - n1. * n.1 / n..)[!exclude], na.rm = TRUE)^2 /
            sum((n1. * n2. * n.1 * n.2 / n..^3)[!exclude], na.rm = TRUE))
  #
  p.e <- (n11 + incr.e) / (n1. + 2 * incr.e)
  p.c <- (n21 + incr.c) / (n2. + 2 * incr.c)
  #
  # Estimation of treatment effects in individual studies
  #
  if (sm %in% c("OR", "DOR")) {
    if (method != "Peto") {
      #
      # Cooper & Hedges (1994), p. 251-2
      #
      TE <- log(((n11 + incr.e) * (n22 + incr.c)) /
                ((n12 + incr.e) * (n21 + incr.c)))
      seTE <- sqrt((1 / (n11 + incr.e) + 1 / (n12 + incr.e) +
                    1 / (n21 + incr.c) + 1 / (n22 + incr.c)))
    }
    else {
      #
      # Cooper & Hedges (1994), p. 252
      #
      O <- n11
      E <- n1. * n.1 / n..
      V <- n1. * n2. * n.1 * n.2 / ((n.. - 1) * n..^2)
      #
      TE <- (O - E) / V
      seTE <- sqrt(1 / V)
    }
  }
  else if (sm %in% c("RR", "VE")) {
    #
    # Cooper & Hedges (1994), p. 247-8
    #
    if (!RR.Cochrane) {
      TE <- log(((n11 + incr.e) / (n1. + incr.e)) /
                ((n21 + incr.c) / (n2. + incr.c)))
      #
      # Hartung & Knapp (2001), Stat Med, equation (18)
      #
      seTE <- sqrt((1 / (n11 + incr.e * (!allevents)) - 1 / (n1. + incr.e) +
                    1 / (n21 + incr.c * (!allevents)) - 1 / (n2. + incr.c)))
    }
    else {
      TE <- log(((n11 + incr.e) / (n1. + 2 * incr.e)) /
                ((n21 + incr.c) / (n2. + 2 * incr.c)))
      seTE <- sqrt((1 / (n11 + incr.e) - 1 / (n1. + 2 * incr.e) +
                    1 / (n21 + incr.c) - 1 / (n2. + 2 * incr.c)))
    }
  }
  else if (sm == "RD") {
    #
    # Cooper & Hedges (1994), p. 246-7
    #
    TE <- n11 / n1. - n21 / n2.
    seTE <- sqrt((n11 + incr.e) * (n12 + incr.e) / (n1. + 2 * incr.e)^3 +
                 (n21 + incr.c) * (n22 + incr.c) / (n2. + 2 * incr.c)^3)
  }
  else if (sm == "ASD") {
    #
    # Ruecker et al. (2009)
    #
    TE <- asin(sqrt(n11 / n1.)) - asin(sqrt(n21 / n2.))
    seTE <- sqrt(0.25 * (1 / n1. + 1 / n2.))
  }
  #
  # Set NaN to NA
  #
  TE[is.nan(TE)] <- NA
  
  
  #
  #
  # (9) Additional checks for three-level model
  #
  #
  
  three.level <- FALSE
  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (with.cluster &&
      length(unique(cluster[sel.ni])) != length(cluster[sel.ni]))
    three.level <- TRUE
  #
  if (three.level) {
    chkmlm(method.tau, missing.method.tau, method.predict,
           method, missing.method)
    #
    common <- FALSE
    method <- "Inverse"
    is.glmm <- FALSE
    is.lrp <- FALSE
    #
    if (!(method.tau %in% c("REML", "ML")))
      method.tau <- "REML"
  }
  #
  if (is.lrp) {
    is_installed_package("brglm2", fun, "method", " = \"LRP\"")
    #
    if (!missing.method.tau & method.tau != "DL")
      warn_ignore_input(method.tau, text = "for penalised logistic regression")
    #
    if (by & tau.common)
      stop("Subgroup analysis not defined for penalised logistic regression ",
           "assuming a common tau-squared.",
           call. = FALSE)
  }
  
  
  #
  #
  # (10) Additional checks for GLMM, penalised logistic regression,
  #      Peto method or SSW
  #
  #
  
  if (sm != "OR") {
    if (method == "Peto")
      stop("Peto's method only possible with argument 'sm = \"OR\"'")
    else if (method == "SSW")
      stop("Sample size weighting only available with argument 'sm = \"OR\"'")
    else if (is.glmm)
      stop("Generalised linear mixed models only possible with ",
           "argument 'sm = \"OR\"'.")
    else if (is.lrp)
      stop("Logistic regression with penalised likelihood only possible with ",
           "argument 'sm = \"OR\"'.")
  }
  #
  if (is.glmm) {
    chkglmm(sm, method.tau, method.random.ci, method.predict,
            adhoc.hakn.ci, adhoc.hakn.pi,
            "OR")
    #
    if (!is.null(TE.tau)) {
      if (warn)
        warning("Argument 'TE.tau' not considered for GLMM.",
                call. = FALSE)
      TE.tau <- NULL
    }
    #
    if (!is.null(tau.preset)) {
      if (warn)
        warning("Argument 'tau.preset' not considered for GLMM.",
                call. = FALSE)
      tau.preset <- NULL
    }
    #
    if (model.glmm == "CM.EL")
      is_installed_package("BiasedUrn", fun, "model.glmm", " = \"CM.EL\"")
  }
  #
  if (is.lrp) {
    chklrp(sm, method.tau, method.random.ci, method.predict,
           adhoc.hakn.ci, adhoc.hakn.pi, "OR")
    #
    if (warn) {
      txt.warn <- "for penalised logistic regression"
      #
      warn_ignore_input(TE.tau, !is.null(TE.tau), txt.warn)
      warn_ignore_input(tau.preset, !is.null(tau.preset), txt.warn)
      #
      if (!missing.method.I2 & method.I2 == "tau2")
        warning("Argument 'method.I2' set to \"Q\" for ",
                "penalised logistic regression.",
                call. = FALSE)
    }
    #
    TE.tau <- NULL
    tau.preset <- NULL
    method.I2 <- "Q"
  }
  #
  # No need to add anything to cell counts for
  #  (i)  arcsine difference as summary measure
  #  (ii) Peto method, GLMM, or penalised logistic regression
  #
  if (!avail.incr.both) {
    if (sm == "ASD" | method %in% c("Peto", "GLMM", "LRP")) {
      if ((!missing.incr & any(incr != 0)) |
          allincr | addincr |
          (!missing.allstudies & allstudies)
      )
        if (sm == "ASD") {
          if ((sparse | addincr) & warn) {
            warning("Note, no continuity correction considered ",
                    "for arcsine difference (sm = \"ASD\").",
                    call. = FALSE)
          }
        }
      else if (method == "Peto") {
        if ((sparse | addincr) & warn)
          warning("Note, no continuity correction considered ",
                  "for method = \"Peto\".",
                  call. = FALSE)
      }
    }
  }
  
  
  #
  #
  # (11) Do meta-analysis
  #
  #
  
  k <- sum(!is.na(event.e[!exclude]) & !is.na(event.c[!exclude]) &
           !is.na(n.e[!exclude]) & !is.na(n.c[!exclude]))
  #
  for (i in seq_along(method.random.ci))
    if (k == 1 & method.random.ci[i] == "HK")
      method.random.ci[i] <- "classic"
  #
  if (method == "MH") {
    incr.e <- incr.e * (!MH.exact)
    incr.c <- incr.c * (!MH.exact)
    #
    if (sm %in% c("OR", "DOR")) {
      #
      # Cooper & Hedges (1994), p. 253-5 (MH.exact == TRUE)
      #
      # Bradburn, Deeks, Altman, Stata-procedure "metan"
      # und RevMan 3.1 (MH.exact == FALSE)
      #
      A <- (n11 + incr.e) * (n22 + incr.c) / (n.. + 2 * incr.e + 2 * incr.c)
      B <- (n11 + incr.e + n22 + incr.c) / (n.. + 2 * incr.e + 2 * incr.c)
      C <- (n12 + incr.e) * (n21 + incr.c) / (n.. + 2 * incr.e + 2 * incr.c)
      D <- (n12 + incr.e + n21 + incr.c) / (n.. + 2 * incr.e + 2 * incr.c)
      #
      A[exclude] <- B[exclude] <- C[exclude] <- D[exclude] <- 0
      #
      # Cooper & Hedges (1994), p. 265-6
      #
      w.common <- C
      TE.common <- log(sum(A, na.rm = TRUE) / sum(C, na.rm = TRUE))
      seTE.common <- sqrt((1 / (2 * sum(A, na.rm = TRUE)^2)  *
                           (sum(A * B, na.rm = TRUE) +
                            exp(TE.common) * (sum(B * C, na.rm = TRUE) +
                                              sum(A * D, na.rm = TRUE)) +
                            exp(TE.common)^2 * sum(C * D, na.rm = TRUE))))
    }
    else if (sm %in% c("RR", "VE")) {
      #
      # Greenland, Robins (1985) (MH.exact == TRUE)
      #
      # Bradburn, Deeks, Altman, Stata-procedure "metan"
      # (MH.exact == FALSE)
      #
      D <- ((n1. + 2 * incr.e) * (n2. + 2 * incr.c) * (n.1 + incr.e + incr.c) -
            (n11 + incr.e) * (n21 + incr.c) * (n.. + 2 * incr.e + 2 * incr.c)) /
        (n.. + 2 * incr.e + 2 * incr.c)^2
      R <- (n11 + incr.e) * (n2. + 2 * incr.c) / (n.. + 2 * incr.e + 2 * incr.c)
      S <- (n21 + incr.c) * (n1. + 2 * incr.e) / (n.. + 2 * incr.e + 2 * incr.c)
      #
      D[exclude] <- R[exclude] <- S[exclude] <- 0
      #
      w.common <- S
      TE.common <- log(sum(R, na.rm = TRUE) / sum(S, na.rm = TRUE))
      seTE.common <- sqrt(sum(D, na.rm = TRUE) / (sum(R, na.rm = TRUE) *
                                                  sum(S, na.rm = TRUE)))
    }
    else if (sm == "RD") {
      #
      # Jon Deeks (1999) (MH.exact == TRUE)
      #
      # Bradburn, Deeks, Altman, Stata-procedure "metan"
      # und RevMan 3.1 (MH.exact == FALSE)
      #
      R <- ((n11 + incr.e) * (n12 + incr.e) * (n2. + 2 * incr.c)^3 +
            (n21 + incr.c) * (n22 + incr.c) * (n1. + 2 * incr.e)^3) /
        ((n1. + 2 * incr.e) * (n2. + 2 * incr.c) * (n.. + 2 * incr.e + 2 * incr.c)^2)
      S <- (n1. + 2 * incr.e) * (n2. + 2 * incr.c) /
        (n.. + 2 * incr.e + 2 * incr.c)
      #
      R[exclude] <- S[exclude] <- 0
      #
      w.common <- S
      TE.common <- weighted.mean(TE, w.common, na.rm = TRUE)
      seTE.common <- sqrt(sum(R, na.rm = TRUE) / sum(S, na.rm = TRUE)^2)
    }
    #
    w.common[is.na(w.common)] <- 0
  }
  else if (method == "Peto") {
    w.common <- 1 / seTE^2
    w.common[exclude] <- 0
    TE.common   <- weighted.mean(TE, w.common, na.rm = TRUE)
    seTE.common <- sqrt(1 / sum(w.common, na.rm = TRUE))
    #
    w.common[is.na(w.common)] <- 0
  }
  else if (is.glmm) {
    list.bin <- list(ai = event.e[!exclude], n1i = n.e[!exclude],
                     ci = event.c[!exclude], n2i = n.c[!exclude],
                     measure = "OR", model = model.glmm)
    #
    use.random <-
      sum(!exclude) > 1 &
      !((sum(event.e[!exclude], na.rm = TRUE) == 0 &
         sum(event.c[!exclude], na.rm = TRUE) == 0) |
        (!any(event.e[!exclude] != n.e[!exclude]) |
         !any(event.c[!exclude] != n.c[!exclude])))
    #
    res.glmm <-
      runGLMM(list.bin,
              method.tau = method.tau,
              method.random.ci = method.random.ci,
              level = level.ma,
              control = list(control),
              use.random = use.random,
              warn = warn)
    #
    TE.common   <- as.numeric(res.glmm$glmm.common$b)
    seTE.common <- as.numeric(res.glmm$glmm.common$se)
    #
    w.common <- rep(NA, length(event.e))
  }
  else if (is.lrp) {
    fit.lrp <- runLRP(event.e[!exclude], n1 = n.e[!exclude],
                      event2 = event.c[!exclude], n2 = n.c[!exclude],
                      ...)
    #
    TE.common   <- fit.lrp$TE.common
    seTE.common <- fit.lrp$seTE.common
    #
    w.common <- rep(NA, length(event.e))
  }
  else if (method == "SSW") {
    w.common <- n.e * n.c / (n.e + n.c)
    w.common[exclude] <- 0
    TE.common <- weighted.mean(TE, w.common, na.rm = TRUE)
    seTE.common <- sqrt(sum(w.common^2 * seTE^2, na.rm = TRUE) /
                        sum(w.common, na.rm = TRUE)^2)
    #
    w.common[is.na(w.common)] <- 0
  }
  #
  m <- metagen(TE, seTE, studlab,
               exclude = if (missing.exclude) NULL else exclude,
               cluster = cluster, rho = rho,
               #
               weights.common = weights.common,
               weights.random = weights.random,
               #
               sm = sm,
               level = level,
               #
               common = common,
               random = random,
               overall = overall,
               overall.hetstat = overall.hetstat,
               prediction = prediction,
               #
               method.tau = if (is.glmm | is.lrp) "DL" else method.tau,
               method.tau.ci = if (is.glmm | is.lrp) "" else method.tau.ci,
               level.hetstat = level.hetstat,
               tau.preset = tau.preset,
               TE.tau = if (Q.Cochrane) TE.common else TE.tau,
               tau.common = FALSE,
               detail.tau = detail.tau,
               #
               method.I2 = method.I2,
               #
               level.ma = level.ma,
               method.common.ci = method.common.ci,
               method.random.ci = method.random.ci,
               adhoc.hakn.ci = adhoc.hakn.ci,
               #
               level.predict = level.predict,
               method.predict = method.predict,
               adhoc.hakn.pi = adhoc.hakn.pi,
               seed.predict = seed.predict,
               #
               method.bias = method.bias,
               #
               backtransf = backtransf,
               #
               text.common = text.common, text.random = text.random,
               text.predict = text.predict,
               text.w.common = text.w.common, text.w.random = text.w.random,
               #
               title = title, complab = complab, outclab = outclab,
               #
               label.e = label.e, label.c = label.c,
               label.left = label.left, label.right = label.right,
               col.label.left = col.label.left,
               col.label.right = col.label.right,
               #
               keepdata = FALSE,
               warn = warn,
               #
               control = control)
  #
  if (method == "SSW") {
    w.random <- n.e * n.c / (n.e + n.c)
    w.random[exclude] <- 0
    TE.random <- weighted.mean(TE, w.random, na.rm = TRUE)
    seTE.random <- sqrt(sum(w.random^2 * (seTE^2 + m$tau^2), na.rm = TRUE) /
                          sum(w.random, na.rm = TRUE)^2)
    #
    w.random[is.na(w.random)] <- 0
  }
  #
  # Estimate common tau-squared across subgroups
  #
  if (by & tau.common & !(is.glmm | is.lrp))
    hcc <- hetcalc(TE, seTE, method.tau, m$method.tau.ci,
                   if (Q.Cochrane & method == "MH") TE.common else TE.tau,
                   method.I2, level.hetstat, subgroup, control)
  
  
  #
  #
  # (12) Generate R object
  #
  #
  
  res <- list(event.e = event.e, n.e = n.e,
              event.c = event.c, n.c = n.c,
              method = method, method.random = method,
              incr = if (length(unique(incr)) == 1) unique(incr) else incr,
              method.incr = method.incr,
              sparse = sparse,
              allstudies = allstudies,
              doublezeros = doublezeros,
              MH.exact = MH.exact, RR.Cochrane = RR.Cochrane,
              Q.Cochrane = Q.Cochrane,
              Q.CMH = Q.CMH, df.Q.CMH = 1, pval.Q.CMH = pvalQ(Q.CMH, 1),
              print.CMH = print.CMH,
              incr.e = incr.e, incr.c = incr.c,
              k.MH = if (method == "MH") sum(w.common > 0) else NA)
  #
  # Add meta-analysis results
  # (after removing unneeded list elements)
  #
  m$method <- NULL
  m$method.random <- NULL
  m$n.e <- NULL
  m$n.c <- NULL
  m$pscale <- NULL
  m$irscale <- NULL
  m$irunit <- NULL
  m$method.ci <- NULL
  m$method.mean <- NULL
  m$approx.TE <- NULL
  m$approx.seTE <- NULL
  #
  res <- c(res, m)
  #
  # Add data
  #
  res$TE.tau <- TE.tau
  #
  res$pscale <- pscale
  #
  if (is.pairwise | data.pairwise) {
    res$pairwise <- TRUE
    res$k.study <- length(unique(res$studlab[!is.na(res$TE)]))
  }
  #
  res$call <- match.call()
  #
  if (method %in% c("MH", "Peto", "GLMM", "LRP", "SSW")) {
    res <- ci2meta(res, ci.c = ci(TE.common, seTE.common, level = level.ma))
    res$w.common <- w.common
  }
  #
  if (is.glmm) {
    res$method.tau <- method.tau
    res <- addGLMM(res, res.glmm, method.I2)
    res$model.glmm <- model.glmm
    #
    if (by) {
      n.subgroups <- length(unique(subgroup[!exclude]))
      if (n.subgroups > 1)
        subgroup.glmm <-
          factor(subgroup[!exclude], bylevs(subgroup[!exclude]))
      #
      hcc <-
        hccGLMM(
          res,
          runGLMM(list.bin,
                  method.tau = method.tau,
                  method.random.ci = method.random.ci,
                  level = level.hetstat,
                  data =
                    if (n.subgroups > 1)
                      list(data.frame(subgroup.glmm))
                    else NULL,
                  mods =
                    if (n.subgroups > 1)
                      as.call(~ subgroup.glmm)
                    else
                      NULL,
                  control = list(control),
                  use.random = use.random,
                  warn = warn)$glmm.random[[1]],
          method.I2
        )
    }
  }
  else if (is.lrp) {
    res <- ci2meta(res,
                   ci.r = ci(fit.lrp$TE.random, fit.lrp$seTE.random,
                             level = level.ma,
                             df = ifelse(method.random.ci == "HK",
                                         m$k - 1, Inf)))
    res$w.random <- w.common
    res$phi <- fit.lrp$phi
  }
  else if (method == "SSW") {
    res <- ci2meta(res,
                   ci.r = ci(TE.random, seTE.random, level = level.ma,
                             df = ifelse(method.random.ci == "HK",
                                         m$k - 1, Inf)))
    res$w.random <- w.random
  }
  #
  if (keepdata) {
    res$data <- data
    if (!missing.subset)
      res$subset <- subset
  }
  #
  class(res) <- c(fun, "meta")
  #
  # Add results from subgroup analysis
  #
  if (by) {
    res$subgroup <- subgroup
    res$subgroup.name <- subgroup.name
    res$print.subgroup.name <- print.subgroup.name
    res$sep.subgroup <- sep.subgroup
    res$test.subgroup <- test.subgroup
    res$prediction.subgroup <- prediction.subgroup
    res$tau.common <- tau.common
    #
    if (!tau.common) {
      res <- c(res, subgroup(res, seed = seed.predict.subgroup))
      if (res$three.level)
        res <- setNA3(res)
    }
    else if (!is.null(tau.preset))
      res <-
        c(res, subgroup(res, tau.preset, seed = seed.predict.subgroup))
    else {
      if (is.glmm)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup)), ...))
      else if (res$three.level)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup))))
      else
        res <-
          c(res, subgroup(res, hcc$tau.resid, seed = seed.predict.subgroup))
    }
    #
    if (tau.common && is.null(tau.preset))
      res <- addHet(res, hcc, !(is.glmm | is.lrp))
    #
    res$n.w <- NULL
    res$event.w <- NULL
    #
    res$n.harmonic.mean.w <- NULL
    #
    res$time.e.w <- NULL
    res$time.c.w <- NULL
    res$t.harmonic.mean.w <- NULL
    #
    res <- setNAwithin(res, res$three.level | is.glmm)
  }
  #
  # Mantel-Haenszel method is common effect method
  #
  if (res$method.random == "MH")
    res$method.random <- "Inverse"
  #
  # Do not return tau^2 and tau for penalised logistic regression
  #
  if (is.lrp) {
    res$method.random <- "LRP"
    #
    res$tau2 <- res$lower.tau2 <- res$upper.tau2 <- NA
    res$tau <- res$lower.tau <- res$upper.tau <- NA
    res$method.tau <- ""
    #
    res <- calcPI(res)
    #
    res$version.brglm2 <- packageDescription("brglm2")$Version
  }
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  class(res) <- c(fun, "meta")
  
  
  res
}

#' Funnel plot
#' 
#' @description
#' Draw a funnel plot which can be used to assess small study effects
#' in meta-analysis. A contour-enhanced funnel plot can also be
#' produced to assess causes of funnel plot asymmetry.
#' 
#' @aliases funnel funnel.meta
#' 
#' @param x An object of class \code{meta}.
#' @param type A character string indicating type of funnel plot. Either
#'   \code{"standard"} or \code{"contour"}, can be abbreviated.
#' @param xlim The x limits (min,max) of the plot.
#' @param ylim The y limits (min,max) of the plot.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param common A logical indicating whether the common effect
#'   estimate should be plotted.
#' @param random A logical indicating whether the random effects
#'   estimate should be plotted.
#' @param axes A logical indicating whether axes should be drawn on
#'   the plot.
#' @param pch The plotting symbol(s) used for individual studies.
#' @param text A character vector specifying the text to be used
#'   instead of plotting symbol.
#' @param cex The magnification to be used for plotting symbols.
#' @param lty.common Line type (common effect estimate).
#' @param lty.random Line type (random effects estimate).
#' @param col A vector with colour of plotting symbols.
#' @param bg A vector with background colour of plotting symbols (only
#'   used if \code{pch} in \code{21:25}).
#' @param col.common Colour of line representing common effect estimate.
#' @param col.random Colour of line representing random effects
#'   estimate.
#' @param lwd The line width for confidence intervals (if \code{level}
#'   is not \code{NULL}).
#' @param lwd.common The line width for common effect estimate (if
#'   \code{common} is not \code{NULL}).
#' @param lwd.random The line width for random effects estimate (if
#'   \code{random} is not \code{NULL}).
#' @param log A character string which contains \code{"x"} if the
#'   x-axis is to be logarithmic, \code{"y"} if the y-axis is to be
#'   logarithmic and \code{"xy"} or \code{"yx"} if both axes are to be
#'   logarithmic.
#' @param yaxis A character string indicating which type of weights
#'   are to be used. Either \code{"se"}, \code{"invvar"},
#'   \code{"invse"}, \code{"size"}, \code{"invsqrtsize"}, or
#'   \code{"ess"}.
#' @param contour.levels A numeric vector specifying contour levels to
#'   produce contour-enhanced funnel plot.
#' @param col.contour Colour of contours.
#' @param ref Reference value (null effect) used to produce
#'   contour-enhanced funnel plot.
#' @param level The confidence level utilised in the plot. For the
#'   funnel plot, confidence limits are not drawn if
#'   \code{yaxis="size"} or \code{yaxis="invsqrtsize"}.
#' @param studlab A logical indicating whether study labels should be
#'   printed in the graph. A vector with study labels can also be
#'   provided (must be of same length as \code{x$TE} then).
#' @param cex.studlab Size(s) of study labels, see argument \code{cex} in
#'   \code{\link{text}}.
#' @param pos.studlab Position of study labels, see argument
#'   \code{pos} in \code{\link{text}}.
#' @param ref.triangle A logical indicating whether approximate
#'   confidence limits should be printed around reference value (null
#'   effect).
#' @param lty.ref Line type (reference value).
#' @param lwd.ref The line width for the reference value and
#'   corresponding confidence intervals (if \code{ref.triangle} is
#'   TRUE and \code{level} is not \code{NULL}).
#' @param col.ref Colour of line representing reference value.
#' @param lty.ref.triangle Line type (confidence intervals of
#'   reference value).
#' @param backtransf A logical indicating whether results for relative
#'   summary measures (argument \code{sm} equal to \code{"OR"},
#'   \code{"RR"}, \code{"HR"}, or \code{"IRR"}) should be back
#'   transformed in funnel plots. If \code{backtransf=TRUE}, results
#'   for \code{sm="OR"} are printed as odds ratios rather than log
#'   odds ratios, for example.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param vals Vector with values used in \code{setvals} (see Examples).
#' @param \dots Additional arguments (passed on to plot.default).
#' 
#' @details
#' A funnel plot (Light & Pillemer, 1984) is drawn in the active
#' graphics window. If \code{common} is TRUE, the estimate of the
#' common effect model is plotted as a vertical line. Similarly, if
#' \code{random} is TRUE, the estimate of the random effects model is
#' plotted. If \code{level} is not NULL, the corresponding approximate
#' confidence limits are drawn around the common effect estimate (if
#' \code{common} is TRUE) or the random effects estimate (if
#' \code{random} is TRUE and \code{common} is FALSE).
#' 
#' In the funnel plot, the standard error of the treatment estimates
#' is plotted on the y-axis by default (\code{yaxis = "se"}) which is
#' likely to be the best choice (Sterne & Egger, 2001). Only exception
#' is meta-analysis of diagnostic test accuracy studies (Deeks et al.,
#' 2005) where the inverse of the square root of the \emph{effective
#' study size} is used (\code{yaxis = "ess"}). Other possible choices
#' for \code{yaxis} are \code{"invvar"} (inverse of the variance),
#' \code{"invse"} (inverse of the standard error), \code{"size"}
#' (study size), and \code{"invsqrtsize"} (1 / sqrt(study size)).
#' 
#' If argument \code{yaxis} is not equal to \code{"size"},
#' \code{"invsqrtsize"} or \code{"ess"}, contour-enhanced funnel plots
#' can be produced (Peters et al., 2008) by specifying the contour
#' levels (argument \code{contour.levels}). By default (argument
#' \code{col.contour} missing), suitable gray levels will be used to
#' distinguish the contours. Different colours can be chosen by
#' argument \code{col.contour}.
#'
#' @note
#' R function \code{setvals} can be used to easily define the input for the
#' arguments \code{pch}, \code{text}, \code{cex}, \code{col}, \code{bg},
#' and \code{cex.studlab}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}, Petra
#'   Graham \email{pgraham@@efs.mq.edu.au}
#' 
#' @seealso \code{\link{metabias}}, \code{\link{metabin}},
#'   \code{\link{metagen}}, \code{\link{radial}}
#' 
#' @references
#' Deeks JJ, Macaskill P, Irwig L (2005):
#' The performance of tests of publication bias and other sample size
#' effects in systematic reviews of diagnostic test accuracy was
#' assessed.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{58}:882--93
#'
#' Light RJ & Pillemer DB (1984):
#' \emph{Summing Up. The Science of Reviewing Research}.
#' Cambridge: Harvard University Press
#' 
#' Peters JL, Sutton AJ, Jones DR, Abrams KR, Rushton L (2008):
#' Contour-enhanced meta-analysis funnel plots help distinguish
#' publication bias from other causes of asymmetry.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{61}, 991--6
#' 
#' Sterne JAC & Egger M (2001):
#' Funnel plots for detecting bias in meta-analysis: Guidelines on
#' choice of axis.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{54}, 1046--55
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Olkin1995)
#' ma <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, subset = c(41, 47, 51, 59),
#'   studlab = paste(author, year),
#'   sm = "RR", method = "I")
#' 
#' # Standard funnel plot
#' #
#' funnel(ma)
#' 
#' # Funnel plot with confidence intervals, common effect estimate and
#' # contours
#' #
#' fun <- funnel(ma, common = TRUE, level = 0.95, type = "contour")
#' legend("topleft", fun$text.contour, fill = fun$col.contour, bg = "white")
#' 
#' # Contour-enhanced funnel plot with user-chosen colours
#' #
#' funnel(ma, common = TRUE,
#'   level = 0.95, contour = c(0.9, 0.95, 0.99),
#'   col.contour = c("darkgreen", "green", "lightgreen"),
#'   lwd = 2, cex = 2, pch = 16, studlab = TRUE, cex.studlab = 1.25)
#' legend(0.05, 0.05,
#'   c("0.1 > p > 0.05", "0.05 > p > 0.01", "< 0.01"),
#'   fill = c("darkgreen", "green", "lightgreen"))
#' 
#' fun <- funnel(ma, common = TRUE,
#'   level = 0.95, contour = c(0.9, 0.95, 0.99),
#'   col.contour = c("darkgreen", "green", "lightgreen"),
#'   lwd = 2, cex = 2, pch = 16, studlab = TRUE, cex.studlab = 1.25)
#' legend(0.05, 0.05, fun$text.contour, fill = fun$col.contour)
#' 
#' # Use different colours for log risk ratios below and above 0
#' #
#' funnel(ma, bg = setvals(TE < 0, c("green", "red")))
#'
#' @method funnel meta
#' @export

funnel.meta <- function(x,
                        #
                        type = "standard",
                        #
                        xlim = NULL, ylim = NULL, xlab = NULL, ylab = NULL,
                        #
                        common = x$common, random = x$random,
                        #
                        axes = TRUE,
                        pch = if (!inherits(x, "trimfill"))
                                21 else ifelse(x$trimfill, 1, 21),
                        text = NULL, cex = 1,
                        lty.common = 2, lty.random = 9,
                        lwd = 1, lwd.common = lwd, lwd.random = lwd,
                        col = "black", bg = "darkgray",
                        col.common = "black", col.random = "black",
                        #
                        log, yaxis,
                        contour.levels =
                          if (type == "contour")
                            c(0.90, 0.95, 0.99) else NULL,
                        col.contour =
                          if (type == "contour")
                            c("gray80", "gray70", "gray60") else NULL,
                        #
                        ref = ifelse(is_relative_effect(x$sm), 1, 0),
                        #
                        level = if (common | random) x$level else NULL,
                        studlab = FALSE, cex.studlab = 0.8, pos.studlab = 2,
                        #
                        ref.triangle = FALSE,
                        lty.ref = 1,
                        lwd.ref = lwd,
                        col.ref = "black",
                        lty.ref.triangle = 5,
                        #
                        backtransf = x$backtransf,
                        warn.deprecated = gs("warn.deprecated"),
                        ...) {
  
  
  #
  #
  # (1) Check for meta object and upgrade older meta object
  #
  #
  chkclass(x, "meta")
  chksuitable(x, "Funnel plot", "metamerge", check.mlm = FALSE)
  #
  x.name <- deparse(substitute(x))
  x <- updateversion(x)
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  type <- setchar(type, c("standard", "contour"))
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  chklogical(random)
  chklogical(axes)
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (!missing(pch)) {
    error <- try(pch <- catch("pch", mc, x, sfsp), silent = TRUE)
    if (inherits(error, "try-error")) {
      pch <- catch("pch", mc, x$data, sfsp)
      if (isCol(x$data, ".subset"))
        pch <- pch[x$data$.subset]
    }
  }
  chknumeric(pch)
  #
  if (!missing(text)) {
    error <- try(text <- catch("text", mc, x, sfsp), silent = TRUE)
    if (inherits(error, "try-error")) {
      text <- catch("text", mc, x$data, sfsp)
      if (isCol(x$data, ".subset"))
        text <- text[x$data$.subset]
    }
  }
  #
  if (!missing(cex)) {
    error <- try(cex <- catch("cex", mc, x, sfsp), silent = TRUE)
    if (inherits(error, "try-error")) {
      cex <- catch("cex", mc, x$data, sfsp)
      if (isCol(x$data, ".subset"))
        cex <- cex[x$data$.subset]
    }
  }
  if (length(cex) == 1 & length(x$TE) > 1)
    cex <- rep(cex, length(x$TE))
  chknumeric(cex)
  #
  lty.common <- deprecated(lty.common, missing(lty.common), args, "lty.fixed",
                           warn.deprecated)
  chknumeric(lty.common, length = 1)
  chknumeric(lty.random, length = 1)
  chknumeric(lwd, length = 1)
  lwd.common <- deprecated(lwd.common, missing(lwd.common), args, "lwd.fixed",
                           warn.deprecated)
  chknumeric(lwd.common, length = 1)
  chknumeric(lwd.random, length = 1)
  #
  if (!missing(col)) {
    error <- try(col <- catch("col", mc, x, sfsp), silent = TRUE)
    if (inherits(error, "try-error")) {
      col <- catch("col", mc, x$data, sfsp)
      if (isCol(x$data, ".subset"))
        col <- col[x$data$.subset]
    }
  }
  #
  if (length(col) == 1 & length(x$TE) > 1)
    col <- rep(col, length(x$TE))
  #
  if (!missing(bg)) {
    error <- try(bg <- catch("bg", mc, x, sfsp), silent = TRUE)
    if (inherits(error, "try-error")) {
      bg <- catch("bg", mc, x$data, sfsp)
      if (isCol(x$data, ".subset"))
        bg <- bg[x$data$.subset]
    }
  }
  #
  if (length(bg) == 1 & length(x$TE) > 1)
    bg <- rep(bg, length(x$TE))
  #
  if (length(pch) == 1 & length(x$TE) > 1)
    pch <- rep(pch, length(x$TE))
  #
  col.common <- deprecated(col.common, missing(col.common), args, "col.fixed",
                           warn.deprecated)
  #
  if (missing(yaxis))
    if (inherits(x, "metabin") && x$sm == "DOR")
      yaxis <- "ess"
    else
      yaxis <- "se"
  #
  yaxis <- setchar(yaxis, c("se", "invse", "invvar",
                            "size", "invsqrtsize", "ess"))
  #
  if (!is.null(contour.levels))
    chklevel(contour.levels, length = 0, ci = FALSE)
  chknumeric(ref)
  if (!is.null(level))
    chklevel(level)
  #
  if (!missing(cex.studlab)) {
    error <-
      try(cex.studlab <- catch("cex.studlab", mc, x, sfsp), silent = TRUE)
    if (inherits(error, "try-error")) {
      cex.studlab <- catch("cex.studlab", mc, x$data, sfsp)
      if (isCol(x$data, ".subset"))
        cex.studlab <- cex.studlab[x$data$.subset]
    }
  }
  if (length(cex.studlab) == 1 & length(x$TE) > 1)
    cex.studlab <- rep(cex.studlab, length(x$TE))
  chknumeric(cex.studlab)
  #
  pos.studlab <- as.numeric(setchar(pos.studlab, as.character(1:4)))
  chklogical(ref.triangle)
  chknumeric(lty.ref, length = 1)
  chknumeric(lwd.ref, length = 1)
  chknumeric(lty.ref.triangle, length = 1)
  chklogical(backtransf)
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  TE <- x$TE
  k.All <- length(TE)
  seTE <- x$seTE
  seTE[is.infinite(seTE)] <- NA
  #
  if (is.logical(studlab))
    if (studlab) {
      slab <- TRUE
      studlab <- x$studlab
    }
    else
      slab <- FALSE
  else
    slab <- TRUE
  #
  fun <- "funnel"
  chklength(seTE, k.All, fun)
  if (slab)
    chklength(studlab, k.All, fun)
  #
  if (!is.null(text))
    chklength(text, k.All, fun)
  #
  # Exclude studies from funnel plot
  #
  if (!is.null(x$exclude)) {
    TE <- TE[!x$exclude]
    seTE <- seTE[!x$exclude]
    if (slab)
      studlab <- studlab[!x$exclude]
    #
    if (!is.null(pch))
      pch <- pch[!x$exclude]
    if (!is.null(text))
      text <- text[!x$exclude]
    if (!is.null(cex))
      cex <- cex[!x$exclude]
    if (!is.null(col))
      col <- col[!x$exclude]
    if (!is.null(bg))
      bg <- bg[!x$exclude]
    if (!is.null(cex.studlab))
      cex.studlab <- cex.studlab[!x$exclude]
  }
  
  
  #
  #
  # (4) Further assignments
  #
  #
  TE.common <- x$TE.common
  TE.random <- x$TE.random
  sm <- x$sm
  #
  if (missing(log))
    if (backtransf & is_relative_effect(sm))
      log <- "x"
    else
      log <- ""
  #  
  if (yaxis == "se")
    seTE.min <- 0
  else
    seTE.min <- min(seTE, na.rm = TRUE)
  #
  seTE.max <- max(seTE, na.rm = TRUE)
  #
  if (is_relative_effect(sm))
    ref <- log(ref)
  #
  ref.contour <- ref
  #
  if (!is.null(level)) {
    #
    seTE.seq <- seq(seTE.min, seTE.max, length.out = 500)
    #
    if (random & !common)
      ciTE <- ci(TE.random, seTE.seq, level)
    else
      ciTE <- ci(TE.common, seTE.seq, level)
    #
    ciTE.ref <- ci(ref, seTE.seq, level)
    #
    if ((common | random) & ref.triangle) {
      xlim.lower <- min(c(TE, ciTE$lower, ciTE.ref$lower), na.rm = TRUE)
      xlim.upper <- max(c(TE, ciTE$upper, ciTE.ref$upper), na.rm = TRUE)
    }
    #
    else if (ref.triangle) {
      xlim.lower <- min(c(TE, ciTE.ref$lower), na.rm = TRUE)
      xlim.upper <- max(c(TE, ciTE.ref$upper), na.rm = TRUE)
    }
    #
    else {
      xlim.lower <- min(c(TE, ciTE$lower), na.rm = TRUE)
      xlim.upper <- max(c(TE, ciTE$upper), na.rm = TRUE)
    }
    #
    TE.xlim <- c(xlim.lower * ifelse(xlim.lower < 0, 1.025, 1 / 1.025),
                 xlim.upper * ifelse(xlim.upper > 0, 1.025, 1 / 1.025))
  }
  #
  if (backtransf & is_relative_effect(sm)) {
    TE <- exp(TE)
    TE.common <- exp(TE.common)
    TE.random <- exp(TE.random)
    ref <- exp(ref)
    #
    if (!is.null(level)) {
      ciTE$lower <- exp(ciTE$lower)
      ciTE$upper <- exp(ciTE$upper)
      #
      ciTE.ref$lower <- exp(ciTE.ref$lower)
      ciTE.ref$upper <- exp(ciTE.ref$upper)
      #
      TE.xlim <- exp(TE.xlim)
    }
  }
  else if (sm == "VE" & backtransf) {
    TE <- -TE
    TE.common <- -TE.common
    TE.random <- -TE.random
    ref <- -ref
    #
    if (!is.null(level)) {
      tl <- -ciTE$upper
      ciTE$lower <- -ciTE$lower
      ciTE$upper <- tl
      #
      tr <- -ciTE.ref$upper
      ciTE.ref$lower <- -ciTE.ref$lower
      ciTE.ref$upper <- tr
      #
      TE.xlim <- rev(-TE.xlim)
    }
  }
  #
  # y-value: weight
  #
  if (yaxis == "se")
    weight <- seTE
  else if (yaxis == "invse")
    weight <- 1 / seTE
  else if (yaxis == "invvar")
    weight <- 1 / seTE^2
  else if (yaxis %in% c("size", "invsqrtsize")) {
    if (inherits(x, "metabin") || inherits(x, "metacont"))
      weight <- floor(x$n.e) + floor(x$n.c)
    else if (length(x$n.e) > 0 & length(x$n.c) > 0)
      weight <- floor(x$n.e) + floor(x$n.c)
    else if (inherits(x, "metaprop"))
      weight <- floor(x$n)
    else if (length(x$n) > 0)
      weight <- floor(x$n)
    else if (length(x$n.e) > 0)
      weight <- floor(x$n.e)
    else if (length(x$n.c) > 0)
      weight <- floor(x$n.c)
    else
      stop("No information on sample size available in object '",
           x.name, "'.")
  }
  else if (yaxis == "ess") {
    if (inherits(x, "metabin") || inherits(x, "metacont"))
      weight <- 4 * floor(x$n.e) * floor(x$n.c) /
        (floor(x$n.e) + floor(x$n.c))
    else if (length(x$n.e) > 0 & length(x$n.c) > 0)
      weight <- 4 * floor(x$n.e) * floor(x$n.c) /
        (floor(x$n.e) + floor(x$n.c))
    else
      stop("No information on sample size available in object '",
           x.name, "'.")
  }
  #
  if (yaxis %in% c("invsqrtsize", "ess"))
    weight <- 1 / sqrt(weight)
  #
  # x-axis: labels / xlim
  #
  if (is.null(xlab))
    if (is_relative_effect(sm) | sm == "VE")
      xlab <- xlab_meta(sm, backtransf)
    else if (sm == "PRAW")
      xlab <- "Proportion"
    else
      xlab <- xlab_meta(sm, FALSE)
  #
  if (is.null(xlim) & !is.null(level) &
      (yaxis == "se" |
         yaxis == "invse" |
           yaxis == "invvar"))
    xlim <- TE.xlim
  else if (is.null(xlim))
    xlim <- range(TE, na.rm = TRUE)
  #
  # y-axis: labels / ylim
  #
  if (yaxis == "se" & is.null(ylab))
    ylab <- "Standard Error"
  else if (yaxis == "invse" & is.null(ylab))
    ylab <- "Inverse of Standard Error"
  else if (yaxis == "invvar" & is.null(ylab))
    ylab <- "Inverse of Variance"
  else if (yaxis == "size" & is.null(ylab))
    ylab <- "Study Size"
  else if (yaxis == "invsqrtsize" & is.null(ylab))
    ylab <- "1 / root(Study Size)"
  else if (yaxis == "ess" & is.null(ylab))
    ylab <- "1 / root(Effective Study Size)"
  #
  if (is.null(ylim) & yaxis %in% c("se", "invsqrtsize", "ess"))
    ylim <- c(max(weight, na.rm = TRUE), 0)
  if (is.null(ylim)) ylim <- range(weight, na.rm = TRUE)
  
  
  #
  #
  # (5) Produce funnel plot
  #
  #
  args$x <- TE
  args$y <- weight
  args$type <- "n"
  args$xlim <- xlim
  args$ylim <- ylim
  args$xlab <- xlab
  args$ylab <- ylab
  #
  if (sm == "VE" & backtransf) {
    axes.orig <- axes
    axes <- FALSE
  }
  args$axes <- axes
  #
  args$log <- log
  #
  args$fixed <- NULL
  args$lty.fixed <- NULL
  args$lwd.fixed <- NULL
  args$col.fixed <- NULL
  #
  do.call(plot, args)
  #
  if (sm == "VE" & backtransf && axes.orig) {
    xs <- sort(c(0, TE))
    axis(1, at = xs, labels = round(logVR2VE(-xs), 1))
    axis(2)
    box()
  }
  #
  # Add contour shades (enhanced funnel plots)
  #
  if (!is.null(contour.levels) &
      !(yaxis %in% c("size", "invsqrtsize", "ess"))) {
    #
    if (is.null(col.contour))
      if (length(contour.levels) < 2)
        col.contour <- "gray60"
      else
        col.contour <- gray(seq(0.6, 0.9, len = length(contour.levels)))
    #
    if (length(contour.levels) != length(col.contour))
      stop("Arguments 'contour.levels' and 'col.contour' must be of ",
           "the same length.")
    #
    seTE.cont <- seq(seTE.max, seTE.min, length.out = 500)
    #
    j <- 0
    #
    for (i in contour.levels) {
      #
      j <- j + 1
      #
      ciContour <- ci(ref.contour, seTE.cont, i)
      #
      if (backtransf & is_relative_effect(sm)) {
        ciContour$TE    <- exp(ciContour$TE)
        ciContour$lower <- exp(ciContour$lower)
        ciContour$upper <- exp(ciContour$upper)
      }
      sel.l <- ciContour$lower > min(xlim) & ciContour$lower < max(xlim)
      sel.u <- ciContour$upper > min(xlim) & ciContour$upper < max(xlim)
      #
      min.l <- min(ciContour$lower)
      max.l <- max(ciContour$lower)
      min.u <- min(ciContour$upper)
      max.u <- max(ciContour$upper)
      #
      if (yaxis == "se") {
        if (max.u < min(xlim) | min.l > max(xlim)) {
          contour.u.x <- c(min(xlim), min(xlim), max(xlim), max(xlim))
          contour.u.y <- c(min(ylim), max(ylim), max(ylim), min(ylim))
          contour.l.x <- NA
          contour.l.y <- NA
        }
        else {
          contour.l.x <- c(min(xlim), min(xlim),
                           ciContour$lower[sel.l],
                           if (any(sel.l)) max(ciContour$lower[sel.l]) else NA)
          contour.l.y <- c(min(ylim), max(ylim), seTE.cont[sel.l],
                           min(ylim))
          #
          contour.u.x <- c(max(xlim), max(xlim),
                           ciContour$upper[sel.u],
                           if (any(sel.u)) min(ciContour$upper[sel.u]) else NA)
          contour.u.y <- c(min(ylim), max(ylim), seTE.cont[sel.u],
                           min(ylim))
        }
      }
      #
      if (yaxis == "invvar") {
        if (max.u < min(xlim) | min.l > max(xlim)) {
          contour.u.x <- c(min(xlim), min(xlim), max(xlim), max(xlim))
          contour.u.y <- c(max(ylim), min(ylim), min(ylim), max(ylim))
          contour.l.x <- NA
          contour.l.y <- NA
        }
        else {
          contour.l.x <- c(min(xlim), min(xlim),
                           ciContour$lower[sel.l],
                           if (any(sel.l)) max(ciContour$lower[sel.l]) else NA)
          contour.l.y <- c(max(ylim), min(ylim), 1 / seTE.cont[sel.l]^2,
                           max(ylim))
          #
          contour.u.x <- c(max(xlim), max(xlim),
                           ciContour$upper[sel.u],
                           if (any(sel.u)) min(ciContour$upper[sel.u]) else NA)
          contour.u.y <- c(max(ylim), min(ylim), 1 / seTE.cont[sel.u]^2,
                           max(ylim))
        }
      }
      #
      if (yaxis == "invse") {
        if (max.u < min(xlim) | min.l > max(xlim)) {
          contour.u.x <- c(min(xlim), min(xlim), max(xlim), max(xlim))
          contour.u.y <- c(max(ylim), min(ylim), min(ylim), max(ylim))
          contour.l.x <- NA
          contour.l.y <- NA
        }
        else {
          contour.l.x <- c(min(xlim), min(xlim),
                           ciContour$lower[sel.l],
                           if (any(sel.l)) max(ciContour$lower[sel.l]) else NA)
          contour.l.y <- c(max(ylim), min(ylim), 1 / seTE.cont[sel.l],
                           max(ylim))
          #
          contour.u.x <- c(max(xlim), max(xlim),
                           ciContour$upper[sel.u],
                           if (any(sel.u)) min(ciContour$upper[sel.u]) else NA)
          contour.u.y <- c(max(ylim), min(ylim), 1 / seTE.cont[sel.u],
                           max(ylim))
        }
      }
      #
      polygon(contour.l.x, contour.l.y,
              col = col.contour[j], border = FALSE)
      #
      polygon(contour.u.x, contour.u.y,
              col = col.contour[j], border = FALSE)
    }
  }
  #
  # Add results for individual studies
  #
  if (is.null(text))
    points(TE, weight, pch = pch, cex = cex, col = col, bg = bg)
  else
    text(TE, weight, labels = text, cex = cex, col = col)
  #
  # Add results for meta-analysis
  #
  if (common)
    lines(c(TE.common, TE.common), range(ylim),
          lty = lty.common, lwd = lwd.common, col = col.common)
  #
  if (random)
    lines(c(TE.random, TE.random), range(ylim),
          lty = lty.random, lwd = lwd.random, col = col.random)
  #
  if (ref.triangle)
    lines(c(ref, ref), range(ylim),
          lty = lty.ref, lwd = lwd.ref, col = col.ref)
  #
  # Add approximate confidence intervals
  #
  if (!is.null(level)) {
    if (common | random | !ref.triangle) {
      tlow <- ciTE$lower
      tupp <- ciTE$upper
      #
      lty.lines <- if (random & !common) lty.random else lty.common
      lwd.lines <- if (random & !common) lwd.random else lwd.common
      col.lines <- if (random & !common) col.random else col.common
      #
      if (yaxis == "se") {
        points(tlow, seTE.seq, type = "l", lty = lty.lines, lwd = lwd.lines,
               col = col.lines)
        points(tupp, seTE.seq, type = "l", lty = lty.lines, lwd = lwd.lines,
               col = col.lines)
      }
      else if (yaxis == "invse") {
        points(tlow, 1 / seTE.seq, type = "l",
               lty = lty.lines, lwd = lwd.lines,
               col = col.lines)
        points(tupp, 1 / seTE.seq, type = "l",
               lty = lty.lines, lwd = lwd.lines,
               col = col.lines)
      }
      else if (yaxis == "invvar") {
        points(tlow, 1 / seTE.seq^2, type = "l",
               lty = lty.lines, lwd = lwd.lines,
               col = col.lines)
        points(tupp, 1 / seTE.seq^2, type = "l",
               lty = lty.lines, lwd = lwd.lines,
               col = col.lines)
      }
    }
    #
    if (ref.triangle) {
      tlow <- ciTE.ref$lower
      tupp <- ciTE.ref$upper
      #
      if (yaxis == "se") {
        points(tlow, seTE.seq, type = "l", lty = lty.ref.triangle,
               lwd = lwd.ref, col = col.ref)
        points(tupp, seTE.seq, type = "l", lty = lty.ref.triangle,
               lwd = lwd.ref, col = col.ref)
      }
      else if (yaxis == "invse") {
        points(tlow, 1 / seTE.seq, type = "l", lty = lty.ref.triangle,
               lwd = lwd.ref, col = col.ref)
        points(tupp, 1 / seTE.seq, type = "l", lty = lty.ref.triangle,
               lwd = lwd.ref, col = col.ref)
      }
      else if (yaxis == "invvar") {
        points(tlow, 1 / seTE.seq^2, type = "l", lty = lty.ref.triangle,
               lwd = lwd.ref, col = col.ref)
        points(tupp, 1 / seTE.seq^2, type = "l", lty = lty.ref.triangle,
               lwd = lwd.ref, col = col.ref)
      }
    }
  }
  #
  # Add study labels
  #
  if (!is.logical(studlab) && length(studlab) > 0)
    text(TE, weight, labels = studlab, pos = pos.studlab, cex = cex.studlab) 
  
  
  #
  #
  # (6) Return information on funnel plot
  #
  #
  res <- list(xvals = TE, yvals = weight,
              pch = pch, text = text, cex = cex, col = col,
              bg = bg, cex.studlab = cex.studlab)
  #
  res$xlim <- xlim
  res$ylim <- ylim
  #
  if (!is.null(contour.levels) &
      !(yaxis %in% c("size", "invsqrtsize", "ess"))) {
    res$contour.levels <- contour.levels
    res$col.contour <- col.contour
    #
    tc <- vector("character", 0)
    #
    while (length(contour.levels) >= 2) {
      tc <- c(tc, paste(1 - contour.levels[1], "> p >", 1 - contour.levels[2]))
      contour.levels <- contour.levels[-1]
    }
    #
    res$text.contour <- c(tc, paste("<", 1 - contour.levels))
  }
  
  invisible(res)
}





#' @rdname funnel.meta
#' @export setvals

setvals <- function(x, vals = seq_along(unique(x))) {
  if (is.factor(x))
    levs <- levels(x)
  else
    levs <- sort(unique(x))
  #
  if (length(levs) != length(vals))
    stop("Length of argument 'vals' must be identical to the number of ",
         "unique values in argument 'x'.", call. = FALSE)
  #
  res <- factor(x, levels = levs, labels = vals)
  #
  res <- as.character(res)
  #
  if (is.numeric(vals))
    res <- as.numeric(res)
  #
  res
}

#' Add pooled results from external analysis to meta-analysis
#' 
#' @description
#' Add pooled results from external analysis to an existing
#' meta-analysis object. This is useful, for example, to add results
#' from a Bayesian meta-analysis which is not implemented in R package
#' \bold{meta}.
#' 
#' @param x Meta-analysis object.
#' @param type A character string or vector indicating whether added
#'   results are from common effect, random effects model or
#'   prediction interval. Either \code{"common"}, \code{"random"},
#'   \code{"prediction"}, or \code{"tau2"} can be abbreviated.
#' @param TE Pooled estimate(s) or between-study variance.
#' @param lower Lower limit(s) of confidence or prediction interval.
#' @param upper Upper limit(s) of confidence or prediction interval.
#' @param statistic Test statistic(s).
#' @param pval P-value(s).
#' @param df Degrees of freedom for confidence or prediction intervals.s
#' @param se Standard error(s).
#' @param method A character string or vector to describe the
#'   method used to get the pooled estimate(s), prediction interval(s) or
#'   between-study variance(s).
#' @param method.ci A character string or vector to describe the
#'   method used to get the confidence or prediction interval.
#' @param text A character string or vector used in printouts and
#'   forest plot to label the added results.
#' @param data An optional data frame containing the new results or an
#'   object of class \code{meta}.
#' @param transf A logical indicating whether inputs for arguments
#'   \code{TE}, \code{lower} and \code{upper} are already
#'   appropriately transformed to conduct the meta-analysis or on the
#'   original scale. If \code{transf = TRUE} (default), inputs are
#'   expected to be log odds ratios instead of odds ratios for
#'   \code{sm = "OR"} and Fisher's z transformed correlations instead
#'   of correlations for \code{sm = "ZCOR"}, for example.
#' 
#' @details
#' In R package \bold{meta}, objects of class \code{"meta"} contain
#' results of both common effect and random effects
#' meta-analyses. This function enables the user to add the pooled
#' results of an additional analysis to an existing meta-analysis
#' object. This is useful, for example, to add the result of a
#' Bayesian meta-analysis.
#'
#' If argument \code{data} is a meta-analysis object created with R
#' package \bold{meta}, arguments \code{TE}, \code{lower},
#' \code{upper}, \code{statistic} and \code{pval} are ignored as this
#' information is extracted from the meta-analysis.
#'
#' Otherwise, arguments \code{TE}, \code{lower} and \code{upper} have
#' to be provided if \code{type = "common"} or \code{type =
#' "random"}. For \code{type = "prediction"}, only arguments
#' \code{lower} and \code{upper} are mandatory.
#'
#' Note, R function \code{\link{metamerge}} can be used to add
#' meta-analysis results of another meta-analysis object (see
#' \code{\link{meta-object}}).
#' 
#' @return
#' An object of class \code{"meta"} with corresponding generic
#' functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metagen}}, \code{\link{metamerge}}
#' 
#' @examples
#' data(Fleiss1993bin)
#' 
#' # Common effect and random effects meta-analysis
#' ma1 <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
#'   studlab = paste(study, year), sm = "OR")
#'
#' # Naive pooling
#' ma2 <- metabin(sum(d.asp), sum(n.asp), sum(d.plac), sum(n.plac),
#'   data = Fleiss1993bin, sm = "OR", text.common = "Naive pooling")
#'
#' # Add results of second meta-analysis from common effect model
#' ma12 <- metaadd(ma1, data = ma2)
#' ma12
#'
#' forest(ma12)
#' 
#' @export metaadd

metaadd <- function(x, type = NULL,
                    #
                    TE = NA, lower = NA, upper = NA,
                    #
                    statistic = NA, pval = NA, df = NA, se = NA,
                    #
                    method = "", method.ci = "",
                    #
                    text = "Added result",
                    #
                    data = NULL,
                    #
                    transf = gs("transf")) {
  
  #
  #
  # (1) Check arguments
  #
  #
  
  chkclass(x, "meta")
  res <- updateversion(x)
  #
  if (res$k < 2)
    stop("Meta-analysis object provided in argument 'x' must combine ",
         "at least two studies.",
         call. = FALSE)
  #
  missing.type <- missing(type)
  #
  missing.TE <- missing(TE)
  missing.lower <- missing(lower)
  missing.upper <- missing(upper)
  missing.statistic <- missing(statistic)
  missing.pval <- missing(pval)
  missing.df <- missing(df)
  missing.se <- missing(se)
  #
  missing.method <- missing(method)
  missing.method.ci <- missing(method.ci)
  #
  if (missing.type & missing.TE & missing.lower & missing.upper &
      missing.statistic & missing.pval & missing.df & missing.se)
    return(x)
  #
  missing.text <- missing(text)
  #
  chklogical(transf)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  # Catch 'type', 'TE', 'lower', 'upper', 'statistic', 'pval', 'df', 'se',
  # 'method' and 'method.ci' from data:
  #
  if (inherits(data, "meta")) {
    if (inherits(data, "metaadd"))
      stop("Argument 'data' cannot be of class 'metaadd'.",
           call. = FALSE)
    #
    is.meta <- TRUE
    #
    warnmeta <- function(miss, name)
      if (!miss)
        warning("Argument '", name, "' ignored as argument 'data' is a ",
                "meta-analysis object.",
                call. = FALSE)
    #
    warnmeta(missing.type, "type")
    warnmeta(missing.TE, "TE")
    warnmeta(missing.lower, "lower")
    warnmeta(missing.upper, "upper")
    warnmeta(missing.statistic, "statistic")
    warnmeta(missing.pval, "pval")
    warnmeta(missing.df, "df")
    warnmeta(missing.se, "se")
    #
    warnmeta(missing.method, "method")
    warnmeta(missing.method.ci, "method.ci")
    
    dat.c <- dat.r <- dat.t <- dat.p <- NULL
    #
    if (data$common | !(data$random | data$prediction)) {
      if (any(data$method == "GLMM") & is.null(res$model.glmm))
        res$model.glmm <- data$model.glmm
      #
      dat.c <- data.frame(type = "common",
                          #
                          TE = data$TE.common,
                          se = data$seTE.common,
                          lower = data$lower.common,
                          upper = data$upper.common,
                          statistic = data$statistic.common,
                          pval = data$pval.common,
                          df = NA,
                          #
                          method = data$method,
                          method.ci = data$method.common.ci,
                          #
                          text = data$text.common,
                          phi = NA)
    }
    #
    if (data$random) {
      if (any(data$method.random == "GLMM") & is.null(res$model.glmm))
        res$model.glmm <- data$model.glmm
      #
      dat.r <- data.frame(type = "random",
                          #
                          TE = data$TE.random,
                          se = data$seTE.random,
                          lower = data$lower.random,
                          upper = data$upper.random,
                          statistic = data$statistic.random,
                          pval = data$pval.random,
                          df = data$df.random,
                          #
                          method = data$method.random,
                          method.ci = data$method.random.ci,
                          #
                          text = data$text.random,
                          phi = NA)
      #
      dat.t <- data.frame(type = "tau2",
                          #
                          TE = data$tau2,
                          se = data$se.tau2,
                          lower = data$lower.tau2,
                          upper = data$upper.tau2,
                          statistic = NA,
                          pval = NA,
                          df = NA,
                          #
                          method = data$method.tau,
                          method.ci = data$method.tau.ci,
                          #
                          text = "",
                          #
                          phi = replaceNULL(data$phi))
    }
    #
    if (data$prediction) {
      dat.p <- data.frame(type = "prediction",
                          #
                          TE = NA,
                          se = data$seTE.predict,
                          lower = data$lower.predict,
                          upper = data$upper.predict,
                          statistic = NA,
                          pval = NA,
                          df = data$df.predict,
                          #
                          method = data$method.predict,
                          method.ci = data$method.predict.ci,
                          #
                          text = data$text.predict,
                          phi = NA)
      #
      n.prd <- nrow(dat.p)
    }
    #
    dat <- rbind(dat.c, dat.r, dat.t, dat.p)
    #
    if (nrow(dat) == 0) {
      warning("No pooled results in meta-analysis object.",
              call. = FALSE)  
      return(res)
    }
  }
  else {
    is.meta <- FALSE
    #
    if (!missing.type)
      type <- catch("type", mc, data, sfsp)
    type <- replaceNULL(type, "")
    type <- setchar(type, c("common", "random", "prediction", "tau2"))
    #
    if (any(type %in% c("common", "random", "tau2")))
      if (missing.TE)
        stop("Argument 'TE' must be provided for common effect, ",
             "random effects model, or between-study variance.",
             call. = FALSE)
    #
    if (missing.lower & any(type %in% c("common", "random", "prediction")))
      stop("Argument 'lower' must be provided.",
           call. = FALSE)
    #
    if (missing.upper & any(type %in% c("common", "random", "prediction")))
      stop("Argument 'upper' must be provided.",
           call. = FALSE)
    #
    if (!missing.TE) {
      TE <- catch("TE", mc, data, sfsp)
      chknumeric(TE)
    }
    if (!missing.lower) {
      lower <- catch("lower", mc, data, sfsp)
      chknumeric(lower)
    }
    if (!missing.upper) {
      upper <- catch("upper", mc, data, sfsp)
      chknumeric(upper)
    }
    #
    if (!missing.statistic) {
      statistic <- catch("statistic", mc, data, sfsp)
      chknumeric(statistic)
    }
    if (!missing.pval) {
      pval <- catch("pval", mc, data, sfsp)
      chknumeric(pval)
    }
    if (!missing.df) {
      df <- catch("df", mc, data, sfsp)
      chknumeric(df)
    }
    if (!missing.se) {
      se <- catch("se", mc, data, sfsp)
      chknumeric(se, min = 0)
    }
    #
    if (!missing.method) {
      method <- catch("method", mc, data, sfsp)
      chkchar(method)
    }
    if (!missing.method.ci) {
      method.ci <- catch("method.ci", mc, data, sfsp)
      chkchar(method.ci)
    }
    #
    if (!missing.text) {
      text <- catch("text", mc, data, sfsp)
      chkchar(text)
    }
    #
    dat <- data.frame(type,
                      TE, se, lower, upper, statistic, pval, df,
                      method, method.ci, text)
    #
    # Transform added results
    #
    for (i in seq_len(nrow(dat))) {
      if (!transf && dat$type[i] != "tau2") {
        dat$TE[i] <- transf(dat$TE[i], x$sm, x$func.transf, x$args.transf)
        dat$lower[i] <- transf(dat$lower[i], x$sm, x$func.transf, x$args.transf)
        dat$upper[i] <- transf(dat$upper[i], x$sm, x$func.transf, x$args.transf)
      }
    }
  }
  
  
  #
  #
  # (3) Add results
  #
  #
  
  init.random <- res$random
  #
  j.t <- 0
  #
  for (i in seq_len(nrow(dat))) {
    if (dat$type[i] == "common") {
      if (!res$common)
        res$w.common[!is.na(res$w.common)] <- NA
      #
      res$TE.common <-
        c(if (res$common) res$TE.common, dat$TE[i])
      res$seTE.common <-
        c(if (res$common) res$seTE.common, dat$se[i])
      res$lower.common <-
        c(if (res$common) res$lower.common, dat$lower[i])
      res$upper.common <-
        c(if (res$common) res$upper.common, dat$upper[i])
      res$statistic.common <-
        c(if (res$common) res$statistic.common, dat$statistic[i])
      res$pval.common <-
        c(if (res$common) res$pval.common, dat$pval[i])
      #
      res$method <-
        c(if (res$common) res$method, dat$method[i])
      res$method.common.ci <-
        c(if (res$common) res$method.common.ci, dat$method.ci[i])
      #
      res$text.common <-
        c(if (res$common) res$text.common, dat$text[i])
      #
      res$common <- TRUE
      res$overall <- TRUE
    }
    #
    if (dat$type[i] == "random") {
      if (!res$random)
        res$w.random[!is.na(res$w.random)] <- NA
      #
      res$TE.random <-
        c(if (res$random) res$TE.random, dat$TE[i])
      res$seTE.random <-
        c(if (res$random) res$seTE.random, dat$se[i])
      res$lower.random <-
        c(if (res$random) res$lower.random, dat$lower[i])
      res$upper.random <-
        c(if (res$random) res$upper.random, dat$upper[i])
      res$statistic.random <-
        c(if (res$random) res$statistic.random, dat$statistic[i])
      res$pval.random <-
        c(if (res$random) res$pval.random, dat$pval[i])
      res$df.random <-
        c(if (res$random) res$df.random, dat$df[i])
      #
      res$method.random <-
        c(if (res$random) res$method.random, dat$method[i])
      res$method.random.ci <-
        c(if (res$random) res$method.random.ci, dat$method.ci[i])
      #
      res$text.random <-
        c(if (res$random) res$text.random, dat$text[i])
      #
      res$random <- TRUE
      res$overall <- TRUE
    }
    #
    if (dat$type[i] == "prediction") {
      res$seTE.predict <-
        c(if (res$prediction) res$seTE.predict, dat$se[i])
      res$lower.predict <-
        c(if (res$prediction) res$lower.predict, dat$lower[i])
      res$upper.predict <-
        c(if (res$prediction) res$upper.predict, dat$upper[i])
      res$df.prediction <-
        c(if (res$prediction) res$df.predict, dat$df[i])
      #
      res$method.predict <-
        c(if (res$prediction) res$method.predict, dat$method[i])
      res$method.predict.ci <-
        c(if (res$prediction) res$method.predict.ci, dat$method.ci[i])
      #
      res$text.predict <-
        c(if (res$prediction) res$text.predict, dat$text[i])
      #
      res$prediction <- TRUE
      res$overall <- TRUE
    }
    #
    if (dat$type[i] == "tau2") {
      j.t <- j.t + 1
      #
      sel.t.i <- res$random & ((is.meta & (init.random & j.t > 1)) | !is.meta)
      #
      res$tau2 <- c(if (sel.t.i) res$tau2, dat$TE[i])
      res$se.tau2 <- c(if (sel.t.i) res$se.tau2, dat$se[i])
      res$lower.tau2 <- c(if (sel.t.i) res$lower.tau2, dat$lower[i])
      res$upper.tau2 <- c(if (sel.t.i) res$upper.tau2, dat$upper[i])
      #
      res$method.tau <- c(if (sel.t.i) res$method.tau, dat$method[i])
      res$method.tau.ci <- c(if (sel.t.i) res$method.tau.ci, dat$method.ci[i])
      #
      if (!is.na(dat$phi[i]))
        res$phi <- c(if (sel.t.i) res$phi, dat$phi[i])
      #
      res$random <- TRUE
    }
  }
  #
  # Change results for between-study standard deviation
  #
  res$tau <- sqrt(res$tau2)
  res$lower.tau <- sqrt(res$lower.tau2)
  res$upper.tau <- sqrt(res$upper.tau2)
  #
  if (all(is.na(res$phi)))
    res$phi <- NULL
  #
  class(res) <- c(class(res), "metaadd")
  res
}

#' Meta-analysis of correlations
#' 
#' @description
#' Calculation of common effect and random effects estimates for
#' meta-analyses with correlations; inverse variance weighting is used
#' for pooling.
#' 
#' @param cor Correlations.
#' @param n Number of observations.
#' @param studlab An optional vector with study labels.
#' @param data An optional data frame containing the study
#'   information, i.e., cor and n.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots.
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param sm A character string indicating which summary measure
#'   (\code{"ZCOR"} or \code{"COR"}) is to be used for pooling of
#'   studies.
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param null.effect A numeric value specifying the effect under the
#'   null hypothesis.
#' @param method.bias A character string indicating which test is to
#'   be used. Either \code{"Begg"}, \code{"Egger"}, or
#'   \code{"Thompson"}, can be abbreviated. See function
#'   \code{\link{metabias}}.
#' @param backtransf A logical indicating whether results for Fisher's
#'   z transformed correlations (\code{sm = "ZCOR"}) should be back
#'   transformed in printouts and plots. If TRUE (default), results
#'   will be presented as correlations; otherwise Fisher's z
#'   transformed correlations will be shown.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param adhoc.hakn Deprecated argument (replaced by 'adhoc.hakn.ci').
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}}.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' This function conducts common effect and random effects meta-analysis of
#' correlations based either on Fisher's z transformation of correlations
#' (\code{sm = "ZCOR"}) or direct combination of (untransformed) correlations
#' (\code{sm = "COR"}) (see Cooper et al., 2009, p264-5 and p273-4). Note, the
#' input to argument \code{cor} is always correlations and not Fisher's z
#' transformed correlations if \code{sm = "ZCOR"}.
#' 
#' Only few statisticians would advocate the use of untransformed correlations
#' unless sample sizes are very large (see Cooper et al., 2009, p265). The
#' artificial example given below shows that the smallest study gets the largest
#' weight if correlations are combined directly because the correlation is
#' closest to 1.
#' 
#' A three-level random effects meta-analysis model (Van den Noortgate
#' et al., 2013) is utilised if argument \code{cluster} is used and at
#' least one cluster provides more than one estimate. Internally,
#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
#' "rowsum"} is used to calculate random effects weights.
#' 
#' Default settings are utilised for several arguments (assignments
#' using \code{\link{gs}} function). These defaults can be changed for
#' the current R session using the \code{\link{settings.meta}}
#' function.
#' 
#' Furthermore, R function \code{\link{update.meta}} can be used to
#' rerun a meta-analysis with different settings.
#'
#' \subsection{Subgroup analysis}{
#' 
#' Argument \code{subgroup} can be used to conduct subgroup analysis for
#' a categorical covariate. The \code{\link{metareg}} function can be
#' used instead for more than one categorical covariate or continuous
#' covariates.
#' }
#' 
#' \subsection{Exclusion of studies from meta-analysis}{
#'
#' Arguments \code{subset} and \code{exclude} can be used to exclude
#' studies from the meta-analysis. Studies are removed completely from
#' the meta-analysis using argument \code{subset}, while excluded
#' studies are shown in printouts and forest plots using argument
#' \code{exclude} (see Examples in \code{\link{metagen}}).
#' Meta-analysis results are the same for both arguments.
#' }
#' 
#' \subsection{Presentation of meta-analysis results}{
#' 
#' Internally, both common effect and random effects models are
#' calculated regardless of values choosen for arguments
#' \code{common} and \code{random}. Accordingly, the estimate
#' for the random effects model can be extracted from component
#' \code{TE.random} of an object of class \code{"meta"} even if
#' argument \code{random = FALSE}. However, all functions in R
#' package \bold{meta} will adequately consider the values for
#' \code{common} and \code{random}. E.g. functions
#' \code{\link{print.meta}} and \code{\link{forest.meta}} will not
#' print results for the random effects model if \code{random =
#' FALSE}.
#'
#' A prediction interval will only be shown if \code{prediction =
#' TRUE}.
#' }
#' 
#' @note
#' The function \code{\link{metagen}} is called internally to
#' calculate individual and overall treatment estimates and standard
#' errors.
#' 
#' @return
#' An object of class \code{c("metacor", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{update.meta}},
#'   \code{\link{metacont}}, \code{\link{metagen}},
#'   \code{\link{print.meta}}
#' 
#' @references
#' Cooper H, Hedges LV, Valentine JC (2009):
#' \emph{The Handbook of Research Synthesis and Meta-Analysis},
#' 2nd Edition.
#' New York: Russell Sage Foundation
#'
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#' 
#' @examples
#' ma1 <- metacor(c(0.85, 0.7, 0.95), c(20, 40, 10))
#' 
#' # Print correlations (back transformed from Fisher's z
#' # transformation)
#' #
#' summary(ma1)
#' 
#' # Print Fisher's z transformed correlations 
#' #
#' print(summary(ma1), backtransf = FALSE)
#' 
#' # Forest plot with back transformed correlations
#' #
#' forest(ma1)
#' 
#' # Forest plot with Fisher's z transformed correlations
#' #
#' forest(ma1, backtransf = FALSE)
#' 
#' ma2 <- update(ma1, sm = "cor")
#' summary(ma2)
#'
#' \dontrun{
#' # Identical forest plots (as back transformation is the identity
#' # transformation)
#' forest(ma2)
#' forest(ma2, backtransf = FALSE)
#' }
#' 
#' @export metacor

metacor <- function(cor, n, studlab,
                    #
                    data = NULL, subset = NULL, exclude = NULL,
                    cluster = NULL, rho = 0,
                    #
                    weights = NULL,
                    weights.common = weights, weights.random = weights,
                    #
                    sm = gs("smcor"),
                    level = gs("level"),
                    #
                    common = gs("common"),
                    random = gs("random") | !is.null(tau.preset),
                    overall = common | random,
                    overall.hetstat =
                      if (is.null(gs("overall.hetstat")))
                        common | random
                      else
                        gs("overall.hetstat"),   
                    prediction = gs("prediction") | !missing(method.predict),
                    #
                    method.tau = gs("method.tau"),
                    method.tau.ci = gs("method.tau.ci"),
                    level.hetstat = gs("level.hetstat"),
                    tau.preset = NULL, TE.tau = NULL,
                    tau.common = gs("tau.common"),
                    detail.tau = NULL,
                    #
                    method.I2 = gs("method.I2"),
                    #
                    level.ma = gs("level.ma"),
                    method.common.ci = gs("method.common.ci"),
                    method.random.ci = gs("method.random.ci"),
                    adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                    #
                    level.predict = gs("level.predict"),
                    method.predict = gs("method.predict"),
                    adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                    seed.predict = NULL,
                    #
                    null.effect = 0,
                    #
                    method.bias = gs("method.bias"),
                    #
                    backtransf = gs("backtransf"),
                    #
                    text.common = gs("text.common"),
                    text.random = gs("text.random"),
                    text.predict = gs("text.predict"),
                    text.w.common = gs("text.w.common"),
                    text.w.random = gs("text.w.random"),
                    #
                    title = gs("title"), complab = gs("complab"),
                    outclab = "",
                    #
                    label.left = gs("label.left"),
                    label.right = gs("label.right"),
                    col.label.left = gs("col.label.left"),
                    col.label.right = gs("col.label.right"),
                    #
                    subgroup, subgroup.name = NULL,
                    print.subgroup.name = gs("print.subgroup.name"),
                    sep.subgroup = gs("sep.subgroup"),
                    test.subgroup = gs("test.subgroup"),
                    prediction.subgroup = gs("prediction.subgroup"),
                    seed.predict.subgroup = NULL,
                    #
                    byvar, adhoc.hakn,
                    #
                    keepdata = gs("keepdata"),
                    warn.deprecated = gs("warn.deprecated"),
                    #
                    control = NULL,
                    ...) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  chknumeric(rho, min = -1, max = 1)
  #
  chknull(sm)
  chklevel(level)
  #
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  missing.method.tau <- missing(method.tau)
  method.tau <- setchar(method.tau, gs("meth4tau"))
  #
  missing.tau.common <- missing(tau.common)
  tau.common <- replaceNULL(tau.common, FALSE)
  chklogical(tau.common)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  chklogical(prediction)
  chklevel(level.predict)
  #
  missing.method.predict <- missing(method.predict)
  #
  method.tau <-
    set_method_tau(method.tau, missing.method.tau,
                 method.predict, missing.method.predict)
  method.predict <-
    set_method_predict(method.predict, missing.method.predict,
                     method.tau, missing.method.tau)
  #
  if (any(method.predict == "NNF"))
    is_installed_package("pimeta", argument = "method.predict", value = "NNF")
  #
  adhoc.hakn.pi <- setchar(replaceNA(adhoc.hakn.pi, ""), gs("adhoc4hakn.pi"))
  #
  chknumeric(null.effect, length = 1)
  #
  method.bias <- setmethodbias(method.bias)
  #
  chklogical(backtransf)
  #
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  if (!is.null(text.random))
    chkchar(text.random)
  if (!is.null(text.predict))
    chkchar(text.predict)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chklogical(keepdata)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing(level.ma), args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                      warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.random.ci <-
    deprecated(method.random.ci, missing(method.random.ci),
               args, "hakn", warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  adhoc.hakn.ci <-
    deprecated2(adhoc.hakn.ci, missing(adhoc.hakn.ci),
                adhoc.hakn, missing(adhoc.hakn), warn.deprecated)
  adhoc.hakn.ci <- setchar(replaceNA(adhoc.hakn.ci, ""), gs("adhoc4hakn.ci"))
  #
  missing.subgroup.name <- missing(subgroup.name)
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing(print.subgroup.name),
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing(sep.subgroup), args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  # Some more checks
  #
  chklogical(overall)
  chklogical(overall.hetstat)
  #
  # Additional arguments / checks for metacor objects
  #
  fun <- "metacor"
  sm <- setchar(sm, gs("sm4cor"))
  
  
  #
  #
  # (2) Read data
  #
  #
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  # Catch 'cor' and 'n' from data:
  #
  cor <- catch("cor", mc, data, sfsp)
  chknull(cor)
  k.All <- length(cor)
  #
  n <- catch("n", mc, data, sfsp)
  chknull(n)
  #
  # Catch 'studlab', 'subgroup', 'subset', 'exclude' and 'cluster'
  # from data:
  #
  studlab <- catch("studlab", mc, data, sfsp)
  studlab <- setstudlab(studlab, k.All)
  #
  missing.subgroup <- missing(subgroup)
  subgroup <- catch("subgroup", mc, data, sfsp)
  missing.byvar <- missing(byvar)
  byvar <- catch("byvar", mc, data, sfsp)
  subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar)
  by <- !is.null(subgroup)
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  exclude <- catch("exclude", mc, data, sfsp)
  missing.exclude <- is.null(exclude)
  #
  cluster <- catch("cluster", mc, data, sfsp)
  with.cluster <- !is.null(cluster)
  #
  # Additional checks
  #
  if (!by & tau.common) {
    warning("Value for argument 'tau.common' set to FALSE as ",
            "argument 'subgroup' is missing.")
    tau.common <- FALSE
  }
  if (by & !tau.common & !is.null(tau.preset)) {
    warning("Argument 'tau.common' set to TRUE as ",
            "argument tau.preset is not NULL.")
    tau.common <- TRUE
  }
  #
  # Catch 'weights', 'weights.common', and 'weights.random' from data:
  #
  if (!missing(weights))
    weights <- catch("weights", mc, data, sfsp)
  if (!missing(weights.common))
    weights.common <- catch("weights.common", mc, data, sfsp)
  if (!missing(weights.random))
    weights.random <- catch("weights.random", mc, data, sfsp)
  #
  if (!is.null(weights) & is.null(weights.common))
    weights.common <- weights
  #
  if (!is.null(weights) & is.null(weights.random))
    weights.random <- weights
  #
  usw.common <- !is.null(weights.common)
  usw.random <- !is.null(weights.random)
  #
  if (usw.common)
    chknumeric(weights.common, min = 0)
  #
  if (usw.random)
    chknumeric(weights.random, min = 0)
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  chklength(n, k.All, fun)
  chklength(studlab, k.All, fun)
  if (with.cluster)
    chklength(cluster, k.All, fun)
  #
  if (usw.common) {
    if (length(weights.common) == 1)
      weights.common <- rep(weights.common, k.All)
    else
      chklength(weights.common, k.All, fun)
  }
  #
  if (usw.random) {
    if (length(weights.random) == 1)
      weights.random <- rep(weights.random, k.All)
    else
      chklength(weights.random, k.All, fun)
  }
  #
  if (by) {
    chklength(subgroup, k.All, fun)
    chklogical(test.subgroup)
    chklogical(prediction.subgroup)
  }
  
  
  #
  #
  # (4) Subset, exclude studies, and subgroups
  #
  #
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of subset is larger than number of studies.")
  #
  if (!missing.exclude) {
    if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
        (length(exclude) > k.All))
      stop("Length of argument 'exclude' is larger than number of studies.")
    #
    exclude2 <- rep(FALSE, k.All)
    exclude2[exclude] <- TRUE
    exclude <- exclude2
  }
  else
    exclude <- rep(FALSE, k.All)
  
  
  #
  #
  # (5) Store complete dataset in list object data
  #     (if argument keepdata is TRUE)
  #
  #
  if (keepdata) {
    if (nulldata)
      data <- data.frame(.cor = cor)
    else
      data$.cor <- cor
    #
    data$.n <- n
    data$.studlab <- studlab
    #
    if (by)
      data$.subgroup <- subgroup
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
    #
    if (!missing.exclude)
      data$.exclude <- exclude
    #
    if (with.cluster)
      data$.id <- data$.cluster <- cluster
    #
    if (usw.common)
      data$.weights.common <- weights.common
    #
    if (usw.random)
      data$.weights.random <- weights.random
  }
  
  
  #
  #
  # (6) Use subset for analysis
  #
  #
  if (!missing.subset) {
    cor <- cor[subset]
    n   <- n[subset]
    studlab <- studlab[subset]
    #
    cluster <- cluster[subset]
    exclude <- exclude[subset]
    #
    weights.common <- weights.common[subset]
    weights.random <- weights.random[subset]
    #
    if (by)
      subgroup <- subgroup[subset]
  }
  #
  # Determine total number of studies
  #
  k.all <- length(cor)
  #
  if (k.all == 0)
    stop("No studies to combine in meta-analysis.")
  #
  # No meta-analysis for a single study
  #
  if (k.all == 1) {
    common <- FALSE
    random <- FALSE
    prediction <- FALSE
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  #
  # Check variable values
  #
  chknumeric(cor, -1, 1)
  chknumeric(n, 0, zero = TRUE)
  #
  if (by) {
    chkmiss(subgroup)
    #
    if (missing.subgroup.name & is.null(subgroup.name)) {
      if (!missing.subgroup)
        subgroup.name <- byvarname("subgroup", mc)
      else if (!missing.byvar)
        subgroup.name <- byvarname("byvar", mc)
    }
  }
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  
  
  #
  #
  # (7) Calculate results for individual studies
  #
  #
  if (sm == "ZCOR") {
    TE   <- cor2z(cor)
    seTE <- sqrt(1 / (n - 3))
    transf.null.effect <- 0.5 * log((1 + null.effect) / (1 - null.effect))
  }
  if (sm == "COR") {
    TE <- cor
    seTE <- sqrt((1 - cor^2)^2 / (n - 1))
    transf.null.effect <- null.effect
  }
  
  
  #
  #
  # (8) Additional checks for three-level model
  #
  #
  three.level <- FALSE
  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (with.cluster &&
      length(unique(cluster[sel.ni])) != length(cluster[sel.ni]))
    three.level <- TRUE
  #
  if (three.level) {
    chkmlm(method.tau, missing.method.tau, method.predict)
    #
    common <- FALSE
    #
    if (!(method.tau %in% c("REML", "ML")))
      method.tau <- "REML"
  }
  
  
  #
  #
  # (9) Do meta-analysis
  #
  #
  m <- metagen(TE, seTE, studlab,
               exclude = if (missing.exclude) NULL else exclude,
               cluster = cluster, rho = rho,
               #
               weights.common = weights.common,
               weights.random = weights.random,
               #
               sm = sm,
               level = level,
               #
               common = common,
               random = random,
               overall = overall,
               overall.hetstat = overall.hetstat,
               prediction = prediction,
               #
               method.tau = method.tau, method.tau.ci = method.tau.ci,
               level.hetstat = level.hetstat,
               tau.preset = tau.preset,
               TE.tau = TE.tau,
               tau.common = FALSE,
               detail.tau = detail.tau,
               #
               method.I2 = method.I2,
               #
               level.ma = level.ma,
               method.common.ci = method.common.ci,
               method.random.ci = method.random.ci,
               adhoc.hakn.ci = adhoc.hakn.ci,
               #
               level.predict = level.predict,
               method.predict = method.predict,
               adhoc.hakn.pi = adhoc.hakn.pi,
               seed.predict = seed.predict,
               #
               null.effect = transf.null.effect,
               #
               method.bias = method.bias,
               #
               backtransf = backtransf,
               #
               text.common = text.common, text.random = text.random,
               text.predict = text.predict,
               text.w.common = text.w.common, text.w.random = text.w.random,
               #
               title = title, complab = complab, outclab = outclab,
               #
               label.left = label.left, label.right = label.right,
               col.label.left = col.label.left,
               col.label.right = col.label.right,
               #
               keepdata = FALSE,
               warn = FALSE,
               #
               control = control)
  #
  # Estimate common tau-squared across subgroups
  #
  if (by & tau.common)
    hcc <- hetcalc(TE, seTE, method.tau, "", TE.tau,
                   method.I2, level.hetstat, subgroup, control)
  
  
  #
  #
  # (9) Generate R object
  #
  #
  res <- list(cor = cor, n = n)
  #
  # Add meta-analysis results
  # (after removing unneeded list elements)
  #
  m$n.e <- NULL
  m$n.c <- NULL
  m$pscale <- NULL
  m$irscale <- NULL
  m$irunit <- NULL
  m$method.ci <- NULL
  m$method.mean <- NULL
  m$approx.TE <- NULL
  m$approx.seTE <- NULL
  #
  m$label.e <- ""
  m$label.c <- ""
  m$warn <- NULL
  #
  res <- c(res, m)
  res$null.effect <- null.effect
  #
  # Add data
  #
  res$pairwise <- FALSE
  #
  res$call <- match.call()
  #
  if (keepdata) {
    res$data <- data
    if (!missing.subset)
      res$subset <- subset
  }
  #
  class(res) <- c(fun, "meta")
    #
  # Add results from subgroup analysis
  #
  if (by) {
    res$subgroup <- subgroup
    res$subgroup.name <- subgroup.name
    res$print.subgroup.name <- print.subgroup.name
    res$sep.subgroup <- sep.subgroup
    res$test.subgroup <- test.subgroup
    res$prediction.subgroup <- prediction.subgroup
    res$tau.common <- tau.common
    #
    if (!tau.common) {
      res <- c(res, subgroup(res, seed = seed.predict.subgroup))
      if (res$three.level)
        res <- setNA3(res)
    }
    else if (!is.null(tau.preset))
      res <-
        c(res, subgroup(res, tau.preset, seed = seed.predict.subgroup))
    else {
      if (res$three.level)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup))))
      else
        res <-
          c(res, subgroup(res, hcc$tau.resid, seed = seed.predict.subgroup))
    }
    #
    if (tau.common && is.null(tau.preset))
      res <- addHet(res, hcc)
    #
    res$event.w <- NULL
    #
    res$n.e.w <- NULL
    res$n.c.w <- NULL
    res$n.harmonic.mean.w <- NULL
    #
    res$event.e.w <- NULL
    res$event.c.w <- NULL
    #
    res$time.e.w <- NULL
    res$time.c.w <- NULL
    res$t.harmonic.mean.w <- NULL
    #
    res <- setNAwithin(res, res$three.level)
  }
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  class(res) <- c(fun, "meta")
  
  
  res
}

setlistname <- function(x, name) {
  if (!is.list(x))
    x <- list(x)
  #
  if (!is.null(name) & is.null(names(x)))
    names(x) <- name
  #
  x
}

setname <- function(x, name) {
  if (!is.null(x) &&
      is.null(names(x)) &&
      !(is.null(name) || all(name == ""))) {
    nx <- length(x)
    #
    if (length(name) == nx)
      names(x) <- name
    else if (length(name) == 1)
      names(x) <- rep_len(name, nx)
  }
  #
  x
}

samedata <- function(x, y, stop = TRUE) {
  mismatch <- FALSE
  #
  if (!is.null(x$data) & !is.null(y$data)) {
    #
    if (nrow(x$data) != nrow(y$data)) {
      if (stop)
        stop("Meta-analyses based on different data sets.",
             call. = FALSE)
      else
        return(TRUE)
    }
    #
    if (inherits(x, "metabin")) {
      mismatch1 <-
        mismatch(x, y, ".event.e")
      mismatch2 <-
        mismatch(x, y, ".n.e")
      mismatch3 <-
        mismatch(x, y, ".event.c")
      mismatch4 <-
        mismatch(x, y, ".n.c")
      #
      mismatch <- mismatch1 | mismatch2 | mismatch3 | mismatch4
    }
    else if (inherits(x, "metacont")) {
      mismatch1 <-
        mismatch(x, y, ".n.e")
      mismatch2 <-
        mismatch(x, y, ".mean.e")
      mismatch3 <-
        mismatch(x, y, ".sd.e")
      mismatch4 <-
        mismatch(x, y, ".n.c")
      mismatch5 <-
        mismatch(x, y, ".mean.c")
      mismatch6 <-
        mismatch(x, y, ".sd.c")
      #
      mismatch <-
        mismatch1 | mismatch2 | mismatch3 | mismatch4 |
        mismatch5 | mismatch6
    }
    else if (inherits(x, "metacor")) {
      mismatch1 <-
        mismatch(x, y, ".cor")
      mismatch2 <-
        mismatch(x, y, ".n")
      #
      mismatch <- mismatch1 | mismatch2
    }
    else if (inherits(x, "metagen")) {
      #
      mismatch1 <-
        mismatch(x, y, ".n.e")
      mismatch2 <-
        mismatch(x, y, ".n.c")
      #
      mismatch <- mismatch1 | mismatch2
    }
    else if (inherits(x, "metainc")) {
      mismatch1 <-
        mismatch(x, y, ".time.e")
      mismatch2 <-
        mismatch(x, y, ".n.e")
      mismatch3 <-
        mismatch(x, y, ".time.c")
      mismatch4 <-
        mismatch(x, y, ".n.c")
      #
      mismatch <- mismatch1 | mismatch2 | mismatch3 | mismatch4
    }
    else if (inherits(x, "metamean")) {
      mismatch1 <-
        mismatch(x, y, ".n")
      mismatch2 <-
        mismatch(x, y, ".mean")
      mismatch3 <-
        mismatch(x, y, ".sd")
      #
      mismatch <-
        mismatch1 | mismatch2 | mismatch3
    }
    else if (inherits(x, "metaprop")) {
      mismatch1 <-
        mismatch(x, y, ".event")
      mismatch2 <-
        mismatch(x, y, ".n")
      #
      mismatch <- mismatch1 | mismatch2
    }
    else if (inherits(x, "metarate")) {
      mismatch1 <-
        mismatch(x, y, ".time")
      mismatch2 <-
        mismatch(x, y, ".n")
      #
      mismatch <- mismatch1 | mismatch2
    }
    #
    if (mismatch & stop)
      stop("Meta-analyses based on different data sets.",
           call. = FALSE)
  }
  #
  invisible(mismatch)
}


samesm <- function(x, y) {
  if (!is.null(x$sm) & !is.null(y$sm)) {
    if (is_relative_effect(x$sm) != is_relative_effect(y$sm))
      stop("Summary measures used in meta-analyses do not fit.",
           call. = FALSE)
  }
  #
  if (inherits(x, "metabin")) {
    if ((x$sm != y$sm) &
        any(c(x$sm, y$sm) %in% c("RD", "ASD")))
      stop("Summary measures used in meta-analyses do not fit.",
           call. = FALSE)
  }
  #
  invisible(NULL)
}


samesubgroups <- function(x, y) {
  if (!is.null(x$subgroup.levels) &
      !is.null(y$subgroup.levels)) {
    #
    if (length(x$subgroup.levels) !=
        length(y$subgroup.levels))
      stop("Meta-analyses have different number of subgroups.",
           call. = FALSE)
    #
    if (any(x$subgroup.levels != y$subgroup.levels))
      stop("Meta-analyses based on different subgroup-analyses.",
           call. = FALSE)
  }
  #
  invisible(NULL)
}


updateobj <- function(x,
                      label.common, label.random, label.predict,
                      hetlabel, taulabel, label.subgroup,
                      text.common, text.random, text.predict,
                      text.w.common, text.w.random) {
  
  is.merge <- inherits(x, "metamerge")
  is.copas <- !is.merge & inherits(x, "copas")
  is.limit <- !is.merge & inherits(x, "limitmeta")
  is.robu  <- !is.merge & inherits(x, "robu")
  is.tf    <- !is.merge & inherits(x, "trimfill")
  #
  res <- x
  #
  if (is.null(x$hetlabel) || all(x$hetlabel == ""))
    res$hetlabel <- replaceNULL(hetlabel, "")
  #
  if (all(x$detail.tau == ""))
    res$detail.tau <- replaceNULL(taulabel, "")
  #
  if (!is.merge) {    
    res$TE.common <- setname(res$TE.common, label.common)
    res$seTE.common <- setname(res$seTE.common, label.common)
    res$statistic.common <- setname(res$statistic.common, label.common)
    res$pval.common <- setname(res$pval.common, label.common)
    res$lower.common <- setname(res$lower.common, label.common)
    res$upper.common <- setname(res$upper.common, label.common)
    res$zval.common <- setname(res$zval.common, label.common)
    #
    res$TE.random <- setname(res$TE.random, label.random)
    res$seTE.random <- setname(res$seTE.random, label.random)
    res$statistic.random <- setname(res$statistic.random, label.random)
    res$pval.random <- setname(res$pval.random, label.random)
    res$method.random.ci <- setname(res$method.random.ci, label.random)
    res$df.random <- setname(res$df.random, label.random)
    res$lower.random <- setname(res$lower.random, label.random)
    res$upper.random <- setname(res$upper.random, label.random)
    res$zval.random <- setname(res$zval.random, label.random)
    res$seTE.classic <- setname(res$seTE.classic, label.random)
    res$adhoc.hakn.ci <- setname(res$adhoc.hakn.ci, label.random)
    res$df.hakn <- setname(res$df.hakn, label.random)
    res$seTE.hakn.ci <- setname(res$seTE.hakn.ci, label.random)
    res$seTE.hakn.adhoc.ci <- setname(res$seTE.hakn.adhoc.ci, label.random)
    res$df.kero <- setname(res$df.kero, label.random)
    res$seTE.kero <- setname(res$seTE.kero, label.random)
    #
    res$seTE.predict <- setname(res$seTE.predict, label.predict)
    res$df.predict <- setname(res$df.predict, label.predict)
    res$lower.predict <- setname(res$lower.predict, label.predict)
    res$upper.predict <- setname(res$upper.predict, label.predict)
    res$adhoc.hakn.pi <- setname(res$adhoc.hakn.pi, label.predict)
    res$seTE.hakn.pi <- setname(res$seTE.hakn.pi, label.predict)
    res$seTE.hakn.adhoc.pi <- setname(res$seTE.hakn.adhoc.pi, label.predict)
    #
    res$Q <- setname(res$Q, hetlabel)
    res$df.Q <- setlistname(res$df.Q, hetlabel)
    res$pval.Q <- setname(res$pval.Q, hetlabel)
    #
    res$I2 <- setname(res$I2, hetlabel)
    res$lower.I2 <- setname(res$lower.I2, hetlabel)
    res$upper.I2 <- setname(res$upper.I2, hetlabel)
    #
    res$H <- setname(res$H, hetlabel)
    res$lower.H <- setname(res$lower.H, hetlabel)
    res$upper.H <- setname(res$upper.H, hetlabel)
    #
    res$Rb <- setname(res$Rb, hetlabel)
    res$lower.Rb <- setname(res$lower.Rb, hetlabel)
    res$upper.Rb <- setname(res$upper.Rb, hetlabel)
    #
    res$TE.common.w <- setlistname(res$TE.common.w, label.subgroup)
    res$seTE.common.w <- setlistname(res$seTE.common.w, label.subgroup)
    res$statistic.common.w <-
      setlistname(res$statistic.common.w, label.subgroup)
    res$pval.common.w <- setlistname(res$pval.common.w, label.subgroup)
    res$lower.common.w <- setlistname(res$lower.common.w, label.subgroup)
    res$upper.common.w <- setlistname(res$upper.common.w, label.subgroup)
    #
    res$w.common.w <- setlistname(res$w.common.w, label.subgroup)
    #
    res$TE.random.w <- setlistname(res$TE.random.w, label.subgroup)
    res$seTE.random.w <- setlistname(res$seTE.random.w, label.subgroup)
    res$statistic.random.w <-
      setlistname(res$statistic.random.w, label.subgroup)
    res$pval.random.w <- setlistname(res$pval.random.w, label.subgroup)
    res$df.random.w <- setlistname(res$df.random.w, label.subgroup)
    res$lower.random.w <- setlistname(res$lower.random.w, label.subgroup)
    res$upper.random.w <- setlistname(res$upper.random.w, label.subgroup)
    res$df.hakn.w <- setlistname(res$df.hakn.w, label.subgroup)
    res$df.kero.w <- setlistname(res$df.kero.w, label.subgroup)
    #
    res$w.random.w <- setlistname(res$w.random.w, label.subgroup)
    #
    res$seTE.predict.w <- setlistname(res$seTE.predict.w, label.subgroup)
    res$df.predict.w <- setlistname(res$df.predict.w, label.subgroup)
    res$lower.predict.w <- setlistname(res$lower.predict.w, label.subgroup)
    res$upper.predict.w <- setlistname(res$upper.predict.w, label.subgroup)
    #
    res$Q.w <- setname(res$Q.w, label.subgroup)
    res$tau.w <- setlistname(res$tau.w, label.subgroup)
    #
    res$Q.b.common <- setname(res$Q.b.common, label.subgroup)
    res$df.Q.b.common <- setlistname(res$df.Q.b.common, label.subgroup)
    res$pval.Q.b.common <- setname(res$pval.Q.b.common, label.subgroup)
    #
    res$Q.b.random <- setname(res$Q.b.random, label.subgroup)
    res$pval.Q.b.random <- setname(res$pval.Q.b.random, label.subgroup)
  }
  #
  # Act upon ordinary meta-analysis object
  #
  if (!(is.copas | is.limit | is.robu | is.tf)) {
    if (!(is.null(label.common) || label.common == "") &
        is.null(text.common))
      res$text.common <- paste0(res$text.common, " (", label.common, ")")
    else if (!is.null(text.common))
      res$text.common <- text.common
    #
    if (!is.null(label.random) & is.null(text.random))
      res$text.random <- paste0(res$text.random, " (", label.random, ")")
    else if (!is.null(text.random))
      res$text.random <- text.random
    #
    if (!is.null(label.predict) & is.null(text.predict))
      res$text.predict <- paste0(res$text.predict, " (", label.predict, ")")
    else if (!is.null(text.predict))
      res$text.predict <- text.predict
    #
    if (!is.null(label.common) & is.null(text.w.common))
      res$text.w.common <- label.common
    else if (!is.null(text.w.common))
      res$text.w.common <- text.w.common
    #
    if (!is.null(label.random) & is.null(text.w.random))
      res$text.w.random <- label.random
    else if (!is.null(text.w.random))
      res$text.w.random <- text.w.random
    #
    return(res)
  }
  #
  # Other objects
  #
  res$text.predict <- ""
  #
  if (is.copas | is.limit | is.robu) {
    res$method.random <- "Inverse"
    #
    res$method.random.ci <- "classic"
    res$adhoc.hakn.ci <- ""
    res$df.random <- Inf
    #
    res$method.predict <- NULL
    res$adhoc.hakn.pi <- NULL
    res$seTE.predict <- NULL
    res$df.predict <- NULL
    res$level.predict <- NULL
    res$lower.predict <- NULL
    res$upper.predict <- NULL
    res$seTE.hakn.pi <- NULL
    res$seTE.hakn.adhoc.pi <- NULL
  }
  #
  if (is.copas) {
    if (is.null(label.random))
      label.random <- "copas"
    #
    if (is.null(hetlabel))
      hetlabel <- "copas"
    #
    if (is.null(taulabel))
      taulabel <- "copas"
    #
    res$TE.random <- setname(res$TE.adjust, label.random)
    res$seTE.random <- setname(res$seTE.adjust, label.random)
    res$lower.random <- setname(res$lower.adjust, label.random)
    res$upper.random <- setname(res$upper.adjust, label.random)
    res$statistic.random <- setname(res$statistic.adjust, label.random)
    res$pval.random <- setname(res$pval.adjust, label.random)
    #
    res$w.random <- rep(0, length(res$w.random))
    #
    res$tau <- setname(res$tau.adjust, taulabel)
    res$lower.tau <- NA
    res$upper.tau <- NA
    res$tau2 <- setname(res$tau.adjust^2, taulabel)
    res$lower.tau2 <- NA
    res$upper.tau2 <- NA
    res$se.tau <- NA
    #
    res$method.tau <- "ML"
    #
    res$hetlabel <- hetlabel
    res$detail.tau <- taulabel
    #
    if (is.null(text.random))
      res$text.random <- "Copas selection model"
    else
      res$text.random <- text.random
    #
    if (is.null(text.w.random))
      res$text.w.random <- "Copas"
    else
      res$text.w.random <- text.w.random
    #
    res$k.study <- sum(is.finite(res$TE))
  }
  else if (is.limit) {
    if (is.null(label.random))
      label.random <- "limit"
    #
    if (is.null(hetlabel))
      hetlabel <- "limit"
    #
    if (is.null(taulabel))
      taulabel <- "limit"
    #
    res$TE.random <- setname(res$TE.adjust, label.random)
    res$seTE.random <- setname(res$seTE.adjust, label.random)
    res$lower.random <- setname(res$lower.adjust, label.random)
    res$upper.random <- setname(res$upper.adjust, label.random)
    res$statistic.random <- setname(res$statistic.adjust, label.random)
    res$pval.random <- setname(res$pval.adjust, label.random)
    #
    res$w.random <- rep(0, length(res$w.random))
    #
    res$tau <- setname(res$tau, taulabel)
    res$lower.tau <- NA
    res$upper.tau <- NA
    res$tau2 <- setname(res$tau2, taulabel)
    res$lower.tau2 <- NA
    res$upper.tau2 <- NA
    res$se.tau <- NA
    #
    res$method.tau <- res$x$method.tau
    #
    res$hetlabel <- hetlabel
    res$detail.tau <- taulabel
    #
    if (is.null(text.random))
      res$text.random <- "Limit meta-analysis"
    else
      res$text.random <- text.random
    #
    if (is.null(text.w.random))
      res$text.w.random <- "limit"
    else
      res$text.w.random <- text.w.random
    #
    res$k.study <- res$k
  }
  else if (is.robu) {
    if (is.null(label.random))
      label.random <- "RVE"
    #
    if (is.null(hetlabel))
      hetlabel <- "RVE"
    #
    if (is.null(taulabel))
      taulabel <- "RVE"
    #
    res$TE.random <- setname(res$reg_table$b.r[1], label.random)
    res$seTE.random <- setname(res$reg_table$SE[1], label.random)
    res$lower.random <- setname(res$reg_table$CI.L[1], label.random)
    res$upper.random <- setname(res$reg_table$CI.U[1], label.random)
    res$statistic.random <- setname(res$reg_table$t[1], label.random)
    res$pval.random <- setname(res$reg_table$prob[1], label.random)
    #
    res$w.random <- res$data.full$r.weights
    #
    res$level.ma <- 0.95
    #
    res$tau <- setname(sqrt(res$mod_info$tau.sq), taulabel)
    res$lower.tau <- NA
    res$upper.tau <- NA
    res$tau2 <- setname(res$mod_info$tau.sq, taulabel)
    res$lower.tau2 <- NA
    res$upper.tau2 <- NA
    res$se.tau <- NA
    #
    res$method.tau <- "DL"
    #
    res$hetlabel <- hetlabel
    res$detail.tau <- taulabel
    #
    if (is.null(text.random))
      res$text.random <- "RVE model"
    else
      res$text.random <- text.random
    #
    if (is.null(text.w.random))
      res$text.w.random <- "RVE"
    else
      res$text.w.random <- text.w.random
  }
  else if (is.tf) {
    if (is.null(hetlabel))
      hetlabel <- "TF"
    #
    if (is.null(taulabel))
      taulabel <- "TF"
    #
    if (!(is.null(label.common) || label.common == "") &
        is.null(text.common))
      res$text.common <- paste0(res$text.common, " (", label.common, ")")
    else if (!is.null(text.common))
      res$text.common <- text.common
    else
      res$text.common <- "Trim-and-fill method (CE)"
    #
    if (!is.null(label.random) & is.null(text.random))
      res$text.random <- paste0(res$text.random, " (", label.random, ")")
    else if (!is.null(text.random))
      res$text.random <- text.random
    else
      res$text.random <- "Trim-and-fill method (RE)"
    #
    res$hetlabel <- hetlabel
    res$detail.tau <- taulabel
    #
    if ((!is.null(label.common) & is.null(text.w.common)) ||
        !is.null(text.w.common))
      res$text.w.common <- text.w.common
    else
      res$text.w.common <- "CE-TF"
    #
    if ((!is.null(label.random) & is.null(text.w.random)) ||
        !is.null(text.w.random))
      res$text.w.random <- text.w.random
    else
      res$text.w.random <- "RE-TF"
  }
  
  res
}

dropcommon <- function(x) {
  res <- x
  #
  res$method <- NULL
  #
  res$w.common <- NULL
  res$TE.common <- NULL
  res$seTE.common <- NULL
  res$statistic.common <- NULL
  res$pval.common <- NULL
  res$lower.common <- NULL
  res$upper.common <- NULL
  res$zval.common <- NULL
  res$text.common <- NULL
  #
  res$k.MH <- NULL
  #
  res$TE.common.w <- NULL
  res$seTE.common.w <- NULL
  res$statistic.common.w <- NULL
  res$pval.common.w <- NULL
  res$lower.common.w <- NULL
  res$upper.common.w <- NULL
  res$w.common.w <- NULL
  #
  res$Q.w.common <- NULL
  res$pval.Q.w.common <- NULL
  #
  res$Q.b.common <- NULL
  res$df.Q.b.common <- NULL
  res$pval.Q.b.common <- NULL
  #
  res$zval.common.w <- NULL
  res$TE.fixed.w <- NULL
  res$seTE.fixed.w <- NULL
  res$lower.fixed.w <- NULL
  res$upper.fixed.w <- NULL
  res$statistic.fixed.w <- NULL
  res$pval.fixed.w <- NULL
  res$zval.fixed.w <- NULL
  res$w.fixed.w <- NULL
  res$Q.w.fixed <- NULL
  res$pval.Q.w.fixed <- NULL
  res$Q.b.fixed <- NULL
  res$pval.Q.b.fixed <- NULL
  #
  res
}

droprandom <- function(x) {
  res <- x
  #
  res$method.random <- NULL
  #
  res$w.random <- NULL
  res$TE.random <- NULL
  res$seTE.random <- NULL
  res$statistic.random <- NULL
  res$pval.random <- NULL
  res$method.random.ci <- NULL
  res$df.random <- NULL
  res$lower.random <- NULL
  res$upper.random <- NULL
  res$zval.random <- NULL
  #
  res$seTE.classic <- NULL
  #
  res$adhoc.hakn.ci <- NULL
  res$df.hakn <- NULL
  res$seTE.hakn.ci <- NULL
  res$seTE.hakn.adhoc.ci <- NULL
  #
  res$df.kero <- NULL
  res$seTE.kero <- NULL
  #
  res$text.random <- NULL
  #
  res$cluster <- FALSE
  res$three.level <- FALSE
  #
  res$k.study <- res$k
  #
  res$TE.random.w <- NULL
  res$seTE.random.w <- NULL
  res$statistic.random.w <- NULL
  res$pval.random.w <- NULL
  res$df.random.w <- NULL
  res$lower.random.w <- NULL
  res$upper.random.w <- NULL
  res$w.random.w <- NULL
  #
  res$seTE.classic.w <- NULL
  #
  res$df.hakn.ci.w <- NULL
  res$seTE.hakn.ci.w <- NULL
  res$seTE.hakn.adhoc.ci.w <- NULL
  #
  res$df.kero.w <- NULL
  res$seTE.kero.w <- NULL
  #
  res$Q.w.random <- NULL
  res$pval.Q.w.random <- NULL
  #
  res$Q.b.random <- NULL
  res$df.Q.b.random <- NULL
  res$pval.Q.b.random <- NULL
  #
  res$zval.random.w <- NULL
  #
  res
}

droppredict <- function(x) {
  res <- x
  #
  res$method.predict <- NULL
  res$adhoc.hakn.pi <- NULL
  res$df.hakn.pi <- NULL
  res$seTE.predict <- NULL
  res$df.predict <- NULL
  res$lower.predict <- NULL
  res$upper.predict <- NULL
  res$seTE.hakn.pi <- NULL
  res$seTE.hakn.adhoc.pi <- NULL
  res$text.predict <- NULL
  #      
  res$seTE.predict.w <- NULL
  res$df.predict.w <- NULL
  res$lower.predict.w <- NULL
  res$upper.predict.w <- NULL
  res$seTE.hakn.pi.w <- NULL
  res$seTE.hakn.adhoc.pi.w <- NULL
  #      
  res$prediction.subgroup <- FALSE
  #
  res
}


expandmerge <- function(x, y, nc1 = 0, nr1 = 0, nc2 = 0, nr2 = 0) {
  n1 <- nc1 + nr1
  n2 <- nc2 + nr2
  #
  if ((is.null(x) & is.null(y)))
    return(NULL)
  else if (length(x) == n1 & length(y) == n2)
    return(c(x[seq_len(nc1)], y[seq_len(nc2)],
             x[nc1 + seq_len(nr1)], y[nc2 + seq_len(nr2)]))
  #
  if (length(x) == 1 & n1 > 1)
    x <- rep(x, n1)
  if (length(y) == 1 & n2 > 1)
    y <- rep(y, n2)
  #
  if (length(x) == 0) {
    if (is.character(y))
      x <- rep_len("", n1)
    else
      x <- rep_len(NA, n1)
  }
  if (length(y) == 0) {
    if (is.character(x))
      y <- rep_len("", n2)
    else
      y <- rep_len(NA, n2)
  }
  #
  res <- c(x[seq_len(nc1)], y[seq_len(nc2)],
           x[nc1 + seq_len(nr1)], y[nc2 + seq_len(nr2)])
  res
}

#' Description of R object of class "meta"
#' 
#' @description
#' Detailed description of R objects of class "meta".
#' 
#' @details
#'
#' The following R functions create an object of class \code{"meta"}:
#' \itemize{
#' \item \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metacor}}, \code{\link{metagen}},
#'   \code{\link{metainc}}, \code{\link{metamean}},
#'   \code{\link{metaprop}}, \code{\link{metarate}},
#'   \code{\link{metacr}}, \code{\link{metamerge}},
#'   \code{\link{trimfill}}
#' }
#'
#' The following generic functions are available for an object of
#' class \code{"meta"}:
#'
#' \itemize{
#' \item \code{\link{as.data.frame.meta}}, \code{\link{labels.meta}},
#'   \code{\link{print.meta}}, \code{\link{print.summary.meta}},
#'   \code{\link{summary.meta}}, \code{\link{update.meta}},
#'   \code{\link{weights.meta}}
#' }
#'
#' An object of class \code{"meta"} is a list containing the following
#' components.
#' \tabular{ll}{
#' \code{studlab} \tab Study labels \cr
#' \code{sm} \tab Effect measure \cr
#' \code{null.effect} \tab Effect under the null hypothesis \cr
#' \code{TE} \tab Effect estimates (individual studies) \cr
#' \code{seTE} \tab Standard error of effect estimates (individual studies) \cr
#' \code{statistic} \tab Statistics for test of effect (individual studies) \cr
#' \code{pval} \tab P-values for test of effect (individual studies) \cr
#' \code{df} \tab Degrees of freedom (individual studies) \cr
#' \code{level} \tab Level of confidence intervals for individual studies \cr
#' \code{lower} \tab Lower confidence limits (individual studies) \cr
#' \code{upper} \tab Upper confidence limits (individual studies) \cr
#' \code{three.level} \tab Indicator variable for three-level
#'   meta-analysis model \cr
#' \code{cluster} \tab Cluster variable (three-level meta-analysis
#'   model) \cr
#' \code{rho} \tab Within-cluster correlation (three-level
#'   meta-analysis model) \cr
#' \code{k} \tab Number of estimates combined in meta-analysis \cr
#' \code{k.study} \tab Number of studies combined in meta-analysis \cr
#' \code{k.all} \tab Number of all studies \cr
#' \code{k.TE} \tab Number of studies with estimable effects \cr
#' \code{overall} \tab Print meta-analysis results \cr
#' \code{overall.hetstat} \tab Print overall heterogeneity statistics \cr
#' \code{common} \tab Print results for common effect meta-analysis \cr
#' \code{random} \tab Print results for random effects meta-analysis \cr
#' \code{prediction} \tab Print prediction interval \cr
#' \code{backtransf} \tab Back transform results in printouts and plots \cr
#' \code{method} \tab Meta-analysis method (common effect model) \cr
#' \code{method.random} \tab Meta-analysis method (random effects model) \cr
#' \code{w.common} \tab Weights for common effect model (individual studies) \cr
#' \code{TE.common} \tab Estimated overall effect (common effect model) \cr
#' \code{seTE.common} \tab Standard error of overall effect (common effect model) \cr
#' \code{statistic.common} \tab Statistic for test of overall effect (common effect model) \cr
#' \code{pval.common} \tab P-value for test of overall effect (common effect model) \cr
#' \code{level.ma} \tab Level of confidence interval for meta-analysis
#'   estimates \cr
#' \code{lower.common} \tab Lower confidence limit (common effect model) \cr
#' \code{upper.common} \tab Upper confidence limit (common effect model) \cr
#' \code{w.random} \tab Weight for random effects model (individual studies) \cr
#' \code{TE.random} \tab Estimated overall effect (random effects model) \cr
#' \code{seTE.random} \tab Standard error of overall effect (random effects model) \cr
#' \code{statistic.random} \tab Statistic for test of overall effect (random effects model) \cr
#' \code{pval.random} \tab P-value for test of overall effect (random effects model) \cr
#' \code{method.random.ci} \tab Confidence interval method (random effects model) \cr
#' \code{df.random} \tab Degrees of freedom (random effects model) \cr
#' \code{lower.random} \tab Lower confidence limit (random effects model) \cr
#' \code{upper.random} \tab Upper confidence limit (random effects model) \cr
#' \code{seTE.classic} \tab Standard error (classic random effects method) \cr
#' \code{adhoc.hakn.ci} \tab \emph{Ad hoc} correction for
#'   Hartung-Knapp method (confidence interval) \cr
#' \code{df.hakn.ci} \tab Degrees of freedom for Hartung-Knapp method
#'   \cr \tab (if used in meta-analysis) \cr
#' \code{seTE.hakn.ci} \tab Standard error for Hartung-Knapp method
#'   \cr
#' \tab (not taking \emph{ad hoc} variance correction into account)
#'   \cr
#' \code{seTE.hakn.adhoc.ci} \tab Standard error for Hartung-Knapp
#'   method \cr
#' \tab (taking \emph{ad hoc} variance correction into account) \cr
#' \code{df.kero} \tab Degrees of freedom for Kenward-Roger method \cr
#' \tab  (if used in meta-analysis) \cr 
#' \code{seTE.kero} \tab Standard error for Kenward-Roger
#'   method \cr
#' \code{method.predict} \tab Method to calculate prediction interval \cr
#' \code{adhoc.hakn.pi} \tab \emph{Ad hoc} correction for
#'   Hartung-Knapp method (prediction interval) \cr
#' \code{df.hakn.ci} \tab Degrees of freedom for Hartung-Knapp method
#'   \cr \tab (prediction interval) \cr
#' \code{seTE.predict} \tab Standard error used to calculate
#'   prediction interval \cr
#' \code{df.predict} \tab Degrees of freedom for prediction interval \cr
#' \code{level.predict} \tab Level of prediction interval \cr
#' \code{lower.predict} \tab Lower limit of prediction interval \cr
#' \code{upper.predict} \tab Upper limit of prediction interval \cr
#' \code{seTE.hakn.pi} \tab Standard error for Hartung-Knapp method
#'   \cr
#' \tab (not taking \emph{ad hoc} variance correction into account)
#'   \cr
#' \code{seTE.hakn.adhoc.pi} \tab Standard error for Hartung-Knapp
#'   method \cr
#' \tab (taking \emph{ad hoc} variance correction into account) \cr
#' \code{Q} \tab Heterogeneity statistic \cr
#' \code{df.Q} \tab Degrees of freedom for heterogeneity statistic
#'   \code{Q} \cr
#' \code{pval.Q} \tab P-value of heterogeneity test \cr
#' \code{method.tau} \tab Method to estimate between-study variance \eqn{\tau^2} \cr
#' \code{control} \tab Additional arguments for iterative estimation
#'   of \eqn{\tau^2} \cr
#' \code{method.tau.ci} \tab Method for confidence interval of \eqn{\tau^2} \cr
#' \code{method.I2} \tab Method to estimate heterogeneity statistics I\eqn{^2}
#'   and H \cr
#' \code{level.hetstat} \tab Level of confidence intervals for heterogeneity
#'   statistics \cr
#' \code{tau2} \tab Between-study variance \eqn{\tau^2} \cr
#' \code{se.tau2} \tab Standard error of \eqn{\tau^2} \cr
#' \code{lower.tau2} \tab Lower confidence limit for \eqn{\tau^2} [1] \cr
#' \code{upper.tau2} \tab Upper confidence limit for \eqn{\tau^2} [1] \cr
#' \code{tau} \tab Square-root of between-study variance \eqn{\tau} \cr
#' \code{lower.tau} \tab Lower confidence limit for \eqn{\tau} [1] \cr
#' \code{upper.tau} \tab Upper confidence limit for \eqn{\tau} [1] \cr
#' \code{tau.preset} \tab Prespecified value for \eqn{\tau} \cr
#' \code{TE.tau} \tab Effect estimate used to estimate \eqn{\tau^2} \cr
#' \code{detail.tau} \tab Detail on between-study variance estimate \cr
#' \code{phi} \tab Multiplicative heterogeneity parameter \eqn{\phi} in
#'   penalised logistic regression \cr
#' \code{H} \tab Heterogeneity statistic H \cr
#' \code{lower.H} \tab Lower confidence limit for H [2] \cr
#' \code{upper.H} \tab Upper confidence limit for H [2] \cr
#' \code{I2} \tab Heterogeneity statistic I\eqn{^2} \cr
#' \code{lower.I2} \tab Lower confidence limit for I\eqn{^2} [2] \cr
#' \code{upper.I2} \tab Upper confidence limit for I\eqn{^2} [2] \cr
#' \code{Rb} \tab Heterogeneity statistic R\eqn{_b} \cr
#' \code{lower.Rb} \tab Lower confidence limit for R\eqn{_b} \cr
#' \code{upper.Rb} \tab Upper confidence limit for R\eqn{_b} \cr
#' \code{method.bias} \tab Method to test for funnel plot asymmetry \cr
#' \code{text.common} \tab Label for common effect model \cr
#' \code{text.random} \tab Label for random effects model \cr
#' \code{text.predict} \tab Label for prediction interval \cr
#' \code{text.w.common} \tab Label for weights (common effect model) \cr
#' \code{text.w.random} \tab Label for weights (random effects model) \cr
#' \code{title} \tab Title of meta-analysis / systematic review \cr
#' \code{complab} \tab Comparison label \cr
#' \code{outclab} \tab Outcome label \cr
#' \code{label.e} \tab Label for experimental group \cr
#' \code{label.c} \tab Label for control group \cr
#' \code{label.left} \tab Graph label on left side of forest plot \cr
#' \code{label.right} \tab Graph label on right side of forest plot \cr
#' \code{keepdata} \tab Keep original data \cr
#' \code{data} \tab Original data (set) used in function call (if
#'   \code{keepdata = TRUE}) \cr
#' \code{subset} \tab Information on subset of original data used in
#'   meta-analysis \cr
#' \tab (if \code{keepdata = TRUE}) \cr
#' \code{exclude} \tab Studies excluded from meta-analysis \cr
#' \code{warn} \tab Print warnings \cr
#' \code{call} \tab Function call \cr
#' \code{version} \tab Version of R package \bold{meta} used to create
#'   object
#' }
#' with
#' [1] not available for generalized linear mixed model;
#' [2] not available for generalized linear mixed model or multi-level model
#'   if calculation of I\eqn{^2} is based on \eqn{\tau^2}
#'    (argument \code{method.I2 = "tau2"}).
#' 
#' For subgroup analysis (argument \code{subgroup}), the following
#' additional components are added to the list.
#' \tabular{ll}{
#' \code{subgroup} \tab Subgroup information (for individual studies) \cr
#' \code{subgroup.name} \tab Name of subgroup variable \cr
#' \code{print.subgroup.name} \tab Print name of subgroup variable \cr
#' \code{sep.subgroup} \tab Separator between name of subgroup variable and value \cr
#' \code{test.subgroup} \tab Print test for subgroup differences \cr
#' \code{prediction.subgroup} \tab Print prediction interval for subgroup(s) \cr
#' \code{tau.common} \tab Assumption of common between-study variance in subgroups \cr
#' \code{subgroup.levels} \tab Levels of grouping variable \cr
#' \code{k.w} \tab Number of estimates combined in subgroups \cr
#' \code{k.study.w} \tab Number of studies combined in subgroups \cr
#' \code{k.all.w} \tab Number of studies in subgroups \cr
#' \code{k.TE.w} \tab Number of studies with estimable effects in subgroups \cr
#' \code{TE.common.w} \tab Estimated effect in subgroups (common
#'   effect model) \cr
#' \code{seTE.common.w} \tab Standard error in subgroups (common effect model) \cr
#' \code{statistic.common.w} \tab Statistic for test of effect in subgroups (common effect model) \cr
#' \code{pval.common.w} \tab P-value for test of effect in subgroups (common effect model) \cr
#' \code{lower.common.w} \tab Lower confidence limit in subgroups (common effect model) \cr
#' \code{upper.common.w} \tab Upper confidence limit in subgroups (common effect model) \cr
#' \code{w.common.w} \tab Total weight in subgroups (common effect model) \cr
#' \code{TE.random.w} \tab Estimated effect in subgroups (random
#'   effect model) \cr
#' \code{seTE.random.w} \tab Standard error in subgroups (random effects model) \cr
#' \code{statistic.random.w} \tab Statistic for test of effect in subgroups (random effects model) \cr
#' \code{pval.random.w} \tab P-value for test of effect in subgroups (random effects model) \cr
#' \code{df.random.w} \tab Degrees of freedom in subgroups (random effects model) \cr
#' \code{lower.random.w} \tab Lower confidence limit in subgroups (random effects model) \cr
#' \code{upper.random.w} \tab Upper confidence limit in subgroups (random effects model) \cr
#' \code{w.random.w} \tab Total weight in subgroups (random effects model) \cr
#' \code{seTE.classic.w} \tab Standard error (classic random effects method) \cr
#' \code{df.hakn.ci.w} \tab Degrees of freedom for Hartung-Knapp
#'   method in subgroups \cr
#' \code{seTE.hakn.ci.w} \tab Standard error for Hartung-Knapp method in subgroups
#'   \cr
#' \tab (not taking \emph{ad hoc} variance correction into account)
#'   \cr
#' \code{seTE.hakn.adhoc.ci.w} \tab Standard error for Hartung-Knapp
#'   method in subgroups \cr
#' \code{df.kero.w} \tab Degrees of freedom for Kenward-Roger method in subgroups \cr 
#' \code{seTE.kero.w} \tab Standard error for Kenward-Roger
#'   method in subgroups \cr
#' \code{seTE.predict.w} \tab Standard error for prediction interval in subgroups \cr
#' \code{df.predict.w} \tab Degrees of freedom for prediction interval in subgroups \cr
#' \code{lower.predict.w} \tab Lower limit of prediction interval in subgroups \cr
#' \code{upper.predict.w} \tab Upper limit of prediction interval in subgroups \cr
#' \code{seTE.hakn.pi.w} \tab Standard error for Hartung-Knapp method in
#'   subgroups (prediction intervals) \cr
#' \tab (not taking \emph{ad hoc} variance correction into account) \cr
#' \code{seTE.hakn.adhoc.pi.w} \tab Standard error for Hartung-Knapp
#'   method in subgroups (prediction intervals) \cr
#' \code{Q.w} \tab Heterogeneity statistic Q in subgroups \cr
#' \code{pval.Q.w} \tab P-value for test of heterogeneity in subgroups \cr
#' \code{tau2.w} \tab Between-study variance \eqn{\tau^2} in subgroups \cr
#' \code{lower.tau2.w} \tab Lower confidence limit for \eqn{\tau^2} in subgroups \cr
#' \code{upper.tau2.w} \tab Upper confidence limit for \eqn{\tau^2} in subgroups \cr
#' \code{tau.w} \tab Square-root of between-study variance \eqn{\tau} in subgroups \cr
#' \code{lower.tau.w} \tab Lower confidence limit for \eqn{\tau} in subgroups \cr
#' \code{upper.tau.w} \tab Upper confidence limit for \eqn{\tau} in subgroups \cr
#' \code{H.w} \tab Heterogeneity statistic H in subgroups \cr
#' \code{lower.H.w} \tab Lower confidence limit for H in subgroups \cr
#' \code{upper.H.w} \tab Upper confidence limit for H in subgroups \cr
#' \code{I2.w} \tab Heterogeneity statistic I\eqn{^2} in subgroups \cr
#' \code{lower.I2.w} \tab Lower confidence limit for I\eqn{^2} in subgroups \cr
#' \code{upper.I2.w} \tab Upper confidence limit for I\eqn{^2} in subgroups \cr
#' \code{Rb.w} \tab Heterogeneity statistic R\eqn{_b} in subgroups \cr
#' \code{lower.Rb.w} \tab Lower confidence limit for R\eqn{_b} in subgroups \cr
#' \code{upper.Rb.w} \tab Upper confidence limit for R\eqn{_b} in subgroups \cr
#' \code{Q.w.common} \tab Within-group heterogeneity statistic Q
#'   (common effect model) \cr
#' \code{Q.w.random} \tab Within-group heterogeneity statistic
#'   Q (random effects model) \cr
#' \tab (only calculated if argument \code{tau.common = TRUE}) \cr
#' \code{df.Q.w} \tab Degrees of freedom for \code{Q.w.common} and \code{Q.w.random} \cr
#' \code{pval.Q.w.common} \tab P-value of test for residual
#'   heterogeneity (common effect model) \cr
#' \code{pval.Q.w.random} \tab P-value of test for residual
#'   heterogeneity (random effects model) \cr
#' \code{Q.b.common} \tab Between-groups heterogeneity statistic
#'   Q (common effect model) \cr
#' \code{df.Q.b.common} \tab Degrees of freedom for \code{Q.b.common} \cr
#' \code{pval.Q.b.common} \tab P-value of test for subgroup
#'   differences (common effect model) \cr
#' \code{Q.b.random} \tab Between-groups heterogeneity statistic
#'   Q (random effects model) \cr
#' \code{df.Q.b.random} \tab Degrees of freedom for \code{Q.b.random} \cr
#' \code{pval.Q.b.random} \tab P-value of test for subgroup
#'   differences (random effects model)
#' }
#' 
#' For subgroup analysis assuming a common between-study variance in subgroups
#' (argument \code{tau.common = TRUE}), the following additional components are
#' added to the list.
#' \tabular{ll}{
#' \code{tau2.resid} \tab Residual between-study variance after accounting for
#'   \cr
#' \code{} \tab subgroup differences (equal to entries in list element
#'   \code{tau2.w}) \cr
#' \code{lower.tau2.resid} \tab Lower confidence limit for
#'   residual \eqn{\tau^2} [1] \cr
#' \code{upper.tau2.resid} \tab Upper confidence limit for
#'   residual \eqn{\tau^2} [1] \cr
#' \code{tau.resid} \tab Square-root of residual between-study variance \cr
#' \code{lower.tau.resid} \tab Lower confidence limit for
#'   residual \eqn{\tau} [1] \cr
#' \code{upper.tau.resid} \tab Upper confidence limit for
#'   residual \eqn{\tau} [1] \cr
#' \code{H.resid} \tab Heterogeneity statistic H after accounting for
#'   subgroup differences \cr
#' \code{lower.H.resid} \tab Lower confidence limit for residual H [2] \cr
#' \code{upper.H.resid} \tab Upper confidence limit for residual H [2] \cr
#' \code{I2.resid} \tab Heterogeneity statistic I\eqn{^2} after accounting for
#'   subgroup differences \cr
#' \code{lower.I2.resid} \tab Lower confidence limit for
#'   residual I\eqn{^2} [2] \cr
#' \code{upper.I2.resid} \tab Upper confidence limit for
#'   residual I\eqn{^2} [2] \cr
#' \code{Q.resid} \tab Heterogeneity statistic for residual heterogeneity \cr
#' \code{df.Q.resid} \tab Degrees of freedom for heterogeneity statistic
#'   \code{Q.resid} \cr
#' \code{pval.Q.resid} \tab P-value of test for residual heterogeneity \cr
#' }
#' with
#' [1] not available for generalized linear mixed model;
#' [2] not available for generalized linear mixed model or multi-level model
#'   if calculation of I\eqn{^2} is based on \eqn{\tau^2}
#'    (argument \code{method.I2 = "tau2"}).
#' 
#' An object created with \code{\link{metabin}} has the additional
#' class \code{"metabin"} and the following components.
#' 
#' \tabular{ll}{
#' \code{event.e} \tab Events in experimental group (individual
#'   studies) \cr
#' \code{n.e} \tab Sample size in experimental group (individual
#'   studies) \cr
#' \code{event.e} \tab Events in control group (individual studies)
#'   \cr
#' \code{n.e} \tab Sample size in control group (individual studies)
#'   \cr
#' \code{incr} \tab Increment added to zero cells \cr
#' \code{method.incr} \tab Continuity correction method \cr
#' \code{sparse} \tab Continuity correction applied \cr
#' \code{allstudies} \tab Include studies with double zeros \cr
#' \code{doublezeros} \tab Indicator for studies with double zeros \cr
#' \code{MH.exact} \tab Exact Mantel-Haenszel method \cr
#' \code{RR.Cochrane} \tab Cochrane method to calculate risk ratio \cr
#' \code{Q.Cochrane} \tab Cochrane method to calculate \eqn{\tau^2}
#'   \cr
#' \code{Q.CMH} \tab Cochran-Mantel-Haenszel statistic \cr
#' \code{df.Q.CMH} \tab Degrees of freedom for \code{Q.CMH} \cr
#' \code{pval.Q.CMH} \tab P-value of Cochran-Mantel-Haenszel test \cr
#' \code{print.CMH} \tab Print results for Cochran-Mantel-Haenszel
#'   statistic \cr
#' \code{incr.e} \tab Continuity correction in experimental group
#'   (individual studies) \cr
#' \code{incr.c} \tab Continuity correction in control group
#'   (individual studies) \cr
#' \code{k.MH} \tab Number of studies (Mantel-Haenszel method)
#' }
#' 
#' An object created with \code{\link{metacont}} has the additional
#' class \code{"metacont"} and the following components.
#' 
#' \tabular{ll}{
#' \code{n.e} \tab Sample size in experimental group (individual
#'   studies) \cr
#' \code{mean.e} \tab Estimated mean in experimental group (individual
#'   studies) \cr
#' \code{sd.e} \tab Standard deviation in experimental group
#'   (individual studies) \cr
#' \code{n.c} \tab Sample size in control group (individual studies)
#'   \cr
#' \code{mean.c} \tab Estimated mean in control group (individual
#'   studies) \cr
#' \code{sd.c} \tab Standard deviation in control group (individual
#'   studies) \cr
#' \code{pooledvar} \tab Use pooled variance for mean difference \cr
#' \code{method.smd} \tab Method for standardised mean difference
#'   (SMD) \cr
#' \code{sd.glass} \tab Denominator in Glass' method \cr
#' \code{exact.smd} \tab Use exact formulae for SMD \cr
#' \code{method.ci} \tab Method to calculate confidence limits \cr
#' \code{method.mean} \tab Method to approximate mean \cr
#' \code{method.sd} \tab Method to approximate standard deviation
#' }
#' 
#' An object created with \code{\link{metacor}} has the additional
#' class \code{"metacor"} and the following components.
#' 
#' \tabular{ll}{
#' \code{cor} \tab Correlation (individual studies) \cr
#' \code{n} \tab Sample size (individual studies)
#' }
#' 
#' An object created with \code{\link{metainc}} has the additional
#' class \code{"metainc"} and the following components.
#' 
#' \tabular{ll}{
#' \code{event.e} \tab Events in experimental group (individual
#'   studies) \cr
#' \code{time.e} \tab Person time in experimental group (individual
#'   studies) \cr
#' \code{n.e} \tab Sample size in experimental group (individual
#'   studies) \cr 
#' \code{event.c} \tab Events in control group (individual studies)
#'   \cr
#' \code{time.c} \tab Person time in control group (individual studies)
#'   \cr
#' \code{n.c} \tab Sample size in control group (individual studies)
#'   \cr
#' \code{incr} \tab Increment added to zero cells \cr
#' \code{method.incr} \tab Continuity correction method \cr
#' \code{sparse} \tab Continuity correction applied \cr
#' \code{incr.e} \tab Continuity correction in experimental group
#'   (individual studies) \cr
#' \code{incr.c} \tab Continuity correction in control group
#'   (individual studies) \cr
#' \code{k.MH} \tab Number of studies (Mantel-Haenszel method)
#' }
#' 
#' An object created with \code{\link{metamean}} has the additional
#' class \code{"metamean"} and the following components.
#' 
#' \tabular{ll}{
#' \code{n} \tab Sample size (individual studies) \cr
#' \code{mean} \tab Estimated mean (individual studies) \cr
#' \code{sd} \tab Standard deviation (individual studies) \cr
#' \code{method.ci} \tab Method to calculate confidence limits \cr
#' \code{method.mean} \tab Method to approximate mean \cr
#' \code{method.sd} \tab Method to approximate standard deviation \cr
#' }
#' 
#' An object created with \code{\link{metaprop}} has the additional
#' class \code{"metaprop"} and the following components.
#' 
#' \tabular{ll}{
#' \code{event} \tab Events (individual studies) \cr
#' \code{n} \tab Sample size (individual studies) \cr
#' \code{incr} \tab Increment added to zero cells \cr
#' \code{method.incr} \tab Continuity correction method \cr
#' \code{sparse} \tab Continuity correction applied \cr
#' \code{method.ci} \tab Method to calculate confidence limits \cr
#' \code{incr.event} \tab Continuity correction (individual studies)
#' }
#' 
#' An object created with \code{\link{metarate}} has the additional
#' class \code{"metarate"} and the following components.
#' 
#' \tabular{ll}{
#' \code{event} \tab Events (individual studies) \cr
#' \code{time} \tab Person time (individual studies) \cr
#' \code{n} \tab Sample size (individual studies) \cr
#' \code{incr} \tab Increment added to zero cells \cr
#' \code{method.incr} \tab Continuity correction method \cr
#' \code{sparse} \tab Continuity correction applied \cr
#' \code{method.ci} \tab Method to calculate confidence limits \cr
#' \code{incr.event} \tab Continuity correction (individual studies)
#' }
#' 
#' An object created with \code{\link{trimfill}} has the additional
#' classes \code{"trimfill"} and \code{"metagen"} and the following
#' components.
#' 
#' \tabular{ll}{
#' \code{k0} \tab Number of added studies \cr
#' \code{left} \tab Studies missing on left side \cr
#' \code{ma.common} \tab Use common effect or random effects model to
#'   estimate \cr
#' \tab number of missing studies \cr
#' \code{type} \tab Method to estimate missing studies \cr
#' \code{n.iter.max} \tab Maximum number of iterations \cr
#' \code{n.iter} \tab Number of iterations \cr
#' \code{trimfill} \tab Filled studies (individual studies) \cr
#' \code{class.x} \tab Primary class of meta-analysis object
#' }
#' 
#' An object created with \code{\link{metamerge}} has the additional
#' class \code{"metamerge"}. Furthermore, the following components
#' have a different meaning:
#' 
#' \tabular{ll}{
#' \code{k} \tab Vector with number of estimates \cr
#' \code{k.study} \tab Vector with number of studies \cr
#' \code{k.all} \tab Vector with total number of studies \cr
#' \code{k.TE} \tab Vector with number of studies with estimable
#'   effects \cr
#' \code{k.MH} \tab Vector with number of studies combined with
#'   Mantel-Haenszel method \cr
#' \code{TE.common} \tab Vector with common effect estimates \cr
#' \code{seTE.common} \tab Vector with standard errors of common effect
#'   estimates \cr
#' \code{lower.common} \tab Vector with lower confidence limits (common
#'   effect model) \cr
#' \code{upper.common} \tab Vector with upper confidence limits (common
#'   effect model) \cr
#' \code{statistic.common} \tab Vector with test statistics for test of
#'   overall effect (common effect model) \cr
#' \code{pval.common} \tab Vector with p-value of test for overall effect
#'   (common effect model) \cr
#' \code{TE.random} \tab Vector with random effects estimates \cr
#' \code{seTE.random} \tab Vector with standard errors of random effects
#'   estimates \cr
#' \code{lower.random} \tab Vector with lower confidence limits (random
#'   effects model) \cr
#' \code{upper.random} \tab Vector with upper confidence limits (random
#'   effects model) \cr
#' \code{statistic.random} \tab Vector with test statistics for test of
#'   overall effect (random effects model) \cr
#' \code{pval.random} \tab Vector with p-value of test for overall effect
#'   (random effects model) \cr
#' \code{w.common} \tab Vector or matrix with common effect weights \cr
#' \code{w.random} \tab Vector or matrix with random effects weights
#' }
#'
#' @name meta-object
#' 
#' @aliases meta-object
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{meta-sm}},
#'   \code{\link{print.meta}}, \code{\link{summary.meta}},
#'   \code{\link{forest.meta}}
#'
#' @keywords list


NULL

#' Test for funnel plot asymmetry
#' 
#' @description
#' Test for funnel plot asymmetry, based on rank correlation or linear
#' regression method.
#' 
#' @aliases metabias metabias.meta metabias.default
#' 
#' @param x An object of class \code{meta} or estimated treatment
#'   effect in individual studies.
#' @param seTE Standard error of estimated treatment effect (mandatory
#'   if \code{x} not of class \code{meta}).
#' @param method.bias A character string indicating which test is to
#'   be used (see Details), can be abbreviated.
#' @param plotit A logical indicating whether a plot should be
#'   produced (see Details).
#' @param correct A logical indicating whether a continuity corrected
#'   statistic is used for rank correlation tests.
#' @param k.min Minimum number of studies to perform test for funnel
#'   plot asymmetry.
#' @param digits Minimal number of significant digits for estimates,
#'   see \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value of test for test of funnel plot asymmetry, see
#'   \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of test for test of funnel plot asymmetry, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   errors, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   residual heterogeneity variance, see \code{print.default}.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param big.mark A character used as thousands separator.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param text.tau2 Text printed to identify residual heterogeneity
#'   variance \eqn{\tau^2}.
#' @param details.methods A logical specifying whether details on
#'   statistical methods should be printed.
#' @param \dots Additional arguments passed on to
#'   \code{\link[metafor]{rma.uni}}.
#' 
#' @details
#' Functions to conduct rank correlation or linear regression tests
#' for funnel plot asymmetry.
#'
#' \subsection{Classic generic tests}{
#' The following tests are generic tests for funnel plot asymmetry
#' which only require estimates of the treatment effect and
#' corresponding standard errors. Accordingly, these are the only
#' tests provided by R function \code{metabias.default}.
#' 
#' If argument \code{method.bias} is \code{"Begg"}, the test statistic
#' is based on the rank correlation between standardised treatment
#' estimates and variance estimates of estimated treatment effects;
#' Kendall's tau is used as correlation measure (Begg & Mazumdar,
#' 1994). The test statistic follows a standard normal
#' distribution. By default (if \code{correct} is FALSE), no
#' continuity correction is utilised (Kendall & Gibbons, 1990).
#' 
#' If argument \code{method.bias} is \code{"Egger"}, the test
#' statistic is based on a weighted linear regression of the treatment
#' effect on its standard error (Egger et al., 1997). The test
#' statistic follows a t distribution with \code{number of studies -
#' 2} degrees of freedom.
#' 
#' If argument \code{method.bias} is \code{"Thompson"}, the test
#' statistic is based on a weighted linear regression of the treatment
#' effect on its standard error using an additive between-study
#' variance component denoted as methods (3a) - (3d) in Thompson &
#' Sharp (1999). The test statistic follows a t distribution with
#' \code{number of studies - 2} degrees of freedom.  }
#' 
#' \subsection{Tests for meta-analysis with binary outcomes}{
#' The following tests for funnel plot asymmetry are only available
#' for meta-analyses comparing two binary outcomes, i.e. meta-analyses
#' generated with the \code{metabin} function. The only exception is
#' the test by Peters et al. (2006) which can also be used in a
#' meta-analysis of single proportions generated with \code{metaprop}.
#' 
#' If argument \code{method.bias} is \code{"Harbord"}, the test
#' statistic is based on a weighted linear regression utilising
#' efficient score and score variance (Harbord et al., 2006,
#' 2009). The test statistic follows a t distribution with
#' \code{number of studies - 2} degrees of freedom.
#' 
#' In order to calculate an arcsine test for funnel plot asymmetry
#' (Rücker et al., 2008), one has to use the \code{metabin} function
#' with argument \code{sm = "ASD"} as input to the \code{metabias}
#' command. The three arcsine tests described in Rücker et al. (2008)
#' can be calculated by setting \code{method.bias} to \code{"Begg"},
#' \code{"Egger"} and \code{"Thompson"}, respectively.
#' 
#' If argument \code{method.bias} is \code{"Macaskill"}, the test
#' statistic is based on a weighted linear regression of the treatment
#' effect on the total sample size with weights reciprocal to the
#' variance of the average event probability (Macaskill et al., 2001,
#' \emph{method FPV}). The test statistic follows a t distribution
#' with \code{number of studies - 2} degrees of freedom.
#' 
#' If argument \code{method.bias} is \code{"Peters"}, the test
#' statistic is based on a weighted linear regression of the treatment
#' effect on the inverse of the total sample size with weights
#' reciprocal to the variance of the average event probability (Peters
#' et al., 2006). The test statistic follows a t distribution with
#' \code{number of studies - 2} degrees of freedom. Note, this test is
#' a variant of Macaskill et al. (2001), \emph{method FPV}, using the
#' inverse sample size as covariate.
#' 
#' If argument \code{method.bias} is \code{"Schwarzer"}, the test
#' statistic is based on the rank correlation between a standardised
#' cell frequency and the inverse of the variance of the cell
#' frequency; Kendall's tau is used as correlation measure (Schwarzer
#' et al., 2007). The test statistic follows a standard normal
#' distribution. By default (if \code{correct} is FALSE), no
#' continuity correction is utilised (Kendall & Gibbons, 1990).
#' 
#' Finally, for meta-analysis of diagnostic test accuracy studies, if
#' argument \code{method.bias} is \code{"Deeks"}, the test statistic
#' is based on a weighted linear regression of the log diagnostic odds
#' ratio on the inverse of the squared effective sample size using the
#' effective sample size as weights (Deeks et al., 2005). The test
#' statistic follows a t distribution with \code{number of studies -
#' 2} degrees of freedom.
#' }
#'
#' \subsection{Test for the standardised mean difference}{
#' If argument \code{method.bias} is \code{"Pustejovsky"}, the test
#' statistic is based on a weighted linear regression of the treatment
#' effect on the square root of the sum of the inverse group sample
#' sizes using the treatment effect variance as weights (Pustejovsky &
#' Rodgers, 2019). The test statistic follows a t distribution with
#' \code{number of studies - 2} degrees of freedom.
#' }
#' 
#' \subsection{Recommendations and default settings}{
#' Following recommendations by Sterne et al. (2011), by default, a
#' test for funnel plot asymmetry is only conducted if the number of
#' studies is ten or larger (argument \code{k.min = 10}). This
#' behaviour can be changed by setting a smaller value for argument
#' \code{k.min}. Note, the minimum number of studies is three.
#' 
#' If argument \code{method.bias} is missing, the Harbord test
#' (\code{method.bias = "Harbord"}) is used in meta-analyses with a binary
#' outcome for the odds ratio and Deeks' test (\code{method.bias = "Deeks"})
#' for the diagnostic odds ratios. In all other settings, the Egger test
#' (\code{method.bias = "Egger"}) is used (Sterne et al., 2011).
#' 
#' No test for funnel plot asymmetry is conducted in meta-analyses
#' with subgroups.
#' }
#'
#' If argument \code{plotit = TRUE}, a scatter plot is shown if
#' argument \code{method.bias} is equal to \code{"Begg"},
#' \code{"Egger"}, \code{"Thompson"}, \code{"Harbord"}, or
#' \code{"Deeks"}.
#' 
#' @return
#' A list with class \code{metabias} containing the following
#' components if a test for funnel plot asymmetry is conducted:
#' \item{statistic}{Test statistic.}
#' \item{df}{The degrees of freedom of the test statistic in
#'   the case that it follows a t distribution.}
#' \item{pval}{The p-value for the test.}
#' \item{estimate}{Estimates used to calculate test statisic.}
#' \item{method}{A character string indicating what type of test was
#'   used.}
#' \item{title}{Title of Cochrane review.}
#' \item{complab}{Comparison label.}
#' \item{outclab}{Outcome label.}
#' \item{var.model}{A character string indicating whether none,
#'   multiplicative, or additive residual heterogeneity variance was
#'   assumed.}
#' \item{method.bias}{As defined above.}
#' \item{x}{Meta-analysis object.}
#' \item{version}{Version of R package \bold{meta} used to create
#'   object.}
#'
#' Or a list with the following elements if test is not conducted due
#' to the number of studies:
#' \item{k}{Number of studies in meta-analysis.}
#' \item{k.min}{Minimum number of studies to perform test for funnel
#'   plot asymmetry.}
#' \item{version}{Version of R package \bold{meta} used to create
#'   object.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{funnel}}, \code{\link{funnel.meta}},
#'   \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}
#' 
#' @references
#' Begg CB & Mazumdar M (1994):
#' Operating characteristics of a rank correlation test for
#' publication bias.
#' \emph{Biometrics},
#' \bold{50}, 1088--101
#'
#' Deeks JJ, Macaskill P, Irwig L (2005):
#' The performance of tests of publication bias and other sample size
#' effects in systematic reviews of diagnostic test accuracy was
#' assessed.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{58}:882--93
#' 
#' Egger M, Smith GD, Schneider M & Minder C (1997):
#' Bias in meta-analysis detected by a simple, graphical test.
#' \emph{British Medical Journal},
#' \bold{315}, 629--34
#' 
#' Harbord RM, Egger M & Sterne J (2006):
#' A modified test for small-study effects in meta-analyses of
#' controlled trials with binary endpoints.
#' \emph{Statistics in Medicine},
#' \bold{25}, 3443--57
#'
#' Harbord RM, Harris RJ, Sterne JAC (2009):
#' Updated tests for small-study effects in meta–analyses.
#' \emph{The Stata Journal},
#' \bold{9}, 197--210
#' 
#' Kendall M & Gibbons JD (1990):
#' \emph{Rank Correlation Methods}.
#' London: Edward Arnold
#'
#' Macaskill P, Walter SD, Irwig L (2001):
#' A comparison of methods to detect publication bias in
#' meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{20}, 641--54
#' 
#' Peters JL, Sutton AJ, Jones DR, Abrams KR & Rushton L (2006):
#' Comparison of two methods to detect publication bias in
#' meta-analysis.
#' \emph{Journal of the American Medical Association},
#' \bold{295}, 676--80
#'
#' Pustejovsky JE, Rodgers MA (2019):
#' Testing for funnel plot asymmetry of standardized mean differences.
#' \emph{Research Synthesis Methods},
#' \bold{10}, 57--71
#' 
#' Rücker G, Schwarzer G, Carpenter JR (2008):
#' Arcsine test for publication bias in meta-analyses with binary
#' outcomes.
#' \emph{Statistics in Medicine},
#' \bold{27}, 746--63
#' 
#' Schwarzer G, Antes G & Schumacher M (2007):
#' A test for publication bias in meta-analysis with sparse binary
#' data.
#' \emph{Statistics in Medicine},
#' \bold{26}, 721--33
#' 
#' Sterne, JAC et al. (2011):
#' Recommendations for examining and interpreting funnel plot
#' asymmetry in meta-analyses of randomised controlled trials.
#' \emph{BMJ (Clinical research ed.)},
#' \bold{343}, 1
#' 
#' Thompson SG & Sharp, SJ (1999):
#' Explaining heterogeneity in meta-analysis: a comparison of methods,
#' \emph{Statistics in Medicine},
#' \bold{18}, 2693--708
#' 
#' @examples
#' data(Olkin1995)
#' ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, subset = 1:10, sm = "RR", method = "I")
#' 
#' metabias(ma1)
#' metabias(ma1, plotit = TRUE)
#' 
#' metabias(ma1, method.bias = "Begg")
#' metabias(ma1, method.bias = "Begg", correct = TRUE)
#' 
#' metabias(ma1, method.bias = "Schwarzer")
#' metabias(ma1, method.bias = "Egger")$pval
#' 
#' # Arcsine test (based on linear regression)
#' #
#' ma1.as <- update(ma1, sm = "ASD")
#' metabias(ma1.as)
#' # Same result (using function metabias.default)
#' metabias(ma1.as$TE, ma1.as$seTE)
#' 
#' # No test for funnel plot asymmetry calculated
#' #
#' ma2 <- update(ma1, subset = 1:5)
#' metabias(ma2)
#' 
#' ma3 <- update(ma1, subset = 1:2)
#' metabias(ma3)
#' 
#' # Test for funnel plot asymmetry calculated (use of argument k.min)
#' #
#' metabias(ma2, k.min = 5)
#' 
#' @rdname metabias
#' @method metabias meta
#' @export

metabias.meta <- function(x, method.bias = x$method.bias,
                          plotit = FALSE, correct = FALSE,
                          k.min = 10, ...) {
  
  #
  #
  # (1) Check for meta object
  #
  #
  chkclass(x, "meta")
  chksuitable(x, "Test for funnel plot asymmetry", "metamerge")
  #
  x <- updateversion(x)
  x.name <- deparse(substitute(x))
  
  
  #
  #
  # (2) Check / set other arguments
  #
  #
  if (length(method.bias) == 0) {
    # Set default for method.bias (see Sterne et al., BMJ 2011;343:d4002):
    # - "Harbord" for OR as effect measure
    # - "Egger" otherwise
    if (inherits(x, "metabin") & x$sm == "OR")
      method.bias <- "Harbord"
    else
      method.bias <- "Egger"
  }
  #
  method.bias <- setmethodbias(method.bias)
  #
  if (method.bias %in% c("Begg", "Schwarzer"))
    lab.method <-
      paste0("Rank correlation test of funnel plot asymmetry",
             if (correct == TRUE) " (with continuity correction)")
  else if (method.bias == "Deeks")
    lab.method <- "Funnel plot test for diagnostic odds ratios"
  else
    lab.method <- "Linear regression test of funnel plot asymmetry"
  #
  chklogical(plotit)
  chklogical(correct)
  chknumeric(k.min, 1, length = 1)
  
  
  #
  #
  # (3) Select studies for inclusion in test of funnel plot asymmetry
  #
  #
  TE <- x$TE
  seTE <- x$seTE
  n.e <- x$n.e
  n.c <- x$n.c
  #
  if (inherits(x, "metabin")) {
    event.e <- x$event.e
    event.c <- x$event.c
    #
    if (x$method == "Peto") {
      warning("Inverse variance method used for pooling ",
              "to perform test of funnel plot asymmetry.",
              call = FALSE)
      m <- metabin(event.e, n.e, event.c, n.c, sm = x$sm, method = "Inverse",
                   incr = x$incr, allincr = x$allincr,
                   allstudies = x$allstudies, MH.exact = x$MH.exact,
                   RR.Cochrane = x$RR.Cochrane)
      TE <- m$TE
      seTE <- m$seTE
    }
  }
  #
  if (inherits(x, "metaprop")) {
    event <- x$event
    n <- x$n
  }
  
  if(length(TE) != length(seTE))
    stop("Length of argument TE and seTE must be equal.",
         call. = FALSE)
  #
  # Exclude studies from meta-analysis
  #
  if (!is.null(x$exclude)) {
    TE <- TE[!x$exclude]
    seTE <- seTE[!x$exclude]
    #
    if (!is.null(n.e))
      n.e <- n.e[!x$exclude]
    if (!is.null(n.c))
      n.c <- n.c[!x$exclude]
    #
    if (inherits(x, "metabin")) {
      event.e <- event.e[!x$exclude]
      event.c <- event.c[!x$exclude]
    }
    #
    if (inherits(x, "metaprop")) {
      n <- n[!x$exclude]
      event <- event[!x$exclude]
    }
  }
  #
  sel <- !is.na(TE) & !is.na(seTE)
  if (length(TE) != sum(sel))
    warning(paste(length(TE) - sum(sel),
                  "observation(s) dropped due to missing values"))
  #
  TE <- TE[sel]
  seTE <- seTE[sel]
  #
  if (inherits(x, c("metabin", "metacont"))) {
    n.e <- n.e[sel]
    n.c <- n.c[sel]
  }
  #
  if (inherits(x, "metabin")) {
    event.e <- event.e[sel]
    event.c <- event.c[sel]
  }
  #
  k <- length(TE)
  
  
  #
  #
  # (4) Conduct test of funnel plot asymmetry
  #
  #
  if (k < k.min | k < 3)
    res <- list(k = k, k.min = k.min)
  else if (length(x$subgroup) != 0)
    res <- list(subgroup = TRUE)
  else {
    #
    # Check whether standard errors differ
    #
    if (method.bias %in% c("Begg", "Egger", "Thompson",
                           "Macaskill", "Pustejovsky"))
      if (length(unique(seTE)) == 1)
        stop("Test for small-study effects not feasible as ",
             "all studies have the same precision.",
             call. = FALSE)
    #
    # (a) Rank correlation tests
    #
    if (method.bias %in% c("Begg", "Schwarzer")) {
      if (method.bias == "Begg") {
        #
        # Begg und Mazumdar (1994), Biometrics, 50, 1088-1101
        #
        m <- metagen(TE, seTE, method.tau = "DL", method.tau.ci = "")
        TE.common <- m$TE.common
        seTE.common <- m$seTE.common
        #
        varTE.s <- seTE^2 - seTE.common^2
        TE.s <- (TE - TE.common) / sqrt(varTE.s)
        #
        ktau <- kentau(TE.s, seTE^2, correct = correct)
      }
      else {
        #
        # Schwarzer, Antes, Schumacher (2006), Stat Med
        #
        if (inherits(x, "metabin")) {
          TE.MH <- metabin(x$event.e, x$n.e, x$event.c, x$n.c,
                           sm = "OR", method = "MH",
                           method.tau = "DL", method.tau.ci = "",
                           warn = FALSE)$TE.common
          #
          n.. <- n.e + n.c
          n11 <- event.e
          n1. <- n.e
          n.1 <- event.e + event.c
          #
          n11.e <- rep(NA, k)
          n11.v <- rep(NA, k)
          #
          for ( i in seq(1:k) ) {
            obj <- hypergeometric(n1.[i], n.1[i], n..[i], exp(TE.MH))
            n11.e[i] <- obj$mean()
            n11.v[i] <- obj$var()
          }
          #
          ktau <- kentau((n11 - n11.e) / sqrt(n11.v),
                         1 / n11.v, correct = correct)
        }
      }
      #
      res <- list(statistic = ktau$ks / ktau$se.ks,
                  pval = ktau$p.value,
                  estimate = c(ktau$ks, ktau$se.ks),
                  p.value = ktau$p.value)
      #
      names(res$statistic) <- "z"
      names(res$estimate) <- c("ks", "se.ks")
    }
    #
    # (b) Linear regression tests
    #
    else {
      if (method.bias == "Egger") {
        #
        # Egger, Smith, Schneider, Minder (1997), BMJ, 315, 629-34
        #
        lreg <- linregcore(TE, seTE, ...)
      }
      else if (method.bias == "Thompson") {
        #
        # Thompson und Sharp (1999), Stat Med, 18, 2693-2708
        #
        lreg <- linregcore(TE, seTE, model = "rma", method.tau = x$method.tau,
                           ...)
      }
      else if (method.bias == "Harbord") {
        if (inherits(x, "metabin")) {
          if (x$sm == "RR") {
            #
            # Harbord et al. (2009), The Stata Journal
            #
            Z <- (event.e * (n.e + n.c) - (event.e + event.c) * n.e) /
              (n.e - event.e + n.c - event.c)
            V <- n.e * n.c * (event.e + event.c) /  
              ((n.e + n.c) * (n.e - event.e + n.c - event.c))
            V <- (n.e + n.c) / 4 *
              (event.e + event.c) / (n.e - event.e + n.c - event.c)
          }
          else {
            if (x$sm != "OR")
              warning("Using odds ratio as effect measure in Harbord test.",
                      call. = FALSE)
            #
            # Harbord et al. (2006), Statistics in Medicine
            #
            Z <- event.e - (event.e + event.c) * n.e / (n.e + n.c)
            V <- n.e * n.c * (event.e + event.c) * 
              (n.e - event.e + n.c - event.c) /
              ((n.e + n.c)^2 * ((n.e + n.c) - 1))
          }
          #
          TE.score <- Z / V
          seTE.score <- 1 / sqrt(V)
          #
          lreg <- linregcore(TE.score, seTE.score, ...)
        }
        else
          stoponly("method.bias", method.bias, "metabin()")
      }
      else if (method.bias == "Macaskill") {
        #
        # Macaskill et al. (2001), Stat Med
        #
        if (inherits(x, "metabin") || inherits(x, "metacont"))
          covar <- n.e + n.c
        else if (length(n.e) > 0 & length(n.c) > 0)
          covar <- n.e + n.c
        else if (inherits(x, "metaprop"))
          covar <- n
        else if (length(n.e) > 0)
          covar <- n.e
        else if (length(n.c) > 0)
          covar <- n.c
        else
          stop("No information on sample size available.")
        #
        lreg <- linregcore(TE, seTE, covar, ...)
      }
      else if (method.bias == "Peters") {
        #
        # Peters et al. (2006), JAMA
        #
        if (inherits(x, "metabin")) {
          seTE.Peters <- sqrt(1 / (event.e + event.c) +
                              1 / (n.e - event.e + n.c - event.c))
          covar <- 1 / (n.e + n.c)
        }
        else if (inherits(x, "metaprop")) {
          seTE.Peters <- sqrt(1 / event + 1 / (n - event))
          covar <- 1 / n
        }
        else
          stoponly("method.bias", method.bias, "metabin() or metaprop()")
        #
        lreg <- linregcore(TE, seTE.Peters, covar, ...)
      }
      else if (method.bias == "Deeks") {
        #
        # Deeks et al. (2005), J Clin Epid
        #
        if (inherits(x, "metabin")) {
          ESS <- 4 * n.e * n.c / (n.e + n.c)
          #
          lreg <- linregcore(TE, sqrt(1 / ESS), 1 / sqrt(ESS), ...)
        }
        else
          stoponly("method.bias", method.bias, "metabin()")
      }
      else if (method.bias == "Pustejovsky") {
        #
        # Pustejovsky & Rodgers (2019)
        #
        if (is.null(n.e))
          stop("Sample size in experimental group (n.e) missing.",
               call. = FALSE)
        if (is.null(n.c))
          stop("Sample size in control group (n.c) missing.",
               call. = FALSE)
        #
        lreg <- linregcore(TE, seTE, sqrt(1 / n.e + 1 / n.c), ...)
      }
      #
      res <- list(statistic = lreg$statistic, df = lreg$df, pval = lreg$pval,
                  estimate = c(lreg$slope, lreg$se.slope),
                  tau = lreg$tau, p.value = lreg$pval,
                  intercept = lreg$intercept,
                  se.intercept = lreg$se.intercept)
      names(res$estimate) <- c("bias", "se.bias")
    }
    
    
    #
    # Optional plot
    #
    if (plotit) {
      #
      if (method.bias == "Egger" | method.bias == "Thompson") {
        radial(TE, seTE, common = FALSE)
        abline(lreg$slope, lreg$intercept)
      }
      else if (method.bias == "Begg") {
        #
        if (plotit) {
          plot(TE.s, seTE^2,
               xlab = "Standardised treatment effect",
               ylab = "Estimated variance of treatment estimate")
        }
      }
      else if (method.bias == "Harbord") {
        #
        radial(TE.score, seTE.score, common = FALSE)
        abline(lreg$slope, lreg$intercept)
      }
    }
    
    
    res$method <- lab.method
    #
    res$var.model <- ifelse(method.bias %in% c("Begg", "Schwarzer"),
                            "",
                     ifelse(method.bias == "Thompson",
                            "additive", "multiplicative"))
    #
    res$method.bias <- method.bias
    
    
    if (inherits(x, "meta")) {
      if (!is.null(x$title))
        res$title <- x$title
      if (!is.null(x$complab))
        res$complab <- x$complab
      if (!is.null(x$outclab))
        res$outclab <- x$outclab
    }
  }
  
  
  res$x <- x
  res$version <- packageDescription("meta")$Version
  
  class(res) <- "metabias"
  
  res
}





#' @rdname metabias
#' @method print metabias
#' @export

print.metabias <- function(x,
                           digits = gs("digits"),
                           digits.stat = gs("digits.stat"),
                           digits.pval = max(gs("digits.pval"), 2),
                           digits.se = gs("digits.se"),
                           digits.tau2 = gs("digits.tau2"),
                           #
                           scientific.pval = gs("scientific.pval"),
                           big.mark = gs("big.mark"),
                           zero.pval = gs("zero.pval"),
                           JAMA.pval = gs("JAMA.pval"),
                           #
                           text.tau2 = gs("text.tau2"),
                           #
                           details.methods = gs("details"),
                           #
                           ...) {
  
  
  #
  #
  # (1) Check for metabias object
  #
  #
  
  chkclass(x, "metabias")
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  #
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chkchar(text.tau2, length = 1)
  chklogical(details.methods)
  
  # Update old metabias objects
  #
  if (inherits(x, "htest")) {
    x$pval <- x$p.value
    x$x <- list(method.tau = "DL")
    x$df <- x$parameters
    x$var.model <- "multiplicative"
    #
    if (grepl("Rank correlation test of funnel plot asymmetry",
              x$method)) {
      if (!grepl("counts", x$method))
        x$method.bias <- "Begg"
      else
        x$method.bias <- "Schwarzer"
      #
      x$method <- "Rank correlation test of funnel plot asymmetry"
      x$var.model <- ""
    }
    else if (grepl("Deeks", x$method)) {
      x$method.bias <- "Deeks"
      x$method <- "Funnel plot test for diagnostic odds ratios"
    }
    else {
      #
      if (grepl("score", x$method))
        x$method.bias <- "Harbord"
      else if (grepl("sample size", x$method))
        x$method.bias <- "Peters"
      else if (grepl("methods of moment", x$method)) {
        x$method.bias <- "Thompson"
      x$var.model <- "additive"
      }
      else
        x$method.bias <- "Egger"
      #
      x$method <- "Linear regression test of funnel plot asymmetry"
    }
  }
  
  # Print information for meta-analysis from Cochrane Review
  #
  crtitle(x)
  
  if (length(x$pval) != 0) {
    format.pvalue <- formatPT(x$pval, digits = digits.pval,
                              scientific = scientific.pval,
                              zero = zero.pval, JAMA = JAMA.pval)
    
    if (!grepl("<", format.pvalue))
      format.pvalue <- paste("=", format.pvalue)
    
    cat(paste0(x$method, "\n\n"))
    #
    cat(paste0("Test result: ",
               names(x$statistic), " = ",
               formatN(x$statistic, digits.stat, "NA",
                       big.mark = big.mark),
               ", ",
               if (!is.null(x$df))
                 paste0("df = ", x$df, ", "),
               "p-value ",
               format.pvalue,
               "\n",
               #
               "Bias estimate: ",
               formatN(x$estimate[1], digits, "NA", big.mark = big.mark),
               " (SE = ",
               formatN(x$estimate[2], digits.se, "NA", big.mark = big.mark),
               ")\n"
    )
    )
    #
    if (details.methods) {
      if (x$var.model != "") {
        cat("\nDetails:\n")
        cat(paste0("- ", x$var.model,
                   " residual heterogeneity variance (",
                   formatPT(x$tau^2,
                            lab = TRUE, labval = text.tau2,
                            digits = digits.tau2,
                            lab.NA = "NA",
                            big.mark = big.mark),
                   ")\n"))
        #
        if (x$var.model == "additive") {
          i.lab.method.tau <-
            charmatch(x$x$method.tau, c(gs("meth4tau"), ""), nomatch = NA)
          #
          lab.method.tau <-
            c("- DerSimonian-Laird estimator",
              "- Paule-Mandel estimator",
              "- restricted maximum-likelihood estimator",
              "- maximum-likelihood estimator",
              "- Hunter-Schmidt estimator",
              "- Sidik-Jonkman estimator",
              "- Hedges estimator",
              "- empirical Bayes estimator",
              ""
            )[i.lab.method.tau]
          #
          if (lab.method.tau != "")
            lab.method.tau <- paste(lab.method.tau, "for", text.tau2)
          #
          cat(paste0(lab.method.tau, "\n"))
        }
        #
        if (x$method.bias == "Egger")
          detail.predictor <- "standard error"
        else if (x$method.bias == "Thompson")
          detail.predictor <- "standard error"
        else if (x$method.bias == "Harbord")
          detail.predictor <- "standard error of score"
        else if (x$method.bias == "Macaskill")
          detail.predictor <- "total sample size"
        else if (x$method.bias == "Peters")
          detail.predictor <- "inverse of total sample size"
        else if (x$method.bias == "Deeks")
          detail.predictor <- "inverse of the squared effective sample size"
        else if (x$method.bias == "Pustejovsky")
          detail.predictor <-
          "square root of the sum of the inverse group sample sizes"
        #
        cat(paste0("- predictor: ", detail.predictor, "\n"))
        #
        if (x$method.bias == "Egger")
          detail.weights <- "inverse variance"
        else if (x$method.bias == "Thompson")
          detail.weights <- "inverse variance"
        else if (x$method.bias == "Harbord")
          detail.weights <- "inverse variance of score"
        else if (x$method.bias == "Macaskill")
          detail.weights <- "inverse variance of average event probability"
        else if (x$method.bias == "Peters")
          detail.weights <- "inverse variance of average event probability"
        else if (x$method.bias == "Deeks")
          detail.weights <- "effective sample size"
        else if (x$method.bias == "Pustejovsky")
          detail.weights <- "inverse variance"
        #
        cat(paste0("- weight:    ", detail.weights, "\n"))
      }
    }
    #
    if (x$method.bias == "Begg")
      detail.ref <- "Begg & Mazumdar (1993), Biometrics"
    else if (x$method.bias == "Egger")
      detail.ref <- "Egger et al. (1997), BMJ"
    else if (x$method.bias == "Thompson")
      detail.ref <- "Thompson & Sharp (1999), Stat Med"
    else if (x$method.bias == "Schwarzer")
      detail.ref <- "Schwarzer et al. (2006), Stat Med"
    else if (x$method.bias == "Harbord")
      detail.ref <- "Harbord et al. (2006), Stat Med"
    else if (x$method.bias == "Macaskill")
      detail.ref <- "Macaskill et al. (2001), Stat Med, method FPV"
    else if (x$method.bias == "Peters")
      detail.ref <- "Peters et al. (2006), JAMA"
    else if (x$method.bias == "Deeks")
      detail.ref <- "Deeks et al. (2005), J Clin Epid"
    else if (x$method.bias == "Pustejovsky")
      detail.ref <- "Pustejovsky & Rodgers (2019), RSM"
    else
      detail.ref <- ""
    #
    if (detail.ref != "")
      cat(
        paste0(
          if (details.methods && x$var.model != "")
            "- reference: "
          else
            "\nReference: ",
          detail.ref, "\n"))
  }
  else {
    #
    # Check whether number of studies is too small:
    #
    if (length(x$k) != 0 & length(x$k.min != 0)) {
      if (x$k <= x$k.min) {
        if (x$k <= 2)
          warning("Number of studies (k=",  x$k,
                  ") too small to test for small study effects.",
                  call. = FALSE)
        else
          warning("Number of studies (k=",  x$k,
                  ") too small to test for small study effects (k.min=",
                  x$k.min, "). Change argument 'k.min' if appropriate.",
                  call. = FALSE)
      }
    }
    #
    # Check whether meta-analysis has subgroups:
    #
    if (length(x$subgroup) != 0)
      warning("No test for small study effects conducted ",
              "for meta-analysis with subgroups.",
              call. = FALSE)
  }
  
  invisible(NULL)
}





#' @rdname metabias
#' @export metabias

metabias <- function(x, ...) 
  UseMethod("metabias")





#' @rdname metabias
#' @method metabias default
#' @export

metabias.default <- function(x, seTE,
                             method.bias = "Egger",
                             plotit = FALSE, correct = FALSE,
                             k.min = 10, ...) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  k.All <- length(x)
  #
  chknumeric(x)
  chknumeric(seTE)
  #
  fun <- "metabias"
  chklength(seTE, k.All, fun)
  #
  method.bias <- setchar(method.bias, c("Begg", "Egger", "Thompson"))
  
  
  #
  #
  # (2) Do meta-analysis
  #
  #
  m <- metagen(x, seTE, method.tau.ci = "")
  
  
  #
  #
  # (3) Conduct test for funnel plot asymmetry
  #
  #
  res <- metabias(m, method.bias = method.bias,
                  plotit = plotit, correct = correct,
                  k.min = k.min, ...)
  
  
  res
}

#' Elevated CO_2 and total biomass of woody plants
#' 
#' @description
#' Meta-analysis on effects of elevated CO_2 on total biomass of woody
#' plants
#' 
#' This dataset has been used as an example in Hedges et al. (1999) to
#' describe methods for the meta-analysis of response ratios. The
#' complete dataset with 102 observations and 26 variables is
#' available online as a supplement. Here only a subset of 10
#' variables is provided and used in the examples.
#' 
#' @name woodyplants
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{obsno}}\tab observation number \cr
#' \bold{\emph{papno}}\tab database paper number \cr
#' \bold{\emph{treat}}\tab treatment code \cr
#' \bold{\emph{level}}\tab treatment level \cr
#' \bold{\emph{n.elev}}\tab number of observations in experimental
#'   group (elevated CO_2-level) \cr
#' \bold{\emph{mean.elev}}\tab estimated mean in experimental group
#'   \cr
#' \bold{\emph{sd.elev}}\tab standard deviation in experimental group
#'   \cr
#' \bold{\emph{n.amb}}\tab number of observations in control group
#'   (ambient CO_2-level) \cr
#' \bold{\emph{mean.amb}}\tab estimated mean in control group \cr
#' \bold{\emph{sd.amb}}\tab standard deviation in control group
#' }
#' 
#' @references
#' Hedges LV, Gurevitch J, Curtis PS (1999):
#' The meta-analysis of response ratios in experimental ecology.
#' \emph{Ecology},
#' \bold{80}, 1150--6
#' 
#' @keywords datasets
#' 
#' @examples
#' data(woodyplants)
#' 
#' # Meta-analysis of response ratios (Hedges et al., 1999)
#' #
#' ma1 <- metacont(n.elev, mean.elev, sd.elev, n.amb, mean.amb, sd.amb,
#'   data = woodyplants, sm = "ROM", studlab = paste(obsno, papno, sep = " / "))
#' print(ma1, prediction = TRUE)
#' 
#' # Meta-analysis for plants grown with low soil fertility treatment
#' #
#' ma2 <- update(ma1, subset = (treat == "fert" & level == "low"))
#' print(ma2, prediction = TRUE)
#' 
#' # Meta-analysis for plants grown under low light conditions
#' #
#' ma3 <- update(ma1, subset = (treat == "light" & level == "low"))
#' print(ma3, prediction = TRUE)

NULL

ciAgrestiCoull <- function(event, n, level) {
  chknumeric(event, 0)
  chknumeric(n, 0, zero = TRUE)
  chklevel(level)
  #
  if (length(event) == 1 & length(n) > 1)
    event <- rep(event, length(n))
  else if (length(event) > 1 & length(n) == 1)
    n <- rep(n, length(event))
  else
    if (length(event) != length(n))
      stop("Arguments 'event' and 'n' must be of same length.", call. = FALSE)
  #
  if (any(event > n, na.rm = TRUE))
    stop("Number of events must be smaller equal sample size.", call. = FALSE)
  #
  z <- qnorm(1 - (1 - level) / 2)
  #
  n <- n + z^2
  prop <- 1 / n * (event + 0.5 * z^2)
  #
  lower <- prop - z * sqrt(1 / n * prop * (1 - prop))
  upper <- prop + z * sqrt(1 / n * prop * (1 - prop))
  #
  lower[lower < 0] <- 0
  upper[upper > 1] <- 1
  #
  list(event = event, n = n,
       prop = prop, lower = lower, upper = upper,
       statistic = NA, p = NA, level = level,
       df = NA, null.effect = NA)
}


ciClopperPearson <- function(event, n, level, null.effect) {
  chknumeric(event, 0)
  chknumeric(n, 0, zero = TRUE)
  chklevel(level)
  chknumeric(null.effect, min = 0, max = 1, length = 1)
  #
  if (length(event) == 1 & length(n) > 1)
    event <- rep(event, length(n))
  else if (length(event) > 1 & length(n) == 1)
    n <- rep(n, length(event))
  else
    if (length(event) != length(n))
      stop("Arguments 'event' and 'n' must be of same length.", call. = FALSE)
  #
  if (any(event > n, na.rm = TRUE))
    stop("Number of events must be smaller equal sample size.", call. = FALSE)
  #
  k <- length(event)
  lower <- upper <- statistic <- pval <- rep(NA, k)
  #
  for (i in seq_len(k)) {
    if (!is.na(event[i] & !is.na(n[i]))) {
      cint <- binom.test(event[i], n[i], conf.level = level,
                         p = if (!is.na(null.effect)) null.effect else 0.5)
      #
      lower[i] <- cint$conf.int[[1]]
      upper[i] <- cint$conf.int[[2]]
      if (!is.na(null.effect))
        pval[i] <- cint$p.value
    }
    else {
      lower[i] <- NA
      upper[i] <- NA
      pval[i] <- NA
    }
  }
  #
  list(event = event, n = n,
       prop = event / n, lower = lower, upper = upper,
       statistic = statistic, p = pval, level = level,
       df = NA, null.effect = null.effect)
}


ciSimpleAsymptotic <- function(event, n, level, correct = FALSE) {
  #
  # Newcombe RG. Two-sided confidence intervals for the single
  # proportion: Comparison of seven methods.
  # Stat Med 1998, Apr 30;17(8): 857-72
  #
  chknumeric(event, 0)
  chknumeric(n, 0, zero = TRUE)
  chklevel(level)
  chklogical(correct)
  #
  if (length(event) == 1 & length(n) > 1)
    event <- rep(event, length(n))
  else if (length(event) > 1 & length(n) == 1)
    n <- rep(n, length(event))
  else
    if (length(event) != length(n))
      stop("Arguments 'event' and 'n' must be of same length.", call. = FALSE)
  #
  if (any(event > n, na.rm = TRUE))
    stop("Number of events must be smaller equal sample size.", call. = FALSE)
  #
  prop <- event / n
  z <- qnorm(1 - (1 - level) / 2)
  #
  if (!correct) {
    lower  <- prop - z * sqrt(prop * (1 - prop) / n)
    upper  <- prop + z * sqrt(prop * (1 - prop) / n)
  }
  else {
    lower  <- prop - (z * sqrt(prop * (1 - prop) / n) + 1 / (2 * n))
    upper  <- prop + (z * sqrt(prop * (1 - prop) / n) + 1 / (2 * n))
  }
  #
  lower[lower < 0] <- 0
  upper[upper > 1] <- 1
  #
  list(event = event, n = n,
       prop = prop, lower = lower, upper = upper,
       statistic = NA, p = NA, level = level,
       df = NA, null.effect = NA)
}


ciWilsonScore <- function(event, n, level, correct = FALSE) {
  #
  # Newcombe RG. Two-sided confidence intervals for the single
  # proportion: Comparison of seven methods.
  # Stat Med 1998, Apr 30;17(8): 857-72
  #
  chknumeric(event, 0)
  chknumeric(n, 0, zero = TRUE)
  chklevel(level)
  #
  if (length(event) == 1 & length(n) > 1)
    event <- rep(event, length(n))
  else if (length(event) > 1 & length(n) == 1)
    n <- rep(n, length(event))
  else
    if (length(event) != length(n))
      stop("Arguments 'event' and 'n' must be of same length.", call. = FALSE)
  #
  if (any(event > n, na.rm = TRUE))
    stop("Number of events must be smaller equal sample size.", call. = FALSE)
  #
  prop <- event / n
  z <- qnorm(1 - (1 - level) / 2)
  #
  if (!correct) {
    lower  <- (2 * n * prop + z^2 -
               z * sqrt(z^2 + 4 * n * prop * (1 - prop))) / (2 * (n + z^2))
    upper  <- (2 * n * prop + z^2 +
               z * sqrt(z^2 + 4 * n * prop * (1 - prop))) / (2 * (n + z^2))
    }
  else {
      lower <- (2 * n * prop + z^2 - 1 -
                z * sqrt(z^2 - 2 - 1/n + 4 * prop * (n * (1 - prop) + 1))) /
        (2 * (n + z^2))
    lower[lower < 0] <- 0
      upper <- (2 * n * prop + z^2 + 1 +
                z * sqrt(z^2 + 2 - 1/n + 4 * prop * (n * (1 - prop) - 1))) /
        (2 * (n + z^2))
    upper[upper > 1] <- 1
  }
  #
  list(event = event, n = n,
       prop = prop, lower = lower, upper = upper,
       statistic = NA, p = NA, level = level,
       df = NA, null.effect = NA)
}


ciPoisson <- function(event, time, level, null.effect) {
  chknumeric(event, 0)
  chknumeric(time, 0, zero = TRUE)
  chklevel(level)
  chknumeric(null.effect, min = 0, max = 1, length = 1)
  #
  if (length(event) == 1 & length(time) > 1)
    event <- rep(event, length(time))
  else if (length(event) > 1 & length(time) == 1)
    time <- rep(time, length(event))
  else
    if (length(event) != length(time))
      stop("Arguments 'event' and 'time' must be of same length.",
           call. = FALSE)
  #
  k <- length(event)
  lower <- upper <- statistic <- pval <- rep(NA, k)
  #
  for (i in seq_len(k)) {
    if (!is.na(event[i] & !is.na(time[i]))) {
      cint <-
        poisson.test(event[i], time[i], conf.level = level,
                     r = if (!is.na(null.effect)) null.effect else 1)
      #
      lower[i] <- cint$conf.int[[1]]
      upper[i] <- cint$conf.int[[2]]
      if (!is.na(null.effect))
        pval[i] <- cint$p.value
    }
    else {
      lower[i] <- NA
      upper[i] <- NA
      pval[i] <- NA
    }
  }
  #
  list(event = event, time = time,
       rate = event / time, lower = lower, upper = upper,
       statistic = statistic, p = pval, level = level,
       df = NA, null.effect = null.effect)
}

#' Meta-regression
#' 
#' @description
#' Meta-regression for objects of class \code{meta}. This is a wrapper
#' function for the R function \code{\link[metafor]{rma.uni}} in the R
#' package \bold{metafor} (Viechtbauer 2010).
#' 
#' @details
#' This R function is a wrapper function for R function
#' \code{\link[metafor]{rma.uni}} in the R package \bold{metafor}
#' (Viechtbauer 2010).
#' 
#' Note, results are not back-transformed in printouts of
#' meta-analyses using summary measures with transformations, e.g.,
#' log risk ratios are printed instead of the risk ratio if argument
#' \code{sm = "RR"} and logit transformed proportions are printed if
#' argument \code{sm = "PLOGIT"}.
#' 
#' Argument '\dots{}' can be used to pass additional arguments to R
#' function \code{\link[metafor]{rma.uni}}. For example, argument
#' \code{control} to provide a list of control values for the
#' iterative estimation algorithm. See help page of R function
#' \code{\link[metafor]{rma.uni}} for more details.
#' 
#' @param x An object of class \code{meta}.
#' @param formula Either a character string or a formula object.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance tau-squared. Either
#'   \code{"FE"}, \code{"DL"}, \code{"REML"}, \code{"ML"},
#'   \code{"HS"}, \code{"SJ"}, \code{"HE"}, or \code{"EB"}, can be
#'   abbreviated.
#' @param hakn A logical indicating whether the method by Hartung and
#'   Knapp should be used to adjust test statistics and confidence
#'   intervals.
#' @param level.ma The level used to calculate confidence intervals
#'   for parameter estimates in the meta-regression model.
#' @param intercept A logical indicating whether an intercept should
#'   be included in the meta-regression model.
#' @param \dots Additional arguments passed to R function
#'   \code{\link[metafor]{rma.uni}}.
#' 
#' @return
#' An object of class \code{c("metareg", "rma.uni", "rma")}. Please
#' look at the help page of R function \code{\link[metafor]{rma.uni}}
#' for more details on the output from this function.
#' 
#' In addition, a list \code{.meta} is added to the output containing
#' the following components:
#' \item{x, formula, method.tau, hakn, level.ma, intercept}{As
#'   defined above.}
#' \item{dots}{Information provided in argument '\dots{}'.}
#' \item{call}{Function call.}
#' \item{version}{Version of R package \bold{meta} used to create
#'   object.}
#' \item{version.metafor}{Version of R package \bold{metafor} used to
#'   create object.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{bubble}}, \code{\link{summary.meta}},
#'   \code{\link{metagen}}
#' 
#' @references
#' Viechtbauer W (2010):
#' Conducting Meta-Analyses in R with the Metafor Package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#' 
#' @keywords models regression
#' 
#' @examples
#' data(Fleiss1993cont)
#' # Add some (fictitious) grouping variables:
#' Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
#' Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")
#' 
#' ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD")
#' \dontrun{
#' # Warning as no information on covariate is available
#' #
#' metareg(ma1)
#' }
#' 
#' # Do meta-regression for covariate region
#' #
#' ma2 <- update(ma1, subgroup = region, tau.common = TRUE, common = FALSE)
#' metareg(ma2)
#' 
#' # Same result for
#' # - tau-squared
#' # - test of heterogeneity
#' # - test for subgroup differences
#' # (as argument 'tau.common' was used to create ma2)
#' #
#' ma2
#' metareg(ma2, intercept = FALSE)
#' metareg(ma1, region)
#' 
#' # Different result for
#' # - tau-squared
#' # - test of heterogeneity
#' # - test for subgroup differences
#' # (as argument 'tau.common' is - by default - FALSE)
#' #
#' ma3 <- update(ma1, subgroup = region)
#' ma3
#' 
#' # Generate bubble plot
#' #
#' bubble(metareg(ma2))
#' 
#' # Do meta-regression with two covariates
#' #
#' metareg(ma3, region + age)
#' 
#' # Do same meta-regressions using formula notation
#' #
#' metareg(ma1, ~ region)
#' metareg(ma3, ~ region + age)
#' 
#' # Print intermediate results for iterative estimation algorithm;
#' # furthermore print results with three digits.
#' #
#' metareg(ma3, region, control = list(verbose = TRUE), digits = 3)
#' 
#' @rdname metareg
#' @method metareg meta
#' @export

metareg.meta <- function(x, formula, method.tau = x$method.tau,
                         hakn = x$method.random.ci == "HK",
                         level.ma = x$level.ma,
                         intercept = TRUE, ...) {

  if ("data" %in% names(list(...))) {
    warning("Please note, argument 'data' has been renamed to 'x' ",
            "in version 3.0-0 of R package meta ",
            "(see help page of R function metareg). ",
            "No meta-regression conducted.")
    return(invisible(NULL))
  }

  if (is.call(x)) {
    warning("Please note, first two arguments of R function metareg ",
            "have been interchanged in version 3.0-0 of R package meta. ",
            "No meta-regression conducted.")
    return(invisible(NULL))
  }


  #
  #
  # (1) Check for meta object
  #
  #
  chkclass(x, "meta")
  chksuitable(x, "Meta-regression", c("metamerge", "netpairwise"),
               check.mlm = FALSE)
  #
  x <- updateversion(x)
  
  
  #
  # Assignments
  #
  TE <- x$TE
  seTE <- x$seTE
  method <- x$method
  #
  model.glmm <- x$model.glmm
  #
  three.level <- !is.null(x$k.study) && x$k != x$k.study
  #
  metabin <- inherits(x, "metabin")
  metainc <- inherits(x, "metainc")
  metaprop <- inherits(x, "metaprop")
  metarate <- inherits(x, "metarate")
  #
  if (metabin) {
    if (x$method == "LRP")
      stop("Meta-regression not implemented for penalised logistic regression.",
           call. = FALSE)
    #
    event.e <- x$event.e
    n.e <- x$n.e
    event.c <- x$event.c
    n.c <- x$n.c
  }
  else if (metainc) {
    event.e <- x$event.e
    time.e <- x$time.e
    event.c <- x$event.c
    time.c <- x$time.c
  }
  else if (metaprop) {
    event <- x$event
    n <- x$n
  }
  else if (metarate) {
    event <- x$event
    time <- x$time
  }
  
  
  if (missing(formula)) {
    if (isCol(x$data, ".subgroup"))
      if (intercept)
        formula <- as.call(~ .subgroup)
      else
        formula <- as.call(~ .subgroup - 1)
    else {
      warning("No meta-regression conducted as argument 'formula' ",
              "is missing and no information is provided on subgroup ",
              "variable, i.e. list element 'subgroup' in meta-analysis object ",
              "'x' (see help page of R function metareg).")
      return(invisible(NULL))
    }
  }
  else {
    as.char <-
      try(is.character(formula) || is.numeric(formula) ||
          is.function(formula), silent = TRUE)
    #
    if (inherits(as.char, "try-error"))
      formula.text <- deparse(substitute(formula))
    else {
      if ((as.char & length(formula) != 1) ||
          is.function(formula))
        formula.text <- paste("~", deparse(substitute(formula)))
      else
        formula.text <- deparse(formula)
    }
    #
    formula.text <- gsub("~", "", formula.text)
    formula.text <- gsub("\\\"", "", formula.text)
    formula.text <- gsub("\\\'", "", formula.text)
    #
    nulldata <- is.null(x$data)
    #
    for (i in as.vector(sapply(strsplit(formula.text, "+", fixed = TRUE),
                               trimws, which = "both"))) {
      if (".GlobalEnv" %in% find(i)) {
        if (nulldata) {
          if (is.null(x$data)) {
            x$data <- data.frame(first = .GlobalEnv[[i]])
            names(x$data) <- i
          }
          else
            x$data[[i]] <- .GlobalEnv[[i]]
        }
        else {
          if (isCol(x$data, i))
            warning("R object '", i, "' found in .GlobalEnv used in ",
                    "meta-regression instead of data from ",
                    "meta-analysis object.")
          x$data[[i]] <- .GlobalEnv[[i]]
        }
      }
    }
    #
    if (!intercept)
      formula.text <- paste0(formula.text, " - 1")
    #
    formula <- as.formula(paste("~", formula.text))
  }
  
  
  if (is.null(method.tau))
    method.tau <- "DL"
  #
  method.tau <- setchar(method.tau, c(gs("meth4tau"), "FE"))
  #
  chklogical(hakn)
  #
  chklevel(level.ma)
  chklogical(intercept)

  if (is.null(x$data)) {
    warning("Necessary data not available. Please, recreate meta-analysis ",
            "object without option 'keepdata = FALSE'.")
    return(invisible(NULL))
  }


  #
  # Use subset of studies in meta-regression
  #
  if (!is.null(x$subset))
    dataset <- x$data[x$subset, ]
  else
    dataset <- x$data


  #
  # Exclude studies from meta-regression
  #
  if (!is.null(x$exclude)) {
    exclude <- dataset$.exclude
    dataset <- dataset[!dataset$.exclude, ]
  }
  else
    exclude <- rep(FALSE, nrow(dataset))


  #
  # Argument test in rma.uni(), rma.glmm() and rma.mv()
  #
  test <- ifelse(!hakn, "z",
                 ifelse(method == "GLMM" | three.level, "t", "knha"))

  #
  # Covariate 'x' makes problems without removing meta-analysis object x
  #
  ..x <- x
  rm(x)
  #
  warn.FE <- paste("Fallback to common effect model (argument",
                   "method.tau = \"FE\") due to small number of studies.")
  #
  if (method != "GLMM") {
    #
    # Three-level model
    #
    if (three.level) {
      #
      dataset$.idx <- seq_len(nrow(dataset))
      #
      res <-
        runNN(rma.mv,
              list(yi = TE[!exclude], V = seTE[!exclude]^2,
                   data = dataset,
                   mods = formula, method = method.tau,
                   random = as.call(~ 1 | .id / .idx),
                   test = test, level = 100 * level.ma,
                   ...))
    }
    else
      res <-
        runNN(rma.uni,
              list(yi = TE[!exclude], sei = seTE[!exclude],
                   data = dataset,
                   mods = formula, method = method.tau,
                   test = test, level = 100 * level.ma,
                   ...))
  }
  else {
    if (metabin) {
      if (sum(!exclude) > 2)
        res <-
          runNN(rma.glmm,
                list(ai = event.e[!exclude], n1i = n.e[!exclude],
                     ci = event.c[!exclude], n2i = n.c[!exclude],
                     data = dataset,
                     mods = formula, method = method.tau,
                     test = test, level = 100 * level.ma,
                     measure = "OR", model = model.glmm,
                     ...))
      else {
        if (method.tau != "FE")
          warning(warn.FE)
        res <-
          runNN(rma.glmm,
                list(ai = event.e[!exclude], n1i = n.e[!exclude],
                     ci = event.c[!exclude], n2i = n.c[!exclude],
                     data = dataset,
                     mods = formula, method = "FE",
                     test = test, level = 100 * level.ma,
                     measure = "OR", model = model.glmm,
                     ...))
      }
    }
    else if (metainc) {
      if (sum(!exclude) > 2)
        res <-
          runNN(rma.glmm,
                list(x1i = event.e[!exclude], t1i = time.e[!exclude],
                     x2i = event.c[!exclude], t2i = time.c[!exclude],
                     data = dataset,
                     mods = formula, method = method.tau,
                     test = test, level = 100 * level.ma,
                     measure = "IRR", model = model.glmm,
                     ...))
      else {
        if (method.tau != "FE")
          warning(warn.FE)
        res <-
          runNN(rma.glmm,
                list(x1i = event.e[!exclude], t1i = time.e[!exclude],
                     x2i = event.c[!exclude], t2i = time.c[!exclude],
                     data = dataset,
                     mods = formula, method = "FE",
                     test = test, level = 100 * level.ma,
                     measure = "IRR", model = model.glmm,
                     ...))
      }
    }
    else if (metaprop) {
      if (sum(!exclude) > 2)
        res <-
          runNN(rma.glmm,
                list(xi = event[!exclude], ni = n[!exclude],
                      data = dataset,
                     mods = formula, method = method.tau,
                     test = test, level = 100 * level.ma,
                     measure = "PLO",
                     ...))
      else {
        if (method.tau != "FE")
          warning(warn.FE)
        res <-
          runNN(rma.glmm,
                 list(xi = event[!exclude], ni = n[!exclude],
                      data = dataset,
                      mods = formula, method = "FE",
                      test = test, level = 100 * level.ma,
                      measure = "PLO",
                      ...))
      }
    }
    else if (metarate) {
      if (sum(!exclude) > 2)
        res <-
          runNN(rma.glmm,
                list(xi = event[!exclude], ti = time[!exclude],
                     data = dataset,
                     mods = formula, method = method.tau,
                     test = test, level = 100 * level.ma,
                     measure = "IRLN",
                     ...))
      else {
        if (method.tau != "FE")
          warning(warn.FE)
        res <-
          runNN(rma.glmm,
                list(xi = event[!exclude], ti = time[!exclude],
                     data = dataset,
                     mods = formula, method = "FE",
                     test = test, level = 100 * level.ma,
                     measure = "IRLN",
                     ...))
      }
    }
  }
  
  
  res$.meta <- list(x = ..x,
                    formula = formula,
                    method.tau = method.tau,
                    hakn = hakn,
                    level.ma = level.ma,
                    intercept = intercept,
                    dots = list(...),
                    call = match.call(),
                    version = packageDescription("meta")$Version,
                    version.metafor = packageDescription("metafor")$Version)
  
  class(res) <- c("metareg", class(res))
  res$call <- NULL
  
  res
}





#' @rdname metareg
#' @export metareg

metareg <- function(x, ...) 
  UseMethod("metareg")





#' @rdname metareg
#' @method metareg default
#' @export

metareg.default <- function(x, ...)
  stop("Meta-regression not available for an object of class '",
       class(x)[1], "'.",
       call. = FALSE)

#' Import RevMan 5 analysis data
#' 
#' @description
#' Reads analysis data from Cochrane intervention review created with
#' RevMan 5 and creates a data frame from it.
#' 
#' @aliases read.rm5 Fleiss1993_CR
#' 
#' @param file The name of a file to read data values from.
#' @param sep The field separator character (only considered for
#'   CSV-files). Values on each line of the file are separated by this
#'   character. The comma is the default field separator character in
#'   RevMan 5.
#' @param quote The set of quoting characters (only considered for
#'   CSV-files). In RevMan 5 a "\"" is the default quoting character.
#' @param title Title of Cochrane review.
#' @param numbers.in.labels A logical indicating whether comparison
#'   number and outcome number should be printed at the beginning of
#'   the comparison (argument \code{complab}) and outcome label
#'   (argument \code{outclab}); this is the default in RevMan 5.
#' @param debug An integer between 0 and 3 indicating whether to print
#'   debug messages (only considered for RM5-files).
#' 
#' @details
#' Review Manager 5 (RevMan 5) was the software used for preparing and
#' maintaining Cochrane reviews. RevMan 5 includes the ability to write
#' systematic reviews of interventions, diagnostic test accuracy reviews,
#' methodology reviews and overviews of reviews.
#' 
#' This function provides the ability to read the analysis data from a
#' Cochrane intervention review created with RevMan 5; a data frame is
#' created from it. Cochrane intervention reviews are based on
#' comparisons of two interventions.
#' 
#' By default in RevMan 5, the name of the exported CSV data file is
#' the title of the Cochrane review. Furthermore, the title is part of
#' the RM5-file. Argument \code{title} can be used to overwrite the
#' title of the Cochrane review.
#'
#' \subsection{Import RM5-file}{
#' 
#' A RM5-file (which is in a specific XML format) can be used directly
#' to import the analysis dataset.
#'
#' If the import fails, use argument \code{debug = 3} for more details.
#' }
#'
#' \subsection{Import CSV-file}{
#' 
#' In the past, the following (rather complicated) procedure based on
#' a CSV-file generated within RevMan 5 was necessary - which is only
#' described here for backward compatibility.
#' 
#' In order to generate a data analysis file in RevMan 5 use the
#' following Menu points: \code{"File"} - \code{"Export"} -
#' \code{"Data and analyses"}. It is mandatory to include the
#' following fields in the exported data file by selecting them with
#' the mouse cursor in the Export Analysis Data Wizard: (i) Comparison
#' Number, (ii) Outcome Number, (iii) Subgroup Number. When these
#' fields are not selected a corresponding error message will be
#' printed in R.  It is recommended to include all fields in the
#' exported data file except for the last field
#' "Risk of bias tables". For example, in order to redo the
#' meta-analysis in R for the RevMan 5 data type
#' \code{"O-E and Variance"} the fields \code{"O-E"} and
#' \code{"Variance"} have to be selected in the Export Analysis Data
#' Wizard. If the last field "Risk of bias tables" is selected the
#' import in R fails with an error message
#' "line X did not have Y elements".
#' }
#' 
#' @return
#' A data frame containing the following components:
#' \item{comp.no}{Comparison number.}
#' \item{outcome.no}{Outcome number.}
#' \item{group.no}{Group number.}
#' \item{studlab}{Study label.}
#' \item{year}{Year of publication.}
#' \item{event.e}{Number of events in experimental group.}
#' \item{n.e}{Number of observations in experimental group.}
#' \item{event.c}{Number of events in control group.}
#' \item{n.c}{Number of observations in control group.}
#' \item{mean.e}{Estimated mean in experimental group.}
#' \item{sd.e}{Standard deviation in experimental group.}
#' \item{mean.c}{Estimated mean in control group.}
#' \item{sd.c}{Standard deviation in control group.}
#' \item{O.E}{Observed minus expected (IPD analysis).}
#' \item{V}{Variance of \code{O.E} (IPD analysis).}
#' \item{TE, seTE}{Estimated treatment effect and standard error of
#'   individual studies.}
#' \item{lower, upper}{Lower and upper limit of 95\% confidence
#'   interval for treatment effect in individual studies.}
#' \item{weight}{Weight of individual studies (according to
#'   meta-analytical method used in respective meta-analysis - see
#'   details).}
#' \item{order}{Ordering of studies.}
#' \item{grplab}{Group label.}
#' \item{type}{Type of outcome. D = dichotomous, C = continuous, P =
#'   IPD.}
#' \item{method}{A character string indicating which method has been
#'   used for pooling of studies. One of \code{"Inverse"},
#'   \code{"MH"}, or \code{"Peto"}.}
#' \item{sm}{A character string indicating which summary measure has
#'   been used for pooling of studies.}
#' \item{model}{A character string indicating which meta-analytical
#'   model has been used (either \code{"Fixed"} or \code{"Random"}).}
#' \item{common}{A logical indicating whether common effect
#'   meta-analysis has been used in respective meta-analysis (see
#'   details).}
#' \item{random}{A logical indicating whether random effects
#'   meta-analysis has been used in respective meta-analysis (see
#'   details).}
#' \item{outclab}{Outcome label.}
#' \item{k}{Total number of studies combined in respective
#'   meta-analysis).}
#' \item{event.e.pooled}{Number of events in experimental group in
#'   respective meta-analysis (see details).}
#' \item{n.e.pooled}{Number of observations in experimental group in
#'   respective meta-analysis (see details).}
#' \item{event.c.pooled}{Number of events in control group in
#'   respective meta-analysis (see details).}
#' \item{n.c.pooled}{Number of observations in control group in
#'   respective meta-analysis (see details).}
#' \item{TE.pooled}{Estimated treatment effect in respective
#'   meta-analysis (see details).}
#' \item{lower, upper}{Lower and upper limit of 95\% confidence
#'   interval for treatment effect in respective meta-analysis (see
#'   details).}
#' \item{weight.pooled}{Total weight in respective meta-analysis (see
#'   details).}
#' \item{Z.pooled}{Z-score for test of overall treatment effect in
#'   respective meta-analysis (see details).}
#' \item{pval.pooled}{P-value for test of overall treatment effect in
#'   respective meta-analysis (see details).}
#' \item{Q}{Heterogeneity statistic Q in respective meta-analysis (see
#'   details).}
#' \item{pval.Q}{P-value of heterogeneity statistic Q in respective
#'   meta-analysis (see details).}
#' \item{I2}{Heterogeneity statistic I\eqn{^2} in respective meta-analysis
#'   (see details).}
#' \item{tau2}{Between-study variance (moment estimator of
#'   DerSimonian-Laird) in respective meta-analysis.}
#' \item{Q.w}{Heterogeneity statistic Q within groups in respective
#'   meta-analysis (see details).}
#' \item{pval.Q.w}{P-value of heterogeneity statistic Q within groups
#'   in respective meta-analysis (see details).}
#' \item{I2.w}{Heterogeneity statistic I\eqn{^2} within groups in respective
#'   meta-analysis (see details).}
#' \item{label.e}{Label for experimental group.}
#' \item{label.c}{Label for control group.}
#' \item{label.left}{Graph label on left side of forest plot.}
#' \item{label.right}{Graph label on right side of forest plot.}
#' \item{complab}{Comparison label.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{summary.rm5}}, \code{\link{metabias.rm5}},
#'   \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}, \code{\link{metacr}},
#'   \code{\link{print.rm5}}
#' 
#' @references
#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
#' The Cochrane Collaboration, 2020
#' 
#' @keywords datagen
#' 
#' @examples
#' # Locate export data file "Fleiss1993_CR.csv"
#' # in sub-directory of package "meta"
#' #
#' filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
#' Fleiss1993_CR <- read.rm5(filename)
#' 
#' # Same result as R command example(Fleiss1993bin):
#' #
#' metacr(Fleiss1993_CR)
#' 
#' # Same result as R command example(Fleiss1993cont):
#' #
#' metacr(Fleiss1993_CR, 1, 2)
#'
#' \dontrun{
#' # Locate file "Fleiss1993.rm5" in sub-directory of R package meta
#' #
#' filename <- system.file("extdata/Fleiss1993.rm5", package = "meta")
#' Fleiss1993_CR <- read.cdir(filename)
#' Fleiss1993_CR
#' 
#' # Same result as R Command example(Fleiss1993bin):
#' #
#' metacr(Fleiss1993_CR)
#' }
#'
#' @importFrom xml2 as_xml_document xml_attr xml_find_all xml_text
#' 
#' @rdname read.rm5
#' @export read.rm5

read.rm5 <- function(file, sep = ",", quote = "\"",
                     title, numbers.in.labels = TRUE,
                     debug = 0) {
  #
  if (missing(file))
    stop("File name must be provided", call. = FALSE)
  #
  chkchar(file, length = 1)
  #
  chkchar(sep, length = 1)
  chkchar(quote, length = 1)
  #
  missing.title <- missing(title)
  if (!missing.title)
    chkchar(title, length = 1)
  #
  chklogical(numbers.in.labels)
  #
  if (!is.logical(debug))
    chknumeric(debug)
  if (!(debug %in% 0:3))
    stop("Argument 'debug' must be 0, 1, 2, or 3.", call. = FALSE)
  
  
  nc <- nchar(file)
  #
  if (tolower(substring(file, nc - 2, nc)) == "csv") {
    if (missing.title)
      res <- read.rm5.csv(file, sep = sep, quote = quote,
                          numbers.in.labels = numbers.in.labels)
    else
      res <- read.rm5.csv(file, sep = sep, quote = quote,
                          title = title, numbers.in.labels = numbers.in.labels)
  }
  else if (tolower(substring(file, nc - 2, nc)) == "rm5") {
    if (missing.title)
      res <- read.rm5.rm5(file,
                          numbers.in.labels = numbers.in.labels,
                          debug = debug)
    else
      res <- read.rm5.rm5(file, title = title,
                          numbers.in.labels = numbers.in.labels,
                          debug = debug)
  }
  else
    stop("Argument 'file' must refer to a RM5- or CSV-file.", call. = FALSE)
  #
  if (!is.null(res))
    attr(res, "filename") <- file
  #
  res
}

#' Import RevMan 4 data files (.mtv)
#' 
#' @description
#' Reads a file created with RevMan 4 and creates a data frame from
#' it.
#' 
#' @param file The name of a file to read data values from.
#' 
#' @details
#' Reads a file created with RevMan 4 (Menu: "File" - "Export" -
#' "Analysis data file...") and creates a data frame from it.
#' 
#' @return
#' A data frame containing the following components:
#' \item{comp.no}{Comparison number.}
#' \item{outcome.no}{Outcome number.}
#' \item{group.no}{Group number.}
#' \item{studlab}{Study label.}
#' \item{year}{Year of publication.}
#' \item{event.e}{Number of events in experimental group.}
#' \item{n.e}{Number of observations in experimental group.}
#' \item{event.c}{Number of events in control group.}
#' \item{n.c}{Number of observations in control group.}
#' \item{mean.e}{Estimated mean in experimental group.}
#' \item{sd.e}{Standard deviation in experimental group.}
#' \item{mean.c}{Estimated mean in control group.}
#' \item{sd.c}{Standard deviation in control group.}
#' \item{O.E}{Observed minus expected (IPD analysis).}
#' \item{V}{Variance of \code{O.E} (IPD analysis).}
#' \item{order}{Ordering of studies.}
#' \item{conceal}{Concealment of treatment allocation.}
#' \item{grplab}{Group label.}
#' \item{type}{Type of outcome. D = dichotomous, C = continuous, P =
#'   IPD.}
#' \item{outclab}{Outcome label.}
#' \item{graph.exp}{Graph label for experimental group.}
#' \item{graph.cont}{Graph label for control group.}
#' \item{label.exp}{Label for experimental group.}
#' \item{label.cont}{Label for control group.}
#' \item{complab}{Comparison label.}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}
#' 
#' @references
#' \emph{Review Manager (RevMan)} [Computer program]. Version 4.2.
#' Copenhagen: The Nordic Cochrane Centre, The Cochrane Collaboration, 2003
#' 
#' @keywords datagen
#' 
#' @examples
#' # Locate MTV-data file "FLEISS1993.MTV" in sub-directory of R package
#' # meta
#' #
#' filename <- system.file("extdata/FLEISS1993.MTV", package = "meta")
#' fleiss1933.cc <- read.mtv(filename)
#' 
#' # Same result as R Command example(Fleiss1993bin):
#' #
#' metabin(event.e, n.e, event.c, n.c,
#'   data = fleiss1933.cc, subset = type == "D",
#'   studlab = paste(studlab, year))
#' 
#' # Same result: example(Fleiss1993cont)
#' #
#' metacont(n.e, mean.e, sd.e, n.c, mean.c, sd.c,
#'   data = fleiss1933.cc, subset = type == "C",
#'   studlab = paste(studlab, year))
#' 
#' @export read.mtv

read.mtv <- function(file) {
  #
  line <- scan(file,
               what = "character",
               blank.lines.skip = FALSE,
               comment.char = "",
               sep = "\n")
  #
  #line <- line[substring(line, 1, 10) != "START DATA"]
  #line <- line[substring(line, 1,  8) != "END DATA"]
  #line <- line[substring(line, 1,  8) != "STUDIES:"]
  #
  sel <- substring(line, 1, 9) == "METAVIEW:"
  title <- rmSpace(substring(line[sel], 10), end = TRUE)
  #line <- line[!sel]
  #
  #
  comp  <- line[substring(line, 1, 1) == 3]
  outc  <- line[substring(line, 1, 1) == 2]
  sgrp  <- line[substring(line, 1, 1) == 1]
  study <- line[substring(line, 1, 1) == 0]
  #
  #
  res <- list(title = title,
              #
              comparison =
                data.frame(comp.no = I(substring(comp, 3, 4)),
                           complab = I(rmSpace(substring(comp, 9), end = TRUE))
                           ),
              #
              outcome =
                data.frame(#id         = substring(outc, 5, 9),
                  comp.no    = I(substring(outc, 5, 6)),
                  outcome.no = I(substring(outc, 8, 9)),
                  type       = I(substring(outc, 3, 3)),
                  totals     = I(substring(outc, 11, 11)),
                  outclab    = I(rmSpace(substring(outc,  26, 132), end = TRUE)),
                  graph.exp  = I(rmSpace(substring(outc, 134, 153), end = TRUE)),
                  graph.cont = I(rmSpace(substring(outc, 155, 174), end = TRUE)),
                  label.exp  = I(rmSpace(substring(outc, 176, 195), end = TRUE)),
                  label.cont = I(rmSpace(substring(outc, 197, 216), end = TRUE))),
              #  details    = I(rmSpace(substring(outc,   5,  24), end = TRUE))),
              #
              group =
                data.frame(#id         = I(substring(sgrp, 3, 10)),
                  comp.no    = I(substring(sgrp, 3,  4)),
                  outcome.no = I(substring(sgrp, 6,  7)),
                  group.no   = I(substring(sgrp, 9, 10)),
                  grplab     = I(rmSpace(substring(sgrp, 15), end = TRUE))),
              #
              study =
                data.frame(#id         = I(substring(study, 3, 10)),
                  comp.no    = I(substring(study, 3,  4)),
                  outcome.no = I(substring(study, 6,  7)),
                  group.no   = I(substring(study, 9, 10)),
                  #
                  studlab    = I(rmSpace(substring(study, 12, 31), end = TRUE)),
                  year       = as.numeric(substring(study, 33, 36)),
                  #
                  event.e    = as.numeric(substring(study, 39, 44)),
                  n.e        = as.numeric(substring(study, 46, 51)),
                  event.c    = as.numeric(substring(study, 73, 78)),
                  n.c        = as.numeric(substring(study, 80, 85)),
                  #
                  mean.e     = as.numeric(substring(study, 53,  61)),
                  sd.e       = as.numeric(substring(study, 63,  71)),
                  mean.c     = as.numeric(substring(study, 87,  95)),
                  sd.c       = as.numeric(substring(study, 97, 105)),
                  #
                  O.E        = as.numeric(substring(study, 107, 115)),
                  V          = as.numeric(substring(study, 117, 125)),
                  #
                  order      = as.numeric(substring(study, 127, 130)),
                  conceal    = I(substring(study, 132, 132)))
              )
  #
  if (dim(res$group)[[1]] > 0)
    res2 <- merge(res$study, res$group, by = c("comp.no", "outcome.no", "group.no"), all.x = TRUE)
  else
    res2 <- res$study
  res2 <- merge(res2, res$outcome, by = c("comp.no", "outcome.no"))
  res2 <- merge(res2, res$comparison, by = c("comp.no"))
  #
  res2$event.e[res2$type == "I" & res2$event.e == 0 & res2$n.e == 1] <- NA
  res2$n.e[res2$type == "I" & res2$n.e == 1] <- NA
  res2$event.c[res2$type == "I" & res2$event.c == 0 & res2$n.c == 1] <- NA
  res2$n.c[res2$type == "I" & res2$n.c == 1] <- NA
  res2$mean.c[res2$type == "I" & res2$mean.c == 0 & res2$sd.c == 0] <- NA
  res2$sd.c[res2$type == "I" & res2$sd.c == 0] <- NA
  res2$O.E[res2$type == "I" & res2$O.E == 0 & res2$V == 0] <- NA
  res2$V[res2$type == "I" & res2$V == 0] <- NA
  #
  res2$mean.e[res2$type == "D" & res2$mean.e == 0 & res2$sd.e == 0] <- NA
  res2$sd.e[res2$type == "D" & res2$sd.e == 0] <- NA
  res2$mean.c[res2$type == "D" & res2$mean.c == 0 & res2$sd.c == 0] <- NA
  res2$sd.c[res2$type == "D" & res2$sd.c == 0] <- NA
  res2$O.E[res2$type == "D" & res2$O.E == 0 & res2$V == 0] <- NA
  res2$V[res2$type == "D" & res2$V == 0] <- NA
  #
  res2$event.e[res2$type == "C" & res2$event.e == 0] <- NA
  res2$event.c[res2$type == "C" & res2$event.c == 0] <- NA
  res2$O.E[res2$type == "C" & res2$O.E == 0 & res2$V == 0] <- NA
  res2$V[res2$type == "C" & res2$V == 0] <- NA
  #
  res2$mean.e[res2$type == "P" & res2$mean.e == 0 & res2$sd.e == 0] <- NA
  res2$sd.e[res2$type == "P" & res2$sd.e == 0] <- NA
  res2$mean.c[res2$type == "P" & res2$mean.c == 0 & res2$sd.c == 0] <- NA
  res2$sd.c[res2$type == "P" & res2$sd.c == 0] <- NA
  #
  attr(res2, "title") <- res$title
  
  attr(res2, "version") <- packageDescription("meta")$Version
  
  res2
}

#' Meta-analysis of outcome data from Cochrane review
#' 
#' @description
#' Wrapper function to perform meta-analysis for a single outcome of a
#' Cochrane Intervention review.
#' 
#' @param x An object of class \code{rm5} or \code{cdir} created by R
#'   function \code{read.rm5} or \code{read.cdir} .
#' @param comp.no Comparison number.
#' @param outcome.no Outcome number.
#' @param method A character string indicating which method is to be
#'   used for pooling of studies. One of \code{"Inverse"},
#'   \code{"MH"}, or \code{"Peto"}, can be abbreviated.
#' @param sm A character string indicating which summary measure
#'   (\code{"RR"}, \code{"OR"}, \code{"RD"}, \code{"ASD"},
#'   \code{"HR"}, \code{"MD"}, or \code{"SMD"}, or \code{"ROM"}) is to
#'   be used for pooling of studies.
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param Q.Cochrane A logical indicating if the Mantel-Haenszel
#'   estimate is used in the calculation of the heterogeneity
#'   statistic Q which is implemented in RevMan 5.
#' @param swap.events A logical indicating whether events and
#'   non-events should be interchanged.
#' @param logscale A logical indicating whether effect estimates are
#'   entered on log-scale (ignored for \code{cdir} objects).
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and plots. If
#'   \code{backtransf=TRUE} (default), results for \code{sm="OR"} are
#'   printed as odds ratios rather than log odds ratios and results
#'   for \code{sm="ZCOR"} are printed as correlations rather than
#'   Fisher's z transformed correlations, for example.
#' @param rob A logical indicating whether risk of bias (RoB)
#'   assessment should be considered in meta-analysis (only for
#'   \code{read.cdir} objects).
#' @param tool Risk of bias (RoB) tool (only for \code{read.cdir}
#'   objects).
#' @param categories Possible RoB categories (only for
#'   \code{read.cdir} objects).
#' @param col Colours for RoB categories (only for \code{read.cdir}
#'   objects).
#' @param symbols Corresponding symbols for RoB categories (only for
#'   \code{read.cdir} objects).
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if \code{incr} is added to studies with zero cell
#'   frequencies).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' Cochrane intervention reviews are based on the comparison of two
#' interventions. Each Cochrane intervention review can have a
#' variable number of comparisons. For each comparison, a variable
#' number of outcomes can be define. For each outcome, a separate
#' meta-analysis is conducted. Review Manager 5 (RevMan 5) was the
#' software used for preparing and maintaining Cochrane Reviews.
#' 
#' This wrapper function can be used to perform meta-analysis for a
#' single outcome of a Cochrane intervention review. Internally, R
#' functions \code{\link{metabin}}, \code{\link{metacont}}, and
#' \code{\link{metagen}} are called - depending on the definition of
#' the outcome in RevMan 5.
#'
#' Information on the risk of bias RoB) assessment can be provided
#' with arguments \code{tool}, \code{categories}, \code{col} and
#' \code{symbols}. This is not useful if an overall RoB assessment has
#' been done. In this case use \code{\link{rob}} to add the full
#' flexible RoB information to a \code{\link{metacr}} object.
#' 
#' Note, it is recommended to choose the RevMan 5 settings before
#' executing \code{metacr}, i.e., \code{settings.meta("revman5")}.
#' 
#' @return
#' An object of class \code{"meta"} and - depending on outcome type
#' utilised in Cochrane intervention review for selected outcome -
#' \code{"metabin"}, \code{"metacont"}, or \code{"metagen"} with
#' corresponding generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{rob}},
#'   \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}, \code{\link{read.cdir}},
#'   \code{\link{read.rm5}}, \code{\link{settings.meta}}
#' 
#' @references
#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
#' The Cochrane Collaboration, 2020
#' 
#' @examples
#' # Locate export data file "Fleiss1993_CR.csv"
#' # in sub-directory of package "meta"
#' #
#' filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
#' #
#' Fleiss1993_CR <- read.rm5(filename)
#' 
#' # Choose RevMan 5 settings and store old settings
#' #
#' oldset <- settings.meta("revman5", quietly = FALSE)
#' 
#' # Same result as R command example(Fleiss1993bin)
#' #
#' metacr(Fleiss1993_CR)
#' 
#' # Same result as R command example(Fleiss1993cont)
#' #
#' metacr(Fleiss1993_CR, 1, 2)
#' forest(metacr(Fleiss1993_CR, 1, 2))
#' 
#' # Change summary measure to RR
#' #
#' ma <- metacr(Fleiss1993_CR)
#' update(ma, sm = "RR")
#' 
#' # Use old settings
#' #
#' settings.meta(oldset)
#' 
#' @export metacr

metacr <- function(x, comp.no = 1, outcome.no = 1,
                   #
                   method, sm, level = gs("level"),
                   #
                   common, random,
                   prediction = gs("prediction") | !missing(method.predict),
                   #
                   method.tau = "DL",
                   method.tau.ci = gs("method.tau.ci"),
                   level.hetstat = gs("level.hetstat"),
                   tau.common = FALSE,
                   #
                   method.I2 = gs("method.I2"),
                   #
                   level.ma = gs("level.ma"),
                   method.common.ci = "classic",
                   method.random.ci = "classic",
                   adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                   #
                   level.predict = gs("level.predict"),
                   method.predict = gs("method.predict"),
                   adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                   seed.predict = NULL,
                   #
                   Q.Cochrane, swap.events, logscale,
                   #
                   backtransf = gs("backtransf"),
                   #
                   test.subgroup,
                   prediction.subgroup = gs("prediction.subgroup"),
                   seed.predict.subgroup = NULL,
                   #
                   rob = NULL,
                   tool = NULL,
                   categories = NULL,
                   col = NULL,
                   symbols = NULL,
                   #
                   text.common = gs("text.common"),
                   text.random = gs("text.random"),
                   text.predict = gs("text.predict"),
                   text.w.common = gs("text.w.common"),
                   text.w.random = gs("text.w.random"),
                   #
                   title, complab, outclab,
                   #
                   label.left, label.right,
                   col.label.left = gs("col.label.left"),
                   col.label.right = gs("col.label.right"),
                   #
                   keepdata = gs("keepdata"),
                   warn = FALSE,
                   warn.deprecated = gs("warn.deprecated"),
                   ...) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  chkclass(x, c("rm5", "cdir"))
  #
  if (inherits(x, "cdir")) {
    x.list <- x
    if (!is.null(x$rob)) {
      robdata <- x$rob
      #
      if (is.null(rob))
        rob <- attr(robdata, "rob")
      chklogical(rob)
      #     
      domains <- attr(robdata, "domains")
      n.domains <- length(domains)
      #
      if (is.null(tool))
        tool <- attr(robdata, "tool")
      if (is.null(categories))
        categories <- attr(robdata, "categories")
      if (is.null(col))
        col <- attr(robdata, "col")
      if (is.null(symbols))
        symbols <- attr(robdata, "symbols")
      #
      robdata <- robdata[, !grepl(".details", names(robdata))]
    }
    else
      rob <- FALSE
    #
    x <- x$data
    #
    if (rob)
      x <- merge(x, robdata, all.x = TRUE)
  }
  else
    rob <- FALSE
  #
  if (is.numeric(comp.no))
    chknumeric(comp.no, length = 1)
  else
    chkchar(comp.no, length = 1)
  #
  if (is.numeric(outcome.no))
    chknumeric(outcome.no, length = 1)
  else
    chkchar(outcome.no, length = 1)
  #
  chklogical(warn.deprecated)
  args <- list(...)
  #
  missing.common <- missing(common)
  missing.fixed <- is.na(argid(names(args), "fixed"))
  if (!missing.fixed) {
    common <-
      deprecated(common, missing.common, args, "fixed",
                 warn.deprecated)
    missing.common <- FALSE
  }
  #
  chklevel(level)
  chklevel(level.ma)
  #
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  method.tau <- setchar(method.tau, gs("meth4tau"))
  if (is.null(method.tau.ci))
    method.tau.ci <- if (method.tau == "DL") "J" else "QP"
  method.tau.ci <- setchar(method.tau.ci, gs("meth4tau.ci"))
  chklogical(tau.common)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  chklogical(prediction)
  chklevel(level.predict)
  chklogical(prediction.subgroup)
  if (!is.null(seed.predict))
    chknumeric(seed.predict)
  if (!is.null(seed.predict.subgroup))
    chknumeric(seed.predict.subgroup)
  #
  missing.Q.Cochrane <- missing(Q.Cochrane)
  if (!missing.Q.Cochrane)
    chklogical(Q.Cochrane)
  #
  if (!missing(swap.events))
    chklogical(swap.events)
  #
  chklogical(backtransf)
  #
  text.common <-
    deprecated(text.common, missing(text.common), args, "text.fixed",
               warn.deprecated)
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  #
  if (!is.null(text.random))
    chkchar(text.random, length = 1)
  if (!is.null(text.predict))
    chkchar(text.predict, length = 1)
  #
  text.w.common <-
    deprecated(text.w.common, missing(text.w.common), args, "text.w.fixed",
               warn.deprecated)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  #
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chklogical(keepdata)
  chklogical(warn)
  #
  method.random.ci <-
    deprecated(method.random.ci, missing(method.random.ci),
               args, "hakn", warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  
  
  #
  #
  # (2) Select data for meta-analysis
  #
  #
  sel <- x$comp.no == comp.no & x$outcome.no == outcome.no
  #
  if (sum(sel) == 0) {
    warning("No data available for comp.no = ", comp.no,
            " and outcome.no = ", outcome.no, ".")
    return(NULL)
  }
  #
  x$sel <- sel
  #
  # Additional checks
  #
  if (missing(title))
    title   <- attributes(x)$title
  #
  if (missing(complab))
    complab <- unique(x$complab[sel])
  #
  if (missing(outclab))
    outclab <- unique(x$outclab[sel])
  #
  label.e <- unique(x$label.e[sel])
  label.c <- unique(x$label.c[sel])
  #
  if (missing(label.left))
    label.left  <- unique(x$label.left[sel])
  else
    chkchar(label.left, length = 1)
  #
  if (missing(label.right))
    label.right  <- unique(x$label.right[sel])
  else
    chkchar(label.right, length = 1)
  #
  overall <- replaceNULL(unique(x$overall[sel]), TRUE)
  if (is.na(overall))
    overall <- TRUE
  type <- unique(x$type[sel])
  if (missing(test.subgroup))
    test.subgroup <- unique(x$test.subgroup[sel])
  #
  if (missing(method))
    method <- unique(x$method[sel])
  else
    method <- setchar(method, c("Inverse", "MH", "Peto"))
  chkchar(method, length = 1)
  #
  if (missing(sm))
    sm <- unique(x$sm[sel])
  chkchar(sm, length = 1)
  if (sm == "PETO_OR")
    sm <- "OR"
  #
  if (missing.common)
    common <- unique(x$common[sel])
  #
  if (missing(random))
    random <- unique(x$random[sel])
  #  
  if (tau.common & method == "Peto") {
    if (warn)
      warning("Argument 'tau.common' not considered for Peto method.")
    tau.common <- FALSE
  }
  #
  if (tau.common & method == "MH") {
    if (warn)
      warning("Argument 'tau.common' not considered for Mantel-Haenszel method.")
    tau.common <- FALSE
  }
  
  
  #
  #
  # (3) Calculate results for individual studies
  #
  #

  if (inherits(x, "rm5")) {
    if (!all(is.na(x$logscale[sel]))) {
      if (!unique(x$logscale[sel])) {
        x$TE[sel] <- log(x$TE[sel])
        logscale <- FALSE
      }
      else
        logscale <- TRUE
    }
    else {
      if (!missing(logscale)) {
        if (!logscale)
          x$TE[sel] <- log(x$TE[sel])
      }
      else {
        if ((type == "I" & method != "Peto") & is_relative_effect(sm)) {
          warning("Assuming that values for 'TE' are on log scale. ",
                  "Please use argument 'logscale = FALSE' if ",
                  "values are on natural scale.",
                  call. = FALSE)
          logscale <- TRUE
        }
        else
          logscale <- NA
      }
    }
  }
  else
    logscale <- NULL
  
  
  #
  #
  # (4) Do meta-analysis
  #
  #
  O.E <- TE <- V <- event.c <- event.e <- grplab <- mean.c <- mean.e <-
    n.c <- n.e <- sd.c <- sd.e <- seTE <- studlab <- NULL
  #
  dropnames <- c("comp.no", "outcome.no", "group.no",
                 "overall", "test.subgroup",
                 "type", "method", "sm", "model", "common", "random",
                 "outclab", "k",
                 "event.e.pooled", "n.e.pooled",
                 "event.c.pooled", "n.c.pooled",
                 "TE.pooled", "lower.pooled", "upper.pooled",
                 "weight.pooled", "Z.pooled", "pval.TE.pooled",
                 "Q", "pval.Q", "I2", "tau2", "Q.w", "pval.Q.w", "I2.w",
                 "swap.events", "enter.n", "logscale", "label.e", "label.c",
                 "label.left", "label.right", "complab", "sel")
  #
  varnames <- names(x)[!(names(x) %in% dropnames)]
  #
  if (missing.Q.Cochrane)
    Q.Cochrane <- if (method == "MH" & method.tau == "DL") TRUE else FALSE
  #
  if (length(unique(x$group.no[sel])) > 1) {
    if (type == "D") {
      #
      if (missing(swap.events)) {
        swap.events <- unique(x$swap.events[sel])
        swap.events <- !is.na(swap.events) && swap.events
      }
      #
      if (swap.events)
        m1 <- metabin(n.e - event.e, n.e, n.c - event.c, n.c,
                      studlab = studlab,
                      data = x[sel, varnames],
                      #
                      method = method, sm = sm, level = level,
                      #
                      common = common, random = random,
                      overall = overall,
                      prediction = prediction,
                      #
                      method.tau = method.tau, method.tau.ci = method.tau.ci,
                      level.hetstat = level.hetstat,
                      tau.common = tau.common,
                      #
                      method.I2 = method.I2,
                      #
                      level.ma = level.ma,
                      method.common.ci = method.common.ci,
                      method.random.ci = method.random.ci,
                      adhoc.hakn.ci = adhoc.hakn.ci,
                      #
                      level.predict = level.predict,
                      method.predict = method.predict,
                      adhoc.hakn.pi = adhoc.hakn.pi,
                      seed.predict = seed.predict,
                      #
                      subgroup = grplab,
                      subgroup.name = "grp",
                      print.subgroup.name = FALSE,
                      test.subgroup = test.subgroup,
                      prediction.subgroup = prediction.subgroup,
                      seed.predict.subgroup = seed.predict.subgroup,
                      #
                      backtransf = backtransf,
                      #
                      text.common = text.common, text.random = text.random,
                      text.predict = text.predict,
                      text.w.common = text.w.common,
                      text.w.random = text.w.random,
                      #
                      title = title,
                      complab = complab, outclab = outclab,
                      #
                      label.e = label.e, label.c = label.c,
                      label.left = label.left, label.right = label.right,
                      col.label.left = col.label.left,
                      col.label.right = col.label.right,
                      #
                      RR.Cochrane = TRUE,
                      Q.Cochrane = Q.Cochrane,
                      #
                      warn = warn, keepdata = keepdata)
      else
        m1 <- metabin(event.e, n.e, event.c, n.c, studlab = studlab,
                      data = x[sel, varnames],
                      #
                      method = method, sm = sm, level = level,
                      #
                      common = common, random = random,
                      overall = overall,
                      prediction = prediction,
                      #
                      method.tau = method.tau, method.tau.ci = method.tau.ci,
                      level.hetstat = level.hetstat,
                      tau.common = tau.common,
                      #
                      method.I2 = method.I2,
                      #
                      level.ma = level.ma,
                      method.common.ci = method.common.ci,
                      method.random.ci = method.random.ci,
                      adhoc.hakn.ci = adhoc.hakn.ci,
                      #
                      level.predict = level.predict,
                      method.predict = method.predict,
                      adhoc.hakn.pi = adhoc.hakn.pi,
                      seed.predict = seed.predict,
                      #
                      subgroup = grplab,
                      subgroup.name = "grp",
                      print.subgroup.name = FALSE,
                      test.subgroup = test.subgroup,
                      prediction.subgroup = prediction.subgroup,
                      seed.predict.subgroup = seed.predict.subgroup,
                      #
                      backtransf = backtransf,
                      #
                      text.common = text.common, text.random = text.random,
                      text.predict = text.predict,
                      text.w.common = text.w.common,
                      text.w.random = text.w.random,
                      #
                      title = title,
                      complab = complab, outclab = outclab,
                      #
                      label.e = label.e, label.c = label.c,
                      label.left = label.left, label.right = label.right,
                      col.label.left = col.label.left,
                      col.label.right = col.label.right,
                      #
                      RR.Cochrane = TRUE,
                      Q.Cochrane = Q.Cochrane,
                      #
                      warn = warn, keepdata = keepdata)
    }
    #
    if (type == "C")
      m1 <- metacont(n.e, mean.e, sd.e, n.c, mean.c, sd.c, studlab = studlab,
                     data = x[sel, varnames],
                     #
                     sm = sm, level = level,
                     #
                     common = common, random = random,
                     overall = overall,
                     prediction = prediction,
                     #
                     method.tau = method.tau, method.tau.ci = method.tau.ci,
                     level.hetstat = level.hetstat,
                     tau.common = tau.common,
                     #
                     method.I2 = method.I2,
                     #
                     level.ma = level.ma,
                     method.common.ci = method.common.ci,
                     method.random.ci = method.random.ci,
                     adhoc.hakn.ci = adhoc.hakn.ci,
                     #
                     level.predict = level.predict,
                     method.predict = method.predict,
                     adhoc.hakn.pi = adhoc.hakn.pi,
                     seed.predict = seed.predict,
                     #
                     subgroup = grplab,
                     subgroup.name = "grp",
                     print.subgroup.name = FALSE,
                     test.subgroup = test.subgroup,
                     prediction.subgroup = prediction.subgroup,
                     seed.predict.subgroup = seed.predict.subgroup,
                     #
                     text.common = text.common, text.random = text.random,
                     text.predict = text.predict,
                     text.w.common = text.w.common,
                     text.w.random = text.w.random,
                     #
                     title = title,
                     complab = complab, outclab = outclab,
                     #
                     label.e = label.e, label.c = label.c,
                     label.left = label.left, label.right = label.right,
                     col.label.left = col.label.left,
                     col.label.right = col.label.right,
                     #
                     keepdata = keepdata)
    #
    if (type == "P")
      m1 <- metagen(O.E / V, sqrt(1 / V), studlab = studlab,
                    data = x[sel, varnames],
                    #
                    sm = sm, level = level,
                    #
                    common = common, random = random,
                    overall = overall,
                    prediction = prediction,
                    #
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    tau.common = tau.common,
                    #
                    method.I2 = method.I2,
                    #
                    level.ma = level.ma,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    #
                    level.predict = level.predict,
                    method.predict = method.predict,
                    adhoc.hakn.pi = adhoc.hakn.pi,
                    seed.predict = seed.predict,
                    #
                    subgroup = grplab,
                    subgroup.name = "grp",
                    print.subgroup.name = FALSE,
                    test.subgroup = test.subgroup,
                    prediction.subgroup = prediction.subgroup,
                    seed.predict.subgroup = seed.predict.subgroup,
                    #
                    backtransf = backtransf,
                    #
                    text.common = text.common, text.random = text.random,
                    text.predict = text.predict,
                    text.w.common = text.w.common,
                    text.w.random = text.w.random,
                    #
                    title = title,
                    complab = complab, outclab = outclab,
                    #
                    label.e = label.e, label.c = label.c,
                    label.left = label.left, label.right = label.right,
                    col.label.left = col.label.left,
                    col.label.right = col.label.right,
                    #
                    keepdata = keepdata)
    #
    if (type == "I" & method != "Peto")
      m1 <- metagen(TE, seTE, studlab = studlab,
                    data = x[sel, varnames],
                    #
                    sm = sm, level = level,
                    #
                    common = common, random = random,
                    overall = overall,
                    prediction = prediction,
                    #
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    tau.common = tau.common,
                    #
                    method.I2 = method.I2,
                    #
                    level.ma = level.ma,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    #
                    level.predict = level.predict,
                    method.predict = method.predict,
                    adhoc.hakn.pi = adhoc.hakn.pi,
                    seed.predict = seed.predict,
                    #
                    subgroup = grplab,
                    subgroup.name = "grp",
                    print.subgroup.name = FALSE,
                    test.subgroup = test.subgroup,
                    prediction.subgroup = prediction.subgroup,
                    seed.predict.subgroup = seed.predict.subgroup,
                    #
                    n.e = n.e,
                    n.c = n.c,
                    #
                    backtransf = backtransf,
                    #
                    text.common = text.common, text.random = text.random,
                    text.predict = text.predict,
                    text.w.common = text.w.common,
                    text.w.random = text.w.random,
                    #
                    title = title,
                    complab = complab, outclab = outclab,
                    #
                    label.e = label.e, label.c = label.c,
                    label.left = label.left, label.right = label.right,
                    col.label.left = col.label.left,
                    col.label.right = col.label.right,
                    #
                    keepdata = keepdata)
    #
    if (type == "I" & method == "Peto")
      m1 <- metagen(O.E / V, sqrt(1 / V), studlab = studlab,
                    data = x[sel, varnames],
                    #
                    sm = sm, level = level,
                    #
                    common = common, random = random,
                    overall = overall,
                    prediction = prediction,
                    #
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    tau.common = tau.common,
                    #
                    method.I2 = method.I2,
                    #
                    level.ma = level.ma,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    #
                    level.predict = level.predict,
                    method.predict = method.predict,
                    adhoc.hakn.pi = adhoc.hakn.pi,
                    seed.predict = seed.predict,
                    #
                    subgroup = grplab,
                    subgroup.name = "grp",
                    print.subgroup.name = FALSE,
                    test.subgroup = test.subgroup,
                    prediction.subgroup = prediction.subgroup,
                    seed.predict.subgroup = seed.predict.subgroup,
                    #
                    backtransf = backtransf,
                    #
                    text.common = text.common, text.random = text.random,
                    text.predict = text.predict,
                    text.w.common = text.w.common,
                    text.w.random = text.w.random,
                    #
                    title = title,
                    complab = complab, outclab = outclab,
                    #
                    label.e = label.e, label.c = label.c,
                    label.left = label.left, label.right = label.right,
                    col.label.left = col.label.left,
                    col.label.right = col.label.right,
                    #
                    keepdata = keepdata)
  }
  else {
    if (type == "D") {
      #
      if (missing(swap.events)) {
        swap.events <- unique(x$swap.events[sel])
        swap.events <- !is.na(swap.events) && swap.events
      }
      #
      if (swap.events)
        m1 <- metabin(n.e - event.e, n.e, n.c - event.c, n.c,
                      studlab = studlab,
                      data = x[sel, varnames],
                      #
                      method = method, sm = sm, level = level,
                      #
                      common = common, random = random,
                      overall = overall,
                      prediction = prediction,
                      #
                      method.tau = method.tau, method.tau.ci = method.tau.ci,
                      level.hetstat = level.hetstat,
                      tau.common = tau.common,
                      #
                      method.I2 = method.I2,
                      #
                      level.ma = level.ma,
                      method.common.ci = method.common.ci,
                      method.random.ci = method.random.ci,
                      adhoc.hakn.ci = adhoc.hakn.ci,
                      #
                      level.predict = level.predict,
                      method.predict = method.predict,
                      adhoc.hakn.pi = adhoc.hakn.pi,
                      seed.predict = seed.predict,
                      #
                      backtransf = backtransf,
                      #
                      text.common = text.common, text.random = text.random,
                      text.predict = text.predict,
                      text.w.common = text.w.common,
                      text.w.random = text.w.random,
                      #
                      title = title,
                      complab = complab, outclab = outclab,
                      #
                      label.e = label.e, label.c = label.c,
                      label.left = label.left, label.right = label.right,
                      col.label.left = col.label.left,
                      col.label.right = col.label.right,
                      #
                      RR.Cochrane = TRUE,
                      Q.Cochrane = Q.Cochrane,
                      #
                      warn = warn, keepdata = keepdata)
      else
        m1 <- metabin(event.e, n.e, event.c, n.c, studlab = studlab,
                      data = x[sel, varnames],
                      #
                      method = method, sm = sm, level = level,
                      #
                      common = common, random = random,
                      overall = overall,
                      prediction = prediction,
                      #
                      method.tau = method.tau, method.tau.ci = method.tau.ci,
                      level.hetstat = level.hetstat,
                      tau.common = tau.common,
                      #
                      method.I2 = method.I2,
                      #
                      level.ma = level.ma,
                      method.common.ci = method.common.ci,
                      method.random.ci = method.random.ci,
                      adhoc.hakn.ci = adhoc.hakn.ci,
                      #
                      level.predict = level.predict,
                      method.predict = method.predict,
                      adhoc.hakn.pi = adhoc.hakn.pi,
                      seed.predict = seed.predict,
                      #
                      backtransf = backtransf,
                      #
                      text.common = text.common, text.random = text.random,
                      text.predict = text.predict,
                      text.w.common = text.w.common,
                      text.w.random = text.w.random,
                      #
                      title = title,
                      complab = complab, outclab = outclab,
                      #
                      label.e = label.e, label.c = label.c,
                      label.left = label.left, label.right = label.right,
                      col.label.left = col.label.left,
                      col.label.right = col.label.right,
                      #
                      RR.Cochrane = TRUE,
                      Q.Cochrane = Q.Cochrane,
                      warn = warn, keepdata = keepdata)
    }
    #
    if (type == "C")
      m1 <- metacont(n.e, mean.e, sd.e, n.c, mean.c, sd.c, studlab = studlab,
                     data = x[sel, varnames],
                     #
                     sm = sm, level = level,
                     #
                     common = common, random = random,
                     overall = overall,
                     prediction = prediction,
                     #
                     method.tau = method.tau, method.tau.ci = method.tau.ci,
                     level.hetstat = level.hetstat,
                     tau.common = tau.common,
                     #
                     method.I2 = method.I2,
                     #
                     level.ma = level.ma,
                     method.common.ci = method.common.ci,
                     method.random.ci = method.random.ci,
                     adhoc.hakn.ci = adhoc.hakn.ci,
                     #
                     level.predict = level.predict,
                     method.predict = method.predict,
                     adhoc.hakn.pi = adhoc.hakn.pi,
                     seed.predict = seed.predict,
                     #
                     text.common = text.common, text.random = text.random,
                     text.predict = text.predict,
                     text.w.common = text.w.common,
                     text.w.random = text.w.random,
                     #
                     title = title,
                     complab = complab, outclab = outclab,
                     #
                     label.e = label.e, label.c = label.c,
                     label.left = label.left, label.right = label.right,
                     col.label.left = col.label.left,
                     col.label.right = col.label.right,
                     #
                     keepdata = keepdata)
    #
    if (type == "P")
      m1 <- metagen(O.E / V, sqrt(1 / V), studlab = studlab,
                    data = x[sel, varnames],
                    #
                    sm = sm, level = level,
                    #
                    common = common, random = random,
                    overall = overall,
                    prediction = prediction,
                    #
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    tau.common = tau.common,
                    #
                    method.I2 = method.I2,
                    #
                    level.ma = level.ma,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    #
                    level.predict = level.predict,
                    method.predict = method.predict,
                    adhoc.hakn.pi = adhoc.hakn.pi,
                    seed.predict = seed.predict,
                    #
                    backtransf = backtransf,
                    #
                    text.common = text.common, text.random = text.random,
                    text.predict = text.predict,
                    text.w.common = text.w.common,
                    text.w.random = text.w.random,
                    #
                    title = title,
                    complab = complab, outclab = outclab,
                    #
                    label.e = label.e, label.c = label.c,
                    label.left = label.left, label.right = label.right,
                    col.label.left = col.label.left,
                    col.label.right = col.label.right,
                    #
                    keepdata = keepdata)
    #
    if (type == "I" & method != "Peto")
      m1 <- metagen(TE, seTE, studlab = studlab,
                    data = x[sel, varnames],
                    #
                    sm = sm, level = level,
                    #
                    common = common, random = random,
                    overall = overall,
                    prediction = prediction,
                    #
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    tau.common = tau.common,
                    #
                    method.I2 = method.I2,
                    #
                    level.ma = level.ma,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    #
                    level.predict = level.predict,
                    method.predict = method.predict,
                    adhoc.hakn.pi = adhoc.hakn.pi,
                    seed.predict = seed.predict,
                    #
                    n.e = n.e,
                    n.c = n.c,
                    #
                    backtransf = backtransf,
                    #
                    text.common = text.common, text.random = text.random,
                    text.predict = text.predict,
                    text.w.common = text.w.common,
                    text.w.random = text.w.random,
                    #
                    title = title,
                    complab = complab, outclab = outclab,
                    #
                    label.e = label.e, label.c = label.c,
                    label.left = label.left, label.right = label.right,
                    col.label.left = col.label.left,
                    col.label.right = col.label.right,
                    #
                    keepdata = keepdata)
    #
    if (type == "I" & method == "Peto")
      m1 <- metagen(O.E / V, sqrt(1 / V), studlab = studlab,
                    data = x[sel, varnames],
                    #
                    sm = sm, level = level,
                    #
                    common = common, random = random,
                    overall = overall,
                    prediction = prediction,
                    #
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    tau.common = tau.common,
                    #
                    method.I2 = method.I2,
                    #
                    level.ma = level.ma,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    #
                    level.predict = level.predict,
                    method.predict = method.predict,
                    adhoc.hakn.pi = adhoc.hakn.pi,
                    seed.predict = seed.predict,
                    #
                    backtransf = backtransf,
                    #
                    text.common = text.common, text.random = text.random,
                    text.predict = text.predict,
                    text.w.common = text.w.common,
                    text.w.random = text.w.random,
                    #
                    title = title,
                    complab = complab, outclab = outclab,
                    #
                    label.e = label.e, label.c = label.c,
                    label.left = label.left, label.right = label.right,
                    col.label.left = col.label.left,
                    col.label.right = col.label.right,
                    #
                    keepdata = keepdata)
  }
  
  if (sm == "OTHER") {
    warning('Meta-analysis not possible for sm = "OTHER".')
    return(NULL)
  }


  #
  #
  # (5) Add risk of bias assessment
  #
  #
  if (rob) {
    rd <- m1$data
    m1 <- rob(
      studlab = rd$studlab,
      item1 = if (n.domains >= 1) rd$D1 else NULL,
      item2 = if (n.domains >= 2) rd$D2 else NULL,
      item3 = if (n.domains >= 3) rd$D3 else NULL,
      item4 = if (n.domains >= 4) rd$D4 else NULL,
      item5 = if (n.domains >= 5) rd$D5 else NULL,
      item6 = if (n.domains >= 6) rd$D6 else NULL,
      item7 = if (n.domains >= 7) rd$D7 else NULL,
      item8 = if (n.domains >= 8) rd$D8 else NULL,
      item9 = if (n.domains >= 9) rd$D9 else NULL,
      item10 = if (n.domains >= 10) rd$D10 else NULL,
      weight = if (m1$random) m1$w.random else m1$w.common,
      #
      tool = tool,
      categories = categories,
      col = col,
      symbols = symbols,
      domains = domains,
      #
      data = m1, warn = FALSE)
  }
  
  
  res <- m1
  #
  attr(res, "comp.no") <- comp.no
  attr(res, "outcome.no") <- outcome.no
  attr(res, "type") <- type
  #
  if (type == "D")
    attr(res, "swap.events") <- swap.events
    attr(res, "logscale") <- logscale
  #
  if (!is.null(x$enter.n))
    attr(res, "enter.n") <- x$enter.n
  #
  res
}

# Auxiliary functions to calculate heterogeneity measures
#
# Package: meta
# Author: Guido Schwarzer <guido.schwarzer@@uniklinik-freiburg.de>
# License: GPL (>= 2)
#
Rb <- function(seTE, seTE.random, tau2, Q, df.Q, level) {
  #
  # Calculate Rb
  # Crippa et al. (2016), Statistics in Medicine, 35 3661-75.
  #
  
  
  if (is.na(df.Q) || df.Q == 0)
    return(list(TE = NaN, lower = NaN, upper = NaN))
  #
  k <- df.Q + 1
  varTE <- seTE^2
  w.common <- 1 / varTE
  
  
  #
  # Equation (4) in Crippa et al. (2016)
  #
  # Rb <- 1 / k * sum(tau2 / (varTE + tau2))
  Rb <- tau2 / (k * seTE.random^2)
  
  
  #
  # Appendix D in Crippa et al. (2016)
  #
  S <- function(n, w) sum(w^n)
  #
  a <- varTE * (S(1, w.common) - S(2, w.common) / S(1, w.common))
  #
  seQ <- sqrt(2 * (k - 1) +
              4 * (S(1, w.common) - S(2, w.common) / S(1, w.common)) * tau2 +
              2 * (S(2, w.common) - 2 * S(3, w.common) / S(1, w.common) +
                   S(2, w.common)^2 / S(1, w.common)^2) * tau2^2
              )
  #  
  seRb <- sqrt((1 / k * sum(a / (Q + a - (k - 1))^2))^2 * seQ^2)
  
  cint <- ci(Rb, seRb, level)
  #
  cint$lower <- ifelse(cint$lower < 0, 0, cint$lower)
  cint$upper <- ifelse(cint$upper > 1, 1, cint$upper)
  
  res <- list(TE = cint$TE, lower = cint$lower, upper = cint$upper)
  #
  res
}
calcH <- function(Q, df, level) {
  #
  # Calculate H
  # Higgins & Thompson (2002), Statistics in Medicine, 21, 1539-58
  #
  k <- df + 1
  #
  if (!is.na(k)) {
    if (k > 1)
      H <- sqrt(Q / (k - 1))
    else
      H <- NA
    #
    selogH <- ifelse(Q > k,
              ifelse(k >= 2,
                     0.5 * (log(Q) - log(k - 1)) /
                     (sqrt(2 * Q) - sqrt(2 * k - 3)),
                     NA),
              ifelse(k > 2,
                     sqrt(1 / (2 * (k - 2)) * (1 - 1 / (3 * (k - 2)^2))),
                     NA))
  }
  else {
    H <- NA
    selogH <- NA
  }
  #
  tres <- ci(log(max(c(H, 1))), selogH, level)
  #
  res <- list(TE = exp(tres$TE),
              lower = max(exp(tres$lower), 1),
              upper = max(exp(tres$upper), 1))
  
  res
}
isquared <- function(Q, df, level) {
  #
  # Calculate I-Squared
  # Higgins & Thompson (2002), Statistics in Medicine, 21, 1539-58
  #
  tres <- calcH(Q, df, level)
  #
  func.t <- function(x) (x^2 - 1) / x^2
  #
  res <- list(TE = func.t(tres$TE),
              lower = func.t(tres$lower),
              upper = func.t(tres$upper))
  
  res
}
pvalQ <- function(Q, df, lower.tail = FALSE) {
  #
  if (length(df) == 1 & length(Q) > 1)
    df <- rep(df, length(Q))
  else if (length(df) > 1 & length(Q) == 1)
    Q <- rep(Q, length(df))
  else if (length(df) > 1 & length(Q) > 1 & length(df) != length(Q))
    stop("Length of arguments 'Q' and 'df' do not match.")
  #
  res <- ifelse(is.na(df) | df < 1,
                NA,
                pchisq(Q, df, lower.tail = lower.tail))
  #
  res
}

addvars2long <- function(x) {
  levs <- NULL
  fac <- is.factor(x$var1) | is.factor(x$var2)
  #
  if (!(is.numeric(x$var1) & is.numeric(x$var2))) {
    levs <- unique(c(x$var1, x$var2))
    x$var1 <- as.numeric(factor(x$var1, levels = levs))
    x$var2 <- as.numeric(factor(x$var2, levels = levs))
  }
  #
  res <-
    to.long("RD", ai = x$var1, n1i = x$var1, ci = x$var2, n2i = x$var2)$out1
  #
  if (!is.null(levs)) {
    res <- factor(res, levels = seq_along(levs), labels = levs)
    #
    if (!fac)
      res <- as.character(res)
  }
  #
  res
}

#' Caffeine for daytime drowsiness
#' 
#' @description
#' Caffeine for daytime drowsiness (Cochrane Practice review)
#' 
#' @details
#' Data come from the Cochrane Practice review on caffeine for daytime
#' drowsiness. Eight fictitous studies evaluate the risk of headaches
#' after drinking either caffeinated or decaffeinated coffee.
#' 
#' @name caffeine
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{year}}\tab year of publication \cr
#' \bold{\emph{h.caf}}\tab Number of participants with headaches (caffeine group) \cr
#' \bold{\emph{n.caf}}\tab Number of participants (caffeine group) \cr
#' \bold{\emph{h.decaf}}\tab Number of participants with headaches (decaf group) \cr
#' \bold{\emph{n.decaf}}\tab Number of participants (decaf group) \cr
#' \bold{\emph{D1}}\tab Domain 1 of risk of bias 2 tool (RoB 2) \cr
#' \bold{\emph{D2}}\tab Domain 2 (RoB 2) \cr
#' \bold{\emph{D3}}\tab Domain 3 (RoB 2) \cr
#' \bold{\emph{D4}}\tab Domain 4 (RoB 2) \cr
#' \bold{\emph{D5}}\tab Domain 5 (RoB 2) \cr
#' \bold{\emph{rob}}\tab Overall RoB 2 assessment
#' }
#' 
#' @seealso \code{\link{metabin}}, \code{\link{rob}}
#' 
#' @references
#' Higgins JPT, Savović J, Page MJ, Sterne JA on behalf of the RoB2
#' Development Group (2019):
#' Revised Cochrane risk-of-bias tool for randomized trials.
#' \url{https://www.riskofbias.info/welcome/rob-2-0-tool}
#' 
#' 
#' @keywords datasets
#' 
#' @examples
#' oldset <- settings.meta("RevMan5")
#' 
#' data(caffeine)
#' head(caffeine)
#'
#' ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
#'   data = caffeine, studlab = paste(study, year))
#'
#' # Add risk of bias assessment to meta-analysis
#' ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")
#'
#' # Print risk of bias assessment
#' rob(ma2)
#' 
#' # Forest plot with risk of bias assessment
#' forest(ma2)
#'
#' settings.meta(oldset)


NULL

#' Produce weighted bar plot of risk of bias assessment
#' 
#' @description
#' Produce weighted bar plot of risk of bias assessment
#' 
#' @param height An object of class \code{rob}.
#' @param overall A logical indicating whether to include a bar for
#'   overall risk of bias in the figure.
#' @param weighted A logical indicating whether weights should be used
#'   in the bar plot.
#' @param colour Specify colour scheme for the bar plot; see
#'   \code{\link[robvis]{rob_summary}}.
#' @param quiet A logical to suppress the display of the bar plot.
#' @param \dots Additional arguments (ignored)
#' 
#' @details
#' This is a wrapper function for \code{\link[robvis]{rob_summary}} of
#' R package \bold{robvis} to produce a weighted bar plot of risk of
#' bias assessment.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{rob}}, \code{\link{traffic_light}},
#'   \code{\link[robvis]{rob_summary}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Use RevMan 5 settings
#' oldset <- settings.meta("RevMan5")
#' 
#' data(caffeine)
#' 
#' ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
#'   data = caffeine, studlab = paste(study, year))
#'
#' # Add risk of bias assessment to meta-analysis
#' ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")
#' 
#' # Print risk of bias assessment
#' rob(ma2)
#'
#' \dontrun{
#' # Weighted bar plot (R package 'robvis' must be available)
#' if (requireNamespace("robvis", quietly = TRUE))
#'  barplot(rob(ma2))
#' }
#'
#' # Use previous settings
#' settings.meta(oldset)
#' 
#' @method barplot rob
#' @export

barplot.rob <- function(height,
                        overall = FALSE, weighted = TRUE,
                        colour = "cochrane",
                        quiet = FALSE, ...) {
  
  chkclass(height, "rob")
  rob <- height
  #
  tool <- attr(rob, "tool")
  #
  chklogical(overall)
  chklogical(weighted)
  chklogical(quiet)
  
  
  if (!is_installed_package("robvis", stop = FALSE))
    stop(paste0("Package 'robvis' missing.",
                "\n  ",
                "Please use the following R command for installation:",
                "\n  install.packages(\"robvis\")"),
         call. = FALSE)
  
  
  if (!(tool %in% c("RoB1", "RoB2", "ROBINS-I", "user-defined"))) {
    warning("R function 'rob_summary() in R package robvis not usable with ",
            "risk of bias tool '", tool, "'.",
            call. = FALSE)
    return(invisible(NULL))
  }
  else {
    if (tool %in% c("RoB2", "ROBINS-I") & is.null(rob$Overall)) {
      warning("No summary weighted barplot created as ",
              "overall risk of bias assessment is missing.",
              call. = FALSE)
      return(invisible(NULL))
    }
    #
    if (is.null(rob$Weight)) {
      rob$Weight <- 1
      if (!missing(weighted) & weighted)
        warning("Argument 'weighted' set to FALSE as no weights are ",
                "available.",
                call. = FALSE)
      weighted <- FALSE
    }
    #
    if (tool %in% c("RoB1", "user-defined")) {
      domains <- attr(rob, "domains")
      #
      nam <- names(rob)
      #
      if (tool == "RoB1") {
        nam[nam == "A"] <- "Random.sequence.generation."
        nam[nam == "B"] <- "Allocation.concealment."
        nam[nam == "C"] <- "Blinding.of.participants.and.personnel."
        nam[nam == "D"] <- "Blinding.of.outcome.assessment."
        nam[nam == "E"] <- "Incomplete.outcome.data."
        nam[nam == "F"] <- "Selective.reporting."
        nam[nam == "G"] <- "Other.bias."
        #
        if (length(domains) > 8)
          nam[nam == "H"] <- paste0(domains[8], ".")
        if (length(domains) > 9)
          nam[nam == "I"] <- paste0(domains[9], ".")
        if (length(domains) > 10)
          nam[nam == "J"] <- paste0(domains[10], ".")
      }
      else if (tool == "user-defined") {
        nam[nam == "Overall"] <- "O"
        nam[nam %in% LETTERS[1:15]] <- paste0(gsub(" ", ".", domains), ".")
        #
        tool <- "RoB1"
      }
      #
      names(rob) <- nam
    }
    #
    return(robvis::rob_summary(rob, tool = toupper(tool),
                               overall = overall, weighted = weighted,
                               colour = colour, quiet = quiet))
  }
}

#' Forest plot to display the result of a leave-one-out meta-analysis
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid
#' graphics system).
#' 
#' @aliases forest.metainf
#' 
#' @param x An object of class \code{\link{metainf}}.
#' @param prediction A logical indicating whether prediction
#'   intervals should be printed.
#' @param overall A logical indicating whether overall results should be
#'   shown.
#' @param just.addcols Justification of text for additional columns
#'   (possible values: "left", "right", "center").
#' @param smlab A label for the summary measure (printed at top of
#'   figure).
#' @param type A character string or vector specifying how to
#'   plot treatment effects and confidence intervals for cumulative
#'   meta-analysis results.
#' @param lab.NA A character string to label missing values.
#' @param layout A character string specifying the layout of the
#'   forest plot (see \code{\link{forest.meta}}).
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param big.mark A character used as thousands separator.
#' @param digits Minimal number of significant digits for treatment
#'   effects, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for
#'   p-values.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic.
#' @param digits.cid Minimal number of significant digits for
#'   CID / decision thresholds, see \code{print.default}.
#' @param digits.percent Minimal number of significant digits for
#'   probabilities, printed as percentages, see \code{print.default}.
#' @param col The colour for cumulative meta-analysis results (only considered
#'   if \code{type = "square"}).
#' @param col.bg The background colour for squares and diamonds of
#'   cumulative meta-analysis results.
#' @param col.border The colour for the outer lines of squares and diamonds of
#'   cumulative meta-analysis results.
#' @param col.bg.predict The background colour for prediction intervals of
#'   cumulative meta-analysis results.
#' @param col.border.predict The colour for the outer lines of prediction
#'   intervals of cumulative meta-analysis results.
#' @param addrows.below.overall A numeric value indicating how many
#'   empty rows are printed between meta-analysis results and
#'   meta-analysis details.
#' @param details A logical specifying whether details on statistical
#'   methods should be printed.
#' @param \dots Additional graphical arguments (passed on to
#'   \code{\link{forest.meta}}).
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window. Internally, R function
#' \code{\link{forest.metacum}} is called to produce the forest plot.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.metacum}}, \code{\link{forest.meta}},
#'   \code{\link{metainf}}, \code{\link{settings.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Fleiss1993bin)
#' ma <- metabin(d.asp, n.asp, d.plac, n.plac,
#'   data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
#' ma
#' metainf(ma)
#' metainf(ma, pooled = "random")
#' 
#' forest(metainf(ma))
#' forest(metainf(ma, pooled = "random"))
#' forest(metainf(ma, pooled = "random", prediction = TRUE))
#'
#' @method forest metainf
#' @export

forest.metainf <- function(x,
                           #
                           prediction = x$prediction,
                           overall = x$overall,
                           just.addcols = "right",
                           smlab = "Leave-One-Out Meta-Analysis",
                           type = "square",
                           layout = gs("layout"),
                           lab.NA = ".",
                           #
                           backtransf = x$backtransf,
                           #
                           big.mark = gs("big.mark"),
                           digits = gs("digits.forest"),
                           digits.pval = gs("digits.pval"),
                           digits.tau2 = gs("digits.tau2"),
                           digits.tau = gs("digits.tau"),
                           digits.I2 = gs("digits.I2"),
                           digits.cid = gs("digits.cid"),
                           digits.percent = 1,
                           #
                           col = gs("col.study"),
                           col.bg = 
                             ifelse(type == "diamond",
                                    gs("col.diamond"), gs("col.square")),
                           col.border =
                             ifelse(type == "diamond",
                                    gs("col.diamond.lines"),
                                    gs("col.square.lines")),
                           col.bg.predict = gs("col.predict"),
                           col.border.predict = gs("col.predict.lines"),
                           #
                           addrows.below.overall = 1L * details,
                           details = gs("forest.details"),
                           ...) {
  
  chkclass(x, "metainf")
  #
  x$prediction <- prediction
  x$overall <- overall
  x$backtransf <- backtransf
  #
  res <- forest.metacum(x,
                        #
                        just.addcols = just.addcols,
                        smlab = smlab,
                        type = type,
                        layout = layout,
                        lab.NA = lab.NA,
                        #
                        big.mark = big.mark,
                        digits = digits,
                        digits.pval = digits.pval,
                        digits.tau2 = digits.tau2,
                        digits.tau = digits.tau,
                        digits.I2 = digits.I2,
                        digits.cid = digits.cid,
                        digits.percent = digits.percent,
                        #
                        col = col,
                        col.bg = col.bg,
                        col.border = col.border,
                        col.bg.predict = col.bg.predict,
                        col.border.predict = col.border.predict,
                        #
                        addrows.below.overall = addrows.below.overall,
                        details = details,
                        ...)
  #
  return(invisible(res))
}

smlab <- function(sm, backtransf, pscale, irscale) {
  res <- sm
  #
  if (backtransf) {
    if (sm == "ZCOR")
      res <- "COR"
    else if (is_mean(sm))
      res <- "mean"
    else if (is_prop(sm)) {
      if (pscale == 1)
        res <- "proportion"
      else
        res <- "events"
    }
    else if (is_rate(sm)) {
      if (irscale == 1)
        res <- "rate"
      else
        res <- "events"
    }
  }
  else {
    if (is_relative_effect(sm))
      res <- paste0("ln", sm)
    else if (sm == "VE")
      res <- "lnVR"
  }
  #
  res
}

#' Coerce to a data frame
#' 
#' @description
#' The \code{as.data.frame} method returns a data frame containing
#' information on individual studies, e.g., estimated treatment effect
#' and its standard error.
#' 
#' @param x An object of class \code{meta}.
#' @param row.names \code{NULL} or a character vector giving the row
#'   names for the data frame.
#' @param optional logical. If \code{TRUE}, setting row names and
#'   converting column names (to syntactic names) is optional.
#' @param \dots other arguments
#' 
#' @return A data frame is returned by the function \code{as.data.frame}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metagen}}, \code{\link{forest.meta}}
#' 
#' @examples
#' data(Fleiss1993cont)
#' #
#' # Generate additional variable with grouping information
#' #
#' Fleiss1993cont$group <- c(1, 2, 1, 1, 2)
#' #
#' # Do meta-analysis without grouping information
#' #
#' ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD", studlab = paste(study, year))
#' #
#' # Update meta-analysis object and do subgroup analyses
#' #
#' update(ma1, subgroup = group)
#' 
#' # Same result using metacont function directly
#' #
#' ma2 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD", studlab = paste(study, year),
#'   subgroup = group)
#' ma2
#' 
#' # Compare printout of the following two commands
#' #
#' as.data.frame(ma1)
#' ma1$data
#'
#' @method as.data.frame meta
#' @export

as.data.frame.meta <- function(x, row.names = NULL, optional = FALSE, ...) {
  
  
  #
  #
  # (1) Check for meta object and upgrade older meta objects
  #
  #
  chkclass(x, "meta")
  
  
  # Remove element 'call' from object of class meta to get rid
  # of an error message in meta-analyses with six studies:
  # 'Error: evaluation nested too deeply: infinite recursion ...'
  #
  # NB: Element 'call' which is of length six contains information
  #     on the function call.
  #
  x$call <- NULL
  x$call.object <- NULL
  
  # Remove data set from output
  #
  x$data <- NULL
  
  # Remove risk of bias table from output
  #
  x$rob <- NULL
  
  # Remove list information from metabind()
  #
  x$list <- NULL
  
  # Remove debug information
  #
  x$debug <- x$tau2.calc <- x$rma.three.level <- NULL
  
  if (!is.null(x$approx.TE) && all(x$approx.TE == ""))
    x$approx.TE <- NULL
  #
  if (!is.null(x$approx.seTE) && all(x$approx.seTE == ""))
    x$approx.seTE <- NULL
  
  sel <- as.vector(lapply(x, length) == length(x$TE))
  
  res <- as.data.frame(x[names(x)[sel]], ...)
  #
  if (!is.null(row.names)) {
    if (length(row.names) != nrow(res))
      stop("Argument 'row.names' must be of length ", nrow(res))
    row.names(res) <- row.names
  }
  else
    row.names(res) <- seq_len(nrow(res))
  
  attr(res, "version") <- packageDescription("meta")$Version
  
  res
}

#' Merge results of two meta-analyses on the same data set
#' 
#' @description
#' This function can be used to merge results of two meta-analyses
#' into a single meta-analysis object if they are based on the same
#' data set. This is, for example, useful to produce a forest plot of
#' a random-effects meta-analysis with different estimates of the
#' between-study variance \eqn{\tau^2}.
#' 
#' @param meta1 First meta-analysis object (see Details).
#' @param meta2 Second meta-analysis object (see Details).
#' @param common1 A logical indicating whether results of common
#'   effect model should be considered for first meta-analysis.
#' @param random1 A logical indicating whether results of random
#'   effects model should be considered for first meta-analysis.
#' @param prediction1 A logical indicating whether prediction interval
#'   should be considered for first meta-analysis.
#' @param common2 A logical indicating whether results of common
#'   effect model should be considered for second meta-analysis.
#' @param random2 A logical indicating whether results of random
#'   effects model should be considered for second meta-analysis.
#' @param prediction2 A logical indicating whether prediction interval
#'   should be considered for second meta-analysis.
#' @param label1 Default setting for arguments 'label1.common',
#'   'label1.random', 'label1.predict' and 'label1.subgroup'.
#' @param label2 Default setting for arguments 'label2.common',
#'   'label2.random', 'label2.predict' and 'label2.subgroup'.
#' @param label1.common A character string to label the common effect estimate
#'   from the first meta-analysis.
#' @param label2.common A character string to label the common effect estimate
#'   from the second meta-analysis.
#' @param label1.random A character string to label the random effects estimate
#'   from the first meta-analysis (default label for arguments 'hetlabel1' and
#'   'taulabel1').
#' @param label2.random A character string to label the random effects estimate
#'   from the second meta-analysis (default label for arguments 'hetlabel2' and
#'   'taulabel2').
#' @param label1.predict A character string to label the prediction interval
#'   from the first meta-analysis.
#' @param label2.predict A character string to label the prediction interval
#'   from the second meta-analysis.
#' @param label1.subgroup A character string to label the subgroup results
#'   from the first meta-analysis.
#' @param label2.subgroup A character string to label the subgroup results
#'   from the second meta-analysis.
#' @param text.pooled1 A character string used in printouts and forest
#'   plot to label the results from the first meta-analysis.
#' @param text.pooled2 A character string used in printouts and forest
#'   plot to label the results from the second meta-analysis.
#' @param text.w.pooled1 A character string used to label weights of
#'   the first meta-analysis; can be of same length as the number of
#'   pooled estimates requested in argument \code{pooled1}.
#' @param text.w.pooled2 A character string used to label weights of
#'   the second meta-analysis; can be of same length as the number of
#'   pooled estimates requested in argument \code{pooled1}.
#' @param hetlabel1 A character string used to label heterogeneity
#'   statistics of the first meta-analysis.
#' @param hetlabel2 A character string used to label heterogeneity
#'   statistics of the second meta-analysis.
#' @param taulabel1 A character string used to label estimate of
#'   between-study variance of the first meta-analysis.
#' @param taulabel2 A character string used to label estimate of
#'   between-study variance of the second meta-analysis.
#' @param text.common1 A character string used in printouts and forest
#'   plot to label results for common effect models from the first
#'   meta-analysis.
#' @param text.common2 A character string used in printouts and forest
#'   plot to label results for common effect models from the second
#'   meta-analysis.
#' @param text.random1 A character string used in printouts and forest
#'   plot to label results for random effects models from the first
#'   meta-analysis.
#' @param text.random2 A character string used in printouts and forest
#'   plot to label results for random effects models from the second
#'   meta-analysis.
#' @param text.predict1 A character string used in printouts and
#'   forest plot to label prediction interval from the first
#'   meta-analysis.
#' @param text.predict2 A character string used in printouts and
#'   forest plot to label prediction interval from the second
#'   meta-analysis.
#' @param text.w.common1 A character string used to label common
#'   effect weights of the first meta-analysis; can be of same length
#'   as the number of common effect estimates.
#' @param text.w.common2 A character string used to label common
#'   effect weights of the second meta-analysis; can be of same length
#'   as the number of common effect estimates.
#' @param text.w.random1 A character string used to label random
#'   effects weights of the first meta-analysis; can be of same length
#'   as the number of random effects estimates.
#' @param text.w.random2 A character string used to label random
#'   effects weights of the second meta-analysis; can be of same
#'   length as the number of random effects estimates.
#' @param keep A logical indicating whether to keep additional
#'   information from second meta-analysis.
#' @param keep.Q A logical indicating whether heterogeneity statistic
#'   Q of second meta-analysis should be kept or ignored.
#' @param keep.I2 A logical indicating whether heterogeneity statistic
#'   I2 of second meta-analysis should be kept or ignored.
#' @param keep.w A logical indicating whether weights of the second
#'   meta-analysis should be kept or ignored.
#' @param common A logical indicating whether results of common effect
#'   meta-analyses should be reported.
#' @param random A logical indicating whether results of random
#'   effects meta-analyses should be reported.
#' @param overall A logical indicating whether overall summaries
#'   should be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons.
#' @param prediction A logical indicating whether prediction intervals
#'   should be reported.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and plots. If
#'   \code{backtransf=TRUE} (default), results for \code{sm="OR"} are
#'   printed as odds ratios rather than log odds ratios, for example.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param pooled1 Deprecated argument (replaced by 'common1',
#'   'random1', 'prediction1'). A character string indicating whether
#'   results of common effect or random effects model should be
#'   considered for first meta-analysis. Either \code{"both"},
#'   \code{"common"} or \code{"random"}, can be abbreviated.
#' @param pooled2 Deprecated argument (replaced by 'common2',
#'   'random2', 'prediction2'). A character string indicating whether
#'   results of common effect or random effects model should be
#'   considered for second meta-analysis. Either \code{"both"},
#'   \code{"common"} or \code{"random"}, can be abbreviated.
#' 
#' @details
#' In R package \bold{meta}, objects of class \code{"meta"} contain
#' results of both common effect and random effects
#' meta-analyses. This function enables the user to merge the results
#' of two meta-analysis object if they are based on the same data set.
#'
#' Applications of this function include printing and plotting results
#' of the common effect or random effects meta-analysis and the
#' \itemize{
#' \item trim-and-fill method (\code{\link{trimfill}}),
#' \item limit meta-analyis (\code{\link[metasens]{limitmeta}} from R
#'   package \bold{metasens}),
#' \item Copas selection model (\code{\link[metasens]{copas}} from R
#'   package \bold{metasens}),
#' \item robust variance meta-analysis model
#'   (\code{\link[robumeta]{robu}} from R package \bold{robumeta}).
#' }
#'
#' The first argument (\code{meta1}) must be an object created by a
#' meta-analysis function (see \code{\link{meta-object}}). If an
#' object created with \code{\link[metasens]{limitmeta}} or
#' \code{\link[metasens]{copas}} is provided as the first argument,
#' this object will be returned, i.e., argument \code{meta2} will be
#' ignored.
#'
#' The second meta-analysis could be an object created by a
#' meta-analysis function or with \code{\link{trimfill}},
#' \code{\link[metasens]{limitmeta}}, \code{\link[metasens]{copas}},
#' or \code{\link[robumeta]{robu}}.
#'
#' The created meta-analysis object only contains the study results,
#' i.e., estimated effects and confidence intervals, from the first
#' meta-analysis which are shown in printouts and forest plots. This
#' only makes a difference for meta-analysis methods where individual
#' study results differ, e.g., Mantel-Haenszel and Peto method for
#' binary outcomes (see \code{\link{metabin}}).
#'
#' R function \code{\link{metaadd}} can be used to add pooled results
#' from any (external) meta-analysis.
#'
#' R function \code{\link{metabind}} can be used to print and plot the
#' results of several meta-analyses without the restriction that the
#' same data set has to be used. Accordingly, individual study results
#' are ignored.
#' 
#' @return
#' An object of class \code{"meta"} and \code{"metamerge"} with
#' corresponding generic functions (see \code{\link{meta-object}}).
#' 
#' The following list elements have a different meaning:
#' \item{TE, seTE, studlab}{Treatment estimate, standard error, and
#'   study labels (first meta-analysis).}
#' \item{lower, upper}{Lower and upper confidence interval limits for
#'   individual studies (first meta-analysis).}
#' \item{statistic, pval}{Statistic and p-value for test of treatment
#'   effect for individual studies (first meta-analysis.}
#' \item{w.common}{Vector or matrix with common effect weights.}
#' \item{w.random}{Vector or matrix with random effects weights.}
#' \item{k}{Vector with number of estimates (same length as number of
#'   common effect and random effects estimates).}
#' \item{k.study}{Vector with number of studies (same length as
#'   number of common effect and random effects estimates).}
#' \item{k.all}{Vector with total number of studies (same length as
#'   number of common effect and random effects estimates).}
#' \item{k.TE}{Vector with number of studies with estimable effects
#'   (same length as number of common effect and random effects
#'   estimates).}
#' \item{k.MH}{Vector with number of studies combined with
#'   Mantel-Haenszel method (same length as number of common effect
#'   and random effects estimates).}
#' \item{TE.common}{Vector with common effect estimates.}
#' \item{seTE.common}{Vector with standard errors of common effect
#'   estimates.}
#' \item{lower.common}{Vector with lower confidence limits (common
#'   effect model).}
#' \item{upper.common}{Vector with upper confidence limits (common
#'   effect model).}
#' \item{statistic.common}{Vector with test statistics for test of
#'   overall effect (common effect model).}
#' \item{pval.common}{Vector with p-value of test for overall effect
#'   (common effect model).}
#' \item{TE.random}{Vector with random effects estimates.}
#' \item{seTE.random}{Vector with standard errors of random effects
#'   estimates.}
#' \item{lower.random}{Vector with lower confidence limits (random
#'   effects model).}
#' \item{upper.random}{Vector with upper confidence limits (random
#'   effects model).}
#' \item{statistic.random}{Vector with test statistics for test of
#'   overall effect (random effects model).}
#' \item{pval.random}{Vector with p-value of test for overall effect
#'   (random effects model).}
#' 
#' Furthermore, meta-analysis results of common effect or random
#' effects model are taken from first meta-analysis if only random
#' effects or common effects models are selected from both
#' meta-analyses (arguments \code{pooled1} and \code{pooled2}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metagen}}, \code{\link{metabind}},
#'   \code{\link{metaadd}}
#' 
#' @examples
#' # Print results with more significant digits and do not show confidence
#' # intervals for tau^2 and tau
#' oldset <- settings.meta(digits = 6, digits.stat = 4, digits.pval = 6,
#'   digits.Q = 6, digits.I2 = 4, digits.H = 4,
#'   print.tau2.ci = FALSE, print.tau.ci = FALSE)
#' oldopts <- options(width = 120)
#' 
#' data(Fleiss1993bin)
#' 
#' # Mantel-Haenszel method
#' ma1 <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
#'   studlab = paste(study, year), sm = "OR")
#' # Peto method
#' ma2 <- update(ma1, method = "Peto")
#' # Inverse variance method (only common effect model)
#' ma3 <- update(ma2, method = "Inverse", random = FALSE)
#' 
#' # Merge results from MH and Peto method
#' # - show individual results for MH method
#' #   (as this is the first meta-analysis)
#' # - keep all additional information from Peto meta-analysis (i.e.,
#' #   weights, Q statistic and I2 statistic)
#' ma12 <- metamerge(ma1, ma2,
#'   label1 = "REML", label2 = "REML-Peto",
#'   label1.common = "MH", label2.common = "Peto", 
#'   text.common1 = "Mantel-Haenszel method",
#'   text.common2 = "Peto method",
#'   text.w.random1 = "REML", text.w.random2 = "REML-Peto",
#'   hetlabel1 = "MH/IV", hetlabel2 = "Peto",
#'   keep = TRUE)
#' 
#' # Add common effect results from inverse variance method
#' # - keep weights from IV meta-analysis
#' # - Q and I2 statistic are identical for sm = "MH" and sm = "Inverse"
#' #   as inverse variance method is used for sm = "MH" under random
#' #   effects model
#' ma123 <- metamerge(ma12, ma3,
#'   label2 = "IV",
#'   text.common2 = "Inverse variance method",
#'   keep.w = TRUE)
#' summary(ma123)
#' \dontrun{
#' forest(ma123, digits = 6)
#' 
#' # Merge results (show individual results for Peto method)
#' ma21 <- metamerge(ma2, ma1,
#'   label1 = "REML-Peto", label2 = "REML",
#'   label1.common = "Peto", label2.common = "MH", 
#'   hetlabel1 = "Peto", hetlabel2 = "MH/IV",
#'   text.common1 = "Peto method",
#'   text.common2 = "Mantel-Haenszel method",
#'   keep = TRUE)
#' 
#' # Add results from inverse variance method
#' # - keep weights from IV meta-analysis
#' # - Q and I2 statistic are identical for sm = "MH" and sm = "Inverse"
#' #   as inverse variance method is used for sm = "MH" under random
#' #   effects model
#' ma213 <- metamerge(ma21, ma3,
#'   label2 = "IV",
#'   text.common2 = "Inverse variance method",
#'   keep.w = TRUE)
#' summary(ma213)
#' 
#' # Random effects method using ML estimator for between-study variance tau2
#' ma4 <- update(ma1, common = FALSE, method.tau = "ML")
#' 
#' # Use DerSimonian-Laird estimator for tau2
#' ma5 <- update(ma4, method.tau = "DL")
#' 
#' # Use Paule-Mandel estimator for tau2
#' ma6 <- update(ma4, method.tau = "PM")
#' 
#' # Merge random effects results for ML and DL estimators
#' # - keep weights for DL estimator (which are different from ML)
#' ma45 <- metamerge(ma4, ma5, label1 = "ML", label2 = "DL",
#'   text.w.random1 = "RE-ML", text.w.random2 = "RE-DL", keep.w = TRUE)
#' summary(ma45)
#' 
#' # Add results for PM estimator
#' # - keep weights
#' ma456 <- metamerge(ma45, ma6, label2 = "PM",
#'   text.w.random2 = "RE-PM", keep.w = TRUE)
#' summary(ma456)
#' 
#' ma123456 <- metamerge(ma123, ma456)
#' ma123456
#' 
#' # Use Hartung-Knapp confidence intervals
#' # - do not keep information on Q, I2 and weights
#' ma7 <- update(ma4, method.random.ci = "HK",
#'   text.random = "Hartung-Knapp method")
#' ma8 <- update(ma5, method.random.ci = "HK",
#'   text.random = "Hartung-Knapp method")
#' ma9 <- update(ma6, method.random.ci = "HK",
#'   text.random = "Hartung-Knapp method")
#' 
#' # Merge results for Hartung-Knapp method (with REML and DL estimator)
#' # - RE weights for REML estimator are shown
#' ma78 <- metamerge(ma7, ma8, label1 = "ML", label2 = "DL")
#' summary(ma78)
#' 
#' ma789 <- metamerge(ma78, ma9, label2 = "PM")
#' summary(ma789)
#' 
#' # Merge everything
#' ma1to9 <- metamerge(metamerge(ma123, ma456, keep.w = TRUE), ma789)
#' summary(ma1to9)
#' 
#' ma10 <- update(ma1, method = "GLMM")
#' 
#' ma.all <- metamerge(ma1to9, ma10, keep.Q = TRUE,
#'   label2 = "GLMM", taulabel2 = "ML-GLMM")
#' summary(ma.all)
#' 
#' forest(ma.all, layout = "JAMA")
#' forest(ma.all, details = TRUE)
#' }
#' 
#' settings.meta(oldset)
#' options(oldopts)
#' 
#' @export metamerge

metamerge <- function(meta1, meta2,
                      #
                      common1 = meta1$common,
                      random1 = meta1$random,
                      prediction1 = meta1$prediction,
                      common2 = meta2$common,
                      random2 = meta2$random,
                      prediction2 = meta2$prediction,
                      #
                      label1 = NULL, label2 = NULL,
                      label1.common = label1, label2.common = label2,
                      label1.random = label1, label2.random = label2,
                      label1.predict = label1, label2.predict = label2,
                      label1.subgroup = label1, label2.subgroup = label2,
                      #
                      hetlabel1 = label1.random,
                      hetlabel2 = label2.random,
                      taulabel1 = label1.random,
                      taulabel2 = label2.random,
                      #
                      text.pooled1 = NULL, text.pooled2 = NULL,
                      text.w.pooled1 = NULL, text.w.pooled2 = NULL,
                      #
                      text.common1 = text.pooled1,
                      text.common2 = text.pooled2,
                      text.random1 = text.pooled1,
                      text.random2 = text.pooled2,
                      text.predict1 = text.pooled1,
                      text.predict2 = text.pooled2,
                      #
                      text.w.common1 = text.w.pooled1,
                      text.w.common2 = text.w.pooled2,
                      text.w.random1 = text.w.pooled1,
                      text.w.random2 = text.w.pooled2,
                      #
                      keep = FALSE,
                      keep.Q = keep, keep.I2 = keep.Q,
                      keep.w = keep,
                      #
                      common = common1 | common2,
                      random = random1 | random2,
                      overall = common | random,
                      overall.hetstat = common | random,
                      prediction = prediction1 | prediction2,
                      #
                      backtransf,
                      #
                      warn.deprecated = gs("warn.deprecated"),
                      pooled1, pooled2) {
  
  #
  #
  # (1) Check arguments
  #
  #
  
  if (missing(meta1))
    stop("Argument 'meta1' must be provided.",
         call. = FALSE)
  #
  if (missing(meta2)) {
    if (!inherits(meta1, "metamerge") & inherits(meta1, "copas"))
      return(metamerge(update(meta1$x), meta1,
                       label2 = if (is.null(label2)) "copas" else label2))
    #
    else if (!inherits(meta1, "metamerge") & inherits(meta1, "limitmeta"))
      return(metamerge(update(meta1$x), meta1,
                       label2 = if (is.null(label2)) "limit" else label2))
    #
    else if (!inherits(meta1, "metamerge") & inherits(meta1, "robu"))
      stop("Argument 'meta1' cannot be of class 'robu'.",
           call. = FALSE)
    #
    else
      return(meta1)
  }
  else if (!inherits(meta1, "metamerge")) {
    if (inherits(meta1, "copas"))
      stop("Argument 'meta1' cannot be of class 'copas' ",
           "(use argument 'meta2').",
           call. = FALSE)
    else if (inherits(meta1, "limitmeta"))
      stop("Argument 'meta1' cannot be of class 'limitmeta' ",
           "(use argument 'meta2').",
           call. = FALSE)
    else if (inherits(meta1, "robu"))
      stop("Argument 'meta1' cannot be of class 'robu' ",
           "(use argument 'meta2').",
           call. = FALSE)
  }
  #
  chkclass(meta1, "meta")
  chksuitable(meta1, "R function metamerge()", classes = "metaprop",
              check.mlm = FALSE)
  meta1 <- updateversion(meta1)
  #
  chkclass(meta2, c("meta", "limitmeta", "copas", "robu"))
  chksuitable(meta2, "R function metamerge()", classes = "metaprop",
              check.mlm = FALSE)
  #
  if (inherits(meta1, "netpairwise") | inherits(meta2, "netpairwise"))
    stop("R objects of class 'netpairwise' cannot be merged.",
         call. = FALSE)
  #
  if (inherits(meta2, "meta"))
    meta2 <- updateversion(meta2)
  #
  is.copas <- !inherits(meta2, "metamerge") & inherits(meta2, "copas")
  is.limit <- !inherits(meta2, "metamerge") & inherits(meta2, "limitmeta")
  is.robu  <- !inherits(meta2, "metamerge") & inherits(meta2, "robu")
  #
  if (is.copas | is.limit | is.robu) {
    common2 <- FALSE
    random2 <- TRUE
    prediction2 <- FALSE
    meta2$three.level <- FALSE
    meta2$rho <- NA
    #
    if (is.copas)
      meta2$k <- length(meta2$TE)
    #
    if (is.limit)
      meta2$tau2 <- meta2$tau^2
    #
    keep <- FALSE
    keep.Q <- FALSE
    keep.I2 <- FALSE
    keep.w <- FALSE
  }
  #
  chklogical(warn.deprecated)
  #
  missing.pooled1 <- missing(pooled1)
  missing.pooled2 <- missing(pooled2)
  #
  missing.common1 <- missing(common1)
  missing.random1 <- missing(random1)
  #
  missing.common2 <- missing(common2)
  missing.random2 <- missing(random2)
  #
  deprecated2(common1, missing(common1),
              pooled1, missing.pooled1, warn.deprecated)
  deprecated2(random1, missing(random1),
              pooled1, missing.pooled1, warn.deprecated)
  #
  if (missing.common1 & !missing(pooled1)) {
    pooled1 <- setchar(pooled1, c("both", "common", "random", "fixed"))
    pooled1[pooled1 == "fixed"] <- "common"
    common1 <- pooled1 == "common"
  }
  #
  if (missing.random1 & !missing(pooled1)) {
    pooled1 <- setchar(pooled1, c("both", "common", "random", "fixed"))
    random1 <- pooled1 %in% c("common", "both")
  }
  #
  chklogical(common1)
  chklogical(random1)
  #
  deprecated2(common2, missing(common2),
              pooled2, missing.pooled2, warn.deprecated)
  deprecated2(random2, missing(random2),
              pooled2, missing.pooled2, warn.deprecated)
  #
  if (missing.common2 & !missing(pooled2)) {
    pooled2 <- setchar(pooled2, c("both", "common", "random", "fixed"))
    pooled2[pooled2 == "fixed"] <- "common"
    common2 <- pooled2 == "common"
  }
  #
  if (missing.random2 & !missing(pooled2)) {
    pooled2 <- setchar(pooled2, c("both", "common", "random", "fixed"))
    random2 <- pooled2 %in% c("common", "both")
  }
  #
  chklogical(common2)
  chklogical(random2)
  #
  chklogical(keep)
  chklogical(keep.Q)
  chklogical(keep.I2)
  chklogical(keep.w)
  #
  chkchar(label1, length = 1, NULL.ok = TRUE)
  chkchar(label2, length = 1, NULL.ok = TRUE)
  #
  chkchar(label1.common, length = 1, NULL.ok = TRUE)
  chkchar(label2.common, length = 1, NULL.ok = TRUE)
  #
  chkchar(label1.random, length = 1, NULL.ok = TRUE)
  chkchar(label2.random, length = 1, NULL.ok = TRUE)
  #
  chkchar(label1.predict, length = 1, NULL.ok = TRUE)
  chkchar(label2.predict, length = 1, NULL.ok = TRUE)
  #
  chkchar(hetlabel1, length = 1, NULL.ok = TRUE)
  chkchar(taulabel1, length = 1, NULL.ok = TRUE)
  #
  chkchar(hetlabel2, length = 1, NULL.ok = TRUE)
  chkchar(taulabel2, length = 1, NULL.ok = TRUE)
  #
  chkchar(text.pooled1, length = 1, NULL.ok = TRUE)
  chkchar(text.common1, length = 1, NULL.ok = TRUE)
  chkchar(text.random1, length = 1, NULL.ok = TRUE)
  chkchar(text.predict1, length = 1, NULL.ok = TRUE)
  #
  chkchar(text.common2, length = 1, NULL.ok = TRUE)
  chkchar(text.pooled2, length = 1, NULL.ok = TRUE)
  chkchar(text.random2, length = 1, NULL.ok = TRUE)
  chkchar(text.predict2, length = 1, NULL.ok = TRUE)
  #
  chkchar(text.w.pooled1, length = 1, NULL.ok = TRUE)
  chkchar(text.w.common1, length = 1, NULL.ok = TRUE)
  chkchar(text.w.random1, length = 1, NULL.ok = TRUE)
  #
  chkchar(text.w.pooled2, length = 1, NULL.ok = TRUE)
  chkchar(text.w.common2, length = 1, NULL.ok = TRUE)
  chkchar(text.w.random2, length = 1, NULL.ok = TRUE)
  #
  chklogical(common)
  chklogical(random)
  chklogical(overall)
  chklogical(overall.hetstat)
  chklogical(prediction)
  #
  if (!missing(backtransf))
    chklogical(backtransf)
  else {
    if (!is.null(meta1$backtransf) & !is.null(meta2$backtransf))
      backtransf <- meta1$backtransf | meta2$backtransf
    else if (!is.null(meta1$backtransf))
      backtransf <- meta1$backtransf
    else if (!is.null(meta2$backtransf))
      backtransf <- meta2$backtransf
    else
      backtransf <- FALSE
  }
  #
  # Check summary measures
  #
  if (!is.robu)
    samesm(meta1, meta2)
  #
  # Check original data (if available)
  #
  if (!is.robu)
    samedata(meta1, meta2)
  #
  # Check subgroup levels
  #
  samesubgroups(meta1, meta2)
  #
  # Check levels of confidence intervals
  #
  lvls <- sort(unique(c(meta1$level, meta2$level)))
  lvls.ma <- sort(unique(c(meta1$level.ma, meta2$level.ma)))
  #
  if (length(lvls) != 1)
    stop("Different level for confidence intervals of individual studies: ",
         paste0(lvls, collapse = ", "),
         call. = FALSE)
  if (length(lvls.ma) != 1)
    stop("Different level for meta-analysis confidence intervals: ",
         paste0(lvls.ma, collapse = ", "),
         call. = FALSE)
  
  
  #
  #
  # (2) Some assignments for trim-and-fill method (meta1 / meta2),
  #     Copas selection model, limit meta-analysis and robust
  #     variance meta-analysis (meta2)
  #
  
  meta1 <- updateobj(meta1,
                     label1.common, label1.random, label1.predict,
                     hetlabel1, taulabel1, label1.subgroup,
                     text.common1, text.random1, text.predict1,
                     text.w.common1, text.w.random1)
  #
  hetlabel1 <- meta1$hetlabel
  taulabel1 <- meta1$taulabel
  #
  meta2 <- updateobj(meta2,
                     label2.common, label2.random, label2.predict,
                     hetlabel2, taulabel2, label2.subgroup,
                     text.common2, text.random2, text.predict2,
                     text.w.common2, text.w.random2)
  #
  hetlabel2 <- meta2$hetlabel
  taulabel2 <- meta2$taulabel
  
  
  #
  #
  # (3) Some more assignments
  #
  #
  
  meta1$detail.tau <- replaceNULL(meta1$detail.tau, "")
  meta2$detail.tau <- replaceNULL(meta2$detail.tau, "")
  #
  meta1$method.tau <- replaceNULL(meta1$method.tau, "")
  meta2$method.tau <- replaceNULL(meta2$method.tau, "")
  #
  meta1$method.tau.ci <- replaceNULL(meta1$method.tau.ci, "")
  meta2$method.tau.ci <- replaceNULL(meta2$method.tau.ci, "")
  #
  #meta1$method.random.ci <- replaceNULL(meta1$method.random.ci, "")
  #meta2$method.random.ci <- replaceNULL(meta2$method.random.ci, "")
  #
  meta1$df.random <- replaceNULL(meta1$df.random, NA)
  meta2$df.random <- replaceNULL(meta2$df.random, NA)
  
  
  #
  #
  # (4) Remove results from first meta-analysis (if necessary)
  #
  #
  
  if (!common1 & common2)
    meta1 <- dropcommon(meta1)
  #
  if (!random1 & random2)
    meta1 <- droprandom(meta1)
  #
  if (!prediction1 & prediction2)
    meta1 <- droppredict(meta1)
  
  
  #
  #
  # (5) Remove results from second meta-analysis (if necessary)
  #
  #
  
  if (!common2)
    meta2 <- dropcommon(meta2)
  #
  if (!random2)
    meta2 <- droprandom(meta2)
  #
  if (!prediction2)
    meta2 <- droppredict(meta2)
  
  
  #
  #
  # (6) Merge results
  #
  #
  
  res <- meta1
  #
  ncom1 <- length(meta1$TE.common)
  nran1 <- length(meta1$TE.random)
  ncom2 <- length(meta2$TE.common)
  nran2 <- length(meta2$TE.random)
  #
  # Individual study weights
  #
  if (!common1 & common2) {
    res$w.common <- meta2$w.common
    res$text.w.common <- meta2$text.w.common
  }
  else if (!is.null(meta1$w.common) & !is.null(meta2$w.common) & keep.w) {
    res$text.w.common <- c(meta1$text.w.common, meta2$text.w.common)
    #
    res$w.common <- cbind(meta1$w.common, meta2$w.common)
    colnames(res$w.common) <- res$text.w.common
    rownames(res$w.common) <- meta1$studlab
  }
  #
  if (is.null(res$w.random) & !is.null(meta2$w.random)) {
    res$w.random <- meta2$w.random
    res$text.w.random <- meta2$text.w.random
  }
  else if (!is.null(res$w.random) & !is.null(meta2$w.random) & keep.w) {
    if (length(res$w.random) != length(meta2$w.random)) {
      warning("Argument 'keep.w' set to FALSE as number of weights differs ",
              "between meta-analyses.",
              call. = FALSE)
      keep.w <- FALSE
    }
    else {
      res$text.w.random <- c(meta1$text.w.random, meta2$text.w.random)
      #
      res$w.random <- cbind(meta1$w.random, meta2$w.random)
      colnames(res$w.random) <- c(meta1$text.w.random, meta2$text.w.random)
      rownames(res$w.random) <- meta1$studlab
    }
  }
  #
  res$hetlabel <- c(meta1$hetlabel, meta2$hetlabel)
  #
  res$method <-
    expandmerge(meta1$method, meta2$method,
                nc1 = ncom1, nc2 = ncom2)
  res$method.random <-
    expandmerge(meta1$method.random, meta2$method.random,
                nr1 = nran1, nr2 = nran2)
  #
  # Number of studies
  #
  if (!inherits(meta1, "metamerge") & (is.limit | is.copas | is.robu)) {
    res$k <- meta1$k
    res$k.all <- meta1$k.all
    res$k.MH <- meta1$k.MH
    res$k.study <- meta1$k.study
    res$k.TE <- meta1$k.TE
  }
  else {
    res$k <-
      expandmerge(meta1$k, meta2$k,
                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
    res$k.all <-
      expandmerge(meta1$k.all, meta2$k.all,
                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
    res$k.MH <-
      expandmerge(meta1$k.MH, meta2$k.MH,
                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
    res$k.study <-
      expandmerge(meta1$k.study, meta2$k.study,
                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
    res$k.TE <-
      expandmerge(meta1$k.TE, meta2$k.TE,
                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
    res$k0 <-
      expandmerge(meta1$k0, meta2$k0,
                  nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
  }
  #
  # Common effect model
  #
  res$TE.common <- c(meta1$TE.common, meta2$TE.common)
  res$seTE.common <- c(meta1$seTE.common, meta2$seTE.common)
  res$statistic.common <- c(meta1$statistic.common, meta2$statistic.common)
  res$pval.common <- c(meta1$pval.common, meta2$pval.common)
  res$lower.common <- c(meta1$lower.common, meta2$lower.common)
  res$upper.common <- c(meta1$upper.common, meta2$upper.common)
  res$zval.common <- c(meta1$zval.common, meta2$zval.common)
  res$text.common <- c(meta1$text.common, meta2$text.common)
  #
  # Random effects model
  #
  res$TE.random <- c(meta1$TE.random, meta2$TE.random)
  res$seTE.random <- c(meta1$seTE.random, meta2$seTE.random)
  res$statistic.random <- c(meta1$statistic.random, meta2$statistic.random)
  res$pval.random <- c(meta1$pval.random, meta2$pval.random)
  res$method.random.ci <-
    replaceNULL(c(meta1$method.random.ci, meta2$method.random.ci), "")
  res$df.random <- c(meta1$df.random, meta2$df.random)
  res$lower.random <- c(meta1$lower.random, meta2$lower.random)
  res$upper.random <- c(meta1$upper.random, meta2$upper.random)
  res$zval.random <- c(meta1$zval.random, meta2$zval.random)
  res$seTE.classic <- c(meta1$seTE.classic, meta2$seTE.classic)
  res$adhoc.hakn.ci <- c(meta1$adhoc.hakn.ci, meta2$adhoc.hakn.ci)
  res$df.hakn <- c(meta1$df.hakn, meta2$df.hakn)
  res$seTE.hakn.ci <- c(meta1$seTE.hakn.ci, meta2$seTE.hakn.ci)
  res$seTE.hakn.adhoc.ci <-
    c(meta1$seTE.hakn.adhoc.ci, meta2$seTE.hakn.adhoc.ci)
  res$df.kero <- c(meta1$df.kero, meta2$df.kero)
  res$seTE.kero <- c(meta1$seTE.kero, meta2$seTE.kero)
  res$text.random <- c(meta1$text.random, meta2$text.random)
  #
  # Prediction interval
  #
  res$method.predict <- c(meta1$method.predict, meta2$method.predict)
  res$seTE.predict <- c(meta1$seTE.predict, meta2$seTE.predict)
  res$df.predict <- c(meta1$df.predict, meta2$df.predict)
  res$lower.predict <- c(meta1$lower.predict, meta2$lower.predict)
  res$upper.predict <- c(meta1$upper.predict, meta2$upper.predict)
  res$adhoc.hakn.pi <- c(meta1$adhoc.hakn.pi, meta2$adhoc.hakn.pi)
  res$seTE.hakn.pi <- c(meta1$seTE.hakn.pi, meta2$seTE.hakn.pi)
  res$seTE.hakn.adhoc.pi <-
    c(meta1$seTE.hakn.adhoc.pi, meta2$seTE.hakn.adhoc.pi)
  res$text.predict <- c(meta1$text.predict, meta2$text.predict)
  #
  res$prediction.subgroup <-
    any(c(meta1$prediction.subgroup, meta2$prediction.subgroup))
  #
  # Heterogeneity statistics
  #
  if (keep.Q) {
    res$Q <- c(meta1$Q, meta2$Q)
    res$df.Q <- c(meta1$df.Q, meta2$df.Q)
    res$pval.Q <- c(meta1$pval.Q, meta2$pval.Q)
  }
  #
  if (keep.I2) {
    res$I2 <- c(meta1$I2, meta2$I2)
    res$lower.I2 <- c(meta1$lower.I2, meta2$upper.I2)
    res$upper.I2 <- c(meta1$upper.I2, meta2$upper.I2)
    #
    res$H <- c(meta1$H, meta2$H)
    res$lower.H <- c(meta1$lower.H, meta2$upper.H)
    res$upper.H <- c(meta1$upper.H, meta2$upper.H)
    #
    res$Rb <- c(meta1$Rb, meta2$Rb)
    res$lower.Rb <- c(meta1$lower.Rb, meta2$upper.Rb)
    res$upper.Rb <- c(meta1$upper.Rb, meta2$upper.Rb)
  }
  #
  # Trim-and-fill method
  #
  res$left <-
    expandmerge(meta1$left, meta2$left,
                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
  res$ma.common <-
    expandmerge(meta1$ma.common, meta2$ma.common,
                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
  res$type <-
    expandmerge(meta1$type, meta2$type,
                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
  res$n.iter.max <-
    expandmerge(meta1$n.iter.max, meta2$n.iter.max,
                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
  res$n.iter <-
    expandmerge(meta1$n.iter, meta2$n.iter,
                nc1 = ncom1, nc2 = ncom2, nr1 = nran1, nr2 = nran2)
  #
  # Subgroup analyses
  #
  replaceNULL(label1.subgroup, "")
  replaceNULL(label2.subgroup, "")
  #
  res$TE.common.w <- c(meta1$TE.common.w, meta2$TE.common.w)
  res$seTE.common.w <- c(meta1$seTE.common.w, meta2$seTE.common.w)
  res$statistic.common.w <-
    c(meta1$statistic.common.w, meta2$statistic.common.w)
  res$pval.common.w <- c(meta1$pval.common.w, meta2$pval.common.w)
  res$lower.common.w <- c(meta1$lower.common.w, meta2$lower.common.w)
  res$upper.common.w <- c(meta1$upper.common.w, meta2$upper.common.w)
  #
  res$w.common.w <- c(meta1$w.common.w, meta2$w.common.w)
  #
  res$tau.w <- c(meta1$tau.w, meta2$tau.w)
  #
  res$Q.w.common <- c(meta1$Q.w.common, meta2$Q.w.common)
  res$pval.Q.w.common <- c(meta1$pval.Q.w.common, meta2$pval.Q.w.common)
  #
  res$Q.b.common <- c(meta1$Q.b.common, meta2$Q.b.common)
  res$df.Q.b.common <- c(meta1$df.Q.b.common, meta2$df.Q.b.common)
  res$pval.Q.b.common <- c(meta1$pval.Q.b.common, meta2$pval.Q.b.common)
  #
  res$TE.random.w <- c(meta1$TE.random.w, meta2$TE.random.w)
  res$seTE.random.w <- c(meta1$seTE.random.w, meta2$seTE.random.w)
  res$statistic.random.w <-
    c(meta1$statistic.random.w, meta2$statistic.random.w)
  res$pval.random.w <- c(meta1$pval.random.w, meta2$pval.random.w)
  res$df.random.w <- c(meta1$df.random.w, meta2$df.random.w)
  res$lower.random.w <- c(meta1$lower.random.w, meta2$lower.random.w)
  res$upper.random.w <- c(meta1$upper.random.w, meta2$upper.random.w)
  res$df.hakn.w <- c(meta1$df.hakn.w, meta2$df.hakn.w)
  res$df.kero.w <- c(meta1$df.kero.w, meta2$df.kero.w)
  #
  res$w.random.w <- c(meta1$w.random.w, meta2$w.random.w)
  #
  res$Q.w.random <- c(meta1$Q.w.random, meta2$Q.w.random)
  res$pval.Q.w.random <- c(meta1$pval.Q.w.random, meta2$pval.Q.w.random)
  #
  res$Q.b.random <- c(meta1$Q.b.random, meta2$Q.b.random)
  res$df.Q.b.random <- c(meta1$df.Q.b.random, meta2$df.Q.b.random)
  res$pval.Q.b.random <- c(meta1$pval.Q.b.random, meta2$pval.Q.b.random)
  #
  res$seTE.predict.w <- c(meta1$seTE.predict.w, meta2$seTE.predict.w)
  res$df.predict.w <- c(meta1$df.predict.w, meta2$df.predict.w)
  res$lower.predict.w <- c(meta1$lower.predict.w, meta2$lower.predict.w)
  res$upper.predict.w <- c(meta1$upper.predict.w, meta2$upper.predict.w)
  
  
  #
  #
  # (7) More settings
  #
  #
  if (is.null(taulabel1))
    taulabel1 <- names(meta1$tau)
  if (is.null(taulabel2))
    taulabel2 <- names(meta2$tau)
  #
  names(meta1$tau) <- names(meta1$tau2) <- taulabel1
  names(meta2$tau) <- names(meta2$tau2) <- taulabel2
  #
  if (!random1 & random2) {
    res$method.tau <- meta2$method.tau
    res$method.tau.ci <- meta2$method.tau.ci
    res$detail.tau <- meta2$detail.tau
    #
    res$tau <- meta2$tau
    names(res$tau) <- taulabel2
    res$lower.tau <- meta2$lower.tau
    res$upper.tau <- meta2$upper.tau
    #
    res$tau2 <- meta2$tau2
    names(res$tau2) <- taulabel2
    res$lower.tau2 <- meta2$lower.tau2
    res$upper.tau2 <- meta2$upper.tau2
    #
    res$se.tau <- meta2$se.tau
    #
    res$tau.preset <- meta2$tau.preset
    #
    res$Q.Cochrane <- meta2$Q.Cochrane
  }
  else if (random1 & random2) {
    if (!inherits(meta1, "metamerge") && meta1$three.level) {
        meta1$tau2 <- sum(meta1$tau2)
        meta1$tau <- sqrt(meta1$tau2)
        meta1$lower.tau <- meta1$upper.tau <-
          meta1$lower.tau2 <- meta1$upper.tau2 <- NA
        meta1$sign.lower.tau <- meta1$sign.upper.tau <- ""
    }
    #
    if (!inherits(meta2, "metamerge") && meta2$three.level) {
        meta2$tau2 <- sum(meta2$tau2)
        meta2$tau <- sqrt(meta2$tau2)
        meta2$lower.tau <- meta2$upper.tau <-
          meta2$lower.tau2 <- meta2$upper.tau2 <- NA
        meta2$sign.lower.tau <- meta2$sign.upper.tau <- ""
    }
    #
    res$method.tau <-
      expandmerge(meta1$method.tau, meta2$method.tau,
                  nr1 = nran1, nr2 = nran2)
    res$method.tau.ci <-
      expandmerge(meta1$method.tau.ci, meta2$method.tau.ci,
                  nr1 = nran1, nr2 = nran2)
    #
    res$tau <- expandmerge(meta1$tau, meta2$tau, nr1 = nran1, nr2 = nran2)
    res$lower.tau <-
      expandmerge(meta1$lower.tau, meta2$lower.tau, nr1 = nran1, nr2 = nran2)
    res$upper.tau <-
      expandmerge(meta1$upper.tau, meta2$upper.tau, nr1 = nran1, nr2 = nran2)
    #
    res$tau2 <- expandmerge(meta1$tau2, meta2$tau2, nr1 = nran1, nr2 = nran2)
    res$lower.tau2 <- expandmerge(meta1$lower.tau2, meta2$lower.tau2,
                                  nr1 = nran1, nr2 = nran2)
    res$upper.tau2 <- expandmerge(meta1$upper.tau2, meta2$upper.tau2,
                                  nr1 = nran1, nr2 = nran2)
    #
    res$se.tau <-
      expandmerge(meta1$se.tau, meta2$se.tau, nr1 = nran1, nr2 = nran2)
    #
    res$detail.tau <-
      expandmerge(meta1$detail.tau, meta2$detail.tau,
                  nr1 = nran1, nr2 = nran2)
    #
    res$tau.preset <-
      expandmerge(meta1$tau.preset, meta2$tau.preset, nr1 = nran1, nr2 = nran2)
    #
    res$Q.Cochrane <-
      expandmerge(meta1$Q.Cochrane, meta2$Q.Cochrane, nr1 = nran1, nr2 = nran2)
  }
  #
  res$sm <- if (meta1$sm != meta2$sm) "" else meta1$sm
  #
  res$common <- any(common)
  res$random <- any(random)
  res$overall <- any(overall)
  res$overall.hetstat <- overall.hetstat
  res$prediction <- prediction
  #
  res$backtransf <- backtransf
  #
  # Three-level model
  #
  if (!inherits(meta1, "metamerge") & is.limit)
    res$three.level <- unique(res$three.level)
  else
    res$three.level <-
      expandmerge(meta1$three.level, meta2$three.level,
                  nr1 = nran1, nr2 = nran2)
  #
  res$rho <-
    expandmerge(meta1$rho, meta2$rho, nr1 = nran1, nr2 = nran2)
  #
  if (!random1 & random2)
    res$cluster <- meta2$cluster
  #
  # Additional arguments from metabin(), metainc(), metaprop()
  #
  res$incr <- expandmerge(meta1$incr, meta2$incr,
                          ncom1, nran1, ncom2, nran2)
  res$method.incr <-
    expandmerge(meta1$method.incr, meta2$method.incr,
                ncom1, nran1, ncom2, nran2)
  res$sparse <-
    expandmerge(meta1$sparse, meta2$sparse,
                ncom1, nran1, ncom2, nran2)
  #
  # Additional arguments from metabin()
  #
  res$allstudies <-
    expandmerge(meta1$allstudies, meta2$allstudies,
                ncom1, nran1, ncom2, nran2)
  res$doublezeros <-
    expandmerge(meta1$doublezeros, meta2$doublezeros,
                ncom1, nran1, ncom2, nran2)
  res$MH.exact <-
    expandmerge(meta1$MH.exact, meta2$MH.exact,
                ncom1, nran1, ncom2, nran2)
  res$RR.Cochrane <-
    expandmerge(meta1$RR.Cochrane, meta2$RR.Cochrane,
                ncom1, nran1, ncom2, nran2)
  res$model.glmm <-
    expandmerge(meta1$model.glmm, meta2$model.glmm,
                ncom1, nran1, ncom2, nran2)
  res$phi <-
    expandmerge(meta1$phi, meta2$phi,
                ncom1, nran1, ncom2, nran2)
  #
  # Additional arguments from metacont()
  #
  res$pooledvar <- expandmerge(meta1$pooledvar, meta2$pooledvar,
                               ncom1, nran1, ncom2, nran2)
  res$method.smd <- expandmerge(meta1$method.smd, meta2$method.smd,
                                ncom1, nran1, ncom2, nran2)
  res$sd.glass <- expandmerge(meta1$sd.glass, meta2$sd.glass,
                              ncom1, nran1, ncom2, nran2)
  res$exact.smd <- expandmerge(meta1$exact.smd, meta2$exact.smd,
                               ncom1, nran1, ncom2, nran2)
  res$method.mean <- expandmerge(meta1$method.mean, meta2$method.mean,
                                 ncom1, nran1, ncom2, nran2)
  res$method.sd <- expandmerge(meta1$method.sd, meta2$method.sd,
                               ncom1, nran1, ncom2, nran2)
  
  
  #
  #
  # (8) Backward compatibility
  #
  #
  res <- backward(res)
  
  
  #
  #
  # (9) Set class
  #
  #
  #  
  class(res) <- c(class(res), "metamerge")
  #
  if (inherits(meta2, "trimfill"))
    class(res) <- c(class(res), "trimfill")
  #
  res$call <- match.call()
  #
  class(res) <- unique(class(res))
  
  
  res
}

hetcalc <- function(TE, seTE,
                    method.tau, method.tau.ci, TE.tau,
                    method.I2, level,
                    subgroup, control,
                    cluster = NULL, rho = 0) {
  
  Ccalc <- function(x) {
    res <- (sum(x, na.rm = TRUE) -
            sum(x^2, na.rm = TRUE) /
            sum(x, na.rm = TRUE))
    #
    res
  }
  
  tau.common <- !missing(subgroup)
  #
  sel.noInf <- !is.infinite(TE) & !is.infinite(seTE)
  TE <- TE[sel.noInf]
  seTE <- seTE[sel.noInf]
  if (!is.null(cluster))
    cluster <- cluster[sel.noInf]
  if (tau.common)
    subgroup <- subgroup[sel.noInf]
  #
  sel.noNA <- !(is.na(TE) | is.na(seTE))
  TE <- TE[sel.noNA]
  seTE <- seTE[sel.noNA]
  if (!is.null(cluster))
    cluster <- cluster[sel.noNA]
  if (tau.common)
    subgroup <- subgroup[sel.noNA]
  #
  noHet <- all(!sel.noNA) || sum(sel.noNA) < 2
  allNA <- all(!sel.noNA)
  #
  three.level <- FALSE
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (!is.null(cluster) &&
      length(unique(cluster)) != length(cluster))
    three.level <- TRUE
  
  
  #
  # Meta-analysis without subgroups
  #
  if (!(is.null(TE.tau)) & method.tau == "DL") {
    #
    # Mantel-Haenszel estimator to calculate Q and tau (like RevMan 5)
    #
    w.common <- 1 / seTE^2
    w.common[is.na(w.common)] <- 0
    #
    Q <- sum(w.common * (TE - TE.tau)^2, na.rm = TRUE)
    df.Q <- sum(!is.na(seTE)) - 1
    pval.Q <- pvalQ(Q, df.Q)
    #
    if (df.Q == 0)
      tau2 <- NA
    else if (round(Q, digits = 18) <= df.Q)
      tau2 <- 0
    else
      tau2 <- (Q - df.Q) / Ccalc(w.common)
    #
    se.tau2 <- lower.tau2 <- upper.tau2 <- NA
    tau <- sqrt(tau2)
    lower.tau <- upper.tau <- NA
    #
    sign.lower.tau <- sign.upper.tau <- method.tau.ci <- ""
    #
    H  <- calcH(Q, df.Q, level)
    I2 <- isquared(Q, df.Q, level)
  }
  else {
    if (noHet) {
      if (allNA)
        Q <- NA
      else
        Q <- 0
      #
      df.Q <- 0
      pval.Q <- pvalQ(Q, df.Q)
      #
      tau2 <- NA
      se.tau2 <- lower.tau2 <- upper.tau2 <- NA
      tau <- sqrt(tau2)
      lower.tau <- upper.tau <- NA
      #
      sign.lower.tau <- sign.upper.tau <- method.tau.ci <- ""
      #
      H  <- calcH(Q, df.Q, level)
      I2 <- isquared(Q, df.Q, level)
    }
    else {
      if (!three.level) {
        mf0 <- runNN(rma.uni,
                     list(yi = TE, sei = seTE, method = method.tau,
                          control = control))
        #
        tau2 <- mf0$tau2
        se.tau2 <- mf0$se.tau2
        # Calculate Cochran's Q
        w <- 1 / seTE^2
        Q <- sum(w * (TE - weighted.mean(TE, w))^2)
      }
      else {
        idx <- seq_along(TE)
        mf0 <-
          runNN(rma.mv,
                list(yi = TE,
                     V = vcalc(vi = seTE^2, cluster = cluster, obs = idx,
                               rho = rho),
                     method = method.tau,
                     random = as.call(~ 1 | cluster / idx),
                     control = control,
                     data = data.frame(cluster, idx)),
                warn = FALSE)
        #
        tau2 <- mf0$sigma2
        se.tau2 <- NA
        #
        Q <- mf0$QE
      }
      #
      tau <- sqrt(tau2)
      #
      df.Q <- mf0$k - mf0$p
      pval.Q <- pvalQ(Q, df.Q)
      #
      if (df.Q < 1)
        method.tau.ci <- ""
      else if (three.level & method.tau.ci != "")
        method.tau.ci <- "PL"
      #
      # Confidence interval for overall meta-analysis
      #
      if (method.tau.ci == "BJ")
        ci0 <-
          confint.rma.uni(
            runNN(rma.uni,
                  list(yi = TE, sei = seTE, weights = 1 / seTE^2,
                       method = "GENQ", control = control)),
            level = 100 * level)
      else if (method.tau.ci == "J")
        ci0 <-
          confint.rma.uni(
            runNN(rma.uni,
                  list(yi = TE, sei = seTE, weights = 1 / seTE,
                       method = "GENQ", control = control)),
            level = 100 * level)
      else if (method.tau.ci == "QP")
        ci0 <- confint.rma.uni(mf0, level = 100 * level)
      else if (method.tau.ci == "PL")
        ci0 <- confint.rma.mv(mf0, level = 100 * level)
      #
      if (method.I2 == "Q") {
        H  <- calcH(Q, df.Q, level)
        I2 <- isquared(Q, df.Q, level)
      }
      else {
        H <- list(TE = sqrt(replaceNULL(mf0$H2)), lower = NA, upper = NA)
        I2 <- list(TE = replaceNULL(mf0$I2) / 100, lower = NA, upper = NA)
        #
        # No confidence interval for H and I2 for three-level model
        #
        if (method.tau.ci %in% c("BJ", "J", "QP")) {
          H$lower <- sqrt(ci0$random["H^2", "ci.lb"])
          H$upper <- sqrt(ci0$random["H^2", "ci.ub"])
          #
          I2$lower <- ci0$random["I^2(%)", "ci.lb"] / 100
          I2$upper <- ci0$random["I^2(%)", "ci.ub"] / 100
        }
      }
    }
  }
  
  
  #
  # Meta-analysis with subgroups
  #
  useFE <- FALSE
  #
  if (tau.common) {
    if (is.numeric(subgroup))
      subgroup <- as.factor(subgroup)
    #
    if (!three.level) {
      if (length(unique(subgroup)) == 1)
        mf1 <-
          runNN(rma.uni,
                list(yi = TE, sei = seTE, method = method.tau,
                     control = control))
      else {
        mf1 <-
          try(runNN(rma.uni,
                    list(yi = TE, sei = seTE, method = method.tau,
                         mods = as.call(~ subgroup), control = control,
                         data = data.frame(TE, seTE, subgroup))),
              silent = TRUE)
        #
        if ("try-error" %in% class(mf1))
          if (grepl(paste0("Number of parameters to be estimated is ",
                           "larger than the number of observations"),
                    mf1)) {
            useFE <- TRUE
            mf1 <-
              runNN(rma.uni,
                    list(yi = TE, sei = seTE, method = "FE",
                         mods = as.call(~ subgroup), control = control,
                         data = data.frame(TE, seTE, subgroup)))
          }
          else
            stop(mf1)
      }
      #
      tau2.resid <- mf1$tau2
      se.tau2.resid <- mf1$se.tau2
    }
    else {
      idx <- seq_along(TE)
      #
      if (length(unique(subgroup)) == 1)
        mf1 <-
          runNN(rma.mv,
                list(yi = TE,
                     V = vcalc(vi = seTE^2, cluster = cluster, obs = idx,
                               rho = rho),
                     method = method.tau,
                     random = as.call(~ 1 | cluster / idx),
                     control = control,
                     data = data.frame(cluster, idx)),
                warn = FALSE)
      else {
        mf1 <-
          try(
            runNN(rma.mv,
                  list(yi = TE,
                       V = vcalc(vi = seTE^2, cluster = cluster, obs = idx,
                                 rho = rho),
                       method = method.tau,
                       random = as.call(~ 1 | cluster / idx),
                       mods = as.call(~ subgroup), control = control,
                       data = data.frame(TE, seTE, subgroup, cluster, idx)),
                  warn = FALSE),
            silent = TRUE)
        #
        if ("try-error" %in% class(mf1))
          if (grepl(paste0("Number of parameters to be estimated is ",
                           "larger than the number of observations"),
                    mf1)) {
            useFE <- TRUE
            mf1 <-
              runNN(rma.mv,
                    list(yi = TE,
                         V = vcalc(vi = seTE^2, cluster = cluster, obs = idx,
                                   rho = rho),
                         method = "FE",
                         random = as.call(~ 1 | cluster / idx),
                         mods = as.call(~ subgroup), control = control,
                         data = data.frame(TE, seTE, subgroup, cluster, idx)),
                    warn = FALSE)
          }
          else
            stop(mf1)
      }
      #
      tau2.resid <- mf1$sigma2
      se.tau2.resid <- NA
    }
    #
    tau.resid <- sqrt(tau2.resid)
    #
    Q.resid <- mf1$QE
    df.Q.resid <- mf1$k - mf1$p
    pval.Q.resid <- pvalQ(Q.resid, df.Q.resid)
    #
    if (method.I2 == "Q") {
      H.resid <- calcH(Q.resid, df.Q.resid, level)
      I2.resid <- isquared(Q.resid, df.Q.resid, level)
    }
    else {
      H.resid <- list(TE = sqrt(replaceNULL(mf1$H2)), lower = NA, upper = NA)
      I2.resid <- list(TE = replaceNULL(mf1$I2) / 100, lower = NA, upper = NA)
    }
    #
    if (df.Q < 2 || useFE)
      method.tau.ci <- ""
    else if (three.level & method.tau.ci != "")
      method.tau.ci <- "PL"
    #
    # Confidence interval for residual heterogeneity
    #
    if (method.tau.ci == "BJ")
      ci1 <-
        confint.rma.uni(
          runNN(rma.uni,
                list(yi = TE, sei = seTE, weights = 1 / seTE^2,
                     method = "GENQ",
                     mods = as.call(~ subgroup), control = control,
                     data = data.frame(TE, seTE, subgroup))),
          level = 100 * level)
    else if (method.tau.ci == "J")
      ci1 <-
        confint.rma.uni(
          runNN(rma.uni,
                list(yi = TE, sei = seTE, weights = 1 / seTE,
                     method = "GENQ",
                     mods = as.call(~ subgroup), control = control,
                     data = data.frame(TE, seTE, subgroup))),
          level = 100 * level)
    else if (method.tau.ci == "QP")
      ci1 <- confint.rma.uni(mf1, level = 100 * level)
    else if (method.tau.ci == "PL")
      ci1 <- confint.rma.mv(mf1, level = 100 * level)
    #
    # No confidence interval for H and I2 for three-level model
    #
    if (method.tau.ci %in% c("BJ", "J", "QP")) {
      H.resid$lower <- sqrt(ci1$random["H^2", "ci.lb"])
      H.resid$upper <- sqrt(ci1$random["H^2", "ci.ub"])
      #
      I2.resid$lower <- ci1$random["I^2(%)", "ci.lb"] / 100
      I2.resid$upper <- ci1$random["I^2(%)", "ci.ub"] / 100
    }
  }
  
  
  #
  # Confidence interval for tau2 and tau
  #
  if (method.tau.ci %in% c("QP", "BJ", "J")) {
    lower.tau2 <- ci0$random["tau^2", "ci.lb"]
    upper.tau2 <- ci0$random["tau^2", "ci.ub"]
    #
    lower.tau <- ci0$random["tau", "ci.lb"]
    upper.tau <- ci0$random["tau", "ci.ub"]
    #
    sign.lower.tau <- ci0$lb.sign
    sign.upper.tau <- ci0$ub.sign
  }
  else if (method.tau.ci == "PL") {
    if (any(names(ci0) == "random")) {      
      lower.tau2 <- c(NA, ci0$random["sigma^2.2", "ci.lb"])
      upper.tau2 <- c(NA, ci0$random["sigma^2.2", "ci.ub"])
      #
      lower.tau <- c(NA, ci0$random["sigma.2", "ci.lb"])
      upper.tau <- c(NA, ci0$random["sigma.2", "ci.ub"])
      #
      sign.lower.tau <- c("", ci0$lb.sign)
      sign.upper.tau <- c("", ci0$ub.sign)
    }
    else {
      lower.tau2 <- c(ci0[[1]]$random["sigma^2.1", "ci.lb"],
                      ci0[[2]]$random["sigma^2.2", "ci.lb"])
      upper.tau2 <- c(ci0[[1]]$random["sigma^2.1", "ci.ub"],
                      ci0[[2]]$random["sigma^2.2", "ci.ub"])
      #
      lower.tau <- c(ci0[[1]]$random["sigma.1", "ci.lb"],
                     ci0[[2]]$random["sigma.2", "ci.lb"])
      upper.tau <- c(ci0[[1]]$random["sigma.1", "ci.ub"],
                     ci0[[2]]$random["sigma.2", "ci.ub"])
      #
      sign.lower.tau <- c(ci0[[1]]$lb.sign, ci0[[2]]$lb.sign)
      sign.upper.tau <- c(ci0[[1]]$ub.sign, ci0[[2]]$ub.sign)
    }
  }
  else {
    lower.tau2 <- upper.tau2 <- lower.tau <- upper.tau <- NA
    sign.lower.tau <- sign.upper.tau <- "" 
  }
  #
  # Confidence interval for tau2.resid and tau.resid
  #
  if (tau.common) {
    if (method.tau.ci %in% c("QP", "BJ", "J")) {
      lower.tau2.resid <- ci1$random["tau^2", "ci.lb"]
      upper.tau2.resid <- ci1$random["tau^2", "ci.ub"]
      #
      lower.tau.resid <- ci1$random["tau", "ci.lb"]
      upper.tau.resid <- ci1$random["tau", "ci.ub"]
      #
      sign.lower.tau.resid <- ci1$lb.sign
      sign.upper.tau.resid <- ci1$ub.sign
    }
    else if (method.tau.ci == "PL") {
      if (any(names(ci0) == "random")) {      
        lower.tau2.resid <- c(NA, ci0$random["sigma^2.2", "ci.lb"])
        upper.tau2.resid <- c(NA, ci0$random["sigma^2.2", "ci.ub"])
        #
        lower.tau.resid <- c(NA, ci0$random["sigma.2", "ci.lb"])
        upper.tau.resid <- c(NA, ci0$random["sigma.2", "ci.ub"])
      }
      else {
        lower.tau2.resid <- c(ci1[[1]]$random["sigma^2.1", "ci.lb"],
                              ci1[[2]]$random["sigma^2.2", "ci.lb"])
        upper.tau2.resid <- c(ci1[[1]]$random["sigma^2.1", "ci.ub"],
                              ci1[[2]]$random["sigma^2.2", "ci.ub"])
        #
        lower.tau.resid <- c(ci1[[1]]$random["sigma.1", "ci.lb"],
                             ci1[[2]]$random["sigma.2", "ci.lb"])
        upper.tau.resid <- c(ci1[[1]]$random["sigma.1", "ci.ub"],
                             ci1[[2]]$random["sigma.2", "ci.ub"])
      }
    }
    else {
      lower.tau2.resid <- upper.tau2.resid <-
        lower.tau.resid <- upper.tau.resid <- NA
    }
  }
  
  
  res <- list(tau2 = tau2,
              se.tau2 = se.tau2,
              lower.tau2 = lower.tau2,
              upper.tau2 = upper.tau2,
              #
              tau = tau,
              lower.tau = lower.tau,
              upper.tau = upper.tau,
              #
              method.tau.ci = method.tau.ci,
              sign.lower.tau = sign.lower.tau,
              sign.upper.tau = sign.upper.tau,
              #
              level = level,
              #
              Q = Q,
              df.Q = df.Q,
              pval.Q = pval.Q,
              #
              H = H$TE,
              lower.H = H$lower,
              upper.H = H$upper,
              #
              I2 = I2$TE,
              lower.I2 = I2$lower,
              upper.I2 = I2$upper,
              #
              tau2.resid = if (tau.common) tau2.resid else NA,
              se.tau2.resid = if (tau.common) se.tau2.resid else NA,
              lower.tau2.resid = if (tau.common) lower.tau2.resid else NA,
              upper.tau2.resid = if (tau.common) upper.tau2.resid else NA,
              #
              tau.resid = if (tau.common) tau.resid else NA,
              lower.tau.resid = if (tau.common) lower.tau.resid else NA,
              upper.tau.resid = if (tau.common) upper.tau.resid else NA,
              #
              Q.resid = if (tau.common) Q.resid else NA,
              df.Q.resid = if (tau.common) df.Q.resid else NA,
              pval.Q.resid = if (tau.common) pval.Q.resid else NA,
              #
              H.resid = if (tau.common) H.resid$TE else NA,
              lower.H.resid = if (tau.common) H.resid$lower else NA,
              upper.H.resid = if (tau.common) H.resid$upper else NA,
              #
              I2.resid = if (tau.common) I2.resid$TE else NA,
              lower.I2.resid = if (tau.common) I2.resid$lower else NA,
              upper.I2.resid = if (tau.common) I2.resid$upper else NA
              )
  #
  res
}

#' Produce traffic light plot of risk of bias assessment
#' 
#' @description
#' Produce traffic light plot of risk of bias assessment
#' 
#' @param object An object of class \code{rob}.
#' @param colour Specify colour scheme for the traffic light plot; see
#'   \code{\link[robvis]{rob_summary}}.
#' @param psize Size of the traffic lights.
#' @param quiet A logical to suppress the display of the traffic light
#'   plot.
#' 
#' @details
#' This is a wrapper function for
#' \code{\link[robvis]{rob_traffic_light}} of R package \bold{robvis}
#' to produce a traffic light plot of risk of bias assessment.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{rob}}, \code{\link{barplot.rob}},
#'   \code{\link[robvis]{rob_traffic_light}}
#' 
#' @keywords hplot
#' 
#' @examples
#' # Use RevMan 5 settings
#' oldset <- settings.meta("RevMan5")
#' 
#' data(caffeine)
#' 
#' ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
#'   data = caffeine, studlab = paste(study, year))
#'
#' # Add risk of bias assessment to meta-analysis
#' ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")
#' 
#' # Print risk of bias assessment
#' rob(ma2)
#'
#' \dontrun{
#' # Traffic light plot (R package 'robvis' must be available)
#' if (requireNamespace("robvis", quietly = TRUE))
#'  traffic_light(rob(ma2))
#' }
#' 
#' # Use previous settings
#' settings.meta(oldset)
#' 
#' @export traffic_light

traffic_light <- function(object,
                          colour = "cochrane",
                          psize = 15,
                          quiet = FALSE) {
  
  chkclass(object, "rob")
  rob <- object
  #
  tool <- attr(object, "tool")
  #
  chklogical(quiet)
  
  
  if (!is_installed_package("robvis", stop = FALSE))
    stop(paste0("Package 'robvis' missing.",
                "\n  ",
                "Please use the following R command for installation:",
                "\n  install.packages(\"robvis\")"),
         call. = FALSE)
  
  
  if (!(tool %in% c("RoB1", "RoB2", "ROBINS-I", "user-defined"))) {
    warning("R function 'rob_traffic_light() in R package robvis not usable ",
            "with risk of bias tool '", tool, "'.",
            call. = FALSE)
    return(invisible(NULL))
  }
  else {
    if (tool %in% c("RoB2", "ROBINS-I") & is.null(rob$Overall)) {
      warning("No traffic light plot created as ",
              "overall risk of bias assessment is missing.",
              call. = FALSE)
      return(invisible(NULL))
    }
    #
    if (is.null(rob$Weight))
      rob$Weight <- 1
    #
    if (tool %in% c("RoB1", "user-defined")) {
      domains <- attr(rob, "domains")
      #
      nam <- names(rob)
      #
      if (tool == "RoB1") {
        nam[nam == "A"] <- "Random.sequence.generation."
        nam[nam == "B"] <- "Allocation.concealment."
        nam[nam == "C"] <- "Blinding.of.participants.and.personnel."
        nam[nam == "D"] <- "Blinding.of.outcome.assessment."
        nam[nam == "E"] <- "Incomplete.outcome.data."
        nam[nam == "F"] <- "Selective.reporting."
        nam[nam == "G"] <- "Other.bias."
        #
        if (length(domains) > 8)
          nam[nam == "H"] <- paste0(domains[8], ".")
        if (length(domains) > 9)
          nam[nam == "I"] <- paste0(domains[9], ".")
        if (length(domains) > 10)
          nam[nam == "J"] <- paste0(domains[10], ".")
      }
      else if (tool == "user-defined") {
        domains[domains == "Overall risk of bias"] <- "Overall"
        #
        nam[nam == "Overall"] <- "O"
        nam[nam %in% LETTERS[1:15]] <- paste0(gsub(" ", ".", domains), ".")
        #
        tool <- "RoB1"
      }
      #
      names(rob) <- nam
    }
    #
    return(robvis::rob_traffic_light(rob, tool = toupper(tool),
                                     colour = colour, psize = psize,
                                     quiet = quiet))
  }
}

setcat <- function(x, labels) {
  if (is.null(labels))
    return(x)
  #
  x <- setchar(x, labels, stop.at.error = FALSE, setNA = TRUE)
  factor(x, levels = labels)
}


setdom <- function(dm, tool, domains, domain.available) {

  n <- length(dm)
  #
  n.names <- length(domains)
  n.domains <- sum(domain.available)
  #
  n.tool <-
    if (tool %in% c("RoB1", "ROBINS-I", "ROBINS-E"))
      7
    else if (tool %in% c("RoB2-cluster", "RoB2-crossover"))
      6
    else if (tool == "RoB2")
      5
  
  if (n.names == n.domains)
    return(domains)
  else if (n.names == n - n.tool |
           n.names == n.domains - n.tool) {
    domains.tool <- dm[seq_len(n.tool)]
    domains.add <- vector(mode = "character")
    for (i in seq_len(n.names))
      domains.add[i] <- domains[i]
    return(c(domains.tool, domains.add))
  }
  else if (n.names == n.domains - n.tool) {
    domains.add <- 
    for (i in seq_len(n.names))
      domains[n.tool + i] <- domains[i]
    return(domains[domain.available])
  }
  else
    stop("Wrong number of domains provided for '", tool,
         "' (must be ", n.domains, n.tool, " or ", n.domains - n.tool, ").",
         call. = FALSE)
  
  invisible(NULL)
}


catleg <- function(x) {
  domains <- attr(x, "domains")
  #
  vars <- names(x)
  vars <- vars[!(vars %in% c("Study", "Weight"))]
  vars[vars == "Overall"] <- "O"
  #
  paste0("(", vars, ") ", domains)
}


catcat <- function(x) {
  cat <- attr(x, "categories")
  #
  vars <- names(x)
  vars <- vars[!(vars %in% c("Study", "Weight"))]
  vars[vars == "Overall"] <- "O"
  #
  unique.list <- function(x) {
    if (length(unique(sapply(x, length))) != 1)
      return(FALSE)
    #
    for (i in seq_len(length(x[[1]])))
      if (length(unique(sapply(x, "[[", i))) != 1)
        return(FALSE)
    #
    return(TRUE)
  }
  if (unique.list(cat)) {
    if (suppressWarnings(all(!is.na(as.numeric(cat[[1]])))))
      return(collapse(cat[[1]], collapse = ", ", quote = ""))
    else
      return(collapse(cat[[1]], collapse = ", "))
  }
  else
    return(paste0("(", vars, ") ", lapply(cat, collapse, collapse = ", ")))
}


catcol <- function(x) {
  col <- attr(x, "col")
  #
  vars <- names(x)
  vars <- vars[!(vars %in% c("Study", "Weight"))]
  vars[vars == "Overall"] <- "O"
  #
  unique.list <- function(x) {
    if (length(unique(sapply(x, length))) != 1)
      return(FALSE)
    #
    for (i in seq_len(length(x[[1]])))
      if (length(unique(sapply(x, "[[", i))) != 1)
        return(FALSE)
    #
    return(TRUE)
  }
  if (unique.list(col)) {
    if (suppressWarnings(all(!is.na(as.numeric(col[[1]])))))
      return(collapse(col[[1]], collapse = ", ", quote = ""))
    else
      return(collapse(col[[1]], collapse = ", "))
  }
  else
    return(paste0("(", vars, ") ",
                  lapply(col, collapse, collapse = ", ", quote = "")))
}


catsymb <- function(x) {
  symb <- attr(x, "symbols")
  #
  vars <- names(x)
  vars <- vars[!(vars %in% c("Study", "Weight"))]
  vars[vars == "Overall"] <- "O"
  #
  unique.list <- function(x) {
    if (length(unique(sapply(x, length))) != 1)
      return(FALSE)
    #
    for (i in seq_len(length(x[[1]])))
      if (length(unique(sapply(x, "[[", i))) != 1)
        return(FALSE)
    #
    return(TRUE)
  }
  if (unique.list(symb)) {
    if (suppressWarnings(all(!is.na(as.numeric(symb[[1]])))))
      return(collapse(symb[[1]], collapse = ", ", quote = ""))
    else
      return(collapse(symb[[1]], collapse = ", "))
  }
  else
    return(paste0("(", vars, ") ",
                  lapply(symb, collapse, collapse = ", ", quote = "")))
}


definecat <- function(avail, x, var, tool, warn) {

  if (!avail)
    return(NULL)
  
  varname <- deparse(substitute(x))
  #
  robins <- tolower(substring(tool, 1, 6)) == "robins"
  rob <- tolower(substring(tool, 1, 3)) == "rob" & !robins
  
  if (is.null(x)) {
    if (tool == "RoB1")
      x <-
        c("Low risk of bias", "Unclear risk of bias", "High risk of bias")
    else if (rob)
      x <-
        c("Low risk of bias", "Some concerns", "High risk of bias")
    else if (robins)
      x <-
        c("Low risk", "Some concerns", "High risk", "Very high risk", "NI")
    else {
      if (warn)
        warning("Alpha-numeric order of available categories used as ",
                "argument '",  varname, "' is missing.",
                call. = FALSE)
      x <- unique(sort(var))
    }
  }
  #
  x
}


definesymb <- function(avail, x, cat, tool, warn) {
  
  if (!avail)
    return(NULL)
  
  varname <- deparse(substitute(x))
  #
  robins <- tolower(substring(tool, 1, 6)) == "robins"
  rob <- tolower(substring(tool, 1, 3)) == "rob" & !robins
  
  if (is.null(x)) {
    if (tool == "user-defined")
      x <- FALSE
    else {
      if (rob)
        x <- c("+", "?", "-")
      else if (robins)
        x <- FALSE
    }
  }
  else {
    chkchar(x, nchar = 1)
    #
    if (length(x) == 1 && is.logical(x)) {
      if (x) {
        if (tool == "user-defined")
          x <- seq_along(cat)
        else if (rob)
          x <- c("+", "?", "-")
        else
          x <- FALSE
      }
    }
    else 
      chklength(x, length(cat),
                text =
                  paste0("Wrong number of RoB symbols ",
                         "(argument '", varname, "')."))
  }
  #
  x
}


definecol <- function(avail, x, cat, tool, warn) {
  
  if (!avail)
    return(NULL)
  
  varname <- deparse(substitute(x))
  #
  robins <- tolower(substring(tool, 1, 6)) == "robins"
  rob <- tolower(substring(tool, 1, 3)) == "rob" & !robins
  
  if (is.null(x)) {
    if (tool == "user-defined")
      x <- seq_len(length(cat))
    else {
      if (rob)
        x <- c("green", "yellow", "red")
      else if (robins)
        x <-  c("green", "yellow", "red", "darkred", "darkgrey")
    }
  }
  else
    chklength(x, length(cat),
              text =
                paste0("Wrong number of RoB colours ",
                       "(argument '", varname, "')."))
  #
  x
}

#' Smoking example
#' 
#' @description
#' Meta-analyses on the effect of smoking on mortality risk.
#' 
#' @details
#' Data have been reconstructed based on the famous Smoking and Health
#' Report to the Surgeon General (Bayne-Jones S et al., 1964). Data
#' sets can be used to evaluate the risk of smoking on overall
#' mortality (dataset \code{smoking}) and lung-cancer deaths (dataset
#' \code{lungcancer}), respectively.
#'
#' The person time is attributed such that the rate ratios are equal
#' to the reported mortality ratios implicitly assuming that the data
#' have arisen from a homogeneous age group; more detailed information
#' by age is not available from the report. Note, the group of
#' "non-smokers" actually consists of all participants except those
#' who are smokers of cigarettes only. Information on real non-smokers
#' is not available from the published Smoking and Health Report.
#' 
#' @name smoking
#' 
#' @aliases smoking lungcancer
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{participants}}\tab total number of participants \cr
#' \bold{\emph{d.smokers}}\tab number of deaths in smokers' group \cr
#' \bold{\emph{py.smokers}}\tab person years at risk in smokers' group
#'   \cr
#' \bold{\emph{d.nonsmokers}}\tab number of deaths in non-smokers'
#'   group \cr
#' \bold{\emph{py.nonsmokers}}\tab person years at risk in
#'   non-smokers' group
#' }
#' 
#' @seealso \code{\link{metainc}}
#' 
#' @source
#' Bayne-Jones S et al. (1964):
#' Smoking and Health: Report of the Advisory Committee to the Surgeon
#' General of the United States.
#' U-23 Department of Health, Education, and Welfare.
#' Public Health Service Publication No. 1103.
#' 
#' @keywords datasets
#' 
#' @examples
#' data(smoking)
#' 
#' ma1 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
#'   data = smoking, studlab = study)
#' print(ma1, digits = 2)
#' 
#' data(lungcancer)
#' 
#' ma2 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
#'   data = lungcancer, studlab = study)
#' print(ma2, digits = 2)

NULL

#' Extract results from meta-analysis object
#' 
#' @description
#' Extract study and meta-analysis results from meta-analysis object
#' which can be stored in an Excel file.
#' 
#' @param x A meta-analysis object of class \code{meta}.
#' @param sortvar An optional vector used to sort the individual
#'   studies (must be of same length as \code{x$TE}).
#' @param study.results A logical indicating whether study results
#'   should be extracted.
#' @param common A logical indicating whether results of common effect
#'   meta-analysis should be extracted.
#' @param random A logical indicating whether results of random
#'   effects meta-analysis should be extracted.
#' @param prediction A logical indicating whether prediction interval
#'   should be extracted.
#' @param overall A logical indicating whether overall summaries
#'   should be extracted. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be extracted.
#' @param subgroup A logical indicating whether subgroup results
#'   should be extracted.
#' @param prediction.subgroup A single logical or logical vector
#'   indicating whether / which prediction intervals should be
#'   extracted for subgroups.
#' @param backtransf A logical indicating whether extracted results
#'   should be back transformed.
#' @param se A logical indicating whether standard errors should be
#'   extracted.
#' @param ci A logical indicating whether confidence / prediction
#'   interval should be extracted.
#' @param statistic A logical indicating whether to extract statistic
#'   of test for overall effect.
#' @param pval A logical indicating whether to extract p-value of test
#'   for overall effect.
#' @param n A logical indicating whether sample sizes should be
#'   extracted (if available).
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard
#'   errors, see \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-statistic for test of overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance, see \code{print.default}.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param big.mark A character used as thousands separator.
#' @param details A logical specifying whether details on statistical
#'   methods should be printed.
#' @param writexl A logical indicating whether an Excel file should be
#'   created (R package \bold{writexl} must be available).
#' @param path A character string specifying the filename of the Excel
#'   file.
#' @param overwrite A logical indicating whether an existing Excel
#'   file should be overwritten.
#' @param \dots Additional arguments passed on to \code{prmatrix}.
#' 
#' @details
#' Extract study and meta-analysis results from meta-analysis object.
#' By default, a data frame with the results is
#' generated. Alternatively, an Excel file is created if argument
#' \code{writexl = TRUE}.
#'
#' The following information is extracted.
#'
#' \tabular{ll}{
#' \bold{Variable} \tab \bold{Content} \cr
#' studlab \tab Study label / descriptor of meta-analysis result \cr
#' estimate \tab (Back transformed) estimate for individual studies /
#'   meta-analysis \cr
#' se \tab Standard error \cr
#' lower \tab Lower (back transformed) confidence / prediction
#'   interval limit \cr
#' upper \tab Upper (back transformed) confidence / prediction
#'   interval limit \cr
#' k \tab Number of studies \cr
#' df \tab Degrees of freedom for confidence / prediction intervals \cr
#' statistic \tab Statistic for test of effect \cr
#' pval \tab P-value for test of effect \cr
#' n \tab Total sample size \cr
#' n.e \tab Sample size in first (experimental) group \cr
#' n.c \tab Sample size in second (control) group
#' }
#'
#' Some variables are only extracted if the corresponding logical
#' argument \code{se}, \code{ci}, \code{statistic} or \code{n} is
#' \code{TRUE}. Furthermore, (group) sample sizes are only extracted
#' if available in the meta-analysis object.
#'
#' The variables \code{estimate}, \code{lower} and \code{upper}
#' contain the back transformed effects and confidence interval
#' limits, e.g., odds ratio (argument \code{sm = "OR"} in
#' \code{\link{metabin}} or \code{\link{metagen}}) or correlation
#' (\code{sm = "ZCOR"} in \code{\link{metacor}} or
#' \code{\link{metagen}}), if argument \code{backtransf} is
#' \code{TRUE}. Otherwise, these variables contain the transformed
#' values, e.g., log odds ratio (\code{sm = "OR"}) or Fisher's Z
#' transformed correlations (\code{sm = "ZCOR"}). See
#' \code{\link{meta-sm}} for available summary measures and
#' \code{\link{meta-transf}} for the corresponding transformations and
#' back transformations.
#' 
#' @return
#' A data frame with additional class 'extract.meta' or an Excel file.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-transf}}, \code{\link{meta-sm}},
#'   \code{\link{as.data.frame.meta}}
#' 
#' @examples
#' ma <- metacor(c(0.85, 0.7, 0.95), c(20, 40, 10))
#' summary(ma)
#' estimates(ma)
#' estimates(ma, backtransf = FALSE)
#' estimates(update(ma, common = FALSE, random = FALSE))
#' estimates(update(ma, prediction = TRUE))
#' estimates(update(ma, prediction = TRUE,
#'   level.ma = 0.99, level.predict = 0.9))
#'
#' \dontrun{
#' # Create Excel file with extracted results
#' # (R package 'writexl' must be available)
#' if (requireNamespace("writexl", quietly = TRUE)) {
#'  fname1 <- tempfile(fileext = ".xlsx")
#'  estimates(ma, path = fname1)
#'  # An existing Excel file is not overwritten but a warning is printed
#'  estimates(ma, path = fname1)
#'  # Overwrite an existing Excel file
#'  estimates(ma, path = fname1, overwrite = TRUE)
#'  # Suppress message on file creation and overwrite existing file
#'  suppressMessages(estimates(ma, path = fname1, overwrite = TRUE))
#' }
#'
#' # Save the extracted results in a text file
#' fname2 <- tempfile(fileext = ".csv")
#' fname2
#' write.csv(estimates(ma), file = fname2, row.names = FALSE)
#' }
#'
#' @rdname estimates
#' @method estimates meta
#' @export

estimates.meta <- function(x,
                           sortvar,
                           #
                           study.results = TRUE,
                           common = x$common,
                           random = x$random,
                           prediction = x$prediction,
                           overall = x$overall,
                           subgroup,
                           prediction.subgroup = x$prediction.subgroup,
                           #
                           se = FALSE,
                           ci = TRUE,
                           statistic = FALSE,
                           pval = FALSE,
                           n = TRUE,
                           #
                           backtransf = x$backtransf,
                           #
                           digits = gs("digits"),
                           digits.se = gs("digits.se"),
                           digits.stat = gs("digits.stat"),
                           digits.pval = gs("digits.pval"),
                           #
                           writexl = !missing(path),
                           path = "estimates.xlsx",
                           overwrite = FALSE,
                           #
                           ...) {
  
  chkclass(x, "meta")
  x <- updateversion(x)
  chksuitable(x, method = "estimates", classes = "metabind", check.mlm = FALSE)
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  chklogical(study.results)
  chklogical(common)
  chklogical(random)
  chklogical(prediction)
  chklogical(overall)
  #
  if (missing(subgroup)) {
    subgroup <- !is.null(x$subgroup)
    n.subgroups <- length(x$subgroup.levels)
  }
  else {
    chklogical(subgroup)
    if (is.null(x$subgroup))
      subgroup <- FALSE
    else
      n.subgroups <- length(x$subgroup.levels)
  }
  #
  missing.prediction.subgroup <- missing(prediction.subgroup)
  #
  if (!subgroup) {
    if (!missing.prediction.subgroup)
      warning("Argument 'prediction.subgroup' only considered for ",
              "meta-analysis with subgroups.",
              call. = FALSE)
    #
    prediction.subgroup <- FALSE
  }
  else {
    prediction.subgroup <- catch("prediction.subgroup", mc, x, sfsp)
    prediction.subgroup <- replaceNULL(prediction.subgroup, FALSE)
    #
    if (length(prediction.subgroup) == 1) {
      if (is.matrix(x$lower.predict.w))
        prediction.subgroup.logical <-
          prediction.subgroup &
          apply(x$lower.predict.w, 1, notallNA) &
          apply(x$upper.predict.w, 1, notallNA)
      else {
        prediction.subgroup.logical <-
          prediction.subgroup &
          notallNA(x$lower.predict.w) &
          notallNA(x$upper.predict.w)
        prediction.subgroup.logical <-
          rep(prediction.subgroup.logical, n.subgroups)
      }
    }
    else {
      chklength(prediction.subgroup, n.subgroups,
                text = paste("Length of argument 'prediction.subgroup' must be",
                             "equal to 1 or number of subgroups."))
    }
    #
    chklogical(prediction.subgroup[1])
    #
    if (length(prediction.subgroup) == 1 & n.subgroups > 1)
      prediction.subgroup <- rep(prediction.subgroup, n.subgroups)
  }
  #
  chklogical(se)
  chklogical(ci)
  chklogical(statistic)
  chklogical(pval)
  chklogical(n)
  #
  chklogical(backtransf)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  #
  chklogical(writexl)
  chkchar(path, length = 1)
  chklogical(overwrite)
  
  
  if (!study.results & !common & !random &
      !prediction & !any(prediction.subgroup)) {
    warning("Nothing to extract.")
    return(NULL)
  }
  #
  avail.n <- n & !is.null(x$n)
  avail.n.e <- n & !is.null(x$n.e)
  avail.n.c <- n & !is.null(x$n.c)
  #
  res.s <- NULL
  res.c <- res.c.w <- NULL
  res.r <- res.r.w <- NULL
  res.p <- res.p.w <- NULL
  #
  n.s <- n.e.s <- n.c.s <- NULL
  n.c <- n.e.c <- n.c.c <- NULL
  n.c.w <- n.e.c.w <- n.c.c.w <- NULL
  n.r <- n.e.r <- n.c.r <- NULL
  n.r.w <- n.e.r.w <- n.c.r.w <- NULL
  n.p <- n.e.p <- n.c.p <- NULL
  n.p.w <- n.e.p.w <- n.c.p.w <- NULL
  #
  if (study.results) {
    error <-
      try(sortvar <-
            catch("sortvar", mc, x, sfsp),
          silent = TRUE)
    if (inherits(error, "try-error")) {
      sortvar <- catch("sortvar", mc, x$data,  NULL)
      if (isCol(x$data, ".subset"))
        sortvar <- sortvar[x$data$.subset]
    }
    sort <- !is.null(sortvar)
    if (sort && (length(sortvar) != length(x$TE)))
      stop("Number of studies in object 'x' and ",
           "argument 'sortvar' have different length.")
    if (!sort)
      sortvar <- seq_along(x$TE)
    #
    o <- order(sortvar)
    #
    res.s <- data.frame(studlab = x$studlab,
                        subgroup = if (subgroup) x$subgroup else "",
                        estimate = x$TE,
                        se = x$seTE,
                        lower = x$lower,
                        upper = x$upper,
                        k = NA,
                        df = if (is.null(x$df)) NA else x$df,
                        statistic = x$statistic,
                        pval = x$pval)[o, ]
    #
    if (avail.n)
      n.s <- x$n[o]
    else if (avail.n.e & avail.n.c)
      n.s <- x$n.e[o] + x$n.c[o]
    #
    if (avail.n.e)
      n.e.s <- x$n.e[o]
    #
    if (avail.n.c)
      n.c.s <- x$n.c[o]
  }
  #
  if (common) {
    if (overall) {
      res.c <- data.frame(studlab = x$text.common,
                          subgroup = "",
                          estimate = x$TE.common,
                          se = x$seTE.common,
                          lower = x$lower.common,
                          upper = x$upper.common,
                          k = x$k,
                          df = NA,
                          statistic = x$statistic.common,
                          pval = x$pval.common)
      #
      if (avail.n)
        n.c <- rep(sum(x$n), length(x$lower.common))
      else if (avail.n.e & avail.n.c)
        n.c <- rep(sum(x$n.e + x$n.c), length(x$lower.common))
      #
      if (avail.n.e)
        n.e.c <- rep(sum(x$n.e), length(x$lower.common))
      #
      if (avail.n.c)
        n.c.c <- rep(sum(x$n.c), length(x$lower.common))
    }
    #
    if (subgroup) {
      rep.c <- rep(length(x$text.common), n.subgroups)
      #
      res.c.w <- data.frame(studlab = rep(x$text.common, n.subgroups),
                            subgroup = rep(x$subgroup.levels, rep.c),
                            estimate = x$TE.common.w,
                            se = as.vector(t(x$seTE.common.w)),
                            lower = as.vector(t(x$lower.common.w)),
                            upper = as.vector(t(x$upper.common.w)),
                            k = rep(x$k.w, rep.c),
                            df = NA,
                            statistic = as.vector(t(x$statistic.common.w)),
                            pval = as.vector(t(x$pval.common.w)),
                            row.names =
                              seq_along(as.vector(t(x$lower.common.w))))
      #
      if (avail.n)
        n.c.w <- rep(x$n.w, rep.c)
      else if (avail.n.e & avail.n.c)
        n.c.w <- rep(x$n.e.w + x$n.c.w, rep.c)
      #
      if (avail.n.e)
        n.e.c.w <- rep(x$n.e.w, rep.c)
      #
      if (avail.n.c)
        n.c.c.w <- rep(x$n.c.w, rep.c)
    }
  }
  #
  if (random) {
    if (overall) {
      res.r <- data.frame(studlab = x$text.random,
                          subgroup = "",
                          estimate = x$TE.random,
                          se = x$seTE.random,
                          lower = x$lower.random,
                          upper = x$upper.random,
                          k = x$k,
                          df = x$df.random,
                          statistic = x$statistic.random,
                          pval = x$pval.random)
      #
      if (avail.n)
        n.r <- rep(sum(x$n), length(x$lower.random))
      else if (avail.n.e & avail.n.c)
        n.r <- rep(sum(x$n.e + x$n.c), length(x$lower.random))
      #
      if (avail.n.e)
        n.e.r <- rep(sum(x$n.e), length(x$lower.random))
      #
      if (avail.n.c)
        n.c.r <- rep(sum(x$n.c), length(x$lower.random))
    }
    #
    if (subgroup) {
      rep.r <- rep(length(x$text.random), n.subgroups)
      #
      res.r.w <- data.frame(studlab = rep(x$text.random, n.subgroups),
                            subgroup = rep(x$subgroup.levels, rep.r),
                            estimate = x$TE.random.w,
                            se = as.vector(t(x$seTE.random.w)),
                            lower = as.vector(t(x$lower.random.w)),
                            upper = as.vector(t(x$upper.random.w)),
                            k = rep(x$k.w, rep.r),
                            df = as.vector(t(x$df.random.w)),
                            statistic = as.vector(t(x$statistic.random.w)),
                            pval = as.vector(t(x$pval.random.w)),
                            row.names =
                              seq_along(as.vector(t(x$lower.random.w))))
      #
      if (avail.n)
        n.r.w <- rep(x$n.w, rep.r)
      else if (avail.n.e & avail.n.c)
        n.r.w <- rep(x$n.e.w + x$n.c.w, rep.r)
      #
      if (avail.n.e)
        n.e.r.w <- rep(x$n.e.w, rep.r)
      #
      if (avail.n.c)
        n.c.r.w <- rep(x$n.c.w, rep.r)
    }
  }
  #
  if (prediction) {
    if (overall) {
      res.p <- data.frame(studlab = x$text.predict,
                          subgroup = "",
                          estimate = NA,
                          se = NA,
                          lower = x$lower.predict,
                          upper = x$upper.predict,
                          k = x$k,
                          df = x$df.predict,
                          statistic = NA,
                          pval = NA)
      #
      if (avail.n)
        n.p <- rep(sum(x$n), length(x$lower.predict))
      else if (avail.n.e & avail.n.c)
        n.p <- rep(sum(x$n.e + x$n.c), length(x$lower.predict))
      #
      if (avail.n.e)
        n.e.p <- rep(sum(x$n.e), length(x$lower.predict))
      #
      if (avail.n.c)
        n.c.p <- rep(sum(x$n.c), length(x$lower.predict))
    }
  }
  #
  if (any(prediction.subgroup)) {
    rep.p <- rep(length(x$text.predict), n.subgroups)
    sel.p <- rep(prediction.subgroup, rep.p)
    #
    res.p.w <-
      data.frame(studlab = rep(x$text.predict, n.subgroups),
                 subgroup = rep(x$subgroup.levels, rep.p),
                 estimate = NA,
                 se = NA,
                 lower = as.vector(t(x$lower.predict.w)),
                 upper = as.vector(t(x$upper.predict.w)),
                 k = rep(x$k.w, rep.p),
                 df = as.vector(t(x$df.predict.w)),
                 statistic = NA,
                 pval = NA,
                 row.names =
                   seq_along(as.vector(t(x$lower.predict.w))))[sel.p, ]
    #
    if (avail.n)
      n.p.w <- rep(x$n.w, rep.p)
    else if (avail.n.e & avail.n.c)
      n.p.w <- rep(x$n.e.w, rep.p) + rep(x$n.c.w, rep.p)
    #
    if (avail.n.e)
      n.e.p.w <- rep(x$n.e.w, rep.p)
    #
    if (avail.n.c)
      n.c.p.w <- rep(x$n.c.w, rep.p)
    #
    n.p.w <- n.p.w[sel.p]
    n.e.p.w <- n.e.p.w[sel.p]
    n.c.p.w <- n.c.p.w[sel.p]
  }
  #
  res <- rbind(res.s, res.c, res.c.w, res.r, res.r.w, res.p, res.p.w)
  #
  if (n) {
    res$n <- c(n.s, n.c, n.c.w, n.r, n.r.w, n.p, n.p.w)
    res$n.e <- c(n.e.s, n.e.c, n.e.c.w, n.e.r, n.e.r.w, n.e.p, n.e.p.w)
    res$n.c <- c(n.c.s, n.c.c, n.c.c.w, n.c.r, n.c.r.w, n.c.p, n.c.p.w)
  }
  #
  if (backtransf) {
    if (inherits(x, "metaprop")) {
      harmonic.mean <- 1 / mean(1 / x$n)
      #
      nback <- c(if (study.results)
                   x$n[o],
                 if (common & overall)
                   rep(harmonic.mean, length(x$lower.common)),
                 if (common & subgroup)
                   rep(x$n.harmonic.mean.w, rep.c),
                 if (random & overall)
                   rep(harmonic.mean, length(x$lower.random)),
                 if (random & subgroup)
                   rep(x$n.harmonic.mean.w, rep.r),
                 if (prediction & overall)
                   rep(harmonic.mean, length(x$lower.predict)),
                 if (any(prediction.subgroup))
                   rep(x$n.harmonic.mean.w, rep.p)[sel.p]
                 )
    }
    else
      nback <- NULL
    #
    if (inherits(x, "metarate")) {
      harmonic.mean <- 1 / mean(1 / x$time)
      #
      timeback <- c(if (study.results)
                      x$time[o],
                    if (common & overall)
                      rep(harmonic.mean, length(x$lower.common)),
                    if (common & subgroup)
                      rep(x$t.harmonic.mean.w, rep.c),
                    if (random & overall)
                      rep(harmonic.mean, length(x$lower.random)),
                    if (random & subgroup)
                      rep(x$t.harmonic.mean.w, rep.r),
                    if (prediction & overall)
                      rep(harmonic.mean, length(x$lower.predict)),
                    if (any(prediction.subgroup))
                      rep(x$t.harmonic.mean.w, rep.p)[sel.p]
                    )
    }
    else
      timeback <- NULL
    #
    res$estimate <- backtransf(res$estimate, x$sm, nback, timeback,
                               x$func.backtransf, x$args.backtransf)
    if (ci)
      res$lower <- backtransf(res$lower, x$sm, nback, timeback,
                              x$func.backtransf, x$args.backtransf)
    if (ci)
      res$upper <- backtransf(res$upper, x$sm, nback, timeback,
                              x$func.backtransf, x$args.backtransf)
    #
    if (inherits(x, "metaprop") && x$pscale != 1) {
      res$estimate <- x$pscale * res$estimate
      if (ci) {
        res$lower <- x$pscale * res$lower
        res$upper <- x$pscale * res$upper
      }
    }
    #
    if (inherits(x, "metarate") && x$irscale != 1) {
      res$estimate <- x$irscale * res$estimate
      if (ci) {
        res$lower <- x$irscale * res$lower
        res$upper <- x$irscale * res$upper
      }
    }
  }
  #
  res$estimate <- round(res$estimate, digits = digits)
  #
  if (se)
    res$se <- round(res$se, digits = digits.se)
  else
    res$se <- NULL
  #
  if (ci)
    res$lower <- round(res$lower, digits = digits)
  else
    res$lower <- NULL
  #
  if (ci)
    res$upper <- round(res$upper, digits = digits)
  else
    res$upper <- NULL
  #
  if (statistic)
    res$statistic <- round(res$statistic, digits = digits)
  else
    res$statistic <- NULL
  #
  if (pval)
    res$pval <- round(res$pval, digits = digits.pval)
  else
    res$pval <- NULL
  #
  if (!any(is.finite(res$df)))
    res$df <- NULL
  else
    res$df <- round(ifelse(is.na(res$df), Inf, res$df), 2)
  #
  if (!subgroup)
    res$subgroup <- NULL
  
  if (writexl) {
    if (!is_installed_package("writexl", stop = FALSE))
      stop(paste0("Package 'writexl' missing.",
                  "\n  ",
                  "Please use the following R command for installation:",
                  "\n  install.packages(\"writexl\")"),
           call. = FALSE)
    #
    if (file.exists(path) & !overwrite)
      warning("File '", path, "' exists. ",
              "Use argument 'overwrite = TRUE' to overwrite file.",
              call. = FALSE)
    else {
      names(res)[names(res) == "estimate"] <-
        smlab(x$sm, backtransf, x$pscale, x$irscale)
      #
      writexl::write_xlsx(res, path = path, col_names = TRUE, ...)
      message(paste0("Extracted information saved in file '", path, "'."))
    }
    #
    return(invisible(NULL))
  }
  
  
  attr(x, ".print.study.results.") <- study.results
  #
  attr(res, "x") <- x
  #
  attr(res, "study.results") <- study.results
  attr(res, "common") <- common
  attr(res, "random") <- random
  attr(res, "prediction") <- prediction
  attr(res, "overall") <- overall
  #
  attr(res, "se") <- se
  attr(res, "ci") <- ci
  attr(res, "statistic") <- statistic
  attr(res, "pval") <- pval
  attr(res, "n") <- n
  #
  attr(res, "backtransf") <- backtransf
  #
  attr(res, "digits") <- digits
  attr(res, "digits.se") <- digits.se
  attr(res, "digits.stat") <- digits.stat
  attr(res, "digits.pval") <- digits.pval
  #
  class(res) <- c("estimates.meta", "estimates", "data.frame")  
  
  res
}


#' @rdname estimates
#' @export estimates

estimates <- function(x, ...) 
  UseMethod("estimates")


#' @rdname estimates
#' @method print estimates.meta
#' @export

print.estimates.meta <- function(x,
                                 digits.tau = gs("digits.tau"),
                                 text.tau2 = gs("text.tau2"),
                                 text.tau = gs("text.tau"),
                                 big.mark = gs("big.mark"),
                                 details = TRUE, ...) {
  
  chkclass(x, "estimates.meta")
  
  meta <- attr(x, "x")
  #
  digits <- attr(x, "digits")
  digits.se <- attr(x, "digits.se")
  digits.stat <- attr(x, "digits.stat")
  digits.pval <- attr(x, "digits.pval")
  #
  chknumeric(digits.tau, min = 0, length = 1)
  
  common <- attr(x, "common")
  random <- attr(x, "random")
  prediction <- attr(x, "prediction")
  overall <- attr(x, "overall")
  #
  backtransf <- attr(x, "backtransf")
  sm.lab <- smlab(meta$sm, backtransf, meta$pscale, meta$irscale)
  #
  ci.lab <- paste0(round(100 * meta$level, 1), "%-CI")
  #
  se <- attr(x, "se")
  ci <- attr(x, "ci")
  statistic <- attr(x, "statistic")
  pval <- attr(x, "pval")
  
  res <- x[, names(x) != "studlab"]
  #
  res$estimate <- round(res$estimate, digits = digits)
  if (ci)
    res$lower <- round(res$lower, digits = digits)
  if (ci)
    res$upper <- round(res$upper, digits = digits)
  #
  res$estimate <- formatN(res$estimate, digits, "", big.mark = big.mark)
  #
  if (se)
    res$se <- formatN(res$se, digits, "", big.mark = big.mark)
  #
  if (ci)
    res$lower <-
      ifelse(is.na(res$lower) & is.na(res$upper), "",
             formatCI(
               formatN(res$lower, digits, "", big.mark = big.mark),
               formatN(res$upper, digits, "", big.mark = big.mark)))
  #
  if (statistic)
    res$statistic <-
      formatN(res$statistic, digits.stat, "", big.mark = big.mark)
  if (pval)
    res$pval <- formatPT(res$pval, digits = digits.pval,
                         lab.NA = "", big.mark = big.mark)
  #
  if (all(is.na(res$k)))
    res$k <- NULL
  else
    res$k <- formatN(res$k, 0, "", big.mark = big.mark)
  #
  if (!is.null(res$df))
    res$df <- ifelse(is.finite(res$df), res$df, "")
  #
  res$upper <- NULL
  #
  res <- as.matrix(res)
  #
  rownames(res) <- x$studlab
  #
  cn <- colnames(res)
  cn[cn == "estimate"] <- sm.lab
  cn[cn == "lower"] <- ci.lab
  colnames(res) <- cn
  
  crtitle(meta)
  #
  prmatrix(res, quote = FALSE, right = TRUE, ...)
  #
  if (details) {
    catmeth(meta,
            common, random, prediction, overall, random | prediction,
            #
            func.transf = NULL,
            backtransf = backtransf, func.backtransf = NULL,
            #
            big.mark = gs("big.mark"), digits = digits, digits.tau = digits.tau,
            text.tau = gs("text.tau"), text.tau2 = gs("text.tau2"))
    #
    if ((common | random) && meta$level != meta$level.ma)
      cat(paste0("- ", round(100 * meta$level.ma, 1),
                 "%-CI calculated for meta-analysis results\n"))
    if (prediction && meta$level != meta$level.predict)
      if (meta$level != meta$level.ma)
        cat(paste0("- ", round(100 * meta$level.predict, 1),
                   "%-PI calculated\n"))
  }
  
  invisible(NULL)
}

#' meta: Brief overview of methods and general hints
#' 
#' @description
#' R package \bold{meta} is a user-friendly general package providing
#' standard methods for meta-analysis and supporting Schwarzer et
#' al. (2015),
#' \url{https://link.springer.com/book/10.1007/978-3-319-21416-0}.
#' 
#' @details
#' R package \bold{meta} (Schwarzer, 2007; Balduzzi et al., 2019)
#' provides the following statistical methods for meta-analysis.
#' \enumerate{
#' \item Common effect (also called fixed effect) and random effects model:
#' \itemize{
#'  \item Meta-analysis of continuous outcome data (\code{\link{metacont}})
#'  \item Meta-analysis of binary outcome data (\code{\link{metabin}})
#'  \item Meta-analysis of incidence rates (\code{\link{metainc}})
#'  \item Generic inverse variance meta-analysis (\code{\link{metagen}})
#'  \item Meta-analysis of single correlations (\code{\link{metacor}})
#'  \item Meta-analysis of single means (\code{\link{metamean}})
#'  \item Meta-analysis of single proportions (\code{\link{metaprop}})
#'  \item Meta-analysis of single incidence rates (\code{\link{metarate}})
#' }
#' \item Several plots for meta-analysis:
#' \itemize{
#'  \item Forest plot (\code{\link{forest.meta}}, \code{\link{forest.metabind}})
#'  \item Funnel plot (\code{\link{funnel.meta}})
#'  \item Galbraith plot / radial plot (\code{\link{radial.meta}})
#'  \item L'Abbe plot for meta-analysis with binary outcome data
#'   (\code{\link{labbe.metabin}}, \code{\link{labbe.default}})
#'  \item Baujat plot to explore heterogeneity in meta-analysis
#'   (\code{\link{baujat.meta}})
#'  \item Bubble plot to display the result of a meta-regression
#'   (\code{\link{bubble.metareg}})
#' }
#' \item Three-level meta-analysis model (Van den Noortgate et al.,
#'   2013)
#' \item Generalised linear mixed models (GLMMs) for binary and count
#'   data (Stijnen et al., 2010) (\code{\link{metabin}},
#'   \code{\link{metainc}}, \code{\link{metaprop}}, and
#'   \code{\link{metarate}})
#' \item Logistic regression with penalised likelihood for rare binary data
#'   (Evrenoglou et al., 2022) (\code{\link{metabin}},
#' \item Various estimators for the between-study variance
#'  \eqn{\tau^2} in a random effects model (Veroniki et al., 2016);
#'  see description of argument \code{method.tau} below
#' \item Two methods to estimate the I-squared statistic
#'   (Higgins and Thompson, 2002); see description of argument
#'   \code{method.I2} below
#' \item Hartung-Knapp method for random effects meta-analysis
#'  (Hartung & Knapp, 2001a,b), see description of arguments
#'  \code{method.random.ci} and \code{adhoc.hakn.ci} below
#' \item Kenward-Roger method for random effects meta-analysis
#'  (Partlett and Riley, 2017), see description of arguments
#'  \code{method.random.ci} and \code{method.predict} below
#' \item Inverse variance heterogeneity method (Doi et al., 2015),
#'  see description of argument \code{method.common.ci} below
#' \item Prediction interval for the treatment effect of a new study
#'  (Veroniki et al., 2019; Higgins et al., 2009; Partlett and Riley, 2017;
#'  Nagashima et al., 2019), see description of argument \code{method.predict}
#'  below
#' \item Expected proportion of comparable studies with clinically important
#'  benefit or harm derived from the prediction distribution
#'  (Siemens et al., 2025)
#' \item Statistical tests for funnel plot asymmetry
#'  (\code{\link{metabias.meta}}, \code{\link{metabias.rm5}}) and
#'  trim-and-fill method (\code{\link{trimfill.meta}},
#'  \code{\link{trimfill.default}}) to evaluate bias in meta-analysis
#' \item Meta-regression (\code{\link{metareg}})
#' \item Cumulative meta-analysis (\code{\link{metacum}}) and
#'   leave-one-out meta-analysis (\code{\link{metainf}})
#' \item Import data from RevMan Web (\code{\link{read.cdir}}), RevMan
#'   5 (\code{\link{read.rm5}}), see also \code{\link{metacr}} to
#'   conduct meta-analysis for a single comparison and outcome from a
#'   Cochrane review
#' }
#'
#' R package \bold{meta} provides two vignettes:
#' \itemize{
#'  \item \code{vignette("meta-workflow")} with an overview of main
#'    functions,
#'  \item \code{vignette("meta-tutorial")} with up-to-date commands for
#'    Balduzzi et al. (2019).
#' }
#' 
#' Additional statistical meta-analysis methods are provided by add-on
#' R packages:
#' \itemize{
#' \item Frequentist methods for network meta-analysis (R package
#'   \bold{netmeta})
#' \item Statistical methods for sensitivity analysis in meta-analysis
#'   (R package \bold{metasens})
#' \item Statistical methods for meta-analysis of diagnostic accuracy
#'   studies with several cutpoints (R package \bold{diagmeta})
#' }
#' 
#' In the following, more details on available and default statistical
#' meta-analysis methods are provided and R function
#' \code{\link{settings.meta}} is briefly described which can be used
#' to change the default settings. Additional information on
#' meta-analysis objects and available summary measures can be found
#' on the help pages \code{\link{meta-object}} and
#' \code{\link{meta-sm}}.
#' 
#' \subsection{Estimation of between-study variance}{
#' 
#' The following methods are available in all meta-analysis functions
#' to estimate the between-study variance \eqn{\tau^2}.
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Method} \cr 
#' \code{method.tau = "REML"}
#'  \tab Restricted maximum-likelihood estimator (Viechtbauer, 2005) \cr
#' \tab (default) \cr
#' \code{method.tau = "PM"}
#'  \tab Paule-Mandel estimator (Paule and Mandel, 1982) \cr
#' \code{method.tau = "DL"}
#'  \tab DerSimonian-Laird estimator (DerSimonian and Laird, 1986) \cr
#' \code{method.tau = "ML"}
#'  \tab Maximum-likelihood estimator (Viechtbauer, 2005) \cr
#' \code{method.tau = "HS"}
#'  \tab Hunter-Schmidt estimator (Hunter and Schmidt, 2015) \cr
#' \code{method.tau = "SJ"}
#'  \tab Sidik-Jonkman estimator (Sidik and Jonkman, 2005) \cr
#' \code{method.tau = "HE"}
#'  \tab Hedges estimator (Hedges and Olkin, 1985) \cr
#' \code{method.tau = "EB"}
#'  \tab Empirical Bayes estimator (Morris, 1983)
#' }
#'
#' For GLMMs, only the maximum-likelihood method is available.
#' 
#' Historically, the DerSimonian-Laird method was the de facto
#' standard to estimate the between-study variance \eqn{\tau^2} and is
#' the default in some software packages including Review Manager 5
#' (RevMan 5) and R package \bold{meta}, version 4 and below. However,
#' its role has been challenged and especially the REML and
#' Paule-Mandel estimators have been recommended (Veroniki et al.,
#' 2016; Langan et al., 2019). Accordingly, the currenct default in R
#' package \bold{meta} is the REML estimator.
#' 
#' The following R command could be used to employ the Paule-Mandel
#' instead of the REML estimator in all meta-analyses of the current R
#' session:
#' \itemize{
#' \item \code{settings.meta(method.tau = "PM")}
#' }
#' 
#' Other estimators for \eqn{\tau^2} could be selected in a similar
#' way.
#'
#' Note, for binary outcomes, two variants of the DerSimonian-Laird
#' estimator are available if the Mantel-Haenszel method is used for
#' pooling. If argument \code{Q.Cochrane = TRUE} (default), the
#' heterogeneity statistic Q is based on the Mantel-Haenszel instead
#' of the inverse variance estimator under the common effect
#' model. This is the estimator for \eqn{\tau^2} implemented in RevMan
#' 5.
#' }
#' 
#' 
#' \subsection{Estimation of I-squared statistic}{
#' 
#' The following methods are available in all meta-analysis functions
#' to estimate the I-squared statistic (Higgins and Thompson, 2002).
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Method} \cr 
#' \code{method.I2 = "Q"}
#'  \tab Based on heterogeneity statistic Q (default) \cr
#' \code{method.I2 = "tau2"}
#'  \tab Based on between-study variance \eqn{\tau^2}
#' }
#'
#' Using \code{method.I2 = "Q"} (Higgins and Thompson, 2002, section 3.3), the
#' value of I\eqn{^2} does not change if the estimate of \eqn{\tau^2} changes.
#' Furthermore, the value of I\eqn{^2} and the test of heterogeneity based on
#' the Q statistic are in agreement. R package \bold{metafor} uses the second
#' method (\code{method.I2 = "tau2"}) which is described in Higgins and Thompson
#' (2002), section 3.2. This method is more general in the way that the value
#' of I\eqn{^2} changes with the estimate of \eqn{\tau^2}.
#' }
#' 
#' \subsection{Confidence interval for common effect estimate}{
#'
#' The following methods are available in all meta-analysis functions
#' to calculate a confidence interval for the common effect estimate.
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Method} \cr
#' \code{method.common.ci = "classic"} \tab Based on standard normal
#'   quantile \cr
#' \code{method.common.ci = "IVhet"} \tab Method by Doi et al. (2015) going
#'   back to Henmi and Copas (2010) \cr
#' }
#' 
#' The inverse variance heterogeneity method (IVhet) uses the weights under the
#' random effects model to calculate the confidence interval. This method is
#' only available if argument \code{method = "Inverse"}.
#' }
#' 
#' \subsection{Confidence interval for random effects estimate}{
#'
#' The following methods are available in all meta-analysis functions
#' to calculate a confidence interval for the random effects estimate.
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Method} \cr
#' \code{method.random.ci = "classic"} \tab Based on standard normal
#'   quantile \cr
#' \tab (DerSimonian and Laird, 1986) (default) \cr
#' \code{method.random.ci = "HK"} \tab Method by Hartung and Knapp
#'   (2001a/b) \cr
#' \code{method.random.ci = "KR"} \tab Kenward-Roger method (Partlett and
#'   Riley, 2017)
#' }
#' 
#' DerSimonian and Laird (1986) introduced the classic random effects
#' model using a quantile of the standard normal distribution to
#' calculate a confidence interval for the random effects
#' estimate. This method implicitly assumes that the weights in the
#' random effects meta-analysis are not estimated but
#' given. Particularly, the uncertainty in the estimation of the
#' between-study variance \eqn{\tau^2} is ignored.
#'
#' Hartung and Knapp (2001a,b) proposed an alternative method for
#' random effects meta-analysis based on a refined variance estimator
#' for the treatment estimate and a quantile of a
#' \emph{t}-distribution with \emph{k-1} degrees of freedom where
#' \emph{k} corresponds to the number of studies in the
#' meta-analysis.
#'
#' The Kenward-Roger method is only available for the REML estimator
#' (\code{method.tau = "REML"}) of the between-study variance
#' \eqn{\tau^2} (Partlett and Riley, 2017). This method is based on an
#' adjusted variance estimate for the random effects
#' estimate. Furthermore, a quantile of a \emph{t}-distribution with
#' adequately modified degrees of freedom is used to calculate the
#' confidence interval.
#'
#' For GLMMs and three-level models, the Kenward-Roger method is not
#' available, but a method similar to Knapp and Hartung (2003) is used
#' if \code{method.random.ci = "HK"}. For this method, the variance
#' estimator is not modified, however, a quantile of a
#' \emph{t}-distribution with \emph{k-1} degrees of freedom is used;
#' see description of argument \code{test} in
#' \code{\link[metafor]{rma.glmm}} and \code{\link[metafor]{rma.mv}}.
#' 
#' Simulation studies (Hartung and Knapp, 2001a,b; IntHout et al.,
#' 2014; Langan et al., 2019) show improved coverage probabilities of
#' the Hartung-Knapp method compared to the classic random effects
#' method. However, in rare settings with very homogeneous treatment
#' estimates, the Hartung-Knapp variance estimate can be arbitrarily
#' small resulting in a very narrow confidence interval (Knapp and
#' Hartung, 2003; Wiksten et al., 2016). In such cases, an \emph{ad
#' hoc} variance correction has been proposed by utilising the
#' variance estimate from the classic random effects model with the
#' Hartung-Knapp method (Knapp and Hartung, 2003; IQWiQ, 2022). An
#' alternative \emph{ad hoc} approach is to use the confidence
#' interval of the classic common or random effects meta-analysis if
#' it is wider than the interval from the Hartung-Knapp method
#' (Wiksten et al., 2016; Jackson et al., 2017).
#'
#' Argument \code{adhoc.hakn.ci} can be used to choose the \emph{ad
#' hoc} correction for the Hartung-Knapp (HK) method:
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{\emph{Ad hoc} method} \cr
#' \code{adhoc.hakn.ci = ""} \tab no \emph{ad hoc} correction (default)
#'   \cr
#' \code{adhoc.hakn.ci = "se"} \tab use variance correction if HK standard
#'  error is smaller \cr
#'  \tab than standard error from classic random effects
#'  \cr
#'  \tab meta-analysis (Knapp and Hartung, 2003) \cr
#' \code{adhoc.hakn.ci = "IQWiG6"} \tab use variance correction if HK
#'  confidence interval \cr
#'  \tab is narrower than CI from classic random effects model \cr
#'  \tab with DerSimonian-Laird estimator (IQWiG, 2022) \cr
#' \code{adhoc.hakn.ci = "ci"} \tab use wider confidence interval of
#'  classic random effects \cr
#'  \tab and HK meta-analysis \cr
#'  \tab (Hybrid method 2 in Jackson et al., 2017)
#' }
#' 
#' For GLMMs and three-level models, the \emph{ad hoc} variance
#' corrections are not available.
#' }
#' 
#' \subsection{Prediction interval}{
#'
#' The following methods are available in all meta-analysis functions
#' to calculate a prediction interval for the treatment effect in a
#' single new study.
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Method} \cr
#' \code{method.predict = "V"} \tab Based on \emph{t}-distribution
#'   with \emph{k-1} degrees of freedom \cr
#'   \tab (Veroniki et al., 2019) (default) \cr
#' \code{method.predict = "HTS"} \tab Based on \emph{t}-distribution
#'   with \emph{k-2} degrees of freedom \cr
#'   \tab (Higgins et al., 2009) \cr
#' \code{method.predict = "HK"} \tab Based on Hartung-Knapp standard error
#'   and \cr
#'   \tab \emph{t}-distribution with \emph{k-1} degrees of freedom \cr
#' \code{method.predict = "HK-PR"} \tab Based on Hartung-Knapp standard error
#'   and \cr
#'   \tab \emph{t}-distribution with \emph{k-2} degrees of freedom \cr
#'   \tab (Partlett and Riley, 2017) \cr
#' \code{method.predict = "KR"} \tab Based on Kenward-Roger standard error
#'   and \cr
#'   \tab \emph{t}-distribution with approximate Kenward-Roger \cr
#'   \tab degrees of freedom \cr
#' \code{method.predict = "KR-PR"} \tab Based on Kenward-Roger standard error
#'   and \cr
#'   \tab \emph{t}-distribution with approximate Kenward-Roger \cr
#'   \tab degrees of freedom minus 1 (Partlett and Riley, 2017) \cr
#' \code{method.predict = "NNF"} \tab Bootstrap approach (Nagashima et
#'   al., 2019) \cr
#' \code{method.predict = "S"} \tab Based on standard normal quantile
#'   (Skipka, 2006)
#' }
#'
#' By default (\code{method.predict = "V"}), the prediction interval
#' is based on a \emph{t}-distribution with \emph{k-1} degrees of
#' freedom where \emph{k} corresponds to the number of studies in the
#' meta-analysis (Veroniki et al., 2019). The method by Higgins et al., (2009),
#' which is based on a \emph{t}-distribution with \emph{k-2} degrees of freedom,
#' has been the default in R package \bold{meta}, version 7.0-0 or lower.
#' 
#' The Hartung-Knapp prediction intervals are also based on a
#' \emph{t}-distribution, however, use a different standard error.
#'
#' The Kenward-Roger method is only available for the REML estimator
#' (\code{method.tau = "REML"}) of the between-study variance
#' \eqn{\tau^2} (Partlett and Riley, 2017). This method is based on an
#' adjusted variance estimate for the random effects
#' estimate. Furthermore, a quantile of a \emph{t}-distribution with
#' adequately modified degrees of freedom is used to calculate the prediction
#' interval.
#'
#' The bootstrap approach is only available if R package \bold{pimeta}
#' is installed (Nagashima et al., 2019). Internally, the
#' \code{\link[pimeta]{pima}} function is called with argument
#' \code{method = "boot"}. Argument \code{seed.predict} can be used to
#' get a reproducible bootstrap prediction interval and argument
#' \code{seed.predict.subgroup} for reproducible bootstrap prediction
#' intervals in subgroups.
#' 
#' The method of Skipka (2006) ignores the uncertainty in the
#' estimation of the between-study variance \eqn{\tau^2} and thus has
#' too narrow limits for meta-analyses with a small number of studies.
#'
#' For GLMMs and three-level models, only the methods by Veroniki et al.
#' (2019), Higgins et al. (2009) and Skipka (2006) are available. Argument
#' \code{method.predict = "V"} in R package \bold{meta} gives the same
#' prediction intervals as R functions \code{\link[metafor]{rma.glmm}} or
#' \code{\link[metafor]{rma.mv}} with argument \code{test = "t"}.
#'
#' Note, in R package \bold{meta}, version 7.0-0 or lower, the methods
#' \code{method.predict = "HK-PR"} and \code{method.predict = "KR-PR"} have
#' been available as \code{method.predict = "HK"} and
#' \code{method.predict = "KR"}.
#'
#' Argument \code{adhoc.hakn.pi} can be used to choose the \emph{ad
#' hoc} correction for the Hartung-Knapp method:
#' 
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{\emph{Ad hoc} method} \cr
#' \code{adhoc.hakn.pi = ""} \tab no \emph{ad hoc} correction (default)
#'   \cr
#' \code{adhoc.hakn.pi = "se"} \tab use variance correction if HK
#'  standard error is smaller
#' }
#' }
#'
#' \subsection{Confidence interval for the between-study variance}{
#'
#' The following methods are available in all meta-analysis functions
#' to calculate a confidence interval for \eqn{\tau^2} and \eqn{\tau}.
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Method} \cr 
#' \code{method.tau.ci = "J"} \tab Method by Jackson (2013) \cr
#' \code{method.tau.ci = "BJ"} \tab Method by Biggerstaff and Jackson (2008) \cr
#' \code{method.tau.ci = "QP"} \tab Q-Profile method (Viechtbauer, 2007) \cr
#' \code{method.tau.ci = "PL"} \tab Profile-Likelihood method for three-level \cr
#'  \tab meta-analysis model (Van den Noortgate et al., 2013) \cr
#' \code{method.tau.ci = ""} \tab No confidence interval
#' }
#' The first three methods have been recommended by Veroniki et
#' al. (2016). By default, the Jackson method is used for the
#' DerSimonian-Laird estimator of \eqn{\tau^2} and the Q-profile
#' method for all other estimators of \eqn{\tau^2}.
#'
#' The Profile-Likelihood method is the only method available for the
#' three-level meta-analysis model.
#' 
#' For GLMMs, no confidence intervals for \eqn{\tau^2} and \eqn{\tau}
#' are calculated.
#' }
#'
#' \subsection{Change default settings for R session}{
#'
#' R function \code{\link{settings.meta}} can be used to change the
#' previously described and several other default settings for the
#' current R session.
#' 
#' Some pre-defined general settings are available:
#' \itemize{
#' \item \code{settings.meta("RevMan5")}
#' \item \code{settings.meta("JAMA")}
#' \item \code{settings.meta("BMJ")}
#' \item \code{settings.meta("IQWiG5")}
#' \item \code{settings.meta("IQWiG6")}
#' \item \code{settings.meta("geneexpr")}
#' }
#' 
#' The first command can be used to reproduce meta-analyses from
#' Cochrane reviews conducted with \emph{Review Manager 5} (RevMan 5)
#' and specifies to use a RevMan 5 layout in forest plots.
#'
#' The second command can be used to generate forest plots following
#' instructions for authors of the \emph{Journal of the American
#' Medical Association}. Study labels according to JAMA guidelines can be
#' generated using \code{\link{labels.meta}}.
#'
#' The third command can be used to generate forest plots in the current layout
#' of the \emph{British Medical Journal}.
#'
#' The next two commands implement the recommendations of the
#' Institute for Quality and Efficiency in Health Care (IQWiG),
#' Germany accordinging to General Methods 5 and 6, respectively
#' (\url{https://www.iqwig.de/en/about-us/methods/methods-paper/}).
#'
#' The last setting can be used to print p-values in scientific
#' notation and to suppress the calculation of confidence intervals
#' for the between-study variance.
#'
#' See \code{\link{settings.meta}} for more details on these
#' pre-defined general settings.
#' 
#' In addition, \code{\link{settings.meta}} can be used to define
#' individual settings for the current R session. For example, the
#' following R command specifies the use of Hartung-Knapp and
#' Paule-Mandel method, and the printing of prediction intervals for
#' any meta-analysis generated after execution of this command:
#' \itemize{
#' \item \code{settings.meta(method.random.ci = "HK", method.tau =
#'   "PM", prediction = TRUE)}
#' }
#' }
#'
#' \subsection{Data sets}{
#' The following data sets are available in R package \bold{meta}.
#'
#' \tabular{ll}{
#' \bold{Data set} \tab \bold{Description} \cr
#' \code{\link{Fleiss1993bin}} \tab Aspirin after myocardial infarction \cr
#' \code{\link{Fleiss1993cont}} \tab Mental health treatment on medical utilisation\cr
#' \code{\link{Olkin1995}} \tab Thrombolytic therapy after acute myocardial infarction \cr
#' \code{\link{Pagliaro1992}} \tab Prevention of first bleeding in cirrhosis \cr
#' \code{\link{amlodipine}} \tab Amlodipine for work capacity \cr
#' \code{\link{caffeine}} \tab Caffeine for daytime drowsiness (Cochrane Practice review) \cr
#' \code{\link{cisapride}} \tab Cisapride in non-ulcer dispepsia \cr
#' \code{\link{lungcancer}} \tab Smoking example \cr
#' \code{\link{smoking}} \tab Smoking example \cr
#' \code{\link{woodyplants}} \tab Elevated CO$_2$ and total biomass of woody plants
#' }
#' 
#' \bold{R} packages \bold{metabook} and \bold{metadat} have a large collection
#'  of meta-analysis data sets.
#' }
#'
#' @note
#' Balduzzi et al. (2019) is the preferred citation in publications
#' for \bold{meta}. Type \code{citation("meta")} for a BibTeX entry of
#' this publication.
#' 
#' Type \code{help(package = "meta")} for a listing of all R functions
#' and datasets available in \bold{meta}. For example, results of
#' several meta-analyses can be combined with \code{\link{metabind}}
#' which is useful to generate a forest plot with results of several
#' subgroup analyses.
#' 
#' R package \bold{meta} imports R functions from \bold{metafor}
#' (Viechtbauer, 2010) to
#' \itemize{
#' \item estimate the between-study variance \eqn{\tau^2},
#' \item conduct meta-regression,
#' \item estimate three-level models,
#' \item estimate generalised linear mixed models.
#' }
#' 
#' To report problems and bugs
#' \itemize{
#' \item type \code{bug.report(package = "meta")} if you do not use
#'   RStudio,
#' \item send an email to Guido Schwarzer
#'   \email{guido.schwarzer@@uniklinik-freiburg.de} if you use RStudio.
#' }
#' 
#' The development version of \bold{meta} is available on GitHub
#' \url{https://github.com/guido-s/meta/}.
#' 
#' @name meta-package
#' 
#' @aliases meta-package meta
#'
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @references
#' Balduzzi S, Rücker G, Schwarzer G (2019):
#' How to perform a meta-analysis with R: a practical tutorial.
#' \emph{Evidence-Based Mental Health},
#' \bold{22}, 153--160
#' 
#' Biggerstaff BJ, Jackson D (2008):
#' The exact distribution of Cochran’s heterogeneity statistic in
#' one-way random effects meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{27}, 6093--110
#'
#' DerSimonian R & Laird N (1986):
#' Meta-analysis in clinical trials.
#' \emph{Controlled Clinical Trials},
#' \bold{7}, 177--88
#' 
#' Doi SAR, Barendregt JJ, Khan S, Thalib L, Williams GM (2015):
#' Advances in the meta-analysis of heterogeneous clinical trials I:
#' The inverse variance heterogeneity model.
#' \emph{Contemporary Clinical Trials},
#' \bold{45}, 130--8
#' 
#' Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A. (2022):
#' Network meta-analysis of rare events using penalized likelihood regression.
#' \emph{Statistics in Medicine},
#' \bold{41}, 5203--19
#'
#' Hartung J, Knapp G (2001a):
#' On tests of the overall treatment effect in meta-analysis with
#' normally distributed responses.
#' \emph{Statistics in Medicine},
#' \bold{20}, 1771--82
#' 
#' Hartung J, Knapp G (2001b):
#' A refined method for the meta-analysis of controlled clinical
#' trials with binary outcome.
#' \emph{Statistics in Medicine},
#' \bold{20}, 3875--89
#' 
#' Hedges LV & Olkin I (1985):
#' \emph{Statistical methods for meta-analysis}.
#' San Diego, CA: Academic Press
#' 
#' Henmi M & Copas JB (2010):
#' Confidence intervals for random effects meta-analysis and
#' robustness to publication bias.
#' \emph{Statistics in Medicine},
#' \bold{29}, 2969--83
#' 
#' Higgins JPT & Thompson SG (2002):
#' Quantifying heterogeneity in a meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{21}, 1539--58
#' 
#' Higgins JPT, Thompson SG, Spiegelhalter DJ (2009):
#' A re-evaluation of random-effects meta-analysis.
#' \emph{Journal of the Royal Statistical Society: Series A},
#' \bold{172}, 137--59
#' 
#' Hunter JE & Schmidt FL (2015):
#' \emph{Methods of Meta-Analysis: Correcting Error and Bias in
#' Research Findings} (Third edition).
#' Thousand Oaks, CA: Sage
#' 
#' IntHout J, Ioannidis JPA, Borm GF (2014):
#' The Hartung-Knapp-Sidik-Jonkman method for random effects
#' meta-analysis is straightforward and considerably outperforms the
#' standard DerSimonian-Laird method.
#' \emph{BMC Medical Research Methodology},
#' \bold{14}, 25
#' 
#' IQWiG (2022):
#' General Methods: Version 6.1.
#' \url{https://www.iqwig.de/en/about-us/methods/methods-paper/}
#'
#' Jackson D (2013):
#' Confidence intervals for the between-study variance in random
#' effects meta-analysis using generalised Cochran heterogeneity
#' statistics.
#' \emph{Research Synthesis Methods},
#' \bold{4}, 220--229
#'
#' Jackson D, Law M, Rücker G, Schwarzer G (2017): 
#' The Hartung-Knapp modification for random-effects meta-analysis: A
#' useful refinement but are there any residual concerns?
#' \emph{Statistics in Medicine},
#' \bold{36}, 3923--34
#' 
#' Knapp G & Hartung J (2003):
#' Improved tests for a random effects meta-regression with a single
#' covariate.
#' \emph{Statistics in Medicine},
#' \bold{22}, 2693--710
#'
#' Langan D, Higgins JPT, Jackson D, Bowden J, Veroniki AA,
#' Kontopantelis E, et al. (2019):
#' A comparison of heterogeneity variance estimators in simulated
#' random-effects meta-analyses.
#' \emph{Research Synthesis Methods},
#' \bold{10}, 83--98
#' 
#' Schwarzer G (2007):
#' meta: An R package for meta-analysis.
#' \emph{R News},
#' \bold{7}, 40--5
#' 
#' Schwarzer G, Carpenter JR and Rücker G (2015):
#' \emph{Meta-Analysis with R (Use-R!)}.
#' Springer International Publishing, Switzerland
#'
#' Siemens W, Borenstein M, Evrenoglou T, Meerpohl JJ, Schwarzer G (2025):
#' Beyond prediction intervals in meta-analysis: reporting the expected
#' proportion of comparable studies with clinically relevant benefit or harm.
#' \emph{BMC Medical Research Methodology},
#' \bold{25}, 275
#' 
#' Skipka G (2006):
#' The inclusion of the estimated inter-study variation into forest
#' plots for random effects meta-analysis - a suggestion for a
#' graphical representation [abstract].
#' \emph{XIV Cochrane Colloquium, Dublin}, 23-26.
#' 
#' Stijnen T, Hamza TH, Ozdemir P (2010):
#' Random effects meta-analysis of event outcome in the framework of
#' the generalized linear mixed model with applications in sparse
#' data.
#' \emph{Statistics in Medicine},
#' \bold{29}, 3046--67
#' 
#' Veroniki AA, Jackson D, Viechtbauer W, Bender R, Bowden J, Knapp G,
#' et al. (2016):
#' Methods to estimate the between-study variance and its uncertainty
#' in meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{7}, 55--79 
#' 
#' Veroniki AA, Jackson D, Bender R, Kuss O, Higgins JPT, Knapp G, Salanti G
#' (2019):
#' Methods to calculate uncertainty in the estimated overall effect size from a
#' random-effects meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{10}, 23--43
#' 
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#' 
#' Viechtbauer W (2005):
#' Bias and efficiency of meta-analytic variance estimators in the
#' random-effects model.
#' \emph{Journal of Educational and Behavioral Statistics},
#' \bold{30}, 261--93
#' 
#' Viechtbauer W (2007):
#' Confidence intervals for the amount of heterogeneity in
#' meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{26}, 37--52
#' 
#' Viechtbauer W (2010):
#' Conducting Meta-Analyses in R with the metafor Package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#' 
#' Wiksten A, Rücker G, Schwarzer G (2016):
#' Hartung-Knapp method is not always conservative compared with
#' fixed-effect meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{35}, 2503--15
#'
#' @keywords package
#' 
#' @seealso \code{\link{meta-object}}, \code{\link{meta-sm}}
#'
#' @import metabook
#'
#' @import metadat
#' 
#' @importFrom rlang :=
#'
#' @importFrom dplyr %>% across mutate all_of select rename rename_with mutate
#'   if_else tibble filter bind_rows group_by group_map relocate
#'
#' @importFrom tibble column_to_rownames
#'
#' @importFrom scales number_format
#'
#' @importFrom stringr str_pad
#'
#' @importFrom magrittr %<>%
#'
#' @importFrom purrr compact
#'
#' @importFrom readr read_csv cols
#'
#' @importFrom grid arrow gpar grid.circle grid.draw grid.layout
#'   grid.lines grid.newpage grid.polygon grid.rect grid.text
#'   grid.xaxis textGrob popViewport pushViewport viewport unit unit.c
#'   convertX grobTree rectGrob
#'   grid.get grid.gget
#'
#' @importFrom grDevices gray gray.colors cairo_pdf cairo_ps pdf
#'   postscript svg bmp jpeg png tiff hcl.colors
#'
#' @importFrom graphics abline axis barplot box mtext lines par plot
#'   points polygon text
#'   
#' @importFrom ggplot2 aes annotate geom_area geom_line geom_point
#'   geom_polygon ggplot scale_x_continuous xlab ylab
#' 
#' @importFrom stats as.formula binom.test coef cor lm pchisq pf pnorm
#'   poisson.test pt qlogis qnorm qt runif update var weighted.mean
#'   weights glm binomial vcov fitted residuals dt
#'
#' @importFrom utils count.fields read.table assignInNamespace
#'   getFromNamespace packageDescription packageVersion head tail find
#'   unzip combn
#'
#' @importFrom metafor forest funnel funnel.default baujat labbe
#'   radial trimfill rma.uni rma.glmm rma.mv predict.rma
#'   confint.rma.uni confint.rma.mv escalc regtest to.long vcalc blup
#'
#' @importFrom lme4 glmer
#'
#' @importFrom CompQuadForm farebrother
#'
#' @importFrom methods formalArgs
#' 
#' @importFrom xml2 as_xml_document xml_attr xml_find_all xml_text
#' 
#' @export forest funnel baujat labbe radial trimfill blup

"_PACKAGE"

NULL

#' Transform meta-analysis data from two arm-based formats into contrast-based
#' format
#' 
#' @description
#' This function transforms data that are given in wide or long arm-based
#' format (e.g. input format for WinBUGS) to a contrast-based format that is
#' needed as input to R functions \code{\link{metabin}}, \code{\link{metacont}},
#' \code{\link{metainc}}, \code{\link{metagen}}, or
#' \code{\link[netmeta]{netmeta}} from R package \bold{netmeta}. The function
#' can transform data with binary, continuous, or generic outcomes as well as
#' incidence rates from arm-based to contrast-based format.
#' 
#' @param treat A list or vector with treatment information for
#'   individual treatment arms (see Details).
#' @param event A list or vector with information on number of events
#'   for individual treatment arms (see Details).
#' @param n A list or vector with information on number of
#'   observations for individual treatment arms (see Details).
#' @param mean A list or vector with estimated means for individual
#'   treatment arms (see Details).
#' @param sd A list or vector with information on the standard
#'   deviation for individual treatment arms (see Details).
#' @param TE A list or vector with estimated treatment effects for
#'   individual treatment arms (see Details).
#' @param seTE A list or vector with standard errors of estimated
#'   treatment effect for individual treatment arms (see Details).
#' @param time A list or vector with information on person time at
#'   risk for individual treatment arms (see Details).
#' @param agent A list or vector with agent information for
#'   individual treatment arms (see Details).
#' @param dose A list or vector with dose information for
#'   individual treatment arms (see Details).
#' @param data An optional data frame containing the study
#'   information.
#' @param studlab A vector with study labels (optional).
#' @param method A character string indicating which method is to be
#'   used to calculate treatment estimates (see Details).
#' @param sm A character string indicating which summary measure is to be
#'   used to calculate treatment estimates (see Details).
#' @param incr A numerical value which is added to cell frequencies
#'   for studies with a zero cell count, see Details.
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (\code{"only0"},
#'   \code{"if0all"}, or \code{"all"}), see \code{\link{metabin}}.
#' @param allstudies A logical indicating if studies with zero or all
#'   events in two treatment arms are to be included in the
#'   meta-analysis (applies only if \code{sm} is equal to \code{"RR"}
#'   or \code{"OR"}).
#' @param relative.effects A logical indicating whether treatment estimates
#'   and standard errors provided in \code{TE} and \code{seTE} are relative
#'   treatment effects.
#' @param reference.group Reference treatment (first treatment is used
#'   if argument is missing).
#' @param sep.ag A character used as separator between agent and dose to
#'   create treatment labels.
#' @param keep.all.comparisons A logical indicating whether all
#'   pairwise comparisons or only comparisons with the study-specific
#'   reference group should be kept ('basic parameters').
#' @param varnames Character vector of length 2 with the variable names for the
#'   treatment estimate and its standard error; by default, "TE" and "seTE".
#' @param append Either a logical indicating whether variables from the dataset
#'   provided in argument \code{data} are appended to the dataset with
#'   pairwise comparisons or a character vector with variable names to append to
#'   the dataset.
#' @param allincr Deprecated argument (replaced by 'method.incr');
#'   see \code{\link{metabin}}.
#' @param addincr Deprecated argument (replaced by 'method.incr');
#'   see \code{\link{metabin}}.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if studies are excluded due to only providing a single
#'   treatment arm).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments passed-through to the functions
#'   to calculate effects.
#' 
#' @details
#' The pairwise function transforms data given in (wide or long)
#' arm-based format into the contrast-based format which consists of
#' \emph{pairwise} comparisons and which is needed as input to R functions
#' \code{\link{metabin}}, \code{\link{metacont}}, \code{\link{metainc}},
#' \code{\link{metagen}}, or \code{\link[netmeta]{netmeta}} from R package
#' \bold{netmeta}.
#' 
#' The pairwise function can transform data with binary outcomes
#' continuous outcomes (\code{\link{metacont}} function), incidence
#' rates (\code{\link{metainc}} function), and generic outcomes
#' (\code{\link{metagen}} function). Depending on the outcome, the
#' following arguments are mandatory:
#' \itemize{
#' \item treat, event, n (see \code{\link{metabin}});
#' \item treat, n, mean, sd (see \code{\link{metacont}});
#' \item treat, event, time (see \code{\link{metainc}});
#' \item treat, TE, seTE (see \code{\link{metagen}}).
#' }
#' 
#' Admissible values for arguments \code{method} and \code{sm} are outcome
#' specific; see help pages of R functions \code{\link{metabin}},
#' \code{\link{metacont}}, \code{\link{metainc}}, and \code{\link{metagen}}.
#' 
#' It is also possible to provide data in the common BUGS format, i.e.,
#' relative treatment effects in comparison to a reference treatment. In this
#' case, the user has to specify argument \code{relative.effects = TRUE} and
#' the data set must provide the relative treatment estimates and standard
#' errors. The relative treatment effect for the study-specific reference
#' treatment must be equal to \code{NA} (see Examples).
#' 
#' Argument \code{treat} is mandatory to identify the individual
#' treatments. The other arguments contain outcome specific
#' data. These arguments must be either lists (wide arm-based format,
#' i.e., one row per study) or vectors (long arm-based format,
#' i.e., multiple rows per study) of the same length.
#' 
#' For the wide arm-based format, each list consists of as many
#' vectors of the same length as the multi-arm study with the largest
#' number of treatments. If a single multi-arm study has five arms,
#' five vectors have to be provided for each lists. Two-arm studies
#' have entries with \code{NA} for the third and subsequent
#' vectors. Each list entry is a vector with information for each
#' individual study; i.e., the length of this vector corresponds to
#' the total number of studies incorporated in the network
#' meta-analysis. Typically, list elements are part of a data frame
#' (argument \code{data}, optional); see Examples. An optional vector
#' with study labels can be provided which can be part of the data
#' frame.
#' 
#' In the long arm-based format, argument \code{studlab} is mandatory
#' to identify rows contributing to individual studies.
#' 
#' Additional arguments for meta-analysis functions can be provided
#' using argument '\dots'; see help pages of R functions
#' \code{\link{metabin}}, \code{\link{metacont}},
#' \code{\link{metainc}}, and \code{\link{metagen}}.
#'
#' For standardised mean differences (argument \code{sm = "SMD"}),
#' equations (4) and (5) in Crippa & Orsini (2016) are used to
#' calculated SMDs and standard errors. These equations guarantee
#' consistent SMDs and standard errors for multi-arm studies. Note,
#' the summary measure is actually Cohen's d as Hedges' g is not
#' consistent in multi-arm studies.
#' 
#' For binary outcomes, 0.5 is added to all cell frequencies (odds
#' ratio) or only the number of events (risk ratio) for studies with a
#' zero cell count. For odds ratio and risk ratio, treatment estimates
#' and standard errors are only calculated for studies with zero or
#' all events in both groups if \code{allstudies} is \code{TRUE}. This
#' continuity correction is used both to calculate individual study
#' results with confidence limits and to conduct meta-analysis based
#' on the inverse variance method. For the risk difference, 0.5 is
#' only added to all cell frequencies to calculate the standard error.
#' 
#' For incidence rates, 0.5 is added to all cell frequencies for the
#' incidence rate ratio as summary measure. For the incidence risk
#' difference, 0.5 is only added to all cell frequencies to calculate
#' the standard error.
#' 
#' The value of pairwise is a data frame with as many rows as there
#' are pairwise comparisons. For each study with \emph{p} treatments,
#' \emph{p*(p-1) / 2} contrasts are generated. Each row contains the
#' treatment effect (\code{TE}), its standard error (\code{seTE}), the
#' treatments compared ((\code{treat1}), (\code{treat2})) and the
#' study label ((\code{studlab})). Further columns are added according
#' to type of data.
#' 
#' All variables from the original dataset are also part of the output
#' dataset if argument \code{append = TRUE}. If data are provided in the long
#' arm-based format, the value of a variable can differ between treatment arms;
#' for example, the mean age or percentage of women in the treatment arm. In
#' this situation, two variables instead of one variable will be included
#' in the output dataset. The values "1" and "2" are added to the
#' names for these variables, e.g. "mean.age1" and "mean.age2" for the
#' mean age.
#' 
#' In general, any variable names in the original dataset that are
#' identical to the main variable names (i.e., "TE", "seTE", ...) will
#' be renamed to variable names with ending ".orig".
#'
#' A reduced dataset with basic comparisons (Rücker & Schwarzer,
#' 2014) can be generated using argument \code{keep.all.comparisons =
#' FALSE}. Furthermore, the reference group for the basic comparisons
#' can be specified with argument \code{reference.group}.
#'
#' \subsection{Use in network meta-analysis}{
#' R function \code{\link[netmeta]{netmeta}} expects data in a
#' \bold{contrast-based format}, where each row corresponds to a
#' comparison of two treatments and contains a measure of the
#' treatment effect comparing two treatments with standard error,
#' labels for the two treatments and an optional study label.  In
#' contrast-based format, a three-arm study contributes three rows
#' with treatment comparison and corresponding standard error for
#' pairwise comparison \emph{A} vs \emph{B}, \emph{A} vs \emph{C}, and
#' \emph{B} vs \emph{C} whereas a four-arm study contributes six rows
#' / pairwise comparisons: \emph{A} vs \emph{B}, \emph{A} vs \emph{C},
#' \dots{}, \emph{C} vs \emph{D}.
#' 
#' Other programs for network meta-analysis in WinBUGS and Stata
#' require data in an \emph{arm-based} format, i.e. treatment estimate
#' for each treatment arm instead of a difference of two treatments. A
#' common \bold{(wide) arm-based format} consists of one data row per
#' study, containing treatment and other necessary information for all
#' study arms. For example, a four-arm study contributes one row with
#' four treatment estimates and corresponding standard errors for
#' treatments \emph{A}, \emph{B}, \emph{C}, and \emph{D}.  Another
#' possible arm-based format is a long format where each row
#' corresponds to a single study arm. Accordingly, in the \bold{long
#' arm-based format} a study contributes as many rows as treatments
#' considered in the study.
#' }
#' 
#' @note
#' This function must not be confused with \code{\link[netmeta]{netpairwise}}
#' which can be used to conduct pairwise meta-analyses for all
#' comparisons with direct evidence in a network meta-analysis.
#' 
#' @return
#' A data frame with the following columns:
#' \item{TE}{Treatment estimate comparing treatment 'treat1' and
#'   'treat2'.}
#' \item{seTE}{Standard error of treatment estimate.}
#' \item{studlab}{Study labels.}
#' \item{treat1}{First treatment in comparison.}
#' \item{treat2}{Second treatment in comparison.}
#' \item{event1}{Number of events for first treatment arm (for metabin
#'   and metainc).}
#' \item{event2}{Number of events for second treatment arm (for
#'   metabin and metainc).}
#' \item{n1}{Number of observations for first treatment arm (for
#'   metabin and metacont).}
#' \item{n2}{Number of observations for second treatment arm (for
#'   metabin and metacont).}
#' \item{mean1}{Estimated mean for first treatment arm (for
#'   metacont).}
#' \item{mean2}{Estimated mean for second treatment arm (for
#'   metacont).}
#' \item{sd1}{Standard deviation for first treatment arm (for
#'   metacont).}
#' \item{sd2}{Standard deviation for second treatment arm (for
#'   metacont).}
#' \item{TE1}{Estimated treatment effect for first treatment arm (for
#'   metagen).}
#' \item{TE2}{Estimated treatment effect for second treatment arm (for
#'   metagen).}
#' \item{seTE1}{Standard error of estimated treatment effect for first
#'   treatment arm (for metagen).}
#' \item{seTE2}{Standard error of estimated treatment effect for
#'   second treatment arm (for metagen).}
#' \item{time1}{Person time at risk for first treatment arm (for
#'   metainc).}
#' \item{time2}{Person time at risk for second treatment arm (for
#'   metainc).}
#' \item{agent1}{First agent in comparison.}
#' \item{agent2}{Second agent in comparison.}
#' \item{dose1}{Dose of first agent in comparison.}
#' \item{dose2}{Dose of second agent in comparison.}
#' 
#' All variables from the original dataset are also part of the output
#' dataset; see Details.
#' 
#' @author Gerta Rücker\email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de},
#'   Gert van Valkenhoef \email{gvanvalkenhoef@@cochrane.org}
#' 
#' @seealso \code{\link{longarm}}, \code{\link{metabin}},
#'   \code{\link{metacont}}, \code{\link{metagen}},
#'   \code{\link{metainc}}, \code{\link[netmeta]{netmeta}},
#'   \code{\link[netmeta]{netgraph.netmeta}},
#'   \code{\link[metabook]{Senn2013}},
#'   \code{\link[metabook]{Franchini2012}}
#'
#' @references
#' Crippa A, Orsini N (2016):
#' Dose-response meta-analysis of differences in means.
#' \emph{BMC Medical Research Methodology},
#' \bold{16}:91.
#' 
#' @keywords datagen
#' 
#' @examples
#' pw0 <- pairwise(studlab = study, treat = treatment,
#'   n = n, mean = mean, sd = sd, data = Senn2013,
#'   varnames = c("MD", "seMD"), append = c("study", "type"))
#' head(pw0)
#' # Meta-analysis of studies comparing metformin to placebo
#' metagen(pw0, subset = treat1 == "Metformin" & treat2 == "Placebo")
#' 
#' \dontrun{
#' # Use pairwise() to run network meta-analyses
#' # (R package 'netmeta' must be available)
#' if (requireNamespace("netmeta", quietly = TRUE)) {
#'  # Example with continuous outcome - transform data from wide arm-based to
#'  # contrast-based format (internal call of function metacont)
#'  pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
#'    n = list(n1, n2, n3),
#'    mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
#'    data = Franchini2012, studlab = Study)
#'  pw1
#' 
#'  # Conduct network meta-analysis
#'  library("netmeta")
#'  #
#'  nma1 <- netmeta(pw1)
#'  nma1
#' 
#'  # Draw network graphs
#'  #
#'  netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.5,
#'    thickness = "se.common")
#'  netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.5,
#'    plastic = TRUE, thickness = "se.common",
#'    iterate = TRUE)
#'  netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.5,
#'    plastic = TRUE, thickness = "se.common",
#'    iterate = TRUE, start = "eigen")
#' 
#'  # Example using generic outcome (internal call of function
#'  # metagen)
#'  #
#'  # Calculate standard error for means y1, y2, y3
#'  Franchini2012$se1 <- with(Franchini2012, sqrt(sd1^2 / n1))
#'  Franchini2012$se2 <- with(Franchini2012, sqrt(sd2^2 / n2))
#'  Franchini2012$se3 <- with(Franchini2012, sqrt(sd3^2 / n3))
#'  # Transform data from arm-based format to contrast-based format
#'  # using means and standard errors (note, argument 'sm' has to be
#'  # used to specify that argument 'TE' is a mean difference)
#'  pw2 <- pairwise(list(Treatment1, Treatment2, Treatment3),
#'    TE = list(y1, y2, y3), seTE = list(se1, se2, se3),
#'    n = list(n1, n2, n3),
#'    data = Franchini2012, studlab = Study,
#'    sm = "MD")
#'  pw2
#' 
#'  # Compare pairwise objects pw1 (based on continuous outcomes) and pw2
#'  # (based on generic outcomes)
#'  #
#'  all.equal(
#'    pw1[, c("TE", "seTE", "studlab", "treat1", "treat2")],
#'    pw2[, c("TE", "seTE", "studlab", "treat1", "treat2")])
#'  # Only difference:
#'  attributes(pw1)$type
#'  attributes(pw2)$type
#'
#'  # Same result as network meta-analysis based on continuous outcomes
#'  # (object nma1)
#'  nma2 <- netmeta(pw2)
#'  nma2
#' 
#'  # Example with binary data
#'  #
#'  data(smokingcessation)
#'  # Transform data from arm-based format to contrast-based format
#'  # (internal call of metabin function). Argument 'sm' has to be used
#'  # for odds ratio as risk ratio (sm = "RR") is default of metabin
#'  # function.
#'  #
#'  pw3 <- pairwise(list(treat1, treat2, treat3),
#'    list(event1, event2, event3), list(n1, n2, n3),
#'    data = smokingcessation,
#'    sm = "OR")
#'  pw3
#' 
#'  # Conduct network meta-analysis
#'  #
#'  nma3 <- netmeta(pw3)
#'  nma3
#' 
#'  # Example with incidence rates
#'  #
#'  data(dietaryfat)
#' 
#'  # Transform data from arm-based format to contrast-based format
#'  #
#'  pw4 <- pairwise(list(treat1, treat2, treat3),
#'    list(d1, d2, d3), time = list(years1, years2, years3),
#'    studlab = ID,
#'    data = dietaryfat)
#'  pw4
#' 
#'  # Conduct network meta-analysis using incidence rate ratios (sm =
#'  # "IRR"). Note, the argument 'sm' is not necessary as this is the
#'  # default in R function metainc called internally.
#'  #
#'  nma4 <- netmeta(pw4, sm = "IRR")
#'  summary(nma4)
#' 
#'  # Example with long data format
#'  #
#'  # Transform data from long arm-based format to contrast-based
#'  # format Argument 'sm' has to be used for odds ratio as summary
#'  # measure; by default the risk ratio is used in the metabin
#'  # function called internally.
#'  #
#'  pw5 <- pairwise(treatment, event = r, n = N,
#'    studlab = author, data = Woods2010, sm = "OR")
#'  pw5
#' 
#'  # Conduct network meta-analysis
#'  nma5 <- netmeta(pw5)
#'  nma5
#'  
#'  # Relative treatment effects as input, i.e., common BUGS format
#'  #
#'  if (requireNamespace("gemtc", quietly = TRUE) ) {
#'  library(gemtc)
#'  
#'  dat1 <- parkinson_diff$data
#'  dat1
#'  
#'  pw6 <- pairwise(treatment, TE = diff, seTE = std.err, studlab = study,
#'    data = dat1, relative.effects = TRUE, sm = "MD")
#'  pw6
#'  #
#'  nma6 <- netmeta(pw6, common = FALSE, method.tau = "REML",
#'    small.values = "undesirable")
#'  nma6
#' 
#'  datE <- dat1[nrow(dat1), ]
#'  datE$treatment <- "E"
#'  dat2 <- rbind(dat1, datE)
#'  #
#'  pw7 <- pairwise(treatment, TE = diff, seTE = std.err, studlab = study,
#'    data = dat2, relative.effects = TRUE, sm = "MD")
#'  nma7 <- netmeta(pw7, common = FALSE, method.tau = "REML",
#'    small.values = "undesirable")
#'  nma7
#'  #
#'  # Number of pairwise comparisons
#'  #
#'  nma6$m
#'  nma7$m
#'  #
#'  detach("package:gemtc")
#' }
#' }
#' }
#' 
#' @export pairwise

pairwise <- function(treat,
                     event, n, mean, sd, TE, seTE, time,
                     agent, dose,
                     data = NULL, studlab,
                     #
                     method = "Inverse",
                     sm = NULL,
                     incr = gs("incr"),
                     method.incr = gs("method.incr"),
                     allstudies = gs("allstudies"),
                     #
                     relative.effects = FALSE,
                     #
                     reference.group,
                     keep.all.comparisons,
                     #
                     sep.ag = "*",
                     #
                     varnames = c("TE", "seTE"),
                     #
                     append = !is.null(data),
                     #
                     addincr = gs("addincr"),
                     allincr = gs("allincr"),
                     #
                     warn = FALSE, warn.deprecated = gs("warn.deprecated"),
                     ...) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  
  missing.event <- missing(event)
  missing.n <- missing(n)
  missing.mean <- missing(mean)
  missing.sd <- missing(sd)
  missing.TE <- missing(TE)
  missing.seTE <- missing(seTE)
  missing.time <- missing(time)
  missing.agent <- missing(agent)
  missing.dose <- missing(dose)
  missing.studlab <- missing(studlab)
  #
  missing.incr <- missing(incr)
  missing.method.incr <- missing(method.incr)
  missing.allstudies <- missing(allstudies)
  missing.allincr <- missing(allincr)
  missing.addincr <- missing(addincr)
  #
  missing.append <- missing(append)
  missing.sep.ag <- missing(sep.ag)
  missing.reference.group <- missing(reference.group)
  missing.keep.all.comparisons <- missing(keep.all.comparisons)
  missing.varnames <- missing(varnames)
  #
  chknumeric(incr, min = 0, length = 1)
  chklogical(allstudies)
  #
  chklogical(warn.deprecated)
  allincr <-
    deprecated2(method.incr, missing.method.incr, allincr, missing.allincr,
                warn.deprecated)
  addincr <-
    deprecated2(method.incr, missing.method.incr, addincr, missing.addincr,
                warn.deprecated)
  if (missing.method.incr) {
    if (is.logical(addincr) && addincr)
      method.incr <- "all"
    else if (is.logical(allincr) && allincr)
      method.incr <- "if0all"
  }
  #
  method.incr <-
    setchar(method.incr, gs("meth4incr")[gs("meth4incr") != "user"])
  #
  if (!is.character(append)) {
    chklogical(append, text = "or vector with variable names")
    append.logical <- append
  }
  else {
    if (is.null(data))
      append.logical <- NULL
    else {
      append <- setchar(append, names(data),
                        pre = "a logical or a character vector with ")
      #
      append.logical <- !is.null(append)
      #
      append <- c(append, paste0(append, 1), paste0(append, 2))
    }
  }
  #
  chklogical(warn)
  #
  chkchar(sep.ag)
  #
  chkchar(varnames, length = 2)
  #
  args <- list(...)
  nam.args <- names(args)
  
  
  #
  # Auxiliary functions
  #
  sumzero <- function(x)
    sum(x[!is.na(x)] == 0)
  #
  anytrue <- function(x)
    any(x == TRUE, na.rm = TRUE)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  #
  # Catch studlab, treat, agent, dose, event, n, mean, sd, time from data:
  #
  treat <- catch("treat", mc, data, sfsp)
  #
  if (inherits(treat, "pairwise")) {
    is.pairwise <- TRUE
    #
    res.attr <- attributes(treat)
    res <- treat
    #
    if (any(replaceNULL(res.attr$varnames, c("TE", "seTE")) !=
            c("TE", "seTE"))) {
      names(res)[names(res) == res.attr$varnames[1]] <- "TE"
      names(res)[names(res) == res.attr$varnames[2]] <- "seTE"
      #
      if (missing.varnames)
        varnames <- res.attr$varnames
    }
    #
    if (missing.append) {
      append <- res.attr$append
      append.logical <- res.attr$append.logical
    }
    #
    if (missing.reference.group)
      if (!is.null(attributes(treat)$reference.group)) {
        reference.group <- attributes(treat)$reference.group
        missing.reference.group <- FALSE
      }
    #
    if (missing.keep.all.comparisons) {
      if (!is.null(attributes(treat)$keep.all.comparisons))
        keep.all.comparisons <- attributes(treat)$keep.all.comparisons
      else
        keep.all.comparisons <- TRUE
    }
    #
    txt.ignore <- "as first argument is a pairwise object"
    #
    warn_ignore_input(event, !missing.event, txt.ignore)
    warn_ignore_input(n, !missing.n, txt.ignore)
    warn_ignore_input(mean, !missing.mean, txt.ignore)
    warn_ignore_input(sd, !missing.sd, txt.ignore)
    warn_ignore_input(TE, !missing.TE, txt.ignore)
    warn_ignore_input(seTE, !missing.seTE, txt.ignore)
    warn_ignore_input(time, !missing.time, txt.ignore)
    warn_ignore_input(agent, !missing.agent, txt.ignore)
    warn_ignore_input(dose, !missing.dose, txt.ignore)
    warn_ignore_input(data, !nulldata, txt.ignore)
    warn_ignore_input(studlab, !missing.studlab, txt.ignore)
    warn_ignore_input(incr, !missing.incr, txt.ignore)
    warn_ignore_input(method.incr, !missing.method.incr, txt.ignore)
    warn_ignore_input(allincr, !missing.allincr, txt.ignore)
    warn_ignore_input(addincr, !missing.addincr, txt.ignore)
    warn_ignore_input(allstudies, !missing.allstudies, txt.ignore)
    #
    type <- attributes(res)$type
    #
    avail.treat <- TRUE
    avail.agent <- avail.dose <- FALSE
  }
  else {
    is.pairwise <- FALSE
    #
    if (missing.keep.all.comparisons)
      keep.all.comparisons <- TRUE
    chklogical(keep.all.comparisons)
    #
    agent <- catch("agent", mc, data, sfsp)
    dose <- catch("dose", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    event <- catch("event", mc, data, sfsp)
    n <- catch("n", mc, data, sfsp)
    mean <- catch("mean", mc, data, sfsp)
    sd <- catch("sd", mc, data, sfsp)
    TE <- catch("TE", mc, data, sfsp)
    seTE <- catch("seTE", mc, data, sfsp)
    time <- catch("time", mc, data, sfsp)
    #
    avail.studlab <- !is.null(studlab)
    #
    avail.event <- !is.null(event)
    avail.n <- !is.null(n)
    avail.mean <- !is.null(mean)
    avail.sd <- !is.null(sd)
    avail.TE <- !is.null(TE)
    avail.seTE <- !is.null(seTE)
    avail.time <- !is.null(time)
    #
    avail.treat <- !is.null(treat)
    avail.agent <- !is.null(agent)
    avail.dose <- !is.null(dose)
    
    #if (!(avail.treat || (avail.agent & avail.dose)))
    #  stop("Mandatory argument(s) are 'treat' or 'agent' and 'dose'.")
    #
    if (avail.treat & avail.agent & avail.dose)
      stop("Either provide argument 'treat' or arguments 'agent' and 'dose'.")
    #
    #if (!avail.treat & (avail.agent + avail.dose != 2))
    #  stop("Mandatory arguments 'agent' and 'dose' for dose-response data.")
    #
    if (avail.treat)
      if (is.list(treat))
        chklist(treat)
    #
    if (avail.agent)
      if (is.list(agent))
        chklist(agent)
    #
    if (avail.dose)
      if (is.list(dose))
        chklist(dose)
    #
    if (avail.event)
      if (is.list(event))
        chklist(event)
    else
      chknumeric(event)
    #
    if (avail.n)
      if (is.list(n))
        chklist(n)
    else
      chknumeric(n)
    #
    if (avail.mean)
      if (is.list(mean))
        chklist(mean)
    else
      chknumeric(mean)
    #
    if (avail.sd)
      if (is.list(sd))
        chklist(sd)
    else
      chknumeric(sd)
    #
    if (avail.TE)
      if (is.list(TE))
        chklist(TE)
    else
      chknumeric(TE)
    #
    if (avail.seTE)
      if (is.list(seTE))
        chklist(seTE)
    else
      chknumeric(seTE)
    #
    if (avail.time)
      if (is.list(time))
        chklist(time)
    else
      chknumeric(time)
    
    
    if (avail.TE & avail.seTE)
      type <- "generic"
    else if (avail.event & avail.time & !avail.mean & !avail.sd)
      type <- "count"
    else if (avail.event & avail.n & !avail.mean & !avail.sd)
      type <- "binary"
    else if (avail.n & avail.mean & avail.sd)
      type <- "continuous"
    else if (!avail.event & !avail.mean & !avail.sd &
             !avail.TE & !avail.seTE & !avail.time)
      type <- "onlytreat"
    else
      stop("Type of outcome unclear. Please provide the necessary ",
           "information:\n  - event, n (binary outcome)\n  - n, ",
           "mean, sd (continuous outcome)\n  - TE, seTE (generic outcome)\n",
           "  - event, time (incidence rates).")
    
    
    #
    # Determine whether data is in long, wide or comparison-based format
    #
    if (type == "generic") {
      if (is.list(TE) & is.list(seTE)) {
        if (length(TE) == 1 & length(seTE) == 1 &
            (length(treat) == 1 || (length(agent) == 1 & length(dose) == 1))) {
          data.format <- "long"
          #
          TE <- unlist(TE)
          seTE <- unlist(seTE)
          #
          if (avail.treat)
            treat <- unlist(treat)
          else {
            agent <- unlist(agent)
            dose <- unlist(dose)
            #
            sep.ag <- setsep(agent, sep.ag, type = "agent")
            #
            treat <- paste(agent, dose, sep = sep.ag)
          }
        }
        else if (length(TE) == 2 & length(seTE) == 2 &
                 (length(treat) == 2 ||
                  (length(agent) == 2 & length(dose) == 2) ||
                  (!avail.treat & !avail.agent & !avail.dose))) {
          data.format <- "comparison"
          #
          if (!avail.treat & !avail.agent & !avail.dose) {
            treat <- list(expand("trt1", TE[[1]]), expand("trt2", TE[[2]]))
            avail.treat <- TRUE
          }
          #
          dat.st <- data.frame(studlab)
          #
          if (avail.treat) {
            dat.st$treat1 <- treat[[1]]
            dat.st$treat2 <- treat[[2]]
          }
          else {
            dat.st$agent1 <- agent[[1]]
            dat.st$dose1 <- dose[[1]]
            #
            dat.st$agent2 <- agent[[2]]
            dat.st$dose2 <- dose[[2]]
            #
            sep.ag <-
              setsep(c(dat.st$agent1, dat.st$agent2), sep.ag, type = "agent")
            #
            dat.st$treat1 <- paste(dat.st$agent1, dat.st$dose1, sep = sep.ag)
            dat.st$treat2 <- paste(dat.st$agent2, dat.st$dose2, sep = sep.ag)
          }
        }
        else {
          data.format <- "wide"
          #
          if (!avail.treat) {
            sep.ag <- setsep(unlist(agent), sep.ag, type = "agent")
            agent.without.NA <- lapply(agent, replaceNA, replace = "")
            dose.without.NA <- lapply(dose, replaceNA, replace = "")
            #
            treat.without.NA <-
              mapply(paste,
                     agent.without.NA, dose.without.NA,
                     MoreArgs = list(sep = sep.ag),
                     SIMPLIFY = FALSE)
            #
            treat <- lapply(treat.without.NA, replaceVal,
                            old = sep.ag, new = NA)
          }
        }
      }
      else {
        data.format <- "long"
        #
        if (!avail.treat) {
          sep.ag <- setsep(agent, sep.ag, type = "agent")
          treat <- paste(agent, dose, sep = sep.ag)
        }
      }
    }
    #
    else if (type == "binary") {
      if (is.list(event) & is.list(n)) {
        if (length(event) == 1 & length(n) == 1 &
            (length(treat) == 1 || (length(agent) == 1 & length(dose) == 1))) {
          data.format <- "long"
          #
          event <- unlist(event)
          n <- unlist(n)
          #
          if (avail.treat)
            treat <- unlist(treat)
          else {
            agent <- unlist(agent)
            dose <- unlist(dose)
            #
            sep.ag <- setsep(agent, sep.ag, type = "agent")
            #
            treat <- paste(agent, dose, sep = sep.ag)
          }
        }
        else if (length(event) == 2 & length(n) == 2 & 
                 (length(treat) == 2 ||
                  (length(agent) == 2 & length(dose) == 2) ||
                  (!avail.treat & !avail.agent & !avail.dose))) {
          data.format <- "comparison"
          #
          if (!avail.treat & !avail.agent & !avail.dose) {
            treat <- list(expand("trt1", n[[1]]), expand("trt2", n[[2]]))
            avail.treat <- TRUE
          }
          #
          dat.st <- data.frame(studlab)
          #
          if (avail.treat) {
            dat.st$treat1 <- treat[[1]]
            dat.st$treat2 <- treat[[2]]
          }
          else {
            dat.st$agent1 <- agent[[1]]
            dat.st$dose1 <- dose[[1]]
            #
            dat.st$agent2 <- agent[[2]]
            dat.st$dose2 <- dose[[2]]
            #
            sep.ag <-
              setsep(c(dat.st$agent1, dat.st$agent2), sep.ag, type = "agent")
            #
            dat.st$treat1 <- paste(dat.st$agent1, dat.st$dose1, sep = sep.ag)
            dat.st$treat2 <- paste(dat.st$agent2, dat.st$dose2, sep = sep.ag)
          }
        }
        else {
          data.format <- "wide"
          #
          if (!avail.treat) {
            sep.ag <- setsep(unlist(agent), sep.ag, type = "agent")
            agent.without.NA <- lapply(agent, replaceNA, replace = "")
            dose.without.NA <- lapply(dose, replaceNA, replace = "")
            #
            treat.without.NA <-
              mapply(paste,
                     agent.without.NA, dose.without.NA,
                     MoreArgs = list(sep = sep.ag),
                     SIMPLIFY = FALSE)
            #
            treat <- lapply(treat.without.NA, replaceVal,
                            old = sep.ag, new = NA)
          }
        }
      }
      else {
        data.format <- "long"
        #
        if (!avail.treat) {
          sep.ag <- setsep(agent, sep.ag, type = "agent")
          treat <- paste(agent, dose, sep = sep.ag)
        }
      }
    }
    #
    else if (type == "continuous") {
      if (is.list(n) & is.list(mean) & is.list(sd)) {
        if (length(n) == 1 & length(mean) == 1 & length(sd) == 1 &
            (length(treat) == 1 || (length(agent) == 1 & length(dose) == 1))) {
          data.format <- "long"
          #
          n <- unlist(n)
          mean <- unlist(mean)
          sd <- unlist(sd)
          #
          if (avail.treat)
            treat <- unlist(treat)
          else {
            agent <- unlist(agent)
            dose <- unlist(dose)
            #
            sep.ag <- setsep(agent, sep.ag, type = "agent")
            #
            treat <- paste(agent, dose, sep = sep.ag)
          }
        }
        else if (length(n) == 2 & length(mean) == 2 & length(sd) == 2 &
                 (length(treat) == 2 ||
                  (length(agent) == 2 & length(dose) == 2) ||
                  (!avail.treat & !avail.agent & !avail.dose))) {
          data.format <- "comparison"
          #
          if (!avail.treat & !avail.agent & !avail.dose) {
            treat <- list(expand("trt1", n[[1]]), expand("trt2", n[[2]]))
            avail.treat <- TRUE
          }
          #
          dat.st <- data.frame(studlab)
          #
          if (avail.treat) {
            dat.st$treat1 <- treat[[1]]
            dat.st$treat2 <- treat[[2]]
          }
          else {
            dat.st$agent1 <- agent[[1]]
            dat.st$dose1 <- dose[[1]]
            #
            dat.st$agent2 <- agent[[2]]
            dat.st$dose2 <- dose[[2]]
            #
            sep.ag <-
              setsep(c(dat.st$agent1, dat.st$agent2), sep.ag, type = "agent")
            #
            dat.st$treat1 <- paste(dat.st$agent1, dat.st$dose1, sep = sep.ag)
            dat.st$treat2 <- paste(dat.st$agent2, dat.st$dose2, sep = sep.ag)
          }
        }
        else {
          data.format <- "wide"
          #
          if (!avail.treat) {
            sep.ag <- setsep(unlist(agent), sep.ag, type = "agent")
            agent.without.NA <- lapply(agent, replaceNA, replace = "")
            dose.without.NA <- lapply(dose, replaceNA, replace = "")
            #
            treat.without.NA <-
              mapply(paste,
                     agent.without.NA, dose.without.NA,
                     MoreArgs = list(sep = sep.ag),
                     SIMPLIFY = FALSE)
            #
            treat <- lapply(treat.without.NA, replaceVal,
                            old = sep.ag, new = NA)
          }
        }
      }
      else {
        data.format <- "long"
        #
        if (!avail.treat) {
          sep.ag <- setsep(agent, sep.ag, type = "agent")
          treat <- paste(agent, dose, sep = sep.ag)
        }
      }
    }
    #
    else if (type == "count") {
      if (is.list(event) & is.list(time)) {
        if (length(event) == 1 & length(time) == 1 &
            (length(treat) == 1 || (length(agent) == 1 & length(dose) == 1))) {
          data.format <- "long"
          #
          event <- unlist(event)
          time <- unlist(time)
          #
          if (avail.n && is.list(n) && length(n) == 1)
            n <- unlist(n)
          #
          if (avail.treat)
            treat <- unlist(treat)
          else {
            agent <- unlist(agent)
            dose <- unlist(dose)
            #
            sep.ag <- setsep(agent, sep.ag, type = "agent")
            #
            treat <- paste(agent, dose, sep = sep.ag)
          }
        }
        else if (length(event) == 2 & length(time) == 2 &
                 (length(treat) == 2 ||
                  (length(agent) == 2 & length(dose) == 2) ||
                  (!avail.treat & !avail.agent & !avail.dose))) {
          data.format <- "comparison"
          #
          if (!avail.treat & !avail.agent & !avail.dose) {
            treat <- list(expand("trt1", time[[1]]), expand("trt2", time[[2]]))
            avail.treat <- TRUE
          }
          #
          dat.st <- data.frame(studlab)
          #
          if (avail.treat) {
            dat.st$treat1 <- treat[[1]]
            dat.st$treat2 <- treat[[2]]
          }
          else {
            dat.st$agent1 <- agent[[1]]
            dat.st$dose1 <- dose[[1]]
            #
            dat.st$agent2 <- agent[[2]]
            dat.st$dose2 <- dose[[2]]
            #
            sep.ag <-
              setsep(c(dat.st$agent1, dat.st$agent2), sep.ag, type = "agent")
            #
            dat.st$treat1 <- paste(dat.st$agent1, dat.st$dose1, sep = sep.ag)
            dat.st$treat2 <- paste(dat.st$agent2, dat.st$dose2, sep = sep.ag)
          }
        }
        else {
          data.format <- "wide"
          #
          if (!avail.treat) {
            sep.ag <- setsep(unlist(agent), sep.ag, type = "agent")
            agent.without.NA <- lapply(agent, replaceNA, replace = "")
            dose.without.NA <- lapply(dose, replaceNA, replace = "")
            #
            treat.without.NA <-
              mapply(paste,
                     agent.without.NA, dose.without.NA,
                     MoreArgs = list(sep = sep.ag),
                     SIMPLIFY = FALSE)
            #
            treat <- lapply(treat.without.NA, replaceVal,
                            old = sep.ag, new = NA)
          }
        }
      }
      else {
        data.format <- "long"
        #
        if (!avail.treat) {
          sep.ag <- setsep(agent, sep.ag, type = "agent")
          treat <- paste(agent, dose, sep = sep.ag)
        }
      }
    }
    #
    else if (type == "onlytreat") {
      if (is.list(treat)) {
        if ((length(treat) == 1 || (length(agent) == 1 & length(dose) == 1))) {
          data.format <- "long"
          #
          if (avail.treat)
            treat <- unlist(treat)
          else {
            agent <- unlist(agent)
            dose <- unlist(dose)
            #
            sep.ag <- setsep(agent, sep.ag, type = "agent")
            #
            treat <- paste(agent, dose, sep = sep.ag)
          }
        }
        else if (length(treat) == 2 ||
                 (length(agent) == 2 & length(dose) == 2)) {
          data.format <- "comparison"
          #
          dat.st <- data.frame(studlab)
          #
          if (avail.treat) {
            dat.st$treat1 <- treat[[1]]
            dat.st$treat2 <- treat[[2]]
          }
          else {
            dat.st$agent1 <- agent[[1]]
            dat.st$dose1 <- dose[[1]]
            #
            dat.st$agent2 <- agent[[2]]
            dat.st$dose2 <- dose[[2]]
            #
            sep.ag <-
              setsep(c(dat.st$agent1, dat.st$agent2), sep.ag, type = "agent")
            #
            dat.st$treat1 <- paste(dat.st$agent1, dat.st$dose1, sep = sep.ag)
            dat.st$treat2 <- paste(dat.st$agent2, dat.st$dose2, sep = sep.ag)
          }
        }
        else {
          data.format <- "wide"
          #
          if (!avail.treat) {
            sep.ag <- setsep(unlist(agent), sep.ag, type = "agent")
            agent.without.NA <- lapply(agent, replaceNA, replace = "")
            dose.without.NA <- lapply(dose, replaceNA, replace = "")
            #
            treat.without.NA <-
              mapply(paste,
                     agent.without.NA, dose.without.NA,
                     MoreArgs = list(sep = sep.ag),
                     SIMPLIFY = FALSE)
            #
            treat <- lapply(treat.without.NA, replaceVal,
                            old = sep.ag, new = NA)
          }
        }
      }
      else {
        data.format <- "long"
        #
        if (!avail.treat) {
          sep.ag <- setsep(agent, sep.ag, type = "agent")
          treat <- paste(agent, dose, sep = sep.ag)
        }
      }
    }
    
    
    #
    #
    #
    if (relative.effects & type == "generic") {
      if (data.format != "long")
        stop("Data must be in long format for relative effects.",
             call. = FALSE)
      #
      append <- FALSE
      append.logical <- FALSE
      #
      studlab <- as.character(studlab)
      treat <- as.character(treat)
      #
      sel.ref <- is.na(TE)
      #
      if (sum(sel.ref) != length(unique(studlab)))
        stop("In each study, relative effect must be 'NA' for the ",
             "reference treatment.",
             call. = FALSE)
      #
      dat.ref <- data.frame(studlab, treat) %>% filter(sel.ref)
      treat.ref <- sapply(studlab, function(x) {
        dat.ref$treat[dat.ref$studlab == x]
      })
      #
      res <- pwgen(studlab, treat, treat.ref, TE, seTE, n)
    }
    else {
      #
      # Use longarm() to transform outcome variables from comparison-based
      # to long arm-based format
      #
      if (data.format == "comparison") {
        if (!avail.studlab)
          stop("Argument 'studlab' mandatory for comparison-based format.")
        #
        if (type == "binary") {
          ldat <- longarm(studlab = unlist(studlab),
                          treat1 = expand(treat[[1]], n[[1]]),
                          treat2 = expand(treat[[2]], n[[2]]),
                          event1 = event[[1]], event2 = event[[2]],
                          n1 = n[[1]], n2 = n[[2]])
          #
          studlab <- ldat$studlab
          treat <- ldat$treat
          event <- ldat$event
          n <- ldat$n
        }
        #
        else if (type == "continuous") {
          ldat <- longarm(studlab = unlist(studlab),
                          treat1 = expand(treat[[1]], n[[1]]),
                          treat2 = expand(treat[[2]], n[[2]]),
                          n1 = n[[1]], n2 = n[[2]],
                          mean1 = mean[[1]], mean2 = mean[[2]],
                          sd1 = sd[[1]], sd2 = sd[[2]])
          #
          studlab <- ldat$studlab
          treat <- ldat$treat
          n <- ldat$n
          mean <- ldat$mean
          sd <- ldat$sd
        }
        #
        else if (type == "count") {
          avail.n.comp <- avail.n && is.list(n) && length(n) == 2
          #
          ldat <- longarm(studlab = unlist(studlab),
                          treat1 = expand(treat[[1]], time[[1]]),
                          treat2 = expand(treat[[2]], time[[2]]),
                          event1 = event[[1]], event2 = event[[2]],
                          time1 = time[[1]], time2 = time[[2]],
                          n1 = if (avail.n.comp) n[[1]] else NULL,
                          n2 = if (avail.n.comp) n[[2]] else NULL)
          #
          studlab <- ldat$studlab
          treat <- ldat$treat
          event <- ldat$event
          time <- ldat$time
          #
          if (avail.n.comp)
            n <- ldat$n
        }
      }
      
      
      #
      # Transform outcome variables from long arm-based to list format
      #
      if (data.format %in% c("comparison", "long")) {
        if (!avail.studlab)
          stop("Argument 'studlab' mandatory for long arm-based format.")
        #
        studlab <- as.character(studlab)
        #
        treat <- as.character(treat)
        #
        ttab <- table(studlab, treat)
        n.arms <- apply(ttab, 1, sum)
        max.arms <- max(n.arms)
        #
        treat.list <- vector("list", max.arms)
        event.list <- vector("list", max.arms)
        n.list     <- vector("list", max.arms)
        mean.list  <- vector("list", max.arms)
        sd.list    <- vector("list", max.arms)
        TE.list    <- vector("list", max.arms)
        seTE.list  <- vector("list", max.arms)
        time.list  <- vector("list", max.arms)
        #
        if (!nulldata)
          adddata <- vector("list", max.arms)
        #
        if (type == "binary") {
          #
          # Generate lists
          #
          tdat <- data.frame(studlab, treat, event, n,
                             .order = seq_along(treat), stringsAsFactors = FALSE)
          #
          if (!nulldata & data.format == "long") {
            tdat <- cbind(tdat, data)
            dupl <- duplicated(names(tdat))
            if (any(dupl))
              names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
          }
          #
          studlab <- names(n.arms)
          dat.studlab <- data.frame(studlab, stringsAsFactors = FALSE)
          #
          for (i in 1:max.arms) {
            sel.i <- !duplicated(tdat$studlab)
            tdat.i <- merge(dat.studlab, tdat[sel.i, ],
                            by = "studlab", all.x = TRUE)
            #
            treat.list[[i]] <- tdat.i$treat
            event.list[[i]] <- tdat.i$event
            n.list[[i]]     <- tdat.i$n
            #
            tdat.i$event <- NULL
            tdat.i$n     <- NULL
            #
            if (!nulldata)
              adddata[[i]] <- tdat.i
            #
            tdat <- tdat[!sel.i, ]
          }
          #
          treat <- treat.list
          event <- event.list
          n     <- n.list
        }
        #
        else if (type == "continuous") {
          #
          # Generate lists
          #
          tdat <- data.frame(studlab, treat, n, mean, sd,
                             .order = seq_along(treat), stringsAsFactors = FALSE)
          #
          if (!nulldata & data.format == "long") {
            tdat <- cbind(tdat, data)
            dupl <- duplicated(names(tdat))
            if (any(dupl))
              names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
          }
          #
          studlab <- names(n.arms)
          dat.studlab <- data.frame(studlab, stringsAsFactors = FALSE)
          #
          for (i in 1:max.arms) {
            sel.i <- !duplicated(tdat$studlab)
            tdat.i <- merge(dat.studlab, tdat[sel.i, ],
                            by = "studlab", all.x = TRUE)
            #
            treat.list[[i]] <- tdat.i$treat
            n.list[[i]]     <- tdat.i$n
            mean.list[[i]]  <- tdat.i$mean
            sd.list[[i]]    <- tdat.i$sd
            #
            tdat.i$n    <- NULL
            tdat.i$mean <- NULL
            tdat.i$sd   <- NULL
            #
            if (!nulldata)
              adddata[[i]] <- tdat.i
            #
            tdat <- tdat[!sel.i, ]
          }
          #
          treat <- treat.list
          n     <- n.list
          mean  <- mean.list
          sd    <- sd.list
        }
        #
        else if (type == "count") {
          #
          # Generate lists
          #
          tdat <- data.frame(studlab, treat, event, time,
                             .order = seq_along(treat), stringsAsFactors = FALSE)
          #
          if (avail.n)
            tdat$n <- n
          #
          if (!nulldata & data.format == "long") {
            tdat <- cbind(tdat, data)
            dupl <- duplicated(names(tdat))
            if (any(dupl))
              names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
          }
          #
          studlab <- names(n.arms)
          dat.studlab <- data.frame(studlab, stringsAsFactors = FALSE)
          #
          for (i in 1:max.arms) {
            sel.i <- !duplicated(tdat$studlab)
            tdat.i <- merge(dat.studlab, tdat[sel.i, ],
                            by = "studlab", all.x = TRUE)
            #
            treat.list[[i]] <- tdat.i$treat
            event.list[[i]] <- tdat.i$event
            time.list[[i]]  <- tdat.i$time
            #
            if (avail.n)
              n.list[[i]] <- tdat.i$n
            #
            tdat.i$event <- NULL
            tdat.i$time  <- NULL
            #
            if (!nulldata)
              adddata[[i]] <- tdat.i
            #
            tdat <- tdat[!sel.i, ]
          }
          #
          treat <- treat.list
          event <- event.list
          time  <- time.list
          #
          if (avail.n)
            n  <- n.list
        }
        #
        else if (type == "generic") {
          #
          # Generate lists
          #
          tdat <- data.frame(studlab, treat, TE, seTE,
                             .order = seq_along(treat), stringsAsFactors = FALSE)
          #
          if (avail.n)
            tdat$n <- n
          #
          if (avail.event)
            tdat$event <- event
          #
          if (!nulldata & data.format == "long") {
            tdat <- cbind(tdat, data)
            dupl <- duplicated(names(tdat))
            if (any(dupl))
              names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
          }
          #
          studlab <- names(n.arms)
          dat.studlab <- data.frame(studlab, stringsAsFactors = FALSE)
          #
          for (i in 1:max.arms) {
            sel.i <- !duplicated(tdat$studlab)
            tdat.i <- merge(dat.studlab, tdat[sel.i, ],
                            by = "studlab", all.x = TRUE)
            #
            treat.list[[i]] <- tdat.i$treat
            TE.list[[i]]    <- tdat.i$TE
            seTE.list[[i]]  <- tdat.i$seTE
            #
            tdat.i$TE   <- NULL
            tdat.i$seTE <- NULL
            #
            if (!nulldata)
              adddata[[i]] <- tdat.i
            #
            tdat <- tdat[!sel.i, ]
          }
          #
          treat <- treat.list
          TE    <- TE.list
          seTE  <- seTE.list
        }
        #
        else if (type == "onlytreat") {
          #
          # Generate lists
          #
          tdat <- data.frame(studlab, treat,
                             .order = seq_along(treat), stringsAsFactors = FALSE)
          #
          if (avail.n)
            tdat$n <- n
          #
          if (!nulldata) {
            if (data.format == "long")
              tdat <- cbind(tdat, data)
            dupl <- duplicated(names(tdat))
            if (any(dupl))
              names(tdat)[dupl] <- paste(names(tdat)[dupl], "orig", sep = ".")
          }
          #
          studlab <- names(n.arms)
          dat.studlab <- data.frame(studlab, stringsAsFactors = FALSE)
          #
          for (i in 1:max.arms) {
            sel.i <- !duplicated(tdat$studlab)
            tdat.i <- merge(dat.studlab, tdat[sel.i, ],
                            by = "studlab", all.x = TRUE)
            #
            treat.list[[i]] <- tdat.i$treat
            #
            if (!nulldata)
              adddata[[i]] <- tdat.i
            #
            tdat <- tdat[!sel.i, ]
          }
          #
          treat <- treat.list
        }
      }
      
      
      
      
      #
      # Check and set study labels
      #
      if (!avail.studlab)
        studlab <- seq(along = treat[[1]])
      #
      if (length(treat) != 2 && length(studlab) != length(unique(studlab)))
        stop("Study labels must all be distinct.")
      #
      levs <- unique(studlab)
      
      
      narms <- length(unique(treat))
      nstud <- length(unique(studlab))
      
      
      #
      #
      # Generate dataset with variables from original dataset
      #
      #
      if (!nulldata & data.format == "long") {
        names.adddata <- names(adddata[[1]])
        #
        notunique <- matrix(NA,
                            ncol = length(names.adddata),
                            nrow = narms * (narms - 1) / 2)
        colnames(notunique) <- names.adddata
        #
        oneNA <- matrix(NA,
                        ncol = length(names.adddata),
                        nrow = narms * (narms - 1) / 2)
        #
        allNA <- matrix(NA,
                        ncol = length(names.adddata),
                        nrow = narms * (narms - 1) / 2)
        colnames(oneNA) <- names.adddata
        #
        n.ij <- 0
        #
        for (i in 1:(narms - 1)) {
          for (j in (i + 1):narms) {
            n.ij <- n.ij + 1
            notunique[n.ij, ] <-
              apply(adddata[[i]] != adddata[[j]], 2, anytrue)
            #
            allNA[n.ij, ] <- apply(is.na(adddata[[j]]), 2, all)
            #
            oneNA[n.ij, ] <-
              apply(is.na(adddata[[i]]) & !is.na(adddata[[j]]), 2, anytrue)
          }
        }
        #
        notunique <- apply(notunique, 2, anytrue)
        oneNA <- apply(oneNA, 2, anytrue)
        allNA <- apply(allNA, 2, anytrue)
        notunique <- apply(rbind(notunique, oneNA, allNA), 2, anytrue)
        # print(notunique)
        #
        for (i in 1:(narms - 1)) {
          for (j in (i + 1):narms) {
            dat.i <- adddata[[i]]
            dat.j <- adddata[[j]]
            #
            if (any(!notunique))
              dat.ij <- dat.i[, names.adddata[!notunique], drop = FALSE]
            else
              stop("Study label must be unique for single treatment arm.")
            #
            for (nam in names.adddata[notunique]) {
              dat.ij[, paste0(nam, 1)] <- adddata[[i]][nam]
              dat.ij[, paste0(nam, 2)] <- adddata[[j]][nam]
            }
            #
            if (i == 1 & j == 2)
              newdata <- dat.ij
            else
              newdata <- rbind(newdata, dat.ij)
          }
        }
        #
        names.basic <- c("studlab", "treat1", "treat2")
        names.newdata <- names(newdata)
        #
        newdata <- newdata[, c(names.basic,
                               names.newdata[!(names.newdata %in% names.basic)])]
        newdata <- newdata[!is.na(newdata$treat1) & !is.na(newdata$treat2), ]
      }
      
      
      if (type == "binary") {
        #
        method <- setchar(method, gs("meth4bin"))
        #
        if (length(event) != narms)
          stop("Different length of lists 'treat' and 'event'.")
        if (length(n) != narms)
          stop("Different length of lists 'treat' and 'n'.")
        #
        # Determine increment for individual studies
        #
        n.zeros <- apply(matrix(unlist(event), ncol = length(event)), 1, sumzero)
        n.all   <- apply(matrix(unlist(n), ncol = length(event)) -
                           matrix(unlist(event), ncol = length(event)),
                         1, sumzero)
        #
        incr.study <- rep(0, length(n.zeros))
        #
        if (is.null(sm))
          sm <- if (method == "Peto") "OR" else gs("smbin")
        else
          sm <- setchar(sm, gs("sm4bin"))
        #
        addincr <- allincr <- FALSE
        #
        if (!(sm == "ASD" | method == "Peto")) {
          if (method.incr == "all")
            addincr <- TRUE
          else if (method.incr == "if0all")
            allincr <- TRUE
        }
        #
        sparse <- switch(sm,
                         OR = (n.zeros > 0) | (n.all > 0),
                         RD = (n.zeros > 0) | (n.all > 0),
                         RR = (n.zeros > 0) | (n.all > 0),
                         ASD = rep(FALSE, length(n.zeros)))
        #
        if (!allincr & !addincr)
          incr.study[sparse] <- incr
        else if (addincr)
          incr.study[] <- incr
        else {
          if (any(n.zeros > 0))
            incr.study[] <- incr
          else
            incr.study[] <- 0
        }
        #
        for (i in 1:(narms - 1)) {
          #
          if (i == 1 & (length(treat[[i]]) != length(event[[i]])))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'event'.")
          if (i == 1 & (length(event[[i]]) != length(n[[i]])))
            stop("Different length of element ", i, " of ",
                 "lists 'event' and 'n'.")
          #
          for (j in (i + 1):narms) {
            #
            if (length(treat[[j]]) != length(event[[j]]))
              stop("Different length of element ", j, " of ",
                   "lists 'treat' and 'event'.")
            if (length(event[[j]]) != length(n[[j]]))
              stop("Different length of element ", j, " of ",
                   "lists 'event' and 'n'.")
            #
            dat <- data.frame(studlab, treat1 = treat[[i]], treat2 = treat[[j]],
                              #
                              TE = NA, seTE = NA,
                              #
                              event1 = event[[i]], n1 = n[[i]],
                              event2 = event[[j]], n2 = n[[j]],
                              #
                              incr1 = incr.study, incr2 = incr.study,
                              #
                              .order = seq_along(studlab),
                              stringsAsFactors = FALSE, row.names = NULL)
            #
            if (!nulldata & data.format == "wide") {
              dat <- cbind(dat, data, stringsAsFactors = FALSE)
              dupl <- duplicated(names(dat))
              if (any(dupl))
                names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
            }
            #
            dat <- dat[!(is.na(dat$event1) & is.na(dat$n1)), ]
            dat <- dat[!(is.na(dat$event2) & is.na(dat$n2)), ]
            #
            if (nrow(dat) > 0) {
              m1 <- metabin(dat$event1, dat$n1, dat$event2, dat$n2,
                            #
                            method = method, sm = sm,
                            #
                            incr.e = dat$incr1, incr.c = dat$incr2,
                            method.incr = "user", allstudies = allstudies,
                            #
                            method.tau = "DL", method.tau.ci = "",
                            #
                            warn = warn,
                            warn.deprecated = FALSE, ...)
              #
              dat$TE <- m1$TE
              dat$seTE <- m1$seTE
              #
              dat$TE[is.infinite(dat$TE)] <- NA
              dat$seTE[is.infinite(dat$seTE)] <- NA
              #
              dat$incr1 <- m1$incr.e
              dat$incr2 <- m1$incr.c
              #
              dat.NAs <- dat[is.na(dat$TE) | is.na(dat$seTE) | dat$seTE <= 0, ]
              #
              if (i == 1 & j == 2) {
                res <- dat
                res.NAs <- dat.NAs
              }
              else {
                res <- rbind(res, dat)
                res.NAs <- rbind(res.NAs, dat.NAs)
              }
            }
            else
              if (i == 1 & j == 2)
                stop("No studies available for comparison of ",
                     "first and second treatment.")
          }
        }
      }
      #
      else if (type == "continuous") {
        if (length(n) != narms)
          stop("Different length of lists 'treat' and 'n'.")
        if (length(mean) != narms)
          stop("Different length of lists 'treat' and 'mean'.")
        if (length(sd) != narms)
          stop("Different length of lists 'treat' and 'sd'.")
        #
        if (is.null(sm))
          sm <- gs("smcont")
        else
          sm <- setchar(sm, gs("sm4cont"))
        #
        for (i in seq_len(narms)) {
          #
          if (length(treat[[i]]) != length(n[[i]]))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'n'.",
                 call. = FALSE)
          if (length(treat[[i]]) != length(mean[[i]]))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'mean'.",
                 call. = FALSE)
          if (length(treat[[i]]) != length(sd[[i]]))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'sd'.",
                 call. = FALSE)
          if (length(treat[[i]]) != nstud)
            stop("Different length of study labels and ",
                 "element ", i, " of list 'treat'.",
                 call. = FALSE)
        }
        #
        # For standardised mean difference, calculate pooled standard
        # deviation for multi-arm studies
        #
        if (sm == "SMD" & narms > 2) {
          pooled.sd <- function(sd, n) {
            sel <- !is.na(sd) & !is.na(n)
            #
            if (any(sel))
              res <- sqrt(sum((n[sel] - 1) * sd[sel]^2) / sum(n[sel] - 1))
            else
              res <- NA
            #
            res
          }
          #
          N <- matrix(unlist(n), ncol = narms, nrow = nstud, byrow = FALSE)
          M <- matrix(unlist(mean), ncol = narms, nrow = nstud, byrow = FALSE)
          S <- matrix(unlist(sd), ncol = narms, nrow = nstud, byrow = FALSE)
          #
          sel.n <- apply(!is.na(N) & N > 0, 1, sum) > 2
          sel.mean <- apply(!is.na(M), 1, sum) > 2
          sel.sd <- apply(!is.na(S) & S > 0, 1, sum) > 2
          sel <- sel.n & sel.mean & sel.sd
          #
          if (any(sel)) {
            N <- N[sel, , drop = FALSE]
            S <- S[sel, , drop = FALSE]
            sd.p <- rep_len(NA, nrow(N))
            #
            for (i in seq_len(nrow(N)))
              sd.p[i] <- pooled.sd(S[i, ], N[i, ])
          }
          #
          for (i in seq_len(narms))
            sd[[i]][sel] <- ifelse(is.na(sd[[i]][sel]), NA, sd.p)
        }
        #
        for (i in 1:(narms - 1)) {
          for (j in (i + 1):narms) {
            dat <- data.frame(studlab, treat1 = treat[[i]], treat2 = treat[[j]],
                              #
                              TE = NA, seTE = NA,
                              n1 = n[[i]], mean1 = mean[[i]], sd1 = sd[[i]],
                              n2 = n[[j]], mean2 = mean[[j]], sd2 = sd[[j]],
                              #
                              .order = seq_along(studlab),
                              stringsAsFactors = FALSE, row.names = NULL)
            #
            if (data.format == "wide") {
              dat <- cbind(dat, data, stringsAsFactors = FALSE)
              dupl <- duplicated(names(dat))
              if (any(dupl))
                names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
            }
            #
            dat <- dat[!(is.na(dat$n1) & is.na(dat$mean1) & is.na(dat$sd1)), ]
            dat <- dat[!(is.na(dat$n2) & is.na(dat$mean2) & is.na(dat$sd2)), ]
            #
            if (nrow(dat) > 0) {
              m1 <- metacont(dat$n1, dat$mean1, dat$sd1,
                             dat$n2, dat$mean2, dat$sd2,
                             #
                             sm = sm,
                             method.tau = "DL", method.tau.ci = "",
                             method.smd = "Cohen",
                             #
                             warn = warn,
                             warn.deprecated = FALSE, ...)
              #
              dat$TE <- m1$TE
              dat$seTE <- m1$seTE
              #
              dat$TE[is.infinite(dat$TE)] <- NA
              dat$seTE[is.infinite(dat$seTE)] <- NA
              #
              dat.NAs <- dat[is.na(dat$TE) | is.na(dat$seTE) | dat$seTE <= 0, ]
              #
              if (i == 1 & j == 2) {
                res <- dat
                res.NAs <- dat.NAs
              }
              else {
                res <- rbind(res, dat)
                res.NAs <- rbind(res.NAs, dat.NAs)
              }
            }
            else
              if (i == 1 & j == 2)
                stop("No studies available for comparison of ",
                     "first and second treatment.",
                     call. = FALSE)
          }
        }
      }
      #
      else if (type == "generic") {
        if (length(TE) != narms)
          stop("Different length of lists 'treat' and 'TE'.",
               call. = FALSE)
        if (length(seTE) != narms)
          stop("Different length of lists 'treat' and 'seTE'.",
               call. = FALSE)
        #
        if (is.null(sm))
          sm <- ""
        #
        for (i in 1:(narms - 1)) {
          #
          if (i == 1 & (length(treat[[i]]) != length(TE[[i]])))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'TE'.",
                 call. = FALSE)
          if (i == 1 & (length(treat[[i]]) != length(seTE[[i]])))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'seTE'.",
                 call. = FALSE)
          #
          for (j in (i + 1):narms) {
            #
            if (length(treat[[j]]) != length(TE[[j]]))
              stop("Different length of element ", j, " of ",
                   "lists 'treat' and 'TE'.",
                   call. = FALSE)
            if (length(treat[[j]]) != length(seTE[[j]]))
              stop("Different length of element ", j, " of ",
                   "lists 'treat' and 'seTE'.",
                   call. = FALSE)
            #
            dat <- data.frame(studlab, treat1 = treat[[i]], treat2 = treat[[j]],
                              #
                              TE = NA, seTE = NA,
                              TE1 = TE[[i]], seTE1 = seTE[[i]],
                              TE2 = TE[[j]], seTE2 = seTE[[j]],
                              #
                              .order = seq_along(studlab),
                              stringsAsFactors = FALSE, row.names = NULL)
            #
            if (avail.event) {
              dat$event1 <- event[[i]]
              dat$event2 <- event[[j]]
            }
            #
            if (avail.n) {
              dat$n1 <- n[[i]]
              dat$n2 <- n[[j]]
            }
            #
            if (avail.mean) {
              dat$mean1 <- mean[[i]]
              dat$mean2 <- mean[[j]]
            }
            #
            if (avail.sd) {
              dat$sd1 <- sd[[i]]
              dat$sd2 <- sd[[j]]
            }
            #
            if (avail.time) {
              dat$time1 <- time[[i]]
              dat$time2 <- time[[j]]
            }
            #
            if (data.format == "wide") {
              dat <- cbind(dat, data, stringsAsFactors = FALSE)
              dupl <- duplicated(names(dat))
              if (any(dupl))
                names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
            }
            #
            dat <- dat[!(is.na(dat$TE1) & is.na(dat$seTE1)), ]
            dat <- dat[!(is.na(dat$TE2) & is.na(dat$seTE2)), ]
            #
            if (nrow(dat) > 0) {
              m1 <- metagen(dat$TE1 - dat$TE2,
                            sqrt(dat$seTE1^2 + dat$seTE2^2),
                            #
                            sm = sm,
                            method.tau = "DL", method.tau.ci = "",
                            #
                            warn = warn,
                            warn.deprecated = FALSE, ...)
              #
              dat$TE <- m1$TE
              dat$seTE <- m1$seTE
              #
              dat$TE[is.infinite(dat$TE)] <- NA
              dat$seTE[is.infinite(dat$seTE)] <- NA
              #
              dat.NAs <- dat[is.na(dat$TE) | is.na(dat$seTE) | dat$seTE <= 0, ]
              #
              if (i == 1 & j == 2) {
                res <- dat
                res.NAs <- dat.NAs
              }
              else {
                res <- rbind(res, dat)
                res.NAs <- rbind(res.NAs, dat.NAs)
              }
            }
            else
              if (i == 1 & j == 2)
                stop("No studies available for comparison of ",
                     "first and second treatment.",
                     call. = FALSE)
          }
        }
      }
      #
      else if (type == "count") {
        #
        method <- setchar(method, gs("meth4inc"))
        #
        if (length(event) != narms)
          stop("Different length of lists 'treat' and 'event'.",
               call. = FALSE)
        if (length(time) != narms)
          stop("Different length of lists 'treat' and 'time'.",
               call. = FALSE)
        #
        if (is.null(sm))
          sm <- gs("sminc")
        else
          sm <- setchar(sm, gs("sm4inc"))
        #
        addincr <- allincr <- FALSE
        #
        if (method.incr == "all")
          addincr <- TRUE
        else if (method.incr == "if0all")
          allincr <- TRUE
        #
        # Determine increment for individual studies
        #
        n.zeros <- apply(matrix(unlist(event), ncol = length(event)), 1, sumzero)
        #
        incr.study <- rep(0, length(n.zeros))
        #
        sparse <- n.zeros > 0
        #
        if (!allincr & !addincr)
          incr.study[sparse] <- incr
        else if (addincr)
          incr.study[] <- incr
        else {
          if (any(n.zeros > 0))
            incr.study[] <- incr
          else
            incr.study[] <- 0
        }
        #
        for (i in 1:(narms - 1)) {
          #
          if (i == 1 & (length(treat[[i]]) != length(event[[i]])))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'event'.",
                 call. = FALSE)
          #
          if (i == 1 & (length(treat[[i]]) != length(time[[i]])))
            stop("Different length of element ", i, " of ",
                 "lists 'treat' and 'time'.",
                 call. = FALSE)
          #
          if (avail.n)
            if (i == 1 & (length(treat[[i]]) != length(n[[i]])))
              stop("Different length of element ", i, " of ",
                   "lists 'treat' and 'n'.",
                   call. = FALSE)
          #
          for (j in (i + 1):narms) {
            #
            if (length(treat[[j]]) != length(event[[j]]))
              stop("Different length of element ", j, " of ",
                   "lists 'treat' and 'event'.",
                   call. = FALSE)
            #
            if (length(treat[[j]]) != length(time[[j]]))
              stop("Different length of element ", j, " of ",
                   "lists 'treat' and 'time'.",
                   call. = FALSE)
            #
            if (avail.n)
              if (length(treat[[j]]) != length(n[[j]]))
                stop("Different length of element ", j, " of ",
                     "lists 'treat' and 'n'.",
                     call. = FALSE)
            #
            dat <- data.frame(studlab, treat1 = treat[[i]], treat2 = treat[[j]],
                              #
                              TE = NA, seTE = NA,
                              #
                              event1 = event[[i]], time1 = time[[i]],
                              event2 = event[[j]], time2 = time[[j]],
                              #
                              incr1 = incr.study, incr2 = incr.study,
                              #
                              n1 = if (avail.n) n[[i]] else NA,
                              n2 = if (avail.n) n[[j]] else NA,
                              #
                              .order = seq_along(studlab),
                              stringsAsFactors = FALSE, row.names = NULL)
            #
            if (data.format == "wide") {
              dat <- cbind(dat, data, stringsAsFactors = FALSE)
              dupl <- duplicated(names(dat))
              if (any(dupl))
                names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
            }
            #
            dat <- dat[!(is.na(dat$event1) & is.na(dat$time1)), ]
            dat <- dat[!(is.na(dat$event2) & is.na(dat$time2)), ]
            #
            if (nrow(dat) > 0) {
              m1 <- metainc(dat$event1, dat$time1,
                            dat$event2, dat$time2,
                            #
                            n.e = if (avail.n) dat$n1 else NULL,
                            n.c = if (avail.n) dat$n2 else NULL,
                            #
                            method = method, sm = sm,
                            #
                            incr.e = dat$incr1, incr.c = dat$incr2,
                            method.incr = "user",
                            #
                            method.tau = "DL", method.tau.ci = "",
                            #
                            warn = warn, warn.deprecated = FALSE, ...)
              #
              dat$TE <- m1$TE
              dat$seTE <- m1$seTE
              #
              dat$TE[is.infinite(dat$TE)] <- NA
              dat$seTE[is.infinite(dat$seTE)] <- NA
              #
              dat$incr1 <- m1$incr.e
              dat$incr2 <- m1$incr.c
              #
              dat.NAs <- dat[is.na(dat$TE) | is.na(dat$seTE) | dat$seTE <= 0, ]
              #
              if (i == 1 & j == 2) {
                res <- dat
                res.NAs <- dat.NAs
              }
              else {
                res <- rbind(res, dat)
                res.NAs <- rbind(res.NAs, dat.NAs)
              }
            }
            else
              if (i == 1 & j == 2)
                stop("No studies available for comparison of ",
                     "first and second treatment.",
                     call. = FALSE)
          }
        }
      }
      #
      else if (type == "onlytreat") {
        if (is.null(sm))
          sm <- ""
        #
        for (i in 1:(narms - 1)) {
          for (j in (i + 1):narms) {
            #
            dat <- data.frame(studlab, treat1 = treat[[i]], treat2 = treat[[j]],
                              .order = seq_along(studlab),
                              stringsAsFactors = FALSE, row.names = NULL)
            #
            if (avail.n) {
              dat$n1 <- n[[i]]
              dat$n2 <- n[[j]]
            }
            #
            if (data.format == "wide") {
              dat <- cbind(dat, data, stringsAsFactors = FALSE)
              dupl <- duplicated(names(dat))
              if (any(dupl))
                names(dat)[dupl] <- paste(names(dat)[dupl], "orig", sep = ".")
            }
            #
            dat <- dat[!is.na(dat$treat2), ]
            #
            if (nrow(dat) > 0) {
              dat.NAs <- data.frame()
              #
              if (i == 1 & j == 2) {
                res <- dat
                res.NAs <- dat.NAs
              }
              else {
                res <- rbind(res, dat)
                res.NAs <- rbind(res.NAs, dat.NAs)
              }
            }
            else
              if (i == 1 & j == 2)
                stop("No studies available for comparison of ",
                     "first and second treatment.",
                     call. = FALSE)
          }
        }
      }
      #
      method <- if (type == "onlytreat") "" else m1$method
      #
      if (!nulldata & data.format == "long")
        res <- merge(res, newdata,
                     by = c("studlab", "treat1", "treat2"),
                     suffixes = c("",".orig"),
                     all.x = TRUE)
      #
      if (!nulldata & data.format == "comparison") {
        res$.order <- NULL
        res2 <- data.frame()
        for (i in seq_len(nrow(dat.st))) {
          sel.i <-
            res$studlab == dat.st$studlab[i] &
            res$treat1 == dat.st$treat1[i] &
            res$treat2 == dat.st$treat2[i]
          #
          sel.wo.i <-
            res$studlab == dat.st$studlab[i] &
            res$treat1 == dat.st$treat2[i] &
            res$treat2 == dat.st$treat1[i]
          #
          if (any(sel.i))
            res2 <- rbind(res2, res[sel.i, ])
          else if (any(sel.wo.i)) {
            res2.i <- res[sel.wo.i, ]
            #
            res2.i$TE <- -res2.i$TE
            #
            t1 <- res2.i$treat1
            res2.i$treat1 <- res2.i$treat2
            res2.i$treat2 <- t1
            #
            if (type == "binary") {
              ev1 <- res2.i$event1
              res2.i$event1 <- res2.i$event2
              res2.i$event2 <- ev1
              #
              tn1 <- res2.i$n1
              res2.i$n1 <- res2.i$n2
              res2.i$n2 <- tn1
            }
            #
            else if (type == "continuous") {
              tn1 <- res2.i$n1
              res2.i$n1 <- res2.i$n2
              res2.i$n2 <- tn1
              #
              me1 <- res2.i$mean1
              res2.i$mean1 <- res2.i$mean2
              res2.i$mean2 <- me1
              #
              ts1 <- res2.i$sd1
              res2.i$sd1 <- res2.i$sd2
              res2.i$sd2 <- ts1
            }
            #
            else if (type == "count") {
              ev1 <- res2.i$event1
              res2.i$event1 <- res2.i$event2
              res2.i$event2 <- ev1
              #
              tt1 <- res2.i$time1
              res2.i$time1 <- res2.i$time2
              res2.i$time2 <- tt1
              #
              if (avail.n) {
                tn1 <- res2.i$n1
                res2.i$n1 <- res2.i$n2
                res2.i$n2 <- tn1
              }
            }
            res2 <- rbind(res2, res2.i)
          }
        }
        #
        res <- res2
        #
        if (!nulldata) {
          res <- cbind(res, data)
          dupl <- duplicated(names(res))
          if (any(dupl))
            names(res)[dupl] <- paste(names(res)[dupl], "orig", sep = ".")
        }
      }
      
      
      #
      # Additional checks
      #
      #
      # a) Duplicate treatments ?
      #
      sel.treat <- as.character(res$treat1) == as.character(res$treat2)
      #
      if (any(sel.treat)) {
        sel.stud <- unique(sort(res$studlab[sel.treat]))
        #
        stop(paste0("Identical treatments for the following stud",
                    if (length(sel.stud) == 1) "y: " else "ies:\n  ",
                    paste0(paste0("'", sel.stud, "'"),
                           collapse = " - "),
                    "\n  Please check dataset."),
             call. = FALSE)
      }
      #
      # b) Studies missing ?
      #
      sel.study <- !(studlab %in% unique(as.character(res$studlab)))
      #
      if (any(sel.study) & warn)
        warning(paste0("The following stud",
                       if (sum(sel.study) == 1) "y is " else "ies are ",
                       "excluded from the analysis\n  ",
                       "(due to a single study arm or missing values):",
                       if (sum(sel.study) == 1) " " else "\n  ",
                       paste0(paste0("'", studlab[sel.study], "'"),
                              collapse = " - ")),
                call. = FALSE)
      #
      # c) Missing treatment estimates or standard errors?
      #
      if (type != "onlytreat" && nrow(res.NAs) > 0 & warn) {
        warning("Comparison",
                if (nrow(res.NAs) > 1) "s",
                " with missing TE / seTE or zero seTE",
                " will not be considered in network meta-analysis.",
                call. = FALSE)
        cat("Comparison",
            if (nrow(res.NAs) > 1) "s",
            " will not be considered in network meta-analysis:\n",
            sep = "")
        #
        res.NAs$.order <- NULL
        res.NAs$.order1 <- NULL
        res.NAs$.order2 <- NULL
        #
        prmatrix(res.NAs,
                 quote = FALSE, right = TRUE, na.print = "NA",
                 rowlab = rep("", nrow(res.NAs)))
      }
      
      
      # Calculate standard error for SMDs
      #
      if (type == "continuous" && m1$sm == "SMD") {
        res$.seTE <- res$seTE
        #
        for (i in unique(res$studlab)) {
          sel.i <- res$studlab == i
          dat.i <- res[sel.i, ]
          #
          # Calculate total sample size in study i
          #
          ndat.i <- rbind(data.frame(treat = dat.i$treat1, n = dat.i$n1),
                          data.frame(treat = dat.i$treat2, n = dat.i$n2))
          n.i <- sum(ndat.i[!duplicated(ndat.i), ]$n)
          #
          # Crippa & Orsini (2016), BMC Med Res Meth, equation (5)
          #
          varTE.i <-
            1 / res$n1[sel.i] + 1 / res$n2[sel.i] + res$TE[sel.i]^2 / (2 * n.i)
          #
          res$seTE[sel.i] <- sqrt(varTE.i)
        }
      }
      
      
      if (data.format != "comparison") {
        if (!is.null(res$.order1)) {
          res <- res[order(res$.order1), ]
          res$.order1 <- NULL
          res$.order2 <- NULL
          res$.order <- NULL
          res <- unique(res)
        }
        else if (!is.null(res$.order)) {
          res <- res[order(res$.order), ]
          res$.order <- NULL
          res$.order.orig <- NULL
          res <- unique(res)
        }
        else {
          res <- res[order(factor(res$studlab, levels = levs),
                           res$treat1, res$treat2), ]
        }
      }
      #
      rownames(res) <- 1:nrow(res)
    }
    #
    # Only keep core variables
    #
    corevars <- c("TE", "seTE", "studlab", "treat1", "treat2",
                  "event1", "event2", "n1", "n2",
                  "mean1", "mean2", "sd1", "sd2",
                  "TE1", "TE2", "seTE1", "seTE2",
                  "time1", "time2", "incr1", "incr2",
                  ".seTE")
    #
    if (!append.logical)
      res <- res[, names(res) %in% corevars]
    else if (is.character(append))
      res <- res[, names(res) %in% c(corevars, append)]
    #
    # Drop columns 'n1' and 'n2' if argument 'n' is missing 
    #
    if (type == "count" & missing.n) {
      res$n1 <- NULL
      res$n2 <- NULL
    }
  }
  
  #
  # Use first treatment with estimable effect as reference if
  # argument is missing
  #
  labels <- unique(sort(c(res$treat1, res$treat2)))
  #
  if (missing.reference.group) {
    if (type == "onlytreat")
      reference.group <- ""
    else {
      go.on <- TRUE
      i <- 0
      while (go.on) {
        i <- i + 1
        sel.i <-
          !is.na(res$TE) & !is.na(res$seTE) &
          (res$treat1 == labels[i] | res$treat2 == labels[i])
        if (sum(sel.i) > 0) {
          go.on <- FALSE
          reference.group <- labels[i]
          }
        else if (i == length(labels)) {
          go.on <- FALSE
          reference.group <- ""
        }
      }
    }
  }
  #
  if (is.factor(reference.group))
    reference.group <- as.character(reference.group)
  #
  if (is.numeric(reference.group))
    chknumeric(reference.group, length = 1)
  else
    chkchar(reference.group, length = 1)
  #
  reference.group <- setchar(reference.group, c(labels, ""))
  #
  if (!keep.all.comparisons) {
    #
    drop <- logical(0)
    #
    for (i in unique(res$studlab)) {
      d.i <- res[res$studlab == i, , drop = FALSE]
      trts.i <- unique(sort(c(d.i$treat1, d.i$treat2)))
      #
      # Keep comparisons with reference group or first treatment if
      # reference treatment is missing in study
      #
      if (reference.group %in% trts.i)
        ref.i <- reference.group
      else
        ref.i <- rev(trts.i)[1]
      #
      drop.i <- d.i$treat1 != ref.i & d.i$treat2 != ref.i
      #
      drop <- c(drop, drop.i)
    }
    #
    if (!keep.all.comparisons)
      res <- res[!drop, , drop = FALSE]
  }
  
  if (!avail.treat) {
    if (isCol(res, "agent1"))
      names(res)[names(res) == "agent1"] <- "agent1.orig"
    if (isCol(res, "dose1"))
      names(res)[names(res) == "dose1"] <- "dose1.orig"
    if (isCol(res, "agent2"))
      names(res)[names(res) == "agent2"] <- "agent2.orig"
    if (isCol(res, "dose2"))
      names(res)[names(res) == "dose2"] <- "dose2.orig"
    #
    res$agent1 <- sapply(compsplit(res$treat1, sep.ag), first)
    res$dose1 <- sapply(compsplit(res$treat1, sep.ag), second)
    res$agent2 <- sapply(compsplit(res$treat2, sep.ag), first)
    res$dose2 <- sapply(compsplit(res$treat2, sep.ag), second)
    #
    res$dose1 <- as.numeric(res$dose1)
    res$dose2 <- as.numeric(res$dose2)
    #
    nam.res <- names(res)
    nam1 <- c("TE", "seTE", "studlab")
    nam2 <- c("agent1", "dose1", "agent2", "dose2")
    #
    res <- res[, c(nam1, nam2, nam.res[!nam.res %in% c(nam1, nam2)])]
  }
  
  if (any(varnames != c("TE", "seTE"))) {
    names(res)[names(res) == "TE"] <- varnames[1]
    names(res)[names(res) == "seTE"] <- varnames[2]
  }
  
  attr(res, "pairwise") <- TRUE
  attr(res, "reference.group") <- reference.group
  attr(res, "keep.all.comparisons") <- keep.all.comparisons
  attr(res, "type") <- type
  #
  if (is.pairwise) {
    attr(res, "sm") <- res.attr$sm
    attr(res, "method") <- res.attr$method
    #
    attr(res, "incr") <- res.attr$incr
    attr(res, "method.incr") <- res.attr$method.incr
    #
    attr(res, "allstudies") <- res.attr$allstudies
  }
  else {
    attr(res, "sm") <- if (type != "onlytreat") replaceNULL(sm, "") else ""
    attr(res, "method") <- if (type != "onlytreat") method else ""
    #
    if (type %in% c("binary", "count")) {
      attr(res, "incr") <- incr
      attr(res, "method.incr") <- method.incr
      #
      if (type == "binary")
        attr(res, "allstudies") <- allstudies
    }
  }
  #
  attr(res, "varnames") <- varnames
  attr(res, "append") <- append
  attr(res, "append.logical") <- append.logical
  attr(res, "version") <- packageDescription("meta")$Version
  #
  attr(res, "args") <- args
  
  class(res) <- unique(c("pairwise", class(res)))
  
  res
}

#' Forest plot to display the result of a cumulative meta-analysis
#' 
#' @description
#' Draws a forest plot in the active graphics window (using grid
#' graphics system).
#' 
#' @aliases forest.metacum
#' 
#' @param x An object of class \code{\link{metacum}}.
#' @param leftcols A character vector specifying (additional) columns
#'   to be plotted on the left side of the forest plot or a logical
#'   value.
#' @param leftlabs A character vector specifying labels for
#'   (additional) columns on left side of the forest plot.
#' @param rightcols A character vector specifying (additional) columns
#'   to be plotted on the right side of the forest plot or a logical
#'   value.
#' @param rightlabs A character vector specifying labels for
#'   (additional) columns on right side of the forest plot.
#' @param prediction A logical indicating whether prediction
#'   intervals should be printed.
#' @param overall A logical indicating whether overall results should be
#'   shown.
#' @param just.addcols Justification of text for additional columns
#'   (possible values: "left", "right", "center").
#' @param smlab A label for the summary measure (printed at top of
#'   figure).
#' @param type A character string or vector specifying how to
#'   plot treatment effects and confidence intervals for cumulative
#'   meta-analysis results.
#' @param layout A character string specifying the layout of the
#'   forest plot (see \code{\link{forest.meta}}).
#' @param lab.NA A character string to label missing values.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in forest plots. If \code{backtransf = TRUE},
#'   results for \code{sm = "OR"} are presented as odds ratios rather
#'   than log odds ratios, for example.
#' @param big.mark A character used as thousands separator.
#' @param digits Minimal number of significant digits for treatment
#'   effects, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for
#'   p-values.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance.
#' @param digits.tau Minimal number of significant digits for square
#'   root of between-study variance.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   statistic.
#' @param digits.cid Minimal number of significant digits for
#'   CID / decision thresholds, see \code{print.default}.
#' @param digits.percent Minimal number of significant digits for
#'   probabilities, printed as percentages, see \code{print.default}.
#' @param col The colour for cumulative meta-analysis results (only considered
#'   if \code{type = "square"}).
#' @param col.bg The background colour for squares and diamonds of
#'   cumulative meta-analysis results.
#' @param col.border The colour for the outer lines of squares and diamonds of
#'   cumulative meta-analysis results.
#' @param col.bg.predict The background colour for prediction intervals of
#'   cumulative meta-analysis results.
#' @param col.border.predict The colour for the outer lines of prediction
#'   intervals of cumulative meta-analysis results.
#' @param addrows.below.overall A numeric value indicating how many
#'   empty rows are printed between meta-analysis results and
#'   meta-analysis details.
#' @param details A logical specifying whether details on statistical
#'   methods should be printed.
#' @param \dots Additional graphical arguments (passed on to
#'   \code{\link{forest.meta}}).
#' 
#' @details
#' A forest plot, also called confidence interval plot, is drawn in
#' the active graphics window. The forest functions in R package
#' \bold{meta} are based on the grid graphics system. In order to
#' print the forest plot, resize the graphics window and either use
#' \code{\link{dev.copy2eps}} or \code{\link{dev.copy2pdf}}. Another
#' possibility is to create a file using \code{\link{pdf}},
#' \code{\link{png}}, or \code{\link{svg}} and to specify the width
#' and height of the graphic (see \code{\link{forest.meta}} examples).
#' 
#' The arguments \code{leftcols} and \code{rightcols} can be used to
#' specify columns which are plotted on the left and right side of the
#' forest plot, respectively.
#' 
#' The arguments \code{leftlabs} and \code{rightlabs} can be used to
#' specify column headings which are plotted on left and right side of
#' the forest plot, respectively. For certain columns predefined
#' labels exist. For other columns, the column name will be used as a
#' label. It is possible to only provide labels for new columns (see
#' \code{\link{forest.meta}} examples). Otherwise the length of
#' \code{leftlabs} and \code{rightlabs} must be the same as the number
#' of printed columns, respectively. The value \code{NA} can be used
#' to specify columns which should use default labels.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.meta}}, \code{\link{metacum}},
#'   \code{\link{settings.meta}}
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Fleiss1993bin)
#' ma <- metabin(d.asp, n.asp, d.plac, n.plac,
#'   data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
#' ma
#' metacum(ma)
#' metacum(ma, pooled = "random")
#' 
#' forest(metacum(ma))
#' forest(metacum(ma, pooled = "random"))
#' forest(metacum(ma, pooled = "random", prediction = TRUE))
#'
#' @method forest metacum
#' @export

forest.metacum <- function(x,
                           #
                           leftcols = NULL, leftlabs = NULL,
                           rightcols = NULL, rightlabs = NULL,
                           #
                           prediction = x$prediction,
                           overall = x$overall,
                           just.addcols = "right",
                           smlab = "Cumulative Meta-Analysis",
                           type = "square",
                           layout = gs("layout"),
                           lab.NA = ".",
                           #
                           backtransf = x$backtransf,
                           #
                           big.mark = gs("big.mark"),
                           digits = gs("digits.forest"),
                           digits.pval = gs("digits.pval"),
                           digits.tau2 = gs("digits.tau2"),
                           digits.tau = gs("digits.tau"),
                           digits.I2 = gs("digits.I2"),
                           digits.cid = gs("digits.cid"),
                           digits.percent = 1,
                           #
                           col = gs("col.study"),
                           col.bg = 
                             ifelse(type == "diamond",
                                  gs("col.diamond"), gs("col.square")),
                           col.border =
                             ifelse(type == "diamond",
                                    gs("col.diamond.lines"),
                                    gs("col.square.lines")),
                           col.bg.predict = gs("col.predict"),
                           col.border.predict = gs("col.predict.lines"),
                           #
                           addrows.below.overall = 1L * details,
                           details = gs("forest.details"),
                           ...) {
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  chkclass(x, c("metacum", "metainf"))
  x <- updateversion(x)
  #
  type <- setchar(type, c("square", "diamond", "circle", "squarediamond"))
  #
  just.addcols <- setchar(just.addcols, c("left", "center", "right"))
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  common <- x$pooled == "common"
  #
  if (!missing(prediction))
    prediction <- catch("prediction", mc, x, sfsp)
  #
  k.all <- length(x$TE)
  #
  if (length(prediction) > 1 && length(prediction) != k.all)
    stop("Argument 'prediction' must be of length 1 or number of studies.",
         call. = FALSE)
  #
  if (!is.logical(prediction))
    stop("Argument 'prediction' must be of type logical.",
         call. = FALSE)
  #
  if (length(prediction) == 1)
    prediction <- rep(prediction, k.all)
  #
  prediction <- prediction & !common
  #
  chklogical(overall)
  #
  layout <- setchar(layout, c("meta", "BMJ", "RevMan5", "JAMA"))
  #
  chklogical(backtransf)
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  chknumeric(digits.cid, min = 0, length = 1)
  chknumeric(digits.percent, min = 0, length = 1)
  chknumeric(addrows.below.overall, length = 1)
  #
  chklogical(details)
  #
  missing.leftcols <- missing(leftcols)
  missing.rightcols <- missing(rightcols)
  #
  missing.col.bg <- missing(col.bg)
  missing.col.border <- missing(col.border)
  
  
  avail.prop.cid.below.null <-
    !is.null(x$prop.cid.below.null) && !(all(is.na(x$prop.cid.below.null)))
  avail.prop.cid.above.null <-
    !is.null(x$prop.cid.above.null) && !(all(is.na(x$prop.cid.above.null)))
  #
  avail.prop.cid <- avail.prop.cid.below.null | avail.prop.cid.above.null
  #
  pvalNA <- all(is.na(x$pval))
  #
  if (is.null(leftcols))
    leftcols <- "studlab"
  #
  if (is.null(leftlabs))
    leftlabs <- rep(NA, length(leftcols))
  #
  if (is.null(rightcols)) {
    rightcols <- c("effect", "ci", if (!pvalNA) "pval", "tau2", "tau", "I2")
    #
    if (avail.prop.cid.below.null)
      rightcols <- c(rightcols, "prop.cid.below.null")
    #
    if (avail.prop.cid.above.null)
      rightcols <- c(rightcols, "prop.cid.above.null")
  }
  #
  if (is.null(rightlabs))
    rightlabs <- rep(NA, length(rightcols))
  #
  print.tau2 <- any(c("tau2" %in% leftcols, "tau2" %in% rightcols))
  print.tau <- any(c("tau" %in% leftcols, "tau" %in% rightcols))
  print.I2 <- any(c("I2" %in% leftcols, "I2" %in% rightcols))
  #
  print.cid.below.null <- any(c("prop.cid.below.null" %in% leftcols,
                                "prop.cid.below.null" %in% rightcols))
  #
  print.cid.above.null <- any(c("prop.cid.above.null" %in% leftcols,
                                "prop.cid.above.null" %in% rightcols))
  #
  pval <- formatPT(x$pval, digits = digits.pval, lab.NA = lab.NA)
  tau2 <- formatPT(x$tau2, digits = digits.tau2, lab.NA = lab.NA)
  tau <- formatPT(x$tau, digits = digits.tau2, lab.NA = lab.NA)
  I2 <- ifelse(is.na(x$I2), lab.NA,
               paste0(formatPT(100 * x$I2, digits = digits.I2,
                               lab.NA = lab.NA), "%"))
  #
  if (avail.prop.cid.below.null) {
    x$prop.cid.below.null <-
      ifelse(is.na(x$prop.cid.below.null), lab.NA,
             paste0(formatPT(100 * x$prop.cid.below.null,
                             digits = digits.percent), "%"))
  }
  #
  if (avail.prop.cid.above.null) {
    x$prop.cid.above.null <-
      ifelse(is.na(x$prop.cid.above.null), lab.NA,
             paste0(formatPT(100 * x$prop.cid.above.null,
                             digits = digits.percent), "%"))
  }
  #
  x.tmp <- x
  x.tmp$prediction <- any(prediction)
  class(x.tmp) <-
    c(class(x.tmp), if (inherits(x, "metainf")) "metainf" else "metacum")
  #
  text.details <-
    catmeth(x.tmp,
            x$common, x$random, any(prediction), overall, TRUE,
            #
            func.transf = x$func.transf,
            backtransf = backtransf, func.backtransf = x$func.backtransf,
            #
            big.mark = big.mark, digits = digits,
            digits.tau = digits.tau,
            text.tau = gs("text.tau"), text.tau2 = gs("text.tau2"),
            #
            print.tau2 = print.tau2 | x$pooled == "random",
            print.tau2.ci = FALSE,
            print.tau = print.tau | x$pooled == "random",
            print.tau.ci = FALSE,
            #
            print.I2 = print.I2, text.I2 = gs("text.I2"),
            #
            print.df = TRUE, prediction.subgroup = FALSE,
            #
            forest = TRUE)
  #
  if (avail.prop.cid)
    svd <- x$small.values == "desirable"
  #
  if (avail.prop.cid.below.null) {
    text.details <-
      paste0(text.details,
             paste0("\n- Lower decision threshold (",
                    if (svd) "beneficial " else "harmful ",
                    "effects): ",
                    formatN(x$cid.below.null, digits = digits.cid,
                            big.mark = big.mark)))
  }
  #
  if (avail.prop.cid.above.null) {
    text.details <-
      paste0(text.details,
             paste0("\n- Upper decision threshold (",
                    if (svd) "harmful " else "beneficial ",
                    "effects): ",
                    formatN(x$cid.above.null, digits = digits.cid,
                            big.mark = big.mark)))
  }
  
  
  #
  # Print prediction intervals in separate rows
  #
  if (any(prediction)) {
    TE <- as.vector(
      matrix(c(x$TE, rep(NA, k.all)),
             ncol = k.all, byrow = TRUE))
    #
    seTE <- as.vector(
      matrix(c(x$seTE, rep(NA, k.all)),
             ncol = k.all, byrow = TRUE))
    #
    lower <- as.vector(
      matrix(c(x$lower, x$lower.predict),
               ncol = k.all, byrow = TRUE))
    #
    upper <- as.vector(
      matrix(c(x$upper,
               x$upper.predict),
               ncol = k.all, byrow = TRUE))
    #
    studlab <- as.vector(
      matrix(c(x$studlab,
               rep("", k.all)),
               ncol = k.all, byrow = TRUE))
    #
    if (print.cid.below.null)
      prop.cid.below.null <- as.vector(
        matrix(c(x$prop.cid.below.null,
                 rep("", k.all)),
               ncol = k.all, byrow = TRUE))
    #
    if (print.cid.above.null)
      prop.cid.above.null <- as.vector(
        matrix(c(x$prop.cid.above.null,
                 rep("", k.all)),
               ncol = k.all, byrow = TRUE))
    #
    if (!is.null(col.bg))
      col.bg <- as.vector(
        matrix(c(rep(col.bg, k.all),
                 rep(col.bg.predict, k.all)),
               ncol = k.all, byrow = TRUE))
    #
    if (!is.null(col.border))
      col.border <- as.vector(
        matrix(c(rep(col.border, k.all),
                 rep(col.border.predict, k.all)),
               ncol = k.all, byrow = TRUE))
    #
    sel.pred <- as.vector(
      matrix(c(rep(TRUE, k.all), prediction),
             ncol = k.all, byrow = TRUE))
    #
    m <- metagen(TE[sel.pred], seTE[sel.pred], studlab = studlab[sel.pred],
                 common = common, random = !common,
                 prediction = any(prediction),
                 overall = overall,
                 sm = x$sm,
                 backtransf = backtransf,
                 func.backtransf = x$func.backtransf,
                 #
                 null.effect = x$null.effect,
                 #
                 label.left = x$label.left,
                 label.right = x$label.right,
                 #
                 method.tau = "DL")
    #
    m$lower <- lower[sel.pred]
    m$upper <- upper[sel.pred]
    #
    m$pval <- as.vector(
      matrix(c(pval, rep("", k.all)),
             ncol = k.all, byrow = TRUE))[sel.pred]
    #
    m$I2 <- as.vector(
      matrix(c(I2, rep("", k.all)),
             ncol = k.all, byrow = TRUE))[sel.pred]
    m$tau2 <- as.vector(
      matrix(c(tau2, rep("", k.all)),
             ncol = k.all, byrow = TRUE))[sel.pred]
    m$tau <- as.vector(
      matrix(c(tau, rep("", k.all)),
             ncol = k.all, byrow = TRUE))[sel.pred]
    #
    if (print.cid.below.null)
      m$prop.cid.below.null <- prop.cid.below.null[sel.pred]
    #
    if (print.cid.above.null)
      m$prop.cid.above.null <- prop.cid.above.null[sel.pred]
    #
    type.study <- rep(c(type, "predict"), k.all)[sel.pred]
    #
    if (!is.null(col.bg))
      col.bg <- col.bg[sel.pred]
    if (!is.null(col.border))
      col.border <- col.border[sel.pred]
  }
  #
  else {
    TE <- x$TE
    seTE <- x$seTE
    #
    lower <- x$lower
    upper <- x$upper
    #
    studlab <- x$studlab
    #
    m <- metagen(TE, seTE, studlab = studlab,
                 common = common, random = !common,
                 prediction = any(prediction),
                 overall = overall,
                 sm = x$sm,
                 backtransf = backtransf,
                 func.backtransf = x$func.backtransf,
                 #
                 null.effect = x$null.effect,
                 #
                 label.left = x$label.left,
                 label.right = x$label.right,
                 #
                 method.tau = "DL")
    #
    m$lower <- lower
    m$upper <- upper
    #
    m$pval <- pval
    m$I2 <- I2
    m$tau2 <- tau2
    m$tau <- tau
    #
    if (print.cid.below.null)
      m$prop.cid.below.null <- x$prop.cid.below.null
    #
    if (print.cid.below.null)
      m$prop.cid.above.null <- x$prop.cid.above.null
    #
    type.study <- type
  }
  #
  m$TE.common <- x$TE.pooled
  m$lower.common <- x$lower.pooled
  m$upper.common <- x$upper.pooled
  m$statistic.common <- x$statistic.pooled
  m$pval.common <- x$pval.pooled
  #
  m$TE.random <- x$TE.pooled
  m$lower.random <- x$lower.pooled
  m$upper.random <- x$upper.pooled
  m$statistic.random <- x$statistic.pooled
  m$pval.random <- x$pval.pooled
  #
  m$df.random <- x$df.random.pooled
  #
  if (any(prediction)) {
    m$lower.predict <- x$lower.predict.pooled
    m$upper.predict <- x$upper.predict.pooled
    m$method.predict <- x$method.predict
  }
  #
  m$level <- x$level.ma <- x$level.ma
  m$level.predict <- x$level.predict
  #
  m$method <- x$method
  m$method.random <- x$method.random
  #
  m$method.random.ci <- x$method.random.ci
  m$adhoc.hakn.ci <- x$adhoc.hakn.ci
  #
  m$method.tau <- x$method.tau
  m$method.tau.ci <- x$method.tau.ci
  #
  m$tau.preset <- x$tau.preset
  m$TE.tau <- x$TE.tau
  #
  m$method.I2 <- x$method.I2
  #
  m$k <- x$k.pooled
  m$k.study <- x$k.study.pooled
  m$k.all <- x$k.all.pooled
  m$k <- x$k.TE.pooled
  #
  if (any(rightcols %in% c("ci", "effect.ci")) |
      any(leftcols %in% c("ci", "effect.ci"))) {
    level.ma <- x$level.ma
    level.predict <- x$level.predict
    #
    if (any(prediction)) {
      if (level.ma == level.predict)
        ci.lab <- paste0(100 * level.ma, "%-CI/PI")
      else
        ci.lab <-
          paste0(100 * level.ma, "%-CI / ", 100 * level.predict, "%-PI")
    }
    else
      ci.lab <- paste0(100 * level.ma, "%-CI")
    #
    sel.left <- leftcols == "ci"
    #
    if (any(sel.left) && is.na(leftlabs[sel.left]))
      leftlabs[sel.left] <- ci.lab
    #
    sel.right <- rightcols == "ci"
    #
    if (any(sel.right) && is.na(rightlabs[sel.right]))
      rightlabs[sel.right] <- ci.lab
    #
    sel.left <- leftcols == "effect.ci"
    #
    if (any(sel.left) && is.na(leftlabs[sel.left]))
      leftlabs[sel.left] <-
        paste(smlab(x$sm, backtransf, x$pscale, x$irscale), ci.lab)
    #
    sel.right <- rightcols == "effect.ci"
    #
    if (any(sel.right) && is.na(rightlabs[sel.right]))
      rightlabs[sel.right] <-
        paste(smlab(x$sm, backtransf, x$pscale, x$irscale), ci.lab)
  }
  #
  # Set column labels for decision threshold probabilites
  #
  if (print.cid.below.null) {
    sel.left <- leftcols == "prop.cid.below.null"
    #
    if (any(sel.left) && is.na(leftlabs[sel.left]))
      leftlabs[sel.left] <-
        paste0("P(",
               if (x$small.values == "desirable") "benefit" else "harm",
               ")")
    #
    sel.right <- rightcols == "prop.cid.below.null"
    #
    if (any(sel.right) && is.na(rightlabs[sel.right]))
      rightlabs[sel.right] <-
      paste0("P(",
             if (x$small.values == "desirable") "benefit" else "harm",
             ")")
  }
  #
  if (print.cid.above.null) {
    sel.left <- leftcols == "prop.cid.above.null"
    #
    if (any(sel.left) && is.na(leftlabs[sel.left]))
      leftlabs[sel.left] <-
        paste0("P(",
               if (x$small.values == "desirable") "harm" else "benefit",
               ")")
    #
    sel.right <- rightcols == "prop.cid.above.null"
    #
    if (any(sel.right) && is.na(rightlabs[sel.right]))
      rightlabs[sel.right] <-
      paste0("P(",
             if (x$small.values == "desirable") "harm" else "benefit",
             ")")
  }
  #
  m$.text.details.methods <- text.details
  #
  # Move columns to left side of forest plot for JAMA and RevMan5 layouts
  #
  if (missing.leftcols & missing.rightcols & layout %in% c("JAMA", "RevMan5")) {
    leftcols <- c(leftcols, rightcols[-(1:2)], rightcols[1:2])
    rightcols <- NULL
    leftlabs <- c(leftlabs, rightlabs[-(1:2)], rightlabs[1:2])
    rightlabs <- NULL
  }
  #
  # Use default colours for JAMA and RevMan5 layouts
  #
  if (missing.col.bg & layout %in% c("JAMA", "RevMan5"))
    col.bg <- NULL
  #
  if (missing.col.border & layout %in% c("JAMA", "RevMan5"))
    col.border <- NULL
  
  
  data.p <-
    data.frame(pval = formatPT(x$pval.pooled, digits = digits.pval,
                               lab.NA = lab.NA),
               tau2 = formatPT(x$tau2.pooled, digits = digits.tau2,
                               lab.NA = lab.NA),
               tau = formatPT(x$tau.pooled, digits = digits.tau,
                              lab.NA = lab.NA),
               I2 = ifelse(is.na(x$I2.pooled), lab.NA,
                           paste0(formatPT(100 * x$I2.pooled,
                                           digits = digits.I2,
                                           lab.NA = lab.NA), "%")))
  #
  if (avail.prop.cid.below.null) {
    data.p$prop.cid.below.null <-
      ifelse(is.na(x$prop.cid.below.null.pooled), lab.NA,
             paste0(formatPT(100 * x$prop.cid.below.null.pooled,
                             digits = digits.percent), "%"))
  }
  #
  if (avail.prop.cid.above.null) {
    data.p$prop.cid.above.null <-
      ifelse(is.na(x$prop.cid.above.null.pooled), lab.NA,
             paste0(formatPT(100 * x$prop.cid.above.null.pooled,
                             digits = digits.percent), "%"))
  }
  
  
  class(m) <- c(class(m), class(x))
  m$classes <- x$classes
  #
  dots_list <- drop_from_dots(list(...),
                              c("col.study", "col.square", "col.square.lines",
                                "overall.hetstat", "overall.hetstat",
                                "data.pooled"),
                              c("col", "col.bg", "col.border",
                                "col.bg.predict", "col.border.predict",
                                ""))
  #
  args_list <-
    list(x = m,
         leftcols = leftcols, leftlabs = leftlabs,
         rightcols = rightcols, rightlabs = rightlabs,
         overall.hetstat = FALSE,
         type.study = type.study,
         weight.study = "same",
         lab.NA = lab.NA, smlab = smlab,
         data.pooled = data.p,
         just.addcols = just.addcols,
         #
         layout = layout,
         #
         backtransf = backtransf,
         #
         big.mark = big.mark,
         #
         digits = digits,
         digits.tau2 = digits.tau2,
         digits.tau = digits.tau,
         digits.I2 = digits.I2,
         #
         addrows.below.overall = addrows.below.overall,
         #
         col.study = col,
         col.square = col.bg, col.square.lines = col.border,
         #
         details = details)
  #
  res <- do.call("forest.meta", c(args_list, dots_list))
  #
  invisible(res)
}

xlab_meta <- function(sm, backtransf,
                      pscale = 1, irscale = 1, irunit = "person-years",
                      newline = FALSE, revman5 = FALSE,
                      big.mark = gs("big.mark"),
                      func.transf = NULL, func.backtransf = NULL) {
  
  res <- sm
  
  
  newline <- if (newline) "\n" else " "
  
  
  if (sm == "SMD")
    res <- paste0(if (revman5) "Std. Mean" else "Standardised Mean",
                  newline, "Difference")
  #
  else if (sm == "WMD" | sm == "MD")
    res <- "Mean Difference"
  #
  else if (sm == "COR")
    res <- "Correlation"
  #
  else if (sm == "RD")
    if (pscale == 1)
      res <- "Risk Difference"
    else
      res <- paste0("Risk Difference\n(events per ",
                    format(pscale, scientific = FALSE, big.mark = big.mark),
                    " obs.)")
  #
  else if (sm == "ASD")
    res <- paste0("Arcus Sinus", newline, "Difference")
  #
  else if (sm == "IRD")
    if (irscale == 1)
      res <- paste0("Incidence Rate", newline, "Difference")
    else
      res <- paste0("Incidence Rate Diff.\n(events per ",
                    format(irscale, scientific = FALSE, big.mark = big.mark),
                    newline, irunit)
  #
  else if (sm == "IR")
    res <- "Incidence Rate"
  #
  else if (sm == "MRAW")
    res <- "Mean"
  #
  else if (backtransf) {
    if (sm == "OR")
      res <- "Odds Ratio"
    #
    else if (sm == "RR")
      res <- "Risk Ratio"
    #
    else if (sm == "DOR")
      res <- "Diagnostic Odds Ratio"
    #
    else if (sm == "ROM")
      res <- "Ratio of Means"
    #
    else if (sm == "ZCOR")
      res <- "Correlation"
    #
    else if (sm == "HR")
      res <- "Hazard Ratio"
    #
    else if (sm == "IRR")
      res <- paste0("Incidence Rate", newline, "Ratio")
    #
    else if (sm == "VE")
      res <- "Vaccine Eff."
    #
    else if (is_prop(sm)) {
      if (pscale == 1)
        res <- ""
      else
        res <- paste0("Events per ",
                      format(pscale, scientific = FALSE, big.mark = big.mark),
                      newline, "observations")
    }
    #
    else if (is_rate(sm)) {
      if (irscale == 1)
        res <- "Incidence Rate"
      else
        res <- paste0("Events per ",
                      format(irscale, scientific = FALSE, big.mark = big.mark),
                      newline, irunit)
    }
    #
    else if (sm == "MLN")
      res <- "Mean"
  }
  else {
    if (sm == "OR")
      res <- "Log Odds Ratio"
    #
    else if (sm == "RR")
      res <- "Log Risk Ratio"
    #
    else if (sm == "DOR")
      res <- "Log Diagnostic Odds Ratio"
    #
    else if (sm == "ROM")
      res <- paste0("Log Ratio of", newline, "Means")
    #
    else if (sm == "HR")
      res <- paste0("Log Hazard", newline, "Ratio")
    #
    else if (sm == "IRR")
      res <- paste0("Log Incidence Rate", newline, "Ratio")
    #
    else if (sm == "ZCOR")
      res <- paste0("Fisher's z transformed", newline, "correlation")
    #
    else if (sm == "VE")
      res <- "Log Risk Ratio"
    #
    else if (sm == "PFT")
      res <- paste0("Freeman-Tukey Double Arcsine", newline,
                    "Transformed Proportion")
    #
    else if (sm == "PAS")
      res <- paste0("Arcsine Transformed", newline, "Proportion")
    #
    else if (sm == "PLN")
      res <- paste0("Log Transformed", newline, "Proportion")
    #
    else if (sm == "PLOGIT")
      res <- paste0("Logit Transformed", newline, "Proportion")
    #
    else if (sm == "PRAW")
      res <- paste0("Untransformed", newline, "Proportion")
    #
    else if (sm == "IR")
      res <- "Incidence Rate"
    #
    else if (sm == "IRLN")
      res <- paste0("Log Incidence", newline, "Rate")
    #
    else if (sm == "IRS")
      res <- paste0("Square Root of", newline, "Incidence Rate")
    #
    else if (sm == "IRFT")
      res <- paste0("Freeman-Tukey Double Arcsine", newline,
                   "Transformed Rate")
    #
    else if (sm == "MLN")
      res <- "Log Mean"
    else if (!is.null(func.transf))
      res <- paste0(func.transf, "(", sm, ")")
    else if (!is.null(func.backtransf)) {
      if (func.backtransf == "exp")
        res <- paste0("log(", sm, ")")
      else if (func.backtransf == "z2cor")
        res <-  paste0("Fisher's z transformed", newline, "correlation")
      else if (func.backtransf == "logit2p")
        res <-  paste0("Logit Transformed", newline, "Proportion")
      else if (func.backtransf == "logVR2VE")
        res <-  "Log Vaccine Ratio"
    }
  }
  
  if (is.null(res))
    res <- sm

  
  res
}

#' Drapery plot
#' 
#' @description
#' Draw a drapery plot with (scaled) p-value curves for individual
#' studies and meta-analysis estimates.
#' 
#' @aliases drapery
#' 
#' @param x An object of class \code{meta}.
#' @param type A character string indicating whether to plot test
#'   statistics (\code{"zvalue"}) or p-values (\code{"pvalue"}), can
#'   be abbreviated.
#' @param layout A character string for the line layout of individual
#'   studies: \code{"grayscale"}, \code{"equal"}, or
#'   \code{"linewidth"} (see Details), can be abbreviated.
#' @param study.results A logical indicating whether results for
#'   individual studies should be shown in the figure.
#' @param lty.study Line type for individual studies.
#' @param lwd.study Line width for individual studies.
#' @param col.study Colour of lines for individual studies.
#' @param labels A logical or character string indicating whether
#'   study labels should be shown at the top of the drapery plot;
#'   either \code{FALSE}, \code{"id"}, or \code{"studlab"}; see
#'   Details.
#' @param col.labels Colour of study labels.
#' @param cex.labels The magnification for study labels.
#' @param subset.labels A vector specifying which study labels should
#'   be shown in the drapery plot.
#' @param srt.labels A numerical vector or single numeric (between 0
#'   and 90) specifying the angle to rotate study labels; see Details.
#' @param common A logical indicating whether to show result for the
#'   common effect model.
#' @param random A logical indicating whether to show result for the
#'   random effects model.
#' @param lty.common Line type for common effect meta-analysis.
#' @param lwd.common Line width for common effect meta-analysis.
#' @param col.common Colour of lines for common effect meta-analysis.
#' @param lty.random Line type for random effects meta-analysis.
#' @param lwd.random Line width for random effects meta-analysis.
#' @param col.random Colour of lines for random effects meta-analysis.
#' @param prediction A logical indicating whether to show prediction
#'   region.
#' @param col.predict Colour of prediction region
#' @param sign Significance level used to highlight significant values
#'   in curves.
#' @param lty.sign Line type for significant values.
#' @param lwd.sign Line width for significant values.
#' @param col.sign Line colour for significant values.
#' @param alpha Horizonal lines are printed for the specified alpha
#'   values.
#' @param lty.alpha Line type of horizonal lines for alpha values.
#' @param lwd.alpha Line width of horizonal lines for alpha values.
#' @param col.alpha Colour of horizonal lines for alpha values.
#' @param cex.alpha The magnification for the text of the alpha
#' @param col.null.effect Colour of vertical line indicating null
#'   effect.
#' @param legend A logical indicating whether a legend should be
#'   printed.
#' @param pos.legend A character string with position of legend (see
#'   \code{\link{legend}}).
#' @param bg Background colour of legend (see \code{\link{legend}}).
#' @param bty Type of the box around the legend; either \code{"o"} or
#'   \code{"n"} (see \code{\link{legend}}).
#' @param backtransf A logical indicating whether results should be
#'   back transformed on the x-axis. For example, if \code{backtransf
#'   = FALSE}, log odds ratios instead of odds ratios are shown on the
#'   x-axis.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param xlim The x limits (min, max) of the plot.
#' @param ylim The y limits (min, max) of the plot (ignored if
#'   \code{type = "pvalue"}).
#' @param lwd.max The maximum line width (only considered if argument
#'   \code{layout} is equal to \code{"linewidth"}).
#' @param lwd.study.weight A character string indicating whether to
#'   determine line width for individual studies using weights from
#'   common effect (\code{"common"}) or random effects model
#'   (\code{"random"}), can be abbreviated (only considered if
#'   argument \code{layout} is equal to \code{"linewidth"}).
#' @param at Points at which tick-marks are to be drawn on the x-axis.
#' @param n.grid The number of grid points to calculate the p-value or
#'   test statistic functions.
#' @param mar Physical plot margin, see \code{\link{par}}.
#' @param plot A logical indicating whether to generate a figure.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param fixed Deprecated argument (replaced by 'common').
#' @param lwd.fixed Deprecated argument (replaced by 'lwd.common').
#' @param lty.fixed Deprecated argument (replaced by 'lty.common').
#' @param col.fixed Deprecated argument (replaced by 'col.common').
#' @param \dots Graphical arguments as in \code{par} may also be
#'   passed as arguments.
#' 
#' @details
#' The concept of a p-value function, also called confidence curve,
#' goes back to Birnbaum (1961). A drapery plot, showing p-value
#' functions (or a scaled version based on the corresponding test
#' statistics) for individual studies as well as meta-analysis
#' estimates, is drawn in the active graphics window. Furthermore, a
#' prediction region for a single future study is shown as a shaded
#' area. In contrast to a forest plot, a drapery plot does not provide
#' information for a single confidence level however for any
#' confidence level.
#'
#' Argument \code{type} can be used to either show p-value functions
#' (Birnbaum, 1961) or a scaled version (Infanger, 2019) with test
#' statistics (default).
#' 
#' Argument \code{layout} determines how curves for individual studies
#' are presented:
#' \itemize{
#' \item darker gray tones with increasing precision (\code{layout =
#'   "grayscale"})
#' \item thicker lines with increasing precision (\code{layout =
#'   "linewidth"})
#' \item equal lines (\code{layout = "equal"})
#' }
#'
#' Argument \code{labels} determines how curves of individual studies
#' are labelled:
#' \itemize{
#' \item number of the study in the (unsorted) forest plot / printout
#'   of a meta-analysis (\code{labels = "id"})
#' \item study labels provided by argument \code{studlab} in
#'   meta-analysis functions (\code{labels = "studlab"})
#' \item no study labels (\code{labels = FALSE})
#' }
#' By default, study labels are used (\code{labels = "studlab"}) if no
#' label has more than three characters; otherwise IDs are used
#' (\code{labels = "id"}). The connection between IDs and study labels
#' (among other information) is part of a data frame which is
#' invisibly returned (if argument \code{ study.results = TRUE}).
#'
#' Argument \code{srt.labels} can be used to change the rotation of
#' IDs or study labels. By default, study labels are rotated by +/- 45
#' degrees if at least one study label has more than three characters;
#' otherwise labels are not rotated.
#' 
#' If \code{labels = "studlab"}, labels are rotated by -45 degrees for
#' studies with a treatment estimate below the common effect estimate
#' and otherwise by 45 degrees.
#' 
#' @author Gerta Rücker \email{gerta.ruecker@@uniklinik-freiburg.de}, Guido
#'   Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest}}, \code{\link{radial}}
#' 
#' @references
#' 
#' Birnbaum A (1961):
#' Confidence Curves: An Omnibus Technique for Estimation and Testing
#' Statistical Hypotheses.
#' \emph{Journal of the American Statistical Association},
#' \bold{56}, 246--9
#'
#' Infanger D and Schmidt-Trucksäss A (2019):
#' P value functions: An underused method to present research results
#' and to promote quantitative reasoning
#' \emph{Statistics in Medicine},
#' \bold{38}, 4189--97
#' 
#' @keywords hplot
#' 
#' @examples
#' data("lungcancer")
#' ma1 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
#'   data = lungcancer, studlab = study)
#' 
#' # Drapery plot
#' #
#' drapery(ma1, xlim = c(0.5, 50))
#' 
#' \dontrun{
#' data(Fleiss1993bin)
#' ma2 <- metabin(d.asp, n.asp, d.plac, n.plac,
#'   data = Fleiss1993bin, studlab = paste(study, year),
#'   sm = "OR", random = FALSE)
#'
#' # Produce drapery plot and print data frame with connection between
#' # IDs and study labels
#' #
#' (drapery(ma2))
#' 
#' # For studies with a significant effect (p < 0.05), show
#' # study labels and print labels and lines in red
#' #
#' drapery(ma2,
#'   labels = "studlab", subset.labels = pval < 0.05,
#'   srt.labels = 0, col.labels = "red",
#'   col.study = ifelse(pval < 0.05, "red", "darkgray"))
#' }
#' 
#' @rdname drapery
#' @export drapery

drapery <- function(x, type = "zvalue", layout = "grayscale",
                    #
                    study.results = TRUE,
                    lty.study = 1, lwd.study = 1, col.study = "darkgray",
                    #
                    labels, col.labels = "black", cex.labels = 0.7,
                    subset.labels, srt.labels,
                    #
                    common = x$common, random = x$random,
                    lty.common = 1, lwd.common = max(3, lwd.study),
                    col.common = "blue",
                    lty.random = 1, lwd.random = lwd.common,
                    col.random = "red",
                    #
                    sign = NULL,
                    lty.sign = 1, lwd.sign = 1, col.sign = "black",
                    prediction = random, col.predict = "lightblue",
                    #
                    alpha = if (type == "zvalue") c(0.001, 0.01, 0.05, 0.1)
                            else c(0.01, 0.05, 0.1),
                    lty.alpha = 2, lwd.alpha = 1, col.alpha = "black",
                    cex.alpha = 0.7,
                    col.null.effect = "black",
                    #
                    legend = TRUE, pos.legend = "topleft",
                    bg = "white", bty = "o",
                    #
                    backtransf = x$backtransf,
                    xlab, ylab,
                    xlim, ylim,
                    lwd.max = 2.5,
                    lwd.study.weight = if (random) "random" else "common",
                    at = NULL,
                    n.grid = if (type == "zvalue") 10000 else 1000,
                    mar = c(5.1, 4.1, 4.1, 4.1),
                    plot = TRUE,
                    #
                    warn.deprecated = gs("warn.deprecated"),
                    fixed, lwd.fixed, lty.fixed, col.fixed,
                    #
                    ...) {
  
  
  pvf <- function(x, TE, seTE)
    2 * pnorm(abs(TE - x) / seTE, lower.tail = FALSE)
  
  
  if (plot) {
    oldpar <- par(mar = mar)
    on.exit(par(oldpar))
  }
  
  
  #
  #
  # (1) Check and set arguments
  #
  #
  chkclass(x, "meta")
  x <- updateversion(x)
  #
  seq.TE <- seq(along = x$TE)
  k.all <- length(seq.TE)
  #
  mf <- match.call()
  fun <- "drapery"
  #
  types <- c("zvalue", "pvalue", "surprisal")
  type <- setchar(type, types)
  layout <- setchar(layout, c("grayscale", "linewidth", "equal"))
  #
  chklogical(study.results)
  #
  # Argument lty.study
  #
  if (!missing(lty.study)) {
    lty.study <- catchvar("lty.study", x, mf)
    chknumeric(lty.study, min = 0, zero = TRUE)
  }
  lty.study <- augment(lty.study, k.all, fun)
  #
  # Argument lwd.study
  #
  missing.lwd.study <- missing(lwd.study)
  #
  if (!missing.lwd.study) {
    lwd.study <- catchvar("lwd.study", x, mf)
    chknumeric(lwd.study, min = 0, zero = TRUE)
  }
  lwd.study <- augment(lwd.study, k.all, fun)
  #
  # Argument col.study
  #
  missing.col.study <- missing(col.study)
  #
  if (!missing.col.study)
    col.study <- catchvar("col.study", x, mf)
  col.study <- augment(col.study, k.all, fun)
  #
  # Argument labels
  #
  if (missing(labels) || (is.logical(labels) && labels)) {
    if (max(nchar(x$studlab)) < 4)
      labels <- "studlab"
    else
      labels <- "id"
  }
  else {
    if (is.logical(labels) && !labels)
      labels <- ""
    else
      labels <- setchar(labels, c("", "id", "studlab"))
  }
  #
  # Argument col.labels
  #
  if (!missing(col.labels))
    col.labels <- catchvar("col.labels", x, mf)
  col.labels <- augment(col.labels, k.all, fun)
  #
  # Argument cex.labels
  #
  if (!missing(cex.labels)) {
    cex.labels <- catchvar("cex.labels", x, mf)
    chknumeric(cex.labels, min = 0, zero = TRUE)
  }
  cex.labels <- augment(cex.labels, k.all, fun)
  #
  # Argument subset.labels
  #
  if (missing(subset.labels))
    subset.labels <- rep(TRUE, k.all)
  else {
    subset.labels <- catchvar("subset.labels", x, mf)
    chklength(subset.labels, k.all, fun)
    #
    if (is.numeric(subset.labels)) {
      if (any(!(subset.labels %in% seq.TE)))
        stop("Numerical input for argument 'subset.labels' ",
             "must contain integers between 1 and ",
             k.all, ".",
             call. = FALSE)
      sel.labels <- rep(FALSE, k.all)
      sel.labels[subset.labels] <- TRUE
    }
    else if (!is.logical(subset.labels))
      stop("Argument 'subset.labels' must be a logical or ",
           "numeric vector.",
           call. = FALSE)
  }
  #
  # Argument srt.labels
  #
  if (missing(srt.labels)) {
    if (labels == "studlab" & max(nchar(x$studlab)) > 3)
      srt.labels <- ifelse(x$TE < x$TE.common, 45, -45)
    else
      srt.labels <- rep(0, k.all)
  }
  else {
    srt.labels <- catchvar("srt.labels", x, mf)
    chknumeric(srt.labels, min = -90, max = 90)
  }
  srt.labels <- augment(srt.labels, k.all, fun)
  #
  # Other arguments
  #
  common <-
    deprecated2(common, missing(common), fixed, missing(fixed),
                warn.deprecated)
  chklogical(common)
  chklogical(random)
  chklogical(prediction)
  #
  lwd.common <-
    deprecated2(lwd.common, missing(lwd.common),
                lwd.fixed, missing(lwd.fixed),
                warn.deprecated)
  chknumeric(lwd.common, length = 1)
  #
  lty.common <-
    deprecated2(lty.common, missing(lty.common),
                lty.fixed, missing(lty.fixed),
                warn.deprecated)
  chknumeric(lty.common, length = 1)
  #
  col.common <-
    deprecated2(col.common, missing(col.common),
                col.fixed, missing(col.fixed),
                warn.deprecated)
  chklength(col.common, 1,
            text = "Argument 'col.common' must be of length 1.")
  chklength(col.random, 1,
            text = "Argument 'col.random' must be of length 1.")
  #
  if (!is.null(sign)) {
    chklevel(sign, length = 0)
    if (type == "zvalue")
      sign <- qnorm(sign / 2)
    #
    chknumeric(lty.sign, min = 0, zero = TRUE, length = 1)
    chknumeric(lwd.sign, min = 0, zero = TRUE, length = 1)
  }
  #
  if (!all(is.na(alpha)))
    chklevel(alpha, length = 0)
  chknumeric(lty.alpha, min = 0, zero = TRUE, length = 1)
  chknumeric(lwd.alpha, min = 0, zero = TRUE, length = 1)
  chknumeric(cex.alpha, min = 0, zero = TRUE, length = 1)
  #
  chklogical(legend)
  bty <- setchar(bty, c("o", "n"))
  chklogical(backtransf)
  #
  if (missing(xlab)) {
    xlab <- ""
    xlab <- xlab_meta(x$sm, backtransf = backtransf)
  }
  else
    chkchar(xlab, length = 1)
  #
  if (missing(ylab))
    ylab <- c("Test statistic",
              "P-value",
              "Binary S-value / surprisal")[charmatch(type, types)]
  else
    chkchar(ylab, length = 1)
  #
  chknumeric(lwd.max, min = 0, zero = TRUE, length = 1)
  lwd.study.weight <- setchar(lwd.study.weight, c("random", "common"))
  #
  chknumeric(n.grid, min = 0, zero = TRUE, length = 1)
  chklogical(plot)
  #
  # Additional check
  #
  if (!any(c(study.results, common, random, prediction))) {
    warning("Nothing selected to show in drapery plot.")
    return(invisible(NULL))
  }
  
  
  #
  #
  # (2) Set variables for plotting
  #
  #
  if (missing(ylim))
    ylim <-
      if (type == "pvalue") c(0, 1)
      else if (type == "zvalue") c(qnorm(0.0005), 0)
      else c(0, 12)
  else
    if (type == "pvalue") {
      warning("Argument 'ylim' ignored for p-value function plot." )
      ylim <- c(0, 1)
    }
  #
  x.backtransf <- is_relative_effect(x$sm) & backtransf
  #
  missing.xlim <- missing(xlim)
  #
  if (missing.xlim) {
    mn <- min(x$lower, na.rm = TRUE)
    mx <- max(x$upper, na.rm = TRUE)
    #
    xlim <- c(mn, mx)
  }
  else {
    if (x.backtransf)
      xlim <- log(xlim)
    #
    mn <- min(xlim)
    mx <- max(xlim)
  }
  #
  grid <- c(seq(mn, mx, length.out = n.grid), x$TE, x$TE.common, x$TE.random)
  #
  if (type == "zvalue")
    grid <- c(grid,
              x$TE + ylim[1] * x$seTE,
              x$TE - ylim[1] * x$seTE,              
              x$TE.common + ylim[1] * x$seTE.common,
              x$TE.common - ylim[1] * x$seTE.common,
              x$TE.random + ylim[1] * x$seTE.random,
              x$TE.random - ylim[1] * x$seTE.random)
  #
  grid <- sort(grid)
  #
  if (x.backtransf) {
    mn <- exp(mn)
    mx <- exp(mx)
    x.grid <- exp(grid)
    xlim <- exp(xlim)
    null.effect <- exp(x$null.effect)
  }
  else {
    x.grid <- grid
    null.effect <- x$null.effect
  }
  #
  o <- order(x$seTE)
  #
  TE <- x$TE[o]
  seTE <- x$seTE[o]
  lower <- x$lower[o]
  upper <- x$upper[o]
  pval <- x$pval[o]
  statistic <- x$statistic[o]
  w.common <- x$w.common[o]
  w.random <- x$w.random[o]
  seq.TE <- seq.TE[o]
  studlab <- x$studlab[o]
  #
  lty.study <- lty.study[o]
  lwd.study <- lwd.study[o]
  col.study <- col.study[o]
  #
  col.labels <- col.labels[o]
  cex.labels <- cex.labels[o]
  subset.labels <- subset.labels[o]
  srt.labels <- srt.labels[o]
  #
  TE.common <- x$TE.common
  seTE.common <- x$seTE.common
  TE.random <- x$TE.random
  seTE.random <- x$seTE.random
  seTE.predict <- x$seTE.predict
  t.quantile <- (x$upper.predict - x$lower.predict) / seTE.predict / 2
  #
  if (layout == "grayscale" & missing.col.study)
    col.study <- gray.colors(k.all)
  #
  else if (layout == "linewidth" & missing.lwd.study) {
    if (lwd.study.weight == "common")
      lwd.study <- lwd.max * w.common / max(w.common)
    else
      lwd.study <- lwd.max * w.random / max(w.random)
  }
  #  
  y.common <- pvf(grid, TE.common, seTE.common)
  y.random <- pvf(grid, TE.random, seTE.random)
  y.predict <- pvf(grid, TE.random, t.quantile / qnorm(0.975) * seTE.predict)
  y.alpha <- alpha
  #
  if (type == "zvalue") {
    y.common <- qnorm(y.common / 2)
    y.random <- qnorm(y.random / 2)
    y.predict <- qnorm(y.predict / 2)
    y.alpha <- qnorm(y.alpha / 2)
  }
  else if (type == "surprisal") {
    y.common <- -log(y.common, base = 2)
    y.random <- -log(y.random, base = 2)
    y.predict <- -log(y.predict, base = 2)
    y.alpha <- -log(y.alpha, base = 2)
  }
  #
  sel.common <- y.common >= min(ylim)
  sel.random <- y.random >= min(ylim)
  sel.predict <- y.predict >= min(ylim)
  #
  if (!study.results & missing.xlim) {
    if (prediction) {
      xvals <- x.grid[sel.predict]
      if (common)
        xvals <- c(xvals, x.grid[sel.common])
      xlim <- range(xvals)
    }
    else {
      if (common & random)
        xvals <- c(x.grid[sel.common], x.grid[sel.random])
      else if (!common)
        xvals <- x.grid[sel.random]
      else
        xvals <- x.grid[sel.common]
      xlim <- range(xvals)
    }
  }
  
  
  #
  #
  # (3) Generate drapery plot
  #
  #
  if (plot) {
    plot(x.grid, y.common,
         xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim,
         type = "n", las = 1, if (x.backtransf) log = "x" else log = "",
         axes = FALSE, ...)
    axis(1, at = at)
    axis(2)
    box()
    #
    # Add prediction region
    #
    if (prediction) {
      polygon(x.grid[sel.predict], y.predict[sel.predict],
              col = col.predict, border = NA)
      #
      if (type != "surprisal")
        polygon(c(min(x.grid[sel.predict]),
                  max(x.grid[sel.predict]),
                  max(x.grid[sel.predict])),
                c(min(y.predict[sel.predict]),
                  min(y.predict[sel.predict]),
                  tail(y.predict[sel.predict], n = 1)),
                col = col.predict, border = NA)
      else
        polygon(c(max(x.grid[sel.predict]),
                  min(x.grid[sel.predict]),
                  min(x.grid[sel.predict])),
                c(max(y.predict[sel.predict]),
                  max(y.predict[sel.predict]),
                  head(y.predict[sel.predict], n = 1)),
                col = col.predict, border = NA)
      #
      if (all(y.predict > min(ylim)) & type != "surprisal") {
        x.min <- min(x.grid)
        x.max <- max(x.grid)
        y.min <- min(ylim)
        y.max <- min(y.predict)
        polygon(c(x.min, x.min, x.max, x.max),
                c(y.min, y.max, y.max, y.min),
                col = col.predict, border = NA)
      }
    }
    #
    # Add studies
    #
    if (study.results) {
      for (i in k.all:1) {
        y.i <- pvf(grid, TE[i], seTE[i])
        if (type == "zvalue")
          y.i <- qnorm(y.i / 2)
        else if (type == "surprisal")
          y.i <- -log(y.i, base = 2)
        sel.i <- y.i >= min(ylim)
        lines(x.grid[sel.i], y.i[sel.i],
              lty = lty.study[i], lwd = lwd.study[i], col = col.study[i])
        #
        if (!is.null(sign)) {
          sel.sign.i <- sel.i & y.i < sign
          seq.i <- seq(along = y.i)[sel.sign.i]
          sel.seq.i.u <- seq.i[c(FALSE, diff(seq.i) != 1)]
          sel.seq.i.l <- seq.i[c(diff(seq.i) != 1, FALSE)]
          sel.sign.i.l <- sel.sign.i & seq(along = sel.sign.i) <= sel.seq.i.l
          sel.sign.i.u <- sel.sign.i & seq(along = sel.sign.i) >= sel.seq.i.u
          #
          lines(x.grid[sel.sign.i.l], y.i[sel.sign.i.l],
                lty = lty.sign, lwd = lwd.sign, col = col.sign)
          lines(x.grid[sel.sign.i.u], y.i[sel.sign.i.u],
                lty = lty.sign, lwd = lwd.sign, col = col.sign)
        }
        #
        if (subset.labels[i]) {
          if (type != "surprisal") {
            yval <- ylim[2] + 0.005 * abs(ylim[1] - ylim[2])
            y.adj <- 0
          }
          else {
            yval <- ylim[1] - 0.005 * abs(ylim[1] - ylim[2])
            y.adj <- 1
          }
          #
          if (labels != "") {
            if (srt.labels[i] == 0)
              adj.i <- c(0.5, y.adj)
            else
              adj.i <- if (srt.labels[i] > 0) c(1, y.adj) else c(0, y.adj)
            #
            text(if (x.backtransf) exp(TE[i]) else TE[i],
                 yval,
                 labels = if (labels == "id") seq.TE[i] else studlab[i],
                 col = col.labels[i], cex = cex.labels[i],
                 srt = srt.labels[i], adj = adj.i)
          }
        }
      }
    }
    #
    # Add common effect lines
    #
    if (common)
      lines(x.grid[sel.common], y.common[sel.common],
            lty = lty.common, lwd = lwd.common, col = col.common)
    #
    # Add random effects lines
    #
    if (random)
      lines(x.grid[sel.random], y.random[sel.random],
            lty = lty.random, lwd = lwd.random, col = col.random)
    #
    # Add null effect line
    #
    abline(v = null.effect, col = col.null.effect)
    #
    # Add p-value lines
    #
    if (type != "surprisal")
      for (alpha.i in y.alpha)
        abline(h = alpha.i, lty = lty.alpha, lwd = lwd.alpha, col = col.alpha)
    #
    # Add text for alpha levels
    #
    if (type != "surprisal")
      for (i in seq(along = y.alpha))
        text(mn, y.alpha[i] + (ylim[2] - ylim[1]) / 200,
             paste("p =", alpha[i]),
             cex = cex.alpha, col = col.alpha, adj = c(0, 0))
    #
    # Add text for confidence levels
    #
    if (type != "surprisal")
      for (i in seq(along = y.alpha))
        text(mx, y.alpha[i] + (ylim[2] - ylim[1]) / 200,
             paste0(100 * (1 - alpha[i]), "%-CI"),
             cex = cex.alpha, col = col.alpha, adj = c(1, 0))
    #
    # Add legend
    #
    if (legend & any(c(common, random, prediction)))
      legend(pos.legend,
             col = c(if (common) col.common,
                     if (random) col.random,
                     if (prediction) col.predict),
             lwd = c(if (common) lwd.common,
                     if (random) lwd.random,
                     if (prediction) lwd.random),
             bty = bty, bg = bg,
             c(if (common) gs("text.common"),
               if (random) gs("text.random"),
               if (prediction) "Range of prediction"))
    #
    # Add y-axis on right side
    #
    if (type == "pvalue") {
      par(new = TRUE)
      plot(x.grid, y.common,
           xlab = xlab, ylab = ylab,
           xlim = xlim, ylim = rev(ylim),
           type = "n", las = 1, if (x.backtransf) log = "x" else log = "",
           axes = FALSE,
           ...)
      axis(4, las = 1)
    }
    else if (type == "zvalue") {
      axis(4, las = 1,
           at = qnorm(c(0.001, 0.01, 0.05) / 2),
           labels = 1 - c(0.001, 0.01, 0.05))
      axis(4, las = 1,
           at = qnorm(seq(0.2, 1, by = 0.2) / 2),
           labels = format(seq(0.8, 0, by = -0.2)))
    }
    #
    if (type %in% c("pvalue", "zvalue"))
      mtext("Confidence level", 4, line = 3)
  }
  
  
  if (study.results) {
    res <- data.frame(ID = seq.TE, studlab = studlab,
                      lty.study, lwd.study, col.study,
                      col.labels, cex.labels,
                      subset.labels, srt.labels,
                      TE, seTE, lower, upper, statistic, pval,
                      w.common, w.random,
                      stringsAsFactors = FALSE)
    #
    res <- res[order(res$ID), , drop = FALSE]
    #
    attr(res, "xlim") <- xlim
    attr(res, "ylim") <- ylim
  }
  else
    res <- NULL
  
  
  invisible(res)
}

#' Radial plot
#' 
#' @description
#' Draw a radial plot (also called Galbraith plot) which can be used
#' to assess bias in meta-analysis.
#' 
#' @aliases radial radial.default radial.meta
#' 
#' @param x An object of class \code{meta}, or estimated treatment
#'   effect in individual studies.
#' @param y Standard error of estimated treatment effect.
#' @param xlim The x limits (min, max) of the plot.
#' @param ylim The y limits (min, max) of the plot.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param common A logical indicating whether the pooled common effect
#'   estimate should be plotted.
#' @param axes A logical indicating whether axes should be drawn on
#'   the plot.
#' @param pch The plotting symbol used for individual studies.
#' @param text A character vector specifying the text to be used
#'   instead of plotting symbol.
#' @param cex The magnification to be used for plotting symbol.
#' @param col A vector with colour of plotting symbols.
#' @param level The confidence level utilised in the plot.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param fixed Deprecated argument (replaced by 'common').
#' @param \dots Graphical arguments as in \code{par} may also be
#'   passed as arguments.
#' 
#' @details
#' A radial plot (Galbraith 1988a,b), also called Galbraith plot, is
#' drawn in the active graphics window. If \code{common} is TRUE, the
#' pooled estimate of the common effect model is plotted. If
#' \code{level} is not NULL, the corresponding confidence limits are
#' drawn.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabias}}, \code{\link{metabin}},
#'   \code{\link{metagen}}, \code{\link{funnel}}
#' 
#' @references
#' Galbraith RF (1988a):
#' Graphical display of estimates having differing standard errors.
#' \emph{Technometrics},
#' \bold{30}, 271--81
#' 
#' Galbraith RF (1988b):
#' A note on graphical presentation of estimated odds ratios from
#' several clinical trials.
#' \emph{Statistics in Medicine},
#' \bold{7}, 889--94
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Olkin1995)
#' ma <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, subset = c(41, 47, 51, 59),
#'   studlab = paste(author, year), sm = "RR", method = "I")
#' 
#' # Radial plot
#' #
#' radial(ma, level = 0.95)
#' 
#' @rdname radial
#' @method radial meta
#' @export

radial.meta <- function(x,
                        #
                        xlim = NULL, ylim = NULL,
                        xlab = "Inverse of standard error",
                        ylab = "Standardised treatment effect (z-score)",
                        #
                        common = TRUE,
                        #
                        axes = TRUE,
                        pch = 1, text = NULL, cex = 1, col = NULL,
                        #
                        level = NULL,
                        warn.deprecated = gs("warn.deprecated"),
                        fixed,
                        ...) {
  
  
  #
  #
  # (1) Check for meta object
  #
  #
  chkclass(x, "meta")
  chksuitable(x, "Radial plot", "metamerge", check.mlm = FALSE)
  #
  x <- updateversion(x)
  #
  TE <- x$TE
  seTE <- x$seTE
  
  
  #
  #
  # (2) Check arguments
  #
  #
  if (!is.null(xlim))
    chknumeric(xlim, length = 2)
  if (!is.null(ylim))
    chknumeric(ylim, length = 2)
  chkchar(xlab, length = 1)
  chkchar(ylab, length = 1)
  #
  chklogical(warn.deprecated)
  #
  common <-
    deprecated2(common, missing(common), fixed, missing(fixed),
                warn.deprecated)
  chklogical(common)
  #
  chklogical(axes)
  if (!is.null(level))
    chklevel(level)
  #
  if (!is.null(text))
    chklength(text, length(TE),
              text = paste(
                "Argument 'text' must have same length as number of",
                "studies in meta-analysis."))
  
  
  #
  #
  # (3) Produce radial plot
  #
  #
  if (!is.null(x$exclude)) {
    TE <- TE[!x$exclude]
    seTE <- seTE[!x$exclude]
    if (!is.null(text))
      text <- text[!x$exclude]
  }
  #
  zscore <- TE / seTE
  #
  if (is.null(xlim)) xlim <- c(0, max(1 / seTE, na.rm = TRUE))
  if (is.null(ylim)) ylim <- c(min(c(-2, zscore), na.rm = TRUE),
                               max(c( 2, zscore), na.rm = TRUE))
  #
  plot(1 / seTE, zscore,
       xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim,
       axes = axes, type = "n", ...)

  if (is.null(text)) {
    if (is.null(col))
      points(1 / seTE, zscore, pch = pch, cex = cex)
    else
      points(1 / seTE, zscore, pch = pch, cex = cex, col = col)
  }
  else {
    if (is.null(col))
      text(1 / seTE, zscore, labels = text, cex = cex)
    else
      text(1 / seTE, zscore, labels = text, cex = cex, col = col)
  }

  if (common) {
    lmcomb <- lm(zscore ~ I(1 / seTE) - 1)
    abline(lmcomb, lty = 4)
    if (!is.null(level)) {
      alpha <- 1 - level
      abline(qnorm(1 - alpha / 2), coef(lmcomb), lty = 2)
      abline(qnorm(    alpha / 2), coef(lmcomb), lty = 2)
    }
  }
  
  invisible(NULL)
}





#' @rdname radial
#' @method radial default
#' @export

radial.default <- function(x, y,
                           #
                           xlim = NULL, ylim = NULL,
                           xlab = "Inverse of standard error",
                           ylab = "Standardised treatment effect (z-score)",
                           #
                           common = TRUE,
                           #
                           axes = TRUE,
                           #
                           pch = 1, text = NULL, cex = 1, col = NULL,
                           #
                           level = NULL, ...) {
  
  
  #
  #
  # (1) Check essential arguments
  #
  #
  chknumeric(x)
  chknumeric(y)
  #
  chklength(y, length(x), "x")
  
  
  #
  #
  # (2) Do meta-analysis
  #
  #
  m <- metagen(x, y, method.tau.ci = "")
  
  
  #
  #
  # (3) Produce radial plot
  #
  #
  radial(m, xlim = xlim, ylim = ylim,
         xlab = xlab, ylab = ylab,
         common = common, axes = axes,
         pch = pch, text = text,
         cex = cex, col = col, level = level, ...)
  
  
  invisible(NULL)
}

#' Extract parts of longarm object
#' 
#' @description
#' Auxiliary function to extract parts of \code{\link{longarm}} object.
#' 
#' @param x An object of class \code{\link{longarm}}.
#' @param \dots Additional arguments (passed on to [.data.frame).
#' 
#' @aliases [,longarm
#' 
#' @name [.longarm
#' 
#' @usage \method{[}{longarm}(x, ...)
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{longarm}}, \code{\link{subset.longarm}},
#'   \code{\link[metabook]{Franchini2012}}
#' 
#' @examples
#' # Transform data from wide arm-based format to contrast-based format
#' pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
#'   n = list(n1, n2, n3),
#'   mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
#'   data = Franchini2012, studlab = Study)
#' 
#' # Transform data from contrast-based to long arm-based format
#' # and only keep the main variables
#' la1 <- longarm(pw1, append = FALSE)
#' head(la1)
#' 
#' la1[la1$studlab == "Lieberman 1998", ]
#'
#' @method [ longarm
#' @export

`[.longarm` <- function(x, ...) {
  
  chkclass(x, "longarm")
  #
  attribs <- attributes(x)
  #
  attribs$names <- attribs$row.names <- NULL
  
  res <- x
  class(res) <- "data.frame"
  res <- res[...]
  
  # Check whether 'reference.group' is available in the subset
  #
  reference.group <- attr(x, "reference.group")
  #
  trts <- sort(unique(c(res$treat1, res$treat2)))
  if (!is.null(trts) && !(reference.group %in% trts))
    reference.group <- trts[1]
  
  # Return results
  #
  for (i in names(attribs))
    attr(res, i) <- attr(x, i)
  #
  class(res) <- unique(c("longarm", class(res)))
  #
  res
}

#' Print meta-analysis results
#' 
#' @description
#' Print method for objects of class \code{meta}.
#'
#' R function cilayout can be utilised to change the layout to print
#' confidence intervals (both in printout from print.meta and
#' print.summary.meta function as well as in forest plots). The
#' default layout is "[lower; upper]". Another popular layout is
#' "(lower - upper)" which is used throughout an R session by using R
#' command \code{cilayout("(", " - ")}.
#' 
#' Argument \code{pscale} can be used to rescale single proportions or
#' risk differences, e.g. \code{pscale = 1000} means that proportions
#' are expressed as events per 1000 observations. This is useful in
#' situations with (very) low event probabilities.
#' 
#' Argument \code{irscale} can be used to rescale single rates or rate
#' differences, e.g. \code{irscale = 1000} means that rates are
#' expressed as events per 1000 time units, e.g. person-years. This is
#' useful in situations with (very) low rates. Argument \code{irunit}
#' can be used to specify the time unit used in individual studies
#' (default: "person-years"). This information is printed in summaries
#' and forest plots if argument \code{irscale} is not equal to 1.
#'
#' @aliases print.meta cilayout
#' 
#' @param x An object of class \code{meta}.
#' @param common A logical indicating whether results for common
#'   effect meta-analysis should be printed.
#' @param random A logical indicating whether results for random
#'   effects meta-analysis should be printed.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param test.subgroup.common A logical value indicating whether to
#'   print results of test for subgroup differences (based on common
#'   effect model).
#' @param test.subgroup.random A logical value indicating whether to
#'   print results of test for subgroup differences (based on random
#'   effects model).
#' @param prediction.subgroup A single logical or logical vector
#'   indicating whether / which prediction intervals should be printed
#'   for subgroups.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf=TRUE}, results
#'   for \code{sm="OR"} are printed as odds ratios rather than log
#'   odds ratios and results for \code{sm="ZCOR"} are printed as
#'   correlations rather than Fisher's z transformed correlations, for
#'   example.
#' @param pscale A numeric giving scaling factor for printing of
#'   single event probabilities or risk differences, i.e. if argument
#'   \code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
#'   \code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.
#' @param irscale A numeric defining a scaling factor for printing of
#'   single incidence rates or incidence rate differences, i.e. if
#'   argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
#'   \code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.
#' @param irunit A character specifying the time unit used to
#'   calculate rates, e.g. person-years.
#' @param subgroup.name A character string with a name for the
#'   grouping variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the grouping variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between label and levels of grouping variable.
#' @param nchar.subgroup A numeric specifying the number of characters
#'   to print from subgroup labels.
#' @param sort.overall An optional vector used to sort meta-analysis
#'   results.
#' @param sort.tau An optional vector used to sort estimators of the
#'   between-study heterogeneity variance.
#' @param sort.het An optional vector used to sort heterogeneity
#'   statistics.
#' @param sort.Q An optional vector used to sort test of
#'   heterogeneity.
#' @param header A logical indicating whether information on title of
#'   meta-analysis, comparison and outcome should be printed at the
#'   beginning of the printout.
#' @param print.CMH A logical indicating whether result of the
#'   Cochran-Mantel-Haenszel test for overall effect should be
#'   printed.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value of test for overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance \eqn{\tau^2}, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param digits.Q Minimal number of significant digits for
#'   heterogeneity statistic Q, see \code{print.default}.
#' @param digits.df Minimal number of significant digits for degrees
#'   of freedom.
#' @param digits.pval.Q Minimal number of significant digits for
#'   p-value of heterogeneity test, see \code{print.default}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   and Rb statistic, see \code{print.default}.
#' @param digits.H Minimal number of significant digits for H
#'   statistic, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau2.ci A logical value indicating whether to print
#'   the confidence interval of \eqn{\tau^2}.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.tau.ci A logical value indicating whether to print the
#'   confidence interval of \eqn{\tau}.
#' @param print.Q A logical value indicating whether to print the
#'   results of the test of heterogeneity.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param print.H A logical specifying whether heterogeneity statistic
#'   H should be printed.
#' @param print.Rb A logical specifying whether heterogeneity
#'   statistic R\eqn{_b} should be printed.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param text.I2 Text printed to identify heterogeneity statistic
#'   I\eqn{^2}.
#' @param text.Rb Text printed to identify heterogeneity statistic
#'   R\eqn{_b}.
#' @param details.methods A logical specifying whether details on
#'   statistical methods should be printed.
#' @param warn.backtransf Deprecated argument (ignored).
#' @param func.backtransf A function used to back-transform results.
#' @param bracket A character with bracket symbol to print lower
#'   confidence interval: "[", "(", "\{", "".
#' @param separator A character string with information on separator
#'   between lower and upper confidence interval.
#' @param lower.blank A logical indicating whether blanks between left
#'   bracket and lower confidence limit should be printed.
#' @param upper.blank A logical indicating whether blanks between
#'   separator and upper confidence limit should be printed.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (passed on to \code{prmatrix}).
#' 
#' @rdname print.meta
#' @method print meta
#' @export

print.meta <- function(x,
                       common = x$common,
                       random = x$random,
                       prediction = x$prediction,
                       overall = x$overall,
                       overall.hetstat = x$overall.hetstat,
                       #
                       test.subgroup = x$test.subgroup,
                       test.subgroup.common = test.subgroup & common,
                       test.subgroup.random = test.subgroup & random,
                       prediction.subgroup = x$prediction.subgroup,
                       #
                       backtransf = x$backtransf,
                       pscale = x$pscale,
                       irscale = x$irscale,
                       irunit = x$irunit,
                       #
                       subgroup.name = x$subgroup.name,
                       print.subgroup.name = x$print.subgroup.name,
                       sep.subgroup = x$sep.subgroup,
                       nchar.subgroup = 35,
                       #
                       sort.overall = NULL,
                       sort.tau = NULL,
                       sort.het = NULL,
                       sort.Q = NULL,
                       #
                       header = TRUE,
                       print.CMH = x$print.CMH,
                       #
                       digits = gs("digits"),
                       digits.stat = gs("digits.stat"),
                       digits.pval = max(gs("digits.pval"), 2),
                       #
                       digits.tau2 = gs("digits.tau2"),
                       digits.tau = gs("digits.tau"),
                       #
                       digits.Q = gs("digits.Q"),
                       digits.df = gs("digits.df"),
                       digits.pval.Q = max(gs("digits.pval.Q"), 2),
                       #
                       digits.H = gs("digits.H"),
                       digits.I2 = gs("digits.I2"),
                       #
                       big.mark = gs("big.mark"),
                       scientific.pval = gs("scientific.pval"),
                       zero.pval = gs("zero.pval"),
                       JAMA.pval = gs("JAMA.pval"),
                       #
                       print.tau2 = gs("print.tau2"),
                       print.tau2.ci = gs("print.tau2.ci"),
                       print.tau = gs("print.tau"),
                       print.tau.ci = gs("print.tau.ci"),
                       #
                       print.Q = gs("print.Q"),
                       print.I2 = gs("print.I2"),
                       print.I2.ci = gs("print.I2.ci"),
                       print.H = gs("print.H"),
                       print.Rb = gs("print.Rb"),
                       #
                       text.tau2 = gs("text.tau2"),
                       text.tau = gs("text.tau"),
                       text.I2 = gs("text.I2"),
                       text.Rb = gs("text.Rb"),
                       #
                       details.methods = gs("details"),
                       #
                       warn.backtransf = FALSE,
                       func.backtransf = x$func.backtransf,
                       warn.deprecated = gs("warn.deprecated"),
                       ...) {
  
  
  #
  #
  # (1) Check for meta object
  #
  #
  chkclass(x, "meta")
  x <- updateversion(x)
  #
  if (inherits(x, "metainf"))
    return(invisible(NULL))
  #
  is.metabind <- inherits(x, "metabind")
  is.netpairwise <- inherits(x, "netpairwise")
  #
  method <- x$method
  method.random <- x$method.random
  #
  by <- !is.null(x$subgroup)
  #
  if (by)
    n.by <- length(x$subgroup.levels)
  #
  method.random.ci <- replaceNULL(x$method.random.ci, "")
  method.predict <- replaceNULL(x$method.predict, "")
  #
  if (is.function(func.backtransf))
    fbt <- deparse(substitute(func.backtransf))
  else
    fbt <- func.backtransf
  #
  abt <- x$args.backtransf
  #
  print.I2 <- print.I2 & ((overall.hetstat & !by) | by)
  #
  pairwise <- replaceNULL(x$pairwise, FALSE) & any(table(x$studlab) > 1)
  
  
  #
  #
  # (2) Check and set other arguments
  #
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 1, length = 1)
  chknumeric(digits.pval.Q, min = 1, length = 1)
  chknumeric(digits.Q, min = 0, length = 1)
  chknumeric(digits.H, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.tau2)
  chklogical(print.tau2.ci)
  chklogical(print.tau)
  chklogical(print.tau.ci)
  #
  if (all(method == "LRP") & all(method.random == "LRP")) {
    print.tau2 <- FALSE
    print.tau <- FALSE
  }
  #
  chklogical(print.I2)
  chklogical(print.I2.ci)
  chklogical(print.H)
  chklogical(print.Rb)
  #
  chkchar(text.tau2, length = 1)
  chkchar(text.tau, length = 1)
  chkchar(text.I2, length = 1)
  chkchar(text.Rb, length = 1)
  chklogical(print.Q)
  #
  chklogical(warn.deprecated)
  #
  missing.pscale <- missing(pscale)
  if (!is.null(pscale))
    chknumeric(pscale, length = 1)
  else
    pscale <- 1
  #
  missing.irscale <- missing(irscale)
  if (!is.null(irscale))
    chknumeric(irscale, length = 1)
  else
    irscale <- 1
  if (!is.null(irunit) && !is.na(irunit))
    chkchar(irunit)
  #
  chklogical(common)
  chklogical(random)
  chklogical(prediction)
  overall <- replaceNULL(overall, TRUE)
  chklogical(overall)
  overall.hetstat <- replaceNULL(overall.hetstat, TRUE)
  chklogical(overall.hetstat)
  test.subgroup <- replaceNULL(test.subgroup, TRUE)
  if (is.na(test.subgroup))
    test.subgroup <- FALSE
  chklogical(test.subgroup)
  test.subgroup.common <- replaceNULL(test.subgroup.common, test.subgroup)
  chklogical(test.subgroup.common)
  test.subgroup.random <- replaceNULL(test.subgroup.random, test.subgroup)
  chklogical(test.subgroup.random)
  #
  missing.prediction.subgroup <- missing(prediction.subgroup)
  #
  if (by) {
    x$lower.predict.w <- list2mat(x$lower.predict.w)
    x$upper.predict.w <- list2mat(x$upper.predict.w)
    x$df.predict.w <- list2mat(x$df.predict.w)
    #
    if (!missing.prediction.subgroup)
      prediction.subgroup <- catch("prediction.subgroup", mc, x, sfsp)
    #
    prediction.subgroup <- replaceNULL(prediction.subgroup, FALSE)
    #
    if (length(prediction.subgroup) == 1) {
      if (is.matrix(x$lower.predict.w)) {
        prediction.subgroup.logical <-
          prediction.subgroup &
          apply(x$lower.predict.w, 1, notallNA) &
          apply(x$upper.predict.w, 1, notallNA)
      }
      else {
        prediction.subgroup.logical <-
          prediction.subgroup &
          notallNA(x$lower.predict.w) &
          notallNA(x$upper.predict.w)
        #
        prediction.subgroup.logical <-
          rep(prediction.subgroup.logical, n.by)
      }
    }
    else {
      chklength(prediction.subgroup, n.by,
                text = paste("Length of argument 'prediction.subgroup' must be",
                             "equal to 1 or number of subgroups."))
      prediction.subgroup.logical <- prediction.subgroup
    }
    #
    if (length(prediction.subgroup.logical) == 0)
      prediction.subgroup.logical <- FALSE
    #
    chklogical(prediction.subgroup.logical[1])
    #
    if (missing.prediction.subgroup) {
      if (any(method.predict %in% c("", "S")))
        prediction.w <- prediction.subgroup.logical
      else
        prediction.w <- prediction.subgroup.logical & !is.na(x$df.predict.w)
    }
    else
      prediction.w <- prediction.subgroup.logical
    #
    prediction.w[is.na(prediction.w)] <- FALSE
    prediction.w <- any(prediction.w) & !is.metabind
  }
  else
    prediction.w <- FALSE
  #
  if (!is.null(print.CMH))
    chklogical(print.CMH)
  #
  sort.tau <- catch("sort.tau", mc, x, sfsp)
  if (is.list(sort.tau))
    sort.tau <- unlist(sort.tau)
  #
  sort.het <- catch("sort.het", mc, x, sfsp)
  if (is.list(sort.het))
    sort.het <- unlist(sort.het)
  #
  sort.Q <- catch("sort.Q", mc, x, sfsp)
  if (is.list(sort.Q))
    sort.Q <- unlist(sort.Q)
  #
  chklogical(header)
  #
  chklogical(details.methods)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing(random), args, "comb.random", FALSE)
  chklogical(random)
  #
  subgroup.name <-
    deprecated(subgroup.name, missing(subgroup.name), args, "bylab", FALSE)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing(print.subgroup.name),
               args, "print.byvar", FALSE)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing(sep.subgroup), args, "byseparator",
               FALSE)
  #
  nchar.subgroup <-
    deprecated(nchar.subgroup, missing(nchar.subgroup), args, "nchar.subgroup",
               warn.deprecated)
  chknumeric(nchar.subgroup, min = 1, length = 1)
  #
  backtransf <-
    deprecated(backtransf, missing(backtransf), args, "logscale",
               warn.deprecated)
  if (is_untransformed(x$sm))
    backtransf <- TRUE
  chklogical(backtransf)
  #
  # Additional settings
  #
  if (!backtransf & pscale != 1 & !is_untransformed(x$sm)) {
    if (!missing.pscale)
      warning("Argument 'pscale' set to 1 as argument 'backtransf' is FALSE.")
    pscale <- 1
  }
  if (!backtransf & irscale != 1 & !is_untransformed(x$sm)) {
    if (!missing.irscale)
      warning("Argument 'irscale' set to 1 as argument 'backtransf' is FALSE.")
    irscale <- 1
  }
  #
  if (by) {
    chklogical(print.subgroup.name)
    chkchar(sep.subgroup)
  }
  else {
    if (!missing.prediction.subgroup)
      warning("Argument 'prediction.subgroup' only considered for ",
              "meta-analysis with subgroups.",
              call. = FALSE)
  }
  
  
  #
  #
  # (3) Some additional settings
  #
  #
  k.all <- replaceNULL(x$k.all, length(x$TE))
  k <- x$k
  k.study <- replaceNULL(x$k.study, k)
  sm <- x$sm
  #
  bip <- inherits(x, c("metabin", "metainc", "metaprop", "metarate"))
  metabin <- inherits(x, "metabin")
  metaprop <- inherits(x, "metaprop")
  metarate <- inherits(x, "metarate")
  #
  null.effect <- x$null.effect
  null.given <- !is.null(null.effect) && !is.na(null.effect)
  #
  if (null.given & !backtransf) {
    #
    if (sm %in% c("PFT", "PAS"))
      null.effect <- asin(sqrt(null.effect))
    else if (is_log_effect(sm))
      null.effect <- log(null.effect)
    else if (sm == c("PLOGIT"))
      null.effect <- log(null.effect / (1 - null.effect))
    else if (sm %in% c("IRS", "IRFT"))
      null.effect <- sqrt(null.effect)
    else if (sm == "ZCOR")
      null.effect <- 0.5 * log((1 + null.effect) / (1 - null.effect))
  }
  #
  x$null.effect <- null.effect
  #    
  if (is.null(prediction) || is.na(prediction))
    prediction <- FALSE
  #
  sm.lab <- smlab(sm, backtransf, pscale, irscale)
  #
  if (length(x$tau.common) == 0)
    x$tau.common <- FALSE
  #
  if (length(x$tau.common) == 0)
    x$tau.common <- FALSE
  #
  if (by)
    subgroup.levels <-
      ifelse(nchar(x$subgroup.levels) > nchar.subgroup,
             paste0(substring(x$subgroup.levels, 1, nchar.subgroup - 4),
                    " ..."),
             x$subgroup.levels)
  #
  if (is.null(x$text.common))
    text.common <- gs("text.common")
  else
    text.common <- x$text.common
  #
  if (is.null(x$text.random))
    text.random <- gs("text.random")
  else
    text.random <- x$text.random
  #
  if (is.null(x$text.predict))
    text.predict <- gs("text.predict")
  else
    text.predict <- x$text.predict
  #
  if (any(substring(text.common, 1, 5) %in% c("Fixed", "Commo"))) {
    text.common.br <- gsub("Fixed", "fixed", text.common)
    text.common.br <- gsub("Common", "common", text.common.br)
    text.common.br <- gsub("Effect", "effect", text.common.br)
  }
  else
    text.common.br <- text.common
  #
  if (any(substring(text.random, 1, 5) %in% c("Rando"))) {
    text.random.br <- gsub("Random", "random", text.random)
    text.random.br <- gsub("Effect", "effect", text.random.br)
  }
  else
    text.random.br <- text.random
  #
  if (common | random)
    ci.lab <- paste0(round(100 * x$level.ma, 1), "%-CI")
  else if (prediction)
    ci.lab <- paste0(round(100 * x$level.predict, 1), "%-PI")
  else if (all(k.all == 1))
    ci.lab <- paste0(round(100 * x$level.ma, 1), "%-CI")
  else
    ci.lab <- ""
  #
  details <- NULL
  
  
  #
  #
  # (4) Set and backtransform results of meta-analysis
  #
  #
  TE.common <- x$TE.common
  lowTE.common <- x$lower.common
  uppTE.common <- x$upper.common
  pTE.common <- x$pval.common
  sTE.common <- x$statistic.common
  #
  TE.random <- x$TE.random
  lowTE.random <- x$lower.random
  uppTE.random <- x$upper.random
  if (length(TE.random) == 1 &&
      length(TE.random) != length(lowTE.random))
    TE.random <- rep_len(TE.random, length(lowTE.random))
  #
  lowTE.predict <- x$lower.predict
  uppTE.predict <- x$upper.predict
  #
  Q <- unlist(x$Q)
  df.Q <- replaceNULL(unlist(x$df.Q), k - 1)
  pval.Q <- replaceNULL(unlist(x$pval.Q), pvalQ(Q, df.Q))
  #
  if (!is.null(x$Q.CMH)) {
    Q.CMH <- unlist(x$Q.CMH)
    df.Q.CMH <- replaceNULL(unlist(x$df.Q.CMH), 1)
    pval.Q.CMH <- replaceNULL(unlist(x$pval.Q.CMH), pvalQ(Q.CMH, df.Q.CMH))
  }
  #
  if (by) {
    k.w <- x$k.w
    #
    TE.common.w    <- list2mat(x$TE.common.w)
    lowTE.common.w <- list2mat(x$lower.common.w)
    uppTE.common.w <- list2mat(x$upper.common.w)
    pval.common.w  <- list2mat(x$pval.common.w)
    #
    lowTE.random.w <- list2mat(x$lower.random.w)
    TE.random.w    <- list2mat(x$TE.random.w, lowTE.random.w)
    #
    uppTE.random.w <- list2mat(x$upper.random.w)
    pval.random.w  <- list2mat(x$pval.random.w)
    #
    if (!is.matrix(TE.random.w) & is.matrix(lowTE.random.w)) {
      TE.random.w <-
        matrix(TE.random.w,
               nrow = nrow(lowTE.random.w),
               ncol = ncol(lowTE.random.w))
      rownames(TE.random.w) <- rownames(lowTE.random.w)
      colnames(TE.random.w) <- colnames(lowTE.random.w)
    }
    #
    lowTE.predict.w <- x$lower.predict.w
    uppTE.predict.w <- x$upper.predict.w
    #
    harmonic.mean.w <- x$n.harmonic.mean.w
    #
    Q.b.common <- unlist(x$Q.b.common)
    names(Q.b.common) <- colnames(lowTE.common.w)
    Q.w.common <- x$Q.w.common
    #
    Q.b.random <- unlist(x$Q.b.random)
    Q.w.random <- x$Q.w.random
    #
    Q.w <- x$Q.w
    #
    df.Q.w <- replaceNULL(x$df.Q.w, sum((k.w - 1)[!is.na(x$Q.w)]))
    df.Q.b <- replaceNULL(x$df.Q.b, (k - 1) - sum((k.w - 1)[!is.na(x$Q.w)]))
    df.Q.b.common <- replaceNULL(x$df.Q.b.common, df.Q.b)
    df.Q.b.random <- replaceNULL(x$df.Q.b.random, df.Q.b)    
    #
    pval.Q.b.common <-
      replaceNULL(x$pval.Q.b.common, pvalQ(Q.b.common, df.Q.b.common))
    pval.Q.w.common <-
      replaceNULL(x$pval.Q.w.common, pvalQ(Q.w.common, df.Q.w))
    #
    pval.Q.b.random <-
      unlist(replaceNULL(x$pval.Q.b.random, pvalQ(Q.b.random, df.Q.b.random)))
    pval.Q.w.random <-
      unlist(replaceNULL(x$pval.Q.w.random, pvalQ(Q.w.random, df.Q.w)))
  }
  #
  if (backtransf) {
    if (sm == "IRFT") {
      if (is.metabind)
        harmonic.mean <- x$t.harmonic.mean.ma
      else
        harmonic.mean <- 1 / mean(1 / x$time)
    }
    else {
      if (is.metabind)
        harmonic.mean <- x$n.harmonic.mean.ma
      else
        harmonic.mean <- 1 / mean(1 / x$n)
    }
    #
    TE.common <-
      backtransf(TE.common, sm, harmonic.mean, harmonic.mean, fbt, abt)
    lowTE.common <-
      backtransf(lowTE.common, sm, harmonic.mean, harmonic.mean, fbt, abt)
    uppTE.common <-
      backtransf(uppTE.common, sm, harmonic.mean, harmonic.mean, fbt, abt)
    #
    TE.random <-
      backtransf(TE.random, sm, harmonic.mean, harmonic.mean, fbt, abt)
    lowTE.random <-
      backtransf(lowTE.random, sm, harmonic.mean, harmonic.mean, fbt, abt)
    uppTE.random <-
      backtransf(uppTE.random, sm, harmonic.mean, harmonic.mean, fbt, abt)
    #
    lowTE.predict <-
      backtransf(lowTE.predict, sm, harmonic.mean, harmonic.mean, fbt, abt)
    uppTE.predict <-
      backtransf(uppTE.predict, sm, harmonic.mean, harmonic.mean, fbt, abt)
    #
    if (by) {
      TE.common.w <-
        backtransf(TE.common.w, sm, harmonic.mean.w, harmonic.mean.w,
                   fbt, abt)
      lowTE.common.w <-
        backtransf(lowTE.common.w, sm, harmonic.mean.w, harmonic.mean.w,
                   fbt, abt)
      uppTE.common.w  <-
        backtransf(uppTE.common.w, sm, harmonic.mean.w, harmonic.mean.w)
      #
      TE.random.w <-
        backtransf(TE.random.w, sm, harmonic.mean.w, harmonic.mean.w, fbt, abt)
      lowTE.random.w <-
        backtransf(lowTE.random.w, sm, harmonic.mean.w, harmonic.mean.w,
                   fbt, abt)
      uppTE.random.w <-
        backtransf(uppTE.random.w, sm, harmonic.mean.w, harmonic.mean.w,
                   fbt, abt)
      #
      if (prediction.w) {
        lowTE.predict.w <-
          backtransf(lowTE.predict.w, sm, harmonic.mean.w, harmonic.mean.w,
                     fbt, abt)
        uppTE.predict.w <-
          backtransf(uppTE.predict.w, sm, harmonic.mean.w, harmonic.mean.w,
                     fbt, abt)
      }
    }
  }
  #
  # Apply argument 'pscale' to proportions / risk differences and
  # 'irscale' to rates / incidence rate differences
  #
  if (pscale != 1 || irscale != 1) {
    if (pscale != 1 && irscale != 1)
      stop("Provide either arguments 'pscale' or 'irscale' but not ",
           "both arguments.",
           call. = FALSE)
    if (pscale != 1)
      scale <- pscale
    else
      scale <- irscale
    #
    TE.common    <- scale * TE.common
    lowTE.common <- scale * lowTE.common
    uppTE.common <- scale * uppTE.common
    #
    TE.random    <- scale * TE.random
    lowTE.random <- scale * lowTE.random
    uppTE.random <- scale * uppTE.random
    #
    lowTE.predict <- scale * lowTE.predict
    uppTE.predict <- scale * uppTE.predict
    #
    if (by) {
      TE.common.w    <- scale * TE.common.w
      lowTE.common.w <- scale * lowTE.common.w
      uppTE.common.w <- scale * uppTE.common.w
      #   
      TE.random.w    <- scale * TE.random.w
      lowTE.random.w <- scale * lowTE.random.w
      uppTE.random.w <- scale * uppTE.random.w
      #   
      if (prediction.w) {
        lowTE.predict.w <- scale * lowTE.predict.w
        uppTE.predict.w <- scale * uppTE.predict.w
      }
    }
  }
  #
  # Switch lower and upper limit for VE if results have been
  # backtransformed
  #
  if (backtransf & sm == "VE") {
    tmp.l <- lowTE.common
    lowTE.common <- uppTE.common
    uppTE.common <- tmp.l
    #
    tmp.l <- lowTE.random
    lowTE.random <- uppTE.random
    uppTE.random <- tmp.l
    #
    tmp.l <- lowTE.predict
    lowTE.predict <- uppTE.predict
    uppTE.predict <- tmp.l
    #
    if (by) {
      tmp.l <- lowTE.common.w
      lowTE.common.w <- uppTE.common.w
      uppTE.common.w <- tmp.l
      #   
      tmp.l <- lowTE.random.w
      lowTE.random.w <- uppTE.random.w
      uppTE.random.w <- tmp.l
      #   
      if (prediction.w) {
        tmp.l <- lowTE.predict.w
        lowTE.predict.w <- uppTE.predict.w
        uppTE.predict.w <- tmp.l
      }
    }
  }
  #
  # Round and round ...
  #
  TE.common    <- round(TE.common, digits)
  lowTE.common <- round(lowTE.common, digits)
  uppTE.common <- round(uppTE.common, digits)
  sTE.common <- round(sTE.common, digits.stat)
  #
  TE.random    <- round(TE.random, digits)
  lowTE.random <- round(lowTE.random, digits)
  uppTE.random <- round(uppTE.random, digits)
  pTE.random <- x$pval.random
  sTE.random <- round(x$statistic.random, digits.stat)
  #
  lowTE.predict <- round(lowTE.predict, digits)
  uppTE.predict <- round(uppTE.predict, digits)
  #
  if (by) {
    TE.common.w     <- round(TE.common.w, digits)
    lowTE.common.w  <- round(lowTE.common.w, digits)
    uppTE.common.w  <- round(uppTE.common.w, digits)
    #
    TE.random.w    <- round(TE.random.w, digits)
    lowTE.random.w <- round(lowTE.random.w, digits)
    uppTE.random.w <- round(uppTE.random.w, digits)
    #
    if (prediction.w) {
      lowTE.predict.w <- round(lowTE.predict.w, digits)
      uppTE.predict.w <- round(uppTE.predict.w, digits)
    }
    #
    if (print.I2)
      I2.w <- round(100 * x$I2.w, digits.I2)
    else
      I2.w <- NA
    #
    if (print.Rb)
      Rb.w <- round(100 * x$Rb.w, digits.I2)
    else
      Rb.w <- NA
  }
  #
  if (print.H) {
    H <- round(unlist(x$H), digits.H)
    lowH <- round(unlist(x$lower.H), digits.H)
    uppH <- round(unlist(x$upper.H), digits.H)
    if (all(!is.na(lowH) & !is.na(uppH)) && all(lowH == uppH)) {
      lowH <- NA
      uppH <- NA
    }
  }
  else {
    H <- lowH <- uppH <- NULL
  }
  #
  if (print.I2) {
    I2 <- round(100 * unlist(x$I2), digits.I2)
    lowI2 <- round(100 * unlist(x$lower.I2), digits.I2)
    uppI2 <- round(100 * unlist(x$upper.I2), digits.I2)
    #
    if (is.null(names(Q)))
      Q4I2 <- Q
    else
      Q4I2 <- Q[!grepl("LRT", names(Q))]
    #
    print.I2.ci <- print.I2.ci &
      ifelse(((Q4I2 > max(k, na.rm = TRUE) & max(k, na.rm = TRUE) >= 2) |
              (Q4I2 <= max(k, na.rm = TRUE) & max(k, na.rm = TRUE) > 2)) &
             !(is.na(lowI2) | is.na(uppI2)), TRUE, FALSE)
    #
    print.I2.ci[is.na(print.I2.ci)] <- FALSE
    print.I2.ci <- print.I2.ci & !(lowI2 == uppI2)
    print.I2.ci <- any(print.I2.ci)
  }
  else {
    I2 <- lowI2 <- uppI2 <- NULL
    print.I2.ci <- FALSE
  }
  #
  if (print.Rb) {
    Rb <- round(100 * unlist(x$Rb), digits.I2)
    lowRb <- round(100 * unlist(x$lower.Rb), digits.I2)
    uppRb <- round(100 * unlist(x$upper.Rb), digits.I2)
    #
    if (all(!is.na(lowRb) & !is.na(uppRb)) && all(lowRb == uppRb)) {
      lowRb <- rep(NA, length(Rb))
      uppRb <- rep(NA, length(Rb))
    }
  }
  else {
    Rb <- lowRb <- uppRb <- NULL
    print.Rb.ci <- FALSE    
  }
  #
  three.level <- if (is.null(x$three.level)) FALSE else any(x$three.level)
  is.glmm <- any(method == "GLMM") | any(method.random == "GLMM")
  #
  sel.n <- inherits(x, c("metacor", "metaprop", "metamean", "metarate"))
  #
  sel.ev <- inherits(x, c("metaprop", "metarate"))
  
  
  #
  #
  # (5) Print result for meta-analysis
  #
  #
  is.metamiss <- inherits(x, "metamiss")
  if (header) {
    if (is.metamiss)
      cat("Sensitivity analysis for missing binary data\n")
    #
    crtitle(x)
  }
  #
  if (all(is.na(k.all))) {
    # Do nothing
    return(invisible(NULL))
  }
  else if (all(k.all == 1)) {
    #
    # Print results for a single study
    #
    if (!(metaprop | metarate) | (overall & (common | random)) |
        overall.hetstat | by)
      method <- ifelse(method == "Peto", method, "")
    #
    if (!is.metabind) {
      if (is.metamiss)
        cat("\n")
      else if (sel.n)
        catobsev(if (is.null(x$exclude)) x$n else x$n[!x$exclude],
                 type = "n")
      else
        catobsev(if (is.null(x$exclude)) x$n.e else x$n.e[!x$exclude],
                 if (is.null(x$exclude)) x$n.c else x$n.c[!x$exclude],
                 type = "n")
      #
      if (sel.ev)
        catobsev(if (is.null(x$exclude)) x$event else x$event[!x$exclude],
                 type = "e", addrow = TRUE)
      else if (!is.metamiss)
        catobsev(if (is.null(x$exclude)) x$event.e else x$event.e[!x$exclude],
                 if (is.null(x$exclude)) x$event.c else x$event.c[!x$exclude],
                 type = "e", addrow = TRUE)
    }
    #
    res <- cbind(formatN(TE.common, digits, "NA",
                         big.mark = big.mark),
                 formatCI(formatN(lowTE.common, digits, "NA",
                                  big.mark = big.mark),
                          formatN(uppTE.common, digits, "NA",
                                  big.mark = big.mark)),
                 if (null.given)
                   formatN(sTE.common, digits.stat, big.mark = big.mark),
                 if (null.given)
                   formatPT(pTE.common, digits = digits.pval,
                            scientific = scientific.pval,
                            zero = zero.pval, JAMA = JAMA.pval))
    dimnames(res) <- list(x$studlab,
                          c(sm.lab, ci.lab,
                            if (null.given) "z",
                            if (null.given) "p-value"))
    prmatrix(res, quote = FALSE, right = TRUE, ...)
    #
    # Print information on meta-analysis method:
    #
    x$pscale <- pscale
    x$irscale <- irscale
    x$irunit <- irunit
    #
    if (!is.null(x$cycles))
      x$sd.n_of_1 <-
        formatPT(x$sd.n_of_1, digits = gs("digits.sd"), big.mark = big.mark)
    #
    if (details.methods)
      details <-
        catmeth(x,
                common, random, prediction, overall, overall.hetstat,
                #
                func.transf = x$func.transf,
                backtransf = backtransf, func.backtransf = fbt,
                #
                big.mark = big.mark, digits = digits,
                digits.tau = digits.tau,
                text.tau = text.tau, text.tau2 = text.tau2,
                #
                print.tau2 = FALSE)
  }
  else {
    #
    #
    # Print results for meta-analysis with more than one study
    #
    #
    zlab <- ""
    #
    if (header & is.metamiss)
      cat("\n")
    #
    if (all(!is.na(k))) {
      if (overall & (common | random | prediction)) {
        if (!inherits(x, "trimfill")) {
          if (any(method == "MH") &&
              (inherits(x, c("metabin", "metainc")) &
               common & sm %in% c("RD", "IRD") &
               (!is.null(x$k.MH) && any(k != x$k.MH, na.rm = TRUE))))
            cat(paste0("Number of studies:   k.MH = ",
                       cond(x$k.MH, digits = 0),
                       " (", text.common.br[1], "), k = ",
                       format(cond(k, digits = 0),
                              big.mark = big.mark),
                       " (", text.random.br[1], ")\n",
                       collapse = ""))
          else {
            if (any(k.study != k, na.rm = TRUE)) {
              cat(paste0("Number of studies: n = ",
                         format(cond(x$k.study, digits = 0),
                                big.mark = big.mark), "\n",
                         collapse = ""))
              cat(paste0("Number of estimates: k = ",
                         format(cond(k, digits = 0),
                                big.mark = big.mark), "\n",
                         collapse = ""))
            }
            else
              cat(paste0("Number of studies: k = ",
                         format(cond(k, digits = 0),
                                big.mark = big.mark), "\n",
                         collapse = ""))
          }
        }
        else
          cat(paste0("Number of studies: k = ",
                     format(cond(k, digits = 0),
                            big.mark = big.mark),
                     " (with ",
                     format(cond(x$k0, digits = 0),
                            big.mark = big.mark),
                     " added studies)\n",
                     collapse = ""))
        #
        if (!is.metabind) {
          if (is.metamiss)
            cat("\n")
          else if (sel.n)
            catobsev(if (is.null(x$exclude)) x$n else x$n[!x$exclude],
                     type = "n")
          else
            catobsev(if (is.null(x$exclude)) x$n.e else x$n.e[!x$exclude],
                     if (is.null(x$exclude)) x$n.c else x$n.c[!x$exclude],
                     type = "n")
          #
          if (sel.ev)
            catobsev(if (is.null(x$exclude)) x$event else x$event[!x$exclude],
                     type = "e", addrow = TRUE)
          else if (!is.metamiss)
            catobsev(if (is.null(x$exclude)) x$event.e
                     else x$event.e[!x$exclude],
                     if (is.null(x$exclude)) x$event.c
                     else x$event.c[!x$exclude],
                     type = "e", addrow = TRUE)
        }
        #
        res <- cbind(formatN(c(if (common) TE.common,
                               if (random) TE.random,
                               if (prediction)
                                 rep(NA, length(lowTE.predict))),
                             digits, "NA",
                             big.mark = big.mark),
                     formatCI(formatN(c(if (common) lowTE.common,
                                        if (random) lowTE.random,
                                        if (prediction) lowTE.predict),
                                      digits, "NA", big.mark = big.mark),
                              formatN(c(if (common) uppTE.common,
                                        if (random) uppTE.random,
                                        if (prediction) uppTE.predict),
                                      digits, "NA", big.mark = big.mark)),
                     if (null.given)
                       formatN(c(if (common) sTE.common,
                                 if (random) sTE.random,
                                 if (prediction)
                                   rep(NA, length(lowTE.predict))),
                               digits = digits.stat, big.mark = big.mark),
                     if (null.given)
                       formatPT(c(if (common) pTE.common,
                                  if (random) pTE.random,
                                  if (prediction)
                                    rep(NA, length(lowTE.predict))),
                                digits = digits.pval,
                                scientific = scientific.pval,
                                zero = zero.pval, JAMA = JAMA.pval))
        if (prediction) {
          seq <- (nrow(res) - length(lowTE.predict) + 1):nrow(res)
          res[seq, 1] <- ""
          if (null.given)
            res[seq, 3:4] <- ""
        }
        #
        if (any(method.random.ci %in% c("HK", "KR"))) {
          if ((common & random) | any(method.random.ci == "classic"))
            zlab <- "z|t"
          else if (common & !random)
            zlab <- "z"
          else if (!common & random)
            zlab <- "t"
        }
        else
          zlab <- "z"
        #
        if (prediction)
          if (x$level.ma == x$level.predict)
            lab.predict <- text.predict
          else
            lab.predict <-
              paste0(text.predict,
                    " (", round(100 * x$level.predict, 1), "%-PI)")
        #
        dimnames(res) <- list(c(if (common) text.common,
                                if (random) text.random,
                                if (prediction) lab.predict),
                              c(sm.lab, ci.lab,
                                if (null.given) zlab,
                                if (null.given) "p-value"))
        #
        sort.overall <- setsort(sort.overall, nrow(res), "overall results")
        #
        if (prediction & !(common | random))
          res <- res[, ci.lab, drop = FALSE]
        #
        prmatrix(res[sort.overall, , drop = FALSE],
                 quote = FALSE, right = TRUE, ...)
        #
        if (metabin && print.CMH) {
          Qdata <- cbind(formatN(round(Q.CMH, digits.Q), digits.Q, "NA",
                                 big.mark = big.mark),
                         df.Q.CMH,
                         formatPT(pval.Q.CMH,
                                  digits = digits.pval.Q,
                                  scientific = scientific.pval,
                                  zero = zero.pval, JAMA = JAMA.pval))
          dimnames(Qdata) <- list("", c("Q", "d.f.", "p-value"))
          #
          cat("\nCochran-Mantel-Haenszel (CMH) test for overall effect: \n")
          prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
        }
      }
      else {
        if (any(k.study != k)) {
          cat(paste0("Number of studies: ",
                     if (is.netpairwise) "k" else "n", " = ",
                     format(cond(x$k.study, digits = 0),
                            big.mark = big.mark), "\n"))
          cat(paste0("Number of ",
                     if (is.netpairwise | pairwise)
                       "pairwise comparisons: m"
                     else
                       "estimates: k",
                     " = ",
                     format(cond(k, digits = 0),
                            big.mark = big.mark), "\n"))
        }
        else
          cat(paste0("Number of studies: k",
                     " = ",
                     format(cond(k, digits = 0),
                            big.mark = big.mark), "\n"))
        #
        if (!is.metabind) {
          if (is.metamiss)
            cat("\n")
          else if (sel.n)
            catobsev(if (is.null(x$exclude)) x$n else x$n[!x$exclude],
                     type = "n")
          else
            catobsev(if (is.null(x$exclude)) x$n.e else x$n.e[!x$exclude],
                     if (is.null(x$exclude)) x$n.c else x$n.c[!x$exclude],
                     type = "n")
          #
          if (sel.ev)
            catobsev(if (is.null(x$exclude)) x$event else x$event[!x$exclude],
                     type = "e")
          else if (!is.metamiss)
            catobsev(if (is.null(x$exclude)) x$event.e else x$event.e[!x$exclude],
                     if (is.null(x$exclude)) x$event.c else x$event.c[!x$exclude],
                     type = "e")
        }
      }
    }
    #
    # Print information on heterogeneity
    #
    if (overall.hetstat) {
      print.tau2.ci <-
        print.tau2.ci & !all(is.na(x$lower.tau2) & is.na(x$upper.tau2))
      if (print.tau2.ci &&
          (all(x$lower.tau2 == 0, na.rm = TRUE) &
           all(x$upper.tau2 == 0, na.rm = TRUE)))
        print.tau2.ci <- FALSE
      #
      print.tau.ci <-
        print.tau.ci & !all(is.na(x$lower.tau) & is.na(x$upper.tau))
      if (print.tau.ci &&
          (all(x$lower.tau == 0, na.rm = TRUE) &
           all(x$upper.tau == 0, na.rm = TRUE)))
        print.tau.ci <- FALSE
      #
      cat(paste0("\nQuantifying heterogeneity",
                 if (!(is.null(x$level.hetstat) || is.na(x$level.hetstat)) &
                     (print.tau2.ci | print.tau.ci | print.I2.ci))
                   paste0(" (with ", 100 * x$level.hetstat, "%-CI",
                          if (print.tau2.ci + print.tau.ci +
                              print.I2.ci + print.H * print.I2.ci > 1) "s",
                          ")"),
                 ":"))
      #
      if (sum(c(print.tau2, print.tau, print.I2, print.H, print.Rb)) > 1)
        cat("\n")
      else
        cat("")
      #
      cathet(k,
             x$method.tau, x$detail.tau,
             x$tau2, x$lower.tau2, x$upper.tau2,
             print.tau2, print.tau2.ci, text.tau2, digits.tau2,
             x$tau, x$lower.tau, x$upper.tau,
             print.tau, print.tau.ci, text.tau, digits.tau,
             x$sign.lower.tau, x$sign.upper.tau,
             I2, lowI2, uppI2,
             print.I2, print.I2.ci, text.I2, digits.I2,
             H, lowH, uppH,
             print.H, digits.H,
             Rb, lowRb, uppRb,
             print.Rb, text.Rb,
             big.mark,
             sort.tau, sort.het)
      #
      # Print information on residual heterogeneity
      #
      if (by & !is.metabind) {
        #
        Q.resid <- x$Q.w.common
        k.resid <- x$df.Q.w + 1
        #
        if (print.H) {
          H.resid <- round(replaceNULL(x$H.resid), digits.H)
          lowH.resid <- round(replaceNULL(x$lower.H.resid), digits.H)
          uppH.resid <- round(replaceNULL(x$upper.H.resid), digits.H)
        }
        #
        I2.resid <- round(100 * replaceNULL(x$I2.resid), digits.I2)
        #
        if (print.I2) {
          lowI2.resid <- round(100 * replaceNULL(x$lower.I2.resid), digits.I2)
          uppI2.resid <- round(100 * replaceNULL(x$upper.I2.resid), digits.I2)
          print.I2.ci <-
            ((replaceNULL(Q.resid) > replaceNULL(k.resid) &
              replaceNULL(k.resid) >= 2) |
             (replaceNULL(Q.resid) <= replaceNULL(k.resid) &
              replaceNULL(k.resid) > 2)) &
            !(is.na(replaceNULL(lowI2.resid)) |
              is.na(replaceNULL(uppI2.resid)))
          #
          print.I2.ci <- unique(print.I2.ci)          
          if (is.na(print.I2.ci))
            print.I2.ci <- FALSE
        }
        #
        if (!is.na(replaceNULL(I2.resid))) {
          cat(paste0("\nQuantifying residual heterogeneity",
                     if (!(is.null(x$level.hetstat) || is.na(x$level.hetstat)) &
                         (print.tau2.ci | print.tau.ci | print.I2.ci))
                       paste0(" (with ", 100 * x$level.hetstat, "%-CI",
                              if (print.tau2.ci + print.tau.ci +
                                  print.I2.ci + print.H * print.I2.ci > 1) "s",
                              ")"),
                     ":"))
          #
          if (sum(c(print.tau2, print.tau, print.I2, print.H, print.Rb)) > 1)
            cat("\n")
          else
            cat("")
          #
          cathet(k.resid, 
                 x$method.tau, x$detail.tau,
                 x$tau2.resid, x$lower.tau2.resid, x$upper.tau2.resid,
                 x$tau.common, print.tau2.ci, text.tau2, digits.tau2,
                 x$tau.resid, x$lower.tau.resid, x$upper.tau.resid,
                 x$tau.common, print.tau.ci, text.tau, digits.tau,
                 "", "",
                 I2.resid, lowI2.resid, uppI2.resid,
                 print.I2, print.I2.ci, text.I2, digits.I2,
                 H.resid, lowH.resid, uppH.resid,
                 print.H, digits.H,
                 NA, NA, NA,
                 FALSE, "",
                 big.mark)
        }
      }
      #
      # Test of heterogeneity
      #
      if (print.Q & (common | random)) {
        if (any(k > 1, na.rm = TRUE)) {
          #
          Qdat <- qdat(Q, df.Q, pval.Q, x$hetlabel, text.common)
          #
          sort.Q <- setsort(sort.Q, nrow(Qdat), "tests of heterogeneity")
          #
          Qdata <- cbind(formatN(round(Qdat$Q, digits.Q), digits.Q, "NA",
                                 big.mark = big.mark),
                         format(Qdat$df.Q, big.mark = big.mark),
                         formatPT(Qdat$pval.Q,
                                  digits = digits.pval.Q,
                                  scientific = scientific.pval,
                                  zero = zero.pval, JAMA = JAMA.pval))
          dimnames(Qdata) <- list(Qdat$names, c("Q", "d.f.", "p-value"))
          #
          Qdata <- Qdata[sort.Q, , drop = FALSE]
          #
          cat("\nTest of heterogeneity:\n")
          prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
        }
      }
    }
    else if (!by) {
      print.tau2 <- FALSE
      print.tau2.ci <- FALSE
      print.tau <- FALSE
      print.tau.ci <- FALSE
    }
    else {
      print.tau2.ci <- FALSE
      print.tau.ci <- FALSE
    }
    #
    # Print information for subgroup analysis
    #
    if (by) {
      if (is.metabind)
        anaunit <- "meta-analyses"
      else if (is.netpairwise)
        anaunit <- "pairwise comparisons"
      else
        anaunit <- "subgroups"
      #
      if (common) {
        #
        # Subgroup analysis based on common effect model
        #
        if (is.vector(TE.common.w)) {
          nam <- names(TE.common.w)
          TE.common.w <- matrix(TE.common.w, ncol = 1)
          lowTE.common.w <- matrix(lowTE.common.w, ncol = 1)
          uppTE.common.w <- matrix(uppTE.common.w, ncol = 1)
          #
          if (!is.null(nam))
            rownames(TE.common.w) <- rownames(lowTE.common.w) <-
              rownames(uppTE.common.w) <- nam
        }
        #
        n.common <- ncol(TE.common.w)
        #
        if (!is.list(df.Q.b.common))
          df.Q.b.common <- list(df.Q.b.common)
        #
        df.Q.b.common <- do.call("c", df.Q.b.common)
        #
        for (i in seq_len(n.common)) {
          Tdata <- cbind(format(k.w, big.mark = big.mark),
                         formatN(TE.common.w[, i], digits, "NA",
                                 big.mark = big.mark),
                         formatCI(formatN(lowTE.common.w[, i], digits, "NA",
                                          big.mark = big.mark),
                                  formatN(uppTE.common.w[, i], digits, "NA",
                                          big.mark = big.mark)),
                         if (print.Q)
                           formatN(round(Q.w, digits.Q), digits.Q,
                                   big.mark = big.mark),
                         if (print.I2)
                           ifelse(is.na(I2.w),
                                  "--",
                                  paste0(formatN(I2.w, digits.I2), "%")),
                         if (print.Rb)
                           ifelse(is.na(Rb.w),
                                  "--",
                                  paste0(formatN(Rb.w, digits.I2), "%")),
                         if (print.tau2 & !random & !is.list(x$tau.w))
                           ifelse(k.w == 1 & !x$tau.common, "--",
                                  formatPT(x$tau.w^2,
                                           digits = digits.tau2,
                                           big.mark = big.mark,
                                           noblanks = TRUE)),
                         if (print.tau & !random & !is.list(x$tau.w))
                           ifelse(k.w == 1 & !x$tau.common, "--",
                                  formatPT(x$tau.w,
                                           digits = digits.tau,
                                           big.mark = big.mark,
                                           noblanks = TRUE))
                         )
          #
          bylab.txt <- bylabel(subgroup.name, subgroup.levels,
                               print.subgroup.name, sep.subgroup,
                               big.mark = big.mark)
          #
          dimnames(Tdata) <- list(bylab.txt,
                                  c(if (is.netpairwise | pairwise)
                                      "  m" else "  k",
                                    sm.lab, ci.lab,
                                    if (print.Q) "Q",
                                    if (print.I2) text.I2,
                                    if (print.Rb) text.Rb,
                                    if (print.tau2 & !random) text.tau2,
                                    if (print.tau & !random) text.tau)
                                  )
          #
          if (is.metabind & !is.null(x$common.meta))
            Tdata <- Tdata[x$common.meta, , drop = FALSE]
          #
          cat(paste0("\nResults for ", anaunit,
                     paste0(" (", text.common.br[i], "):\n")))
          #
          prmatrix(Tdata, quote = FALSE, right = TRUE, ...)
          #
          df.i <- df.Q.b.common[[i]]
          #
          if (length(df.i) > 1) {
            dfs.b.i <-
              rmSpace(paste(formatN(unlist(df.i), digits = 0,
                                    big.mark = big.mark),
                            collapse = ", "), end = TRUE)
            Q.lab <- "F"
          }
          else {
            dfs.b.i <- formatN(df.i, digits = 0, big.mark = big.mark)
            Q.lab <- "Q"
          }
          #
          if (test.subgroup.common & !is.metabind) {
            cat(paste0("\nTest for subgroup differences (",
                       text.common.br[i], "):\n"))
            if (any(method == "MH") | all(is.na(Q.w.common))) {
              Qdata <- cbind(formatN(round(Q.b.common[i], digits.Q),
                                     digits.Q, "NA", big.mark = big.mark),
                             formatN(dfs.b.i, digits = 0,
                                     big.mark = big.mark),
                             formatPT(pval.Q.b.common[i],
                                      digits = digits.pval.Q,
                                      scientific = scientific.pval,
                                      zero = zero.pval, JAMA = JAMA.pval))
              dimnames(Qdata) <- list("Between groups",
                                      c(Q.lab, "d.f.", "p-value"))
              prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
            }
            else {
              Qs  <- c(Q.b.common[i], Q.w.common[i])
              dfs <- c(dfs.b.i,
                       formatN(df.Q.w, digits = 0, big.mark = big.mark))
              if (is.glmm & Q.lab == "F")
                Q.lab <- "F/Q"
              #
              pvals <- c(pval.Q.b.common[i], pval.Q.w.common[i])
              #
              Qdata <- cbind(formatN(round(Qs, digits.Q), digits.Q, "NA",
                                     big.mark = big.mark),
                             dfs,
                             formatPT(pvals,
                                      digits = digits.pval.Q,
                                      scientific = scientific.pval,
                                      zero = zero.pval, JAMA = JAMA.pval))
              dimnames(Qdata) <- list(c("Between groups", "Within groups"),
                                      c(Q.lab, "d.f.", "p-value"))
              prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
            }
          }
        }
      }
      #
      if (random) {
        #
        # Subgroup analysis based on random effects model
        #
        if (is.vector(TE.random.w)) {
          nam <- names(TE.random.w)
          TE.random.w <- matrix(TE.random.w, ncol = 1)
          lowTE.random.w <- matrix(lowTE.random.w, ncol = 1)
          uppTE.random.w <- matrix(uppTE.random.w, ncol = 1)
          #
          if (!is.null(nam))
            rownames(TE.random.w) <- rownames(lowTE.random.w) <-
              rownames(uppTE.random.w) <- nam
        }
        #
        n.random <- ncol(TE.random.w)
        #
        if (!is.list(x$tau.w))
          x$tau.w <- list(x$tau.w)
        #
        if (!is.list(df.Q.b.random))
          df.Q.b.random <- list(df.Q.b.random)
        #
        df.Q.b.random <- do.call("c", df.Q.b.random)
        #
        for (i in seq_len(n.random)) {
          tau.i <- length(x$tau.w) >= i
          #
          Tdata <- cbind(format(k.w, big.mark = big.mark),
                         formatN(TE.random.w[, i], digits, "NA",
                                 big.mark = big.mark),
                         formatCI(formatN(lowTE.random.w[, i], digits, "NA",
                                          big.mark = big.mark),
                                  formatN(uppTE.random.w[, i], digits, "NA",
                                          big.mark = big.mark)),
                         if (print.tau2 & (i == 1 | tau.i))
                           ifelse(k.w == 1 & !x$tau.common, "--",
                                  formatPT(x$tau.w[[i]]^2,
                                           digits = digits.tau2,
                                           big.mark = big.mark,
                                           noblanks = TRUE)),
                         if (print.tau & (i == 1 | tau.i))
                           ifelse(k.w == 1 & !x$tau.common, "--",
                                  formatPT(x$tau.w[[i]],
                                           digits = digits.tau,
                                           big.mark = big.mark,
                                           noblanks = TRUE)),
                         if (i == 1 & !common & print.Q)
                           formatN(round(Q.w, digits.Q), digits.Q,
                                   big.mark = big.mark),
                         if (i == 1 & !common & print.I2)
                           ifelse(is.na(I2.w),
                                  "--",
                                  paste0(formatN(I2.w, digits.I2), "%")),
                         if (i == 1 & !common & print.Rb)
                           ifelse(is.na(Rb.w),
                                  "--",
                                  paste0(formatN(Rb.w, digits.I2,
                                                 big.mark = big.mark), "%"))
                         )
          #
          bylab.txt <- bylabel(subgroup.name, subgroup.levels,
                               print.subgroup.name, sep.subgroup,
                               big.mark = big.mark)
          #
          dimnames(Tdata) <- list(bylab.txt,
                                  c(if (is.netpairwise | pairwise)
                                      "  m" else "  k",
                                    sm.lab, ci.lab,
                                    if (print.tau2 & (i == 1 | tau.i))
                                      text.tau2,
                                    if (print.tau & (i == 1 | tau.i))
                                      text.tau,
                                    if (i == 1 & !common & print.Q) "Q",
                                    if (i == 1 & !common & print.I2) text.I2,
                                    if (i == 1 & !common & print.Rb) text.Rb)
                                  )
          #
          if (is.metabind & !is.null(x$random.meta))
            Tdata <- Tdata[x$random.meta, , drop = FALSE]
          #
          cat(paste0("\nResults for ", anaunit,
                     paste0(" (", text.random.br[i], "):\n")))
          #
          prmatrix(Tdata, quote = FALSE, right = TRUE, ...)
          #
          df.i <- df.Q.b.random[[i]]
          #
          if (length(df.i) > 1) {
            dfs.b.i <-
              rmSpace(paste(formatN(df.i, digits = 0,
                                    big.mark = big.mark),
                            collapse = ", "), end = TRUE)
            Q.lab <- "F"
          }
          else {
            dfs.b.i <- formatN(df.i, digits = 0, big.mark = big.mark)
            Q.lab <- "Q"
          }
          #
          if (test.subgroup.random & !is.na(Q.b.random[i]) & !is.metabind) {
            cat(paste0("\nTest for subgroup differences (",
                       text.random.br[i],
                       "):\n"))
            if (all(is.na(Q.w.random))) {
              Qdata <- cbind(formatN(round(Q.b.random[i], digits.Q),
                                     digits.Q, "NA", big.mark = big.mark),
                             formatN(dfs.b.i, digits = 0,
                                     big.mark = big.mark),
                             formatPT(pval.Q.b.random[i],
                                      digits = digits.pval.Q,
                                      scientific = scientific.pval,
                                      zero = zero.pval, JAMA = JAMA.pval))
              dimnames(Qdata) <- list("Between groups",
                                      c(Q.lab, "d.f.", "p-value"))
            }
            else {
              Qs  <- c(Q.b.random[i], Q.w.random[i])
              dfs <- c(dfs.b.i,
                       formatN(df.Q.w, digits = 0, big.mark = big.mark))
              Q.lab <-
                ifelse((three.level | is.glmm) && Q.lab == "F", "F/Q", Q.lab)
              #
              pvals <- c(pval.Q.b.random[i], pval.Q.w.random[i])
              #
              Qdata <- cbind(formatN(round(Qs, digits.Q), digits.Q, "NA",
                                     big.mark = big.mark),
                             dfs,
                             formatPT(pvals,
                                      digits = digits.pval.Q,
                                      scientific = scientific.pval,
                                      zero = zero.pval, JAMA = JAMA.pval))
              dimnames(Qdata) <- list(c("Between groups", "Within groups"),
                                      c(Q.lab, "d.f.", "p-value"))
            }
            prmatrix(Qdata, quote = FALSE, right = TRUE, ...)
          }
        }
      }
      #
      if (prediction.w) {
        #
        # Prediction intervals for subgroups
        #
        if (is.vector(lowTE.predict.w)) {
          nam <- names(lowTE.predict.w)
          lowTE.predict.w <- matrix(lowTE.predict.w, ncol = 1)
          uppTE.predict.w <- matrix(uppTE.predict.w, ncol = 1)
          #
          rownames(lowTE.predict.w) <- rownames(uppTE.predict.w) <- nam
        }
        #
        for (i in seq_len(ncol(lowTE.predict.w))) {
          Pdata <- cbind(formatCI(formatN(lowTE.predict.w[, i], digits, "NA",
                                          big.mark = big.mark),
                                  formatN(uppTE.predict.w[, i], digits, "NA",
                                          big.mark = big.mark)))
          #
          bylab.txt <-
            bylabel(subgroup.name, subgroup.levels,
                    print.subgroup.name, sep.subgroup,
                    big.mark = big.mark)
          lab.predict <- paste0(round(100 * x$level.predict, 1), "%-PI")
          dimnames(Pdata) <- list(bylab.txt, lab.predict)
          #
          Pdata <- Pdata[prediction.subgroup.logical, , drop = FALSE]
          #
          cat(paste0("\n", text.predict[i], " for subgroups:\n"))
          prmatrix(Pdata, quote = FALSE, right = TRUE, ...)
        }
      }
    }
    #
    # Print information on meta-analysis method:
    #
    x$pscale <- pscale
    x$irscale <- irscale
    x$irunit <- irunit
    #
    if (!is.null(x$cycles))
      x$sd.n_of_1 <-
      formatPT(x$sd.n_of_1, digits = gs("digits.sd"), big.mark = big.mark)
    #
    if (details.methods &
        (common | random | prediction | overall.hetstat | by))
      details <-
        catmeth(x,
                common, random, prediction, overall, overall.hetstat,
                #
                func.transf = x$func.transf,
                backtransf = backtransf, func.backtransf = fbt,
                #
                big.mark = big.mark, digits = digits,
                digits.tau = digits.tau,
                text.tau = text.tau, text.tau2 = text.tau2,
                #
                print.tau2 = print.tau2,
                print.tau2.ci = print.tau2 & print.tau2.ci,
                print.tau = print.tau,
                print.tau.ci = print.tau & print.tau.ci,
                #
                print.I2 = print.I2, text.I2 = text.I2,
                #
                print.df = overall, prediction.subgroup = prediction.subgroup)
  }
  
  invisible(details)
}





#' @rdname print.meta
#' @export cilayout

cilayout <- function(bracket = gs("CIbracket"),
                     separator = gs("CIseparator"),
                     lower.blank = gs("CIlower.blank"),
                     upper.blank = gs("CIupper.blank")) {

  chkchar(bracket, length = 1)
  chkchar(separator, length = 1)
  #
  bracket <- setchar(bracket, c("[", "(", "{", ""))
  #
  chklogical(lower.blank)
  chklogical(upper.blank)
  
  setOption("CIbracket", bracket)
  setOption("CIseparator", separator)
  setOption("CIlower.blank", lower.blank)
  setOption("CIupper.blank", upper.blank)
  
  invisible(NULL)
}

#' Meta-analysis of incidence rates
#' 
#' @description
#' Calculation of common effect and random effects estimates (incidence
#' rate ratio or incidence rate difference) for meta-analyses with
#' event counts.  Mantel-Haenszel, Cochran, inverse variance method,
#' and generalised linear mixed model (GLMM) are available for
#' pooling. For GLMMs, the \code{\link[metafor]{rma.glmm}} function
#' from R package \bold{metafor} (Viechtbauer 2010) is called
#' internally.
#' 
#' @param event.e Number of events in experimental group or an R object
#'   created with \code{\link{pairwise}}.
#' @param time.e Person time at risk in experimental group.
#' @param event.c Number of events in control group.
#' @param time.c Person time at risk in control group.
#' @param studlab An optional vector with study labels.
#' @param data An optional data frame containing the study
#'   information, i.e., event.e, time.e, event.c, and time.c.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots.
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param method A character string indicating which method is to be
#'   used for pooling of studies. One of \code{"MH"},
#'   \code{"Inverse"}, \code{"Cochran"}, or \code{"GLMM"} can be
#'   abbreviated.
#' @param sm A character string indicating which summary measure
#'   (\code{"IRR"}, \code{"IRD"}, \code{"IRSD"}, or \code{"VE"}) is to
#'   be used for pooling of studies, see Details.
#' @param incr A numerical value which is added to cell frequencies
#'   for studies with a zero cell count or a numeric vector with the continuity
#'   correction for each study, see Details.
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (\code{"only0"},
#'   \code{"if0all"}, \code{"all"}, or \code{"user"}), see Details.
#' @param incr.e Continuity correction in experimental group, see Details.
#' @param incr.c Continuity correction in control group, see Details.
#' @param model.glmm A character string indicating which GLMM should
#'   be used. One of \code{"UM.FS"}, \code{"UM.RS"}, and
#'   \code{"CM.EL"}, see Details.
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param method.bias A character string indicating which test is to
#'   be used. Either \code{"Begg"}, \code{"Egger"}, or
#'   \code{"Thompson"}, can be abbreviated. See function
#'   \code{\link{metabias}}.
#' @param n.e Number of observations in experimental group (optional).
#' @param n.c Number of observations in control group (optional).
#' @param backtransf A logical indicating whether results for
#'   incidence rate ratio (\code{sm = "IRR"}) and vaccine efficacy or
#'   vaccine effectiveness (\code{sm = "VE"}) should be back
#'   transformed in printouts and plots. If TRUE (default), results
#'   will be presented as incidence rate ratios or vaccine efficacy /
#'   effectiveness; otherwise log incidence rate ratios or log vaccine
#'   rate ratios will be shown.
#' @param irscale A numeric defining a scaling factor for printing of
#'   incidence rate differences.
#' @param irunit A character string specifying the time unit used to
#'   calculate rates, e.g. person-years.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.e Label for experimental group.
#' @param label.c Label for control group.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param hakn Deprecated argument (replaced by 'method.random.ci').
#' @param adhoc.hakn Deprecated argument (replaced by
#'   'adhoc.hakn.ci').
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if \code{incr} is added to studies with zero cell
#'   frequencies or if estimation problems exist in fitting a GLMM).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}} or
#'   \code{\link[metafor]{rma.glmm}}, respectively.
#' @param \dots Additional arguments passed on to
#'   \code{\link[metafor]{rma.glmm}} function and to catch deprecated
#'   arguments.
#' 
#' @details
#' Calculation of common and random effects estimates for meta-analyses
#' comparing two incidence rates.
#' 
#' The following measures of treatment effect are available:
#' \itemize{
#' \item Incidence Rate Ratio (\code{sm = "IRR"})
#' \item Incidence Rate Difference (\code{sm = "IRD"})
#' \item Square root transformed Incidence Rate Difference (\code{sm =
#'   "IRSD"})
#' \item Vaccine efficacy or vaccine effectiveness (\code{sm = "VE"})
#' }
#'
#' Note, log incidence rate ratio (logIRR) and log vaccine ratio
#' (logVR) are mathematical identical, however, back-transformed
#' results differ as vaccine efficacy or effectiveness is defined as
#' \code{VE = 100 * (1 - IRR)}.
#' 
#' A three-level random effects meta-analysis model (Van den Noortgate
#' et al., 2013) is utilised if argument \code{cluster} is used and at
#' least one cluster provides more than one estimate. Internally,
#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
#' "rowsum"} is used to calculate random effects weights.
#' 
#' Default settings are utilised for several arguments (assignments
#' using \code{\link{gs}} function). These defaults can be changed for
#' the current R session using the \code{\link{settings.meta}}
#' function.
#' 
#' Furthermore, R function \code{\link{update.meta}} can be used to
#' rerun a meta-analysis with different settings.
#' 
#' \subsection{Meta-analysis method}{
#' 
#' By default, both common effect and random effects models are
#' considered (see arguments \code{common} and \code{random}). If
#' \code{method} is \code{"MH"} (default), the Mantel-Haenszel method
#' is used to calculate the common effect estimate (Greenland &
#' Robbins, 1985); if \code{method} is \code{"Inverse"}, inverse
#' variance weighting is used for pooling; if \code{method} is
#' \code{"Cochran"}, the Cochran method is used for pooling
#' (Bayne-Jones, 1964, Chapter 8). For these three methods, the random
#' effects estimate is always based on the inverse variance method.
#' 
#' A distinctive and frequently overlooked advantage of incidence
#' rates is that individual patient data (IPD) can be extracted from
#' count data. Accordingly, statistical methods for IPD, i.e.,
#' generalised linear mixed models, can be utilised in a meta-analysis
#' of incidence rate ratios (Stijnen et al., 2010). These methods are
#' available (argument \code{method = "GLMM"}) for the common effect
#' and random effects model by calling the
#' \code{\link[metafor]{rma.glmm}} function from R package
#' \bold{metafor} internally.
#'
#' Three different GLMMs are available for meta-analysis of incidence
#' rate ratios using argument \code{model.glmm} (which corresponds to
#' argument \code{model} in the \code{\link[metafor]{rma.glmm}}
#' function):
#' \tabular{cl}{
#' 1. \tab Poisson regression model with fixed study effects (default)
#'  \cr
#'  \tab (\code{model.glmm = "UM.FS"}, i.e., \bold{U}nconditional
#'  \bold{M}odel - \bold{F}ixed \bold{S}tudy effects) \cr
#' 2. \tab Mixed-effects Poisson regression model with random study
#'  effects \cr
#'  \tab (\code{model.glmm = "UM.RS"}, i.e., \bold{U}nconditional
#'  \bold{M}odel - \bold{R}andom \bold{S}tudy effects) \cr
#' 3. \tab Generalised linear mixed model (conditional Poisson-Normal)
#'  \cr
#'  \tab (\code{model.glmm = "CM.EL"}, i.e., \bold{C}onditional
#'   \bold{M}odel - \bold{E}xact \bold{L}ikelihood)
#' }
#'
#' Details on these three GLMMs as well as additional arguments which
#' can be provided using argument '\code{\dots}' in \code{metainc}
#' are described in \code{\link[metafor]{rma.glmm}} where you can also
#' find information on the iterative algorithms used for estimation.
#' Note, regardless of which value is used for argument
#' \code{model.glmm}, results for two different GLMMs are calculated:
#' common effect model (with fixed treatment effect) and random effects
#' model (with random treatment effects).
#' }
#' 
#' \subsection{Continuity correction}{
#'
#' Four approaches are available to apply a continuity correction:
#' \itemize{
#' \item Only studies with a zero cell count (\code{method.incr =
#'   "only0", default})
#' \item All studies if at least one study has a zero cell count
#'   (\code{method.incr = "if0all"})
#' \item All studies irrespective of zero cell counts
#'   (\code{method.incr = "all"})
#' \item Use values provided in arguments \code{incr.e} and \code{incr.c}
#'   (\code{method.incr = "user"})
#' }
#'
#' For studies with a zero cell count, by default, 0.5 is added to all
#' cell frequencies of these studies (argument \code{incr}). This
#' continuity correction is used both to calculate individual study
#' results with confidence limits and to conduct meta-analysis based
#' on the inverse variance method. For Mantel-Haenszel method, Cochran
#' method, and GLMMs, nothing is added to zero cell counts.
#' Accordingly, estimates for these methods are not defined if the
#' number of events is zero in all studies either in the experimental
#' or control group.
#' }
#' 
#' \subsection{Subgroup analysis}{
#' 
#' Argument \code{subgroup} can be used to conduct subgroup analysis for
#' a categorical covariate. The \code{\link{metareg}} function can be
#' used instead for more than one categorical covariate or continuous
#' covariates.
#' }
#' 
#' \subsection{Exclusion of studies from meta-analysis}{
#'
#' Arguments \code{subset} and \code{exclude} can be used to exclude
#' studies from the meta-analysis. Studies are removed completely from
#' the meta-analysis using argument \code{subset}, while excluded
#' studies are shown in printouts and forest plots using argument
#' \code{exclude} (see Examples in \code{\link{metagen}}).
#' Meta-analysis results are the same for both arguments.
#' }
#' 
#' \subsection{Presentation of meta-analysis results}{
#' 
#' Internally, both common effect and random effects models are
#' calculated regardless of values choosen for arguments
#' \code{common} and \code{random}. Accordingly, the estimate
#' for the random effects model can be extracted from component
#' \code{TE.random} of an object of class \code{"meta"} even if
#' argument \code{random = FALSE}. However, all functions in R
#' package \bold{meta} will adequately consider the values for
#' \code{common} and \code{random}. E.g. function
#' \code{\link{print.meta}} will not print results for the random
#' effects model if \code{random = FALSE}.
#'
#' A prediction interval will only be shown if \code{prediction =
#' TRUE}.
#' }
#' 
#' @return
#' An object of class \code{c("metainc", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{metabin}},
#'   \code{\link{update.meta}}, \code{\link{print.meta}}
#' 
#' @references
#' Bayne-Jones S et al. (1964):
#' Smoking and Health: Report of the Advisory Committee to the Surgeon
#' General of the United States.
#' U-23 Department of Health, Education, and Welfare.
#' Public Health Service Publication No. 1103.
#' 
#' Greenland S & Robins JM (1985):
#' Estimation of a common effect parameter from sparse follow-up data.
#' \emph{Biometrics},
#' \bold{41}, 55--68
#' 
#' Stijnen T, Hamza TH, Ozdemir P (2010):
#' Random effects meta-analysis of event outcome in the framework of
#' the generalized linear mixed model with applications in sparse
#' data.
#' \emph{Statistics in Medicine},
#' \bold{29}, 3046--67
#'
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#' 
#' Viechtbauer W (2010):
#' Conducting Meta-Analyses in R with the Metafor Package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#' 
#' @examples
#' data(smoking)
#' ma1 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
#'   data = smoking, studlab = study)
#' print(ma1, digits = 2)
#' 
#' ma2 <- update(ma1, method = "Cochran")
#' print(ma2, digits = 2)
#' 
#' data(lungcancer)
#' ma3 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
#'   data = lungcancer, studlab = study)
#' print(ma3, digits = 2)
#' 
#' # Redo Cochran meta-analysis with inflated standard errors
#' #
#' # All cause mortality
#' #
#' TEa <- log((smoking$d.smokers/smoking$py.smokers) /
#'   (smoking$d.nonsmokers/smoking$py.nonsmokers))
#' seTEa <- sqrt(1 / smoking$d.smokers + 1 / smoking$d.nonsmokers +
#'   2.5 / smoking$d.nonsmokers)
#' metagen(TEa, seTEa, sm = "IRR", studlab = smoking$study)
#' 
#' # Lung cancer mortality
#' #
#' TEl <- log((lungcancer$d.smokers/lungcancer$py.smokers) /
#'   (lungcancer$d.nonsmokers/lungcancer$py.nonsmokers))
#' seTEl <- sqrt(1 / lungcancer$d.smokers + 1 / lungcancer$d.nonsmokers +
#'   2.25 / lungcancer$d.nonsmokers)
#' metagen(TEl, seTEl, sm = "IRR", studlab = lungcancer$study)
#'
#' \dontrun{
#' # Meta-analysis using generalised linear mixed models
#' # (only if R packages 'metafor' and 'lme4' are available)
#' 
#' # Poisson regression model (fixed study effects)
#' #
#' ma4 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
#'   data = smoking, studlab = study, method = "GLMM")
#' ma4
#' 
#' # Mixed-effects Poisson regression model (random study effects)
#' #
#' update(ma4, model.glmm = "UM.RS", nAGQ = 1)
#' #
#' # Generalised linear mixed model (conditional Poisson-Normal)
#' #
#' update(ma4, model.glmm = "CM.EL")
#' }
#' 
#' @export metainc

metainc <- function(event.e, time.e, event.c, time.c, studlab,
                    #
                    data = NULL, subset = NULL, exclude = NULL,
                    cluster = NULL, rho = 0,
                    #
                    weights = NULL,
                    weights.common = weights, weights.random = weights,
                    #
                    method = if (sm == "IRSD") "Inverse" else "MH",
                    sm = gs("sminc"),
                    incr = gs("incr"), method.incr = gs("method.incr"),
                    incr.e = if (length(incr) > 1) incr else NULL,
                    incr.c = if (length(incr) > 1) incr else NULL,
                    model.glmm = "UM.FS",
                    #
                    level = gs("level"),
                    common = gs("common"),
                    random = gs("random") | !is.null(tau.preset),
                    overall = common | random,
                    overall.hetstat =
                      if (is.null(gs("overall.hetstat")))
                        common | random
                      else
                        gs("overall.hetstat"),   
                    prediction = gs("prediction") | !missing(method.predict),
                    #
                    method.tau =
                      ifelse(!is.na(charmatch(tolower(method), "glmm",
                                              nomatch = NA)),
                             "ML", gs("method.tau")),
                    method.tau.ci = gs("method.tau.ci"),
                    level.hetstat = gs("level.hetstat"),
                    tau.preset = NULL, TE.tau = NULL,
                    tau.common = gs("tau.common"),
                    detail.tau = NULL,
                    #
                    method.I2 = gs("method.I2"),
                    #
                    level.ma = gs("level.ma"),
                    method.common.ci = gs("method.common.ci"),
                    method.random.ci = gs("method.random.ci"),
                    adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                    #
                    level.predict = gs("level.predict"),
                    method.predict = gs("method.predict"),
                    adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                    seed.predict = NULL,
                    #
                    method.bias = gs("method.bias"),
                    #
                    n.e = NULL, n.c = NULL,
                    #
                    backtransf = if (sm == "IRSD") FALSE else gs("backtransf"),
                    irscale = 1, irunit = "person-years",
                    #
                    text.common = gs("text.common"),
                    text.random = gs("text.random"),
                    text.predict = gs("text.predict"),
                    text.w.common = gs("text.w.common"),
                    text.w.random = gs("text.w.random"),
                    #
                    title = gs("title"), complab = gs("complab"),
                    outclab = "",
                    #
                    label.e = gs("label.e"), label.c = gs("label.c"),
                    label.left = gs("label.left"),
                    label.right = gs("label.right"),
                    col.label.left = gs("col.label.left"),
                    col.label.right = gs("col.label.right"),
                    #
                    subgroup, subgroup.name = NULL,
                    print.subgroup.name = gs("print.subgroup.name"),
                    sep.subgroup = gs("sep.subgroup"),
                    test.subgroup = gs("test.subgroup"),
                    prediction.subgroup = gs("prediction.subgroup"),
                    seed.predict.subgroup = NULL,
                    #
                    byvar, hakn, adhoc.hakn,
                    #
                    keepdata = gs("keepdata"),
                    warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
                    #
                    control = NULL,
                    ...
                    ) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  
  args <- list(...)
  nam.args <- names(args)
  #
  missing.sm <- missing(sm)
  missing.subgroup <- missing(subgroup)
  missing.byvar <- missing(byvar)
  missing.overall <- missing(overall)
  missing.overall.hetstat <- missing(overall.hetstat)
  missing.test.subgroup <- missing(test.subgroup)
  #
  missing.event.c <- missing(event.c)
  missing.time.e <- missing(time.e)
  missing.time.c <- missing(time.c)
  missing.n.e <- missing(n.e)
  missing.n.c <- missing(n.c)
  #
  missing.studlab <- missing(studlab)
  #
  missing.incr <- missing(incr)
  avail.incr <- !missing.incr && !is.null(incr)
  #
  missing.method.incr <- missing(method.incr)
  avail.method.incr <- !missing.method.incr && !is.null(method.incr)
  #
  missing.incr.e <- missing(incr.e)
  missing.incr.c <- missing(incr.c)
  #
  missing.method.tau <- missing(method.tau)
  missing.tau.common <- missing(tau.common)
  missing.method.predict <- missing(method.predict)
  missing.method <- missing(method)
  missing.level.ma <- missing(level.ma)
  missing.common <- missing(common)
  missing.random <- missing(random)
  missing.method.common.ci <- missing(method.common.ci)
  missing.method.random.ci <- missing(method.random.ci)
  #
  missing.hakn <- missing(hakn)
  missing.adhoc.hakn.ci <- missing(adhoc.hakn.ci)
  missing.adhoc.hakn <- missing(adhoc.hakn)
  #
  missing.subgroup.name <- missing(subgroup.name)
  missing.print.subgroup.name <- missing(print.subgroup.name)
  missing.sep.subgroup <- missing(sep.subgroup)
  #
  missing.label.e <- missing(label.e)
  missing.label.c <- missing(label.c)
  missing.complab <- missing(complab)
  #
  missing.cluster <- missing(cluster)
  #
  chknumeric(rho, min = -1, max = 1)
  #
  chknull(sm)
  sm <- setchar(sm, gs("sm4inc"))
  #
  chklevel(level)
  #
  method.tau <- setchar(method.tau, gs("meth4tau"))
  #
  if (is.null(method.tau.ci))
    method.tau.ci <- if (method.tau == "DL") "J" else "QP"
  method.tau.ci <- setchar(method.tau.ci, gs("meth4tau.ci"))
  #
  tau.common <- replaceNULL(tau.common, FALSE)
  chklogical(tau.common)
  #
  chklogical(prediction)
  chklevel(level.predict)
  #
  method.predict <- setchar(method.predict, gs("meth4pi"))
  #
  method.tau <-
    set_method_tau(method.tau, missing.method.tau,
                 method.predict, missing.method.predict)
  method.predict <-
    set_method_predict(method.predict, missing.method.predict,
                     method.tau, missing.method.tau)
  #
  if (any(method.predict == "NNF"))
    is_installed_package("pimeta", argument = "method.predict", value = "NNF")
  #
  adhoc.hakn.pi <- setchar(replaceNA(adhoc.hakn.pi, ""), gs("adhoc4hakn.pi"))
  #
  chklogical(backtransf)
  #
  chknumeric(irscale, length = 1)
  chkchar(irunit)
  #
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  if (!is.null(text.random))
    chkchar(text.random)
  if (!is.null(text.predict))
    chkchar(text.predict)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chklogical(keepdata)
  #
  # Additional arguments / checks for metainc objects
  #
  fun <- "metainc"
  #
  if (sm != "IRD" & irscale != 1) {
    warning("Argument 'irscale' only considered for ",
            "incidence rate differences.",
            call. = FALSE)
    irscale <- 1
  }
  #
  method <- setchar(method, gs("meth4inc"))
  #
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  if (method != "Inverse" & method.common.ci == "IVhet") {
    if (!missing.method.common.ci)
      warning("Argument 'method.common.ci = \"IVhet\"' only available ",
              "if 'method = \"Inverse\".",
              call. = FALSE)
    method.common.ci <- "classic"
  }
  #
  method.incr <- setchar(method.incr, gs("meth4incr"))
  #
  is.glmm <- method == "GLMM"
  #
  model.glmm <- setchar(model.glmm, c("UM.FS", "UM.RS", "CM.EL"))
  chklogical(warn)
  #
  # Check for deprecated arguments in '...'
  #
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing.level.ma, args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing.random, args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.random.ci <-
    deprecated2(method.random.ci, missing.method.random.ci,
                hakn, missing.hakn,
                warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  adhoc.hakn.ci <-
    deprecated2(adhoc.hakn.ci, missing.adhoc.hakn.ci,
                adhoc.hakn, missing.adhoc.hakn, warn.deprecated)
  adhoc.hakn.ci <- setchar(replaceNA(adhoc.hakn.ci, ""), gs("adhoc4hakn.ci"))
  #
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing.print.subgroup.name,
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing.sep.subgroup, args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  # Some more checks
  #
  chklogical(overall)
  chklogical(overall.hetstat)
  #
  # Ignore deprecated arguments 'addincr' and 'allincr'
  #
  txt.ignore <- "(deprecated); use argument 'method.incr'"
  #
  if (!is.na(charmatch("addincr", nam.args)))
    warn_ignore_input(addincr, TRUE, txt.ignore)
  if (!is.na(charmatch("allincr", nam.args)))
    warn_ignore_input(allincr, TRUE, txt.ignore)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata) {
    data <- sfsp
    data.pairwise <- FALSE
  }
  else
    data.pairwise <- inherits(data, "pairwise")
  #
  # Catch 'event.e', 'time.e', 'event.c', 'time.c', 'n.e', 'n.c',
  # 'incr.e', 'incr.c', studlab', and 'subgroup' from data:
  #
  event.e <- catch("event.e", mc, data, sfsp)
  chknull(event.e)
  #
  if (inherits(event.e, "pairwise")) {
    is.pairwise <- TRUE
    #
    type <- attr(event.e, "type")
    if (type != "count")
      stop("Wrong type for pairwise() object: '", type, "'.", call. = FALSE)
    #
    txt.ignore <- "as first argument is a pairwise object"
    #
    warn_ignore_input(event.c, !missing.event.c, txt.ignore)
    warn_ignore_input(time.e, !missing.time.e, txt.ignore)
    warn_ignore_input(time.c, !missing.time.c, txt.ignore)
    warn_ignore_input(n.e, !missing.n.e, txt.ignore)
    warn_ignore_input(n.c, !missing.n.c, txt.ignore)
    warn_ignore_input(incr.e, !missing.incr.e, txt.ignore)
    warn_ignore_input(incr.c, !missing.incr.c, txt.ignore)
    #
    warn_ignore_input(studlab, !missing.studlab, txt.ignore)
    #
    if (missing.sm)
      sm <- attr(event.e, "sm")
    #
    if (missing.method)
      method <- attr(event.e, "method")
    #
    if (!avail.method.incr & !avail.incr)
      method.incr <- "user"
    #
    missing.sm <- FALSE
    missing.method <- FALSE
    #
    avail.method.incr <- TRUE
    missing.incr <- FALSE
    missing.method.incr <- FALSE
    #
    reference.group <- attr(event.e, "reference.group")
    #
    studlab <- event.e$studlab
    #
    treat1 <- event.e$treat1
    treat2 <- event.e$treat2
    #
    event.c <- event.e$event2
    #
    time.e <- event.e$time1
    time.c <- event.e$time2
    #
    n.e <- event.e$n1
    n.c <- event.e$n2
    #
    avail.n.e <- !is.null(n.e)
    avail.n.c <- !is.null(n.c)
    #
    incr.e <- event.e$incr1
    incr.c <- event.e$incr2
    #
    if (avail.incr | method.incr != "user") {
      incr.e <- NULL
      incr.c <- NULL
    }
    #
    if (!avail.incr)
      incr <- attr(event.e, "incr")
    #
    pairdata <- event.e
    data <- event.e
    nulldata <- FALSE
    #
    event.e <- event.e$event1
    #
    wo <- treat1 == reference.group
    #
    if (any(wo)) {
      ttreat1 <- treat1
      treat1[wo] <- treat2[wo]
      treat2[wo] <- ttreat1[wo]
      #
      tevent.e <- event.e
      event.e[wo] <- event.c[wo]
      event.c[wo] <- tevent.e[wo]
      #
      ttime.e <- time.e
      time.e[wo] <- time.c[wo]
      time.c[wo] <- ttime.e[wo]
      #
      if (avail.n.e & avail.n.c) {
        tn.e <- n.e
        n.e[wo] <- n.c[wo]
        n.c[wo] <- tn.e[wo]
      }
      #
      if (!(is.null(incr.e) | is.null(incr.c))) {
        tincr.e <- incr.e
        incr.e[wo] <- incr.c[wo]
        incr.c[wo] <- tincr.e[wo]
      }
    }
    #
    if (missing.subgroup & missing.byvar) {
      subgroup <- paste(treat1, treat2, sep = " vs ")
      #
      if (length(unique(subgroup)) == 1) {
        if (missing.complab)
          complab <- unique(subgroup)
        #
        if (missing.label.e)
          label.e <- unique(treat1)
        if (missing.label.c)
          label.c <- unique(treat2)
        #
        subgroup <- NULL
      }
      else {
        if (missing.overall)
          overall <- FALSE
        if (missing.overall.hetstat)
          overall.hetstat <- FALSE
        if (missing.test.subgroup)
          test.subgroup <- FALSE
      }
    }
    else {
      subgroup <- catch("subgroup", mc, data, sfsp)
      byvar <- catch("byvar", mc, data, sfsp)
      #
      subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                              warn.deprecated)
    }
  }
  else {
    is.pairwise <- FALSE
    #
    if (missing.sm && !is.null(data) && !is.null(attr(data, "sm")))
      sm <- attr(data, "sm")
    #
    time.e <- catch("time.e", mc, data, sfsp)
    n.e <- catch("n.e", mc, data, sfsp)
    #
    event.c <- catch("event.c", mc, data, sfsp)
    time.c <- catch("time.c", mc, data, sfsp)
    n.c <- catch("n.c", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    subgroup <- catch("subgroup", mc, data, sfsp)
    byvar <- catch("byvar", mc, data, sfsp)
    #
    subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                            warn.deprecated)
    #
    if (!missing.incr)
      incr <- catch("incr", mc, data, sfsp)
    #
    if (!missing.incr.e)
      incr.e <- catch("incr.e", mc, data, sfsp)
    if (!missing.incr.c)
      incr.c <- catch("incr.c", mc, data, sfsp)
  }
  #
  method.bias <- setmethodbias(method.bias)
  #
  chknumeric(incr, min = 0)
  #
  avail.n.e <- !is.null(n.e)
  avail.n.c <- !is.null(n.c)
  #
  avail.incr.e <- !is.null(incr.e)
  avail.incr.c <- !is.null(incr.c)
  avail.incr.both <- avail.incr.e & avail.incr.c
  #
  if (avail.incr.e + avail.incr.c == 1)
    stop("Arguments 'incr.e' and 'incr.c' are required together.",
         call. = FALSE)
  #
  if (avail.incr.e)
    chknumeric(incr.e, min = 0, NA.ok = FALSE)
  #
  if (avail.incr.c)
    chknumeric(incr.c, min = 0, NA.ok = FALSE)
  #
  if (avail.incr.both) {
    if (!avail.method.incr)
      method.incr <- "user"
    #
    txt.ignore <- "as arguments 'incr.e' and 'incr.c' are provided"
    #
    warn_set_input(method.incr, method.incr != "user", '"user"', txt.ignore)
    #
    if (length(incr) != length(incr.e) | any(incr != incr.e))
      warn_ignore_input(incr, avail.incr, txt.ignore)
  }
  else {
    addincr <- allincr <- FALSE
    #
    if (method.incr == "all")
      addincr <- TRUE
    else if (method.incr == "if0all")
      allincr <- TRUE
  }
  #
  k.All <- length(event.e)
  #
  chknull(time.e)
  chknull(event.c)
  chknull(time.c)
  #
  studlab <- setstudlab(studlab, k.All)
  #
  by <- !is.null(subgroup)
  #
  # Catch 'subset', 'exclude' and 'cluster' from data:
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  exclude <- catch("exclude", mc, data, sfsp)
  missing.exclude <- is.null(exclude)
  #
  cluster <- catch("cluster", mc, data, sfsp)
  with.cluster <- !is.null(cluster)
  #
  # Catch 'weights', 'weights.common', and 'weights.random' from data:
  #
  if (!missing(weights))
    weights <- catch("weights", mc, data, sfsp)
  if (!missing(weights.common))
    weights.common <- catch("weights.common", mc, data, sfsp)
  if (!missing(weights.random))
    weights.random <- catch("weights.random", mc, data, sfsp)
  #
  if (!is.null(weights) & is.null(weights.common))
    weights.common <- weights
  #
  if (!is.null(weights) & is.null(weights.random))
    weights.random <- weights
  #
  usw.common <- !is.null(weights.common)
  usw.random <- !is.null(weights.random)
  #
  if (usw.common)
    chknumeric(weights.common, min = 0)
  #
  if (usw.random)
    chknumeric(weights.random, min = 0)
  #
  if (usw.common & method != "Inverse")
    stop("User-specified weights for the common effect model only implemented ",
         "for the inverse variance method (method = \"Inverse\").",
         call. = FALSE)
  #
  if (usw.random & method == "GLMM")
    stop("User-specified weights for the random effects model not implemented ",
         "for generalized linear mixed models (method = \"GLMM\").",
         call. = FALSE)
  #
  # Check variable values
  #
  chknumeric(event.e, 0)
  chknumeric(time.e, 0, zero = TRUE)
  chknumeric(event.c, 0)
  chknumeric(time.c, zero = TRUE)
  #
  # Recode integer as numeric:
  #
  event.e <- int2num(event.e)
  time.e  <- int2num(time.e)
  event.c <- int2num(event.c)
  time.c  <- int2num(time.c)
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  
  chklength(time.e, k.All, fun)
  chklength(event.c, k.All, fun)
  chklength(time.c, k.All, fun)
  chklength(studlab, k.All, fun)
  #
  if (with.cluster)
    chklength(cluster, k.All, fun)
  #
  if (usw.common) {
    if (length(weights.common) == 1)
      weights.common <- rep(weights.common, k.All)
    else
      chklength(weights.common, k.All, fun)
  }
  #
  if (usw.random) {
    if (length(weights.random) == 1)
      weights.random <- rep(weights.random, k.All)
    else
      chklength(weights.random, k.All, fun)
  }
  #
  if (length(incr) > 1)
    chklength(incr, k.All, fun)
  #
  if (avail.incr.e) {
    if (length(incr.e) == 1)  
      incr.e <- rep_len(incr.e, k.All)
    else
      chklength(incr.e, k.All, fun)
  }
  #
  if (avail.incr.c) {
    if (length(incr.c) == 1)  
      incr.c <- rep_len(incr.c, k.All)
    else
      chklength(incr.c, k.All, fun)
  }
  #
  if (avail.n.e)
    chklength(n.e, k.All, fun)
  #
  if (avail.n.c)
    chklength(n.c, k.All, fun)
  #
  if (by) {
    chklength(subgroup, k.All, fun)
    chklogical(test.subgroup)
    chklogical(prediction.subgroup)
  }
  #
  # Additional checks
  #
  if (!by & tau.common) {
    warning("Value for argument 'tau.common' set to FALSE as ",
            "argument 'subgroup' is missing.",
            call. = FALSE)
    tau.common <- FALSE
  }
  #
  if (by & !tau.common & !is.null(tau.preset)) {
    warning("Argument 'tau.common' set to TRUE as ",
            "argument tau.preset is not NULL.",
            call. = FALSE)
    tau.common <- TRUE
  }
  #
  if (method.incr == "user" & !avail.incr.both)
    stop("Arguments 'incr.e' and 'incr.c' must be provided if ",
         "'method.incr = \"user\".",
         call. = FALSE)
  
  
  #
  #
  # (4) Subset, exclude studies, and subgroups
  #
  #
  
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of subset is larger than number of studies.")
  #
  if (!missing.exclude) {
    if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
        (length(exclude) > k.All))
      stop("Length of argument 'exclude' is larger than number of studies.")
    #
    exclude2 <- rep(FALSE, k.All)
    exclude2[exclude] <- TRUE
    exclude <- exclude2
  }
  else
    exclude <- rep(FALSE, k.All)
  
  
  #
  #
  # (5) Store complete dataset in list object data
  #     (if argument keepdata is TRUE)
  #
  #
  
  if (keepdata) {
    if (nulldata)
      data <- data.frame(.studlab = studlab)
    else
      data$.studlab <- studlab
    #
    data$.event.e <- event.e
    data$.time.e <- time.e
    data$.event.c <- event.c
    data$.time.c <- time.c
    #
    data$.incr.e <- NA
    data$.incr.c <- NA
    #
    if (by)
      data$.subgroup <- subgroup
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
    #
    if (!missing.exclude)
      data$.exclude <- exclude
    #
    if (with.cluster)
      data$.id <- data$.cluster <- cluster
    #
    if (usw.common)
      data$.weights.common <- weights.common
    #
    if (usw.random)
      data$.weights.random <- weights.random
    #
    if (avail.n.e)
      data$.n.e <- n.e
    if (avail.n.e)
      data$.n.c <- n.c
  }  
  
  
  #
  #
  # (6) Use subset for analysis
  #
  #
  
  if (!missing.subset) {
    event.e <- event.e[subset]
    time.e <- time.e[subset]
    event.c <- event.c[subset]
    time.c <- time.c[subset]
    studlab <- studlab[subset]
    #
    cluster <- cluster[subset]
    exclude <- exclude[subset]
    #
    weights.common <- weights.common[subset]
    weights.random <- weights.random[subset]
    #
    if (avail.n.e)
      n.e <- n.e[subset]
    if (avail.n.c)
      n.c <- n.c[subset]
    #
    incr.e <- incr.e[subset]
    incr.c <- incr.c[subset]
    #
    if (length(incr) > 1)
      incr <- incr[subset]
    #
    if (by)
      subgroup <- subgroup[subset]
  }
  #
  if (missing.subgroup & is.pairwise & by) {
    if (length(unique(subgroup)) == 1) {
      by <- FALSE
      #
      if (missing.complab)
        complab <- unique(subgroup)
      #
      subgroup <- NULL
      #
      if (keepdata)
        data$.subgroup <- NULL
      #
      if (missing.overall)
        overall <- TRUE
      if (missing.overall.hetstat)
        overall.hetstat <- TRUE
    }
  }
  #
  # Determine total number of studies
  #
  k.all <- length(event.e)
  #
  if (k.all == 0)
    stop("No studies to combine in meta-analysis.")
  #
  # No meta-analysis for a single study
  #
  if (k.all == 1) {
    common <- FALSE
    random <- FALSE
    prediction <- FALSE
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  #
  if (by) {
    chkmiss(subgroup)
    #
    if (missing.subgroup.name & is.null(subgroup.name)) {
      if (!missing.subgroup)
        subgroup.name <- byvarname("subgroup", mc)
      else if (!missing.byvar)
        subgroup.name <- byvarname("byvar", mc)
    }
  }
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  
  
  #
  #
  # (7) Continuity correction
  #
  #
  
  sel <- switch(sm,
                IRD = event.e == 0 | event.c == 0,
                IRR = event.e == 0 | event.c == 0,
                VE = event.e == 0 | event.c == 0,
                IRSD = event.e == 0 | event.c == 0)
  #
  # Sparse computation
  #
  sparse <- any(sel, na.rm = TRUE)
  #
  if (avail.incr.both) {
    chknumeric(incr.e, min = 0, NA.ok = FALSE)
    chknumeric(incr.c, min = 0, NA.ok = FALSE)
  }
  else {
    if (addincr)
      incr.event <- if (length(incr) == 1) rep(incr, k.all) else incr
    else {
      if (sparse) {
        if (allincr)
          incr.event <- if (length(incr) == 1) rep(incr, k.all) else incr
        else
          incr.event <- incr * sel
      }
      else
        incr.event <- rep(0, k.all)
    }
    #
    incr.e <- incr.c <- incr.event
  }
  #
  if (keepdata) {
    if (missing.subset) {
      data$.incr.e <- incr.e
      data$.incr.c <- incr.c
    }
    else {
      data$.incr.e <- NA
      data$.incr.c <- NA
      #
      data$.incr.e[subset] <- incr.e
      data$.incr.c[subset] <- incr.c
    }
  }
  
  
  #
  #
  # (8) Calculate results for individual studies
  #
  #
  
  if (sm %in% c("IRR", "VE")) {
    TE <- log(((event.e + incr.e) / time.e) / ((event.c + incr.c) / time.c))
    seTE <- sqrt(1 / (event.e + incr.e) + 1 / (event.c + incr.c))
  }
  else if (sm == "IRD") {
    TE <- event.e / time.e - event.c / time.c
    seTE <- sqrt((event.e + incr.e) / time.e^2 + (event.c + incr.c) / time.c^2)
  }
  else if (sm == "IRSD") {
    TE <- sqrt(event.e / time.e) - sqrt(event.c / time.c)
    seTE <- sqrt(0.25 / time.e + 0.25 / time.c)
  }
  #
  # Set NaN to NA
  #
  TE[is.nan(TE)] <- NA
  
  
  #
  #
  # (8) Additional checks for three-level model
  #
  #
  
  three.level <- FALSE
  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (with.cluster &&
      length(unique(cluster[sel.ni])) != length(cluster[sel.ni]))
    three.level <- TRUE
  #
  if (three.level) {
    chkmlm(method.tau, missing.method.tau, method.predict,
           method, missing.method)
    #
    common <- FALSE
    method <- "Inverse"
    is.glmm <- FALSE
    #
    if (!(method.tau %in% c("REML", "ML")))
      method.tau <- "REML"
  }
  
  
  #
  #
  # (9) Additional checks for GLMMs
  #
  #
  
  if (is.glmm) {
    chkglmm(sm, method.tau, method.random.ci, method.predict,
            adhoc.hakn.ci, adhoc.hakn.pi,
            c("IRR", "VE"))
    #
    if (!is.null(TE.tau)) {
      if (warn)
        warning("Argument 'TE.tau' not considered for GLMM.",
                call. = FALSE)
      TE.tau <- NULL
    }
    #
    if (!is.null(tau.preset)) {
      if (warn)
        warning("Argument 'tau.preset' not considered for GLMM.",
                call. = FALSE)
      tau.preset <- NULL
    }
  }
  
  
  #
  #
  # (10) Do meta-analysis
  #
  #
  
  k <- sum(!is.na(event.e[!exclude]) & !is.na(event.c[!exclude]) &
           !is.na(time.e[!exclude]) & !is.na(time.c[!exclude]))
  #
  for (i in seq_along(method.random.ci))
    if (k == 1 & method.random.ci[i] == "HK")
      method.random.ci[i] <- "classic"
  #
  if (method == "MH") {
    #
    # Greenland, Robins (1985)
    #
    x.k <- event.e
    y.k <- event.c
    n.k <- time.e
    m.k <- time.c
    #
    N.k <- n.k + m.k
    t.k <- x.k + y.k
    #
    if (sm %in% c("IRR", "VE")) {
      D <- n.k * m.k * t.k / N.k^2
      R <- x.k * m.k / N.k
      S <- y.k * n.k / N.k
      #
      D[exclude] <- R[exclude] <- S[exclude] <- 0
      #
      w.common <- S
      TE.common <- log(sum(R, na.rm = TRUE) / sum(S, na.rm = TRUE))
      seTE.common <- sqrt(sum(D, na.rm = TRUE) / (sum(R, na.rm = TRUE) *
                                                  sum(S, na.rm = TRUE)))
    }
    else if (sm == "IRD") {
      L <- (x.k * m.k^2 + y.k * n.k^2) / N.k^2
      S <- n.k * m.k / N.k
      #
      L[exclude] <- S[exclude] <- 0
      #
      w.common <- S
      TE.common <- weighted.mean(TE, w.common, na.rm = TRUE)
      seTE.common <- sqrt(sum(L, na.rm = TRUE) / sum(S, na.rm = TRUE)^2)
    }
  }
  #
  else if (method == "Cochran") {
    #
    # Smoking and Health - Report of the Advisory Committee to the
    # Surgeon General of the Public Health Service,
    # Chapter 8
    #
    if (sm %in% c("IRR", "VE")) {
      w.common <- event.c * time.e / time.c
      w.common[exclude] <- 0
      TE.common <- weighted.mean(TE, w.common)
      seTE.common <- sqrt(1 / sum(event.e) + 1 / sum(event.c))
    }
    else if (sm == "IRD") {
      warning("Cochran method only available for ",
              "Incidence Rate Ratio (sm = \"IRR\") ",
              "and Vaccine Efficacy / Effectiveness (sm = \"VE\")",
              call. = FALSE)
      return(NULL)
    }
  }
  else if (is.glmm) {
    list.inc <- list(x1i = event.e[!exclude], t1i = time.e[!exclude],
                     x2i = event.c[!exclude], t2i = time.c[!exclude],
                     measure = "IRR", model = model.glmm)
    #
    use.random <-
      sum(!exclude) > 1 &
      !((sum(event.e[!exclude], na.rm = TRUE) == 0 &
         sum(event.c[!exclude], na.rm = TRUE) == 0) |
        (!any(event.e[!exclude] != time.e[!exclude]) |
         !any(event.c[!exclude] != time.c[!exclude])))
    #
    res.glmm <-
      runGLMM(list.inc,
              method.tau = method.tau,
              method.random.ci = method.random.ci,
              level = level.ma,
              control = control, use.random = use.random,
              warn = warn)
    #
    TE.common   <- as.numeric(res.glmm$glmm.common$b)
    seTE.common <- as.numeric(res.glmm$glmm.common$se)
    #
    w.common <- rep(NA, length(event.e))
  }
  #
  m <- metagen(TE, seTE, studlab,
               exclude = if (missing.exclude) NULL else exclude,
               cluster = cluster, rho = rho,
               #
               weights.common = weights.common,
               weights.random = weights.random,
               #
               sm = sm,
               level = level,
               #
               common = common,
               random = random,
               overall = overall,
               overall.hetstat = overall.hetstat,
               prediction = prediction,
               #
               method.tau = if (is.glmm) "DL" else method.tau,
               method.tau.ci = if (is.glmm) "" else method.tau.ci,
               level.hetstat = level.hetstat,
               tau.preset = tau.preset,
               TE.tau = TE.tau,
               tau.common = FALSE,
               detail.tau = detail.tau,
               #
               method.I2 = method.I2,
               #
               level.ma = level.ma,
               method.common.ci = method.common.ci,
               method.random.ci = method.random.ci,
               adhoc.hakn.ci = adhoc.hakn.ci,
               #
               level.predict = level.predict,
               method.predict = method.predict,
               adhoc.hakn.pi = adhoc.hakn.pi,
               seed.predict = seed.predict,
               #
               method.bias = method.bias,
               #
               backtransf = backtransf,
               #
               text.common = text.common, text.random = text.random,
               text.predict = text.predict,
               text.w.common = text.w.common, text.w.random = text.w.random,
               #
               title = title, complab = complab, outclab = outclab,
               #
               label.e = label.e, label.c = label.c,
               label.left = label.left, label.right = label.right,
               col.label.left = col.label.left,
               col.label.right = col.label.right,
               #
               keepdata = FALSE,
               warn = warn,
               #
               control = control)
  #
  # Estimate common tau-squared across subgroups
  #
  if (by & tau.common & !is.glmm)
    hcc <- hetcalc(TE, seTE, method.tau, "",
                   if (method == "Inverse") TE.tau else TE.common,
                   method.I2, level.hetstat, subgroup, control)
  
  
  #
  #
  # (11) Generate R object
  #
  #
  
  res <- list(event.e = event.e, time.e = time.e,
              event.c = event.c, time.c = time.c,
              method = method, method.random = method,
              incr = if (length(unique(incr)) == 1) unique(incr) else incr,
              method.incr = method.incr,
              sparse = sparse,
              incr.e = incr.e,
              incr.c = incr.c,
              k.MH = if (method == "MH") sum(w.common > 0) else NA)
  #
  # Add meta-analysis results
  # (after removing unneeded list elements)
  #
  m$method <- NULL
  m$method.random <- NULL
  m$n.e <- NULL
  m$n.c <- NULL
  m$pscale <- NULL
  m$irscale <- NULL
  m$irunit <- NULL
  m$method.ci <- NULL
  m$method.mean <- NULL
  m$approx.TE <- NULL
  m$approx.seTE <- NULL
  #
  res <- c(res, m)
  #
  # Add data
  #
  res$n.e <- n.e
  res$n.c <- n.c
  res$TE.tau <- TE.tau
  #
  res$irscale <- irscale
  res$irunit  <- irunit
  #
  if (is.pairwise | data.pairwise) {
    res$pairwise <- TRUE
    res$k.study <- length(unique(res$studlab[!is.na(res$TE)]))
  }
  #
  res$call <- match.call()
  #
  if (method %in% c("MH", "Cochran", "GLMM")) {
    res <- ci2meta(res, ci.c = ci(TE.common, seTE.common, level = level.ma))
    res$w.common <- w.common
  }
  #
  if (is.glmm) {
    res$method.tau <- method.tau
    #
    res <- addGLMM(res, res.glmm, method.I2)
    res$model.glmm <- model.glmm
    #
    if (by) {
      n.subgroups <- length(unique(subgroup[!exclude]))
      if (n.subgroups > 1)
        subgroup.glmm <-
          factor(subgroup[!exclude], bylevs(subgroup[!exclude]))
      #
      hcc <-
        hccGLMM(
          res,
          runGLMM(list.inc,
                  method.tau = method.tau,
                  method.random.ci = method.random.ci,
                  level = level.ma,
                  data =
                    if (n.subgroups > 1)
                      list(data = data.frame(subgroup.glmm))
                    else
                      NULL,
                  mods =
                    if (n.subgroups > 1)
                      as.call(~ subgroup.glmm)
                    else
                      NULL,
                  control = control, use.random = use.random,
                  warn = warn)$glmm.random[[1]],
          method.I2
        )
    }
  }
  #
  if (keepdata) {
    res$data <- data
    if (!missing.subset)
      res$subset <- subset
  }
  #
  class(res) <- c(fun, "meta")
  #
  # Add results from subgroup analysis
  #
  if (by) {
    res$subgroup <- subgroup
    res$subgroup.name <- subgroup.name
    res$print.subgroup.name <- print.subgroup.name
    res$sep.subgroup <- sep.subgroup
    res$test.subgroup <- test.subgroup
    res$prediction.subgroup <- prediction.subgroup
    res$tau.common <- tau.common
    #
    if (!tau.common) {
      res <- c(res, subgroup(res, seed = seed.predict.subgroup))
      if (res$three.level)
        res <- setNA3(res)
    }
    else if (!is.null(tau.preset))
      res <-
        c(res, subgroup(res, tau.preset, seed = seed.predict.subgroup))
    else {
      if (is.glmm)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup)), ...))
      else if (res$three.level)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup))))
      else
        res <-
          c(res, subgroup(res, hcc$tau.resid, seed = seed.predict.subgroup))
    }
    #
    if (tau.common && is.null(tau.preset))
      res <- addHet(res, hcc, !is.glmm)
    #
    res$n.w <- NULL
    res$event.w <- NULL
    #
    if (!avail.n.e)
      res$n.e.w <- NULL
    if (!avail.n.c)
      res$n.c.w <- NULL
    #
    res$n.harmonic.mean.w <- NULL
    res$t.harmonic.mean.w <- NULL
    #
    res <- setNAwithin(res, res$three.level | is.glmm)
  }
  #
  # Mantel-Haenszel and Cochran method are common effect methods
  #
  if (res$method.random %in% c("MH", "Cochran"))
    res$method.random <- "Inverse"
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  class(res) <- c(fun, "meta")
  
  
  res
}

#' Thrombolytic Therapy after Acute Myocardial Infarction
#' 
#' @description
#' Meta-analysis on Thrombolytic Therapy after Acute Myocardial Infarction
#' 
#' @name Olkin1995
#' @aliases Olkin95
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{author}}\tab first author \cr
#' \bold{\emph{year}}\tab year of publication \cr
#' \bold{\emph{ev.exp}}\tab number of events in experimental group
#'   \cr
#' \bold{\emph{n.exp}}\tab number of observations in experimental group
#'   \cr
#' \bold{\emph{ev.cont}}\tab number of events in control group \cr
#' \bold{\emph{n.cont}}\tab number of observations in control group
#' }
#' 
#' @source
#' Olkin I (1995):
#' Statistical and theoretical considerations in meta-analysis.
#' \emph{Journal of Clinical Epidemiology},
#' \bold{48}, 133--46
#' 
#' @keywords datasets
#' 
#' @examples
#' data(Olkin1995)
#' metabin(ev.exp, n.exp, ev.cont, n.cont, data = Olkin1995)


NULL

#' Bubble plot to display the result of a meta-regression
#' 
#' @description
#' Draw a bubble plot to display the result of a meta-regression.
#' 
#' @aliases bubble bubble.metareg
#' 
#' @param x An object of class \code{metareg}.
#' @param xlim The x limits (min,max) of the plot.
#' @param ylim The y limits (min,max) of the plot.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param cex The magnification to be used for plotting symbols.
#' @param min.cex Minimal magnification for plotting symbols.
#' @param max.cex Maximal magnification for plotting symbols.
#' @param pch The plotting symbol(s) used for individual studies.
#' @param col A vector with colour of plotting symbols.
#' @param bg A vector with background colour of plotting symbols (only
#'   used if \code{pch} in \code{21:25}).
#' @param lty The line type for the meta-regression line.
#' @param lwd The line width for the meta-regression line.
#' @param col.line Colour for the meta-regression line.
#' @param studlab A logical indicating whether study labels should be
#'   printed in the graph. A vector with study labels can also be
#'   provided (must be of same length as the numer of studies in the
#'   meta-analysis then).
#' @param cex.studlab The magnification for study labels.
#' @param pos.studlab Position of study labels, see argument
#'   \code{pos} in \code{\link{text}}.
#' @param offset Offset for study labels (see \code{\link{text}}).
#' @param regline A logical indicating whether a regression line
#'   should be added to the bubble plot.
#' @param col.line Colour for the meta-regression line.
#' @param backtransf A logical indicating whether results for relative
#'   summary measures (argument \code{sm} equal to \code{"OR"},
#'   \code{"RR"}, \code{"HR"}, or \code{"IRR"}) should be back
#'   transformed. If \code{backtransf=TRUE}, results for
#'   \code{sm="OR"} are printed as odds ratios rather than log odds
#'   ratios, for example.
#' @param ref A numerical giving the reference value to be plotted as
#'   a line in the bubble plot. No reference line is plotted if
#'   argument \code{ref} is equal to \code{NA}.
#' @param col.ref Colour of the reference line.
#' @param lty.ref The line type for the reference line.
#' @param lwd.ref The line width for the reference line.
#' @param pscale A numeric giving scaling factor for printing of
#'   probabilities.
#' @param irscale A numeric defining a scaling factor for printing of
#'   incidence rates.
#' @param axes Either a logical or a character string equal to \code{"x"},
#'   \code{"y"} or \code{"xy"} indicating whether x- and y-axis should be
#'   printed.
#' @param box A logical indicating whether a box should be printed.
#' @param \dots Graphical arguments as in \code{par} may also be
#'   passed as arguments.
#' 
#' @details
#' A bubble plot can be used to display the result of a
#' meta-regression. It is a scatter plot with the treatment effect for
#' each study on the y-axis and the covariate used in the
#' meta-regression on the x-axis. Typically, the size of the plotting
#' symbol is inversely proportional to the variance of the estimated
#' treatment effect (Thompson & Higgins, 2002).
#' 
#' Argument \code{cex} specifies the plotting size for each individual
#' study. If this argument is missing the weights from the
#' meta-regression model will be used (which typically is a random
#' effects model). Use \code{cex="common"} in order to utilise weights
#' from a common effect model to define the size of the plotted
#' symbols (even for a random effects meta-regression). If a vector
#' with individual study weights is provided, the length of this
#' vector must be of the same length as the number of studies.
#' 
#' Arguments \code{min.cex} and \code{max.cex} can be used to define
#' the size of the smallest and largest plotting symbol. The plotting
#' size of the most precise study is set to \code{max.cex} whereas the
#' plotting size of all studies with a plotting size smaller than
#' \code{min.cex} will be set to \code{min.cex}.
#' 
#' For a meta-regression with more than one covariate. Only a scatter plot of
#' the first covariate in the regression model is shown. In this case the
#' effect of the first covariate adjusted for other covariates in the
#' meta-regression model is shown.
#' 
#' For a factor or categorial covariate separate bubble plots for each
#' group compared to the baseline group are plotted.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metagen}}, \code{\link{metainf}}
#' 
#' @references
#' Thompson SG, Higgins JP (2002):
#' How should meta-regression analyses be undertaken and interpreted?
#' \emph{Statistics in Medicine},
#' \bold{21}, 1559--73
#'
#' @keywords hplot
#' 
#' @examples
#' data(Fleiss1993cont)
#' 
#' # Add some (fictitious) grouping variables:
#' Fleiss1993cont$age <- c(55, 65, 52, 65, 58)
#' Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")
#' 
#' ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD")
#' 
#' mr1 <- metareg(ma, region)
#' mr1
#' 
#' bubble(mr1)
#' bubble(mr1, lwd = 2, col.line = "blue")
#' 
#' mr2 <- metareg(ma, age)
#' mr2
#' 
#' bubble(mr2, lwd = 2, col.line = "blue", xlim = c(50, 70))
#' bubble(mr2, lwd = 2, col.line = "blue", xlim = c(50, 70), cex = "common")
#' 
#' # Do not print regression line
#' #
#' bubble(mr2, lwd = 2, col.line = "blue", xlim = c(50, 70), regline = FALSE)
#' 
#' @rdname bubble.metareg
#' @method bubble metareg
#' @export

bubble.metareg <- function(x,
                           xlim, ylim,
                           xlab, ylab,
                           cex, min.cex = 0.5, max.cex = 5,
                           pch = 21, col = "black", bg = "darkgray",
                           lty = 1, lwd = 1, col.line = "black",
                           studlab = FALSE, cex.studlab = 0.8,
                           pos.studlab = 2, offset = 0.5,
                           regline = TRUE,
                           backtransf = x$.meta$x$backtransf,
                           ref,
                           col.ref = "lightgray", lty.ref = 1, lwd.ref = 1,
                           pscale = x$.meta$x$pscale,
                           irscale = x$.meta$x$irscale,
                           axes = TRUE, box = TRUE, ...) {
  
  
  #
  #
  # (1) Check for meta object
  #
  #
  chkclass(x, "metareg")
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  chknumeric(min.cex)
  chknumeric(max.cex)
  chknumeric(lty)
  chknumeric(lwd)
  chknumeric(cex.studlab)
  pos.studlab <- as.numeric(setchar(pos.studlab, as.character(1:4)))
  chknumeric(offset)
  chklogical(regline)
  chklogical(backtransf)
  chknumeric(lty.ref)
  chknumeric(lwd.ref)
  #
  if (!is.null(pscale))
    chknumeric(pscale, length = 1)
  else
    pscale <- 1
  #
  if (!is.null(irscale))
    chknumeric(irscale, length = 1)
  else
    irscale <- 1
  #
  if (!is.character(axes)) {
    chklogical(axes)
    axis.x <- axis.y <- axes
  }
  else {
    axes <- setchar(axes, c("x", "y", "xy"))
    #
    axis.x <- grepl("x", axes)
    axis.y <- grepl("y", axes)
  }
  #
  chklogical(box)
  
  
  m0 <- update(x$.meta$x)
  method.tau0 <- x$.meta$method.tau
  #
  if (method.tau0 != "FE" & (method.tau0 != m0$method.tau))
    m1 <- update(m0, method.tau = method.tau0)
  else
    m1 <- m0
  #
  TE <- m1$TE
  sm <- m1$sm

  log.y <- backtransf &&
    (is_relative_effect(sm) || is_log_effect(sm) ||
     (!is.null(m1$func.backtransf) && m1$func.backtransf == "exp"))
  #
  if (log.y) {
    log <- "y"
    TE <- exp(TE)
  }
  else
    log <- ""
  #
  if (missing(ref)) {
    if (is_prop(sm) | is_rate(sm) | is_mean(sm))
      ref <- NA
    else if (log.y)
      ref <- 1
    else
      ref <- 0
  }
  #
  chknumeric(ref, length = 1)
  
  
  if (is.logical(studlab)) {
    if (studlab)
      studlab <- m1$studlab
    else
      studlab <- rep("", length(TE))
  }
  else {
    studlab <- as.character(studlab)
    if (length(studlab) != length(TE))
      stop("Length of argument 'studlab' must be the same as ",
           "number of studies in meta-analysis.")
  }
  
  
  charform <- as.character(x$.meta$formula)[2]
  splitform <- strsplit(charform, " ")[[1]]
  covar.name <- splitform[1]
  if (covar.name == "1" | covar.name == "-1")
    covar.name <- splitform[3]
  #
  covar.names <- names(coef(x))
  covar.names.without.intrcpt <- covar.names[covar.names != "intrcpt"]
  if (length(covar.names.without.intrcpt) == 0) {
    warning("No covariate in meta-regression.")
    return(invisible(NULL))
  }
  #
  nointrcpt <- ifelse("intrcpt" %in% covar.names, FALSE, TRUE)
  #
  if (covar.name == ".subgroup")
    covar.name <- x$.meta$x$subgroup.name
  #
  if (covar.name %in% names(x$.meta$x$data))
    covar <- x$.meta$x$data[[covar.name]]
  else if (covar.name %in% names(x$.meta$x))
    covar <- x$.meta$x[[covar.name]]
  else if (".subgroup" %in% names(x$.meta$x$data))
    covar <- x$.meta$x$data[[".subgroup"]]
  else
    covar <- get(covar.name)
  #
  if (!is.null(x$.meta$x$subset))
    covar <- covar[x$.meta$x$subset]
  #
  if (is.character(covar))
    covar <- as.factor(covar)
  #
  if (is.factor(covar)) {
    levs <- levels(covar)
    xs <- as.numeric(covar) - 1
    at.x <- sort(unique(xs))
    if (missing(xlim))
      xlim <- c(-0.5, 1.5)
    if (missing(xlab))
      xlab <- ""
  }
  else {
    xs <- covar
    if (missing(xlim))
      xlim <- range(xs, na.rm = TRUE)
  }
  #
  alpha <- ifelse(nointrcpt, 0, coef(x)["intrcpt"])
  #
  if (covar.name %in% names(coef(x)))
    beta <- coef(x)[covar.name]
  else
    beta <- coef(x)[".subgroup"]
  #
  if (length(covar.names.without.intrcpt) > 1 & !is.factor(covar)) {
    warning(paste0("Only first covariate in meta-regression ",
                   "('", covar.name, "') considered in bubble plot. ",
                   "No regression line plotted.")
            )
    regline <- FALSE
    if (missing(xlab))
      xlab <- paste0("Covariate ", covar.name,
                     " (meta-regression: ", charform, ")")
  }
  else
    if (missing(xlab))
      xlab = paste("Covariate", covar.name)
  
  
  if (backtransf && (pscale != 1 || irscale != 1)) {
    if (pscale != 1 && irscale != 1)
      stop("Provide either arguments 'pscale' or 'irscale'",
           call. = FALSE)
    if (pscale != 1)
      scale <- pscale
    else
      scale <- irscale
  }
  else
    scale <- 1
  #
  ys <- TE
  #
  if (backtransf & !log.y) {
    if (!is.null(m1$func.backtransf))
      func.backtransf <- m1$func.backtransf
    else if (sm == "PLOGIT")
      func.backtransf <- logit2p
    else if (sm == "PAS")
      func.backtransf <- asin2p
    else if (sm == "IRS")
      func.backtransf <- function(x) x^2
    else if (sm == "ZCOR")
      func.backtransf <- z2cor
    else
      func.backtransf <- I
    #
    ys <- do.call(func.backtransf, list(ys))
  }
  #
  if (missing(ylim))
    ylim <- scale * range(ys)
  #
  if (missing(ylab)) {
    ylab <- xlab_meta(sm, backtransf, func.transf = m1$func.transf,
                 func.backtransf = m1$func.backtransf)
    #
    if (ylab == "") {
      if (sm == "PRAW" | (backtransf & sm %in% c("PLN", "PAS", "PLOGIT")))
        ylab <- "Proportion"
      else if (sm == "IR" | (backtransf & sm %in% c("IRLN", "IRS")))
        ylab <- "Incidence Rate"
      else if (sm == "MRAW" | (backtransf & sm %in% c("IRLN", "IRS")))
        ylab <- "Incidence Rate"
      else if (sm == "COR" | (backtransf & sm == "ZCOR"))
        ylab <- "Correlation"
    }
  }
  
  
  missing.cex <- missing(cex)
  #
  if (!missing.cex && is.character(cex)) {
    cex.type <- setchar(cex, c("common", "random", "fixed"),
                        "must be numeric or equal to \"common\" or \"random\"")
    cex.type[cex.type == "fixed"] <- "common"
    if (length(unique(cex.type)) != 1)
      stop("Argument 'cex' must be numeric or equal to ",
           "\"common\" or \"random\".")
    common.cex <- all(cex.type == "common")
    random.cex <- all(cex.type == "random")
  }
  else {
    common.cex <- FALSE
    random.cex <- FALSE
  }
  #
  if (missing.cex)
    if (method.tau0 == "FE")
      cex <- m1$w.common
    else
      cex <- m1$w.random
  else if (common.cex)
    cex <- m1$w.common
  else if (random.cex)
    cex <- m1$w.random
  else if (length(cex) == 1)
    cex <- rep_len(cex, length(TE))
  #
  if (all(is.na(cex))) {
    cex <- rep_len(1, length(TE))
    min.cex <- 1
    max.cex <- 1
  }
  #
  if (length(cex) != length(TE))
    stop("Length of argument 'cex' must be the same as ",
         "number of studies in meta-analysis.")
  #
  if (missing.cex | common.cex) {
    cexs <- max.cex * (cex / max(cex))
    cexs[cexs < min.cex] <- min.cex
  }
  else
    cexs <- cex
  #
  if (length(col) > 1 && length(col) != length(TE))
    stop("Length of argument 'col' must be 1 or the same as ",
         "number of studies in meta-analysis.")
  #
  if (length(bg) > 1 && length(bg) != length(TE))
    stop("Length of argument 'col' must be 1 or the same as ",
         "number of studies in meta-analysis.")
  #
  o <- rev(order(cex))
  xs <- xs[o]
  ys <- ys[o]
  studlab <- studlab[o]
  cexs <- cexs[o]
  #
  if (length(pch) > 1)
    pchs <- pch[o]
  else
    pchs <- rep(pch, length(o))
  #
  if (length(col) > 1)
    cols <- col[o]
  else
    cols <- rep(col, length(o))
  #
  if (length(bg) > 1)
    bgs <- bg[o]
  else
    bgs <- rep(bg, length(o))
  #
  if (length(cex.studlab) > 1)
    cex.studlab <- cex.studlab[o]
  #
  if (length(pos.studlab) > 1)
    pos.studlab <- pos.studlab[o]
  #
  if (length(offset) > 1)
    offset <- offset[o]
  
  
  #
  #
  # Generate bubble plot
  #
  #
  if (is.factor(covar)) {
    for (i in 2:length(levs)) {
      sel <- xs %in% c(0, i - 1)
      xs.i <- xs[sel]
      xs.i[xs.i > 0] <- 1
      ys.i <- ys[sel]
      studlab.i <- studlab[sel]
      cexs.i <- cexs[sel]
      pch.i <- pchs[sel]
      col.i <- cols[sel]
      bg.i <- bgs[sel]
      #
      plot(xs.i, scale * ys.i,
           pch = pch.i, cex = cexs.i,
           xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim,
           type = "n", axes = FALSE, log = log, ...)
      #
      # Add reference line
      #
      abline(h = scale * ref, col = col.ref, lty = lty.ref, lwd = lwd.ref)
      #
      # Add regression line
      #
      if (regline) {
        x.reg <- seq(0, 1, length.out = 500)
        y.reg <- seq(alpha, alpha + coef(x)[i], length.out = 500)
        #
        if (log.y)
          y.reg <- scale * exp(y.reg)
        else if (backtransf)
          y.reg <- scale * do.call(func.backtransf, list(y.reg))
        #
        lines(x.reg, y.reg, lty = lty, lwd = lwd, col = col.line)
      }
      #
      for (j in seq_along(xs.i))
        points(xs.i[j], scale * ys.i[j], cex = cexs.i[j], pch = pch.i[j],
               col = col.i[j], bg = bg.i[j])
      #
      # x-axis
      #
      if (axis.x)
        axis(1, at = 0:1, labels = levs[c(1, i)], ...)
      #
      # y-axis
      #
      if (axis.y)
        axis(2, ...)
      #
      text(xs.i, scale * ys.i, labels = studlab.i, cex = cex.studlab,
           pos = pos.studlab, offset = offset)
      #
      if (box)
        box()
    }
  }
  else {
    plot(xs, scale * ys,
         pch = pchs, cex = cexs,
         xlab = xlab, ylab = ylab, xlim = xlim, ylim = ylim,
         type = "n", axes = FALSE, log = log, ...)
    #
    # Add reference line
    #
    abline(h = scale * ref, col = col.ref, lty = lty.ref, lwd = lwd.ref)
    #
    # Add regression line
    #
    if (regline) {
      x.reg <- seq(xlim[1], xlim[2], length.out = 500)
      y.reg <- seq(alpha + beta * xlim[1], alpha + beta * xlim[2],
                   length.out = 500)
      #
      if (log.y)
        y.reg <- scale * exp(y.reg)
      else if (backtransf)
        y.reg <- scale * do.call(func.backtransf, list(y.reg))
      #
      lines(x.reg, y.reg, lty = lty, lwd = lwd, col = col.line)
    }
    #
    points(xs, scale * ys, cex = cexs, pch = pchs, col = cols, bg = bgs)
    #
    # x-axis
    #
    if (axis.x)
      axis(1, ...)
    #
    # y-axis
    #
    if (axis.y)
      axis(2, ...)
    #
    text(xs, scale * ys, labels = studlab, cex = cex.studlab,
         pos = pos.studlab, offset = offset)
    #
    if (box)
      box()
  }
  
  
  invisible(NULL)
}





#' @rdname bubble.metareg
#' @export bubble

bubble <- function(x, ...) 
  UseMethod("bubble")

#' Amlodipine for Work Capacity
#' 
#' @description
#' Meta-analysis on the effect of amlodipine on work capacity.
#' 
#' This meta-analysis is used as a data example in Hartung and Knapp (2001).
#' 
#' @name amlodipine
#'
#' @docType data
#'
#' @format
#'
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{n.amlo}}\tab number of observations in amlodipine group
#'   \cr
#' \bold{\emph{mean.amlo}}\tab estimated mean in amlodipine group \cr
#' \bold{\emph{var.amlo}}\tab variance in amlodipine group \cr
#' \bold{\emph{n.plac}}\tab number of observations in placebo group
#'   \cr
#' \bold{\emph{mean.plac}}\tab estimated mean in placebo group \cr
#' \bold{\emph{var.plac}}\tab variance in placebo group
#' }
#' 
#' @seealso \code{\link{metacont}}
#' 
#' @source
#' Hartung J & Knapp G (2001):
#' On tests of the overall treatment effect in meta-analysis with
#' normally distributed responses.
#' \emph{Statistics in Medicine},
#' \bold{20}, 1771--82
#' 
#' @keywords datasets
#' 
#' @examples
#' data(amlodipine)
#' 
#' ma <- metacont(n.amlo, mean.amlo, sqrt(var.amlo),
#'   n.plac, mean.plac, sqrt(var.plac),
#'   data = amlodipine, studlab = study,
#'   method.tau = "DL")
#' ma.hk <- update(ma, method.random.ci = "HK")
#' 
#' # Same results for mean difference as in Table III in Hartung and
#' # Knapp (2001)
#' #
#' vars.common <- c("TE.common", "lower.common", "upper.common")
#' vars.random <- c("TE.random", "lower.random", "upper.random")
#' #
#' res.common <- as.data.frame(ma[vars.common])
#' names(res.common) <- vars.random
#' #
#' res.md <- rbind(res.common,
#'   as.data.frame(ma[vars.random]),
#'   as.data.frame(ma.hk[vars.random]))
#' #
#' res.md <- round(res.md, 5)
#' #
#' row.names(res.md) <- c("CE", "RE", "RE (HaKn)")
#' names(res.md) <- c("Absolute difference", "CI lower", "CI upper")
#' #
#' res.md

NULL

#' Calculation of confidence intervals (based on normal approximation
#' or t-distribution)
#' 
#' @description
#' Calculation of confidence intervals; based on normal approximation
#' or t-distribution.
#' 
#' @param TE Estimated treatment effect.
#' @param seTE Standard error of treatment estimate.
#' @param level The confidence level required.
#' @param df Degrees of freedom (for confidence intervals based on
#'   t-distribution).
#' @param null.effect A numeric value specifying the effect under the
#'   null hypothesis.
#' 
#' @return
#' List with components
#' \item{TE}{Estimated treatment effect}
#' \item{seTE}{Standard error of treatment estimate}
#' \item{lower}{Lower confidence limits}
#' \item{upper}{Upper confidence limits}
#' \item{statistic}{Test statistic (either z-score or t-score)}
#' \item{p}{P-value of text.with null hypothesis \code{TE=0}}
#' \item{level}{The confidence level required}
#' \item{df}{Degrees of freedom (t-distribution)}
#' 
#' @note
#' This function is primarily called from other functions of the
#' library \code{meta}, e.g. \code{forest.meta}, \code{summary.meta}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @examples
#' data.frame(ci(170, 10))
#' data.frame(ci(170, 10, 0.99))
#' data.frame(ci(1.959964, 1))
#' data.frame(ci(2.2621571628, 1, df = 9))
#' 
#' @export ci

ci <- function(TE, seTE, level = 0.95, df = NULL, null.effect = 0) {
  
  chklevel(level)
  alpha <- 1 - level
  
  df.orig <- df
  
  if (is.null(df)) {
    lower <- TE - qnorm(1 - alpha / 2) * seTE
    upper <- TE + qnorm(1 - alpha / 2) * seTE
    statistic <- (TE - null.effect) / seTE
    pval <- 2 * pnorm(abs(statistic), lower.tail = FALSE)
    #
    df <- Inf
    df.orig <- "NULL"
  }
  else {
    statistic <- (TE - null.effect) / seTE
    #
    if (length(df) == 1) {
      if (is.na(df) || df <= 0) {
        lower <- TE - qnorm(1 - alpha / 2) * seTE
        upper <- TE + qnorm(1 - alpha / 2) * seTE
        #
        pval <- 2 * pnorm(abs(statistic), lower.tail = FALSE)
      }
      else {
        lower <- TE - qt(1 - alpha / 2, df = df) * seTE
        upper <- TE + qt(1 - alpha / 2, df = df) * seTE
        #
        pval <- 2 * pt(abs(statistic), df = df, lower.tail = FALSE)
      }
    }
    else {
      sel0 <- is.na(df) | df <= 0
      df[sel0] <- NA
      lower <- ifelse(!is.na(df),
                      TE - qt(1 - alpha / 2, df = df) * seTE,
                      TE - qnorm(1 - alpha / 2) * seTE)
      upper <- ifelse(!is.na(df),
                      TE + qt(1 - alpha / 2, df = df) * seTE,
                      TE + qnorm(1 - alpha / 2) * seTE)
      #        
      pval <- ifelse(!is.na(df),
                     2 * pt(abs(statistic), df = df, lower.tail = FALSE),
                     2 * pnorm(abs(statistic), lower.tail = FALSE))
      #
      df[sel0] <- df.orig[sel0]
    }
    #
    df[is.na(df)] <- Inf
  }
  
  res <- list(TE = TE, seTE = seTE,
              lower = lower, upper = upper,
              statistic = statistic, p = pval, level = level,
              df = df, null.effect = null.effect)
  
  res
}

#' Aspirin after Myocardial Infarction
#' 
#' @description
#' Meta-analysis on aspirin in preventing death after myocardial
#' infarction.
#' 
#' Data example in Fleiss (1993) for meta-analysis with binary
#' outcomes.
#' 
#' @name Fleiss1993bin
#' @aliases Fleiss93
#' 
#' @docType data
#' 
#' @format
#' A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{year}}\tab year of publication \cr
#' \bold{\emph{d.asp}}\tab number of deaths in aspirin group \cr
#' \bold{\emph{n.asp}}\tab number of observations in aspirin group \cr
#' \bold{\emph{d.plac}}\tab number of deaths in placebo group \cr
#' \bold{\emph{n.plac}}\tab number of observations in placebo group
#' }
#' 
#' @source
#' Fleiss JL (1993):
#' The statistical basis of meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{2}, 121--45
#' 
#' @keywords datasets
#' 
#' @seealso \code{\link{metabin}}
#' 
#' @examples
#' data(Fleiss1993bin)
#' metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
#'   studlab = paste(study, year), sm = "OR", random = FALSE)


NULL

#' Print results of a leave-one-out meta-analysis
#' 
#' @description
#' Print results of a leave-one-out meta-analysis
#' 
#' @aliases print.metainf
#' 
#' @param x An object of class \code{\link{metainf}}.
#' @param prediction A logical indicating whether prediction
#'   intervals should be printed.
#' @param overall A logical indicating whether overall results should be
#'   printed.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf=TRUE}, results
#'   for \code{sm="OR"} are printed as odds ratios rather than log
#'   odds ratios, for example.
#' @param header A logical indicating whether information on title of
#'   meta-analysis, comparison and outcome should be printed at the
#'   beginning of the printout.
#' @param lab.NA A character string to label missing values.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value of test for overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance \eqn{\tau^2}, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   and Rb statistic, see \code{print.default}.
#' @param digits.cid Minimal number of significant digits for
#'   CID / decision thresholds, see \code{print.default}.
#' @param digits.percent Minimal number of significant digits for
#'   probabilities, printed as percentages, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.stat A logical value indicating whether z- or t-value
#'   for test of treatment effect should be printed.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau2.ci A logical value indicating whether to print
#'   the confidence interval of \eqn{\tau^2}.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.tau.ci A logical value indicating whether to print the
#'   confidence interval of \eqn{\tau}.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param print.prob A logical specifying whether to print probabilities
#'   of clinically important benefit or harm.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param text.I2 Text printed to identify heterogeneity statistic
#'   I\eqn{^2}.
#' @param details.methods A logical specifying whether details on
#'   statistical methods should be printed.
#' @param \dots Additional arguments (ignored).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metainf}}, \code{\link{settings.meta}}
#' 
#' @examples
#' data(Fleiss1993bin)
#' ma <- metabin(d.asp, n.asp, d.plac, n.plac,
#'   data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
#' ma
#' metainf(ma)
#' metainf(ma, pooled = "random")
#' metainf(ma, pooled = "random", prediction = TRUE)
#'
#' @method print metainf
#' @export

print.metainf <- function(x,
                          #
                          prediction = x$prediction,
                          overall = x$overall,
                          backtransf = x$backtransf,
                          header = TRUE,
                          #
                          lab.NA = ".",
                          #
                          digits = gs("digits"),
                          digits.stat = gs("digits.stat"),
                          digits.pval = gs("digits.pval"),
                          digits.tau2 = gs("digits.tau2"),
                          digits.tau = gs("digits.tau"),
                          digits.I2 = gs("digits.I2"),
                          digits.cid = gs("digits.cid"),
                          digits.percent = 1,
                          #
                          big.mark = gs("big.mark"),
                          scientific.pval = gs("scientific.pval"),
                          zero.pval = gs("zero.pval"),
                          JAMA.pval = gs("JAMA.pval"),
                          #
                          print.stat = FALSE,
                          print.tau2 = TRUE,
                          print.tau2.ci = FALSE,
                          print.tau = TRUE,
                          print.tau.ci = FALSE,
                          print.I2 = TRUE,
                          print.I2.ci = FALSE,
                          print.prob = TRUE,
                          #
                          text.tau2 = gs("text.tau2"),
                          text.tau = gs("text.tau"),
                          text.I2 = gs("text.I2"),
                          #
                          details.methods = gs("details"),
                          ...) {
  
  chkclass(x, "metainf")
  #
  print.metacum(x,
                #
                prediction = prediction,
                overall = overall,
                backtransf = backtransf,
                header = header,
                #
                digits = digits,
                digits.stat = digits.stat,
                digits.pval = digits.pval,
                digits.tau2 = digits.tau2,
                digits.tau = digits.tau,
                digits.I2 = digits.I2,
                digits.cid = digits.cid,
                digits.percent = digits.percent,
                #
                big.mark = big.mark,
                scientific.pval = scientific.pval,
                zero.pval = zero.pval,
                JAMA.pval = JAMA.pval,
                #
                print.stat = print.stat,
                print.tau2 = print.tau2,
                print.tau2.ci = print.tau2.ci,
                print.tau = print.tau,
                print.tau.ci = print.tau.ci,
                print.I2 = print.I2,
                print.I2.ci = print.I2.ci,
                print.prob = print.prob,
                #
                text.tau2 = text.tau2,
                text.tau = text.tau,
                text.I2 = text.I2,
                #
                details.methods = details.methods,
                ...)
  #
  invisible(NULL)
}

#' Get default for a meta-analysis setting.
#' 
#' @description
#' Get default for a meta-analysis setting in R package \bold{meta}.
#' 
#' @param x A character string or vector with setting name(s).
#' @param unname A logical whether to remove names from attributes.
#' 
#' @details
#' This function can be used to get the default for a meta-analysis
#' setting defined using \code{\link{settings.meta}}.
#' 
#' This function is primarily used to define default settings in
#' meta-analysis functions, e.g., \code{\link{metabin}} or
#' \code{\link{metacont}}. A list of all arguments with current
#' settings is printed using the command
#' \code{settings.meta("print")}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{settings.meta}}
#' 
#' @examples
#' # Get default setting for confidence interval of random effects
#' # model
#' #
#' gs("method.random.ci")
#' 
#' # Get default setting for summary measure in metabin()
#' #
#' gs("smbin")
#' 
#' @export gs

gs <- function(x = NULL, unname = NULL) {
  
  if (is.null(x))
    return(NULL)
  #
  chkchar(x)
  if (!is.null(unname))
    chklogical(unname)
  
  x.list <- vector("list", length(x))
  #
  for (i in seq_along(x)) {
    x.list[[i]] <-
    setchar(x[i],
            c(.settings$argslist, .settings$argslist.internal),
            "unmatched", name = x[i],
            stop.at.error = FALSE)
  }
  x.list <- compact(x.list)
  #
  if (is.null(x.list) | length(x.list) == 0)
    return(NULL)
  
  res <- vector("list", length(x.list))
  #
  for (i in seq_along(x.list))
    res[[i]] <- settings.meta(quietly = TRUE)[[x.list[[i]]]]
  #
  if (is.null(res) | length(res) == 0)
    return(NULL)
  #
  if (is.null(unname))
    unname <- length(res) <= 1
  #
  if (!unname)
    names(res) <- unlist(x.list)
  
  if (length(res) == 1)
    res <- unlist(res)
  
  res
}

#' Summary of meta-analysis results
#' 
#' @description
#' Summary method for objects of class \code{meta}.
#' 
#' @param object An object of class \code{meta}.
#' @param \dots Additional arguments (ignored).
#'
#' @details
#' Summary method for objects of class \code{meta}.
#' 
#' @return
#' An object of classes \code{summary.meta} and \code{meta} (see
#' \code{\link{meta-object}}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{print.summary.meta}}, \code{\link{metabin}},
#'   \code{\link{metacont}}, \code{\link{metagen}}
#' 
#' @references
#' Cooper H & Hedges LV (1994):
#' \emph{The Handbook of Research Synthesis}.
#' Newbury Park, CA: Russell Sage Foundation
#' 
#' Crippa A, Khudyakov P, Wang M, Orsini N, Spiegelman D (2016):
#' A new measure of between-studies heterogeneity in meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{35}, 3661--75
#' 
#' Higgins JPT & Thompson SG (2002):
#' Quantifying heterogeneity in a meta-analysis.
#' \emph{Statistics in Medicine},
#' \bold{21}, 1539--58
#' 
#' @examples
#' data(Fleiss1993cont)
#' ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, studlab = paste(study, year), sm = "SMD")
#' summary(ma)
#' 
#' summary(update(ma, subgroup = c(1, 2, 1, 1, 2), subgroup.name = "group"))
#' forest(update(ma, subgroup = c(1, 2, 1, 1, 2), subgroup.name = "group"))
#' 
#' \dontrun{
#' # Use unicode characters to print tau^2, tau, and I^2
#' print(summary(ma),
#'   text.tau2 = "\u03c4\u00b2", text.tau = "\u03c4", text.I2 = "I\u00b2")
#' }
#' 
#' @method summary meta
#' @export

summary.meta <- function(object, ...) {
  
  #
  #
  # (1) Check for meta object and upgrade older meta objects
  #
  #
  
  chkclass(object, "meta")
  #
  if (inherits(object, c("metacum", "metainf")))
    return(object)
  #
  object <- updateversion(object)
  #
  metaprop <- inherits(class(object), "metaprop")
  metarate <- inherits(class(object), "metarate")
  
  #
  #
  # (2) Check other arguments
  #
  #
  
  addargs <- names(list(...))
  #
  if (length(addargs) > 0)
    warning("Additional arguments provided in '...' are ignored.",
            call. = FALSE)
  
  #
  #
  # (3) Results for individual studies
  #
  #
  
  object$df <- replaceNULL(object$df, Inf)
  method.ci <- replaceNULL(object$method.ci, "")
  object$statistic <- replaceNULL(object$statistic, object$zval)
  #
  ci.study <- list(TE = object$TE,
                   seTE = object$seTE,
                   lower = object$lower,
                   upper = object$upper,
                   statistic = object$statistic,
                   p = object$pval,
                   level = object$level,
                   df = object$df)
  #
  if (metaprop) {
    ci.study$event <- object$event
    ci.study$n <- object$n
  }
  
  #
  #
  # (4) Results for meta-analysis
  #
  #
  
  ci.c <- list(TE = object$TE.common,
               seTE = object$seTE.common,
               lower = object$lower.common,
               upper = object$upper.common,
               statistic = object$statistic.common,
               p = object$pval.common,
               level = object$level.ma)
  if (metaprop)
    ci.c$harmonic.mean <- 1 / mean(1 / object$n)
  else if (metarate)
    ci.c$harmonic.mean <- 1 / mean(1 / object$time)
  #
  ci.r <- list(TE = object$TE.random,
               seTE = object$seTE.random,
               lower = object$lower.random,
               upper = object$upper.random,
               statistic = object$statistic.random,
               p = object$pval.random,
               level = object$level.ma,
               df = object$df.random)
  if (metaprop)
    ci.r$harmonic.mean <- 1 / mean(1 / object$n)
  else if (metarate)
    ci.r$harmonic.mean <- 1 / mean(1 / object$time)
  #
  ci.p <- list(TE = NA,
               seTE = object$seTE.predict,
               lower = object$lower.predict,
               upper = object$upper.predict,
               statistic = NA,
               p = NA,
               level = object$level.predict,
               df = object$df.predict)
  
  #
  #
  # (5) Generate R object
  #
  #
  
  res <- object
  #
  res$common <- ci.c
  res$random <- ci.r
  res$predict <- ci.p
  #
  # Backward compatibility
  #
  res$fixed <- ci.c
  #
  # Add results from subgroup analysis
  #
  if (length(object$subgroup) > 0) {
    #
    n.subgroups <- length(object$subgroup.levels)
    #
    ci.common.w <- list(TE = object$TE.common.w,
                        seTE = object$seTE.common.w,
                        lower = object$lower.common.w,
                        upper = object$upper.common.w,
                        statistic = object$statistic.common.w,
                        p = object$pval.common.w,
                        level = object$level.ma,
                        harmonic.mean = object$n.harmonic.mean.w)
    #
    if (metarate)
      ci.common.w$harmonic.mean <- object$t.harmonic.mean.w
    #
    ci.random.w <- list(TE = object$TE.random.w,
                        seTE = object$seTE.random.w,
                        lower = object$lower.random.w,
                        upper = object$upper.random.w,
                        statistic = object$statistic.random.w,
                        p = object$pval.random.w,
                        level = object$level.ma,
                        df = object$df.random.w,
                        harmonic.mean = object$n.harmonic.mean.w)
    #
    ci.predict.w <- list(TE = rep(NA, n.subgroups),
                         seTE = object$seTE.predict.w,
                         lower = object$lower.predict.w,
                         upper = object$upper.predict.w,
                         statistic = rep(NA, n.subgroups),
                         p = rep(NA, n.subgroups),
                         level = object$level.predict,
                         df = object$df.predict.w,
                         harmonic.mean = object$n.harmonic.mean.w)
    #
    if (metarate)
      ci.random.w$harmonic.mean <- object$t.harmonic.mean.w
    #
    res$within.common  <- ci.common.w
    res$within.random  <- ci.random.w
    res$within.predict <- ci.predict.w
    #
    if (is.null(res$test.subgroup))
      res$test.subgroup <- TRUE
  }
  #
  res$x <- object
  #
  class(res) <- c("summary.meta", class(object))
  #
  attr(res, "class.orig") <- class(object)
  
  res
}

catmeth <- function(x,
                    common, random, prediction, overall, overall.hetstat,
                    prediction.subgroup = FALSE,
                    #
                    func.transf, backtransf, func.backtransf,
                    #
                    big.mark, digits, digits.tau, text.tau, text.tau2,
                    #
                    print.tau2 = TRUE, print.tau2.ci = FALSE,
                    print.tau = FALSE, print.tau.ci = FALSE,
                    #
                    print.I2 = FALSE, text.I2,
                    #
                    print.df = TRUE,
                    #
                    forest = FALSE) {

  #
  #
  # (1) Some settings
  #
  #
  
  metacum.metainf <- inherits(x, "metacum") | inherits(x, "metainf")
  metabind <- inherits(x, "metabind")
  #
  metabin  <- inherits(x, "metabin") & !metacum.metainf
  metacont <- inherits(x, "metacont") & !metacum.metainf
  metacor  <- inherits(x, "metacor") & !metacum.metainf
  metagen  <- inherits(x, "metagen") & !metacum.metainf
  metainc  <- inherits(x, "metainc") & !metacum.metainf
  metamean <- inherits(x, "metamean") & !metacum.metainf
  metaprop <- inherits(x, "metaprop") & !metacum.metainf
  metarate <- inherits(x, "metarate") & !metacum.metainf
  #
  trimfill <- inherits(x, "trimfill") & !metacum.metainf
  metamiss <- inherits(x, "metamiss") & !metacum.metainf
  #
  by <- !is.null(x$subgroup)
  #
  gm <- gm(x)
  meth <- gm$meth
  pred <- gm$pred
  #
  selmod <-
    if (!common & !random)
      "common"
    else if (common & !random)
      "common"
    else if (!common & random)
      "random"
    else
      c("common", "random")
  #
  bothmod <- length(selmod) > 1
  #
  # Print information on meta-analysis of n-of-1-trials first
  #
  if (!is.null(x$cycles)) {
    details <-
      paste0("\n- ",
             if (any(x$k.all > 1)) "Meta-a" else "A",
             "nalysis of n-of-1 trials (pooled SD=",
             x$sd.n_of_1,
             ")")
  }
  else
    details <- NULL
  #
  width <- options()$width
  #
  if (metabin)
    method <- "metabin"
  else if (metacont)
    method <- "metacont"
  else if (metacor)
    method <- "metacor"
  else if (metagen)
    method <- "metagen"
  else if (metainc)
    method <- "metainc"
  else if (metamean)
    method <- "metamean"
  else if (metaprop)
    method <- "metaprop"
  else if (metarate)
    method <- "metarate"
  else if (metacum.metainf | metabind)
    method <- x$classes
  #
  if (forest) {
    text.tau2 <- "tau^2"
    text.tau <- "tau"
    #
    text.I2 <- "I^2"
  }
  #
  text.t <- ""
  #
  if (print.tau2 | metabind | metacum.metainf)
    text.t <- text.tau2
  else if (print.tau)
    text.t <- text.tau
  #
  method.I2 <- replaceNULL(x$method.I2, "Q")
  
  
  #
  #
  # (2) Meta-analysis methods
  #
  #
    
  if (overall | metabind | metacum.metainf | by) {
    
    meth.ma <- meth[meth$model %in% selmod, , drop = FALSE]
    #
    vars.ma <- c("model", "method")
    #
    if ((metabin | metainc) & any(meth.ma$method == "GLMM"))
      vars.ma <- c(vars.ma, "model.glmm")
    #
    vars.ma <- c(vars.ma, "three.level", "rho")
    #
    if (metacont)
      vars.ma <- c(vars.ma, "pooledvar")
    else if (metabin) {
      vars.ma <- c(vars.ma, "incr", "method.incr", "sparse", "MH.exact")
      if (any(meth.ma$method == "LRP"))
        vars.ma <- c(vars.ma, "phi")
    }
    else if (metainc)
      vars.ma <- c(vars.ma, "incr", "method.incr", "sparse")
    #
    meth.ma <- unique(meth.ma[, vars.ma, drop = FALSE])
    #
    details.i <- vector("character", length = nrow(meth.ma))
    for (i in seq_len(nrow(meth.ma)))
      details.i[i] <- text_meth(meth.ma, i, random, method)
    #
    details <- paste(c(details, unique(details.i)), collapse = "")
    #
    # Information on user-specified weights
    #
    if (!is.null(x$weights.common) | !is.null(x$weights.random)) {
      if (!is.null(x$weights.common) & !is.null(x$weights.random))
        details.usw <- ""
      else if  (!is.null(x$weights.common))
        details.usw <- " (common effect model)"
      else
        details.usw <- " (random effects model)"
      #
      details <-
        paste0(details, "\n- User-specified weights", details.usw)
    }
  }
  
  
  #
  #
  # (3) Estimation of between-study variance
  #
  #
  if (((random | (metacum.metainf & (print.tau2 | print.tau))) & 
       (overall | metabind | by)) | overall.hetstat) {
    if (metacum.metainf)
      sel.mt <- meth$model %in% c("common", "random")
    else
      sel.mt <- meth$model == "random"
    #
    dat.mt <- unique(meth[sel.mt, c("tau.preset", "method.tau")])
    #
    tau.preset <- dat.mt$tau.preset[!is.na(dat.mt$tau.preset)]
    #
    if (length(tau.preset) >= 1)
      details <-
        paste0(details,
               "\n- Preset square root of between-study variance: ",
               text.tau, " = ",
               cond(tau.preset, only.finite = FALSE, digits = digits.tau,
                    big.mark = big.mark))
    #
    method.tau <- unique(dat.mt$method.tau[is.na(dat.mt$tau.preset)])
    #
    if ((print.tau2 | print.tau | metabind) & length(method.tau) >= 1) {
      for (mti in method.tau) {
        details <-
          paste0(details,
                 if (mti == "DL")
                   "\n- DerSimonian-Laird estimator"
                 #
                 else if (mti == "PM")
                   "\n- Paule-Mandel estimator"
                 #
                 else if (mti == "REML")
                   "\n- Restricted maximum-likelihood estimator"
                 #
                 else if (mti == "ML")
                   "\n- Maximum-likelihood estimator"
                 #
                 else if (mti == "HS")
                   "\n- Hunter-Schmidt estimator"
                 #
                 else if (mti == "SJ")
                   "\n- Sidik-Jonkman estimator"
                 #
                 else if (mti == "HE")
                   "\n- Hedges estimator"
                 #
                 else if (mti == "EB")
                   "\n- Empirical Bayes estimator")
        #
        if (mti != "")
          details <- paste(details, "for", text.t)
        #
        if (replaceNULL(x$tau.common, FALSE))
          details <-
            paste0(details,
                   if (forest) " " else "\n  ",
                   "(assuming common ", text.t, " in subgroups)")
      }
    }
    #
    if (metabin | metacum.metainf) {
      dat.qc <-
        unique(meth[meth$model %in% "random" & is.na(meth$tau.preset),
                    c("method.tau", "Q.Cochrane")])
      dat.qc$Q.Cochrane <- replaceNA(dat.qc$Q.Cochrane, FALSE)
      #
      if (any(dat.qc$method.tau == "DL" & dat.qc$Q.Cochrane))
        details <-
          paste0(details,
                 "\n- Mantel-Haenszel estimator used in ",
                 "calculation of Q and ", text.t,
                 if (width > 70 | forest) " (like RevMan 5)"
                 else "\n  (like RevMan 5)")
    }
  }
  
  
  #
  #
  # (4) Confidence interval for between-study variance
  #
  #

  if (print.tau.ci | print.tau2.ci) {
    method.tau.ci <- unique(meth$method.tau.ci[meth$model %in% "random"])
    #
    if (length(method.tau) >= 1) {
      for (mtci in method.tau.ci) {
        details <-
          paste0(details,
                 if (mtci == "QP")
                   "\n- Q-Profile method"
                 #
                 else if (mtci == "BJ")
                   "\n- Biggerstaff and Jackson method"
                 #
                 else if (mtci == "J")
                   "\n- Jackson method"
                 #
                 else if (mtci == "PL")
                   "\n- Profile-Likelihood method")
        #
        if (mtci != "")
          details <-
            paste0(details,
                  " for confidence interval of ",
                  if (print.tau2 & print.tau)
                    paste(text.tau2, "and", text.tau)
                  else if (print.tau2)
                    text.tau2
                  else
                    text.tau)
      }
    }
  }
  
  
  #
  #
  # (5) Method to estimate I2
  #
  #
    
  if (print.I2) {
    method.I2 <- unique(meth$method.I2)
    #
    if (length(method.I2) >= 1) {
      for (mi2i in method.I2) {
        details <-
          paste0(details,
                 if (mi2i == "Q")
                   paste0("\n- Calculation of ", text.I2,
                          " based on Q")
                 else if (mi2i == "tau2")
                   paste0("\n- Calculation of ", text.I2,
                          " based on ", text.tau2))
      }
    }
  }
  
  
  #
  #
  # (6) Confidence interval of common effect estimate
  #
  #
  
  if (common &
      any(meth$model == "common" & meth$method.common.ci == "IVhet"))
    details <- paste0(details,
                      "\n- Inverse variance heterogeneity model")
  
  
  #
  #
  # (7) Confidence interval of random effects estimate
  #
  #

  if (random) {
    vars <- c("method", "method.random.ci", "df.random", "three.level")
    #
    dat.rc <- unique(meth[meth$model == "random", vars])
    dat.rc.hk <-
      unique(meth[meth$model == "random", c(vars, "adhoc.hakn.ci")])
    #
    dat.rc.hk <-
      subset(dat.rc.hk,
             dat.rc.hk$method.random.ci == "HK" &
             !(dat.rc.hk$method %in% c("GLMM", "LRP") | dat.rc.hk$three.level))
    #
    dat.rc.hk.tdist <-
      subset(dat.rc,
             dat.rc$method.random.ci == "HK" &
             (dat.rc$method %in% c("GLMM", "LRP") | dat.rc$three.level))
    #
    dat.rc.ckr <- subset(dat.rc, dat.rc$method.random.ci == "classic-KR")
    dat.rc.kr <- subset(dat.rc, dat.rc$method.random.ci == "KR")
    #
    more.ci <- sum(1L * (nrow(dat.rc.hk) > 0) +
                   1L * (nrow(dat.rc.hk.tdist) > 0) +
                   1L * (nrow(dat.rc.ckr) > 0) +
                   1L * (nrow(dat.rc.kr) > 0)) > 1
    #
    if (nrow(dat.rc.hk) > 0) {
      details <-
        paste0(
          details,
          "\n- Hartung-Knapp ",
          if (more.ci) "(HK) ",
          "adjustment for random effects model",
          if (print.df)
            paste0(" (df = ", cond(dat.rc.hk$df.random, digits = 0), ")")
        )
      #
      if (any(dat.rc.hk$adhoc.hakn.ci != ""))
        details <- paste0(
          details,
          if (forest) " " else "\n  ", "(with ",
          if (any(dat.rc.hk$adhoc.hakn.ci == ""))
            "and without ",
          "ad hoc correction)")
    }
    #
    if (nrow(dat.rc.hk.tdist) > 0) {
      details <-
        paste0(
          details,
          "\n- Random effects confidence interval based on t-distribution",
          if (more.ci) " (T)",
          if (print.df)
            paste0(" (df = ", cond(dat.rc.hk.tdist$df.random, digits = 0), ")")
        )
    }
    #
    if (nrow(dat.rc.ckr) > 0)
      details <-
        paste0(
          details,
          "\n- Classic method instead of Kenward-Roger adjustment ",
          if (more.ci) "(classic-KR) ",
          "used for random effects model")
    #
    if (nrow(dat.rc.kr) > 0)
      details <-
        paste0(
          details,
          "\n- Kenward-Roger ",
          if (more.ci) "(KR) ",
          "adjustment for random effects model",
          if (print.df)
            paste0(" (df = ", cond(dat.rc.kr$df.random), ")")
        )
  }
  
  
  #
  #
  # (8) Prediction interval
  #
  #

  more.pi <- NULL
  #
  if (!prediction & replaceNULL(prediction.subgroup, FALSE)) {
    prediction <- TRUE
    print.df <- FALSE
    more.pi <- TRUE
  }
  #
  if (prediction) {
    dat.pr <- unique(pred)
    #
    dat.pr.v <- subset(dat.pr, dat.pr$method.predict == "V")
    dat.pr.v.kr <- subset(dat.pr, dat.pr$method.predict == "V-KR")
    dat.pr.hts1 <- subset(dat.pr, dat.pr$method.predict == "HTS")
    dat.pr.hk <- subset(dat.pr, dat.pr$method.predict == "HK")
    dat.pr.hts2 <- subset(dat.pr, dat.pr$method.predict == "HTS-KR")
    dat.pr.kr <- subset(dat.pr, dat.pr$method.predict == "KR")
    dat.pr.nnf <- subset(dat.pr, dat.pr$method.predict == "NNF")
    dat.pr.s <- subset(dat.pr, dat.pr$method.predict == "S")
    #
    if (is.null(more.pi))
      more.pi <- sum(1L * (nrow(dat.pr.v) > 0) +
                       1L * (nrow(dat.pr.v.kr) > 0) +
                       1L * (nrow(dat.pr.hts1) > 0) +
                       1L * (nrow(dat.pr.hk) > 0) +
                       1L * (nrow(dat.pr.hts2) > 0) +
                       1L * (nrow(dat.pr.kr) > 0) +
                       1L * (nrow(dat.pr.nnf) > 0) +
                       1L * (nrow(dat.pr.s) > 0)) > 1
    #
    if (nrow(dat.pr.v) > 0)
      details <-
        paste0(
          details,
          "\n- Prediction interval based on t-distribution",
          if (more.pi) " (V)",
          if (print.df)
            paste0(" (df = ",
                   cond(dat.pr.v$df.predict, digits = 0),
                   ")")
          )
    #
    if (nrow(dat.pr.v.kr) > 0)
      details <-
      paste0(
        details,
        "\n- Prediction interval based on t-distribution",
        if (more.pi) " (V)",
        if (print.df)
          paste0(" (df = ",
                 cond(dat.pr.v.kr$df.predict, digits = 0),
                 ")"),
        " instead of Kenward-Roger adjustment")
    #
    if (nrow(dat.pr.hts1) > 0)
      details <-
      paste0(
        details,
        "\n- Prediction interval based on t-distribution",
        if (more.pi) " (HTS)",
        if (print.df)
          paste0(" (df = ",
                 cond(dat.pr.hts1$df.predict, digits = 0),
                 ")")
        )
    #
    if (nrow(dat.pr.hk) > 0) {
      details <-
        paste0(
          details,
          "\n- Hartung-Knapp",
          if (more.pi) " (HK)",
          " prediction interval",
          if (print.df)
            paste0(" (df = ",
                   cond(dat.pr.hk$df.predict, digits = 0),
                   ")")
        )
      #
      if (any(dat.pr.hk$adhoc.hakn.pi != ""))
        details <- paste0(
          details,
          if (forest) " " else "\n  ", "(with ",
          if (any(dat.pr.hk$adhoc.hakn.pi == ""))
            "and without ",
          "ad hoc correction)")
    }
    #
    if (nrow(dat.pr.hts2) > 0)
      details <-
        paste0(
          details,
          "\n- Prediction interval based on t-distribution",
          if (more.pi) " (HTS) ",
          if (print.df)
            paste0(" (df = ",
                   cond(dat.pr.hts2$df.predict, digits = 0),
                   ")"),
          " instead of Kenward-Roger adjustment")
    #
    if (nrow(dat.pr.kr) > 0)
      details <-
        paste0(
          details,
          "\n- Kenward-Roger",
          if (more.pi) " (KR)",
          " prediction interval",
          if (print.df)
            paste0(" (df = ",
                   cond(dat.pr.kr$df.predict),
                   ")")
        )
    #
    if (nrow(dat.pr.nnf) > 0)
      details <-
        paste0(
          details,
          "\n- Boot-strap prediction interval",
          if (more.pi) " (NNF)",
          if (print.df)
            paste0(" (df = ",
                   cond(dat.pr.nnf$df.predict, digits = 0),
                   ")")
        )
    #
    if (nrow(dat.pr.s) > 0)
      details <-
        paste0(
          details,
          "\n- Prediction interval based on standard normal distribution",
          if (more.pi) " (S)")
  }
  
  
  #
  #
  # (9) Trim-and-fill method
  #
  #

  if (overall & trimfill) {
    type <- unique(meth$type[meth$model %in% selmod])
    type <- type[type != ""]
    #
    if (length(type) > 0) {
      details <-
        paste0(details,
               "\n- Trim-and-fill method to adjust for funnel plot asymmetry",
               if (length(type) == 1 | forest)
                 " ("
               else
                 "\n  (",
               paste0(type, "-estimator", collapse = ", "), ")")
    }
  }
  
  
  #
  #
  # (10) R function metamiss()
  #
  #

  if (metamiss) {
    method.miss <- x$method.miss
    IMOR.e <- x$IMOR.e
    IMOR.c <- x$IMOR.c
    #
    if (method.miss == "IMOR") {
      mmiss <- "Informative Missingness Odds Ratio"
      if (length(unique(IMOR.e)) == 1 & length(unique(IMOR.c)) == 1)
        mmiss <-
          paste0(if (forest) " " else "\n  ", mmiss,
                 " (IMOR.e = ", round(unique(IMOR.e), 4),
                 ", IMOR.c = ", round(unique(IMOR.c), 4), ")")
      else
        mmiss <- paste(mmiss, "(IMOR)")
    }
    else {
      meths <- c("Gamble-Hollis analysis",
                 "impute no events (ICA-0)",
                 "impute events (ICA-1)",
                 "observed risk in control group (ICA-pc)",
                 "observed risk in experimental group (ICA-pe)",
                 "observed group-specific risks (ICA-p)",
                 "best-case scenario (ICA-b)",
                 "worst-case scenario (ICA-w)")
      #
      mm <- c("GH", "0", "1", "pc", "pe", "p", "b", "w")
      #
      idx <- charmatch(method.miss, mm)
      #
      mmiss <- meths[idx]
    }
    #
    details <- paste0(details, "\n- Imputation method: ", mmiss)
  }
  
  
  #
  #
  # (11) Information on effect measure
  #
  #
  
  sm <- x$sm
  #
  details <-
    paste0(details,
           if (sm == "ZCOR")
             "\n- Fisher's z transformation of correlations"
           else if (sm == "COR")
             "\n- Untransformed correlations"
           else if (sm == "PFT" | sm == "IRFT")
             "\n- Freeman-Tukey double arcsine transformation"
           else if (sm == "PAS")
             "\n- Arcsine transformation"
           else if (is_log_effect(sm))
             "\n- Log transformation"
           else if (sm == "PLOGIT")
             "\n- Logit transformation"
           else if (sm == "PRAW")
             "\n- Untransformed proportions"
           else if (sm == "IR")
             "\n- Untransformed rates"
           else if (sm == "IRS")
             "\n- Square root transformation"
           else if (sm == "MRAW")
             "\n- Untransformed (raw) means")
  #
  if (!is.null(func.transf))
    details <-
      paste0(details, "\n- User-specified transformation: ", func.transf)
  #
  if (backtransf & !is.null(func.backtransf))
    details <-
      paste0(details,
             "\n- User-specified back-transformation: ", func.backtransf)
  #
  # Add information on method for standardised mean difference
  #
  if (!is.null(meth$method.smd)) {
    dat.ms <-
      unique(meth[meth$model %in% selmod,
                  c("method.smd", "exact.smd", "sd.glass")])
    #
    for (i in seq_len(nrow(dat.ms))) {
      method.smd.i <- dat.ms$method.smd[i]
      exact.smd.i <- dat.ms$exact.smd[i]
      #
      if (method.smd.i == "Hedges")
        details <-
          paste0(details,
                 "\n- Hedges' g (bias corrected standardised mean ",
                 "difference",
                 if (exact.smd.i)
                   paste0(if (width >= 80 | forest) "; " else ";\n  ",
                          "using exact formulae)")
                 else
                   ")")
      else if (method.smd.i == "Cohen")
        details <-
          paste0(details,
                 "\n- Cohen's d (standardised mean difference",
                 if (exact.smd.i)
                   "; using exact formulae)"
                 else
                   ")")
      else if (method.smd.i == "Glass")
        details <-
          paste0(details,
                 "\n- Glass' delta (standardised mean difference; ",
                 if (dat.ms$sd.glass[i] == "control")
                   "based on second group)"
                 else
                   "based on first group)")
    }
  }
  
  
  #
  #
  # (12) Information on confidence interval for individual studies
  #
  #
  
  info.ci <- attr(x, ".print.study.results.")
  details.ci <- ""
  #
  if ((!is.null(info.ci) && info.ci) & !is.null(x$method.ci)) {
    if (any(x$k.all > 1)) {
      if (x$method.ci == "WSCC")
        fis <- paste0(if (forest) " " else "\n  ",
                      "for individual studies")
      else
        fis <- " for individual studies"
    }
    else
      fis <- ""
    #
    details.ci <-
      if (x$method.ci == "CP")
        paste0("\n- Clopper-Pearson confidence interval", fis)
      else if (x$method.ci == "WS")
        paste0("\n- Wilson Score confidence interval", fis)
      else if (x$method.ci == "WSCC")
        paste0("\n- Wilson Score confidence interval with ",
               "continuity correction", fis)
      else if (x$method.ci == "AC")
        paste0("\n- Agresti-Coull confidence interval", fis)
      else if (x$method.ci == "SA")
        paste0("\n- Simple approximation confidence interval", fis)
      else if (x$method.ci == "SACC")
        paste0("\n- Simple approximation confidence interval with ",
               "continuity correction", fis)
      else if (x$method.ci == "NAsm")
        paste0("\n- Normal approximation confidence interval", fis)
      else if (x$method.ci == "Poisson")
        paste0("\n- Exact Poisson confidence interval", fis)
      else if (x$method.ci == "t")
        paste0("\n- Confidence interval", fis, " based on t-distribution")
  }
  #
  details <- paste0(details, details.ci)
  
  
  #
  #
  # (13) Information on continuity correction
  #
  #
  
  if (metabin | metainc | metaprop | metarate) {
    vars <- c("method", "incr", "method.incr", "sparse", "k.all")
    #
    if (metabin)
      vars <- c(vars, "allstudies", "doublezeros", "MH.exact", "RR.Cochrane")
    #
    dat.cc <- unique(meth[meth$model %in% selmod, vars])
    #
    if (!(metabin & sm == "ASD")) {
      details.cc <- ""
      #
      for (i in seq_len(nrow(dat.cc))) {
        method.incr.i <- dat.cc$method.incr[i]
        if (!is.na(dat.cc$incr[i]))
          incr.i <- dat.cc$incr[i]
        else
          incr.i <- 0
        #
        sparse.i <- !is.na(dat.cc$sparse[i]) && dat.cc$sparse[i]
        k.all.i <- dat.cc$k.all[i]
        #
        details.rr <- NULL
        #
        if (metabin) {
          txtCC.ind.i <-
            (dat.cc$method[i] == "MH" & dat.cc$MH.exact[i]) |
            dat.cc$method[i] %in% c("GLMM", "LRP")
          #
          if (!is.na(dat.cc$RR.Cochrane[i]) &&
              dat.cc$RR.Cochrane[i] &
              (method.incr.i == "all" | (sparse.i & incr.i != 0))) {
            details.rr <-
              if (width >= 70 | forest)
                " (applied twice to sample sizes, like RevMan 5)"
              else
                "\n  (applied twice to sample sizes, like RevMan 5)"
          }
        }
        else
          txtCC.ind.i <- dat.cc$method[i] %in% c("GLMM", "LRP")
        #
        # No continuity correction for generalized linear mixed model and
        # argument 'method.ci != "NAsm"'
        #
        if (!(txtCC.ind.i &
              (!is.null(x$method.ci) && x$method.ci != "NAsm"))) {
          if (method.incr.i == "user") {
            if (incr.i != 0)
              details.cc <- "\n- User-defined continuity correction"
          }
          else  if (method.incr.i == "all") {
            if (incr.i == "TACC") {
              details.cc <- c(
                details.cc,
                if (k.all.i > 1)
                  "\n- Treatment arm continuity correction in all studies"
                else
                  "\n- Treatment arm continuity correction")
            }
            else if (incr.i != 0)
              details.cc <- c(
                details.cc,
                paste0("\n- Continuity correction of ",
                       incr.i,
                       if (k.all.i > 1)
                         " in all studies",
                       details.rr))
          }
          else if (sparse.i) {
            if (incr.i == "TACC") {
              details.cc <- c(
                details.cc,
                paste0(
                  "\n- Treatment arm continuity correction in ",
                  if (k.all.i > 1)
                    "studies "
                  else
                    "study ",
                  "with",
                  if (width > 70 | forest)
                    " "
                  else
                    "\n  ",
                  "zero cell frequencies",
                  details.rr
                ))
            }
            else if (incr.i != 0) {
              details.cc <- c(
                details.cc,
                paste0(
                  "\n- Continuity correction of ",
                  incr.i,
                  if (k.all.i > 1)
                    " in studies with",
                  if (k.all.i > 1 & width > 70)
                    " "
                  else if (k.all.i > 1 & !forest)
                    "\n  ",
                  if (k.all.i > 1)
                    "zero cell frequencies",
                  details.rr))
            }
            #
            if ((incr.i == "TACC" || incr.i != 0) && txtCC.ind.i)
              details.cc <- c(
                details.cc,
                if (forest) " " else "\n  ",
                "(only used to calculate individual study results)")
          }
        }
        #
        if (metabin & method.incr.i != "user") {
          if ((!is.na(dat.cc$allstudies[i]) && dat.cc$allstudies[i]) &
              (!is.na(dat.cc$doublezeros[i]) && dat.cc$doublezeros[i]))
            details.cc <- c(
              details.cc,
              if (k.all.i > 1)
                "\n- Studies with double zeros included in meta-analysis"
              else
                "\n- Study with double zeros considered")
        }
      }
      #
      details.cc <- paste0(unique(details.cc), collapse = "")
      details <- paste0(details, details.cc)
    }
  }
  
  
  #
  #
  # (14) Information on number of events and null hypothesis
  #
  #
  
  pscale <- x$pscale
  irscale <- x$irscale
  irunit <- x$irunit
  #
  if (!is.null(pscale) && pscale != 1)
    if (is_prop(sm) || sm == "RD")
      details <-
        paste0(details,
               "\n- Events per ",
               format(pscale, scientific = FALSE, big.mark = big.mark),
               " observations")
    else
      details <-
        paste0(details,
               "\n- Scaling factor for results: ",
               format(pscale, scientific = FALSE, big.mark = big.mark))   
  #
  if (!is.null(irscale) && irscale != 1)
    if (is_rate(sm) || sm == "IRD")
      details <-
        paste0(details,
               "\n- Events per ",
               format(irscale, scientific = FALSE, big.mark = big.mark),
               " ", irunit)
    else
      details <-
        paste0(details,
               "\n- Scaling factor for results: ",
               format(irscale, scientific = FALSE, big.mark = big.mark))   
  #
  null.effect <- x$null.effect
  #
  if (!is.na(null.effect) && (null.effect != 0 | metaprop | metarate)) {
    details <- paste0(details, "\n- Null hypothesis: effect is equal to ")
    #
    if (!is.null(pscale) && pscale != 1)
      details <-
        paste0(details,
               format(round(null.effect * pscale, digits), scientific = FALSE,
                      big.mark = big.mark),
               " events per ",
               format(pscale, scientific = FALSE, big.mark = big.mark),
               " observations")
    else if (!is.null(irscale) && irscale != 1)
      details <-
        paste0(details,
               format(round(null.effect * irscale, digits),
                      scientific = FALSE, big.mark = big.mark),
               " events per ",
               format(irscale, scientific = FALSE, big.mark = big.mark),
               " ", irunit)
    else
      details <- paste0(details,
                        format(null.effect, scientific = FALSE,
                               big.mark = big.mark))
  }
  
  
  if (!is.null(details) && length(details) > 0 && details != "") {
    details <-
      paste0("\nDetails",
             if ((common | random | prediction) && any(x$k.all > 1))
               " of meta-analysis methods",
             ":", details)
    #
    if (!forest)
      cat(paste0(details, "\n"))
  }
  
  invisible(details)
}


NULL

#' Print results of a cumulative meta-analysis
#' 
#' @description
#' Print results of a cumulative meta-analysis
#' 
#' @aliases print.metacum
#' 
#' @param x An object of class \code{\link{metacum}}.
#' @param prediction A logical indicating whether prediction intervals should
#'   be printed.
#' @param overall A logical indicating whether overall results should be
#'   printed.
#' @param backtransf A logical indicating whether printed results
#'   should be back transformed. If \code{backtransf=TRUE}, results
#'   for \code{sm="OR"} are printed as odds ratios rather than log
#'   odds ratios, for example.
#' @param header A logical indicating whether information on title of
#'   meta-analysis, comparison and outcome should be printed at the
#'   beginning of the printout.
#' @param lab.NA A character string to label missing values.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.stat Minimal number of significant digits for z- or
#'   t-value of test for overall effect, see \code{print.default}.
#' @param digits.pval Minimal number of significant digits for p-value
#'   of overall treatment effect, see \code{print.default}.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance \eqn{\tau^2}, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param digits.I2 Minimal number of significant digits for I-squared
#'   and Rb statistic, see \code{print.default}.
#' @param digits.cid Minimal number of significant digits for
#'   CID / decision thresholds, see \code{print.default}.
#' @param digits.percent Minimal number of significant digits for
#'   probabilities, printed as percentages, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param scientific.pval A logical specifying whether p-values should
#'   be printed in scientific notation, e.g., 1.2345e-01 instead of
#'   0.12345.
#' @param zero.pval A logical specifying whether p-values should be
#'   printed with a leading zero.
#' @param JAMA.pval A logical specifying whether p-values for test of
#'   overall effect should be printed according to JAMA reporting
#'   standards.
#' @param print.stat A logical value indicating whether z- or t-value
#'   for test of treatment effect should be printed.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau2.ci A logical value indicating whether to print
#'   the confidence interval of \eqn{\tau^2}.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param print.tau.ci A logical value indicating whether to print the
#'   confidence interval of \eqn{\tau}.
#' @param print.I2 A logical specifying whether heterogeneity
#'   statistic I\eqn{^2} should be printed.
#' @param print.I2.ci A logical specifying whether confidence interval for
#'   heterogeneity statistic I\eqn{^2} should be printed.
#' @param print.prob A logical specifying whether to print probabilities
#'   of clinically important benefit or harm.
#' @param text.tau2 Text printed to identify between-study variance
#'   \eqn{\tau^2}.
#' @param text.tau Text printed to identify \eqn{\tau}, the square
#'   root of the between-study variance \eqn{\tau^2}.
#' @param text.I2 Text printed to identify heterogeneity statistic
#'   I\eqn{^2}.
#' @param details.methods A logical specifying whether details on
#'   statistical methods should be printed.
#' @param \dots Additional arguments (ignored).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metacum}}, \code{\link{settings.meta}}
#' 
#' @examples
#' data(Fleiss1993bin)
#' ma <- metabin(d.asp, n.asp, d.plac, n.plac,
#'   data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
#' ma
#' metacum(ma)
#' metacum(ma, pooled = "random")
#' metacum(ma, pooled = "random", prediction = TRUE)
#'
#' @method print metacum
#' @export

print.metacum <- function(x,
                          #
                          prediction = x$prediction,
                          overall = x$overall,
                          backtransf = x$backtransf,
                          header = TRUE,
                          #
                          lab.NA = ".",
                          #
                          digits = gs("digits"),
                          digits.stat = gs("digits.stat"),
                          digits.pval = gs("digits.pval"),
                          digits.tau2 = gs("digits.tau2"),
                          digits.tau = gs("digits.tau"),
                          digits.I2 = gs("digits.I2"),
                          digits.cid = gs("digits.cid"),
                          digits.percent = 1,
                          #
                          big.mark = gs("big.mark"),
                          scientific.pval = gs("scientific.pval"),
                          zero.pval = gs("zero.pval"),
                          JAMA.pval = gs("JAMA.pval"),
                          #
                          print.stat = FALSE,
                          print.tau2 = TRUE,
                          print.tau2.ci = FALSE,
                          print.tau = TRUE,
                          print.tau.ci = FALSE,
                          print.I2 = TRUE,
                          print.I2.ci = FALSE,
                          print.prob = TRUE,
                          #
                          text.tau2 = gs("text.tau2"),
                          text.tau = gs("text.tau"),
                          text.I2 = gs("text.I2"),
                          #
                          details.methods = gs("details"),
                          ...) {
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  chkclass(x, c("metacum", "metainf"))
  x <- updateversion(x)
  #
  chklogical(prediction)
  chklogical(overall)
  chklogical(backtransf)
  chklogical(header)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.stat, min = 0, length = 1)
  chknumeric(digits.pval, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chknumeric(digits.I2, min = 0, length = 1)
  chknumeric(digits.cid, min = 0, length = 1)
  chknumeric(digits.percent, min = 0, length = 1)
  #
  chkchar(big.mark, length = 1)
  chklogical(scientific.pval)
  chklogical(zero.pval)
  chklogical(JAMA.pval)
  #
  chklogical(print.stat)
  chklogical(print.tau2)
  chklogical(print.tau2.ci)
  chklogical(print.tau)
  chklogical(print.tau.ci)
  #
  if (all(x$method == "LRP") & all(x$method.random == "LRP")) {
    print.tau2 <- FALSE
    print.tau <- FALSE
  }
  #
  chklogical(print.I2)
  chklogical(print.I2.ci)
  #
  chklogical(print.prob)
  #
  chklogical(details.methods)
  #
  print.tau2.ci <- print.tau2 & print.tau2.ci
  print.tau.ci <- print.tau & print.tau.ci
  print.I2.ci <- print.I2 & print.I2.ci
  #
  sm <- x$sm
  #
  if (is.function(x$func.backtransf))
    fbt <- deparse(substitute(x$func.backtransf))
  else
    fbt <- x$func.backtransf
  #
  abt <- x$args.backtransf
  
  
  #
  #
  # (3) Create data set
  #
  #
  
  # Get rid of warning 'no visible binding for global variable'
  #
  TE <- lower <- upper <- statistic <- pval <-
    tau2 <- lower.tau2 <- upper.tau2 <-
    tau <- lower.tau <- upper.tau <-
    I2 <- lower.I2 <- upper.I2 <-
    lower.predict <- upper.predict <-
    prop.cid.below.null <- prop.cid.above.null <-
    harmonic.mean <- n.harmonic.mean <- t.harmonic.mean <- NULL
  #
  dat <-
    with(x,
         data.frame(TE, lower, upper,
                    statistic, pval,
                    tau2, lower.tau2, upper.tau2,
                    tau, lower.tau, upper.tau,
                    I2, lower.I2, upper.I2,
                    lower.predict, upper.predict,
                    n.harmonic.mean,
                    t.harmonic.mean,
                    row.names = studlab))
  #
  avail.prop.cid.below.null <-
    !is.null(x$prop.cid.below.null) && !all(is.na(x$prop.cid.below.null))
  #
  avail.prop.cid.above.null <-
    !is.null(x$prop.cid.above.null) && !all(is.na(x$prop.cid.above.null))
  #
  avail.prop.cid <- avail.prop.cid.below.null | avail.prop.cid.above.null
  #
  if (avail.prop.cid.below.null)
    dat$prop.cid.below.null <- x$prop.cid.below.null
  else
    dat$prop.cid.below.null <- NA
  #
  if (avail.prop.cid.above.null)
    dat$prop.cid.above.null <- x$prop.cid.above.null
  else
    dat$prop.cid.above.null <- NA
  #
  if (overall) {
    space <- data.frame(TE = NA, lower = NA, upper = NA,
                        statistic = NA, pval = NA,
                        tau2 = NA, lower.tau2 = NA, upper.tau2 = NA,
                        tau = NA, lower.tau = NA, upper.tau = NA, 
                        I2 = NA, lower.I2 = NA, upper.I2 = NA,
                        lower.predict = NA, upper.predict = NA,
                        prop.cid.below.null = NA, prop.cid.above.null = NA,
                        n.harmonic.mean = NA, t.harmonic.mean = NA,
                        row.names = "")
    #
    dat.pooled <-
      with(x,
           data.frame(TE.pooled, lower.pooled, upper.pooled,
                      statistic.pooled, pval.pooled,
                      tau2.pooled, lower.tau2.pooled, upper.tau2.pooled,
                      tau.pooled, lower.tau.pooled, upper.tau.pooled,
                      I2.pooled, lower.I2.pooled, upper.I2.pooled,
                      lower.predict.pooled, upper.predict.pooled,
                      n.harmonic.mean.pooled, t.harmonic.mean.pooled)) %>%
      rename_with(~ gsub(".pooled", "", .x, fixed = TRUE))
    #
    rownames(dat.pooled) <- x$text.pooled
    #
    if (avail.prop.cid.below.null)
      dat.pooled$prop.cid.below.null <- x$prop.cid.below.null.pooled
    else
      dat.pooled$prop.cid.below.null <- NA
    #
    if (avail.prop.cid.above.null)
      dat.pooled$prop.cid.above.null <- x$prop.cid.above.null.pooled
    else
      dat.pooled$prop.cid.above.null <- NA
    #
    dat <- rbind(dat, space, dat.pooled)
  }
  #
  if (sm == "IRFT")
    dat %<>% rename(harmonic.mean = t.harmonic.mean) %>%
      select(-n.harmonic.mean)
  else
    dat %<>% rename(harmonic.mean = n.harmonic.mean) %>%
      select(-t.harmonic.mean)
  #
  if (backtransf) {
    dat %<>%
      mutate(
        TE = backtransf(TE, sm, harmonic.mean, harmonic.mean, fbt, abt),
        lower = backtransf(lower, sm, harmonic.mean, harmonic.mean, fbt, abt),
        upper = backtransf(upper, sm, harmonic.mean, harmonic.mean, fbt, abt),
        lower.predict =
          backtransf(lower.predict, sm, harmonic.mean, harmonic.mean, fbt, abt),
        upper.predict =
          backtransf(upper.predict, sm, harmonic.mean, harmonic.mean, fbt, abt),
      )    
  }
  #
  dat %<>% select(-harmonic.mean)
  #
  dat %<>%
    mutate(TE = formatN(TE, digits = digits, text.NA = lab.NA, big.mark = big.mark),
           lower = 
             if_else(is.na(lower) & is.na(upper), lab.NA,
                     formatCI(formatN(lower, digits = digits, text.NA = lab.NA,
                                      big.mark = big.mark),
                              formatN(upper, digits = digits, text.NA = lab.NA,
                                      big.mark = big.mark))),
           #
           statistic = formatN(statistic, digits = digits.stat, text.NA = lab.NA),
           #
           pval = formatPT(pval, digits = digits.pval, lab.NA = lab.NA,
                           scientific = scientific.pval,
                           zero = zero.pval, JAMA = JAMA.pval),
           #
           tau2 = formatPT(tau2, digits = digits.tau2, lab.NA = lab.NA,
                           big.mark = big.mark),
           lower.tau2 = 
             if_else(is.na(lower.tau2) & is.na(upper.tau2), lab.NA,
                     formatCI(formatPT(lower.tau2, digits = digits,
                                       lab.NA = lab.NA, big.mark = big.mark),
                              formatPT(upper.tau2, digits = digits,
                                       lab.NA = lab.NA, big.mark = big.mark))),
           #
           tau = formatPT(tau, digits = digits.tau, lab.NA = lab.NA,
                          big.mark = big.mark),
           lower.tau = 
             if_else(is.na(lower.tau) & is.na(upper.tau), lab.NA,
                     formatCI(formatPT(lower.tau, digits = digits,
                                       lab.NA = lab.NA, big.mark = big.mark),
                              formatPT(upper.tau, digits = digits,
                                       lab.NA = lab.NA, big.mark = big.mark))),
           #
           I2 = if_else(is.na(I2), lab.NA,
                        paste0(formatPT(100 * I2, digits = digits.I2,
                                        lab.NA = lab.NA), "%")),
           #
           lower.I2 =
             if_else(is.na(lower.I2), lab.NA,
                     paste0(formatPT(100 * lower.I2, digits = digits.I2,
                                     lab.NA = lab.NA), "%")),
           #
           upper.I2 =
             if_else(is.na(upper.I2), lab.NA,
                     paste0(formatPT(100 * upper.I2, digits = digits.I2,
                                     lab.NA = lab.NA), "%")),
           #
           lower.I2 =
             if_else(lower.I2 == lab.NA & upper.I2 == lab.NA, lab.NA,
                     formatCI(lower.I2, upper.I2)),
           #
           lower.predict = 
             if_else(is.na(lower.predict) & is.na(upper.predict), lab.NA,
                     formatCI(formatN(lower.predict, digits = digits,
                                      text.NA = lab.NA, big.mark = big.mark),
                              formatN(upper.predict, digits = digits,
                                      text.NA = lab.NA, big.mark = big.mark))),
           #
           prop.cid.below.null =
             if_else(is.na(prop.cid.below.null), lab.NA,
                     paste0(formatPT(100 * prop.cid.below.null,
                                     digits = digits.percent,
                                     lab.NA = lab.NA), "%")),
           #
           prop.cid.above.null =
             if_else(is.na(prop.cid.above.null), lab.NA,
                     paste0(formatPT(100 * prop.cid.above.null,
                                     digits = digits.percent,
                                     lab.NA = lab.NA), "%"))
    ) %>%
    select(-upper, -upper.predict, -upper.tau2, -upper.tau, -upper.I2)
  #
  # Use "" instead of lab.NA in empty row
  #
  sel.empty <- with(dat, TE == lab.NA & lower == lab.NA & I2 == lab.NA)
  #
  if (sum(sel.empty) > 0) {
    for (i in names(dat))
      dat[sel.empty, i] <- ""
  }
  #
  names(dat)[names(dat) == "TE"] <- smlab(sm, backtransf, x$pscale, x$irscale)
  #
  names(dat)[names(dat) == "lower"] <-
    paste0(round(100 * x$level.ma, 1), "%-CI")
  #
  if (prediction)
    names(dat)[names(dat) == "lower.predict"] <-
    paste0(round(100 * x$level.predict, 1), "%-PI")
  else
    dat$lower.predict <- NULL
  #
  if (all(is.na(c(x$pval, x$pval.pooled))))
    dat$pval <- NULL
  else
    names(dat)[names(dat) == "pval"] <- "p-value"
  #
  if (print.stat) {
    if (x$pooled == "random" & x$method.random.ci %in% c("HK", "KR"))
      names(dat)[names(dat) == "statistic"] <- "t"
    else
      names(dat)[names(dat) == "statistic"] <- "z"
  }
  else
    dat$statistic <- NULL
  #
  if (print.tau2)
    names(dat)[names(dat) == "tau2"] <- text.tau2
  else {
    dat$tau2 <- NULL
    dat$lower.tau2 <- NULL
  }
  #
  if (print.tau2.ci)
    names(dat)[names(dat) == "lower.tau2"] <-
    paste0(round(100 * x$level.hetstat, 1), "%-CI")
  else
    dat$lower.tau2 <- NULL
  #
  if (print.tau)
    names(dat)[names(dat) == "tau"] <- text.tau
  else
    dat$tau <- NULL
  #
  if (print.tau.ci)
    names(dat)[names(dat) == "lower.tau"] <-
    paste0(round(100 * x$level.hetstat, 1), "%-CI")
  else
    dat$lower.tau <- NULL
  #
  if (print.I2)
    names(dat)[names(dat) == "I2"] <- text.I2
  else
    dat$I2 <- NULL
  #
  if (print.I2.ci)
    names(dat)[names(dat) == "lower.I2"] <-
    paste0(round(100 * x$level.hetstat, 1), "%-CI")
  else
    dat$lower.I2 <- NULL
  #
  if (!avail.prop.cid.below.null)
    dat$prop.cid.below.null <- NULL
  #
  if (!avail.prop.cid.above.null)
    dat$prop.cid.above.null <- NULL
  #
  if (print.prob & avail.prop.cid.below.null) {
    names(dat)[names(dat) == "prop.cid.below.null"] <-
      paste0("P(",
             if (x$small.values == "desirable") "benefit" else "harm",
             ")")
  }
  else
    dat$prop.cid.below.null <- NULL
  #
  if (print.prob & avail.prop.cid.above.null) {
    names(dat)[names(dat) == "prop.cid.above.null"] <-
      paste0("P(",
             if (x$small.values == "desirable") "harm" else "benefit",
             ")")
  }
  else
    dat$prop.cid.above.null <- NULL
  #
  dat <- replaceNA(dat, "")
  
  
  #
  #
  # (4) Print results
  #
  #
  
  if (header)
    crtitle(x)
  #
  cat(paste0(if (inherits(x, "metacum")) "Cumulative " else "Leave-one-out ",
             "meta-analysis\n\n"))
  #
  prmatrix(dat, quote = FALSE, right = TRUE, ...)
  #
  if (details.methods) {
    details <-
      catmeth(x,
              x$pooled == "common", x$pooled == "random", prediction,
              overall, TRUE,
              #
              func.transf = x$func.transf,
              backtransf = backtransf,
              func.backtransf = x$func.backtransf,
              #
              big.mark = big.mark, digits = digits,
              digits.tau = digits.tau,
              text.tau = text.tau, text.tau2 = text.tau2,
              #
              print.tau2 = print.tau2,
              print.tau2.ci = print.tau2.ci,
              print.tau = print.tau,
              print.tau.ci = print.tau.ci,
              #
              print.I2 = print.I2, text.I2 = text.I2,
              #
              print.df = TRUE, prediction.subgroup = FALSE)
    #
    if (avail.prop.cid)
      svd <- x$small.values == "desirable"
    #
    if (avail.prop.cid.below.null)
      cat(paste0("- Lower decision threshold (",
                 if (svd) "beneficial " else "harmful ",
                 "effects): ",
                 formatN(x$cid.below.null, digits = digits.cid, big.mark = big.mark),
                 "\n"))
    #
    if (avail.prop.cid.above.null)
      cat(paste0("- Upper decision threshold (",
                 if (svd) "harmful " else "beneficial ",
                 "effects): ",
                 formatN(x$cid.above.null, digits = digits.cid, big.mark = big.mark),
                 "\n"))
  }
  #
  invisible(NULL)
}

kenwardroger <- function(w.random) {
  #
  # Kenward-Roger method for meta-analysis
  # (Partlett & Riley, 2017, Stat Med)
  #
  w1p <- sum(w.random)
  w2p <- sum(w.random^2)
  w3p <- sum(w.random^3)
  #
  IE <- w2p * 0.5 - w3p / w1p + 0.5 * (w2p / w1p)^2
  var <- (1 + 2 * (w3p / w1p - (w2p / w1p)^2) / IE) / w1p
  df <- 2 * IE / (var * w2p)^2
  #
  res <- list(se = if (!is.na(var) && var >= 0) sqrt(var) else NA, df = df,
              IE = IE, w1p = w1p, w2p = w2p, w3p = w3p)
  res
}

subgroup <- function(x, tau.preset = NULL, subgroup.rma,
                     seed = NULL, ...) {
  
  subgroup <- x$subgroup
  method.predict <-
    replaceVal(x$method.predict, "", gs("method.predict"))
  #
  levs <- bylevs(subgroup)
  n.levs <- length(levs)
  #
  methci <- paste(x$method.random.ci,
                  toupper(substring(x$adhoc.hakn.ci, 1, 2)),
                  sep = "-")
  methci <- gsub("-$", "", methci)
  #
  methpi <- paste(method.predict,
                  toupper(substring(x$adhoc.hakn.pi, 1, 2)),
                  sep = "-")
  methpi <- gsub("-$", "", methpi)
  #
  if (!(length(subgroup) > 0)) {
    warning("Argument 'subgroup' is missing.")
    return(NULL)
  }
  #
  if (!is.null(seed) & any(method.predict %in% "NNF")) {
    if (length(seed) == 1)
      seed <- rep_len(seed, n.levs)
    chknumeric(seed, length = n.levs, name = "seed.predict.subgroup")
  }
  #
  bin  <- inherits(x, "metabin")
  cont <- inherits(x, "metacont")
  cor  <- inherits(x, "metacor")
  gen  <- inherits(x, "metagen")
  mean <- inherits(x, "metamean")
  inc  <- inherits(x, "metainc")
  prop <- inherits(x, "metaprop")
  rate <- inherits(x, "metarate")
  #
  bin.cont.gen <- bin | cont | gen
  bin.inc <- bin | inc
  cor.prop.mean.rate <- cor | prop | mean | rate
  #
  is.glmm <- x$method == "GLMM"
  is.mlm <- !is.null(x$three.level) && x$three.level
  #
  sumNA <- function(x, exclude = NULL) {
    if (all(is.na(x)))
      return(NA)
    else {
      if (is.null(exclude))
        return(sum(x, na.rm = TRUE))
      else
        return(sum(x[!exclude], na.rm = TRUE))
    }
  }
  
  
  #
  #
  # (1) Subgroup analysis without common tau2
  #
  #
  res.i <- vector(mode = "list")
  j <- 0
  #
  for (i in levs) {
    j <- j + 1
    sel <- subgroup == i
    #
    if (all(is.na(x$studlab[sel])))
      stop("No data available for subgroup = ", i)
    #
    if (!is.null(seed))
      seed.i <- seed[j]
    else
      seed.i <- NULL
    #
    if (bin) {
      if (x$method.incr == "user") {
        incr <- NULL
        incr.e <- x$incr.e[sel]
        incr.c <- x$incr.c[sel]
      }
      else {
        incr <- x$incr
        if (length(incr) > 1)
          incr <- incr[sel]
        #
        incr.e <- NULL
        incr.c <- NULL
      }
      #
      meta1 <- metabin(x$event.e[sel], x$n.e[sel],
                       x$event.c[sel], x$n.c[sel],
                       studlab = x$studlab[sel],
                       exclude = x$exclude[sel],
                       cluster =
                         if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                       rho = x$rho,
                       #
                       method = x$method,
                       sm = x$sm,
                       #
                       incr = incr,
                       incr.e = incr.e, incr.c = incr.c,
                       method.incr = x$method.incr, allstudies = x$allstudies,
                       #
                       MH.exact = x$MH.exact,
                       RR.Cochrane = x$RR.Cochrane,
                       Q.Cochrane = x$Q.Cochrane,
                       model.glmm = x$model.glmm,
                       #
                       level.ma = x$level.ma,
                       method.common.ci = x$method.common.ci,
                       method.random.ci = x$method.random.ci,
                       adhoc.hakn.ci = x$adhoc.hakn.ci,
                       #
                       level.predict = x$level.predict,
                       method.predict = method.predict,
                       adhoc.hakn.pi = x$adhoc.hakn.pi,
                       seed.predict = seed.i,
                       #
                       method.tau = x$method.tau,
                       method.tau.ci = x$method.tau.ci,
                       level.hetstat = x$level.hetstat,
                       tau.preset = tau.preset,
                       TE.tau = x$TE.tau,
                       #
                       keepdata = FALSE,
                       warn = x$warn,
                       control = x$control)
    }
    #
    else if (cont)
      meta1 <- metacont(x$n.e[sel], x$mean.e[sel],
                        x$sd.e[sel],
                        x$n.c[sel], x$mean.c[sel],
                        x$sd.c[sel],
                        studlab = x$studlab[sel],
                        exclude = x$exclude[sel],
                        cluster =
                          if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                        rho = x$rho,
                        #
                        median.e = subsetVar(x$median.e, sel),
                        q1.e = subsetVar(x$q1.e, sel),
                        q3.e = subsetVar(x$q3.e, sel),
                        min.e = subsetVar(x$min.e, sel),
                        max.e = subsetVar(x$max.e, sel),
                        #
                        median.c = subsetVar(x$median.c, sel),
                        q1.c = subsetVar(x$q1.c, sel),
                        q3.c = subsetVar(x$q3.c, sel),
                        min.c = subsetVar(x$min.c, sel),
                        max.c = subsetVar(x$max.c, sel),
                        #
                        method.mean = subsetVar(x$method.mean, sel),
                        method.sd = subsetVar(x$method.sd, sel),
                        #
                        approx.mean.e = subsetVar(x$approx.mean.e, sel),
                        approx.sd.e = subsetVar(x$approx.sd.e, sel),
                        #
                        approx.mean.c = subsetVar(x$approx.mean.c, sel),
                        approx.sd.c = subsetVar(x$approx.sd.c, sel),
                        #
                        sm = x$sm,
                        #
                        pooledvar = x$pooledvar,
                        method.smd = x$method.smd,
                        sd.glass = x$sd.glass,
                        exact.smd = x$exact.smd,
                        #
                        level.ma = x$level.ma,
                        method.common.ci = x$method.common.ci,
                        method.random.ci = x$method.random.ci,
                        adhoc.hakn.ci = x$adhoc.hakn.ci,
                        #
                        level.predict = x$level.predict,
                        method.predict = method.predict,
                        adhoc.hakn.pi = x$adhoc.hakn.pi,
                        seed.predict = seed.i,
                        #
                        method.tau = x$method.tau,
                        method.tau.ci = x$method.tau.ci,
                        level.hetstat = x$level.hetstat,
                        tau.preset = tau.preset,
                        TE.tau = x$TE.tau,
                        #
                        keepdata = FALSE,
                        warn = x$warn,
                        control = x$control)
    #
    else if (cor)
      meta1 <- metacor(x$cor[sel], x$n[sel],
                       studlab = x$studlab[sel],
                       exclude = x$exclude[sel],
                       cluster =
                         if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                       rho = x$rho,
                       #
                       sm = x$sm,
                       #
                       level.ma = x$level.ma,
                       method.common.ci = x$method.common.ci,
                       method.random.ci = x$method.random.ci,
                       adhoc.hakn.ci = x$adhoc.hakn.ci,
                       #
                       level.predict = x$level.predict,
                       method.predict = method.predict,
                       adhoc.hakn.pi = x$adhoc.hakn.pi,
                       seed.predict = seed.i,
                       #
                       method.tau = x$method.tau,
                       method.tau.ci = x$method.tau.ci,
                       level.hetstat = x$level.hetstat,
                       tau.preset = tau.preset,
                       TE.tau = x$TE.tau,
                       #
                       null.effect = x$null.effect,
                       #
                       keepdata = FALSE,
                       control = x$control)
    #
    else if (gen)
      meta1 <- metagen(x$TE[sel], x$seTE[sel],
                       studlab = x$studlab[sel],
                       exclude = x$exclude[sel],
                       cluster =
                         if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                       rho = x$rho,
                       #
                       weights.common =
                         if (!is.null(x$weights.common)) x$weights.common[sel] else NULL,
                       #
                       weights.random =
                         if (!is.null(x$weights.random)) x$weights.random[sel] else NULL,
                       #
                       sm = x$sm,
                       #
                       level.ma = x$level.ma,
                       method.common.ci = x$method.common.ci,
                       method.random.ci = x$method.random.ci,
                       adhoc.hakn.ci = x$adhoc.hakn.ci,
                       #
                       level.predict = x$level.predict,
                       method.predict = method.predict,
                       adhoc.hakn.pi = x$adhoc.hakn.pi,
                       seed.predict = seed.i,
                       #
                       method.tau = x$method.tau,
                       method.tau.ci = x$method.tau.ci,
                       level.hetstat = x$level.hetstat,
                       tau.preset = tau.preset,
                       TE.tau = x$TE.tau,
                       #
                       null.effect = x$null.effect,
                       n.e = x$n.e[sel], n.c = x$n.c[sel],
                       #
                       keepdata = FALSE,
                       warn = x$warn,
                       control = x$control)
    #
    else if (inc) {
      if (x$method.incr == "user") {
        incr <- NULL
        incr.e <- x$incr.e[sel]
        incr.c <- x$incr.c[sel]
      }
      else {
        incr <- x$incr
        if (length(incr) > 1)
          incr <- incr[sel]
        #
        incr.e <- NULL
        incr.c <- NULL
      }
      #
      meta1 <- metainc(x$event.e[sel], x$time.e[sel],
                       x$event.c[sel], x$time.c[sel],
                       studlab = x$studlab[sel],
                       exclude = x$exclude[sel],
                       cluster =
                         if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                       rho = x$rho,
                       #
                       method = x$method,
                       sm = x$sm,
                       #
                       incr = incr,
                       incr.e = incr.e, incr.c = incr.c,
                       method.incr = x$method.incr,
                       #
                       model.glmm = x$model.glmm,
                       #
                       level.ma = x$level.ma,
                       method.common.ci = x$method.common.ci,
                       method.random.ci = x$method.random.ci,
                       adhoc.hakn.ci = x$adhoc.hakn.ci,
                       #
                       level.predict = x$level.predict,
                       method.predict = method.predict,
                       adhoc.hakn.pi = x$adhoc.hakn.pi,
                       seed.predict = seed.i,
                       #
                       method.tau = x$method.tau,
                       method.tau.ci = x$method.tau.ci,
                       level.hetstat = x$level.hetstat,
                       tau.preset = tau.preset,
                       TE.tau = x$TE.tau,
                       #
                       n.e = x$n.e[sel], n.c = x$n.c[sel],
                       #
                       keepdata = FALSE,
                       warn = x$warn,
                       control = x$control)
    }
    #
    else if (mean)
      meta1 <- metamean(x$n[sel], x$mean[sel], x$sd[sel],
                        studlab = x$studlab[sel],
                        exclude = x$exclude[sel],
                        cluster = subsetVar(x$cluster, sel),
                        rho = x$rho,
                        #
                        median = subsetVar(x$median, sel),
                        q1 = subsetVar(x$q1, sel),
                        q3 = subsetVar(x$q3, sel),
                        min = subsetVar(x$min, sel),
                        max = subsetVar(x$max, sel),
                        method.mean = subsetVar(x$method.mean, sel),
                        method.sd = subsetVar(x$method.sd, sel),
                        approx.mean = subsetVar(x$approx.mean, sel),
                        approx.sd = subsetVar(x$approx.sd, sel),
                        #
                        sm = x$sm,
                        #
                        level.ma = x$level.ma,
                        method.common.ci = x$method.common.ci,
                        method.random.ci = x$method.random.ci,
                        adhoc.hakn.ci = x$adhoc.hakn.ci,
                        #
                        level.predict = x$level.predict,
                        method.predict = method.predict,
                        adhoc.hakn.pi = x$adhoc.hakn.pi,
                        seed.predict = seed.i,
                        #
                        method.tau = x$method.tau,
                        method.tau.ci = x$method.tau.ci,
                        level.hetstat = x$level.hetstat,
                        tau.preset = tau.preset,
                        TE.tau = x$TE.tau,
                        #
                        null.effect = x$null.effect,
                        #
                        keepdata = FALSE,
                        warn = x$warn,
                        control = x$control)
    #
    else if (prop)
      meta1 <- metaprop(x$event[sel], x$n[sel],
                        studlab = x$studlab[sel],
                        exclude = x$exclude[sel],
                        cluster =
                          if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                        rho = x$rho,
                        #
                        method = x$method,
                        sm = x$sm,
                        incr = if (length(x$incr) == 1) x$incr else x$incr[sel],
                        method.incr = x$method.incr,
                        #
                        level.ma = x$level.ma,
                        method.common.ci = x$method.common.ci,
                        method.random.ci = x$method.random.ci,
                        adhoc.hakn.ci = x$adhoc.hakn.ci,
                        #
                        level.predict = x$level.predict,
                        method.predict = method.predict,
                        adhoc.hakn.pi = x$adhoc.hakn.pi,
                        seed.predict = seed.i,
                        #
                        method.tau = x$method.tau,
                        method.tau.ci = x$method.tau.ci,
                        level.hetstat = x$level.hetstat,
                        tau.preset = tau.preset,
                        TE.tau = x$TE.tau,
                        #
                        null.effect = x$null.effect,
                        #
                        keepdata = FALSE,
                        warn = x$warn,
                        control = x$control)
    #
    else if (rate)
      meta1 <- metarate(x$event[sel], x$time[sel],
                        studlab = x$studlab[sel],
                        exclude = x$exclude[sel],
                        cluster =
                          if (!is.null(x$cluster)) x$cluster[sel] else NULL,
                        rho = x$rho,
                        #
                        method = x$method,
                        sm = x$sm,
                        incr = if (length(x$incr) == 1) x$incr else x$incr[sel],
                        method.incr = x$method.incr,
                        #
                        level.ma = x$level.ma,
                        method.common.ci = x$method.common.ci,
                        method.random.ci = x$method.random.ci,
                        adhoc.hakn.ci = x$adhoc.hakn.ci,
                        #
                        level.predict = x$level.predict,
                        method.predict = method.predict,
                        adhoc.hakn.pi = x$adhoc.hakn.pi,
                        seed.predict = seed.i,
                        #
                        method.tau = x$method.tau,
                        method.tau.ci = x$method.tau.ci,
                        level.hetstat = x$level.hetstat,
                        tau.preset = tau.preset,
                        TE.tau = x$TE.tau,
                        #
                        n = if (!is.null(x$n)) x$n[sel] else NULL,
                        #
                        null.effect = x$null.effect,
                        #
                        keepdata = FALSE,
                        warn = x$warn,
                        control = x$control)
    #
    else
      stop("No meta-analysis object used for subgroup analysis.")
    #
    if (length(meta1$TE.random) == 1 &&
        length(meta1$TE.random) != length(meta1$seTE.random))
      meta1$TE.random <- rep_len(meta1$TE.random, length(meta1$seTE.random))
    #
    res.i[[j]] <- list(k = meta1$k,
                       k.study = meta1$k.study,
                       k.all = meta1$k.all,
                       k.TE = meta1$k.TE,
                       #
                       TE.common = meta1$TE.common,
                       seTE.common = meta1$seTE.common,
                       statistic.common = meta1$statistic.common,
                       pval.common = meta1$pval.common,
                       lower.common = meta1$lower.common,
                       upper.common = meta1$upper.common,
                       w.common = sum(x$w.common[sel]),
                       #
                       TE.random = meta1$TE.random,
                       seTE.random = meta1$seTE.random,
                       statistic.random = meta1$statistic.random,
                       pval.random = meta1$pval.random,
                       df.random = meta1$df.random,
                       lower.random = meta1$lower.random,
                       upper.random = meta1$upper.random,
                       w.random = sum(x$w.random[sel]),
                       #
                       seTE.classic = meta1$seTE.classic,
                       #
                       df.hakn.ci = meta1$df.hakn.ci,
                       seTE.hakn.ci = meta1$seTE.hakn.ci,
                       seTE.hakn.adhoc.ci = meta1$seTE.hakn.adhoc.ci,
                       #
                       df.kero = meta1$df.kero,
                       seTE.kero = meta1$seTE.kero,
                       #
                       seTE.predict = meta1$seTE.predict,
                       df.predict = meta1$df.predict,
                       lower.predict = meta1$lower.predict,
                       upper.predict = meta1$upper.predict,
                       #
                       seTE.hakn.pi = meta1$seTE.hakn.pi,
                       seTE.hakn.adhoc.pi = meta1$seTE.hakn.adhoc.pi,
                       #
                       Q = meta1$Q,
                       #
                       tau2 = sum(meta1$tau2),
                       lower.tau2 =
                         if (meta1$three.level) NA else meta1$lower.tau2,
                       upper.tau2 =
                         if (meta1$three.level) NA else meta1$upper.tau2,
                       #
                       tau = sum(meta1$tau),
                       lower.tau =
                         if (meta1$three.level) NA else meta1$lower.tau,
                       upper.tau =
                         if (meta1$three.level) NA else meta1$upper.tau,
                       #
                       H = meta1$H,
                       lower.H = meta1$lower.H,
                       upper.H = meta1$upper.H,
                       #
                       I2 = meta1$I2,
                       lower.I2 = meta1$lower.I2,
                       upper.I2 = meta1$upper.I2,
                       #
                       Rb = meta1$Rb,
                       lower.Rb = meta1$lower.Rb,
                       upper.Rb = meta1$upper.Rb,
                       #
                       event =
                         if (prop) sumNA(meta1$event, meta1$exclude) else NA,
                       n =
                         if (cor.prop.mean.rate) sumNA(meta1$n, meta1$exclude) else NA,
                       #
                       event.e =
                         if (bin.inc) sumNA(meta1$event.e, meta1$exclude) else NA,
                       n.e =
                         if (bin.cont.gen) sumNA(meta1$n.e, meta1$exclude) else NA,
                       event.c =
                         if (bin.inc) sumNA(meta1$event.c, meta1$exclude) else NA,
                       n.c =
                         if (bin.cont.gen) sumNA(meta1$n.c, meta1$exclude) else NA,
                       #
                       time.e =
                         if (inc) sumNA(meta1$time.e, meta1$exclude) else NA,
                       time.c =
                         if (inc) sumNA(meta1$time.c, meta1$exclude) else NA,
                       #
                       n.harmonic.mean = 1 / mean(1 / x$n[sel]),
                       t.harmonic.mean = 1 / mean(1 / x$time[sel]))
  }
  #
  k.w <- extrVec(res.i, "k", levs)
  k.study.w <- extrVec(res.i, "k.study", levs)
  k.all.w <- extrVec(res.i, "k.all", levs)
  k.TE.w <- extrVec(res.i, "k.TE", levs)
  #
  TE.common.w <- extrVec(res.i, "TE.common", levs)
  seTE.common.w <- extrVec(res.i, "seTE.common", levs)
  statistic.common.w <- extrVec(res.i, "statistic.common", levs)
  pval.common.w <- extrVec(res.i, "pval.common", levs)
  lower.common.w <- extrVec(res.i, "lower.common", levs)
  upper.common.w <- extrVec(res.i, "upper.common", levs)
  w.common.w <- extrVec(res.i, "w.common", levs)
  #
  TE.random.w <- extrMat(res.i, "TE.random", levs, methci)    
  seTE.random.w <- extrMat(res.i, "seTE.random", levs, methci)
  if (is.matrix(TE.random.w)) {
    TE.random.w <- as.vector(TE.random.w[, 1])
    names(TE.random.w) <- rownames(seTE.random.w)
  }
  statistic.random.w <- extrMat(res.i, "statistic.random", levs, methci)
  pval.random.w <- extrMat(res.i, "pval.random", levs, methci)
  df.random.w <- extrMat(res.i, "df.random", levs, methci)
  lower.random.w <- extrMat(res.i, "lower.random", levs, methci)
  upper.random.w <- extrMat(res.i, "upper.random", levs, methci)
  w.random.w <- extrVec(res.i, "w.random", levs)
  #
  seTE.classic.w <- extrVec(res.i, "seTE.classic", levs)
  #
  df.hakn.ci.w <- extrMat(res.i, "df.hakn.ci", levs, methci)
  seTE.hakn.ci.w <- extrVec(res.i, "seTE.hakn.ci", levs)
  seTE.hakn.adhoc.ci.w <- extrMat(res.i, "seTE.hakn.adhoc.ci", levs, methci)
  #
  df.kero.w <- extrVec(res.i, "df.kero", levs)
  seTE.kero.w <- extrVec(res.i, "seTE.kero", levs)
  #
  seTE.predict.w <- extrMat(res.i, "seTE.predict", levs, methpi)
  df.predict.w <- extrMat(res.i, "df.predict", levs, methpi)
  lower.predict.w <- extrMat(res.i, "lower.predict", levs, methpi)
  upper.predict.w <- extrMat(res.i, "upper.predict", levs, methpi)
  #
  seTE.hakn.pi.w <- extrVec(res.i, "seTE.hakn.pi", levs)
  seTE.hakn.adhoc.pi.w <- extrMat(res.i, "seTE.hakn.adhoc.pi", levs, methpi)
  #
  Q.w <- extrVec(res.i, "Q", levs, TRUE)
  #
  tau2.w <- extrVec(res.i, "tau2", levs)
  lower.tau2.w <- extrVec(res.i, "lower.tau2", levs)
  upper.tau2.w <- extrVec(res.i, "upper.tau2", levs)
  #
  tau.w <- extrVec(res.i, "tau", levs)
  lower.tau.w <- extrVec(res.i, "lower.tau", levs)
  upper.tau.w <- extrVec(res.i, "upper.tau", levs)
  #
  H.w <- extrVec(res.i, "H", levs)
  lower.H.w <- extrVec(res.i, "lower.H", levs)
  upper.H.w <- extrVec(res.i, "upper.H", levs)
  #
  I2.w <- extrVec(res.i, "I2", levs)
  lower.I2.w <- extrVec(res.i, "lower.I2", levs)
  upper.I2.w <- extrVec(res.i, "upper.I2", levs)
  #
  Rb.w <- extrVec(res.i, "Rb", levs)
  lower.Rb.w <- extrVec(res.i, "lower.Rb", levs)
  upper.Rb.w <- extrVec(res.i, "upper.Rb", levs)
  #
  event.w <- extrVec(res.i, "event", levs)
  n.w <- extrVec(res.i, "n", levs)
  #
  event.e.w <- extrVec(res.i, "event.e", levs)
  n.e.w <- extrVec(res.i, "n.e", levs)
  event.c.w <- extrVec(res.i, "event.c", levs)
  n.c.w <- extrVec(res.i, "n.c", levs)
  #
  time.e.w <- extrVec(res.i, "time.e", levs)
  time.c.w <- extrVec(res.i, "time.c", levs)
  #
  n.harmonic.mean.w <- extrVec(res.i, "n.harmonic.mean", levs)
  t.harmonic.mean.w <- extrVec(res.i, "t.harmonic.mean", levs)
  #
  # Tests for subgroup differences
  #
  Q.w.common <- sum(Q.w, na.rm = TRUE)
  df.Q.w <- sum((k.w - 1)[!is.na(Q.w)])
  pval.Q.w.common <- pvalQ(Q.w.common, df.Q.w)
  #
  Q.b.common <- metagen(TE.common.w, seTE.common.w, method.tau = "DL")$Q
  #
  if (is.matrix(seTE.random.w)) {
    n.random <- ncol(seTE.random.w)
    Q.b.random <- rep(NA, n.random)
    names(Q.b.random) <- colnames(seTE.random.w)
    for (i in seq_len(n.random))
      Q.b.random[i] <-
        metagen(TE.random.w,
                lower = lower.random.w[, i],
                upper = upper.random.w[, i],
                method.tau = "DL")$Q
  }
  else {
    n.random <- 1
    Q.b.random <-
      metagen(TE.random.w, seTE.random.w, method.tau = "DL")$Q
  }
  #
  df.Q.b <- NA
  df.Q.b.common <- NA
  df.Q.b.random <- list()
  #
  pval.Q.b.common <- NA
  pval.Q.b.random <- NA
  #
  if (missing(subgroup.rma)) {
    df.Q.b <- if (x$k == 0) 0 else x$k - 1 - sum((k.w - 1)[!is.na(Q.w)])
    df.Q.b.common <- df.Q.b
    pval.Q.b.common <- pvalQ(Q.b.common, df.Q.b.common)
    #
    if (n.random > 1) {
      pval.Q.b.random <- vector("numeric", n.random)
      df.Q.b.random <- vector("list", n.random)
      for (i in seq_len(n.random)) {
        df.Q.b.random[[i]] <- df.Q.b
        pval.Q.b.random[i] <- pvalQ(Q.b.random[i], df.Q.b.random[[i]])
      }
      #
      names(df.Q.b.random) <- names(pval.Q.b.random) <-
        colnames(seTE.random.w)
    }
    else {
      df.Q.b.random <- df.Q.b
      pval.Q.b.random <- pvalQ(Q.b.random, df.Q.b.random)
    }
  }
  
  
  #
  #
  # (2) Subgroup analysis with common tau-squared (three-level model)
  #
  #
  
  if (!missing(subgroup.rma) && is.mlm) {
    n.methci <- length(x$method.random.ci)
    #
    mod <- as.call(~ subgroup.rma - 1)
    mod.Q <- as.call(~ subgroup.rma)
    #
    if (is.null(x$exclude))
      x$exclude <- rep(FALSE, length(x$TE))
    #
    list.mlm <- list(yi = x$TE[!x$exclude],
                     V = vcalc(vi = x$seTE[!x$exclude]^2,
                               cluster = x$cluster[!x$exclude],
                               obs = seq_along(x$cluster)[!x$exclude],
                               rho = replaceNULL(x$rho, 0)))
    #
    mlm <-
      runMLM(c(list.mlm,
               list(data = 
                      data.frame(subgroup.rma = subgroup.rma,
                                 cluster = x$cluster[!x$exclude],
                                 idx = seq_along(x$cluster)[!x$exclude]))),
             method.tau = x$method.tau,
             method.random.ci = x$method.random.ci,
             level = x$level.ma,
             mods = mod,
             control = list(x$control),
             warn = FALSE)
    #
    # Random effects model
    #
    sel.r <- !is.na(TE.random.w)
    TE.random.w[sel.r] <- as.numeric(mlm[[1]]$b)
    seTE.random.w[sel.r] <- as.numeric(mlm[[1]]$se)
    #
    if (n.methci == 1) {
      lower.random.w[sel.r] <- as.numeric(mlm[[1]]$ci.lb)
      upper.random.w[sel.r] <- as.numeric(mlm[[1]]$ci.ub)
      statistic.random.w[sel.r] <- as.numeric(mlm[[1]]$zval)
      pval.random.w[sel.r] <- as.numeric(mlm[[1]]$pval)
    }
    else {
      for (i in seq_len(n.methci)) {
        lower.random.w[sel.r, i] <- as.numeric(mlm[[i]]$ci.lb)
        upper.random.w[sel.r, i] <- as.numeric(mlm[[i]]$ci.ub)
        statistic.random.w[sel.r, i] <- as.numeric(mlm[[i]]$zval)
        pval.random.w[sel.r, i] <- as.numeric(mlm[[i]]$pval)
      }
    }
    #
    # Heterogeneity measures
    #
    tau2.w <- rep_len(sum(mlm[[1]]$sigma2), n.levs)
    lower.tau2.w <- tau2.w
    upper.tau2.w <- tau2.w
    lower.tau2.w[!is.na(lower.tau2.w)] <- NA
    upper.tau2.w[!is.na(upper.tau2.w)] <- NA
    #
    tau.w <- sqrt(tau2.w)
    lower.tau.w <- tau.w
    upper.tau.w <- tau.w
    lower.tau.w[!is.na(lower.tau.w)] <- NA
    upper.tau.w[!is.na(upper.tau.w)] <- NA
    #
    upper.Rb.w <- lower.Rb.w <- Rb.w <- rep_len(NA, n.levs)
    #
    # Prediction interval
    #
    tau2.calc <- if (is.na(sum(mlm[[1]]$sigma2))) 0 else tau2.w
    seTE.predict.w <- sqrt(seTE.random.w^2 + tau2.w)
    #
    n.methpi <- length(method.predict)
    #
    for (i in seq_len(n.methpi)) {
      df.i <- set_df_predict(method.predict[i], k.w)
      #
      ci.p <- ci(TE.random.w, seTE.predict.w, x$level.predict, df = df.i)
      #
      if (n.methpi == 1) {
        lower.predict.w <- ci.p$lower
        upper.predict.w <- ci.p$upper
      }
      else {
        lower.predict.w[, i] <- ci.p$lower
        upper.predict.w[, i] <- ci.p$upper
      }
      #
      sel.p <-
        !(method.predict[i] == "V" & k.w >= 2 |
            method.predict[i] == "HTS" & k.w >= 3 |
            method.predict[i] == "S")
      #
      if (n.methpi == 1) {
        lower.predict.w[sel.p] <- NA
        upper.predict.w[sel.p] <- NA
      }
      else {
        lower.predict.w[sel.p, i] <- NA
        upper.predict.w[sel.p, i] <- NA
      }
    }
    #
    seTE.predict <- seTE.predict.w
    lower.predict <- lower.predict.w
    upper.predict <- upper.predict.w
    #
    if (length(method.predict) > 1)
      names(lower.predict) <- names(upper.predict) <-
        methpi
    #
    # Test for subgroup differences
    #
    mlm.Q <-
      runMLM(c(list.mlm,
               list(data = 
                      data.frame(subgroup.rma = subgroup.rma,
                                 cluster = x$cluster[!x$exclude],
                                 idx = seq_along(x$cluster)[!x$exclude]))),
             method.tau = x$method.tau,
             method.random.ci = x$method.random.ci,
             level = x$level.ma,
             mods = mod.Q,
             control = list(x$control),
             warn = FALSE)
    #
    # Tests for subgroup differences
    #
    for (i in seq_len(n.methci)) {
      Q.b.random[i] <- mlm.Q[[i]]$QM
      df.i <- mlm.Q[[i]]$QMdf
      df.Q.b.random[[i]] <- df.i[!is.na(df.i)]
      pval.Q.b.random[i] <- mlm.Q[[i]]$QMp
      #
      if (n.methci == 1)
        df.Q.b.random <- df.Q.b.random[[1]]
    }
    #
    Q.w.common <- NA
    Q.w.random <- NA
    df.Q.w <- NA
    pval.Q.w.common <- NA
    pval.Q.w.random <- NA
  }
  
  
  #
  #
  # (3) Subgroup analysis with common tau-squared (GLMM)
  #
  #
  
  if (!missing(subgroup.rma) && is.glmm) {
    n.methci <- length(x$method.random.ci)
    #
    mod <- as.call(~ subgroup.rma - 1)
    mod.Q <- as.call(~ subgroup.rma)
    #
    if (is.null(x$exclude))
      x$exclude <- rep(FALSE, length(x$TE))
    #
    if (prop) {
      list.glmm <-
        list(xi = x$event[!x$exclude], ni = x$n[!x$exclude],
             measure = "PLO")
      #
      use.random <-
        sum(!x$exclude) > 1 &
        sum(x$event[!x$exclude], na.rm = TRUE) > 0 &
        any(x$event[!x$exclude] != x$n[!x$exclude])
    }
    #
    else if (rate) {
      list.glmm <-
        list(xi = x$event[!x$exclude], ti = x$time[!x$exclude],
             measure = "IRLN")
      #
      use.random <-
        sum(!x$exclude) > 1 &
        sum(x$event[!x$exclude], na.rm = TRUE) > 0 &
        any(x$event[!x$exclude] != x$time[!x$exclude])
    }
    #
    else if (inc) {
      list.glmm <-
        list(x1i = x$event.e[!x$exclude], t1i = x$time.e[!x$exclude],
             x2i = x$event.c[!x$exclude], t2i = x$time.c[!x$exclude],
             measure = "IRR", model = x$model.glmm)
      #
      use.random <-
        sum(!x$exclude) > 1 &
        !((sum(x$event.e[!x$exclude], na.rm = TRUE) == 0 &
           sum(x$event.c[!x$exclude], na.rm = TRUE) == 0) |
          (!any(x$event.e[!x$exclude] != x$time.e[!x$exclude]) |
           !any(x$event.c[!x$exclude] != x$time.c[!x$exclude])))
    }
    #
    else if (bin) {
      list.glmm <-
        list(ai = x$event.e[!x$exclude], n1i = x$n.e[!x$exclude],
             ci = x$event.c[!x$exclude], n2i = x$n.c[!x$exclude],
             measure = "OR", model = x$model.glmm)
      #
      use.random <-
        sum(!x$exclude) > 1 &
        !((sum(x$event.e[!x$exclude], na.rm = TRUE) == 0 &
           sum(x$event.c[!x$exclude], na.rm = TRUE) == 0) |
          (!any(x$event.e[!x$exclude] != x$n.e[!x$exclude]) |
           !any(x$event.c[!x$exclude] != x$n.c[!x$exclude])))
    }
    #
    glmms <-
      runGLMM(list.glmm,
              method.tau = x$method.tau,
              method.random.ci = x$method.random.ci,
              level = x$level.ma,
              data = list(data = data.frame(subgroup.rma)),
              mods = mod,
              control = list(x$control),
              use.random = use.random,
              warn = FALSE)
    #
    glmm.c <- glmms$glmm.common
    glmm.r <- glmms$glmm.random
    #
    # Common effect model
    #
    sel.c <- !is.na(TE.common.w)
    TE.common.w[sel.c] <- as.numeric(glmm.c$b)
    seTE.common.w[sel.c] <- as.numeric(glmm.c$se)
    lower.common.w[sel.c] <- as.numeric(glmm.c$ci.lb)
    upper.common.w[sel.c] <- as.numeric(glmm.c$ci.ub)
    statistic.common.w[sel.c] <- as.numeric(glmm.c$zval)
    pval.common.w[sel.c] <- as.numeric(glmm.c$pval)
    #
    # Random effects model
    #
    sel.r <- !is.na(TE.random.w)
    TE.random.w[sel.r] <- as.numeric(glmm.r[[1]]$b)
    seTE.random.w[sel.r] <- as.numeric(glmm.r[[1]]$se)
    #
    if (n.methci == 1) {
      lower.random.w[sel.r] <- as.numeric(glmm.r[[1]]$ci.lb)
      upper.random.w[sel.r] <- as.numeric(glmm.r[[1]]$ci.ub)
      statistic.random.w[sel.r] <- as.numeric(glmm.r[[1]]$zval)
      pval.random.w[sel.r] <- as.numeric(glmm.r[[1]]$pval)
    }
    else {
      for (i in seq_len(n.methci)) {
        lower.random.w[sel.r, i] <- as.numeric(glmm.r[[i]]$ci.lb)
        upper.random.w[sel.r, i] <- as.numeric(glmm.r[[i]]$ci.ub)
        statistic.random.w[sel.r, i] <- as.numeric(glmm.r[[i]]$zval)
        pval.random.w[sel.r, i] <- as.numeric(glmm.r[[i]]$pval)
      }
    }
    #
    # Heterogeneity measures
    #
    tau2.w <- rep_len(glmm.r[[1]]$tau2, n.levs)
    lower.tau2.w <- tau2.w
    upper.tau2.w <- tau2.w
    lower.tau2.w[!is.na(lower.tau2.w)] <- NA
    upper.tau2.w[!is.na(upper.tau2.w)] <- NA
    #
    tau.w <- sqrt(tau2.w)
    lower.tau.w <- tau.w
    upper.tau.w <- tau.w
    lower.tau.w[!is.na(lower.tau.w)] <- NA
    upper.tau.w[!is.na(upper.tau.w)] <- NA
    #
    upper.Rb.w <- lower.Rb.w <- Rb.w <- rep_len(NA, n.levs)
    #
    # Prediction interval
    #
    tau2.calc <- if (is.na(glmm.r[[1]]$tau2)) 0 else tau2.w
    seTE.predict.w <- sqrt(seTE.random.w^2 + tau2.w)
    #
    n.methpi <- length(method.predict)
    #
    for (i in seq_len(n.methpi)) {
      df.i <- set_df_predict(method.predict[i], k.w)
      #
      ci.p <- ci(TE.random.w, seTE.predict.w, x$level.predict, df = df.i)
      #
      if (n.methpi == 1) {
        lower.predict.w <- ci.p$lower
        upper.predict.w <- ci.p$upper
      }
      else {
        lower.predict.w[, i] <- ci.p$lower
        upper.predict.w[, i] <- ci.p$upper
      }
      #
      sel.p <-
        !(method.predict[i] == "V" & k.w >= 2 |
            method.predict[i] == "HTS" & k.w >= 3 |
            method.predict[i] == "S")
      #
      if (n.methpi == 1) {
        lower.predict.w[sel.p] <- NA
        upper.predict.w[sel.p] <- NA
      }
      else {
        lower.predict.w[sel.p, i] <- NA
        upper.predict.w[sel.p, i] <- NA
      }
    }
    #
    seTE.predict <- seTE.predict.w
    lower.predict <- lower.predict.w
    upper.predict <- upper.predict.w
    #
    if (length(method.predict) > 1)
      names(lower.predict) <- names(upper.predict) <-
        methpi
    #
    # Test for subgroup differences
    #
    glmms.Q <-
      runGLMM(list.glmm,
              method.tau = x$method.tau,
              method.random.ci = x$method.random.ci,
              level = x$level.ma,
              data = list(data = data.frame(subgroup.rma)),
              mods = mod.Q,
              control = list(x$control),
              use.random = use.random,
              warn = FALSE)
    #
    glmm.c.Q <- glmms.Q$glmm.common
    glmm.r.Q <- glmms.Q$glmm.random
    #
    # Tests for subgroup differences
    #
    Q.b.common <- glmm.c.Q$QM
    df.Q.b.common <- glmm.c.Q$QMdf[!is.na(glmm.c.Q$QMdf)]
    pval.Q.b.common <- glmm.c.Q$QMp
    #
    for (i in seq_len(n.methci)) {
      Q.b.random[i] <- glmm.r.Q[[i]]$QM
      df.i <- glmm.r.Q[[i]]$QMdf
      df.Q.b.random[[i]] <- df.i[!is.na(df.i)]
      pval.Q.b.random[i] <- glmm.r.Q[[i]]$QMp
      #
      if (n.methci == 1)
        df.Q.b.random <- df.Q.b.random[[1]]
    }
    #
    Q.w.common <- NA
    Q.w.random <- NA
    df.Q.w <- NA
    pval.Q.w.common <- NA
    pval.Q.w.random <- NA
  }
  
  
  #
  #
  # (4) Return list with results of subgroup analysis
  #
  #
  
  res <- list(subgroup.levels = levs,
              #
              k.w = k.w,
              k.study.w = k.study.w,
              k.all.w = k.all.w,
              k.TE.w = k.TE.w,
              n.w = n.w,
              event.w = event.w,
              #
              TE.common.w = TE.common.w,
              seTE.common.w = seTE.common.w,
              statistic.common.w = statistic.common.w,
              pval.common.w = pval.common.w,
              lower.common.w = lower.common.w,
              upper.common.w = upper.common.w,
              w.common.w = w.common.w,
              #
              TE.random.w = TE.random.w,
              seTE.random.w = seTE.random.w,
              statistic.random.w = statistic.random.w,
              pval.random.w = pval.random.w,
              df.random.w = df.random.w,
              lower.random.w = lower.random.w,
              upper.random.w = upper.random.w,
              w.random.w = w.random.w,
              #
              seTE.classic.w = seTE.classic.w,
              #
              df.hakn.ci.w = df.hakn.ci.w,
              seTE.hakn.ci.w = seTE.hakn.ci.w,
              seTE.hakn.adhoc.ci.w = seTE.hakn.adhoc.ci.w,
              #
              df.kero.w = df.kero.w,
              seTE.kero.w = seTE.kero.w,
              #
              seTE.predict.w = seTE.predict.w,
              df.predict.w = df.predict.w,
              lower.predict.w = lower.predict.w,
              upper.predict.w = upper.predict.w,
              seTE.hakn.pi.w = seTE.hakn.pi.w,
              seTE.hakn.adhoc.pi.w = seTE.hakn.adhoc.pi.w,
              #
              Q.w = Q.w,
              pval.Q.w = pvalQ(Q.w, k.w - 1),
              #
              tau2.w = tau2.w,
              lower.tau2.w = lower.tau2.w,
              upper.tau2.w = upper.tau2.w,
              #
              tau.w = tau.w,
              lower.tau.w = lower.tau.w,
              upper.tau.w = upper.tau.w,
              #
              H.w = H.w,
              lower.H.w = lower.H.w,
              upper.H.w = upper.H.w,
              #
              I2.w = I2.w,
              lower.I2.w = lower.I2.w,
              upper.I2.w = upper.I2.w,
              #
              Rb.w = Rb.w,
              lower.Rb.w = lower.Rb.w,
              upper.Rb.w = upper.Rb.w,
              #
              Q.w.common = Q.w.common,
              Q.w.random = NA,
              df.Q.w = df.Q.w,
              pval.Q.w.common = pval.Q.w.common,
              pval.Q.w.random = NA,
              #
              Q.b.common = Q.b.common,
              Q.b.random = Q.b.random,
              df.Q.b = df.Q.b,
              df.Q.b.common = df.Q.b.common,
              df.Q.b.random = df.Q.b.random,
              pval.Q.b.common = pval.Q.b.common,
              pval.Q.b.random = pval.Q.b.random,
              #
              seed.predict.subgroup = seed
              )
  #
  # No general list elements
  #
  res$n.e.w <- n.e.w
  res$n.c.w <- n.c.w
  res$n.harmonic.mean.w <- n.harmonic.mean.w
  #
  res$event.e.w <- event.e.w
  res$event.c.w <- event.c.w
  #
  res$time.e.w <- time.e.w
  res$time.c.w <- time.c.w
  res$t.harmonic.mean.w <- t.harmonic.mean.w
  #
  # Set common effect results to NA for three-level model
  #
  if (is.mlm) {
    res$TE.common.w[!is.na(res$TE.common.w)] <- NA
    res$seTE.common.w[!is.na(res$seTE.common.w)] <- NA
    res$statistic.common.w[!is.na(res$statistic.common.w)] <- NA
    res$pval.common.w[!is.na(res$pval.common.w)] <- NA
    res$lower.common.w[!is.na(res$lower.common.w)] <- NA
    res$upper.common.w[!is.na(res$upper.common.w)] <- NA
    res$w.common.w[!is.na(res$w.common.w)] <- NA
    res$Q.w.common[!is.na(res$Q.w.common)] <- NA
    res$pval.Q.w.common[!is.na(res$pval.Q.w.common)] <- NA
    res$Q.b.common[!is.na(res$Q.b.common)] <- NA
    res$df.Q.b.common[!is.na(res$df.Q.b.common)] <- NA
    res$pval.Q.b.common[!is.na(res$pval.Q.b.common)] <- NA
  }
  #
  # Deprecated list elements
  #
  res$zval.common.w <- statistic.common.w
  res$zval.random.w <- statistic.random.w
  #
  # Backward compatibility
  #
  res$TE.fixed.w <- res$TE.common.w
  res$seTE.fixed.w <- res$seTE.common.w
  res$lower.fixed.w <- res$lower.common.w
  res$upper.fixed.w <- res$upper.common.w
  res$statistic.fixed.w <- res$statistic.common.w
  res$pval.fixed.w <- res$pval.common.w
  res$zval.fixed.w <- res$zval.common.w
  res$w.fixed.w <- res$w.common.w
  #
  res$Q.w.fixed <- res$Q.w.common
  res$pval.Q.w.fixed <- res$pval.Q.w.common
  res$Q.b.fixed <- res$Q.b.common
  res$pval.Q.b.fixed <- res$pval.Q.b.common
  #
  res$bylevs <- res$subgroup.levels
  #
  res$seTE.hakn.w <- res$seTE.hakn.ci.w
  res$seTE.hakn.adhoc.w <- res$seTE.hakn.adhoc.ci.w
  
  res
}

compsplit <- function(x, split) {
  
  if (split %in% gs("special.characters"))
    split <- paste0("\\", split)

  res <- strsplit(x, split)

  if (is.list(res)) {
    withspace <- any(unlist(lapply(res, grepl, pattern = "^\\s+|\\s+$")))
    res <- lapply(res, gsub, pattern = "^\\s+|\\s+$", replacement = "")
  }
  else {
    withspace <- any(grepl("^\\s+|\\s+$", "", res))
    res <- gsub("^\\s+|\\s+$", "", res)
  }
  #
  attr(res, "withspace") <- withspace
  
  res
}

#' Combine and summarize meta-analysis objects
#' 
#' @description
#' This function can be used to combine meta-analysis objects and is,
#' for example, useful to summarize results of various meta-analysis
#' methods or to generate a forest plot with results of several
#' subgroup analyses.
#' 
#' @param ... Any number of meta-analysis objects or a single list
#'   with meta-analyses.
#' @param name An optional character vector providing descriptive
#'   names for the meta-analysis objects.
#' @param subgroup An optional variable to generate a forest plot with
#'   subgroups.
#' @param common A logical vector indicating whether results of common
#'   effect model should be considered.
#' @param random A logical vector indicating whether results of random
#'   effects model should be considered.
#' @param prediction A logical vector indicating whether results of
#'   prediction intervals should be considered.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and plots. If
#'   \code{backtransf=TRUE} (default), results for \code{sm="OR"} are
#'   printed as odds ratios rather than log odds ratios, for example.
#' @param outclab Outcome label for all meta-analyis objects.
#' @param pooled Deprecated argument (replaced by \code{common} and
#'   \code{random}.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' 
#' @details
#' This function can be used to combine any number of meta-analysis
#' objects which is useful, for example, to summarize results of
#' various meta-analysis methods or to generate a forest plot with
#' results of several subgroup analyses (see Examples).
#' 
#' Individual study results are not retained with \code{metabind} as
#' the function allows to combine meta-analyses from different data
#' sets (e.g., with randomised or observational studies). Individual
#' study results are retained with R function \code{\link{metamerge}}
#' which can be used to combine results of meta-analyses of the
#' same dataset.
#' 
#' @return
#' An object of class \code{c("metabind", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metagen}}, \code{\link{forest.metabind}},
#'   \code{\link{metamerge}}
#' 
#' @examples
#' data(Fleiss1993cont)
#' 
#' # Add some (fictitious) grouping variables:
#' #
#' Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
#' Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")
#' 
#' ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD")
#'
#' # Conduct two subgroup analyses
#' #
#' ma1 <- update(ma, subgroup = age, subgroup.name = "Age group")
#' ma2 <- update(ma, subgroup = region, subgroup.name = "Region")
#'
#' # Combine random effects subgroup meta-analyses and show forest
#' # plot with subgroup results
#' #
#' mb <- metabind(ma1, ma2, common = FALSE)
#' mb
#' forest(mb)
#'
#' # Use various estimation methods for between-study heterogeneity
#' # variance
#' #
#' ma.pm <- update(ma, method.tau = "PM")
#' ma.dl <- update(ma, method.tau = "DL")
#' ma.ml <- update(ma, method.tau = "ML")
#' ma.hs <- update(ma, method.tau = "HS")
#' ma.sj <- update(ma, method.tau = "SJ")
#' ma.he <- update(ma, method.tau = "HE")
#' ma.eb <- update(ma, method.tau = "EB")
#'
#' # Combine meta-analyses and show results
#' #
#' taus <- c("Restricted maximum-likelihood estimator",
#'   "Paule-Mandel estimator",
#'   "DerSimonian-Laird estimator",
#'   "Maximum-likelihood estimator",
#'   "Hunter-Schmidt estimator",
#'   "Sidik-Jonkman estimator",
#'   "Hedges estimator",
#'   "Empirical Bayes estimator")
#' #
#' ma.taus <- metabind(ma, ma.pm, ma.dl, ma.ml, ma.hs, ma.sj, ma.he, ma.eb,
#'   name = taus, common = FALSE)
#' ma.taus
#' forest(ma.taus)
#' 
#' @export metabind

metabind <- function(..., subgroup = NULL,
                     name = NULL,
                     common = NULL, random = NULL, prediction = NULL,
                     backtransf = NULL, outclab = NULL,
                     pooled = NULL,
                     warn.deprecated = gs("warn.deprecated")) {
  
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  missing.subgroup <- missing(subgroup)
  subgroup.meta <- subgroup
  missing.name <- missing(name)
  missing.pooled <- missing(pooled)
  missing.backtransf <- missing(backtransf)
  #
  chklogical(warn.deprecated)
  #
  args <- list(...)
  #
  n.meta <- length(args)
  seq.meta <- seq_len(n.meta)
  #
  if (n.meta == 1) {
    if (inherits(args[[1]], "meta.rm5")) {
      args <- args[[1]]
      #
      if (missing.name) {
        name <- unlist(lapply(args, "[[" , "outclab"))
        missing.name <- FALSE
      }
    }
    #
    else if (inherits(args[[1]], "copas"))
      return(metamerge(args[[1]]))
    #
    else if (inherits(args[[1]], "limitmeta"))
      return(metamerge(args[[1]]))
    #
    else if (inherits(args[[1]], "netpairwise"))
      stop("Elements of argument '...' may not be of class 'netpairwise'.",
           call. = FALSE)
    #
    else if (inherits(args[[1]], "meta"))
      return(args[[1]])
    #
    else if (!is.list(args[[1]]))
      stop("All elements of argument '...' must be of class 'meta', ",
           "'limitmeta', or 'copas'.",
           call. = FALSE)
    else {
      n.meta <- length(args[[1]])
      seq.meta <- seq_len(n.meta)
      #
      args2 <- list()
      for (i in seq.meta)
        args2[[i]] <- args[[1]][[i]]
      #
      args <- args2
    }
  }
  #
  if (!missing.pooled) {
    pooled <- setchar(pooled, c("common", "random", "fixed"))
    pooled[pooled == "fixed"] <- "common"
    #
    pooled.common <- pooled == "common"
    pooled.random <- pooled == "random"
  }
  #
  common <-
    deprecated2(common, missing(common), pooled.common, missing.pooled,
                warn.deprecated, "pooled")
  #
  if (!is.null(common)) {
    chklogical(common[1], "common")
    #
    if (length(common) == 1)
      common <- rep_len(common, n.meta)
    else
      chklength(common, n.meta,
                text = paste("Length of argument 'common' differs from",
                             "number of meta-analyses."))
  }
  #
  random <-
    deprecated2(random, missing(random), pooled.random, missing.pooled,
                warn.deprecated, "pooled")
  #
  if (!is.null(random)) {
    chklogical(random[1], "random")
    #
    if (length(random) == 1)
      random <- rep_len(random, n.meta)
    else
      chklength(random, n.meta,
                text = paste("Length of argument 'random' differs from",
                             "number of meta-analyses."))
  }
  #
  if (!missing(prediction)) {
    if (length(prediction) == 1)
      prediction <- rep_len(prediction, n.meta)
    #
    if (!is.logical(prediction))
      stop("Argument 'prediction' must contain logical values.",
           call. = TRUE)
    #
    chklength(prediction, n.meta,
              text = paste("Length of argument 'prediction' differs from",
                           "number of meta-analyses."))
  }
  #
  if (!missing.backtransf)
    chklogical(backtransf)
  #  
  # Act on limitmeta and copas objects
  #
  name.i <- vector("character", n.meta)
  #
  is.limit <- is.copas <- is.trimfill <- is.subgroup <- rep(FALSE, n.meta)
  samedata <- rep(NA, n.meta)
  #
  for (i in seq.meta) {
    if (inherits(args[[i]], "metabind"))
      stop("Elements of argument '...' may not be of class 'metabind'.",
           call. = FALSE)
    #
    if (inherits(args[[i]], "netpairwise"))
      stop("Elements of argument '...' may not be of class 'netpairwise'.",
           call. = FALSE)
    #
    if (inherits(args[[i]], "meta")) {
      if (inherits(args[[i]], "metamerge"))
        stop("Meta-analysis objects created with metamerge() cannot ",
             "be used in metabind().",
             call. = FALSE)
      #
      if (!inherits(args[[i]], "metaadd"))
        args[[i]] <- updateversion(args[[i]])
      #
      if (missing.name) {
        if (inherits(args[[i]], "trimfill")) {
          is.trimfill[i] <- TRUE
          #
          name.i[i] <- "trimfill"
        }
        else
          name.i[i] <- replaceNULL(args[[i]]$subgroup.name, "")
        #
        if (name.i[i] == "")
          name.i[i] <- class(args[[i]])[1]
      }
    }
    #
    else if (inherits(args[[i]], "copas")) {
      args[[i]] <-
        metamerge(update(args[[i]]$x, common = FALSE, random = FALSE),
                  args[[i]], label2 = "copas")
      #
      is.copas[i] <- TRUE
      #
      if (missing.name)
        name.i[i] <- "copas"
    }
    #
    else if (inherits(args[[i]], "limitmeta")) {
      args[[i]] <-
        metamerge(update(args[[i]]$x, common = FALSE, random = FALSE),
                  args[[i]], label2 = "limit")
      #
      is.limit[i] <- TRUE
      #
      if (missing.name)
        name.i[i] <- "limitmeta"
    }
    #
    else
      stop("All elements of argument '...' must be of class 'meta', ",
           "'limitmeta', or 'copas'.",
           call. = FALSE)
    #
    is.subgroup[i] <- !is.null(args[[i]]$subgroup)
    samedata[i] <- !samedata(args[[1]], args[[i]], stop = FALSE)
  }
  #
  # Meta-analyses must all contain subgroups or none
  #
  if (length(unique(is.subgroup)) != 1)
    stop("All or none meta-analyses must contain subgroups.",
         call. = FALSE)
  #
  with.subgroups <- any(is.subgroup)
  is.limit.copas <- is.limit | is.copas
  #
  samedata[is.limit.copas] <- TRUE
  samedata <- all(samedata)
  
  
  #
  #
  # (2) Extract meta-analytical methods
  #
  #
  
  meth.list <- vector("list", n.meta)
  #
  for (i in seq.meta)
    meth.list[[i]] <-
      meta2meth(args[[i]], outclab)
  #
  meth <- list()
  #
  for (i in names(meth.list[[1]]))
    meth[[i]] <- condense(meth.list, i)
  #
  meth$common <- replaceNULL(common, meth$common)
  meth$common[is.limit.copas] <- FALSE
  meth$random <- replaceNULL(random, meth$random)
  meth$prediction <- replaceNULL(prediction, meth$prediction)
  #
  # Use common effect estimate if no result is selected
  #
  meth$common[!(meth$common | meth$random | meth$prediction)] <- TRUE
  #
  # Check whether settings are unique
  #
  meth2 <- meth[c("sm", "level.ma", "level.predict", "null.effect")]
  n.meth <- lapply(meth2, function(x) length(unique(x)))
  #
  if (any(n.meth != 1))
    stop("Setting for the following argument",
         if (sum(n.meth != 1) > 1) "s",
         " must be the same for all meta-analyses: ",
         paste0(paste0("'", names(meth2)[n.meth != 1], "'"),
                collapse = " - "))
  #
  # Unify settings
  #
  meth$sm <- makeunique(meth$sm)
  meth$method <- unique(meth$method)
  meth$method.random <- unique(meth$method.random)
  meth$three.level <- makeunique(meth$three.level, FALSE)
  #
  meth$level <- meth$level.ma <- makeunique(meth$level.ma)
  meth$level.predict <- makeunique(meth$level.predict)
  #
  common <- meth$common
  random <- meth$random
  prediction <- meth$prediction
  #
  meth$overall <- with.subgroups & samedata
  meth$overall.hetstat <- with.subgroups & samedata
  #
  if (missing.pooled & all(meth$common) & all(!meth$random))
    meth$common <- pooled == "onlycommon"
  else
    meth$common <- any(meth$common)
  #
  meth$random <- any(meth$random)
  #
  meth$prediction <- any(meth$prediction)
  #
  meth$method.common.ci <- unique(meth$method.common.ci)
  meth$method.random.ci <- unique(meth$method.random.ci)
  meth$adhoc.hakn.ci <- unique(meth$adhoc.hakn.ci)
  meth$method.tau <- unique(meth$method.tau)
  meth$tau.preset <- unique(meth$tau.preset)
  meth$TE.tau <- unique(meth$TE.tau)
  meth$tau.common <-
    if (!with.subgroups)
      FALSE
    else
      unique(meth$tau.common)
  #
  meth$method.I2 <- unique(meth$method.I2)
  #
  meth$prediction.subgroup <-
    if (!with.subgroups)
      FALSE
    else
      unique(meth$prediction.subgroup)
  meth$method.predict <- unique(meth$method.predict)
  meth$adhoc.hakn.pi <- unique(meth$adhoc.hakn.pi)
  #
  meth$method.bias <- unique(meth$method.bias)
  meth$null.effect <- makeunique(meth$null.effect) 
  #
  meth$title <- makeunique(meth$title, "")
  meth$complab <- makeunique(meth$complab, "")
  meth$outclab <- makeunique(meth$outclab, "")
  #
  meth$label.e <- makeunique(meth$label.e, "")
  meth$label.c <- makeunique(meth$label.c, "")
  meth$label.left <- makeunique(meth$label.left, "")
  meth$label.right <- makeunique(meth$label.right, "")
  #
  meth$print.subgroup.name <- makeunique(meth$print.subgroup.name, FALSE)
  meth$sep.subgroup <- makeunique(meth$sep.subgroup, "")
  meth$warn <- makeunique(meth$warn, FALSE)
  #
  meth$backtransf <-
    if (missing.backtransf)
      any(meth$backtransf)
    else
      backtransf
  meth$pscale <- makeunique(meth$pscale, 1)
  meth$irscale <- makeunique(meth$irscale, 1)
  meth$irunit <- makeunique(meth$irunit, "")
  #
  if (!(any(common) | any(random) | any(prediction))) {
    warning("No results to bind.", call. = FALSE)
    return(NULL)
  }
  #
  # Name of meta-analysis object
  #
  if (missing.name) {
    name <- name.i
    name[name == ""] <- paste0("meta", seq.meta[name == ""])
  }
  else
    chklength(name, length(is.subgroup),
              text =
                paste("Number of meta-analyses and names provided in",
                      "argument 'name' differ."))
  #
  # Names for meta-analyses must be unique
  #
  if (length(unique(name)) != length(name) & missing.subgroup) {
    for (i in seq.meta)
      if (name[i] %in% c("metabin", "metainc", "metaprop", "metarate") &
          !is.trimfill[i])
        name[i] <- paste(name[i], args[[i]]$method, sep = ".")
  }
  #
  if (length(unique(name)) != length(name) & missing.subgroup) {
    for (i in seq.meta)
      if (random[i])
        name[i] <- paste(name[i], args[[i]]$method.tau, sep = ".")
  }
  #
  if (length(unique(name)) != length(name) & missing.subgroup)
    name <- paste0("meta", seq.meta)
  #
  # Check if more than one common effect / random effects CI or PI is
  # provided
  #
  for (i in seq.meta) {
    if (length(args[[i]]$lower.common) > 1) {
      ith <- min(i, 4)
      stop("More than one result for common effect model provided in ",
           i, switch(ith, "st", "nd", "rd", "th"),
           " meta-analysis.",
           call. = FALSE)
    }
  }
  #
  for (i in seq.meta) {
    if (length(args[[i]]$lower.random) > 1) {
      ith <- min(i, 4)
      stop("More than one result for random effects model provided in ",
           i, switch(ith, "st", "nd", "rd", "th"),
           " meta-analysis.",
           call. = FALSE)
    }
  }
  #
  for (i in seq.meta) {
    if (length(args[[i]]$lower.predict) > 1) {
      ith <- min(i, 4)
      stop("More than one prediction interval provided in ",
           i, switch(ith, "st", "nd", "rd", "th"),
           " meta-analysis.",
           call. = FALSE)
    }
  }
  
  
  # #
  # # Show individual results
  # #
  show.studies <- TRUE
  overall.hetstat <- TRUE
  # #
  # if (length(unique(meth$method)) != 1 |
  #     length(unique(meth$method.tau)) != 1) {
  #   show.studies <- FALSE
  #   overall.hetstat <- FALSE
  # }
  
  
  #
  #
  # (3) Extract meta-analysis data
  #
  #
  
  meta.list <- vector("list", n.meta)
  #
  if (with.subgroups)
    for (i in seq.meta)
      meta.list[[i]] <-
        subgr2meta(args[[i]], common[i], random[i], prediction[i], name[i])
  else {
    for (i in seq.meta)
      meta.list[[i]] <-
        overall2meta(args[[i]], common[i], random[i], prediction[i], name[i])
  }
  #
  meta <- list()
  #
  for (i in names(meta.list[[1]]))
    meta[[i]] <- condense(meta.list, i)
  
  
  #
  #
  # (4) Extract meta-analysis results and store them in subgroups
  #
  #
  
  subgroup.list <- vector("list", n.meta)
  #
  for (i in seq.meta)
    subgroup.list[[i]] <- overall2subgr(args[[i]])
  #
  subgroup <- list()
  #
  for (i in names(subgroup.list[[1]]))
    subgroup[[i]] <- condense(subgroup.list, i)
  #
  if (!with.subgroups) {
    subgroup$subgroup.levels <- name
  }
  
  
  #
  #
  # (5) Extract study data
  #
  #
  
  data.list <- vector("list", n.meta)
  #
  if (with.subgroups) {
    for (i in seq.meta)
      data.list[[i]] <-
        subgr2data(args[[i]], common[i], random[i], prediction[i], name[i])
  }
  else {
    if (!is.null(subgroup.meta)) {
      if (length(subgroup.meta == 1))
        subgroup.meta <- rep_len(subgroup.meta, n.meta)
      else if (length(subgroup.meta) != n.meta)
        stop("Argument 'subgroup' must be of same length as ",
             "number of meta-analysis results.",
             call. = FALSE)
    }
    else
      subgroup.meta <- rep("", n.meta)
    #
    for (i in seq.meta)
      data.list[[i]] <-
        overall2data(args[[i]], common[i], random[i], prediction[i], name[i],
                     subgroup.meta[[i]])
  }
  #
  data <- do.call("rbind", data.list)
  #
  if (!with.subgroups & all(subgroup.meta == ""))
    data <- data[, names(data) != "subgroup"]
  
  
  #
  #
  # (6) Generate meta-analysis object
  #
  #
  
  res <- c(meta, meth, subgroup)
  #
  res$data <- data
  #
  res$common.meta <- common
  res$random.meta <- random
  res$prediction.meta <- prediction
  #
  if (!with.subgroups) {
    res$n.harmonic.mean <- res$n.harmonic.mean.ma
    res$t.harmonic.mean <- res$t.harmonic.mean.ma
  }
  else {
    res$n.harmonic.mean.ma <- unique(res$n.harmonic.mean.ma)
    res$t.harmonic.mean.ma <- unique(res$t.harmonic.mean.ma)
  }
  #
  res$call <- match.call()
  res$version <- packageDescription("meta")$Version
  #
  res$show.studies <- show.studies
  #
  res$with.subgroups <- with.subgroups
  res$samedata <- samedata
  #
  res$method <- unique(res$method)
  res$method.random <- unique(res$method.random)
  res$method.common.ci <- unique(res$method.common.ci)
  res$method.random.ci <- unique(res$method.random.ci)
  res$three.level <- unique(res$three.level)
  res$adhoc.hakn.ci <- unique(res$adhoc.hakn.ci)
  res$tau.common <- unique(res$tau.common)
  #
  if (with.subgroups & samedata) {
    sel.c <-
      !duplicated(
         data.frame(res$TE.common,
                    res$lower.common,
                    res$upper.common))
    #
    res$TE.common <- res$TE.common[sel.c]
    res$seTE.common <- res$seTE.common[sel.c]
    res$lower.common <- res$lower.common[sel.c]
    res$upper.common <- res$upper.common[sel.c]
    res$statistic.common <- res$statistic.common[sel.c]
    res$pval.common <- res$pval.common[sel.c]
    #
    res$text.common <-
      if (is.null(res$text.common))
        gs("text.common")
      else
        res$text.common <- res$text.common[sel.c]
    #
    sel.r <-
      !duplicated(
         data.frame(res$TE.random,
                    res$lower.random,
                    res$upper.random))
    #
    res$TE.random <- res$TE.random[sel.r]
    res$seTE.random <- res$seTE.random[sel.r]
    res$lower.random <- res$lower.random[sel.r]
    res$upper.random <- res$upper.random[sel.r]
    res$statistic.random <- res$statistic.random[sel.r]
    res$pval.random <- res$pval.random[sel.r]
    #
    res$text.random <-
      if (is.null(res$text.random))
        gs("text.random")
      else
        res$text.random <- res$text.random[sel.r]
    #
    res$df.random <- unique(res$df.random)
    res$df.hakn <- unique(res$df.hakn)
    res$df.kero <- unique(res$df.kero)
    #
    sel.p <-
      !duplicated(
         data.frame(res$lower.predict,
                    res$upper.predict))
    #
    res$lower.predict <- res$lower.predict[sel.p]
    res$upper.predict <- res$upper.predict[sel.p]
    #
    res$text.predict <- 
      if (is.null(res$text.predict))
        gs("text.predict")
      else
        res$text.predict[sel.p]
    #
    res$n.e <- unique(res$n.e)
    res$n.c <- unique(res$n.c)
    #
    res$k <- unique(res$k)
    res$k.study <- unique(res$k.study)
    res$k.all <- unique(res$k.all)
    res$k.TE <- unique(res$k.TE)
    #
    res$tau2 <- unique(res$tau2)
    res$tau <- unique(res$tau)
    #
    res$detail.tau <- ""
    #
    res$H <- unique(res$H)
    res$lower.H <- unique(res$lower.H)
    res$upper.H <- unique(res$upper.H)
    #
    res$I2 <- unique(res$I2)
    res$lower.I2 <- unique(res$lower.I2)
    res$upper.I2 <- unique(res$upper.I2)
    #
    res$Rb <- unique(res$Rb)
    res$lower.Rb <- unique(res$lower.Rb)
    res$upper.Rb <- unique(res$upper.Rb)
    #
    res$subgroup.name <- "meta-analysis"
    res$w.common <- res$w.random <- rep(0, length(res$TE))
    res$w.common.w <- rep(0, length(res$TE.common.w))
    res$w.random.w <- rep(0, length(res$TE.random.w))
    res$lower.predict.w <- rep(NA, length(res$w.random.w))
    res$upper.predict.w <- rep(NA, length(res$w.random.w))
    #
    res$Q.b.common <- unique(res$Q.b.common)
    res$Q.b.random <- unique(res$Q.b.random)
    res$df.Q.b <- unique(res$df.Q.b)
    res$pval.Q.b.common <- unique(res$pval.Q.b.common)
    res$pval.Q.b.random <- unique(res$pval.Q.b.random)
  }
  else {
    res$TE.common <- res$seTE.common <-
      res$lower.common <- res$upper.common <-
        res$statistic.common <- res$pval.common <- NA
    #
    res$TE.random <- res$seTE.random <-
      res$lower.random <- res$upper.random <-
        res$statistic.random <- res$pval.random <- NA
    #
    res$lower.predict <- res$upper.predict <- NA
    #
    if (!with.subgroups) {
      res$studlab <- res$data$studlab
      res$TE <- res$data$TE
      res$seTE <- res$data$seTE
      res$lower <- res$data$lower
      res$upper <- res$data$upper
      res$statistic <- res$data$statistic
      res$pval <- res$data$pval
      res$zval <- res$data$statistic
      #
      res$w.common <- res$w.random <- rep_len(NA, length(res$studlab))
      #
      res$text.w.common <- res$text.w.random <- ""
    }
  }
  #
  if (all(is.na(res$tau.preset)))
    res$tau.preset <- NULL
  #
  res$Q.w.common <- NA
  res$Q.w.random <- NA
  res$df.Q.w <- NA
  res$pval.Q.w.common <- NA
  res$pval.Q.w.random <- NA
  #
  res$is.limit.copas <- is.limit.copas
  #
  res$classes <- unique(as.vector(sapply(args, function(x) class(x))))
  res$classes <- res$classes[res$classes != "meta"]
  #
  # Backward compatibility
  #
  res <- backward(res)
  #  
  class(res) <- c("metabind", "meta")
  
  res$list <- list(meth = meth, meta = meta, subgroup = subgroup, data = data)
  
  res
}

# Auxiliary functions to transform relative treatment effects to
# pairwise comparisons
#
# Author: Gert van Valkenhoef <gvanvalkenhoef@cochrane.org>
# License: GPL (>= 2)
#

# For use with dplyr::group_map
#
pwgen_study <- function(input, studlab) {
  # Get rid of warning 'Undefined global functions or variables'
  treat1 <- NULL
  #
  studlab <- studlab$studlab
  
  # Get rid of warning 'Undefined global functions or variables'
  n <- NULL
  
  # separate the reference arm from the other rows
  ref <- input[input$treat1 == input$treat2, ]
  stopifnot(nrow(ref) == 1)
  rel <- input %>% filter(treat1 != ref$treat2) %>%
    mutate(studlab = studlab, n1 = n, n2 = ref$n) %>%
    select(!n)
  stopifnot(nrow(rel) == nrow(input) - 1)
  stopifnot(nrow(rel) == length(unique(rel$treat1)))
  
  # calculate other treatment effects
  if (nrow(rel) > 1) {
    # generate the missing treatment pairs
    other <- combn(rel$treat1, 2)
    # for each pair, calculate TE and seTE
    other <- mapply(function(treat1, treat2) {
      r1 <- rel[rel$treat1 == treat1, ]
      r2 <- rel[rel$treat1 == treat2, ]
      data.frame(
        studlab,
        treat1,
        treat2,
        TE = r1$TE - r2$TE,
        seTE = sqrt(r1$seTE^2 + r2$seTE^2 - 2 * ref$seTE^2),
        n1 = r1$n1,
        n2 = r2$n1)
    },
    other[1, ], other[2, ], SIMPLIFY = FALSE) %>% bind_rows()
    #
    bind_rows(rel, other)
  }
  else {
    rel
  }
}

# Expects one row per arm of each study:
#  - treat1 is the current study arm 
#  - treat2 is the study reference arm (e.g. control/placebo)
#  - if treat1 != treat2, TE is the treatment effect estimate, otherwise NA
#  - if treat1 != treat2, seTE is the standard error of the treatment effect
#    estimate, otherwise it is the standard error of the reference arm
#    (or the square root of the covariance between treatment effects) - optional
#    for two-arm studies.
#
pwgen <- function(studlab, treat1, treat2, TE, seTE, n) {
  nulln <- is.null(n)
  if (nulln) {
    n <- rep(NA, length(TE))
  }
  
  # Get rid of warning 'Undefined global functions or variables'
  n1 <- n2 <- NULL
  
  result <- data.frame(studlab, treat1, treat2, TE, seTE, n) %>%
    group_by(studlab) %>%
    group_map(pwgen_study) %>%
    bind_rows() %>%
    relocate(studlab)
  
  if (nulln) {
    result %>% select(!c(n1, n2)) %>% as.data.frame
  }
  else {
    as.data.frame(result)
  }
}

#' Calculate best linear unbiased predictor for \code{meta} object
#' 
#' @description
#' Calculate best linear unbiased predictors (BLUPs) for meta-analysis object
#' created with R package \bold{meta}.
#' 
#' @aliases blup blup.meta
#' 
#' @param x An object of class \code{meta}, \code{blup.meta}, or
#'   \code{estimates.blup.meta}.
#' @param level The level used to calculate prediction intervals for BLUPs.
#' @param backtransf A logical indicating whether BLUPs should be
#'   back transformed. If \code{backtransf = TRUE}, results for
#'   \code{sm = "OR"} will be odds ratios rather than log odds ratios,
#'    for example.
#' @param digits Minimal number of significant digits, see
#'   \code{print.default}.
#' @param digits.se Minimal number of significant digits for standard errors.
#' @param digits.tau2 Minimal number of significant digits for
#'   between-study variance \eqn{\tau^2}, see \code{print.default}.
#' @param digits.tau Minimal number of significant digits for
#'   \eqn{\tau}, the square root of the between-study variance
#'   \eqn{\tau^2}.
#' @param big.mark A character used as thousands separator.
#' @param se A logical indicating whether standard errors should be
#'   printed / extracted.
#' @param print.tau2 A logical specifying whether between-study
#'   variance \eqn{\tau^2} should be printed.
#' @param print.tau A logical specifying whether \eqn{\tau}, the
#'   square root of the between-study variance \eqn{\tau^2}, should be
#'   printed.
#' @param details A logical specifying whether details on
#'   statistical methods should be printed.
#' @param writexl A logical indicating whether an Excel file should be
#'   created (R package \bold{writexl} must be available).
#' @param path A character string specifying the filename of the Excel
#'   file.
#' @param overwrite A logical indicating whether an existing Excel
#'   file should be overwritten.
#' @param \dots Additional arguments (passed on to \code{\link{prmatrix}}).
#' 
#' @return
#' Data frame with variables
#' \item{studlab}{Study label}
#' \item{blup}{estimated best linear unbiased predictor}
#' \item{se.blup}{standard error (only if argument \code{backtransf = FALSE})}
#' \item{lower}{lower prediction limits}
#' \item{upper}{upper prediction limits}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}
#' 
#' @examples
#' \dontrun{
#' if (requireNamespace("metadat", quietly = TRUE)) {
#' data(dat.bcg, package = "metadat")
#' ma <- metabin(tpos, tpos + tneg, cpos, cpos + cneg,
#'   data = dat.bcg, studlab = paste(author, year), method = "Inverse")
#' summary(ma)
#' blup(ma)
#' 
#' # Save estimates in Excel file (R package 'writexl' must be available)
#' if (requireNamespace("writexl", quietly = TRUE))
#'  estimates(blup(ma), path = "blup_ma.xlsx")
#' }
#' }
#' 
#' @method blup meta
#' @export

blup.meta <- function(x, level = x$level, backtransf = x$backtransf,
                      ...) {
  
  #
  # (1) Check for meta object and upgrade older meta object
  #
  chkclass(x, "meta")
  x <- updateversion(x)
  meta <- x
  
  #
  # (2) Check additional arguments
  #
  chklevel(level)
  #
  if (is_untransformed(x$sm))
    backtransf <- TRUE
  chklogical(backtransf)
  #
  by <- !is.null(x$subgroup)
  
  #
  # (3) Calculate BLUP in meta-analysis without subgroups
  #
  if (!by) {
    TE.random <- x$TE.random
    seTE.random <- x$seTE.random
    df.hakn.ci <- x$df.hakn.ci
    df.kero <- round(x$df.kero, 2)
    #
    if (length(x$method.random.ci) == 1) {
      names(TE.random) <- x$method.random.ci
      names(seTE.random) <- x$method.random.ci
      names(df.hakn.ci) <- x$method.random.ci
      names(df.kero) <- x$method.random.ci
    }
    #
    frac <- x$tau2 / (x$tau2 + x$seTE^2)
    res <- data.frame()
    #
    for (i in x$method.random.ci) {
      dat.i <- as.data.frame(x)
      dat.i$blup <- frac * x$TE + (1 - frac) * TE.random[i]
      dat.i$se.blup <- sqrt(frac * x$seTE^2 + (1 - frac)^2 * seTE.random[i]^2)
      dat.i$tau2 <- x$tau2
      dat.i$method <- i
      #
      dat.i$df.hakn <- x$df.hakn.ci[i]
      dat.i$df.kero <- round(x$df.kero, 2)
      #
      if (i %in% c("classic", "classic-KR")) {
        ci.blup.i <- ci(dat.i$blup, dat.i$se.blup, level = level)
      }
      else if (i == "HK") {
        ci.blup.i <-
          ci(dat.i$blup, dat.i$se.blup, level = level, df = dat.i$df.hakn)
      }
      else if (i == "KR") {
        ci.blup.i <-
          ci(dat.i$blup, dat.i$se.blup, level = level, df = dat.i$df.kero)
      }
      else {
        ci.blup.i <- list(lower = NA, upper = NA)
      }
      #
      res.i <- data.frame(studlab = x$studlab,
                          blup = dat.i$blup, se.blup = dat.i$se.blup,
                          lower = ci.blup.i$lower,
                          upper = ci.blup.i$upper)
      #
      if (length(x$method.random.ci) > 1)
        res.i$method <- i
      if (!any(x$method.random.ci == "HK"))
        res.i$df.hakn <- NULL
      if (!any(x$method.random.ci == "KR"))
        res.i$df.kero <- NULL
      #
      if (nrow(res) == 0)
        res <- res.i
      else
        res <- rbind(res, res.i)
    }
  }
  
  #
  # (4) Calculate BLUP in meta-analysis with subgroups
  #
  if (by) {
    TE.random <- x$TE.random.w
    seTE.random <- x$seTE.random.w
    #
    if (length(x$method.random.ci) == 1) {
      seTE.random <- matrix(seTE.random, ncol = 1)
      rownames(seTE.random) <- x$subgroup.levels
      colnames(seTE.random) <- x$method.random.ci
    }
    #
    res <- data.frame()
    #
    for (i in x$method.random.ci) {
      dat.i <- as.data.frame(x)
      dat.i$blup <- NA
      dat.i$se.blup <- NA
      dat.i$tau2 <- NA
      #
      for (g in x$subgroup.levels) {
        sel.g <- dat.i$subgroup == g
        tau2.g <- replaceNA(x$tau2.w[g], 0)
        #
        frac.g <- tau2.g / (tau2.g + dat.i$seTE[sel.g]^2)
        #
        dat.i$blup[sel.g] <-
          frac.g * dat.i$TE[sel.g] + (1 - frac.g) * TE.random[g]
        #
        dat.i$se.blup[sel.g] <-
          sqrt(frac.g * x$seTE[sel.g]^2 + (1 - frac.g)^2 * seTE.random[g, i]^2)
        #
        dat.i$df.hakn[sel.g] <- x$k.w[g] - 1
        dat.i$df.kero[sel.g] <- round(x$df.kero.w[g], 2)
        #
        dat.i$tau2[sel.g] <- tau2.g
      }
      #
      blup.i <- dat.i$blup
      se.blup.i <- dat.i$se.blup
      df.hakn <- dat.i$df.hakn
      df.kero <- dat.i$df.kero
      #
      if (i %in% c("classic", "classic-KR"))
        ci.blup.i <- ci(blup.i, se.blup.i, level = level)
      else if (i == "HK")
        ci.blup.i <- ci(blup.i, se.blup.i, level = level, df = df.hakn)
      else if (i == "KR")
        ci.blup.i <- ci(blup.i, se.blup.i, level = level, df = df.kero)
      else
        ci.blup.i <- list(lower = NA, upper = NA)
      #
      res.i <- data.frame(studlab = x$studlab,
                          subgroup = dat.i$subgroup,
                          blup = blup.i, se.blup = se.blup.i,
                          lower = ci.blup.i$lower,
                          upper = ci.blup.i$upper,
                          tau2 = dat.i$tau2,
                          df.hakn = dat.i$df.hakn,
                          df.kero = dat.i$df.kero)
      #
      if (length(x$method.random.ci) > 1)
        res.i$method <- i
      #
      if (nrow(res) == 0)
        res <- res.i
      else
        res <- rbind(res, res.i)
    }
  }
  
  #
  # (5) Return BLUPs
  #
  meta$data <- NULL
  meta$level <- level
  meta$backtransf <- backtransf
  #
  attr(res, "x") <- meta
  #
  class(res) <- c("blup.meta", "data.frame")
  #
  res
}





#' @rdname blup.meta
#' @method print blup.meta
#' @export

print.blup.meta <- function(x, backtransf = attr(x, "x")$backtransf,
                            #
                            digits = gs("digits"),
                            digits.se = gs("digits.se"),
                            digits.tau2 = gs("digits.tau2"),
                            digits.tau = gs("digits.tau"),
                            big.mark = gs("big.mark"),
                            #
                            se = FALSE,
                            print.tau2 = gs("print.tau2"),
                            print.tau = gs("print.tau"),
                            #
                            details = gs("details"),
                            ...) {
  
  #
  # (1) Check arguments
  #
  chkclass(x, "blup.meta")
  #
  meta <- attr(x, "x")
  #
  sm <- meta$sm
  pscale <- meta$pscale
  irscale <- meta$irscale
  #
  if (is_untransformed(sm))
    backtransf <- TRUE
  #
  chklogical(backtransf)
  chklogical(se)
  if (!isCol(x, "se.blup"))
    se <- FALSE
  #
  print.se <- se
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  chklogical(print.tau2)
  chklogical(print.tau)
  #
  chklogical(details)
  
  #
  # (2) Back-transform and round results
  #
  sm.lab <- smlab(sm, backtransf, pscale, irscale)
  #
  if (backtransf) {
    x$blup <- backtransf(x$blup, sm)
    x$lower <- backtransf(x$lower, sm)
    x$upper <- backtransf(x$upper, sm)
  }
  #
  blup <- round(x$blup, digits)
  lower <- round(x$lower, digits)
  upper <- round(x$upper, digits)
  #
  if (print.se) {
    se.blup <- round(x$se.blup, digits.se)
    #
    if (sm.lab != "")
      se.lab <- paste0("SE(", sm.lab, ")")
    else
      se.lab <- "SE"
  }
  #
  if (!isCol(x, "method"))
    x$method <- meta$method.random.ci
  #
  text.random <- meta$text.random
  if (length(text.random) == 1 & length(unique(x$method)) > 1)
    text.random <- paste0(text.random, "(", unique(x$method), ")")
  #
  by <- !is.null(meta$subgroup)
  
  #
  # (3) Print results
  #
  crtitle(meta)
  #
  j <- 0
  for (i in unique(x$method)) {
    j <- j + 1
    sel.i <- x$method == i
    #
    res.i <-
      cbind(blup = formatN(blup[sel.i], digits, "NA", big.mark = big.mark),
            if (print.se)
              se = formatPT(se.blup, digits = digits.se,
                            big.mark = big.mark,
                            lab = FALSE, lab.NA = "NA"),
            ci = formatCI(formatN(lower[sel.i], digits, "NA", 
                                  big.mark = big.mark),
                          formatN(upper[sel.i], digits, "NA",
                                  big.mark = big.mark))
      )
    #
    colnames(res.i) <- c(sm.lab, if (print.se) se.lab,
                         paste0(100 * meta$level, "%-PI"))
    #
    if (by)
      res.i <- cbind(res.i, subgroup = x$subgroup[sel.i])
    #
    if (print.tau2 & length(unique(x$tau2[sel.i])) > 1)
      res.i <- cbind(res.i, tau2 =
                       rmSpace(formatPT(x$tau2[sel.i], digits = digits.tau2,
                                        big.mark = big.mark,
                                        lab = FALSE, lab.NA = "NA")))
    #
    if (print.tau & length(unique(x$tau2[sel.i])) > 1)
      res.i <- cbind(res.i, tau =
                       rmSpace(formatPT(sqrt(x$tau2[sel.i]),
                                        digits = digits.tau,
                                        big.mark = big.mark,
                                        lab = FALSE, lab.NA = "NA")))
    #
    if (i == "HK" & (by | length(unique(x$df.hakn[sel.i])) > 1))
      res.i <- cbind(res.i, df.hakn = x$df.hakn[sel.i])
    #
    if (i == "KR" & (by | length(unique(x$df.kero[sel.i])) > 1))
      res.i <- cbind(res.i,
                     df.kero = formatN(round(x$df.kero[sel.i], 2),
                                       2, "NA", big.mark = big.mark))
    #
    rownames(res.i) <- x$studlab[sel.i]
    #
    if (length(unique(x$method)) > 1) {
      if (j > 1)
        cat("\n")
      cat(text.random[j], "\n")
    }
    prmatrix(res.i, quote = FALSE, right = TRUE, ...)
  }
  #
  if (details)
    details <-
    catmeth(meta,
            common = FALSE, random = TRUE, prediction = FALSE,
            overall = TRUE, overall.hetstat = TRUE,
            #
            func.transf = NULL,
            backtransf = backtransf, func.backtransf = NULL,
            #
            big.mark = big.mark, digits = digits, digits.tau = digits.tau,
            text.tau = gs("text.tau"), text.tau2 = gs("text.tau2"),
            #
            print.tau2 = print.tau2, print.tau2.ci = FALSE,
            print.tau = print.tau, print.tau.ci = FALSE,
            #
            print.df = !by)
  #
  invisible(details)
}





#' @rdname blup.meta
#' @method estimates blup.meta
#' @export

estimates.blup.meta <- function(x,
                                se = FALSE,
                                backtransf = attr(x, "x")$backtransf,
                                #
                                digits = gs("digits"),
                                digits.se = gs("digits.se"),
                                digits.tau2 = gs("digits.tau2"),
                                digits.tau = gs("digits.tau"),
                                #
                                writexl = !missing(path),
                                path = "estimates_blup.xlsx",
                                overwrite = FALSE,
                                #
                                ...) {
  
  #
  # (1) Check arguments
  #
  chkclass(x, "blup.meta")
  #
  chklogical(se)
  chklogical(backtransf)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.se, min = 0, length = 1)
  chknumeric(digits.tau2, min = 0, length = 1)
  chknumeric(digits.tau, min = 0, length = 1)
  #
  chklogical(writexl)
  chkchar(path, length = 1)
  chklogical(overwrite)
  #
  meta <- attr(x, "x")
  res <- x
  
  #
  # (2) Back-transform
  #
  if (backtransf) {
    res$blup <- backtransf(res$blup, meta$sm)
    res$lower <- backtransf(res$lower, meta$sm)
    res$upper <- backtransf(res$upper, meta$sm)
  }
  #
  sm.lab <- smlab(meta$sm, backtransf, meta$pscale, meta$irscale)
  #
  if (!any(meta$method.random.ci == "HK"))
    res$df.hakn <- NULL
  #
  if (!any(meta$method.random.ci == "KR"))
    res$df.kero <- NULL
  
  #
  # (3) Create Excel file
  #
  if (writexl) {
    if (!is_installed_package("writexl", stop = FALSE))
      stop(paste0("Package 'writexl' missing.",
                  "\n  ",
                  "Please use the following R command for installation:",
                  "\n  install.packages(\"writexl\")"),
           call. = FALSE)
    #
    res$blup <- round(res$blup, digits = digits)
    res$lower <- round(res$lower, digits = digits)
    res$upper <- round(res$upper, digits = digits)
    #
    if (isCol(res, "tau2"))
      res$tau2 <- round(res$tau2, digits = digits.tau2)
    #
    if (isCol(res, "tau"))
      res$tau <- round(res$tau, digits = digits.tau)
    #
    names(res)[names(res) == "blup"] <- sm.lab
    #
    if (se & isCol(res, "se.blup") &
        !(backtransf & is_relative_effect(meta$sm))) {
      res$se.blup <- round(res$se.blup, digits = digits.se)
      if (sm.lab != "")
        names(res)[names(res) == "se.blup"] <- paste0("SE(", sm.lab, ")")
      else
        names(res)[names(res) == "se.blup"]  <- "SE"
    }
    else
      res$se.blup <- NULL
    #
    if (file.exists(path) & !overwrite)
      warning("File '", path, "' exists. ",
              "Use argument 'overwrite = TRUE' to overwrite file.",
              call. = FALSE)
    else {
      writexl::write_xlsx(res, path = path, col_names = TRUE, ...)
      message(paste0("Extracted information saved in file '", path, "'."))
    }
    #
    return(invisible(NULL))
  }
  
  #
  # (4) Return estimates object
  #
  class(res) <- c("estimates.blup.meta", "data.frame")
  #
  attr(res, "digits") <- digits
  attr(res, "digits.se") <- digits.se
  attr(res, "digits.tau2") <- digits.tau2
  attr(res, "digits.tau") <- digits.tau
  #
  meta$backtransf <- backtransf
  attr(res, "x") <- meta
  #
  res
}





#' @rdname blup.meta
#' @method print estimates.blup.meta
#' @export

print.estimates.blup.meta <- function(x,
                                      big.mark = gs("big.mark"),
                                      details = gs("details"),
                                      ...) {
  
  chkclass(x, "estimates.blup.meta")
  
  meta <- attr(x, "x")
  #
  digits <- attr(x, "digits")
  digits.se <- attr(x, "digits.se")
  digits.tau2 <- attr(x, "digits.tau2")
  digits.tau <- attr(x, "digits.tau")
  #
  backtransf <- meta$backtransf
  #
  se <- isCol(x, "se.blup")
  
  sm.lab <- smlab(meta$sm, backtransf, meta$pscale, meta$irscale)
  #
  ci.lab <- paste0(round(100 * meta$level, 1), "%-PI")
  #
  x$blup <- round(x$blup, digits = digits)
  x$lower <- round(x$lower, digits = digits)
  x$upper <- round(x$upper, digits = digits)
  studlab <- x$studlab
  x$studlab <- NULL
  #
  if (se)
    x$se.blup <- round(x$se.blup, digits = digits.se)
  #
  x$blup <- formatN(x$blup, digits, "", big.mark = big.mark)
  #
  x$lower <-
    ifelse(is.na(x$lower) & is.na(x$upper), "",
           formatCI(
             formatN(x$lower, digits, "", big.mark = big.mark),
             formatN(x$upper, digits, "", big.mark = big.mark)))
  x$upper <- NULL
  #
  if (isCol(x, "tau2"))
    x$tau2 <- round(x$tau2, digits = digits.tau2)
  #
  if (isCol(x, "tau"))
    x$tau <- round(x$tau, digits = digits.tau)
  #
  x <- as.matrix(x)
  rownames(x) <- studlab
  colnames(x)[colnames(x) == "blup"] <- sm.lab
  colnames(x)[colnames(x) == "lower"] <- ci.lab
  if (se) {
    if (sm.lab != "")
      colnames(x)[colnames(x) == "se.blup"] <- paste0("SE(", sm.lab, ")")
    else
      colnames(x)[colnames(x) == "se.blup"] <- "SE"
  }
  
  crtitle(meta)
  #
  prmatrix(x, quote = FALSE, right = TRUE, ...)
  #
  if (details)
    details <-
      catmeth(meta,
              common = FALSE, random = TRUE, prediction = FALSE,
              overall = TRUE, overall.hetstat = TRUE,
              #
              func.transf = NULL,
              backtransf = backtransf, func.backtransf = NULL,
              #
              big.mark = big.mark, digits = digits, digits.tau = digits.tau,
              text.tau = gs("text.tau"), text.tau2 = gs("text.tau2"),
              #
              print.tau2 = TRUE, print.tau2.ci = FALSE,
              print.tau = FALSE, print.tau.ci = FALSE)
  #
  invisible(details)
}

#' Transform data from pairwise comparisons to long arm-based format
#' 
#' @description
#' This function transforms data from pairwise comparisons to a long
#' arm-based format, i.e., two rows for a pairwise comparison.
#' 
#' @param treat1 Either label for first treatment or a meta-analysis
#'   or pairwise object (see Details).
#' @param treat2 Label for second treatment.
#' @param event1 Number of events (first treatment).
#' @param n1 Number of observations (first treatment).
#' @param event2 Number of events (second treatment).
#' @param n2 Number of observations (second treatment).
#' @param mean1 Estimated mean (first treatment).
#' @param sd1 Standard deviation (first treatment).
#' @param mean2 Estimated mean (second treatment).
#' @param sd2 Standard deviation (second treatment).
#' @param time1 Person time at risk (first treatment).
#' @param time2 Person time at risk (second treatment).
#' @param agent1 Agent (first treatment).
#' @param agent2 Agent (second treatment).
#' @param dose1 Dose (first treatment).
#' @param dose2 Dose (second treatment).
#' @param sep.ag A character used as separator between agent and dose to
#'   create treatment labels.
#' @param data An optional data frame containing the study
#'   information.
#' @param studlab A vector with study labels (optional).
#' @param id1 Last character(s) of variable names for additional
#'   variables with group specific information for first treatment.
#' @param id2 Last character(s) of variable names for additional
#'   variables with group specific information for second treatment.
#' @param append A logical indicating if data frame provided in
#'   argument 'data' should be returned.
#' @param keep.duplicated A logical indicating if duplicated rows
#'   should be returned (see Details).
#' @param keep.internal A logical indicating if variables generated
#'   internally should be returned (typically only relevant for data
#'   checking).
#' 
#' @details
#' This function transforms data given as one pairwise comparison per
#' row to a long arm-based format with one row per treatment arm. The
#' long arm-based format is, for example, the required input format
#' for WinBUGS.
#' 
#' The function can be used to transform data with a binary,
#' continuous or count outcome. The corresponding meta-analysis
#' functions are \code{\link{metabin}}, \code{\link{metacont}} and
#' \code{\link{metainc}}. Accordingly, a meta-analysis object created
#' with one of these functions can be provided as argument
#' \code{treat1}. It is also possible to use the longarm function with
#' an R objected created with \code{\link{pairwise}}.
#'
#' Otherwise, arguments \code{treat1} and \code{treat2} are mandatory
#' to identify the individual treatments and, depending on the
#' outcome, the following additional arguments are mandatory:
#' 
#' \itemize{
#' \item event1, n1, event2, n2 (binary outcome);
#' \item n1, mean1, sd1, n2, mean2, sd2 (continuous outcome);
#' \item time1, n1, time2, n2 (count outcome).
#' }
#' 
#' Argument \code{studlab} must be provided if several pairwise
#' comparisons come from a single study with more than two treatments.
#'
#' The following variables will be returned:
#'
#' \tabular{rl}{
#' \bold{\emph{studlab}}\tab study label \cr
#' \bold{\emph{treat}}\tab treatment label \cr
#' \bold{\emph{n}}\tab group sample size (count outcome only if provided) \cr
#' \bold{\emph{events}}\tab number of events (binary or count outcome) \cr
#' \bold{\emph{nonevents}}\tab number of non-events (binary outcome) \cr
#' \bold{\emph{mean}}\tab estimated mean (continuous outcome) \cr
#' \bold{\emph{sd}}\tab standard deviation (continuous outcome) \cr
#' \bold{\emph{time}}\tab person time at risk (count outcome) \cr
#' }
#'
#' In addition, the data set provided in argument \code{data} will be
#' returned if argument \code{append = TRUE} (default).
#' 
#' Argument \code{keep.duplicated} can be used to keep duplicated rows
#' from the data set. Duplicated rows can occur, for example, in a
#' three-arm study comparing treatments A and B with placebo. In this
#' situation, the placebo arm will be returned twice in the data set
#' in long arm-based format if \code{keep.duplicated = TRUE}. By
#' default, duplicated rows with not be kept in the data set.
#'
#' @note
#' R function \code{\link[metafor]{to.long}} from R package
#' \bold{metafor} is called internally.
#' 
#' @return
#' A data frame in long arm-based format.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metainc}}, \code{\link{pairwise}}
#' 
#' @keywords datagen
#' 
#' @examples
#' # Artificial example with three studies
#' m <- metabin(1:3, 100:102, 4:6, 200:202, studlab = LETTERS[1:3])
#' # Transform data to long arm-based format
#' longarm(m)
#' # Keep internal variables
#' longarm(m, keep.internal = TRUE)
#' 
#' @export longarm

longarm <- function(treat1, treat2,
                    event1, n1, event2, n2,
                    mean1, sd1, mean2, sd2,
                    time1, time2,
                    agent1, agent2,
                    dose1, dose2,
                    sep.ag = "*",
                    data = NULL, studlab,
                    id1 = NULL,
                    id2 = NULL,
                    append = TRUE,
                    keep.duplicated = FALSE,
                    keep.internal = FALSE) {
  
  #
  #
  # (1) Check arguments
  #
  #
  chklogical(append)
  chklogical(keep.duplicated)
  chklogical(keep.internal)
  
  
  #
  #
  # (2) Read data
  #
  #
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  # Catch treat1 and check for pairwise object
  #
  missing.treat1 <- missing(treat1)
  missing.treat2 <- missing(treat2)
  #
  dosres <-
    !(missing(agent1) | missing(agent2) |missing(dose1) | missing(dose2))
  #
  treat1 <- catch("treat1", mc, data, sfsp)
  #
  ignore <- function(miss, name, func)
    if (!miss)
      warning("Argument '", name,
              "' ignored for object created with ",
              func, "().",
              call. = FALSE)
  #
  if (inherits(treat1, c("metabin", "metacont", "metainc"))) {
    cls <- class(treat1)[1]
    #
    ignore(missing.treat2, "treat2", cls)
    ignore(missing(event1), "event1", cls)
    ignore(missing(event2), "event2", cls)
    ignore(missing(n1), "n1", cls)
    ignore(missing(n2), "n2", cls)
    ignore(missing(mean1), "mean1", cls)
    ignore(missing(mean2), "mean2", cls)
    ignore(missing(sd1), "sd1", cls)
    ignore(missing(sd2), "sd2", cls)
    ignore(missing(time1), "time1", cls)
    ignore(missing(time2), "time2", cls)
    #
    event1 <- event2 <- n1 <- n2 <-
      mean1 <- mean2 <- sd1 <- sd2 <-
        time1 <- time2 <- NULL
    #
    if (!is.null(treat1$data)) {
      data <- treat1$data
      if (isCol(data, ".subset"))
        data <- data[data$.subset, , keep = TRUE]
      if (isCol(data, ".exclude"))
        data <- data[!data$.exclude, , keep = TRUE]
    }
    else
      data <- as.data.frame(treat1)
    #
    data$.treat1 <- rep_len(treat1$label.e, nrow(data))
    data$.treat2 <- rep_len(treat1$label.c, nrow(data))
    #
    if (cls == "metabin") {     
      event1 <- data$.event.e
      event2 <- data$.event.c
      n1 <- data$.n.e
      n2 <- data$.n.c
      #
      type <- "binary"
    }
    else if (cls == "metacont") {
      n1 <- data$.n.e
      n2 <- data$.n.c
      mean1 <- data$.mean.e
      mean2 <- data$.mean.c
      sd1 <- data$.sd.e
      sd2 <- data$.sd.c
      #
      type <- "continuous"
    }
    else if (cls == "metainc") {
      event1 <- data$.event.e
      event2 <- data$.event.c
      time1 <- data$.time.e
      time2 <- data$.time.c
      n1 <- data$.n.e
      n2 <- data$.n.c
      #
      type <- "count"
    }
    #
    studlab <- data$.studlab
    treat2 <- data$.treat2
    treat1 <- data$.treat1
  }
  else if (inherits(treat1, "meta")) {
    cls <- class(treat1)
    cls <- cls[cls != "meta"]
    #
    stop("R function longarm() cannot be used with ",
         "meta-analysis object of class", if (length(cls) > 1) "es", " ",
         paste(paste0("'", cls, "'"), collapse = " - "), ".",
         call. = FALSE)
  }
  else if (is.data.frame(treat1) & !is.null(attr(treat1, "pairwise"))) {
    if (!nulldata)
      warning("Argument 'data' ignored for object created with pairwise().",
              call. = FALSE)
    #
    ignore(missing.treat2, "treat2", "pairwise")
    ignore(missing(event1), "event1", "pairwise")
    ignore(missing(event2), "event2", "pairwise")
    ignore(missing(n1), "n1", "pairwise")
    ignore(missing(n2), "n2", "pairwise")
    ignore(missing(mean1), "mean1", "pairwise")
    ignore(missing(mean2), "mean2", "pairwise")
    ignore(missing(sd1), "sd1", "pairwise")
    ignore(missing(sd2), "sd2", "pairwise")
    ignore(missing(time1), "time1", "pairwise")
    ignore(missing(time2), "time2", "pairwise")
    ignore(missing(agent1), "agent1", cls)
    ignore(missing(agent2), "agent2", cls)
    ignore(missing(dose1), "dose1", cls)
    ignore(missing(dose2), "dose2", cls)
    #
    event1 <- event2 <- n1 <- n2 <-
      mean1 <- mean2 <- sd1 <- sd2 <-
      time1 <- time2 <-
      agent1 <- agent2 <- dose1 <- dose2 <- NULL
    #
    if (attr(treat1, "type") == "binary") {
      event1 <- treat1$event1
      event2 <- treat1$event2
      n1 <- treat1$n1
      n2 <- treat1$n2
      #
      type <- "binary"
    }
    else if (attr(treat1, "type") == "continuous") {
      n1 <- treat1$n1
      n2 <- treat1$n2
      mean1 <- treat1$mean1
      mean2 <- treat1$mean2
      sd1 <- treat1$sd1
      sd2 <- treat1$sd2
      #
      type <- "continuous"
    }
    else if (attr(treat1, "type") == "count") {
      event1 <- treat1$event1
      event2 <- treat1$event2
      time1 <- treat1$time1
      time2 <- treat1$time2
      n1 <- treat1$n1
      n2 <- treat1$n2
      #
      type <- "count"
    }
    else
      stop("Function cannot be used with generic outcome.")
    #
    studlab <- treat1$studlab
    treat2 <- treat1$treat2
    #
    dosres <- !is.null(treat1$agent1) & !is.null(treat1$agent2) &
      !is.null(treat1$dose1) & !is.null(treat1$dose2)
    #
    if (dosres) {
      agent1 <- treat1$agent1
      agent2 <- treat1$agent2
      dose1 <- treat1$dose1
      dose2 <- treat1$dose2
    }
    #
    data <- treat1
    treat1 <- treat1$treat1
    #
    data$.treat1 <- treat1
    data$.treat2 <- treat2
    #
    if (dosres) {
      data$.agent1 <- agent1
      data$.agent2 <- agent2
      data$.dose1 <- dose1
      data$.dose2 <- dose2
    }
    #
    data$.event1 <- event1
    data$.event2 <- event2
    data$.n1 <- n1
    data$.n2 <- n2
    data$.mean1 <- mean1
    data$.mean2 <- mean2
    data$.sd1 <- sd1
    data$.sd2 <- sd2
    data$.time1 <- time1
    data$.time2 <- time2
    data$.studlab <- studlab
  }
  else {
    #
    # Catch studlab, treat2, event1, event2, n1, n2, mean1, mean2,
    # sd1, sd2, time1, time2 from data:
    #
    studlab <- catch("studlab", mc, data, sfsp)
    treat2 <- catch("treat2", mc, data, sfsp)
    event1 <- catch("event1", mc, data, sfsp)
    event2 <- catch("event2", mc, data, sfsp)
    n1 <- catch("n1", mc, data, sfsp)
    n2 <- catch("n2", mc, data, sfsp)
    mean1 <- catch("mean1", mc, data, sfsp)
    mean2 <- catch("mean2", mc, data, sfsp)
    sd1 <- catch("sd1", mc, data, sfsp)
    sd2 <- catch("sd2", mc, data, sfsp)
    time1 <- catch("time1", mc, data, sfsp)
    time2 <- catch("time2", mc, data, sfsp)
    #
    agent1 <- catch("agent1", mc, data, sfsp)
    agent2 <- catch("agent2", mc, data, sfsp)
    dose1 <- catch("dose1", mc, data, sfsp)
    dose2 <- catch("dose2", mc, data, sfsp)
    #
    if (!is.null(event1))
      chknumeric(event1)
    if (!is.null(event2))
      chknumeric(event2)
    #
    if (!is.null(n1))
      chknumeric(n1)
    if (!is.null(n2))
      chknumeric(n2)
    #
    if (!is.null(mean1))
      chknumeric(mean1)
    if (!is.null(mean2))
      chknumeric(mean2)
    #
    if (!is.null(sd1))
      chknumeric(sd1)
    if (!is.null(sd2))
      chknumeric(sd2)
    #
    if (!is.null(time1))
      chknumeric(time1)
    if (!is.null(time2))
      chknumeric(time2)
    #
    if (!is.null(event1) & !is.null(time1) &
        !is.null(event2) & !is.null(time2) &
        is.null(mean1) & is.null(sd1) &
        is.null(mean2) & is.null(sd2))
      type <- "count"
    else if (!is.null(event1) & !is.null(n1) &
             !is.null(event2) & !is.null(n2) &
             is.null(mean1) & is.null(sd1) &
             is.null(mean2) & is.null(sd2))
      type <- "binary"
    else if (!is.null(n1) & !is.null(n2) &
             !is.null(mean1) & !is.null(mean2) &
             !is.null(sd1) & !is.null(sd2))
      type <- "continuous"
    else
      stop("Type of outcome unclear. Please provide the necessary ",
           "information:\n  ",
           "- event1, n1, event2, n2 (binary outcome)\n  ",
           "- n1, mean1, sd1, n2, mean2, sd2 (continuous outcome)\n  ",
           "- event1, time1, event2, time2 (incidence rates).")
    #
    if (type == "count") {
      k.all <-
        max(length(event1), length(time1), length(event2), length(time2))
    }
    else if (type == "binary") {
      k.all <-
        max(length(event1), length(n1), length(event2), length(n2))
    }
    else if (type == "continuous") {
      k.all <-
        max(length(n1), length(mean1), length(sd1),
            length(n2), length(mean2), length(sd2))
    }
    #
    if (missing.treat1 & missing.treat2) {
      if (dosres) {
        treat1 <- paste(agent1, dose1, sep = sep.ag)
        treat2 <- paste(agent2, dose2, sep = sep.ag)
        #
        if (length(treat1) == 1)
          treat1 <- rep(treat1, k.all)
        #
        if (length(treat2) == 1)
          treat2 <- rep(treat2, k.all)
      }
      else {
        treat1 <- rep("B", k.all)
        treat2 <- rep("A", k.all)
      }
      #
      missing.treat2 <- FALSE
    }
    else if (!missing.treat1 && !missing.treat2 &&
             (length(treat1) == 1 | length(treat2) == 1)) {
      if (length(treat1) == 1)
        treat1 <- rep(treat1, k.all)
      if (length(treat2) == 1)
        treat2 <- rep(treat2, k.all)
    }
    #
    if (dosres && (length(agent1) == 1 | length(dose1) == 1)) {
      if (length(agent1) == 1)
        agent1 <- rep(agent1, k.all)
      if (length(dose1) == 1)
        dose1 <- rep(dose1, k.all)
    }
    #
    if (dosres && (length(agent2) == 1 | length(dose2) == 1)) {
      if (length(agent2) == 1)
        agent2 <- rep(agent2, k.all)
      if (length(dose2) == 1)
        dose2<- rep(dose2, k.all)
    }
    #
    if (missing.treat2)
      stop("Argument 'treat2' mandatory.")
    #
    studlab <- replaceNULL(studlab, seq_along(treat1))
    #
    # Keep data set
    #
    if (nulldata) {
      data <-
        data.frame(.studlab = studlab,
                   .treat1 = treat1, .treat2 = treat2,
                   .agent1 = if (dosres) agent1 else NA,
                   .dose1 = if (dosres) dose1 else NA,
                   .agent2 = if (dosres) agent2 else NA,
                   .dose2 = if (dosres) dose2 else NA,
                   .n1 = n1, .n2 = n2)
      data$.mean1 <- mean1
      data$.mean2 <- mean2
      data$.sd1 <- sd1
      data$.sd2 <- sd2
      data$.time1 <- time1
      data$.time2 <- time2
    }
    else {
      data$.treat1 <- treat1
      data$.treat2 <- treat2
      #
      if (dosres) {
        data$.agent1 <- agent1
        data$.agent2 <- agent2
        data$.dose1 <- dose1
        data$.dose2 <- dose2
      }
      #
      data$.event1 <- event1
      data$.event2 <- event2
      data$.n1 <- n1
      data$.n2 <- n2
      data$.mean1 <- mean1
      data$.mean2 <- mean2
      data$.sd1 <- sd1
      data$.sd2 <- sd2
      data$.time1 <- time1
      data$.time2 <- time2
      data$.studlab <- studlab
    }
  }
  
  
  #
  #
  # (3) Check length of variables
  #
  #
  k.all <- length(treat1)
  chklength(treat2, k.all, name = "treat1")
  #
  if (!is.null(event1))
    chklength(event1, k.all, name = "treat1")
  if (!is.null(event2))
    chklength(event2, k.all, name = "treat1")
  #
  if (!is.null(n1))
      chklength(n1, k.all, name = "treat1")
  if (!is.null(n2))
    chklength(n2, k.all, name = "treat1")
  #
  if (!is.null(mean1))
    chklength(mean1, k.all, name = "treat1")
  if (!is.null(mean2))
    chklength(mean2, k.all, name = "treat1")
  #
  if (!is.null(sd1))
    chklength(sd1, k.all, name = "treat1")
  if (!is.null(sd2))
    chklength(sd2, k.all, name = "treat1")
  #
  if (!is.null(time1))
    chklength(time1, k.all, name = "treat1")
  if (!is.null(time2))
    chklength(time2, k.all, name = "treat1")
  #
  if (is.null(studlab)) {
    studlab <- seq_len(k.all)
    data$.studlab <- studlab
  }
  
  
  #
  #
  # (4) Create data set in long arm-based format
  #
  #
  
  if (type == "binary") {
    dat.l <-
      to.long("RD",
              ai = event1, bi = n1 - event1,
              ci = event2, di = n2 - event2,
              slab = studlab,
              data = data,
              var.names = c(".id.m4", ".grp.m4",
                            "events", "nonevents"))
    dat.l$n <- dat.l$events + dat.l$nonevents
    nam <- c("studlab", "treat",
             if (dosres) "agent", if (dosres) "dose",
             "n", "events", "nonevents")
  }
  else if (type == "continuous") {
    dat.l <-
      to.long("MD",
              n1i = n1, m1i = mean1, sd1i = sd1,
              n2i = n2, m2i = mean2, sd2i = sd2,
              slab = studlab,
              data = data,
              var.names = c(".id.m4", ".grp.m4",
                            "mean", "sd", "n"))
    nam <- c("studlab", "treat",
             if (dosres) "agent", if (dosres) "dose",
             "n", "mean", "sd")
  }
  else if (type == "count") {
    dat.l <-
      to.long("IRD",
              x1i = event1, t1i = time1,
              x2i = event2, t2i = time2,
              slab = studlab,
              data = data,
              var.names = c(".id.m4", ".grp.m4", "events", "time"))
    #
    n.given <- FALSE
    if (!is.null(dat.l$.n1) & !is.null(dat.l$.n2)) {
      n.given <- TRUE
      dat.l$n <- ifelse(dat.l$.grp.m4 == 1, dat.l$.n1, dat.l$.n2)
    }
    else if (!is.null(dat.l$.n.e) & !is.null(dat.l$.n.c)) {
      n.given <- TRUE
      dat.l$n <- ifelse(dat.l$.grp.m4 == 1, dat.l$.n.e, dat.l$.n.c)
    } 
    #
    nam <- c("studlab", "treat",
             if (dosres) "agent", if (dosres) "dose",
             if (n.given) "n", "events", "time")
  }
  #
  dat.l$treat <- if_else(dat.l$.grp.m4 == 1, dat.l$.treat1, dat.l$.treat2)
  #
  if (dosres) {
    dat.l$agent <- if_else(dat.l$.grp.m4 == 1, dat.l$.agent1, dat.l$.agent2)
    dat.l$dose <- if_else(dat.l$.grp.m4 == 1, dat.l$.dose1, dat.l$.dose2)
  }
  #
  if (!isCol(dat.l, "studlab") & isCol(dat.l, ".studlab"))
    dat.l$studlab <- dat.l$.studlab
  #
  if (!keep.duplicated)
    dat.l <- dat.l[!duplicated(dat.l[, nam]), ]
  #
  # Catch additional variables with group specific information
  #
  if (!is.null(id1) & !is.null(id2) & !is.null(data)) {
    chklength(id1, 1)
    chklength(id2, 1)
    #
    ext1 <- paste0(id1, "$")
    ext2 <- paste0(id2, "$")
    #
    vars1 <- gsub(ext1, "", names(data)[grepl(ext1, names(data))])
    vars2 <- gsub(ext2, "", names(data)[grepl(ext2, names(data))])
    #
    j <- 0
    both <- character(0)
    for (i in seq_along(vars1)) {
      if (vars1[i] %in% vars2) {
        j <- j + 1
        both[j] <- vars1[i]
      }
    }
    #
    if (length(both) > 0) {
      bothlist <- list()
      for (i in seq_along(both)) {
        bothlist[[i]] <-
          data.frame(var1 = data[[paste0(both[i], id1)]],
                     var2 = data[[paste0(both[i], id2)]])
        names(bothlist)[[i]] <- both[i]
      }
      #
      for (var.i in both) {
        if (!(var.i %in% names(dat.l)))
          dat.l[[var.i]] <- addvars2long(bothlist[[var.i]])
      }
    }
  }
  #
  if (append) {
    allnames <- names(dat.l)
    other <- allnames[!(allnames %in% nam)]
    #
    nam.internal <-
      c(".studlab", ".treat1", ".treat2",
        #
        if (dosres) c(".agent1", ".agent2", ".dose1", ".dose2"),
        #
        ".event1", ".time1", ".n1", ".incr1", ".mean1", ".sd1",
        ".event2", ".time2", ".n2", ".incr2", ".mean2", ".sd2",
        #
        ".event.e", ".time.e", ".n.e", ".incr.e", ".mean.e", ".sd.e",
        ".event.c", ".time.c", ".n.c", ".incr.c", ".mean.c", ".sd.c",
        #
        ".incr",
        #
        ".subset", ".exclude", ".subgroup",
        #
        ".id.m4", ".grp.m4")
    #
    nam.internal <- nam.internal[nam.internal %in% other]
    other <- other[!(other %in% nam.internal)]
    #
    res <- dat.l[, c(nam, other, if (keep.internal) nam.internal)]
  }
  else
    res <- dat.l[, nam]
  
  
  attr(res, "type") <- type
  #
  class(res) <- unique(c("longarm", class(res)))
  #
  res
}

#' Calculate expected proportion of comparable studies with clinically important
#' benefit or harm
#'
#' @description
#' Calculate expected proportion of comparable studies with clinically important
#' benefit or harm which is derived from the prediction interval.
#'
#' @aliases cidprop cidprop.meta
#'
#' @param x An object of class \code{meta}.
#' @param cid A numeric value or vector specifying clinically important
#'   differences (CID) / decision thresholds used to calculate expected
#'   proportions of clinically important benefit or harm (see Details).
#' @param cid.below.null A numeric value or vector specifying CID limits below
#'   the null effect (see Details).
#' @param cid.above.null A numeric value or vector specifying CID limits above
#'   the null effect (see Details).
#' @param label.cid A character string or vector specifying labels for
#'   clinically important differences. Must be of same length as argument
#'   \code{cid}.
#' @param label.cid.below.null A character string or vector specifying labels
#'   for clinically important differences below the null effect. Must be of
#'   same length as argument \code{cid.below.null} (or \code{cid}).
#' @param label.cid.above.null A character string or vector specifying labels
#'   for clinically important differences above the null effect. Must be of
#'   same length as argument \code{cid.above.null} (or \code{cid}).
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}), can be abbreviated.
#' @param big.mark A character used as thousands separator.
#' @param digits.cid Minimal number of significant digits for
#'   CIDs / decision thresholds, see \code{print.default}.
#' @param digits.percent Minimal number of significant digits for
#'   expected proportions, printed as percentages, see \code{print.default}.
#' @param details.methods A logical specifying whether details on
#'   statistical methods should be printed.
#' @param \dots Additional arguments (ignored)
#' 
#' @details
#' Expected proportions of comparable studies with clinically important
#' benefit or harm are derived from the prediction interval in
#' the meta-analysis (Siemens et al., 2025).
#' 
#' Clinically important benefit or harm can be defined using either argument
#' \code{cid} or \code{cid.below.null} and \code{cid.above.null}.
#' Input for the later arguments will be ignored if argument \code{cid} was
#' specified. In this case, the values of \code{cid.below.null} and
#' \code{cid.above.null} will be equal to
#' \itemize{
#' \item \code{cid} and \code{1 / cid} for ratio measures,
#' \item \code{cid} and \code{-cid} for difference measures.
#' }
#' 
#' By default, i.e., if no input is provided for argument \code{cid}, the
#' null effect is used as threshold.
#' 
#' Thresholds based on argument \code{cid} will always be symmetric. Asymmetric
#' thresholds can be defined using arguments \code{cid.below.null} and
#' \code{cid.above.null}.
#'
#' @return
#' A list with elements
#' \item{prop.cid.below.null}{Expected proportion of comparable studies below
#'   lower CID(s)}
#' \item{prop.cid.above.null}{Expected proportion of comparable studies above
#'   upper CID(s)}
#' \item{prop.within.cid}{Expected proportion of comparable studies between
#'   lower and upper CID(s)}
#' \item{cid, cid.below.null, cid.above.null, small.values, x}{As defined
#'   above}
#' \item{label.cid, label.cid.below.null, label.cid.above.null}{As defined
#'   above}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{plot.cidprop}}
#' 
#' @references
#' Siemens W, Borenstein M, Evrenoglou T, Meerpohl JJ, Schwarzer G (2025):
#' Beyond prediction intervals in meta-analysis: reporting the expected
#' proportion of comparable studies with clinically relevant benefit or harm.
#' \emph{BMC Medical Research Methodology},
#' \bold{25}, 275
#' 
#' @examples
#' oldset <- settings.meta(digits.cid = 0)
#' 
#' m <- metagen(1:10 - 3, 1:10, sm = "MD")
#' #
#' pp1 <- cidprop(m, cid = 2)
#' pp1
#' #
#' pp2 <- cidprop(m, cid.below = 0.5, cid.above = 2)
#' pp2
#' #
#' pp3 <- cidprop(m, cid.below = 0.5, cid.above = 2, small.values = "u")
#' pp3
#' 
#' pp4 <- cidprop(m, cid = 1:2, label.cid = c("moderate", "large"))
#' pp4
#' #
#' pp5 <- cidprop(m, cid.below = -1.5, cid.above = 1:2,
#'   label.cid.below = "large", label.cid.above = c("moderate", "large"))
#' pp5
#' 
#' settings.meta(oldset)
#'
#' @rdname cidprop
#' @method cidprop meta
#' @export

cidprop.meta <- function(x,
                         cid = NULL,
                         cid.below.null = NULL, cid.above.null = NULL,
                         #
                         label.cid = "",
                         label.cid.below.null = NULL,
                         label.cid.above.null = NULL,
                         #
                         small.values = "desirable",
                         ...) {
  
  #
  #
  # (1) Check meta-analysis object and arguments
  #
  #
  
  chkclass(x, "meta")
  #
  sm <- x$sm
  is_relative <- is_relative_effect(sm)
  #
  missing.cid <- missing(cid)
  #
  if (is_prop(sm) | is_rate(sm) | is_mean(sm)) {
    if (is_prop(sm))
      outcome <- "proportions"
    else if (is_prop(sm))
      outcome <- "rates"
    else
      outcome <- "means"
    #
    ref <- replaceNULL(x$null.effect)
    #
    if (is.na(ref) & !missing.cid)
      stop("Argument 'cid' can only be used for meta-analysis of single ",
           outcome, " if argument 'null.effect' was specified.",
           call. = FALSE)
  }
  else if (is_relative)
    ref <- 1
  else
    ref <- 0
  #
  small.values <- setchar(small.values, c("desirable", "undesirable"))
  #
  # CID
  #
  avail.cid <- !missing.cid & !is.null(cid) & !all(is.na(cid))
  avail.cid.below.null <-
    !missing(cid.below.null) & !is.null(cid.below.null) &
    !all(is.na(cid.below.null))
  avail.cid.above.null <-
    !missing(cid.above.null) & !is.null(cid.above.null) &
    !all(is.na(cid.above.null))
  #
  if (!avail.cid & !avail.cid.below.null & !avail.cid.above.null) {
    cid <- x$null.effect
    #
    if (is.na(cid))
      stop("At least one decision threshold (argument 'cid', ",
           "'cid.below.null', or 'cid.above.null') must be specified.",
           call. = FALSE)
    else if (!(is_prop(x$sm) | is_rate(x$sm) | is_mean(x$sm)))
      cid <- backtransf(cid, x$sm)
    #
    avail.cid <- TRUE
  }
  #
  if (avail.cid) {
    if (any(is.na(cid)))
      stop("Missing values not allows in argument 'cid'.",
           call. = FALSE)
    #
    if (avail.cid.below.null + avail.cid.above.null == 2)
      warning("Arguments 'cid.below.null' and 'cid.above.null' ignored as ",
              "argument 'cid' is provided.",
              call. = FALSE)
    else if (avail.cid.below.null)
      warning(
        "Argument 'cid.below.null' ignored as argument 'cid' is provided.",
        call. = FALSE)
    else if (avail.cid.above.null)
      warning(
        "Argument 'cid.above.null' ignored as argument 'cid' is provided.",
        call. = FALSE)
    #
    if (any(diff(cid) <= 0))
      stop("Values for argument 'cid' must be increasing.",
           call. = FALSE)
    #
    if (any(cid < ref) & any(cid > ref))
      stop("All values provided for argument 'cid' must be either ",
           "smaller or larger than reference value of ", ref, ".",
           call. = FALSE)
    #
    if (is_prop(x$sm) | is_rate(x$sm) | is_mean(x$sm)) {
        cid.below.null <- cid
        cid.above.null <- cid
    }
    else {
      if (all(cid == ref)) {
        cid.below.null <- cid
        cid.above.null <- cid
      }
      else if (all(cid < ref)) {
        cid.below.null <- cid
        #
        if (is_relative)
          cid.above.null <- rev(1 / cid)
        else
          cid.above.null <- rev(-cid)
      }
      else {
        cid.above.null <- cid
        #
        if (is_relative)
          cid.below.null <- rev(1 / cid)
        else
          cid.below.null <- rev(-cid)
      }
    }
    #
    avail.cid.below.null <- TRUE
    avail.cid.above.null <- TRUE
  }
  #
  if (avail.cid.below.null) {
    chknumeric(cid.below.null)
    #
    cid.below.null.transf <- cid.below.null
    #
    if (is_relative)
      cid.below.null.transf <- log(cid.below.null)
  }
  else {
    cid.below.null <- NA
    cid.below.null.transf <- NA
  }
  #
  if (avail.cid.above.null) {
    chknumeric(cid.above.null)
    #
    cid.above.null.transf <- cid.above.null
    #
    if (is_relative)
      cid.above.null.transf <- log(cid.above.null)
  }
  else {
    cid.above.null <- NA
    cid.above.null.transf <- NA
  }
  #
  # CID labels
  #
  avail.label.cid <- !missing(label.cid) & !is.null(label.cid)
  avail.label.cid.below.null <-
    !missing(label.cid.below.null) & !is.null(label.cid.below.null)
  avail.label.cid.above.null <-
    !missing(label.cid.above.null) & !is.null(label.cid.above.null)
  #
  if (avail.cid & avail.label.cid && length(cid) != length(label.cid))
    stop("Arguments 'cid' and 'label.cid' must be of same length.",
         call. = FALSE)
  #
  if (avail.cid.below.null & avail.label.cid.below.null &&
      length(cid.below.null) != length(label.cid.below.null))
    stop("Arguments 'cid.below.null' and 'label.cid.below.null' must be of same length.",
         call. = FALSE)
  #
  if (avail.cid.above.null & avail.label.cid.above.null &&
      length(cid.above.null) != length(label.cid.above.null))
    stop("Arguments 'cid.above.null' and 'label.cid.above.null' must be of same length.",
         call. = FALSE)
  #
  if (avail.label.cid) {
    if (avail.label.cid.below.null + avail.label.cid.above.null == 2)
      warning("Arguments 'label.cid.below.null' and 'label.cid.above.null' ignored as ",
              "argument 'label.cid' is provided.",
              call. = FALSE)
    else if (avail.label.cid.below.null)
      warning("Argument 'label.cid.below.null' ignored as argument 'label.cid' is provided.",
              call. = FALSE)
    else if (avail.label.cid.above.null)
      warning("Argument 'label.cid.above.null' ignored as argument 'label.cid' is provided.",
              call. = FALSE)
    #
    #
    if (all(cid < ref)) {
      label.cid.below.null <- label.cid
      label.cid.above.null <- rev(label.cid)
    }
    else {
      label.cid.below.null <- rev(label.cid)
      label.cid.above.null <- label.cid
    }
    #
    avail.label.cid.below.null <- TRUE
    avail.label.cid.above.null <- TRUE
  }
  #
  if (!avail.label.cid.below.null)
    label.cid.below.null <-
    if (length(cid.below.null) == 1) "" else rev(seq_along(cid.below.null))
  #
  if (!avail.label.cid.above.null)
    label.cid.above.null <-
    if (length(cid.above.null) == 1) "" else seq_along(cid.above.null)
  
  
  #
  #
  # (2) Only consider results of first random effects meta-analysis
  #
  #
  
  method.predict <- x$method.predict[1]
  #
  TE.random <- x$TE.random[1]
  lower.random <- x$lower.random[1]
  upper.random <- x$upper.random[1]
  lower.predict <- x$lower.predict[1]
  upper.predict <- x$upper.predict[1]
  seTE.predict <- x$seTE.predict[1]
  df.predict <- x$df.predict[1]
  
  
  #
  #
  # (3) Calculate expected proportions
  #
  #
  
  prop.cid.below.null <-
    pt((cid.below.null.transf - TE.random) / seTE.predict, df.predict)
  #
  prop.cid.above.null <-
    pt((cid.above.null.transf - TE.random) / seTE.predict, df.predict,
       lower.tail = FALSE)
  #
  prop.within.cid <-
    1 -
    replaceNA(max(prop.cid.below.null), 0) -
    replaceNA(max(prop.cid.above.null), 0)
  #
  if (is_zero(prop.within.cid))
    prop.within.cid <- 0
  
  
  #
  #
  # (4) Return cidprop object
  #
  #
  
  res <- list(prop.cid.below.null = prop.cid.below.null, 
              prop.cid.above.null = prop.cid.above.null,
              prop.within.cid = prop.within.cid,
              #
              cid = cid,
              cid.below.null = cid.below.null, cid.above.null = cid.above.null,
              #
              label.cid = label.cid,
              label.cid.below.null = label.cid.below.null,
              label.cid.above.null = label.cid.above.null,
              #
              small.values = small.values,
              #
              ref = ref,
              #
              x = x)
  #
  class(res) <- "cidprop"
  #
  res
}


#' @rdname cidprop
#' @export cidprop

cidprop <- function(x, ...)
  UseMethod("cidprop")


#' @rdname cidprop
#' @method print cidprop
#' @export

print.cidprop <- function(x,
                           digits.cid = gs("digits.cid"), digits.percent = 1,
                           big.mark = gs("big.mark"),
                           details.methods = gs("details"),
                           ...) {
  
  #
  #
  # (1) Check arguments
  #
  #
    
  chkclass(x, "cidprop")
  #
  chknumeric(digits.cid, min = 0, length = 1)
  chknumeric(digits.percent, min = 0, length = 1)
  chkchar(big.mark, length = 1)
  chklogical(details.methods)
  
  #
  #
  # (2) Extract information from cidprop object
  #
  #
    
  cid.below.null <- x$cid.below.null
  cid.above.null <- x$cid.above.null
  #
  avail.cid.below.null <- !all(is.na(cid.below.null))
  avail.cid.above.null <- !all(is.na(cid.above.null))
  #
  prop.cid.below.null <- x$prop.cid.below.null
  prop.cid.above.null <- x$prop.cid.above.null
  prop.within.cid <- x$prop.within.cid
  #
  label.cid.below.null <- x$label.cid.below.null
  label.cid.above.null <- x$label.cid.above.null
  #
  # If meta-analysis object is available
  #
  if (!is.null(x$x)) {
    if (!x$x$backtransf) {
      cid.below.null <- transf(cid.below.null, x$x$sm)
      cid.above.null <- transf(cid.above.null, x$x$sm)
    }
    #
    if (x$x$sm == "")
      smlab <- ""
    else if (is_prop(x$x$sm))
      smlab <- "P "
    else if (is_rate(x$x$sm))
      smlab <- "R "
    else if (is_mean(x$x$sm))
      smlab <- "Mean "
    else
      smlab <-
        paste0(smlab(x$x$sm, x$x$backtransf), " ")
    #
    crtitle(x$x)
  }
  else
    smlab <- ""
  #
  svd <- x$small.values == "desirable"
  
  #
  #
  # (3) Data set with CID information
  #
  #
  
  dat.l <- dat.u <- dat.w <- NULL
  #
  if (avail.cid.below.null) {
    dat.l <-
      data.frame(Threshold = cid.below.null, prop = prop.cid.below.null,
                 label = label.cid.below.null,
                 category =
                   if (svd) "Beneficial effect" else "Harmful effect",
                 sign = "\u2264 ")
    #
    max.cid.below.null <- max(cid.below.null, na.rm = TRUE)
  }
  #
  if (avail.cid.above.null) {
    dat.u <-
      data.frame(Threshold = cid.above.null, prop = prop.cid.above.null,
                 label = label.cid.above.null,
                 category =
                   if (svd) "Harmful effect" else "Beneficial effect",
                 sign = "\u2265 ")
    #
    min.cid.above.null <- min(cid.above.null, na.rm = TRUE)
    #
    if (avail.cid.below.null & prop.within.cid ==  0) {
      dat.u %<>%
        mutate(sign = if_else(Threshold == min.cid.above.null, "> ", sign))
    }
  }
  #
  if (prop.within.cid > 0) {
    dat.w <- data.frame(Threshold = NA,
                        prop = prop.within.cid,
                        label = "",
                        category = "Not important effect",
                        sign = "")
    #
    if (avail.cid.below.null & avail.cid.above.null) {
      within.cid <-
        formatN(c(max.cid.below.null, min.cid.above.null),
                digits = digits.cid, big.mark = big.mark)
      #
      within.cid <- paste(within.cid[1], "to", within.cid[2])
    }
    else if (avail.cid.below.null) {
      within.cid <-
        formatN(max.cid.below.null, digits = digits.cid, big.mark = big.mark)
      #
      within.cid <- paste(">", within.cid)
    }
    else if (avail.cid.above.null) {
      within.cid <-
        formatN(min.cid.above.null, digits = digits.cid, big.mark = big.mark)
      #
      within.cid <- paste("<", within.cid)
    }
  }
  else
    within.cid <- ""
  #
  Threshold <- prop <- label <- category <- sign <- NULL
  #
  three.categories <- !is.null(dat.l) + !is.null(dat.w) + !is.null(dat.u)
  dat.cid <- rbind(dat.l, dat.w, dat.u)
  #
  dat.cid %<>%
    mutate(Threshold = formatN(Threshold, digits = digits.cid,
                               big.mark = big.mark, text.NA = ""))
  #
  if (three.categories) {
    dat.cid %<>%
      mutate(Threshold = if_else(category != "Not important effect",
                                 paste0(smlab, sign, Threshold), within.cid))
  }
  else {
    dat.cid %<>%
      mutate(Threshold = if_else(category != "Not important effect",
                                 paste0(smlab, sign, Threshold),
                                 paste0(smlab, within.cid)))
  }
  #
  dat.cid %<>%
    mutate(
      prop = paste0(formatPT(100 * prop, digits = digits.percent), "%"),
      category =
        if_else(label == "", category, paste(category, label)),
      category = paste0(category, " ")) %>%
    column_to_rownames("category") %>%
    rename(Percent = prop) %>%
    select(-label, -sign)
  #
  print(dat.cid)
  #
  if (details.methods & !is.null(x$x)) {
    catmeth(x$x,
            FALSE, FALSE, TRUE, FALSE, TRUE,
            #
            func.transf = x$x$func.transf, backtransf = FALSE,
            func.backtransf = x$x$func.backtransf,
            #
            big.mark = "", digits = 2,
            digits.tau = gs("digits.tau"),
            text.tau = gs("text.tau"), text.tau2 = gs("text.tau2"),
            #
            print.tau2 = TRUE,
            print.tau2.ci = FALSE,
            print.tau = TRUE,
            print.tau.ci = FALSE,
            #
            print.I2 = FALSE, text.I2 = "",
            #
            print.df = TRUE, prediction.subgroup = FALSE)
  }
  #
  invisible(NULL)
}

#' Plot density of prediction distribution highlighting areas of clinically
#' important benefit or harm
#'
#' @description
#' Plot density of prediction distribution highlighting areas of clinically
#' important benefit or harm
#'
#' @param x An object of class \code{cidprop}.
#' @param cid A numeric value or vector specifying clinically important
#'   differences (CID) / decision thresholds used to calculate expected
#'   proportions of clinically important benefit or harm (see Details).
#' @param cid.below.null A numeric value or vector specifying CID limits below
#'   the null effect (see Details).
#' @param cid.above.null A numeric value or vector specifying CID limits above
#'   the null effect (see Details).
#' @param label.cid A character string or vector specifying labels for
#'   clinically important differences. Must be of same length as argument
#'   \code{cid}.
#' @param label.cid.below.null A character string or vector specifying labels
#'   for clinically important differences below the null effect. Must be of
#'   same length as argument \code{cid.below.null} (or \code{cid}).
#' @param label.cid.above.null A character string or vector specifying labels
#'   for clinically important differences above the null effect. Must be of
#'   same length as argument \code{cid.above.null} (or \code{cid}).
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}), can be abbreviated.
#' @param fill.cid.below.null Background colour(s) for CID areas below null
#'   effect.
#' @param fill.cid.above.null Background colour(s) for CID areas above null
#'   effect.
#' @param fill Background colour for area between decision thresholds.
#' @param legend A logical indicating whether to print a legend with
#'   expected proportions of beneficial, harmful, or not important effects.
#' @param studies A logical indicating whether to print estimates of individual
#'   studies.
#' @param random A logical indicating whether to show diamond of the random
#'   effects meta-analysis.
#' @param col.diamond The colour of the diamond representing the results
#'   for the random effects model.
#' @param col.diamond.lines The colour of the outer lines of the diamond
#'   representing the results of the random effects model.
#' @param prediction A logical indicating whether to show the prediction
#'   interval.
#' @param col.predict The colour of the prediction interval.
#' @param col.predict.lines The colour of the outer lines of the prediction
#'   interval.
#' @param big.mark A character used as thousands separator.
#' @param digits.cid Minimal number of significant digits for
#'   decision thresholds, see \code{\link{print.default}}.
#' @param digits.percent Minimal number of significant digits for
#'   expected proportions, printed as percentages, see
#'   \code{\link{print.default}}.
#' @param digits.xaxis Minimal number of significant digits for
#'   labels on x-axis, see \code{\link{print.default}}.
#' @param xlab Label on x-axis.
#' @param ylab Label on y-axis.
#' @param xlim Limits for x-axis.
#' @param ylim Limits for y-axis.
#' @param labels.x Predefined labels for tick marks on x-axis.
#' @param \dots Additional arguments (ignored)
#' 
#' @details
#' This function plots the density of the prediction distribution highlighting
#' areas of clinically important benefit or harm (Siemens et al., 2025).
#' 
#' Arguments \code{cid}, \code{cid.below.null}, \code{cid.above.null},
#' \code{label.cid}, \code{label.cid.below.null}, \code{label.cid.above.null},
#' and \code{small.values} are identical to the main arguments of R function
#' \code{\link{cidprop}} which is called internally if any of these values has
#' been provided by the user.
#' 
#' R packages \bold{ggpubr} and \bold{gridExtra} must be installed in order to
#' add a legend to the plot with the CIDs, expected proportions of clinically
#' benefit or harm, and the area colours (due to using R functions
#' \code{ggarrange} and \code{tableGrob}). The data and colours shown in the
#' legend are stored in the attribute 'data.cid' of the returned ggplot object
#' (see Examples).
#' 
#' UTF-8 code for the less than or equal and greater than or equal signs are
#' used in the legend. Accordingly, graphic devices with full UTF-8 support are
#' required to save graphics, for example, \code{\link[grDevices]{cairo_pdf}}
#' instead of \code{\link[grDevices]{pdf}} from R package \bold{grDevices}.
#' 
#' @return
#' A ggplot object with additional class 'plot.cidprop'.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#'
#' @seealso \code{\link{cidprop}}
#' 
#' @references
#' Siemens W, Borenstein M, Evrenoglou T, Meerpohl JJ, Schwarzer G (2025):
#' Beyond prediction intervals in meta-analysis: reporting the expected
#' proportion of comparable studies with clinically relevant benefit or harm.
#' \emph{BMC Medical Research Methodology},
#' \bold{25}, 275
#'
#' @examples
#' oldset <- settings.meta(digits.cid = 0)
#' 
#' m <- metagen(1:10 - 3, 1:10, sm = "MD")
#' 
#' pp1 <- cidprop(m, cid = 2)
#' pp1
#' plot(pp1, xlim = c(-4, 4))
#' \donttest{
#' pp2 <- cidprop(m, cid.below.null = 0.5, cid.above.null = 2)
#' pp2
#' plot(pp2, xlim = c(-4, 4))
#' 
#' pp3 <- cidprop(m, cid.below.null = 0.5, cid.above.null = 2,
#'   small.values = "u")
#' pp3
#' plot(pp3, xlim = c(-4, 4))
#' 
#' pp4 <- cidprop(m, cid = 1:2, label.cid = c("moderate", "large"))
#' pp4
#' plot(pp4, xlim = c(-4, 4))
#' 
#' pp5 <- cidprop(m, cid.below.null = -1.5, cid.above.null = 1:2,
#'   label.cid.below.null = "large",
#'   label.cid.above.null = c("moderate", "large"))
#' pp5
#' plpp5 <- plot(pp5, xlim = c(-4, 4))
#' plpp5
#' # Information on CIDs and colours
#' attr(plpp5, "data.cid")
#' }
#' \dontrun{
#' # R packages 'ggpubr' and 'gridExtra' must be available
#' if (requireNamespace("ggpubr", quietly = TRUE) &
#'     requireNamespace("gridExtra", quietly = TRUE)) {
#'   plot(pp1, xlim = c(-4, 4), legend = TRUE)
#' }
#' }
#' 
#' settings.meta(oldset)
#' 
#' @method plot cidprop
#' @export

plot.cidprop <- function(x,
                         cid = NULL,
                         cid.below.null = x$cid.below.null,
                         cid.above.null = x$cid.above.null,
                         #
                         label.cid = "",
                         label.cid.below.null = x$label.cid.below.null,
                         label.cid.above.null = x$label.cid.above.null,
                         #
                         small.values = x$small.values,
                         #
                         fill.cid.below.null = NULL,
                         fill.cid.above.null = NULL,
                         fill = "white",
                         #
                         legend = FALSE,
                         #
                         studies = TRUE,
                         #
                         random = TRUE,
                         col.diamond = gs("col.diamond"),
                         col.diamond.lines = gs("col.diamond.lines"),
                         #
                         prediction = TRUE,
                         col.predict = gs("col.predict"),
                         col.predict.lines = gs("col.predict.lines"),
                         #
                         big.mark = gs("big.mark"),
                         digits.cid = gs("digits.cid"), digits.percent = 1,
                         #
                         digits.xaxis = gs("digits.forest"),
                         #
                         xlab = NULL, ylab = NULL,
                         xlim = NULL, ylim = NULL,
                         labels.x = NULL,
                         #
                         ...) {
  
  #
  #
  # (1) Check arguments
  #
  #
  
  chkclass(x, "cidprop")
  #
  missing.xlim <- missing(xlim)
  #
  chklogical(legend)
  chklogical(studies)
  chklogical(random)
  chklogical(prediction)
  #
  chknumeric(digits.cid, min = 0, length = 1)
  chknumeric(digits.percent, min = 0, length = 1)
  chknumeric(digits.xaxis, min = 0, length = 1)
  #
  if (legend) {
    if (!is_installed_package("ggpubr", argument = "legend", value = TRUE,
                              stop = FALSE)) {
      warning("Please install R package 'ggpubr' in order to print a legend.",
      call. = FALSE)
      #
      legend <- FALSE
    }
  }
  
  #
  #
  # (2) Re-run cidprop() if settings changed
  #
  #
  
  avail.cid <- !missing(cid) & !is.null(cid) & !all(is.na(cid))
  avail.cid.below.null <- !missing(cid.below.null) &
    !is.null(cid.below.null) & !all(is.na(cid.below.null))
  avail.cid.above.null <- !missing(cid.above.null) &
    !is.null(cid.above.null) & !all(is.na(cid.above.null))
  #
  avail.label.cid <- !missing(label.cid)
  avail.label.cid.below.null <- !missing(label.cid.below.null)
  avail.label.cid.above.null <- !missing(label.cid.above.null)
  #
  avail.small.values <- !missing(small.values)
  #
  # Rerun cidprop()
  #
  if (avail.cid | avail.cid.below.null | avail.cid.above.null |
      avail.label.cid |
      avail.label.cid.below.null | avail.label.cid.above.null |
      avail.small.values) {
    #
    if (avail.cid) {
      if (any(is.na(cid)))
        stop("Missing values not allows in argument 'cid'.",
             call. = FALSE)
      #
      if (avail.cid.below.null + avail.cid.above.null == 2)
        warning("Arguments 'cid.below.null' and 'cid.above.null' ignored as ",
                "argument 'cid' is provided.",
                call. = FALSE)
      else if (avail.cid.below.null)
        warning("Argument 'cid.below.null' ignored as ", 
                "argument 'cid' is provided.",
                call. = FALSE)
      else if (avail.cid.above.null)
        warning("Argument 'cid.above.null' ignored as ",
                "argument 'cid' is provided.",
                call. = FALSE)
      #
      x <- cidprop(x$x,
                   cid = cid, label.cid = label.cid,
                   small.values = small.values)
    }
    else {
      x <- cidprop(x$x,
                   cid.below.null = cid.below.null,
                   cid.above.null = cid.above.null,
                   label.cid.below.null = label.cid.below.null,
                   label.cid.above.null = label.cid.above.null,
                   small.values = small.values)
    }
  }
  
  #
  #
  # (3) Extract results from cidprop-object
  #
  #
    
  x.meta <- x$x
  #
  sm <- x.meta$sm
  smlab <- smlab(sm, x.meta$backtransf, x.meta$pscale, x.meta$irscale)
  #
  if (is.null(xlab))
    xlab <- xlab_meta(sm, backtransf = TRUE)
  #
  if (is.null(ylab))
    ylab <- "Density"
  #
  is_relative <- is_relative_effect(sm)
  #
  cid.below.null <- x$cid.below.null
  cid.above.null <- x$cid.above.null
  #
  if (is_relative) {
    cid.below.null.transf <- log(cid.below.null)
    cid.above.null.transf <- log(cid.above.null)
  }
  else {
    cid.below.null.transf <- cid.below.null
    cid.above.null.transf <- cid.above.null
  }
  #
  label.cid.below.null <- x$label.cid.below.null
  label.cid.above.null <- x$label.cid.above.null
  #
  svd <- x$small.values == "desirable"
  #
  prop.cid.below.null <- x$prop.cid.below.null
  prop.cid.above.null <- x$prop.cid.above.null
  prop.within.cid <- x$prop.within.cid
  #
  method.predict <- x.meta$method.predict[1]
  #
  TE <- x.meta$TE
  lower <- x.meta$lower
  upper <- x.meta$upper
  w.random <- x.meta$w.random
  #
  TE.random <- x.meta$TE.random[1]
  lower.random <- x.meta$lower.random[1]
  upper.random <- x.meta$upper.random[1]
  lower.predict <- x.meta$lower.predict[1]
  upper.predict <- x.meta$upper.predict[1]
  seTE.predict <- x.meta$seTE.predict[1]
  df.predict <- x.meta$df.predict[1]
  #
  avail.cid.below.null <- !all(is.na(cid.below.null))
  avail.cid.above.null <- !all(is.na(cid.above.null))
  
  #
  #
  # (4) Get / set colors for CID areas
  #
  #
  
  if (svd) {
    if (length(cid.below.null) == 1)
      fill.cid.below.null <- replaceNULL(fill.cid.below.null, "lightgreen")
    else
      fill.cid.below.null <-
        replaceNULL(fill.cid.below.null,
                    hcl.colors(length(cid.below.null) + 1, palette = "Greens",
                               alpha = 1)[seq_along(cid.below.null)])
    #
    if (length(cid.above.null) == 1)
      fill.cid.above.null <- replaceNULL(fill.cid.above.null, "lightpink1")
    else
      fill.cid.above.null <-
        replaceNULL(fill.cid.above.null,
                    hcl.colors(length(cid.above.null) + 1, palette = "Reds",
                               alpha = 1)[rev(seq_along(cid.above.null))])
  }
  else {
    if (length(cid.below.null) == 1)
      fill.cid.below.null <- replaceNULL(fill.cid.below.null, "lightpink1")
    else
      fill.cid.below.null <-
        replaceNULL(fill.cid.below.null,
                    hcl.colors(length(cid.below.null) + 1, palette = "Reds",
                               alpha = 1)[seq_along(cid.below.null)])
    #
    if (length(cid.above.null) == 1)
      fill.cid.above.null <- replaceNULL(fill.cid.above.null, "lightgreen")
    else
      fill.cid.above.null <-
        replaceNULL(fill.cid.above.null,
                    hcl.colors(length(cid.above.null) + 1, palette = "Greens",
                               alpha = 1)[rev(seq_along(cid.above.null))])
  }
  #
  if (length(cid.below.null) != length(fill.cid.below.null))
    stop("Different length for arguments 'cid.below.null' and ",
         "'fill.cid.below.null'.",
         call. = FALSE)
  #
  if (length(cid.above.null) != length(fill.cid.above.null))
    stop("Different length for arguments 'cid.above.null' and ",
         "'fill.cid.above.null'.",
         call. = FALSE)
  #
  if (length(fill) != 1)
    stop("Argument 'fill' must be a single colour",
         call. = FALSE)
  
  #
  #
  # (5) Data set with CID information
  #
  #
  
  dat.l <- dat.u <- dat.w <- NULL
  #
  if (avail.cid.below.null) {
    dat.l <-
      data.frame(Colour = fill.cid.below.null,
                 Threshold = cid.below.null, prop = prop.cid.below.null,
                 label = label.cid.below.null,
                 category =
                   if (svd) "Beneficial effect" else "Harmful effect",
                 sign = "\u2264 ")
    #
    max.cid.below.null <- max(cid.below.null, na.rm = TRUE)
  }
  #
  if (avail.cid.above.null) {
    dat.u <-
      data.frame(Colour = fill.cid.above.null,
                 Threshold = cid.above.null, prop = prop.cid.above.null,
                 label = label.cid.above.null,
                 category =
                   if (svd) "Harmful effect" else "Beneficial effect",
                 sign = "\u2265 ")
    #
    min.cid.above.null <- min(cid.above.null, na.rm = TRUE)
    #
    if (avail.cid.below.null & prop.within.cid ==  0) {
      dat.u %<>%
        mutate(sign = if_else(Threshold == min.cid.above.null, "> ", sign))
    }
  }
  #
  if (prop.within.cid > 0) {
    dat.w <- data.frame(Colour = fill,
                        Threshold = NA,
                        prop = prop.within.cid,
                        label = "",
                        category = "Not important effect",
                        sign = "")
    #
    if (avail.cid.below.null & avail.cid.above.null) {
      within.cid <-
        formatN(c(max.cid.below.null, min.cid.above.null),
                digits = digits.cid, big.mark = big.mark)
      #
     within.cid <- paste(within.cid[1], "to", within.cid[2])
    }
    else if (avail.cid.below.null) {
      within.cid <-
        formatN(max.cid.below.null, digits = digits.cid, big.mark = big.mark)
      #
      within.cid <- paste(">", within.cid)
    }
    else if (avail.cid.above.null) {
      within.cid <-
        formatN(min.cid.above.null, digits = digits.cid, big.mark = big.mark)
      #
      within.cid <- paste("<", within.cid)
    }
  }
  else
    within.cid <- ""
  #
  dat.cid <- rbind(dat.l, dat.w, dat.u)
  #
  Threshold <- prop <- label <- category <- sign <- NULL
  #
  dat.cid %<>%
    mutate(Threshold = formatN(Threshold, digits = digits.cid,
                               big.mark = big.mark, text.NA = ""),
           Threshold = if_else(category != "Not important effect",
                               paste0(sign, Threshold), within.cid),
           prop = paste0(formatPT(100 * prop, digits = digits.percent), "%"),
           category =
             if_else(label == "", category, paste(category, label))) %>%
    column_to_rownames("category") %>%
    rename(Percent = prop) %>%
    select(-label, -sign)
  
  #
  #
  # (6) Data set for plot
  #
  #
  
  # Get rid of warnings
  #
  cid.category <- xval <- yval <- y <- NULL
  #
  vals <- c(lower, upper, lower.predict, upper.predict)
  #
  if (missing.xlim) {
    from <- min(vals, na.rm = TRUE)
    to <- max(vals, na.rm = TRUE)
  }
  else {
    chknumeric(xlim, length = 2)
    #
    from <- if (is_relative) log(xlim[1]) else xlim[1]
    to <- if (is_relative) log(xlim[2]) else xlim[2]
  }
  #
  by <- (to - from) / 1000
  #
  seq <- c(seq(from, to, by),
           TE.random + c(-1e-8, 0, 1e-8), 
           lower.predict + c(-1e-8, 0, 1e-8),
           upper.predict + c(-1e-8, 0, 1e-8))
  #
  if (avail.cid.below.null) {
    for (i in seq_along(cid.below.null))
      seq <- c(seq, cid.below.null.transf[i] + c(-1e-8, 0, 1e-8))
  }
  #
  if (avail.cid.above.null) {
    for (i in seq_along(cid.above.null))
      seq <- c(seq, cid.above.null.transf[i] + c(-1e-8, 0, 1e-8))
  }
  #
  seq <- unique(sort(seq))
  #
  # Calculate density
  #
  dens <- dt((seq - TE.random) / seTE.predict, df.predict)
  #
  dat <- tibble(xval = seq, yval = dens)
  #
  if (avail.cid.below.null & avail.cid.above.null) {
    dat$cid.category <-
      cut(dat$xval,
          breaks = unique(c(-Inf, cid.below.null.transf,
                            cid.above.null.transf, Inf)))
    #
    if (prop.within.cid == 0)
      fill.category <- c(fill.cid.below.null, fill.cid.above.null)
    else
      fill.category <- c(fill.cid.below.null, fill, fill.cid.above.null)
  }
  else if (avail.cid.below.null) {
    dat$cid.category <-
      cut(dat$xval, breaks = c(-Inf, cid.below.null.transf, Inf))
    #
    fill.category <- c(fill.cid.below.null, fill)
  }
  #
  else if (avail.cid.above.null) {
    dat$cid.category <-
      cut(dat$xval, breaks = c(-Inf, cid.above.null.transf, Inf))
    #
    fill.category <- c(fill, fill.cid.above.null)
  }
  #
  if (is_relative) {
    dat$xval <- exp(dat$xval)
    #
    seq <- exp(seq)
  }
  #
  # Only keep values within limits of x-axis
  #
  if (!missing.xlim)
    dat %<>% filter(xval >= xlim[1] & xval <= xlim[2])
  #
  # Data for diamond of random effects model
  #
  x.diamond <- c(TE.random, upper.random, TE.random, lower.random)
  dat.diamond <- data.frame(
    x = if (is_relative) exp(x.diamond) else x.diamond,
    y = -0.03 + c(0.005, 0, -0.005, 0)
  )
  #
  # Data for prediction interval
  #
  dat.predict <- data.frame(
    x1 = if (is_relative) exp(lower.predict) else lower.predict,
    x2 = if (is_relative) exp(upper.predict) else upper.predict,
    y1 = -0.045, y2 = -0.05)
  
  #
  #
  # (7) Create plot
  #
  #
  
  p <- ggplot(dat, aes(xval, yval)) +
    geom_line() +
    geom_area(fill = fill) +
    # Add diamond for random effects model
    geom_polygon(data = dat.diamond, aes(x, y), 
                 fill = col.diamond, color = col.diamond.lines) +
    # Add prediction interval
    annotate("rect",
             xmin = if (is_relative) exp(lower.predict) else lower.predict,
             xmax = if (is_relative) exp(upper.predict) else upper.predict,
             ymin = -0.05, ymax = -0.045,
             fill = col.predict, color = col.predict.lines) +
  xlab(xlab) + ylab(ylab)
  #
  if (avail.cid.below.null | avail.cid.above.null) {
    levs.category <- levels(dat$cid.category)
    #
    for (i in seq_along(levs.category)) {
      p <- p +
        geom_area(data = filter(dat, cid.category == levs.category[i]),
                  fill = fill.category[i])
    }
  }
  #
  # Add individual study results
  #
  if (studies) {
    dat.TE <-
      data.frame(TE = if (is_relative) exp(TE) else TE,
                 yval = 0, size = 10 * w.random / max(w.random))
    #
    if (!missing.xlim)
      dat.TE %<>% filter(TE >= xlim[1] & TE <= xlim[2])
    #
    p <- p +
      geom_point(data = dat.TE, aes(x = TE, y = yval),
                 shape = 1, size = dat.TE$size)
  }
  #
  if (!is.null(ylim)) {
    chknumeric(ylim, length = 2)
    #
    p <- p + ylim(ylim[1], ylim[2])
  }
  #
  # Add labels on x-axis
  #
  if (missing(labels.x)) {
    if (is_relative)
      p <- p +
        scale_x_continuous(
          trans = "log",
          labels = number_format(accuracy = 1 / 10^digits.xaxis))
    else
      p <- p +
        scale_x_continuous(
          labels = number_format(accuracy = 1 / 10^digits.xaxis))
  }
  else {
    if (is_relative)
      p <- p +
        scale_x_continuous(trans = "log", breaks = labels.x, labels = labels.x)
    else
      p <- p +
        scale_x_continuous(breaks = labels.x, labels = labels.x)
  }
  #
  # Add legend with expected proportions
  #
  if (legend) {
    dat.legend <- dat.cid %>% mutate(Colour = "")
    #
    gtab <- gridExtra::tableGrob(dat.legend)
    #
    for (i in seq_len(nrow(gtab$layout))) {
      cell <- gtab$layout[i, ]
      # First column in data set is equal to 'cell$l == 2'
      if (cell$l == 2 && cell$name %in% c("core-fg", "colhead-fg")) {
        #
        row_index <- cell$t
        # Coloured background for each cell in the target column
        if (row_index > 1)
          gtab$grobs[[i]] <-
            grobTree(
              rectGrob(gp =
                         gpar(fill = dat.cid$Colour[row_index - 1], col = NA)),
              gtab$grobs[[i]]
          )
      }
    }
    #
    p <- ggpubr::ggarrange(p, gtab, ncol = 1, nrow = 2, heights = c(3, 1))
  }
  #
  attr(p, "data.cid") <- dat.cid
  #
  class(p) <- c(class(p), "plot.cidprop")
  #
  p
}

#
#
# Definition of auxiliary functions to import RevMan 5 analysis datasets
#
#


extract_outcomes <- function(txt, outcome.type, res,
                             comp.no, complab,
                             debug = 0) {
  #
  pattern1 <- paste0("<", outcome.type, "_OUTCOME")
  pattern2 <- paste0("</", outcome.type, "_OUTCOME")
  sel1 <- grep(pattern1, txt)
  sel2 <- grep(pattern2, txt) 
  #
  if (length(sel1) > 0 | length(sel2) > 0) {
    #
    if (debug)
      cat(paste0("\n*** ", outcome.type, "_OUTCOME ***\n"))
    #
    if (length(sel1) != length(sel2))
      stop("Malformed XML file (tag: ", outcome.type, "_OUTCOME)")
    #
    for (j in seq(along = sel1)) {
      txt.j <- txt[sel1[j]:sel2[j]]
      sel.data.j <- c(grep(paste0("<", outcome.type, "_DATA"), txt.j) - 1,
                      grep(paste0("<", outcome.type, "_SUBGROUP"), txt.j) - 1)
      if (outcome.type %in% c("IV", "IPD"))
        sel.data.j <- c(sel.data.j, grep("</EFFECT_MEASURE>", txt.j))
      else
        sel.data.j <- c(sel.data.j, grep("</GRAPH_LABEL_2>", txt.j))
      if (length(sel.data.j) > 0)
        sel.j <- c(1:(max(c(6, min(sel.data.j)))), length(txt.j))
      else
        sel.j <- c(1:6, length(txt.j))
      #
      xml.j <- as_xml_document(paste(txt.j[sel.j], collapse = " "))
      #
      outcome.no <- as.numeric(xml_attr(xml.j, "NO"))
      outclab <- xml_text(xml_find_all(xml.j, "//NAME"))
      #
      if (debug) {
        cat(paste0("Outcome.no: ", outcome.no, "\n"))
        cat(paste0("outclab: ", outclab, "\n"))
        #
        if (debug == 3)
          print(txt.j[sel.j])
      }
      #
      overall <- xml_attr(xml.j, "TOTALS") == "YES"
      test.subgroup <- xml_attr(xml.j, "SUBGROUP_TEST") == "YES"
      #
      n.studies <- as.numeric(xml_attr(xml.j, "DF")) + 1
      #
      type <- switch(outcome.type,
                     DICH = "D",
                     CONT = "C",
                     IV = "I",
                     IPD = "I")
      #
      method <- xml_attr(xml.j, "METHOD")
      if (is.na(method) & outcome.type == "IPD")
        method <- "Peto"
      else if (is.na(method))
        method <- "Inverse"
      else if (method == "IV")
        method <- "Inverse"
      #
      sm <- xml_attr(xml.j, "EFFECT_MEASURE")
      if (is.na(sm)) {
        sm <- xml_text(xml_find_all(xml.j, "//EFFECT_MEASURE"))
        sm[tolower(sm) == "odds ratio"] <- "OR"
        sm[tolower(sm) == "peto odds ratio"] <- "OR"
        sm[tolower(sm) == "peto_or"] <- "OR"
        sm[tolower(sm) == "odds ratio (non-event)"] <- "OR"
        sm[tolower(sm) == "risk ratio"] <- "RR"
        sm[tolower(sm) == "risk difference"] <- "RD"
        sm[tolower(sm) == "mean difference"] <- "MD"
        sm[tolower(sm) == "standardized mean difference"] <- "SMD"
        sm[tolower(sm) == "std. mean difference"] <- "SMD"
        sm[tolower(sm) == "hazard ratio"] <- "HR"
      }
      #
      logscale <- xml_attr(xml.j, "LOG_DATA") == "NO"
      #
      random <- xml_attr(xml.j, "RANDOM")
      if (is.na(random) & outcome.type == "IPD")
        random <- "NO"
      model <- ifelse(random == "NO", "Fixed", "Random")
      common <- model == "Fixed"
      random <- model == "Random"
      #
      TE.pooled <- as.numeric(xml_attr(xml.j, "EFFECT_SIZE"))
      lower.pooled <- as.numeric(xml_attr(xml.j, "CI_START"))
      upper.pooled <- as.numeric(xml_attr(xml.j, "CI_END"))
      weight.pooled <- as.numeric(xml_attr(xml.j, "WEIGHT"))
      level <- as.numeric(xml_attr(xml.j, "CI_STUDY")) / 100
      level.ma <- as.numeric(xml_attr(xml.j, "CI_TOTAL")) / 100
      #
      Z.pooled <- as.numeric(xml_attr(xml.j, "Z"))
      pval.TE.pooled <- as.numeric(xml_attr(xml.j, "P_Z"))
      Q <- as.numeric(xml_attr(xml.j, "Q"))
      pval.Q <- as.numeric(xml_attr(xml.j, "P_Q"))
      I2 <- as.numeric(xml_attr(xml.j, "I2"))
      tau2 <- as.numeric(xml_attr(xml.j, "TAU2"))
      #
      event.e.pooled <- as.numeric(xml_attr(xml.j, "EVENTS_1"))
      n.e.pooled <- as.numeric(xml_attr(xml.j, "TOTAL_1"))
      event.c.pooled <- as.numeric(xml_attr(xml.j, "EVENTS_2"))
      n.c.pooled <- as.numeric(xml_attr(xml.j, "TOTAL_2"))
      #
      swap.events <- xml_attr(xml.j, "SWAP_EVENTS") != "NO"
      #
      label.e <- xml_text(xml_find_all(xml.j, "//GROUP_LABEL_1"))
      label.c <- xml_text(xml_find_all(xml.j, "//GROUP_LABEL_2"))
      label.left <- xml_text(xml_find_all(xml.j, "//GRAPH_LABEL_1"))
      label.right <- xml_text(xml_find_all(xml.j, "//GRAPH_LABEL_2"))
      #
      # Subgroups
      #
      sel.subgroup1 <- grep(paste0("<", outcome.type, "_SUBGROUP"), txt.j)
      sel.subgroup2 <- grep(paste0("</", outcome.type, "_SUBGROUP"), txt.j)
      #
      if (length(sel.subgroup1 > 0) | length(sel.subgroup2) > 0) {
        if (debug)
          cat(paste0("\n** ", outcome.type, "_SUBGROUP **\n"))
        #
        if (length(sel.subgroup1) != length(sel.subgroup2))
          stop("Malformed XML file (tag: ", outcome.type, "_SUBGROUP)")
        #
        for (k in seq(along = sel.subgroup1)) {
          txt.jk <- txt.j[sel.subgroup1[k]:sel.subgroup2[k]]
          sel.data.jk <- c(grep(paste0("<", outcome.type, "_DATA"), txt.jk) - 1,
                           grep("</NAME>", txt.jk))
          if (length(sel.data.jk) > 0)
            sel.jk <- unique(c(1:(max(c(2, min(sel.data.jk)))), length(txt.jk)))
          else
            sel.jk <- unique(c(1:2, length(txt.jk)))
          #
          xml.jk <- as_xml_document(paste(txt.jk[sel.jk], collapse = " "))
          #
          group.no <- as.numeric(xml_attr(xml.jk, "NO"))
          grplab <- xml_text(xml_find_all(xml.jk, "//NAME"))
          #
          if (debug) {
            cat(paste0("Group.no: ", group.no, "\n"))
            cat(paste0("grplab: ", grplab, "\n"))
            #
            if (debug == 3)
              print(txt.jk[sel.jk])
          }
          #
          Q.w <- as.numeric(xml_attr(xml.jk, "NO"))
          pval.Q.w <- as.numeric(xml_attr(xml.jk, "P_CHI2"))
          I2.w <- as.numeric(xml_attr(xml.jk, "I2"))
          #
          # Data
          #
          sel.data1 <- grep(paste0("<", outcome.type, "_DATA"), txt.jk)
          #
          sel1.data2 <- grepl("/>", txt.jk)
          sel2.data2 <- grepl(paste0("</", outcome.type, "_DATA"), txt.jk)
          sel3.data2 <-
            !(grepl("<GROUP_LABEL", txt.jk) |
              grepl("<GRAPH_LABEL", txt.jk) |
              grepl("<EFFECT_MEASURE", txt.jk))
          sel.data2 <-
            seq_along(txt.jk)[(sel1.data2 | sel2.data2) & sel3.data2]
          #
          if (length(sel.data1) != length(sel.data2)) {
            if (debug == 3) {
              print(txt.jk)
              cat(paste0("sel.data1: ", sel.data1, "\n"))
              cat(paste0("sel.data2: ", sel.data2, "\n"))
              #
              print(data.frame(sel1.data2, sel2.data2, sel3.data2))
            }
            stop("Malformed XML file (tag: ", outcome.type, "_DATA)")
          }
          #
          event.e <- n.e <- event.c <- n.c <-
            mean.e <- sd.e <- mean.c <- sd.c <-
              O.E <- V <-
                TE <- seTE <- lower.TE <- upper.TE <-
                  weight <- order <- 
                    rep(NA, length(sel.data1))
          #
          id <- rep("", length(sel.data1))
          #
          if (length(sel.data1) > 0) {
            if (debug)
              cat(paste0("\n* ", outcome.type, "_DATA *\n"))
            for (l in seq(along = sel.data1)) {
              txt.jkl <- paste(txt.jk[sel.data1[l]:sel.data2[l]],
                               collapse = "")
              #
              if (debug == 3)
                print(txt.jk[sel.data1[l]:sel.data2[l]])
              #
              xml.jkl <- as_xml_document(txt.jkl)
              #
              id[l] <- xml_attr(xml.jkl, "STUDY_ID")
              #
              event.e[l] <- as.numeric(xml_attr(xml.jkl, "EVENTS_1"))
              n.e[l] <- as.numeric(xml_attr(xml.jkl, "TOTAL_1"))
              mean.e[l] <- as.numeric(xml_attr(xml.jkl, "MEAN_1"))
              sd.e[l] <- as.numeric(xml_attr(xml.jkl, "SD_1"))
              #
              event.c[l] <- as.numeric(xml_attr(xml.jkl, "EVENTS_2"))
              n.c[l] <- as.numeric(xml_attr(xml.jkl, "TOTAL_2"))
              mean.c[l] <- as.numeric(xml_attr(xml.jkl, "MEAN_2"))
              sd.c[l] <- as.numeric(xml_attr(xml.jkl, "SD_2"))
              #
              O.E[l] <- as.numeric(xml_attr(xml.jkl, "O_E"))
              V[l] <- as.numeric(xml_attr(xml.jkl, "VAR"))
              #
              TE[l] <- as.numeric(xml_attr(xml.jkl, "EFFECT_SIZE"))
              seTE[l] <- as.numeric(xml_attr(xml.jkl, "SE"))
              lower.TE[l] <- as.numeric(xml_attr(xml.jkl, "CI_START"))
              upper.TE[l] <- as.numeric(xml_attr(xml.jkl, "CI_END"))
              #
              weight[l] <- as.numeric(xml_attr(xml.jkl, "WEIGHT"))
              order[l] <- as.numeric(xml_attr(xml.jkl, "ORDER"))
            }
            #
            res.new <-
              data.frame(comp.no, outcome.no, group.no,
                         id = id,
                         event.e = event.e, n.e = n.e,
                         event.c = event.c, n.c = n.c,
                         mean.e = mean.e, sd.e = sd.e,
                         mean.c = mean.c, sd.c = sd.c,
                         O.E = O.E, V = V,
                         TE = TE, seTE = seTE,
                         lower.TE = lower.TE, upper.TE = upper.TE,
                         weight = weight, order = order,
                         level = level,
                         grplab = grplab,
                         overall = overall,
                         test.subgroup = test.subgroup,
                         type = type, method = method, sm = sm, model = model,
                         common = common, random = random,
                         outclab = outclab,
                         k = n.studies,
                         event.e.pooled = event.e.pooled,
                         n.e.pooled = n.e.pooled,
                         event.c.pooled = event.c.pooled,
                         n.c.pooled = n.c.pooled,
                         TE.pooled = TE.pooled,
                         lower.pooled = lower.pooled,
                         upper.pooled = upper.pooled,
                         weight.pooled = weight.pooled,
                         level.ma = level.ma,
                         Z.pooled = Z.pooled, pval.TE.pooled = pval.TE.pooled,
                         Q = Q, pval.Q = pval.Q,
                         I2 = I2, tau2 = tau2,
                         Q.w = Q.w, pval.Q.w = pval.Q.w, I2.w = I2.w,
                         swap.events = swap.events, logscale = logscale,
                         label.e = label.e, label.c = label.c,
                         label.left = label.left,
                         label.right = label.right,
                         complab = complab)
            #
            if (debug >= 2) {
              cat("\n* New data: *\n")
              print(res.new)
            }
            res <- rbind(res, res.new)
          }
        }
      }
      else {
        group.no <- NA
        grplab <- ""
        Q.w <- NA
        pval.Q.w <- NA
        I2.w <- NA
        #
        # Data
        #
        sel.data1 <- grep(paste0("<", outcome.type, "_DATA"), txt.j)
        #
        sel1.data2 <- grepl("/>", txt.j)
        sel2.data2 <- grepl(paste0("</", outcome.type, "_DATA"), txt.j)
        sel3.data2 <-
          !(grepl("<GROUP_LABEL", txt.j) |
            grepl("<GRAPH_LABEL", txt.j) |
            grepl("<EFFECT_MEASURE", txt.j))
        sel.data2 <-
          seq_along(txt.j)[(sel1.data2 | sel2.data2) & sel3.data2]
        #
        if (length(sel.data1) != length(sel.data2)) {
          if (debug == 3) {
            print(txt.j)
            cat(paste0("sel.data1: ", sel.data1, "\n"))
            cat(paste0("sel.data2: ", sel.data2, "\n"))
            #
            print(data.frame(sel1.data2, sel2.data2, sel3.data2))
          }
          stop("Malformed XML file (tag: ", outcome.type, "_DATA)")
        }
        #
        event.e <- n.e <- event.c <- n.c <-
          mean.e <- sd.e <- mean.c <- sd.c <-
            O.E <- V <-
              TE <- seTE <- lower.TE <- upper.TE <-
                weight <- order <- 
                  rep(NA, length(sel.data1))
        #
        id <- rep("", length(sel.data1))
        #
        if (length(sel.data1) > 0) {
          if (debug) {
            cat(paste0("\n* ", outcome.type, "_DATA *\n"))
            print(sel.data1)
          }
          #
          for (k in seq(along = sel.data1)) {
            txt.jk <- paste(txt.j[sel.data1[k]:sel.data2[k]], collapse = "")
            #
            if (debug == 3)
              print(txt.j[sel.data1[k]:sel.data2[k]])
            #
            xml.jk <- as_xml_document(txt.jk)
            #
            id[k] <- xml_attr(xml.jk, "STUDY_ID")
            #
            event.e[k] <- as.numeric(xml_attr(xml.jk, "EVENTS_1"))
            n.e[k] <- as.numeric(xml_attr(xml.jk, "TOTAL_1"))
            mean.e[k] <- as.numeric(xml_attr(xml.jk, "MEAN_1"))
            sd.e[k] <- as.numeric(xml_attr(xml.jk, "SD_1"))
            #
            event.c[k] <- as.numeric(xml_attr(xml.jk, "EVENTS_2"))
            n.c[k] <- as.numeric(xml_attr(xml.jk, "TOTAL_2"))
            mean.c[k] <- as.numeric(xml_attr(xml.jk, "MEAN_2"))
            sd.c[k] <- as.numeric(xml_attr(xml.jk, "SD_2"))
            #
            O.E[k] <- as.numeric(xml_attr(xml.jk, "O_E"))
            V[k] <- as.numeric(xml_attr(xml.jk, "VAR"))
            #
            TE[k] <- as.numeric(xml_attr(xml.jk, "EFFECT_SIZE"))
            seTE[k] <- as.numeric(xml_attr(xml.jk, "SE"))
            lower.TE[k] <- as.numeric(xml_attr(xml.jk, "CI_START"))
            upper.TE[k] <- as.numeric(xml_attr(xml.jk, "CI_END"))
            #
            weight[k] <- as.numeric(xml_attr(xml.jk, "WEIGHT"))
            order[k] <- as.numeric(xml_attr(xml.jk, "ORDER"))
          }
          #
          res.new <-
            data.frame(comp.no, outcome.no, group.no,
                       id = id,
                       event.e = event.e, n.e = n.e,
                       event.c = event.c, n.c = n.c,
                       mean.e = mean.e, sd.e = sd.e,
                       mean.c = mean.c, sd.c = sd.c,
                       O.E = O.E, V = V,
                       TE = TE, seTE = seTE,
                       lower.TE = lower.TE, upper.TE = upper.TE,
                       weight = weight, order = order,
                       level = level,
                       grplab = grplab,
                       overall = overall,
                       test.subgroup = test.subgroup,
                       type = type, method = method, sm = sm, model = model,
                       common = common, random = random,
                       outclab = outclab,
                       k = n.studies,
                       event.e.pooled = event.e.pooled,
                       n.e.pooled = n.e.pooled,
                       event.c.pooled = event.c.pooled,
                       n.c.pooled = n.c.pooled,
                       TE.pooled = TE.pooled,
                       lower.pooled = lower.pooled,
                       upper.pooled = upper.pooled,
                       weight.pooled = weight.pooled,
                       level.ma = level.ma,
                       Z.pooled = Z.pooled, pval.TE.pooled = pval.TE.pooled,
                       Q = Q, pval.Q = pval.Q,
                       I2 = I2, tau2 = tau2,
                       Q.w = Q.w, pval.Q.w = pval.Q.w, I2.w = I2.w,
                       swap.events = swap.events, logscale = logscale,
                       label.e = label.e, label.c = label.c,
                       label.left = label.left,
                       label.right = label.right,
                       complab = complab)
          #
          if (debug >= 2) {
            cat("\n* New data: *\n")
            print(res.new)
          }
          #
          res <- rbind(res, res.new)        
        }
      }
    }
  }
  #
  res
}


oct2txt <- function(txt) {
  txt <- gsub("\200", "EUR", txt, useBytes = TRUE)
  txt <- gsub("\202", "'", txt, useBytes = TRUE)
  txt <- gsub("\203", "f", txt, useBytes = TRUE)
  txt <- gsub("\204", '"', txt, useBytes = TRUE)
  txt <- gsub("\205", "...", txt, useBytes = TRUE)
  txt <- gsub("\206", "+", txt, useBytes = TRUE)
  txt <- gsub("\207", "+", txt, useBytes = TRUE)
  txt <- gsub("\210", "^", txt, useBytes = TRUE)
  txt <- gsub("\211", "%%", txt, useBytes = TRUE)
  txt <- gsub("\212", "S", txt, useBytes = TRUE)
  txt <- gsub("\213", "'", txt, useBytes = TRUE)
  txt <- gsub("\214", "OE", txt, useBytes = TRUE)
  txt <- gsub("\216", "Z", txt, useBytes = TRUE)
  txt <- gsub("\221", "'", txt, useBytes = TRUE)
  txt <- gsub("\222", "'", txt, useBytes = TRUE)
  txt <- gsub("\223", '"', txt, useBytes = TRUE)
  txt <- gsub("\224", '"', txt, useBytes = TRUE)
  txt <- gsub("\225", "-", txt, useBytes = TRUE)
  txt <- gsub("\226", "-", txt, useBytes = TRUE)
  txt <- gsub("\227", "-", txt, useBytes = TRUE)
  txt <- gsub("\230", "~", txt, useBytes = TRUE)
  txt <- gsub("\231", "TM", txt, useBytes = TRUE)
  txt <- gsub("\232", "S", txt, useBytes = TRUE)
  txt <- gsub("\233", "'", txt, useBytes = TRUE)
  txt <- gsub("\234", "oe", txt, useBytes = TRUE)
  txt <- gsub("\237", "Y", txt, useBytes = TRUE)
  txt <- gsub("\240", " ", txt, useBytes = TRUE)
  txt <- gsub("\241", "!", txt, useBytes = TRUE)
  txt <- gsub("\242", "cent", txt, useBytes = TRUE)
  txt <- gsub("\243", "Pound", txt, useBytes = TRUE)
  txt <- gsub("\244", "C", txt, useBytes = TRUE)
  txt <- gsub("\245", "Yen", txt, useBytes = TRUE)
  txt <- gsub("\246", "|", txt, useBytes = TRUE)
  txt <- gsub("\247", "S", txt, useBytes = TRUE)
  txt <- gsub("\250", "", txt, useBytes = TRUE)
  txt <- gsub("\251", "(C)", txt, useBytes = TRUE)
  txt <- gsub("\252", "", txt, useBytes = TRUE)
  txt <- gsub("\253", "'", txt, useBytes = TRUE)
  txt <- gsub("\254", "-", txt, useBytes = TRUE)
  txt <- gsub("\255", "-", txt, useBytes = TRUE)
  txt <- gsub("\256", "(R)", txt, useBytes = TRUE)
  txt <- gsub("\257", "-", txt, useBytes = TRUE)
  txt <- gsub("\260", "o", txt, useBytes = TRUE)
  txt <- gsub("\261", "+/-", txt, useBytes = TRUE)
  txt <- gsub("\262", "2", txt, useBytes = TRUE)
  txt <- gsub("\263", "3", txt, useBytes = TRUE)
  txt <- gsub("\264", "", txt, useBytes = TRUE)
  txt <- gsub("\265", "mu", txt, useBytes = TRUE)
  txt <- gsub("\266", "", txt, useBytes = TRUE)
  txt <- gsub("\267", ".", txt, useBytes = TRUE)
  txt <- gsub("\270", " ", txt, useBytes = TRUE)
  txt <- gsub("\271", "1", txt, useBytes = TRUE)
  txt <- gsub("\272", "o", txt, useBytes = TRUE)
  txt <- gsub("\273", '"', txt, useBytes = TRUE)
  txt <- gsub("\274", "1/4", txt, useBytes = TRUE)
  txt <- gsub("\275", "1/2", txt, useBytes = TRUE)
  txt <- gsub("\276", "3/4", txt, useBytes = TRUE)
  txt <- gsub("\277", "?", txt, useBytes = TRUE)
  txt <- gsub("\300", "A", txt, useBytes = TRUE)
  txt <- gsub("\301", "A", txt, useBytes = TRUE)
  txt <- gsub("\302", "A", txt, useBytes = TRUE)
  txt <- gsub("\303", "A", txt, useBytes = TRUE)
  txt <- gsub("\304", "A", txt, useBytes = TRUE)
  txt <- gsub("\305", "A", txt, useBytes = TRUE)
  txt <- gsub("\306", "AE", txt, useBytes = TRUE)
  txt <- gsub("\307", "C", txt, useBytes = TRUE)
  txt <- gsub("\310", "E", txt, useBytes = TRUE)
  txt <- gsub("\311", "E", txt, useBytes = TRUE)
  txt <- gsub("\312", "E", txt, useBytes = TRUE)
  txt <- gsub("\313", "E", txt, useBytes = TRUE)
  txt <- gsub("\314", "I", txt, useBytes = TRUE)
  txt <- gsub("\315", "I", txt, useBytes = TRUE)
  txt <- gsub("\316", "I", txt, useBytes = TRUE)
  txt <- gsub("\317", "I", txt, useBytes = TRUE)
  txt <- gsub("\320", "E", txt, useBytes = TRUE)
  txt <- gsub("\321", "N", txt, useBytes = TRUE)
  txt <- gsub("\322", "O", txt, useBytes = TRUE)
  txt <- gsub("\323", "O", txt, useBytes = TRUE)
  txt <- gsub("\324", "O", txt, useBytes = TRUE)
  txt <- gsub("\325", "O", txt, useBytes = TRUE)
  txt <- gsub("\326", "O", txt, useBytes = TRUE)
  txt <- gsub("\327", "x", txt, useBytes = TRUE)
  txt <- gsub("\330", "O", txt, useBytes = TRUE)
  txt <- gsub("\331", "U", txt, useBytes = TRUE)
  txt <- gsub("\332", "U", txt, useBytes = TRUE)
  txt <- gsub("\333", "U", txt, useBytes = TRUE)
  txt <- gsub("\334", "U", txt, useBytes = TRUE)
  txt <- gsub("\335", "Y", txt, useBytes = TRUE)
  txt <- gsub("\336", "T", txt, useBytes = TRUE)
  txt <- gsub("\337", "ss", txt, useBytes = TRUE)
  txt <- gsub("\340", "a", txt, useBytes = TRUE)
  txt <- gsub("\341", "a", txt, useBytes = TRUE)
  txt <- gsub("\342", "a", txt, useBytes = TRUE)
  txt <- gsub("\343", "a", txt, useBytes = TRUE)
  txt <- gsub("\344", "a", txt, useBytes = TRUE)
  txt <- gsub("\345", "a", txt, useBytes = TRUE)
  txt <- gsub("\346", "a", txt, useBytes = TRUE)
  txt <- gsub("\347", "c", txt, useBytes = TRUE)
  txt <- gsub("\350", "e", txt, useBytes = TRUE)
  txt <- gsub("\351", "e", txt, useBytes = TRUE)
  txt <- gsub("\352", "e", txt, useBytes = TRUE)
  txt <- gsub("\353", "e", txt, useBytes = TRUE)
  txt <- gsub("\354", "i", txt, useBytes = TRUE)
  txt <- gsub("\355", "i", txt, useBytes = TRUE)
  txt <- gsub("\356", "i", txt, useBytes = TRUE)
  txt <- gsub("\357", "i", txt, useBytes = TRUE)
  txt <- gsub("\360", "e", txt, useBytes = TRUE)
  txt <- gsub("\361", "n", txt, useBytes = TRUE)
  txt <- gsub("\362", "o", txt, useBytes = TRUE)
  txt <- gsub("\363", "o", txt, useBytes = TRUE)
  txt <- gsub("\364", "o", txt, useBytes = TRUE)
  txt <- gsub("\365", "o", txt, useBytes = TRUE)
  txt <- gsub("\366", "o", txt, useBytes = TRUE)
  txt <- gsub("\367", "/", txt, useBytes = TRUE)
  txt <- gsub("\370", "o", txt, useBytes = TRUE)
  txt <- gsub("\371", "u", txt, useBytes = TRUE)
  txt <- gsub("\372", "u", txt, useBytes = TRUE)
  txt <- gsub("\373", "u", txt, useBytes = TRUE)
  txt <- gsub("\374", "u", txt, useBytes = TRUE)
  txt <- gsub("\375", "y", txt, useBytes = TRUE)
  txt <- gsub("\376", "t", txt, useBytes = TRUE)
  txt <- gsub("\377", "y", txt, useBytes = TRUE)
  txt <- gsub("\303\237", "AY", txt, useBytes = TRUE)
  
  # Replace any remaining non UTF-8 characters with question marks
  # otherwise they crash the `extract_outcomes` function
  txt <- iconv(txt, from = "", to = "UTF-8", sub = "???")
  
  #
  txt
}


read.rm5.csv <- function(file, sep = ",", quote = "\"",
                         title, numbers.in.labels = TRUE) {
  #
  selvar <- function(x, sel, value = NA) {
    res <-
      if (!is.null(x))
        x[sel]
      else value[sel]
    res
  }
  #
  numchar <- function(x) {
    res <- as.numeric(as.character(x))
    res
  }
  #
  nachar <- function(x) {
    res <- x
    res[is.na(res)] <- ""
    res
  }
  #
  if (missing(title)) {
    title <- strsplit(file, "\\.csv$")[[1]]
    tmp <- strsplit(title, "\\/")
    title <- tmp[[1]][length(tmp[[1]])]
  }
  #
  # Check number of fields in input data file
  #
  if (length(unique(count.fields(file, sep = sep, quote = quote,
                                 comment.char = ""))) != 1)
    stop("Input file has different number of elements. ",
         "Probably your export data file contains the field ",
         "'Risk of bias tables'. Please create a new export data file in ",
         "RevMan 5 without this field.")
  #
  tdata <- read.table(file, header = TRUE,
                      sep = sep, quote = quote,
                      comment.char = "")
  #
  nam <- names(tdata)
  
  #
  # Overcome problems to import files with UTF-8, byte order mark encoding
  # (see https://en.wikipedia.org/wiki/Byte_order_mark)
  #
  nam[grep("Comparison.Number$", nam)] <- "Comparison.Number"
  #
  if (!all(c("Comparison.Number", "Outcome.Number",
             "Subgroup.Number") %in% nam))
    stop("Mandatory fields 'Comparison Number', 'Outcome Number',",
         " and 'Subgroup Number' not included in export file ",
         deparse(substitute(file)),
         " (see help page of function read.rm5).")
  #
  nam[nam == "Comparison.Number"] <- "comp.no"
  nam[nam == "Outcome.Number"] <- "outcome.no"
  nam[nam == "Subgroup.Number"] <- "group.no"
  nam[nam == "Name"] <- "author"
  #
  nam[nam == "Data.Type"] <- "type"
  nam[nam == "Statistical.Method"] <- "method"
  nam[nam == "Effect.Measure"] <- "sm"
  nam[nam == "Analysis.Model"] <- "model"
  #
  # Variable 'Totals' not included in data frame 
  # Variable 'Study.Confidence.Interval' not included in data frame 
  # Variable 'Total.Confidence.Interval' not included in data frame 
  # Variable 'Test.for.subgroup.differences' not included in data frame 
  #
  nam[nam == "Swap.event.and.non.event"] <- "swap.events"
  nam[nam == "Entered.data.are.on.log.scale..Generic.Inverse.Variance.only."] <-
    "logscale"
  nam[nam == "Enter.number.of.participants..Generic.Inverse.Variance..for.display.only."] <-
    "enter.n"
  #
  nam[nam == "Events.1"] <- "event.e"
  nam[nam == "Mean.1"] <- "mean.e"
  nam[nam == "SD.1"] <- "sd.e"
  nam[nam == "Total.1"] <- "n.e"
  #
  nam[nam == "Events.2"] <- "event.c"
  nam[nam == "Mean.2"] <- "mean.c"
  nam[nam == "SD.2"] <- "sd.c"
  nam[nam == "Total.2"] <- "n.c"
  #
  nam[nam == "O.E"] <- "O.E"
  nam[nam == "Var"] <- "V"
  #
  nam[nam == "Effect.Estimate"] <- "TE"
  nam[nam == "SE"] <- "seTE"
  #
  nam[nam == "CI.Start"] <- "lower"
  nam[nam == "CI.End"]   <- "upper"
  #
  nam[nam == "Weight"]   <- "weight"
  #
  # Variable 'Q' is Q
  nam[nam == "P.Q."]  <- "pval.Q"
  nam[nam == "I..Q."] <- "I2"
  nam[nam == "Tau."]  <- "tau2"
  #
  # Variable 'Z' is Z
  nam[nam == "P.Z."]     <- "pval.TE"
  # Variable 'Qint' is Qint
  nam[nam == "P.Qint."]  <- "pval.Qint"
  nam[nam == "I..Qint."] <- "I2.Qint"
  # Variable 'df' is df
  #
  nam[nam == "Group.Label.1"]     <- "label.e"
  nam[nam == "Group.Label.2"]     <- "label.c"
  nam[nam == "Left.Graph.Label"]  <- "label.left"
  nam[nam == "Right.Graph.Label"] <- "label.right"
  #
  nam[nam == "Year.of.study"]      <- "year"
  nam[nam == "User.defined.order"] <- "order"
  #
  # Fancy coding to set variable names of I^2 and tau^2
  # as "I2" and "tau2" (not necessary for Linux)
  # I don't know how to do this differently with Windows ...
  #
  if (!all(c("I2", "tau2") %in% nam)) {
    pos1 <- seq(along=nam)[nam == "pval.Q"]
    pos2 <- seq(along=nam)[nam == "Z"]
    if ((pos2 - pos1) == 3)
      nam[pos1 + 1:2] <- c("I2", "tau2")
  }
  if (!"I2.Qint" %in% nam) {
    pos3 <- seq(along=nam)[nam == "pval.Qint"]
    pos4 <- seq(along=nam)[nam == "df"]
    if ((pos4 - pos3) == 2)
      nam[pos1 + 1] <- c("I2.Qint")
  }
  #
  names(tdata) <- nam
  #
  tdata$author <- as.character(tdata$author)
  tdata$type <- as.character(tdata$type)
  tdata$method <- as.character(tdata$method)
  tdata$method[tdata$method == "IV"] <- "Inverse"
  tdata$sm <- as.character(tdata$sm)
  tdata$model <- as.character(tdata$model)
  tdata$common <- tdata$model == "Fixed"
  tdata$random <- tdata$model == "Random"
  #
  tdata$sm <- em2sm(tdata$sm)
  #
  sel.oe <- tdata$method == "EXP_O_E_VAR"
  tdata$method[sel.oe] <- "Peto"
  tdata$sm[sel.oe] <- "OR"
  #
  sel.peto <- tdata$method == "PETO" & tdata$sm == "PETO_OR"
  tdata$method[sel.peto] <- "Peto"
  tdata$sm[sel.peto] <- "OR"
  #
  tdata$event.e <- as.numeric(gsub(",", "", tdata$event.e))
  tdata$n.e <- as.numeric(gsub(",", "", tdata$n.e))
  tdata$event.c <- as.numeric(gsub(",", "", tdata$event.c))
  tdata$n.c <- as.numeric(gsub(",", "", tdata$n.c))
  tdata$mean.e <- as.numeric(gsub(",", "", tdata$mean.e))
  tdata$sd.e <- as.numeric(gsub(",", "", tdata$sd.e))
  tdata$mean.c <- as.numeric(gsub(",", "", tdata$mean.c))
  tdata$sd.c <- as.numeric(gsub(",", "", tdata$sd.c))
  if (!is.null(tdata$O.E))
    tdata$O.E <- as.numeric(gsub(",", "", tdata$O.E))
  if (!is.null(tdata$V))
    tdata$V <- as.numeric(gsub(",", "", tdata$V))
  tdata$TE <- as.numeric(gsub(",", "", tdata$TE))
  # No warning concerning infinite values for
  # seTE, lower and upper CI bound
  tdata$seTE <- suppressWarnings(as.numeric(gsub(",", "", tdata$seTE)))
  tdata$lower <- suppressWarnings(as.numeric(gsub(",", "", tdata$lower)))
  tdata$upper <- suppressWarnings(as.numeric(gsub(",", "", tdata$upper)))
  #
  tdata$weight <- as.numeric(gsub(",", "", tdata$weight))
  tdata$Q <- as.numeric(gsub(",", "", tdata$Q))
  tdata$tau2 <- as.numeric(gsub(",", "", tdata$tau2))
  tdata$Qint <- as.numeric(gsub(",", "", tdata$Qint))
  #
  diffcomp <- c(1, diff(tdata$comp.no))
  diffoutc <- c(1, diff(tdata$outcome.no))
  diffgrp  <- c(1, diff(tdata$group.no))
  #
  sel.comp  <- diffcomp != 0
  sel.outc  <- diffoutc != 0 & !sel.comp
  sel.grp   <- tdata$type != "" & diffgrp  != 0 & !sel.comp & !sel.outc
  sel.study <- !sel.comp & !sel.outc & !sel.grp
  #
  res <- list(title = title,
              comparison = 
                data.frame(comp.no = selvar(tdata$comp.no, sel.comp),
                           complab = selvar(tdata$author, sel.comp)
                           ),
              outcome = 
                data.frame(comp.no    = selvar(tdata$comp.no, sel.outc),
                           outcome.no = selvar(tdata$outcome.no, sel.outc),
                           #
                           type        = selvar(tdata$type, sel.outc),
                           method      = selvar(tdata$method, sel.outc),
                           sm          = selvar(tdata$sm, sel.outc),
                           model       = selvar(tdata$model, sel.outc),
                           common = selvar(tdata$common, sel.outc),
                           random = selvar(tdata$random, sel.outc),
                           outclab     = selvar(tdata$author, sel.outc),
                           #
                           k = selvar(tdata$df, sel.outc) + 1,
                           #
                           event.e.pooled = selvar(tdata$event.e, sel.outc),
                           n.e.pooled     = selvar(tdata$n.e, sel.outc),
                           event.c.pooled = selvar(tdata$event.c, sel.outc),
                           n.c.pooled     = selvar(tdata$n.c, sel.outc),
                           #
                           TE.pooled    = selvar(tdata$TE, sel.outc),
                           lower.pooled = selvar(tdata$lower, sel.outc),
                           upper.pooled = selvar(tdata$upper, sel.outc),
                           #
                           weight.pooled = selvar(tdata$weight, sel.outc),
                           #
                           Z.pooled       = selvar(tdata$Z, sel.outc),
                           pval.TE.pooled = selvar(tdata$pval.TE, sel.outc),
                           #
                           Q      = selvar(tdata$Q, sel.outc),
                           pval.Q = selvar(tdata$pval.Q, sel.outc),
                           I2     = selvar(tdata$I2, sel.outc),
                           tau2   = selvar(tdata$tau2, sel.outc),
                           #
                           Q.w      = selvar(tdata$Qint, sel.outc),
                           pval.Q.w = selvar(tdata$pval.Qint, sel.outc),
                           I2.w     = selvar(tdata$I2.Qint, sel.outc),
                           #
                           swap.events = selvar(tdata$swap.events, sel.outc),
                           enter.n     = selvar(tdata$enter.n, sel.outc),
                           logscale    = selvar(tdata$logscale, sel.outc),
                           #
                           label.e     = selvar(tdata$label.e, sel.outc),
                           label.c     = selvar(tdata$label.c, sel.outc),
                           label.left  = selvar(tdata$label.left, sel.outc),
                           label.right = selvar(tdata$label.right, sel.outc)
                           ),
              group =
                if (sum(sel.grp) > 0 ) {
                  data.frame(comp.no    = selvar(tdata$comp.no, sel.grp),
                             outcome.no = selvar(tdata$outcome.no, sel.grp),
                             group.no   = selvar(tdata$group.no, sel.grp),
                             grplab     = selvar(tdata$author, sel.grp)
                             )}
                else {
                  NA
                             },
              study =
                data.frame(comp.no    = selvar(tdata$comp.no, sel.study),
                           outcome.no = selvar(tdata$outcome.no, sel.study),
                           group.no   = selvar(tdata$group.no, sel.study),
                           studlab    = selvar(tdata$author, sel.study),
                           year       = selvar(tdata$year, sel.study),
                           #
                           event.e = selvar(tdata$event.e, sel.study),
                           n.e     = selvar(tdata$n.e, sel.study),
                           event.c = selvar(tdata$event.c, sel.study),
                           n.c     = selvar(tdata$n.c, sel.study),
                           #
                           mean.e = selvar(tdata$mean.e, sel.study),
                           sd.e   = selvar(tdata$sd.e, sel.study),
                           mean.c = selvar(tdata$mean.c, sel.study),
                           sd.c   = selvar(tdata$sd.c, sel.study),
                           #
                           O.E = selvar(tdata$O.E, sel.study),
                           V   = selvar(tdata$V, sel.study),
                           #
                           TE   = selvar(tdata$TE, sel.study),
                           seTE = selvar(tdata$seTE, sel.study),
                           #
                           lower.TE = selvar(tdata$lower, sel.study),
                           upper.TE = selvar(tdata$upper, sel.study),
                           weight   = selvar(tdata$weight, sel.study),
                           #
                           order = selvar(tdata$order, sel.study)
                           )
              )
  #
  if (is.data.frame(res$group))
    res2 <- merge(res$study, res$group,
                  by = c("comp.no", "outcome.no", "group.no"),
                  all.x = TRUE)
  else
    res2 <- res$study
  res2 <- merge(res2, res$outcome,
                by = c("comp.no", "outcome.no"))
  res2 <- merge(res2, res$comparison,
                by = c("comp.no"))
  #
  sel.nocont <- (res2$mean.e == 0 & res2$sd.e == 0 &
                 res2$mean.c == 0 & res2$sd.c == 0)
  res2$mean.e[sel.nocont] <- NA
  res2$sd.e[sel.nocont]   <- NA
  res2$mean.c[sel.nocont] <- NA
  res2$sd.c[sel.nocont]   <- NA
  #
  if (is.factor(res2$O.E))
    res2$O.E <- numchar(res2$O.E)
  if (is.factor(res2$V))
    res2$V <- numchar(res2$V)
  sel.noOE <- res2$O.E == 0 & res2$V == 0
  res2$O.E[sel.noOE] <- NA
  res2$V[sel.noOE]   <- NA
  #
  res2$complab <- as.character(res2$complab)
  res2$outclab <- nachar(as.character(res2$outclab))
  res2$studlab <- nachar(as.character(res2$studlab))
  if (is.data.frame(res$group)) {
    res2$grplab <- nachar(as.character(res2$grplab))
    # Replace some XML code:
    res2$grplab <- gsub("\\&lt;", "<", res2$grplab)
    res2$grplab <- gsub("\\&gt;", ">", res2$grplab)
  }
  #
  res2$sm     <- as.character(res2$sm)
  res2$method <- as.character(res2$method)
  res2$type   <- substring(res2$type, 1, 1)
  res2$model  <- as.character(res2$model)
  #
  res2$label.e     <- nachar(as.character(res2$label.e))
  res2$label.c     <- nachar(as.character(res2$label.c))
  res2$label.left  <- nachar(as.character(res2$label.left))
  res2$label.right <- nachar(as.character(res2$label.right))
  #
  if (numbers.in.labels & length(res2$comp.no) > 0) {
    res2$complab <- paste0(res2$comp.no, " ", res2$complab)
    res2$outclab <- paste0(res2$comp.no, ".", res2$outcome.no, " ",
                           res2$outclab)
    if (is.data.frame(res$group))
      res2$grplab <- paste0(res2$comp.no, ".", res2$outcome.no, ".",
                            res2$group.no, " ", res2$grplab)
  }
  #
  res2 <- res2[order(res2$comp.no,
                     res2$outcome.no,
                     res2$group.no),]
  #
  attr(res2, "title") <- res$title
  #
  if (length(res2$comp.no) > 0) {
    for (i in unique(res2$comp.no)) {
      for (j in unique(res2$outcome.no[res2$comp.no == i])) {
        sel2 <- res2$comp.no == i & res2$outcome.no == j
        if (unique(res2$sm[sel2]) == "OTHER")
          warning("Summary measure unclear for outcome '",
                  #paste0(unique(res2$comp.no[sel2]), ".",
                  #       unique(res2$outcome.no[sel2])),
                  unique(res2$outclab[sel2]),
                  "'. Please use argument 'sm' in function metacr ",
                  "to choose adequate summary measure.")
      }
    }
  }
  
  
  #res2$TE[res2$sm == "HR"] <- log(res2$TE[res2$sm == "HR"])
  #res2$sm[res2$sm == "log HR"] <- "HR"

  iswap.events <- charmatch(tolower(as.character(res2$swap.events)),
                            c("yes", "no"), nomatch = NA)
  res2$swap.events <- ifelse(iswap.events == 1, TRUE,
                      ifelse(iswap.events == 2, FALSE, NA))
  #
  ienter.n <- charmatch(tolower(as.character(res2$enter.n)),
                        c("yes", "no"), nomatch = NA)
  res2$enter.n <- ifelse(ienter.n == 1, TRUE,
                  ifelse(ienter.n == 2, FALSE, NA))
  #
  ilogscale <- charmatch(tolower(as.character(res2$logscale)),
                         c("no", "yes"), nomatch = NA)
  res2$logscale <- ifelse(ilogscale == 1, TRUE,
                   ifelse(ilogscale == 2, FALSE, NA))
  
  attr(res2, "version") <- packageDescription("meta")$Version
  
  class(res2) <- c("rm5", "data.frame")
  res2
}


read.rm5.rm5 <- function(file, title, numbers.in.labels = TRUE, debug = 0) {
  
  
  #
  # Read data file
  #
  rdata <- readLines(file, warn = FALSE)
  
  
  #
  # Convert extended ASCII codes
  #
  rdata <- oct2txt(rdata)
  
  
  #
  # Deal with missing line breaks
  #
  if (length(rdata) == 1 && sum(gregexpr(">", rdata)[[1]] > 0)) {
    rdata <- unlist(strsplit(rdata, "(?<=>)", perl = TRUE))
  }

  
  #
  # Extract title
  #
  if (missing(title)) {
    sel1 <- grep("<TITLE", rdata)
    sel2 <- grep("</TITLE", rdata)
    title.orig <- rdata[unique(c(sel1[1], sel2[1]))]
    title <- xml_text(as_xml_document(paste(title.orig, collapse = " ")))
  }
  if (debug) {
    cat("\n***** TITLE *****\n")
    cat(paste0("Title: ", title, "\n"))
    #
    if (debug == 3)
      print(title.orig)
  }
  
  
  #
  # Determine whether study data are available
  #
  sel.data1 <- grepl("<ANALYSES_AND_DATA", rdata)
  sel.data2 <- grepl("</ANALYSES_AND_DATA", rdata)
  #
  if (!any(sel.data2)) {
    warning("Cochrane review does not contain any study data.",
            call. = FALSE)
    return(NULL)
  }
  #
  sel.data1 <- grep("<ANALYSES_AND_DATA", rdata)
  sel.data2 <- grep("</ANALYSES_AND_DATA", rdata)
  
  
  #
  # Extract data (as vector of character strings)
  #
  txt <- rdata[(sel.data1 + 1):(sel.data2 - 1)]
  #
  sel.data1 <- grepl("<DICH_DATA", txt)
  sel.data2 <- grepl("<CONT_DATA", txt)
  sel.data3 <- grepl("<IV_DATA", txt)
  sel.data4 <- grepl("<IPD_DATA", txt)
  #
  if (!any(sel.data1 | sel.data2 | sel.data3 | sel.data4)) {
    warning("Cochrane review does not contain any usable study data.",
            call. = FALSE)
    return(NULL)
  }
  
  
  res <- data.frame(comp.no = NA, outcome.no = NA, group.no = NA,
                    id = "",
                    event.e = NA, n.e = NA, event.c = NA, n.c = NA,
                    mean.e = NA, sd.e = NA, mean.c = NA, sd.c = NA,
                    O.E = NA, V = NA,
                    TE = NA, seTE = NA, lower.TE = NA, upper.TE = NA,
                    weight = NA, order = NA, level = NA,
                    grplab = "",
                    overall = NA,
                    test.subgroup = NA,
                    type = "", method = "", sm = "", model = "",
                    common = NA, random = NA,
                    outclab = "",
                    k = NA,
                    event.e.pooled = NA, n.e.pooled = NA,
                    event.c.pooled = NA, n.c.pooled = NA,
                    TE.pooled = NA, lower.pooled = NA, upper.pooled = NA,
                    level.ma = NA,
                    weight.pooled = NA,
                    Z.pooled = NA, pval.TE.pooled = NA,
                    Q = NA, pval.Q = NA,
                    I2 = NA, tau2 = NA,
                    Q.w = NA, pval.Q.w = NA, I2.w = NA,
                    swap.events = NA, logscale = NA,
                    label.e = "", label.c = "",
                    label.left = "", label.right = "",
                    complab = "")
  
  
  #
  #
  # Comparisons
  #
  #
  sel.comp1 <- grep("<COMPARISON", txt)
  sel.comp2 <- grep("</COMPARISON", txt)
  #
  if (length(sel.comp1) != length(sel.comp2))
    stop("Malformed XML file (tag: COMPARISON)")
  #
  for (i in seq(along = sel.comp1)) {
    txt.i <- txt[sel.comp1[i]:sel.comp2[i]]
    sel.data.i <- c(grep("</NAME>", txt.i))
    if (length(sel.data.i) > 0)
      sel.i <- c(1:(max(c(2, min(sel.data.i)))), length(txt.i))
    else
      sel.i <- c(1:2, length(txt.i))
    #
    xml.i <- as_xml_document(paste(txt.i[sel.i], collapse = " "))
    #
    comp.no <- as.numeric(xml_attr(xml.i, "NO"))
    complab <- xml_text(xml_find_all(xml.i, "//NAME"))
    #
    if (debug) {
      cat("\n**** COMPARISON ****\n")
      cat(paste0("Comp.no: ", comp.no, "\n"))
      cat(paste0("complab: ", complab, "\n"))
      #
      if (debug == 3)
        print(txt.i[sel.i])
    }
    #
    # Binary outcomes
    #
    res <- extract_outcomes(txt.i, "DICH", res,
                            comp.no, complab,
                            debug = debug)
    #
    # Continuous outcomes
    #
    res <- extract_outcomes(txt.i, "CONT", res,
                            comp.no, complab,
                            debug = debug)
    #
    # IV outcomes
    #
    res <- extract_outcomes(txt.i, "IV", res,
                            comp.no, complab,
                            debug = debug)
    #
    # IPD outcomes
    #
    res <- extract_outcomes(txt.i, "IPD", res,
                            comp.no, complab,
                            debug = debug)
  }
  #
  res <- res[-1, ]
  
  
  #
  # Determine whether information on included studies are available
  #
  sel.incl1 <- grepl("<INCLUDED_STUDIES ", rdata) |
    grepl("<INCLUDED_STUDIES>", rdata)
  sel.incl2 <- grepl("</INCLUDED_STUDIES>", rdata)
  #
  if (any(sel.incl2)) {
    #
    # Extract study information (as vector of character strings)
    #
    sel.incl1 <- c(grep("<INCLUDED_STUDIES ", rdata),
                   grep("<INCLUDED_STUDIES>", rdata))
    sel.incl2 <- grep("</INCLUDED_STUDIES>", rdata)
    #
    study <- rdata[(sel.incl1 + 1):(sel.incl2 - 1)]
    #
    sel.study1 <- grep("<STUDY", study)
    sel.study2 <- grep("</STUDY", study)
    #
    id <- studlab <- year <- rep("", length(sel.study1))
    #
    if ((length(sel.study1) > 0 | length(sel.study2) > 0) &&
        (length(sel.study1) == length(sel.study2))) {
      if (debug)
        cat(paste0("\n* STUDY *\n"))
      for (s in seq(along = sel.study1)) {
        study.s <- study[sel.study1[s]:sel.study2[s]]
        xml.s <- as_xml_document(paste(study.s, collapse = " "))
        #
        id[s] <- xml_attr(xml.s, "ID")
        studlab[s] <- xml_attr(xml.s, "NAME")
        year[s] <- xml_attr(xml.s, "YEAR")
        #
        if (debug >= 2) {
          cat("- id:\n")
          print(id)
          cat("- studlab:\n")
          print(studlab)
          cat("- year:\n")
          print(year)
          #
          if (debug == 3)
            print(study.s)
        }
      }
      #
      names.res <- names(res)
      res.study <- data.frame(id, studlab, year)
      res <- merge(res, res.study, by = "id",
                   all.x = TRUE)
      #
      names.res <- c(names.res[1:4], "studlab", "year",
                     names.res[5:length(names.res)])
      #
      res <- res[, names.res]
    }
  }
  
  
  res <- res[order(res$comp.no, res$outcome.no, res$group.no), ]
  #
  if (numbers.in.labels & length(res$comp.no) > 0) {
    res$complab <- paste0(res$comp.no, " ", res$complab)
    res$outclab <- paste0(res$comp.no, ".", res$outcome.no, " ",
                          res$outclab)
    sel.grp <- !is.na(res$group.no)
    if (any(sel.grp))
      res$grplab[sel.grp] <-
        paste0(res$comp.no, ".", res$outcome.no, ".",
               res$group.no, " ", res$grplab)[sel.grp]
  }
  #
  res$test.subgroup[is.na(res$group.no)] <- NA
  #
  row.names(res) <- 1:nrow(res)
  #
  res$id <- as.character(res$id)
  res$studlab <- as.character(res$studlab)
  res$year <- as.character(res$year)
  res$grplab <- as.character(res$grplab)
  res$type <- as.character(res$type)
  res$method <- as.character(res$method)
  res$sm <- as.character(res$sm)
  res$model <- as.character(res$model)
  res$label.e <- as.character(res$label.e)
  res$label.c <- as.character(res$label.c)
  res$label.left <- as.character(res$label.left)
  res$label.right <- as.character(res$label.right)
  #
  # sel.rel <- is_relative_effect(res$sm)
  # res$TE[sel.rel] <- log(res$TE[sel.rel])
  # res$lower.TE[sel.rel] <- log(res$lower.TE[sel.rel])
  # res$upper.TE[sel.rel] <- log(res$upper.TE[sel.rel])
  # res$seTE <- TE.seTE.ci(res$lower.TE, res$upper.TE, res$level)$seTE
  #
  res$fixed <- res$common
  #
  attr(res, "title") <- title
  attr(res, "version") <- packageDescription("meta")$Version
  
  class(res) <- c("rm5", "data.frame")
  res
}

estimate.missing <- function(TE, TE.sum, type) {
  #
  # 1. Center around mean
  #
  TE.c <- TE - TE.sum
  n <- length(TE.c)
  #
  # 2. Rank absolute values of centered values
  #
  r.star <- rank(abs(TE.c)) * sign(TE.c)
  #
  if (type == "L") {
    #
    # 3. Sum the positive ranks only
    #
    S.rank <- sum(r.star[r.star>0])
    #
    # 4. Estimate for L0
    #
    res0 <- (4 * S.rank - n * (n + 1)) / (2 * n - 1)
    res0.plus <- max(0, res0 + 0.5) %/% 1
  }
  if (type == "R") {
    #
    # 5. Estimate for R0
    #
    res0 <- n - abs(min(r.star)) - 1.5
    res0.plus <- max(0, res0 + 0.5) %/% 1
  }
  #
  res <- list(res0 = res0, res0.plus = res0.plus)
  res
}
hypergeometric <- function(n1, m1, N, psi) {
  #
  # R program for computing the mean, variance, density, cumulative
  # distribution and generating random deviates.
  #
  # Based on
  # Liao and Rosen (2001): Fast and Stable Algorithms for Computing
  # and Sampling from the Noncentral Hypergeometric Distribution,
  # The American Statistician, 55, 236-369.
  #
  # this is how to use the function
  #
  # n1 <- 100
  # n2 <- 100
  # m1 <- 100
  # N <- n1 + n2
  # odds.ratio <- 3
  # obj <- hypergeometric(n1, m1, N, odds.ratio)
  # obj$mean()
  # obj$var()
  # obj$d(40)
  # obj$p(40)
  # obj$r()
  #
  n2 <- N - n1
  #
  if (n1 < 0 | n2 < 0 | m1 < 0 | m1 > N | psi <= 0)
    stop("Wrong argument in hypergeometric")
  #
  mode.compute <- function() {
    a <- psi - 1
    b <- -((m1 + n1 + 2) * psi + n2 - m1)
    c <- psi * (n1 + 1) * (m1 + 1)
    q <- b + sign(b) * sqrt(b * b - 4 * a * c)
    q <- -q / 2
    #
    mode <- trunc(c / q)
    if (uu >= mode && mode >= ll)
      return(mode)
    else
      return(trunc(q / a))
  }
  #
  r.function <- function(i)
    (n1 - i + 1) * (m1 - i + 1) / i / (n2 - m1 + i) * psi
  #
  mean <- function()
    sum(prob[(ll:uu) + shift] * (ll:uu))
  #
  var <- function()
    sum(prob[(ll:uu) + shift] * (ll:uu)^2) - mean()^2
  #
  d <- function(x)
    return(prob[x + shift])
  #
  p <- function(x, lower.tail = TRUE) {
    if (lower.tail)
      return(sum(prob[ll:(x + shift)]))
    else
      return(sum(prob[(x + shift):uu]))
  }
  #
  sample.low.to.high <- function(lower.end, ran) {
    for (i in lower.end:uu) {
      if (ran <= prob[i + shift]) return(i)
      ran <- ran - prob[i + shift]
    }
  }
  #
  sample.high.to.low <- function(upper.end, ran) {
    for (i in upper.end:ll) {
      if (ran <= prob[i + shift])
        return(i)
      ran <- ran - prob[i + shift]
    }
  }
  #
  r <- function() {
    ran <- runif(1)
    #
    if (mode == ll)
      return(sample.low.to.high(ll, ran))
    #
    if (mode == uu)
      return(sample.high.to.low(uu, ran))
    #
    if (ran < prob[mode + shift])
      return(mode)
    #
    ran <- ran - prob[mode + shift]
    #
    lower <- mode - 1
    upper <- mode + 1
    #
    repeat {
      if (prob[upper + shift] >= prob[lower + shift]) {
        if (ran < prob[upper + shift])
          return(upper)
        ran <- ran - prob[upper + shift]
        if (upper == uu)
          return(sample.high.to.low(lower, ran))
        upper <- upper + 1
      }
      else {
        if (ran < prob[lower + shift])
          return(lower)
        ran <- ran - prob[lower + shift]
        if (lower == ll)
          return(sample.low.to.high(upper, ran))
        lower <- lower - 1
      }
    }
  }
  #
  ll <- max(0, m1 - n2)
  uu <- min(n1, m1)
  mode <- mode.compute()
  #
  prob <- array(1, uu - ll + 1)
  #
  shift <- 1 - ll
  if (mode < uu) {
    # note the shift of location
    r1 <- r.function((mode + 1):uu)
    prob[(mode + 1 + shift):(uu + shift)] <- cumprod(r1)
  }
  #
  if (mode > ll) {
    r1 <- 1 / r.function(mode:(ll + 1))
    prob[(mode - 1 + shift):(ll + shift)] <- cumprod(r1)
  }
  #
  prob <- prob / sum(prob)
  #
  return(list(mean = mean, var = var, d = d, p = p, r = r))
}
kentau <- function(x, y, correct = FALSE, keep.data = FALSE) {
  #
  # Check:
  #
  if (length(x) != length(y))
    stop("length of argument x and y must be equal")
  #
  sel <- !is.na(x) & !is.na(y)
  if (length(x) != sum(sel))
    warning(paste(length(x) - sum(sel),
                  "observation(s) dropped due to missing values"))
  #
  x <- x[sel]
  y <- y[sel]
  n <- length(x)
  #
  t <- rle(sort(x))$lengths
  u <- rle(sort(y))$lengths
  #
  N <- 0.5 * n * (n - 1)
  N1 <- N - sum(t * (t - 1) / 2)
  N2 <- N - sum(u * (u - 1) / 2)
  #
  ks <- sqrt(N1) * sqrt(N2) * cor(x, y, method = "kendall")
  #
  # ks <- .C(kenscore,
  #          kenscore = as.double(0),
  #          x = as.double(x),
  #          y = as.double(y),
  #          n = as.integer(n))$kenscore
  #
  # Calculate S and s.e(S) according to
  # Stata, release 5, Reference P-Z, p.239-240
  #
  # see also Kendall, Gibbons (1990), Rank Correlation Methods
  # p. 66-68
  #
  se.ks <- sqrt(1 / 18 * (n * (n - 1) * (2 * n + 5) -
                          sum(t * (t - 1) * (2 * t + 5)) -
                          sum(u * (u - 1) * (2 * u + 5))) +
                1 / (9 * n * (n - 1) * (n - 2))  *
                sum(t * (t - 1) * (t - 2)) *
                sum(u * (u - 1) * (u - 2)) +
                1 / (2 * n * (n - 1))  *
                sum(t * (t - 1))  *
                sum(u * (u - 1)))
  #
  if (as.logical(correct) &
      any(c(length(unique(x)), length(unique(y))) == 2))
    warning("Continuity corrected statistic may be inappropriate,\n",
            "see Kendall, Gibbons (1990), Rank Correlation Methods, p.67.")
  #
  statistic <- (ks - sign(ks) * as.logical(correct)) / se.ks
  p.value <- 2 * pnorm(abs(statistic), lower.tail = FALSE)
  #
  res <- list(tau.a = ks / N,
              tau.b = ks / (sqrt(N1) * sqrt(N2)),
              ks = ks - sign(ks) * as.logical(correct),
              se.ks = se.ks,
              statistic = statistic,
              p.value = p.value,
              correction = as.logical(correct))
  #
  if (keep.data) {
    res$x <- x
    res$y <- y
  }
  #
  res
}
linregcore <- function(TE, seTE, covar = NULL,
                       model = "lm", method.tau = "DL",
                       ...) {
  #
  if (is.null(covar))
    predictor <- "sei"
  else
    predictor <- "ni"
  #
  rma1 <-
    suppressWarnings(
      runNN(rma.uni,
            list(yi = TE, sei = seTE, ni = covar, method = method.tau,
                 test = "t", ...)))
  #
  reg <- suppressWarnings(regtest(rma1, predictor = predictor,
                                  model = model))
  #
  if (inherits(reg$fit, c("lm", "summary.lm"))) {
    if (!inherits(reg$fit, "summary.lm"))
      creg <- suppressWarnings(coef(summary(reg$fit)))
    else
      creg <- suppressWarnings(coef(reg$fit))
    #
    res <- list(intercept = creg[1, 1], se.intercept = creg[1, 2],
                slope = creg[2, 1], se.slope = creg[2, 2])
  }
  else
    res <- list(intercept = reg$fit$beta[1], se.intercept = reg$fit$se[1],
                slope = reg$fit$beta[2], se.slope = reg$fit$se[2])
  #
  res$statistic <- res$slope / res$se.slope
  res$df <- reg$dfs
  res$pval <- 2 * pt(abs(res$statistic), df = res$df, lower.tail = FALSE)
  #
  if (inherits(reg$fit, c("lm", "summary.lm"))) {
    if (!inherits(reg$fit, "summary.lm"))
      res$tau <- suppressWarnings(summary(reg$fit)$sigma)
    else
      res$tau <- reg$fit$sigma
  }
  else
    res$tau <- sqrt(reg$fit$tau2)
  #
  res$TE <- TE
  res$seTE <- seTE
  res$covar <- covar
  res$predictor <- predictor
  res$model <- model
  res$method.tau <- method.tau
  #
  names(res$statistic) <- "t"
  #
  res
}

#' Trim-and-fill method to adjust for bias in meta-analysis
#' 
#' @description
#' Trim-and-fill method for estimating and adjusting for the number
#' and outcomes of missing studies in a meta-analysis.
#' 
#' @aliases trimfill trimfill.meta trimfill.default
#' 
#' @param x An object of class \code{meta}, or estimated treatment
#'   effect in individual studies.
#' @param seTE Standard error of estimated treatment effect.
#' @param left A logical indicating whether studies are supposed to be
#'   missing on the left or right side of the funnel plot. If NULL,
#'   the linear regression test for funnel plot symmetry (i.e.,
#'   function \code{metabias(..., method="Egger")}) is used to
#'   determine whether studies are missing on the left or right side.
#' @param ma.common A logical indicating whether a common effect or
#'   random effects model is used to estimate the number of missing
#'   studies.
#' @param type A character indicating which method is used to estimate
#'   the number of missing studies. Either \code{"L"} or \code{"R"}.
#' @param n.iter.max Maximum number of iterations to estimate number
#'   of missing studies.
#' @param sm An optional character string indicating underlying
#'   summary measure, e.g., \code{"RD"}, \code{"RR"}, \code{"OR"},
#'   \code{"ASD"}, \code{"HR"}, \code{"MD"}, \code{"SMD"}, or
#'   \code{"ROM"}; ignored if \code{x} is of class \code{meta}.
#' @param studlab An optional vector with study labels; ignored if
#'   \code{x} is of class \code{meta}.
#' @param level The level used to calculate confidence intervals for
#'   individual studies. If existing, \code{x$level} is used as value
#'   for \code{level}; otherwise 0.95 is used.
#' @param level.ma The level used to calculate confidence interval for
#'   the pooled estimate. If existing, \code{x$level.ma} is used as
#'   value for \code{level.ma}; otherwise 0.95 is used.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for the
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and plots. If
#'   \code{backtransf=TRUE}, results for \code{sm="OR"} are printed as
#'   odds ratios rather than log odds ratios and results for
#'   \code{sm="ZCOR"} are printed as correlations rather than Fisher's
#'   z transformed correlations, for example.
#' @param pscale A numeric giving scaling factor for printing of
#'   single event probabilities or risk differences, i.e. if argument
#'   \code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
#'   \code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.
#' @param irscale A numeric defining a scaling factor for printing of
#'   single incidence rates or incidence rate differences, i.e. if
#'   argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
#'   \code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.
#' @param irunit A character specifying the time unit used to
#'   calculate rates, e.g. person-years.
#' @param silent A logical indicating whether basic information on
#'   iterations shown.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param \dots Additional arguments (to catch deprecated arguments).
#'
#' @details
#' The trim-and-fill method (Duval, Tweedie 2000a, 2000b) can be used
#' for estimating and adjusting for the number and outcomes of missing
#' studies in a meta-analysis. The method relies on scrutiny of one
#' side of a funnel plot for asymmetry assumed due to publication
#' bias.
#' 
#' Three different methods have been proposed originally to estimate
#' the number of missing studies. Two of these methods (L- and
#' R-estimator) have been shown to perform better in simulations, and
#' are available in this R function (argument \code{type}).
#' 
#' A common effect or random effects model can be used to estimate the
#' number of missing studies (argument \code{ma.common}). Furthermore,
#' a common effect and/or random effects model can be used to
#' summaries study results (arguments \code{common} and
#' \code{random}). Simulation results (Peters et al. 2007) indicate
#' that the common-random model, i.e. using a common effect model to
#' estimate the number of missing studies and a random effects model
#' to summaries results, (i) performs better than the common-common
#' model, and (ii) performs no worse than and marginally better in
#' certain situations than the random-random model. Accordingly, the
#' common-random model is the default.
#' 
#' An empirical comparison of the trim-and-fill method and the Copas
#' selection model (Schwarzer et al. 2010) indicates that the
#' trim-and-fill method leads to excessively conservative inference in
#' practice. The Copas selection model is available in R package
#' \bold{metasens}.
#' 
#' The function \code{\link{metagen}} is called internally.
#' 
#' @return
#' An object of class \code{c("trimfill", "metagen", "meta")} with
#' corresponding generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metagen}}, \code{\link{metabias}},
#'   \code{\link{funnel}}
#' 
#' @references
#' Duval S & Tweedie R (2000a):
#' A nonparametric "Trim and Fill" method of accounting for
#' publication bias in meta-analysis.
#' \emph{Journal of the American Statistical Association},
#' \bold{95}, 89--98
#' 
#' Duval S & Tweedie R (2000b):
#' Trim and Fill: A simple funnel-plot-based method of testing and
#' adjusting for publication bias in meta-analysis.
#' \emph{Biometrics},
#' \bold{56}, 455--63
#' 
#' Peters JL, Sutton AJ, Jones DR, Abrams KR, Rushton L (2007):
#' Performance of the trim and fill method in the presence of
#' publication bias and between-study heterogeneity.
#' \emph{Statisics in Medicine},
#' \bold{10}, 4544--62
#' 
#' Schwarzer G, Carpenter J, Rücker G (2010):
#' Empirical evaluation suggests Copas selection model preferable to
#' trim-and-fill method for selection bias in meta-analysis
#' \emph{Journal of Clinical Epidemiology},
#' \bold{63}, 282--8
#' 
#' @examples
#' data(Fleiss1993bin)
#' ma <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin, sm = "OR")
#' tf <- trimfill(ma)
#' tf
#' funnel(tf)
#' funnel(tf, pch = ifelse(tf$trimfill, 1, 16), level = 0.9, random = FALSE)
#' #
#' # Use log odds ratios on x-axis
#' #
#' funnel(tf, backtransf = FALSE)
#' funnel(tf, pch = ifelse(tf$trimfill, 1, 16), level = 0.9, random = FALSE,
#'   backtransf = FALSE)
#' 
#' trimfill(ma$TE, ma$seTE, sm = ma$sm)
#' 
#' @rdname trimfill
#' @method trimfill meta
#' @export

trimfill.meta <- function(x, left = NULL, ma.common = TRUE,
                          type = "L", n.iter.max = 50,
                          #
                          common = FALSE, random = TRUE,
                          prediction = x$prediction,
                          #
                          backtransf = x$backtransf, pscale = x$pscale,
                          irscale = x$irscale, irunit = x$irunit,
                          silent = TRUE,
                          warn.deprecated = gs("warn.deprecated"),
                          ...) {
  
  #
  #
  # (1) Check for (inadmissible) meta object
  #
  #
  chkclass(x, "meta")
  chksuitable(x, "Trim-and-fill method",
              c("metacum", "metainf", "metamerge", "netpairwise"))
  #
  if (!is.null(x$weights.common) | !is.null(x$weights.random))
    stop("Trim-and-fill method not implemented for user-specified weights.",
         call. = FALSE)
  #
  x <- updateversion(x)
  #
  if (x$three.level)
    stop("Trim-and-fill method not implemented for three-level model.",
         call. = FALSE)
  
  
  #
  #
  # (2) Check arguments
  #
  #
  args <- list(...)
  #
  ma.common <- deprecated(ma.common, missing(ma.common), args, "ma.fixed",
                          warn.deprecated)
  chklogical(ma.common)
  #
  type <- setchar(type, c("L", "R"))
  chknumeric(n.iter.max, min = 1, length = 1)
  #
  common <- deprecated(common, missing(common), args, "fixed",
                       warn.deprecated)
  chklogical(common)
  chklogical(random)
  chklogical(prediction)
  chklogical(backtransf)
  #
  sm <- x$sm
  if (!is_prop(sm))
    pscale <- 1
  chknumeric(pscale, length = 1)
  if (!backtransf & pscale != 1) {
    warning("Argument 'pscale' set to 1 as argument 'backtransf' is FALSE.")
    pscale <- 1
  }
  if (!is_rate(sm))
    irscale <- 1
  chknumeric(irscale, length = 1)
  if (!backtransf & irscale != 1) {
    warning("Argument 'irscale' set to 1 as argument 'backtransf' is FALSE.")
    irscale <- 1
  }
  #
  chklogical(silent)
  
  
  TE <- x$TE
  seTE <- x$seTE
  studlab <- x$studlab
  #
  n.e <- x$n.e
  n.c <- x$n.c
  n <- x$n
  #
  event.e <- x$event.e
  event.c <- x$event.c
  event <- x$event
  #
  time.e <- x$time.e
  time.c <- x$time.c
  time <- x$time
  #
  cor <- x$cor
  #
  mean.e <- x$mean.e
  mean.c <- x$mean.c
  #
  sd.e <- x$sd.e
  sd.c <- x$sd.c
  #
  transf.null.effect <- null.effect <- x$null.effect
  #
  if (sm %in% c("PFT", "PAS"))
    transf.null.effect <- asin(sqrt(null.effect))
  else if (is_log_effect(sm))
    transf.null.effect <- log(null.effect)
  else if (sm == c("PLOGIT"))
    transf.null.effect <- log(null.effect / (1 - null.effect))
  else if (sm %in% c("IRS", "IRFT"))
    transf.null.effect <- sqrt(null.effect)
  else if (sm == "ZCOR")
    transf.null.effect <- 0.5 * log((1 + null.effect) / (1 - null.effect))
  
  
  #
  # Exclude studies from meta-analysis
  #
  if (!is.null(x$exclude)) {
    exclude <- x$exclude
    nomiss <- !is.na(TE) & !is.na(seTE)
    miss <- !nomiss & !exclude
    #
    sel <- nomiss & !exclude
  }
  else {
    exclude <- exclude.na <- NULL
    nomiss <- !is.na(TE) & !is.na(seTE)
    miss <- !nomiss
    #
    sel <- nomiss
  }
  #
  if (any(miss))
    warning(paste(sum(miss),
                  "observation(s) dropped due to missing values"))
  
  
  TE <- TE[sel]
  seTE <- seTE[sel]
  studlab <- studlab[sel]
  #
  if (!is.null(n.e))
    n.e <- n.e[sel]
  if (!is.null(n.c))
    n.c <- n.c[sel]
  if (!is.null(n))
    n <- n[sel]
  #
  if (!is.null(event.e))
    event.e <- event.e[sel]
  if (!is.null(event.c))
    event.c <- event.c[sel]
  if (!is.null(event))
    event <- event[sel]
  #
  if (!is.null(time.e))
    time.e <- time.e[sel]
  if (!is.null(time.c))
    time.c <- time.c[sel]
  if (!is.null(time))
    time <- time[sel]
  #
  if (!is.null(cor))
    cor <- cor[sel]
  #
  if (!is.null(mean.e))
    mean.e <- mean.e[sel]
  if (!is.null(mean.c))
    mean.c <- mean.c[sel]
  #
  if (!is.null(sd.e))
    sd.e <- sd.e[sel]
  if (!is.null(sd.c))
    sd.c <- sd.c[sel]
  #
  k <- length(TE)
  #
  if (k <= 2) {
    warning("Minimal number of three studies for trim-and-fill method")
    return(invisible(NULL))
  }
  
  
  if (is.null(left))
    left <- as.logical(sign(metabias(TE, seTE, method = "Egger",
                                     k.min = 3)$estimate[1]) == 1)
  #
  if (!left) TE <- -TE
  #
  ord <- order(TE)
  TE <- TE[ord]
  seTE <- seTE[ord]
  studlab <- studlab[ord]
  #
  if (!is.null(n.e))
    n.e <- n.e[ord]
  if (!is.null(n.c))
    n.c <- n.c[ord]
  if (!is.null(n))
    n <- n[ord]
  #
  if (!is.null(event.e))
    event.e <- event.e[ord]
  if (!is.null(event.c))
    event.c <- event.c[ord]
  if (!is.null(event))
    event <- event[ord]
  #
  if (!is.null(time.e))
    time.e <- time.e[ord]
  if (!is.null(time.c))
    time.c <- time.c[ord]
  if (!is.null(time))
    time <- time[ord]
  #
  if (!is.null(cor))
    cor <- cor[ord]
  #
  if (!is.null(mean.e))
    mean.e <- mean.e[ord]
  if (!is.null(mean.c))
    mean.c <- mean.c[ord]
  #
  if (!is.null(sd.e))
    sd.e <- sd.e[ord]
  if (!is.null(sd.c))
    sd.c <- sd.c[ord]
  
  
  if (ma.common)
    TE.sum <-
      metagen(TE, seTE, method.tau = "DL", method.tau.ci = "")$TE.common
  else
    TE.sum <-
      metagen(TE, seTE, method.tau = x$method.tau, method.tau.ci = "")$TE.random
  
  
  if (k == 1) {
    n.iter <- 0
    k0 <- -9
  }
  else {
    n.iter  <-  0
    k0.last <- -1
    k0      <-  0
    #
    while (k0.last != k0 & k0 <= (k - 1) & n.iter < n.iter.max) {
      #
      n.iter <- n.iter + 1
      #
      k0.last <- k0
      #
      sel <- 1:(k - k0)
      #
      if (ma.common)
        TE.sum <- metagen(TE[sel], seTE[sel],
                          method.tau = "DL",
                          method.tau.ci = "")$TE.common
      else
        TE.sum <- metagen(TE[sel], seTE[sel],
                          method.tau = x$method.tau,
                          method.tau.ci = "")$TE.random
      #
      trim1 <- estimate.missing(TE, TE.sum, type)
      #
      if (!silent) {
        cat(paste0("n.iter = ", n.iter, "\n"))
        if (type == "L")
          cat(paste0("L0 = ", round(trim1$res0, 2), "\n\n"))
        if (type == "R")
          cat(paste0("R0 = ", round(trim1$res0 + 0.5, 2), "\n\n"))
      }
      #
      k0 <- trim1$res0.plus
    }
  }
  
  
  if (k0 > (k - 1))
    k0 <- k - 1
  #
  if (k0 > 0) {
    TE.star   <- 2 * TE.sum - TE[(k - k0 + 1):k]
    seTE.star <- seTE[(k - k0 + 1):k]
    #
    trimfill  <- c(rep(FALSE, length(TE)),
                   rep(TRUE, length(TE.star)))
    #
    TE      <- c(TE[order(ord)], TE.star)
    seTE    <- c(seTE[order(ord)], seTE.star)
    studlab <- c(studlab[order(ord)],
                 paste("Filled:", studlab[(k - k0 + 1):k]))
    #
    if (!is.null(n.e))
      n.e <- c(n.e[order(ord)], n.e[(k - k0 + 1):k])
    if (!is.null(n.c))
      n.c <- c(n.c[order(ord)], n.c[(k - k0 + 1):k])
    if (!is.null(n))
      n <- c(n[order(ord)], n[(k - k0 + 1):k])
    #
    if (!is.null(event.e))
      event.e <- c(event.e[order(ord)], event.e[(k - k0 + 1):k])
    if (!is.null(event.c))
      event.c <- c(event.c[order(ord)], event.c[(k - k0 + 1):k])
    if (!is.null(event))
      event <- c(event[order(ord)], event[(k - k0 + 1):k])
    #
    if (!is.null(time.e))
      time.e <- c(time.e[order(ord)], time.e[(k - k0 + 1):k])
    if (!is.null(time.c))
      time.c <- c(time.c[order(ord)], time.c[(k - k0 + 1):k])
    if (!is.null(time))
      time <- c(time[order(ord)], time[(k - k0 + 1):k])
    #
    if (!is.null(cor))
      cor <- c(cor[order(ord)], cor[(k - k0 + 1):k])
    #
    if (!is.null(mean.e))
      mean.e <- c(mean.e[order(ord)], mean.e[(k - k0 + 1):k])
    if (!is.null(mean.c))
      mean.c <- c(mean.c[order(ord)], mean.c[(k - k0 + 1):k])
    #
    if (!is.null(sd.e))
      sd.e <- c(sd.e[order(ord)], sd.e[(k - k0 + 1):k])
    if (!is.null(sd.c))
      sd.c <- c(sd.c[order(ord)], sd.c[(k - k0 + 1):k])
  }
  else {
    TE.star   <- NA
    seTE.star <- NA
    trimfill  <- rep(FALSE, length(TE))
    TE        <- TE[order(ord)]
    seTE      <- seTE[order(ord)]
    studlab   <- studlab[order(ord)]
    #
    if (!is.null(n.e))
      n.e <- n.e[order(ord)]
    if (!is.null(n.c))
      n.c <- n.c[order(ord)]
    if (!is.null(n))
      n <- n[order(ord)]
    #
    if (!is.null(event.e))
      event.e <- event.e[order(ord)]
    if (!is.null(event.c))
      event.c <- event.c[order(ord)]
    if (!is.null(event))
      event <- event[order(ord)]
    #
    if (!is.null(time.e))
      time.e <- time.e[order(ord)]
    if (!is.null(time.c))
      time.c <- time.c[order(ord)]
    if (!is.null(time))
      time <- time[order(ord)]
    #
    if (!is.null(cor))
      cor <- cor[order(ord)]
    #
    if (!is.null(mean.e))
      mean.e <- mean.e[order(ord)]
    if (!is.null(mean.c))
      mean.c <- mean.c[order(ord)]
    #
    if (!is.null(sd.e))
      sd.e <- sd.e[order(ord)]
    if (!is.null(sd.c))
      sd.c <- sd.c[order(ord)]
  }
  
  
  if (!left)
    m <- metagen(-TE, seTE, studlab = studlab, level = x$level,
                 #
                 level.ma = x$level.ma,
                 method.common.ci = x$method.common.ci,
                 method.random.ci = x$method.random.ci,
                 adhoc.hakn.ci = x$adhoc.hakn.ci,
                 #
                 method.tau = x$method.tau, method.tau.ci = x$method.tau.ci,
                 level.hetstat = x$level.hetstat,
                 #
                 method.I2 = replaceNULL(x$method.I2, gs("method.I2")),
                 #
                 method.predict = x$method.predict,
                 prediction = prediction, level.predict = x$level.predict,
                 adhoc.hakn.pi = x$adhoc.hakn.pi,
                 seed.predict = x$seed.predict,
                 #
                 null.effect = transf.null.effect)
  else
    m <- metagen(TE, seTE, studlab = studlab, level = x$level,
                 #
                 level.ma = x$level.ma,
                 method.common.ci = x$method.common.ci,
                 method.random.ci = x$method.random.ci,
                 adhoc.hakn.ci = x$adhoc.hakn.ci,
                 #
                 method.tau = x$method.tau, method.tau.ci = x$method.tau.ci,
                 level.hetstat = x$level.hetstat,
                 #
                 method.I2 = replaceNULL(x$method.I2, gs("method.I2")),
                 #
                 method.predict = x$method.predict,
                 prediction = prediction, level.predict = x$level.predict,
                 adhoc.hakn.pi = x$adhoc.hakn.pi,
                 seed.predict = x$seed.predict,
                 #
                 null.effect = transf.null.effect)
  
  
  #
  # Calculate H, I-Squared, and Rb
  #
  Hres  <- list(TE = m$H, lower = m$lower.H, upper = m$upper.H)
  I2res <- list(TE = m$I2, lower = m$lower.I2, upper = m$upper.I2)
  Rbres <- list(TE = m$Rb, lower = m$lower.Rb, upper = m$upper.Rb)
  
  
  #
  # Number of filled studies
  #
  k0 <- sum(trimfill)
  
  
  if (!is.null(exclude) && any(exclude)) {
    exclude.na <- c(exclude, rep(NA, k0))
    exclude <- c(exclude, rep(FALSE, k0))
    TE.all <- seTE.all <- studlab.all <- rep(NA, length(exclude))
    #
    TE.all[exclude] <- x$TE[exclude]
    TE.all[!exclude] <- TE
    #
    seTE.all[exclude] <- x$seTE[exclude]
    seTE.all[!exclude] <- seTE
    #
    studlab.all[exclude] <- x$studlab[exclude]
    studlab.all[!exclude] <- studlab
    #
    if (!left)
      m <- metagen(-TE.all, seTE.all, studlab = studlab.all,
                   exclude = exclude, level = x$level,
                   #
                   level.ma = x$level.ma,
                   method.common.ci = x$method.common.ci,
                   method.random.ci = x$method.random.ci,
                   adhoc.hakn.ci = x$adhoc.hakn.ci,
                   #
                   method.tau = x$method.tau, method.tau.ci = x$method.tau.ci,
                   level.hetstat = x$level.hetstat,
                   #
                   method.I2 = replaceNULL(x$method.I2, gs("method.I2")),
                   #
                   method.predict = x$method.predict,
                   prediction = prediction,
                   level.predict = x$level.predict,
                   adhoc.hakn.pi = x$adhoc.hakn.pi,
                   seed.predict = x$seed.predict,
                   #
                   null.effect = transf.null.effect)
    else
      m <- metagen(TE.all, seTE.all, studlab = studlab.all,
                   exclude = exclude, level = x$level,
                   #
                   level.ma = x$level.ma,
                   method.common.ci = x$method.common.ci,
                   method.random.ci = x$method.random.ci,
                   adhoc.hakn.ci = x$adhoc.hakn.ci,
                   #
                   method.tau = x$method.tau, method.tau.ci = x$method.tau.ci,
                   level.hetstat = x$level.hetstat,
                   #
                   method.I2 = replaceNULL(x$method.I2, gs("method.I2")),
                   #
                   method.predict = x$method.predict,
                   prediction = prediction,
                   level.predict = x$level.predict,
                   adhoc.hakn.pi = x$adhoc.hakn.pi,
                   seed.predict = x$seed.predict,
                   #
                   null.effect = transf.null.effect)
  }
  
  
  res <- list(studlab = m$studlab,
              #
              sm = sm,
              null.effect = x$null.effect,
              #
              TE = m$TE, seTE = m$seTE,
              statistic = m$statistic, pval = m$pval,
              df = rep(NA, length(m$TE)),
              level = x$level,
              lower = m$lower, upper = m$upper,
              #
              three.level = FALSE,
              cluster = NULL,
              #
              k = m$k,
              k.study = m$k.study,
              k.all = m$k.all,
              k.TE = m$k.TE,
              k0 = k0,
              #
              overall = common | random,
              overall.hetstat = TRUE,
              common = common,
              random = random,
              prediction = prediction,
              backtransf = backtransf,
              #
              method = m$method,
              method.random = m$method.random,
              level = x$level,
              #
              w.common = m$w.common,
              TE.common = m$TE.common, seTE.common = m$seTE.common,
              statistic.common = m$statistic.common,
              pval.common = m$pval.common,
              level.ma = x$level.ma,
              lower.common = m$lower.common,
              upper.common = m$upper.common,
              #
              w.random = m$w.random,
              TE.random = m$TE.random, seTE.random = m$seTE.random,
              statistic.random = m$statistic.random,
              pval.random = m$pval.random,
              method.common.ci = x$method.common.ci,
              method.random.ci = x$method.random.ci,
              df.random = m$df.random,
              lower.random = m$lower.random, upper.random = m$upper.random,
              #
              seTE.classic = m$seTE.classic,
              #
              adhoc.hakn.ci = m$adhoc.hakn.ci,
              df.hakn = m$df.hakn,
              seTE.hakn.ci = m$seTE.hakn.ci,
              seTE.hakn.adhoc.ci = m$seTE.hakn.adhoc.ci,
              #
              df.kero = m$df.kero,
              seTE.kero = m$seTE.kero,
              #
              method.predict = m$method.predict,
              adhoc.hakn.pi = m$adhoc.hakn.pi,
              seTE.predict = m$seTE.predict,
              df.predict = m$df.predict,
              level.predict = x$level.predict,
              lower.predict = m$lower.predict,
              upper.predict = m$upper.predict,
              seTE.hakn.pi = m$seTE.hakn.pi,
              seTE.hakn.adhoc.pi = m$seTE.hakn.adhoc.pi,
              seed.predict = m$seed.predict,
              #
              Q = m$Q, df.Q = m$df.Q, pval.Q = m$pval.Q,
              #
              method.tau = m$method.tau, method.tau.ci = m$method.tau.ci,
              level.hetstat = m$level.hetstat,
              tau2 = m$tau2,
              se.tau2 = m$se.tau2,
              lower.tau2 = m$lower.tau2, upper.tau2 = m$upper.tau2,
              tau = m$tau,
              lower.tau = m$lower.tau, upper.tau = m$upper.tau,
              detail.tau = m$detail.tau,
              sign.lower.tau = m$sign.lower.tau,
              sign.upper.tau = m$sign.upper.tau,
              #
              method.I2 = m$method.I2,
              #
              H = Hres$TE, lower.H = Hres$lower, upper.H = Hres$upper,
              #
              I2 = I2res$TE, lower.I2 = I2res$lower, upper.I2 = I2res$upper,
              #
              Rb = Rbres$TE, lower.Rb = Rbres$lower, upper.Rb = Rbres$upper,
              #
              method.bias = m$method.bias,
              #
              text.common = x$text.common,
              text.random = x$text.random,
              text.predict = x$text.predict,
              text.w.common = x$text.w.common,
              text.w.random = x$text.w.random,
              #
              title = x$title,
              complab = x$complab,
              outclab = x$outclab,
              label.e = x$label.e,
              label.c = x$label.c,
              label.left = x$label.left,
              label.right = x$label.right,
              #
              keepdata = FALSE,
              exclude = exclude.na,
              #
              left = left,
              ma.common = ma.common,
              type = type,
              n.iter.max = n.iter.max,
              n.iter = n.iter,
              #
              trimfill = trimfill,
              #
              class.x = class(x)[1]
              )
  #
  res$pscale <- pscale
  res$irscale <- irscale
  res$irunit <- irunit
  #
  res$n.e <- n.e
  res$n.c <- n.c
  res$n <- n
  #
  res$event.e <- event.e
  res$event.c <- event.c
  res$event <- event
  #
  res$time.e <- time.e
  res$time.c <- time.c
  res$time <- time
  #
  res$cor <- cor
  #
  res$mean.e <- mean.e
  res$mean.c <- mean.c
  #
  res$sd.e <- sd.e
  res$sd.c <- sd.c
  #
  call <- match.call()
  res$version <- packageDescription("meta")$Version
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  res$ma.fixed <- res$ma.common
  res$hakn <- m$hakn
  #  
  class(res) <- c("metagen", "meta", "trimfill")
  
  
  res
}





#' @rdname trimfill
#' @method trimfill default
#' @export

trimfill.default <- function(x, seTE, left = NULL, ma.common = TRUE,
                             type = "L", n.iter.max = 50,
                             sm = "", studlab = NULL,
                             level = 0.95, level.ma = level,
                             common = FALSE, random = TRUE,
                             #
                             method.common.ci = gs("method.common.ci"),
                             method.random.ci = gs("method.random.ci"),
                             adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                             #
                             method.tau = gs("method.tau"),
                             method.tau.ci =
                               if (method.tau == "DL") "J" else "QP",
                             level.hetstat = gs("level.hetstat"),
                             #
                             prediction = FALSE, level.predict = level,
                             method.predict = gs("method.predict"),
                             adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                             seed.predict = NULL,
                             #
                             backtransf = TRUE, pscale = 1,
                             irscale = 1, irunit = "person-years",
                             silent = TRUE, ...) {
  
  
  #
  #
  # (1) Check essential arguments
  #
  #
  k.All <- length(x)
  #
  chknumeric(x)
  chknumeric(seTE)
  chknull(sm)
  #
  fun <- "trimfill"
  chklength(seTE, k.All, fun)
  #
  if (!is.null(studlab))
    chklength(studlab, k.All, fun)
  else
    studlab <- seq(along = x)
  #
  if (is.null(sm))
    sm <- ""
  
  
  #
  #
  # (2) Do meta-analysis
  #
  #
  m <- metagen(x, seTE, studlab = studlab, sm = sm,
               level = level, level.ma = level.ma,
               common = common, random = random,
               #
               method.common.ci = method.common.ci,
               method.random.ci = method.random.ci,
               adhoc.hakn.ci = adhoc.hakn.ci,
               #
               method.tau = method.tau, method.tau.ci = method.tau.ci,
               level.hetstat = level.hetstat,
               #
               method.I2 = gs("method.I2"),
               #
               prediction = prediction,
               method.predict = method.predict,
               adhoc.hakn.pi = adhoc.hakn.pi,
               level.predict = level.predict,
               seed.predict = seed.predict,
               #
               backtransf = backtransf, pscale = pscale,
               irscale = irscale, irunit = irunit,
               ...)
  
  
  #
  #
  # (3) Run trim-and-fill method
  #
  #
  res <- trimfill(m, left = left, ma.common = ma.common,
                  type = type, n.iter.max = n.iter.max,
                  silent = silent, ...)
  
  
  res
}

#' L'Abbé plot for meta-analysis with binary outcomes
#' 
#' @description
#' Draw a L'Abbé plot for meta-analysis with binary outcomes.
#' 
#' @aliases labbe labbe.metabin labbe.default
#' 
#' @param x An object of class \code{metabin}. Alternatively, the x
#'   coordinates of points of the L'Abbé plot.
#' @param y The y coordinates of the L'Abbé plot, if argument \code{x}
#'   is not an object of class \code{metabin}.
#' @param xlim The x limits (min, max) of the plot.
#' @param ylim The y limits (min, max) of the plot.
#' @param xlab A label for the x-axis.
#' @param ylab A label for the y-axis.
#' @param TE.common A numeric or vector specifying combined common
#'   effect estimate(s).
#' @param TE.random A numeric or vector specifying combined random
#'   effects estimate(s).
#' @param common A logical indicating whether the common effect
#'   estimate should be plotted.
#' @param random A logical indicating whether the random effects
#'   estimate should be plotted.
#' @param backtransf A logical indicating which values should be
#'   printed on x- and y-axis (see Details).
#' @param axes A logical indicating whether axes should be drawn on
#'   the plot.
#' @param pch The plotting symbol used for individual studies.
#' @param text A character vector specifying the text to be used
#'   instead of plotting symbol.
#' @param cex The magnification to be used for plotting symbol.
#' @param col A vector with colour of plotting symbols.
#' @param bg A vector with background colour of plotting symbols (only
#'   used if \code{pch} in \code{21:25}).
#' @param lwd The line width.
#' @param lwd.common The line width(s) for common effect estimate(s)
#'   (if \code{common} is not \code{NULL} or \code{FALSE}).
#' @param lwd.random The line width(s) for random effects estimate(s)
#'   (if \code{random} is not \code{NULL} or \code{FALSE}).
#' @param lty.common Line type(s) for common effect estimate(s).
#' @param lty.random Line type(s) for random effects estimate(s).
#' @param col.common Colour of line(s) for common effect estimate(s).
#' @param col.random Colour of line(s) for random effects estimate(s).
#' @param nulleffect A logical indicating whether line for null effect
#'   should be added to the plot..
#' @param lwd.nulleffect Width of line for null effect.
#' @param col.nulleffect Colour of line for null effect.
#' @param sm A character string indicating underlying summary measure,
#'   i.e., \code{"RD"}, \code{"RR"}, \code{"OR"}, or \code{"ASD"}.
#' @param weight Either a numeric vector specifying relative sizes of
#'   plotting symbols or a character string indicating which type of
#'   plotting symbols is to be used for individual treatment
#'   estimates. One of missing (see Details), \code{"same"},
#'   \code{"common"}, or \code{"random"}, can be abbreviated. Plot
#'   symbols have the same size for all studies or represent study
#'   weights from common effect or random effects model.
#' @param studlab A logical indicating whether study labels should be
#'   printed in the graph. A vector with study labels can also be
#'   provided (must be of same length as \code{x$event.e} then).
#' @param cex.studlab Size of study labels.
#' @param pos.studlab Position of study labels, see argument
#'   \code{pos} in \code{\link{text}}.
#' @param label.e Label for experimental group.
#' @param label.c Label for control group.
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param TE.fixed Deprecated argument (replaced by 'TE.common').
#' @param fixed Deprecated argument (replaced by 'common').
#' @param lwd.fixed Deprecated argument (replaced by 'lwd.common').
#' @param lty.fixed Deprecated argument (replaced by 'lty.common').
#' @param col.fixed Deprecated argument (replaced by 'col.common').
#' @param \dots Graphical arguments as in \code{par} may also be
#'   passed as arguments.
#'
#' @details
#' A L'Abbé plot is a scatter plot with the risk in the control group
#' on the x-axis and the risk in the experimental group on the y-axis
#' (L'Abbé et al., 1987). It can be used to evaluate heterogeneity in
#' meta-analysis.  Furthermore, this plot can aid to choose a summary
#' measure (odds ratio, risk ratio, risk difference) that will result
#' in more consistent results (Jiménez et al., 1997; Deeks, 2002).
#' 
#' If argument \code{backtransf} is TRUE (default), event
#' probabilities will be printed on x- and y-axis. Otherwise,
#' transformed event probabilities will be printed as defined by the
#' summary measure, i.e., log odds of probabilities for odds ratio as
#' summary measure (\code{sm = "OR"}), log probabilities for \code{sm
#' = "RR"}, and arcsine-transformed probabilities for \code{sm =
#' "ASD"}.
#' 
#' If \code{common} is TRUE, the estimate of the common effct model is
#' plotted as a line. If \code{random} is TRUE, the estimate of the
#' random effects model is plotted as a line.
#' 
#' Information from object \code{x} is utilised if argument
#' \code{weight} is missing. Weights from the common effect model are
#' used (\code{weight = "common"}) if argument \code{x$common} is
#' \code{TRUE}; weights from the random effects model are used
#' (\code{weight = "random"}) if argument \code{x$random} is
#' \code{TRUE} and \code{x$common} is \code{FALSE}.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}
#' 
#' @references
#' Deeks JJ (2002):
#' Issues in the selection of a summary statistic for meta-analysis of
#' clinical trials with binary outcomes.
#' \emph{Statistics in Medicine},
#' \bold{21}, 1575--600
#'
#' Jiménez FJ, Guallar E, Martín-Moreno JM (1997):
#' A graphical display useful for meta-analysis.
#' \emph{European Journal of Public Health},
#' \bold{1}, 101--5
#' 
#' L'Abbé KA, Detsky AS, O'Rourke K (1987):
#' Meta-analysis in clinical research.
#' \emph{Annals of Internal Medicine},
#' \bold{107}, 224--33
#' 
#' @keywords hplot
#' 
#' @examples
#' data(Olkin1995)
#' ma <- metabin(ev.exp, n.exp, ev.cont, n.cont,
#'   data = Olkin1995, studlab = paste(author, year),
#'   sm = "RR", method = "I")
#' 
#' # L'Abbe plot for risk ratio
#' #
#' labbe(ma)
#' 
#' # L'Abbe plot for odds ratio
#' #
#' labbe(ma, sm = "OR")
#' # same plot
#' labbe(update(ma, sm = "OR"))
#' 
#' # L'Abbe plot for risk difference
#' #
#' labbe(ma, sm = "RD")
#' 
#' # L'Abbe plot on log odds scale
#' #
#' labbe(ma, sm = "OR", backtransf = FALSE)
#' 
#' # L'Abbe plot for odds ratio with coloured lines for various
#' # treatment effects (defined as log odds ratios)
#' #
#' mycols <- c("blue", "yellow", "green", "red", "green", "yellow", "blue")
#' labbe(ma, sm = "OR", random = FALSE,
#'   TE.common = log(c(1 / 10, 1 / 5, 1 / 2, 1, 2, 5, 10)),
#'   col.common = mycols, lwd.common = 2)
#' 
#' # L'Abbe plot on log odds scale with coloured lines for various
#' # treatment effects (defined as log odds ratios)
#' #
#' labbe(ma, sm = "OR", random = FALSE, backtransf = FALSE,
#'   TE.common = log(c(1 / 10, 1 / 5, 1 / 2, 1, 2, 5, 10)),
#'   col.common = mycols, lwd.common = 2)
#' 
#' @rdname labbe
#' @method labbe metabin
#' @export

labbe.metabin <- function(x,
                          xlim, ylim,
                          xlab = NULL, ylab = NULL,
                          TE.common = x$TE.common,
                          TE.random = x$TE.random,
                          common = x$common,
                          random = x$random,
                          backtransf = x$backtransf,
                          axes = TRUE,
                          pch = 21, text = NULL, cex = 1,
                          col = "black", bg = "lightgray",
                          lwd = 1, lwd.common = lwd, lwd.random = lwd,
                          lty.common = 2, lty.random = 9,
                          col.common = col, col.random = col,
                          nulleffect = TRUE,
                          lwd.nulleffect = lwd, col.nulleffect = "lightgray",
                          sm = x$sm, weight,
                          studlab = FALSE, cex.studlab = 0.8, pos.studlab = 2,
                          label.e = x$label.e, label.c = x$label.c,
                          warn.deprecated = gs("warn.deprecated"),
                          #
                          TE.fixed, fixed, lwd.fixed, lty.fixed, col.fixed,
                          #
                          ...) {
  
  
  #
  #
  # (1) Check for meta object and upgrade older meta objects
  #
  #
  chkclass(x, "metabin")
  x <- updateversion(x)
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  common <-
    deprecated2(common, missing(common), fixed, missing(fixed),
                warn.deprecated)
  chklogical(common)
  chklogical(random)
  chklogical(backtransf)
  chklogical(axes)
  chknumeric(cex)
  chknumeric(lwd)
  lwd.common <-
    deprecated2(lwd.common, missing(lwd.common),
                lwd.fixed, missing(lwd.fixed),
                warn.deprecated)
  chknumeric(lwd.common)
  lty.common <-
    deprecated2(lty.common, missing(lty.common),
                lty.fixed, missing(lty.fixed),
                warn.deprecated)
  chknumeric(lty.common)
  chknumeric(lty.random)
  col.common <-
    deprecated2(col.common, missing(col.common),
                col.fixed, missing(col.fixed),
                warn.deprecated)
  chklogical(nulleffect)
  chknumeric(lwd.nulleffect)
  chknull(sm)
  sm <- setchar(sm, gs("sm4bin"))
  chknumeric(cex.studlab)
  chklogical(warn.deprecated)
  #
  pos.studlab <- as.numeric(setchar(pos.studlab, as.character(1:4)))
  
  
  if (!backtransf) {
    xpos <- (x$event.c + x$incr.c) / (x$n.c + 2 * x$incr.c)
    ypos <- (x$event.e + x$incr.e) / (x$n.e + 2 * x$incr.e)
  }
  else {
    xpos <- x$event.c / x$n.c
    ypos <- x$event.e / x$n.e
  }
  #
  if(length(xpos) != length(ypos))
    stop("event rates must be of same length")
  
  
  if (sm != x$sm) {
    m <- update(x, sm = sm)
    if (missing(TE.common)) {
      if (!missing(TE.fixed)) {
        if (warn.deprecated)
          warning(paste("Use argument 'TE.common' instead of",
                        "'TE.fixed' (deprecated)."),
                  call. = FALSE)
        TE.common <- TE.fixed
      }
    }
    else
      TE.common <- m$TE.common
    #
    if (missing(TE.random))
      TE.random <- m$TE.random
    w.common <- m$w.common
    w.random <- m$w.random
  }
  else {
    w.common <- x$w.common
    w.random <- x$w.random
  }
  #
  # Exclude studies from meta-analysis
  #
  if (!is.null(x$exclude)) {
    xpos <- xpos[!x$exclude]
    ypos <- ypos[!x$exclude]
    w.common <- w.common[!x$exclude]
    w.random <- w.random[!x$exclude]
  }
  
  
  if (!backtransf) {
    if (sm %in% c("OR", "DOR")) {
      xpos <- log(xpos / (1 - xpos))
      ypos <- log(ypos / (1 - ypos))
    }
    else if (sm == "RR") {
      xpos <- log(xpos)
      ypos <- log(ypos)
    }
    else if (sm == "ASD") {
      xpos <- asin(sqrt(xpos))
      ypos <- asin(sqrt(ypos))
    }
  }
  
  
  if (missing(weight))
    weight <- ifelse(random & !common, "random", "common")
  #
  iweight <- charmatch(tolower(weight),
                       c("same", "common", "random"), nomatch = NA)
  #
  if(is.na(iweight))
    stop("weight should be \"same\", \"common\", or \"random\"")
  #
  weight <- c("same", "common", "random")[iweight]
  #
  if (weight == "common")
    cex.i <- 4 * cex * sqrt(w.common) / sqrt(max(w.common))
  else if (weight == "random")
    cex.i <- 4 * cex * sqrt(w.random) / sqrt(max(w.random))
  else if (weight == "same")
    cex.i <- cex
  #
  if (min(cex.i) < 0.5)
    cex.i <- cex.i + (0.5 - min(cex.i))
  
  
  if (is.logical(studlab) && studlab)
    studlab <- x$studlab
  
  
  if (length(common) == 0)
    common <- TRUE
  if (length(random) == 0)
    random <- TRUE
  
  
  if (!backtransf)
    minval <- min(c(xpos, ypos), na.rm = TRUE)
  else
    minval <- 0
  #
  if (missing(xlim) & missing(ylim)) {
    xlim <- c(minval, max(c(xpos, ypos), na.rm = TRUE))
    ylim <- xlim
  }
  if (missing(xlim))
    xlim <- c(minval, max(c(xpos, ypos), na.rm = TRUE))
  if (missing(ylim))
    ylim <- xlim
  #
  chknumeric(xlim, length = 2)
  chknumeric(ylim, length = 2)
  
  
  oldpar <- par(pty = "s")
  on.exit(par(oldpar))
  
  
  if (is.null(xlab)) {
    if (length(label.c) > 0) {
      xlab <- paste0("Event rate (", label.c, ")")
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          xlab <- paste0("log(odds) ", label.c)
        else if (sm == "RR")
          xlab <- paste0("log(event rate) ", label.c)
        else if (sm == "ASD")
          xlab <- paste0("Arcsin-transformed event rate (", label.c, ")")
    }
    else {
      xlab <- "Event rate (Control)"
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          xlab <- "log(odds) Control"
        else if (sm == "RR")
          xlab <- "log(event rate) Control"
        else if (sm == "ASD")
          xlab <- "Arcsin-transformed event rate (Control)"
    }
  }
  #
  if (is.null(ylab)) {
    if (length(label.e) > 0) {
      ylab <- paste0("Event rate (", label.e, ")")
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          ylab <- paste0("log(odds) ", label.e)
        else if (sm == "RR")
          ylab <- paste0("log(event rate) ", label.e)
        else if (sm == "ASD")
          ylab <- paste0("Arcsin-transformed event rate (", label.e, ")")
    }
    else {
      ylab <- "Event rate (Experimental)"
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          ylab <- "log(odds) Experimental"
        else if (sm == "RR")
          ylab <- "log(event rate) Experimental"
        else if (sm == "ASD")
          ylab <- "Arcsin-transformed event rate (Experimental)"
    }
  }
  
  
  if (common && length(lty.common) == 1 & length(TE.common) > 1)
    lty.common <- rep(lty.common, length(TE.common))
  #
  if (common && length(lwd.common) == 1 & length(TE.common) > 1)
    lwd.common <- rep(lwd.common, length(TE.common))
  #
  if (common && length(col.common) == 1 & length(TE.common) > 1)
    col.common <- rep(col.common, length(TE.common))
  
  if (random && length(lty.random) == 1 & length(TE.random) > 1)
    lty.random <- rep(lty.random, length(TE.random))
  #
  if (random && length(lwd.random) == 1 & length(TE.random) > 1)
    lwd.random <- rep(lwd.random, length(TE.random))
  #
  if (random && length(col.random) == 1 & length(TE.random) > 1)
    col.random <- rep(col.random, length(TE.random))
  
  
  #
  # Generate L'Abbe plot
  #
  plot(xpos, ypos, type = "n", 
       xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab,
       axes = axes, ...)
  #
  if (nulleffect)
    abline(0, 1, lwd = lwd.nulleffect, col = col.nulleffect)
  #
  points(xpos, ypos, pch = pch, cex = cex.i, col = col, bg = bg, lwd = lwd)
  
  
  #
  # Auxillary function
  #
  addlines <- function(x.line, y.line, ylim, lty, lwd, col) {
    sel <- min(ylim) <= y.line & y.line <= max(ylim)
    if (sum(sel) > 1)
      lines(x.line[sel], y.line[sel],
            lty = lty, lwd = lwd, col = col)
  }
  
  
  #
  # Add results for common effect model
  #
  if (common & length(TE.common) > 0) {
    x.line <- seq(min(xlim), max(xlim), len = 100)
    #
    if (!backtransf)
      for (i in 1:length(TE.common))
        abline(TE.common[i], 1,
               lty = lty.common[i], lwd = lwd.common[i],
               col = col.common[i])
    else {
      if (sm == "RR") {
        for (i in 1:length(TE.common)) {
          y.line <- x.line * exp(TE.common[i])
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
      else if (sm == "RD") {
        for (i in 1:length(TE.common)) {
          y.line <- x.line + TE.common[i]
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
      else if (sm %in% c("OR", "DOR")) {
        for (i in 1:length(TE.common)) {
          y.line <- exp(TE.common[i]) * (x.line / (1 - x.line)) /
            (1 + exp(TE.common[i]) * x.line / (1 - x.line))
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
      else if (sm == "ASD" & length(TE.common) > 0) {
        for (i in 1:length(TE.common)) {
          y.line <- sin(asin(sqrt(x.line)) + TE.common[i])^2
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
    }
  }
  
  
  #
  # Add results for random effects model
  #
  if (random & length(TE.random) > 0) {
    x.line <- seq(min(xlim), max(xlim), len = 100)
    #
    if (!backtransf)
      for (i in 1:length(TE.random))
        abline(TE.random[i], 1,
               lty = lty.random[i], lwd = lwd.random[i],
               col = col.random[i])
    else {
      if (sm == "RR") {
        for (i in 1:length(TE.random)) {
          y.line <- x.line * exp(TE.random[i])
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
      else if (sm == "RD") {
        for (i in 1:length(TE.random)) {
          y.line <- x.line + TE.random[i]
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
      else if (sm %in% c("OR", "DOR")) {
        for (i in 1:length(TE.random)) {
          y.line <- exp(TE.random[i]) * (x.line / (1 - x.line)) /
            (1 + exp(TE.random[i]) * x.line / (1 - x.line))
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
      else if (sm == "ASD" & length(TE.random) > 0) {
        for (i in 1:length(TE.random)) {
          y.line <- sin(asin(sqrt(x.line)) + TE.random[i])^2
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
    }
  }


  #
  # Add study labels
  #
  if (!is.logical(studlab) && length(studlab) > 0)
    text(xpos, ypos, labels = studlab, pos = pos.studlab, cex = cex.studlab)
  
  
  invisible(NULL)
}





#' @rdname labbe
#' @method labbe default
#' @export

labbe.default <- function(x, y,
                          xlim, ylim,
                          xlab = NULL, ylab = NULL,
                          TE.common = NULL, TE.random = NULL,
                          common = !is.null(TE.common),
                          random = !is.null(TE.random),
                          backtransf = TRUE,
                          axes = TRUE,
                          pch = 21, text = NULL, cex = 1,
                          col = "black", bg = "lightgray",
                          lwd = 1, lwd.common = lwd, lwd.random = lwd,
                          lty.common = 2, lty.random = 9,
                          col.common = col, col.random = col,
                          nulleffect = TRUE,
                          lwd.nulleffect = lwd, col.nulleffect = "lightgray",
                          sm = "", weight,
                          studlab = FALSE, cex.studlab = 0.8, pos.studlab = 2,
                          label.e = NULL, label.c = NULL,
                          #
                          warn.deprecated = gs("warn.deprecated"),
                          TE.fixed, fixed, lwd.fixed, lty.fixed, col.fixed,
                          ...) {
  
  #
  #
  # (1) Check arguments
  #
  #
  xpos <- x
  ypos <- y
  #
  if(length(xpos) != length(ypos))
    stop("arguments 'x' and 'y' must be of same length")
  #
  common <-
    deprecated2(common, missing(common), fixed, missing(fixed),
                warn.deprecated)
  chklogical(common)
  chklogical(random)
  chklogical(backtransf)
  chklogical(axes)
  chknumeric(cex)
  chknumeric(lwd)
  lwd.common <-
    deprecated2(lwd.common, missing(lwd.common),
                lwd.fixed, missing(lwd.fixed),
                warn.deprecated)
  chknumeric(lwd.common)
  chknumeric(lwd.random)
  lty.common <-
    deprecated2(lty.common, missing(lty.common),
                lty.fixed, missing(lty.fixed),
                warn.deprecated)
  chknumeric(lty.common)
  chknumeric(lty.random)
  col.common <-
    deprecated2(col.common, missing(col.common),
                col.fixed, missing(col.fixed),
                warn.deprecated)
  chklogical(nulleffect)
  chknumeric(lwd.nulleffect)
  chknull(sm)
  sm <- setchar(sm, gs("sm4bin"))
  chknumeric(cex.studlab)
  chklogical(warn.deprecated)
  #
  pos.studlab <- as.numeric(setchar(pos.studlab, as.character(1:4)))
  
  
  if (!backtransf) {
    if (sm %in% c("OR", "DOR")) {
      xpos <- log(xpos / (1 - xpos))
      ypos <- log(ypos / (1 - ypos))
    }
    else if (sm == "RR") {
      xpos <- log(xpos)
      ypos <- log(ypos)
    }
    else if (sm == "ASD") {
      xpos <- asin(sqrt(xpos))
      ypos <- asin(sqrt(ypos))
    }
  }
  
  
  if (!missing(weight))
    cex.i <- 4 * cex * sqrt(weight) / sqrt(max(weight))
  else
    cex.i <- rep(cex, length(xpos))
  #
  if (min(cex.i) < 0.5)
    cex.i <- cex.i + (0.5 - min(cex.i))
  
  
  if (backtransf)
    minval <- 0
  else
    minval <- min(c(xpos, ypos), na.rm = TRUE)
  #
  if (missing(xlim) & missing(ylim)) {
    xlim <- c(minval, max(c(xpos, ypos), na.rm = TRUE))
    ylim <- xlim
  }
  if (missing(xlim))
    xlim <- c(minval, max(c(xpos, ypos), na.rm = TRUE))
  if (missing(ylim))
    ylim <- xlim
  
  
  oldpar <- par(pty = "s")
  on.exit(par(oldpar))
  
  
  if (is.null(xlab)) {
    if (length(label.c) > 0) {
      xlab <- paste0("Event rate (", label.c, ")")
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          xlab <- paste0("log(odds) ", label.c)
        else if (sm == "RR")
          xlab <- paste0("log(event rate) ", label.c)
        else if (sm == "ASD")
          xlab <- paste0("Arcsin-transformed event rate (", label.c, ")")
    }
    else {
      xlab <- "Event rate (Control)"
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          xlab <- "log(odds) Control"
        else if (sm == "RR")
          xlab <- "log(event rate) Control"
        else if (sm == "ASD")
          xlab <- "Arcsin-transformed event rate (Control)"
    }
  }
  #
  if (is.null(ylab)) {
    if (length(label.e) > 0) {
      ylab <- paste0("Event rate (", label.e, ")")
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          ylab <- paste0("log(odds) ", label.e)
        else if (sm == "RR")
          ylab <- paste0("log(event rate) ", label.e)
        else if (sm == "ASD")
          ylab <- paste0("Arcsin-transformed event rate (", label.e, ")")
    }
    else {
      ylab <- "Event rate (Experimental)"
      if (!backtransf)
        if (sm %in% c("OR", "DOR"))
          ylab <- "log(odds) Experimental"
        else if (sm == "RR")
          ylab <- "log(event rate) Experimental"
        else if (sm == "ASD")
          ylab <- "Arcsin-transformed event rate (Experimental)"
    }
  }
  
  
  if (common && length(lty.common) == 1 & length(TE.common) > 1)
    lty.common <- rep(lty.common, length(TE.common))
  #
  if (common && length(lwd.common) == 1 & length(TE.common) > 1)
    lwd.common <- rep(lwd.common, length(TE.common))
  #
  if (common && length(col.common) == 1 & length(TE.common) > 1)
    col.common <- rep(col.common, length(TE.common))
  
  if (random && length(lty.random) == 1 & length(TE.random) > 1)
    lty.random <- rep(lty.random, length(TE.random))
  #
  if (random && length(lwd.random) == 1 & length(TE.random) > 1)
    lwd.random <- rep(lwd.random, length(TE.random))
  #
  if (random && length(col.random) == 1 & length(TE.random) > 1)
    col.random <- rep(col.random, length(TE.random))
  
  
  #
  # Generate L'Abbe plot
  #
  plot(xpos, ypos, type = "n", 
       xlim = xlim, ylim = ylim, xlab = xlab, ylab = ylab,
       axes = axes, ...)
  #
  if (nulleffect)
    abline(0, 1, lwd = lwd.nulleffect, col = col.nulleffect)
  #
  points(xpos, ypos, pch = pch, cex = cex.i, col = col, bg = bg, lwd = lwd)
  
  
  #
  # Auxillary function
  #
  addlines <- function(x.line, y.line, ylim, lty, lwd, col) {
    sel <- min(ylim) <= y.line & y.line <= max(ylim)
    if (sum(sel) > 1)
      lines(x.line[sel], y.line[sel],
            lty = lty, lwd = lwd, col = col)
  }
  
  
  #
  # Add results for common effect model
  #
  if (common & length(TE.common) > 0) {
    x.line <- seq(min(xlim), max(xlim), len = 100)
    #
    if (!backtransf)
      for (i in 1:length(TE.common))
        abline(TE.common[i], 1,
               lty = lty.common[i], lwd = lwd.common[i],
               col = col.common[i])
    else {
      if (sm == "RR") {
        for (i in 1:length(TE.common)) {
          y.line <- x.line * exp(TE.common[i])
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
      else if (sm == "RD") {
        for (i in 1:length(TE.common)) {
          y.line <- x.line + TE.common[i]
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
      else if (sm %in% c("OR", "DOR")) {
        for (i in 1:length(TE.common)) {
          y.line <- exp(TE.common[i]) * (x.line / (1 - x.line)) /
            (1 + exp(TE.common[i]) * x.line / (1 - x.line))
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
      else if (sm == "ASD" & length(TE.common) > 0) {
        for (i in 1:length(TE.common)) {
          y.line <- sin(asin(sqrt(x.line)) + TE.common[i])^2
          addlines(x.line, y.line, ylim,
                   lty.common[i], lwd.common[i], col.common[i])
        }
      }
    }
  }
  
  
  #
  # Add results for random effects model
  #
  if (random & length(TE.random) > 0) {
    x.line <- seq(min(xlim), max(xlim), len = 100)
    #
    if (!backtransf)
      for (i in 1:length(TE.random))
        abline(TE.random[i], 1,
               lty = lty.random[i], lwd = lwd.random[i],
               col = col.random[i])
    else {
      if (sm == "RR") {
        for (i in 1:length(TE.random)) {
          y.line <- x.line * exp(TE.random[i])
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
      else if (sm == "RD") {
        for (i in 1:length(TE.random)) {
          y.line <- x.line + TE.random[i]
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
      else if (sm %in% c("OR", "DOR")) {
        for (i in 1:length(TE.random)) {
          y.line <- exp(TE.random[i]) * (x.line / (1 - x.line)) /
            (1 + exp(TE.random[i]) * x.line / (1 - x.line))
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
      else if (sm == "ASD" & length(TE.random) > 0) {
        for (i in 1:length(TE.random)) {
          y.line <- sin(asin(sqrt(x.line)) + TE.random[i])^2
          addlines(x.line, y.line, ylim,
                   lty.random[i], lwd.random[i], col.random[i])
        }
      }
    }
  }


  #
  # Add study labels
  #
  if (!is.logical(studlab) && length(studlab) > 0)
    text(xpos, ypos, labels = studlab, pos = pos.studlab, cex = cex.studlab)
  
  
  invisible(NULL)
}

#' Import data of Cochrane intervention review
#' 
#' @description
#' Reads Cochrane data package (version 1) of a Cochrane intervention
#' review and creates a data frame from it.
#' 
#' @aliases read.cdir
#' 
#' @param file The name of a file to read data values from.
#' @param title Title of Cochrane review.
#' @param exdir The directory to extract files to (the equivalent of
#'   ‘unzip -d’). It will be created if necessary.
#' @param numbers.in.labels A logical indicating whether comparison
#'   number and outcome number should be printed at the beginning of
#'   the comparison (argument \code{complab}) and outcome label
#'   (argument \code{outclab}); this is the default in RevMan Web.
#' @param rob A logical indicating whether risk of bias (RoB)
#'   assessment should be considered in meta-analyses.
#' @param tool Risk of bias (RoB) tool.
#' @param categories Possible RoB categories.
#' @param col Colours for RoB categories.
#' @param symbols Corresponding symbols for RoB categories.
#' @param keep.orig A logical indicating whether to return the
#'   original data files.
#' @param \dots Additional arguments (passed on to
#'   \code{\link[utils]{unzip}})
#' @param x An object of class "cdir".
#' 
#' @details
#' RevMan Web is the current software used for preparing and maintaining
#' Cochrane reviews. RevMan Web includes the ability to write systematic
#' reviews of interventions or diagnostic test accuracy reviews.
#' 
#' This function provides the ability to read the Cochrane data
#' package from a Cochrane intervention review created with RevMan
#' Web. The ZIP-file is extracted with \code{\link[utils]{unzip}}.
#' 
#' Argument \code{title} can be used to overwrite the title of the
#' Cochrane review.
#'
#' Information on the risk of bias (RoB) assessment can be provided
#' with arguments \code{tool}, \code{categories}, \code{col} and
#' \code{symbols}. This is only useful if (i) all outcomes are based
#' on the same RoB categories and (ii) an overall RoB assessment has
#' not been done. If no overall RoB assessment was conducted, R
#' function \code{\link{metacr}} can be used to provide the RoB
#' information for a single outcome. R function \code{\link{rob}} is
#' the most flexible way to add RoB information to a meta-analysis
#' object.
#' 
#' \subsection{Creation of Cochrane data package}{
#' 
#' Two possible ways exist to create the ZIP-file.
#'
#' In RevMan Web, press the "Export" button at the bottom of the
#' \emph{Default view} website. After a couple of seconds, the data package will
#' be shown at the bottom of the \emph{Default view} website under "Downloads".
#'
#' In the Cochrane Library, press on "Download statistical data" in the
#' Contents menu to download an rm5-file. This file can be converted to a data
#' package in RevMan Web using \emph{Help} - \emph{Convert a RevMan 5 file}.
#' }
#' 
#' @return
#' A list consisting of a data frame 'data' with the study data and
#' (if available) a data frame 'rob' with information on the risk of
#' bias assessment. If \code{keep.orig = TRUE}, an additional list
#' 'orig' is returned containing elements 'settings', 'datarows',
#' 'subgroup' and 'rob' (if available).
#'
#' The data frame 'data' contains the following variables:
#' 
#' \item{comp.no}{Comparison number.}
#' \item{outcome.no}{Outcome number.}
#' \item{group.no}{Group number.}
#' \item{studlab}{Study label.}
#' \item{year}{Year of publication.}
#' \item{event.e}{Number of events in experimental group.}
#' \item{n.e}{Number of observations in experimental group.}
#' \item{event.c}{Number of events in control group.}
#' \item{n.c}{Number of observations in control group.}
#' \item{mean.e}{Estimated mean in experimental group.}
#' \item{sd.e}{Standard deviation in experimental group.}
#' \item{mean.c}{Estimated mean in control group.}
#' \item{sd.c}{Standard deviation in control group.}
#' \item{O.E}{Observed minus expected (IPD analysis).}
#' \item{V}{Variance of \code{O.E} (IPD analysis).}
#' \item{TE, seTE}{Estimated treatment effect and standard error of
#'   individual studies.}
#' \item{lower, upper}{Lower and upper limit of 95\% confidence
#'   interval for treatment effect in individual studies.}
#' \item{weight}{Weight of individual studies (according to
#'   meta-analytical method used in respective meta-analysis - see
#'   details).}
#' \item{order}{Ordering of studies.}
#' \item{grplab}{Group label.}
#' \item{type}{Type of outcome. D = dichotomous, C = continuous, P =
#'   IPD.}
#' \item{method}{A character string indicating which method has been
#'   used for pooling of studies. One of \code{"Inverse"},
#'   \code{"MH"}, or \code{"Peto"}.}
#' \item{sm}{A character string indicating which summary measure has
#'   been used for pooling of studies.}
#' \item{model}{A character string indicating which meta-analytical
#'   model has been used (either \code{"Fixed"} or \code{"Random"}).}
#' \item{common}{A logical indicating whether common effect
#'   meta-analysis has been used in respective meta-analysis (see
#'   details).}
#' \item{random}{A logical indicating whether random effects
#'   meta-analysis has been used in respective meta-analysis (see
#'   details).}
#' \item{title}{Title of Cochrane review.}
#' \item{complab}{Comparison label.}
#' \item{outclab}{Outcome label.}
#' \item{label.e}{Label for experimental group.}
#' \item{label.c}{Label for control group.}
#' \item{label.left}{Graph label on left side of forest plot.}
#' \item{label.right}{Graph label on right side of forest plot.}
#'
#' The data frame 'rob' contains the following variables:
#'
#' \item{studlab}{Study label.}
#' \item{D1, D2, \dots}{Risk of bias domain 1, 2, \dots}
#' \item{D1.details, D2.details, \dots}{Details on risk of bias domain
#'   1, 2, \dots}
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metacr}}, \code{\link{rob}},
#'   \code{\link{read.rm5}}
#' 
#' @references
#' 
#' \url{https://documentation.cochrane.org/revman-kb/data-package-user-guide-243761660.html}
#' 
#' \url{https://documentation.cochrane.org/revman-kb/data-package-specification-249561249.html}
#' 
#' @keywords datagen
#' 
#' @examples
#' # Locate file "Fleiss1993.zip" with Cochrane data package in
#' # sub-directory of R package meta
#' #
#' filename <- system.file("extdata/Fleiss1993.zip", package = "meta")
#' Fleiss1993_CR <- read.cdir(filename)
#' Fleiss1993_CR
#' 
#' # Same result as R Command example(Fleiss1993bin):
#' #
#' metacr(Fleiss1993_CR)
#' 
#' # Same result as R Command example(Fleiss1993cont):
#' #
#' metacr(Fleiss1993_CR, 1, 2)
#' 
#' @rdname read.cdir
#' @export read.cdir

read.cdir <- function(file, title = "Cochrane Review of Interventions",
                      exdir = tempdir(),
                      numbers.in.labels = TRUE,
                      #
                      rob =
                        !missing(tool) | !missing(categories) |
                        !missing(col) | !missing(symbols),
                      tool = NULL,
                      categories = NULL,
                      col = NULL,
                      symbols = NULL,
                      #
                      keep.orig = FALSE,
                      ...) {
  
  #
  #
  # (1) Check argument and extract filename and Cochrane data package
  #     identifier (CD)
  #
  #
  
  if (missing(file))
    stop("File name must be provided", call. = FALSE)
  #
  chkchar(file, length = 1)
  #
  if (!file.exists(file))
    stop("File '", file, "' does not exists.",
         call. = FALSE)
  #
  path <- file
  file <- basename(file)
  if (substring(file, 1, 2) == "CD")
    CD <- unlist(strsplit(file, "-"))[1]
  else
    CD <- NULL
  #
  data.package <- grepl("dataPackage.zip$", file)
  #
  chkchar(title, length = 1)
  chklogical(numbers.in.labels)
  chkchar(exdir, length = 1)
  chklogical(keep.orig)
  chklogical(rob)
  #
  # Get rid of warnings 'no visible binding for global variable'
  #
  common <- comp.no <- complab <- eligibility <-
    event.c <- event.e <- footnotes <- group.no <- label.c <- label.e <-
      label.left <- label.right <- level <- level.ma <- logscale <-
        lower.TE <- mean.c <- mean.e <- method <- model <- n.c <- n.e <-
          O.E <- outclab <- outclab2 <- outcome.no <- overall <- random <-
            sd.c <- sd.e <- seTE <- show.subgroup <- sm <- studlab <-
              Study <- swap.events <- TE <- test.subgroup <- type <-
                upper.TE <- V <- weight <- year <- NULL
  
  
  #
  #
  # (2) Unzip Cochrane data package with subdirectories
  #
  #
  
  dir.create(exdir, showWarnings = FALSE, recursive = TRUE)
  unzip(path, exdir = exdir, ...)
  #
  if (is.null(CD)) {
    lf <- list.files(exdir)
    #
    if (any(grepl("-files$", lf)))
      CD <- unlist(strsplit(lf[grep("-files$", lf)], "-"))[1]
    else {
      ld <- list.dirs(exdir)
      ld <- ld[ld != exdir]
      exdir <- ld[1]
      lf <- list.files(exdir)
      CD <- unlist(strsplit(lf[grep("-files$", lf)], "-"))[1]
    }
  }
  #
  if (data.package)
    wd <- exdir
  else
    wd <- paste0(exdir, "/", CD, "-files")  
  #
  if (!dir.exists(wd))
    stop("Data directory '", wd, "' does not exist.")
  #
  if (!data.package) {
    datazip <- paste0(wd, "/", CD, "-data.zip")
    #
    if (!file.exists(datazip))
      stop("File '", datazip, "' does not exists.",
           call. = FALSE)
    else
      unzip(datazip, exdir = wd)
  }
  
  
  #
  #
  # (3) Read files from directory CD-analysis-data
  #
  #
  
  dir.analysis <- paste0(wd, "/", CD, "-analysis-data")
  #
  if (!dir.exists(dir.analysis))
    stop("Directory '", dir.analysis, "' does not exist.",
         call. = FALSE)
  #
  # Check for file specific to Cochrane review of DTA studies
  #
  if (file.exists(paste0(dir.analysis, "/", CD, "-parameters.csv")))
    stop("Cochrane data package is from diagnostic test accuracy review.",
         call. = FALSE)
  #
  file.settings <-
    paste0(dir.analysis, "/", CD, "-overall-estimates-and-settings.csv")
  file.datarows <-
    paste0(dir.analysis, "/", CD, "-data-rows.csv")
  file.subgroup <-
    paste0(dir.analysis, "/", CD, "-subgroup-estimates.csv")
  #
  if (file.exists(file.settings))
    settings <- read_csv(file.settings, col_types = cols())
  else
    stop("File '", file.settings, "' does not exists.",
         call. = FALSE)
  #
  if (file.exists(file.datarows))
    datarows <- read_csv(file.datarows, col_types = cols())
  else
    stop("File '", file.datarows, "' does not exists.",
         call. = FALSE)
  #
  if (file.exists(file.subgroup))
    subgroup <- read_csv(file.subgroup, col_types = cols())
  else
    stop("File '", file.subgroup, "' does not exists.",
         call. = FALSE)
  #
  if (keep.orig) {
    settings.orig <- settings
    datarows.orig <- datarows
    subgroup.orig <- subgroup
  }
  #
  # Check whether meta-analyses have been conducted
  #
  if (nrow(datarows) == 0) {
    warning("No meta-analyses have been conducted.",
            call. = FALSE)
    return(NULL)
  }
  
  
  #
  #
  # (4) Create study data set (with information on analyses)
  #
  #
  
  settings %<>%
    rename(
      comp.no = "Analysis group",
      outcome.no = "Analysis number",
      type = "Data type",
      method = "Statistical method",
      sm = "Effect measure",
      level = "Study CI",
      level.ma = "Estimate CI",
      unit = "Unit of effect measure",
      model = "Analysis model",
      logscale = "Log-scale data",
      #
      label.e = "Experimental group label",
      label.c = "Control group label",
      #
      overall = "Overall estimates",
      test.subgroup = "Test for subgroup differences",
      show.subgroup = "Subgroup estimates",
      #
      swap.events = "Swap event and non-event",
      #
      complab = "Analysis group name",
      outclab = "Analysis name",
      #
      data.source = "Data source",
      eligibility = "Data source eligibility"
    ) %>%
  mutate(
    comp.no = as.numeric(comp.no),
    outcome.no = as.numeric(outcome.no),
    type = if_else(type == "Contrast level", "I",
                   if_else(type == "O-E and variance", "P",
                           substring(type, 1, 1))),
    common = model == "Fixed effect",
    random = model == "Random effect",
    method = sm2meth(method),
    sm = em2sm(sm, type),
    eligibility = if_else(eligibility == "#N/A", NA, eligibility),
    model = rmSpace(substring(model, 1, 6), end = TRUE),
    level = as.numeric(gsub("%", "", level)) / 100,
    level.ma = as.numeric(gsub("%", "", level.ma)) / 100,
    label.e = if_else(is.na(label.e), "Experimental", label.e),
    label.c = if_else(is.na(label.c), "Control", label.c),
    label.left = "",
    label.right = ""
  ) %>%
  select(
    comp.no, outcome.no, complab, outclab,
    type, method, sm, model, common, random, swap.events, logscale, unit,
    level, level.ma,
    label.e, label.c, label.left, label.right,
    overall, test.subgroup, show.subgroup, swap.events) %>%
  as.data.frame()
  #
  datarows %<>%
    rename(
      comp.no = "Analysis group",
      outcome.no = "Analysis number",
      #
      outclab2 = "Analysis name",
      #
      subgroup = "Subgroup",
      studlab = "Study",
      year = "Study year",
      #
      TE = "GIV Mean",
      seTE = "GIV SE",
      lower.TE = "CI start",
      upper.TE = "CI end",
      #
      event.e = "Experimental cases",
      n.e = "Experimental N",
      mean.e = "Experimental mean",
      sd.e = "Experimental SD",
      #
      event.c = "Control cases",
      n.c = "Control N",
      mean.c = "Control mean",
      sd.c = "Control SD",
      #
      O.E = "O-E",
      V = "Variance",
      weight = "Weight",
      #
      footnotes = "Footnotes"
    ) %>%
  mutate(
    comp.no = as.numeric(comp.no),
    outcome.no = as.numeric(outcome.no),
    lower.TE = if_else(is.na(TE), NA, lower.TE),
    upper.TE = if_else(is.na(TE), NA, upper.TE)
    ) %>%
    select(
      comp.no, outcome.no, outclab2, subgroup,
      studlab, year,
      event.e, n.e, event.c, n.c, mean.e, sd.e, mean.c, sd.c,
      O.E, V, TE, seTE, lower.TE, upper.TE,
      weight, footnotes) %>%
  as.data.frame()
  #
  subgroup %<>%
    rename(
      comp.no = "Analysis group",
      outcome.no = "Analysis number",
      group.no = "Subgroup number",
      subgroup = "Subgroup"
    ) %>%
    mutate(
      comp.no = as.numeric(comp.no),
      outcome.no = as.numeric(outcome.no)
    ) %>%
    select(
      comp.no, outcome.no, group.no, subgroup
    ) %>%
    as.data.frame()
  #
  # Full Cochrane data set
  #
  data <- merge(settings, datarows,
                by = c("comp.no", "outcome.no"), all = TRUE)
  #
  data <- merge(data, subgroup,
                by = c("comp.no", "outcome.no", "subgroup"), all.x = TRUE)
  #
  suppressWarnings(
    data %<>%
    mutate(
      lower.TE = if_else(logscale & !is.na(lower.TE), log(lower.TE), lower.TE),
      upper.TE = if_else(logscale & !is.na(upper.TE), log(upper.TE), upper.TE),
      order = seq_len(nrow(data))
    )
  )
  #
  data %<>%
    select(
      comp.no, outcome.no, group.no, studlab, year,
      event.e, n.e, event.c, n.c, mean.e, sd.e, mean.c, sd.c,
      O.E, V, TE, seTE, lower.TE, upper.TE,
      weight, order,
      type, method, sm, model, common, random, swap.events, logscale,
      level, level.ma,
      label.e, label.c, label.left, label.right, unit,
      complab, outclab, footnotes
    )
  #
  if (numbers.in.labels) {
    data$complab <- paste(data$comp.no, data$complab)
    data$outclab <- paste0(data$comp.no, ".", data$outcome.no, " ", data$outclab)
  }
  #
  attr(data, "title") <- title
  
  
  #
  #
  # (5) Extract risk of bias (if available)
  #
  #
  
  dir.study <- paste0(wd, "/", CD, "-study-data")
  #
  robdata <- NULL
  #
  if (dir.exists(dir.study)) {
    #
    file.robdata <- paste0(dir.study, "/", CD, "-risk-of-bias.csv")
    #
    rob <- rob && file.exists(file.robdata)
    #
    if (rob) {
      robdata <- as.data.frame(read_csv(file.robdata, col_types = cols()))
      rob <- rob && nrow(robdata) >= 1
    }
    #
    if (rob) {
      if (keep.orig)
        rob.orig <- robdata
      #
      if (nrow(robdata) > 0) {
        robdata %<>%
          rename(studlab = Study)
        #
        vars <- names(robdata)
        j <- 0
        k <- 0
        domains <- vector("character", 0)
        #
        for (i in seq_along(vars)) {
          if (substring(vars[i], 1, 18) == "Domain (judgement)") {
            j <- j + 1
            robdata[paste0("D", j)] <- robdata[vars[i]]
            domains <- c(domains, substring(vars[i], 21))
          }
          #
          if (substring(vars[i], 1, 16) == "Domain (support)") {
            k <- k + 1
            robdata[paste0("D", k, ".details")] <- robdata[vars[i]]
          }
        }
        #
        selvars <- "studlab"
        #
        if (j > 0)
          selvars <- c(selvars, paste0("D", seq_len(j)))
        if (k > 0)
          selvars <- c(selvars, paste0("D", seq_len(j), ".details"))
        #
        robdata %<>% select(all_of(selvars))
        #
        domains <- gsub(": All outcomes", "", domains)
        #
        attr(robdata, "rob") <- rob
        attr(robdata, "domains") <- domains
        attr(robdata, "tool") <- tool
        attr(robdata, "categories") <- categories
        attr(robdata, "col") <- col
        attr(robdata, "symbols") <- symbols
      }
    }
  }
  else
    rob <- FALSE
  
  
  #
  #
  # (6) Extract risk of bias (if available)
  #
  #
  
  res <- list(data = data, rob = robdata)
  #
  if (keep.orig)
    res$orig <-
      list(settings = settings.orig, datarows = datarows.orig,
           subgroup = subgroup.orig, if (rob) rob = rob.orig)
  #
  class(res) <- "cdir"
  res
}





#' @rdname read.cdir
#' @method print cdir
#' @export

print.cdir <- function(x, ...) {
  
  chkclass(x, "cdir")
  #
  tl <- options()$width - 12
  newline <- FALSE
  #
  title <- attr(x$data, "title")
  #
  if (!is.null(title)) {
    if (title != "") {
      newline <- TRUE
      if (nchar(title) <= tl)
        cat(paste0("Review:     ", title, "\n"))
      else
        cat(paste0("Review:     ", substring(title, 1, tl - 4), " ...\n"))
    }
  }
  #
  if (!is.null(x$data$complab)) {
    cat("Available comparisons:\n")
    #
    cat(paste0(unique(x$data$complab), "\n", collapse = ""))
  }
  #
  invisible(NULL)
}

#' Risk of bias assessment
#' 
#' @description
#' Create table with risk of bias assessment or add table to existing
#' meta-analysis
#' 
#' @param item1 Risk of bias item 1 or a meta-analysis object of class
#'   \code{meta} with information on risk of bias assessment.
#' @param item2 Risk of bias item 2.
#' @param item3 Risk of bias item 3.
#' @param item4 Risk of bias item 4.
#' @param item5 Risk of bias item 5.
#' @param item6 Risk of bias item 6.
#' @param item7 Risk of bias item 7.
#' @param item8 Risk of bias item 8.
#' @param item9 Risk of bias item 9.
#' @param item10 Risk of bias item 10.
#' @param overall Overall risk of bias assess.
#' @param weight Weight for each study.
#' @param studlab Study labels.
#' @param data A data frame or a meta-analysis object of class
#'   \code{meta}.
#' @param tool Risk of bias (RoB) tool.
#' @param domains A character vector with names of RoB domains.
#' @param categories Possible RoB categories.
#' @param cat1 Possible categories for RoB item 1.
#' @param cat2 Possible categories for RoB item 2.
#' @param cat3 Possible categories for RoB item 3.
#' @param cat4 Possible categories for RoB item 4.
#' @param cat5 Possible categories for RoB item 5.
#' @param cat6 Possible categories for RoB item 6.
#' @param cat7 Possible categories for RoB item 7.
#' @param cat8 Possible categories for RoB item 8.
#' @param cat9 Possible categories for RoB item 9.
#' @param cat10 Possible categories for RoB item 10.
#' @param cat.overall Possible categories for overall RoB.
#' @param col Colours for RoB categories.
#' @param col1 Colours for categories for RoB item 1.
#' @param col2 Colours for categories for RoB item 2.
#' @param col3 Colours for categories for RoB item 3.
#' @param col4 Colours for categories for RoB item 4.
#' @param col5 Colours for categories for RoB item 5.
#' @param col6 Colours for categories for RoB item 6.
#' @param col7 Colours for categories for RoB item 7.
#' @param col8 Colours for categories for RoB item 8.
#' @param col9 Colours for categories for RoB item 9.
#' @param col10 Colours for categories for RoB item 10.
#' @param col.overall Colours for categories for overall RoB.
#' @param symbols Corresponding symbols for RoB categories.
#' @param symb1 Corresponding symbols for RoB item 1.
#' @param symb2 Corresponding symbols for RoB item 2.
#' @param symb3 Corresponding symbols for RoB item 3.
#' @param symb4 Corresponding symbols for RoB item 4.
#' @param symb5 Corresponding symbols for RoB item 5.
#' @param symb6 Corresponding symbols for RoB item 6.
#' @param symb7 Corresponding symbols for RoB item 7.
#' @param symb8 Corresponding symbols for RoB item 8.
#' @param symb9 Corresponding symbols for RoB item 9.
#' @param symb10 Corresponding symbols for RoB item 10.
#' @param symb.overall Corresponding symbols for overall RoB.
#' @param legend A logical specifying whether legend with RoB domains
#'   should be printed.
#' @param overwrite A logical indicating whether an existing risk of
#'   bias table in a meta-analysis object should be overwritten.
#' @param warn A logical indicating whether warnings should be
#'   printed.
#' @param x An object of class \code{rob}.
#' @param details A logical indicating whether to print details on
#'   categories and colours.
#' @param \dots Additional printing arguments.
#' 
#' @details
#' This function can be used to define a risk of bias (RoB) assessment
#' for a meta-analysis which can be shown in a forest plot
#' (\code{\link{forest.meta}}), summary weighted barplot
#' (\code{\link{barplot.rob}}) or traffic light plot
#' (\code{\link{traffic_light}}). It is also possible to extract the
#' risk of bias assessment from a meta-analysis with RoB information.
#'
#' The risk of bias table contains
#'
#' \itemize{
#' \item study labels;
#' \item variables for individual RoB domains (with variable names A,
#'   B, \dots);
#' \item an overall RoB assessment if argument \code{overall} is
#'   provided;
#' \item weights for individual studies used in summary weighted
#'   barplots.
#' }
#' Note, an overall RoB assessment is mandatory to create a summary
#' weighted barplot or a traffic light plot.
#'
#' The RoB table is directly returned if argument \code{data} is a
#' data frame or argument \code{item1} is a meta-analysis with risk of
#' bias assessment. The RoB table is added as a new list element 'rob'
#' to a meta-analysis object if argument \code{data} is a
#' meta-analysis.
#'
#' The user must either specify the categories and (optionally)
#' domains of the RoB tool (using the eponymous arguments) or one of
#' the following RoB tools.
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Risk of bias tool} \cr
#' \code{tool = "RoB1"} \tab RoB 1 tool for randomized studies
#'   (Higgins et al., 2011) \cr
#' \code{tool = "RoB2"} \tab RoB 2 tool for randomized studies
#'   (Sterne et al., 2019) \cr
#' \code{tool = "RoB2-cluster"} \tab RoB 2 tool for
#'   cluster-randomized trials \cr
#' \code{tool = "RoB2-crossover"} \tab RoB 2 tool for crossover
#'   trials \cr
#' \code{tool = "ROBINS-I"} \tab Risk Of Bias In Non-randomized
#'   Studies - of Interventions \cr
#' \tab (Sterne et al., 2016) \cr
#' \code{tool = "ROBINS-E"} \tab Risk Of Bias In Non-randomized
#'   Studies - of Exposures \cr
#' \tab (ROBINS-E Development Group, 2023)
#' }
#' These RoB tools are described on the website
#' \url{https://www.riskofbias.info/}.
#'
#' \subsection{Risk of bias domains}{
#'
#' By default, i.e., if argument \code{domains} is not provided by the
#' user, the following names are used for RoB domains.
#'
#' \itemize{
#' \item RoB 1 tool for randomized studies (RoB1):
#'
#' \enumerate{
#'  \item Random sequence generation (selection bias)
#'  \item Allocation concealment (selection bias)
#'  \item Blinding of participants and personnel (performance bias)
#'  \item Blinding of outcome assessment (detection bias)
#'  \item Incomplete outcome data (attrition bias)
#'  \item Selective reporting (reporting bias)
#'  \item Other bias
#' }
#'
#' \item RoB 2 tool for randomized studies (RoB2):
#' \enumerate{
#'  \item Bias arising from the randomization process
#'  \item Bias due to deviations from intended intervention"
#'  \item Bias due to missing outcome data
#'  \item Bias in measurement of the outcome
#'  \item Bias in selection of the reported result
#' }
#'
#' \item RoB 2 tool for cluster-randomized trials (RoB2-cluster):
#' \enumerate{
#'  \item Bias arising from the randomization process
#'  \item Bias arising from the identification or recruitment of
#'    participants into clusters
#'  \item Bias due to deviations from intended intervention
#'  \item Bias due to missing outcome data
#'  \item Bias in measurement of the outcome
#'  \item Bias in selection of the reported result
#' }
#'
#' \item RoB 2 tool for crossover trials (RoB2-crossover)
#' \enumerate{
#'  \item Bias arising from the randomization process
#'  \item Bias arising from period and carryover effects
#'  \item Bias due to deviations from intended intervention
#'  \item Bias due to missing outcome data
#'  \item Bias in measurement of the outcome
#'  \item Bias in selection of the reported result
#' }
#' 
#' \item Risk Of Bias In Non-randomized Studies - of Intervention (ROBINS-I):
#' \enumerate{
#'  \item Risk of bias due to confounding
#'  \item Risk of bias in selection of participants into the study
#'  \item Risk of bias in classification of interventions
#'  \item Risk of bias due to deviations from intented interventions
#'  \item Risk of bias due to missing outcome data
#'  \item Risk of bias in measurement of the outcome
#'  \item Risk of bias in the selection of the reported results
#' }
#' 
#' \item Risk Of Bias In Non-randomized Studies - of Exposures (ROBINS-E):
#' \enumerate{
#'  \item Risk of bias due to confounding
#'  \item Risk of bias arising from measurement of the exposure into
#'    the study (or into the analysis)
#'  \item Risk of bias due to post-exposure interventions
#'  \item Risk of bias due to deviations from intented interventions
#'  \item Risk of bias due to missing outcome data
#'  \item Risk of bias in measurement of the outcome
#'  \item Risk of bias in the selection of the reported results
#' }
#' 
#' \item User-defined RoB assessment:
#' \enumerate{
#'  \item First item
#'  \item Second item
#'  \item \dots
#' }
#' }
#'
#' It is possible to define additional bias domains for the available
#' RoB tools. In this case, only the names for new RoB domains have to
#' be provided in argument \code{domains}. If argument \code{domains}
#' is not used to specify new domains, the names "Additional item 1"
#' etc. will be used. It is also possible to modify the pre-defined
#' domain names using argument \code{domains}.
#'
#' The maximum number of bias domains / items is ten (see arguments
#' \code{item1}, ..., \code{item10}).
#' }
#'
#' \subsection{Risk of bias categories, colours and symbols}{
#' 
#' By default, the following settings are used.
#'
#' RoB 1 tool:
#'
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Values} \cr
#' \code{categories} \tab "Low risk of bias", "Unclear risk of bias",
#'   "High risk of bias" \cr
#' \code{col} \tab "green", "yellow", "red" \cr
#' \code{symbols} \tab "+", "?", "-"
#' }
#'
#' RoB 2 tools:
#'
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Values} \cr
#' \code{categories} \tab "Low risk of bias", "Some concerns",
#'   "High risk of bias" \cr
#' \code{col} \tab "green", "yellow", "red" \cr
#' \code{symbols} \tab "+", "?", "-"
#' }
#'
#' ROBINS tools:
#'
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Values} \cr
#' \code{categories} \tab "Low risk", "Some concerns", "High risk",
#'   "Very high risk", "NI" \cr
#' \code{col} \tab "green", "yellow", "red", "darkred", "darkgrey" \cr
#' \code{symbols} \tab none
#' }
#'
#' User-defined RoB tools:
#'
#' \tabular{ll}{
#' \bold{Argument} \tab \bold{Values} \cr
#' \code{categories} \tab Must be specified by the user \cr
#' \code{col} \tab 1, 2, ... \cr
#' \code{symbols} \tab none
#' }
#'
#' If colours (\code{col}) and symbols (\code{symbols}) are provided,
#' they must be of the same length as the number of categories. 
#' }
#' 
#' @return
#' A data frame with study labels and risk of bias items and
#' additional class "rob".
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.meta}}, \code{\link{barplot.rob}},
#'   \code{\link{traffic_light}}
#'
#' @references
#' Higgins JPT, Altman DG, Gøtzsche PC, Jüni P, Moher D, Oxman AD et
#' al. (2011):
#' The Cochrane Collaboration's tool for assessing risk of bias in
#' randomised trials.
#' \emph{British Medical Journal}, \bold{343}: d5928
#'
#' ROBINS-E Development Group (Higgins J, Morgan R, Rooney A et al.) (2023):
#' Risk Of Bias In Non-randomized Studies - of Exposure (ROBINS-E)
#' Available from: \url{https://www.riskofbias.info/welcome/robins-e-tool}.
#' 
#' Sterne JA, Hernán MA, Reeves BC, Savović J, Berkman ND, Viswanathan
#' M, et al. (2016):
#' ROBINS-I: a tool for assessing risk of bias in non-randomised
#' studies of interventions.
#' \emph{British Medical Journal}, \bold{355}: i4919
#' 
#' Sterne JAC, Savović J, Page MJ, Elbers RG, Blencowe NS, Boutron I,
#' et al. (2019):
#' RoB 2: a revised tool for assessing risk of bias in randomised trials.
#' \emph{British Medical Journal}, \bold{366}: l4898. 
#' 
#' @examples
#' # Use RevMan 5 settings
#' oldset <- settings.meta("RevMan5", quietly = FALSE)
#' 
#' data(caffeine)
#' 
#' ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
#'   data = caffeine, studlab = paste(study, year))
#'
#' # Add risk of bias assessment to meta-analysis
#' ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")
#' 
#' # Print risk of bias assessment
#' rob(ma2)
#' 
#' # Forest plot with risk of bias assessment
#' forest(ma2)
#'
#' # Use previous settings
#' settings.meta(oldset)
#' 
#' @export rob

rob <- function(item1,
                item2 = NULL,
                item3 = NULL,
                item4 = NULL,
                item5 = NULL,
                item6 = NULL,
                item7 = NULL,
                item8 = NULL,
                item9 = NULL,
                item10 = NULL,
                studlab = NULL,
                overall = NULL,
                weight = NULL,
                data = NULL,
                #
                tool = gs("tool.rob"),
                domains = NULL,
                #
                categories = NULL,
                cat1 = categories,
                cat2 = categories,
                cat3 = categories,
                cat4 = categories,
                cat5 = categories,
                cat6 = categories,
                cat7 = categories,
                cat8 = categories,
                cat9 = categories,
                cat10 = categories,
                cat.overall = categories,
                #
                col = NULL,
                col1 = col,
                col2 = col,
                col3 = col,
                col4 = col,
                col5 = col,
                col6 = col,
                col7 = col,
                col8 = col,
                col9 = col,
                col10 = col,
                col.overall = col,
                #
                symbols = NULL,
                symb1 = symbols,
                symb2 = symbols,
                symb3 = symbols,
                symb4 = symbols,
                symb5 = symbols,
                symb6 = symbols,
                symb7 = symbols,
                symb8 = symbols,
                symb9 = symbols,
                symb10 = symbols,
                symb.overall = symbols,
                #
                legend = TRUE,
                #
                overwrite = FALSE,
                warn = TRUE) {
  
  #
  #
  # (1) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  meta.object <- NULL
  #
  is.meta <- inherits(data, "meta")
  #
  if (nulldata)
    data <- sfsp
  else if (is.meta) {
    meta.object <- data
    #
    data <- data$data
    if (!is.null(meta.object$subset))
      data <- data[meta.object$subset, ]
    #
    if (is.null(studlab))
      studlab <- data$.studlab    
  }
  #
  # Catch 'item1', etc.
  #
  item1 <- catch("item1", mc, data, sfsp)
  if (!is.null(item1) && inherits(item1, "meta")) {
    if (is.null(item1$rob))
      return(NULL)
    else
      return(item1$rob)
  }
  #
  item2 <- catch("item2", mc, data, sfsp)
  item3 <- catch("item3", mc, data, sfsp)
  item4 <- catch("item4", mc, data, sfsp)
  item5 <- catch("item5", mc, data, sfsp)
  item6 <- catch("item6", mc, data, sfsp)
  item7 <- catch("item7", mc, data, sfsp)
  item8 <- catch("item8", mc, data, sfsp)
  item9 <- catch("item9", mc, data, sfsp)
  item10 <- catch("item10", mc, data, sfsp)
  #
  missing.overall <- missing(overall)
  overall <- catch("overall", mc, data, sfsp)
  avail.overall <- !missing.overall & !is.null(overall)
  #
  missing.weight <- missing(weight)
  weight <- catch("weight", mc, data, sfsp)
  avail.weight <- !missing.weight & !is.null(weight)
  #
  studlab <- catch("studlab", mc, data, sfsp)
  
  
  #
  #
  # (2) Check data
  #
  #
  
  fun <- "rob"
  #
  avail1 <- !is.null(item1)
  avail2 <- !is.null(item2)
  avail3 <- !is.null(item3)
  avail4 <- !is.null(item4)
  avail5 <- !is.null(item5)
  avail6 <- !is.null(item6)
  avail7 <- !is.null(item7)
  avail8 <- !is.null(item8)
  avail9 <- !is.null(item9)
  avail10 <- !is.null(item10)
  #
  if (avail1)
    k.All <- length(item1)
  else if (avail2)
    k.All <- length(item2)
  else if (avail3)
    k.All <- length(item3)
  else if (avail4)
    k.All <- length(item4)
  else if (avail5)
    k.All <- length(item5)
  else if (avail6)
    k.All <- length(item6)
  else if (avail7)
    k.All <- length(item7)
  else if (avail8)
    k.All <- length(item8)
  else if (avail9)
    k.All <- length(item9)
  else if (avail10)
    k.All <- length(item10)
  else
    stop("No information on risk of bias domains provided.", call. = FALSE)
  #
  if (avail2)
    chklength(item2, k.All, fun)
  #
  if (avail3)
    chklength(item3, k.All, fun)
  #
  if (avail4)
    chklength(item4, k.All, fun)
  #
  if (avail5)
    chklength(item5, k.All, fun)
  #
  if (avail6)
    chklength(item6, k.All, fun)
  #
  if (avail7)
    chklength(item7, k.All, fun)
  #
  if (avail8)
    chklength(item8, k.All, fun)
  #
  if (avail9)
    chklength(item9, k.All, fun)
  #
  if (avail10)
    chklength(item10, k.All, fun)
  #
  if (avail.overall)
    chklength(overall, k.All, fun)
  #
  if (avail.weight & length(weight) > 1)
    chklength(weight, k.All, fun)
  #
  if (is.null(studlab)) {
    if (is.meta)
      studlab <- data$.studlab
    else
      studlab <- seq_len(k.All)
  }
  else
    chklength(studlab, k.All, fun)
  #
  chklogical(legend)
  chklogical(overwrite)
  chklogical(warn)
  #
  if (!is.null(tool))
    tool <- setchar(tool, gs("tool4rob"))
  else
    tool <- "user-defined"
  #
  
  
  #
  #
  # (3) Risk of bias categories
  #
  #
  cat1 <- definecat(avail1, cat1, item1, tool, warn)
  cat2 <- definecat(avail2, cat2, item2, tool, warn)
  cat3 <- definecat(avail3, cat3, item3, tool, warn)
  cat4 <- definecat(avail4, cat4, item4, tool, warn)
  cat5 <- definecat(avail5, cat5, item5, tool, warn)
  cat6 <- definecat(avail6, cat6, item6, tool, warn)
  cat7 <- definecat(avail7, cat7, item7, tool, warn)
  cat8 <- definecat(avail8, cat8, item8, tool, warn)
  cat9 <- definecat(avail9, cat9, item9, tool, warn)
  cat10 <- definecat(avail10, cat10, item10, tool, warn)
  #
  cat.overall <- definecat(avail.overall, cat.overall, overall, tool, warn)
  
  
  #
  #
  # (4) Create risk of bias table
  #
  #
  
  rob <- data.frame(Study = studlab)
  #
  if (avail1)
    rob$A <- setcat(item1, cat1)
  #
  if (avail2)
    rob$B <- setcat(item2, cat2)
  #
  if (avail3)
    rob$C <- setcat(item3, cat3)
  #
  if (avail4)
    rob$D <- setcat(item4, cat4)
  #
  if (avail5)
    rob$E <- setcat(item5, cat5)
  #
  if (avail6)
    rob$F <- setcat(item6, cat6)
  #
  if (avail7)
    rob$G <- setcat(item7, cat7)
  #
  if (avail8)
    rob$H <- setcat(item8, cat8)
  #
  if (avail9)
    rob$I <- setcat(item9, cat9)
  #
  if (avail10)
    rob$J <- setcat(item10, cat10)
  #
  domain.available <- c(avail1, avail2, avail3, avail4, avail5, avail6, avail7,
                        avail8, avail9, avail10)
  names(domain.available) <- paste0("D", 1:10)
  #
  if (avail.overall)
    rob$Overall <- setcat(overall, cat.overall)
  #
  if (avail.weight && length(weight) == 1 && is.character(weight)) {
    if (is.meta)
      weight <- setchar(weight, c("common", "random"))
    else
      weight <- NULL
  }
  #
  if (!avail.weight && is.meta && meta.object$overall)
    weight <-
      if (meta.object$random)
        "random"
      else if (meta.object$common)
        "common"
      else
        NULL
  #
  if (!is.null(weight)) {
    if (length(weight) == 1 && is.character(weight)) {
      sel.w <-
        if (is.null(meta.object$subset))
          seq_along(meta.object$TE)
        else
          meta.object$subset
      #
      if (weight == "random")
        rob$Weight <- meta.object$w.random[sel.w]
      else if (weight == "common")
        rob$Weight <- meta.object$w.common[sel.w]
    }
    else
      rob$Weight <- weight
    #
    avail.weight <- TRUE
  }
  else
    avail.weight <- FALSE
  
  
  #
  #
  # (4) Names for risk of bias domains
  #
  #
  
  if (tool == "user-defined")
    dm <- paste("Item", 1:10)
  else if (tool == "RoB1")
    dm <-
      c("Random sequence generation (selection bias)",
        "Allocation concealment (selection bias)",
        "Blinding of participants and personnel (performance bias)",
        "Blinding of outcome assessment (detection bias)",
        "Incomplete outcome data (attrition bias)",
        "Selective reporting (reporting bias)",
        "Other bias",
        rep("", 10 - 7))
  else if (tool == "RoB2")
    dm <-
      c("Bias arising from the randomization process",
        "Bias due to deviations from intended intervention",
        "Bias due to missing outcome data",
        "Bias in measurement of the outcome",
        "Bias in selection of the reported result",
        rep("", 10 - 5))
  else if (tool == "RoB2-cluster")
    dm <-
      c("Bias arising from the randomization process",
        paste("Bias arising from the identification or",
              "recruitment of participants into clusters"),
        "Bias due to deviations from intended intervention",
        "Bias due to missing outcome data",
        "Bias in measurement of the outcome",
        "Bias in selection of the reported result",
        rep("", 10 - 6))
  else if (tool == "RoB2-crossover")
    dm <-
      c("Bias arising from the randomization process",
        "Bias arising from period and carryover effects",
        "Bias due to deviations from intended intervention",
        "Bias due to missing outcome data",
        "Bias in measurement of the outcome",
        "Bias in selection of the reported result",
        rep("", 10 - 6))
  else if (tool == "ROBINS-I")
    dm <-
      c("Risk of bias due to confounding",
        "Risk of bias in selection of participants into the study",
        "Risk of bias in classification of interventions",
        "Risk of bias due to deviations from intented interventions",
        "Risk of bias due to missing outcome data",
        "Risk of bias in measurement of the outcome",
        "Risk of bias in the selection of the reported results",
        rep("", 10 - 7))
  else if (tool == "ROBINS-E")
    dm <-
      c("Risk of bias due to confounding",
        "Risk of bias arising from measurement of the exposure",
        paste("Risk of bias in selection of participants",
              "into the study (or into the analysis)"),
        "Risk of bias due to post-exposure interventions",
        "Risk of bias due to missing data",
        "Risk of bias in measurement of the outcome",
        "Risk of bias in selection of the reported results",
        rep("", 10 - 7))
  else
    dm <- rep("", 10)
  #
  if (tool %in% c("RoB1", "ROBINS-I", "ROBINS-E")) {
    if (avail8)
      dm[8] <- "Additional item 1"
    #
    if (avail9)
      dm[9] <- "Additional item 2"
    #
    if (avail10)
      dm[10] <- "Additional item 3"
  }
  else if (tool %in% c("RoB2-cluster", "RoB2-crossover")) {
    if (avail7)
      dm[7] <- "Additional item 1"
    #
    if (avail8)
      dm[8] <- "Additional item 2"
    #
    if (avail9)
      dm[9] <- "Additional item 3"
    #
    if (avail10)
      dm[10] <- "Additional item 4"
  }
  else if (tool == "RoB2") {
    if (avail6)
      dm[6] <- "Additional item 1"
    #
    if (avail7)
      dm[7] <- "Additional item 2"
    #
    if (avail8)
      dm[8] <- "Additional item 3"
    #
    if (avail9)
      dm[9] <- "Additional item 4"
    #
    if (avail10)
      dm[10] <- "Additional item 5"
  }
  #
  if (is.null(domains))
    domains <- dm[domain.available]
  #
  else if (tool == "user-defined") {
    if (length(domains) != sum(domain.available))
      stop("Number of domain names does not match number of domains.",
           call. = FALSE)
  }
  else
    domains <- setdom(dm, tool, domains, domain.available)
  #
  if (avail.overall)
    domains <- c(domains, "Overall risk of bias")
  
  
  #
  #
  # (6) Risk of bias symbols
  #
  #
  
  symb1 <- definesymb(avail1, symb1, cat1, tool, warn)
  symb2 <- definesymb(avail2, symb2, cat2, tool, warn)
  symb3 <- definesymb(avail3, symb3, cat3, tool, warn)
  symb4 <- definesymb(avail4, symb4, cat4, tool, warn)
  symb5 <- definesymb(avail5, symb5, cat5, tool, warn)
  symb6 <- definesymb(avail6, symb6, cat6, tool, warn)
  symb7 <- definesymb(avail7, symb7, cat7, tool, warn)
  symb8 <- definesymb(avail8, symb8, cat8, tool, warn)
  symb9 <- definesymb(avail9, symb9, cat9, tool, warn)
  symb10 <- definesymb(avail10, symb10, cat10, tool, warn)
  #
  symb.overall <-
    definesymb(avail.overall, symb.overall, cat.overall, tool, warn)
  
  
  #
  #
  # (7) Risk of bias colours
  #
  #
  
  col1 <- definecol(avail1, col1, cat1, tool, warn)
  col2 <- definecol(avail2, col2, cat2, tool, warn)
  col3 <- definecol(avail3, col3, cat3, tool, warn)
  col4 <- definecol(avail4, col4, cat4, tool, warn)
  col5 <- definecol(avail5, col5, cat5, tool, warn)
  col6 <- definecol(avail6, col6, cat6, tool, warn)
  col7 <- definecol(avail7, col7, cat7, tool, warn)
  col8 <- definecol(avail8, col8, cat8, tool, warn)
  col9 <- definecol(avail9, col9, cat9, tool, warn)
  col10 <- definecol(avail10, col10, cat10, tool, warn)
  #
  col.overall <- definecol(avail.overall, col.overall, cat.overall, tool, warn)
  
  
  #
  #
  # (8) Return risk of bias table
  #
  #
  
  attr(rob, "tool") <- tool
  attr(rob, "domains") <- domains
  #
  attr(rob, "overall") <- avail.overall
  attr(rob, "weight") <- avail.weight
  #
  attr(rob, "categories") <-
    compact(list(cat1 = cat1, cat2 = cat2, cat3 = cat3, cat4 = cat4,
                 cat5 = cat5, cat6 = cat6, cat7 = cat7, cat8 = cat8,
                 cat9 = cat9, cat10 = cat10, cat.overall = cat.overall))
  attr(rob, "symbols") <-
    compact(list(symb1 = symb1, symb2 = symb2, symb3 = symb3, symb4 = symb4,
                 symb5 = symb5, symb6 = symb6, symb7 = symb7, symb8 = symb8,
                 symb9 = symb9, symb10 = symb10, symb.overall = symb.overall))
  attr(rob, "col") <-
    compact(list(col1 = col1, col2 = col2, col3 = col3, col4 = col4,
                 col5 = col5, col6 = col6, col7 = col7, col8 = col8,
                 col9 = col9, col10 = col10, col.overall = col.overall))
  #
  attr(rob, "legend") <- legend
  #
  class(rob) <- c("rob", class(rob))
  #
  if (is.null(meta.object))
    res <- rob
  else {
    res <- meta.object
    if (!is.null(res$rob) & !overwrite)
      stop("Risk of bias available ",
           "(use argument 'overwrite = TRUE' to replace RoB table",
           call. = FALSE)
    res$rob <- rob
  }
  
  
  res
}





#' @rdname rob
#' @method print rob
#' @export

print.rob <- function(x, legend = attr(x, "legend"), details = TRUE, ...) {

  chkclass(x, "rob")
  #
  chklogical(legend)
  chklogical(details)
  
  x.prt <- x
  class(x.prt) <- "data.frame"
  
  print(x.prt, ...)
  
  if (legend) {
    cat(
      paste0("\nRisk of bias domains:",
             paste0("\n", catleg(x), collapse = ""),
             "\n")
    )
    #
    if (details) {
      cat(
        paste0("\nCategories:",
               paste0("\n", catcat(x), collapse = ""),
               "\n")
      )
      #
      cat(
        paste0("\nColours:",
               paste0("\n", catcol(x), collapse = ""),
               "\n")
      )
      #
      symbols <- unique(unlist(attr(x, "symbols")))
      if (!(length(symbols) == 1 && is.logical(symbols) && !symbols))
      cat(
        paste0("\nSymbols:",
               paste0("\n", catsymb(x), collapse = ""),
               "\n")
      )        
    }
  }
  
  
  invisible(NULL)
}

# Auxiliary functions to set arguments
#
# Package: meta
# Author: Guido Schwarzer <guido.schwarzer@@uniklinik-freiburg.de>
# License: GPL (>= 2)
#

setchar <- function(x, val, text, list = FALSE, name = NULL,
                    stop.at.error = TRUE, addtext = "",
                    return.NULL = TRUE, nchar.equal = FALSE,
                    setNA = FALSE, pre = "",
                    unique = FALSE) {
  val <- unique(val)
  #
  if (is.null(name))
    name <- deparse(substitute(x))
  nval <- length(val)
  #
  if (is.numeric(x)) {
    numeric.x <- TRUE
    idx <- x
    idx[idx < 1] <- NA
    idx[idx >= nval + 1] <- NA
  }
  else {
    numeric.x <- FALSE
    #
    if (length(unique(tolower(x))) != length(unique(x)) |
        length(unique(tolower(val))) != length(unique(val)))
      idx <- charmatch(x, val, nomatch = NA)
    else
      idx <- charmatch(tolower(x), tolower(val), nomatch = NA)
  }
  #
  if ((anyNA(idx) || any(idx == 0)) && !setNA) {
    if (list)
      first <- "List element '"
    else
      first <- "Argument '"
    #
    if (missing(text)) {
      if (numeric.x) {
        if (nval == 1)
          vlist <- "1"
        else if (nval == 2)
          vlist <- "1 or 2"
        else
          vlist <- paste("between 1 and", nval)
      }
      else {
        if (nval == 1)
          vlist <- paste0('"', val, '"')
        else if (nval == 2)
          vlist <- paste0('"', val, '"', collapse = " or ")
        else
          vlist <- paste0(paste0('"', val[-nval], '"', collapse = ", "),
                          ', or ', '"', val[nval], '"')
      }
      #
      if (stop.at.error)
        stop(first, name, "' must be ", pre,
             vlist, addtext, ".", call. = FALSE)
      else {
        if (return.NULL)
          return(NULL)
        else
          return(x)
      }
    }
    else {
      if (stop.at.error)
        stop(first, name, "' ", text, ".", call. = FALSE)
      else {
        if (return.NULL)
          return(NULL)
        else
          return(x)
      }
    }
  }
  #
  if (is.null(x))
    return(NULL)
  else
    res <- val[idx]
  #
  if (nchar.equal && nchar(res) != nchar(x))
    res <- x
  #
  if (unique)
    res <- unique(res)
  #
  res
}

setstudlab <- function(x, k) {
  #
  # Set study labels
  #
  if (is.null(x)) {
    if (k == 1)
      x <- ""
    else
      x <- seq(along = rep(1, k))
  }
  #
  if (is.factor(x))
    x <- as.character(x)
  #
  x
}

setlength <- function(x, len, text) {
  if (length(x) == 1)
    x <- rep(x, len)
  else
    chklength(x, len,
              text =
                paste0("Length of argument '", deparse(substitute(x)),
                       "' must be equal to 1 or ", text, "."))
  #
  x
}

setunit <- function(x) {
  xname <- deparse(substitute(x))
  
  if (is.character(x)) {
    if (length(x) != 1)
      stop("Argument '", xname, "' must be a character string.")
    #
    plotunit <- ""
    if (length(grep("cm", tolower(x))) == 1)
      plotunit <- "cm"
    else if (length(grep("inch", tolower(x))) == 1)
      plotunit <- "inch"
    else if (length(grep("mm", tolower(x))) == 1)
      plotunit <- "mm"
    #
    if (plotunit == "")
      stop("Argument '", xname, "' must contain 'cm', 'inch', or 'mm'.")
    #
    if (plotunit == "cm") {
      plotval <- substring(x, 1, nchar(x) - 2)
      if (length(plotval) == 0 | is.na(as.numeric(plotval)))
        stop("Argument '", xname, "' must contain at least one number.")
      #
      res <- grid::unit(as.numeric(plotval), "cm")
    }
    else if (plotunit == "inch") {
      plotval <- substring(x, 1, nchar(x) - 4)
      if (length(plotval) == 0 | is.na(as.numeric(plotval)))
        stop("Argument '", xname, "' must contain at least one number.")
      #
      res <- grid::unit(as.numeric(plotval), "inch")
    }
    else if (plotunit == "mm") {
      plotval <- substring(x, 1, nchar(x) - 2)
      if (length(plotval) == 0 | is.na(as.numeric(plotval)))
        stop("Argument '", xname, "' must contain at least one number.")
      #
      res <- grid::unit(as.numeric(plotval), "mm")
    }
  }
  else
    res <- x
  
  res
}

setmethodbias <- function(x, subset) {
  oldmethod <- setchar(x, gs("meth4bias.old"),
                       stop.at.error = FALSE)
  #
  if (is.null(oldmethod))
    if (missing(subset))
      res <- setchar(x, gs("meth4bias"), name = "method.bias")
    else
      res <- setchar(x, gs("meth4bias")[subset], name = "method.bias")
  else
    res <- switch(oldmethod,
                  rank = "Begg",
                  linreg = "Egger",
                  mm = "Thompson",
                  count = "Schwarzer",
                  score = "Harbord")
  #
  res
}

set_method_tau <- function(method.tau, missing.tau,
                           method.predict, missing.predict,
                           warn = TRUE) {
  if (method.tau != "REML" & any(method.predict == "KR")) {
    if (missing.tau & !missing.predict) {
      if (warn)
        warning("Argument 'method.tau' set to \"REML\" as ",
                "'method.predict' = \"KR\".",
                call. = FALSE)
      #
      method.tau <- "REML"
    }
  }
  #
  method.tau
}

set_method_predict <- function(method.predict, missing.predict,
                               method.tau, missing.tau,
                               warn = TRUE) {
  any_KR <- any(method.predict %in% "KR")
  any_KR_PR <- any(method.predict %in% "KR-PR")
  #
  if (method.tau != "REML" & (any_KR | any_KR_PR)) {
    if (!missing.tau & missing.predict) {
      method.predict[method.predict == "KR"] <- "V"
      method.predict[method.predict == "KR-PR"] <- "V"
    }
    else if (!missing.tau & !missing.predict) {
      if (warn)
        warning("Argument 'method.predict' set to \"V\" instead of ",
                if (any_KR) "\"KR\"", if (any_KR & any_KR_PR) " / ",
                if (any_KR_PR) "\"KR-PR\"",
                " as 'method.tau' != \"REML\".",
                call. = FALSE)
      method.predict[method.predict == "KR"] <- "V"
      method.predict[method.predict == "KR-PR"] <- "V"
    }
    else {
      method.predict[method.predict == "KR"] <- "V"
      method.predict[method.predict == "KR-PR"] <- "V"
    }
  }
  #
  method.predict
}

# Function only used with MLM or GLMM
#
set_df_predict <- function(method.predict, k)
  ifelse(method.predict == "V" & k >= 2, k - 1,
         ifelse(method.predict == "HTS" & k >= 3, k - 2,
                ifelse(method.predict == "S", Inf, NA)))

setVal <- function(data, varname, default = NULL) {
  if (isCol(data, varname))
    return(data[[varname]])
  else
    return(default)
}

setsort <- function(sort, n, text) {
  if (is.null(sort))
    res <- seq_len(n)
  else {
    chklength(sort, n,
              text = paste0("Argument '", deparse(substitute(sort)),
                           "' must be of same length as ",
                           "number of ", text, "."))
    #
    res <- sort
    if (!(is.numeric(res) & min(res) == 1 & max(res) == n))
      res <- order(res)
  }
  #
  res
}
setsep <- function(x, sep, type = "treatment",
                   argname = deparse(substitute(sep)),
                   missing.sep) {
  labels <- sort(unique(x))
  #
  if (compmatch(labels, sep)) {
    if (!missing.sep)
      warning("Separator '", sep,
              "' used in at least one ",
              type, " label. ",
              "Trying to use predefined separators: ",
              "':', '-', '_', '/', '+', '.', '|', '*'.",
              call. = FALSE)
    #
    if (!compmatch(labels, ":"))
      sep <- ":"
    else if (!compmatch(labels, "-"))
      sep <- "-"
    else if (!compmatch(labels, "_"))
      sep <- "_"
    else if (!compmatch(labels, "/"))
      sep <- "/"
    else if (!compmatch(labels, "+"))
      sep <- "+"
    else if (!compmatch(labels, "."))
      sep <- "-"
    else if (!compmatch(labels, "|"))
      sep <- "|"
    else if (!compmatch(labels, "*"))
      sep <- "*"
    else
      stop("All predefined separators (':', '-', '_', '/', '+', ",
           "'.', '|', '*') are used in at least one ",
           type, " label. ",
           "Please specify a different character that should be ",
           "used as separator (argument '", argname, "').",
           call. = FALSE)
  }
  #
  sep
}

.onAttach <- function(libname, pkgname) {
  msg <-
    paste0("Loading 'meta' package (version ",
           utils::packageDescription("meta")$Version,
           ").",
           "\nType 'help(meta)' for a brief overview.")
  packageStartupMessage(msg)
}


updateversion <- function(x) {
  #
  # Update older meta objects, see gs("major.update") and gs("minor.update")
  #
  if (update_needed(x$version))
    x <- update(x, warn = FALSE, warn.deprecated = FALSE)
  #
  x
}


update_needed <- function(version,
                          major = gs("major.update"),
                          minor = gs("minor.update"),
                          verbose = FALSE) {
  if (is.null(version)) {
    version <- 0.1
    major.cur <- 0
    minor.cur <- 1
  }
  else {
    version <- unlist(strsplit(version, "-"))
    divide <- unlist(strsplit(version[1], ".", fixed = TRUE))
    major.cur <- as.numeric(divide[1])
    minor.cur <- as.numeric(paste(divide[2], version[2], sep = "."))
  }
  #
  res <-
    ifelse(major.cur < major,
           TRUE, ifelse(major.cur > major,
                        FALSE, minor.cur < as.numeric(gsub("-", ".", minor))))
  if (res & verbose)
    message(paste0("Update to meta, version ", major, ".", minor))
  #
  res
}


taudat <- function(method.tau, detail.tau,
                   tau, lower.tau, upper.tau, print.tau.ci, digits.tau,
                   tau2, lower.tau2, upper.tau2, print.tau2.ci, digits.tau2,
                   sign.lower, sign.upper) {
  dat <- data.frame(method.tau, names = detail.tau, tau, tau2)
  #
  # In order to use duplicated()
  #
  if (!is.null(dat$tau))
    dat$tau <- round(dat$tau, 14)
  if (!is.null(dat$tau2))
    dat$tau2 <- round(dat$tau2, 14)
  #
  if (!is.null(lower.tau))
    dat$lower.tau <- round(lower.tau, 14)
  if (!is.null(lower.tau2))
    dat$lower.tau2 <- round(lower.tau2, 14)
  #
  if (!is.null(upper.tau))
    dat$upper.tau <- round(upper.tau, 14)
  if (!is.null(upper.tau2))
    dat$upper.tau2 <- round(upper.tau2, 14)
  #
  if (!is.null(sign.lower))
    dat$sign.lower <- sign.lower
  if (!is.null(sign.upper))
    dat$sign.upper <- sign.upper
  #
  if (!is.null(names(tau)))
    dat$names <- names(tau)
  #
  sel1 <- !duplicated(dat[, c("method.tau", "tau", "tau2")])
  #
  if (!is.null(dat$lower.tau) & !is.null(dat$lower.tau2) &
      !is.null(dat$upper.tau) & !is.null(dat$upper.tau2))
    sel2 <- !duplicated(dat[, c("method.tau", "tau", "tau2",
                                "lower.tau", "upper.tau",
                                "lower.tau2", "upper.tau2")])
  else
    sel2 <- sel1
  #
  sel <- sel1 & sel2
  #
  dat <- dat[sel, , drop = FALSE]
  #
  if (print.tau.ci) {
    dat$lower.tau <- round(dat$lower.tau, digits.tau)
    dat$upper.tau <- round(dat$upper.tau, digits.tau)
  }
  #
  if (print.tau2.ci) {
    dat$lower.tau2 <- round(dat$lower.tau2, digits.tau2)
    dat$upper.tau2 <- round(dat$upper.tau2, digits.tau2)
  }
  
  dat
}


hetdat <- function(I2, lowI2, uppI2, print.I2, print.I2.ci, digits.I2,
                   H, lowH, uppH, print.H, digits.H,
                   Rb, lowRb, uppRb, print.Rb) {
  dat <- data.frame()
  #
  if (print.I2) {
    if (nrow(dat) == 0)
      dat <- data.frame(I2 = round(I2, digits.I2))
    else
      dat$I2 <- round(I2, digits.I2)
    #
    if (print.I2.ci) {
      dat$lowI2 <- round(lowI2, digits.I2)
      dat$uppI2 <- round(uppI2, digits.I2)
    }
  }
  #
  if (print.H) {
    if (nrow(dat) == 0)
      dat <- data.frame(H = round(H, digits.H))
    else
      dat$H <- round(H, digits.H)
    #
    if (print.I2.ci) {
      dat$lowH <- round(lowH, digits.H)
      dat$uppH <- round(uppH, digits.H)
    }
  }
  #
  if (print.Rb) {
    if (nrow(dat) == 0)
      dat <- data.frame(Rb = round(Rb, digits.I2))
    else
      dat$Rb <- round(Rb, digits.I2)
    #
    if (print.I2.ci) {
      dat$lowRb <- round(lowRb, digits.I2)
      dat$uppRb <- round(uppRb, digits.I2)
    }
  }
  #
  if (!is.null(names(I2)))
    dat$names <- names(I2)
  else if (!is.null(names(H)))
    dat$names <- names(H)
  else if (!is.null(names(Rb)))
    dat$names <- names(Rb)
  #
  dat <- unique(dat)
  dat
}


qdat <- function(Q, df.Q, pval.Q, hetlabel, text.common) {
  if (length(Q) > 1) {
    if (!is.null(names(Q)))
      rownames.Q <- names(Q)
    else if (!is.null(hetlabel) && length(hetlabel) == length(Q))
      rownames.Q <- hetlabel
    else if (length(text.common) == length(Q))
      rownames.Q <- text.common
    else
      rownames.Q <- rep("", length(Q))
  }
  else
    rownames.Q <- rep("", length(Q))
  #
  sel <- rownames.Q != ""
  rownames.Q[sel] <- paste0(" ", rownames.Q[sel])
  #
  dat <- data.frame(Q, df.Q, pval.Q, names = rownames.Q)
  dat <- unique(dat)
  dat
}


collapse <- function(x, quote = '"', collapse = ", ", sort = FALSE)
  paste0(paste0(quote, if (sort) sort(x) else x, quote, collapse = collapse))

collapse2 <- function(x, quote = "", collapse = ", ", br1 = "", br2 = "",
                      sort = FALSE, max.len = 5) {
  if (sort)
    x <- sort(x)
  #
  if (length(x) == 0)
    res <- ""
  else if (length(x) == 1)
    res <- x
  else {
    if (is.numeric(x) && (all(diff(x) == 1) & length(x) > max.len))
      res <-
        paste0(br1, min(x, na.rm = TRUE), ", ..., ", max(x, na.rm = TRUE), br2)
    else
      res <- paste0(br1, paste(x, collapse = collapse), br2)
  }
  #
  res
}

condense <- function(x, var)
  unlist(lapply(x, "[[" , var))


cathet <- function(k,
                   method.tau, detail.tau,
                   tau2, lower.tau2, upper.tau2,
                   print.tau2, print.tau2.ci, text.tau2, digits.tau2,
                   tau, lower.tau, upper.tau,
                   print.tau, print.tau.ci, text.tau, digits.tau,
                   sign.lower.tau, sign.upper.tau,
                   I2, lowI2, uppI2, 
                   print.I2, print.I2.ci, text.I2, digits.I2,
                   H, lowH, uppH,
                   print.H, digits.H,
                   Rb, lowRb, uppRb,
                   print.Rb, text.Rb,
                   big.mark,
                   sort.tau = NULL, sort.het = NULL) {
  
  
  if (is.null(lower.tau2))
    lower.tau2 <- NA
  if (is.null(upper.tau2))
    upper.tau2 <- NA
  if (is.null(lower.tau))
    lower.tau <- NA
  if (is.null(upper.tau))
    upper.tau <- NA
  #
  if (all(is.na(lower.tau2) & is.na(upper.tau2)))
    print.tau2.ci <- FALSE
  if (all(is.na(lower.tau) & all(is.na(upper.tau))))
    print.tau.ci <- FALSE
  
  
  # Text for tau2 and tau
  #
  dtau <- taudat(method.tau, detail.tau,
                 tau, lower.tau, upper.tau, print.tau.ci, digits.tau,
                 tau2, lower.tau2, upper.tau2, print.tau2.ci, digits.tau2,
                 sign.lower.tau, sign.upper.tau)
  #
  ntau <- nrow(dtau)
  sort.tau <- setsort(sort.tau, ntau, "tau2 estimates")
  dtau <- dtau[sort.tau, , drop = FALSE]
  #
  if (print.tau2 | print.tau) {
    if (ntau > 1) {
      text.tau2 <- paste(text.tau2, seq_len(ntau), sep = ".")
      text.tau <- paste(text.tau, seq_len(ntau), sep = ".")
    }
  }
  #
  label.tau <-
    if (ntau > 1)
      ifelse(dtau$names == "", "", paste0(" (", dtau$names, ")"))
    else
      ""
  #
  if (print.tau2.ci) {
    text.tau2.ci <-
      pasteCI(dtau$lower.tau2, dtau$upper.tau2,
              digits.tau2, big.mark,
              dtau$sign.lower, dtau$sign.upper)
    text.tau2.ci[text.tau2.ci == " "] <- ""
  }
  else
    text.tau2.ci <- ""
  #
  if (print.tau.ci) {
    text.tau.ci <-
      pasteCI(dtau$lower.tau, dtau$upper.tau,
              digits.tau, big.mark,
              dtau$sign.lower, dtau$sign.upper)
    text.tau.ci[text.tau.ci == " "] <- ""
  }
  else
    text.tau.ci <- ""
  #
  hettxt <- 
    paste0(
      if (print.tau2 | print.tau | print.I2 | print.H | print.Rb)
        " ",
      if (print.tau2)
        paste0(formatPT(dtau$tau2,
                        lab = TRUE, labval = text.tau2,
                        digits = digits.tau2,
                        lab.NA = "NA",
                        big.mark = big.mark),
               text.tau2.ci,
               if (!print.tau) label.tau),
      #
      if (print.tau)
             paste0(
               if (print.tau2) "; " else "",
               formatPT(dtau$tau,
                        lab = TRUE, labval = text.tau,
                        digits = digits.tau,
                        lab.NA = "NA",
                   big.mark = big.mark),
               text.tau.ci,
               label.tau),
      collapse = "\n")
  
  
  # Text for I2, H and Rb
  #
  dhet <- hetdat(I2, lowI2, uppI2, print.I2, print.I2.ci, digits.I2,
                 H, lowH, uppH, print.H, digits.H,
                 Rb, lowRb, uppRb, print.Rb)
  #
  nhet <- nrow(dhet)
  sort.het <- setsort(sort.het, nhet, "heterogeneity estimates")
  dhet <- dhet[sort.het, , drop = FALSE]
  #
  label.het <-
    if (nhet > 1)
      ifelse(dhet$names == "", "", paste0(" (", dhet$names, ")"))
    else
      ""
  #
  if (print.I2) {
    if (nhet > 1)
      text.I2 <- paste(text.I2, seq_len(nhet), sep = ".")
  }
  #
  if (print.H) {
    text.H <- "H"
    if (nhet > 1)
      text.H <- paste(text.H, seq_len(nhet), sep = ".")
  }
  #  
  if (print.Rb) {
    if (nhet > 1)
      text.Rb <- paste(text.Rb, seq_len(nhet), sep = ".")
  }
  #
  if (print.I2)
    hettxt <-
      paste0(hettxt,
             ifelse(print.tau2 | print.tau,
             ifelse(ntau > 1 | print.tau2.ci | print.tau.ci |
                    (options()$width < 70 & print.I2.ci),
                    "\n", ";"),
             ""))
  #
  hettxt <-
    paste0(hettxt,
           paste0(
             if (print.I2)
               paste0(
                 if (print.tau2 | print.tau)
                   " ",
                 text.I2, " = ",
                 ifelse(is.na(dhet$I2), "NA",
                        paste0(formatN(dhet$I2, digits.I2), "%")),
                 if (print.I2.ci)
                   pasteCI(dhet$lowI2, dhet$uppI2, digits.I2,
                           big.mark, unit = "%"),
                 if (nhet > 1 & !print.H)
                   label.het
               ),
             #
             if (print.H)
               paste0(
                 if (print.tau2 | print.tau | print.I2)
                   "; " else " ",
                 text.H, " = ",
                 ifelse(is.na(dhet$H), "NA",
                        formatN(dhet$H, digits.H, "NA", big.mark = big.mark)),
                 if (print.I2.ci & any(!is.na(lowH) & !is.na(uppH)))
                   pasteCI(dhet$lowH, dhet$uppH, digits.H, big.mark),
                 if (nhet > 1)
                   label.het),
             collapse = "\n")
           )
  #
  if (print.Rb)
    hettxt <-
      paste0(hettxt,
             ifelse(print.tau2 | print.tau | print.I2 | print.H,
             ifelse(ntau > 1 | nhet > 1 |
                    print.tau2.ci | print.tau.ci |
                    print.I2.ci |
                    (options()$width < 70 & print.I2.ci),
                    "\n", ";"),
             ""))
  #
  if (print.Rb)
    hettxt <-
      paste0(hettxt,
             paste0(
               " ",
               text.Rb, " = ",
               ifelse(is.na(dhet$Rb), "NA",
                      paste0(formatN(dhet$Rb, digits.I2,
                                     big.mark = big.mark), "%")),
               if (any(!is.na(dhet$lowRb) & !is.na(dhet$uppRb)))
                 pasteCI(dhet$lowRb, dhet$uppRb, digits.I2,
                         big.mark, unit = "%"),
               label.het,
               collapse = "\n")
             )
  
  
  # Remove spaces before ";", " (" and a line break
  #
  hettxt <- gsub("(*UCP)\\s+(;)", "\\1", hettxt, perl = TRUE)
  hettxt <- gsub("(*UCP)\\s+( \\()", "\\1", hettxt, perl = TRUE)
  hettxt <- gsub("(*UCP)\\s+(\\\n)", "\\1", hettxt, perl = TRUE)
  
  
  # Print information on heterogeneity
  #
  cat(hettxt)
  
  
  # Empty row(s)
  #
  if (print.tau2 | print.tau | print.I2 | print.H | print.Rb)
    cat("\n")
  
  invisible(NULL)
}


catobsev <- function(var1, var2 = NULL, type = "n", addrow = FALSE,
                     big.mark = gs("big.mark")) {
  if (type == "n") {
    txt <- "observations"
    idx <- "o"
  }
  else if (type == "e") {
    txt <- "events"
    idx <- "e"
  }
  #
  if (!is.null(var1) & !is.null(var2)) {
    if (!(all(is.na(var1)) | all(is.na(var2)))) {
      sum1 <- sum(var1, na.rm = TRUE)
      sum2 <- sum(var2, na.rm = TRUE)
      #
      cat(paste0("Number of ", txt, ": ", idx, " = ",
                 format(sum1 + sum2, big.mark = big.mark),
                 if (type == "n")
                   paste0(" (", idx, ".e = ",
                          format(sum1, big.mark = big.mark),
                          ", ", idx, ".c = ",
                          format(sum2, big.mark = big.mark),
                          ")"),
                 "\n"))
    }
  }
  else if (!is.null(var1)) {
    if (!all(is.na(var1))) {
      cat(paste0("Number of ", txt, ": ", idx, " = ",
                 format(sum(var1, na.rm = TRUE),
                        big.mark = big.mark),
                 "\n"))
    }
  }
  else if (!is.null(var2)) {
    if (!all(is.na(var2))) {
      cat(paste0("Number of ", txt, ": ", idx, " = ",
                 format(sum(var2, na.rm = TRUE), big.mark = big.mark),
                 "\n"))
    }
  }
  #
  if (addrow)
    cat("\n")
  #
  invisible(NULL)
}


# The following R code is based on the file snowfall-internal.R from
# R package snowfall (Maintainer: Jochen Knaus <jo@imbi.uni-freiburg.de>)
#
# Helpers for managing the internal variables in the package namespace without
# awake the R CMD check for later R versions (which basically blaims many
# global assignments).
#
# The given solution has an advantage: only writing is affected. Reading of the
# objects can remain the same (thanks to Uwe Ligges for the tipp):
#   reading:  gs("CIbracket")
#   writing:  setOption("CIbracket", "(")
#

#
# Set an option in the meta option list.
# (Basically this is the setting of a list entry).
# key - character: object name
# val - object (everything is allowed, even NULL)
#
setOption <- function(key = NULL, val = NULL) {
  if(!is.null(key) && is.character(key)) {
    option <- getVar(".settings") # Get from NS
    option[[key]] <- val
    setVar(".settings", option) # Write to NS
    #
    return(invisible(TRUE))
  }
  #
  stop("Argument 'key' or 'val' is NULL or 'key' is no string.")
}


#
# Get a specific variable from the meta namespace.
# var - character: object name
#
getVar <- function(var = NULL) {
  if(!is.null(var) && is.character(var)) {
    tmp <- try(getFromNamespace(var, "meta"))
    #
    if(inherits(tmp, "try-error"))
      stop("Object", var, "not found in meta package.")
    #
    return(tmp)
  }

  stop("Argument 'var' is NULL or not a string.")
}


#
# Write a specific variable to the meta namespace.
# var - character: object name
# arg - object (NULL allowed)
#
setVar <- function(var = NULL, arg = NULL) {
  if(!is.null(var) && is.character(var)) {
    assignInNamespace(var, arg, "meta")

    return(invisible(TRUE))
  }

  stop("var is NULL or no character");
}

second <- function(x) x[2]

setOptionDepr <- function(x, new, old, func, ...) {
  newval <- do.call(func, list(argname = new, args = x, ...))
  oldval <- do.call(func, list(argname = old, args = x, ...))
  #
  if (is.na(newval) & !is.na(oldval))
    setOption(new, x[[oldval]])
  #
  invisible(NULL)
}





.settings <- list()
#
# List of internal settings
#
argslist.internal <-
  c("Wan2014.Table1", "Wan2014.Table2",
    "sm4bin", "sm4cont", "sm4cor", "sm4inc", "sm4mean", "sm4prop", "sm4rate",
    "ci4cont", "ci4prop", "ci4rate",
    "meth4bin", "meth4inc", "meth4prop", "meth4rate",
    "meth4tau", "meth4tau.ci", "meth4i2",
    "meth4common.ci",
    "meth4random.ci", "meth4pi",
    "adhoc4hakn.ci", "adhoc4hakn.pi",
    "meth4bias", "meth4bias.old",
    "tool4rob",
    "meth4incr",
    "text.fixed", "text.w.fixed",
    "major.update", "minor.update",
    #
    "special.characters")
#
setOption("argslist.internal", argslist.internal)
#
# Set defaults (for internal options)
#
setOption("sm4bin", c("OR", "RD", "RR", "ASD", "DOR", "VE"))
setOption("sm4cont", c("MD", "SMD", "ROM"))
setOption("sm4cor", c("ZCOR", "COR"))
setOption("sm4inc", c("IRR", "IRD", "IRSD", "VE"))
setOption("sm4mean", c("MRAW", "MLN"))
setOption("sm4prop", c("PLOGIT", "PLN", "PRAW", "PAS", "PFT"))
setOption("sm4rate", c("IR", "IRLN", "IRS", "IRFT"))
#
setOption("ci4cont", c("z", "t"))
setOption("ci4prop", c("CP", "WS", "WSCC", "AC", "SA", "SACC", "NAsm"))
setOption("ci4rate", c("NAsm", "Poisson"))
#
setOption("meth4bin", c("Inverse", "MH", "Peto", "GLMM", "LRP", "SSW"))
setOption("meth4inc", c("Inverse", "MH", "Cochran", "GLMM"))
setOption("meth4prop", c("Inverse", "GLMM"))
setOption("meth4rate", c("Inverse", "GLMM"))
#
setOption("meth4tau", c("DL", "PM", "REML", "ML", "HS", "SJ", "HE", "EB"))
setOption("meth4tau.ci", c("QP", "BJ", "J", "PL", ""))
setOption("meth4i2", c("Q", "tau2"))
setOption("meth4common.ci", c("classic", "IVhet"))
setOption("meth4random.ci", c("classic", "HK", "KR"))
setOption("meth4pi",
          c("V", "HTS", "HK", "HK-PR", "KR", "KR-PR", "NNF", "S", ""))
setOption("adhoc4hakn.ci", c("", "se", "ci", "IQWiG6"))
setOption("adhoc4hakn.pi", c("", "se"))
#
setOption("meth4bias.old", c("rank", "linreg", "mm", "count", "score"))
setOption("meth4bias", c("Begg", "Egger", "Thompson", "Schwarzer",
                         "Harbord", "Peters", "Deeks",
                         "Pustejovsky", "Macaskill"))
#
setOption("tool4rob",
          c("RoB1", "RoB2", "RoB2-cluster", "RoB2-crossover",
            "ROBINS-I", "ROBINS-E"))
#
setOption("meth4incr", c("only0", "if0all", "all", "user"))
#
setOption("special.characters", c("+", ".", "&", "$", "#", "|", "*", "^"))
#
setOption("major.update", 5)
setOption("minor.update", 6)
#
# List of arguments that can be changed by user
#
argslist <-
  c("level", "level.ma", "common", "random",
    "method.common.ci", "method.random.ci", "method.predict",
    "adhoc.hakn.ci", "adhoc.hakn.pi",
    "method.tau", "method.tau.ci", "level.hetstat", "tau.common",
    "method.I2",
    "prediction", "level.predict",
    "method.bias",
    "tool.rob",
    "overall.hetstat",
    "text.common", "text.random", "text.predict",
    "text.w.common", "text.w.random",
    "title", "complab",
    "CIbracket", "CIseparator", "CIlower.blank", "CIupper.blank",
    "print.subgroup.name", "sep.subgroup",
    "keepdata", "keeprma", "warn", "warn.deprecated",
    "transf", "backtransf",
    "smbin", "smcont", "smcor", "sminc", "smmean", "smprop", "smrate",
    "incr", "method.incr",
    "method", "allstudies", "MH.exact",
    "RR.Cochrane", "Q.Cochrane", "model.glmm", "print.CMH",
    "pooledvar", "method.smd", "sd.glass", "exact.smd",
    "method.ci.cont", "method.ci.prop", "method.ci.rate",
    "label.e", "label.c", "label.left", "label.right",
    "layout", "forest.details",
    "test.overall", "test.subgroup", "prediction.subgroup",
    "print.tau2.ci.subgroup", "print.tau.ci.subgroup",
    "test.effect.subgroup",
    "digits", "digits.mean", "digits.sd", "digits.se", "digits.stat",
    "digits.Q", "digits.tau2", "digits.tau", "digits.H", "digits.I2",
    "digits.prop", "digits.weight",
    "digits.pval", "digits.pval.Q",
    "digits.forest", "digits.TE.forest",
    "digits.df", "digits.cid",
    "scientific.pval", "big.mark", "zero.pval", "JAMA.pval",
    "details",
    "print.tau2", "print.tau2.ci", "print.tau", "print.tau.ci",
    "print.I2", "print.I2.ci", "print.H", "print.Rb",
    "text.tau2", "text.tau", "text.I2", "text.Rb",
    "print.Q",
    #
    "lty.common", "lty.random", "col.common", "col.random",
    "sort.subgroup",
    "pooled.events", "pooled.times", "study.results",
    "cid", "cid.below.null", "cid.above.null", "lty.cid", "col.cid", "fill.cid",
    "cid.pooled.only",
    "fill", "fill.equi",
    "leftcols", "rightcols", "leftlabs", "rightlabs", 
    "label.e.attach", "label.c.attach",
    "bottom.lr",
    "lab.NA", "lab.NA.effect", "lab.NA.weight",
    "lwd", "lwd.square", "lwd.diamond",
    "arrow.type", "arrow.length",
    "type.study", "type.common",
    "col.study", "col.square", "col.square.lines", "col.circle", "col.inside",
    "col.diamond", "col.diamond.lines",
    "col.predict", "col.predict.lines",
    "col.subgroup",
    "col.label.right", "col.label.left",
    "col.lines", "col.label",
    "hetlab", "resid.hetstat", "resid.hetlab",
    "forest.I2", "forest.I2.ci", "forest.tau2", "forest.tau2.ci",
    "forest.tau", "forest.tau.ci", "forest.Q", "forest.pval.Q",
    "forest.Rb", "forest.Rb.ci",
    "label.n", "label.events",
    "label.mean", "label.sd",
    "label.cor", "label.time",
    "label.pval", "label.tau2", "label.tau", "label.I2",
    "label.cluster", "label.cycles",
    "text.subgroup.nohet",
    "LRT",
    "forest.stat", "forest.Q.subgroup",
    "header.line",
    "fontsize", "fontfamily",
    "fs.common", "fs.random", "fs.predict",
    "fs.common.labels", "fs.random.labels", "fs.predict.labels",
    "fs.hetstat", "fs.test.overall", "fs.test.subgroup",
    "fs.test.effect.subgroup", "fs.addline",
    "ff.heading", "ff.common", "ff.random", "ff.predict",
    "ff.common.labels", "ff.random.labels", "ff.predict.labels",
    "ff.study", "ff.hetstat", "ff.test.overall", "ff.test.subgroup",
    "ff.test.effect.subgroup", "ff.addline",
    "ff.axis", "ff.smlab", "ff.xlab", "ff.lr",
    "colgap", "colgap.forest",
    "width",
    "calcwidth.predict", "calcwidth.hetstat",
    "calcwidth.tests", "calcwidth.subgroup", "calcwidth.addline",
    "just.studlab", "just.addcols",
    "spacing",
    "addrow", "addrow.overall", "addrow.subgroups", "addrows.below.overall"
    )
#
args.depr <- c("fixed", "comb.fixed", "comb.random", "level.comb",
               "hakn", "adhoc.hakn",
               "digits.zval", "print.byvar", "byseparator",
               "addincr", "allincr",
               "lower.equi", "upper.equi", "lty.equi", "col.equi")
#
setOption("argslist", c(argslist, args.depr))
#
setOption("argslist.meta", c(argslist, args.depr))
#
# General settings
#
setOption("level", 0.95)
setOption("level.ma", 0.95)
setOption("level.comb", 0.95)
#
setOption("common", TRUE)
setOption("fixed", TRUE)
setOption("comb.fixed", TRUE)
setOption("random", TRUE)
setOption("comb.random", TRUE)
setOption("method.common.ci", "classic")
setOption("method.random.ci", "classic")
setOption("hakn", FALSE)
setOption("adhoc.hakn", "")
setOption("adhoc.hakn.ci", "")
setOption("adhoc.hakn.pi", "")
setOption("prediction", FALSE)
setOption("level.predict", 0.95)
setOption("method.predict", "V")
setOption("method.tau", "REML")
setOption("method.tau.ci", NULL)
setOption("level.hetstat", 0.95)
setOption("tau.common", FALSE)
setOption("method.I2", "Q")
setOption("method.bias", "Egger")
setOption("tool.rob", NULL)
setOption("overall.hetstat", NULL)
setOption("text.common", "Common effect model")
setOption("text.fixed", "Common effect model")
setOption("text.random", "Random effects model")
setOption("text.predict", "Prediction interval")
setOption("text.w.common", "common")
setOption("text.w.fixed", "common")
setOption("text.w.random", "random")
setOption("title", "")
setOption("complab", "")
setOption("CIbracket", "[")
setOption("CIseparator", "; ")
setOption("CIlower.blank", TRUE)
setOption("CIupper.blank", TRUE)
setOption("print.subgroup.name", TRUE)
setOption("print.byvar", TRUE)
setOption("sep.subgroup", " = ")
setOption("byseparator", " = ")
setOption("test.subgroup", TRUE)
setOption("prediction.subgroup", FALSE)
setOption("print.tau2.ci.subgroup", NULL)
setOption("print.tau.ci.subgroup", NULL)
setOption("keepdata", TRUE)
setOption("keeprma", FALSE)
setOption("warn", TRUE)
setOption("warn.deprecated", TRUE)
setOption("transf", TRUE)
setOption("backtransf", TRUE)
setOption("digits", 4)
setOption("digits.mean", 2)
setOption("digits.sd", 4)
setOption("digits.se", 4)
setOption("digits.stat", 2)
setOption("digits.zval", 2)
setOption("digits.Q", 2)
setOption("digits.tau2", 4)
setOption("digits.tau", 4)
setOption("digits.H", 2)
setOption("digits.I2", 1)
setOption("digits.prop", 4)
setOption("digits.weight", 1)
setOption("digits.pval", 4)
setOption("digits.pval.Q", 4)
setOption("digits.df", 4)
setOption("digits.cid", 4)
setOption("scientific.pval", FALSE)
setOption("big.mark", "")
setOption("zero.pval", TRUE)
setOption("JAMA.pval", FALSE)
#
setOption("details", TRUE)
#
setOption("print.tau2", TRUE)
setOption("print.tau2.ci", TRUE)
setOption("print.tau", TRUE)
setOption("print.tau.ci", TRUE)
setOption("print.I2", TRUE)
setOption("print.I2.ci", TRUE)
setOption("print.H", TRUE)
setOption("print.Rb", FALSE)
setOption("text.tau2", "tau^2")
setOption("text.tau", "tau")
setOption("text.I2", "I^2")
setOption("text.Rb", "Rb")
#
setOption("print.Q", TRUE)
#
# Default summary measure
#
setOption("smbin", "RR")
setOption("smcont", "MD")
setOption("smcor", "ZCOR")
setOption("sminc", "IRR")
setOption("smmean", "MRAW")
setOption("smprop", "PLOGIT")
setOption("smrate", "IRLN")
#
# Settings for R functions metabin, metainc, metaprop
#
setOption("incr", 0.5)
setOption("method.incr", "only0")
setOption("allincr", FALSE)
setOption("addincr", FALSE)
#
# Additional settings for R function metabin
#
setOption("method", "MH")
setOption("allstudies", FALSE)
setOption("MH.exact", FALSE)
setOption("RR.Cochrane", FALSE)
setOption("Q.Cochrane", TRUE)
setOption("model.glmm", "UM.FS")
setOption("print.CMH", FALSE)
#
# Additional setting for R function metacont
#
setOption("pooledvar", FALSE)
setOption("method.smd", "Hedges")
setOption("sd.glass", "control")
setOption("exact.smd", TRUE)
setOption("method.ci.cont", "z")
#
# Additional setting for R function metaprop
#
setOption("method.ci.prop", "CP")
#
# Additional setting for R function metarate
#
setOption("method.ci.rate", "NAsm")
#
# Settings for R functions comparing two treatments
#
setOption("label.e", "Experimental")
setOption("label.c", "Control")
setOption("label.left", "")
setOption("label.right", "")
#
# Settings for R function forest.meta
#
setOption("layout", "meta")
setOption("forest.details", FALSE)
setOption("test.overall", FALSE)
setOption("test.effect.subgroup", FALSE)
setOption("digits.forest", 2)
setOption("digits.TE.forest", 4)
#
setOption("lty.common", 2)
setOption("lty.random", 3)
setOption("col.common", "black")
setOption("col.random", "black")
#
setOption("sort.subgroup", FALSE)
#
setOption("pooled.events", FALSE)
setOption("pooled.times", FALSE)
setOption("study.results", TRUE)
#
setOption("cid", NA)
setOption("cid.below.null", NA)
setOption("cid.above.null", NA)
setOption("lty.cid", 1)
setOption("col.cid", "blue")
setOption("fill.cid", "transparent")
setOption("cid.pooled.only", FALSE)
#
setOption("lower.equi", NA)
setOption("upper.equi", NA)
setOption("lty.equi", 1)
setOption("col.equi", "blue")
setOption("fill.equi", "transparent")
#
setOption("fill", "transparent")
#
setOption("leftcols", NULL)
setOption("rightcols", NULL)
setOption("leftlabs", NULL)
setOption("rightlabs", NULL)
#
setOption("label.e.attach", NULL)
setOption("label.c.attach", NULL)
#
setOption("bottom.lr", TRUE)
#
setOption("lab.NA", ".")
setOption("lab.NA.effect", NULL)
setOption("lab.NA.weight", ".")
#
setOption("lwd", 1)
setOption("lwd.square", 1)
setOption("lwd.diamond", 1)
#
setOption("arrow.type", "open")
setOption("arrow.length", 0.05)
#
setOption("type.study", "square")
setOption("type.common", "diamond")
#
setOption("col.study", "black")
setOption("col.square", "gray")
setOption("col.square.lines", "gray")
setOption("col.circle", "royalblue")
setOption("col.inside", "white")
setOption("col.diamond", "gray")
setOption("col.diamond.lines", "black")
setOption("col.predict", "red")
setOption("col.predict.lines", "black")
setOption("col.subgroup", "black")
setOption("col.label.right", "black")
setOption("col.label.left", "black")
#
setOption("col.lines", "black")
setOption("col.label", "black")
#
setOption("hetlab", "Heterogeneity: ")
setOption("resid.hetstat", NULL)
setOption("resid.hetlab", "Residual heterogeneity: ")
#
setOption("forest.I2", NULL)
setOption("forest.I2.ci", FALSE)
setOption("forest.tau2", NULL)
setOption("forest.tau2.ci", FALSE)
setOption("forest.tau", FALSE)
setOption("forest.tau.ci", FALSE)
setOption("forest.Q", FALSE)
setOption("forest.pval.Q", NULL)
setOption("forest.Rb", FALSE)
setOption("forest.Rb.ci", FALSE)
#
setOption("label.n", "Total")
setOption("label.events", "Events")
setOption("label.mean", "Mean")
setOption("label.sd", "SD")
setOption("label.cor", "Cor")
setOption("label.time", "Time")
setOption("label.pval", "P-value")
setOption("label.tau2", "Tau2")
setOption("label.tau", "Tau")
setOption("label.I2", "I2")
setOption("label.cluster", "Cluster")
setOption("label.cycles", "Cycles")
#
setOption("text.subgroup.nohet", "not applicable")
#
setOption("LRT", FALSE)
#
setOption("forest.stat", TRUE)
setOption("forest.Q.subgroup", TRUE)
#
setOption("header.line", FALSE)
#
setOption("fontsize", 12)
setOption("fontfamily", NULL)
setOption("fs.common", NULL)
setOption("fs.random", NULL)
setOption("fs.predict", NULL)
setOption("fs.common.labels", NULL)
setOption("fs.random.labels", NULL)
setOption("fs.predict.labels", NULL)
setOption("fs.hetstat", NULL)
setOption("fs.test.overall", NULL)
setOption("fs.test.subgroup", NULL)
setOption("fs.test.effect.subgroup", NULL)
setOption("fs.addline", NULL)
#
setOption("ff.heading", "bold")
setOption("ff.common", NULL)
setOption("ff.random", NULL)
setOption("ff.predict", NULL)
setOption("ff.common.labels", NULL)
setOption("ff.random.labels", NULL)
setOption("ff.predict.labels", NULL)
setOption("ff.study", "plain")
setOption("ff.hetstat", NULL)
setOption("ff.test.overall", NULL)
setOption("ff.test.subgroup", NULL)
setOption("ff.test.effect.subgroup", NULL)
setOption("ff.addline", NULL)
setOption("ff.axis", "plain")
setOption("ff.smlab", "bold")
setOption("ff.xlab", "plain")
setOption("ff.lr", "plain")
#
setOption("colgap", "2mm")
setOption("colgap.forest", "2mm")
#
setOption("width", NULL)
#
setOption("calcwidth.predict", FALSE)
setOption("calcwidth.hetstat", FALSE)
setOption("calcwidth.tests", FALSE)
setOption("calcwidth.subgroup", FALSE)
setOption("calcwidth.addline", FALSE)
#
setOption("just.studlab", "left")
setOption("just.addcols", "center")
#
setOption("spacing", 1)
setOption("addrow", NULL)
setOption("addrow.overall", NULL)
setOption("addrow.subgroups", NULL)
setOption("addrows.below.overall", NULL)


setOption("Wan2014.Table1",
          c(0.000, 1.128, 1.693, 2.059, 2.326,
            2.534, 2.704, 2.847, 2.970, 3.078,
            3.173, 3.259, 3.336, 3.407, 3.472,
            3.532, 3.588, 3.640, 3.689, 3.735,
            3.778, 3.819, 3.858, 3.895, 3.931,
            3.964, 3.997, 4.027, 4.057, 4.086,
            4.113, 4.139, 4.165, 4.189, 4.213,
            4.236, 4.259, 4.280, 4.301, 4.322,
            4.341, 4.361, 4.379, 4.398, 4.415,
            4.433, 4.450, 4.466, 4.482, 4.498))
#
setOption("Wan2014.Table2",
          c(0.990, 1.144, 1.206, 1.239, 1.260,
            1.274, 1.284, 1.292, 1.298, 1.303,
            1.307, 1.311, 1.313, 1.316, 1.318,
            1.320, 1.322, 1.323, 1.324, 1.326,
            1.327, 1.328, 1.329, 1.330, 1.330,
            1.331, 1.332, 1.332, 1.333, 1.333,
            1.334, 1.334, 1.335, 1.335, 1.336,
            1.336, 1.336, 1.337, 1.337, 1.337,
            1.338, 1.338, 1.338, 1.338, 1.339,
            1.339, 1.339, 1.339, 1.339, 1.340))

#' Mental Health Treatment
#' 
#' @description
#' Meta-analysis on the Effect of Mental Health Treatment on Medical
#' Utilisation.
#' 
#' Data example in Fleiss (1993) for meta-analysis with continuous outcomes.
#' 
#' @name Fleiss1993cont
#' @aliases Fleiss93cont
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{year}}\tab year of publication \cr
#' \bold{\emph{n.psyc}}\tab number of observations in psychotherapy
#'   group \cr
#' \bold{\emph{mean.psyc}}\tab estimated mean in psychotherapy group
#'   \cr
#' \bold{\emph{sd.psyc}}\tab standard deviation in psychotherapy group
#'   \cr
#' \bold{\emph{n.cont}}\tab number of observations in control group
#'   \cr
#' \bold{\emph{mean.cont}}\tab estimated mean in control group \cr
#' \bold{\emph{sd.cont}}\tab standard deviation in control group
#' }
#' 
#' @seealso \code{\link{Fleiss1993bin}}
#' 
#' @source
#' Fleiss JL (1993):
#' The statistical basis of meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{2}, 121--45
#'
#' @keywords datasets
#' 
#' @seealso \code{\link{metacont}}
#'
#' @examples
#' data(Fleiss1993cont)
#' # Note, the following command uses the bias-corrected version of
#' # Hedges' g. Accordingly, results differ from Fleiss (1993), section 3,
#' # using the uncorrected version of Hedges' g.
#' metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, studlab = paste(study, year),
#'   random = FALSE, sm = "SMD")


NULL

#' Calculate the number needed to treat (NNT)
#' 
#' @description
#' Calculate the number needed to treat (NNT) from the estimated risk
#' difference, risk ratio, odds ratio, or hazard ratio, and a baseline
#' probability, i.e., control group event probability for binary outcomes or
#' survival probability for hazard ratios.
#' 
#' @aliases nnt nnt.default nnt.meta
#' 
#' @param x An object of class \code{meta}, or estimated treatment
#'   effect(s), i.e., risk difference(s), risk ratio(s), odds
#'   ratio(s), or hazard ratio(s).
#' @param p.c Baseline probability, i.e., control group event probability
#'   for binary outcomes or survival probability in control group for
#'   hazard ratios.
#' @param sm Summary measure.
#' @param lower Lower confidence interval limit.
#' @param upper Upper confidence interval limit.
#' @param common A logical indicating whether NNTs should be
#'   calculated based on common effect estimate.
#' @param random A logical indicating whether NNTs should be
#'   calculated based on random effects estimate.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated.
#' @param transf A logical indicating whether treatment estimates and
#'   confidence limits are transformed or on the original scale.
#'   If \code{transf = TRUE}, inputs are expected to be log odds ratios instead
#'   of odds ratios for \code{sm = "OR"}, for example.
#' @param digits Minimal number of significant digits to print NNT and its
#'   confidence interval, see \code{print.default}.
#' @param digits.prop Minimal number of significant digits for
#'   proportions, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' The number needed to treat (NNT) is the estimated number of
#' patients who need to be treated with a new treatment instead of a
#' standard for one additional patient to benefit (Laupacis et al.,
#' 1988; Cook & Sackett, 1995). This definition of the NNT implies
#' that the new treatment is more beneficial than the standard. If the
#' new treatment is indeed less beneficial than the standard, the NNT
#' gives the number of patients treated with the new treatment to
#' observe an additional harmful event. Accordingly, the abbreviations
#' NNTB and NNTH can be used to distinguish between beneficial and
#' harmful NNTs (Altman, 1998).
#'
#' NNTs can be easily computed from an estimated risk difference (RD),
#' risk ratio (RR), or odds ratio (OR) and a given baseline probability
#' (Higgins et al., 2023, section 15.4.4). It is also possible to
#' calculate NNTs from hazard ratios (HR) (Altman & Andersen,
#' 1999). Accordingly, NNTs can be calculated for meta-analyses
#' generated with \code{\link{metabin}} or \code{\link{metagen}} if
#' argument \code{sm} was equal to \code{"RD"}, \code{"RR"},
#' \code{"OR"}, or \code{"HR"}. It is also possible to provide only
#' estimated treatment effects and baseline probabilities (see Examples).
#'
#' The baseline probability can be specified using argument \code{p.c}. If
#' this argument is missing, the minimum, mean, and maximum of the
#' control event probabilities in the meta-analysis are used for
#' \code{\link{metabin}} and control event probabilities of 0.1, 0.2,
#' \dots, 0.9 are used for \code{\link{metagen}}. Note, the survival instead of
#' mortality probability must be provided for hazard ratios.
#'
#' Argument \code{small.values} can be used to specify whether small
#' treatment effects indicate a beneficial (\code{"desirable"}) or harmful
#' (\code{"undesirable"}) effect. For \code{small.values = "desirable"}, odds,
#' risk and hazard ratios below 1 and risk differences below 0 indicate that the
#' new treatment is beneficial. For \code{small.values = "undesirable"}, odds,
#' risk and hazard ratios above 1 and risk differences above 0 indicate that
#' the new treatment is beneficial.
#'
#' \subsection{Interpretation of (positive and negative) NNTs}{
#' A positive value for the estimated NNT indicates that the new
#' treatment is beneficial, i.e., the NNT is actually an NNTB. On the
#' other hand, a negative value for the estimated NNT indicates that
#' the new treatment is harmful, i.e., the NNT is actually an NNTH.
#' 
#' The minimal value for the NNTB is 1. In this extreme case the new
#' treatment is 100\% effective and the standard treatment is 0\%
#' effective, i.e., only one patient has to be treated with the new
#' treatment for one additional patient to benefit. The NNTB increases
#' with decreasing difference between the two risks. If both risks are
#' equal, the NNTB is infinite.
#'
#' The other extreme is an NNT of -1 if the new treatment is 0\%
#' effective and the standard is 100\% effective. Here, one additional
#' harmful event is observed for each patient treated with the new
#' treatment. The NNT approaches minus infinity if the difference
#' between the two risks decreases to 0. Finally, an NNT of -1
#' translates to an NNTH of 1 with possible values from 1 to infinity.
#' }
#' 
#' \subsection{Confidence interval for the NNT}{
#' Confidence limits for an NNT are derived from the lower and upper
#' confidence limits of the summary measure using the same formulae as
#' for the NNT (Higgins et al., 2023, section 15.4.4).
#'
#' A peculiar problem arises if the confidence interval for the
#' summary measure includes the null effect (i.e., RR = 1, OR = 1, HR
#' = 1, or RD = 0). In this case the confidence interval for the NNT
#' contains both NNTB and NNTH values and it seemingly does not
#' include the estimated NNT.
#'
#' As described above, a positive NNT value corresponds to an NNTB and
#' the absolute value of a negative NNT is equal to an
#' NNTH. Accordingly, a confidence interval for the NNT from 20 to -5
#' translates to NNTB values between 20 and infinity and NNTH values
#' between 5 and infinity (Altman, 1998).
#' }
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metagen}}
#' 
#' @references
#' Altman DG (1998):
#' Confidence intervals for the number needed to treat.
#' \emph{British Medical Journal},
#' \bold{317}, 1309--12
#'
#' Altman DG, Andersen PK (1999):
#' Calculating the number needed to treat for trials where the outcome
#' is time to an event.
#' \emph{British Medical Journal},
#' \bold{319}, 1492--95
#' 
#' Cook RJ, Sackett DL (1995):
#' The number needed to treat: a clinically useful measure of
#' treatment effect.
#' \emph{British Medical Journal},
#' \bold{310}, 452--54
#' 
#' Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
#' VA (editors) (2023):
#' \emph{Cochrane Handbook for Systematic Reviews of Interventions
#'   Version 6.4 (updated August 2023)}.
#' Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}
#'
#' Laupacis A, Sackett DL, Roberts RS (1988):
#' An assessment of clinically useful measures of the consequences of
#' treatment.
#' \emph{New England Journal of Medicine},
#' \bold{318}, 1728--33
#' 
#' @examples
#' # Calculate NNTs for risk differences of -0.12 and -0.22
#' # (Cochrane Handbook, version 6.3, subsection 15.4.4.1)
#' nnt(c(-0.12, -0.22), sm = "RD")
#' 
#' # Calculate NNT for risk ratio of 0.92 and baseline risk of 0.3
#' # (Cochrane Handbook, version 6.3, subsection 15.4.4.2)
#' nnt(0.92, p.c = 0.3, sm = "RR")
#'
#' # Calculate NNT for odds ratio of 0.73 and baseline risk of 0.3
#' # (Cochrane Handbook, version 6.3, subsection 15.4.4.3)
#' nnt(0.73, p.c = 0.3, sm = "OR")
#'
#' # Calculate NNTs for Mantel-Haenszel odds ratio
#' data(Olkin1995)
#' ma <-
#'   metabin(ev.exp, n.exp, ev.cont, n.cont, data = Olkin1995, random = FALSE)
#' nnt(ma)
#'
#' # Calculate NNTs from hazard ratio at two and four years (example from
#' # Altman & Andersen, 1999). Note, argument 'p.c' must provide survival
#' # probabilities instead of mortality rates for the control group.
#' nnt(0.72, lower = 0.55, upper = 0.92, sm = "HR", p.c = 1 - c(0.33, 0.49))
#' 
#' @rdname nnt
#' @export nnt

nnt <- function(x, ...) 
  UseMethod("nnt")





#' @rdname nnt
#' @method nnt meta
#' @export

nnt.meta <- function(x, p.c,
                     common = x$common,
                     random = x$random,
                     small.values = "desirable",
                     ...) {
  
  
  #
  # (1) Check for meta object and summary measure
  #
  chkclass(x, "meta")
  x <- updateversion(x)
  #
  if (!(x$sm %in% c("RD", "RR", "OR", "HR")))
    stop("Calculation of NNTs only possible for risk difference, ",
         "risk ratio, odds ratio, or hazard ratio as summary measure ",
         "(argument 'sm').")
  
  
  #
  # (2) Check / set baseline risks
  #
  if (missing(p.c)) {
    if (x$sm == "RD")
      p.c <- NA
    else {
      if (!is.null(x$event.c) & !is.null(x$n.c)) {
        p.c <-
          unique(as.vector(
            summary(x$event.c / x$n.c,
                    na.rm = TRUE)[c("Min.", "Mean", "Max.")]))
        if (!all(p.c == 0))
          p.c <- p.c[p.c != 0]
      }
      else {
        p.c <- seq(0.1, 0.9, by = 0.1)
        if (x$sm == "HR")
          p.c <- rev(p.c)
      }
    }
  }
  #
  chknumeric(p.c, 0, 1)
  
  
  #
  # (3) Check other arguments
  #
  small.values <- setsv(small.values)
  #
  args  <- list(...)
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "fixed", FALSE)
  chklogical(common)
  chklogical(random)
  #
  if (missing.common & !common & !random & x$k == 1)
    common <- TRUE
  #
  if (missing.common & x$k == 0 & x$k.all == 1 &
      all(is.na(x$TE.common)) & any(!is.na(x$TE))) {
    common <- TRUE
    x$TE.common <- x$TE
  }
  
  
  #
  # (4) Calculate NNTs
  #
  res <- list()
  #
  if (common) {
    res$nnt.common <- data.frame(p.c = p.c, NNT = NA,
                                 lower.NNT = NA, upper.NNT = NA)
    #
    res$TE.common    <- x$TE.common
    res$lower.common <- x$lower.common
    res$upper.common <- x$upper.common
  }
  #
  if (random) {
    res$nnt.random <- data.frame(p.c = p.c, NNT = NA,
                                 lower.NNT = NA, upper.NNT = NA)
    #
    res$TE.random    <- x$TE.random
    res$lower.random <- x$lower.random
    res$upper.random <- x$upper.random
  }
  #
  if (x$sm == "RD") {
    if (common) {
      res$nnt.common$NNT <- -1 / res$TE.common
      res$nnt.common$lower.NNT <- -1 / res$lower.common
      res$nnt.common$upper.NNT <- -1 / res$upper.common
    }
    #
    if (random) {
      res$nnt.random$NNT <- -1 / res$TE.random
      res$nnt.random$lower.NNT <- -1 / res$lower.random
      res$nnt.random$upper.NNT <- -1 / res$upper.random
    }
  }
  #
  else if (x$sm == "RR") {
    if (common) {
      res$nnt.common$NNT <- logRR2nnt(res$TE.common, p.c)
      res$nnt.common$lower.NNT <- logRR2nnt(res$lower.common, p.c)
      res$nnt.common$upper.NNT <- logRR2nnt(res$upper.common, p.c)
    }
    #
    if (random) {
      res$nnt.random$NNT <- logRR2nnt(res$TE.random, p.c)
      res$nnt.random$lower.NNT <- logRR2nnt(res$lower.random, p.c)
      res$nnt.random$upper.NNT <- logRR2nnt(res$upper.random, p.c)
    }
  }
  #
  else if (x$sm == "OR") {
    if (common) {
      res$nnt.common$NNT <- logOR2nnt(res$TE.common, p.c)
      res$nnt.common$lower.NNT <- logOR2nnt(res$lower.common, p.c)
      res$nnt.common$upper.NNT <- logOR2nnt(res$upper.common, p.c)
    }
    #
    if (random) {
      res$nnt.random$NNT <- logOR2nnt(res$TE.random, p.c)
      res$nnt.random$lower.NNT <- logOR2nnt(res$lower.random, p.c)
      res$nnt.random$upper.NNT <- logOR2nnt(res$upper.random, p.c)
    }
  }
  #
  else if (x$sm == "HR") {
    if (common) {
      res$nnt.common$NNT <- logHR2nnt(res$TE.common, p.c)
      res$nnt.common$lower.NNT <- logHR2nnt(res$lower.common, p.c)
      res$nnt.common$upper.NNT <- logHR2nnt(res$upper.common, p.c)
    }
    #
    if (random) {
      res$nnt.random$NNT <- logHR2nnt(res$TE.random, p.c)
      res$nnt.random$lower.NNT <- logHR2nnt(res$lower.random, p.c)
      res$nnt.random$upper.NNT <- logHR2nnt(res$upper.random, p.c)
    }
  }
  #
  # Switch direction and lower and upper limits
  #
  if (small.values == "undesirable") {
    if (common) {
      res$nnt.common$NNT <- -res$nnt.common$NNT
      tmp.lower <- res$nnt.common$lower.NNT
      res$nnt.common$lower.NNT <- -res$nnt.common$upper.NNT
      res$nnt.common$upper.NNT <- -tmp.lower
    }
    #
    if (random) {
      res$nnt.random$NNT <- -res$nnt.random$NNT
      tmp.lower <- res$nnt.random$lower.NNT
      res$nnt.random$lower.NNT <- -res$nnt.random$upper.NNT
      res$nnt.random$upper.NNT <- -tmp.lower
    }
  }
  
  #
  res$sm <- x$sm
  res$common <- common
  res$random <- random
  #
  res$level.ma <- x$level.ma
  #
  res$call <- match.call()
  res$version <- packageDescription("meta")$Version
  #
  res$fixed <- common
  #
  class(res) <- "nnt.meta"
  
  
  res
}





#' @rdname nnt
#' @method nnt default
#' @export

nnt.default <- function(x, p.c, sm, lower, upper,
                        small.values = "desirable",
                        transf = FALSE,
                        ...) {
  
  #
  # (1) Check arguments
  #
  if (missing(x))
    stop("Argument 'x' is mandatory.")
  #
  if (missing(sm))
    stop("Argument 'sm' is mandatory.")
  #
  sm <- setchar(sm, c("RD", "RR", "OR", "HR"))
  #
  if (missing(p.c)) {
    if (sm == "RD")
      p.c <- NA
    else
      stop("Argument 'p.c' is mandatory.")
  }
  #
  chknumeric(p.c, 0, 1)
  #
  missing.lower <- missing(lower)
  if (!missing.lower)
    chklength(lower, length(x), "nnt.default")
  #
  missing.upper <- missing(upper)
  if (!missing.upper)
    chklength(upper, length(x), "nnt.default")
  #
  # Check length of arguments 'x' and 'p.c'
  #
  if (length(x) != length(p.c)) {
    if (length(x) > length(p.c))
      bad <- length(x) %% length(p.c) != 0
    else
      bad <- length(p.c) %% length(x) != 0
    #
    if (bad)
      stop("Lengths of arguments 'x' and 'p.c' do not match.",
           call. = FALSE)
  }
  #
  small.values <- setsv(small.values)
  #
  chklogical(transf)
  
  
  #
  # (2) Calculate NNTs
  #
  res <- data.frame(x = x, p.c = p.c,
                    NNT = NA, lower.NNT = NA, upper.NNT = NA)
  #
  if (missing.lower)
    res$lower.NNT <- NULL
  if (missing.upper)
    res$upper.NNT <- NULL
  #
  if (!missing.lower)
    res$lower.x <- lower
  if (!missing.upper)
    res$upper.x <- upper
  #
  if (sm == "RD") {
    res$NNT <- -1 / res$x
    if (!missing.lower)
      res$lower.NNT <- -1 / res$lower.x
    if (!missing.upper)
      res$upper.NNT <- -1 / res$upper.x
  }
  #
  else if (sm == "RR") {
    res$NNT <- logRR2nnt(if (!transf) log(res$x) else res$x, res$p.c)
    if (!missing.lower)
      res$lower.NNT <-
        logRR2nnt(if (!transf) log(res$lower.x) else res$lower.x, res$p.c)
    if (!missing.upper)
      res$upper.NNT <-
        logRR2nnt(if (!transf) log(res$upper.x) else res$upper.x, res$p.c)
  }
  #
  else if (sm == "OR") {
    res$NNT <- logOR2nnt(if (!transf) log(res$x) else res$x, res$p.c)
    if (!missing.lower)
      res$lower.NNT <-
        logOR2nnt(if (!transf) log(res$lower.x) else res$lower.x, res$p.c)
    if (!missing.upper)
      res$upper.NNT <-
        logOR2nnt(if (!transf) log(res$upper.x) else res$upper.x, res$p.c)
  }
  #
  else if (sm == "HR") {
    res$NNT <- logHR2nnt(if (!transf) log(res$x) else res$x, res$p.c)
    if (!missing.lower)
      res$lower.NNT <-
        logHR2nnt(if (!transf) log(res$lower.x) else res$lower.x, res$p.c)
    if (!missing.upper)
      res$upper.NNT <-
        logHR2nnt(if (!transf) log(res$upper.x) else res$upper.x, res$p.c)
  }
  #
  names(res)[names(res) == "x"] <- sm
  names(res)[names(res) == "lower.x"] <- paste0("lower.", sm)
  names(res)[names(res) == "upper.x"] <- paste0("upper.", sm)
  #
  # Remove baseline probability for risk difference
  #
  if (sm == "RD")
    res[["p.c"]] <- NULL
  else if (sm == "HR")
    names(res)[names(res) == "p.c"] <- "Surv.c"
  #
  # Switch direction and lower and upper limits
  #
  if (small.values == "undesirable") {
    res$NNT <- -res$NNT
    #
    tmp.lower <- res$lower.NNT
    if (!is.null(res$upper.NNT))
      res$lower.NNT <- -res$upper.NNT
    if (!is.null(tmp.lower))
      res$upper.NNT <- -tmp.lower
  }
  #
  attr(res, "small.values") <- small.values
  #
  class(res) <- c("nnt.default", class(res))
  
  res
}





#' @rdname nnt
#' @method print nnt.meta
#' @export

print.nnt.meta <- function(x,
                           common = x$common,
                           random = x$random,
                           digits = gs("digits"),
                           digits.prop = gs("digits.prop"),
                           big.mark = gs("big.mark"),
                           ...) {
  
  
  #
  # (1) Check for nnt.meta object
  #
  chkclass(x, "nnt.meta")
  
  
  #
  # (2) Check arguments
  #
  args  <- list(...)
  common <- deprecated(common, missing(common), args, "fixed", FALSE)
  if (is.null(common) & !is.null(x$fixed))
    common <- x$fixed
  chklogical(common)
  chklogical(random)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  
  
  ci.lab <- paste0(round(100 * replaceNULL(x$level.ma), 1), "%-CI")
  p.lab <- if (x$sm == "HR") "Surv.c" else "p.c"
  
  
  if (common) {
    cat(paste0("Number needed to treat (",
               tolower(gs("text.common")), "): \n\n"))
    #
    nnt.common <- cbind(TE = x$TE.common, x$nnt.common)
    if (x$sm != "RD")
      nnt.common$TE <- exp(nnt.common$TE)
    #
    ci.nnt <-
      sum(!is.na(x$lower.common)) > 0 & sum(!is.na(x$upper.common)) > 0
    #
    sign <-
      all(x$lower.common[!is.na(x$lower.common)] > 0) |
      all(x$upper.common[!is.na(x$upper.common)] < 0)
    #
    nnt.common$p.c <- formatN(round(nnt.common$p.c, digits.prop),
                              digits.prop, big.mark = big.mark)
    #
    if (all(nnt.common$NNT < 0)) {
      lab.nnt <- "NNTH"
      lab.nnt2 <- "NNTB"
      #
      nnt.common$NNT <- -nnt.common$NNT
      tmp.u <- -nnt.common$upper.NNT
      nnt.common$upper.NNT <- -nnt.common$lower.NNT
      nnt.common$lower.NNT <- tmp.u
    }
    else if (all(nnt.common$NNT > 0)) {
      lab.nnt <- "NNTB"
      lab.nnt2 <- "NNTH"
    }
    else {
      lab.nnt <- "NNT"
      lab.nnt2 <- "NNT2"
    }
    #
    nnt.common$TE <-
      formatN(round(nnt.common$TE, digits),
              digits, big.mark = big.mark)
    #
    nnt.common$NNT <- formatN(round(nnt.common$NNT, digits),
                              digits, big.mark = big.mark)
    #
    if (sign) {
      nnt.common$lower.NNT <-
        formatCI(formatN(round(nnt.common$lower.NNT, digits),
                         digits, big.mark = big.mark),
                 formatN(round(nnt.common$upper.NNT, digits),
                         digits, big.mark = big.mark))
      nnt.common$upper.NNT <- NULL
      #
      colnames(nnt.common) <- c(x$sm, p.lab, lab.nnt, ci.lab)
      #
      if (!ci.nnt)
        nnt.common <- nnt.common[, -4]
      #
      prmatrix(nnt.common, quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(nnt.common)))
    }
    else {
      bracktype <- setchar(gs("CIbracket"), c("[", "(", "{", ""))
      if (bracktype == "[") {
        bracketLeft <- paste0("[", lab.nnt, " ")
        bracketRight <- "]"
      }
      else if (bracktype == "(") {
        bracketLeft <- paste0("(", lab.nnt, " ")
        bracketRight <- ")"
      }
      else if (bracktype == "{") {
        bracketLeft <- paste0("{", lab.nnt, " ")
        bracketRight <- "}"
      }
      else if (bracktype == "") {
        bracketLeft <- paste0("", lab.nnt, " ")
        bracketRight <- ""
      }
      #
      nnt.common$lower.NNT <-
        formatCI(formatN(round(nnt.common$lower.NNT, digits),
                         digits, big.mark = big.mark),
                 formatN(round(-nnt.common$upper.NNT, digits),
                         digits, big.mark = big.mark),
                 bracket.left = bracketLeft,
                 bracket.right = bracketRight,
                 separator = paste0(" to Inf to ", lab.nnt2, " "))
      nnt.common$upper.NNT <- NULL
      #
      colnames(nnt.common) <- c(x$sm, p.lab, lab.nnt, ci.lab)
      #
      if (!ci.nnt)
        nnt.common <- nnt.common[, -4]
      #
      prmatrix(nnt.common, quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(nnt.common)))
    }
    #
    if (random)
      cat("\n")
  }
  #
  if (random) {
    cat(paste0("Number needed to treat (",
               tolower(gs("text.random")), "): \n\n"))
    #
    nnt.random <- cbind(TE = x$TE.random, x$nnt.random)
    if (x$sm != "RD")
      nnt.random$TE <- exp(nnt.random$TE)
    #
    ci.nnt <-
      sum(!is.na(x$lower.random)) > 0 & sum(!is.na(x$upper.random)) > 0
    #
    sign <-
      all(x$lower.random[!is.na(x$lower.random)] > 0) |
      all(x$upper.random[!is.na(x$upper.random)] < 0)
    #
    nnt.random$p.c <- formatN(round(nnt.random$p.c, digits.prop),
                              digits.prop, big.mark = big.mark)
    #
    if (all(nnt.random$NNT < 0)) {
      lab.nnt <- "NNTH"
      lab.nnt2 <- "NNTB"
      #
      nnt.random$NNT <- -nnt.random$NNT
      tmp.u <- -nnt.random$upper.NNT
      nnt.random$upper.NNT <- -nnt.random$lower.NNT
      nnt.random$lower.NNT <- tmp.u
    }
    else if (all(nnt.random$NNT > 0)) {
      lab.nnt <- "NNTB"
      lab.nnt2 <- "NNTH"
    }
    else {
      lab.nnt <- "NNT"
      lab.nnt2 <- "NNT2"
    }
    #
    nnt.random$TE <-
      formatN(round(nnt.random$TE, digits),
              digits, big.mark = big.mark)
    #
    nnt.random$NNT <- formatN(round(nnt.random$NNT, digits),
                              digits, big.mark = big.mark)
    #
    if (sign) {
      nnt.random$lower.NNT <-
        formatCI(formatN(round(nnt.random$lower.NNT, digits),
                         digits, big.mark = big.mark),
                 formatN(round(nnt.random$upper.NNT, digits),
                         digits, big.mark = big.mark))
      nnt.random$upper.NNT <- NULL
      #
      colnames(nnt.random) <- c(x$sm, p.lab, lab.nnt, ci.lab)
      #
      if (!ci.nnt)
        nnt.random <- nnt.random[, -4]
      #
      prmatrix(nnt.random, quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(nnt.random)))
    }
    else {
      bracktype <- setchar(gs("CIbracket"), c("[", "(", "{", ""))
      if (bracktype == "[") {
        bracketLeft <- paste0("[", lab.nnt, " ")
        bracketRight <- "]"
      }
      else if (bracktype == "(") {
        bracketLeft <- paste0("(", lab.nnt, " ")
        bracketRight <- ")"
      }
      else if (bracktype == "{") {
        bracketLeft <- paste0("{", lab.nnt, " ")
        bracketRight <- "}"
      }
      else if (bracktype == "") {
        bracketLeft <- paste0("", lab.nnt, " ")
        bracketRight <- ""
      }
      #
      nnt.random$lower.NNT <-
        formatCI(formatN(round(nnt.random$lower.NNT, digits),
                         digits, big.mark = big.mark),
                 formatN(round(-nnt.random$upper.NNT, digits),
                         digits, big.mark = big.mark),
                 bracket.left = bracketLeft,
                 bracket.right = bracketRight,
                 separator = paste0(" to Inf to ", lab.nnt2, " "))
      nnt.random$upper.NNT <- NULL
      #
      colnames(nnt.random) <- c(x$sm, p.lab, lab.nnt, ci.lab)
      #
      if (!ci.nnt)
        nnt.random <- nnt.random[, -4]
      #
      prmatrix(nnt.random, quote = FALSE, right = TRUE,
               rowlab = rep("", nrow(nnt.random)))
    }
  }
  
  
  invisible(NULL)
}





#' @rdname nnt
#' @method print nnt.default
#' @export

print.nnt.default <- function(x,
                              digits = gs("digits"),
                              digits.prop = gs("digits.prop"),
                              big.mark = gs("big.mark"),
                              ...) {
  
  #
  # (1) Check for nnt.default object
  #
  chkclass(x, "nnt.default")
  
  #
  # (2) Check arguments
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  
  #
  # (3) Round results
  #
  x$NNT <- formatN(round(x$NNT, digits), digits, big.mark = big.mark)
  if (isCol(x, "lower.NNT"))
    x$lower.NNT <-
      formatN(round(x$lower.NNT, digits), digits, big.mark = big.mark)
  if (isCol(x, "upper.NNT"))
    x$upper.NNT <-
      formatN(round(x$upper.NNT, digits), digits, big.mark = big.mark)
  #
  if (isCol(x, "p.c"))
    x$p.e <- formatN(round(x$p.c, digits.prop), digits.prop)
  if (isCol(x, "Surv.c"))
    x$p.e <- formatN(round(x$Surv.c, digits.prop), digits.prop)
  
  #
  # (4) Print results
  #
  class(x) <- class(x)[class(x) != "nnt.default"]
  print(x)
  
  invisible(NULL)
}





#
# Auxiliary R functions
#
logRR2nnt <- function(x, p.c)
    1 / (p.c - exp(x) * p.c)
#
logOR2nnt <- function(x, p.c)
  1 / (p.c - exp(x) * p.c / (1 - p.c + exp(x) * p.c))
#
logHR2nnt <- function(x, Surv.c)
  1 / (Surv.c^exp(x) - Surv.c)

#' Meta-analysis on Prevention of First Bleeding in Cirrhosis
#' 
#' @description
#' Meta-analysis on Prevention of First Bleeding in Cirrhosis
#' comparing beta-blocker or sclerotherapy with placebo.
#' 
#' @name Pagliaro1992
#' @aliases Pagliaro1992
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{id}}\tab study id \cr
#' \bold{\emph{treat.exp}}\tab treatment in experimental group \cr
#' \bold{\emph{logOR}}\tab log odds ratio
#'   \cr
#' \bold{\emph{selogOR}}\tab standard error of log odds ratio
#'   \cr
#' \bold{\emph{bleed.exp}}\tab number of bleedings in experimental group \cr
#' \bold{\emph{n.cont}}\tab number of observations in experimental group \cr
#' \bold{\emph{bleed.plac}}\tab number of bleedings in placebo group \cr
#' \bold{\emph{n.plac}}\tab number of observations in placebo group
#' }
#' 
#' @source
#' Pagliaro L, D’Amico G et al. (1992):
#' Prevention of first bleeding in cirrhosis.
#' \emph{Annals in Internal Medicine},
#' \bold{117}, 59--70 
#' 
#' @keywords datasets
#' 
#' @examples
#' data(Pagliaro1992)
#' sclero <- subset(Pagliaro1992, treat.exp == "Sclerotherapy")
#' 
#' m <- metagen(logOR, selogOR, data = sclero, sm = "OR")
#' m
#'
#' # Thompson & Sharp (1999), Table IV, method (2)
#' metabias(m, method = "Egger")
#'
#' # Thompson & Sharp (1999), Table IV, method (3a)
#' metabias(m, method = "Thompson")
#'
#' # Thompson & Sharp (1999), Table IV, method (3b)
#' update(m, method.tau = "ML")
#' metabias(update(m, method.tau = "ML"), method = "Thompson")
#' 


NULL

#' Create study labels in JAMA layout (deprecated function)
#' 
#' @description
#' Deprecated function to create study labels in JAMA layout (for
#' forest plot). Replaced by \code{\link{labels.meta}}.
#' 
#' @details
#' This auxiliary function can be used to create study labels in JAMA
#' layout which can be added to a forest plot using argument
#' 'studlab'.
#' 
#' @param author A vector providing study authors.
#' @param year A vector providing year of publication.
#' @param citation A vector providing citation numbers.
#' @param data An optional data frame containing the study
#'   information.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{labels.meta}}, \code{\link{forest.meta}}
#' 
#' @examples
#' data(Fleiss1993bin)
#' 
#' refs <- 20 + 1:7
#' 
#' Fleiss1993bin$mylabs <-
#'   JAMAlabels(study, year, refs, data = Fleiss1993bin)
#'
#' m <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
#'   studlab = paste(study, year),
#'   sm = "OR", random = FALSE)
#' 
#' forest(m, studlab = mylabs, layout = "JAMA",
#'   fontfamily = "Times", fontsize = 10)
#' 
#' @export JAMAlabels

JAMAlabels <- function(author, year, citation, data = NULL) {
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  if (missing(author) | missing(year) | missing(citation))
    stop("Mandatory arguments: 'author', 'year', and 'citation'.",
         call. = FALSE)
  #
  # Catch 'author', 'year', and 'citation' from data:
  #
  author <- catch("author", mc, data, sfsp)
  year <- catch("year", mc, data, sfsp)
  citation <- catch("citation", mc, data, sfsp)
  #
  if (length(author) != length(year))
    stop("Arguments 'author' and 'year' must be of same length.",
         call. = FALSE)
  #
  if (length(author) != length(citation))
    stop("Arguments 'author' and 'citation' must be of same length.",
         call. = FALSE)
  
  
  res <- NA
  for (i in seq(along = author))
    res[i] <-
      as.expression(substitute(study^ref~(year),
                               list(study = author[i],
                                    ref = citation[i],
                                    year = year[i])))
  #
  res
}

#' Meta-analysis of single proportions
#' 
#' @description
#' Calculation of an overall proportion from studies reporting a
#' single proportion. Inverse variance method and generalised linear
#' mixed model (GLMM) are available for pooling. For GLMMs, the
#' \code{\link[metafor]{rma.glmm}} function from R package
#' \bold{metafor} (Viechtbauer 2010) is called internally.
#' 
#' @param event Number of events.
#' @param n Number of observations.
#' @param studlab An optional vector with study labels.
#' @param data An optional data frame containing the study
#'   information, i.e., event and n.
#' @param subset An optional vector specifying a subset of studies to
#'   be used.
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots.
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param method A character string indicating which method is to be
#'   used for pooling of studies. One of \code{"Inverse"} and
#'   \code{"GLMM"}, can be abbreviated.
#' @param sm A character string indicating which summary measure
#'   (\code{"PLOGIT"}, \code{"PAS"}, \code{"PFT"}, \code{"PLN"}, or
#'   \code{"PRAW"}) is to be used for pooling of studies, see Details.
#' @param incr A numeric which is added to event number and sample
#'   size of studies with zero or all events, i.e., studies with an
#'   event probability of either 0 or 1. Or a numeric vector with the
#'   continuity correction for each study.
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (\code{"only0"},
#'   \code{"if0all"}, or \code{"all"}), see Details.
#' @param method.ci A character string indicating which method is used
#'   to calculate confidence intervals for individual studies, see
#'   Details.
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param null.effect A numeric value specifying the effect under the
#'   null hypothesis.
#' @param method.bias A character string indicating which test is to
#'   be used. Either \code{"Begg"}, \code{"Egger"}, or
#'   \code{"Thompson"}, can be abbreviated. See function
#'   \code{\link{metabias}}.
#' @param backtransf A logical indicating whether results for
#'   transformed proportions (argument \code{sm != "PRAW"}) should be
#'   back transformed in printouts and plots. If TRUE (default),
#'   results will be presented as proportions; otherwise transformed
#'   proportions will be shown. See Details for presentation of
#'   confidence intervals.
#' @param pscale A numeric defining a scaling factor for printing of
#'   single event probabilities.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param hakn Deprecated argument (replaced by 'method.random.ci').
#' @param adhoc.hakn Deprecated argument (replaced by 'adhoc.hakn.ci').
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if estimation problems exist in fitting a GLMM).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}} or
#'   \code{\link[metafor]{rma.glmm}}, respectively.
#' @param \dots Additional arguments passed on to
#'   \code{\link[metafor]{rma.glmm}} function and to catch deprecated
#'   arguments.
#' 
#' @details
#' This function provides methods for common effect and random effects
#' meta-analysis of single proportions to calculate an overall
#' proportion. Note, you should use R function \code{\link{metabin}}
#' to compare proportions of pairwise comparisons instead of using
#' \code{metaprop} for each treatment arm separately which will break
#' randomisation in randomised controlled trials.
#' 
#' The following transformations of proportions are
#' implemented to calculate an overall proportion:
#' 
#' \itemize{
#' \item Logit transformation (\code{sm = "PLOGIT"}, default)
#' \item Arcsine transformation (\code{sm = "PAS"})
#' \item Freeman-Tukey Double arcsine transformation (\code{sm = "PFT"})
#' \item Log transformation (\code{sm = "PLN"})
#' \item No transformation (\code{sm = "PRAW"})
#' }
#'
#' List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
#' contain the transformed proportions. In printouts and plots these
#' values are back transformed if argument \code{backtransf = TRUE}
#' (default).
#'
#' A generalised linear mixed model (GLMM) - more specific, a random
#' intercept logistic regression model - can be utilised for the
#' meta-analysis of proportions (Stijnen et al., 2010). This is the
#' default method for the logit transformation (argument \code{sm =
#' "PLOGIT"}). Internally, the \code{\link[metafor]{rma.glmm}}
#' function from R package \bold{metafor} is called to fit a GLMM.
#'
#' Classic meta-analysis (Borenstein et al., 2010) utilising the
#' (un)transformed proportions and corresponding standard errors in
#' the inverse variance method is conducted by calling the
#' \code{\link{metagen}} function internally. This is the only
#' available method for all transformations but the logit
#' transformation. The classic meta-analysis model with logit
#' transformed proportions is used by setting argument \code{method =
#' "Inverse"}.
#' 
#' A three-level random effects meta-analysis model (Van den Noortgate
#' et al., 2013) is utilised if argument \code{cluster} is used and at
#' least one cluster provides more than one estimate. Internally,
#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
#' "rowsum"} is used to calculate random effects weights.
#' 
#' Default settings are utilised for several arguments (assignments
#' using \code{\link{gs}} function). These defaults can be changed for
#' the current R session using the \code{\link{settings.meta}}
#' function.
#' 
#' Furthermore, R function \code{\link{update.meta}} can be used to
#' rerun a meta-analysis with different settings.
#' 
#' \subsection{Choice of transformation / meta-analysis method}{
#' 
#' Contradictory recommendations on the use of transformations of
#' proportions have been published in the literature. For example,
#' Barendregt et al. (2013) recommend the use of the Freeman-Tukey
#' double arcsine transformation instead of the logit transformation
#' whereas Warton & Hui (2011) strongly advise to use generalised
#' linear mixed models with the logit transformation instead of the
#' arcsine transformation.
#'
#' Schwarzer et al. (2019) describe seriously misleading results in a
#' meta-analysis with very different sample sizes due to problems with
#' the back-transformation of the Freeman-Tukey transformation which
#' requires a single sample size (Miller, 1978). Accordingly,
#' Schwarzer et al. (2019) also recommend to use GLMMs for the
#' meta-analysis of single proportions, however, admit that individual
#' study weights are not available with this method. Meta-analysts
#' which require individual study weights should consider the inverse
#' variance method with the arcsine or logit transformation.
#'
#' In order to prevent misleading conclusions for the Freeman-Tukey
#' double arcsine transformation, sensitivity analyses using other
#' transformations or using a range of sample sizes should be
#' conducted (Schwarzer et al., 2019).
#' }
#' 
#' \subsection{Continuity correction}{
#'
#' Three approaches are available to apply a continuity correction:
#' \itemize{
#' \item Only studies with a zero cell count (\code{method.incr =
#'   "only0"})
#' \item All studies if at least one study has a zero cell count
#'   (\code{method.incr = "if0all"})
#' \item All studies irrespective of zero cell counts
#'   (\code{method.incr = "all"})
#' }
#' 
#' If the summary measure is equal to "PLOGIT", "PLN", or "PRAW", the
#' continuity correction is applied if a study has either zero or all
#' events, i.e., an event probability of either 0 or 1.
#'
#' By default, 0.5 is used as continuity correction (argument
#' \code{incr}). This continuity correction is used both to calculate
#' individual study results with confidence limits and to conduct
#' meta-analysis based on the inverse variance method. For GLMMs no
#' continuity correction is used. Furthermore, the value of \code{incr} is
#' only considered in the calculation of confidence intervals for individual
#' studies if \code{method.ci = "NAsm"} (see next subsection).
#' }
#' 
#' \subsection{Confidence intervals for individual studies}{
#' 
#' Various methods are available to calculate confidence intervals for
#' individual study results (see Agresti & Coull 1998 and Newcombe
#' 1988):
#' \itemize{
#' \item Clopper-Pearson interval also called 'exact' binomial
#'   interval (\code{method.ci = "CP"}, default)
#' \item Wilson Score interval (\code{method.ci = "WS"})
#' \item Wilson Score interval with continuity correction
#'   (\code{method.ci = "WSCC"})
#' \item Agresti-Coull interval (\code{method.ci = "AC"})
#' \item Simple approximation interval (\code{method.ci = "SA"})
#' \item Simple approximation interval with continuity correction
#'   (\code{method.ci = "SACC"})
#' \item Normal approximation interval based on summary measure,
#'   i.e. defined by argument \code{sm} (\code{method.ci = "NAsm"})
#' }
#' 
#' Note, with exception of the normal approximation based on the
#' summary measure, i.e. \code{method.ci = "NAsm"}, the same
#' confidence interval is calculated for individual studies for any
#' summary measure (argument \code{sm}) as only number of events and
#' observations are used in the calculation disregarding the chosen
#' transformation. Furthermore, the continuity correction 
#'
#' Results will be presented for transformed proportions if argument
#' \code{backtransf = FALSE}. In this case, argument \code{method.ci =
#' "NAsm"} is used, i.e. confidence intervals based on the normal
#' approximation based on the summary measure.
#' }
#' 
#' \subsection{Subgroup analysis}{
#' 
#' Argument \code{subgroup} can be used to conduct subgroup analysis for
#' a categorical covariate. The \code{\link{metareg}} function can be
#' used instead for more than one categorical covariate or continuous
#' covariates.
#' }
#' 
#' \subsection{Specify the null hypothesis of test for an overall proportion}{
#'
#' Argument \code{null.effect} can be used to specify the proportion
#' used under the null hypothesis in a test for an overall effect.
#'
#' By default (\code{null.effect = NA}), no hypothesis test is
#' conducted as it is unclear which value is a sensible choice for the
#' data at hand.  An overall proportion of 50\%, for example, could be
#' tested by setting argument \code{null.effect = 0.5}.
#'
#' Note, all tests for an overall effect are two-sided with the
#' alternative hypothesis that the effect is unequal to
#' \code{null.effect}.
#' }
#' 
#' \subsection{Exclusion of studies from meta-analysis}{
#'
#' Arguments \code{subset} and \code{exclude} can be used to exclude
#' studies from the meta-analysis. Studies are removed completely from
#' the meta-analysis using argument \code{subset}, while excluded
#' studies are shown in printouts and forest plots using argument
#' \code{exclude} (see Examples in \code{\link{metagen}}).
#' Meta-analysis results are the same for both arguments.
#' }
#' 
#' \subsection{Presentation of meta-analysis results}{
#' 
#' Internally, both common effect and random effects models are
#' calculated regardless of values choosen for arguments
#' \code{common} and \code{random}. Accordingly, the estimate
#' for the random effects model can be extracted from component
#' \code{TE.random} of an object of class \code{"meta"} even if
#' argument \code{random = FALSE}. However, all functions in R
#' package \bold{meta} will adequately consider the values for
#' \code{common} and \code{random}. E.g. function
#' \code{\link{print.meta}} will not print results for the random
#' effects model if \code{random = FALSE}.
#' 
#' Argument \code{pscale} can be used to rescale proportions, e.g.
#' \code{pscale = 1000} means that proportions are expressed as events
#' per 1000 observations. This is useful in situations with (very) low
#' event probabilities.
#'
#' A prediction interval will only be shown if \code{prediction =
#' TRUE}.
#' }
#' 
#' @return
#' An object of class \code{c("metaprop", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{update.meta}},
#'   \code{\link{metacont}}, \code{\link{metagen}},
#'   \code{\link{print.meta}}, \code{\link{forest.meta}}
#' 
#' @references
#' Agresti A & Coull BA (1998):
#' Approximate is better than "exact" for interval estimation of
#' binomial proportions.
#' \emph{The American Statistician},
#' \bold{52}, 119--26
#' 
#' Barendregt JJ, Doi SA, Lee YY, Norman RE, Vos T (2013):
#' Meta-analysis of prevalence.
#' \emph{Journal of Epidemiology and Community Health},
#' \bold{67}, 974--8
#' 
#' Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
#' A basic introduction to fixed-effect and random-effects models for
#' meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{1}, 97--111
#' 
#' Freeman MF & Tukey JW (1950):
#' Transformations related to the angular and the square root.
#' \emph{Annals of Mathematical Statistics},
#' \bold{21}, 607--11
#' 
#' Miller JJ (1978):
#' The inverse of the Freeman-Tukey double arcsine transformation.
#' \emph{The American Statistician},
#' \bold{32}, 138
#' 
#' Newcombe RG (1998):
#' Two-sided confidence intervals for the single proportion:
#' comparison of seven methods.
#' \emph{Statistics in Medicine},
#' \bold{17}, 857--72
#' 
#' Pettigrew HM, Gart JJ, Thomas DG (1986):
#' The bias and higher cumulants of the logarithm of a binomial
#' variate.
#' \emph{Biometrika},
#' \bold{73}, 425--35
#' 
#' Schwarzer G, Chemaitelly H, Abu-Raddad LJ, Rücker G (2019):
#' Seriously misleading results using inverse of Freeman-Tukey double
#' arcsine transformation in meta-analysis of single proportions.
#' \emph{Research Synthesis Methods},
#' \bold{10}, 476--83
#' 
#' Stijnen T, Hamza TH, Ozdemir P (2010):
#' Random effects meta-analysis of event outcome in the framework of
#' the generalized linear mixed model with applications in sparse
#' data.
#' \emph{Statistics in Medicine},
#' \bold{29}, 3046--67
#'
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#' 
#' Viechtbauer W (2010):
#' Conducting meta-analyses in R with the metafor package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#'
#' Warton DI, Hui FKC (2011):
#' The arcsine is asinine: the analysis of proportions in ecology.
#' \emph{Ecology},
#' \bold{92}, 3--10
#' 
#' @examples
#' # Meta-analysis using generalised linear mixed model
#' #
#' metaprop(4:1, 10 * 1:4)
#' 
#' # Apply various classic meta-analysis methods to estimate
#' # proportions
#' #
#' ma1 <- metaprop(4:1, 10 * 1:4, method = "Inverse")
#' ma2 <- update(ma1, sm = "PAS")
#' ma3 <- update(ma1, sm = "PRAW")
#' ma4 <- update(ma1, sm = "PLN")
#' ma5 <- update(ma1, sm = "PFT")
#' #
#' ma1
#' ma2
#' ma3
#' ma4
#' ma5
#' #
#' forest(ma1)
#' \dontrun{
#' forest(ma2)
#' forest(ma3)
#' forest(ma3, pscale = 100)
#' forest(ma4)
#' forest(ma5)
#' }
#' 
#' # Do not back transform results, e.g. print logit transformed
#' # proportions if sm = "PLOGIT" and store old settings
#' #
#' oldset <- settings.meta(backtransf = FALSE)
#' #
#' ma6  <- metaprop(4:1, c(10, 20, 30, 40), method = "Inverse")
#' ma7  <- update(ma6, sm = "PAS")
#' ma8  <- update(ma6, sm = "PRAW")
#' ma9  <- update(ma6, sm = "PLN")
#' ma10 <- update(ma6, sm = "PFT")
#' #
#' forest(ma6)
#' \dontrun{
#' forest(ma7)
#' forest(ma8)
#' forest(ma8, pscale = 100)
#' forest(ma9)
#' forest(ma10)
#' }
#' 
#' # Use old settings
#' #
#' settings.meta(oldset)
#' 
#' # Examples with zero events
#' #
#' ma11 <- metaprop(c(0, 0, 10, 10), rep(100, 4), method = "Inverse")
#' ma12 <- metaprop(c(0, 0, 10, 10), rep(100, 4), incr = 0.1, method = "Inverse")
#' #
#' ma11
#' ma12
#' #
#' \dontrun{
#' forest(ma11)
#' forest(ma12)
#' }
#' 
#' # Example from Miller (1978):
#' #
#' death <- c(3, 6, 10, 1)
#' animals <- c(11, 17, 21, 6)
#' #
#' ma13 <- metaprop(death, animals, sm = "PFT")
#' forest(ma13)
#' 
#' # Data examples from Newcombe (1998)
#' # - apply various methods to estimate confidence intervals for
#' #   individual studies
#' #
#' event <- c(81, 15, 0, 1)
#' n <- c(263, 148, 20, 29)
#' #
#' ma14 <- metaprop(event, n, method.ci = "SA", method = "Inverse")
#' ma15 <- update(ma14, method.ci = "SACC")
#' ma16 <- update(ma14, method.ci = "WS")
#' ma17 <- update(ma14, method.ci = "WSCC")
#' ma18 <- update(ma14, method.ci = "CP")
#' #
#' lower <- round(logit2p(rbind(NA, ma14$lower, ma15$lower, NA, ma16$lower,
#'   ma17$lower, NA, ma18$lower)), 4)
#' upper <- round(logit2p(rbind(NA, ma14$upper, ma15$upper, NA, ma16$upper,
#'   ma17$upper, NA, ma18$upper)), 4)
#' #
#' tab1 <- data.frame(
#'   scen1 = meta:::formatCI(lower[, 1], upper[, 1]),
#'   scen2 = meta:::formatCI(lower[, 2], upper[, 2]),
#'   scen3 = meta:::formatCI(lower[, 3], upper[, 3]),
#'   scen4 = meta:::formatCI(lower[, 4], upper[, 4])
#'   )
#' names(tab1) <- c("r=81, n=263", "r=15, n=148",
#'   "r=0, n=20", "r=1, n=29")
#' row.names(tab1) <- c("Simple", "- SA", "- SACC",
#'   "Score", "- WS", "- WSCC", "Binomial", "- CP")
#' tab1[is.na(tab1)] <- ""
#' # Newcombe (1998), Table I, methods 1-5:
#' tab1
#' 
#' # Same confidence interval, i.e. unaffected by choice of summary
#' # measure
#' #
#' print(metaprop(event, n, method.ci = "WS", method = "Inverse"), ma = FALSE)
#' print(metaprop(event, n, sm = "PLN", method.ci = "WS"), ma = FALSE)
#' print(metaprop(event, n, sm = "PFT", method.ci = "WS"), ma = FALSE)
#' print(metaprop(event, n, sm = "PAS", method.ci = "WS"), ma = FALSE)
#' print(metaprop(event, n, sm = "PRAW", method.ci = "WS"), ma = FALSE)
#' 
#' # Different confidence intervals as argument sm = "NAsm"
#' #
#' print(metaprop(event, n, method.ci = "NAsm", method = "Inverse"), ma = FALSE)
#' print(metaprop(event, n, sm = "PLN", method.ci = "NAsm"), ma = FALSE)
#' print(metaprop(event, n, sm = "PFT", method.ci = "NAsm"), ma = FALSE)
#' print(metaprop(event, n, sm = "PAS", method.ci = "NAsm"), ma = FALSE)
#' print(metaprop(event, n, sm = "PRAW", method.ci = "NAsm"), ma = FALSE)
#' 
#' # Different confidence intervals as argument backtransf = FALSE.
#' # Accordingly, method.ci = "NAsm" used internally.
#' #
#' print(metaprop(event, n, method.ci = "WS", method = "Inverse"),
#'   ma = FALSE, backtransf = FALSE)
#' print(metaprop(event, n, sm = "PLN", method.ci = "WS"),
#'   ma = FALSE, backtransf = FALSE)
#' print(metaprop(event, n, sm = "PFT", method.ci = "WS"),
#'   ma = FALSE, backtransf = FALSE)
#' print(metaprop(event, n, sm = "PAS", method.ci = "WS"),
#'   ma = FALSE, backtransf = FALSE)
#' print(metaprop(event, n, sm = "PRAW", method.ci = "WS"),
#'   ma = FALSE, backtransf = FALSE)
#' 
#' # Same results (printed on original and log scale, respectively)
#' #
#' print(metaprop(event, n, sm = "PLN", method.ci = "NAsm"), ma = FALSE)
#' print(metaprop(event, n, sm = "PLN"), ma = FALSE, backtransf = FALSE)
#' # Results for first study (on log scale)
#' round(log(c(0.3079848, 0.2569522, 0.3691529)), 4)
#' 
#' # Print results as events per 1000 observations
#' #
#' print(metaprop(6:8, c(100, 1200, 1000), method = "Inverse"),
#'   pscale = 1000, digits = 1)
#' 
#' @export metaprop

metaprop <- function(event, n, studlab,
                     #
                     data = NULL, subset = NULL, exclude = NULL,
                     cluster = NULL, rho = 0,
                     #
                     weights = NULL,
                     weights.common = weights, weights.random = weights,
                     #
                     method,
                     #
                     sm = gs("smprop"),
                     #
                     incr = gs("incr"), method.incr = gs("method.incr"),
                     #
                     method.ci = gs("method.ci.prop"), level = gs("level"),
                     #
                     common = gs("common"),
                     random = gs("random") | !is.null(tau.preset),
                     overall = common | random,
                     overall.hetstat =
                       if (is.null(gs("overall.hetstat")))
                         common | random
                       else
                         gs("overall.hetstat"),   
                     prediction = gs("prediction") | !missing(method.predict),
                     #
                     method.tau =
                       ifelse(!is.na(charmatch(tolower(method), "glmm",
                                               nomatch = NA)),
                              "ML", gs("method.tau")),
                     method.tau.ci = gs("method.tau.ci"),
                     level.hetstat = gs("level.hetstat"),
                     tau.preset = NULL, TE.tau = NULL,
                     tau.common = gs("tau.common"),
                     detail.tau = NULL,
                     #
                     method.I2 = gs("method.I2"),
                     #
                     level.ma = gs("level.ma"),
                     method.common.ci = gs("method.common.ci"),
                     method.random.ci = gs("method.random.ci"),
                     adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                     #
                     level.predict = gs("level.predict"),
                     method.predict = gs("method.predict"),
                     adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                     seed.predict = NULL,
                     #
                     null.effect = NA,
                     #
                     method.bias = gs("method.bias"),
                     #
                     backtransf = gs("backtransf"),
                     pscale = 1,
                     #
                     text.common = gs("text.common"),
                     text.random = gs("text.random"),
                     text.predict = gs("text.predict"),
                     text.w.common = gs("text.w.common"),
                     text.w.random = gs("text.w.random"),
                     #
                     title = gs("title"), complab = gs("complab"),
                     outclab = "",
                     #
                     label.left = gs("label.left"),
                     label.right = gs("label.right"),
                     col.label.left = gs("col.label.left"),
                     col.label.right = gs("col.label.right"),
                     #
                     subgroup, subgroup.name = NULL,
                     print.subgroup.name = gs("print.subgroup.name"),
                     sep.subgroup = gs("sep.subgroup"),
                     test.subgroup = gs("test.subgroup"),
                     prediction.subgroup = gs("prediction.subgroup"),
                     seed.predict.subgroup = NULL,
                     #
                     byvar, hakn, adhoc.hakn,
                     #
                     keepdata = gs("keepdata"),
                     warn = gs("warn"), warn.deprecated = gs("warn.deprecated"),
                     #
                     control = NULL,
                     ...
                     ) {
  
  
  #
  #
  # (1) Check and set arguments
  #
  #
  
  chknumeric(rho, min = -1, max = 1)
  #
  chknull(sm)
  sm <- setchar(sm, gs("sm4prop"))
  #
  missing.method <- missing(method)
  if (missing.method)
    method <- if (sm == "PLOGIT") "GLMM" else "Inverse"
  else
    method <- setchar(method, gs("meth4prop"))
  is.glmm <- method == "GLMM"
  #
  missing.method.common.ci <- missing(method.common.ci)
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  if (method != "Inverse" & method.common.ci == "IVhet") {
    if (!missing.method.common.ci)
      warning("Argument 'method.common.ci = \"IVhet\"' only available ",
              "if 'method = \"Inverse\".",
              call. = FALSE)
    method.common.ci <- "classic"
  }
  #
  missing.method.incr <- missing(method.incr)
  method.incr <- setchar(method.incr, gs("meth4incr"))
  #
  chklevel(level)
  #
  missing.method.tau <- missing(method.tau)
  if (missing.method.tau)
    method.tau <- if (method == "GLMM") "ML" else gs("method.tau")
  method.tau <- setchar(method.tau, gs("meth4tau"))
  #
  missing.tau.common <- missing(tau.common)
  tau.common <- replaceNULL(tau.common, FALSE)
  chklogical(tau.common)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  chklogical(prediction)
  chklevel(level.predict)
  #
  missing.method.predict <- missing(method.predict)
  #
  method.tau <-
    set_method_tau(method.tau, missing.method.tau,
                 method.predict, missing.method.predict)
  method.predict <-
    set_method_predict(method.predict, missing.method.predict,
                     method.tau, missing.method.tau)
  #
  if (any(method.predict == "NNF"))
    is_installed_package("pimeta", argument = "method.predict", value = "NNF")
  #
  adhoc.hakn.pi <- setchar(replaceNA(adhoc.hakn.pi, ""), gs("adhoc4hakn.pi"))
  #
  if (!anyNA(null.effect) | length(null.effect) != 1)
    chknumeric(null.effect, min = 0, max = 1, length = 1)
  #
  method.bias <- setmethodbias(method.bias)
  #
  chklogical(backtransf)
  #
  chknumeric(pscale, length = 1)
  if (!backtransf & pscale != 1 & !is_untransformed(sm)) {
    warning("Argument 'pscale' set to 1 as argument 'backtransf' is FALSE.",
            call. = FALSE)
    pscale <- 1
  }
  #
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  if (!is.null(text.random))
    chkchar(text.random)
  if (!is.null(text.predict))
    chkchar(text.predict)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chklogical(keepdata)
  #
  # Additional arguments / checks for metainc objects
  #
  fun <- "metaprop"
  #
  method.ci <- setchar(method.ci, gs("ci4prop"))
  chklogical(warn)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing(level.ma), args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.random.ci <-
    deprecated2(method.random.ci, missing(method.random.ci),
                hakn, missing(hakn),
                warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  adhoc.hakn.ci <-
    deprecated2(adhoc.hakn.ci, missing(adhoc.hakn.ci),
                adhoc.hakn, missing(adhoc.hakn), warn.deprecated)
  adhoc.hakn.ci <- setchar(replaceNA(adhoc.hakn.ci, ""), gs("adhoc4hakn.ci"))
  #
  missing.subgroup.name <- missing(subgroup.name)
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing(print.subgroup.name),
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing(sep.subgroup), args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  addincr <-
    deprecated(method.incr, missing.method.incr, args, "addincr",
               warn.deprecated)
  allincr <-
    deprecated(method.incr, missing.method.incr, args, "allincr",
               warn.deprecated)
  if (missing.method.incr) {
    method.incr <- gs("method.incr")
    #
    if (is.logical(addincr) && addincr)
      method.incr <- "all"
    else if (is.logical(allincr) && allincr)
      method.incr <- "if0all"
  }
  #
  addincr <- allincr <- FALSE
  if (method.incr == "all")
    addincr <- TRUE
  else if (method.incr == "if0all")
    allincr <- TRUE
  #
  # Some more checks
  #
  chklogical(overall)
  chklogical(overall.hetstat)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata)
    data <- sfsp
  #
  # Catch 'event' and 'n' from data:
  #
  event <- catch("event", mc, data, sfsp)
  chknull(event)
  k.All <- length(event)
  #
  n <- catch("n", mc, data, sfsp)
  chknull(n)
  #
  # Catch 'incr' from data:
  #
  if (!missing(incr))
    incr <- catch("incr", mc, data, sfsp)
  chknumeric(incr, min = 0)
  #
  # Catch 'studlab', 'subgroup', 'subset', 'exclude' and 'cluster'
  # from data:
  #
  studlab <- catch("studlab", mc, data, sfsp)
  studlab <- setstudlab(studlab, k.All)
  #
  missing.subgroup <- missing(subgroup)
  subgroup <- catch("subgroup", mc, data, sfsp)
  missing.byvar <- missing(byvar)
  byvar <- catch("byvar", mc, data, sfsp)
  #
  subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                          warn.deprecated)
  by <- !is.null(subgroup)
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  exclude <- catch("exclude", mc, data, sfsp)
  missing.exclude <- is.null(exclude)
  #
  cluster <- catch("cluster", mc, data, sfsp)
  with.cluster <- !is.null(cluster)
  #
  # Catch 'weights', 'weights.common', and 'weights.random' from data:
  #
  if (!missing(weights))
    weights <- catch("weights", mc, data, sfsp)
  if (!missing(weights.common))
    weights.common <- catch("weights.common", mc, data, sfsp)
  if (!missing(weights.random))
    weights.random <- catch("weights.random", mc, data, sfsp)
  #
  if (!is.null(weights) & is.null(weights.common))
    weights.common <- weights
  #
  if (!is.null(weights) & is.null(weights.random))
    weights.random <- weights
  #
  usw.common <- !is.null(weights.common)
  usw.random <- !is.null(weights.random)
  #
  if (usw.common)
    chknumeric(weights.common, min = 0)
  #
  if (usw.random)
    chknumeric(weights.random, min = 0)
  #
  if (usw.common & method != "Inverse")
    stop("User-specified weights for the common effect model only implemented ",
         "for the inverse variance method (method = \"Inverse\").",
         call. = FALSE)
  #
  if (usw.random & method == "GLMM")
    stop("User-specified weights for the random effects model not implemented ",
         "for generalized linear mixed models (method = \"GLMM\").",
         call. = FALSE)
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  
  chklength(n, k.All, fun)
  chklength(studlab, k.All, fun)
  #
  if (with.cluster)
    chklength(cluster, k.All, fun)
  #
  if (usw.common) {
    if (length(weights.common) == 1)
      weights.common <- rep(weights.common, k.All)
    else
      chklength(weights.common, k.All, fun)
  }
  #
  if (usw.random) {
    if (length(weights.random) == 1)
      weights.random <- rep(weights.random, k.All)
    else
      chklength(weights.random, k.All, fun)
  }
  #
  if (length(incr) > 1)
    chklength(incr, k.All, fun)
  #
  if (by) {
    chklength(subgroup, k.All, fun)
    chklogical(test.subgroup)
    chklogical(prediction.subgroup)
  }
  #
  # Additional checks
  #
  if (!by & tau.common) {
    warning("Value for argument 'tau.common' set to FALSE as ",
            "argument 'subgroup' is missing.",
            call. = FALSE)
    tau.common <- FALSE
  }
  if (by & !tau.common & !is.null(tau.preset)) {
    warning("Argument 'tau.common' set to TRUE as ",
            "argument tau.preset is not NULL.",
            call. = FALSE)
    tau.common <- TRUE
  }
  
  
  #
  #
  # (4) Subset, exclude studies, and subgroups
  #
  #
  
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of subset is larger than number of studies.")
  #
  if (!missing.exclude) {
    if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
        (length(exclude) > k.All))
      stop("Length of argument 'exclude' is larger than number of studies.")
    #
    exclude2 <- rep(FALSE, k.All)
    exclude2[exclude] <- TRUE
    exclude <- exclude2
  }
  else
    exclude <- rep(FALSE, k.All)
  
  
  #
  #
  # (5) Store complete dataset in list object data
  #     (if argument keepdata is TRUE)
  #
  #
  
  if (keepdata) {
    if (nulldata)
      data <- data.frame(.event = event)
    else
      data$.event <- event
    #
    data$.n <- n
    data$.studlab <- studlab
    #
    data$.incr <- NA
    #
    if (by)
      data$.subgroup <- subgroup
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
    #
    if (!missing.exclude)
      data$.exclude <- exclude
    #
    if (with.cluster)
      data$.id <- data$.cluster <- cluster
    #
    if (usw.common)
      data$.weights.common <- weights.common
    #
    if (usw.random)
      data$.weights.random <- weights.random
  }
  
  
  #
  #
  # (6) Use subset for analysis
  #
  #
  
  if (!missing.subset) {
    event <- event[subset]
    n <- n[subset]
    studlab <- studlab[subset]
    #
    cluster <- cluster[subset]
    exclude <- exclude[subset]
    #
    weights.common <- weights.common[subset]
    weights.random <- weights.random[subset]
    #
    if (length(incr) > 1)
      incr <- incr[subset]
    #
    if (by)
      subgroup <- subgroup[subset]
  }
  #
  # Determine total number of studies
  #
  k.all <- length(event)
  #
  if (k.all == 0)
    stop("No studies to combine in meta-analysis.")
  #
  # No meta-analysis for a single study
  #
  if (k.all == 1) {
    common <- FALSE
    random <- FALSE
    prediction <- FALSE
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  #
  # Check variable values
  #
  chknumeric(event, 0)
  chknumeric(n, 0, zero = TRUE)
  #
  if (any(event > n, na.rm = TRUE))
    stop("Number of events must not be larger than number of observations")
  #
  # Recode integer as numeric:
  #
  event <- int2num(event)
  n     <- int2num(n)
  #
  # Check for whole numbers
  #
  if (method.ci != "NAsm") {
    if (any(!is_wholenumber(event), na.rm = TRUE)) {
      warning("Normal approximation confidence interval ",
              "(argument method.ci = \"NAsm\") used as\n",
              "at least one number of events contains a non-integer value.",
              call. = FALSE)
      method.ci <- "NAsm"
    }
    else if (any(!is_wholenumber(n), na.rm = TRUE)) {
      warning("Normal approximation confidence interval ",
              "(argument method.ci = \"NAsm\") used as\n",
              "at least one sample size contains a non-integer value.",
              call. = FALSE)
      method.ci <- "NAsm"
    }
  }
  #
  if (by) {
    chkmiss(subgroup)
    #
    if (missing.subgroup.name & is.null(subgroup.name)) {
      if (!missing.subgroup)
        subgroup.name <- byvarname("subgroup", mc)
      else if (!missing.byvar)
        subgroup.name <- byvarname("byvar", mc)
    }
  }
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  
  
  #
  #
  # (7) Continuity correction
  #
  #
  
  sel <- switch(sm,
                PLOGIT = event == 0 | (n - event) == 0,
                PAS = rep(FALSE, length(event)),
                PFT = rep(FALSE, length(event)),
                PLN =    event == 0 | (n - event) == 0,
                PRAW =   event == 0 | (n - event) == 0)
  #
  sparse <- any(sel, na.rm = TRUE)
  #
  # No need to add anything to cell counts for arcsine transformation
  #
  if (addincr | method.incr == "user")
    incr.event <- if (length(incr) == 1) rep(incr, k.all) else incr
  else {
    if (sparse) {
      if (allincr)
        incr.event <- if (length(incr) == 1) rep(incr, k.all) else incr
      else
        incr.event <- incr * sel
    }
    else
      incr.event <- rep(0, k.all)
  }
  #
  if (keepdata) {
    if (missing.subset) {
      data$.incr <- incr.event
    }
    else {
      data$.incr <- NA
      #
      data$.incr[subset] <- incr.event
    }
  }
  
  
  #
  #
  # (8) Calculate results for individual studies
  #
  #
  
  if (sm == "PLOGIT") {
    TE <- log((event + incr.event) / (n - event + incr.event))
    seTE <- sqrt(1 / (event + incr.event) +
                 1 / ((n - event + incr.event)))
    transf.null.effect <- p2logit(null.effect)
  }
  else if (sm == "PAS") {
    TE <- asin(sqrt(event / n))
    seTE <- sqrt(1 / (4 * n))
    transf.null.effect <- p2asin(null.effect)
  }
  else if (sm == "PFT") {
    TE <-
      0.5 * (asin(sqrt(event / (n + 1))) + asin(sqrt((event + 1) / (n + 1))))
    seTE <- sqrt(1 / (4 * n + 2))
    transf.null.effect <- p2asin(null.effect)
  }
  else if (sm == "PLN") {
    TE <- log((event + incr.event) / (n + incr.event))
    # Hartung, Knapp (2001), p. 3880, formula (18):
    seTE <- ifelse(event == n,
                   sqrt(1 / event                - 1 / (n + incr.event)),
                   sqrt(1 / (event + incr.event) - 1 / (n + incr.event))
                   )
    transf.null.effect <- log(null.effect)
  }
  else if (sm == "PRAW") {
    TE <- event / n
    seTE <- sqrt((event + incr.event) * (n - event + incr.event) /
                 (n + 2 * incr.event)^3)
    transf.null.effect <- null.effect
  }
  #
  # Calculate confidence intervals
  #
  if (method.ci == "CP")
    ci.study <- ciClopperPearson(event, n, level, null.effect)
  else if (method.ci == "WS")
    ci.study <- ciWilsonScore(event, n, level)
  else if (method.ci == "WSCC")
    ci.study <- ciWilsonScore(event, n, level, correct = TRUE)
  else if (method.ci == "AC")
    ci.study <- ciAgrestiCoull(event, n, level)
  else if (method.ci == "SA")
    ci.study <- ciSimpleAsymptotic(event, n, level)
  else if (method.ci == "SACC")
    ci.study <- ciSimpleAsymptotic(event, n, level, correct = TRUE)
  else if (method.ci == "NAsm")
    ci.study <- ci(TE, seTE, level, null.effect = null.effect)
  #
  lower.study <- ci.study$lower
  upper.study <- ci.study$upper
  #
  if (method.ci != "NAsm") {
    if (sm == "PLOGIT") {
      lower.study <- p2logit(lower.study)
      upper.study <- p2logit(upper.study)
    }
    #
    else if (sm == "PAS") {
      lower.study <- p2asin(lower.study)
      upper.study <- p2asin(upper.study)
    }
    #
    else if (sm == "PFT") {
      lower.ev <- n * lower.study 
      upper.ev <- n * upper.study 
      #
      lower.study <-
        0.5 * (asin(sqrt(lower.ev / (n + 1))) +
               asin(sqrt((lower.ev + 1) / (n + 1))))
      upper.study <-
        0.5 * (asin(sqrt(upper.ev / (n + 1))) +
               asin(sqrt((upper.ev + 1) / (n + 1))))
    }
    #
    else if (sm == "PLN") {
      lower.study <- log(lower.study)
      upper.study <- log(upper.study)
    }
  }
  
  
  #
  #
  # (9) Additional checks for three-level model
  #
  #
  
  three.level <- FALSE
  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (with.cluster &&
      length(unique(cluster[sel.ni])) != length(cluster[sel.ni]))
    three.level <- TRUE
  #
  if (three.level) {
    chkmlm(method.tau, missing.method.tau, method.predict,
           method, missing.method)
    #
    common <- FALSE
    method <- "Inverse"
    is.glmm <- FALSE
    #
    if (!(method.tau %in% c("REML", "ML")))
      method.tau <- "REML"
  }
  
  
  #
  #
  # (10) Additional checks for GLMM
  #
  #
  
  if (is.glmm) {
    chkglmm(sm, method.tau, method.random.ci, method.predict,
            adhoc.hakn.ci, adhoc.hakn.pi,
            "PLOGIT")
    #
    if (!is.null(TE.tau)) {
      if (warn)
        warning("Argument 'TE.tau' not considered for GLMM.",
                call. = FALSE)
      TE.tau <- NULL
    }
    #
    if (!is.null(tau.preset)) {
      if (warn)
        warning("Argument 'tau.preset' not considered for GLMM.",
                call. = FALSE)
      tau.preset <- NULL
    }
  }
  
  
  #
  #
  # (11) Do meta-analysis
  #
  #
  
  k <- sum(!is.na(event[!exclude]) & !is.na(n[!exclude]))
  #
  for (i in seq_along(method.random.ci))
    if (k == 1 & method.random.ci[i] == "HK")
      method.random.ci[i] <- "classic"
  #
  m <- metagen(TE, seTE, studlab,
               exclude = if (missing.exclude) NULL else exclude,
               cluster = cluster, rho = rho,
               #
               weights.common = weights.common,
               weights.random = weights.random,
               #
               sm = sm,
               level = level,
               #
               common = common,
               random = random,
               overall = overall,
               overall.hetstat = overall.hetstat,
               prediction = prediction,
               #
               method.tau = if (is.glmm) "DL" else method.tau,
               method.tau.ci = if (is.glmm) "" else method.tau.ci,
               level.hetstat = level.hetstat,
               tau.preset = tau.preset,
               TE.tau = TE.tau,
               tau.common = FALSE,
               detail.tau = detail.tau,
               #
               method.I2 = method.I2,
               #
               level.ma = level.ma,
               method.common.ci = method.common.ci,
               method.random.ci = method.random.ci,
               adhoc.hakn.ci = adhoc.hakn.ci,
               #
               level.predict = level.predict,
               method.predict = method.predict,
               adhoc.hakn.pi = adhoc.hakn.pi,
               seed.predict = seed.predict,
               #
               null.effect = transf.null.effect,
               #
               method.bias = method.bias,
               #
               backtransf = backtransf,
               #
               text.common = text.common, text.random = text.random,
               text.predict = text.predict,
               text.w.common = text.w.common, text.w.random = text.w.random,
               #
               title = title, complab = complab, outclab = outclab,
               #
               label.left = label.left, label.right = label.right,
               col.label.left = col.label.left,
               col.label.right = col.label.right,
               #
               keepdata = FALSE,
               warn = warn,
               #
               control = control)
  #
  # Estimate common tau-squared across subgroups
  #
  if (by & tau.common & !is.glmm)
    hcc <- hetcalc(TE, seTE, method.tau, "", TE.tau,
                   method.I2, level.hetstat, subgroup, control)
  
  
  #
  #
  # (12) Generate R object
  #
  #
  
  res <- list(event = event, n = n,
              incr = if (length(unique(incr)) == 1) unique(incr) else incr,
              method.incr = method.incr,
              sparse = sparse,
              method.ci = method.ci,
              incr.event = incr.event)
  #
  # Add meta-analysis results
  # (after removing unneeded list elements)
  #
  m$n.e <- NULL
  m$n.c <- NULL
  m$pscale <- NULL
  m$irscale <- NULL
  m$irunit <- NULL
  m$method.ci <- NULL
  m$method.mean <- NULL
  m$approx.TE <- NULL
  m$approx.seTE <- NULL
  #
  m$label.e <- ""
  m$label.c <- ""
  #
  if (method.ci == "CP") {
    m$statistic <- rep(NA, length(m$statistic))
    m$pval <- ci.study$p
  }    
  else if (method.ci != "NAsm") {
    m$statistic <- rep(NA, length(m$statistic))
    m$pval <- rep(NA, length(m$pval))
  }
  #
  if (is.glmm) 
    m$method.tau <- method.tau
  #
  if (is.glmm | three.level) {
    m$seTE.hakn.ci <- m$seTE.hakn.adhoc.ci <-
      m$seTE.hakn.pi <- m$seTE.hakn.adhoc.pi <-
        m$seTE.kero <- NA
    #
    m$text.random <- gsub("(HK)", "(T)", m$text.random, fixed = TRUE)
  }
  #
  res <- c(res, m)
  res$null.effect <- null.effect
  #
  # Run GLMM and add data
  #
  if (is.glmm & k > 0) {
    res$method <- "GLMM"
    res$method.random <- "GLMM"
    #
    list.prop <- list(xi = event[!exclude], ni = n[!exclude], measure = "PLO")
    #
    use.random <-
      sum(!exclude) > 1 &
      sum(event[!exclude], na.rm = TRUE) > 0 &
      any(event[!exclude] != n[!exclude])
    #
    res.glmm <-
      runGLMM(list.prop,
              method.tau = method.tau,
              method.random.ci = method.random.ci,
              level = level.ma,
              control = control, use.random = use.random,
              warn = warn)
    #
    res <- addGLMM(res, res.glmm, method.I2, transf.null.effect)
    #
    if (by) {
      n.subgroups <- length(unique(subgroup[!exclude]))
      if (n.subgroups > 1)
        subgroup.glmm <-
          factor(subgroup[!exclude], bylevs(subgroup[!exclude]))
      #
      hcc <-
        hccGLMM(
          res,
          runGLMM(list.prop,
                  method.tau = method.tau,
                  method.random.ci = method.random.ci,
                  level = level.ma,
                  data =
                    if (n.subgroups > 1)
                      list(data = data.frame(subgroup.glmm))
                    else
                      NULL,
                  mods =
                    if (n.subgroups > 1)
                      as.call(~ subgroup.glmm)
                    else
                      NULL,
                  control = control, use.random = use.random,
                  warn = warn)$glmm.random[[1]],
          method.I2
        )
    }
  }
  #
  res$lower <- lower.study
  res$upper <- upper.study
  #
  res$pscale <- pscale
  #
  res$pairwise <- FALSE
  #
  res$call <- match.call()
  #
  if (keepdata) {
    res$data <- data
    if (!missing.subset)
      res$subset <- subset
  }
  #
  class(res) <- c(fun, "meta")
  #
  # Add results from subgroup analysis
  #
  if (by) {
    res$subgroup <- subgroup
    res$subgroup.name <- subgroup.name
    res$print.subgroup.name <- print.subgroup.name
    res$sep.subgroup <- sep.subgroup
    res$test.subgroup <- test.subgroup
    res$prediction.subgroup <- prediction.subgroup
    res$tau.common <- tau.common
    #
    if (!tau.common) {
      res <- c(res, subgroup(res, seed = seed.predict.subgroup))
      if (res$three.level)
        res <- setNA3(res)
    }
    else if (!is.null(tau.preset))
      res <-
        c(res, subgroup(res, tau.preset, seed = seed.predict.subgroup))
    else {
      if (is.glmm)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup)), ...))
      else if (res$three.level)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup))))
      else
        res <-
          c(res, subgroup(res, hcc$tau.resid, seed = seed.predict.subgroup))
    }
    #
    if (tau.common && is.null(tau.preset))
      res <- addHet(res, hcc, !is.glmm)
    #
    res$n.e.w <- NULL
    res$n.c.w <- NULL
    res$event.e.w <- NULL
    res$event.c.w <- NULL
    #
    res$time.e.w <- NULL
    res$time.c.w <- NULL
    res$t.harmonic.mean.w <- NULL
    #
    res <- setNAwithin(res, res$three.level | is.glmm)
  }
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  class(res) <- c(fun, "meta")
  
  
  res
}

#' Print and change default meta-analysis settings in R package \bold{meta}
#' 
#' @description
#' Print and change default settings to conduct and print or plot
#' meta-analyses in R package \bold{meta}. The following general
#' settings are available: \emph{Review Manager 5}, \emph{Journal of
#' the American Medical Association}.
#' 
#' @param ... Arguments to change default settings.
#' @param quietly A logical indicating whether information on settings
#'   should be printed.
#' 
#' @details
#' This function can be used to define defaults for several arguments
#' (i.e., assignments using \code{\link{gs}}) of the following R
#' functions: \code{\link{metabin}}, \code{\link{metacont}},
#' \code{\link{metacor}}, \code{\link{metacr}}, \code{\link{metagen}},
#' \code{\link{metainc}}, \code{\link{metaprop}},
#' \code{\link{metarate}}
#' 
#' Furthermore, some of these settings are considered to print
#' meta-analysis results and to produce forest plots.
#' 
#' The function can be used to either change individual settings (see
#' Examples) or use one of the following general settings:
#' \itemize{
#' \item \code{settings.meta("RevMan5")}
#' \item \code{settings.meta("BMJ")}
#' \item \code{settings.meta("JAMA")}
#' \item \code{settings.meta("IQWiG5")}
#' \item \code{settings.meta("IQWiG6")}
#' \item \code{settings.meta("geneexpr")}
#' \item \code{settings.meta("IVhet")}
#' \item \code{settings.meta("meta4")}
#' \item \code{settings.meta("meta7")}
#' }
#'
#' The first command can be used to reproduce meta-analyses from
#' Cochrane reviews conducted with \emph{Review Manager 5} (RevMan 5)
#' and specifies to use a RevMan 5 layout in forest plots.
#'
#' The second command can be used to generate forest plots in BMJ layout.
#'
#' The third command can be used to generate forest plots following
#' instructions for authors of the \emph{Journal of the American
#' Medical Association}. Study labels according to JAMA guidelines can be
#' generated using \code{\link{labels.meta}}.
#'
#' The next commands implement the recommendations of the Institute
#' for Quality and Efficiency in Health Care, Germany (IQWiG)
#' accordinging to General Methods 5 and 6, respectively
#' (\url{https://www.iqwig.de/en/about-us/methods/methods-paper/}).
#'
#' The setting \code{"geneexpr"} can be used to print p-values in
#' scientific notation and to suppress the calculation of confidence
#' intervals for the between-study variance.
#'
#' The setting \code{"IVhet"} can be used for the inverse variance
#' heterogeneity model (Henmi & Copas, 2010; Doi et al., 2015).
#'
#' The last settings use the default settings of R package
#' \bold{meta}, version 4 and 7.0-0, respectively, or below.
#' 
#' RevMan 5 settings, in detail:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{method.random.ci} \tab "classic" \tab only available method
#'   in RevMan 5 \cr
#' \code{method.tau} \tab "DL" \tab only available method in RevMan 5
#'   \cr
#' \code{method.I2} \tab "Q" \tab only available method in RevMan 5 \cr
#' \code{tau.common} \tab FALSE \tab common between-study variance in
#'   subgroups \cr
#' \code{MH.exact} \tab FALSE \tab exact Mantel-Haenszel method \cr
#' \code{RR.Cochrane} \tab TRUE \tab calculation of risk ratios \cr
#' \code{Q.Cochrane} \tab TRUE \tab calculation of heterogeneity
#'   statistic \cr
#' \code{exact.smd} \tab FALSE \tab exact formulae for Hedges' g and
#'   Cohen's d \cr
#' \code{layout} \tab "RevMan5" \tab layout for forest plots \cr
#' \code{prediction} \tab FALSE \tab no prediction interval \cr
#' \code{test.overall} \tab TRUE \tab print information on test of
#'   overall effect \cr
#' \code{test.subgroup} \tab TRUE \tab print information on test for
#'   subgroup differences \cr
#' \code{test.effect.subgroup} \tab TRUE \tab print information on
#'   test for effect in subgroups \cr
#' \code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
#'   forest plots \cr
#' \code{forest.tau2} \tab TRUE \tab show between-study heterogeneity \cr
#'   \tab \tab variance in forest plots \cr
#' \code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity \cr
#'   \tab \tab standard deviation in forest plots \cr
#' \code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
#'   forest plots
#'   \cr
#' \code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
#'   in forest plots
#'   \cr
#' \code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
#'   forest plots
#'   \cr
#' \code{digits.tau2} \tab 3 \tab number of digits for tau-squared \cr
#' \code{digits.tau} \tab 4 \tab number of digits for square root of
#'   tau-squared \cr
#' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
#'   \cr
#' \code{CIbracket}, \tab "[" \tab \cr
#' \code{CIseparator} \tab ", " \tab print confidence intervals as
#'   "\code{[., .]}" \cr
#' \code{header.line}, \tab TRUE \tab print header line
#' }
#'
#' BMJ settings:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{layout} \tab "BMJ" \tab layout for forest plots \cr
#' \code{test.overall} \tab TRUE \tab print information on test of
#'   overall effect \cr
#' \code{test.subgroup} \tab FALSE \tab print information on test for
#'   subgroup differences \cr
#' \code{test.effect.subgroup} \tab FALSE \tab print information on
#'   test for effect in subgroups \cr
#' \code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
#'   forest plots
#'   \cr
#' \code{forest.tau2} \tab TRUE \tab show between-study heterogeneity \cr
#'   \tab \tab variance in forest plots \cr
#' \code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity \cr
#'   \tab \tab standard deviation in forest plots \cr
#' \code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
#'   forest plots
#'   \cr
#' \code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
#'   in forest plots
#'   \cr
#' \code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
#'   forest plots
#'   \cr
#' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
#'   \cr
#' \code{digits.pval} \tab 2 \tab number of digits for p-values \cr
#' \code{CIbracket}, \tab "(" \tab \cr
#' \code{CIseparator} \tab " to " \tab print confidence intervals as
#'   "\code{(. to .)}" \cr
#' \code{hetlab}, \tab \tab "Test for heterogeneity: " \cr
#' \code{header.line}, \tab TRUE \tab print header line
#' }
#'
#' JAMA settings:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{layout} \tab "JAMA" \tab layout for forest plots \cr
#' \code{test.overall} \tab TRUE \tab print information on test of
#'   overall effect \cr
#' \code{test.subgroup} \tab FALSE \tab print information on test for
#'   subgroup differences \cr
#' \code{test.effect.subgroup} \tab FALSE \tab print information on
#'   test for effect in subgroups \cr
#' \code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
#'   forest plots
#'   \cr
#' \code{forest.tau2} \tab FALSE \tab do not show between-study heterogeneity \cr
#'   \tab \tab variance in forest plots \cr
#' \code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity \cr
#'   \tab \tab standard deviation in forest plots \cr
#' \code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
#'   forest plots
#'   \cr
#' \code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
#'   in forest plots
#'   \cr
#' \code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
#'   forest plots
#'   \cr
#' \code{digits.I2} \tab 0 \tab number of digits for I-squared measure
#'   \cr
#' \code{digits.pval} \tab 3 \tab number of digits for p-values \cr
#' \code{CIbracket}, \tab "(" \tab \cr
#' \code{CIseparator} \tab "-" \tab print confidence intervals as
#'   "\code{(.-.)}" \cr
#' \code{zero.pval}, \tab FALSE \tab print p-values with leading zero
#' \cr
#' \code{JAMA.pval}, \tab TRUE \tab round p-values to three digits
#'   (for 0.001 < p \eqn{\le} 0.01) \cr
#'   \tab \tab or two digits (p > 0.01) \cr
#' \code{header.line}, \tab TRUE \tab print header line
#' }
#' 
#' IQWiG, General Methods 5 settings:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{method.random.ci} \tab "HK" \tab Hartung-Knapp method \cr
#' \code{prediction} \tab TRUE \tab Prediction interval \cr
#' }
#' 
#' IQWiG, General Methods 6 settings:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{method.random.ci} \tab "HK" \tab Hartung-Knapp method \cr
#' \code{adhoc.hakn.ci} \tab "IQWiG6" \tab \emph{ad hoc} variance correction \cr
#' \code{method.tau} \tab "PM" \tab Paule-Mandel estimator for
#'   between-study variance \cr
#' \code{prediction} \tab TRUE \tab Prediction interval \cr
#' }
#' 
#' Settings for gene expression data:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{scientific.pval} \tab TRUE \tab Scientific notation for p-values \cr
#' \code{method.tau.ci} \tab FALSE \tab
#'   no confidence interval for between-study \cr
#'  \tab \tab heterogeneity variance \cr
#' }
#'
#' IVhet settings:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{method.common.ci} \tab "IVhet" \tab inverse variance heterogeneity \cr
#' \code{text.common} \tab "IVhet model" \tab  \cr
#' \code{text.w.common} \tab "IVhet" \tab  \cr
#' }
#' 
#' Settings for \bold{meta}, version 4 or below:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{method.tau} \tab "DL" \tab DerSimonian-Laird estimator \cr
#' \code{method.I2} \tab "Q" \tab Use Q to calculate I-squared \cr
#' \code{method.predict} \tab "HTS" \tab prediction interval with \emph{k-2}
#'   degrees \cr
#'   \tab \tab of freedom \cr
#' \code{exact.smd} \tab FALSE \tab Use exact formula for standardised mean \cr
#'   \tab \tab difference (White and Thomas, 2005) \cr
#' \code{text.common} \tab "Fixed effect model" \tab \cr
#' \code{text.w.common} \tab "fixed" \tab \cr
#' \code{warn.deprecated} \tab FALSE \tab Do not print warnings for deprecated
#'   \cr
#'   \tab \tab arguments
#' }
#' 
#' Settings for \bold{meta}, version 7.0-0 or below:
#' \tabular{lll}{
#' \bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
#' \code{method.tau} \tab "REML" \tab REML estimator \cr
#' \code{method.I2} \tab "Q" \tab Use Q to calculate I-squared \cr
#' \code{method.predict} \tab "HTS" \tab prediction interval with \emph{k-2}
#'   degrees \cr
#'   \tab \tab of freedom \cr
#' \code{exact.smd} \tab TRUE \tab Use exact formula for standardised mean \cr
#'   \tab \tab difference (White and Thomas, 2005) \cr
#' \code{text.common} \tab "Common effect model" \tab \cr
#' \code{text.w.common} \tab "common" \tab \cr
#' \code{warn.deprecated} \tab FALSE \tab Do not print warnings for deprecated
#'   \cr
#'   \tab \tab arguments
#' }
#' 
#' A list of all arguments with current settings is printed using the
#' command \code{settings.meta()}.
#' 
#' In order to reset all settings of R package \bold{meta} the command
#' \code{settings.meta(reset = TRUE)} can be used.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{gs}}, \code{\link{forest.meta}},
#'   \code{\link{print.meta}}, \code{\link{labels.meta}}
#'
#' @references
#' Doi SAR, Barendregt JJ, Khan S, Thalib L, Williams GM (2015):
#' Advances in the meta-analysis of heterogeneous clinical trials I:
#' The inverse variance heterogeneity model.
#' \emph{Contemporary Clinical Trials},
#' \bold{45}, 130--8
#' 
#' Henmi M & Copas JB (2010):
#' Confidence intervals for random effects meta-analysis and
#' robustness to publication bias.
#' \emph{Statistics in Medicine},
#' \bold{29}, 2969--83
#' 
#' White IR, Thomas J (2005):
#' Standardized mean differences in individually-randomized and
#' cluster-randomized trials, with applications to meta-analysis.
#' \emph{Clinical Trials},
#' \bold{2}, 141--51
#' 
#' @examples
#' # Get listing of current settings
#' #
#' settings.meta()
#' 
#' # Meta-analyses using default settings
#' #
#' metabin(10, 20, 15, 20)
#' metaprop(4, 20)
#' metabin(10, 20, 15, 20, sm = "RD")
#' metaprop(4, 20, sm = "PLN")
#'
#' # Change summary measure for R functions metabin and metaprop
#' # and store old settings
#' #
#' oldset <- settings.meta(smbin = "RD", smprop = "PLN")
#' #
#' metabin(10, 20, 15, 20)
#' metaprop(4, 20)
#'
#' # Use old settings
#' #
#' settings.meta(oldset)
#' 
#' # Change level used to calculate confidence intervals
#' # (99%-CI for studies, 99.9%-CI for pooled effects)
#' #
#' metagen(1:3, 2:4 / 10, sm = "MD")
#' settings.meta(level = 0.99, level.ma = 0.999)
#' metagen(1:3, 2:4 / 10, sm = "MD")
#' 
#' # Always print a prediction interval
#' #
#' settings.meta(prediction = TRUE)
#' metagen(1:3, 2:4 / 10, sm = "MD")
#' metagen(4:6, 4:2 / 10, sm = "MD")
#' 
#' # Try to set unknown argument results in a warning
#' #
#' try(settings.meta(unknownarg = TRUE))
#' 
#' # Reset to default settings of R package meta
#' #
#' settings.meta("reset")
#' metabin(10, 20, 15, 20)
#' metaprop(4, 20)
#' metagen(1:3, 2:4 / 10, sm = "MD")
#' 
#' # Do not back transform results (e.g. print log odds ratios instead
#' # of odds ratios, print transformed correlations / proportions
#' # instead of correlations / proportions)
#' #
#' settings.meta(backtransf = FALSE)
#' metabin(10, 20, 15, 20)
#' metaprop(4, 20)
#' metacor(c(0.85, 0.7, 0.95), c(20, 40, 10))
#' 
#' # Forest plot using RevMan 5 style
#' #
#' settings.meta("RevMan5")
#' forest(metagen(1:3, 2:4 / 10, sm = "MD", common = FALSE),
#'   label.left = "Favours A", label.right = "Favours B",
#'   colgap.studlab = "2cm", colgap.forest.left = "0.2cm")
#' 
#' # Forest plot using JAMA style
#' #
#' settings.meta("JAMA")
#' forest(metagen(1:3, 2:4 / 10, sm = "MD", common = FALSE),
#'   label.left = "Favours A", label.right = "Favours B",
#'   colgap.studlab = "2cm", colgap.forest.left = "0.2cm")
#'
#' # Use slightly different layout for confidence intervals
#' # (especially useful if upper confidence limit can be negative)
#' #
#' settings.meta(CIseparator = " - ")
#' forest(metagen(-(1:3), 2:4 / 10, sm = "MD", common = FALSE),
#'   label.left = "Favours A", label.right = "Favours B",
#'   colgap.studlab = "2cm", colgap.forest.left = "0.2cm")
#' 
#' # Use old settings
#' #
#' settings.meta(oldset)
#' 
#' @export settings.meta

settings.meta <- function(..., quietly = TRUE) {
  
  #
  # Check argument
  #
  missing.quietly <- missing(quietly)
  chklogical(quietly)
  
  
  #
  # Save object with current settings
  #
  oldset <- .settings
  oldset$argslist <- oldset$argslist.internal <- NULL
  
  
  #
  # Set internal variables
  #
  settings <- c("BMJ", "JAMA", "RevMan5", 
                "IQWiG5", "IQWiG6",
                "geneexpr", "IVhet",
                "meta4", "meta7")
  layouts <- c(settings[1:2], "meta")
  #
  print.settings <- FALSE
  reset.settings <- FALSE
  specific.settings <- FALSE
  #
  args  <- list(...)
  
  
  #
  # Print settings if no argument is provided
  #
  if (length(args) == 0) {
    if (missing.quietly || !quietly)
      settings.meta("print", quietly = FALSE)
    return(invisible(oldset))
  }
  
  
  #
  # Check whether first argument is a list. In this case only use
  # this list as input.
  #
  warn.depr <- TRUE
  if (length(args) > 0 && is.list(args[[1]])) {
    if (!is.null(names(args))) {
      print(names(args))
      warning("Additional arguments ignored as first argument is a list.",
              call. = FALSE)
    }
    warn.depr <- FALSE
    args <- args[[1]]
  }
  
  
  #
  # Unnamed first (and only) argument must be character string or a
  # logical
  #
  if (length(args) == 1 & is.null(names(args))) {
    if (is.character(unlist(args)))
      action <- setchar(unlist(args), c(settings, "reset", "print"),
                        stop.at.error = FALSE)
    else
      action <- unlist(args)
    #
    if (is.null(action))
      stop("First argument can be one of the following character strings:",
           "\n reset, print, ", paste(settings, collapse = ", "),
           call. = FALSE)
    else if (action == "reset")
      settings.meta(reset = TRUE, quietly = quietly)
    else if (action == "print" | (is.logical(action) && action))
      settings.meta(print = TRUE, quietly = FALSE)
    else
      settings.meta(setting = action, quietly = quietly)
    #
    return(invisible(oldset))
  }
  #
  else if (length(args) > 1 & names(args)[1] == "") {
    if (is.character(unlist(args[[1]])))
      action <- setchar(unlist(args[[1]]), c(settings, "reset", "print"),
                        stop.at.error = FALSE)
    else
      action <- unlist(args[[1]])
    #
    if (is.null(action))
      stop("First argument can be one of the following character strings:",
           "\n reset, print, ", paste(settings, collapse = ", "),
           call. = FALSE)
    else if (action == "reset")
      settings.meta(reset = TRUE, quietly = quietly)
    else if (action == "print")
      settings.meta(print = TRUE, quietly = FALSE)
    else
      settings.meta(setting = action, quietly = quietly)
  }
  
  
  #
  # Check and warn about deprecated arguments
  #
  names.all <- names(args)
  #
  if (warn.depr) {
    chkdeprecated(names.all, "level.ma", "level.comb")
    chkdeprecated(names.all, "common", "comb.fixed")
    chkdeprecated(names.all, "common", "fixed")
    chkdeprecated(names.all, "random", "comb.random")
    #
    chkdeprecated(names.all, "method.random.ci", "hakn")
    chkdeprecated(names.all, "adhoc.hakn.ci", "adhoc.hakn")
    #
    chkdeprecated(names.all, "digits.stat", "digits.zval")
    chkdeprecated(names.all, "print.subgroup.name", "print.byvar")
    chkdeprecated(names.all, "sep.subgroup", "byseparator")
    #
    chkdeprecated(names.all, "method.incr", "addincr")
    chkdeprecated(names.all, "method.incr", "allincr")
    #
    chkdeprecated(names.all, "cid.below.null", "lower.equi")
    chkdeprecated(names.all, "cid.above.null", "upper.equi")
    chkdeprecated(names.all, "lty.cid", "lty.equi")
    chkdeprecated(names.all, "col.cid", "col.equi")
  }
  
  
  #
  # Check for new argument names
  #
  newargs <- vector("character", 0)
  #
  for (i in seq_along(names.all)) {
    if (is.null(
      setchar(names.all[i],
              c(.settings$argslist.meta, "reset", "print", "setting", ""),
              stop.at.error = FALSE)))
      newargs <- c(newargs, names.all[i])
  }
  #
  for (i in seq_len(length(newargs))) {
    setOption("argslist", c(.settings$argslist, newargs[i]))
    setOption(newargs[i], args[[newargs[i]]])
  }
  #
  names <- names.all[!(names.all %in% .settings$argslist.internal)]
  
  
  #
  # Check argument names
  #
  if (length(names) != length(unique(names)))
    stop("Arguments must be unique.")
  
  
  #
  # Determine whether to print, reset or use specific settings
  #
  if (any(names == "print") && args[["print"]]) {
    print.settings <- TRUE
    quietly <- FALSE
  }
  if (any(names == "reset") && args[["reset"]])
    reset.settings <- TRUE
  if (any(names == "setting")) {
    setting <- setchar(args[["setting"]], settings)
    specific.settings <- TRUE
  }
  
  
  #
  # Reset settings
  #
  if (reset.settings) {
    if (!quietly)
      cat("\n** Reset all meta-analysis settings (R package meta). **\n\n")
    #
    # General settings
    #
    setOption("level", 0.95)
    setOption("level.ma", 0.95)
    #
    setOption("common", TRUE)
    setOption("fixed", TRUE)
    setOption("comb.fixed", TRUE)
    setOption("method.common.ci", "classic")
    setOption("random", TRUE)
    setOption("comb.random", TRUE)
    setOption("method.random.ci", "classic")
    setOption("hakn", FALSE)
    setOption("adhoc.hakn", "")
    setOption("adhoc.hakn.ci", "")
    setOption("adhoc.hakn.pi", "")
    setOption("method.tau", "REML")
    setOption("method.tau.ci", NULL)
    setOption("level.hetstat", 0.95)
    setOption("tau.common", FALSE)
    setOption("method.I2", "Q")
    setOption("prediction", FALSE)
    setOption("level.predict", 0.95)
    setOption("method.predict", "V")
    setOption("test.subgroup", TRUE)
    setOption("prediction.subgroup", FALSE)
    setOption("print.tau2.ci.subgroup", NULL)
    setOption("print.tau.ci.subgroup", NULL)
    setOption("method.bias", "Egger")
    setOption("tool.rob", NULL)
    setOption("overall.hetstat", NULL)
    setOption("text.common", "Common effect model")
    setOption("text.random", "Random effects model")
    setOption("text.predict", "Prediction interval")
    setOption("text.w.common", "common")
    setOption("text.w.random", "random")
    setOption("title", "")
    setOption("complab", "")
    setOption("CIbracket", "[")
    setOption("CIseparator", "; ")
    setOption("CIlower.blank", TRUE)
    setOption("CIupper.blank", TRUE)
    setOption("print.subgroup.name", TRUE)
    setOption("print.byvar", TRUE)
    setOption("sep.subgroup", " = ")
    setOption("byseparator", " = ")
    setOption("keepdata", TRUE)
    setOption("keeprma", FALSE)
    setOption("warn", TRUE)
    setOption("warn.deprecated", TRUE)
    setOption("transf", TRUE)
    setOption("backtransf", TRUE)
    setOption("digits", 4)
    setOption("digits.mean", 2)
    setOption("digits.sd", 4)
    setOption("digits.se", 4)
    setOption("digits.stat", 2)
    setOption("digits.zval", 2)
    setOption("digits.Q", 2)
    setOption("digits.tau2", 4)
    setOption("digits.tau", 4)
    setOption("digits.H", 2)
    setOption("digits.I2", 1)
    setOption("digits.prop", 4)
    setOption("digits.weight", 1)
    setOption("digits.pval", 4)
    setOption("digits.pval.Q", 4)
    setOption("scientific.pval", FALSE)
    setOption("big.mark", "")
    setOption("zero.pval", TRUE)
    setOption("JAMA.pval", FALSE)
    setOption("digits.df", 4)
    setOption("digits.cid", 4)
    #
    setOption("details", TRUE)
    #
    setOption("print.tau2", TRUE)
    setOption("print.tau2.ci", TRUE)
    setOption("print.tau", TRUE)
    setOption("print.tau.ci", TRUE)
    setOption("print.I2", TRUE)
    setOption("print.I2.ci", TRUE)
    setOption("print.H", TRUE)
    setOption("print.Rb", FALSE)
    #
    setOption("text.tau2", "tau^2")
    setOption("text.tau", "tau")
    setOption("text.I2", "I^2")
    setOption("text.Rb", "Rb")
    #
    setOption("print.Q", TRUE)
    #
    # R function metabin
    #
    setOption("smbin", "RR")
    setOption("method", "MH")
    setOption("incr", 0.5)
    setOption("method.incr", "only0")
    setOption("allincr", FALSE)
    setOption("addincr", FALSE)
    setOption("allstudies", FALSE)
    setOption("MH.exact", FALSE)
    setOption("RR.Cochrane", FALSE)
    setOption("Q.Cochrane", TRUE)
    setOption("model.glmm", "UM.FS")
    setOption("print.CMH", FALSE)
    #
    # R function metacont
    #
    setOption("smcont", "MD")
    setOption("pooledvar", FALSE)
    setOption("method.smd", "Hedges")
    setOption("sd.glass", "control")
    setOption("exact.smd", TRUE)
    setOption("method.ci.cont", "z")
    #
    # R function metaprop
    #
    setOption("smprop", "PLOGIT")
    setOption("method.ci.prop", "CP")
    #
    # R function metarate
    #
    setOption("smrate", "IRLN")
    setOption("method.ci.rate", "NAsm")
    #
    # Other meta-analysis functions
    #
    setOption("smcor", "ZCOR")
    setOption("sminc", "IRR")
    setOption("smmean", "MRAW")
    #
    # R functions comparing two treatments
    #
    setOption("label.e", "Experimental")
    setOption("label.c", "Control")
    setOption("label.left", "")
    setOption("label.right", "")
    #
    # R function forest.meta
    #
    setOption("layout", "meta")
    setOption("forest.details", FALSE)
    setOption("test.overall", FALSE)
    setOption("test.effect.subgroup", FALSE)
    setOption("digits.forest", 2)
    setOption("digits.TE.forest", 4)
    #
    setOption("lty.common", 2)
    setOption("lty.random", 3)
    setOption("col.common", "black")
    setOption("col.random", "black")
    #
    setOption("sort.subgroup", FALSE)
    #
    setOption("pooled.events", FALSE)
    setOption("pooled.times", FALSE)
    setOption("study.results", TRUE)
    #
    setOption("cid", NA)
    setOption("cid.below.null", NA)
    setOption("cid.above.null", NA)
    setOption("lty.cid", 1)
    setOption("col.cid", "blue")
    setOption("fill.cid", "transparent")
    setOption("cid.pooled.only", FALSE)
    #
    setOption("fill", "transparent")
    setOption("fill.equi", "transparent")
    #
    setOption("leftcols", NULL)
    setOption("rightcols", NULL)
    setOption("leftlabs", NULL)
    setOption("rightlabs", NULL)
    #
    setOption("label.e.attach", NULL)
    setOption("label.c.attach", NULL)
    #
    setOption("bottom.lr", TRUE)
    #
    setOption("lab.NA", ".")
    setOption("lab.NA.effect", NULL)
    setOption("lab.NA.weight", ".")
    #
    setOption("lwd", 1)
    setOption("lwd.square", 1)
    setOption("lwd.diamond", 1)
    #
    setOption("arrow.type", "open")
    setOption("arrow.length", 0.05)
    #
    setOption("type.study", "square")
    setOption("type.common", "diamond")
    #
    setOption("col.study", "black")
    setOption("col.square", "gray")
    setOption("col.square.lines", "gray")
    setOption("col.circle", "royalblue")
    setOption("col.inside", "white")
    setOption("col.diamond", "gray")
    setOption("col.diamond.lines", "black")
    setOption("col.predict", "red")
    setOption("col.predict.lines", "black")
    setOption("col.subgroup", "black")
    setOption("col.label.right", "black")
    setOption("col.label.left", "black")
    #
    setOption("col.lines", "black")
    setOption("col.label", "black")
    #
    setOption("hetlab", "Heterogeneity: ")
    setOption("resid.hetstat", NULL)
    setOption("resid.hetlab", "Residual heterogeneity: ")
    #
    setOption("forest.I2", NULL)
    setOption("forest.I2.ci", FALSE)
    setOption("forest.tau2", NULL)
    setOption("forest.tau2.ci", FALSE)
    setOption("forest.tau", FALSE)
    setOption("forest.tau.ci", FALSE)
    setOption("forest.Q", FALSE)
    setOption("forest.pval.Q", NULL)
    setOption("forest.Rb", FALSE)
    setOption("forest.Rb.ci", FALSE)
    #
    setOption("label.n", "Total")
    setOption("label.events", "Events")
    setOption("label.mean", "Mean")
    setOption("label.sd", "SD")
    setOption("label.cor", "Cor")
    setOption("label.time", "Time")
    setOption("label.pval", "P-value")
    setOption("label.tau2", "Tau2")
    setOption("label.tau", "Tau")
    setOption("label.I2", "I2")
    setOption("label.cluster", "Cluster")
    setOption("label.cycles", "Cycles")
    #
    setOption("text.subgroup.nohet", "not applicable")
    #
    setOption("LRT", FALSE)
    #
    setOption("forest.stat", TRUE)
    setOption("forest.Q.subgroup", TRUE)
    #
    setOption("header.line", FALSE)
    #
    setOption("fontsize", 12)
    setOption("fontfamily", NULL)
    setOption("fs.common", NULL)
    setOption("fs.random", NULL)
    setOption("fs.predict", NULL)
    setOption("fs.common.labels", NULL)
    setOption("fs.random.labels", NULL)
    setOption("fs.predict.labels", NULL)
    setOption("fs.hetstat", NULL)
    setOption("fs.test.overall", NULL)
    setOption("fs.test.subgroup", NULL)
    setOption("fs.test.effect.subgroup", NULL)
    setOption("fs.addline", NULL)
    #
    setOption("ff.heading", "bold")
    setOption("ff.common", NULL)
    setOption("ff.random", NULL)
    setOption("ff.predict", NULL)
    setOption("ff.common.labels", NULL)
    setOption("ff.random.labels", NULL)
    setOption("ff.predict.labels", NULL)
    setOption("ff.study", "plain")
    setOption("ff.hetstat", NULL)
    setOption("ff.test.overall", NULL)
    setOption("ff.test.subgroup", NULL)
    setOption("ff.test.effect.subgroup", NULL)
    setOption("ff.addline", NULL)
    setOption("ff.axis", "plain")
    setOption("ff.smlab", "bold")
    setOption("ff.xlab", "plain")
    setOption("ff.lr", "plain")
    #
    setOption("colgap", "2mm")
    setOption("colgap.forest", "2mm")
    #
    setOption("width", NULL)
    #
    setOption("calcwidth.predict", FALSE)
    setOption("calcwidth.hetstat", FALSE)
    setOption("calcwidth.tests", FALSE)
    setOption("calcwidth.subgroup", FALSE)
    setOption("calcwidth.addline", FALSE)
    #
    setOption("just.studlab", "left")
    setOption("just.addcols", "center")
    #
    setOption("spacing", 1)
    setOption("addrow", NULL)
    setOption("addrow.overall", NULL)
    setOption("addrow.subgroups", NULL)
    setOption("addrows.below.overall", NULL)
  }
  
  
  #
  # Specific settings
  #
  if (specific.settings) {
    #
    if (setting == "BMJ") {
      specificSettings(
        args = c("layout", "test.overall",
                 "test.subgroup", "test.effect.subgroup",
                 #
                 "forest.I2", "forest.tau2", "forest.tau",
                 "forest.Q", "forest.pval.Q", "forest.Rb",
                 #
                 "digits.I2", "digits.tau2", "digits.tau", "digits.pval",
                 "digits.forest", "digits.mean", "digits.sd",
                 #
                 "CIbracket", "CIseparator",
                 #
                 "colgap.forest",
                 #
                 "col.common", "col.random", "col.subgroup",
                 "col.study", "col.square", "col.square.lines",
                 "col.diamond", "col.diamond.lines",
                 #
                 "col.lines",
                 #
                 "lwd.square", "lwd.diamond",
                 #
                 "arrow.type",
                 #
                 "ff.lr",
                 "zero.pval", "JAMA.pval",
                 #
                 "hetlab", "header.line"),
        new = list("BMJ",
                   replaceNULL(args[["test.overall"]], TRUE),
                   replaceNULL(args[["test.subgroup"]], TRUE),
                   replaceNULL(args[["test.effect.subgroup"]], TRUE),
                   #
                   replaceNULL(args[["forest.I2"]], TRUE),
                   replaceNULL(args[["forest.tau2"]], TRUE),
                   replaceNULL(args[["forest.tau"]], FALSE),
                   replaceNULL(args[["forest.Q"]], TRUE),
                   replaceNULL(args[["forest.pval.Q"]], TRUE),
                   replaceNULL(args[["forest.Rb"]], FALSE),
                   #
                   replaceNULL(args[["digits.I2"]], 0),
                   replaceNULL(args[["digits.tau2"]], 2),
                   replaceNULL(args[["digits.tau"]], 2),
                   replaceNULL(args[["digits.pval"]], 2),
                   replaceNULL(args[["digits.forest"]], 2),
                   replaceNULL(args[["digits.mean"]], 1),
                   replaceNULL(args[["digits.sd"]], 1),
                   #
                   replaceNULL(args[["CIbracket"]], "("),
                   replaceNULL(args[["CIseparator"]], " to "),
                   #
                   replaceNULL(args[["colgap.forest"]], "5mm"),
                   #
                   replaceNULL(args[["col.common"]], "#6b58a6"),
                   replaceNULL(args[["col.random"]], "#6b58a6"),
                   replaceNULL(args[["col.subgroup"]], "black"),
                   replaceNULL(args[["col.study"]], "#6b58a6"),
                   replaceNULL(args[["col.square"]], "#6b58a6"),
                   replaceNULL(args[["col.square.lines"]], "white"),
                   replaceNULL(args[["col.diamond"]], "#6b58a6"),
                   replaceNULL(args[["col.diamond.lines"]], "white"),
                   #
                   replaceNULL(args[["col.lines"]], "#a7a9ac"),
                   #
                   replaceNULL(args[["lwd.square"]], 0.5),
                   replaceNULL(args[["lwd.diamond"]], 0.5),
                   #
                   replaceNULL(args[["arrow.type"]], "closed"),
                   #
                   replaceNULL(args[["ff.lr"]], "bold"),
                   replaceNULL(args[["zero.pval"]], TRUE),
                   replaceNULL(args[["JAMA.pval"]], TRUE),
                   #
                   replaceNULL(args[["hetlab"]], "Test for heterogeneity: "),
                   replaceNULL(args[["header.line"]], TRUE)),
        setting = "BMJ settings",
        quietly = quietly)
    }
    #
    else if (setting == "JAMA") {
      specificSettings(
        args = c("layout", "test.overall",
                 "test.subgroup", "test.effect.subgroup",
                 #
                 "forest.I2", "forest.tau2", "forest.tau",
                 "forest.Q", "forest.pval.Q", "forest.Rb",
                 #
                 "digits.I2", "digits.pval",
                 "CIbracket", "CIseparator",
                 "zero.pval", "JAMA.pval",
                 "hetlab", "header.line"),
        new = list("JAMA",
                   replaceNULL(args[["test.overall"]], TRUE),
                   replaceNULL(args[["test.subgroup"]], FALSE),
                   replaceNULL(args[["test.effect.subgroup"]], FALSE),
                   #
                   replaceNULL(args[["forest.tau2"]], FALSE),
                   replaceNULL(args[["forest.tau"]], FALSE),
                   replaceNULL(args[["forest.I2"]], TRUE),
                   replaceNULL(args[["forest.Q"]], TRUE),
                   replaceNULL(args[["forest.pval.Q"]], TRUE),
                   replaceNULL(args[["forest.Rb"]], FALSE),
                   #
                   replaceNULL(args[["digits.I2"]], 0),
                   replaceNULL(args[["digits.pval"]], 3),
                   replaceNULL(args[["CIbracket"]], "("),
                   replaceNULL(args[["CIseparator"]], "-"),
                   replaceNULL(args[["zero.pval"]], FALSE),
                   replaceNULL(args[["JAMA.pval"]], TRUE),
                   replaceNULL(args[["hetlab"]], "Heterogeneity: "),
                   replaceNULL(args[["header.line"]], TRUE)),
        setting = "JAMA settings",
        quietly = quietly)
    }
    #
    else if (setting == "RevMan5") {
      specificSettings(
        args = c("method.random.ci", "method.tau", "method.I2",
                 "tau.common",
                 "MH.exact", "RR.Cochrane", "Q.Cochrane",
                 "exact.smd",
                 "layout", "prediction", "test.overall",
                 "test.subgroup", "test.effect.subgroup",
                 "col.subgroup",
                 #
                 "forest.I2", "forest.tau2", "forest.tau",
                 "forest.Q", "forest.pval.Q", "forest.Rb",
                 #
                 "digits", "digits.I2", "digits.tau2", "digits.tau",
                 #
                 "CIbracket", "CIseparator",
                 "zero.pval", "JAMA.pval",
                 "text.common", "text.w.common",
                 "hetlab", "header.line"),
        new = list(replaceNULL(args[["method.random.ci"]], "classic"),
                   replaceNULL(args[["method.tau"]], "DL"),
                   replaceNULL(args[["method.I2"]], "Q"),
                   replaceNULL(args[["tau.common"]], FALSE),
                   replaceNULL(args[["MH.exact"]], FALSE),
                   replaceNULL(args[["RR.Cochrane"]], TRUE),
                   replaceNULL(args[["Q.Cochrane"]], TRUE),
                   replaceNULL(args[["exact.smd"]], FALSE), "RevMan5",
                   replaceNULL(args[["prediction"]], FALSE),
                   replaceNULL(args[["test.overall"]], TRUE),
                   replaceNULL(args[["test.subgroup"]], TRUE),
                   replaceNULL(args[["test.effect.subgroup"]], TRUE),
                   replaceNULL(args[["col.subgroup"]], "black"),
                   #
                   replaceNULL(args[["forest.I2"]], TRUE),
                   replaceNULL(args[["forest.tau2"]], TRUE),
                   replaceNULL(args[["forest.tau"]], FALSE),
                   replaceNULL(args[["forest.Q"]], TRUE),
                   replaceNULL(args[["forest.pval.Q"]], TRUE),
                   replaceNULL(args[["forest.Rb"]], FALSE),
                   #
                   replaceNULL(args[["digits"]], 2),
                   replaceNULL(args[["digits.I2"]], 0),
                   replaceNULL(args[["digits.tau2"]], 3),
                   replaceNULL(args[["digits.tau"]], 4),
                   #
                   replaceNULL(args[["CIbracket"]], "["),
                   replaceNULL(args[["CIseparator"]], ", "),
                   replaceNULL(args[["zero.pval"]], TRUE),
                   replaceNULL(args[["JAMA.pval"]], FALSE),
                   replaceNULL(args[["text.common"]], "Fixed effect model"),
                   replaceNULL(args[["text.w.common"]], "fixed"),
                   replaceNULL(args[["hetlab"]], "Heterogeneity: "),
                   replaceNULL(args[["header.line"]], TRUE)
                   ),
        setting = "RevMan 5 settings",
        quietly = quietly)
    }
    #
    else if (setting == "IQWiG5") {
      specificSettings(args = c("method.random.ci", "prediction"),
                       new = list(replaceNULL(args[["method.random.ci"]],
                                              "HK"),
                                  replaceNULL(args[["prediction"]], TRUE)),
                       setting = "IQWiG 5 settings",
                       quietly = quietly)
    }
    #
    else if (setting == "IQWiG6") {
      specificSettings(args = c("method.random.ci", "adhoc.hakn.ci",
                                "method.tau", "prediction"),
                       new = list(replaceNULL(args[["method.random.ci"]],
                                              "HK"),
                                  replaceNULL(args[["adhoc.hakn.ci"]],
                                              "IQWiG6"),
                                  replaceNULL(args[["method.tau"]], "PM"),
                                  replaceNULL(args[["prediction"]], TRUE)),
                       setting = "IQWiG 6 settings",
                       quietly = quietly)
    }
    #
    else if (setting == "meta4") {
      specificSettings(args = c("method.tau", "method.I2", "method.predict",
                                "exact.smd",
                                "text.common", "text.w.common",
                                "warn.deprecated"),
                       new = list(replaceNULL(args[["method.tau"]], "DL"),
                                  replaceNULL(args[["method.I2"]], "Q"),
                                  replaceNULL(args[["method.predict"]], "HTS"),
                                  replaceNULL(args[["exact.smd"]], FALSE),
                                  replaceNULL(args[["text.common"]],
                                              "Fixed effect model"),
                                  replaceNULL(args[["text.w.common"]],
                                              "fixed"),
                                  replaceNULL(args[["warn.deprecated"]],
                                              FALSE)),
                       setting =
                         "settings from meta, version 4 or below",
                       quietly = quietly)
    }
    #
    else if (setting == "meta7") {
      specificSettings(args = c("method.tau", "method.I2", "method.predict",
                                "exact.smd",
                                "text.common", "text.w.common",
                                "warn.deprecated"),
                       new = list(replaceNULL(args[["method.tau"]], "REML"),
                                  replaceNULL(args[["method.I2"]], "Q"),
                                  replaceNULL(args[["method.predict"]], "HTS"),
                                  replaceNULL(args[["exact.smd"]], TRUE),
                                  replaceNULL(args[["text.common"]],
                                              "Common effect model"),
                                  replaceNULL(args[["text.w.common"]],
                                              "common"),
                                  replaceNULL(args[["warn.deprecated"]],
                                              FALSE)),
                       setting =
                         "settings from meta, version 7.0-0 or below",
                       quietly = quietly)
    }
    #
    else if (setting == "geneexpr") {
      specificSettings(args = c("scientific.pval", "method.tau.ci"),
                       new = list(replaceNULL(args[["scientific.pval"]], TRUE),
                                  replaceNULL(args[["method.tau.ci"]], "")),
                       setting = "Settings for gene expression data",
                       quietly = quietly)
    }
    #
    else if (setting == "IVhet") {
      specificSettings(args = c("method.common.ci",
                                "text.common", "text.w.common"),
                       new =
                         list(replaceNULL(args[["method.common.ci"]], "IVhet"),
                              replaceNULL(args[["text.common"]], "IVhet model"),
                              replaceNULL(args[["text.w.common"]], "IVhet")),
                       setting = "Settings for IVhet model",
                       quietly = quietly)
    }
  }
  
  
  #
  # Print settings
  #
  if (print.settings & !quietly) {
    cat(paste0("\n** Settings for meta-analysis method (R package meta, ",
               "version ", utils::packageDescription("meta")$Version,
               ") **\n\n"))
    #
    cat(paste0("* General settings *\n"))
    catarg("level              ")
    catarg("level.ma           ")
    catarg("common             ")
    catarg("method.common.ci   ")
    catarg("random             ")
    catarg("method.random.ci   ")
    catarg("adhoc.hakn.ci      ")
    catarg("adhoc.hakn.pi      ")
    catarg("method.tau         ")
    catarg("method.tau.ci      ")
    catarg("level.hetstat      ")
    catarg("tau.common         ")
    catarg("method.I2          ")
    catarg("prediction         ")
    catarg("level.predict      ")
    catarg("method.predict     ")
    catarg("test.subgroup      ")
    catarg("prediction.subgroup")
    catarg("method.bias        ")
    catarg("tool.rob           ")
    catarg("overall.hetstat    ")
    catarg("cid                ")
    catarg("cid.below.null     ")
    catarg("cid.above.null     ")
    catarg("text.common        ")
    catarg("text.random        ")
    catarg("text.predict       ")
    catarg("text.w.common      ")
    catarg("text.w.random      ")
    catarg("title              ")
    catarg("complab            ")
    catarg("CIbracket          ")
    catarg("CIseparator        ")
    catarg("CIlower.blank      ")
    catarg("CIupper.blank      ")
    catarg("print.subgroup.name")
    catarg("sep.subgroup       ")
    catarg("keepdata           ")
    catarg("keeprma            ")
    catarg("warn               ")
    catarg("warn.deprecated    ")
    catarg("backtransf         ")
    catarg("digits             ")
    catarg("digits.mean        ")
    catarg("digits.sd          ")
    catarg("digits.se          ")
    catarg("digits.stat        ")
    catarg("digits.Q           ")
    catarg("digits.tau2        ")
    catarg("digits.tau         ")
    catarg("digits.H           ")
    catarg("digits.I2          ")
    catarg("digits.prop        ")
    catarg("digits.weight      ")
    catarg("digits.pval        ")
    catarg("digits.pval.Q      ")
    catarg("scientific.pval    ")
    catarg("big.mark           ")
    catarg("zero.pval          ")
    catarg("JAMA.pval          ")
    catarg("digits.df          ")
    catarg("digits.cid         ")
    catarg("details            ")
    catarg("print.tau2         ")
    catarg("print.tau2.ci      ")
    catarg("print.tau          ")
    catarg("print.tau.ci       ")
    catarg("print.I2           ")
    catarg("print.I2.ci        ")
    catarg("print.H            ")
    catarg("print.Rb           ")
    catarg("text.tau2          ")
    catarg("text.tau           ")
    catarg("text.I2            ")
    catarg("text.Rb            ")
    catarg("print.Q            ")
    #
    cat(paste("\n* Default summary measure (argument 'sm' in",
               "corresponding function) *\n"))
    cat("- metabin():  ")
    catarg("smbin ", newline = FALSE)
    cat("- metacont(): ")
    catarg("smcont", newline = FALSE)
    cat("- metacor():  ")
    catarg("smcor ", newline = FALSE)
    cat("- metainc():  ")
    catarg("sminc ", newline = FALSE)
    cat("- metamean(): ")
    catarg("smmean", newline = FALSE)
    cat("- metaprop(): ")
    catarg("smprop", newline = FALSE)
    cat("- metarate(): ")
    catarg("smrate", newline = FALSE)
    #
    cat(paste("\n* Additional settings for metabin(), metainc(),",
              "metaprop(), and metarate() *\n"))
    catarg("incr       ")
    catarg("method.incr")
    #catarg("allincr")
    #catarg("addincr")
    #
    cat("\n* Additional settings for metabin() *\n")
    catarg("method     ")
    catarg("allstudies ")
    catarg("MH.exact   ")
    catarg("RR.Cochrane")
    catarg("Q.Cochrane ")
    catarg("model.glmm ")
    catarg("print.CMH  ")
    #
    cat("\n* Additional settings for metacont() *\n")
    catarg("pooledvar ")
    catarg("method.smd")
    catarg("sd.glass  ")
    catarg("exact.smd ")
    catarg("method.ci.cont")
    #
    cat("\n* Additional settings for metagen() *\n")
    catarg("transf ")
    #
    cat("\n* Additional setting for metaprop() *\n")
    catarg("method.ci.prop")
    #
    cat("\n* Additional setting for metarate() *\n")
    catarg("method.ci.rate")
    #
    cat("\n* Settings for R functions comparing two treatments *\n")
    catarg("label.e    ")
    catarg("label.c    ")
    catarg("label.left ")
    catarg("label.right")
    #
    cat("\n* Settings for forest.meta() *\n")
    catarg("layout                 ")
    catarg("forest.details         ")
    catarg("test.overall           ")
    catarg("test.effect.subgroup   ")
    catarg("digits.forest          ",
           end = "\n  (argument 'digits' in forest.meta())")
    catarg("digits.TE.forest          ",
           end = "\n  (argument 'digits.TE' in forest.meta())")
    #
    catarg("lty.common             ")
    catarg("lty.random             ")
    catarg("col.common             ")
    catarg("col.random             ")
    #
    catarg("sort.subgroup          ")
    #
    catarg("pooled.events          ")
    catarg("pooled.times           ")
    catarg("study.results          ")
    #
    catarg("lty.cid                ")
    catarg("col.cid                ")
    catarg("fill.cid               ")
    catarg("cid.pooled.only        ")
    catarg("fill.equi              ")
    #
    catarg("fill                   ")
    #
    catarg("print.tau2.ci.subgroup ")
    catarg("print.tau.ci.subgroup  ")
    #
    catarg("leftcols               ")
    catarg("rightcols              ")
    catarg("leftlabs               ")
    catarg("rightlabs              ")
    #
    catarg("label.e.attach         ")
    catarg("label.c.attach         ")
    #
    catarg("bottom.lr              ")
    #
    catarg("lab.NA                 ")
    catarg("lab.NA.effect          ")
    catarg("lab.NA.weight          ")
    #
    catarg("lwd                    ")
    catarg("lwd.square             ")
    catarg("lwd.diamond            ")
    #
    catarg("arrow.type             ")
    catarg("arrow.length           ")
    #
    catarg("type.study             ")
    catarg("type.common            ")
    #
    catarg("col.study              ")
    catarg("col.square             ")
    catarg("col.square.lines       ")
    catarg("col.circle             ")
    catarg("col.inside             ")
    catarg("col.diamond            ")
    catarg("col.diamond.lines      ")
    catarg("col.predict            ")
    catarg("col.predict.lines      ")
    catarg("col.subgroup           ")
    catarg("col.label.right        ")
    catarg("col.label.left         ")
    #
    catarg("col.lines              ")
    catarg("col.label              ")
    #
    catarg("hetlab                 ")
    catarg("resid.hetstat          ")
    catarg("resid.hetlab           ")
    #
    catarg("forest.I2              ")
    catarg("forest.I2.ci           ")
    catarg("forest.tau2            ")
    catarg("forest.tau2.ci         ")
    catarg("forest.tau             ")
    catarg("forest.tau.ci          ")
    catarg("forest.Q               ")
    catarg("forest.pval.Q          ")
    catarg("forest.Rb              ")
    catarg("forest.Rb.ci           ")
    #
    catarg("label.n                ")
    catarg("label.events           ")
    catarg("label.mean             ")
    catarg("label.sd               ")
    catarg("label.cor              ")
    catarg("label.time             ")
    catarg("label.pval             ")
    catarg("label.tau2             ")
    catarg("label.tau              ")
    catarg("label.I2               ")
    catarg("label.cluster          ")
    catarg("label.cycles           ")
    #
    catarg("text.subgroup.nohet    ")
    #
    catarg("LRT                    ")
    #
    catarg("forest.stat            ")
    catarg("forest.Q.subgroup      ")
    #
    catarg("header.line            ")
    #
    catarg("fontsize               ")
    catarg("fontfamily             ")
    catarg("fs.common              ")
    catarg("fs.random              ")
    catarg("fs.predict             ")
    catarg("fs.common.labels       ")
    catarg("fs.random.labels       ")
    catarg("fs.predict.labels      ")
    catarg("fs.hetstat             ")
    catarg("fs.test.overall        ")
    catarg("fs.test.subgroup       ")
    catarg("fs.test.effect.subgroup")
    catarg("fs.addline             ")
    #
    catarg("ff.heading             ")
    catarg("ff.common              ")
    catarg("ff.random              ")
    catarg("ff.predict             ")
    catarg("ff.common.labels       ")
    catarg("ff.random.labels       ")
    catarg("ff.predict.labels      ")
    catarg("ff.study               ")
    catarg("ff.hetstat             ")
    catarg("ff.test.overall        ")
    catarg("ff.test.subgroup       ")
    catarg("ff.test.effect.subgroup")
    catarg("ff.addline             ")
    catarg("ff.axis                ")
    catarg("ff.smlab               ")
    catarg("ff.xlab                ")
    catarg("ff.lr                  ")
    #
    catarg("colgap                 ")
    catarg("colgap.forest          ")
    #
    catarg("width                  ")
    #
    catarg("calcwidth.predict      ")
    catarg("calcwidth.hetstat      ")
    catarg("calcwidth.tests        ")
    catarg("calcwidth.subgroup     ")
    catarg("calcwidth.addline      ")
    #
    catarg("just.studlab           ")
    catarg("just.addcols           ")
    #
    catarg("spacing                ")
    catarg("addrow                 ")
    catarg("addrow.overall         ")
    catarg("addrow.subgroups       ")
    catarg("addrows.below.overall  ")
  }
  
  
  #
  # Set individual settings
  #
  if (any(!(names %in% c("reset", "print", "setting")))) {
    #
    # General settings
    #
    setlevel("level", args)
    #
    setOptionDepr(args, "level.ma", "level.comb", setlevel)
    #
    setOptionDepr(args, "common", "comb.fixed", setlogical)
    setOptionDepr(args, "common", "fixed", setlogical)
    #
    setOptionDepr(args, "random", "comb.random", setlogical)
    #
    setcharacter("method.common.ci", args, gs("meth4common.ci"))
    #
    na <- is.na(setcharacter("method.random.ci", args, gs("meth4random.ci")))
    depr <- setlogical("hakn", args)
    if (na & !is.na(depr)) {
      if (gs("hakn"))
        setOption("method.random.ci", "HK")
      else
        setOption("method.random.ci", "classic")
    }
    #
    setOptionDepr(args, "adhoc.hakn.ci", "adhoc.hakn", setcharacter,
                  set = gs("adhoc4hakn.ci"))
    #
    setcharacter("adhoc.hakn.pi", args, gs("adhoc4hakn.pi"))
    setcharacter("method.tau", args, gs("meth4tau"))
    setcharacter("method.tau.ci", args, c("J", "BJ", "QP", "PL", ""))
    setlevel("level.hetstat", args)
    setlogical("tau.common", args)
    setcharacter("method.I2", args, gs("meth4i2"))
    setlogical("prediction", args)
    setlevel("level.predict", args)
    setcharacter("method.predict", args, gs("meth4pi"))
    setcharacter("method.bias", args, gs("meth4bias"))
    setcharacter("tool.rob", args, gs("tool4rob"), NULL.ok = TRUE)
    setlogical("overall.hetstat", args, NULL.ok = TRUE)
    setcharacter("text.common", args)
    setcharacter("text.random", args)
    setcharacter("text.predict", args)
    setcharacter("text.w.common", args)
    setcharacter("text.w.random", args)
    setcharacter("title", args)
    setcharacter("complab", args)
    setcharacter("CIbracket", args, c("[", "(", "{", ""))
    setcharacter("CIseparator", args)
    setlogical("CIlower.blank", args)
    setlogical("CIupper.blank", args)
    #
    setOptionDepr(args, "print.subgroup.name", "print.byvar", setlogical)
    #
    setOptionDepr(args, "sep.subgroup", "byseparator", setcharacter)
    #
    setlogical("keepdata", args)
    setlogical("keeprma", args)
    setlogical("warn", args)
    setlogical("warn.deprecated", args)
    setlogical("backtransf", args)
    setnumeric("digits", args)
    setnumeric("digits.mean", args)
    setnumeric("digits.sd", args)
    setnumeric("digits.se", args)
    #
    setOptionDepr(args, "digits.stat", "digits.zval", setnumeric)
    #
    setnumeric("digits.Q", args) 
    setnumeric("digits.tau2", args)
    setnumeric("digits.tau", args)
    setnumeric("digits.H", args) 
    setnumeric("digits.I2", args)
    setnumeric("digits.prop", args)
    setnumeric("digits.weight", args)
    setnumeric("digits.pval", args)
    setnumeric("digits.pval.Q", args)
    setlogical("scientific.pval", args)
    setcharacter("big.mark", args)
    setlogical("zero.pval", args)
    setlogical("JAMA.pval", args)
    setnumeric("digits.df", args)
    setnumeric("digits.cid", args)
    #
    setlogical("details", args)
    setlogical("print.tau2", args)
    setlogical("print.tau2.ci", args)
    setlogical("print.tau", args)
    setlogical("print.tau.ci", args)
    setlogical("print.I2", args)
    setlogical("print.I2.ci", args)
    setlogical("print.H", args)
    setlogical("print.Rb", args)
    #
    setcharacter("text.tau2", args)
    setcharacter("text.tau", args)
    setcharacter("text.I2", args)
    setcharacter("text.Rb", args)
    #
    setlogical("print.Q", args)
    #
    setnumeric("cid", args)
    setOptionDepr(args, "cid.below.null", "lower.equi", setnumeric)
    setOptionDepr(args, "cid.above.null", "upper.equi", setnumeric)
    setOptionDepr(args, "lty.cid", "lty.equi", setnumeric)
    setOptionDepr(args, "col.cid", "col.equi", setcolor)
    setcolor("fill.cid", args)
    setcolor("fill.equi", args)
    setlogical("cid.pooled.only", args)
    #
    # R function metabin
    #
    setcharacter("smbin", args, gs("sm4bin"))
    setcharacter("method", args, gs("meth4bin"))
    setcharacter("model.glmm", args, c("UM.FS", "UM.RS", "CM.EL", "CM.AL"))
    idincr <- argid(names.all, "incr")
    if (!is.na(idincr)) {
      incr <- args[[idincr]]
      if (!is.numeric(incr))
        incr <- setchar(incr, "TACC",
                        "should be numeric or the character string \"TACC\"")
      setOption("incr", incr)
    }
    #
    na <- is.na(setcharacter("method.incr", args, gs("meth4incr")))
    depr1 <- setlogical("allincr", args)
    depr2 <- setlogical("addincr", args)
    if (na & !is.na(depr1) & gs("allincr"))
      setOption("method.incr", "if0all")
    if (na & !is.na(depr2) & gs("addincr"))
      setOption("method.incr", "all")
    #
    setlogical("allstudies", args)
    setlogical("MH.exact", args)
    setlogical("RR.Cochrane", args)
    setlogical("Q.Cochrane", args)
    setlogical("print.CMH", args)
    #
    # R function metacont
    #
    setcharacter("smcont", args, gs("sm4cont"))
    setlogical("pooledvar", args)
    setcharacter("method.smd", args, c("Hedges", "Cohen", "Glass"))
    setcharacter("sd.glass", args, c("control", "experimental"))
    setlogical("exact.smd", args)
    setcharacter("method.ci.cont", args, gs("ci4cont"))
    #
    # R function metagen
    #
    setlogical("transf", args)
    #
    # R function metaprop
    #
    setcharacter("smprop", args, gs("sm4prop"))
    setcharacter("method.ci.prop", args, gs("ci4prop"))
    #
    # R function metarate
    #
    setcharacter("smrate", args, gs("sm4rate"))
    setcharacter("method.ci.rate", args, gs("ci4rate"))
    #
    # Other meta-analysis functions
    #
    setcharacter("smcor" , args, gs("sm4cor"))
    setcharacter("sminc" , args, gs("sm4inc"))
    setcharacter("smmean", args, gs("sm4mean"))
    #
    # R functions comparing two treatments
    #
    setcharacter("label.e", args)
    setcharacter("label.c", args)
    setcharacter("label.left", args)
    setcharacter("label.right", args)
    #
    # R function forest.meta
    #
    setcharacter("layout", args, layouts)
    setlogical("forest.details", args)
    setlogical("test.overall", args)
    setlogical("test.subgroup", args)
    setlogical("prediction.subgroup", args)
    setlogical("print.tau2.ci.subgroup", args, NULL.ok = TRUE)
    setlogical("print.tau.ci.subgroup", args, NULL.ok = TRUE)
    setlogical("test.effect.subgroup", args)
    setnumeric("digits.forest", args)
    setnumeric("digits.TE.forest", args)
    #
    setnumeric("lty.common", args)
    setnumeric("lty.random", args)
    setcolor("col.common", args)
    setcolor("col.random", args)
    #
    setlogical("sort.subgroup", args)
    #
    setlogical("pooled.events", args)
    setlogical("pooled.times", args)
    setlogical("study.results", args)
    #
    setcolor("fill", args)
    #
    setcharacter("leftcols", args, length = 0,
                 NULL.ok = TRUE, logical.ok = TRUE)
    setcharacter("rightcols", args, length = 0,
                 NULL.ok = TRUE, logical.ok = TRUE)
    setcharacter("leftlabs", args, length = 0, NULL.ok = TRUE)
    setcharacter("rightlabs", args, length = 0, NULL.ok = TRUE)
    #
    setcharacter("label.e.attach", args, NULL.ok = TRUE)
    setcharacter("label.c.attach", args, NULL.ok = TRUE)
    #
    setlogical("bottom.lr", args)
    #
    setcharacter("lab.NA", args)
    setcharacter("lab.NA.effect", args, NULL.ok = TRUE)
    setcharacter("lab.NA.weight", args)
    #
    setnumeric("lwd", args)
    setnumeric("lwd.square", args)
    setnumeric("lwd.diamond", args)
    #
    setcharacter("arrow.type", args)
    setnumeric("arrow.length", args)
    #
    setcharacter("type.study", args, c("square", "diamond", "predict"))
    setcharacter("type.common", args, c("square", "diamond", "predict"))
    #
    setcolor("col.study", args)
    setcolor("col.square", args)
    setcolor("col.square.lines", args)
    setcolor("col.circle", args)
    setcolor("col.inside", args)
    setcolor("col.diamond", args)
    setcolor("col.diamond.lines", args)
    setcolor("col.predict", args)
    setcolor("col.predict.lines", args)
    setcolor("col.subgroup", args)
    setcolor("col.label.right", args)
    setcolor("col.label.left", args)
    #
    setcolor("col.lines", args)
    setcolor("col.label", args)
    #
    setcharacter("hetlab", args)
    setlogical("resid.hetstat", args, TRUE)
    setcharacter("resid.hetlab", args)
    #
    setlogical("forest.I2", args, TRUE)
    setlogical("forest.I2.ci", args)
    setlogical("forest.tau2", args, TRUE)
    setlogical("forest.tau2.ci", args)
    setlogical("forest.tau", args)
    setlogical("forest.tau.ci", args)
    setlogical("forest.Q", args)
    setlogical("forest.pval.Q", args, TRUE)
    setlogical("forest.Rb", args)
    setlogical("forest.Rb.ci", args)
    #
    setcharacter("label.n", args)
    setcharacter("label.events", args)
    setcharacter("label.mean", args)
    setcharacter("label.sd", args)
    setcharacter("label.cor", args)
    setcharacter("label.time", args)
    setcharacter("label.pval", args)
    setcharacter("label.tau2", args)
    setcharacter("label.tau", args)
    setcharacter("label.I2", args)
    setcharacter("label.cluster", args)
    setcharacter("label.cycles", args)
    #
    setcharacter("text.subgroup.nohet", args)
    #
    setlogical("LRT", args)
    #
    setlogical("forest.stat", args)
    setlogical("forest.Q.subgroup", args)
    #
    setlogical("header.line", args, ignore.other = TRUE)
    setcharacter("header.line", args, c("both", "below", ""),
                 ignore.other = TRUE)
    #
    setnumeric("fontsize", args)
    setcharacter("fontfamily", args, NULL.ok = TRUE)
    setnumeric("fs.common", args, TRUE)
    setnumeric("fs.random", args, TRUE)
    setnumeric("fs.predict", args, TRUE)
    setnumeric("fs.common.labels", args, TRUE)
    setnumeric("fs.random.labels", args, TRUE)
    setnumeric("fs.predict.labels", args, TRUE)
    setnumeric("fs.hetstat", args, TRUE)
    setnumeric("fs.test.overall", args, TRUE)
    setnumeric("fs.test.subgroup", args, TRUE)
    setnumeric("fs.test.effect.subgroup", args, TRUE)
    setnumeric("fs.addline", args, TRUE)
    #
    setcharacter("ff.heading", args)
    setcharacter("ff.common", args, NULL.ok = TRUE)
    setcharacter("ff.random", args, NULL.ok = TRUE)
    setcharacter("ff.predict", args, NULL.ok = TRUE)
    setcharacter("ff.common.labels", args, NULL.ok = TRUE)
    setcharacter("ff.random.labels", args, NULL.ok = TRUE)
    setcharacter("ff.predict.labels", args, NULL.ok = TRUE)
    setcharacter("ff.study", "plain")
    setcharacter("ff.hetstat", args, NULL.ok = TRUE)
    setcharacter("ff.test.overall", args, NULL.ok = TRUE)
    setcharacter("ff.test.subgroup", args, NULL.ok = TRUE)
    setcharacter("ff.test.effect.subgroup", args, NULL.ok = TRUE)
    setcharacter("ff.addline", args, NULL.ok = TRUE)
    setcharacter("ff.axis", args)
    setcharacter("ff.smlab", args)
    setcharacter("ff.xlab", args)
    setcharacter("ff.lr", args)
    #
    setcharacter("colgap", args)
    setcharacter("colgap.forest", args)
    #
    setnumeric("width", args, TRUE)
    #
    setlogical("calcwidth.predict", args)
    setlogical("calcwidth.hetstat", args)
    setlogical("calcwidth.tests", args)
    setlogical("calcwidth.subgroup", args)
    setlogical("calcwidth.addline", args)
    #
    setcharacter("just.studlab", args, c("right", "center", "left"))
    setcharacter("just.addcols", args, c("right", "center", "left"))
    #
    setnumeric("spacing", args)
    setlogical("addrow", args, TRUE)
    setlogical("addrow.overall", args, TRUE)
    setlogical("addrow.subgroups", args, TRUE)
    setnumeric("addrows.below.overall", args)
  }
  
  
  if (any(gs("method.random.ci") == "KR") & gs("method.tau") != "REML")
    warning("Default settings for arguments 'method.tau' and ",
            "'method.random.ci' do not match:\n",
            "- Kenward-Roger method (method.random.ci = \"KR\") ",
            "can only be used with\n  ",
            "REML estimator (method.tau = \"REML\").",
            call. = FALSE)
  #
  if (any(gs("method.predict") == "KR") & gs("method.tau") != "REML")
    warning("Default settings for arguments 'method.tau' and ",
            "'method.predict' do not match:\n",
            "- Kenward-Roger method (method.predict = \"KR\") ",
            "can only be used with\n  ",
            "REML estimator (method.tau = \"REML\").",
            call. = FALSE)
  #
  if (any(gs("method.predict") == "KR-PR") & gs("method.tau") != "REML")
    warning("Default settings for arguments 'method.tau' and ",
            "'method.predict' do not match:\n",
            "- Kenward-Roger method (method.predict = \"KR-PR\") ",
            "can only be used with\n  ",
            "REML estimator (method.tau = \"REML\").",
            call. = FALSE)
  
  
  invisible(oldset)
}

mean_sd_iqr <- function(n, median, q1, q3, method.mean = "Luo") {
  
  #
  # Check arguments
  #
  if (missing(n))
    stop("Mandatory argument 'n' missing.", call. = FALSE)
  if (missing(median))
    stop("Mandatory argument 'median' missing.", call. = FALSE)
  if (missing(q1))
    stop("Mandatory argument 'q1' missing.", call. = FALSE)
  if (missing(q3))
    stop("Mandatory argument 'q3' missing.", call. = FALSE)
  #
  chknumeric(n, min = 0, zero = TRUE)
  #
  k <- length(n)
  arg <- "n"
  chklength(median, k, arg)
  chklength(q1, k, arg)
  chklength(q3, k, arg)
  #
  if (any(median < q1, na.rm = TRUE))
    stop("Median must be larger than first quartile.", call. = FALSE)
  if (any(median > q3, na.rm = TRUE))
    stop("Median must be smaller than third quartile.", call. = FALSE)
  if (any(q1 >= q3, na.rm = TRUE))
    stop("First quartile must be smaller than third quartile.", call. = FALSE)
  
  
  #
  # Estimation of mean
  #
  if (method.mean == "Luo") {
    # Luo et al. (2018), equation (15)
    mean <-
      (0.7 + 0.39 / n) * (q1 + q3) / 2 +
      (0.3 - 0.39 / n) * median
  }
  else if (method.mean == "Wan") {
    # Wan et al. (2014), equation (14)
    mean <- (q1 + median + q3) / 3
  }
  else if (method.mean == "Cai") {
    # Cai et al. (2021)
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::mln.mean.sd(q1.val = q1[i],
                               med.val = median[i],
                               q3.val = q3[i],
                               n = n[i])$est.mean
  }
  else if (method.mean == "QE-McGrath") {
    # McGrath et al. (2020), QE method
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::qe.mean.sd(q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              n = n[i])$est.mean
  }
  else if (method.mean == "BC-McGrath") {
    # McGrath et al. (2020), BC method
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::bc.mean.sd(q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              n = n[i])$est.mean
  }
  else
    mean <- NA
  
  
  #
  # Estimation of standard deviation
  # Wan et al. (2014), equations (15) and (16)
  #
  method.sd <- method.mean
  #
  if (method.sd %in% c("Luo", "Wan")) {
    sd <- (q3 - q1) /
      ifelse(n > 201, 2 * qnorm((0.75 * n - 0.125) / (n + 0.25)),
             gs("Wan2014.Table2")[ceiling(0.25 * (n - 1))])
  }
  else if (method.sd == "Cai") {
    # Cai et al. (2021)
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::mln.mean.sd(q1.val = q1[i],
                               med.val = median[i],
                               q3.val = q3[i],
                               n = n[i])$est.sd
  }
  else if (method.sd == "QE-McGrath") {
    # McGrath et al. (2020), QE method
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::qe.mean.sd(q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              n = n[i])$est.sd
  }
  else if (method.sd == "BC-McGrath") {
    # McGrath et al. (2020), BC method
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::bc.mean.sd(q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              n = n[i])$est.sd
  }

  
  #
  # Calculation of standard error
  #
  se <- sd / sqrt(n)
  
  
  res <- list(mean = mean, sd = sd, se = se,
              median = median, q1 = q1, q3 = q3, n = n,
              method.mean = method.mean)
  #
  res
}


mean_sd_iqr_range <- function(n, median, q1, q3, min, max,
                              method.mean = "Luo", method.sd = "Shi") {
  
  
  #
  # Check arguments
  #
  if (missing(n))
    stop("Mandatory argument 'n' missing.", call. = FALSE)
  if (missing(median))
    stop("Mandatory argument 'median' missing.", call. = FALSE)
  if (missing(q1))
    stop("Mandatory argument 'q1' missing.", call. = FALSE)
  if (missing(q3))
    stop("Mandatory argument 'q3' missing.", call. = FALSE)
  if (missing(min))
    stop("Mandatory argument 'min' missing.", call. = FALSE)
  if (missing(max))
    stop("Mandatory argument 'max' missing.", call. = FALSE)
  #
  chknumeric(n, min = 0, zero = TRUE)
  #
  k <- length(n)
  arg <- "n"
  chklength(median, k, arg)
  chklength(q1, k, arg)
  chklength(q3, k, arg)
  chklength(min, k, arg)
  chklength(max, k, arg)
  #
  if (any(median < q1, na.rm = TRUE))
    stop("Median must be larger than first quartile.", call. = FALSE)
  if (any(median > q3, na.rm = TRUE))
    stop("Median must be smaller than third quartile.", call. = FALSE)
  if (any(q1 >= q3, na.rm = TRUE))
    stop("First quartile must be smaller than third quartile.", call. = FALSE)
  #
  if (any(median < min, na.rm = TRUE))
    stop("Median must be larger than minumum.", call. = FALSE)
  if (any(median > max, na.rm = TRUE))
    stop("Median must be smaller than maximum.", call. = FALSE)
  if (any(min >= max, na.rm = TRUE))
    stop("Minimum must be smaller than maximum.", call. = FALSE)
  #
  if (any(q1 < min, na.rm = TRUE))
    stop("First quartile must be larger than minumum.", call. = FALSE)
  if (any(q3 > max, na.rm = TRUE))
    stop("Third quartile must be smaller than maximum.", call. = FALSE)
  
  
  #
  # Estimation of mean
  #
  if (method.mean == "Luo") {
    # Luo et al. (2018), equation (15)
    mean <-
      2.2 / (2.2 + n^0.75) * (min + max) / 2 +
      (0.7 - 0.72 / n^0.55) * (q1 + q3) / 2 +
      (0.3 + 0.72 / n^0.55 - 2.2 / (2.2 + n^0.75)) * median
  }
  else if (method.mean == "Wan") {
    # Wan et al. (2014), equation (10)
    mean <- (min + 2 * q1 + 2 * median + 2 * q3 + max) / 8
  }
  else if (method.mean == "Cai") {
    # Cai et al. (2021)
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::mln.mean.sd(min.val = min[i],
                               q1.val = q1[i],
                               med.val = median[i],
                               q3.val = q3[i],
                               max.val = max[i],
                               n = n[i])$est.mean
  }
  else if (method.mean == "QE-McGrath") {
    # McGrath et al. (2020), QE method
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::qe.mean.sd(min.val = min[i],
                              q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              max.val = max[i],
                              n = n[i])$est.mean
  }
  else if (method.mean == "BC-McGrath") {
    # McGrath et al. (2020), BC method
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::bc.mean.sd(min.val = min[i],
                              q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              max.val = max[i],
                              n = n[i])$est.mean
  }
  else
    mean <- rep_len(NA, k)
  
  
  #
  # Estimation of standard deviation
  #
  if (method.sd == "Shi") {
    # Shi et al. (2020), equation (11)
    theta1 <- (2 + 0.14 * n^0.6) * qnorm((n - 0.375) / (n + 0.25))
    theta2 <- (2 + 2 / (0.07 * n^0.6)) * qnorm((0.75 * n - 0.125) / (n + 0.25))
    sd <- (max - min) / theta1 + (q3 - q1) / theta2
  }
  else if (method.sd == "Wan") {
    # Wan et al. (2014), equations (12) and (13)
    #
    sd <- 0.5 * (
      (max - min) / ifelse(n > 50, 2 * qnorm((n - 0.375) / (n + 0.25)),
                           gs("Wan2014.Table1")[n]) +
      (q3 - q1) / ifelse(n > 201, 2 * qnorm((0.75 * n - 0.125) / (n + 0.25)),
                         gs("Wan2014.Table2")[ceiling(0.25 * (n - 1))])
    )
  }
  else if (method.sd == "Cai") {
    # Cai et al. (2021)
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::mln.mean.sd(min.val = min[i],
                               q1.val = q1[i],
                               med.val = median[i],
                               q3.val = q3[i],
                               max.val = max[i],
                               n = n[i])$est.sd
  }
  else if (method.sd == "QE-McGrath") {
    # McGrath et al. (2020), QE method
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::qe.mean.sd(min.val = min[i],
                              q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              max.val = max[i],
                              n = n[i])$est.sd
  }
  else if (method.sd == "BC-McGrath") {
    # McGrath et al. (2020), BC method
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::bc.mean.sd(min.val = min[i],
                              q1.val = q1[i],
                              med.val = median[i],
                              q3.val = q3[i],
                              max.val = max[i],
                              n = n[i])$est.sd
  }
  else
    sd <- rep_len(NA, k)
  
  
  #
  # Calculation of standard error
  #
  se <- sd / sqrt(n)
  
  
  res <- list(mean = mean, sd = sd, se = se,
              median = median, q1 = q1, q3 = q3, min = min, max = max, n = n,
              method.mean = method.mean)
  #
  res
}


mean_sd_range <- function(n, median, min, max, method.mean = "Luo") {
  
  
  #
  # Check arguments
  #
  if (missing(n))
    stop("Mandatory argument 'n' missing.", call. = FALSE)
  if (missing(median))
    stop("Mandatory argument 'median' missing.", call. = FALSE)
  if (missing(min))
    stop("Mandatory argument 'min' missing.", call. = FALSE)
  if (missing(max))
    stop("Mandatory argument 'max' missing.", call. = FALSE)
  #
  chknumeric(n, min = 0, zero = TRUE)
  #
  k <- length(n)
  #
  fun <- "TE.seTE.range"
  chklength(median, k, fun,
            text = "Arguments 'n' and 'median' must have the same length.")
  chklength(min, k, fun,
            text = "Arguments 'n' and 'min' must have the same length.")
  chklength(max, k, fun,
            text = "Arguments 'n' and 'max' must have the same length.")
  #
  if (any(median < min, na.rm = TRUE))
    stop("Median must be larger than minumum.", call. = FALSE)
  if (any(median > max, na.rm = TRUE))
    stop("Median must be smaller than maximum.", call. = FALSE)
  if (any(min >= max, na.rm = TRUE))
    stop("Minimum must be smaller than maximum.", call. = FALSE)
  
  
  #
  # Estimation of mean
  #
  if (method.mean == "Luo") {
    # Luo et al. (2018), equation (15)
    mean <-
      4 / (4 + n^0.75) * (min + max) / 2 +
      n^0.75 / (4 + n^0.75) * median
  }
  else if (method.mean == "Wan") {
    # Wan et al. (2014), equation (2)
    mean <- (min + 2 * median + max) / 4 +
      ifelse(is.na(n), 0, (min - 2 * median + max) / (4 * n))
  }
  else if (method.mean == "Cai") {
    # Cai et al. (2021)
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::mln.mean.sd(min.val = min[i],
                               med.val = median[i],
                               max.val = max[i],
                               n = n[i])$est.mean
  }
  else if (method.mean == "QE-McGrath") {
    # McGrath et al. (2020), QE method
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::qe.mean.sd(min.val = min[i],
                              med.val = median[i],
                              max.val = max[i],
                              n = n[i])$est.mean
  }
  else if (method.mean == "BC-McGrath") {
    # McGrath et al. (2020), BC method
    mean <- vector("numeric", k)
    for (i in seq_len(k))
      mean[i] <-
        estmeansd::bc.mean.sd(min.val = min[i],
                              med.val = median[i],
                              max.val = max[i],
                              n = n[i])$est.mean
  }
  else
    mean <- rep_len(NA, k)
  
  
  #
  # Estimation of standard deviation
  # Wan et al. (2014), equations (7) and (9)
  #
  method.sd <- method.mean
  #
  if (method.sd %in% c("Luo", "Wan")) {
    sd <- (max - min) /
      ifelse(n > 50, 2 * qnorm((n - 0.375) / (n + 0.25)),
             gs("Wan2014.Table1")[n])
  }
  else if (method.sd == "Cai") {
    # Cai et al. (2021)
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::mln.mean.sd(min.val = min[i],
                               med.val = median[i],
                               max.val = max[i],
                               n = n[i])$est.sd
  }
  else if (method.sd == "QE-McGrath") {
    # McGrath et al. (2020), QE method
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::qe.mean.sd(min.val = min[i],
                              med.val = median[i],
                              max.val = max[i],
                              n = n[i])$est.sd
  }
  else if (method.sd == "BC-McGrath") {
    # McGrath et al. (2020), BC method
    sd <- vector("numeric", k)
    for (i in seq_len(k))
      sd[i] <-
        estmeansd::bc.mean.sd(min.val = min[i],
                              med.val = median[i],
                              max.val = max[i],
                              n = n[i])$est.sd
  }
  
  
  #
  # Calculation of standard error
  #
  se <- sd / sqrt(n)
  
  
  res <- list(mean = mean, sd = sd, se = se,
              median = median, min = min, max = max, n = n,
              method.mean = method.mean)
  #
  res
}

#' Update a meta-analysis object
#' 
#' @description
#' Update an existing meta-analysis object.
#' 
#' @param object An object of class \code{meta}.
#' @param data Dataset.
#' @param subset Subset.
#' @param studlab Study label.
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots.
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param cycles A numeric vector with the number of cycles per patient / study
#'   in n-of-1 trials (see \code{\link{metagen}}).
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param method A character string indicating which method is to be
#'   used for pooling of studies (see \code{\link{metabin}},
#'   \code{\link{metainc}}, \code{\link{metaprop}} and
#'   \code{\link{metarate}}).
#' @param sm A character string indicating which summary measure is
#'   used for pooling.
#' @param incr Information on increment added to cell frequencies of
#'   studies with zero cell counts (see \code{\link{metabin}},
#'   \code{\link{metainc}}, \code{\link{metaprop}} and
#'   \code{\link{metarate}}).
#' @param method.incr A character string indicating which continuity
#'   correction method should be used (see \code{\link{metabin}},
#'   \code{\link{metainc}}, \code{\link{metaprop}} and
#'   \code{\link{metarate}}).
#' @param allstudies A logical indicating if studies with zero or all
#'   events in both groups are to be included in the meta-analysis
#'   (applies only to \code{\link{metabin}} object with \code{sm}
#'   equal to \code{"RR"} or \code{"OR"}).
#' @param incr.e Continuity correction in experimental group (see
#'   \code{\link{metabin}} and \code{\link{metainc}}).
#' @param incr.c Continuity correction in control group (see
#'   \code{\link{metabin}} and \code{\link{metainc}}).
#' @param incr.event Continuity correction (see
#'   \code{\link{metaprop}} and \code{\link{metarate}}).
#' @param MH.exact A logical indicating if \code{incr} is not to be
#'   added to all cell frequencies for studies with a zero cell count
#'   to calculate the pooled estimate based on the Mantel-Haenszel
#'   method (applies only to \code{\link{metabin}} object).
#' @param RR.Cochrane A logical indicating which method to use as
#'   continuity correction for the risk ratio (see
#'   \code{\link{metabin}}).
#' @param Q.Cochrane A logical indicating which method to use to
#'   calculate the heterogeneity statistic Q (see
#'   \code{\link{metabin}}).
#' @param model.glmm A character string indicating which GLMM model
#'   should be used (see \code{\link{metabin}} and
#'   \code{\link{metainc}}).
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param null.effect A numeric value specifying the effect under the
#'   null hypothesis.
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param method.bias A character string indicating which test for
#'   funnel plot asymmetry is to be used, can be abbreviated. See
#'   function \code{\link{metabias}.}
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and plots. If \code{backtransf =
#'   TRUE}, results for \code{sm = "OR"} are printed as odds ratios
#'   rather than log odds ratios and results for \code{sm = "ZCOR"}
#'   are printed as correlations rather than Fisher's z transformed
#'   correlations, for example.
#' @param func.backtransf A function used to back-transform results
#'   (see \code{\link{metagen}}).
#' @param args.backtransf An optional list to provide additional
#'   arguments to \code{func.backtransf} (see \code{\link{metagen}}).
#' @param pscale A numeric giving scaling factor for printing of
#'   single event probabilities or risk differences, i.e. if argument
#'   \code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
#'   \code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.
#' @param irscale A numeric defining a scaling factor for printing of
#'   single incidence rates or incidence rate differences, i.e. if
#'   argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
#'   \code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.
#' @param irunit A character specifying the time unit used to
#'   calculate rates, e.g. person-years.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.e Label for experimental group.
#' @param label.c Label for control group.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param n.e Number of observations in experimental group (only for
#'   \code{\link{metagen}} or \code{\link{metainc}} object).
#' @param n.c Number of observations in control group (only for
#'   \code{\link{metagen}} or \code{\link{metainc}} object).
#' @param method.mean A character string indicating which method to
#'   use to approximate the mean from the median and other statistics
#'   (see \code{\link{metacont}} and \code{\link{metamean}}).
#' @param method.sd A character string indicating which method to use
#'   to approximate the standard deviation from sample size, median,
#'   interquartile range and range (see \code{\link{metacont}} and
#'   \code{\link{metamean}}).
#' @param approx.mean.e Approximation method to estimate means in
#'   experimental group (see \code{\link{metacont}}).
#' @param approx.mean.c Approximation method to estimate means in
#'   control group (see \code{\link{metacont}}).
#' @param approx.sd.e Approximation method to estimate standard
#'   deviations in experimental group (see \code{\link{metacont}}).
#' @param approx.sd.c Approximation method to estimate standard
#'   deviations in control group (see \code{\link{metacont}}).
#' @param approx.mean Approximation method to estimate means (see
#'   \code{\link{metamean}}).
#' @param approx.sd Approximation method to estimate standard
#'   deviations (see \code{\link{metamean}}).
#' @param approx.TE Approximation method to estimate treatment
#'   estimate (see \code{\link{metagen}}).
#' @param approx.seTE Approximation method to estimate standard error
#'   (see \code{\link{metagen}}).
#' @param pooledvar A logical indicating if a pooled variance should
#'   be used for the mean difference or ratio of means (see
#'   \code{\link{metacont}}).
#' @param method.smd A character string indicating which method is
#'   used to estimate the standardised mean difference (see
#'   \code{\link{metacont}}).
#' @param sd.glass A character string indicating which standard
#'   deviation is used in the denominator for Glass' method to
#'   estimate the standardised mean difference (only for metacont
#'   object with \code{sm = "SMD"}). Either \code{"control"} using the
#'   standard deviation in the control group (\code{sd.c}) or
#'   \code{"experimental"} using the standard deviation in the
#'   experimental group (\code{sd.e}), can be abbreviated.
#' @param exact.smd A logical indicating whether exact formulae should
#'   be used in estimation of the standardised mean difference and its
#'   standard error.
#' @param method.ci A character string indicating which method is used
#'   to calculate confidence intervals for individual studies. Either
#'   \code{"z"}, \code{"t"}, \code{"WS"}, \code{"WSCC"}, \code{"AC"},
#'   \code{"SA"}, \code{"SACC"}, \code{"NAsm"}, or \code{"Poisson"},
#'   can be abbreviated. See functions \code{\link{metacont}},
#'   \code{\link{metaprop}} and \code{\link{metarate}}.
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param id Deprecated argument (replaced by 'cluster').
#' @param print.CMH A logical indicating whether result of the
#'   Cochran-Mantel-Haenszel test for overall effect should be
#'   printed.
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param left A logical indicating whether studies are supposed to be
#'   missing on the left or right side of the funnel plot. If NULL,
#'   the linear regression test for funnel plot symmetry (i.e.,
#'   function \code{metabias(..., method = "linreg")}) is used to
#'   determine whether studies are missing on the left or right side.
#' @param ma.common A logical indicating whether a common effect or
#'   random effects model is used to estimate the number of missing
#'   studies.
#' @param type A character indicating which method is used to estimate
#'   the number of missing studies. Either \code{"L"} or \code{"R"}.
#' @param n.iter.max Maximum number of iterations to estimate number
#'   of missing studies.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if \code{incr} is added to studies with zero cell
#'   frequencies).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param verbose A logical indicating whether to print information on
#'   updates of older meta versions.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}} or
#'   \code{\link[metafor]{rma.glmm}}, respectively.
#' @param \dots Additional arguments (ignored at the moment).
#' 
#' @details
#' Wrapper function to update an existing meta-analysis object which
#' was created with R function \code{\link{metabin}},
#' \code{\link{metacont}}, \code{\link{metacor}},
#' \code{\link{metagen}}, \code{\link{metainc}},
#' \code{\link{metamean}}, \code{\link{metaprop}}, or
#' \code{\link{metarate}}. More details on function arguments are
#' available in help files of respective R functions.
#' 
#' This function can also be used for objects of class 'trimfill',
#' 'metacum', and 'metainf'.
#' 
#' @return
#' An object of class \code{"meta"} and \code{"metabin"},
#' \code{"metacont"}, \code{"metacor"}, \code{"metainc"},
#' \code{"metagen"}, \code{"metamean"}, \code{"metaprop"}, or
#' \code{"metarate"} (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metacont}},
#'   \code{\link{metacor}}, \code{\link{metagen}},
#'   \code{\link{metainc}}, \code{\link{metamean}},
#'   \code{\link{metaprop}}, \code{\link{metarate}}
#' 
#' @examples
#' data(Fleiss1993cont)
#' ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, studlab = paste(study, year), sm = "SMD")
#' ma1
#' 
#' # Change summary measure (from 'SMD' to 'MD')
#' #
#' update(ma1, sm = "MD")
#' 
#' # Restrict analysis to subset of studies
#' #
#' update(ma1, subset = 1:2)
#' 
#' # Use different levels for confidence intervals
#' #
#' ma2 <- update(ma1, level = 0.66, level.ma = 0.99)
#' print(ma2, digits = 2)
#' forest(ma2)
#' 
#' @method update meta
#' @export

update.meta <- function(object, 
                        data = object$data,
                        subset, studlab, exclude, cluster,
                        rho = object$rho,
                        cycles,
                        #
                        weights = NULL,
                        weights.common = object$weights.common,
                        weights.random = object$weights.random,
                        #
                        method,
                        sm = object$sm,
                        incr = object$incr,
                        method.incr = object$method.incr,
                        allstudies = object$allstudies,
                        #
                        incr.e, incr.c, incr.event,
                        #
                        MH.exact = object$MH.exact,
                        RR.Cochrane = object$RR.Cochrane,
                        Q.Cochrane = object$Q.Cochrane,
                        model.glmm = object$model.glmm,
                        #
                        level = object$level,
                        level.ma = object$level.ma,
                        #
                        common = object$common,
                        random = object$random,
                        overall = object$overall,
                        overall.hetstat = object$overall.hetstat,
                        method.common.ci = object$method.common.ci,
                        method.random.ci = object$method.random.ci,
                        adhoc.hakn.ci = object$adhoc.hakn.ci,
                        method.predict = object$method.predict,
                        adhoc.hakn.pi = object$adhoc.hakn.pi,
                        seed.predict = object$seed.predict,
                        method.tau = object$method.tau,
                        method.tau.ci = object$method.tau.ci,
                        level.hetstat = object$level.hetstat,
                        tau.preset = object$tau.preset,
                        TE.tau = object$TE.tau,
                        tau.common = object$tau.common,
                        detail.tau = object$detail.tau,
                        #
                        method.I2 = object$method.I2,
                        #
                        prediction = object$prediction,
                        level.predict = object$level.predict,
                        null.effect = object$null.effect,
                        method.bias = object$method.bias,
                        #
                        backtransf = object$backtransf,
                        func.backtransf = object$func.backtransf,
                        args.backtransf = object$args.backtransf,    
                        #
                        pscale = object$pscale,
                        irscale = object$irscale,
                        irunit = object$irunit,
                        #
                        text.common = object$text.common,
                        text.random = object$text.random,
                        text.predict = object$text.predict,
                        text.w.common = object$text.w.common,
                        text.w.random = object$text.w.random,
                        #
                        title = object$title,
                        complab = object$complab,
                        outclab = object$outclab,
                        #
                        label.e = object$label.e,
                        label.c = object$label.c,
                        label.left = object$label.left,
                        label.right = object$label.right,
                        col.label.left = object$col.label.left,
                        col.label.right = object$col.label.right,
                        #
                        n.e = object$n.e,
                        n.c = object$n.c,
                        #
                        method.mean = object$method.mean,
                        method.sd = object$method.sd,
                        #
                        approx.mean.e = object$approx.mean.e,
                        approx.mean.c = object$approx.mean.c,
                        approx.sd.e = object$approx.sd.e,
                        approx.sd.c = object$approx.sd.c,
                        #
                        approx.mean = object$approx.mean,
                        approx.sd = object$approx.sd,
                        #
                        approx.TE = object$approx.TE,
                        approx.seTE = object$approx.seTE,
                        #
                        pooledvar = object$pooledvar,
                        method.smd = object$method.smd,
                        sd.glass = object$sd.glass,
                        exact.smd = object$exact.smd,
                        method.ci = object$method.ci,
                        #
                        subgroup,
                        subgroup.name = object$subgroup.name,
                        print.subgroup.name = object$print.subgroup.name,
                        sep.subgroup = object$sep.subgroup,
                        test.subgroup = object$test.subgroup,
                        prediction.subgroup = object$prediction.subgroup,
                        seed.predict.subgroup = object$seed.predict.subgroup,
                        #
                        byvar, id,
                        #
                        print.CMH = object$print.CMH,
                        keepdata = TRUE,
                        #
                        left = object$left,
                        ma.common = object$ma.common,
                        type = object$type,
                        n.iter.max = object$n.iter.max,
                        #
                        warn = FALSE, warn.deprecated = gs("warn.deprecated"),
                        verbose = FALSE,
                        #
                        control = object$control,
                        ...) {
  
  
  #
  #
  # (1) Check for meta object and update older meta objects
  #
  #
  chkclass(object, "meta")
  suitable <-
    chksuitable(object, "Update",
                c("metabind", "metaadd", "metamerge", "metamiss"),
                check.mlm = FALSE,
                stop = FALSE, status = "possible")
  if (!suitable)
    return(object)
  #
  metabin  <- inherits(object, "metabin")
  metacont <- inherits(object, "metacont")
  metacor  <- inherits(object, "metacor")
  metagen  <- inherits(object, "metagen")
  metainc  <- inherits(object, "metainc")
  metamean <- inherits(object, "metamean")
  metaprop <- inherits(object, "metaprop")
  metarate <- inherits(object, "metarate")
  #
  missing.method.common.ci <- missing(method.common.ci)
  missing.method.random.ci <- missing(method.random.ci)
  missing.adhoc.hakn.ci <- missing(adhoc.hakn.ci)
  missing.text.random <- missing(text.random)
  #
  missing.method.predict <- missing(method.predict)
  missing.adhoc.hakn.pi <- missing(adhoc.hakn.pi)
  missing.text.predict <- missing(text.predict)
  #
  missing.method.bias <- missing(method.bias)
  missing.method.incr <- missing(method.incr)
  #
  chklogical(verbose)
  #
  if (update_needed(object$version, 3, 2, verbose)) {
    #
    # Changes for meta objects with version < 3.2
    #
    object$subset <- NULL
    #
    object$data <- data.frame(.studlab = object$studlab,
                              .exclude = rep_len(FALSE,
                                                 length(object$studlab)))
    #
    if (!is.null(object$byvar))
      object$data$.byvar <- object$byvar
    #
    if (metabin) {
      object$data$.event.e <- object$event.e
      object$data$.n.e <- object$n.e
      object$data$.event.c <- object$event.c
      object$data$.n.c <- object$n.c
    }
    #
    if (metacont) {
      object$data$.n.e <- object$n.e
      object$data$.mean.e <- object$mean.e
      object$data$.sd.e <- object$sd.e
      object$data$.n.c <- object$n.c
      object$data$.mean.c <- object$mean.c
      object$data$.sd.c <- object$sd.c
    }
    #
    if (metacor) {
      object$data$.cor <- object$cor
      object$data$.n <- object$n
    }
    #
    if (metagen) {
      object$data$.TE <- object$TE
      object$data$.seTE <- object$seTE
    }
    #
    if (metaprop) {
      object$data$.event <- object$event
      object$data$.n <- object$n
    }
  }
  #
  if (update_needed(object$version, 4, 8, verbose)) {
    #
    # Changes for meta objects with version < 4.8
    #
    if (metabin | metainc | metaprop | metarate)
      object$data$.incr <- object$incr
    #
    if (metabin | metainc)
      if (object$method == "MH")
        object$k.MH <- sum(object$w.fixed > 0)
      else
        object$k.MH <- NA
  }
  #
  if (update_needed(object$version, 5, 0, verbose)) {
    #
    # Changes for meta objects with version < 5.0
    #
    object$fixed <- object$comb.fixed
    object$random <- object$comb.random
    object$level.ma <- object$level.comb
    #
    object$detail.tau <- ""
    #
    if (!is.null(object$byvar)) {
      object$data$.subgroup <- object$byvar
      object$subgroup.name <- object$bylab
      object$print.subgroup.name <- object$print.byvar
      object$sep.subgroup <- object$byseparator
    }
  }
  #
  if (update_needed(object$version, 5, 5, verbose)) {
    #
    # Changes for meta objects with version < 5.5
    #
    object$common <- object$fixed
    object$w.common <- object$w.fixed
    #
    object$TE.common <- object$TE.fixed
    object$seTE.common <- object$seTE.fixed
    object$lower.common <- object$lower.fixed
    object$upper.common <- object$upper.fixed
    object$statistic.common <- object$statistic.fixed
    object$pval.common <- object$pval.fixed
    object$zval.common <- object$zval.fixed
    #
    object$text.common <- object$text.fixed
    object$text.w.common <- object$text.w.fixed
    #
    if (!is.null(object$pooled) && object$pooled == "fixed")
      object$pooled <- "common"
    #
    if (!is.null(object$byvar)) {
      object$TE.common.w <- object$TE.fixed.w
      object$seTE.common.w <- object$seTE.fixed.w
      object$lower.common.w <- object$lower.fixed.w
      object$upper.common.w <- object$upper.fixed.w
      object$statistic.common.w <- object$statistic.fixed.w
      object$pval.common.w <- object$pval.fixed.w
      object$zval.common.w <- object$zval.fixed.w
      object$w.common.w <- object$w.fixed.w
      #
      object$Q.w.common <- object$Q.w.fixed
      object$pval.Q.w.common <- object$pval.Q.w.fixed
      object$Q.b.common <- object$Q.b.fixed
      object$pval.Q.b.common <- object$pval.Q.b.fixed
    }
    #
    method.incr <- gs("method.incr")
    if (is.logical(object$addincr) && object$addincr)
      method.incr <- "all"
    else if (is.logical(object$allincr) && object$allincr)
      method.incr <- "if0all"
    #
    if (!is.null(object$id))
      object$cluster <- object$id
      object$data$.cluster <- object$data$.id
  }
  #
  if (update_needed(object$version, 6, 0, verbose)) {
    #
    # Changes for meta objects with version < 6.0
    #
    object$method.random.ci <- if (object$hakn) "HK" else "classic"
    object$adhoc.hakn.ci <- object$adhoc.hakn
    object$df.random <- object$df.hakn
    object$seTE.hakn.ci <- object$seTE.hakn
    object$seTE.hakn.adhoc.ci <- object$seTE.hakn.adhoc
    #
    object$method.predict <- "HTS"
    object$adhoc.hakn.pi <- ""
    object$df.predict <- object$k - 2
    object$seTE.hakn.pi <- NA
    object$seTE.hakn.adhoc.pi <- NA
    #
    object$seTE.kero <- NA
    object$df.kero <- NA
    #
    if (!is.null(object$subgroup)) {
      object$bylevs <- object$subgroup.levels
      object$df.random.w <- object$df.hakn.w
      object$df.predict.w <- object$k.w - 2
      #
      object$seTE.hakn.ci.w <-
        object$seTE.hakn.adhoc.ci.w <-
          object$seTE.hakn.pi.w <-
            object$seTE.hakn.adhoc.pi.w <- NA
      #
      object$df.Q.b.random <- object$df.Q.b.common <- object$df.Q.b
    }
  }
  #
  if (update_needed(object$version, 6, 1, verbose)) {
    #
    # Changes for meta objects with version < 6.1
    #
    object$transf <- TRUE
  }
  #
  if (update_needed(object$version, 6, 5, verbose)) {
    #
    # Changes for meta objects with version < 6.5
    #
    if (length(object$TE.random) == 1 & length(object$lower.random) > 1) {
      object$TE.random <- rep(object$TE.random, length(object$lower.random))
      names(object$TE.random) <- names(object$lower.random)
    }
    #
    if (length(object$seTE.random) == 1 & length(object$lower.random) > 1) {
      object$seTE.random <- rep(object$seTE.random, length(object$lower.random))
      names(object$seTE.random) <- names(object$lower.random)
    }
    #
    object$method.random <- object$method
    object$method.random[object$method.random %in% c("MH", "Cochran")] <-
      "Inverse"
    #
    if (!is.null(object$Q.LRT)) {
      object$Q <- c(object$Q, object$Q.LRT)
      object$df.Q <- c(object$df.Q, object$df.Q.LRT)
      object$pval.Q <- c(object$pval.Q, object$pval.Q.LRT)
      names(object$Q) <- c("Wald", "LRT")
    }
    #
    if (metacont) {
      object$data$.approx.mean.e <-
        setVal(object$data, ".approx.mean.e", object$approx.mean.e)
      object$data$.approx.mean.c <-
        setVal(object$data, ".approx.mean.c", object$approx.mean.c)
      object$data$.approx.sd.e <-
        setVal(object$data, ".approx.sd.e", object$approx.sd.e)
      object$data$.approx.sd.c <-
        setVal(object$data, ".approx.sd.c", object$approx.sd.c)
    }
    #
    if (metamean) {
      object$data$.approx.mean <-
        setVal(object$data, ".approx.mean", object$approx.mean)
      object$data$.approx.sd <-
        setVal(object$data, ".approx.sd", object$approx.sd)
    }
    #
    object$hetlabel <- object$label
    #
    if (length(object$tau) > 1)
      names(object$tau) <- object$detail.tau
    if (length(object$tau2) > 1)
      names(object$tau2) <- object$detail.tau
    if (length(object$I2) > 1)
      names(object$I2) <- object$detail.tau
    #
    object$seed.predict <- NULL
    if (!is.null(object$byvar))
      object$seed.predict.subgroup <- NULL
  }
  #
  if (update_needed(object$version, 7, 0, verbose)) {
    #
    # Changes for meta objects with version < 7.0
    #
    object$rho <- 0
    #
    if (inherits(object, c("metacum", "metainf"))) {
      object$label.e <- replaceNULL(object$label.e, "")
      object$label.c <- replaceNULL(object$label.c, "")
    }
    #
    if (inherits(object, "metaprop") && object$method.ci != "NAsm") {
      if (object$sm == "PLOGIT") {
        object$lower <- p2logit(object$lower)
        object$upper <- p2logit(object$upper)
      }
      #
      else if (object$sm == "PAS") {
        object$lower <- p2asin(object$lower)
        object$upper <- p2asin(object$upper)
      }
      #
      else if (object$sm == "PFT") {
        lower.ev <- object$n * object$lower 
        upper.ev <- object$n * object$upper 
        #
        object$lower <-
          0.5 * (asin(sqrt(lower.ev / object$n)) +
                 asin(sqrt((lower.ev + 1) / object$n)))
        object$upper <-
          0.5 * (asin(sqrt(upper.ev / object$n)) +
                 asin(sqrt((upper.ev + 1) / object$n)))
      }
      #
      else if (object$sm == "PLN") {
        object$lower <- log(object$lower)
        object$upper <- log(object$upper)
      }
    }
    #
    if (inherits(object, "metarate") && object$method.ci != "NAsm") {
      if (object$sm == "IRLN") {
        object$lower <- log(object$lower)
        object$upper <- log(object$upper)
      }
      else if (object$sm == "IRS") {
        object$lower <- sqrt(object$lower)
        object$upper <- sqrt(object$upper)
      }
      #
      else if (object$sm == "IRFT") {
        lower.ev <- object$time * object$lower 
        upper.ev <- object$time * object$upper 
        #
        object$lower <-
          0.5 * (sqrt(lower.ev / object$time) +
                 sqrt((lower.ev + 1) / object$time))
        object$upper <-
          0.5 * (sqrt(upper.ev / object$time) +
                 sqrt((upper.ev + 1) / object$time))
      }
      #
      if (inherits(object, "metabind")) {
        object$with.subgroups <- any(object$is.subgroup)
        #
        if (object$with.subgroups) {
          object$data$Q.b.common <- object$data$Q.b
          object$data$Q.b.random <- object$data$Q.b
          #
          object$data$pval.Q.b.common <- object$data$pval.Q.b
          object$data$pval.Q.b.random <- object$data$pval.Q.b
        }
      }
    }
    #
    object$seTE.kero <-
      replaceNA(object$seTE.kero, kenwardroger(object$w.random)$se)
    object$df.kero <-
      replaceNA(object$df.kero, kenwardroger(object$w.random)$df)
  }
  #
  if (update_needed(object$version, 8, 0, verbose)) {
    #
    # Changes for meta objects with version < 8.0
    #
    object$method.I2 <- "Q"
    #
    object$col.label.left <- replaceNULL(object$col.label.left, "black")
    object$col.label.right <- replaceNULL(object$col.label.left, "black")
    #
    object$level.hetstat <- object$level.ma
  }
  #
  if (update_needed(object$version, 8, 1, verbose)) {
    #
    # Changes for meta objects with version < 8.1
    #
    object$pairwise <- FALSE
    #
    if (object$keepdata) {
      if (inherits(object, c("metabin", "metainc"))) {
        if (isCol(object$data, ".subset")) {
          object$incr.e <- object$data$.incr[object$data$.subset]
          object$incr.c <- object$data$.incr[object$data$.subset]
        }
        else {
          object$incr.e <- object$data$.incr
          object$incr.c <- object$data$.incr
        }
        #
        object$data$.incr.e <- object$data$.incr
        object$data$.incr.c <- object$data$.incr
        object$data$.incr <- NULL
      }
      #
      if (inherits(object, "metainc"))
        object$incr.event <- NULL
      #
      if (inherits(object, c("metaprop", "metarate"))) {
        object$data$.incr.event <- object$data$.incr
        object$data$.incr <- NULL
      }
    }
  }
  #
  if (update_needed(object$version, 8, 2, verbose)) {
    #
    # Changes for meta objects with version < 8.2
    #
    object$method.common.ci <- "classic"
  }
  
  
  #
  #
  # (2) Check arguments
  #
  #
  if (missing(method))
    method <- object$method
  #
  pscale <- replaceNULL(pscale, 1)
  irscale <- replaceNULL(irscale, 1)
  irunit <- replaceNULL(irunit, "")
  #
  tau.common <- replaceNULL(tau.common, gs("tau.common"))
  method.I2 <- replaceNULL(method.I2, gs("method.I2"))
  sep.subgroup <- replaceNULL(sep.subgroup, gs("sep.subgroup"))
  #
  if (!backtransf & pscale != 1 & !is_untransformed(sm)) {
    warning("Argument 'pscale' set to 1 as argument 'backtransf' is FALSE.")
    pscale <- 1
  }
  if (!backtransf & irscale != 1 & !is_untransformed(sm)) {
    warning("Argument 'irscale' set to 1 as argument 'backtransf' is FALSE.")
    irscale <- 1
  }
  #
  # Check for deprecated arguments in '...'
  #
  args <- list(...)
  nam.args <- names(args)
  #
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing(level.ma), args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                      warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                      warn.deprecated)
  chklogical(common)
  #
  missing.random <- missing(random)
  random <- deprecated(random, missing.random, args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  method.random.ci <-
    deprecated(method.random.ci, missing.method.random.ci,
               args, "hakn", warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  missing.subgroup.name <- missing(subgroup.name)
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing(print.subgroup.name),
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing(sep.subgroup), args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  test.subgroup <- replaceNULL(test.subgroup, gs("test.subgroup"))
  prediction.subgroup <-
    replaceNULL(prediction.subgroup, gs("prediction.subgroup"))
  #
  missing.method.incr <- missing(method.incr)
  #
  # Ignore deprecated arguments 'addincr' and 'allincr'
  #
  txt.ignore <- "(deprecated); use argument 'method.incr'"
  #
  addincr <- allincr <- NULL
  if (!is.na(charmatch("addincr", nam.args)))
    warn_ignore_input(addincr, TRUE, txt.ignore)
  if (!is.na(charmatch("allincr", nam.args)))
    warn_ignore_input(allincr, TRUE, txt.ignore)
  #
  # Some more checks
  #
  overall <- replaceNULL(overall, common | random)
  overall.hetstat <- replaceNULL(overall.hetstat, common | random)
  chklogical(overall)
  chklogical(overall.hetstat)
  
  
  #
  #
  # (3) Update trim-and-fill object
  #
  #
  if (inherits(object, "trimfill")) {
    #
    rmfilled <- function(x) {
      #
      if (!is.null(object[[x]]))
        res <- object[[x]][!object$trimfill]
      else
        res <- NULL
      #
      res
    }
    #
    tfnames <- c("TE", "seTE",
                 "studlab",
                 "n.e", "n.c",
                 "event.e", "event.c",
                 "mean.e", "mean.c", "sd.e", "sd.c",
                 "n", "event", "cor")
    #
    for (i in tfnames)
      object[[i]] <- rmfilled(i)
    #
    oldclass <- object$class.x
    #
    object$class <- object$class[object$class != "trimfill"]
    #
    res <- trimfill(object,
                    left = left, ma.common = ma.common,
                    type = type, n.iter.max = n.iter.max,
                    level = level, level.ma = level.ma,
                    common = common, random = random,
                    method.common.ci = method.common.ci,
                    method.random.ci = method.random.ci,
                    adhoc.hakn.ci = adhoc.hakn.ci,
                    method.tau = method.tau, method.tau.ci = method.tau.ci,
                    level.hetstat = level.hetstat,
                    prediction = prediction | !missing.method.predict,
                    level.predict = level.predict,
                    silent = TRUE,
                    ...)
    #
    res$call.object <- object$call
    res$call <- match.call()
    res$class.x <- oldclass
    #
    return(res)
  }
  
  
  #
  #
  # (4) Update metacum or metainf object
  #
  #
  if (inherits(object, "metacum") | inherits(object, "metainf")) {
    #
    res <- object
    #
    res$common <- ifelse(res$pooled == "common", TRUE, FALSE)
    res$random <- ifelse(res$pooled == "random", TRUE, FALSE)
    #
    res$call.object <- object$call
    res$call <- match.call()
    res$version <- packageDescription("meta")$Version
    #
    return(res)
  }
  #
  if (is.null(object$data)) {
    warning("Necessary data not available. Please, recreate ",
            "meta-analysis object without option 'keepdata = FALSE'.")
    return(invisible(NULL))
  }
  
  
  #
  #
  # (5) Catch variables
  #
  #
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  # Catch argument 'subset'
  #
  if (!missing(subset))
    subset <- catch("subset", mc, data, sfsp)
  else
    subset <- catch2(object, "subset", fromobject = TRUE)
  #
  # Catch argument 'studlab'
  #
  if (!missing(studlab))
    studlab <- catch("studlab", mc, data, sfsp)
  else
    studlab <- catch2(object, "studlab")
  #
  # Catch argument 'exclude'
  #
  if (!missing(exclude))
    exclude <- catch("exclude", mc, data, sfsp)
  else
    exclude <- catch2(object, "exclude")
  #
  # Catch argument 'cluster'
  #
  missing.cluster <- missing(cluster)
  missing.id <- missing(id)
  #
  if (!missing.cluster | !missing.id) {
    cluster <- catch("cluster", mc, data, sfsp)
    id <- catch("id", mc, data, sfsp)
    ...cluster <-
      deprecated2(cluster, missing.cluster, id, missing.id,
                  warn.deprecated)
    #
    data$.cluster <- ...cluster
  }
  else
    ...cluster <- catch2(object, "cluster")
  #
  # Catch argument 'cycles'
  #
  if (!missing(cycles))
    cycles <- catch("cycles", mc, data, sfsp)
  else {
    if (isCol(object$data, ".cycles"))
      cycles <- catch(".cycles", object$data, object$data, sfsp)
    else
      cycles <- NULL
  }
  #
  avail.cycles <- !is.null(cycles)
  #
  # Catch argument 'incr'
  #
  missing.incr <- missing(incr)
  #
  # Catch argument 'incr.e'
  #
  missing.incr.e <- missing(incr.e)
  #
  if (!missing.incr.e)
    incr.e <- catch("incr.e", mc, data, sfsp)
  else
    incr.e <- catch2(object, "incr.e", gs("incr"))
  #
  avail.incr.e <- !missing.incr.e & !is.null(incr.e)
  #
  # Catch argument 'incr.c'
  #
  missing.incr.c <- missing(incr.c)
  #
  if (!missing.incr.c)
    incr.c <- catch("incr.c", mc, data, sfsp)
  else
    incr.c <- catch2(object, "incr.c", gs("incr"))
  #
  avail.incr.c <- !missing.incr.c & !is.null(incr.c)
  #
  # Catch argument 'n.e'
  #
  if (!missing(n.e))
    n.e <- catch("n.e", mc, data, sfsp)
  else
    n.e <- catch2(object, "n.e")
  #
  # Catch argument 'n.c'
  #
  if (!missing(n.c))
    n.c <- catch("n.c", mc, data, sfsp)
  else
    n.c <- catch2(object, "n.c")
  #
  # Catch argument 'approx.mean.e'
  #
  if (!missing(approx.mean.e))
    approx.mean.e <- catch("approx.mean.e", mc, data, sfsp)
  else
    approx.mean.e <- catch2(object, "approx.mean.e")
  #
  # Catch argument 'approx.mean.c'
  #
  if (!missing(approx.mean.c))
    approx.mean.c <- catch("approx.mean.c", mc, data, sfsp)
  else
    approx.mean.c <- catch2(object, "approx.mean.c")
  #
  # Catch argument 'approx.sd.e'
  #
  if (!missing(approx.sd.e))
    approx.sd.e <- catch("approx.sd.e", mc, data, sfsp)
  else
    approx.sd.e <- catch2(object, "approx.sd.e")
  #
  # Catch argument 'approx.sd.c'
  #
  if (!missing(approx.sd.c))
    approx.sd.c <- catch("approx.sd.c", mc, data, sfsp)
  else
    approx.sd.c <- catch2(object, "approx.sd.c")
  #
  # Catch argument 'subgroup'
  #
  missing.subgroup <- missing(subgroup)
  missing.byvar <- missing(byvar)
  #
  if (!missing.subgroup | !missing.byvar) {
    subgroup <- catch("subgroup", mc, data, sfsp)
    byvar <- catch("byvar", mc, data, sfsp)
    subgroup <-
      deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                  warn.deprecated)
    #
    data$.subgroup <- subgroup
  }
  else if (isCol(object$data, ".subgroup"))
    subgroup <- object$data$.subgroup
  else
    subgroup <- NULL
  #
  if (missing.subgroup.name & is.null(subgroup.name)) {
    if (!missing.subgroup)
      subgroup.name <- byvarname("subgroup", mc)
    else if (!missing.byvar)
      subgroup.name <- byvarname("byvar", mc)
  }
  #
  missing.sm <- missing(sm)
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  #
  # Check arguments for common effect model
  #
  if (!missing.method.common.ci & missing.common)
    common <- TRUE
  #
  # Check arguments for random effects model(s)
  #
  if (!(missing.method.random.ci & missing.adhoc.hakn.ci) & missing.random)
    random <- TRUE
  #
  if (!missing.method.random.ci) {
    if (missing.adhoc.hakn.ci)
      adhoc.hakn.ci <- rep("", length(method.random.ci))
    else if (length(method.random.ci) != length(adhoc.hakn.ci))
      stop("Arguments 'method.random.ci' and 'adhoc.hakn.ci' must be of ",
           "same length.",
           call. = FALSE)
  }
  if (!missing.adhoc.hakn.ci) {
    if (missing.method.random.ci)
      missing.method.random.ci <- rep("HK", length(adhoc.hakn.ci))
    else if (length(method.random.ci) != length(adhoc.hakn.ci))
      stop("Arguments 'method.random.ci' and 'adhoc.hakn.ci' must be of ",
           "same length.",
           call. = FALSE)
  }
  #
  if (!missing.method.random.ci | !missing.text.random) {
    if (length(method.random.ci) != length(text.random)) {
      if (!missing.method.random.ci) {
        if (!(length(text.random) == 1 && text.random == gs("text.random")))
          warning("Setting argument 'text.random' to default as number of ",
                  "random effects \n   methods changed by ",
                  "argument 'method.random.ci'.",
                  call. = FALSE)
        text.random <- gs("text.random")
      }
      if (!missing.text.random)
        stop("Argument 'text.random' must be of same length as \n   ",
             "number of random effects methods specified by setting for ",
             "'method.random.ci'.",
             call. = FALSE)
    }
  }
  #
  # Check variables for prediction interval(s)
  #
  if (!missing.method.predict) {
    if (missing.adhoc.hakn.pi)
      adhoc.hakn.pi <- rep("", length(method.predict))
    else if (length(method.predict) != length(adhoc.hakn.pi))
      stop("Arguments 'method.predict' and 'adhoc.hakn.pi' must be of ",
           "same length.",
           call. = FALSE)
  }
  if (!missing.adhoc.hakn.pi) {
    if (missing.method.predict)
      missing.method.predict <- rep("HK", length(adhoc.hakn.pi))
    else if (length(method.predict) != length(adhoc.hakn.pi))
      stop("Arguments 'method.predict' and 'adhoc.hakn.pi' must be of ",
           "same length.",
           call. = FALSE)
  }
  #
  if (!missing.method.predict | !missing.text.predict) {
    if (length(method.predict) != length(text.predict)) {
      if (!missing.method.predict) {
        if (!(length(text.predict) == 1 && text.predict == gs("text.predict")))
          warning("Setting argument 'text.predict' to default as number of ",
                  "prediction intervals \n   changed by ",
                  "argument 'method.predict'.",
                  call. = FALSE)
        text.predict <- gs("text.predict")
      }
      if (!missing.text.predict)
        stop("Argument 'text.predict' must be of same length as \n   ",
             "number of prediction intervals specified by setting for ",
             "'method.predict'.",
             call. = FALSE)
    }
  }
  
  
  #
  #
  # (6) Update meta object
  #
  #
  method.predict <- replaceVal(method.predict, "", gs("method.predict"))
  #
  if (metabin) {
    sm <- setchar(sm, gs("sm4bin"))
    method <- setchar(method, gs("meth4bin"))
    #
    if (!is.null(...cluster))
      method <- "Inverse"
    #
    if (method %in% c("GLMM", "LRP") & !missing.sm & sm != "OR")
      warning("Summary measure 'sm = \"OR\" used as 'method = \"",
              method, "\".")
    #
    if (method == "GLMM") {
      sm <- "OR"
      method.tau <- "ML"
      model.glmm <- replaceNULL(model.glmm, gs("model.glmm"))
    }
    #
    if (method == "LRP") {
      sm <- "OR"
      method.tau <- "DL"
    }
    #
    if (!(method == "MH" & method.tau == "DL" &
          (sm %in% c("OR", "RR", "RD", "DOR"))))
      Q.Cochrane <- FALSE
    #
    if (sm == "DOR" & missing.method.bias)
      method.bias <- "Deeks"
    else if (sm == "OR" & missing.method.bias)
      method.bias <- "Harbord"
    #
    if (!missing.method.incr)
      method.incr <- setchar(method.incr, gs("meth4incr"))
    else {
      if (!missing.incr & missing.incr.e & missing.incr.c)
        method.incr <- gs("method.incr")
    }
    #
    if (method.incr == "user") {
      incr.orig <- incr
      incr <- NULL
    }
    else {
      incr.e <- NULL
      incr.c <- NULL
      #
      if (sm == "ASD") {
        if (!missing.incr && any(incr != 0))
          warning("Note, no continuity correction considered for ",
                  "arcsine difference (sm = \"ASD\").",
                  call. = FALSE)
        incr <- 0
      }
      #
      if (method == "Peto") {
        if (!missing.incr && any(incr != 0))
          warning("Note, no continuity correction considered for ",
                  "method = \"Peto\".",
                  call. = FALSE)
        incr <- 0
      }
      #
      RR.Cochrane <- replaceNULL(RR.Cochrane, gs("RR.cochrane"))
    }
    #
    m <- metabin(event.e = object$data$.event.e,
                 n.e = object$data$.n.e,
                 event.c = object$data$.event.c,
                 n.c = object$data$.n.c,
                 studlab = studlab,
                 #
                 data = data, subset = subset, exclude = exclude,
                 cluster = ...cluster, rho = rho,
                 #
                 weights = weights,
                 weights.common = weights.common,
                 weights.random = weights.random,
                 #
                 method = method,
                 sm = sm,
                 #
                 incr = incr, method.incr = method.incr,
                 allstudies = allstudies,
                 incr.e = incr.e, incr.c = incr.c,
                 #
                 MH.exact = MH.exact, RR.Cochrane = RR.Cochrane,
                 Q.Cochrane = Q.Cochrane, model.glmm = model.glmm,
                 #
                 level = level, level.ma = level.ma,
                 common = common, random = random,
                 overall = overall, overall.hetstat = overall.hetstat,
                 #
                 method.common.ci = method.common.ci,
                 method.random.ci = method.random.ci,
                 adhoc.hakn.ci = adhoc.hakn.ci,
                 method.predict = method.predict,
                 adhoc.hakn.pi = adhoc.hakn.pi,
                 seed.predict = seed.predict,
                 #
                 method.tau = method.tau, method.tau.ci = method.tau.ci,
                 level.hetstat = level.hetstat,
                 tau.preset = tau.preset, TE.tau = TE.tau,
                 tau.common = tau.common,
                 detail.tau = detail.tau,
                 #
                 method.I2 = method.I2,
                 #
                 prediction = prediction | !missing.method.predict,
                 level.predict = level.predict,
                 #
                 method.bias = method.bias,
                 #
                 backtransf = backtransf, pscale = pscale,
                 #
                 text.common = text.common, text.random = text.random,
                 text.predict = text.predict,
                 text.w.common = text.w.common, text.w.random = text.w.random,
                 #
                 title = title, complab = complab, outclab = outclab,
                 #
                 label.e = label.e, label.c = label.c,
                 label.right = label.right, label.left = label.left,
                 col.label.right = col.label.right,
                 col.label.left = col.label.left,
                 #
                 subgroup = subgroup, subgroup.name = subgroup.name,
                 print.subgroup.name = print.subgroup.name,
                 sep.subgroup = sep.subgroup,
                 test.subgroup = test.subgroup,
                 prediction.subgroup = prediction.subgroup,
                 seed.predict.subgroup = seed.predict.subgroup,
                 print.CMH = print.CMH,
                 #
                 warn = warn, warn.deprecated = FALSE,
                 #
                 control = control,
                 ...)
    #
    if (method.incr == "user")
      m$incr <- incr.orig
  }
  #
  if (metacont) {
    if (!isCol(object$data, ".approx.mean.e"))
      mean.e <- object$data$.mean.e
    else
      mean.e <-
        setNA_ifnot(object$data$.mean.e, object$data$.approx.mean.e, "")
    #
    if (!isCol(object$data, ".approx.mean.c"))
      mean.c <- object$data$.mean.c
    else
      mean.c <-
        setNA_ifnot(object$data$.mean.c, object$data$.approx.mean.c, "")
    #
    if (!isCol(object$data, ".approx.sd.e"))
      sd.e <- object$data$.sd.e
    else
      sd.e <-
        setNA_ifnot(object$data$.sd.e, object$data$.approx.sd.e, "")
    #
    if (!isCol(object$data, ".approx.sd.c"))
      sd.c <- object$data$.sd.c
    else
      sd.c <-
        setNA_ifnot(object$data$.sd.c, object$data$.approx.sd.c, "")
    #
    m <- metacont(n.e = object$data$.n.e,
                  mean.e = object$data$.mean.e,
                  sd.e = object$data$.sd.e,
                  n.c = object$data$.n.c,
                  mean.c = object$data$.mean.c,
                  sd.c = object$data$.sd.c,
                  studlab = studlab,
                  #
                  data = data, subset = subset, exclude = exclude,
                  cluster = ...cluster, rho = rho,
                  #
                  weights = weights,
                  weights.common = weights.common,
                  weights.random = weights.random,
                  #
                  median.e = setVal(object$data, ".median.e"),
                  q1.e = setVal(object$data, ".q1.e"),
                  q3.e = setVal(object$data, ".q3.e"),
                  min.e = setVal(object$data, ".min.e"),
                  max.e = setVal(object$data, ".max.e"),
                  #
                  median.c = setVal(object$data, ".median.c"),
                  q1.c = setVal(object$data, ".q1.c"),
                  q3.c = setVal(object$data, ".q3.c"),
                  min.c = setVal(object$data, ".min.c"),
                  max.c = setVal(object$data, ".max.c"),
                  #
                  method.mean =
                    replaceVal(replaceNULL(method.mean, "Luo"), "", "Luo"),
                  method.sd =
                    replaceVal(replaceNULL(method.sd, "Shi"), "", "Shi"),
                  #
                  approx.mean.e = approx.mean.e,
                  approx.mean.c = approx.mean.c,
                  approx.sd.e = approx.sd.e,
                  approx.sd.c = approx.sd.c,
                  #
                  sm = sm,
                  pooledvar = replaceNA(pooledvar, gs("pooledvar")),
                  method.smd = replaceVal(method.smd, "", gs("method.smd")),
                  sd.glass = replaceVal(sd.glass, "", gs("sd.glass")),
                  exact.smd = replaceNA(exact.smd, gs("exact.smd")),
                  #
                  method.ci = replaceNULL(method.ci, gs("method.ci.cont")),
                  level = level, level.ma = level.ma,
                  common = common, random = random,
                  overall = overall, overall.hetstat = overall.hetstat,
                  #
                  method.common.ci = method.common.ci,
                  method.random.ci = method.random.ci,
                  adhoc.hakn.ci = adhoc.hakn.ci,
                  method.predict = method.predict,
                  adhoc.hakn.pi = adhoc.hakn.pi,
                  seed.predict = seed.predict,
                  #
                  method.tau = method.tau, method.tau.ci = method.tau.ci,
                  level.hetstat = level.hetstat,
                  tau.preset = tau.preset, TE.tau = TE.tau,
                  tau.common = tau.common,
                  detail.tau = detail.tau,
                  #
                  method.I2 = method.I2,
                  #
                  prediction = prediction | !missing.method.predict,
                  level.predict = level.predict,
                  #
                  method.bias = method.bias,
                  #
                  text.common = text.common, text.random = text.random,
                  text.predict = text.predict,
                  text.w.common = text.w.common, text.w.random = text.w.random,
                  #
                  title = title, complab = complab, outclab = outclab,
                  #
                  label.e = label.e, label.c = label.c,
                  label.right = label.right, label.left = label.left,
                  col.label.right = col.label.right,
                  col.label.left = col.label.left,
                  #
                  subgroup = subgroup, subgroup.name = subgroup.name,
                  print.subgroup.name = print.subgroup.name,
                  sep.subgroup = sep.subgroup,
                  test.subgroup = test.subgroup,
                  prediction.subgroup = prediction.subgroup,
                  seed.predict.subgroup = seed.predict.subgroup,
                  #
                  warn = warn, warn.deprecated = FALSE,
                  #
                  control = control)
  }
  #
  if (metacor)
    m <- metacor(cor = object$data$.cor,
                 n = object$data$.n,
                 studlab = studlab,
                 #
                 data = data, subset = subset, exclude = exclude,
                 cluster = ...cluster, rho = rho,
                 #
                 weights = weights,
                 weights.common = weights.common,
                 weights.random = weights.random,
                 #
                 sm = sm,
                 level = level, level.ma = level.ma,
                 common = common, random = random,
                 overall = overall, overall.hetstat = overall.hetstat,
                 #
                 method.common.ci = method.common.ci,
                 method.random.ci = method.random.ci,
                 adhoc.hakn.ci = adhoc.hakn.ci,
                 method.predict = method.predict,
                 adhoc.hakn.pi = adhoc.hakn.pi,
                 seed.predict = seed.predict,
                 #
                 method.tau = method.tau, method.tau.ci = method.tau.ci,
                 level.hetstat = level.hetstat,
                 tau.preset = tau.preset, TE.tau = TE.tau,
                 tau.common = tau.common,
                 detail.tau = detail.tau,
                 #
                 method.I2 = method.I2,
                 #
                 prediction = prediction | !missing.method.predict,
                 level.predict = level.predict,
                 #
                 null.effect = null.effect,
                 #
                 method.bias = method.bias,
                 #
                 backtransf = backtransf,
                 #
                 text.common = text.common, text.random = text.random,
                 text.predict = text.predict,
                 text.w.common = text.w.common, text.w.random = text.w.random,
                 #
                 title = title, complab = complab, outclab = outclab,
                 #
                 label.right = label.right, label.left = label.left,
                 col.label.right = col.label.right,
                 col.label.left = col.label.left,
                 #
                 subgroup = subgroup, subgroup.name = subgroup.name,
                 print.subgroup.name = print.subgroup.name,
                 sep.subgroup = sep.subgroup,
                 test.subgroup = test.subgroup,
                 prediction.subgroup = prediction.subgroup,
                 seed.predict.subgroup = seed.predict.subgroup,
                 #
                 warn.deprecated = FALSE,
                 #
                 control = control)
  #
  if (metagen) {
    ...n.e <- n.e
    ...n.c <- n.c
    #
    rm(n.e)
    rm(n.c)
    #
    if (missing(approx.TE)) {
      if (isCol(object$data, ".approx.TE"))
        approx.TE <- object$data$.approx.TE
      else
        approx.TE <- NULL
    }
    if (missing(approx.seTE)) {
      if (isCol(object$data, ".approx.seTE"))
        approx.seTE <- object$data$.approx.seTE
      else
        approx.seTE <- NULL
    }
    #
    m <- metagen(TE = object$data$.TE,
                 seTE =
                   if (!avail.cycles) object$data$.seTE
                   else replaceNULL(object$data$.seTE.orig, object$data$.seTE),
                 studlab = studlab,
                 #
                 data = data, subset = subset, exclude = exclude,
                 cluster = ...cluster, rho = rho,
                 #
                 cycles = cycles,
                 #
                 weights = weights,
                 weights.common = weights.common,
                 weights.random = weights.random,
                 #
                 sm = sm,
                 method.ci = method.ci,
                 level = level,
                 common = common, random = random,
                 overall = overall, overall.hetstat = overall.hetstat,
                 #
                 level.ma = level.ma,
                 method.common.ci = method.common.ci,
                 method.random.ci = method.random.ci,
                 adhoc.hakn.ci = adhoc.hakn.ci,
                 method.predict = method.predict,
                 adhoc.hakn.pi = adhoc.hakn.pi,
                 seed.predict = seed.predict,
                 #
                 method.tau = method.tau, method.tau.ci = method.tau.ci,
                 level.hetstat = level.hetstat,
                 tau.preset = tau.preset, TE.tau = TE.tau,
                 tau.common = tau.common,
                 detail.tau = detail.tau,
                 #
                 method.I2 = method.I2,
                 #
                 prediction = prediction | !missing.method.predict,
                 level.predict = level.predict,
                 #
                 method.bias = method.bias,
                 #
                 n.e = ...n.e, n.c = ...n.c,
                 #
                 pval = setVal(object$data, ".pval"),
                 df = setVal(object$data, ".df"),
                 lower = setVal(object$data, ".lower"),
                 upper = setVal(object$data, ".upper"),
                 level.ci = setVal(object$data, ".level.ci"),
                 #
                 median = setVal(object$data, ".median"),
                 q1 = setVal(object$data, ".q1"),
                 q3 = setVal(object$data, ".q3"),
                 min = setVal(object$data, ".min"),
                 max = setVal(object$data, ".max"),
                 #
                 method.mean =
                   replaceVal(replaceNULL(method.mean, "Luo"), "", "Luo"),
                 method.sd =
                   replaceVal(replaceNULL(method.sd, "Shi"), "", "Shi"),
                 #
                 approx.TE = approx.TE,
                 approx.seTE = approx.seTE,
                 #
                 transf = TRUE,
                 backtransf = backtransf,
                 func.transf = object$func.transf,
                 func.backtransf = func.backtransf,
                 args.transf = object$args.transf,
                 args.backtransf = args.backtransf,         
                 #
                 pscale = pscale,
                 irscale = irscale, irunit = irunit,
                 #
                 text.common = text.common, text.random = text.random,
                 text.predict = text.predict,
                 text.w.common = text.w.common, text.w.random = text.w.random,
                 #
                 title = title, complab = complab, outclab = outclab,
                 #
                 label.e = label.e, label.c = label.c,
                 label.right = label.right, label.left = label.left,
                 col.label.right = col.label.right,
                 col.label.left = col.label.left,
                 #
                 subgroup = subgroup, subgroup.name = subgroup.name,
                 print.subgroup.name = print.subgroup.name,
                 sep.subgroup = sep.subgroup,
                 test.subgroup = test.subgroup,
                 prediction.subgroup = prediction.subgroup,
                 seed.predict.subgroup = seed.predict.subgroup,
                 #
                 warn = warn, warn.deprecated = FALSE,
                 #
                 control = control)
  }
  #
  if (metainc) {
    ...n.e <- n.e
    ...n.c <- n.c
    #
    rm(n.e)
    rm(n.c)
    #
    sm <- setchar(sm, gs("sm4inc"))
    method <- setchar(method, gs("meth4inc"))
    #
    if (!is.null(...cluster))
      method <- "Inverse"
    #
    if (method == "GLMM" & !missing.sm & !(sm %in% c("IRR", "VE")))
      warning("Summary measure 'sm = \"IRR\" used as 'method = \"GLMM\".")
    #
    if (method == "GLMM") {
      if (sm != "VE")
        sm <- "IRR"
      method.tau <- "ML"
      model.glmm <- replaceNULL(model.glmm, "UM.FS")
    }
    #
    if (!missing.method.incr)
      method.incr <- setchar(method.incr, gs("meth4incr"))
    else {
      if (!missing.incr & missing.incr.e & missing.incr.c)
        method.incr <- gs("method.incr")
    }
    #
    if (method.incr == "user") {
      incr.orig <- incr
      incr <- NULL
    }
    else {
      incr.e <- NULL
      incr.c <- NULL
    }
    #
    m <- metainc(event.e = object$data$.event.e,
                 time.e = object$data$.time.e,
                 event.c = object$data$.event.c,
                 time.c = object$data$.time.c,
                 studlab = studlab,
                 #
                 data = data, subset = subset, exclude = exclude,
                 cluster = ...cluster, rho = rho,
                 #
                 weights = weights,
                 weights.common = weights.common,
                 weights.random = weights.random,
                 #
                 method = method,
                 sm = sm,
                 #
                 incr = incr, method.incr = method.incr,
                 incr.e = incr.e, incr.c = incr.c,
                 #
                 model.glmm = model.glmm,
                 #
                 level = level, level.ma = level.ma,
                 common = common, random = random,
                 overall = overall, overall.hetstat = overall.hetstat,
                 #
                 method.common.ci = method.common.ci,
                 method.random.ci = method.random.ci,
                 adhoc.hakn.ci = adhoc.hakn.ci,
                 method.predict = method.predict,
                 adhoc.hakn.pi = adhoc.hakn.pi,
                 seed.predict = seed.predict,
                 #
                 method.tau = method.tau, method.tau.ci = method.tau.ci,
                 level.hetstat = level.hetstat,
                 tau.preset = tau.preset, TE.tau = TE.tau,
                 tau.common = tau.common,
                 detail.tau = detail.tau,
                 #
                 method.I2 = method.I2,
                 #
                 prediction = prediction | !missing.method.predict,
                 level.predict = level.predict,
                 #
                 method.bias = method.bias,
                 #
                 n.e = ...n.e, n.c = ...n.c,
                 #
                 backtransf = backtransf, irscale = irscale, irunit = irunit,
                 #
                 text.common = text.common, text.random = text.random,
                 text.predict = text.predict,
                 text.w.common = text.w.common, text.w.random = text.w.random,
                 #
                 title = title, complab = complab, outclab = outclab,
                 #
                 label.e = label.e, label.c = label.c,
                 label.right = label.right, label.left = label.left,
                 col.label.right = col.label.right,
                 col.label.left = col.label.left,
                 #
                 subgroup = subgroup, subgroup.name = subgroup.name,
                 print.subgroup.name = print.subgroup.name,
                 sep.subgroup = sep.subgroup,
                 test.subgroup = test.subgroup,
                 prediction.subgroup = prediction.subgroup,
                 seed.predict.subgroup = seed.predict.subgroup,
                 #
                 warn = warn, warn.deprecated = FALSE,
                 #
                 control = control,
                 ...)
    #
    if (method.incr == "user")
      m$incr <- incr.orig
  }
  #
  if (metamean)
    m <- metamean(n = object$data$.n,
                  mean = object$data$.mean,
                  sd = object$data$.sd,
                  studlab = studlab,
                  #
                  data = data, subset = subset, exclude = exclude,
                  cluster = ...cluster, rho = rho,
                  #
                  weights = weights,
                  weights.common = weights.common,
                  weights.random = weights.random,
                  #
                  median = setVal(object$data, ".median"),
                  q1 = setVal(object$data, ".q1"),
                  q3 = setVal(object$data, ".q3"),
                  min = setVal(object$data, ".min"),
                  max = setVal(object$data, ".max"),
                  #
                  method.mean = replaceVal(method.mean, "", "Luo"),
                  method.sd = replaceVal(method.sd, "", "Shi"),
                  #
                  approx.mean = approx.mean,
                  approx.sd = approx.sd,
                  #
                  sm = sm,
                  method.ci = replaceNULL(method.ci, gs("method.ci.cont")),
                  level = level, level.ma = level.ma,
                  common = common, random = random,
                  overall = overall, overall.hetstat = overall.hetstat,
                  #
                  method.common.ci = method.common.ci,
                  method.random.ci = method.random.ci,
                  adhoc.hakn.ci = adhoc.hakn.ci,
                  method.predict = method.predict,
                  adhoc.hakn.pi = adhoc.hakn.pi,
                  seed.predict = seed.predict,
                  #
                  method.tau = method.tau, method.tau.ci = method.tau.ci,
                  level.hetstat = level.hetstat,
                  tau.preset = tau.preset, TE.tau = TE.tau,
                  tau.common = tau.common,
                  detail.tau = detail.tau,
                  #
                  method.I2 = method.I2,
                  #
                  prediction = prediction | !missing.method.predict,
                  level.predict = level.predict,
                  #
                  null.effect = null.effect,
                  #
                  method.bias = method.bias,
                  #
                  backtransf = backtransf,
                  #
                  text.common = text.common, text.random = text.random,
                  text.predict = text.predict,
                  text.w.common = text.w.common, text.w.random = text.w.random,
                  #
                  title = title, complab = complab, outclab = outclab,
                  #
                  label.right = label.right, label.left = label.left,
                  col.label.right = col.label.right,
                  col.label.left = col.label.left,
                  #
                  subgroup = subgroup, subgroup.name = subgroup.name,
                  print.subgroup.name = print.subgroup.name,
                  sep.subgroup = sep.subgroup,
                  test.subgroup = test.subgroup,
                  prediction.subgroup = prediction.subgroup,
                  seed.predict.subgroup = seed.predict.subgroup,
                  #
                  warn = warn, warn.deprecated = FALSE,
                  #
                  control = control)
  #
  if (metaprop) {
    sm <- setchar(sm, gs("sm4prop"))
    method <- setchar(method, gs("meth4prop"))
    #
    if (!is.null(...cluster))
      method <- "Inverse"
    if (method == "GLMM" & !missing.sm & sm != "PLOGIT")
      warning("Summary measure 'sm = \"PLOGIT\" used as 'method = \"GLMM\".")
    #
    if (method == "GLMM") {
      sm <- "PLOGIT"
      method.tau <- "ML"
    }
    #
    method.ci <- replaceNULL(method.ci, gs("method.ci.prop"))
    #
    m <- metaprop(event = object$data$.event,
                  n = object$data$.n,
                  studlab = studlab,
                  #
                  data = data, subset = subset, exclude = exclude,
                  cluster = ...cluster, rho = rho,
                  #
                  weights = weights,
                  weights.common = weights.common,
                  weights.random = weights.random,
                  #
                  method = method,
                  sm = sm,
                  incr = incr,
                  method.incr = method.incr,
                  #
                  method.ci = method.ci,
                  level = level, level.ma = level.ma,
                  common = common, random = random,
                  overall = overall, overall.hetstat = overall.hetstat,
                  #
                  method.common.ci = method.common.ci,
                  method.random.ci = method.random.ci,
                  adhoc.hakn.ci = adhoc.hakn.ci,
                  method.predict = method.predict,
                  adhoc.hakn.pi = adhoc.hakn.pi,
                  seed.predict = seed.predict,
                  #
                  method.tau = method.tau, method.tau.ci = method.tau.ci,
                  level.hetstat = level.hetstat,
                  tau.preset = tau.preset, TE.tau = TE.tau,
                  tau.common = tau.common,
                  detail.tau = detail.tau,
                  #
                  method.I2 = method.I2,
                  #
                  prediction = prediction | !missing.method.predict,
                  level.predict = level.predict,
                  #
                  null.effect = null.effect,
                  #
                  method.bias = method.bias,
                  #
                  backtransf = backtransf, pscale = pscale,
                  #
                  text.common = text.common, text.random = text.random,
                  text.predict = text.predict,
                  text.w.common = text.w.common, text.w.random = text.w.random,
                  #
                  title = title, complab = complab, outclab = outclab,
                  #
                  label.right = label.right, label.left = label.left,
                  col.label.right = col.label.right,
                  col.label.left = col.label.left,
                  #
                  subgroup = subgroup, subgroup.name = subgroup.name,
                  print.subgroup.name = print.subgroup.name,
                  sep.subgroup = sep.subgroup,
                  test.subgroup = test.subgroup,
                  prediction.subgroup = prediction.subgroup,
                  seed.predict.subgroup = seed.predict.subgroup,
                  #
                  warn = warn, warn.deprecated = FALSE,
                  #
                  control = control,
                  ...)
  }
  #
  if (metarate) {
    sm <- setchar(sm, gs("sm4rate"))
    method <- setchar(method, gs("meth4rate"))
    #
    if (!is.null(...cluster))
      method <- "Inverse"
    #
    if (method == "GLMM" & !missing.sm & sm != "IRLN")
      warning("Summary measure 'sm = \"IRLN\" used as 'method = \"GLMM\".")
    #
    if (method == "GLMM") {
      sm <- "IRLN"
      method.tau <- "ML"
    }
    #
    method.ci <- replaceNULL(method.ci, gs("method.ci.rate"))
    #
    m <- metarate(event = object$data$.event,
                  time = object$data$.time,
                  studlab = studlab,
                  #
                  data = data, subset = subset, exclude = exclude,
                  cluster = ...cluster, rho = rho,
                  #
                  weights = weights,
                  weights.common = weights.common,
                  weights.random = weights.random,
                  #
                  method = method,
                  sm = sm,
                  incr = incr,
                  method.incr = method.incr,
                  #
                  method.ci = method.ci,
                  level = level, level.ma = level.ma,
                  common = common, random = random,
                  overall = overall, overall.hetstat = overall.hetstat,
                  #
                  method.common.ci = method.common.ci,
                  method.random.ci = method.random.ci,
                  adhoc.hakn.ci = adhoc.hakn.ci,
                  method.predict = method.predict,
                  adhoc.hakn.pi = adhoc.hakn.pi,
                  seed.predict = seed.predict,
                  #
                  method.tau = method.tau, method.tau.ci = method.tau.ci,
                  level.hetstat = level.hetstat,
                  tau.preset = tau.preset, TE.tau = TE.tau,
                  tau.common = tau.common,
                  detail.tau = detail.tau,
                  #
                  method.I2 = method.I2,
                  #
                  prediction = prediction | !missing.method.predict,
                  level.predict = level.predict,
                  #
                  null.effect = null.effect,
                  #
                  method.bias = method.bias,
                  #
                  backtransf = backtransf, irscale = irscale, irunit = irunit,
                  #
                  text.common = text.common, text.random = text.random,
                  text.predict = text.predict,
                  text.w.common = text.w.common, text.w.random = text.w.random,
                  #
                  title = title, complab = complab, outclab = outclab,
                  #
                  label.right = label.right, label.left = label.left,
                  col.label.right = col.label.right,
                  col.label.left = col.label.left,
                  #
                  subgroup = subgroup, subgroup.name = subgroup.name,
                  print.subgroup.name = print.subgroup.name,
                  sep.subgroup = sep.subgroup,
                  test.subgroup = test.subgroup,
                  prediction.subgroup = prediction.subgroup,
                  seed.predict.subgroup = seed.predict.subgroup,
                  #
                  warn = warn, warn.deprecated = FALSE,
                  #
                  control = control,
                  ...)
  }
  #
  m$call.object <- object$call
  m$call <- match.call()
  #
  if (!is.null(object$rob)) {
    if (!is.null(m$subset))
      m$rob <- object$rob[m$subset, ]
    else
      m$rob <- object$rob
  }
  #
  if (!keepdata)
    m$data <- NULL
  
  m
}

#' Calculate the risk difference from meta-analysis results
#' 
#' @description
#' Calculate the risk difference from the risk ratio, odds ratio, or hazard
#' ratio, and a baseline probability, i.e., control group event probability for
#' binary outcomes or survival probability for hazard ratios. Results are
#' reported as absolute risk reduction/increase or absolute benefit
#' increase/reduction.
#' 
#' @aliases rd rd.default rd.meta
#' 
#' @param x An object of class \code{meta}, or estimated treatment
#'   effect(s), i.e., risk difference(s), risk ratio(s), odds
#'   ratio(s), or hazard ratio(s).
#' @param p.c Baseline probability, i.e., control group event probability
#'   for binary outcomes or survival probability in control group for
#'   hazard ratios.
#' @param sm Summary measure.
#' @param lower Lower confidence interval limit.
#' @param upper Upper confidence interval limit.
#' @param level The level used to calculate confidence intervals.
#' @param common A logical indicating whether ARRs / ABIs should be
#'   calculated based on common effect estimate.
#' @param random A logical indicating whether ARRs / ABIs should be
#'   calculated based on random effects estimate.
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated.
#' @param transf A logical indicating whether treatment estimates and
#'   confidence limits are transformed or on the original scale.
#'   If \code{transf = TRUE}, inputs are expected to be log odds ratios instead
#'   of odds ratios for \code{sm = "OR"}, for example.
#' @param pscale A numeric defining a scaling factor for printing of
#'   absolute risk reduction or absolute benefit increase.
#' @param digits Minimal number of significant digits to print ARR / ABI and its
#'   confidence interval, see \code{print.default}.
#' @param digits.prop Minimal number of significant digits for
#'   proportions, see \code{print.default}.
#' @param big.mark A character used as thousands separator.
#' @param details A logical specifying whether details should be printed.
#' @param \dots Additional arguments (ignored)
#' 
#' @details
#' Calculate the risk difference from the risk ratio, odds ratio, or hazard
#' ratio, and a baseline probability, i.e., control group event probability for
#' binary outcomes or survival probability for hazard ratios. Report results as
#' absolute risk reduction (ARR), absolute risk increase (ARI), absolute
#' benefit increase (ABI), or absolute benefit reduction (ABR).
#'
#' Absolute measures can be easily computed from an estimated risk
#' difference (RD), or a risk ratio (RR) or odds ratio (OR) and a given
#' baseline probability. It is also possible to calculate absolute
#' measures from hazard ratios (HR) (Altman & Andersen, 1999).
#' Accordingly, these measures can be calculated for meta-analyses
#' generated with \code{\link{metabin}} or \code{\link{metagen}}
#' if argument \code{sm} was equal to \code{"RD"}, \code{"RR"},
#' \code{"OR"}, or \code{"HR"}. It is also possible to provide only
#' estimated treatment effects and baseline probabilities (see Examples).
#'
#' The baseline probability can be specified using argument \code{p.c}. If
#' this argument is missing, the minimum, mean, and maximum of the
#' control event probabilities in the meta-analysis are used for
#' \code{\link{metabin}} and control event probabilities of 0.1, 0.2,
#' \dots, 0.9 are used for \code{\link{metagen}}. Note, the survival instead of
#' mortality probability must be provided for hazard ratios.
#'
#' Argument \code{small.values} can be used to specify whether small
#' treatment effects indicate a beneficial (\code{"desirable"}) or harmful
#' (\code{"undesirable"}) effect. For \code{small.values = "desirable"}, OR < 1,
#' RR < 1, HR < 1, or RD > 0 indicate that the new treatment is beneficial.
#' In this case, calculated risk differences are expressed as absolute risk
#' reductions (ARR). If OR > 1, RR > 1, HR > 1, or RD > 0, calculated risk
#' differences are expressed as absolute risk increases (ARI). For
#' \code{small.values = "undesirable"}, odds, OR > 1, RR > 1, HR > 1, or RD > 0
#' indicate that the new treatment is beneficial. In this case, calculated risk
#' differences are expressed as absolute benefit increases (ABI). If OR < 1,
#' RR < 1, HR < 1, or RD < 0, calculated risk differences are expressed as
#' absolute benefit reductions (ABR).
#' 
#' Argument \code{pscale} can be used to rescale ARRs, ..., ABRs, e.g.,
#' \code{pscale = 1000} means that these quantities are expressed as events
#' per 1000 observations. This is useful in situations with (very) low
#' event probabilities.
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{metabin}}, \code{\link{metagen}}
#' 
#' @references
#' Altman DG, Andersen PK (1999):
#' Calculating the number needed to treat for trials where the outcome
#' is time to an event.
#' \emph{British Medical Journal},
#' \bold{319}, 1492--95
#' 
#' @examples
#' # Calculate ARRs for risk differences of -0.12 and -0.22
#' rd(c(-0.12, -0.22), sm = "RD")
#' 
#' # Calculate ARR for risk ratio of 0.92 and baseline risk of 0.3
#' rd(0.92, p.c = 0.3, sm = "RR")
#'
#' # Calculate ARR for odds ratio of 0.73 and baseline risk of 0.3
#' rd(0.73, p.c = 0.3, sm = "OR")
#'
#' # Calculate ARRs for Mantel-Haenszel odds ratio
#' data(Olkin1995)
#' ma <-
#'   metabin(ev.exp, n.exp, ev.cont, n.cont, data = Olkin1995, random = FALSE)
#' rd(ma)
#'
#' # Calculate ARRs from hazard ratio at two and four years (example from
#' # Altman & Andersen, 1999). Note, argument 'p.c' must provide survival
#' # probabilities instead of mortality rates for the control group.
#' rd(0.72, lower = 0.55, upper = 0.92, sm = "HR", p.c = 1 - c(0.33, 0.49))
#' 
#' @rdname rd
#' @export rd

rd <- function(x, ...) 
  UseMethod("rd")





#' @rdname rd
#' @method rd meta
#' @export

rd.meta <- function(x, p.c,
                    common = x$common,
                    random = x$random,
                    small.values = "desirable",
                    pscale = 1,
                    ...) {
  
  
  #
  # (1) Check for meta object and summary measure
  #
  chkclass(x, "meta")
  x <- updateversion(x)
  #
  if (!(x$sm %in% c("RD", "RR", "OR", "HR")))
    stop("Calculation of ARRs /ABIs only possible for risk difference, ",
         "risk ratio, odds ratio, or hazard ratio as summary measure ",
         "(argument 'sm').")
  #
  chknumeric(pscale, length = 1)
  
  
  #
  # (2) Check / set baseline risks
  #
  if (missing(p.c)) {
    if (x$sm == "RD")
      p.c <- NA
    else {
      if (!is.null(x$event.c) & !is.null(x$n.c)) {
        p.c <-
          unique(as.vector(
            summary(x$event.c / x$n.c,
                    na.rm = TRUE)[c("Min.", "Mean", "Max.")]))
        if (!all(p.c == 0))
          p.c <- p.c[p.c != 0]
      }
      else {
        p.c <- seq(0.1, 0.9, by = 0.1)
        if (x$sm == "HR")
          p.c <- rev(p.c)
      }
    }
  }
  #
  chknumeric(p.c, 0, 1)
  
  
  #
  # (3) Check other arguments
  #
  small.values <- setsv(small.values)
  #
  missing.common <- missing(common)
  chklogical(common)
  chklogical(random)
  #
  if (missing.common & !common & !random & x$k == 1)
    common <- TRUE
  #
  if (missing.common & x$k == 0 & x$k.all == 1 &
      all(is.na(x$TE.common)) & any(!is.na(x$TE))) {
    common <- TRUE
    x$TE.common <- x$TE
  }
  
  
  #
  # (4) Calculate RDs
  #
  res <- list()
  #
  if (common)
    res$common <- data.frame(
      TE = x$TE.common, lower = x$lower.common, upper = x$upper.common,
      p.c = p.c, RD = NA, lower.RD = NA, upper.RD = NA)
  #
  if (random)
    res$random <- data.frame(
      TE = x$TE.random, lower = x$lower.random, upper = x$upper.random,
      p.c = p.c, RD = NA, lower.RD = NA, upper.RD = NA)
  #
  if (x$sm == "RD") {
    if (common) {
      res$common$RD <- x$TE.common
      res$common$lower.RD <- x$lower.common
      res$common$upper.RD <- x$upper.common
    }
    #
    if (random) {
      res$random$RD <- x$TE.random
      res$random$lower.RD <- x$lower.random
      res$random$upper.RD <- x$upper.random
    }
  }
  #
  else if (x$sm == "RR") {
    if (common) {
      res$common$RD <- logRR2rd(x$TE.common, p.c)
      res$common$lower.RD <- logRR2rd(x$lower.common, p.c)
      res$common$upper.RD <- logRR2rd(x$upper.common, p.c)
    }
    #
    if (random) {
      res$random$RD <- logRR2rd(x$TE.random, p.c)
      res$random$lower.RD <- logRR2rd(x$lower.random, p.c)
      res$random$upper.RD <- logRR2rd(x$upper.random, p.c)
    }
  }
  #
  else if (x$sm == "OR") {
    if (common) {
      res$common$RD <- logOR2rd(x$TE.common, p.c)
      res$common$lower.RD <- logOR2rd(x$lower.common, p.c)
      res$common$upper.RD <- logOR2rd(x$upper.common, p.c)
    }
    #
    if (random) {
      res$random$RD <- logOR2rd(x$TE.random, p.c)
      res$random$lower.RD <- logOR2rd(x$lower.random, p.c)
      res$random$upper.RD <- logOR2rd(x$upper.random, p.c)
    }
  }
  #
  else if (x$sm == "HR") {
    if (common) {
      res$common$RD <- logHR2rd(x$TE.common, p.c)
      res$common$lower.RD <- logHR2rd(x$lower.common, p.c)
      res$common$upper.RD <- logHR2rd(x$upper.common, p.c)
    }
    #
    if (random) {
      res$random$RD <- logHR2rd(x$TE.random, p.c)
      res$random$lower.RD <- logHR2rd(x$lower.random, p.c)
      res$random$upper.RD <- logHR2rd(x$upper.random, p.c)
    }
  }
  #
  class(res) <- "rd.meta"
  #
  attr(res, "level") <- x$level.ma
  attr(res, "sm") <- x$sm
  attr(res, "small.values") <- small.values
  #
  attr(res, "pscale") <- pscale
  #
  attr(res, "call") <- match.call()
  attr(res, "version") <- packageDescription("meta")$Version
  
  res
}





#' @rdname rd
#' @method rd default
#' @export

rd.default <- function(x, p.c, sm, lower, upper,
                       level = gs("level.ma"),
                       small.values = "desirable",
                       pscale = 1,
                       transf = FALSE,
                       ...) {
  
  #
  # (1) Check arguments
  #
  if (missing(x))
    stop("Argument 'x' is mandatory.")
  #
  if (missing(sm))
    stop("Argument 'sm' is mandatory.")
  #
  sm <- setchar(sm, c("RD", "RR", "OR", "HR"))
  #
  chklevel(level)
  #
  if (missing(p.c)) {
    if (sm == "RD")
      p.c <- NA
    else
      stop("Argument 'p.c' is mandatory.")
  }
  #
  chknumeric(p.c, 0, 1)
  #
  chknumeric(pscale, length = 1)
  #
  missing.lower <- missing(lower)
  if (!missing.lower)
    chklength(lower, length(x), "rd.default")
  #
  missing.upper <- missing(upper)
  if (!missing.upper)
    chklength(upper, length(x), "rd.default")
  #
  # Check length of arguments 'x' and 'p.c'
  #
  if (length(x) != length(p.c)) {
    if (length(x) > length(p.c))
      bad <- length(x) %% length(p.c) != 0
    else
      bad <- length(p.c) %% length(x) != 0
    #
    if (bad)
      stop("Lengths of arguments 'x' and 'p.c' do not match.",
           call. = FALSE)
  }
  #
  small.values <- setsv(small.values)
  #
  chklogical(transf)
  
  
  #
  # (2) Calculate RDs
  #
  res <- data.frame(TE = if (is_relative_effect(sm) & !transf) log(x) else x,
                    lower = NA, upper = NA,
                    p.c = p.c, RD = NA, lower.RD = NA, upper.RD = NA)
  #
  if (missing(lower)) {
    res$lower <- NULL
    res$lower.RD <- NULL
  }
  else {
    res$lower <- lower
    if (is_relative_effect(sm) & !transf)
      res$lower <- log(res$lower)
  }
  #
  if (missing(upper)) {
    res$upper <- NULL
    res$upper.RD <- NULL
  }
  else {
    res$upper <- upper
    if (is_relative_effect(sm) & !transf)
      res$upper <- log(res$upper)
  }
  #
  if (sm == "RD") {
    res$RD <- res$TE
    if (!missing.lower)
      res$lower.RD <- res$lower
    if (!missing.upper)
      res$upper.RD <- res$upper
  }
  #
  else if (sm == "RR") {
    res$RD <- logRR2rd(res$TE, res$p.c)
    if (!missing.lower)
      res$lower.RD <- logRR2rd(res$lower, res$p.c)
    if (!missing.upper)
      res$upper.RD <- logRR2rd(res$upper, res$p.c)
  }
  #
  else if (sm == "OR") {
    res$RD <- logOR2rd(res$TE, res$p.c)
    if (!missing.lower)
      res$lower.RD <- logOR2rd(res$lower, res$p.c)
    if (!missing.upper)
      res$upper.RD <- logOR2rd(res$upper, res$p.c)
  }
  #
  else if (sm == "HR") {
    res$RD <- logHR2rd(res$TE, res$p.c)
    if (!missing.lower)
      res$lower.RD <- logHR2rd(res$lower, res$p.c)
    if (!missing.upper)
      res$upper.RD <- logHR2rd(res$upper, res$p.c)
  }
  #
  class(res) <- c("rd.default", class(res))
  #
  attr(res, "level") <- level
  attr(res, "sm") <- sm
  attr(res, "small.values") <- small.values
  #
  attr(res, "pscale") <- pscale
  #
  attr(res, "call") <- match.call()
  attr(res, "version") <- packageDescription("meta")$Version
  
  res
}





#' @rdname rd
#' @method print rd.meta
#' @export

print.rd.meta <- function(x,
                          common = !is.null(x$common),
                          random = !is.null(x$random),
                          pscale = attributes(x)$pscale,
                          digits = gs("digits"),
                          digits.prop = gs("digits.prop"),
                          big.mark = gs("big.mark"),
                          details = gs("details"),
                          ...) {
  
  
  #
  # (1) Check for rd.meta object
  #
  chkclass(x, "rd.meta")
  
  
  #
  # (2) Check arguments
  #
  chklogical(common)
  chklogical(random)
  #
  chknumeric(pscale, length = 1)
  #
  chknumeric(digits, min = 0, length = 1)
  chknumeric(digits.prop, min = 0, length = 1)
  #
  chklogical(details)
  
  
  sm <- attributes(x)$sm
  sv <- attributes(x)$small.values
  #
  ci.lab <- paste0(round(100 * replaceNULL(attributes(x)$level), 1), "%-CI")
  p.lab <- if (sm == "HR") "Surv.c" else "Risk.c"
  
  if (common) {
    res.c <- x$common
    #
    with.ci <- sum(!is.na(res.c$lower)) > 0 & sum(!is.na(res.c$upper)) > 0
    #
    if (sm != "RD") {
      res.c$TE <- exp(res.c$TE)
      #
      if (with.ci) {
        res.c$lower <- exp(res.c$lower)
        res.c$upper <- exp(res.c$upper)
      }
    }
    #
    if (all(res.c$RD <= 0) & any(res.c$RD < 0) & sv == "desirable") {
      lab.c <- "ARR"
      title.c <- "Absolute risk reduction"
      #
      res.c$RD <- -res.c$RD
      if (with.ci) {
        tmp.u <- -res.c$upper.RD
        res.c$upper.RD <- -res.c$lower.RD
        res.c$lower.RD <- tmp.u
      }
    }
    else if (all(res.c$RD <= 0) & any(res.c$RD < 0) & sv == "undesirable") {
      lab.c <- "ABR"
      title.c <- "Absolute benefit reduction"
      #
      res.c$RD <- -res.c$RD
      #
      if (with.ci) {
        tmp.u <- -res.c$upper.RD
        res.c$upper.RD <- -res.c$lower.RD
        res.c$lower.RD <- tmp.u
      }
    }
    else if (all(res.c$RD >= 0) & any(res.c$RD > 0) & sv == "desirable") {
      lab.c <- "ARI"
      title.c <- "Absolute risk increase"
    }
    else if (all(res.c$RD >= 0) & any(res.c$RD > 0) & sv == "undesirable") {
      lab.c <- "ABI"
      title.c <- "Absolute benefit increase"
    }
    else {
      lab.c <- "RD"
      title.c <- "Risk difference"
    }
    #
    res.c$TE <- formatN(round(res.c$TE, digits), digits, big.mark = big.mark)
    #
    if (sm != "HR")
      res.c$p.c <- formatN(round(pscale * res.c$p.c, digits.prop),
                           digits.prop, big.mark = big.mark)
    else
      res.c$p.c <- formatN(round(res.c$p.c, 2), 2, big.mark = big.mark)
    #
    res.c$RD <- formatN(round(pscale * res.c$RD, digits.prop),
                        digits.prop, big.mark = big.mark)
    #
    if (with.ci) {
      res.c$lower <-
        formatCI(
          formatN(round(res.c$lower, digits), digits, big.mark = big.mark),
          formatN(round(res.c$upper, digits), digits, big.mark = big.mark))
      res.c$upper <- NULL
      #
      res.c$lower.RD <-
        formatCI(formatN(round(pscale * res.c$lower.RD, digits.prop),
                         digits.prop, big.mark = big.mark),
                 formatN(round(pscale * res.c$upper.RD, digits.prop),
                         digits.prop, big.mark = big.mark))
      res.c$upper.RD <- NULL
    }
    else {
      res.c$lower <- NULL
      res.c$upper <- NULL
      #
      res.c$lower.RD <- NULL
      res.c$upper.RD <- NULL
    }
    #
    no_column_label <- TE <- p.c <- RD <- NULL
    #
    res.c %<>%
      mutate(no_column_label = "") %>%
      relocate(no_column_label, .after = "p.c") %>%
      rename(!!lab.c := RD) %>%
      rename(!!sm := TE) %>%
      rename(!!p.lab := p.c) %>% as.matrix()
    #
    if (with.ci) {
      colnames(res.c)[colnames(res.c) == "lower"] <- ci.lab
      colnames(res.c)[colnames(res.c) == "lower.RD"] <- ci.lab
    }
    #
    colnames(res.c)[colnames(res.c) == "no_column_label"] <- ""
    #
    if (is.null(attributes(x)$rd.default))
      title <- paste0(title.c, " (", tolower(gs("text.common")), ")")
    else
      title <- title.c
    cat(paste0(title, "\n\n"))
    #
    prmatrix(res.c, quote = FALSE, right = TRUE,
             rowlab = rep("", nrow(res.c)))
    #
    if (random)
      cat("\n")
  }
  #
  if (random) {
    res.r <- x$random
    #
    with.ci <- sum(!is.na(res.r$lower)) > 0 & sum(!is.na(res.r$upper)) > 0
    #
    if (sm != "RD") {
      res.r$TE <- exp(res.r$TE)
      #
      if (with.ci) {
        res.r$lower <- exp(res.r$lower)
        res.r$upper <- exp(res.r$upper)
      }
    }
    #
    if (all(res.r$RD <= 0) & any(res.r$RD < 0) & sv == "desirable") {
      lab.r <- "ARR"
      title.r <- "Absolute risk reduction"
      #
      res.r$RD <- -res.r$RD
      if (with.ci) {
        tmp.u <- -res.r$upper.RD
        res.r$upper.RD <- -res.r$lower.RD
        res.r$lower.RD <- tmp.u
      }
    }
    else if (all(res.r$RD <= 0) & any(res.r$RD < 0) & sv == "undesirable") {
      lab.r <- "ABR"
      title.r <- "Absolute benefit reduction"
      #
      res.r$RD <- -res.r$RD
      #
      if (with.ci) {
        tmp.u <- -res.r$upper.RD
        res.r$upper.RD <- -res.r$lower.RD
        res.r$lower.RD <- tmp.u
      }
    }
    else if (all(res.r$RD >= 0) & any(res.r$RD > 0) & sv == "desirable") {
      lab.r <- "ARI"
      title.r <- "Absolute risk increase"
    }
    else if (all(res.r$RD >= 0) & any(res.r$RD > 0) & sv == "undesirable") {
      lab.r <- "ABI"
      title.r <- "Absolute benefit increase"
    }
    else {
      lab.r <- "RD"
      title.r <- "Risk difference"
    }
    #
    res.r$TE <- formatN(round(res.r$TE, digits), digits, big.mark = big.mark)
    #
    if (sm != "HR")
      res.r$p.c <- formatN(round(pscale * res.r$p.c, digits.prop),
                           digits.prop, big.mark = big.mark)
    else
      res.r$p.c <- formatN(round(res.r$p.c, 2), 2, big.mark = big.mark)
    #
    res.r$RD <- formatN(round(pscale * res.r$RD, digits.prop),
                        digits.prop, big.mark = big.mark)
    #
    if (with.ci) {
      res.r$lower <-
        formatCI(
          formatN(round(res.r$lower, digits), digits, big.mark = big.mark),
          formatN(round(res.r$upper, digits), digits, big.mark = big.mark))
      res.r$upper <- NULL
      #
      res.r$lower.RD <-
        formatCI(formatN(round(pscale * res.r$lower.RD, digits.prop),
                         digits.prop, big.mark = big.mark),
                 formatN(round(pscale * res.r$upper.RD, digits.prop),
                         digits.prop, big.mark = big.mark))
      res.r$upper.RD <- NULL
    }
    else {
      res.r$lower <- NULL
      res.r$upper <- NULL
      #
      res.r$lower.RD <- NULL
      res.r$upper.RD <- NULL
    }
    #
    no_column_label <- TE <- p.c <- RD <- NULL
    #
    res.r %<>%
      mutate(no_column_label = "") %>%
      relocate(no_column_label, .after = "p.c") %>%
      rename(!!lab.r := RD) %>%
      rename(!!sm := TE) %>%
      rename(!!p.lab := p.c) %>% as.matrix()
    #
    if (with.ci) {
      colnames(res.r)[colnames(res.r) == "lower"] <- ci.lab
      colnames(res.r)[colnames(res.r) == "lower.RD"] <- ci.lab
    }
    #
    colnames(res.r)[colnames(res.r) == "no_column_label"] <- ""
    #
    if (is.null(attributes(x)$rd.default))
      title <- paste0(title.r, " (", tolower(gs("text.random")), ")")
    else
      title <- title.r
    cat(paste0(title, "\n\n"))
    #
    prmatrix(res.r, quote = FALSE, right = TRUE,
             rowlab = rep("", nrow(res.r)))
  }
  
  if (details) {
    if (sm == "RD")
      abs.details <- ""
    else {
      abs.details <-
        paste0("\n- Absolute effect",
               if (length(unique(c(if (common) x$common$RD,
                                   if (random) x$random$RD))) > 1) "s",
               " calculated from ",
               tolower(xlab_meta(sm, TRUE)),
               if (length(unique(c(if (common) x$common$TE,
                                   if (random) x$random$TE))) > 1) "s",
               " (argument 'sm = \"", sm, "\"')")
    }
    #
    p.details <- paste0("\n- ", p.lab, ": ",
                        if (sm != "HR") "risk " else "survival probability ",
                        "in control group")
    #
    if (pscale != 1)
      ps.details <-
        paste0("\n- Events per ",
               format(pscale, scientific = FALSE, big.mark = big.mark),
               " observations")
    else
      ps.details <- ""
    #
    if (sv == "desirable") {
      sv.details <-
        paste0("\n- ", sm, " < ",
               if (sm == "RD") "0" else "1",
               " correspond to a beneficial effect of the intervention")
    }
    else {
      sv.details <-
        paste0("\n- ", sm, " > ",
               if (sm == "RD") "0" else "1",
               " corresponds to a beneficial effect of the intervention")
    }
    #
    sv.details <-
      paste0(sv.details, "\n  (argument 'small.values = \"", sv, "\"')")
    #
    cat(paste0("\nDetails:",
               abs.details, p.details, ps.details, sv.details, "\n"))
  }
  
  invisible(NULL)
}





#' @rdname rd
#' @method print rd.default
#' @export

print.rd.default <- function(x,
                             pscale = attributes(x)$pscale,
                             digits = gs("digits"),
                             digits.prop = gs("digits.prop"),
                             big.mark = gs("big.mark"),
                             ...) {
  
  #
  # (1) Check for rd.default object
  #
  chkclass(x, "rd.default")
  
  res <- vector("list")
  res$common <- x
  class(res$common) <- "data.frame"
  class(res) <- "rd.meta"
  #
  attr(res, "level") <- attributes(x)$level
  attr(res, "sm") <- attributes(x)$sm
  attr(res, "small.values") <- attributes(x)$small.values
  #
  attr(res, "pscale") <- attributes(x)$pscale
  #
  attr(res, "rd.default") <- TRUE
  #
  print(res, pscale = pscale, digits = digits,  digits.prop = digits.prop,
        big.mark = big.mark)
  #
  invisible(NULL)
}





#
# Auxiliary R functions
#
logRR2rd <- function(x, p.c)
  exp(x) * p.c - p.c
#
logOR2rd <- function(x, p.c)
  exp(x) * p.c / (1 - p.c + exp(x) * p.c) - p.c
#
logHR2rd <- function(x, Surv.c)
  (1 - Surv.c^exp(x)) - (1 - Surv.c)

#' Cisapride in Non-Ulcer Dispepsia
#' 
#' @description
#' Meta-analysis on cisapride in non-ulcer dispepsia.
#' 
#' This meta-analysis is used as a data example in Hartung and Knapp
#' (2001).
#' 
#' @name cisapride
#' 
#' @docType data
#' 
#' @format A data frame with the following columns:
#' \tabular{rl}{
#' \bold{\emph{study}}\tab study label \cr
#' \bold{\emph{event.cisa}}\tab number of events in cisapride group
#'   \cr
#' \bold{\emph{n.cisa}}\tab number of observations in cisapride group
#'   \cr
#' \bold{\emph{event.plac}}\tab number of events in placebo group \cr
#' \bold{\emph{n.plac}}\tab number of observations in placebo group
#' }
#' 
#' @seealso \code{\link{metabin}}
#' 
#' @source
#' Hartung J & Knapp G (2001):
#' A refined method for the meta-analysis of controlled clinical
#' trials with binary outcome.
#' \emph{Statistics in Medicine},
#' \bold{20}, 3875--89
#' 
#' @keywords datasets
#' 
#' @examples
#' data(cisapride)
#' 
#' m.or <- metabin(event.cisa, n.cisa, event.plac, n.plac,
#'   data = cisapride, sm = "OR",
#'   method = "Inverse", method.tau = "DL",
#'   studlab = study, method.incr = "all")
#' 
#' m.or.hk <- update(m.or, method.random.ci = "HK")
#' m.rr <- update(m.or, sm = "RR")
#' m.rr.hk <- update(m.or, sm = "RR", method.random.ci = "HK")
#'
#' vars.common <- c("TE.common", "lower.common", "upper.common")
#' vars.random <- c("TE.random", "lower.random", "upper.random")
#' #
#' res.common.or <- as.data.frame(m.or[vars.common])
#' names(res.common.or) <- vars.random
#' #
#' res.common.rr <- as.data.frame(m.rr[vars.common])
#' names(res.common.rr) <- vars.random
#' 
#' # Results for log risk ratio - see Table VII in Hartung and Knapp (2001) 
#' #
#' res.rr <- rbind(res.common.rr,
#'   as.data.frame(m.rr[vars.random]),
#'   as.data.frame(m.rr.hk[vars.random]))
#' #
#' row.names(res.rr) <- c("CE", "RE", "RE (HaKn)")
#' names(res.rr) <- c("Log risk ratio", "CI lower", "CI upper")
#' #
#' res.rr
#' 
#' 
#' # Results for log odds ratio (Table VII in Hartung and Knapp 2001) 
#' #
#' res.or <- rbind(res.common.or,
#'   as.data.frame(m.or[vars.random]),
#'   as.data.frame(m.or.hk[vars.random]))
#' #
#' row.names(res.or) <- c("CE", "RE", "RE (HaKn)")
#' names(res.or) <- c("Log odds ratio", "CI lower", "CI upper")
#' #
#' res.or


NULL

#' Generic inverse variance meta-analysis
#' 
#' @description
#' Common effect and random effects meta-analysis based on estimates
#' (e.g. log hazard ratios) and their standard errors. The inverse
#' variance method is used for pooling.
#'
#' Three-level random effects meta-analysis (Van den Noortgate et al., 2013) is
#' available by internally calling \code{\link[metafor]{rma.mv}} function from
#' R package \bold{metafor} (Viechtbauer, 2010).
#' 
#' @param TE Estimate of treatment effect, e.g., log hazard ratio or
#'   risk difference or an R object created with \code{\link{pairwise}}.
#' @param seTE Standard error of treatment estimate or standard deviation of
#'   n-of-1 trials.
#' @param studlab An optional vector with study labels.
#' @param data An optional data frame containing the study
#'   information.
#' @param subset An optional vector specifying a subset of studies to
#'   be used (see Details).
#' @param exclude An optional vector specifying studies to exclude
#'   from meta-analysis, however, to include in printouts and forest
#'   plots (see Details).
#' @param cluster An optional vector specifying which estimates come
#'   from the same cluster resulting in the use of a three-level
#'   meta-analysis model.
#' @param rho Assumed correlation of estimates within a cluster.
#' @param cycles A numeric vector with the number of cycles per patient / study
#'   in n-of-1 trials.
#' @param weights A single numeric or vector with user-specified weights.
#' @param weights.common User-specified weights (common effect model).
#' @param weights.random User-specified weights (random effects model).
#' @param sm A character string indicating underlying summary measure,
#'   e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
#'   \code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.
#' @param method.ci A character string indicating which method is used
#'   to calculate confidence intervals for individual studies, see
#'   Details.
#' @param level The level used to calculate confidence intervals for
#'   individual studies.
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param common A logical indicating whether a common effect
#'   meta-analysis should be conducted.
#' @param random A logical indicating whether a random effects
#'   meta-analysis should be conducted.
#' @param overall A logical indicating whether overall summaries
#'   should be reported. This argument is useful in a meta-analysis
#'   with subgroups if overall results should not be reported.
#' @param overall.hetstat A logical value indicating whether to print
#'   heterogeneity measures for overall treatment comparisons. This
#'   argument is useful in a meta-analysis with subgroups if
#'   heterogeneity statistics should only be printed on subgroup
#'   level.
#' @param prediction A logical indicating whether a prediction
#'   interval should be printed.
#' @param method.tau A character string indicating which method is
#'   used to estimate the between-study variance \eqn{\tau^2} and its
#'   square root \eqn{\tau} (see \code{\link{meta-package}}).
#' @param method.tau.ci A character string indicating which method is
#'   used to estimate the confidence interval of \eqn{\tau^2} and
#'   \eqn{\tau} (see \code{\link{meta-package}}).
#' @param level.hetstat The level used to calculate confidence intervals
#'   for heterogeneity statistics.
#' @param tau.preset Prespecified value for the square root of the
#'   between-study variance \eqn{\tau^2}.
#' @param TE.tau Overall treatment effect used to estimate the
#'   between-study variance tau-squared.
#' @param tau.common A logical indicating whether tau-squared should
#'   be the same across subgroups.
#' @param detail.tau Detail on between-study variance estimate.
#' @param method.I2 A character string indicating which method is
#'   used to estimate the heterogeneity statistic I\eqn{^2}. Either
#'   \code{"Q"} or \code{"tau2"}, can be abbreviated
#'   (see \code{\link{meta-package}}).
#' @param level.ma The level used to calculate confidence intervals
#'   for meta-analysis estimates.
#' @param method.common.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   common effect estimate (see \code{\link{meta-package}}).
#' @param method.random.ci A character string indicating which method
#'   is used to calculate confidence interval and test statistic for
#'   random effects estimate (see \code{\link{meta-package}}).
#' @param adhoc.hakn.ci A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied in the case
#'   of an arbitrarily small Hartung-Knapp variance estimate (see
#'   \code{\link{meta-package}}).
#' @param level.predict The level used to calculate prediction
#'   interval for a new study.
#' @param method.predict A character string indicating which method is
#'   used to calculate a prediction interval (see
#'   \code{\link{meta-package}}).
#' @param adhoc.hakn.pi A character string indicating whether an
#'   \emph{ad hoc} variance correction should be applied for
#'   prediction interval (see \code{\link{meta-package}}).
#' @param seed.predict A numeric value used as seed to calculate
#'   bootstrap prediction interval (see \code{\link{meta-package}}).
#' @param null.effect A numeric value specifying the effect under the
#'   null hypothesis.
#' @param method.bias A character string indicating which test is to
#'   be used. Either \code{"Begg"}, \code{"Egger"}, or
#'   \code{"Thompson"}, can be abbreviated. See function
#'   \code{\link{metabias}}.
#' @param n.e Number of observations in experimental group (or total
#'   sample size in study).
#' @param n.c Number of observations in control group.
#' @param pval P-value (used to estimate the standard error).
#' @param df Degrees of freedom (used in test or to construct
#'   confidence intervals).
#' @param lower Lower limit of confidence interval (used to estimate
#'   the standard error).
#' @param upper Upper limit of confidence interval (used to estimate
#'   the standard error).
#' @param level.ci Level of confidence interval.
#' @param median Median (used to estimate the treatment effect and
#'   standard error).
#' @param q1 First quartile (used to estimate the treatment effect and
#'   standard error).
#' @param q3 Third quartile (used to estimate the treatment effect and
#'   standard error).
#' @param min Minimum (used to estimate the treatment effect and
#'   standard error).
#' @param max Maximum (used to estimate the treatment effect and
#'   standard error).
#' @param method.mean A character string indicating which method to
#'   use to approximate the mean from the median and other statistics
#'   (see Details).
#' @param method.sd A character string indicating which method to use
#'   to approximate the standard deviation from sample size, median,
#'   interquartile range and range (see Details).
#' @param approx.TE Approximation method to estimate treatment
#'   estimate (see Details).
#' @param approx.seTE Approximation method to estimate standard error
#'   (see Details).
#' @param transf A logical indicating whether inputs for arguments
#'   \code{TE}, \code{lower} and \code{upper} are already
#'   appropriately transformed to conduct the meta-analysis or on the
#'   original scale. If \code{transf = TRUE} (default), inputs are
#'   expected to be log odds ratios instead of odds ratios for
#'   \code{sm = "OR"} and Fisher's z transformed correlations instead
#'   of correlations for \code{sm = "ZCOR"}, for example.
#' @param backtransf A logical indicating whether results should be
#'   back transformed in printouts and plots. If \code{backtransf =
#'   TRUE} (default), results for \code{sm = "OR"} are printed as odds
#'   ratios rather than log odds ratios and results for \code{sm =
#'   "ZCOR"} are printed as correlations rather than Fisher's z
#'   transformed correlations, for example.
#' @param func.transf A function used to transform inputs for
#'   arguments \code{TE}, \code{lower} and \code{upper}.
#' @param func.backtransf A function used to back-transform results.
#' @param args.transf An optional list to provide additional arguments
#'   to \code{func.transf}.
#' @param args.backtransf An optional list to provide additional
#'   arguments to \code{func.backtransf}.
#' @param pscale A numeric giving scaling factor for printing of
#'   single event probabilities or risk differences, i.e. if argument
#'   \code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
#'   \code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.
#' @param irscale A numeric defining a scaling factor for printing of
#'   single incidence rates or incidence rate differences, i.e. if
#'   argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
#'   \code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.
#' @param irunit A character specifying the time unit used to
#'   calculate rates, e.g. person-years.
#' @param text.common A character string used in printouts and forest
#'   plot to label the pooled common effect estimate.
#' @param text.random A character string used in printouts and forest
#'   plot to label the pooled random effects estimate.
#' @param text.predict A character string used in printouts and forest
#'   plot to label the prediction interval.
#' @param text.w.common A character string used to label weights of
#'   common effect model.
#' @param text.w.random A character string used to label weights of
#'   random effects model.
#' @param title Title of meta-analysis / systematic review.
#' @param complab Comparison label.
#' @param outclab Outcome label.
#' @param label.e Label for experimental group.
#' @param label.c Label for control group.
#' @param label.left Graph label on left side of null effect in forest plot.
#' @param label.right Graph label on right side of null effect in forest plot.
#' @param col.label.left The colour of the graph label on the left side of
#'   the null effect.
#' @param col.label.right The colour of the graph label on the right side of
#'   the null effect.
#' @param subgroup An optional vector to conduct a meta-analysis with
#'   subgroups.
#' @param subgroup.name A character string with a name for the
#'   subgroup variable.
#' @param print.subgroup.name A logical indicating whether the name of
#'   the subgroup variable should be printed in front of the group
#'   labels.
#' @param sep.subgroup A character string defining the separator
#'   between name of subgroup variable and subgroup label.
#' @param test.subgroup A logical value indicating whether to print
#'   results of test for subgroup differences.
#' @param prediction.subgroup A logical indicating whether prediction
#'   intervals should be printed for subgroups.
#' @param seed.predict.subgroup A numeric vector providing seeds to
#'   calculate bootstrap prediction intervals within subgroups. Must
#'   be of same length as the number of subgroups.
#' @param byvar Deprecated argument (replaced by 'subgroup').
#' @param id Deprecated argument (replaced by 'cluster').
#' @param adhoc.hakn Deprecated argument (replaced by
#'   'adhoc.hakn.ci').
#' @param keepdata A logical indicating whether original data (set)
#'   should be kept in meta object.
#' @param keeprma A logical indicating whether \code{\link[metafor]{rma.mv}}
#'   object from three-level meta-analysis should be stored.
#' @param warn A logical indicating whether warnings should be printed
#'   (e.g., if studies are excluded from meta-analysis due to zero
#'   standard errors).
#' @param warn.deprecated A logical indicating whether warnings should
#'   be printed if deprecated arguments are used.
#' @param control An optional list to control the iterative process to
#'   estimate the between-study variance \eqn{\tau^2}. This argument
#'   is passed on to \code{\link[metafor]{rma.uni}} or
#'   \code{\link[metafor]{rma.mv}}.
#' @param \dots Additional arguments (to catch deprecated arguments).
#' 
#' @details
#' This function provides the \emph{generic inverse variance method}
#' for meta-analysis which requires treatment estimates and their
#' standard errors (Borenstein et al., 2010). The method is useful,
#' e.g., for pooling of survival data (using log hazard ratio and
#' standard errors as input). Arguments \code{TE} and \code{seTE} can
#' be used to provide treatment estimates and standard errors
#' directly. However, it is possible to derive these quantities from
#' other information.
#' 
#' Argument \code{cycles} can be used to conduct a meta-analysis of n-of-1
#' trials according to Senn (2024). In this case, argument \code{seTE} does
#' not contain the standard error but standard deviation for individual
#' trials / patients. Trial-specific standard errors are calculated from an
#' average standard deviation multiplied by the number of cycles minus 1, i.e.,
#' the degrees of freedom. Details of the meta-analysis method are provided in
#' Senn (2024). Note, arguments used in the approximation of means or
#' standard errors, like \code{lower} and \code{upper}, or \code{df}, are
#' ignored for the meta-analysis of n-of-1 trials.
#' 
#' A three-level random effects meta-analysis model (Van den Noortgate
#' et al., 2013) is utilised if argument \code{cluster} is used and at
#' least one cluster provides more than one estimate. Internally,
#' \code{\link[metafor]{rma.mv}} is called to conduct the analysis and
#' \code{\link[metafor]{weights.rma.mv}} with argument \code{type =
#' "rowsum"} is used to calculate random effects weights.
#'
#' Default settings are utilised for several arguments (assignments
#' using \code{\link{gs}} function). These defaults can be changed for
#' the current R session using the \code{\link{settings.meta}}
#' function.
#' 
#' Furthermore, R function \code{\link{update.meta}} can be used to
#' rerun a meta-analysis with different settings.
#' 
#' \subsection{Approximate treatment estimates}{
#' 
#' Missing treatment estimates can be derived from
#' \enumerate{
#' \item confidence limits provided by arguments \code{lower} and
#'   \code{upper};
#' \item median, interquartile range and range (arguments
#'   \code{median}, \code{q1}, \code{q3}, \code{min}, and \code{max});
#' \item median and interquartile range (arguments \code{median},
#'   \code{q1} and \code{q3});
#' \item median and range (arguments \code{median}, \code{min} and
#'   \code{max}).
#' }
#' For confidence limits, the treatment estimate is defined as the
#' center of the confidence interval (on the log scale for relative
#' effect measures like the odds ratio or hazard ratio).
#'
#' If the treatment effect is a mean it can be approximated from
#' sample size, median, interquartile range and range.
#'
#' By default, methods described in Luo et al. (2018) are utilised
#' (argument \code{method.mean = "Luo"}):
#' \itemize{
#' \item equation (7) if sample size, median and range are available,
#' \item equation (11) if sample size, median and interquartile range
#'   are available,
#' \item equation (15) if sample size, median, range and interquartile
#'   range are available.
#' }
#' 
#' Instead the methods described in Wan et al. (2014) are used if
#' argument \code{method.mean = "Wan"}:
#' \itemize{
#' \item equation (2) if sample size, median and range are available,
#' \item equation (14) if sample size, median and interquartile range
#'   are available,
#' \item equation (10) if sample size, median, range and interquartile
#'   range are available.
#' }
#'
#' The following methods are also available to estimate means from
#' quantiles or ranges if R package \bold{estmeansd} is installed:
#' \itemize{
#' \item Method for Unknown Non-Normal Distributions (MLN) approach
#'   (Cai et al. (2021), argument \code{method.mean = "Cai"}),
#' \item Quantile Estimation (QE) method (McGrath et al. (2020),
#'   argument \code{method.mean = "QE-McGrath"})),
#' \item Box-Cox (BC) method (McGrath et al. (2020),
#'   argument \code{method.mean = "BC-McGrath"})).
#' }
#'
#' By default, missing treatment estimates are replaced successively
#' using these method, i.e., confidence limits are utilised before
#' interquartile ranges. Argument \code{approx.TE} can be used to
#' overwrite this default for each individual study:
#' \itemize{
#' \item Use treatment estimate directly (entry \code{""} in argument
#'   \code{approx.TE});
#' \item confidence limits (\code{"ci"} in argument \code{approx.TE});
#' \item median, interquartile range and range (\code{"iqr.range"});
#' \item median and interquartile range (\code{"iqr"});
#' \item median and range (\code{"range"}).
#' }
#' }
#'
#' \subsection{Approximate standard errors}{
#' 
#' Missing standard errors can be derived from
#' \enumerate{
#' \item p-value provided by arguments \code{pval} and (optional)
#'   \code{df};
#' \item confidence limits (arguments \code{lower}, \code{upper}, and
#'   (optional) \code{df});
#' \item sample size, median, interquartile range and range (arguments
#'   \code{n.e} and / or \code{n.c}, \code{median}, \code{q1},
#'   \code{q3}, \code{min}, and \code{max});
#' \item sample size, median and interquartile range (arguments
#'   \code{n.e} and / or \code{n.c}, \code{median}, \code{q1} and
#'   \code{q3});
#' \item sample size, median and range (arguments \code{n.e} and / or
#'   \code{n.c}, \code{median}, \code{min} and \code{max}).
#' }
#' For p-values and confidence limits, calculations are either based
#' on the standard normal or \emph{t}-distribution if argument
#' \code{df} is provided. Furthermore, argument \code{level.ci} can be
#' used to provide the level of the confidence interval.
#'
#' Wan et al. (2014) describe methods to estimate the standard
#' deviation (and thus the standard error by deviding the standard
#' deviation with the square root of the sample size) from the sample
#' size, median and additional statistics. Shi et al. (2020) provide
#' an improved estimate of the standard deviation if the interquartile
#' range and range are available in addition to the sample size and
#' median. Accordingly, equation (11) in Shi et al. (2020) is the
#' default (argument \code{method.sd = "Shi"}), if the median,
#' interquartile range and range are provided (arguments
#' \code{median}, \code{q1}, \code{q3}, \code{min} and
#' \code{max}). The method by Wan et al. (2014) is used if argument
#' \code{method.sd = "Wan"} and, depending on the sample size, either
#' equation (12) or (13) is used. If only the interquartile range or
#' range is available, equations (15) / (16) and (7) / (9) in Wan et
#' al. (2014) are used, respectively. The sample size of individual
#' studies must be provided with arguments \code{n.e} and / or
#' \code{n.c}. The total sample size is calculated as \code{n.e} +
#' \code{n.c} if both arguments are provided.
#'
#' The following methods are also available to estimate standard
#' deviations from quantiles or ranges if R package \bold{estmeansd}
#' is installed:
#' \itemize{
#' \item Method for Unknown Non-Normal Distributions (MLN) approach
#'   (Cai et al. (2021), argument \code{method.mean = "Cai"}),
#' \item Quantile Estimation (QE) method (McGrath et al. (2020),
#'   argument \code{method.mean = "QE-McGrath"})),
#' \item Box-Cox (BC) method (McGrath et al. (2020),
#'   argument \code{method.mean = "BC-McGrath"})).
#' }
#'
#' By default, missing standard errors are replaced successively using
#' these method, e.g., p-value before confidence limits before
#' interquartile range and range. Argument \code{approx.seTE} can be
#' used to overwrite this default for each individual study:
#' 
#' \itemize{
#' \item Use standard error directly (entry \code{""} in argument
#'   \code{approx.seTE});
#' \item p-value (\code{"pval"} in argument \code{approx.seTE});
#' \item confidence limits (\code{"ci"});
#' \item median, interquartile range and range (\code{"iqr.range"});
#' \item median and interquartile range (\code{"iqr"});
#' \item median and range (\code{"range"}).
#' }
#' }
#'
#' \subsection{Confidence intervals for individual studies}{
#' 
#' For the mean difference (argument \code{sm = "MD"}), the confidence
#' interval for individual studies can be based on the
#' \itemize{
#' \item standard normal distribution (\code{method.ci = "z"}), or
#' \item \emph{t}-distribution (\code{method.ci = "t"}).
#' }
#'
#' By default, the first method is used if argument \code{df} is
#' missing and the second method otherwise.
#' 
#' Note, this choice does not affect the results of the common effect
#' and random effects meta-analysis.
#' }
#'
#' \subsection{Subgroup analysis}{
#' 
#' Argument \code{subgroup} can be used to conduct subgroup analysis for
#' a categorical covariate. The \code{\link{metareg}} function can be
#' used instead for more than one categorical covariate or continuous
#' covariates.
#' }
#' 
#' \subsection{Specify the null hypothesis of test for an overall effect}{
#'
#' Argument \code{null.effect} can be used to specify the (treatment)
#' effect under the null hypothesis in a test for an overall
#' effect.
#'
#' By default (\code{null.effect = 0}), the null hypothesis
#' corresponds to "no difference" (which is obvious for absolute
#' effect measures like the mean difference (\code{sm = "MD"}) or
#' standardised mean difference (\code{sm = "SMD"})). For relative
#' effect measures, e.g., risk ratio (\code{sm = "RR"}) or odds ratio
#' (\code{sm = "OR"}), the null effect is defined on the log scale,
#' i.e., \emph{log}(RR) = 0 or \emph{log}(OR) = 0 which is equivalent
#' to testing RR = 1 or OR = 1.
#'
#' Use of argument \code{null.effect} is especially useful for summary
#' measures without a "natural" null effect, i.e., in situations
#' without a second (treatment) group. For example, an overall
#' proportion of 50\% could be tested in the meta-analysis of single
#' proportions with argument \code{null.effect = 0.5}.
#'
#' Note, all tests for an overall effect are two-sided with the
#' alternative hypothesis that the effect is unequal to
#' \code{null.effect}.
#' }
#' 
#' \subsection{Exclusion of studies from meta-analysis}{
#'
#' Arguments \code{subset} and \code{exclude} can be used to exclude
#' studies from the meta-analysis. Studies are removed completely from
#' the meta-analysis using argument \code{subset}, while excluded
#' studies are shown in printouts and forest plots using argument
#' \code{exclude} (see Examples).
#' Meta-analysis results are the same for both arguments.
#' }
#' 
#' \subsection{Presentation of meta-analysis results}{
#' 
#' Internally, both common effect and random effects models are
#' calculated regardless of values choosen for arguments \code{common}
#' and \code{random}. Accordingly, the estimate for the random effects
#' model can be extracted from component \code{TE.random} of an object
#' of class \code{"meta"} even if argument \code{random =
#' FALSE}. However, all functions in R package \bold{meta} will
#' adequately consider the values for \code{common} and
#' \code{random}. For example, functions \code{\link{print.meta}} and
#' \code{\link{forest.meta}} will not show results for the random
#' effects model if \code{random = FALSE}.
#'
#' A prediction interval will only be shown if \code{prediction =
#' TRUE}.
#' 
#' Argument \code{pscale} can be used to rescale single proportions or
#' risk differences, e.g. \code{pscale = 1000} means that proportions
#' are expressed as events per 1000 observations. This is useful in
#' situations with (very) low event probabilities.
#' 
#' Argument \code{irscale} can be used to rescale single rates or rate
#' differences, e.g. \code{irscale = 1000} means that rates are
#' expressed as events per 1000 time units, e.g. person-years. This is
#' useful in situations with (very) low rates. Argument \code{irunit}
#' can be used to specify the time unit used in individual studies
#' (default: "person-years"). This information is printed in summaries
#' and forest plots if argument \code{irscale} is not equal to 1.
#'
#' Default settings for \code{common}, \code{random},
#' \code{pscale}, \code{irscale}, \code{irunit} and several other
#' arguments can be set for the whole R session using
#' \code{\link{settings.meta}}.
#' }
#'
#' @note
#' R function \code{\link[metafor]{rma.uni}} from R package
#' \pkg{metafor} (Viechtbauer 2010) is called internally to estimate
#' the between-study variance \eqn{\tau^2}.
#' 
#' @return
#' An object of class \code{c("metagen", "meta")} with corresponding
#' generic functions (see \code{\link{meta-object}}).
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{meta-package}}, \code{\link{update.meta}},
#'   \code{\link{metabin}}, \code{\link{metacont}}, \code{\link{pairwise}},
#'   \code{\link{print.meta}}, \code{\link{settings.meta}}
#' 
#' @references
#' Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
#' A basic introduction to fixed-effect and random-effects models for
#' meta-analysis.
#' \emph{Research Synthesis Methods},
#' \bold{1}, 97--111
#'
#' Cai S, Zhou J, Pan J (2021):
#' Estimating the sample mean and standard deviation from order
#' statistics and sample size in meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{30}, 2701--2719
#' 
#' Luo D, Wan X, Liu J, Tong T (2018):
#' Optimally estimating the sample mean from the sample size, median,
#' mid-range, and/or mid-quartile range.
#' \emph{Statistical Methods in Medical Research},
#' \bold{27}, 1785--805
#'
#' McGrath S, Zhao X, Steele R, et al. and the DEPRESsion Screening
#' Data (DEPRESSD) Collaboration (2020):
#' Estimating the sample mean and standard deviation from commonly
#' reported quantiles in meta-analysis.
#' \emph{Statistical Methods in Medical Research},
#' \bold{29}, 2520--2537
#' 
#' Senn S (2024):
#' The analysis of continuous data from n-of-1 trials using paired cycles:
#' a simple tutorial.
#' \emph{Trials},
#' \bold{25}.
#' 
#' Shi J, Luo D, Weng H, Zeng X-T, Lin L, Chu H, et al. (2020):
#' Optimally estimating the sample standard deviation from the
#' five-number summary.
#' \emph{Research Synthesis Methods}.
#' 
#' Viechtbauer W (2010):
#' Conducting Meta-Analyses in R with the metafor Package.
#' \emph{Journal of Statistical Software},
#' \bold{36}, 1--48
#'
#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
#' Three-level meta-analysis of dependent effect sizes.
#' \emph{Behavior Research Methods},
#' \bold{45}, 576--94
#'
#' Wan X, Wang W, Liu J, Tong T (2014):
#' Estimating the sample mean and standard deviation from the sample
#' size, median, range and/or interquartile range.
#' \emph{BMC Medical Research Methodology},
#' \bold{14}, 135
#' 
#' @examples
#' data(Fleiss1993bin)
#' ma <- metabin(d.asp, n.asp, d.plac, n.plac, study,
#'   data = Fleiss1993bin, sm = "RR", method = "I")
#' ma
#' 
#' # Identical results using the generic inverse variance method with
#' # log risk ratio and its standard error:
#' # Note, argument 'n.e' in metagen() is used to provide the total
#' # sample size which is calculated from the group sample sizes n.e
#' # and n.c in meta-analysis ma.
#' ma.gen <- metagen(TE, seTE, studlab, n.e = n.e + n.c, data = ma, sm = "RR")
#' ma.gen
#' forest(ma.gen, leftcols = c("studlab", "n.e", "TE", "seTE"))
#' 
#' # Meta-analysis with prespecified between-study variance
#' #
#' metagen(ma$TE, ma$seTE, sm = "RR", tau.preset = sqrt(0.1))
#' 
#' # Meta-analysis of survival data:
#' #
#' logHR <- log(c(0.95, 1.5))
#' selogHR <- c(0.25, 0.35)
#' metagen(logHR, selogHR, sm = "HR")
#' 
#' # Paule-Mandel method to estimate between-study variance for data
#' # from Paule & Mandel (1982)
#' #
#' average <- c(27.044, 26.022, 26.340, 26.787, 26.796)
#' variance <- c(0.003, 0.076, 0.464, 0.003, 0.014)
#' #
#' metagen(average, sqrt(variance), sm = "MD", method.tau = "PM")
#' 
#' # Conduct meta-analysis using hazard ratios and 95% confidence intervals
#' #
#' # Data from Steurer et al. (2006), Analysis 1.1 Overall survival
#' # https://doi.org/10.1002/14651858.CD004270.pub2
#' #
#' study <- c("FCG on CLL 1996", "Leporrier 2001", "Rai 2000", "Robak 2000")
#' HR <- c(0.55, 0.92, 0.79, 1.18)
#' lower.HR <- c(0.28, 0.79, 0.59, 0.64)
#' upper.HR <- c(1.09, 1.08, 1.05, 2.17)
#' #
#' # Hazard ratios and confidence intervals as input
#' #
#' summary(metagen(HR, lower = lower.HR, upper = upper.HR,
#'   studlab = study, sm = "HR", transf = FALSE))
#' #
#' # Same result with log hazard ratios as input
#' #
#' summary(metagen(log(HR), lower = log(lower.HR), upper = log(upper.HR),
#'   studlab = study, sm = "HR"))
#' #
#' # Again, same result using an unknown summary measure and
#' # arguments 'func.transf' and 'func.backtransf'
#' #
#' summary(metagen(HR, lower = lower.HR, upper = upper.HR,
#'   studlab = study, sm = "Hazard ratio",
#'   func.transf = log, func.backtransf = exp))
#' #
#' # Finally, same result only providing argument 'func.transf' as the
#' # back-transformation for the logarithm is known
#' #
#' summary(metagen(HR, lower = lower.HR, upper = upper.HR,
#'   studlab = study, sm = "Hazard ratio",
#'   func.transf = log))
#'
#' # Exclude MRC-1 and MRC-2 studies from meta-analysis, however,
#' # show them in printouts and forest plots
#' #
#' metabin(d.asp, n.asp, d.plac, n.plac, study,
#'   data = Fleiss1993bin, sm = "RR", method = "I",
#'   exclude = study %in% c("MRC-1", "MRC-2"))
#' #
#' # Exclude MRC-1 and MRC-2 studies completely from meta-analysis
#' #
#' metabin(d.asp, n.asp, d.plac, n.plac, study,
#'   data = Fleiss1993bin, sm = "RR", method = "I",
#'   subset = !(study %in% c("MRC-1", "MRC-2")))
#'
#' # Exclude studies with total sample size above 1500
#' #
#' metabin(d.asp, n.asp, d.plac, n.plac, study,
#'   data = Fleiss1993bin, sm = "RR", method = "I",
#'   exclude = (n.asp + n.plac) > 1500)
#'
#' # Exclude studies containing "MRC" in study name
#' #
#' metabin(d.asp, n.asp, d.plac, n.plac, study,
#'   data = Fleiss1993bin, sm = "RR", method = "I",
#'   exclude = grep("MRC", study))
#'
#' # Use both arguments 'subset' and 'exclude'
#' #
#' metabin(d.asp, n.asp, d.plac, n.plac, study,
#'   data = Fleiss1993bin, sm = "RR", method = "I",
#'   subset = (n.asp + n.plac) > 1500,
#'   exclude = grep("MRC", study))
#'
#' \dontrun{
#' if (requireNamespace("metadat", quietly = TRUE)) {
#' # Three-level model: effects of modified school calendars on
#' # student achievement
#' data(dat.konstantopoulos2011, package = "metadat")
#' metagen(yi, sqrt(vi), studlab = study, data = dat.konstantopoulos2011,
#'   sm = "SMD",
#'   cluster = district, detail.tau = c("district", "district/school"))
#' }
#' }
#' 
#' @export metagen

metagen <- function(TE, seTE, studlab,
                    #
                    data = NULL, subset = NULL, exclude = NULL,
                    cluster = NULL, rho = 0,
                    #
                    cycles = NULL,
                    #
                    weights = NULL,
                    weights.common = weights, weights.random = weights,
                    #
                    sm = "",
                    #
                    method.ci = if (missing(df)) "z" else "t",
                    level = gs("level"),
                    #
                    common = gs("common"),
                    random = gs("random") | !is.null(tau.preset),
                    overall = common | random,
                    overall.hetstat =
                      if (is.null(gs("overall.hetstat")))
                        common | random
                      else
                        gs("overall.hetstat"),   
                    prediction = gs("prediction") | !missing(method.predict),
                    #
                    method.tau = gs("method.tau"),
                    method.tau.ci = gs("method.tau.ci"),
                    level.hetstat = gs("level.hetstat"),
                    tau.preset = NULL, TE.tau = NULL,
                    tau.common = gs("tau.common"),
                    detail.tau = NULL,
                    #
                    method.I2 = gs("method.I2"),
                    #
                    level.ma = gs("level.ma"),
                    method.common.ci = gs("method.common.ci"),
                    method.random.ci = gs("method.random.ci"),
                    adhoc.hakn.ci = gs("adhoc.hakn.ci"),
                    #
                    level.predict = gs("level.predict"),
                    method.predict = gs("method.predict"),
                    adhoc.hakn.pi = gs("adhoc.hakn.pi"),
                    seed.predict = NULL,
                    #
                    null.effect = 0,
                    #
                    method.bias = gs("method.bias"),
                    #
                    n.e = NULL, n.c = NULL,
                    #
                    pval, df, lower, upper, level.ci = 0.95,
                    median, q1, q3, min, max,
                    method.mean = "Luo",
                    method.sd = "Shi",
                    #
                    approx.TE, approx.seTE,
                    #
                    transf = gs("transf") & missing(func.transf),
                    backtransf = gs("backtransf") | !missing(func.backtransf),
                    func.transf,
                    func.backtransf,
                    args.transf,
                    args.backtransf,
                    pscale = 1,
                    irscale = 1, irunit = "person-years",
                    #
                    text.common = gs("text.common"),
                    text.random = gs("text.random"),
                    text.predict = gs("text.predict"),
                    text.w.common = gs("text.w.common"),
                    text.w.random = gs("text.w.random"),
                    #
                    title = gs("title"), complab = gs("complab"),
                    outclab = "",
                    #
                    label.e = gs("label.e"), label.c = gs("label.c"),
                    label.left = gs("label.left"),
                    label.right = gs("label.right"),
                    col.label.left = gs("col.label.left"),
                    col.label.right = gs("col.label.right"),
                    #
                    subgroup, subgroup.name = NULL,
                    print.subgroup.name = gs("print.subgroup.name"),
                    sep.subgroup = gs("sep.subgroup"),
                    test.subgroup = gs("test.subgroup"),
                    prediction.subgroup = gs("prediction.subgroup"),
                    seed.predict.subgroup = NULL,
                    #
                    byvar, id, adhoc.hakn,
                    #
                    keepdata = gs("keepdata"),
                    keeprma = gs("keeprma"),
                    #
                    warn = gs("warn"),
                    warn.deprecated = gs("warn.deprecated"),
                    #
                    control = NULL,
                    ...) {
  
  
  #
  #
  # (1) Check arguments
  #
  #
  
  missing.studlab <- missing(studlab)
  missing.sm <- missing(sm)
  missing.subgroup <- missing(subgroup)
  missing.overall <- missing(overall)
  missing.overall.hetstat <- missing(overall.hetstat)
  missing.test.subgroup <- missing(test.subgroup)
  missing.label.e <- missing(label.e)
  missing.label.c <- missing(label.c)
  missing.complab <- missing(complab)
  #
  missing.adhoc.hakn.pi <- missing(adhoc.hakn.pi)
  #
  missing.method.tau <- missing(method.tau)
  missing.tau.common <- missing(tau.common)
  avail.detail.tau <- !missing(detail.tau) & !is.null(detail.tau)
  missing.method.predict <- missing(method.predict)
  #
  missing.func.transf <- missing(func.transf)
  missing.args.transf <- missing(args.transf)
  missing.func.backtransf <- missing(func.backtransf)
  missing.args.backtransf <- missing(args.backtransf)
  #
  sm <- replaceNULL(sm, "")
  sm <- setchar(sm,
                unique(c(gs("sm4bin"), gs("sm4cont"), gs("sm4cor"),
                         gs("sm4inc"), gs("sm4mean"),
                         gs("sm4prop"), gs("sm4rate"), "")),
                stop.at.error = FALSE, return.NULL = FALSE,
                nchar.equal = TRUE)
  #
  method.ci <- setchar(method.ci, gs("ci4cont"))
  #
  method.mean <-
    setchar(method.mean, c("Luo", "Wan", "Cai", "QE-McGrath", "BC-McGrath"))
  method.sd <-
    setchar(method.sd, c("Shi", "Wan", "Cai", "QE-McGrath", "BC-McGrath"))
  #
  if (method.mean %in% c("Cai", "QE-McGrath", "BC-McGrath"))
    is_installed_package("estmeansd", argument = "method.mean",
                         value = method.mean)
  if (method.sd %in% c("Cai", "QE-McGrath", "BC-McGrath"))
    is_installed_package("estmeansd", argument = "method.sd",
                         value = method.sd)
  #
  chklevel(level)
  #
  if (length(method.common.ci) != 1)
    stop("Argument 'method.common.ci' must be of length 1.",
         call. = FALSE)
  method.common.ci <- setchar(method.common.ci, gs("meth4common.ci"))
  #
  method.tau <- setchar(method.tau, gs("meth4tau"))
  #
  tau.common <- replaceNULL(tau.common, FALSE)
  chklogical(tau.common)
  #
  method.I2 <- setchar(method.I2, gs("meth4i2"))
  #
  chklogical(prediction)
  chklevel(level.predict)
  #
  chknumeric(rho, min = -1, max = 1)
  #
  method.predict <- setchar(method.predict, gs("meth4pi"))
  #
  method.tau <-
    set_method_tau(method.tau, missing.method.tau,
                   method.predict, missing.method.predict)
  method.predict <-
    set_method_predict(method.predict, missing.method.predict,
                       method.tau, missing.method.tau)
  #
  if (any(method.predict == "NNF"))
    is_installed_package("pimeta", argument = "method.predict", value = "NNF")
  #
  adhoc.hakn.pi <- setchar(replaceNA(adhoc.hakn.pi, ""), gs("adhoc4hakn.pi"))
  #
  if (!is.null(seed.predict))
    chknumeric(seed.predict, length = 1)
  #
  chknumeric(null.effect, length = 1)
  #
  chklogical(transf)
  chklogical(backtransf)
  #
  avail.func.transf <- !missing.func.transf && !is.null(func.transf)
  avail.args.transf <- !missing.args.transf && !is.null(args.transf)
  avail.func.backtransf <-
    !missing.func.backtransf && !is.null(func.backtransf)
  avail.args.backtransf <-
    !missing.args.backtransf && !is.null(args.backtransf)
  #
  if (avail.func.transf) {
    chkfunc(func.transf)
    if (is.function(func.transf))
      func.transf <- deparse(substitute(func.transf))
  }
  else
    func.transf <- NULL
  #
  if (avail.args.transf)
    chklist(args.transf)
  else
    args.transf <- NULL
  #
  if (avail.func.backtransf) {
    chkfunc(func.backtransf)    
    if (is.function(func.backtransf))
      func.backtransf <- deparse(substitute(func.backtransf))
  }
  else
    func.backtransf <- NULL
  #
  if (avail.args.backtransf)
    chklist(args.backtransf)    
  else
    args.backtransf <- NULL
  #
  if (is.null(func.transf) & !is.null(args.transf)) {
    warning("Argument 'args.transf' ignored as argument ",
            "'func.transf' is ",
            if (!avail.func.transf) "missing." else "NULL.",
            call. = FALSE)
    args.transf <- NULL
  }
  #
  if (is.null(func.backtransf) & !is.null(args.backtransf)) {
    warning("Argument 'args.backtransf' ignored as argument ",
            "'func.backtransf' is ",
            if (!avail.func.backtransf) "missing." else "NULL.",
            call. = FALSE)
    args.backtransf <- NULL
  }
  #
  if (!is.null(func.transf) & is.null(func.backtransf)) {
    if (func.transf == "log" & is.null(args.transf))
      func.backtransf <- "exp"
    else if (func.transf == "cor2z" & is.null(args.transf))
      func.backtransf <- "z2cor"
    else if (func.transf == "p2logit" & is.null(args.transf))
      func.backtransf <- "logit2p"
    else if (func.transf == "p2asin" & is.null(args.transf))
      func.backtransf <- "asin2p"
    else if (func.transf == "VE2logVR" & is.null(args.transf))
      func.backtransf <- "logVR2VE"
    else
      stop("Argument 'func.backtransf' must be specified.",
           call. = FALSE)
  }
  #
  missing.pscale <- missing(pscale)
  chknumeric(pscale, length = 1)
  if (!backtransf & pscale != 1) {
    if (!missing.pscale)
      warning("Argument 'pscale' set to 1 as argument 'backtransf' is FALSE.",
              call. = FALSE)
    pscale <- 1
  }
  missing.irscale <- missing(irscale)
  chknumeric(irscale, length = 1)
  if (!backtransf & irscale != 1) {
    if (!missing.irscale)
      warning("Argument 'irscale' set to 1 as argument 'backtransf' is FALSE.",
              call. = FALSE)
    irscale <- 1
  }
  #
  if (!is.null(text.common))
    chkchar(text.common, length = 1)
  if (!is.null(text.random))
    chkchar(text.random)
  if (!is.null(text.predict))
    chkchar(text.predict)
  if (!is.null(text.w.common))
    chkchar(text.w.common, length = 1)
  if (!is.null(text.w.random))
    chkchar(text.w.random, length = 1)
  #
  chkchar(label.e, length = 1)
  chkchar(label.c, length = 1)
  #
  chklogical(keepdata)
  chklogical(keeprma)
  #
  # Additional arguments / checks
  #
  fun <- "metagen"
  chklogical(warn)
  #
  # Check for deprecated arguments in '...'
  #
  args  <- list(...)
  chklogical(warn.deprecated)
  #
  level.ma <- deprecated(level.ma, missing(level.ma), args, "level.comb",
                         warn.deprecated)
  chklevel(level.ma)
  #
  missing.common <- missing(common)
  common <- deprecated(common, missing.common, args, "comb.fixed",
                       warn.deprecated)
  common <- deprecated(common, missing.common, args, "fixed",
                       warn.deprecated)
  chklogical(common)
  #
  random <- deprecated(random, missing(random), args, "comb.random",
                       warn.deprecated)
  chklogical(random)
  #
  method.random.ci <-
    deprecated(method.random.ci, missing(method.random.ci),
               args, "hakn", warn.deprecated)
  if (is.logical(method.random.ci))
    if (method.random.ci)
      method.random.ci <- "HK"
    else
      method.random.ci <- "classic"
  method.random.ci <- setchar(method.random.ci, gs("meth4random.ci"))
  #
  missing.adhoc.hakn.ci <- missing(adhoc.hakn.ci)
  adhoc.hakn.ci <-
    deprecated2(adhoc.hakn.ci, missing.adhoc.hakn.ci,
                adhoc.hakn, missing(adhoc.hakn), warn.deprecated)
  adhoc.hakn.ci <- setchar(replaceNA(adhoc.hakn.ci, ""), gs("adhoc4hakn.ci"))
  #
  missing.subgroup.name <- missing(subgroup.name)
  subgroup.name <-
    deprecated(subgroup.name, missing.subgroup.name, args, "bylab",
               warn.deprecated)
  #
  print.subgroup.name <-
    deprecated(print.subgroup.name, missing(print.subgroup.name),
               args, "print.byvar", warn.deprecated)
  print.subgroup.name <-
    replaceNULL(print.subgroup.name, gs("print.subgroup.name"))
  chklogical(print.subgroup.name)
  #
  sep.subgroup <-
    deprecated(sep.subgroup, missing(sep.subgroup), args, "byseparator",
               warn.deprecated)
  if (!is.null(sep.subgroup))
    chkchar(sep.subgroup, length = 1)
  #
  # Some more checks
  #
  chklogical(overall)
  chklogical(overall.hetstat)
  
  
  #
  #
  # (2) Read data
  #
  #
  
  nulldata <- is.null(data)
  sfsp <- sys.frame(sys.parent())
  mc <- match.call()
  #
  if (nulldata) {
    data <- sfsp
    data.pairwise <- FALSE
  }
  else
    data.pairwise <- inherits(data, "pairwise")
  #
  missing.byvar <- missing(byvar)
  byvar <- catch("byvar", mc, data, sfsp)
  #
  # Catch 'TE', 'seTE', 'median', 'lower', 'upper', 'n.e', 'n.c', and
  # 'cluster' from data:
  #
  missing.TE <- missing(TE)
  missing.seTE <- missing(seTE)
  missing.median <- missing(median)
  missing.lower <- missing(lower)
  missing.upper <- missing(upper)
  #
  if (missing.TE & missing.median & (missing.lower | missing.upper))
    stop("Treatment estimates missing. ",
         "Provide either argument 'TE' or 'median', ",
         "or arguments 'lower' and 'upper'.",
         call. = FALSE)
  #
  TE <- catch("TE", mc, data, sfsp)
  avail.TE <- !(missing.TE || is.null(TE))
  #
  if (inherits(TE, "pairwise")) {
    is.pairwise <- TRUE
    #
    txt.ignore <- "as first argument is a pairwise object"
    #
    warn_ignore_input(sm, !missing.sm, txt.ignore)
    warn_ignore_input(seTE, !missing.seTE, txt.ignore)
    warn_ignore_input(studlab, !missing.studlab, txt.ignore)
    #
    sm <- attr(TE, "sm")
    reference.group <- attr(TE, "reference.group")
    #
    if (is.null(attr(TE, "varnames")))
      seTE <- TE$seTE
    else
      seTE <- TE[[attr(TE, "varnames")[2]]]
    #
    missing.sm <- FALSE
    missing.seTE <- FALSE
    missing.studlab <- FALSE
    #
    studlab <- TE$studlab
    #
    treat1 <- TE$treat1
    treat2 <- TE$treat2
    #
    if (!is.null(TE$n1))
      n.e <- TE$n1
    else
      n.e <- NULL
    #
    if (!is.null(TE$n2))
      n.c <- TE$n2
    else
      n.c <- NULL
    #
    pairdata <- TE
    data <- TE
    nulldata <- FALSE
    #
    if (is.null(attr(TE, "varnames")))
      TE <- TE$TE
    else
      TE <- TE[[attr(TE, "varnames")[1]]]
    #
    avail.TE <- !is.null(TE)
    #
    wo <- treat1 == reference.group
    #
    if (any(wo)) {
      TE[wo] <- -TE[wo]
      #
      ttreat1 <- treat1
      treat1[wo] <- treat2[wo]
      treat2[wo] <- ttreat1[wo]
      #
      if (!is.null(n.e) & !is.null(n.c)) {
        tn.e <- n.e
        n.e[wo] <- n.c[wo]
        n.c[wo] <- tn.e[wo]
      }
    }
    #
    if (missing.subgroup) {
      subgroup <- paste(treat1, treat2, sep = " vs ")
      #
      if (length(unique(subgroup)) == 1) {
        if (missing.complab)
          complab <- unique(subgroup)
        #
        if (missing.label.e)
          label.e <- unique(treat1)
        if (missing.label.c)
          label.c <- unique(treat2)
        #
        subgroup <- NULL
      }
      else {
        if (missing.overall)
          overall <- FALSE
        if (missing.overall.hetstat)
          overall.hetstat <- FALSE
        if (missing.test.subgroup)
          test.subgroup <- FALSE
      }
    }
    else
      subgroup <- catch("subgroup", mc, data, sfsp)
  }
  else {
    is.pairwise <- FALSE
    #
    if (missing.sm)
      if (!is.null(data) && !is.null(attr(data, "sm")))
        sm <- attr(data, "sm")
    else
      sm <- ""
    #
    seTE <- catch("seTE", mc, data, sfsp)
    #
    studlab <- catch("studlab", mc, data, sfsp)
    #
    subgroup <- catch("subgroup", mc, data, sfsp)
    subgroup <- deprecated2(subgroup, missing.subgroup, byvar, missing.byvar,
                            warn.deprecated)
    #
    if (!missing(n.e))
      n.e <- catch("n.e", mc, data, sfsp)
    if (!missing(n.c))
      n.c <- catch("n.c", mc, data, sfsp)
  }
  #
  method.bias <- setmethodbias(method.bias)
  #
  by <- !is.null(subgroup)
  #
  median <- catch("median", mc, data, sfsp)
  lower <- catch("lower", mc, data, sfsp)
  upper <- catch("upper", mc, data, sfsp)
  #
  avail.median <- !(missing.median || is.null(median))
  avail.lower <- !(missing.lower || is.null(lower))
  avail.upper <- !(missing.upper || is.null(upper))
  #
  if (!avail.TE & !avail.median & (!avail.lower | !avail.upper))
    stop("Treatment estimates missing. ",
         "Provide either argument 'TE' or 'median', ",
         "or arguments 'lower' and 'upper'.",
         call. = FALSE)
  #
  TE.orig <- NULL
  lower.orig <- NULL
  upper.orig <- NULL
  #
  if (!transf) {
    if (avail.TE) {
      TE.orig <- TE
      TE <- transf(TE, sm, func.transf, args.transf)
    }
    if (avail.lower) {
      lower.orig <- lower
      lower <- transf(lower, sm, func.transf, args.transf)
    }
    if (avail.upper) {
      upper.orig <- upper
      upper <- transf(upper, sm, func.transf, args.transf)
    }
    if (sm == "VE" && avail.lower & avail.upper) {
      tmp.l <- lower
      lower <- upper
      upper <- tmp.l
      #
      tmp.l <- lower.orig
      lower.orig <- upper.orig
      upper.orig <- tmp.l
    }   
  }
  #
  missing.cluster <- missing(cluster)
  cluster <- catch("cluster", mc, data, sfsp)
  missing.id <- missing(id)
  id <- catch("id", mc, data, sfsp)
  #
  cluster <- deprecated2(cluster, missing.cluster, id, missing.id,
                         warn.deprecated)
  with.cluster <- !is.null(cluster)
  #
  if (with.cluster)
    idx <- seq_along(cluster)
  #
  missing.cycles <- missing(cycles)
  cycles <- catch("cycles", mc, data, sfsp)
  with.cycles <- !is.null(cycles)
  #
  if (!missing(weights))
    weights <- catch("weights", mc, data, sfsp)
  if (!missing(weights.common))
    weights.common <- catch("weights.common", mc, data, sfsp)
  if (!missing(weights.random))
    weights.random <- catch("weights.random", mc, data, sfsp)
  #
  if (!is.null(weights) & is.null(weights.common))
    weights.common <- weights
  #
  if (!is.null(weights) & is.null(weights.random))
    weights.random <- weights
  #
  usw.common <- !is.null(weights.common)
  usw.random <- !is.null(weights.random)
  #
  if (usw.common)
    chknumeric(weights.common, min = 0)
  #
  if (usw.random)
    chknumeric(weights.random, min = 0)
  #
  if (with.cycles) {
    chknumeric(cycles, min = 1)
    #
    if (method.ci != "z")
      method.ci <- "z"
  }
  #
  if (with.cluster & with.cycles)
    stop("Arguments 'cluster' (multi-level model) and 'cycles' (n-of-1 trials)",
         " cannot be used together.",
         call. = FALSE)
  #
  missing.method.tau.ci <- missing(method.tau.ci)
  #
  k.All <- if (avail.TE)
             length(TE)
           else if (avail.median)
             length(median)
           else if (avail.lower)
             length(lower)
           else if (avail.upper)
             length(upper)
           else
             NA
  #
  if (!avail.TE)
    TE <- rep_len(NA, k.All)
  #
  if (missing.seTE)
    seTE <- rep_len(NA, k.All)
  #
  # Catch 'studlab', 'subset', and 'exclude' from data:
  #
  studlab <- catch("studlab", mc, data, sfsp)
  studlab <- setstudlab(studlab, k.All)
  #
  subset <- catch("subset", mc, data, sfsp)
  missing.subset <- is.null(subset)
  #
  exclude <- catch("exclude", mc, data, sfsp)
  missing.exclude <- is.null(exclude)
  #
  # Catch 'pval', 'df', 'level.ci', 'q1', 'q3', 'min', 'max',
  # 'approx.TE' and 'approx.seTE', from data:
  #
  missing.pval <- missing(pval)
  pval <- catch("pval", mc, data, sfsp)
  avail.pval <- !(missing.pval || is.null(pval))
  #
  missing.df <- missing(df)
  df <- catch("df", mc, data, sfsp)
  avail.df <- !(missing.df || is.null(df))
  #
  if (!missing(level.ci))
    level.ci <- catch("level.ci", mc, data, sfsp)
  #
  missing.q1 <- missing(q1)
  q1 <- catch("q1", mc, data, sfsp)
  avail.q1 <- !(missing.q1 || is.null(q1))
  #
  missing.q3 <- missing(q3)
  q3 <- catch("q3", mc, data, sfsp)
  avail.q3 <- !(missing.q3 || is.null(q3))
  #
  missing.min <- missing(min)
  min <- catch("min", mc, data, sfsp)
  avail.min <- !(missing.min || is.null(min))
  #
  missing.max <- missing(max)
  max <- catch("max", mc, data, sfsp)
  avail.max <- !(missing.max || is.null(max))
  #
  missing.approx.TE <- missing(approx.TE)
  approx.TE <- catch("approx.TE", mc, data, sfsp)
  avail.approx.TE <- !(missing.approx.TE || is.null(approx.TE))
  #
  missing.approx.seTE <- missing(approx.seTE)
  approx.seTE <- catch("approx.seTE", mc, data, sfsp)
  avail.approx.seTE <- !(missing.approx.seTE || is.null(approx.seTE))
  #
  missing.text.random <- missing(text.random)
  
  
  #
  #
  # (3) Check length of essential variables
  #
  #
  
  arg <- if (avail.TE) "TE" else "median"
  chklength(seTE, k.All, arg)
  chklength(studlab, k.All, arg)
  #
  if (by) {
    chklength(subgroup, k.All, arg)
    chklogical(test.subgroup)
    chklogical(prediction.subgroup)
  }
  #
  # Additional checks
  #
  if (!by & tau.common) {
    warning("Value for argument 'tau.common' set to FALSE as ",
            "argument 'subgroup' is missing.",
            call. = FALSE)
    tau.common <- FALSE
  }
  if (by & !tau.common & !is.null(tau.preset)) {
    warning("Argument 'tau.common' set to TRUE as ",
            "argument tau.preset is not NULL.",
            call. = FALSE)
    tau.common <- TRUE
  }
  #
  if (usw.common) {
    if (length(weights.common) == 1)
      weights.common <- rep(weights.common, k.All)
    else
      chklength(weights.common, k.All, arg)
  }
  #
  if (usw.random) {
    if (length(weights.random) == 1)
      weights.random <- rep(weights.random, k.All)
    else
      chklength(weights.random, k.All, arg)
  }
  #
  if (!is.null(n.e))
    chklength(n.e, k.All, arg)
  if (!is.null(n.c))
    chklength(n.c, k.All, arg)
  if (with.cluster)
    chklength(cluster, k.All, arg)
  #
  if (avail.approx.TE) {
    if (length(approx.TE) == 1)
      rep_len(approx.TE, k.All)
    else
      chklength(approx.TE, k.All, arg)
    #
    approx.TE <- setchar(approx.TE, c("", "ci", "iqr.range", "iqr", "range"))
  }
  #
  if (avail.approx.seTE) {
    if (length(approx.seTE) == 1)
      rep_len(approx.seTE, k.All)
    else
      chklength(approx.seTE, k.All, arg)
    #
    approx.seTE <- setchar(approx.seTE,
                           c("", "pval", "ci", "iqr.range", "iqr", "range"))
  }
  #
  if (avail.pval)
    chklength(pval, k.All, arg)
  if (avail.df)
    chklength(df, k.All, arg)
  if (avail.lower)
    chklength(lower, k.All, arg)
  if (avail.upper)
    chklength(upper, k.All, arg)
  if (length(level.ci) == 1)
    level.ci <- rep_len(level.ci, k.All)
  else if (!is.null(level.ci))
    chklength(level.ci, k.All, arg)
  if (avail.median)
    chklength(median, k.All, arg)
  if (avail.q1)
    chklength(q1, k.All, arg)
  if (avail.q3)
    chklength(q3, k.All, arg)
  if (avail.min)
    chklength(min, k.All, arg)
  if (avail.max)
    chklength(max, k.All, arg)
  if (with.cycles)
    chklength(cycles, k.All, arg)
  
  
  #
  #
  # (4) Subset, exclude studies, and subgroups
  #
  #
  
  if (!missing.subset)
    if ((is.logical(subset) & (sum(subset) > k.All)) ||
        (length(subset) > k.All))
      stop("Length of argument 'subset' is larger than number of studies.")
  #  
  if (!missing.exclude) {
    if ((is.logical(exclude) & (sum(exclude) > k.All)) ||
        (length(exclude) > k.All))
      stop("Length of argument 'exclude' is larger than number of studies.")
    #
    exclude2 <- rep(FALSE, k.All)
    exclude2[exclude] <- TRUE
    exclude <- exclude2
  }
  else
    exclude <- rep(FALSE, k.All)
  
  
  #
  #
  # (5) Store complete dataset in list object data
  #     (if argument keepdata is TRUE)
  #
  #
  
  if (keepdata) {
    if (inherits(data, "meta")) {
      data <- data$data
      if (isCol(data, ".subset"))
        data <- data[data$.subset, ]
    }
    else if (nulldata & !is.pairwise)
      data <- data.frame(.studlab = studlab)
    else if (nulldata & is.pairwise) {
      data <- pairdata
      data$.studlab <- studlab
    }
    else
      data$.studlab <- studlab
    #
    data$.TE <- TE
    data$.seTE <- seTE
    #
    if (by)
      data$.subgroup <- subgroup
    #
    if (!missing.subset) {
      if (length(subset) == dim(data)[1])
        data$.subset <- subset
      else {
        data$.subset <- FALSE
        data$.subset[subset] <- TRUE
      }
    }
    #
    if (!missing.exclude)
      data$.exclude <- exclude
    #
    if (with.cluster) {
      data$.id <- data$.cluster <- cluster
      data$.idx <- idx
    }
    #
    if (with.cycles)
      data$.cycles <- cycles
    #
    if (usw.common)
      data$.weights.common <- weights.common
    #
    if (usw.random)
      data$.weights.random <- weights.random
    #
    if (avail.pval)
      data$.pval <- pval
    if (avail.df)
      data$.df <- df
    #
    if (avail.lower)
      data$.lower <- lower
    if (avail.upper)
      data$.upper <- upper
    if (avail.lower | avail.upper)
      data$.level.ci <- level.ci
    #
    if (avail.median)
      data$.median <- median
    if (avail.q1)
      data$.q1 <- q1
    if (avail.q3)
      data$.q3 <- q3
    #
    if (avail.min)
      data$.min <- min
    if (avail.max)
      data$.max <- max
    #
    if (avail.approx.TE)
      data$.approx.TE <- approx.TE
    if (avail.approx.seTE)
      data$.approx.seTE <- approx.seTE
    #
    if (!is.null(n.e))
      data$.n.e <- n.e
    if (!is.null(n.c))
      data$.n.c <- n.c
    #
    if (!is.null(TE.orig))
      data$.TE.orig <- TE.orig
    #
    if (!is.null(lower.orig))
      data$.lower.orig <- lower.orig
    #
    if (!is.null(upper.orig))
      data$.upper.orig <- upper.orig
  }
  
  
  #
  #
  # (6) Use subset for analysis
  #
  #
  
  if (!missing.subset) {
    TE <- TE[subset]
    seTE <- seTE[subset]
    studlab <- studlab[subset]
    #
    exclude <- exclude[subset]
    #
    if (with.cluster) {
      cluster <- cluster[subset]
      idx <- idx[subset]
    }
    #
    if (with.cycles)
      cycles <- cycles[subset]
    #
    if (by)
      subgroup <- subgroup[subset]
    #
    if (!is.null(n.e))
      n.e <- n.e[subset]
    if (!is.null(n.c))
      n.c <- n.c[subset]
    #
    if (avail.pval)
      pval <- pval[subset]
    if (avail.df)
      df <- df[subset]
    if (avail.lower)
      lower <- lower[subset]
    if (avail.upper)
      upper <- upper[subset]
    level.ci <- level.ci[subset]
    if (avail.median)
      median <- median[subset]
    if (avail.q1)
      q1 <- q1[subset]
    if (avail.q3)
      q3 <- q3[subset]
    if (avail.min)
      min <- min[subset]
    if (avail.max)
      max <- max[subset]
    if (avail.approx.TE)
      approx.TE <- approx.TE[subset]
    if (avail.approx.seTE)
      approx.seTE <- approx.seTE[subset]
  }
  #
  if (missing.subgroup & is.pairwise & by) {
    if (length(unique(subgroup)) == 1) {
      by <- FALSE
      #
      if (missing.complab)
        complab <- unique(subgroup)
      #
      subgroup <- NULL
      #
      if (keepdata)
        data$.subgroup <- NULL
      #
      if (missing.overall)
        overall <- TRUE
      if (missing.overall.hetstat)
        overall.hetstat <- TRUE
    }
  }
    
  
  #
  # Determine total number of studies
  #
  k.all <- length(TE)
  #
  if (k.all == 0)
    stop("No studies to combine in meta-analysis.")
  #
  # No meta-analysis for a single study
  #
  if (k.all == 1) {
    common <- FALSE
    #
    random <- FALSE
    method.random.ci <- "classic"
    adhoc.hakn.ci <- ""
    #
    prediction <- FALSE
    method.predict <- "V"
    adhoc.hakn.pi <- ""
    #
    overall <- FALSE
    overall.hetstat <- FALSE
  }
  #
  # Check variable values
  #
  chknumeric(TE)
  chknumeric(seTE, 0)
  
  
  #
  #
  # (7) Calculate standard error from other information
  #
  #
  
  if (!with.cycles) {
    if (!avail.approx.seTE) {
      approx.seTE <- rep_len("", length(TE))
      #
      # Use confidence limits
      #
      sel.NA <- is.na(seTE)
      if (any(sel.NA) & avail.lower & avail.upper) {
        j <- sel.NA & !is.na(lower) & !is.na(upper)
        approx.seTE[j] <- "ci"
        if (!avail.df)
          seTE[j] <- TE.seTE.ci(lower[j], upper[j], level.ci[j])$seTE
        else
          seTE[j] <- TE.seTE.ci(lower[j], upper[j], level.ci[j], df[j])$seTE
      }
      #
      # Use p-values
      #
      sel.NA <- is.na(seTE)
      if (any(sel.NA) & avail.pval) {
        j <- sel.NA & !is.na(TE) & !is.na(pval)
        approx.seTE[j] <- "pval"
        if (!avail.df)
          seTE[j] <- seTE.pval(TE[j], pval[j])$seTE
        else
          seTE[j] <- seTE.pval(TE[j], pval[j], df[j])$seTE
      }
      #
      # Use IQR and range
      #
      sel.NA <- is.na(seTE)
      if (any(sel.NA) &
          avail.median & avail.q1 & avail.q3 & avail.min & avail.max &
          !(is.null(n.e) & is.null(n.c))) {
        j <- sel.NA & !is.na(median) & !is.na(q1) & !is.na(q3) &
          !is.na(min) & !is.na(max)
        approx.seTE[j] <- "iqr.range"
        if (is.null(n.c))
          seTE[j] <- mean_sd_iqr_range(n.e[j], median[j], q1[j], q3[j],
                                       min[j], max[j],
                                       method.sd = method.sd)$se
        else if (is.null(n.e))
          seTE[j] <- mean_sd_iqr_range(n.c[j], median[j], q1[j], q3[j],
                                       min[j], max[j],
                                       method.sd = method.sd)$se
        else
          seTE[j] <- mean_sd_iqr_range(n.e[j] + n.c[j], median[j], q1[j], q3[j],
                                       min[j], max[j],
                                       method.sd = method.sd)$se
      }
      #
      # Use IQR
      #
      sel.NA <- is.na(seTE)
      if (any(sel.NA) &
          avail.median & avail.q1 & avail.q3 &
          !(is.null(n.e) & is.null(n.c))) {
        j <- sel.NA & !is.na(median) & !is.na(q1) & !is.na(q3)
        approx.seTE[j] <- "iqr"
        if (is.null(n.c))
          seTE[j] <- mean_sd_iqr(n.e[j], median[j], q1[j], q3[j])$se
        else if (is.null(n.e))
          seTE[j] <- mean_sd_iqr(n.c[j], median[j], q1[j], q3[j])$se
        else
          seTE[j] <- mean_sd_iqr(n.e[j] + n.c[j], median[j], q1[j], q3[j])$se
      }
      #
      # Use range
      #
      sel.NA <- is.na(seTE)
      if (any(sel.NA) &
          avail.median & avail.min & avail.max &
          !(is.null(n.e) & is.null(n.c))) {
        j <- sel.NA & !is.na(median) & !is.na(min) & !is.na(max)
        approx.seTE[j] <- "range"
        if (is.null(n.c))
          seTE[j] <- mean_sd_range(n.e[j], median[j], min[j], max[j])$se
        else if (is.null(n.e))
          seTE[j] <- mean_sd_range(n.c[j], median[j], min[j], max[j])$se
        else
          seTE[j] <- mean_sd_range(n.e[j] + n.c[j], median[j],
                                   min[j], max[j])$se
      }
    }
    else {
      j <- 0
      for (i in approx.seTE) {
        j <- j + 1
        #
        if (i == "ci") {
          if (!avail.df)
            seTE[j] <- TE.seTE.ci(lower[j], upper[j], level.ci[j])$seTE
          else
            seTE[j] <- TE.seTE.ci(lower[j], upper[j], level.ci[j], df[j])$seTE
        }
        else if (i == "pval") {
          if (!avail.df)
            seTE[j] <- seTE.pval(TE[j], pval[j])$seTE
          else
            seTE[j] <- seTE.pval(TE[j], pval[j], df[j])$seTE
        }
        else if (i == "iqr.range") {
          if (is.null(n.e) & is.null(n.c))
            stop("Sample size needed if argument 'approx.seTE' = \"iqr\".",
                 call. = FALSE)
          else if (is.null(n.c))
            seTE[j] <- mean_sd_iqr_range(n.e[j], median[j], q1[j], q3[j],
                                         min[j], max[j],
                                         method.sd = method.sd)$se
          else if (is.null(n.e))
            seTE[j] <- mean_sd_iqr_range(n.c[j], median[j], q1[j], q3[j],
                                         min[j], max[j],
                                         method.sd = method.sd)$se
          else
            seTE[j] <- mean_sd_iqr_range(n.e[j] + n.c[j], median[j], q1[j], q3[j],
                                         min[j], max[j],
                                         method.sd = method.sd)$se
        }
        else if (i == "iqr") {
          if (is.null(n.e) & is.null(n.c))
            stop("Sample size needed if argument 'approx.seTE' = \"iqr\".",
                 call. = FALSE)
          else if (is.null(n.c))
            seTE[j] <- mean_sd_iqr(n.e[j], median[j], q1[j], q3[j])$se
          else if (is.null(n.e))
            seTE[j] <- mean_sd_iqr(n.c[j], median[j], q1[j], q3[j])$se
          else
            seTE[j] <- mean_sd_iqr(n.e[j] + n.c[j], median[j], q1[j], q3[j])$se
        }
        else if (i == "range") {
          if (is.null(n.e) & is.null(n.c))
            stop("Sample size needed if argument 'approx.seTE' = \"range\".",
                 call. = FALSE)
          else if (is.null(n.c))
            seTE[j] <- mean_sd_range(n.e[j], median[j], min[j], max[j])$se
          else if (is.null(n.e))
            seTE[j] <- mean_sd_range(n.c[j], median[j], min[j], max[j])$se
          else
            seTE[j] <- mean_sd_range(n.e[j] + n.c[j], median[j],
                                     min[j], max[j])$se
        }
      }
    }
  }
  
  
  #
  #
  # (8) Calculate treatment estimate from other information
  #
  #
  
  if (!with.cycles) {
    if (!avail.approx.TE) {
      approx.TE <- rep_len("", length(TE))
      #
      # Use confidence limits
      #
      sel.NA <- is.na(TE)
      if (any(sel.NA) & avail.lower & avail.upper) {
        j <- sel.NA & !is.na(lower) & !is.na(upper)
        approx.TE[j] <- "ci"
        TE[j] <- TE.seTE.ci(lower[j], upper[j], level.ci[j])$TE
      }
      #
      # Use IQR and range
      #
      sel.NA <- is.na(TE)
      if (any(sel.NA) &
          avail.median & avail.q1 & avail.q3 & avail.min & avail.max &
          !(is.null(n.e) & is.null(n.c))) {
        j <- sel.NA & !is.na(median) & !is.na(q1) & !is.na(q3) &
          !is.na(min) & !is.na(max)
        approx.TE[j] <- "iqr.range"
        if (is.null(n.c))
          TE[j] <- mean_sd_iqr_range(n.e[j], median[j], q1[j], q3[j],
                                     min[j], max[j], method.mean)$mean
        else if (is.null(n.e))
          TE[j] <- mean_sd_iqr_range(n.c[j], median[j], q1[j], q3[j],
                                     min[j], max[j], method.mean)$mean
        else
          TE[j] <- mean_sd_iqr_range(n.e[j] + n.c[j], median[j], q1[j], q3[j],
                                     min[j], max[j], method.mean)$mean
      }
      #
      # Use IQR
      #
      sel.NA <- is.na(TE)
      if (any(sel.NA) &
          avail.median & avail.q1 & avail.q3 &
          !(is.null(n.e) & is.null(n.c))) {
        j <- sel.NA & !is.na(median) & !is.na(q1) & !is.na(q3)
        approx.TE[j] <- "iqr"
        if (is.null(n.c))
          TE[j] <- mean_sd_iqr(n.e[j], median[j], q1[j], q3[j], method.mean)$mean
        else if (is.null(n.e))
          TE[j] <- mean_sd_iqr(n.c[j], median[j], q1[j], q3[j], method.mean)$mean
        else
          TE[j] <- mean_sd_iqr(n.e[j] + n.c[j], median[j], q1[j], q3[j],
                               method.mean)$mean
      }
      #
      # Use range
      #
      sel.NA <- is.na(TE)
      if (any(sel.NA) &
          avail.median & avail.min & avail.max &
          !(is.null(n.e) & is.null(n.c))) {
        j <- sel.NA & !is.na(median) & !is.na(min) & !is.na(max)
        approx.TE[j] <- "range"
        if (is.null(n.c))
          TE[j] <- mean_sd_range(n.e[j], median[j], min[j], max[j],
                                 method.mean)$mean
        else if (is.null(n.e))
          TE[j] <- mean_sd_range(n.c[j], median[j], min[j], max[j],
                                 method.mean)$mean
        else
          TE[j] <- mean_sd_range(n.e[j] + n.c[j], median[j], min[j], max[j],
                                 method.mean)$mean
      }
    }
    else {
      j <- 0
      for (i in approx.TE) {
        j <- j + 1
        #
        if (i == "ci")
          TE[j] <- TE.seTE.ci(lower[j], upper[j], level.ci[j])$TE
        else if (i == "iqr.range")
          if (is.null(n.e) & is.null(n.c))
            stop("Sample size needed if argument 'approx.TE' = \"iqr.range\".",
                 call. = FALSE)
        else if (is.null(n.c))
          TE[j] <- mean_sd_iqr_range(n.e[j], median[j], q1[j], q3[j],
                                     min[j], max[j], method.mean)$mean
        else if (is.null(n.e))
          TE[j] <- mean_sd_iqr_range(n.c[j], median[j], q1[j], q3[j],
                                     min[j], max[j], method.mean)$mean
        else
          TE[j] <- mean_sd_iqr_range(n.e[j] + n.c[j], median[j], q1[j], q3[j],
                                     min[j], max[j], method.mean)$mean
        else if (i == "iqr") {
          if (is.null(n.e) & is.null(n.c))
            stop("Sample size needed if argument 'approx.TE' = \"iqr\".",
                 call. = FALSE)
          else if (is.null(n.c))
            TE[j] <-
              mean_sd_iqr(n.e[j], median[j], q1[j], q3[j], method.mean)$mean
          else if (is.null(n.e))
            TE[j] <-
              mean_sd_iqr(n.c[j], median[j], q1[j], q3[j], method.mean)$mean
          else
            TE[j] <-
              mean_sd_iqr(n.e[j] + n.c[j], median[j], q1[j], q3[j],
                          method.mean)$mean
        }
        else if (i == "range") {
          cat(paste0("Use 'range' for study", j, "\n"))
          if (is.null(n.e) & is.null(n.c))
            stop("Sample size needed if argument 'approx.TE' = \"range\".",
                 call. = FALSE)
          else if (is.null(n.c))
            TE[j] <- mean_sd_range(n.e[j], median[j], min[j], max[j],
                                   method.mean)$mean
          else if (is.null(n.e))
            TE[j] <- mean_sd_range(n.c[j], median[j], min[j], max[j],
                                   method.mean)$mean
          else
            TE[j] <- mean_sd_range(n.e[j] + n.c[j], median[j], min[j], max[j],
                                   method.mean)$mean
        }
      }
    }
  }
  #
  if (keepdata) {
    if (!isCol(data, ".subset")) {
      data$.TE <- TE
      data$.seTE <- seTE
      #
      if (!avail.approx.TE & any(approx.TE != ""))
        data$.approx.TE <- approx.TE
      if (!avail.approx.seTE & any(approx.seTE != ""))
        data$.approx.seTE <- approx.seTE
    }
    else {
      data$.TE[data$.subset] <- TE
      data$.seTE[data$.subset] <- seTE
      #
      if (!avail.approx.TE & any(approx.TE != "")) {
        data$.approx.TE <- ""
        data$.approx.TE[data$.subset] <- approx.TE
      }
      if (!avail.approx.seTE & any(approx.seTE != "")) {
        data$.approx.seTE <- ""
        data$.approx.seTE[data$.subset] <- approx.seTE
      }
    }
  }
  
  
  #
  #
  # (9) Check standard errors
  #
  #
  
  TE   <- int2num(TE)
  seTE <- int2num(seTE)
  #
  if (any(seTE[!is.na(seTE)] <= 0)) {
    if (warn & !with.cycles)
      warning("Zero values in seTE replaced by NAs.", call. = FALSE)
    seTE[!is.na(seTE) & seTE == 0] <- NA
  }
  #
  tau2.calc <- NA
  
  
  #
  #
  # (10) Additional checks for three-level model
  #
  #
  
  three.level <- FALSE
  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
  #
  # Only conduct three-level meta-analysis if variable 'cluster'
  # contains duplicate values after removing inestimable study
  # results standard errors
  #
  if (with.cluster &&
      length(unique(cluster[sel.ni])) != length(cluster[sel.ni]))
    three.level <- TRUE
  #
  if (is.null(method.tau.ci))
    if (three.level)
      method.tau.ci <- "PL"
    else if (method.tau == "DL")
      method.tau.ci <- "J"
    else
      method.tau.ci <- "QP"
  #
  method.tau.ci <- setchar(method.tau.ci, gs("meth4tau.ci"))
  #
  if (!three.level & method.tau.ci == "PL") {
    if (method.tau == "DL")
      method.tau.ci <- "J"
    else
      method.tau.ci <- "QP"
  }
  #
  chklevel(level.hetstat)
  #
  if (three.level) {
    chkmlm(method.tau, missing.method.tau, method.predict)
    #
    common <- FALSE
    #
    if (!(method.tau %in% c("REML", "ML")))
      method.tau <- "REML"
  }
  #
  if (by) {
    chkmiss(subgroup)
    #
    if (missing.subgroup.name & is.null(subgroup.name)) {
      if (!missing.subgroup)
        subgroup.name <- byvarname("subgroup", mc)
      else if (!missing.byvar)
        subgroup.name <- byvarname("byvar", mc)
    }
  }
  #
  if (!is.null(subgroup.name))
    chkchar(subgroup.name, length = 1)
  
  
  #
  #
  # (11) Additional checks and calculations for n-of-1 trials
  #
  #
  
  if (with.cycles) {
    df.n_of_1 <- cycles - 1
    sd.n_of_1 <-
      sqrt(sum(df.n_of_1 * seTE^2, na.rm = TRUE) / sum(df.n_of_1, na.rm = TRUE))
    #
    seTE <- sd.n_of_1 / sqrt(cycles)
    #
    if (keepdata) {
      data$.seTE.orig <- data$.seTE
      #
      if (!isCol(data, ".subset"))
        data$.seTE <- seTE
      else
        data$.seTE[data$.subset] <- seTE
    }
  }
  
  
  #
  #
  # (12) Do meta-analysis
  #
  #
  
  k <- sum(!is.na(TE[!exclude]) & !is.na(seTE[!exclude]))
  #
  if (three.level) {
    cluster.incl <- cluster[!exclude]
    k.study <-
      length(unique(
        cluster.incl[!is.na(TE[!exclude]) & !is.na(seTE[!exclude])]))
  }
  else
    k.study <- k
  #
  seTE.hakn.ci <- seTE.hakn.adhoc.ci <-
    seTE.hakn.pi <- seTE.hakn.adhoc.pi <-
      seTE.kero <- NA
  #
  pi <- list(seTE = NA, lower = NA, upper = NA, df = NA)
  #
  df.random <- df.predict <-
    df.hakn <- df.hakn.ci <- df.hakn.pi <- df.kero <- NA
  #
  if (k == 0) {
    TE.common <- seTE.common <-
      lower.common <- upper.common <-
        statistic.common <- pval.common <- NA
    w.common <- rep(0, k.all)
    #
    TE.random <- seTE.random <-
      lower.random <- upper.random <-
        statistic.random <- pval.random <- NA
    seTE.classic <- seTE.random
    w.random <- rep(0, k.all)
    #
    seTE.predict <- df.predict <-
      lower.predict <- upper.predict <-
        seTE.hakn.pi <- seTE.hakn.adhoc.pi <- NA
    #
    hc <- list(tau2 = NA, se.tau2 = NA, lower.tau2 = NA, upper.tau2 = NA,
               tau = NA, lower.tau = NA, upper.tau = NA,
               method.tau.ci = "", level = NA,
               sign.lower.tau = "", sign.upper.tau = "",
               #
               Q = NA, df.Q = NA, pval.Q = NA,
               H = NA, lower.H = NA, upper.H = NA,
               I2 = NA, lower.I2 = NA, upper.I2 = NA,
               #
               Q.resid = NA, df.Q.resid = NA, pval.Q.resid = NA,
               H.resid = NA, lower.H.resid = NA, upper.H.resid = NA,
               I2.resid = NA, lower.I2.resid = NA, upper.I2.resid = NA)
  }
  else {
    # At least two studies to perform Hartung-Knapp or Kenward-Roger
    # method
    if (k == 1) {
      method.random.ci <-
        ifelse(method.random.ci %in% c("HK", "KR"), "classic",
               method.random.ci)
      #
      method.predict <- 
        ifelse(method.predict %in% c("HK", "KR"), "V", method.predict)
      #
      adhoc.hakn.ci <- ""
      adhoc.hakn.pi <- ""
    }
    #
    # Estimate tau-squared
    #
    hc <- hetcalc(TE[!exclude], seTE[!exclude],
                  method.tau, method.tau.ci, TE.tau,
                  method.I2, level.hetstat, control = control,
                  cluster = cluster[!exclude], rho = rho)
    #
    # Estimate common tau-squared across subgroups
    #
    if (by & tau.common)
      hcc <- hetcalc(TE[!exclude], seTE[!exclude],
                     method.tau, method.tau.ci, TE.tau,
                     method.I2, level.hetstat,
                     subgroup = subgroup, control = control,
                     cluster = cluster[!exclude], rho = rho)
    #
    # Different calculations for three-level models
    #
    if (!three.level) {
      #
      # Between-study heterogeneity
      #
      if (is.null(tau.preset))
        tau2.calc <- if (is.na(sum(hc$tau2))) 0 else sum(hc$tau2)
      else {
        tau2.calc <- tau.preset^2
        #
        hc$tau2 <- tau.preset^2
        hc$se.tau2 <- hc$lower.tau2 <- hc$upper.tau2 <- NA
        hc$tau <- tau.preset
        hc$lower.tau <- hc$upper.tau <- NA
        hc$method.tau.ci <- ""
        hc$sign.lower.tau <- ""
        hc$sign.upper.tau <- ""
      }
      #
      # Common effect model
      #
      if (!usw.common) {
        #
        # Classic meta-analysis (Cooper & Hedges, 1994, p. 265-6)
        #
        w.common <- 1 / seTE^2
        w.common[is.na(w.common) | is.na(TE) | exclude] <- 0
        #
        TE.common <- weighted.mean(TE, w.common, na.rm = TRUE)
        #
        if (method.common.ci == "classic") {
          seTE.common <- sqrt(1 / sum(w.common, na.rm = TRUE))
        }
        else if (method.common.ci == "IVhet") {
          seTE.common <-
            sqrt(sum((w.common / (sum(w.common)))^2 * (seTE^2 + tau2.calc)))
        }
        else
          seTE.common <- NA
        #
        ci.c <- ci(TE.common, seTE.common, level = level.ma,
                   null.effect = null.effect)
        statistic.common <- ci.c$statistic
        pval.common <- ci.c$p
        lower.common <- ci.c$lower
        upper.common <- ci.c$upper
      }
      else {
        #
        # Conduct common effect meta-analysis with user-specified weights
        #
        sel.4 <- !is.na(TE) & !is.na(seTE) & !exclude
        #
        m4.usw.c <-
          runUSW(list(yi = TE[sel.4], sei = seTE[sel.4],
                      weights = weights.common[sel.4]),
                 method.tau = "FE",
                 method.random.ci = NULL,
                 level = level.ma,
                 control = control)
        #
        res.usw.c <- extrUSW(m4.usw.c, "FE", null.effect)
        #
        w.common <- res.usw.c$w.common
        TE.common <- res.usw.c$TE.common
        seTE.common <- res.usw.c$seTE.common
        lower.common <- res.usw.c$lower.common
        upper.common <- res.usw.c$upper.common
        statistic.common <- res.usw.c$statistic.common
        pval.common <- res.usw.c$pval.common
        #
        # Drop list for single random effects meta-analysis
        #
        if (length(m4.usw.c) == 1)
          m4.usw.c <- m4.usw.c[[1]]
      }
      #
      # Random effects model
      #
      if (!usw.random) {
        #
        # Classic meta-analysis (Cooper & Hedges, 1994, p. 265, 274-5)
        #
        w.random <- 1 / (seTE^2 + tau2.calc)
        w.random[is.na(w.random) | is.na(TE) | exclude] <- 0
        #
        TE.random   <- weighted.mean(TE, w.random, na.rm = TRUE)
        seTE.random <- sqrt(1 / sum(w.random, na.rm = TRUE))
        #
        seTE.classic <- seTE.random
        #
        # Kenward-Roger method for confidence or prediction interval
        #
        kr <- kenwardroger(w.random)
        seTE.kero <- kr$se
        df.kero <- kr$df
        #
        if (any(method.random.ci == "KR") | any(method.predict == "KR")) {
          # Fallback: classic random effects meta-analysis
          if (is.nan(seTE.kero) | is.nan(df.kero)) {
            method.random.ci[method.random.ci == "KR"] <- "classic-KR"
            method.predict[method.predict == "KR"] <- "V-KR"
          }
        }
        #
        # Hartung-Knapp method for confidence or prediction interval
        #
        df.hakn <- k - 1
        q <- 1 / (k - 1) * sum(w.random * (TE - TE.random)^2, na.rm = TRUE)
        #
        seTE.hakn.ci <- seTE.hakn.adhoc.ci <-
          seTE.hakn.pi <- seTE.hakn.adhoc.pi <-
          sqrt(q / sum(w.random))
        #
        # Confidence interval for random effects model
        #
        ci.r <- as.data.frame(ci(1, NA, level = 0.99999))
        #
        if (length(adhoc.hakn.ci) == 1) {
          adhoc.hakn.ci <- ifelse(method.random.ci == "HK", adhoc.hakn.ci, "")
        }
        else if (length(adhoc.hakn.ci) == sum(method.random.ci == "HK")) {
          adho <- rep("", length(method.random.ci))
          adho[method.random.ci == "HK"] <- adhoc.hakn.ci
          adhoc.hakn.ci <- adho
        }
        else if (length(method.random.ci) == 1 && method.random.ci == "HK") {
          method.random.ci <- rep("HK", length(adhoc.hakn.ci))
        }
        else if (length(adhoc.hakn.ci) != length(method.random.ci))
          stop("Argument 'adhoc.hakn.ci' must be of same length as ",
               "'method.random.ci' or number of meta-analyses with ",
               "Hartung-Knapp method",
               call. = FALSE)
        #
        seTE.hakn.adhoc.ci <- rep(seTE.hakn.adhoc.ci, length(method.random.ci))
        df.hakn.ci <- rep(df.hakn, length(method.random.ci))
        #
        for (i in seq_along(method.random.ci)) {
          if (method.random.ci[i] %in% c("classic", "classic-KR")) {
            ci.r.i <- ci(TE.random, seTE.classic, level = level.ma,
                         null.effect = null.effect)
            #
            seTE.hakn.adhoc.ci[i] <- NA
          }
          else if (method.random.ci[i] == "HK") {
            if (adhoc.hakn.ci[i] == "se") {
              #
              # Variance correction if SE_HK < SE_notHK
              # (Knapp and Hartung, 2003), i.e., if q < 1
              #
              if (q < 1)
                seTE.hakn.adhoc.ci[i] <- seTE.classic
            }
            else if (adhoc.hakn.ci[i] == "ci") {
              #
              # Use wider confidence interval, i.e., confidence interval
              # from classic random effects meta-analysis if CI_HK is
              # smaller
              # (Wiksten et al., 2016; Jackson et al., 2017, hybrid 2)
              #
              ci.hk <-
                ci(TE.random, seTE.hakn.ci, level = level.ma, df = df.hakn.ci[i])
              ci.re <-
                ci(TE.random, seTE.classic, level = level.ma)
              #
              width.hk <- ci.hk$upper - ci.hk$lower
              width.re <- ci.re$upper - ci.re$lower
              #
              if (width.hk < width.re) {
                seTE.hakn.adhoc.ci[i] <- seTE.classic
                df.hakn.ci[i] <- NA
              }
            }
            else if (adhoc.hakn.ci[i] == "IQWiG6") {
              #
              # Variance correction if CI_HK < CI_DL (IQWiG, 2020)
              #
              ci.hk <-
                ci(TE.random, seTE.hakn.ci, level = level.ma, df = df.hakn.ci[i])
              #
              m.dl <- metagen(TE, seTE, method.tau = "DL", method.tau.ci = "",
                              method.random.ci = "classic")
              ci.dl <- ci(m.dl$TE.random, m.dl$seTE.classic, level = level.ma)
              #
              width.hk <- ci.hk$upper - ci.hk$lower
              width.dl <- ci.dl$upper - ci.dl$lower
              #
              if (width.hk < width.dl)
                seTE.hakn.adhoc.ci[i] <- seTE.classic
            }
            #
            ci.r.i <- ci(TE.random, seTE.hakn.adhoc.ci[i],
                         level = level.ma, df = df.hakn.ci[i],
                         null.effect = null.effect)
          }
          else if (method.random.ci[i] == "KR") {
            ci.r.i <- ci(TE.random, seTE.kero, level = level.ma, df = df.kero,
                         null.effect = null.effect)
          }
          #
          ci.r <- rbind(ci.r, as.data.frame(ci.r.i))
        }
        #
        ci.r <- ci.r[-1, ]
        #
        seTE.random <- ci.r$seTE
        lower.random <- ci.r$lower
        upper.random <- ci.r$upper
        df.random <- ci.r$df
        statistic.random <- ci.r$statistic
        pval.random <- ci.r$p
        #
        if (missing.text.random ||
            (length(text.random) == 1 & length(method.random.ci) > 1)) {
          text.random <-
            ifelse(method.random.ci == "classic",
                   text.random,
                   ifelse(method.random.ci %in% c("KR", "classic-KR"),
                          paste0(text.random, " (", method.random.ci, ")"),
                          paste0(text.random, " (HK")))
          text.random <-
            paste0(text.random,
                   ifelse(method.random.ci != "HK",
                          "",
                          ifelse(adhoc.hakn.ci == "",
                                 ")",
                                 paste0("-", toupper(substring(adhoc.hakn.ci, 1, 2)),
                                        ")"))))
        }
        #
        # Prediction interval
        #
        pi <- data.frame()
        #
        if (length(adhoc.hakn.pi) == 1) {
          adhoc.hakn.pi <- ifelse(method.predict == "HK", adhoc.hakn.pi, "")
        }
        else if (length(adhoc.hakn.pi) == sum(method.predict == "HK")) {
          adho <- rep("", length(method.predict))
          adho[method.predict == "HK"] <- adhoc.hakn.pi
          adhoc.hakn.pi <- adho
        }
        else if (length(method.predict) == 1 && method.predict == "HK") {
          method.predict <- rep("HK", length(adhoc.hakn.pi))
        }
        else if (length(adhoc.hakn.pi) != length(method.predict)) {
          stop("Argument 'adhoc.hakn.pi' must be of same length as ",
               "'method.predict' or number of prediction intervals using ",
               "Hartung-Knapp method",
               call. = FALSE)
        }
        #
        seTE.hakn.adhoc.pi <- rep(seTE.hakn.adhoc.pi, length(method.predict))
        df.hakn.pi <- rep(df.hakn, length(method.predict))
        #
        for (i in seq_along(method.predict)) {
          if (method.predict[i] == "HK" && df.hakn.pi[i] >= 1) {
            if (adhoc.hakn.pi[i] == "se") {
              #
              # Variance correction if SE_HK < SE_notHK (Knapp and
              # Hartung, 2003), i.e., if q < 1
              #
              if (q < 1)
                seTE.hakn.adhoc.pi[i] <- seTE.classic
            }
            #
            pi.i <- ci(TE.random, sqrt(seTE.hakn.adhoc.pi[i]^2 + tau2.calc),
                       level = level.predict, df = df.hakn.pi[i])
          }
          else if (method.predict[i] == "HK-PR" && df.hakn.pi[i] >= 2) {
            if (adhoc.hakn.pi[i] == "se") {
              #
              # Variance correction if SE_HK < SE_notHK (Knapp and
              # Hartung, 2003), i.e., if q < 1
              #
              if (q < 1)
                seTE.hakn.adhoc.pi[i] <- seTE.classic
            }
            #
            pi.i <- ci(TE.random, sqrt(seTE.hakn.adhoc.pi[i]^2 + tau2.calc),
                       level = level.predict, df = df.hakn.pi[i] - 1)
          }
          else if (method.predict[i] %in% c("V", "V-KR") & k > 1) {
            pi.i <- ci(TE.random, sqrt(seTE.classic^2 + tau2.calc),
                       level.predict, k - 1)
          }
          else if (method.predict[i] %in% c("HTS", "HTS-KR") & k > 2) {
            pi.i <- ci(TE.random, sqrt(seTE.classic^2 + tau2.calc),
                       level.predict, k - 2)
          }
          else if (method.predict[i] == "KR" & df.kero > 0) {
            pi.i <- ci(TE.random, sqrt(seTE.kero^2 + tau2.calc),
                       level.predict, df.kero)
          }
          else if (method.predict[i] == "KR-PR" & df.kero > 1) {
            pi.i <- ci(TE.random, sqrt(seTE.kero^2 + tau2.calc),
                       level.predict, df.kero - 1)
          }
          else if (method.predict[i] == "PR" & df.kero > 0) {
            pi.i <- ci(TE.random, sqrt(seTE.kero^2 + tau2.calc),
                       level.predict, df.kero - 1)
          }
          else if (method.predict[i] == "NNF") {
            res.pima <- pimeta::pima(TE[!exclude], seTE[!exclude],
                                     method = "boot",
                                     alpha = 1 - level.predict,
                                     seed = seed.predict)
            #
            pi.i <- as.data.frame(ci(1, NA, level = level.predict))
            pi.i$seTE <- NA
            pi.i$lower <- res.pima$lpi
            pi.i$upper <- res.pima$upi
            pi.i$df <- res.pima$nup
          }
          else if (method.predict[i] == "S")
            pi.i <- ci(TE.random, sqrt(seTE.classic^2 + tau2.calc), level.predict)
          else if (method.predict[i] == "KR")
            pi.i <- ci(TE.random, NA, level.predict, df.kero - 1)
          else if (method.predict[i] == "V")
            pi.i <- ci(TE.random, NA, level.predict, k - 1)
          else if (method.predict[i] == "HTS")
            pi.i <- ci(TE.random, NA, level.predict, k - 2)
          else
            pi.i <- ci(TE.random, NA, level.predict)
          #
          pi <- rbind(pi, as.data.frame(pi.i))
        }
        #
        seTE.predict <- pi$seTE
        lower.predict <- pi$lower
        upper.predict <- pi$upper
        df.predict <- pi$df
      }
      else {
        #
        # No adhoc method for meta-analysis with user-specified weights
        #
        if ((!missing.adhoc.hakn.ci && any(adhoc.hakn.ci != "")) |
            (!missing.adhoc.hakn.pi && any(adhoc.hakn.pi != ""))) {
          warning("Ad hoc variance correction not implemented ",
                  "for user-specified weights.",
                  call. = FALSE)
          adhoc.hakn.ci[adhoc.hakn.ci != ""] <- ""
          adhoc.hakn.pi[adhoc.hakn.pi != ""] <- ""
        }
        #
        # Conduct random effect meta-analysis with user-specified weights
        #
        sel.4 <- !is.na(TE) & !is.na(seTE) & !exclude
        #
        m4.usw.r <-
          runUSW(list(yi = TE[sel.4], sei = seTE[sel.4],
                      weights = weights.random[sel.4]),
                 method.tau = method.tau,
                 method.random.ci = method.random.ci,
                 level = level.ma,
                 control = control)
        #
        res.usw.r <-
          extrUSW(m4.usw.r, method.tau, null.effect,
                  k, length(TE), sel.4,
                  method.random.ci, method.predict,
                  level.ma, level.predict)
        #
        w.random <- res.usw.r$w.random
        tau2.calc <- sum(res.usw.r$tau2)
        if (is.na(tau2.calc))
          tau2.calc <- 0
        #
        TE.random <- res.usw.r$TE.random
        seTE.random <- res.usw.r$seTE.random
        lower.random <- res.usw.r$lower.random
        upper.random <- res.usw.r$upper.random
        statistic.random <- res.usw.r$statistic.random
        pval.random <- res.usw.r$pval.random
        #
        seTE.classic <- m4.usw.r[[1]]$se
        #
        df.random <- df.hakn <- ifelse(method.random.ci == "HK", k - 1, NA)
        #
        if (missing(text.random) ||
            (length(text.random) == 1 & length(method.random.ci) > 1))
          text.random <-
          ifelse(method.random.ci == "classic",
                 text.random,
                 ifelse(method.random.ci == "HK",
                        paste0(text.random, " (T)"),
                        ""))
        #
        # Prediction interval
        #
        seTE.predict <- res.usw.r$seTE.predict
        lower.predict <- res.usw.r$lower.predict
        upper.predict <- res.usw.r$upper.predict
        df.predict <- res.usw.r$df.predict
        #
        # Drop list for single random effects meta-analysis
        #
        if (length(m4.usw.r) == 1)
          m4.usw.r <- m4.usw.r[[1]]
      }
    }
    else {
      #
      # Conduct three-level meta-analysis
      #
      if (common) {
        #
        # No common effect method for three-level model
        #
        if (!missing.common & common)
          warning(gs("text.common"), " not calculated for three-level model.",
                  call. = FALSE)
        common <- FALSE
      }
      #
      w.common <- rep_len(NA, length(seTE))
      #
      TE.common <- seTE.common <- lower.common <- upper.common <-
        statistic.common <- pval.common <- NA
      #
      # No adhoc method for three-level models
      #
      if ((!missing.adhoc.hakn.ci && any(adhoc.hakn.ci != "")) |
          (!missing.adhoc.hakn.pi && any(adhoc.hakn.pi != ""))) {
        warning("Ad hoc variance correction not implemented ",
                "for three-level model.",
                call. = FALSE)
        adhoc.hakn.ci[adhoc.hakn.ci != ""] <- ""
        adhoc.hakn.pi[adhoc.hakn.pi != ""] <- ""
      }
      #
      # Conduct three-level meta-analysis
      #
      sel.4 <- !is.na(TE) & !is.na(seTE) & !exclude
      #
      list.mlm <- list(yi = TE[sel.4],
                       V = vcalc(vi = seTE[sel.4]^2,
                                 cluster = cluster[sel.4],
                                 obs = idx[sel.4], rho = rho))
             
      #
      m4 <- runMLM(c(list.mlm,
                     list(data = data.frame(cluster = cluster[sel.4],
                                            idx = idx[sel.4]))),
                   method.tau = method.tau,
                   method.random.ci = method.random.ci,
                   level = level.ma,
                   control = control)
      #
      res.mlm <-
        extrMLM(m4, k, length(TE), sel.4,
                method.random.ci, method.predict,
                level.ma, level.predict, null.effect)
      #
      w.random <- res.mlm$w.random
      tau2.calc <- sum(res.mlm$tau2)
      if (is.na(tau2.calc))
        tau2.calc <- 0
      #
      TE.random <- res.mlm$TE.random
      seTE.random <- res.mlm$seTE.random
      lower.random <- res.mlm$lower.random
      upper.random <- res.mlm$upper.random
      statistic.random <- res.mlm$statistic.random
      pval.random <- res.mlm$pval.random
      #
      seTE.classic <- m4[[1]]$se
      #
      df.random <- df.hakn <- ifelse(method.random.ci == "HK", k - 1, NA)
      #
      if (missing(text.random) ||
          (length(text.random) == 1 & length(method.random.ci) > 1))
        text.random <-
          ifelse(method.random.ci == "classic",
                 text.random,
          ifelse(method.random.ci == "HK",
                 paste0(text.random, " (T)"),
                 ""))
      #
      # Prediction interval
      #
      seTE.predict <- res.mlm$seTE.predict
      lower.predict <- res.mlm$lower.predict
      upper.predict <- res.mlm$upper.predict
      df.predict <- res.mlm$df.predict
      #
      # Drop list for single random effects meta-analysis
      #
      if (length(m4) == 1)
        m4 <- m4[[1]]
    }
  }
  #
  if (missing(text.predict) ||
      (length(text.predict) == 1 & length(method.predict) > 1)) {
    if (length(method.predict) > 1) {
      text.predict <- paste0(text.predict, " (", method.predict)
      text.predict <-
        paste0(text.predict,
               ifelse(method.predict != "HK",
                      ")",
               ifelse(adhoc.hakn.pi == "",
                      ")",
                      paste0("-", toupper(adhoc.hakn.pi), ")"))))
    }
  }
  
  
  #
  #
  # (13) Heterogeneity measures
  #
  #
  
  #
  # Calculate Rb (but not for three-level model)
  #
  if (length(tau2.calc) == 1)
    Rbres <- Rb(seTE[!is.na(seTE)], seTE.classic,
                tau2.calc, hc$Q, hc$df.Q, level.ma)
  else
    Rbres <- list(TE = NA, lower = NA, upper = NA)
  
  
  #
  #
  # (14) Generate R object
  #
  #
  if (!avail.detail.tau) {
    if (k != k.study)
      detail.tau <- c("between cluster", "within cluster")
    else
      detail.tau <- ""
  }
  #
  ci.study <- ci(TE, seTE, level = level,
                 df =
                   if (!is.null(method.ci) && method.ci == "t")
                     df
                   else
                     NULL,
                 null.effect = null.effect)
  #
  # Keep original confidence limits
  #
  if (avail.lower)
    ci.study$lower[!is.na(lower)] <- lower[!is.na(lower)]
  if (avail.upper)
    ci.study$upper[!is.na(upper)] <- upper[!is.na(upper)]
  #
  if (length(lower.random) > 1) {
    methci <- paste(method.random.ci,
                    toupper(substring(adhoc.hakn.ci, 1, 2)),
                    sep = "-")
    methci <- gsub("-$", "", methci)
    #
    if (length(TE.random) == 1)
      TE.random <- rep(TE.random, length(lower.random))
    #
    if (length(seTE.random) == 1)
      seTE.random <- rep(seTE.random, length(lower.random))
    #
    names(TE.random) <- names(seTE.random) <-
      names(statistic.random) <- names(pval.random) <-
      names(df.random) <- names(lower.random) <- names(upper.random) <-
      methci
    #
    if (!three.level & !usw.random)
      names(adhoc.hakn.ci) <- names(df.hakn.ci) <-
        names(seTE.hakn.adhoc.ci) <-
        methci
  }
  #
  if (length(lower.predict) > 1) {
    methpi <- paste(method.predict,
                    toupper(substring(adhoc.hakn.pi, 1, 2)),
                    sep = "-")
    methpi <- gsub("-$", "", methpi)
    #
    names(seTE.predict) <- names(df.predict) <-
      names(lower.predict) <- names(upper.predict) <-
      methpi
    #
    if (!three.level & !usw.random)
      names(adhoc.hakn.pi) <- names(df.hakn.pi) <-
        names(seTE.hakn.adhoc.pi) <-
        methpi
  }
  #
  res <- list(studlab = studlab,
              #
              sm = sm,
              null.effect = null.effect,
              #
              TE = TE, seTE = seTE,
              statistic = ci.study$statistic,
              pval = ci.study$p,
              df =
                if (!is.null(method.ci) && method.ci == "t")
                  df
                else
                  rep_len(NA, length(TE)),
              level = level,
              lower = ci.study$lower, upper = ci.study$upper,
              #
              three.level = three.level,
              cluster = cluster, rho = rho,
              #
              k = k, k.study = k.study, k.all = k.all, k.TE = sum(!is.na(TE)),
              #
              cycles = cycles,
              sd.n_of_1 = if (with.cycles) sd.n_of_1 else NULL,
              #
              overall = overall,
              overall.hetstat = overall.hetstat,
              common = common,
              random = random,
              prediction = prediction,
              transf = transf,
              backtransf = backtransf,
              func.transf = func.transf,
              func.backtransf = func.backtransf,
              args.transf = args.transf,
              args.backtransf = args.backtransf,
              #
              method = "Inverse",
              method.random = "Inverse",
              #
              w.common = w.common,
              TE.common = TE.common,
              seTE.common = seTE.common,
              statistic.common = statistic.common,
              pval.common = pval.common,
              level.ma = level.ma,
              method.common.ci = method.common.ci,
              lower.common = lower.common,
              upper.common = upper.common,
              #
              w.random = w.random,
              TE.random = TE.random,
              seTE.random = seTE.random,
              statistic.random = statistic.random,
              pval.random = pval.random,
              method.random.ci = method.random.ci,
              df.random = df.random,
              lower.random = lower.random,
              upper.random = upper.random,
              #
              seTE.classic = seTE.classic,
              #
              weights.common = weights.common,
              weights.random = weights.random,
              #
              adhoc.hakn.ci = adhoc.hakn.ci,
              df.hakn.ci =
                if (any(method.random.ci == "HK")) df.hakn.ci else NA,
              seTE.hakn.ci = seTE.hakn.ci,
              seTE.hakn.adhoc.ci = seTE.hakn.adhoc.ci,
              #
              df.kero = if (any(method.random.ci == "KR") |
                            any(method.predict == "KR")) df.kero else NA,
              seTE.kero = seTE.kero,
              #
              method.predict = method.predict,
              adhoc.hakn.pi = adhoc.hakn.pi,
              df.hakn.pi = if (any(method.predict == "HK")) df.hakn.pi else NA,
              #
              seTE.predict = seTE.predict,
              df.predict = df.predict,
              level.predict = level.predict,
              lower.predict = lower.predict,
              upper.predict = upper.predict,
              seTE.hakn.pi = seTE.hakn.pi,
              seTE.hakn.adhoc.pi = seTE.hakn.adhoc.pi,
              #
              Q = hc$Q, df.Q = hc$df.Q, pval.Q = hc$pval.Q,
              #
              method.tau = method.tau,
              control = control,
              method.tau.ci = hc$method.tau.ci,
              level.hetstat = level.hetstat,
              tau2 = hc$tau2,
              se.tau2 = hc$se.tau2,
              lower.tau2 = hc$lower.tau2, upper.tau2 = hc$upper.tau2,
              tau = hc$tau,
              lower.tau = hc$lower.tau, upper.tau = hc$upper.tau,
              tau.preset = tau.preset,
              TE.tau =
                if (!missing(TE.tau) & method.tau == "DL") TE.tau else NULL,
              detail.tau = detail.tau,
              sign.lower.tau = hc$sign.lower.tau,
              sign.upper.tau = hc$sign.upper.tau,
              #
              method.I2 = method.I2,
              #
              H = hc$H, lower.H = hc$lower.H, upper.H = hc$upper.H,
              #
              I2 = hc$I2, lower.I2 = hc$lower.I2, upper.I2 = hc$upper.I2,
              #
              Rb = Rbres$TE, lower.Rb = Rbres$lower, upper.Rb = Rbres$upper,
              #
              method.bias = method.bias,
              #
              text.common = text.common, text.random = text.random,
              text.predict = text.predict,
              text.w.common = text.w.common, text.w.random = text.w.random,
              #
              title = title, complab = complab, outclab = outclab,
              #
              label.e = label.e, label.c = label.c,
              label.left = label.left, label.right = label.right,
              col.label.left = replaceNULL(col.label.left, "black"),
              col.label.right = replaceNULL(col.label.right, "black"),
              #
              keepdata = keepdata,
              data = if (keepdata) data else NULL,
              subset = if (keepdata) subset else NULL,
              exclude = if (!missing.exclude) exclude else NULL,
              # No general list elements
              n.e = n.e,
              n.c = n.c,
              pscale = pscale,
              irscale = irscale, irunit = irunit,
              method.ci = method.ci,              
              method.mean = method.mean,
              approx.TE = if (all(approx.TE == "")) NULL else approx.TE,
              approx.seTE = if (all(approx.seTE == "")) NULL else approx.seTE,
              #
              seed.predict = seed.predict,
              #
              pairwise = is.pairwise,
              #
              warn = warn,
              call = match.call(),
              version = packageDescription("meta")$Version,
              # Keep debug information
              tau2.calc = tau2.calc,
              rma.usw.c = if (usw.common & keeprma) m4.usw.c else NULL,
              rma.usw.r = if (usw.random & keeprma) m4.usw.r else NULL,
              rma.three.level = if (three.level & keeprma) m4 else NULL,
              # Deprecated list elements
              zval = ci.study$statistic,
              hakn = any(method.random.ci == "HK"),
              zval.common = statistic.common,
              zval.random = statistic.random
              )
  #
  if (is.pairwise | data.pairwise) {
    res$pairwise <- TRUE
    res$k.study <- length(unique(res$studlab[!is.na(res$TE)]))
  }
  #
  class(res) <- c(fun, "meta")
  #
  if (avail.lower | avail.upper)
    res$level.ci <- level.ci
  #
  # Add results from subgroup analysis
  #
  if (by) {
    res$subgroup <- subgroup
    res$subgroup.name <- subgroup.name
    #
    res$tau.common <- tau.common
    res$print.subgroup.name <- print.subgroup.name
    res$sep.subgroup <- sep.subgroup
    res$test.subgroup <- test.subgroup
    res$prediction.subgroup <- prediction.subgroup
    #
    if (!tau.common) {
      res <- c(res, subgroup(res, seed = seed.predict.subgroup))
      if (res$three.level)
        res <- setNA3(res)
    }
    else if (!is.null(tau.preset))
      res <-
        c(res, subgroup(res, tau.preset, seed = seed.predict.subgroup))
    else {
      if (three.level)
        res <- c(res,
                 subgroup(res, NULL,
                          factor(res$subgroup, bylevs(res$subgroup))))
      else
        res <-
          c(res, subgroup(res, hcc$tau.resid, seed = seed.predict.subgroup))
    }
    #
    if (tau.common && is.null(tau.preset))
      res <- addHet(res, hcc)
    else {
      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
      #
      res$Q.resid <- res$df.Q.resid <- res$pval.Q.resid <- NA
      res$H.resid <- res$lower.H.resid <- res$upper.H.resid <- NA
      res$I2.resid <- res$lower.I2.resid <- res$upper.I2.resid <- NA
    }
    #
    res$event.e.w <- NULL
    res$event.c.w <- NULL
    res$event.w   <- NULL
    res$n.w       <- NULL
    res$time.e.w  <- NULL
    res$time.c.w  <- NULL
    #
    res <- setNAwithin(res, res$three.level)
  }
  #
  # Add names to tau2 & rest (if necessary)
  #
  if (length(res$tau2) > 1)
    names(res$tau2) <- res$detail.tau
  #
  if (length(res$tau) > 1)
    names(res$tau) <- res$detail.tau
  #
  if (length(res$tau2.resid) > 1)
    names(res$tau2.resid) <- res$detail.tau
  #
  if (length(res$tau.resid) > 1)
    names(res$tau.resid) <- res$detail.tau
  #
  # Unset variables for prediction intervals
  #
  res$method.predict <-
    ifelse(is.na(res$lower.predict) & is.na(res$upper.predict),
           "", res$method.predict)
  res$df.predict <-
    ifelse(is.na(res$lower.predict) & is.na(res$upper.predict),
           NA, res$df.predict)
  res$adhoc.hakn.pi <-
    ifelse(is.na(res$lower.predict) & is.na(res$upper.predict),
           "", res$adhoc.hakn.pi)
  #
  # Backward compatibility
  #
  res <- backward(res)
  #
  class(res) <- c(fun, "meta")
  
  
  res
}

#' Cumulative meta-analysis
#' 
#' @description
#' Performs a cumulative meta-analysis.
#' 
#' @param x An object of class \code{meta}.
#' @param pooled A character string indicating whether a common effect
#'   or random effects model is used for pooling. Either missing (see
#'   Details), \code{"common"}, or \code{"random"}, can be abbreviated.
#' @param sortvar An optional vector used to sort the individual
#'   studies (must be of same length as \code{x$TE}).
#' @param prediction A logical indicating whether to report prediction
#'   intervals.
#' @param overall A logical indicating whether overall results should be
#'   reported.
#' @param text.pooled A character string used in printouts and forest
#'   plots to label the pooled effect estimate.
#' @param no A numeric specifying which meta-analysis results to
#'   consider.
#' @param cid A numeric value or vector specifying clinically important
#'   differences (CID) / decision thresholds used to calculate expected
#'   proportions of clinically important benefit or harm
#'   (see \code{\link{cidprop}}).
#' @param cid.below.null A single numeric defining the decision threshold below
#'   the null effect to distinguish clinically important from not important
#'   effects (see \code{\link{cidprop}}).
#' @param cid.above.null A single numeric defining the decision threshold above
#'   the null effect to distinguish clinically important from not important
#'   effects (see \code{\link{cidprop}}).
#' @param small.values A character string specifying whether small
#'   treatment effects indicate a beneficial (\code{"desirable"}) or
#'   harmful (\code{"undesirable"}) effect, can be abbreviated
#'   (see \code{\link{cidprop}}).
#' @param \dots Additional arguments (ignored).
#' 
#' @details
#' A cumulative meta-analysis is performed. Studies are included
#' sequentially as defined by \code{sortvar}.
#' 
#' Information from object \code{x} is utilised if argument
#' \code{pooled} is missing. A common effect model is assumed
#' (\code{pooled = "common"}) if argument \code{x$common} is
#' \code{TRUE}; a random effects model is assumed (\code{pooled =
#' "random"}) if argument \code{x$random} is \code{TRUE} and
#' \code{x$common} is \code{FALSE}.
#' 
#' @return
#' An object of class \code{"metacum"} with dedicated print and forest
#' functions.
#' 
#' The following list elements provide results from meta-analyses, each
#' adding one study at a time (see \code{\link{meta-object}} for more
#' information on these list elements):
#' \tabular{l}{
#' \cr
#' studlab, TE, seTE, df.random, lower, upper, statistic, pval, \cr
#' lower.predict, upper.predict, df.predict, w (sum of weights), \cr
#' tau2, se.tau2, lower.tau2, upper.tau2, tau, lower.tau, upper.tau, \cr
#' I2, lower.I2, upper.I2, Rb, n.harmonic.mean, t.harmonic.mean, \cr
#' k, k.study, k.all, k.TE, k.MH.
#' }
#' 
#' The following list elements contain results of the original meta-analysis:
#' \tabular{l}{
#' \cr
#' TE.pooled, seTE.pooled, df.random.pooled, \cr
#' lower.pooled, upper.pooled, statistic.pooled, pval.pooled, \cr
#' lower.predict.pooled, upper.predict.pooled, \cr
#' df.predict.pooled, w.pooled, \cr
#' tau2.pooled, se.tau2.pooled, lower.tau2.pooled, upper.tau2.pooled, \cr
#' tau.pooled, lower.tau.pooled, upper.tau.pooled, \cr
#' I2.pooled, lower.I2.pooled, upper.I2.pooled, Rb.pooled, \cr
#' n.harmonic.mean.pooled, t.harmonic.mean.pooled, \cr
#' k.pooled, k.study.pooled, k.all.pooled, k.TE.pooled, k.MH.pooled.
#' }
#' 
#' @author Guido Schwarzer \email{guido.schwarzer@@uniklinik-freiburg.de}
#' 
#' @seealso \code{\link{forest.metacum}}, \code{\link{print.metacum}},
#'   \code{\link{cidprop}}
#' 
#' @references
#' Cooper H & Hedges LV (1994):
#' \emph{The Handbook of Research Synthesis}.
#' Newbury Park, CA: Russell Sage Foundation
#'
#' @examples
#' data(Fleiss1993bin)
#' ma1 <- metabin(d.asp, n.asp, d.plac, n.plac,
#'   data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
#' ma1
#' metacum(ma1)
#' metacum(ma1, pooled = "random")
#' 
#' forest(metacum(ma1))
#' forest(metacum(ma1, pooled = "random"))
#' 
#' metacum(ma1, sortvar = study)
#' metacum(ma1, sortvar = 7:1)
#' 
#' ma2 <- update(ma1, title = "Fleiss1993bin meta-analysis", backtransf = FALSE)
#' metacum(ma2)
#' 
#' data(Fleiss1993cont)
#' ma3 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
#'   data = Fleiss1993cont, sm = "SMD")
#' metacum(ma3)
#' 
#' @rdname metacum
#' @method metacum meta
#' @export

metacum.meta <- function(x, pooled, sortvar, prediction, overall = x$overall,
                         text.pooled, no = 1,
                         cid = NULL,
                         cid.below.null = NULL, cid.above.null = NULL,
                         small.values = "desirable",
                         ...) {
  
  #
  #
  # (1) Check for meta object and upgrade older meta objects
  #
  #
  
  chkclass(x, "meta")
  chksuitable(x, "Cumulative meta-analysis",
              c("trimfill", "metamerge", "netpairwise"))
  #
  x <- updateversion(x)
  #
  if (!is.null(x$three.level) && x$three.level)
    stop("Cumulative meta-analysis not implemented for a ",
         "multi-level meta-analysis.",
         call. = FALSE)
  #
  k.all <- length(x$TE)
  if (k.all < 2) {
    warning("Nothing calculated (minimum number of studies: 2).")
    return(invisible(NULL))
  }
  
  
  #
  #
  # (2) Check other arguments
  #
  #
  
  if (!missing(pooled)) {
    pooled <- setchar(pooled, c("common", "random", "fixed"))
    pooled[pooled == "fixed"] <- "common"
  }
  else
    if (!x$common & x$random)
      pooled <- "random"
    else
      pooled <- "common"
  #
  if (missing(prediction))
    prediction <- pooled == "random" & x$prediction
  else
    chklogical(prediction)
  #
  chklogical(overall)
  #
  tdist_random <- pooled == "random" & x$method.random.ci %in% c("HK", "KR")
  tdist_predict <- !(x$method.predict %in% c("S", ""))
  #
  mc <- match.call()
  error <-
    try(sortvar <-
          catch("sortvar", mc, x, sys.frame(sys.parent())),
        silent = TRUE)
  if (inherits(error, "try-error")) {
    sortvar <- catch("sortvar", mc, x$data,  NULL)
    if (isCol(x$data, ".subset"))
      sortvar <- sortvar[x$data$.subset]
  }
  sort <- !is.null(sortvar)
  if (sort && (length(sortvar) != k.all))
    stop("Number of studies in object 'x' and argument 'sortvar' ",
         "have different length.")
  if (!sort)
    sortvar <- seq_len(k.all)
  #
  if (!is.null(cid))
    chknumeric(cid, length = 1)
  #
  if (!is.null(cid.below.null))
    chknumeric(cid.below.null, length = 1)
  #
  if (!is.null(cid.above.null))
    chknumeric(cid.above.null, length = 1)
  #
  avail.cid <- !is.null(cid) && !all(is.na(cid))
  avail.cid.below.null <-
    !is.null(cid.below.null) && !all(is.na(cid.below.null))
  avail.cid.above.null <-
    !is.null(cid.above.null) && !all(is.na(cid.above.null))
  #
  run_cidprop <- avail.cid | avail.cid.below.null | avail.cid.above.null
  
  
  #
  #
  # (3) Sort variables
  #
  #
  
  o <- order(sortvar)
  #
  n.e <- x$n.e[o]
  n.c <- x$n.c[o]
  n   <- x$n[o]
  #
  event.e <- x$event.e[o]
  event.c <- x$event.c[o]
  event   <- x$event[o]
  #
  mean.e <- x$mean.e[o]
  mean.c <- x$mean.c[o]
  mean   <- x$mean[o]
  #
  sd.e <- x$sd.e[o]
  sd.c <- x$sd.c[o]
  sd   <- x$sd[o]
  #
  time.e <- x$time.e[o]
  time.c <- x$time.c[o]
  time   <- x$time[o]
  #
  cor <- x$cor[o]
  #
  TE <- x$TE[o]
  seTE <- x$seTE[o]
  #
  if (!is.null(x$cluster))
    cluster <- x$cluster[o]
  else
    cluster <- NULL
  #
  if (!is.null(x$weights.common))
    weights.common <- x$weights.common[o]
  else
    weights.common <- NULL
  #
  if (!is.null(x$weights.random))
    weights.random <- x$weights.random[o]
  else
    weights.random <- NULL
  #
  incr.e <- x$incr.e[o]
  incr.c <- x$incr.c[o]
  #
  if (length(x$incr) > 1)
    incr <- x$incr[o]
  else if (!is.null(x$incr))
    incr <- rep_len(x$incr, k.all)
  else
    incr <- x$incr
  #
  # Exclude studies from meta-analysis
  #
  if (!is.null(x$exclude))
    exclude <- x$exclude[o]
  else
    exclude <- rep_len(FALSE, k.all)
  #
  ncum <- cumsum(!exclude)
  #
  studlab <- x$studlab[o]
  slab <- character(k.all)
  for (i in seq_len(k.all))
    slab[i] <- paste0("Adding ", studlab[i], " (k=", ncum[i], ")")
  #
  chknumeric(no, min = 1, length = 1)
  #
  # Select a single common effect or random effects models
  #
  if (pooled == "common") {
    if (no > length(x$seTE.common))
      stop("Argument 'no' must be smaller or equal to ",
           "number of common effect estimates.",
           call. = FALSE)
    #
    no.c <- no
    no.r <- 1
  }
  else {
    if (no > length(x$seTE.random))
      stop("Argument 'no' must be smaller or equal to ",
           "number of random effects estimates.",
           call. = FALSE)
    #
    no.c <- 1
    no.r <- no
  }
  #
  x$TE.common <- x$TE.common[no.c]
  x$seTE.common <- x$seTE.common[no.c]
  x$statistic.common <- x$statistic.common[no.c]
  x$pval.common <- x$pval.common[no.c]
  x$lower.common <- x$lower.common[no.c]
  x$upper.common <- x$upper.common[no.c]
  x$zval.common <- x$zval.common[no.c]
  x$text.common <- x$text.common[no.c]
  #
  if (length(x$TE.random) == 1 &&
      length(x$TE.random) != length(x$seTE.random))
    x$TE.random <- rep_len(x$TE.random, length(x$seTE.random))
  x$TE.random <- x$TE.random[no.r]
  x$seTE.random <- x$seTE.random[no.r]
  x$df.random <- x$df.random[no.r]
  x$statistic.random <- x$statistic.random[no.r]
  x$pval.random <- x$pval.random[no.r]
  x$lower.random <- x$lower.random[no.r]
  x$upper.random <- x$upper.random[no.r]
  x$zval.random <- x$zval.random[no.r]
  x$seTE.hakn.adhoc.ci <- x$seTE.hakn.adhoc.ci[no.r]
  x$df.hakn.ci <- x$df.hakn.ci[no.r]
  #
  x$text.random <- x$text.random[no.r]
  x$method.random.ci <- x$method.random.ci[no.r]
  x$adhoc.hakn.ci <- x$adhoc.hakn.ci[no.r]
  #
  x$seTE.hakn.adhoc <- x$seTE.hakn.adhoc[no.r]
  x$df.hakn <- x$df.hakn[no.r]
  #
  x$lower.predict <- x$lower.predict[1]
  x$upper.predict <- x$upper.predict[1]
  x$seTE.predict <- x$seTE.predict[1]
  x$df.predict <- x$df.predict[1]
  x$seTE.hakn.adhoc.pi <- x$seTE.hakn.adhoc.pi[1]
  x$df.hakn.pi <- x$df.hakn.pi[1]
  #
  x$text.predict <- x$text.predict[1]
  x$method.predict <- x$method.predict[1]
  x$adhoc.hakn.pi <- x$adhoc.hakn.pi[1]
  
  
  #
  #
  # (4) Do sensitivity analysis
  #
  #
  
  res.i <- matrix(NA, ncol = 31, nrow = k.all)
  add.i <- matrix(NA, ncol = 3, nrow = k.all)
  #
  for (i in seq_len(k.all)) {
    sel <- 1:i
    #
    if (length(incr) > 1)
      incr.i <- incr[sel]
    else
      incr.i <- incr
    #
    if (inherits(x, "metabin"))
      m <- metabin(event.e[sel], n.e[sel], event.c[sel], n.c[sel],
                   #
                   exclude = exclude[sel],
                   #
                   cluster = cluster[sel], rho = x$rho,
                   #
                   weights.common = weights.common[sel],
                   weights.random = weights.random[sel],
                   #
                   method = x$method, sm = x$sm,
                   #
                   incr.e = incr.e[sel], incr.c = incr.c[sel],
                   allstudies = x$allstudies, MH.exact = x$MH.exact,
                   RR.Cochrane = x$RR.Cochrane, Q.Cochrane = x$Q.Cochrane,
                   model.glmm =
                     if (!is.null(x$model.glmm)) x$model.glmm else "UM.FS",
                   #
                   method.tau = x$method.tau,
                   tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                   #
                   level.ma = x$level.ma,
                   method.common.ci = x$method.common.ci,
                   method.random.ci = x$method.random.ci,
                   adhoc.hakn.ci = x$adhoc.hakn.ci,
                   #
                   level.predict = x$level.predict,
                   method.predict = x$method.predict,
                   adhoc.hakn.pi = x$adhoc.hakn.pi,
                   #
                   keepdata = FALSE,
                   warn = FALSE,
                   #
                   control = x$control)
    #
    if (inherits(x, "metacont"))
      m <- metacont(n.e[sel], mean.e[sel], sd.e[sel],
                    n.c[sel], mean.c[sel], sd.c[sel],
                    #
                    exclude = exclude[sel],
                    #
                    cluster = cluster[sel], rho = x$rho,
                    #
                    weights.common = weights.common[sel],
                    weights.random = weights.random[sel],
                    #
                    sm = x$sm,
                    pooledvar = replaceNA(x$pooledvar, gs("pooledvar")),
                    #
                    method.tau = x$method.tau,
                    tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                    #
                    level.ma = x$level.ma,
                    method.common.ci = x$method.common.ci,
                    method.random.ci = x$method.random.ci,
                    adhoc.hakn.ci = x$adhoc.hakn.ci,
                    #
                    level.predict = x$level.predict,
                    method.predict = x$method.predict,
                    adhoc.hakn.pi = x$adhoc.hakn.pi,
                    #
                    keepdata = FALSE,
                    warn = FALSE,
                    #
                    control = x$control)
    #
    if (inherits(x, "metacor"))
      m <- metacor(cor[sel], n[sel],
                   #
                   exclude = exclude[sel],
                   #
                   cluster = cluster[sel], rho = x$rho,
                   #
                   weights.common = weights.common[sel],
                   weights.random = weights.random[sel],
                   #
                   sm = x$sm, null.effect = x$null.effect,
                   #
                   method.tau = x$method.tau,
                   tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                   #
                   level.ma = x$level.ma,
                   method.common.ci = x$method.common.ci,
                   method.random.ci = x$method.random.ci,
                   adhoc.hakn.ci = x$adhoc.hakn.ci,
                   #
                   level.predict = x$level.predict,
                   method.predict = x$method.predict,
                   adhoc.hakn.pi = x$adhoc.hakn.pi,
                   #
                   keepdata = FALSE,
                   #
                   control = x$control)
    #
    if (inherits(x, "metagen"))
      m <- metagen(TE[sel], seTE[sel],
                   #
                   exclude = exclude[sel],
                   #
                   cluster = cluster[sel], rho = x$rho,
                   #
                   weights.common = weights.common[sel],
                   weights.random = weights.random[sel],
                   #
                   sm = x$sm, null.effect = x$null.effect,
                   #
                   method.tau = x$method.tau,
                   tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                   #
                   level.ma = x$level.ma,
                   method.common.ci = x$method.common.ci,
                   method.random.ci = x$method.random.ci,
                   adhoc.hakn.ci = x$adhoc.hakn.ci,
                   #
                   level.predict = x$level.predict,
                   method.predict = x$method.predict,
                   adhoc.hakn.pi = x$adhoc.hakn.pi,
                   #
                   keepdata = FALSE,
                   warn = FALSE,
                   #
                   control = x$control)
    #
    if (inherits(x,"metainc"))
      m <- metainc(event.e[sel], time.e[sel],
                   event.c[sel], time.c[sel],
                   #
                   exclude = exclude[sel],
                   #
                   cluster = cluster[sel], rho = x$rho,
                   #
                   weights.common = weights.common[sel],
                   weights.random = weights.random[sel],
                   #
                   method = x$method, sm = x$sm,
                   #
                   incr.e = incr.e[sel], incr.c = incr.c[sel],
                   model.glmm =
                     if (!is.null(x$model.glmm)) x$model.glmm else "UM.FS",
                   #
                   method.tau = x$method.tau,
                   tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                   #
                   level.ma = x$level.ma,
                   method.common.ci = x$method.common.ci,
                   method.random.ci = x$method.random.ci,
                   adhoc.hakn.ci = x$adhoc.hakn.ci,
                   #
                   level.predict = x$level.predict,
                   method.predict = x$method.predict,
                   adhoc.hakn.pi = x$adhoc.hakn.pi,
                   #
                   keepdata = FALSE,
                   warn = FALSE,
                   #
                   control = x$control)
    #
    if (inherits(x, "metamean"))
      m <- metamean(n[sel], mean[sel], sd[sel],
                    #
                    exclude = exclude[sel],
                    #
                    cluster = cluster[sel], rho = x$rho,
                    #
                    weights.common = weights.common[sel],
                    weights.random = weights.random[sel],
                    #
                    sm = x$sm, null.effect = x$null.effect,
                    #
                    method.tau = x$method.tau,
                    tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                    #
                    level.ma = x$level.ma,
                    method.common.ci = x$method.common.ci,
                    method.random.ci = x$method.random.ci,
                    adhoc.hakn.ci = x$adhoc.hakn.ci,
                    #
                    level.predict = x$level.predict,
                    method.predict = x$method.predict,
                    adhoc.hakn.pi = x$adhoc.hakn.pi,
                    #
                    keepdata = FALSE,
                    warn = FALSE,
                    #
                    control = x$control)
    #
    if (inherits(x, "metaprop"))
      m <- metaprop(event[sel], n[sel],
                    #
                    exclude = exclude[sel],
                    #
                    cluster = cluster[sel], rho = x$rho,
                    #
                    weights.common = weights.common[sel],
                    weights.random = weights.random[sel],
                    #
                    method = x$method, sm = x$sm, null.effect = x$null.effect,
                    #
                    incr = incr.i, method.incr = x$method.incr,
                    method.ci = x$method.ci,
                    #
                    method.tau = x$method.tau,
                    tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                    #
                    level.ma = x$level.ma,
                    method.common.ci = x$method.common.ci,
                    method.random.ci = x$method.random.ci,
                    adhoc.hakn.ci = x$adhoc.hakn.ci,
                    #
                    level.predict = x$level.predict,
                    method.predict = x$method.predict,
                    adhoc.hakn.pi = x$adhoc.hakn.pi,
                    #
                    keepdata = FALSE,
                    warn = FALSE,
                    #
                    control = x$control)
    #
    if (inherits(x, "metarate"))
      m <- metarate(event[sel], time[sel],
                    #
                    exclude = exclude[sel],
                    #
                    cluster = cluster[sel], rho = x$rho,
                    #
                    weights.common = weights.common[sel],
                    weights.random = weights.random[sel],
                    #
                    method = x$method, sm = x$sm, null.effect = x$null.effect,
                    #
                    incr = incr.i, method.incr = x$method.incr,
                    #
                    method.tau = x$method.tau,
                    tau.preset = x$tau.preset, TE.tau = x$TE.tau,
                    #
                    level.ma = x$level.ma,
                    method.common.ci = x$method.common.ci,
                    method.random.ci = x$method.random.ci,
                    adhoc.hakn.ci = x$adhoc.hakn.ci,
                    #
                    level.predict = x$level.predict,
                    method.predict = x$method.predict,
                    adhoc.hakn.pi = x$adhoc.hakn.pi,
                    #
                    keepdata = FALSE,
                    warn = FALSE,
                    #
                    control = x$control)
    #
    sel.pft <- inherits(x, "metaprop") & x$sm == "PFT"
    sel.irft <- inherits(x, "metarate") & x$sm == "IRFT"
    #
    add.i[i, ] <- c(m$method.tau.ci,  # 1
                    m$sign.lower.tau, # 2
                    m$sign.upper.tau  # 3
                    )
    #
    if (pooled == "common") {
      res.i[i, ] <- c(m$TE.common,                                   #  1
                      m$seTE.common,                                 #  2
                      m$lower.common,                                #  3
                      m$upper.common,                                #  4
                      m$statistic.common,                            #  5
                      m$pval.common,                                 #  6
                      #
                      m$tau2,                                        #  7
                      m$lower.tau2,                                  #  8
                      m$upper.tau2,                                  #  9
                      m$se.tau2,                                     # 10
                      #
                      m$tau,                                         # 11
                      m$lower.tau,                                   # 12
                      m$upper.tau,                                   # 13
                      #
                      m$I2,                                          # 14
                      m$lower.I2,                                    # 15
                      m$upper.I2,                                    # 16
                      #
                      m$Rb,                                          # 17
                      #
                      sum(m$w.common, na.rm = TRUE),                 # 18
                      if (sel.pft) 1 / mean(1 / n[sel]) else NA,     # 19
                      if (sel.irft) 1 / mean(1 / time[sel]) else NA, # 20
                      #
                      NA,                                            # 21
                      #
                      NA,                                            # 22
                      NA,                                            # 23
                      NA,                                            # 24
                      #
                      m$k,                                           # 25
                      m$k.study,                                     # 26
                      m$k.all,                                       # 27
                      m$k.TE,                                        # 28
                      replaceNULL(m$k.MH),                           # 29
                      NA,                                            # 30
                      NA                                             # 31
      )
    }
    #
    else if (pooled == "random") {
      if (run_cidprop) {
        pp <-
          cidprop(m,
                   cid = cid,
                   cid.below.null = cid.below.null,
                   cid.above.null = cid.above.null,
                   small.values = small.values)
        #
        prop.cid.below.null <- pp$prop.cid.below.null
        prop.cid.above.null <- pp$prop.cid.above.null
      }
      else {
        prop.cid.below.null <- NA
        prop.cid.above.null <- NA
      }
      #
      res.i[i, ] <- c(m$TE.random,                                   #  1
                      m$seTE.random,                                 #  2
                      m$lower.random,                                #  3
                      m$upper.random,                                #  4
                      m$statistic.random,                            #  5
                      m$pval.random,                                 #  6
                      #
                      m$tau2,                                        #  7
                      m$lower.tau2,                                  #  8
                      m$upper.tau2,                                  #  9
                      m$se.tau2,                                     # 10
                      #
                      m$tau,                                         # 11
                      m$lower.tau,                                   # 12
                      m$upper.tau,                                   # 13
                      #
                      m$I2,                                          # 14
                      m$lower.I2,                                    # 15
                      m$upper.I2,                                    # 16
                      #
                      m$Rb,                                          # 17
                      #
                      sum(m$w.random, na.rm = TRUE),                 # 18
                      if (sel.pft) 1 / mean(1 / n[sel]) else NA,     # 19
                      if (sel.irft) 1 / mean(1 / time[sel]) else NA, # 20
                      #
                      if (tdist_random) m$df.random else NA,         # 21
                      #
                      m$lower.predict,                               # 22
                      m$upper.predict,                               # 23
                      if (tdist_predict) m$df.predict else NA,       # 24
                      #
                      m$k,                                           # 25
                      m$k.study,                                     # 26
                      m$k.all,                                       # 27
                      m$k.TE,                                        # 28
                      replaceNULL(m$k.MH),                           # 29
                      prop.cid.below.null,                           # 30
                      prop.cid.above.null                            # 31
      )
    }
  }
  #
  TE.i <- res.i[, 1]
  seTE.i <- res.i[, 2]
  lower.i <- res.i[, 3]
  upper.i <- res.i[, 4]
  statistic.i <- res.i[, 5]
  pval.i <- res.i[, 6]
  #
  tau2.i <- res.i[, 7]
  lower.tau2.i <- res.i[, 8]
  upper.tau2.i <- res.i[, 9]
  se.tau2.i <- res.i[, 10]
  #
  tau.i <- res.i[, 11]
  lower.tau.i <- res.i[, 12]
  upper.tau.i <- res.i[, 13]
  #
  I2.i <- res.i[, 14]
  lower.I2.i <- res.i[, 15]
  upper.I2.i <- res.i[, 16]
  #
  Rb.i <- res.i[, 17]
  #
  weight.i <- res.i[, 18]
  n.harmonic.mean.i <- res.i[, 19]
  t.harmonic.mean.i <- res.i[, 20]
  #
  df.random.i <- res.i[, 21]
  #
  lower.predict.i <- res.i[, 22]
  upper.predict.i <- res.i[, 23]
  df.predict.i <- res.i[, 24]
  #
  k.i <- res.i[, 25]
  k.study.i <- res.i[, 26]
  k.all.i <- res.i[, 27]
  k.TE.i <- res.i[, 28]
  k.MH.i <- res.i[, 29]
  #
  prop.cid.below.null <- res.i[, 30]
  prop.cid.above.null <- res.i[, 31]
  #
  method.tau.ci <- replaceNULL(unique(add.i[add.i[, 1] != "", 1]), "")
  sign.lower.tau.i <- replaceNULL(unique(add.i[add.i[, 2] != "", 2]), "")
  sign.upper.tau.i <- replaceNULL(unique(add.i[add.i[, 3] != "", 3]), "")
  #
  if (pooled == "common") {
    TE.pooled <- x$TE.common
    seTE.pooled <- x$seTE.common
    lower.pooled <- x$lower.common
    upper.pooled <- x$upper.common
    statistic.pooled <- x$statistic.common
    pval.pooled <- x$pval.common
    w.pooled <- sum(x$w.common, na.rm = TRUE)
    #
    lower.predict.pooled <- NA
    upper.predict.pooled <- NA
  }
  #
  else if (pooled == "random") {
    TE.pooled <- x$TE.random
    seTE.pooled <- x$seTE.random
    lower.pooled <- x$lower.random
    upper.pooled <- x$upper.random
    statistic.pooled <- x$statistic.random
    pval.pooled <- x$pval.random
    w.pooled <- sum(x$w.random, na.rm = TRUE)
    #
    lower.predict.pooled <- x$lower.predict
    upper.predict.pooled <- x$upper.predict
  }
  #
  df.random.pooled <- x$df.random
  #
  tau2.pooled <- x$tau2
  se.tau2.pooled <- x$se.tau2
  lower.tau2.pooled <- x$lower.tau2
  upper.tau2.pooled <- x$upper.tau2
  #
  tau.pooled <- x$tau
  lower.tau.pooled <- x$lower.tau
  upper.tau.pooled <- x$upper.tau
  #
  I2.pooled <- x$I2
  lower.I2.pooled <- x$lower.I2
  upper.I2.pooled <- x$upper.I2
  #
  Rb.pooled <- x$Rb
  #
  n.harmonic.mean.pooled <- 1 / mean(1 / n)
  t.harmonic.mean.pooled <- 1 / mean(1 / time)
  #
  if (pooled == "random" & run_cidprop) {
    pp.pooled <-
      cidprop(x,
               cid = cid,
               cid.below.null = cid.below.null,
               cid.above.null = cid.above.null,
               small.values = small.values)
    #
    prop.cid.below.null.pooled <- pp.pooled$prop.cid.below.null
    prop.cid.above.null.pooled <- pp.pooled$prop.cid.above.null
  }
  else {
    prop.cid.below.null.pooled <- NA
    prop.cid.above.null.pooled <- NA
  }
  
  
  #
  #
  # (5) Generate R object
  #
  #
  
  if (missing(text.pooled))
    text.pooled <- if (pooled == "common") x$text.common else x$text.random
  #
  res <- list(studlab = slab,
              #
              TE = TE.i,
              seTE = seTE.i,
              df.random = df.random.i,
              lower = lower.i,
              upper = upper.i,
              statistic = statistic.i,
              pval = pval.i,
              #
              lower.predict = lower.predict.i,
              upper.predict = upper.predict.i,
              df.predict = df.predict.i,
              #
              w = weight.i,
              #
              tau2 = tau2.i,
              se.tau2 = se.tau2.i,
              lower.tau2 = lower.tau2.i,
              upper.tau2 = upper.tau2.i,
              #
              tau = tau.i,
              lower.tau = lower.tau.i,
              upper.tau = upper.tau.i,
              #
              sign.lower.tau = sign.lower.tau.i,
              sign.upper.tau = sign.upper.tau.i,
              #
              I2 = I2.i,
              lower.I2 = lower.I2.i,
              upper.I2 = upper.I2.i,
              #
              Rb = Rb.i,
              #
              n.harmonic.mean = n.harmonic.mean.i,
              t.harmonic.mean = t.harmonic.mean.i,
              #
              k = k.i,
              k.study = k.study.i,
              k.all = k.all.i,
              k.TE = k.TE.i,
              k.MH = k.MH.i,
              #
              prop.cid.below.null = prop.cid.below.null,
              prop.cid.above.null = prop.cid.above.null,
              cid.below.null = cid.below.null,
              cid.above.null = cid.above.null,
              small.values = small.values,
              #
              sm = x$sm,
              null.effect = x$null.effect,
              #
              pooled = pooled,
              common = pooled == "common",
              random = pooled == "random",
              overall = overall,
              overall.hetstat = FALSE,
              #
              prediction = prediction,
              method.predict = x$method.predict,
              adhoc.hakn.pi = x$adhoc.hakn.pi,
              #
              backtransf = x$backtransf,
              func.backtransf = x$func.backtransf,
              #
              level = x$level.ma,
              level.ma = x$level.ma,
              level.predict = x$level.predict,
              #
              method = x$method,
              method.random = x$method.random,
              #
              method.common.ci = x$method.common.ci,
              method.random.ci = x$method.random.ci,
              adhoc.hakn.ci = x$adhoc.hakn.ci,
              #
              method.tau = x$method.tau,
              method.tau.ci =
                if (length(method.tau.ci) > 0) method.tau.ci else "",
              #
              tau.preset = x$tau.preset,
              TE.tau = x$TE.tau,
              #
              method.I2 = x$method.I2,
              #
              k.pooled = x$k,
              k.study.pooled = x$k.study,
              k.all.pooled = x$k.all,
              k.TE.pooled = x$k.TE,
              k.MH.pooled = x$k.MH,
              #
              TE.pooled = TE.pooled,
              seTE.pooled = seTE.pooled,
              lower.pooled = lower.pooled,
              upper.pooled = upper.pooled,
              df.random.pooled = df.random.pooled,
              statistic.pooled = statistic.pooled,
              pval.pooled = pval.pooled,
              w.pooled = w.pooled,
              text.pooled = text.pooled,
              #
              lower.predict.pooled = lower.predict.pooled,
              upper.predict.pooled = upper.predict.pooled,
              df.predict.pooled = x$df.predict,
              text.predict = x$text.predict,
              #
              prop.cid.below.null.pooled = prop.cid.below.null.pooled,
              prop.cid.above.null.pooled = prop.cid.above.null.pooled,
              #
              Q.pooled = x$Q,
              Q.Cochrane = x$Q.Cochrane,
              #
              tau2.pooled = tau2.pooled,
              se.tau2.pooled = se.tau2.pooled,
              lower.tau2.pooled = lower.tau2.pooled,
              upper.tau2.pooled = upper.tau2.pooled,
              #
              tau.pooled = tau.pooled,
              lower.tau.pooled = x$lower.tau,
              upper.tau.pooled = upper.tau.pooled,
              #
              sign.lower.tau.pooled = x$sign.lower.tau,
              sign.upper.tau.pooled = x$sign.upper.tau,
              #
              I2.pooled = I2.pooled,
              lower.I2.pooled = lower.I2.pooled,
              upper.I2.pooled = upper.I2.pooled,
              #
              Rb.pooled = Rb.pooled,
              #
              n.harmonic.mean.pooled = n.harmonic.mean.pooled,
              t.harmonic.mean.pooled = t.harmonic.mean.pooled,
              #
              pscale = x$pscale,
              irscale = x$irscale, irunit = x$irunit,
              #
              label.e = x$label.e,
              label.c = x$label.c,
              #
              label.left = x$label.left,
              label.right = x$label.right,
              #
              title = x$title, complab = x$complab,
              outclab = x$outclab,
              #
              no = no,
              #
              x = x,
              #
              call = match.call())
  #
  if (!is.null(weights.common) & pooled == "common") {
    res$weights.common <- TRUE
    res$weights.random <- FALSE
  }
  #
  if (!is.null(weights.random) & pooled == "random") {
    res$weights.common <- FALSE
    res$weights.random <- TRUE
  }
  #
  if (run_cidprop) {
    res$cid.below.null <- pp$cid.below.null
    res$cid.above.null <- pp$cid.above.null
  }
  else {
    res$prop.cid.below.null <- NULL
    res$prop.cid.above.null <- NULL
    res$cid.below.null <- NULL
    res$cid.above.null <- NULL
    res$small.values <- NULL
    #
    res$prop.cid.below.null.pooled <- NULL
    res$prop.cid.above.null.pooled <- NULL
  }
  #
  res$version <- packageDescription("meta")$Version
  #
  res$x$common <- res$common
  res$x$random <- res$random
  #
  res$classes <- class(x)[class(x) != "meta"]
  #
  class(res) <- "metacum"
  #
  res
}


#' @rdname metacum
#' @export metacum

metacum <- function(x, ...) 
  UseMethod("metacum")


#' @rdname metacum
#' @method metacum default
#' @export

metacum.default <- function(x, ...)
  stop("Cumulative meta-analysis not available for an object of class '",
       class(x)[1], "'.",
       call. = FALSE)

#
#
# Definition of auxiliary functions for forest plots
#
#


add.label <- function(x, column,
                      xpos, ypos, just, fs.lr, ff.lr, col,
                      fontfamily,
                      ...) {
  #
  pushViewport(viewport(layout.pos.col = column, ...))
  #
  grid.text(x, x = xpos, y = ypos, just = just,
            gp = gpar(fontsize = fs.lr, fontface = ff.lr, col = col,
                      fontfamily = fontfamily))
  #
  popViewport()
  #
  invisible(NULL)
}


add.text <- function(x, column, ...) {
  #
  for (i in seq_len(length(x$rows))) {
    if (!is.na(x$rows[i])) {
      pushViewport(
        viewport(
          layout.pos.row = x$rows[i],
          layout.pos.col = column, ...))
      #
      grid.draw(x$labels[[i]])
      #
      popViewport()
    }
  }
  #
  invisible(NULL)
}


add.xlab <- function(x, column, xlab, xlab.add, newline.xlab,
                     xpos, ypos, fs.xlab, ff.xlab,
                     fontfamily) {
  #
  pushViewport(viewport(layout.pos.col = column, xscale = x$range))
  #
  # Label on x-axis:
  #
  grid.text(xlab,
            x = unit(xpos, "native"),
            y = unit(ypos, "lines"),
            just = "center",
            gp = gpar(fontsize = fs.xlab, fontface = ff.xlab,
                      fontfamily = fontfamily))
  #
  if (newline.xlab)
    grid.text(xlab.add,
              x = unit(xpos, "native"),
              y = unit(ypos - 1, "lines"),
              just = "center",
              gp = gpar(fontsize = fs.xlab, fontface = ff.xlab,
                        fontfamily = fontfamily))
  #
  popViewport()
  #
  invisible(NULL)
}


add.rob <- function(x, column, size, fs, ff, fontfamily,
                    rob, rob.levels, rob.symbols, rob.colour, ...) {
  #
  if (is.null(rob.levels)) {
    if (is.factor(rob))
      rob.levels <- levels(rob)
    else
      rob.levels <- unique(rob)
  }
  #
  n.levs <- length(rob.levels)
  #
  if (is.null(rob.symbols)) {
    if (n.levs == 3)
      rob.symbols <- c("-", "?", "+")
    else
      rob.symbols <- rev(seq_len(n.levs))
  }
  else if (length(rob.symbols) == 1 && is.logical(rob.symbols)) {
    if (rob.symbols) {
      if (n.levs == 3)
        rob.symbols <- c("-", "?", "+")
      else
        rob.symbols <- rev(seq_len(n.levs))
    }
  }
  else if (length(rob.symbols) != n.levs)
    stop("Wrong number of RoB symbols (argument 'rob.symbols').",
         call. = FALSE)
  #
  if (is.null(rob.colour)) {
    if (n.levs == 3)
      rob.colour <- c("red", "yellow", "green")
    else
      rob.colour <- rev(seq_len(n.levs))
  }
  #
  if (!(length(rob.symbols) == 1 && is.logical(rob.symbols) && !rob.symbols))
    txt.rob <-
      as.character(
        factor(rob, levels = rob.levels, labels = rob.symbols))
  else
    txt.rob <- NULL
  #
  col.rob <-
    as.character(
      factor(rob, levels = rob.levels,
             labels = rob.colour))
  #
  j <- 0
  #
  for (i in seq_len(length(x$rows))) {
    if (!is.na(x$rows[i])) {
      pushViewport(
        viewport(
          layout.pos.row = x$rows[i],
          layout.pos.col = column, ...))
      #
      if (i == 1)
        grid.draw(x$labels[[1]])
      else {
        j <- j + 1
        #
        grid.circle(x = unit(0.5, "npc"), y = unit(0.5, "npc"),
                    r = unit(size / 2, "snpc"),
                    gp = gpar(fill = col.rob[j], col = col.rob[j]))
        #
        if (!is.null(txt.rob) && !is.na(txt.rob[j]))
          grid.text(txt.rob[j],
                    x = unit(0.5, "npc"),
                    y = unit(if (txt.rob[j] %in% c("-", "+")) 0.58 else 0.5,
                             "npc"),
                    gp = gpar(fontsize = fs, fontface = ff,
                              fontfamily = fontfamily),
                    just = c("center", "center"))
      }
      #
      popViewport()
    }
  }
  #
  invisible(NULL)
}


draw.axis <- function(x, column, yS, log.xaxis, at, label,
                      fs.axis, ff.axis, fontfamily, lwd,
                      xlim, notmiss.xlim,
                      col.line, col.label) {
  #
  # Function to draw x-axis
  #
  pushViewport(
    viewport(
      layout.pos.row = max(yS, na.rm = TRUE),
      layout.pos.col = column,
      xscale = x$range))
  #
  # x-axis:
  #
  if (log.xaxis) {
    if (is.null(at)) {
      x1000 <- c(0.001, 0.1, 1,  10, 1000)
      x100  <- c(0.01 , 0.1, 1,  10, 100)
      x10   <- c(0.1  , 0.5, 1,   2, 10)
      x5    <- c(0.2  , 0.5, 1,   2, 5)
      x2    <- c(0.5  , 1, 2)
      x1.5  <- c(0.75 , 1, 1.5)
      x1.25 <- c(0.8  , 1, 1.25)
      x1    <- c(0.9  , 1, 1.1)
      #
      min.x <- min(exp(x$range[1]), 1)
      max.x <- max(exp(x$range[2]), 1)
      #
      if (all(x1000 >= min.x) &
          all(x1000 <= max.x))
        label <- x1000
      else if (all(x100 >= min.x) &
               all(x100 <= max.x))
        label <- x100
      else if (all(x10 >= min.x) &
               all(x10 <= max.x))
        label <- x10
      else if (all(x5 >= min.x) &
               all(x5 <= max.x))
        label <- x5
      else if (all(x2 >= min.x) &
               all(x2 <= max.x))
        label <- x2
      else if (all(x1.5 >= min.x) &
               all(x1.5 <= max.x))
        label <- x1.5
      else if (all(x1.25 >= min.x) &
               all(x1.25 <= max.x))
        label <- x1.25
      else if (all(x1 >= min.x) &
               all(x1 <= max.x))
        label <- x1
      else
        label <- 1
      #
      if (notmiss.xlim && is.numeric(xlim[1])) {
        if (exp(min(xlim)) < min(label))
          label <- c(exp(min(xlim)), label)
        if (exp(max(xlim)) > max(label))
          label <- c(label, exp(max(xlim)))
      }
      at <- log(label)
    }
    else {
      if (length(label) == 1 && is.logical(label) && label)
        label <- at
      at <- log(at)
    }
    # Print x-axis labels
    grid.xaxis(name = "xaxis1",
               at = at, label = label,
               gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                         fontfamily = fontfamily, lwd = lwd,
                         col = col.label, tcl = -0.1))
    # Print xaxis and tick marks (in different colour)
    grid.xaxis(name = "xaxis2",
               at = at, label = FALSE,
               gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                         fontfamily = fontfamily, lwd = lwd,
                         col = col.line, tcl = -0.1))
  }
  else {
    if (is.null(at)) {
      # Print x-axis labels
      grid.xaxis(name = "xaxis1",
        gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                  fontfamily = fontfamily, lwd = lwd,
                  col = col.label, tcl = -0.1))
      # Print xaxis and tick marks (in different colour)
      grid.xaxis(name = "xaxis2",
        label = FALSE,
        gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                  fontfamily = fontfamily, lwd = lwd,
                  col = col.line, tcl = -0.1))
    }
    else if ((length(label) == 1 && is.logical(label) && label) |
          (length(label) >= 1 & !is.logical(label))) {
      # Print x-axis labels
      grid.xaxis(name = "xaxis1",
        at = at, label = label,
        gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                  fontfamily = fontfamily, lwd = lwd,
                  col = col.label, tcl = -0.1))
      # Print xaxis and tick marks (in different colour)
      grid.xaxis(name = "xaxis2",
        at = at, label = FALSE,
        gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                  fontfamily = fontfamily, lwd = lwd,
                  col = col.line, tcl = -0.1))
    }
    else {
      # Print x-axis labels
      grid.xaxis(name = "xaxis1",
                 at = at,
                 gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                           fontfamily = fontfamily, lwd = lwd,
                           col = col.label, tcl = -0.1))
      # Print xaxis and tick marks (in different colour)
      grid.xaxis(name = "xaxis2",
        at = at, label = FALSE,
        gp = gpar(fontsize = fs.axis, fontface = ff.axis,
                  fontfamily = fontfamily, lwd = lwd,
                  col = col.line, tcl = -0.1))
    }
  }
  #
  popViewport()
  #
  invisible(NULL)
}


draw.ci.diamond <- function(TE, lower, upper,
                            size, min, max,
                            col.diamond, col.diamond.lines,
                            lwd) {
  #
  if (min > max) {
    tmp <- min
    min <- max
    max <- tmp
  }
  #
  if (!is.na(TE) &&
      ((min <= TE & TE <= max) |
       (min <= lower & lower <= max) |
       (min <= upper & upper <= max))
      ) {
    if (min <= lower & max >= upper) {
      grid.polygon(x = unit(c(lower, TE, upper, TE), "native"),
                   y = unit(0.5 + c(0, 0.4 * size, 0, -0.4 * size), "npc"),
                   gp = gpar(fill = col.diamond, col = col.diamond.lines,
                             lwd = lwd))
    }
    #
    else {
      if (min > lower) {
        x.min <- min
        y.min1 <- 0.5 + -0.4 * size * (lower - min) / (lower - TE)
        y.min2 <- 0.5 +  0.4 * size * (lower - min) / (lower - TE)
      }
      else {
        x.min <- lower
        y.min1 <- y.min2 <- 0.5
      }
      #
      if (max < upper) {
        x.max <- max
        y.max1 <- 0.5 +  0.4 * size * (upper - max) / (upper - TE)
        y.max2 <- 0.5 + -0.4 * size * (upper - max) / (upper - TE)
      }
      else {
        x.max <- upper
        y.max1 <- y.max2 <- 0.5
      }
      #
      grid.polygon(x = unit(c(x.min, x.min, TE, x.max, x.max, TE, x.min),
                            "native"),
                   y = unit(c(y.min1, y.min2, 0.5 + 0.4 * size,
                              y.max1, y.max2, 0.5 - 0.4 * size,
                              y.min1), "npc"),
                   gp = gpar(fill = col.diamond, col = col.diamond.lines,
                             lwd = lwd))
    }
  }
  #
  invisible(NULL)
}


draw.ci.predict <- function(lower.predict, upper.predict,
                            size, min, max,
                            col.predict, col.predict.lines) {
  #
  if (min > max) {
    tmp <- min
    min <- max
    max <- tmp
  }
  #
  if (!(is.na(lower.predict) | is.na(upper.predict))) {
    range <- max - min
    # Plot prediction interval only within plotting range
    if (min > lower.predict)
      x.min <- min + range / 40
    else
      x.min <- lower.predict
    #
    if (max < upper.predict)
      x.max <- max - range / 40
    else
      x.max <- upper.predict
    #
    grid.polygon(x = unit(c(x.min, x.min, x.max, x.max), "native"),
                 y = unit(0.5 + size * c(-1, 1, 1, -1) / 10, "npc"),
                 gp = gpar(fill = col.predict, col = col.predict.lines))
    #
    if (min > lower.predict)
      grid.lines(x = unit(c(min, min + 0.00001), "native"),
                 y = 0.5,
                 gp = gpar(col = col.predict.lines,
                           fill = col.predict),
                 arrow = arrow(ends = "first",
                               length = unit(0.5, "npc"),
                               type = "closed"))
    #
    if (max < upper.predict)
      grid.lines(x = unit(c(max - 0.00001, max), "native"),
                 y = 0.5,
                 gp = gpar(col = col.predict.lines,
                           fill = col.predict),
                 arrow = arrow(ends = "last",
                               length = unit(0.5, "npc"),
                               type = "closed"))
  }
  #
  invisible(NULL)
}


draw.ci <- function(TE, lower, upper,
                    size, min, max,
                    lwd,
                    col,
                    col.square, col.square.lines,
                    col.circle, col.circle.lines,
                    col.inside,
                    type,
                    lwd.square,
                    arrow.type, arrow.length) {
  #
  if (min > max) {
    tmp <- min
    min <- max
    max <- tmp
  }
  #
  if (!is.na(TE)) {
    #
    # Draw lines in colour "col.inside" if totally inside rect
    #
    TElineCol <- col
    #
    if ((!is.na(size) & !is.na(lower) & !is.na(upper)))
      if (size > 0 &&
          (convertX(unit(TE, "native") + unit(0.5 * size, "lines"),
                    "native", valueOnly = TRUE) > upper) &&
          (convertX(unit(TE, "native") - unit(0.5 * size, "lines"),
                    "native", valueOnly = TRUE) < lower))
        TElineCol <- col.inside
  }
  #
  if (!is.na(TE) && (TE >= min & TE <= max)) {
    if (!is.na(size) && size > 0 && !is.na(lower) && !is.na(upper)) {
      if (type == "square") {
        grid.rect(x = unit(TE, "native"),
                  width = unit(size, "snpc"),
                  height = unit(size, "snpc"),
                  gp = gpar(fill = col.square, col = col.square.lines,
                            lwd = lwd.square))
        #
        grid.lines(x = unit(c(TE, TE), "native"),
                   y = unit(c(0.4, 0.6), "npc"),
                   gp = gpar(col = TElineCol, lwd = lwd))
      }
    }
    else
      grid.lines(x = unit(c(TE, TE), "native"),
                 y = unit(c(0.4, 0.6), "npc"),
                 gp = gpar(col = TElineCol, lwd = lwd))
  }
  #
  if (!is.na(TE)) {
    #
    # Draw lines in colour "col.inside" if totally inside rect
    #
    if (!is.na(size)) {
      lineCol <- col
      #
      if (!is.na(lower) & !is.na(upper))
        if (size > 0 &&
            (convertX(unit(TE, "native") + unit(0.5 * size, "lines"),
                      "native", valueOnly = TRUE) > upper) &&
            (convertX(unit(TE, "native") - unit(0.5 * size, "lines"),
                      "native", valueOnly = TRUE) < lower))
          lineCol <- col.inside
      #
      # Draw arrow if exceed col range
      # convertX() used to convert between coordinate systems
      #
      if (!is.na(lower) && !is.na(upper) &&
          (lower >= min & upper <= max))
        grid.lines(x = unit(c(lower, upper), "native"), y = 0.5,
                   gp = gpar(col = lineCol, lwd = lwd))
      #
      if (!is.na(lower) && !is.na(upper) &&
          (lower < min & upper > max))
        grid.lines(x = unit(c(min, max), "native"), y = 0.5,
                   gp = gpar(col = lineCol, lwd = lwd))
      #
      if (!is.na(lower) && !is.na(upper) &&
          (lower < min & (upper <= max & upper > min)))
        grid.lines(x = unit(c(min, upper), "native"), y = 0.5,
                   gp = gpar(col = lineCol, lwd = lwd))
      #
      if (!is.na(lower) && !is.na(upper) &&
          ((lower >= min & lower < max) & upper > max))
        grid.lines(x = unit(c(lower, max), "native"), y = 0.5,
                   gp = gpar(col = lineCol, lwd = lwd))
      #
      if (!is.na(lower) && lower < min)
        grid.lines(x = unit(c(min - 0.00001, min), "native"), y = 0.5,
                   gp = gpar(col = lineCol, lwd = lwd, fill = lineCol),
                   arrow = arrow(ends = "first",
                                 length = unit(arrow.length, "inches"),
                                 type = arrow.type))
      if (!is.na(upper) && upper > max)
        grid.lines(x = unit(c(max, max + 0.00001), "native"), y = 0.5,
                   gp = gpar(col = lineCol, lwd = lwd, fill = lineCol),
                   arrow = arrow(ends = "last",
                                 length = unit(arrow.length, "inches"),
                                 type = arrow.type))
    }
  }
  #
  if (!is.na(TE) && (TE >= min & TE <= max)) {
    if (!is.na(size) && size > 0 && !is.na(lower) && !is.na(upper)) {
      if (type == "circle")
        grid.circle(x = unit(TE, "native"), y = unit(0.5, "npc"),
                    r = unit(size / 2, "snpc"),
                    gp = gpar(fill = col.circle, col = col.circle.lines))
      #
      else if (type == "squarediamond") {
        xmin <- convertX(unit(TE, "native") - unit(0.5 * size, "lines"),
                         "native", valueOnly = TRUE)
        xmax <- convertX(unit(TE, "native") + unit(0.5 * size, "lines"),
                         "native", valueOnly = TRUE)
        #
        grid.polygon(x = unit(c(xmin, TE, xmax, TE), "native"),
                     y = unit(0.5 + c(0, 0.5 * size, 0, -0.5 * size), "npc"),
                     gp = gpar(col = col.square.lines, fill = col.square,
                               lwd = lwd.square))
      }
    }
    else
      grid.lines(x = unit(c(TE, TE), "native"),
                 y = unit(c(0.4, 0.6), "npc"),
                 gp = gpar(col = TElineCol, lwd = lwd))
  }
  #
  invisible(NULL)
}


draw.forest <- function(x, column) {
  #
  # Function to plot results for individual studies and summaries
  #
  for (i in seq_len(length(x$rows))) {
    if (!is.na(x$rows[i])) {
      pushViewport(
        viewport(
          layout.pos.row = x$rows[i],
          layout.pos.col = column,
          xscale = x$range))
      #
      if (x$type[i] %in% c("square", "circle", "squarediamond"))
        draw.ci(x$eff[i], x$low[i], x$upp[i],
                x$sizes[i], x$range[1], x$range[2],
                x$lwd,
                x$col[i],
                x$col.square[i], x$col.square.lines[i],
                x$col.circle[i], x$col.circle.lines[i],
                x$col.inside[i],
                type = x$type[i],
                x$lwd.square,
                x$arrow.type, x$arrow.length)
      #
      else if (x$type[i] == "diamond")
        draw.ci.diamond(x$eff[i], x$low[i], x$upp[i],
                        x$sizes[i], x$range[1], x$range[2],
                        x$col.diamond[i], x$col.diamond.lines[i],
                        x$lwd.diamond)
      #
      else if (x$type[i] == "predict")
        draw.ci.predict(x$low[i], x$upp[i],
                        x$sizes[i], x$range[1], x$range[2],
                        x$col.diamond[i], x$col.diamond.lines[i])
      #
      popViewport()
    }
  }
  #
  invisible(NULL)
}


draw.lines <- function(x, column,
                       ref, TE.common, TE.random,
                       overall, common, random, prediction,
                       ymin.common, ymin.random, ymin.ref, ymax, ymax.ref,
                       lwd, lty.common, lty.random, col.common, col.random,
                       xmin, xmax,
                       cid.below.null, cid.above.null,
                       lty.cid, col.cid,
                       fill.cid.below.null, fill.cid.above.null,
                       fill,
                       col.line) {
  #
  if (xmin > xmax) {
    xmin <- x$range[2]
    xmax <- x$range[1]
  }
  else {
    xmin <- x$range[1]
    xmax <- x$range[2]
  }
  #
  pushViewport(viewport(layout.pos.col = column, xscale = x$range))
  #
  # Add background colour for confidence interval plot
  #
  if (!is.null(fill))
    grid.polygon(x = unit(c(xmin, xmax, xmax, xmin), "native"),
                 y = unit(c(ymin.ref, ymin.ref, ymax, ymax),
                          "lines"),
                 gp = gpar(col = fill, fill = fill))
  #
  # Add equivalence region
  #
  if (is.na(ref) & any(!is.na(cid.below.null)) & any(!is.na(cid.above.null)))
    ref.equi <- min(cid.below.null, na.rm = TRUE) +
      0.5 * (max(cid.above.null, na.rm = TRUE) - min(cid.below.null, na.rm = TRUE))
  else
    ref.equi <- ref
  #
  if (!is.na(ref.equi) && (xmin <= ref.equi & ref.equi <= xmax) &&
      any(!is.na(cid.below.null))) {
    #
    n.cid.below.null <- sum(!is.na(cid.below.null))
    n.fill.cid.below.null <- length(fill.cid.below.null)
    #
    if (n.cid.below.null < n.fill.cid.below.null) {
      firstline <- FALSE
      cid.below.null <- c(xmin, cid.below.null, ref.equi)
    }
    else {
      firstline <- TRUE
      cid.below.null <- c(cid.below.null, ref.equi)
    }
    #
    n.lo <- length(cid.below.null[-1])
    #
    for (i in seq_len(n.lo)) {
      if (!is.na(cid.below.null[i]) && !is.na(cid.below.null[i + 1]) &&
          ((xmin <= cid.below.null[i] & cid.below.null[i] <= cid.below.null[i + 1]) &
           (cid.below.null[i + 1] <= xmax))) {
        #
        grid.polygon(x = unit(c(cid.below.null[i], cid.below.null[i + 1],
                                cid.below.null[i + 1], cid.below.null[i]), "native"),
                     y = unit(c(ymin.ref, ymin.ref, ymax.ref, ymax.ref),
                              "lines"),
                     gp = gpar(lwd = lwd, col = "transparent",
                               fill = fill.cid.below.null[i]))
      }
    }
    #
    for (i in seq_len(n.lo)) {
      if (!(i == 1 & !firstline) &
          !is.na(cid.below.null[i]) && !is.na(cid.below.null[i + 1]) &&
          ((xmin <= cid.below.null[i] & cid.below.null[i] <= cid.below.null[i + 1]) &
           (cid.below.null[i + 1] <= xmax)))
        grid.lines(x = unit(cid.below.null[i], "native"),
                   y = unit(c(ymin.ref, ymax.ref), "lines"),
                   gp = gpar(lwd = lwd, col = col.cid, lty = lty.cid))
    }
  }
  #
  if (!is.na(ref.equi) && (xmin <= ref.equi & ref.equi <= xmax) &&
      any(!is.na(cid.above.null))) {
    #
    n.cid.above.null <- sum(!is.na(cid.above.null))
    n.fill.cid.above.null <- length(fill.cid.above.null)
    #
    if (n.cid.above.null < n.fill.cid.above.null) {
      lastline <- FALSE
      cid.above.null <- c(ref.equi, cid.above.null, xmax)
    }
    else {
      lastline <- TRUE
      cid.above.null <- c(ref.equi, cid.above.null)
    }
    #
    n.up <- length(cid.above.null[-1])
    #
    for (i in seq_len(n.up)) {
      if (!is.na(cid.above.null[i]) && !is.na(cid.above.null[i + 1]) &&
          ((xmin <= cid.above.null[i] & cid.above.null[i] <= cid.above.null[i + 1]) &
           (cid.above.null[i + 1] <= xmax))) {
        #
        grid.polygon(x = unit(c(cid.above.null[i], cid.above.null[i + 1],
                                cid.above.null[i + 1], cid.above.null[i]), "native"),
                     y = unit(c(ymin.ref, ymin.ref, ymax.ref, ymax.ref),
                              "lines"),
                     gp = gpar(lwd = lwd, col = "transparent",
                               fill = fill.cid.above.null[i]))
      }
    }
    #
    for (i in seq_len(n.up)) {
      if ((i != n.up | (i == n.up & lastline)) &
          !is.na(cid.above.null[i + 1]) &&
          (xmin <= cid.above.null[i + 1] & cid.above.null[i + 1] <= xmax))
          grid.lines(x = unit(cid.above.null[i + 1], "native"),
                     y = unit(c(ymin.ref, ymax.ref), "lines"),
                     gp = gpar(lwd = lwd, col = col.cid, lty = lty.cid))
    }
  }
  #
  # Reference line:
  #
  if (!is.na(ref) && (xmin <= ref & ref <= xmax))
    grid.lines(x = unit(ref, "native"),
               y = unit(c(ymin.ref, ymax), "lines"),
               gp = gpar(lwd = lwd, col = col.line))
  #
  # Line for common effect estimate(s):
  #
  if (common & overall)
    for (i in seq_along(TE.common))
      if (!is.na(TE.common[i]))
        if (xmin <= TE.common[i] & TE.common[i] <= xmax)
          if (!is.null(lty.common))
            grid.lines(x = unit(TE.common[i], "native"),
                       y = unit(c(ymin.common + length(TE.common) - i, ymax),
                                "lines"),
                       gp = gpar(lty = lty.common, lwd = lwd, col = col.common))
  #
  # Line for random effects estimate(s):
  #
  if (random & overall)
    for (i in seq_along(TE.random))
      if (!is.na(TE.random[i]))
        if (xmin <= TE.random[i] & TE.random[i] <= xmax)
          if (!is.null(lty.random) & !is.na(TE.random[i]))
            grid.lines(x = unit(TE.random[i], "native"),
                       y = unit(c(ymin.random + length(TE.random) - i, ymax),
                                "lines"),
                       gp = gpar(lty = lty.random,
                                 lwd = lwd, col = col.random))
  #
  popViewport()
  #
  invisible(NULL)
}


formatcol <- function(x, y, rows, just = "right", settings,
                      fontfamily,
                      n.com, n.ran, n.prd,
                      rob = FALSE) {
  #
  if (just == "left")
    xpos <- 0
  if (just == "center")
    xpos <- 0.5
  if (just == "right")
    xpos <- 1
  #
  res <- list(labels = 
                lapply(c(x, as.list(y)),
                       textGrob, x = xpos, just = just,
                       gp = gpar(
                         fontsize = settings$fs.study,
                         fontface = settings$ff.study,
                         fontfamily = fontfamily)
                       ),
              rows = rows,
              rob = rob)
  #
  # Study label:
  #
  res$labels[[1]] <- textGrob(x,
                              x = xpos, just = just,
                              gp = gpar(
                                fontsize = settings$fs.heading,
                                fontface = settings$ff.heading,
                                fontfamily = fontfamily)
                              )
  #
  # Common effect estimate:
  #
  strt <- j <- 1
  for (i in seq_len(n.com)) {
    res$labels[[strt + i]] <- textGrob(y[strt - 1 + i],
                                       x = xpos, just = just,
                                       gp = gpar(
                                         fontsize = settings$fs.common,
                                         fontface = settings$ff.common,
                                         fontfamily = fontfamily)
                                       ) 
    j <- j + 1
  }
  #
  # Random effects estimate:
  #
  strt <- j
  for (i in seq_len(n.ran)) {
    res$labels[[strt + i]] <- textGrob(y[strt - 1 + i],
                                       x = xpos, just = just,
                                       gp = gpar(
                                         fontsize = settings$fs.random,
                                         fontface = settings$ff.random,
                                         fontfamily = fontfamily)
                                       )
    j <- j + 1
  }
  #
  # Prediction interval:
  #
  strt <- j
  for (i in seq_len(n.prd)) {
    res$labels[[strt + i]] <- textGrob(y[strt - 1 + i],
                                       x = xpos, just = just,
                                       gp = gpar(
                                         fontsize = settings$fs.predict,
                                         fontface = settings$ff.predict,
                                         fontfamily = fontfamily)
                                       )
    j <- j + 1
  }
  #
  if (settings$by) {
    n.by <- settings$n.by
    strt <- j
    #
    # Common effect estimates:
    #
    for (i in seq_len(n.by * n.com)) {
      res$labels[[strt + i]] <-
        textGrob(y[strt - 1 + i],
                 x = xpos, just = just,
                 gp = 
                   gpar(
                     fontsize = settings$fs.common,
                     fontface = settings$ff.common,
                     fontfamily = fontfamily,
                     col = settings$col.subgroup)
                 )
      j <- j + 1
    }
    #
    # Random effects estimates:
    #
    strt <- j
    for (i in seq_len(n.by * n.ran)) {
      res$labels[[strt + i]] <-
        textGrob(y[strt - 1 + i],
                 x = xpos, just = just,
                 gp = 
                   gpar(
                     fontsize = settings$fs.random,
                     fontface = settings$ff.random,
                     fontfamily = fontfamily,
                     col = settings$col.subgroup)
                 )
      j <- j + 1
    }
    #
    # Prediction interval:
    #
    strt <- j
    for (i in seq_len(n.by * n.prd)) {
      res$labels[[strt + i]] <-
        textGrob(y[strt - 1 + i],
                 x = xpos, just = just,
                 gp = 
                   gpar(
                     fontsize = settings$fs.predict,
                     fontface = settings$ff.predict,
                     fontfamily = fontfamily,
                     col = settings$col.subgroup)
                 )
      j <- j + 1
    }
  }
  #
  res
}


removeNULL <- function(x, names, varname) {
  if (is.null(x[[varname]]))
    res <- names[names != varname]
  else
    res <- names
  #
  res
}


tg <- function(x, xpos, just, fs, ff, fontfamily, col) {
  if (missing(col))
    res <- textGrob(x,
                    x = xpos, just = just,
                    gp = gpar(fontsize = fs, fontface = ff,
                              fontfamily = fontfamily))
  else
    res <- textGrob(x,
                    x = xpos, just = just,
                    gp = gpar(fontsize = fs, fontface = ff,
                              fontfamily = fontfamily,
                              col = col))
  #
  res
}


tgl <- function(x, xpos, just, fs, ff, fontfamily, rows = 1, col) {
  #
  if (missing(col))
    res <- list(labels = list(tg(x, xpos, just, fs, ff, fontfamily)),
                rows = rows)
  else
    res <- list(labels = list(tg(x, xpos, just, fs, ff, fontfamily, col)),
                rows = rows)
  #
  res
}


twolines <- function(x, xname = deparse(substitute(x)), arg = FALSE) {
  newline <- FALSE
  bottom <- longer <- x
  top <- NULL
  #
  if (!is.null(bottom)) {
    if (grepl("\n", bottom)) {
      wsplit <- unlist(strsplit(bottom, "\n"))
      if (length(wsplit) == 1) {
        if (substring(bottom, 1, 1) == "\n") {
          top <- ""
          bottom <- wsplit[1]
          longer <- bottom
          newline <- TRUE
        }
        else if (substring(bottom, nchar(bottom), nchar(bottom)) == "\n") {
          top <- wsplit[1]
          bottom <- ""
          longer <- top
          newline <- TRUE
        }
      }
      else if (length(wsplit) != 2) {
        if (arg)
          stop("Maximum of two lines for argument '", xname, "'.",
               call. = FALSE)
        else
          stop("Maximum of two lines for label of column '", xname, "'.",
               call. = FALSE)
      }
      else {
        top <- wsplit[1]
        bottom <- wsplit[2]
        longer <- ifelse(nchar(top) > nchar(bottom), top, bottom)
        newline <- TRUE
      }
    }
  }
  #
  list(newline = newline, bottom = bottom, top = top, longer = longer)
}


wcalc <- function(x)
  max(unit(rep(1, length(x)), "grobwidth", x))


collapsemat <- function(x) {
  if (is.list(x)) {
    for (i in rev(seq_len(length(x)))) {
      if (is.null(x[[i]]))
        x[[i]] <- NULL
    }
  }
  #
  if (is.list(x) & length(x) == 1)
    x <- x[[1]]
  #
  if (is.matrix(x)) {
    res <- as.vector(t(x))
    names(res) <- rep(rownames(x), rep(ncol(x), nrow(x)))
  }
  else
    res <- x
  #
  res
}


ordermat <- function(x, levs) {
  o <- order(factor(names(x), levels = levs))
  x[o]
}


selmat <- function(x, levs) {
  o <- order(factor(names(x), levels = levs))
  x[o]
}


notallNA <- function(x)
  any(!is.na(x))


repl <- function(x, n1, n2)
  rep(x, rep(n1, n2))


gh <- function(type.gr, rows.gr,
               #
               n.stud,
               lower.common, lower.random, lower.predict,
               subgroup, subgroup.levels,
               lower.common.w, lower.random.w, lower.predict.w,
               #
               common, random, overall,
               prediction, overall.hetstat,
               study.results,
               #
               spacing,
               #
               xlab, xlab.add, label.right, label.left, bottom.lr,
               #
               prediction.subgroup, subgroup.hetstat,
               test.overall.common, test.overall.random,
               test.subgroup.common, test.subgroup.random,
               #
               text.addline1, text.addline2,
               text.details, text.rob,
               #
               addrow, addrow.overall,
               addrow.subgroups, addrows.below.overall,
               #
               cols, labs,
               text.w.common, text.w.random) {
  
  
  #
  # (1) Determine height per row
  #
  if (grepl("bmp$", tolower(type.gr)) ||
      grepl("jpg$", tolower(type.gr)) ||
      grepl("jpeg$", tolower(type.gr)) ||
      grepl("png$", tolower(type.gr)) ||
      grepl("tif$", tolower(type.gr)) ||
      grepl("tiff$", tolower(type.gr)))
    height_per_row <- 480 / 33
  else
    height_per_row <-  7 / 35
  #
  # (2) Column labels
  #
  labs <- unlist(labs)
  if (any(grepl("col.w.common", cols)))
    labs <- c(labs, text.w.common)
  if (any(grepl("col.w.random", cols)))
    labs <- c(labs, text.w.random)
  #
  rows_column_labels <- 1 + 1L * any(grepl("\n", unlist(labs))) + 1L * addrow
  #
  # (2) Study results
  #
  rows_studies <-
    if (!study.results)
      0
    else
      n.stud
  #
  # (3) Meta-analysis results
  #
  if (overall)
    rows_overall <-
      1L * addrow.overall +
      1L * common * length(lower.common) +
      1L * random * length(lower.random) +
      1L * prediction * length(lower.predict) +
      1L * ((any(common) + any(random) + any(prediction)) == 1)
  else
    rows_overall <- 1L * addrow.overall
  #
  # (4) Text below meta-analysis results
  #
  n.details <- sum(text.details != "")
  n.rob <- sum(text.rob != "")
  rows_below_overall_labels <- addrows.below.overall + overall.hetstat +
    test.overall.common + test.overall.random +
    test.subgroup.common + test.subgroup.random +
    1L * (text.addline1 != "") +
    1L * (text.addline2 != "") +
    n.details + 1L * (n.details > 0) +
    n.rob + 1L * (n.rob > 0) +
    1L * (n.details > 0 | n.rob > 0) +
    2L * (n.details == 0 & n.rob == 0)
  # + 0.25
  #
  # (5) Information below confidence interval plot
  #
  rows_xlab <-
    if (xlab == "")
      0
    else
      2
  #
  rows_xlab.add <-
    if (xlab.add == "")
      0
    else
      2
  #
  rows_xlab <- rows_xlab + rows_xlab.add
  #
  if (!bottom.lr) {
    rows_label.left <- 0
    rows_label.right <- 0
  }
  else {
    rows_label.left <-
      if (is.null(label.left) || label.left == "")
        0
      else if (!grepl("\n", label.left))
        2
      else
        4
    #
    rows_label.right <-
      if (is.null(label.right) || label.right == "")
        0
      else if (!grepl("\n", label.right))
        2
      else
        4
  }
  #
  rows_label <- max(c(rows_label.left, rows_label.right))
  #
  rows_below_forest <- 2 + rows_xlab + rows_label
  #
  # (6) Subgroup results
  #
  if (is.null(subgroup))
    rows_subgroups <- 0
  else {
    n.subgr <- length(subgroup.levels)
    #
    rows_subgroups_common <- common * n.subgr
    rows_subgroups_random <- random * n.subgr
    #
    rows_subgroups_predict <- 
      if (is.vector(prediction.subgroup))
        sum(prediction.subgroup)
      else
        prediction.subgroup * n.subgr
    #
    rows_subgroups_hetstat <- 
      if (length(subgroup.hetstat) > 1)
        sum(subgroup.hetstat)
      else
        subgroup.hetstat * n.subgr
    #
    if (is.matrix(lower.common.w))
      rows_subgroups_common <- rows_subgroups_common * nrow(lower.common.w)
    #
    if (is.matrix(lower.random.w))
      rows_subgroups_random <- rows_subgroups_random * nrow(lower.random.w)
    #
    if (is.matrix(lower.predict.w))
      rows_subgroups_predict <- rows_subgroups_predict * nrow(lower.predict.w)
    #
    rows_subgroups <-
      n.subgr + # Labels
      addrow.subgroups * (n.subgr - 1) +
      rows_subgroups_common + rows_subgroups_random + 
      rows_subgroups_predict + rows_subgroups_hetstat
  }
  #
  # (7) Determine total height of graphics device
  #
  total_rows <-
    rows_column_labels +
    rows_studies +
    rows_overall +
    max(rows_below_overall_labels, rows_below_forest) +
    rows_subgroups +
    rows.gr +
    1
  #
  total_height <- height_per_row * spacing * total_rows
  
  res <- data.frame(total_height, total_rows, height_per_row, spacing)
  #
  res
}


show_subgroup_results <- function(x, n, lower, upper) {
  if (length(x) == 1) {
    if (is.matrix(lower))
      return(x &
             apply(lower, 1, notallNA) &
             apply(upper, 1, notallNA))
    else
      return(rep(x & notallNA(lower) & notallNA(upper), n))
  }
  else {
    chklength(x, n,
              text = paste0("Length of argument '",
                            deparse(substitute(x)),
                            "' must be equal to 1 or number of subgroups."))
    return(x)
  }
}

newCol <- function(varname, label,
                   rob, data1, data2, datap,
                   n.com, n.ran, n.prd,
                   notavail, lab.NA, big.mark,
                   zero.pval, JAMA.pval, scientific.pval,
                   digits, digits.pval, digits.tau2, digits.tau, digits.I2,
                   sel.prd,
                   n.com.w = 0, n.ran.w = 0, n.prd.w = 0, n.stat.w = 0) {
  
  by <- sum(c(n.com.w, n.ran.w, n.prd.w, n.stat.w) > 0)
  #
  n.subgroup <- n.com.w + n.ran.w + n.prd.w + n.stat.w
  #
  if (length(rob[[varname]]) != 0)
    fvar <- rob[[varname]]
  else if (length(data1[[varname]]) != 0)
    fvar <- data1[[varname]]
  else if (length(data2[[varname]]) != 0)
    fvar <- data2[[varname]]
  else
    stop("Variable '", varname,
         "' not available in meta-analysis object.",
         call. = FALSE)
  #
  if (isCol(datap, varname)) {
    if (nrow(datap) == 1) {
      fvar <- c(rep(datap[[varname]], n.com),
                rep(datap[[varname]], n.ran),
                rep(NA, n.prd),
                if (by) rep(NA, n.subgroup) else NULL,
                fvar)
    }
    else if (nrow(datap) == n.com + n.ran)
      fvar <- c(datap[[varname]],
                rep(NA, n.prd),
                if (by) rep(NA, n.subgroup) else NULL,
                fvar)
    else if (nrow(datap) == n.com + n.ran + n.com.w + n.ran.w)
      fvar <- c(datap[[varname]][seq_len(n.com + n.ran)],
                rep(NA, n.prd),
                datap[[varname]][n.com + n.ran + seq_len(n.com.w + n.ran.w)],
                rep(NA, n.prd.w + n.stat.w),
                fvar)
    else
      stop("Wrong number of row in data set 'data.pooled' ",
           "(must be 1",
           if (by)", " else " or ", n.com + n.ran,,
           if (by) " or ", n.com.w + n.ran.w, ").",
           call. = FALSE)
  }
  #
  if (!is.character(fvar)) {
    if (is.factor(fvar))
      fvar <- as.character(fvar)
    else if (notavail & all(is_wholenumber(fvar), na.rm = TRUE))
      fvar <-
        formatN(fvar, digits = 0, text.NA = lab.NA, big.mark = big.mark)
    else if (is.numeric(fvar)) {
      if (varname == "pval")
        fvar <- formatPT(fvar, digits = digits.pval,
                          big.mark = big.mark,
                          lab = FALSE, labval = "",
                          zero = zero.pval, JAMA = JAMA.pval,
                          scientific = scientific.pval,
                          lab.NA = lab.NA)
      else if (varname == "tau2")
        fvar <- formatPT(fvar, digits = digits.tau2,
                          big.mark = big.mark,
                          lab = FALSE, labval = "",
                          lab.NA = lab.NA)
      else if (varname == "tau")
        fvar <- formatPT(fvar, digits = digits.tau,
                          big.mark = big.mark,
                          lab = FALSE, labval = "",
                          lab.NA = lab.NA)
      else if (varname == "I2") {
        sel.r <- !is.na(fvar)
        fvar[sel.r] <-
          paste0(formatN(100 * fvar[sel.r], digits.I2), "%")
        fvar[!sel.r] <- lab.NA
      }
      else
        fvar <-
          formatN(fvar, digits = digits, text.NA = lab.NA, big.mark = big.mark)
    }
  }
  #
  fvar <- ifelse(is.na(fvar), "", fvar)
  #
  # Print nothing for lines with prediction interval
  #
  if (isCol(datap, varname))
    fvar[sel.prd] <- ""
  else
    fvar <- c(rep("", n.com), rep("", n.ran), rep("", n.prd),
              if (by) rep("", n.subgroup),
              fvar)
  #
  # Print nothing for lines for subgroups
  #
  if (by) {
    is_NA <- fvar == lab.NA
    is_w <- rep(TRUE, length(fvar))
    is_w[seq_len(n.com + n.ran + n.prd)] <- FALSE
    #
    fvar[is_NA & is_w] <- ""
  }
  #
  # Check for "\n" in label of new column
  #
  clines <- twolines(label, varname)
  #
  if (clines$newline) {
    label <- clines$bottom
    longer <- clines$longer
  }
  else
    label <- longer <- label
  #
  res <- list(format_var = fvar, colname = paste0("col.", varname),
              label = label, longer = longer,
              rob = length(rob[[varname]]) != 0)
  #
  res
}

warn_only_subgroup <- function(x) {
  x.name <- gsub("missing.", "", deparse(substitute(x)))
  #
  if (!x)
    warning("Argument '", x.name, "' only considered for ",
            "meta-analysis with subgroups.",
            call. = FALSE)
  #
  invisible(NULL)
}

@echo off
setlocal EnableDelayedExpansion
setlocal enableextensions

set baseDir=%~dp0

if exist "%ProgramFiles(x86)%/Microsoft Visual Studio/Installer/" (
	for /f "usebackq delims=#" %%a in (`"%ProgramFiles(x86)%/Microsoft Visual Studio/Installer/vswhere" -version [16.0^,17.0] -latest -property installationPath`) do (
		set "visualStudioTools=%%a"
	)

	set "vcvarsallPath=/VC/Auxiliary/Build/vcvars64.bat"
)

if exist "%visualStudioTools%%vcvarsallPath%" (
    echo "Found %cmakeGenerator% tools: %visualStudioTools%%vcvarsallPath%"
    call "%visualStudioTools%%vcvarsallPath%" 
) else (
    echo,
    echo **************************************************
    echo,
    if exist "%visualStudioTools%" (
        echo ERROR: Could not find C/C++ support in "%visualStudioTools%"
        echo        Please, ensure that you have selected the "Desktop Development with C++" module in your Visual Studio installation.
    ) else (
        echo ERROR: Could not find Visual Studio 2019.
		echo You can try building OpenViBE manually with other version of Visual Studio.
    )
    echo,
    echo **************************************************
    echo,
    pause
    exit 1
)

mkdir %baseDir%\build\
cd %baseDir%\build\

cmake %baseDir% -G Ninja
if not "!ERRORLEVEL!" == "0" goto terminate_error

ninja -j 4
if not "!ERRORLEVEL!" == "0" goto terminate_error


:terminate_success
exit 0

:terminate_error
echo.
echo An error occured during building process !
echo.
exit /b 1

@echo off

set PATH=C:\Program Files (x86)\Microsoft SDKs\Windows\v7.0A\bin;%PATH%
set "SCRIPT_PATH=%~dp0"
set platformTarget=x64
set visualStudioTools=
set configurationType=
set cmakeGenerator=
set vcvarsallPath=

:parameter_parse
if /i "%1" == "--platform-target" (
	set PlatformTarget=%2
	SHIFT
	SHIFT

	Goto parameter_parse
) else if not "%1" == "" (
	echo unrecognized option [%1]
	Goto terminate_error
)
REM ########################################################################################################################

if %platformTarget% NEQ x64 (
    if %platformTarget% NEQ x86 (
        echo "Error: Unsupported platform-target %platformTarget%"
        pause
        exit 1
    )
)

rem Check for MSVC 2017 to 2019
if exist "%ProgramFiles(x86)%/Microsoft Visual Studio/Installer/" (
    for /f "usebackq delims=#" %%a in (`"%ProgramFiles(x86)%/Microsoft Visual Studio/Installer/vswhere" -version [15.0^,17.0] -latest -property catalog_productLineVersion`) do (
        if ["%%a"] == ["2019"] (
            set cmakeGenerator="Visual Studio 16 2019"
        ) else if ["%%a"] == ["2017"] (
            set cmakeGenerator="Visual Studio 15 2017"
        )
        for /f "usebackq delims=#" %%a in (`"%ProgramFiles(x86)%/Microsoft Visual Studio/Installer/vswhere" -version [15.0^,17.0] -latest -property installationPath`) do (
            set "visualStudioTools=%%a"
        )

        set "vcvarsallPath=/VC/Auxiliary/Build/vcvarsall.bat"
        set configurationType=%platformTarget%
        if %platformTarget% == x64 (
            set cmakeGenerator= !cmakeGenerator! -A %platformTarget%
        ) else (
            set cmakeGenerator= !cmakeGenerator! -A Win32
        )
    )
)

rem If MSVC not found above
if ["%visualStudioTools%"] == [""] (
    rem Use Visual Studio 2013 (different tools access method).
    set "visualStudioTools=%VS120COMNTOOLS%"
    set "vcvarsallPath=../../VC/vcvarsall.bat"
    set configurationType=%platformTarget%
    if %platformTarget% == x64 (
        set cmakeGenerator="Visual Studio 12 2013" -A %platformTarget%
        if not exist "%VS120COMNTOOLS%../../VC/bin/amd64" (
            set configurationType=x86_amd64
        )
    ) else (
        set cmakeGenerator="Visual Studio 12 2013" -A Win32
    )
)

if exist "%visualStudioTools%%vcvarsallPath%" (
    echo "Found %cmakeGenerator% tools: %visualStudioTools%%vcvarsallPath% %configurationType%"
    call "%visualStudioTools%%vcvarsallPath%" %configurationType%
) else (
    echo,
    echo **************************************************
    echo,
    if exist "%visualStudioTools%" (
        echo ERROR: Could not find C/C++ support in "%visualStudioTools%"
        echo        Please, ensure that you have selected the "Desktop Development with C++" module in your Visual Studio installation.
    ) else (
        echo ERROR: No supported version of Visual Studio were found.
        echo Supported versions are:
        echo  - Visual Studio 15 2017
        echo  - Visual Studio 12 2013
    )
    echo,
    echo **************************************************
    echo,
    pause
    exit 1
)

@echo off

mkdir build
cd build

set BUILD_CONFIG=Release

cmake ..  -G "Ninja" ^
    -Wno-dev ^
    -DCMAKE_INSTALL_PREFIX=%LIBRARY_PREFIX% ^
    -DCMAKE_PREFIX_PATH=%LIBRARY_PREFIX% ^
    -DCMAKE_INSTALL_RPATH:STRING=%LIBRARY_LIB% ^
    -DCMAKE_INSTALL_NAME_DIR=%LIBRARY_LIB% ^
    -DCMAKE_BUILD_TYPE=Release

if errorlevel 1 exit 1

cmake --build . --target install
if errorlevel 1 exit 1

#!/bin/bash

if [[ -d build ]]; then
    rm -rf build
fi
mkdir build
cd build

if [[ "$c_compiler" == "gcc" ]]; then
  export PATH="${PATH}:${BUILD_PREFIX}/${HOST}/sysroot/usr/lib:${BUILD_PREFIX}/${HOST}/sysroot/usr/include"
fi

cmake .. \
      -DCMAKE_INSTALL_PREFIX="${PREFIX}" \
      -DCMAKE_PREFIX_PATH="${PREFIX}" \
      -DCMAKE_BUILD_TYPE=Release

make -j${CPU_COUNT}
make install

---
#title: "Workflow in R package meta"
#author: "Guido Schwarzer"
output:
  rmarkdown::pdf_document:
   number_sections: true
  rmarkdown::html_vignette:
   toc: false
   number_sections: true
vignette: >
  %\VignetteIndexEntry{meta: General Package for Meta-Analysis - Workflow}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = TRUE,
  warning = FALSE
  )
options(knitr.kable.NA = ".")
```

```{r, echo = FALSE, out.height = "99%"}
knitr::include_graphics("meta-workflow-diagram.pdf")
```

---
title: "How to perform a meta-analysis with R: a practical tutorial"
author: "Sara Balduzzi, Gerta Rücker, Guido Schwarzer"
output:
  rmarkdown::pdf_document:
   number_sections: false
  rmarkdown::html_vignette:
   toc: false
   number_sections: false
bibliography: references.bib 
vignette: >
  %\VignetteIndexEntry{meta: How to perform a meta-analysis with R: a practical tutorial}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  message = TRUE,
  warning = FALSE
  )
options(knitr.kable.NA = ".")

load("mmiss_limit.rda")
```


This vignette provides up-to-date commands for the analyses in "How to
perform a meta-analysis with R: a practical tutorial", *Evid Based
Ment Health* [@Bald:Ruec:Schw:how:2019].


## Install R packages

```{r, eval = FALSE}
install.packages(c("meta", "metasens"))
```

## Make R packages available

```{r}
library(meta)
```

```{r, eval = FALSE}
library(metasens)
```

Note, a similar message would be printed for R package
**metasens**. However, this vignette does not actually load
**metasens** as it might not be installed in addition to **meta**.


## Default settings for R session

Print results with two significant digits and use Paule-Mandel
estimator for between-study variance.

```{r}
settings.meta(digits = 2, method.tau = "PM")
```

Note, in the publication, argument 'method.tau' was used in R function
*metabin()*. Here, we set the Paule-Mandel method as the default for
any meta-analysis conducted in the current R session.

## Import the dataset

```{r}
joy = read.csv("Joy2006.txt")
# Add new variable: miss
joy$miss = ifelse((joy$drop.h + joy$drop.p) == 0, 
  "Without missing data", "With missing data")
head(joy)
str(joy)
```

## Section 'Fixed effect and random effects meta-analysis'

```{r}
m.publ = metabin(resp.h, resp.h + fail.h, resp.p, resp.p + fail.p,
  data = joy, studlab = paste0(author, " (", year, ")"),
  label.e = "Haloperidol", label.c = "Placebo",
  label.left = "Favours placebo", label.right = "Favours haloperidol")
```

Print results of meta-analysis (Figure 1).

```{r}
summary(m.publ)
```

Same printout (result not shown)

```{r, eval = FALSE}
print(summary(m.publ))
```

Create Figure 2 (file 'figure2.pdf').

```{r}
forest(m.publ, sortvar = year, prediction = TRUE,
  digits.pval.Q = 3,
  file = "figure2.pdf", width = 10)
```

```{r, echo = FALSE, out.width = "95%"}
knitr::include_graphics("figure2.pdf")
```

## Section 'Assessing the impact of missing outcome data'

### Subgroup analysis of studies with and without missing data

```{r}
m.publ.sub = update(m.publ, subgroup = miss, print.subgroup.name = FALSE)
m.publ.sub
```

Create Figure 3 (file 'figure3.pdf').

```{r}
forest(m.publ.sub, sortvar = year,
  xlim = c(0.1, 100), at = c(0.1, 0.3, 1, 3, 10, 30, 100),
  digits.pval.Q = 3, test.subgroup.common = FALSE,
  label.test.subgroup.random = "Test for subgroup differences:",
  file = "figure3.pdf", width = 10)
```

```{r, echo = FALSE, out.width = "95%"}
knitr::include_graphics("figure3.pdf")
```

### Use imputation methods

```{r, eval = FALSE}
# Impute as no events (ICA-0) - default
mmiss.0 = metamiss(m.publ, drop.h, drop.p)
# Impute as events (ICA-1)
mmiss.1 = metamiss(m.publ, drop.h, drop.p, method = "1")
# Observed risk in control group (ICA-pc)
mmiss.pc = metamiss(m.publ, drop.h, drop.p, method = "pc")
# Observed risk in experimental group (ICA-pe)
mmiss.pe = metamiss(m.publ, drop.h, drop.p, method = "pe")
# Observed group-specific risks (ICA-p)
mmiss.p = metamiss(m.publ, drop.h, drop.p, method = "p")
# Best-case scenario (ICA-b)
mmiss.b = metamiss(m.publ, drop.h, drop.p, method = "b", small.values = "bad")
# Worst-case scenario (ICA-w)
mmiss.w = metamiss(m.publ, drop.h, drop.p, method = "w", small.values = "bad")
# Gamble-Hollis method
mmiss.gh = metamiss(m.publ, drop.h, drop.p, method = "GH")
# IMOR.e = 2 and IMOR.c = 2 (same as available case analysis)
mmiss.imor2 = metamiss(m.publ, drop.h, drop.p, method = "IMOR", IMOR.e = 2)
# IMOR.e = 0.5 and IMOR.c = 0.5
mmiss.imor0.5 = metamiss(m.publ, drop.h, drop.p, method = "IMOR", IMOR.e = 0.5)
```

Summarise results using R function *metabind()*.

```{r}
meths = c("Available case analysis (ACA)",
  "Impute no events (ICA-0)", "Impute events (ICA-1)",
  "Observed risk in control group (ICA-pc)",
  "Observed risk in experimental group (ICA-pe)",
  "Observed group-specific risks (ICA-p)",
  "Best-case scenario (ICA-b)", "Worst-case scenario (ICA-w)",
  "Gamble-Hollis analysis",
  "IMOR.e = 2, IMOR.c = 2", "IMOR.e = 0.5, IMOR.c = 0.5")
# Use inverse-variance method for pooling (which is used for
# imputation methods)
m.publ.iv = update(m.publ, method = "Inverse")
# Combine results (random effects)
mbr = metabind(m.publ.iv,
  mmiss.0, mmiss.1,
  mmiss.pc, mmiss.pe, mmiss.p,
  mmiss.b, mmiss.w, mmiss.gh,
  mmiss.imor2, mmiss.imor0.5,
  name = meths, pooled = "random")
```

Create Figure 4 (file 'figure4.pdf').

```{r}
forest(mbr, xlim = c(0.5, 4),
  leftcols = c("studlab", "I2", "tau2", "Q", "pval.Q"),
  leftlab = c("Meta-Analysis Method", "I2", "Tau2", "Q", "P-value"),
  type = "diamond",
  digits.addcols = c(4, 2, 2, 2), just.addcols = "right",
  file = "figure4.pdf", width = 10)
```

```{r, echo = FALSE, out.width = "95%"}
knitr::include_graphics("figure4.pdf")
```


## Section 'Assessing and accounting for small-study effects'

### Funnel plot

```{r, eval = FALSE}
funnel(m.publ)
```

### Harbord's score test for funnel plot asymmetry

```{r}
metabias(m.publ, method.bias = "score")
```


### Trim-and-fill method

```{r}
tf.publ = trimfill(m.publ)
tf.publ
```

```{r}
summary(tf.publ)
```

```{r, eval = FALSE}
funnel(tf.publ)
```


### Limit meta-analysis

```{r, eval = FALSE}
l1.publ = limitmeta(m.publ)
```

Note, the printout for the limit meta-analysis is not shown in this
vignette as the installation of R package **metasens** is optional.

```{r, eval = FALSE}
l1.publ
```

Create Figure 5 (file 'figure5.pdf').

```{r, eval = FALSE}
pdf("figure5.pdf", width = 10, height = 10)
#
par(mfrow = c(2, 2), pty = "s",
    oma = c(0, 0, 0, 0), mar = c(4.1, 3.1, 2.1, 1.1))
#
funnel(m.publ, xlim = c(0.05, 50), axes = FALSE)
axis(1, at = c(0.1, 0.2, 0.5, 1, 2, 5, 10, 50))
axis(2, at = c(0, 0.5, 1, 1.5))
box()
title(main = "Panel A: Funnel plot", adj = 0)
#
funnel(m.publ, xlim = c(0.05, 50), axes = FALSE,
       contour.levels = c(0.9, 0.95, 0.99),
       col.contour = c("darkgray", "gray", "lightgray"))
legend("topright",
       c("p < 1%", "1% < p < 5%", "5% < p < 10%", "p > 10%"),
       fill = c("lightgray", "gray", "darkgray", "white"),
       border = "white", bg = "white")
axis(1, at = c(0.1, 0.2, 0.5, 1, 2, 5, 10, 50))
axis(2, at = c(0, 0.5, 1, 1.5))
box()
title(main = "Panel B: Contour-enhanced funnel plot", adj = 0)
#
funnel(tf.publ, xlim = c(0.05, 50), axes = FALSE)
axis(1, at = c(0.1, 0.2, 0.5, 1, 2, 5, 10, 50))
axis(2, at = c(0, 0.5, 1, 1.5))
box()
title(main = "Panel C: Trim-and-fill method", adj = 0)
#
funnel(l1.publ, xlim = c(0.05, 50), axes = FALSE,
       col.line = 8, lwd.line = 3)
axis(1, at = c(0.1, 0.2, 0.5, 1, 2, 5, 10, 50))
axis(2, at = c(0, 0.5, 1, 1.5))
box()
title(main = "Panel D: Limit meta-analysis", adj = 0)
#
dev.off()
```

```{r, echo = FALSE, out.width = "95%"}
knitr::include_graphics("figure5.pdf")
```

# References

# Formating EBML streams {#formating-ebml}

\page FormatingEBML Formating EBML streams

## Introduction

This page tries to teach how a user of this library should use the writing functions in order to build and format an EBML stream correctly. For those who don't know what EBML is, it is basically a binary XML created and used by the matroska developpers. The libmatroska is based on the libebml implementation of these guys. Fore more details, please read  the [What Is EBML](_what_is_e_b_m_l.html) page first and eventually visit the  EBML web page at <http://ebml.sourceforge.net/>

## Concepts

The idea of this formating library is to transform the data provided by the user application into EBML buffers. Those EBML buffers are then given to a user callback object which know what to do with the computed data (maybe write it to a file, send it on a socket, log it in a console, whatever...)

The library is divided into three main components :

- The writer itself that does the formating stuff
- An implementation of the callback object (the implementation is left to the user application developper)
- An optionnal helper object that knows more on the content of the EBML stream.

Here comes the organisation of the different modules and how data go from one to another. Note that the user application and the user callback object may share some information so the callback object communicates with the application itself.

![Concept](images/ebml_formating_concept.png)

Here comes the UML class diagram, presenting the main classes involved in the presented behavior.

![Class Diagram](images/ebml_formating_class.png)

See EBML::IWriter, EBML::IWriterCallback and EBML::IWriterHelper for more details on each of these classes.

## Sample code

In this section, a sample of user application code is presented that opens several child nodes and dumps the created stream into a file for later use. This file can be parsed using the sample code of the page named [Parsing EBML Streams](_parsing_e_b_m_l.html)

The callback object implementation looks something like this :

```cpp
class CWriterCallback : public EBML::IWriterCallback
{
public:
  CWriterCallback(char* filename) { m_file=fopen(filename, "wb"); }
  virtual ~CWriterCallback() { if(m_file) fclose(m_file); }
  virtual void write(const void* buffer, const size_t size) { if(m_file) fwrite(buffer, size, 1, m_file); }
  FILE* m_file;
};
```

Then in the user application code, we can write the
initialisation this way :

```cpp
CWriterCallback oCallback("test.ebml");
EBML::IWriter* writer=EBML::createWriter(oCallback);
EBML::IWriterHelper* writerHelper=EBML::createWriterHelper();
writerHelper->connect(writer);
```

The use of the EBML writer looks something like this :

```cpp
writerHelper->openChild(EBML_Identifier_Header);
writerHelper->openChild(EBML_Identifier_DocType);
writerHelper->setStr("EBML basic sample");
writerHelper->closeChild();
writerHelper->openChild(EBML_Identifier_DocTypeVersion);
writerHelper->setUInt(1);
writerHelper->closeChild();
writerHelper->closeChild();
```

Finally, we have to release the objects and to clean memory :

```cpp
writerHelper->disconnect();
writerHelper->release();
writer->release();
```

# Introduction {#designer-introduction}

\page DesignerIntroduction Introduction

The OpenViBE Designer is an authoring tool dedicated to creating and executing OpenViBE scenarios. It is targeted at a broad range of users, including:

- **researchers** and students of the neuroscience and BCI community
- **neurophysiology experts** who need a tool to realize signal-processing and monitoring of EEG activity
- **clinicians** looking for a tool to conduct neurofeedback experiments
  
It relies on a graphical user interface to provide signal processing tools in an intuitive way, and doesn’t require any programming skills.

Each of these tools comes as a plugin, which communicates with the application via a generic interface hiding implementation details. As a result, it is easy for a programmer to extend the range of tools provided with the platform. Users may arrange any number of these boxes in a very flexible fashion, considering there is virtually no limit as to the number of boxes that may be included in a processing **scenario**.

Once a scenario is created, it may be run from Studio, which provides a toolbar for playing, pausing and stepping through a scenario. A number of box algorithms are available for direct **visualization** of results, from simple 2D displays such as Spectral Analysis and Continuous Oscilloscope to 3D paradigms such as 3D Topography. The layout of these displays may be customized as desired at scenario editing time using the **window manager** module of Studio.

# Parsing EBML streams {#parsing-ebml}

\page ParsingEBML Parsing EBML streams

## Introduction

This page tries to teach how a user of this library should use the reading functions in order to read and parse an EBML stream correctly. For those who don't know what EBML is, it is basically a binary XML created and used by the matroska developpers. The libmatroska is based on the libebml implementation of these guys. Fore more details, please read the [What Is EBML](_what_is_e_b_m_l.html) page first and eventually visit the EBML web page at <http://ebml.sourceforge.net/>

## Concepts

The idea of this parsing library is to transform the stream data provided by the user application into understandable EBML interpreted commands. Once the EBML nodes are found and parsed, they are sent to a callback object that should know what to do with them.

The design of the parsing interface is closed to the one of `eXpat`, an XML parser library (see <http://expat.sourceforge.net> for more details on `eXpat`). Using such interface allows light code and on-the-fly parsing, that means the parser does not need to have all the data ready before starting the parsing process... The data can arrive while the parsing is beeing done.

It is the responsability of the user application to read the EBML stream from a file, a socket, a user input or whatever, and then to send this to the parser...

At least, the callback object may use a reader helper that knows how to read standard EBML types such as integers, floats, strings etc...

The library is divided into three main components :

- The reader itself that does the parsing stuffs
- An implementation of the callback object (the implementation is left to the user application developper)
- An optionnal helper object that knows more on the content of the EBML stream.

Here comes the organisation of the different modules and how data go from one to another. Note that the user application and the user callback object may share some information so the callback object communicates with the application itself.

![Concept](images/ebml_parsing_concept.png)

Here comes the UML class diagram, presenting the main classes involved in the presented behavior.

![Class Diagram](images/ebml_parsing_class.png)

See `EBML::IReader`, `EBML::IReaderCallback` and `EBML::IReaderHelper` for more details on each of these classes.

## Sample code

In this section, a sample of user application code is presented that parses the sample file created in the page named : [Formating EBML Streams](_formating_e_b_m_l.html)

The parsed value are printed in the console.

The callback object looks something like this :

```cpp
class CReaderCallback : public EBML::IReaderCallback
{
public:
  CReaderCallback() { m_helper=EBML::createReaderHelper(); }
  virtual ~CReaderCallback() { if(m_helper) m_helper->release(); }
  virtual bool isMasterChild(const EBML::CIdentifier& identifier)
  {
    if(identifier==EBML_Identifier_Header) { return true; }
    if(identifier==EBML_Identifier_DocType) { return true; }
    if(identifier==EBML_Identifier_DocTypeVersion) { return true; }
    return false;
  }
  virtual void openChild(const EBML::CIdentifier& identifier) { m_oCurrent=identifier; }
  virtual void processChildData(const void* buffer, const size_t size)
  {
    if(m_oCurrent == EBML_Identifier_DocType) { std::cout << "Doc type:" << m_helper->getStr(buffer, size) << std::endl; }
    if(m_oCurrent == EBML_Identifier_DocTypeVersion) { std::cout << "Dox type version:" << m_helper->getUInt(buffer, size) << std::endl; }
  }
  virtual void closeChild()
  {
  }
  EBML::IReaderHelper* m_helper;
};
```

Then in the user application code, we can write the
initialisation this way :

```cpp
CReaderCallback oCallback;
EBML::IReader* reader=EBML::createReader(oCallback);
```

Now suppose the user application got some data from the file in a `buffer` of size `size`; it is sent to the parser this way :

```cpp
reader->processData(buffer, size);
```

Finally, don't forget to release the object and to clean memory :

```cpp
reader->release();
```

         *========================================*
                      |OpenViBE| |AUTHORS|
         *========================================*

         Contact: http://openvibe.inria.fr/contact/
		 
|People|

OpenViBE staff in alphabetical order

 - Guillermo Andrade-Barroso / Inria
 - Laurent Bonnet / Inria, Mensia
 - Alison Cellard / Inria
 - Jerome Chabrol / Inria 
 - Vincent Delannoy / Inria
 - Arthur Desbois / Inria
 - Agnes Dos Santos / Mensia 
 - Morgane Fauvet / Mensia
 - Nathanael Foy / Inria 
 - Charles Garraud / Inria 
 - Thierry Gaugry / Inria 
 - Anatole Lecuyer / Inria
 - Jozef Legeny / Inria, Mensia
 - Jussi T. Lindgren / Inria 
 - Fabien Lotte / Inria
 - Loic Mahe / Inria
 - Thibaut Monseigne / Inria
 - Baptiste Payan / Inria
 - Benoit Perrin / Mensia
 - Alexis Placet / Mensia
 - Thomas Prampart / Inria
 - Yann Renard / Inria, Mensia
 - Bruno Renier / Inria
 - Cedric Riou / Inria 
 - Guillaume Serri�re / Inria
 
Scientific Steering Committee :
 - Laurent Bougrain / Inria
 - Maureen Clerc / Inria
 - Anatole Lecuyer / Inria
 - Fabien Lotte / Inria
 - Theodore Papadopoulo / Inria

Other contributors :

 - Please see 
   
   http://openvibe.inria.fr/contributors/

 
|End.of.file|

 vim:tw=78:ts=8:ft=help:norl:

# What is EBML {#ebml}

\page WhatIsEBML What is EBML

This page tries to explain what EBML is and why it is usefull.

- see http://ebml.sourceforge.net/ for more details
- see http://ebml.sourceforge.net/specs/ for specifications
- see http://www.matroska.org/technical/specs/ for even more details

- check libebml for a sample implementation of EBML
- check libmatroska for a sample of library using libebml
  and a concrete use of EBML

\todo Write more documentation on what EBML is

− \subpage ParsingEBML  
− \subpage FormatingEBML

# Coding Style Guide

## General rules

### Base rules

The base rules applied to this project are from the [Google C++ Style Guide](https://google.github.io/styleguide/cppguide.html).  
Any rule listed below takes precedence over the corresponding rule.  
For any rule/guidance not mentioned here, please refer to the base guide. 

### Naming

- User defined data types are in UpperCamelCase with the following prefixes:

  ```text
  Interface: IUpperCamelCase
  Class:     CUpperCamelCase
  Template:  TUpperCamelCase
  Struct:    SUpperCamelCase
  Enum:      EUpperCamelCase
  Union:     UUpperCamelCase
  ```

- Variables are in lowerCamelCase
- Member variables are prefix with "m_": m_lowerCamelCase
- Private methods are in lowerCamelCase()
- Public methods and functions are UpperCamelCase()
- Enumeration members are in UpperCamelCase_UnderscoreTolerant
- constants are in UPPER_CASE
- Namespaces are in UpperCamelCase

### Formatting

- Indents are using TABs
  - 4 spaces long
  - Final align with spaces if needed
- No indents inside Namespaces
- Braces follow the Allman style for **type declarations and functions**:

  ```cpp
  CNewClass
  {
    public:
      void PublicMethod();
  }

  void CNewClass::PublicMethod()
  {
      ...
  }
  ```

### Commenting

Code must be documented with doxygen comments.  
Commonly used block comments are `///` or `/** */`.  
Commonly used inline comments are `///<` or `/**< */`.  
Below is an example of different styles:

```cpp 
/**
 * \brief Class for handling file writing/reading
 */
class CFileHandling
{
  public:
    ///
    /// \brief Constructor
    /// \param fileName The path to the file to handle
    ///
    explicit CFileHandling(const std::string& fileName);
    /**
     * @brief Opens file for reading or writing
     * @return true on success, false otherwise
     */
     bool OpenFile(const std::string& fileName);

    /// <summary> Writes provided data to file </summary>
    ///
    /// <param name="data"> The data to be written </param>
    /// <param name="append"> Flag set to true to append the data, or false to overwrite the file </param>
    /// <return> true on succes, false otherwise </return>
    bool WriteToFile(const std::string& data, bool append=true);
        
  private:
    std::string m_file;  /**< The path to the file being handled */
    size_t m_fileSize;  ///< The current size of the file
}
```

Be mindful **not to mix styles** within a file or set of files.  
More infos about documenting code on [Doxygen manual](https://www.doxygen.nl/manual/docblocks.html).

# OpenViBE Stream Structures {#stream-structures}

\page StreamStructures OpenViBE Stream Structures

## Introduction

All the streams are defined using EBML (Extensible Binary Meta-Language). They are all structured in 3 sections:

- **HEADER** :
  this part is the first ever sent and received on the stream, when the user presses "play" in a scenario. It usually contains the stream description used by the boxes and algorithms to initialize everything properly (e.g. size of an internal buffer)
- **BUFFERS** :
  the number of buffers that may then transit through the stream is unlimited. Buffers contain the payload, e.g. the current signal matrix.
- **END** :
  this last part is received once, when the user presses "stop" in a scenario. This mechanism is not concretely used in existing boxes, however everything is implemented to handle it.

## Hierarchy

The stream hierarchy is displayed on figure below. Every lower stream also
contains the information specific to the upper stream. The color of the
corresponding input and output in the Designer are also displayed.

![The OpenViBE stream hierarchy](images/streams.png)  
The OpenViBE stream hierarchy

As you can see every stream is an **EBML stream**. Then the EBML container is
specialized in 3 streams : **experiment information**, **stimulations** or
**streamed matrix***.

The **Streamed Matrix** stream can be used as is, or as one of its 4 derived
streams : **channel localisation**, **channel units**, **feature vector**,
**signal** or **spectrum**.

```cpp
#define OVTK_TypeId_EBMLStream                                          OpenViBE::CIdentifier(0x434F6587, 0x2EFD2B7E)
#define   OVTK_TypeId_ExperimentInfo                                    OpenViBE::CIdentifier(0x403488E7, 0x565D70B6)
#define   OVTK_TypeId_Stimulations                                      OpenViBE::CIdentifier(0x6F752DD0, 0x082A321E)
#define   OVTK_TypeId_StreamedMatrix                                    OpenViBE::CIdentifier(0x544A003E, 0x6DCBA5F6)
#define     OVTK_TypeId_FeatureVector                                   OpenViBE::CIdentifier(0x17341935, 0x152FF448)
#define     OVTK_TypeId_Signal                                          OpenViBE::CIdentifier(0x5BA36127, 0x195FEAE1)
#define     OVTK_TypeId_Spectrum                                        OpenViBE::CIdentifier(0x1F261C0A, 0x593BF6BD)
#define     OVTK_TypeId_ChannelLocalisation                             OpenViBE::CIdentifier(0x1E4C0D6E, 0x5204EEB2)
#define     OVTK_TypeId_ChannelUnits                                    OpenViBE::CIdentifier(0x5E330216, 0x2C09724C)
```

The following sections will describe each stream type.

### Standard EBML stream

![EBML stream description](images/ebml-stream-card.png)  
EBML stream description (November 6th 2006, version 1).

From any standard EBML stream header, you can retrieve the stream type identifier, and the stream version.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header                                              EBML::CIdentifier(0x002B395F, 0x108ADFAE)
#define OVTK_NodeId_Header_StreamType                                   EBML::CIdentifier(0x00CDD0F7, 0x46B0278D)
#define OVTK_NodeId_Header_StreamVersion                                EBML::CIdentifier(0x006F5A08, 0x7796EBC5)
#define OVTK_NodeId_Buffer                                              EBML::CIdentifier(0x00CF2101, 0x02375310)
#define OVTK_NodeId_End                                                 EBML::CIdentifier(0x00D9DDC3, 0x0B12873A)
```

### Streamed matrix stream

![Streamed matrix stream description](images/sm-stream-card.png)

Streamed matrix stream description (November 6th 2006, version 1).

The streamed matrix stream is an EBML Stream that also contains a matrix
descriptor in the header, and a **matrix of double precision floating values**
in its buffers. The matrix descriptor gives the **dimension count**, **size**
of each dimension, and **labels** of each dimension.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Header_StreamedMatrix
OVTK_NodeId_Header_StreamedMatrix_DimensionCount (integer:)
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:)
...
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header_StreamedMatrix                               EBML::CIdentifier(0x0072F560, 0x7ED2CBED)
#define OVTK_NodeId_Header_StreamedMatrix_DimensionCount                EBML::CIdentifier(0x003FEBD4, 0x2725D428)
#define OVTK_NodeId_Header_StreamedMatrix_Dimension                     EBML::CIdentifier(0x0000E3C0, 0x3A7D5141)
#define OVTK_NodeId_Header_StreamedMatrix_Dimension_Size                EBML::CIdentifier(0x001302F7, 0x36D8438A)
#define OVTK_NodeId_Header_StreamedMatrix_Dimension_Label               EBML::CIdentifier(0x00153E40, 0x190227E0)
#define OVTK_NodeId_Buffer_StreamedMatrix                               EBML::CIdentifier(0x00120663, 0x08FBC165)
#define OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer                     EBML::CIdentifier(0x00B18C10, 0x427D098C)
```

### Channel localization stream

![Channel localisation description](images/chanloc-stream-card.png)

Channel localisation description (nov 04th 2008, version 1)

The channel localisation stream is a stream matrix stream with some
constraints:

- 2 dimensions : **channels** and **positions**
- First dimension has variable size (channel count)
- Second dimension has a size of 3 : the normalized cartesian coordinates in
  the following frame of reference : X right, Y front and Z up.

Its header also exposes a boolean value : **Dynamic**. If false the coordinates
are assumed to be static, thus sent once and for all (e.g. with an EEG system).
If Dynamic is set, the coordinates of the sensors may change over time, thus
several buffer can be received over time.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:2)
OVTK_NodeId_Header_ChannelLocalisation
OVTK_NodeId_Header_ChannelLocalisation_Dynamic (boolean)
OVTK_NodeId_Header_StreamedMatrix
OVTK_NodeId_Header_StreamedMatrix_DimensionCount (integer:2)
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:channel count)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 1 name)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 2 name)
...
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:3)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:x)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:y)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:z)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header_ChannelLocalisation                          EBML::CIdentifier(0xF2CFE60B, 0xEFD63E3B)
#define OVTK_NodeId_Header_ChannelLocalisation_Dynamic                  EBML::CIdentifier(0x5338AF5C, 0x07C469C3)
```

### Channel units stream

Channel units description (mar 30th 2015, version 1).

The channel units stream is a streamed matrix stream intended to carry channel
measurement unit information from the drivers to Designer (e.g. ‘channel 1 is
in microvolts’). The stream has some constraints:

- 2 dimensions : **channels** and their **properties**
- First dimension has variable size (channel count)
- Second dimension has a size of 2 : the unit and the scaling factor.  Although
  the stream contents can be handled as a matrix, the unit codes and scaling
  factors supported and understood by OpenViBE are limited to a specific set.
  The list is in in `toolkit/include/ovtk_defines.h`.

Its header also exposes a boolean value : **Dynamic**. If false the units are assumed to be static, thus sent once and for all (e.g. with an EEG system). If Dynamic is set, the units may change over time, thus several buffer can be received over time. However, no component in 1.0.0 supports the dynamic units.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:2)
OVTK_NodeId_Header_ChannelUnits
OVTK_NodeId_Header_ChannelUnits_Dynamic (boolean)
OVTK_NodeId_Header_StreamedMatrix
OVTK_NodeId_Header_StreamedMatrix_DimensionCount (integer:2)
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:channel count)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 1 name)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 2 name)
...
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:2)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:unit)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:factor)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header                                              EBML::CIdentifier(0x002B395F, 0x108ADFAE)
#define OVTK_NodeId_Header_StreamType                                   EBML::CIdentifier(0x00CDD0F7, 0x46B0278D)
#define OVTK_NodeId_Header_StreamVersion                                EBML::CIdentifier(0x006F5A08, 0x7796EBC5)
#define OVTK_NodeId_Buffer                                              EBML::CIdentifier(0x00CF2101, 0x02375310)
#define OVTK_NodeId_End                                                 EBML::CIdentifier(0x00D9DDC3, 0x0B12873A)
```

### Feature vector stream

![Feature vector stream description](images/fvec-stream-card.png)

Feature vector stream description (may 24th 2007, version 1).

The feature vector stream is a stream matrix stream with some constraints:

- 1 dimension : the **features**
- Number of features may vary, but remains the same for one stream life time.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Header_StreamedMatrix
OVTK_NodeId_Header_StreamedMatrix_DimensionCount (integer:1)
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:feature count)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:feature 1 name)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:feature 2 name)
...
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
...
OVTK_NodeId_End
```

### Spectrum stream

![Spectrum stream description](images/spec-stream-card.png)

Spectrum stream description (jan. 2017, version 2).

The spectrum stream is a stream matrix stream with some constraints:

// 2 dimensions : **channels** and **number of frequency values**
// First dimension has variable size (channel count)
// Second dimension has variable size (number of frequency values)

Its header also expose the sampling rate of the signal and the list of frequency values used.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Header_StreamedMatrix
OVTK_NodeId_Header_StreamedMatrix_DimensionCount (integer:2)
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:channel count)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 1 name)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 2 name)
...
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:number of frequency band)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:frequency band 1 name)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:frequency band 2 name)
...
OVTK_NodeId_Header_Spectrum
OVTK_NodeId_Header_Spectrum_Sampling
OVTK_NodeId_Header_Spectrum_FrequencyAbscissa (double)
OVTK_NodeId_Header_Spectrum_FrequencyAbscissa (double)
OVTK_NodeId_Header_Spectrum_FrequencyAbscissa (double)
...
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header_Spectrum                                     EBML::CIdentifier(0x00CCFA4B, 0x14F37D4D)
#define OVTK_NodeId_Header_Spectrum_FrequencyBand_Deprecated            EBML::CIdentifier(0x0010983C, 0x21F8BDE5)
#define OVTK_NodeId_Header_Spectrum_FrequencyBand_Start_Deprecated      EBML::CIdentifier(0x00AA5654, 0x2403A2CB)
#define OVTK_NodeId_Header_Spectrum_FrequencyBand_Stop_Deprecated       EBML::CIdentifier(0x00A44C82, 0x05BE50D5)
#define OVTK_NodeId_Header_Spectrum_FrequencyAbscissa                   EBML::CIdentifier(0x00D7287D, 0x622A2BF5)
#define OVTK_NodeId_Header_Spectrum_Sampling                            EBML::CIdentifier(0x006876E9, 0x1DCB0CA1)
```

### Signal stream

![Signal stream description](images/signal-stream-card.png)

Signal stream description (may 24th 2007, version 2).

The signal stream is a stream matrix stream with some constraints:

- 2 dimensions : **channels** and **samples**
- First dimension has variable size (channel count)
- Second dimension has a size corresponding to the **sample count per buffer**.

Its header also expose an integer value : the **Sampling Frequency**. Sampling
frequency and sample count per buffer depends on the data producer (file reader
or acquisition client).

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Header_Signal
OVTK_NodeId_Header_Signal_Sampling (integer)
OVTK_NodeId_Header_StreamedMatrix
OVTK_NodeId_Header_StreamedMatrix_DimensionCount (integer:2)
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:channel count)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 1 name)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string:channel 2 name)
...
OVTK_NodeId_Header_StreamedMatrix_Dimension
OVTK_NodeId_Header_StreamedMatrix_Dimension_Size (integer:number of samples per buffer)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string)
OVTK_NodeId_Header_StreamedMatrix_Dimension_Label (string)
...
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_StreamedMatrix
OVTK_NodeId_Buffer_StreamedMatrix_RawBuffer (array of double)
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header_Signal                                       EBML::CIdentifier(0x007855DE, 0x3748D375)
#define OVTK_NodeId_Header_Signal_Sampling                              EBML::CIdentifier(0x00141C43, 0x0C37006B)
```

### Stimulation stream

![Stimulation stream description](images/stim-stream-card.png)

Stimulation stream description (may 24th 2007, version 3).

The stimulation stream is an EBML stream which buffers contains stimulation
sets. Each stimulation has 3 attributes:

- **Stimulation Date**: a 64bit unsigned integer in 32:32 precision (first 32 bits for the seconds, last 32 bits for the divisions)
- **Stimulation Identifier** as listed in the documentation page.
- **Stimulation Duration**: same encoding as stimulation date. In practice, the duration is never used by boxes, thus the value is most of the time 0.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer_Stimulation
OVTK_NodeId_Buffer_Stimulation_NumberOfStimulations (integer)
OVTK_NodeId_Buffer_Stimulation_Stimulation
OVTK_NodeId_Buffer_Stimulation_Stimulation_ID (integer)
OVTK_NodeId_Buffer_Stimulation_Stimulation_Date (integer)
OVTK_NodeId_Buffer_Stimulation_Stimulation_Duration (integer)
OVTK_NodeId_Stimulation_Stimulation
...
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer
...
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Buffer_Stimulation                                  EBML::CIdentifier(0x006DEABE, 0x7FC05A20)
#define OVTK_NodeId_Buffer_Stimulation_NumberOfStimulations             EBML::CIdentifier(0x00BB790B, 0x2B8574D8)
#define OVTK_NodeId_Buffer_Stimulation_Stimulation                      EBML::CIdentifier(0x0016EAC6, 0x29FBCAA1)
#define OVTK_NodeId_Buffer_Stimulation_Stimulation_ID                   EBML::CIdentifier(0x006FA5DB, 0x4BAC31E9)
#define OVTK_NodeId_Buffer_Stimulation_Stimulation_Date                 EBML::CIdentifier(0x00B866D8, 0x14DA5374)
#define OVTK_NodeId_Buffer_Stimulation_Stimulation_Duration             EBML::CIdentifier(0x14EE055F, 0x87FBCC9C)
```

### Experimentation information stream

![Experimentation information stream description](images/xp-stream-card.png)

Experimentation information stream description (november 6th 2006, version 1).

The experiment information stream is an EBML stream that provides in its header
some information about the experiment being conducted. These information are
entered for example in the acquisition server, when configuring the driver
before starting the acquisition. You can find:

- **Experiment description**: identifier and date.
- **Subject description**: identifier, name, age and gender.
- **Context**: laboratory identifier and name, technician identifier and name.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
OVTK_NodeId_Header_StreamType (integer:)
OVTK_NodeId_Header_StreamVersion (integer:)
OVTK_NodeId_Header_ExperimentInfo
OVTK_NodeId_Header_ExperimentInfo_Experiment
OVTK_NodeId_Header_ExperimentInfo_Experiment_ID (integer)
OVTK_NodeId_Header_ExperimentInfo_Experiment_Date (date)
OVTK_NodeId_Header_ExperimentInfo_Subject
OVTK_NodeId_Header_ExperimentInfo_Subject_ID (integer)
OVTK_NodeId_Header_ExperimentInfo_Subject_Name (string)
OVTK_NodeId_Header_ExperimentInfo_Subject_Age (integer)
OVTK_NodeId_Header_ExperimentInfo_Subject_Gender (integer)
OVTK_NodeId_Header_ExperimentInfo_Context
OVTK_NodeId_Header_ExperimentInfo_Context_LaboratoryID (integer)
OVTK_NodeId_Header_ExperimentInfo_Context_LaboratoryName (string)
OVTK_NodeId_Header_ExperimentInfo_Context_TechnicianID (integer)
OVTK_NodeId_Header_ExperimentInfo_Context_TechnicianName (string)
OVTK_NodeId_End
```

```cpp
// NODE IDENTIFIERS
// ----------------
#define OVTK_NodeId_Header_ExperimentInfo                               EBML::CIdentifier(0x00746BA0, 0x115AE04D)
#define OVTK_NodeId_Header_ExperimentInfo_Experiment                    EBML::CIdentifier(0x0011D6B7, 0x48F1AA39)
#define OVTK_NodeId_Header_ExperimentInfo_Experiment_ID                 EBML::CIdentifier(0x006ACD74, 0x1C960C26)
#define OVTK_NodeId_Header_ExperimentInfo_Experiment_Date               EBML::CIdentifier(0x002F8FB7, 0x6DA7552D)
#define OVTK_NodeId_Header_ExperimentInfo_Subject                       EBML::CIdentifier(0x003EC620, 0x333E0A94)
#define OVTK_NodeId_Header_ExperimentInfo_Subject_ID                    EBML::CIdentifier(0x00D62974, 0x473D4AA5)
#define OVTK_NodeId_Header_ExperimentInfo_Subject_Name                  EBML::CIdentifier(0x0041FD0A, 0x6BCD9A99)
#define OVTK_NodeId_Header_ExperimentInfo_Subject_Age                   EBML::CIdentifier(0x00DF7DD9, 0x33336C51)
// for retro compat
#define OVTK_NodeId_Header_ExperimentInfo_Subject_Sex                   EBML::CIdentifier(0x0069BB84, 0x3FC8E149)
#define OVTK_NodeId_Header_ExperimentInfo_Subject_Gender                EBML::CIdentifier(0x0069BB84, 0x3FC8E149)
#define OVTK_NodeId_Header_ExperimentInfo_Context                       EBML::CIdentifier(0x0018C291, 0x7985DFDD)
#define OVTK_NodeId_Header_ExperimentInfo_Context_LaboratoryID          EBML::CIdentifier(0x003F11B9, 0x26D76D9C)
#define OVTK_NodeId_Header_ExperimentInfo_Context_LaboratoryName        EBML::CIdentifier(0x00EB1F23, 0x51C23B83)
#define OVTK_NodeId_Header_ExperimentInfo_Context_TechnicianID          EBML::CIdentifier(0x00874A7F, 0x60DC34C2)
#define OVTK_NodeId_Header_ExperimentInfo_Context_TechnicianName        EBML::CIdentifier(0x00C8C393, 0x31CE5B3E)
```

### Acquisition stream

![Acquisition stream description](images/acquisition-stream-small.png)

Acquisition stream description (nov 2014, version 3). This is a multiplexed
stream.

The acquisition stream encapsulates 5 streams : experiment information, channel
localisation, channel units, signal and stimulation.

This stream is produced by the Acquisition Server and is sent to every client
connected via tcp/ip network connection. The stream is received by the
Acquisition Client box, that separates each streams and makes them available on
5 corresponding outputs.

#### EBML description & identifiers

```cpp
// STREAM STRUCTURE
// ----------------
OVTK_NodeId_Header
  OVTK_NodeId_Acquisition_Header_ExperimentInfo
    ... some experiment information stream header
  OVTK_NodeId_Acquisition_Header_Signal
    ... some signal stream header
  OVTK_NodeId_Acquisition_Header_Stimulation
    ... some stimulation stream header
  OVTK_NodeId_Acquisition_Header_ChannelLocalisation
    ... some channel localisation stream header
  OVTK_NodeId_Acquisition_Header_ChannelUnits
    ... some channel units stream header
OVTK_NodeId_Buffer
  OVTK_NodeId_Acquisition_Buffer_ExperimentInfo
    ... some experiment information stream buffer
  OVTK_NodeId_Acquisition_Buffer_Signal
    ... some signal stream buffer
  OVTK_NodeId_Acquisition_Buffer_Stimulation
    ... some stimulation stream buffer
  OVTK_NodeId_Acquisition_Buffer_ChannelLocalisation
    ... some channel localisation stream buffer
  OVTK_NodeId_Acquisition_Buffer_ChannelUnits
    ... some channel units stream buffer
OVTK_NodeId_Buffer
OVTK_NodeId_Buffer
...
```

# Metaboxes {#designer-metaboxes}

\page DesignerMetaboxes Metaboxes

In OpenViBE Designer you can create new boxes by assembling them from parts of other boxes, they are called metaboxes. This section describes their usage. Metaboxes are loaded on start-up of the OpenViBE Designer. They are loaded from the directories `$InstallDir/share/openvibe/metaboxes` and `%APPDATA%/mensia/metaboxes`. These metaboxes are written under the formats `.mxb` or `.mbb`.

A Metabox behaves just like a normal box, that is:

- It has an arbitrary number of inputs and outputs using the same types as other boxes.
- It can have an arbitrary number of settings.
- It can be inserted into a scenario.
- It can have visualizations.

## Creating Metaboxes

In order to create a Metabox you first have to create a scenario.

Example: we want to make a Metabox that will apply a notch filter for 48-52Hz and then apply an arbitrary band-pass filter on the signal.

Thus, our Metabox consists of:

- One Signal input
- One Signal output
- Two temporal filter boxes
- Two settings
  - Lower frequency bound
  - Upper frequency bound

### Step 1: Adding boxes and settings

We create a new empty scenario and add two settings to it, we also drag two temporal filter boxes inside and set them up like so:

![Creating a metabox - step 1](images/metaboxes/creating-01.png)  
Creating a metabox - step 1

First Temporal filter box is renamed Band Pass, the second one is renamed Notch filter (right-click + rename box: Notch Filter).

We set the notch filter to a band stop filter with cut off frequencies at 48 and 52Hz.

### Step 2: Adding inputs and outputs

In order to "expose" inputs and outputs of our metabox we need to add some inputs and outputs to a scenario.

To do so we use the Scenario I/O tab. To add an input or output simply click the appropriate add button. A setting has only a name and a type. By default, the created type is Streamed Matrix.

- Add one **input** to the scenario, change its type to **Signal** and rename it to **Input Signal**
- Add one **output** to the scenario, change its type to **Signal** and rename it to **Filtered Signal**

![Adding inputs and outputs](images/metaboxes/creating-02.png)  
Adding inputs and outputs

In order to associate a scenario input with a box input, right click on the box you wish to send the input to. A new context menu item will be available: **Connect scenario inputs**. In this context menu item will be a list of all box inputs which will again open a list of all scenario inputs. Clicking on the scenario input will link the box input to the scenario input.

Right click on the **Notch filter** box. In the **connect scenario inputs** submenu you will see the single input of the box. Incidentally it is also called *Input Signal*, under this item there will be another item called *Input Signal* – this is our scenario’s input.

![Linking a scenario input](images/metaboxes/creating-03.png)  
Linking a scenario input

Now do the same for the output. A circular indicator will be displayed to represent the link between the scenario input and the box input.

![Linked inputs and outputs](images/metaboxes/creating-04.png)  
Linked inputs and outputs

Each scenario input can be linked to exactly one box input. If you wish to use the same input in several boxes, use an identity box.

### Step 3: Adding Metadata

The metabox reads its metadata from the scenario information. You can edit it by clicking on the star button in the toolbar.

The most important attributes are **Name**, **Category** and **Metabox Id**. This is how your box will be identified inside the box-algorithm list.

The **Metabox Id** serves as a unique identifier for this box. You can either specify it yourself (as a group of two 32bit hex integers) or use the refresh button to generate one.

Save your box inside the `%APPDATA%/mensia/metaboxes` folder.

![Change about scenario](images/metaboxes/creating-05.png)  
Change about scenario

## Using a Metabox

Metaboxes will appear in the sidebar just like the other boxes, except they are highlighted in a green colour.

You can drag a metabox into the scenario as you would any other box. It will appear with a double border.

![Metabox in scenario](images/metaboxes/using-in-scenario.png)  
Metabox in scenario

Double clicking on the metabox will reveal all of the settings of the scenario that represents the metabox. You can also right click on a metabox and open it in editor from the context menu *Edit this metabox in the editor*.

# OpenViBE Stimulations Code list {#stimulations}

\page Stimulations OpenViBE Stimulations Code

| Name                                                  | Hexadecimal Code | Decimal Code |
| :---------------------------------------------------- | ---------------: | -----------: |
| OVTK_GDF_100_Watt                                     |     0x00000584   |       1412   |
| OVTK_GDF_125_Watt                                     |     0x00000585   |       1413   |
| OVTK_GDF_150_Watt                                     |     0x00000586   |       1414   |
| OVTK_GDF_175_Watt                                     |     0x00000587   |       1415   |
| OVTK_GDF_200_Watt                                     |     0x00000588   |       1416   |
| OVTK_GDF_225_Watt                                     |     0x00000589   |       1417   |
| OVTK_GDF_25_Watt                                      |     0x00000581   |       1409   |
| OVTK_GDF_250_Watt                                     |     0x0000058A   |       1418   |
| OVTK_GDF_275_Watt                                     |     0x0000058B   |       1419   |
| OVTK_GDF_300_Watt                                     |     0x0000058C   |       1420   |
| OVTK_GDF_325_Watt                                     |     0x0000058D   |       1421   |
| OVTK_GDF_350_Watt                                     |     0x0000058E   |       1422   |
| OVTK_GDF_50_Watt                                      |     0x00000582   |       1410   |
| OVTK_GDF_75_Watt                                      |     0x00000583   |       1411   |
| OVTK_GDF_AEP                                          |     0x00000122   |        290   |
| OVTK_GDF_Arm_Movement                                 |     0x00000449   |       1097   |
| OVTK_GDF_Arm_Right_Movement                           |     0x0000044A   |       1098   |
| OVTK_GDF_Artifact_50_60_Hz_Interference               |     0x00000107   |        263   |
| OVTK_GDF_Artifact_Breathing                           |     0x00000108   |        264   |
| OVTK_GDF_Artifact_ECG                                 |     0x00000102   |        258   |
| OVTK_GDF_Artifact_EMG                                 |     0x00000103   |        259   |
| OVTK_GDF_Artifact_EOG_Large                           |     0x00000101   |        257   |
| OVTK_GDF_Artifact_EOG_Small                           |     0x0000010A   |        266   |
| OVTK_GDF_Artifact_Failing_Electrode                   |     0x00000105   |        261   |
| OVTK_GDF_Artifact_Movement                            |     0x00000104   |        260   |
| OVTK_GDF_Artifact_Pulse                               |     0x00000109   |        265   |
| OVTK_GDF_Artifact_Sweat                               |     0x00000106   |        262   |
| OVTK_GDF_Beep                                         |     0x00000311   |        785   |
| OVTK_GDF_Biting                                       |     0x00000446   |       1094   |
| OVTK_GDF_CAHE                                         |     0x00000403   |       1027   |
| OVTK_GDF_Calibration                                  |     0x0000010F   |        271   |
| OVTK_GDF_class10                                      |     0x0000030A   |        778   |
| OVTK_GDF_class11                                      |     0x0000030B   |        779   |
| OVTK_GDF_class5                                       |     0x00000305   |        773   |
| OVTK_GDF_class7                                       |     0x00000307   |        775   |
| OVTK_GDF_class8                                       |     0x00000308   |        776   |
| OVTK_GDF_class9                                       |     0x00000309   |        777   |
| OVTK_GDF_Correct                                      |     0x00000381   |        897   |
| OVTK_GDF_Cross_On_Screen                              |     0x00000312   |        786   |
| OVTK_GDF_CSB                                          |     0x00000404   |       1028   |
| OVTK_GDF_Cue_Unknown_Undefined                        |     0x0000030F   |        783   |
| OVTK_GDF_Down                                         |     0x00000306   |        774   |
| OVTK_GDF_ECG_Fiducial_Point_QRS_Complex               |     0x00000501   |       1281   |
| OVTK_GDF_ECG_P_Wave                                   |     0x00000502   |       1282   |
| OVTK_GDF_ECG_QRS_Complex                              |     0x00000503   |       1283   |
| OVTK_GDF_ECG_R_Point                                  |     0x00000504   |       1284   |
| OVTK_GDF_ECG_T_Wave                                   |     0x00000506   |       1286   |
| OVTK_GDF_ECG_U_Wave                                   |     0x00000507   |       1287   |
| OVTK_GDF_EEG_Idling_EEG_Eyes_Closed                   |     0x00000115   |        277   |
| OVTK_GDF_EEG_Idling_EEG_Eyes_Open                     |     0x00000114   |        276   |
| OVTK_GDF_EEG_K_Complexes                              |     0x00000112   |        274   |
| OVTK_GDF_EEG_Saw_Tooth_Waves                          |     0x00000113   |        275   |
| OVTK_GDF_EEG_Seizure                                  |     0x00000117   |        279   |
| OVTK_GDF_EEG_Sleep_Splindles                          |     0x00000111   |        273   |
| OVTK_GDF_EEG_Spike                                    |     0x00000116   |        278   |
| OVTK_GDF_End_Of_Session                               |     0x000003F2   |       1010   |
| OVTK_GDF_End_Of_Trial                                 |     0x00000320   |        800   |
| OVTK_GDF_Eye_Blink                                    |     0x00000439   |       1081   |
| OVTK_GDF_Eyes_Down                                    |     0x00000434   |       1076   |
| OVTK_GDF_Eyes_Left                                    |     0x00000431   |       1073   |
| OVTK_GDF_Eyes_Right                                   |     0x00000432   |       1074   |
| OVTK_GDF_Eyes_Up                                      |     0x00000433   |       1075   |
| OVTK_GDF_Feedback_Continuous                          |     0x0000030D   |        781   |
| OVTK_GDF_Feedback_Discrete                            |     0x0000030E   |        782   |
| OVTK_GDF_Flashing_Light                               |     0x00000313   |        787   |
| OVTK_GDF_Foot                                         |     0x00000303   |        771   |
| OVTK_GDF_Foot_Movement                                |     0x00000447   |       1095   |
| OVTK_GDF_Foot_Right_Movement                          |     0x00000448   |       1096   |
| OVTK_GDF_Head_Movement                                |     0x00000443   |       1091   |
| OVTK_GDF_Horizontal_Eye_Movement                      |     0x00000435   |       1077   |
| OVTK_GDF_Incorrect                                    |     0x00000382   |        898   |
| OVTK_GDF_Left                                         |     0x00000301   |        769   |
| OVTK_GDF_Left_Hand_Movement                           |     0x00000441   |       1089   |
| OVTK_GDF_Lights_Off                                   |     0x00008420   |      33824   |
| OVTK_GDF_Lights_On                                    |     0x00000420   |       1056   |
| OVTK_GDF_Maximum_Inspiration                          |     0x0000040E   |       1038   |
| OVTK_GDF_Non_Equidistant_Sampling_Value               |     0x00007FFF   |      32767   |
| OVTK_GDF_OAHE                                         |     0x00000401   |       1025   |
| OVTK_GDF_Rejection                                    |     0x000003FF   |       1023   |
| OVTK_GDF_REM                                          |     0x00000415   |       1045   |
| OVTK_GDF_RERA                                         |     0x00000402   |       1026   |
| OVTK_GDF_Right                                        |     0x00000302   |        770   |
| OVTK_GDF_Right_Hand_Movement                          |     0x00000442   |       1090   |
| OVTK_GDF_Rotation_Clockwise                           |     0x00000437   |       1079   |
| OVTK_GDF_Rotation_Counterclockwise                    |     0x00000438   |       1080   |
| OVTK_GDF_SEP                                          |     0x00000123   |        291   |
| OVTK_GDF_Sleep_Hypoventilation                        |     0x00000405   |       1029   |
| OVTK_GDF_SSAEP                                        |     0x00000132   |        306   |
| OVTK_GDF_SSSEP                                        |     0x00000133   |        307   |
| OVTK_GDF_SSVEP                                        |     0x00000131   |        305   |
| OVTK_GDF_Stage_1                                      |     0x00000411   |       1041   |
| OVTK_GDF_Stage_2                                      |     0x00000412   |       1042   |
| OVTK_GDF_Stage_3                                      |     0x00000413   |       1043   |
| OVTK_GDF_Stage_4                                      |     0x00000414   |       1044   |
| OVTK_GDF_Start                                        |     0x00000580   |       1408   |
| OVTK_GDF_Start_Of_Inspiration                         |     0x0000040F   |       1039   |
| OVTK_GDF_Start_Of_New_Segment                         |     0x00007FFE   |      32766   |
| OVTK_GDF_Start_Of_Trial                               |     0x00000300   |        768   |
| OVTK_GDF_Swallowing                                   |     0x00000445   |       1093   |
| OVTK_GDF_TMS                                          |     0x0000012F   |        303   |
| OVTK_GDF_Tongue                                       |     0x00000304   |        772   |
| OVTK_GDF_Tongue_Movement                              |     0x00000444   |       1092   |
| OVTK_GDF_Up                                           |     0x0000030C   |        780   |
| OVTK_GDF_VEP                                          |     0x00000121   |        289   |
| OVTK_GDF_Vertical_Eye_Movement                        |     0x00000436   |       1078   |
| OVTK_GDF_Wake                                         |     0x00000410   |       1040   |
| OVTK_StimulationId_AddedSamplesBegin                  |     0x00008311   |      33553   |
| OVTK_StimulationId_AddedSamplesEnd                    |     0x00008312   |      33554   |
| OVTK_StimulationId_Artifact                           |     0x00008302   |      33538   |
| OVTK_StimulationId_BaselineStart                      |     0x00008007   |      32775   |
| OVTK_StimulationId_BaselineStop                       |     0x00008008   |      32776   |
| OVTK_StimulationId_Beep                               |     0x00008202   |      33282   |
| OVTK_StimulationId_Button1_Pressed                    |     0x00008012   |      32786   |
| OVTK_StimulationId_Button1_Released                   |     0x00008013   |      32787   |
| OVTK_StimulationId_Button2_Pressed                    |     0x00008014   |      32788   |
| OVTK_StimulationId_Button2_Released                   |     0x00008015   |      32789   |
| OVTK_StimulationId_Button3_Pressed                    |     0x00008016   |      32790   |
| OVTK_StimulationId_Button3_Released                   |     0x00008017   |      32791   |
| OVTK_StimulationId_Button4_Pressed                    |     0x00008018   |      32792   |
| OVTK_StimulationId_Button4_Released                   |     0x00008019   |      32793   |
| OVTK_StimulationId_DoubleBeep                         |     0x00008203   |      33283   |
| OVTK_StimulationId_EndOfFile                          |     0x00008204   |      33284   |
| OVTK_StimulationId_ExperimentStart                    |     0x00008001   |      32769   |
| OVTK_StimulationId_ExperimentStop                     |     0x00008002   |      32770   |
| OVTK_StimulationId_Label_00                           |     0x00008100   |      33024   |
| OVTK_StimulationId_Label_01                           |     0x00008101   |      33025   |
| OVTK_StimulationId_Label_02                           |     0x00008102   |      33026   |
| OVTK_StimulationId_Label_03                           |     0x00008103   |      33027   |
| OVTK_StimulationId_Label_04                           |     0x00008104   |      33028   |
| OVTK_StimulationId_Label_05                           |     0x00008105   |      33029   |
| OVTK_StimulationId_Label_06                           |     0x00008106   |      33030   |
| OVTK_StimulationId_Label_07                           |     0x00008107   |      33031   |
| OVTK_StimulationId_Label_08                           |     0x00008108   |      33032   |
| OVTK_StimulationId_Label_09                           |     0x00008109   |      33033   |
| OVTK_StimulationId_Label_0A                           |     0x0000810a   |      33034   |
| OVTK_StimulationId_Label_0B                           |     0x0000810b   |      33035   |
| OVTK_StimulationId_Label_0C                           |     0x0000810c   |      33036   |
| OVTK_StimulationId_Label_0D                           |     0x0000810d   |      33037   |
| OVTK_StimulationId_Label_0E                           |     0x0000810e   |      33038   |
| OVTK_StimulationId_Label_0F                           |     0x0000810f   |      33039   |
| OVTK_StimulationId_Label_10                           |     0x00008110   |      33040   |
| OVTK_StimulationId_Label_11                           |     0x00008111   |      33041   |
| OVTK_StimulationId_Label_12                           |     0x00008112   |      33042   |
| OVTK_StimulationId_Label_13                           |     0x00008113   |      33043   |
| OVTK_StimulationId_Label_14                           |     0x00008114   |      33044   |
| OVTK_StimulationId_Label_15                           |     0x00008115   |      33045   |
| OVTK_StimulationId_Label_16                           |     0x00008116   |      33046   |
| OVTK_StimulationId_Label_17                           |     0x00008117   |      33047   |
| OVTK_StimulationId_Label_18                           |     0x00008118   |      33048   |
| OVTK_StimulationId_Label_19                           |     0x00008119   |      33049   |
| OVTK_StimulationId_Label_1A                           |     0x0000811a   |      33050   |
| OVTK_StimulationId_Label_1B                           |     0x0000811b   |      33051   |
| OVTK_StimulationId_Label_1C                           |     0x0000811c   |      33052   |
| OVTK_StimulationId_Label_1D                           |     0x0000811d   |      33053   |
| OVTK_StimulationId_Label_1E                           |     0x0000811e   |      33054   |
| OVTK_StimulationId_Label_1F                           |     0x0000811f   |      33055   |
| OVTK_StimulationId_Label_20                           |     0x00008120   |      33056   |
| OVTK_StimulationId_Label_21                           |     0x00008121   |      33057   |
| OVTK_StimulationId_Label_22                           |     0x00008122   |      33058   |
| OVTK_StimulationId_Label_23                           |     0x00008123   |      33059   |
| OVTK_StimulationId_Label_24                           |     0x00008124   |      33060   |
| OVTK_StimulationId_Label_25                           |     0x00008125   |      33061   |
| OVTK_StimulationId_Label_26                           |     0x00008126   |      33062   |
| OVTK_StimulationId_Label_27                           |     0x00008127   |      33063   |
| OVTK_StimulationId_Label_28                           |     0x00008128   |      33064   |
| OVTK_StimulationId_Label_29                           |     0x00008129   |      33065   |
| OVTK_StimulationId_Label_2A                           |     0x0000812a   |      33066   |
| OVTK_StimulationId_Label_2B                           |     0x0000812b   |      33067   |
| OVTK_StimulationId_Label_2C                           |     0x0000812c   |      33068   |
| OVTK_StimulationId_Label_2D                           |     0x0000812d   |      33069   |
| OVTK_StimulationId_Label_2E                           |     0x0000812e   |      33070   |
| OVTK_StimulationId_Label_2F                           |     0x0000812f   |      33071   |
| OVTK_StimulationId_Label_30                           |     0x00008130   |      33072   |
| OVTK_StimulationId_Label_31                           |     0x00008131   |      33073   |
| OVTK_StimulationId_Label_32                           |     0x00008132   |      33074   |
| OVTK_StimulationId_Label_33                           |     0x00008133   |      33075   |
| OVTK_StimulationId_Label_34                           |     0x00008134   |      33076   |
| OVTK_StimulationId_Label_35                           |     0x00008135   |      33077   |
| OVTK_StimulationId_Label_36                           |     0x00008136   |      33078   |
| OVTK_StimulationId_Label_37                           |     0x00008137   |      33079   |
| OVTK_StimulationId_Label_38                           |     0x00008138   |      33080   |
| OVTK_StimulationId_Label_39                           |     0x00008139   |      33081   |
| OVTK_StimulationId_Label_3A                           |     0x0000813a   |      33082   |
| OVTK_StimulationId_Label_3B                           |     0x0000813b   |      33083   |
| OVTK_StimulationId_Label_3C                           |     0x0000813c   |      33084   |
| OVTK_StimulationId_Label_3D                           |     0x0000813d   |      33085   |
| OVTK_StimulationId_Label_3E                           |     0x0000813e   |      33086   |
| OVTK_StimulationId_Label_3F                           |     0x0000813f   |      33087   |
| OVTK_StimulationId_Label_40                           |     0x00008140   |      33088   |
| OVTK_StimulationId_Label_41                           |     0x00008141   |      33089   |
| OVTK_StimulationId_Label_42                           |     0x00008142   |      33090   |
| OVTK_StimulationId_Label_43                           |     0x00008143   |      33091   |
| OVTK_StimulationId_Label_44                           |     0x00008144   |      33092   |
| OVTK_StimulationId_Label_45                           |     0x00008145   |      33093   |
| OVTK_StimulationId_Label_46                           |     0x00008146   |      33094   |
| OVTK_StimulationId_Label_47                           |     0x00008147   |      33095   |
| OVTK_StimulationId_Label_48                           |     0x00008148   |      33096   |
| OVTK_StimulationId_Label_49                           |     0x00008149   |      33097   |
| OVTK_StimulationId_Label_4A                           |     0x0000814a   |      33098   |
| OVTK_StimulationId_Label_4B                           |     0x0000814b   |      33099   |
| OVTK_StimulationId_Label_4C                           |     0x0000814c   |      33100   |
| OVTK_StimulationId_Label_4D                           |     0x0000814d   |      33101   |
| OVTK_StimulationId_Label_4E                           |     0x0000814e   |      33102   |
| OVTK_StimulationId_Label_4F                           |     0x0000814f   |      33103   |
| OVTK_StimulationId_Label_50                           |     0x00008150   |      33104   |
| OVTK_StimulationId_Label_51                           |     0x00008151   |      33105   |
| OVTK_StimulationId_Label_52                           |     0x00008152   |      33106   |
| OVTK_StimulationId_Label_53                           |     0x00008153   |      33107   |
| OVTK_StimulationId_Label_54                           |     0x00008154   |      33108   |
| OVTK_StimulationId_Label_55                           |     0x00008155   |      33109   |
| OVTK_StimulationId_Label_56                           |     0x00008156   |      33110   |
| OVTK_StimulationId_Label_57                           |     0x00008157   |      33111   |
| OVTK_StimulationId_Label_58                           |     0x00008158   |      33112   |
| OVTK_StimulationId_Label_59                           |     0x00008159   |      33113   |
| OVTK_StimulationId_Label_5A                           |     0x0000815a   |      33114   |
| OVTK_StimulationId_Label_5B                           |     0x0000815b   |      33115   |
| OVTK_StimulationId_Label_5C                           |     0x0000815c   |      33116   |
| OVTK_StimulationId_Label_5D                           |     0x0000815d   |      33117   |
| OVTK_StimulationId_Label_5E                           |     0x0000815e   |      33118   |
| OVTK_StimulationId_Label_5F                           |     0x0000815f   |      33119   |
| OVTK_StimulationId_Label_60                           |     0x00008160   |      33120   |
| OVTK_StimulationId_Label_61                           |     0x00008161   |      33121   |
| OVTK_StimulationId_Label_62                           |     0x00008162   |      33122   |
| OVTK_StimulationId_Label_63                           |     0x00008163   |      33123   |
| OVTK_StimulationId_Label_64                           |     0x00008164   |      33124   |
| OVTK_StimulationId_Label_65                           |     0x00008165   |      33125   |
| OVTK_StimulationId_Label_66                           |     0x00008166   |      33126   |
| OVTK_StimulationId_Label_67                           |     0x00008167   |      33127   |
| OVTK_StimulationId_Label_68                           |     0x00008168   |      33128   |
| OVTK_StimulationId_Label_69                           |     0x00008169   |      33129   |
| OVTK_StimulationId_Label_6A                           |     0x0000816a   |      33130   |
| OVTK_StimulationId_Label_6B                           |     0x0000816b   |      33131   |
| OVTK_StimulationId_Label_6C                           |     0x0000816c   |      33132   |
| OVTK_StimulationId_Label_6D                           |     0x0000816d   |      33133   |
| OVTK_StimulationId_Label_6E                           |     0x0000816e   |      33134   |
| OVTK_StimulationId_Label_6F                           |     0x0000816f   |      33135   |
| OVTK_StimulationId_Label_70                           |     0x00008170   |      33136   |
| OVTK_StimulationId_Label_71                           |     0x00008171   |      33137   |
| OVTK_StimulationId_Label_72                           |     0x00008172   |      33138   |
| OVTK_StimulationId_Label_73                           |     0x00008173   |      33139   |
| OVTK_StimulationId_Label_74                           |     0x00008174   |      33140   |
| OVTK_StimulationId_Label_75                           |     0x00008175   |      33141   |
| OVTK_StimulationId_Label_76                           |     0x00008176   |      33142   |
| OVTK_StimulationId_Label_77                           |     0x00008177   |      33143   |
| OVTK_StimulationId_Label_78                           |     0x00008178   |      33144   |
| OVTK_StimulationId_Label_79                           |     0x00008179   |      33145   |
| OVTK_StimulationId_Label_7A                           |     0x0000817a   |      33146   |
| OVTK_StimulationId_Label_7B                           |     0x0000817b   |      33147   |
| OVTK_StimulationId_Label_7C                           |     0x0000817c   |      33148   |
| OVTK_StimulationId_Label_7D                           |     0x0000817d   |      33149   |
| OVTK_StimulationId_Label_7E                           |     0x0000817e   |      33150   |
| OVTK_StimulationId_Label_7F                           |     0x0000817f   |      33151   |
| OVTK_StimulationId_Label_80                           |     0x00008180   |      33152   |
| OVTK_StimulationId_Label_81                           |     0x00008181   |      33153   |
| OVTK_StimulationId_Label_82                           |     0x00008182   |      33154   |
| OVTK_StimulationId_Label_83                           |     0x00008183   |      33155   |
| OVTK_StimulationId_Label_84                           |     0x00008184   |      33156   |
| OVTK_StimulationId_Label_85                           |     0x00008185   |      33157   |
| OVTK_StimulationId_Label_86                           |     0x00008186   |      33158   |
| OVTK_StimulationId_Label_87                           |     0x00008187   |      33159   |
| OVTK_StimulationId_Label_88                           |     0x00008188   |      33160   |
| OVTK_StimulationId_Label_89                           |     0x00008189   |      33161   |
| OVTK_StimulationId_Label_8A                           |     0x0000818a   |      33162   |
| OVTK_StimulationId_Label_8B                           |     0x0000818b   |      33163   |
| OVTK_StimulationId_Label_8C                           |     0x0000818c   |      33164   |
| OVTK_StimulationId_Label_8D                           |     0x0000818d   |      33165   |
| OVTK_StimulationId_Label_8E                           |     0x0000818e   |      33166   |
| OVTK_StimulationId_Label_8F                           |     0x0000818f   |      33167   |
| OVTK_StimulationId_Label_90                           |     0x00008190   |      33168   |
| OVTK_StimulationId_Label_91                           |     0x00008191   |      33169   |
| OVTK_StimulationId_Label_92                           |     0x00008192   |      33170   |
| OVTK_StimulationId_Label_93                           |     0x00008193   |      33171   |
| OVTK_StimulationId_Label_94                           |     0x00008194   |      33172   |
| OVTK_StimulationId_Label_95                           |     0x00008195   |      33173   |
| OVTK_StimulationId_Label_96                           |     0x00008196   |      33174   |
| OVTK_StimulationId_Label_97                           |     0x00008197   |      33175   |
| OVTK_StimulationId_Label_98                           |     0x00008198   |      33176   |
| OVTK_StimulationId_Label_99                           |     0x00008199   |      33177   |
| OVTK_StimulationId_Label_9A                           |     0x0000819a   |      33178   |
| OVTK_StimulationId_Label_9B                           |     0x0000819b   |      33179   |
| OVTK_StimulationId_Label_9C                           |     0x0000819c   |      33180   |
| OVTK_StimulationId_Label_9D                           |     0x0000819d   |      33181   |
| OVTK_StimulationId_Label_9E                           |     0x0000819e   |      33182   |
| OVTK_StimulationId_Label_9F                           |     0x0000819f   |      33183   |
| OVTK_StimulationId_Label_A0                           |     0x000081a0   |      33184   |
| OVTK_StimulationId_Label_A1                           |     0x000081a1   |      33185   |
| OVTK_StimulationId_Label_A2                           |     0x000081a2   |      33186   |
| OVTK_StimulationId_Label_A3                           |     0x000081a3   |      33187   |
| OVTK_StimulationId_Label_A4                           |     0x000081a4   |      33188   |
| OVTK_StimulationId_Label_A5                           |     0x000081a5   |      33189   |
| OVTK_StimulationId_Label_A6                           |     0x000081a6   |      33190   |
| OVTK_StimulationId_Label_A7                           |     0x000081a7   |      33191   |
| OVTK_StimulationId_Label_A8                           |     0x000081a8   |      33192   |
| OVTK_StimulationId_Label_A9                           |     0x000081a9   |      33193   |
| OVTK_StimulationId_Label_AA                           |     0x000081aa   |      33194   |
| OVTK_StimulationId_Label_AB                           |     0x000081ab   |      33195   |
| OVTK_StimulationId_Label_AC                           |     0x000081ac   |      33196   |
| OVTK_StimulationId_Label_AD                           |     0x000081ad   |      33197   |
| OVTK_StimulationId_Label_AE                           |     0x000081ae   |      33198   |
| OVTK_StimulationId_Label_AF                           |     0x000081af   |      33199   |
| OVTK_StimulationId_Label_B0                           |     0x000081b0   |      33200   |
| OVTK_StimulationId_Label_B1                           |     0x000081b1   |      33201   |
| OVTK_StimulationId_Label_B2                           |     0x000081b2   |      33202   |
| OVTK_StimulationId_Label_B3                           |     0x000081b3   |      33203   |
| OVTK_StimulationId_Label_B4                           |     0x000081b4   |      33204   |
| OVTK_StimulationId_Label_B5                           |     0x000081b5   |      33205   |
| OVTK_StimulationId_Label_B6                           |     0x000081b6   |      33206   |
| OVTK_StimulationId_Label_B7                           |     0x000081b7   |      33207   |
| OVTK_StimulationId_Label_B8                           |     0x000081b8   |      33208   |
| OVTK_StimulationId_Label_B9                           |     0x000081b9   |      33209   |
| OVTK_StimulationId_Label_BA                           |     0x000081ba   |      33210   |
| OVTK_StimulationId_Label_BB                           |     0x000081bb   |      33211   |
| OVTK_StimulationId_Label_BC                           |     0x000081bc   |      33212   |
| OVTK_StimulationId_Label_BD                           |     0x000081bd   |      33213   |
| OVTK_StimulationId_Label_BE                           |     0x000081be   |      33214   |
| OVTK_StimulationId_Label_BF                           |     0x000081bf   |      33215   |
| OVTK_StimulationId_Label_C0                           |     0x000081c0   |      33216   |
| OVTK_StimulationId_Label_C1                           |     0x000081c1   |      33217   |
| OVTK_StimulationId_Label_C2                           |     0x000081c2   |      33218   |
| OVTK_StimulationId_Label_C3                           |     0x000081c3   |      33219   |
| OVTK_StimulationId_Label_C4                           |     0x000081c4   |      33220   |
| OVTK_StimulationId_Label_C5                           |     0x000081c5   |      33221   |
| OVTK_StimulationId_Label_C6                           |     0x000081c6   |      33222   |
| OVTK_StimulationId_Label_C7                           |     0x000081c7   |      33223   |
| OVTK_StimulationId_Label_C8                           |     0x000081c8   |      33224   |
| OVTK_StimulationId_Label_C9                           |     0x000081c9   |      33225   |
| OVTK_StimulationId_Label_CA                           |     0x000081ca   |      33226   |
| OVTK_StimulationId_Label_CB                           |     0x000081cb   |      33227   |
| OVTK_StimulationId_Label_CC                           |     0x000081cc   |      33228   |
| OVTK_StimulationId_Label_CD                           |     0x000081cd   |      33229   |
| OVTK_StimulationId_Label_CE                           |     0x000081ce   |      33230   |
| OVTK_StimulationId_Label_CF                           |     0x000081cf   |      33231   |
| OVTK_StimulationId_Label_D0                           |     0x000081d0   |      33232   |
| OVTK_StimulationId_Label_D1                           |     0x000081d1   |      33233   |
| OVTK_StimulationId_Label_D2                           |     0x000081d2   |      33234   |
| OVTK_StimulationId_Label_D3                           |     0x000081d3   |      33235   |
| OVTK_StimulationId_Label_D4                           |     0x000081d4   |      33236   |
| OVTK_StimulationId_Label_D5                           |     0x000081d5   |      33237   |
| OVTK_StimulationId_Label_D6                           |     0x000081d6   |      33238   |
| OVTK_StimulationId_Label_D7                           |     0x000081d7   |      33239   |
| OVTK_StimulationId_Label_D8                           |     0x000081d8   |      33240   |
| OVTK_StimulationId_Label_D9                           |     0x000081d9   |      33241   |
| OVTK_StimulationId_Label_DA                           |     0x000081da   |      33242   |
| OVTK_StimulationId_Label_DB                           |     0x000081db   |      33243   |
| OVTK_StimulationId_Label_DC                           |     0x000081dc   |      33244   |
| OVTK_StimulationId_Label_DD                           |     0x000081dd   |      33245   |
| OVTK_StimulationId_Label_DE                           |     0x000081de   |      33246   |
| OVTK_StimulationId_Label_DF                           |     0x000081df   |      33247   |
| OVTK_StimulationId_Label_E0                           |     0x000081e0   |      33248   |
| OVTK_StimulationId_Label_E1                           |     0x000081e1   |      33249   |
| OVTK_StimulationId_Label_E2                           |     0x000081e2   |      33250   |
| OVTK_StimulationId_Label_E3                           |     0x000081e3   |      33251   |
| OVTK_StimulationId_Label_E4                           |     0x000081e4   |      33252   |
| OVTK_StimulationId_Label_E5                           |     0x000081e5   |      33253   |
| OVTK_StimulationId_Label_E6                           |     0x000081e6   |      33254   |
| OVTK_StimulationId_Label_E7                           |     0x000081e7   |      33255   |
| OVTK_StimulationId_Label_E8                           |     0x000081e8   |      33256   |
| OVTK_StimulationId_Label_E9                           |     0x000081e9   |      33257   |
| OVTK_StimulationId_Label_EA                           |     0x000081ea   |      33258   |
| OVTK_StimulationId_Label_EB                           |     0x000081eb   |      33259   |
| OVTK_StimulationId_Label_EC                           |     0x000081ec   |      33260   |
| OVTK_StimulationId_Label_ED                           |     0x000081ed   |      33261   |
| OVTK_StimulationId_Label_EE                           |     0x000081ee   |      33262   |
| OVTK_StimulationId_Label_EF                           |     0x000081ef   |      33263   |
| OVTK_StimulationId_Label_F0                           |     0x000081f0   |      33264   |
| OVTK_StimulationId_Label_F1                           |     0x000081f1   |      33265   |
| OVTK_StimulationId_Label_F2                           |     0x000081f2   |      33266   |
| OVTK_StimulationId_Label_F3                           |     0x000081f3   |      33267   |
| OVTK_StimulationId_Label_F4                           |     0x000081f4   |      33268   |
| OVTK_StimulationId_Label_F5                           |     0x000081f5   |      33269   |
| OVTK_StimulationId_Label_F6                           |     0x000081f6   |      33270   |
| OVTK_StimulationId_Label_F7                           |     0x000081f7   |      33271   |
| OVTK_StimulationId_Label_F8                           |     0x000081f8   |      33272   |
| OVTK_StimulationId_Label_F9                           |     0x000081f9   |      33273   |
| OVTK_StimulationId_Label_FA                           |     0x000081fa   |      33274   |
| OVTK_StimulationId_Label_FB                           |     0x000081fb   |      33275   |
| OVTK_StimulationId_Label_FC                           |     0x000081fc   |      33276   |
| OVTK_StimulationId_Label_FD                           |     0x000081fd   |      33277   |
| OVTK_StimulationId_Label_FE                           |     0x000081fe   |      33278   |
| OVTK_StimulationId_Label_FF                           |     0x000081ff   |      33279   |
| OVTK_StimulationId_NoArtifact                         |     0x00008301   |      33537   |
| OVTK_StimulationId_NonTarget                          |     0x00008206   |      33286   |
| OVTK_StimulationId_Number_00                          |     0x00000000   |          0   |
| OVTK_StimulationId_Number_01                          |     0x00000001   |          1   |
| OVTK_StimulationId_Number_02                          |     0x00000002   |          2   |
| OVTK_StimulationId_Number_03                          |     0x00000003   |          3   |
| OVTK_StimulationId_Number_04                          |     0x00000004   |          4   |
| OVTK_StimulationId_Number_05                          |     0x00000005   |          5   |
| OVTK_StimulationId_Number_06                          |     0x00000006   |          6   |
| OVTK_StimulationId_Number_07                          |     0x00000007   |          7   |
| OVTK_StimulationId_Number_08                          |     0x00000008   |          8   |
| OVTK_StimulationId_Number_09                          |     0x00000009   |          9   |
| OVTK_StimulationId_Number_0A                          |     0x0000000a   |         10   |
| OVTK_StimulationId_Number_0B                          |     0x0000000b   |         11   |
| OVTK_StimulationId_Number_0C                          |     0x0000000c   |         12   |
| OVTK_StimulationId_Number_0D                          |     0x0000000d   |         13   |
| OVTK_StimulationId_Number_0E                          |     0x0000000e   |         14   |
| OVTK_StimulationId_Number_0F                          |     0x0000000f   |         15   |
| OVTK_StimulationId_Number_10                          |     0x00000010   |         16   |
| OVTK_StimulationId_Number_11                          |     0x00000011   |         17   |
| OVTK_StimulationId_Number_12                          |     0x00000012   |         18   |
| OVTK_StimulationId_Number_13                          |     0x00000013   |         19   |
| OVTK_StimulationId_Number_14                          |     0x00000014   |         20   |
| OVTK_StimulationId_Number_15                          |     0x00000015   |         21   |
| OVTK_StimulationId_Number_16                          |     0x00000016   |         22   |
| OVTK_StimulationId_Number_17                          |     0x00000017   |         23   |
| OVTK_StimulationId_Number_18                          |     0x00000018   |         24   |
| OVTK_StimulationId_Number_19                          |     0x00000019   |         25   |
| OVTK_StimulationId_Number_1A                          |     0x0000001a   |         26   |
| OVTK_StimulationId_Number_1B                          |     0x0000001b   |         27   |
| OVTK_StimulationId_Number_1C                          |     0x0000001c   |         28   |
| OVTK_StimulationId_Number_1D                          |     0x0000001d   |         29   |
| OVTK_StimulationId_Number_1E                          |     0x0000001e   |         30   |
| OVTK_StimulationId_Number_1F                          |     0x0000001f   |         31   |
| OVTK_StimulationId_RemovedSamples                     |     0x00008310   |      33552   |
| OVTK_StimulationId_Reset                              |     0x00008208   |      33288   |
| OVTK_StimulationId_RestStart                          |     0x00008009   |      32777   |
| OVTK_StimulationId_RestStop                           |     0x0000800a   |      32778   |
| OVTK_StimulationId_SegmentStart                       |     0x00008003   |      32771   |
| OVTK_StimulationId_SegmentStop                        |     0x00008004   |      32772   |
| OVTK_StimulationId_Target                             |     0x00008205   |      33285   |
| OVTK_StimulationId_ThresholdPassed_Negative           |     0x00008210   |      33296   |
| OVTK_StimulationId_ThresholdPassed_Positive           |     0x00008209   |      33289   |
| OVTK_StimulationId_Train                              |     0x00008201   |      33281   |
| OVTK_StimulationId_TrainCompleted                     |     0x00008207   |      33287   |
| OVTK_StimulationId_TrialStart                         |     0x00008005   |      32773   |
| OVTK_StimulationId_TrialStop                          |     0x00008006   |      32774   |
| OVTK_StimulationId_VisualSteadyStateStimulationStart  |     0x00008010   |      32784   |
| OVTK_StimulationId_VisualSteadyStateStimulationStop   |     0x00008011   |      32785   |
| OVTK_StimulationId_VisualStimulationStart             |     0x0000800b   |      32779   |
| OVTK_StimulationId_VisualStimulationStop              |     0x0000800c   |      32780   |

# Authoring Scenarios {#designer-authoring-scenarios}

\page DesignerAuthoringScenarios Authoring Scenarios

This section covers the working area of Designer, which is where OpenViBE scenarios are assembled by connecting box algorithms together.

## Boxes

Box algorithms are added to the active scenario by drag and dropping them from the tree view to the scenario edition window. They appear as rounded rectangles with their name inside the box, inputs (if any) on top and outputs (if any) at the bottom. These connectors are displayed as color-coded triangles. Colors vary with the connector type, and help users to make sure they connect boxes properly. See *Connector Types* for an overview of the different connector types and hierarchy.

![Generic Stream Reader Box](images/box.png)  
A file reading box algorithm which has 2 outputs (signal and stimulations streams)

Additionally, if the box has **customizable settings**, they may be set by double clicking on the box (*tip*: when a box has such settings, its name is displayed in bold). The settings are listed in the popup dialog that appears, along with their default values. Settings may be overridden by directly typing in their desired values, or they may be read from a file. In the latter case, one should expand the *Override settings with configuration file* section to check the *File* button and pick a configuration file.

![Box Settings](images/box-settings.png)  
Settings of a 'Generic stream reader' box: filename text entry

### Linking Boxes

The output of a box can be linked to the input of another box if the type of the output is the same type or is a derived type of the input's type. For example, a Signal output can be connected on a Streamed Matrix input as Signal's type derives from Streamed Matrix.

The various stream types are:

- [▼]{.color-stream-unidentified} Unidentified Stream
- [▼]{.color-stream-if-else} If/Else Condition Controller
- [▼]{.color-stream-ebml} EBML Stream
  - [▼]{.color-stream-experiment-information} Experiment Information
  - [▼]{.color-stream-stimulations} Stimulations
  - [▼]{.color-stream-streamed-matrix} Streamed Matrix
  - [▼]{.color-stream-covariance-matrix} Covariance Matrix
  - [▼]{.color-stream-channel-localization} Channel Localization
  - [▼]{.color-stream-feature-vector} Feature Vector
  - [▼]{.color-stream-signal} Signal
  - [▼]{.color-stream-spectrum} Spectrum
  - [▼]{.color-stream-time-frequency} Time/Frequency

There are different connector types differentiated by their color, see figure above. The type "Unidentified stream" is applied on a connector type that couldn't be identified in the launched version of Designer. Some boxes that can have different connector types may also have this type of connectors by default. The type *If / Else Condition Controller* is a special type that is useful in the boxes: *If / Else Separation* and *End If / Else Concatenation*. See the documentation of these boxes for more details.

### Box Status

By default, the status of a box is 'normal', and the box is drawn with a white background. However, some boxes may have a different status depending on the situation. Here are the other possible status of a box:

#### **Update**

![Update](images/box-status-update.png)

When a box is not up to date with the latest version used in a given distribution of OpenViBE, it is drawn as this "Matrix to Signal" box. This situation arises when the prototype of a box has changed between the time the scenario it is stored in is saved and the time it is loaded again. Since scenario files contain information about box prototypes, such as the number of connectors, they may need to be updated when the OpenViBE distribution used to manipulate scenarios changes. To update such a box, simply delete the box from the scenario then add it again. The new box will use the latest version.

Tip: after such an update, one should make sure to reconfigure the box settings if needed!

#### **Deprecated**

![Deprecated](images/box-status-deprecated.png)

As OpenViBE evolves with time, some boxes are added to the platform and others are deleted. It can also happen that a box is replaced with another one (maybe for performance reasons). However, the 'old' box is not necessarily deleted from the platform, but may be kept for backward compatibility with older versions (ensuring older scenarios may still be run, for example). In that case, the documentation should mention that from that time on, the new box should be preferred over the deprecated one (which may be removed from the platform at any time, and in any case which probably won't be maintained anymore). Such boxes are displayed with an orange background and the 'deprecated' label below the box name.

#### **Unstable**

![Unstable](images/box-status-unstable.png)

A box which is under development should be flagged as 'unstable', meaning it may have only been partially implemented or tested. Consequently, it may not behave properly in all conditions, or may be updated in a future release. Such a box is drawn with a dark grey background in Designer, and the 'unstable' label appears below the box name.

#### **Missing**

![Missing](images/box-status-missing.png)

A box that was described in a scenario but is not present in the opened version of Designer is considered as missing and its background is drawn in red color. This means that the launched version of Designer cannot find the information regarding this box and thus cannot play it.
Note that the default tree view shows only stable and fully supported boxes. To use "unstable" boxes set Designer_ShowUnstable to TRUE or tick the corresponding box on top of the tree view. As stated in their documentation pages, unstable boxes can have unexpected behaviors.

### Box Manipulation

All box algorithms may be configured in Designer. However, not all boxes offer the same configuration options. In this overview, we'll focus on functionalities that are common to all boxes. More advanced possibilities are detailed in another tutorial (Tutorial 3: Advanced box configuration).

To illustrate box editing functionalities, let's start by creating a new scenario, and drop a couple of boxes from the box algorithms tab. Simple boxes such as *Time Signal* and (found under the 'Data Generation' category) will do for this tutorial. Drag and drop these boxes in the scenario working area, then right-click on the *Time Signal* box. A context menu should appear, listing different editing functionalities.

Box editing functionalities appear in the lower part of the menu:

- **Rename box (F2)** allows to rename the box. Click this option and enter a new name such as 'Dummy Box' to test this functionality.
- **Delete box (Del)** removes this box from the scenario. Note that a box may also be deleted by selecting it, then pressing the 'Delete' key.
- **About box** displays a dialog containing a summary of the box details, such as its author, version, the version of Designer in which the box was added and last updated, the class name, as well as a short and long description of its purpose. Note that the 'short description' field also appears in the second column of the box algorithms tab.

Another option only gets listed for those boxes that offer configurable settings, such as the *Time Signal* box (this is indicated from its name displayed in bold case). Right click on this box and select:

- **Configure box**, which displays the Box Settings dialog. Note that this dialog may also be displayed by double clicking on the box itself.

## Scenario Manipulation

Standard cut/copy/paste functionalities are supported in Designer. Select a box and right click on it to display a menu from which to select these options.

Groups of boxes may be edited in the same way. Select several boxes at once by maintaining the **Ctrl** key pressed then clicking on the boxes you want to cut or copy. You may also draw a selection area by left clicking in the scenario editing area then drawing a selection rectangle while keeping the button pressed. **Ctrl+A** selects everything in the scenario.

Cut/copy your selection by right clicking on it then selecting the corresponding entry, or by pressing **Ctrl+X/Ctrl+C**.

Paste your selection by right clicking anywhere in the edition area, or pressing **Ctrl+V**. Note that you may also paste selections from one scenario to another.

Delete your selection by selecting 'Delete boxes' in the contextual menu, or pressing the **Delete** key.

Finally, the origin of the scenario editing area may be changed by pressing and holding the Shift key and left click then moving the mouse as desired. This allows you to explore your scenario window without using the scrolling bars.

## Scenario Settings

Scenarios, much like boxes, can have their own settings. To access the settings of a scenario click on the **Scenario Configuration** tab in the right pane.

![Scenario Settings](images/scenario-settings.png)  
Scenario Configuration Pane

### Adding Settings

To add a setting to the current scenario, press the **Configure Settings** button. A new window will appear. In it you are able to modify all settings the currently open scenario has.

Adding a setting is similar to adding a setting to a box. You can use all available types as well.

Once you add a setting it will appear in the list. There are several actions you can do on a setting.

![Scenario Settings Configuration Dialog](images/scenario-settings-annotated.png)  
Scenario Settings Configuration Dialog

1. Rename setting
2. Delete setting
3. Move setting up
4. Move setting down
5. Change default setting's value
6. Change setting's type

When you close the setting configuration dialog box, your new settings will appear in the Scenario Configuration pane.

### Modifying Settings

You can change the setting’s value inside the tab **Scenario Configuration**.

![Scenario Configuration with Settings](images/scenario-configuration-annotated.png)  
Scenario Configuration with Settings

1. Modify the settings value
2. Reset setting to its default value
3. Copy the current setting identifier wrapped in `$var{}` (explained in next step)

### Using Settings

Scenario Settings can be used in place of any setting of a box. In order to do so, right-click on the box, then click on configure box option and simply change the setting inside that box to a token named after the Scenario Setting wrapped inside a `$var{}` token.

In our example for DSP:

![Box Settings with Variables](images/box-settings-variables.png)  
Using scenario settings

In order to simplify the process there is a convenience button that copies the whole string for a particular setting (Button labelled 3 in previous paragraph).

# Configuration Manager {#designer-configuration-manager}

\page DesignerConfiguration Configuration

In this section, we review a component common to the whole OpenViBE software platform: the **configuration manager**. We start with this component as we will mention several times in this document how specific behavior can be changed according to this configuration manager.

The OpenViBE Configuration Manager is a software component in charge of configuring OpenViBE applications and modules according to the user's wishes. Configuration settings are saved in a configuration file, which is loaded at application startup. This file uses a simple syntax, where configuration tokens are listed and given a value. You can check all the configuration tokens in Designer (Menu Edit/Preferences).

## Configuration Files

OpenViBE comes with **a default configuration file** (`share/openvibe/kernel/openvibe.conf`), automatically loaded at startup. This file lists configuration tokens used by the applications.

Users can edit **a personal configuration file** in order to customize their OpenViBE software platform, by overwriting existing token or declaring new ones. In order to do so, create a file named `openvibe-workspace.conf` in the folder next to the scenarios you are running. This file will impact all scenarios next to it.

## Syntax

A configuration file basically looks like successive `token = value` statements.

For example, the application can retrieve its root path (OpenViBE installation path) from the Path_Root variable. The path is expressed relative to the execution directory bin, so the root path is simply:

```ini
  Path_Root = ..
```

Leading and tailing spaces are allowed, and are removed automatically:

```ini
   Path_Root =       ..
```

Comments may be stored on their own lines. They begin after the `#` character:

```ini
   #this is a valid comment
   Path_Root = ..
```

Lines ending with a `\` character continue to the next line, the last `\` character of a line is automatically removed by the parser. Note that `\` is used as an escape character. To write a path, you should use `/`:

```ini
   #this is a line that extends\
    to the next line
   Path_Root = ..
```

Tokens declared anywhere in the configuration file may be used as values for other tokens. The syntax to be used is: `${token}`
For example, the binaries path may be declared as such:

```ini
   Path_Root = ..
   Path_Bin = ${Path_Root}/bin
```

Configuration files can include other configuration files thanks to a simple syntax:

```ini
   Include = path/to/my/config/file.conf
```

All tokens are read sequentially, and thus can be overwritten during the process.

## Existing Tokens

Several tokens such as *Path_Root* and *Path_Bin* are defined in the default configuration file, and can be overwritten in user-defined configuration files. The token *Player_ScenarioDirectory* is useful when designing a scenario. This token is changed as the path of the scenario when playing it. This is useful to load a file in the same folder as your current scenario. To have the complete list of these tokens, and their values, you can open the window **Preferences**.

## Defining and Using Custom Tokens

Defining your own token can be very useful as you can use them afterwards everywhere in OpenViBE. For example, you can define a token `LowPassFrequency = 40`. You can then use it as a setting in several temporal filters (`${LowPassFrequency}`). Changing the token value will automatically change the filtering frequency in every temporal filter at once, which can be very handy: modification is global and switching from one frequency to another is fast and easy.

We will describe a concrete example that illustrates the use of custom configuration tokens, in paragraph "Appendix A: Using Configuration Tokens to Setup an Experiment Environment".

# OpenViBE Documentation

![OpenViBE Banner](Banner.png)

## Preface

Welcome to the Doxygen Documentation for OpenViBE @OPENVIBE_MAJOR_VERSION@.@OPENVIBE_MINOR_VERSION@.@OPENVIBE_PATCH_VERSION@.
  
This document has been automatically generated on Linux. Please note

- The documentation lacks components and classes that are not compiled on Linux.
- Some source files do not have doxygen information and may not be represented here.
- Plugins such as boxes and drivers are not included in the class lists to better illustrate the core part of OpenViBE.  
However, you can see the boxes on the [Box Descriptions](_doc__box_algorithms.html) page.

The tutorials and user manual, as well as build instructions for OpenViBE are located on the [website](http://openvibe.inria.fr/documentation-index)

## Online Manual

- [Documentation Index](http://openvibe.inria.fr/documentation-index)
- [User Documentation](http://openvibe.inria.fr/documentation-index/#User+Documentation)
- [Developer Documentation](http://openvibe.inria.fr/documentation-index/#Developer+Documentation)
- [Video Tutorials](http://openvibe.inria.fr/start)
- [Other versions of this doc](http://openvibe.inria.fr/documentation/)

## Quick Access

- [OpenViBE Home](http://openvibe.inria.fr)
- [Forum](http://openvibe.inria.fr/forum)
- [Code Repository](http://gitlab.inria.fr/openvibe)

## Contact

- [Project Contact](http://openvibe.inria.fr/Contact)

This documentation is licenced under CC-BY-SA  
[![cc-by-sa](cc-by-sa.png)](http://creativecommons.org/licenses/by-sa/3.0/legalcode)

\subpage License  

# Interface Overview {#designer-interface}

\page DesignerInterface Interface

![Interface](images/studio-interface-annotated.png)  
OpenViBE Designer Interface

The following sections are an overview of the main features of Designer tool, and make for a quick and easy introduction to its usage. Here are the topics covered in this overview:

- Top Level **Menu Bar**
- **Toolbar**, which provides immediate access to the most common actions (Scenario File Edition, Window Manager, Active Logs Level, Add Comment Box, Edit Scenario Information, Scenario Control, …)
- **Available Boxes** tree view
  - **Scenario Settings**
  - **Scenario I/O Controls**
- **Status Bar**, which provides performance information when a scenario is being run
- **Message Console** which prints Info, Warning or Error messages
- **Scenario Authoring** window, where boxes may be arranged together in order to fulfill the task at hand.
- Custom Tools

## Menu Bar

Options available from the menu bar are presently restricted to actions involving handling and exiting the application. Scenarios can be saved to MXS (Mensia XML Scenario) or to XML files (as does OpenViBE). Several scenarios may be edited simultaneously, and the active scenario may be changed by clicking onto the corresponding tab at the top of the scenario editor window. Here is a list of options from the menu bar and their shortcuts if available:

The File menu handles standard scenario operations such as creation, saving and closing. Note that the application won't quit as long as at least one scenario is running.

![File Menu](images/menubar-file.png)

- New – **Ctrl+n**: open a new scenario in a new tab:
- Open – **Ctrl+o**: open a file selection dialog to select an existing scenario file
- Recent Scenarios: access recently closed scenarios
- Save – **Ctrl+s**: save current scenario
- Save As: save the current scenario in another file
- Close - **Ctrl+w**: close current scenario tab
- Quit – **Alt+F4**: quit Designer

The Edit menu allows the user to cut/copy/paste selections (in a scenario or across scenarios). These options can also be accessed when right-clicking on a box. User actions are saved during scenario editing and can be retrieved through undo-redo.

![Edit Menu](images/menubar-edit.png)

- Find – **Ctrl+f**: direct to filter box to make a search in the list of boxes
- Undo – **Ctrl+z**: cancel the last edition operation
- Redo – **Ctrl+y**: redo the operation you just cancelled
- Cut – **Ctrl+x**: cut current selection
- Copy – **Ctrl+c**: copy current selection
- Paste – **Ctrl+v**: paste current selection
- Delete – **Del**: delete current selection
- Preferences: this item displays the Configuration Manager dialog. It contains a list of all configuration tokens and their values. This list is read only. To know more about the Configuration Manager, refer to The Configuration Manager documentation paragraph.

The Help menu allows the user to open some various utilities applications.

![Help Menu](images/menubar-help.png)

- About this scenario – allow to add a scenario description see: Adding metadata
- About OpenViBE – display Credits for Designer
- Browse documentation – Open box documentation
- Register License – Open tool to add a license
- Report issue – Open the support website, see: Support
- What's new in version 3.0.x.x of Designer

## Toolbar

![NeuorRT Toolbar](images/toolbar-annotated.png)  
NeuorRT Designer Toolbar

The first section of the tools bar offers direct access to the scenario handling options that you can find in the menu *File*.

Next section contains the Undo and Redo buttons.

After these you have controls for various functionalities of OpenViBE Designer:

## Window Manager

![Window Manager Icon](images/icon-window-manager.png)  

The **Window Manager** button is a toggle button, allowing for displaying and hiding the **window manager**. This tool is displayed in a popup window when the button is pressed. It takes care of arranging visualization boxes in a layout. Such boxes (if any in the current scenario) initially appear under the 'Unaffected display plugins' node of the tree view in the upper left corner. A window containing one tab is also created by default and displayed on the right, as below:

![Window Manager](images/window-manager.png)  
Window manager popup dialog for a scenario with 4 visualization boxes, by default

Users may create any number of windows containing any number of tabs, and then drag and drop visualization boxes onto such tabs in a tree-like structure. If the user chooses to not use the window manager all visualization boxes will be given their own window, displayed when the scenario is being played. For a more in-depth review of the window manager usage, see the Window Manager Tutorial.

![Window Manager with 4 panes](images/window-manager-4-panes.png)  
Window manager popup dialog for a scenario with 4 organized visualization boxes

## Log Levels

![Log Levels Icon](images/icon-log-levels.png)  

The second button displays the log levels dialog (   ). It may be used to configure which log messages should be displayed in the console. It contains 8 levels as shown in the following figure:

![Log Levels](images/log-levels.png)  
Log Levels Dialog

*Warnings* and *Errors* should always be selected. *Information* displays useful messages from signal processing boxes. *Trace* can be used to check step-by-step the processing in the boxes and the kernel. *Benchmark* and *Debug* are very verbose and should not be used with Designer console (only in log files).

## Scenario Comments

![Scenario Comments Icon](images/icon-add-comment.png)

This button adds a dedicated comment box to the scenario. These boxes have of course no input and outputs. Double click on them to edit the comment they display. The syntax of the comments uses `pango style <http://www.gtk.org/api/2.6/pango/PangoMarkupFormat.html>`__, the style buttons can help you format your comment.

![Scenario Comment](images/scenario-comment.png)  
Comment, as it appears in a scenario

![Scenario Comment Editor](images/scenario-comment-editor.png)  
Comment Editor Dialog

## Scenario Description

![Scenario Description Icon](images/icon-scenario-information.png)

This buttons allows editing of some information that will be saved with the scenario (name, authors, date, description, etc.).

![Scenario Description Edit Dialog](images/scenario-information.png)  
Scenario Description

## Scenario Controls

![Scenario Controls Icon](images/icon-scenario-controls.png)

Scenario controls work like a media player would.

- Stop – **F5**: go back to scenario edition mode
- Step – **F6**: play scenario one step of simulation at a time
- Play/Pause – **F7**: play scenario in real time or pause it if it is already running
- Fast Forward – **F8**: play scenario as fast as possible
  
Once a scenario is being played, it is not possible to modify it. Press *Stop* to go back in edition mode. Shortcuts from F5 to F8 can be used only if the focus is on Designer main window (not on visualization windows for example).

Finally, the toolbar contains a time counter which displays the simulation time as a scenario is being run. It is reset to 0 as a scenario is stopped.

## Box Algorithms Tree View

The right-hand part of Designer window displays a list of existing box algorithms, which are the building blocks of OpenViBE scenarios.

Under the *Box algorithms* tab lies a tree view listing available box algorithms, along with a short description of their respective roles. Box algorithms are the smallest granular elements that may be manipulated by a Designer user. They act as black boxes which can be connected together by their inputs and outputs. In order to facilitate their selection, they are grouped into categories, which make up the top level nodes of the tree view. Some categories are related to signal processing, others to scenario serialization or visualization purposes. The default tree view shows only stable and fully supported boxes.

The status of a box varies with the context, and is reflected by the color of the font used in the tree view. The box name and description will use a cyan font if the box is deprecated, and a light grey font if it is unstable. To know more about box status, see section Box status.

## Status Bar

The status bar that lies at the bottom of the window provides performance information. When a scenario is being run, OpenViBE keeps track of the time used by each box and of the overall execution time. Overall performance is displayed in a green gauge overlaid with the same information in percent. A system load 100 percent means the system is barely able to handle the computation load induced by the scenario to run in real-time.

To identify the bottlenecks in a scenario, one can press the button to the right of the gauge as a scenario is being run. This will highlight boxes from green to red, depending on how much time is spent in each box relative to the others.

## Console

Designer console can be expanded/hidden by a simple click. You can choose the log levels you want to see in this particular window. Designer console has been developed in order to be sure to see any warning or error messages.

![OpenViBE Designer Console](images/console-annotated.png)  
OpenViBE Designer Console

# OpenViBE File Format {#file-format}

\page FileFormat OpenViBE File Format

## Scenarios and Metaboxes : `.xml` (`.mxs` and `.mxb` for old mensia files)

Scenarios are used to define the set of consecutive actions to be performed for your *trial*, they are [xml](https://en.wikipedia.org/wiki/XML) files. It looks like this:

```xml
<OpenViBE-Scenario>
	<FormatVersion>2</FormatVersion>		<!-- Version of the xml scenario format unchanged since version 2.0.0 -->
	<Creator>OpenViBE Designer</Creator>	<!-- Currently the only scenario editor -->
	<CreatorVersion>X.X.X</CreatorVersion>	<!-- The version of designer on the last save  -->
	<Settings>...</Settings>				<!-- General settings of scenario -->
	<Inputs>...</Inputs>					<!-- Inputs of scenario used for Metaboxes -->
	<Outputs>...</Outputs>					<!-- Outputs of scenario used for Metaboxes -->
	<Boxes>...</Boxes>						<!-- Boxes on your scenarios -->
	<Links>...</Links>						<!-- Links between boxes on your scenario -->
	<Comments>...</Comments>				<!-- Comments (post-it) on your scenario -->
	<Metadata>...</Metadata>				<!-- Metadata of your scenario is currently only for Display manager -->
	<Attributes>...</Attributes>			<!-- Attributes (author, brief, version...) of your scenario -->
</OpenViBE-Scenario>
```

### Settings Format

The settings appear in the designer's Scenario Configuration tab. They allow the experimenter to create variables that can be used in various boxes (One variable = one `<Setting>` tag). This way, if different boxes share some parameters values, the experiementer can create a variable and use it in all these boxes. Thus, when they want to change a shared parameter, they only have to change the value of the variable instead of changing the parameters of all the boxes one by one. 

```xml
<Setting>
	<Identifier>...</Identifier>			<!-- Random ID -->
	<TypeIdentifier>...</TypeIdentifier>	<!-- Id of setting type (string, filename, stimulation....) -->
	<Name>...</Name>						<!-- Name of your Setting (you can use it with $var{Your-Name} in boxes) -->
	<DefaultValue>...</DefaultValue>		<!-- Obvious -->
	<Value>...</Value>						<!-- Obvious -->
</Setting>
```

### Inputs/Outputs Format

This parts are used to create Inputs/Outputs for your metaboxes.

```xml
<Output>
	<Identifier>...</Identifier>						<!-- Random ID -->
	<TypeIdentifier>...</TypeIdentifier>				<!-- Id of stream type (signal, matrix, stimulations....) -->
	<Name>...</Name>									<!-- Obvious -->
	<LinkedBoxIdentifier>...</LinkedBoxIdentifier>		<!-- ID Of the Box linked -->
	<LinkedBoxOutputIndex>...</LinkedBoxOutputIndex>	<!-- Position of Input/Output in the Box -->
</Output>
```

### Boxes Format

```xml
<Box>
	<Identifier>...</Identifier>								<!-- Random ID -->
	<Name>...</Name>											<!-- Obvious -->
	<AlgorithmClassIdentifier>...</AlgorithmClassIdentifier>	<!-- ID Of Box algorithm -->
	<Inputs>...</Inputs>										<!-- List of Box Inputs with for each Id of stream type (signal, matrix, stimulations....) and name. -->
	<Outputs>...</Outputs>										<!-- List of Box Outputs with for each Id of stream type (signal, matrix, stimulations....) and name. -->
	<Settings>...</Settings>									<!--Same as general settings -->
	<Attributes>...</Attributes>								<!--Attributes are, position x/y, Box flag, Initial hash value (for updates), inputs/outputs/settings number  -->
</Box>
```

### Links Format

```xml
<Link>
	<Identifier>...</Identifier>				<!-- Random ID -->
	<Source>
		<BoxIdentifier>...</BoxIdentifier>		<!-- ID Of the Box linked -->
		<BoxOutputIndex>...</BoxOutputIndex>	<!-- Position of Input/Output in the Box -->
	</Source>
	<Target>
		<BoxIdentifier>...</BoxIdentifier>		<!-- ID Of the Box linked -->
		<BoxInputIndex>...</BoxInputIndex>		<!-- Position of Input/Output in the Box -->
	</Target>
</Link>
```

### Comments Format

```xml
<Comment>
	<Identifier>...</Identifier>	<!-- Random ID -->
	<Text>...</Text>				<!-- Text of your comment -->
	<Attributes>...</Attributes>	<!-- Attributes are, position x/y -->
</Comment>
```

## Config : `.xml` (`.cfg` or any extension)

Config files are used to bypass the settings of a box, they are [xml](https://en.wikipedia.org/wiki/XML) files. They all have a similar format as follows:

```xml
<OpenViBE-SettingsOverride>
	<SettingValue>First Setting Value</SettingValue>
	<SettingValue>Second Setting Value</SettingValue>
	.....
	<SettingValue>Last Setting Value</SettingValue>
</OpenViBE-SettingsOverride>
```

## Electrode Localisation : `.txt`

The electrode localisation file is used to specify the position of the electrodes in a (virtual) space. This is a simple text file as follows:

**For  Cartesian position:**

```text
[
	["Electrode Name 1" "Electrode Name 2" ..... "Last Electrode Name"]
	["x" "y" "z"]
]

[ [ x y1 z1 ] ]
[ [ x2 y2 z2 ] ]
...
[ [ LastX LastY LastZ ] ]
```

See for example [sdk\plugins\processing\file-io\electrode_sets\electrode_set_standard_cartesian.txt](https://gitlab.inria.fr/openvibe/sdk/-/blob/master/plugins/processing/file-io/electrode_sets/electrode_set_standard_cartesian.txt)

**For Spherical position:**

```text
[
	["Electrode Name 1" "Electrode Name 2" ..... "Last Electrode Name"]
	["theta" "phi"]
]

[ [ theta1 phi1 ] ]
[ [ theta2 phi2 ] ]
...
[ [ LastTheta lastPhi ] ]
```

See for example [sdk\plugins\processing\file-io\electrode_sets\electrode_set_standard_spherical.txt](https://gitlab.inria.fr/openvibe/sdk/-/blob/master/plugins/processing/file-io/electrode_sets/electrode_set_standard_spherical.txt)

## Keyboard Stimulation : `.txt`

The Keyboard Stimulation file is used to specify stimulation associated to key pressed and released. We indicate the `Key`, the stimulation when we `press` the key and the stimulation when we `release` the key. This is a simple text file as follows:

```text
a	0x00008101	0x00008100
z	0x00008102	0x00008100
e	0x00008103	0x00008100
r	0x00008104	0x00008100
t	0x00008105	0x00008100
y	0x00008106	0x00008100
```

## Signal Files

### OpenViBE : `.ov`

[OpenViBE](http://openvibe.inria.fr/) has its own file extension, `.ov`, which uses the EBML format. It is a Binay file format, meaning it's not readable by simple text editors.

**Streams** : OV contains any stream you want : Experiment informations, signal, matrix, stimulations.... possibly several stream of the same type.  
**Header** : Header contains definition of each stream.  
**Signal** : Streams are saved after header and are updated on the file at each input. Independant of stream type.

### Comma-Separated Values : `.csv`

[csv](https://en.wikipedia.org/wiki/Comma-separated_values) is the only *Raw text* format for signal, this format is explicit, it's a table with comma (at origin) to separate values. Other separator are possible for example microsoft excel consider `;` as separator.  

**Streams** : CSV contains 1 or 2 streams : Signal stream (optionnal) and stimulations.  
**Header** : Header is columns with time, channels, event (stimulations).  
**Signal** : Signal is saved after header and is update on file at each input.  
**Stimulations** :  Stimulations are saved after Signal on same line. If several stimulations are present they are separated by `:`.

For Openvibe, we have 5 possible definitions for signal files easily identifiable by the number and the name of the columns :

- Signal, example with 4 Channels  
`Time:512Hz,Epoch,C1,C2,C3,C4,Event Id,Event Date,Event Duration`
- Feature, example with 4 Features  
`Time:4,End Time,Feature 1,Feature 2,Feature 3,Feature 4,Event Id,Event Date,Event Duration`
- Spectrum, example with a signal of 128Hz discretized in 3 sections compared to the first 3 (so all here)
`Time:3x3:64,End Time,0:0.000000,0:16.000000,0:32.000000,16:0.000000,16:16.000000,16:32.000000,32:0.000000,32:16.000000,32:32.000000,Event Id,Event Date,Event Duration`
- Matrix, example with a 2x4 matrix (2 channels and 4 sample per channel)  
`Time:2x4,End Time,C1:,C1:,C1:,C1:,C2:,C2:,C2:,C2:,Event Id,Event Date,Event Duration`
- Stimulation  
`Event Id,Event Date,Event Duration`

### General Data Format for Biomedical Signals : `.gdf`

[General Data Format](https://en.wikipedia.org/wiki/General_Data_Format_for_Biomedical_Signals) is a Binay file format so you can't open file with a simple text editor.

**Streams** : GDF contains 3 streams : Experiment informations, signal and stimulations  
**Header** : Header contains Experiment informations, channel names and min/max of this channel  
**Experiment Info** : Experiment Info is saved in the Header.  
**Signal** : Signal is saved after header and is update on file at each input, during this step we update header min/max for each channel  
**Stimulations** :  Stimulations are saved after Signal when the scenario stops normally, so if OpenViBE crashes before that, no stimulation is stored in the file.

### European Data Format : `.edf`

[European Data Format](https://en.wikipedia.org/wiki/European_Data_Format) is a Binay file format so you can't open file with a simple text editor.

EDF is similar to GDF, but the header only contains the Experiment Information. After the header, there are the stimulations (updated at each input). The signal is only saved when the scenario stops normally, so if OpenViBE crashes before that, no signal is stored in the file.

### BrainVision : `.vhdr`, `.eeg`, `.vmrk`

[BrainVision](https://brainvision.com/) has a Binay file format so you can't open file with a simple text editor. It's saved in 3 files : Header in `.vhdr`, signal in `.eeg` and stimulation in `.vmrk`.

**Streams** : BrainVision format contains 2 streams : signal and stimulations.  
**Header** : Header contains channel names.  
**Signal** : Signal is saved without format, each data is saved consecutivly.  
**Stimulations** :  Stimulations are saved one stimulation per lines with Name of stimulation or id (if doesn't exist), time and duration.

### BCI2000 : `.dat`

[BCI2000](https://www.bci2000.org/mediawiki/index.php/Main_Page) has a Binay file format so you can't open file with a simple text editor. OpenViBE can only read this data not write.

**Streams** : BCI2000 format contains 2 streams : signal and "states" equivalent of stimuations.  
**Header** : Header contains only channel number and sampling rate.  
**Signal** : Signal (and state) is saved after header, we must know the sample per buffer previously to have a valid signal. We compute time of signal with sampling rate and number of samples sent.

# OpenViBE Designer {#designer}

\page Designer OpenViBE Designer

The OpenViBE Designer is an authoring tool dedicated to creating and executing Designer scenarios.

− \subpage DesignerIntroduction  
− \subpage DesignerInterface  
− \subpage DesignerAuthoringScenarios  
− \subpage DesignerMetaboxes  
− \subpage DesignerConfiguration

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rd.R
\name{rd}
\alias{rd}
\alias{rd.default}
\alias{rd.meta}
\alias{print.rd.meta}
\alias{print.rd.default}
\title{Calculate the risk difference from meta-analysis results}
\usage{
rd(x, ...)

\method{rd}{meta}(
  x,
  p.c,
  common = x$common,
  random = x$random,
  small.values = "desirable",
  pscale = 1,
  ...
)

\method{rd}{default}(
  x,
  p.c,
  sm,
  lower,
  upper,
  level = gs("level.ma"),
  small.values = "desirable",
  pscale = 1,
  transf = FALSE,
  ...
)

\method{print}{rd.meta}(
  x,
  common = !is.null(x$common),
  random = !is.null(x$random),
  pscale = attributes(x)$pscale,
  digits = gs("digits"),
  digits.prop = gs("digits.prop"),
  big.mark = gs("big.mark"),
  details = gs("details"),
  ...
)

\method{print}{rd.default}(
  x,
  pscale = attributes(x)$pscale,
  digits = gs("digits"),
  digits.prop = gs("digits.prop"),
  big.mark = gs("big.mark"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}, or estimated treatment
effect(s), i.e., risk difference(s), risk ratio(s), odds
ratio(s), or hazard ratio(s).}

\item{\dots}{Additional arguments (ignored)}

\item{p.c}{Baseline probability, i.e., control group event probability
for binary outcomes or survival probability in control group for
hazard ratios.}

\item{common}{A logical indicating whether ARRs / ABIs should be
calculated based on common effect estimate.}

\item{random}{A logical indicating whether ARRs / ABIs should be
calculated based on random effects estimate.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated.}

\item{pscale}{A numeric defining a scaling factor for printing of
absolute risk reduction or absolute benefit increase.}

\item{sm}{Summary measure.}

\item{lower}{Lower confidence interval limit.}

\item{upper}{Upper confidence interval limit.}

\item{level}{The level used to calculate confidence intervals.}

\item{transf}{A logical indicating whether treatment estimates and
confidence limits are transformed or on the original scale.
If \code{transf = TRUE}, inputs are expected to be log odds ratios instead
of odds ratios for \code{sm = "OR"}, for example.}

\item{digits}{Minimal number of significant digits to print ARR / ABI and its
confidence interval, see \code{print.default}.}

\item{digits.prop}{Minimal number of significant digits for
proportions, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{details}{A logical specifying whether details should be printed.}
}
\description{
Calculate the risk difference from the risk ratio, odds ratio, or hazard
ratio, and a baseline probability, i.e., control group event probability for
binary outcomes or survival probability for hazard ratios. Results are
reported as absolute risk reduction/increase or absolute benefit
increase/reduction.
}
\details{
Calculate the risk difference from the risk ratio, odds ratio, or hazard
ratio, and a baseline probability, i.e., control group event probability for
binary outcomes or survival probability for hazard ratios. Report results as
absolute risk reduction (ARR), absolute risk increase (ARI), absolute
benefit increase (ABI), or absolute benefit reduction (ABR).

Absolute measures can be easily computed from an estimated risk
difference (RD), or a risk ratio (RR) or odds ratio (OR) and a given
baseline probability. It is also possible to calculate absolute
measures from hazard ratios (HR) (Altman & Andersen, 1999).
Accordingly, these measures can be calculated for meta-analyses
generated with \code{\link{metabin}} or \code{\link{metagen}}
if argument \code{sm} was equal to \code{"RD"}, \code{"RR"},
\code{"OR"}, or \code{"HR"}. It is also possible to provide only
estimated treatment effects and baseline probabilities (see Examples).

The baseline probability can be specified using argument \code{p.c}. If
this argument is missing, the minimum, mean, and maximum of the
control event probabilities in the meta-analysis are used for
\code{\link{metabin}} and control event probabilities of 0.1, 0.2,
\dots, 0.9 are used for \code{\link{metagen}}. Note, the survival instead of
mortality probability must be provided for hazard ratios.

Argument \code{small.values} can be used to specify whether small
treatment effects indicate a beneficial (\code{"desirable"}) or harmful
(\code{"undesirable"}) effect. For \code{small.values = "desirable"}, OR < 1,
RR < 1, HR < 1, or RD > 0 indicate that the new treatment is beneficial.
In this case, calculated risk differences are expressed as absolute risk
reductions (ARR). If OR > 1, RR > 1, HR > 1, or RD > 0, calculated risk
differences are expressed as absolute risk increases (ARI). For
\code{small.values = "undesirable"}, odds, OR > 1, RR > 1, HR > 1, or RD > 0
indicate that the new treatment is beneficial. In this case, calculated risk
differences are expressed as absolute benefit increases (ABI). If OR < 1,
RR < 1, HR < 1, or RD < 0, calculated risk differences are expressed as
absolute benefit reductions (ABR).

Argument \code{pscale} can be used to rescale ARRs, ..., ABRs, e.g.,
\code{pscale = 1000} means that these quantities are expressed as events
per 1000 observations. This is useful in situations with (very) low
event probabilities.
}
\examples{
# Calculate ARRs for risk differences of -0.12 and -0.22
rd(c(-0.12, -0.22), sm = "RD")

# Calculate ARR for risk ratio of 0.92 and baseline risk of 0.3
rd(0.92, p.c = 0.3, sm = "RR")

# Calculate ARR for odds ratio of 0.73 and baseline risk of 0.3
rd(0.73, p.c = 0.3, sm = "OR")

# Calculate ARRs for Mantel-Haenszel odds ratio
data(Olkin1995)
ma <-
  metabin(ev.exp, n.exp, ev.cont, n.cont, data = Olkin1995, random = FALSE)
rd(ma)

# Calculate ARRs from hazard ratio at two and four years (example from
# Altman & Andersen, 1999). Note, argument 'p.c' must provide survival
# probabilities instead of mortality rates for the control group.
rd(0.72, lower = 0.55, upper = 0.92, sm = "HR", p.c = 1 - c(0.33, 0.49))

}
\references{
Altman DG, Andersen PK (1999):
Calculating the number needed to treat for trials where the outcome
is time to an event.
\emph{British Medical Journal},
\bold{319}, 1492--95
}
\seealso{
\code{\link{metabin}}, \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/or2smd.R
\name{or2smd}
\alias{or2smd}
\title{Conversion from log odds ratio to standardised mean difference}
\usage{
or2smd(
  lnOR,
  selnOR,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  method = "HH",
  ...
)
}
\arguments{
\item{lnOR}{Log odds ratio(s) or meta-analysis object.}

\item{selnOR}{Standard error(s) of log odds ratio(s) (ignored if
argument \code{lnOR} is a meta-analysis object).}

\item{studlab}{An optional vector with study labels (ignored if
argument \code{lnOR} is a meta-analysis object).}

\item{data}{An optional data frame containing the study information
(ignored if argument \code{lnOR} is a meta-analysis object).}

\item{subset}{An optional vector specifying a subset of studies to
be used (ignored if argument \code{lnOR} is a meta-analysis
object).}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots (ignored if argument \code{lnOR} is a meta-analysis
object).}

\item{method}{A character string indicating which method is used to
convert log odds ratios to standardised mean differences. Either
\code{"HH"} or \code{"CS"}, can be abbreviated.}

\item{\dots}{Additional arguments passed on to
\code{\link{metagen}} (ignored if argument \code{lnOR} is a
meta-analysis object).}
}
\value{
An object of class \code{c("metagen", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Conversion from log odds ratio to standardised mean difference
using method by Hasselblad & Hedges (1995) or Cox (1970).
}
\details{
This function implements the following methods for the conversion
from log odds ratios to standardised mean difference:
\itemize{
\item Hasselblad & Hedges (1995) assuming logistic distributions
  (\code{method == "HH"})
\item Cox (1970) and Cox & Snell (1989) assuming normal
  distributions (\code{method == "CS"})
}
Internally, \code{\link{metagen}} is used to conduct a
meta-analysis with the standardised mean difference as summary
measure.

Argument \code{lnOR} can be either a vector of log odds ratios or a
meta-analysis object created with \code{\link{metabin}} or
\code{\link{metagen}} and the odds ratio as summary measure.

Argument \code{selnOR} is mandatory if argument \code{lnOR} is a
vector and ignored otherwise. Additional arguments in \code{\dots}
are only passed on to \code{\link{metagen}} if argument \code{lnOR}
is a vector.
}
\examples{
# Example from Borenstein et al. (2009), Chapter 7
#
ma0 <- or2smd(0.9069, sqrt(0.0676))
# TE = standardised mean difference (SMD); seTE = standard error of SMD
data.frame(SMD = round(ma0$TE, 4), varSMD = round(ma0$seTE^2, 4))

# Use dataset from Fleiss (1993)
#
data(Fleiss1993bin)
ma1 <- metabin(d.asp, n.asp, d.plac, n.plac,
  data = Fleiss1993bin, studlab = paste(study, year),
  sm = "OR", random = FALSE)
or2smd(ma1)

}
\references{
Borenstein M, Hedges LV, Higgins JPT, Rothstein HR (2009):
\emph{Introduction to Meta-Analysis}.
Chichester: Wiley

Cox DR (1970):
\emph{Analysis of Binary Data}.
London: Chapman and Hall / CRC

Cox DR, Snell EJ (1989):
\emph{Analysis of Binary Data} (2nd edition).
London: Chapman and Hall / CRC

Hasselblad V, Hedges LV (1995):
Meta-analysis of screening and diagnostic tests.
\emph{Psychological Bulletin},
\bold{117}, 167--78
}
\seealso{
\code{\link{smd2or}}, \code{\link{metabin}},
  \code{\link{metagen}}, \code{\link{metacont}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labels.meta.R
\name{labels.meta}
\alias{labels.meta}
\title{Create study labels for forest plot}
\usage{
\method{labels}{meta}(
  object,
  author = object$studlab,
  year = "",
  citation = NULL,
  layout = "JAMA",
  data = object$data,
  ...
)
}
\arguments{
\item{object}{An object of class \code{meta}.}

\item{author}{An optional vector providing study authors.}

\item{year}{An optional vector providing year of publication.}

\item{citation}{An optional vector providing citation numbers.}

\item{layout}{A character string specifying layout. Either "JAMA"
or "Lancet".}

\item{data}{An optional data frame containing the study
information.}

\item{\dots}{Additional arguments (ignored at the moment).}
}
\description{
Create study labels for forest plot.
}
\details{
This auxiliary function can be used to create study labels in JAMA
or Lancet layout which can be added to a forest plot using argument
'studlab'.
}
\examples{
data(Fleiss1993bin)

refs <- 20 + 1:7

m <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
 studlab = study, sm = "OR", random = FALSE)

forest(m,
  studlab = labels(m, year = year, citation = refs, layout = "JAMA"),
  layout = "JAMA", fontfamily = "Times", fontsize = 10)

forest(m,
  studlab = labels(m, year = year, citation = refs, layout = "Lancet"))

}
\seealso{
\code{\link{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metacor.R
\name{metacor}
\alias{metacor}
\title{Meta-analysis of correlations}
\usage{
metacor(
  cor,
  n,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  sm = gs("smcor"),
  level = gs("level"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = gs("method.tau"),
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  null.effect = 0,
  method.bias = gs("method.bias"),
  backtransf = gs("backtransf"),
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{cor}{Correlations.}

\item{n}{Number of observations.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information, i.e., cor and n.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{sm}{A character string indicating which summary measure
(\code{"ZCOR"} or \code{"COR"}) is to be used for pooling of
studies.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, or
\code{"Thompson"}, can be abbreviated. See function
\code{\link{metabias}}.}

\item{backtransf}{A logical indicating whether results for Fisher's
z transformed correlations (\code{sm = "ZCOR"}) should be back
transformed in printouts and plots. If TRUE (default), results
will be presented as correlations; otherwise Fisher's z
transformed correlations will be shown.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{adhoc.hakn}{Deprecated argument (replaced by 'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}}.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{c("metacor", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of common effect and random effects estimates for
meta-analyses with correlations; inverse variance weighting is used
for pooling.
}
\details{
This function conducts common effect and random effects meta-analysis of
correlations based either on Fisher's z transformation of correlations
(\code{sm = "ZCOR"}) or direct combination of (untransformed) correlations
(\code{sm = "COR"}) (see Cooper et al., 2009, p264-5 and p273-4). Note, the
input to argument \code{cor} is always correlations and not Fisher's z
transformed correlations if \code{sm = "ZCOR"}.

Only few statisticians would advocate the use of untransformed correlations
unless sample sizes are very large (see Cooper et al., 2009, p265). The
artificial example given below shows that the smallest study gets the largest
weight if correlations are combined directly because the correlation is
closest to 1.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments
\code{common} and \code{random}. Accordingly, the estimate
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"meta"} even if
argument \code{random = FALSE}. However, all functions in R
package \bold{meta} will adequately consider the values for
\code{common} and \code{random}. E.g. functions
\code{\link{print.meta}} and \code{\link{forest.meta}} will not
print results for the random effects model if \code{random =
FALSE}.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\note{
The function \code{\link{metagen}} is called internally to
calculate individual and overall treatment estimates and standard
errors.
}
\examples{
ma1 <- metacor(c(0.85, 0.7, 0.95), c(20, 40, 10))

# Print correlations (back transformed from Fisher's z
# transformation)
#
summary(ma1)

# Print Fisher's z transformed correlations 
#
print(summary(ma1), backtransf = FALSE)

# Forest plot with back transformed correlations
#
forest(ma1)

# Forest plot with Fisher's z transformed correlations
#
forest(ma1, backtransf = FALSE)

ma2 <- update(ma1, sm = "cor")
summary(ma2)

\dontrun{
# Identical forest plots (as back transformation is the identity
# transformation)
forest(ma2)
forest(ma2, backtransf = FALSE)
}

}
\references{
Cooper H, Hedges LV, Valentine JC (2009):
\emph{The Handbook of Research Synthesis and Meta-Analysis},
2nd Edition.
New York: Russell Sage Foundation

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{metacont}}, \code{\link{metagen}},
  \code{\link{print.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.metacum.R
\name{print.metacum}
\alias{print.metacum}
\title{Print results of a cumulative meta-analysis}
\usage{
\method{print}{metacum}(
  x,
  prediction = x$prediction,
  overall = x$overall,
  backtransf = x$backtransf,
  header = TRUE,
  lab.NA = ".",
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.I2 = gs("digits.I2"),
  digits.cid = gs("digits.cid"),
  digits.percent = 1,
  big.mark = gs("big.mark"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  print.stat = FALSE,
  print.tau2 = TRUE,
  print.tau2.ci = FALSE,
  print.tau = TRUE,
  print.tau.ci = FALSE,
  print.I2 = TRUE,
  print.I2.ci = FALSE,
  print.prob = TRUE,
  text.tau2 = gs("text.tau2"),
  text.tau = gs("text.tau"),
  text.I2 = gs("text.I2"),
  details.methods = gs("details"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{metacum}}.}

\item{prediction}{A logical indicating whether prediction intervals should
be printed.}

\item{overall}{A logical indicating whether overall results should be
printed.}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. If \code{backtransf=TRUE}, results
for \code{sm="OR"} are printed as odds ratios rather than log
odds ratios, for example.}

\item{header}{A logical indicating whether information on title of
meta-analysis, comparison and outcome should be printed at the
beginning of the printout.}

\item{lab.NA}{A character string to label missing values.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-value of test for overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance \eqn{\tau^2}, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{digits.I2}{Minimal number of significant digits for I-squared
and Rb statistic, see \code{print.default}.}

\item{digits.cid}{Minimal number of significant digits for
CID / decision thresholds, see \code{print.default}.}

\item{digits.percent}{Minimal number of significant digits for
probabilities, printed as percentages, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{print.stat}{A logical value indicating whether z- or t-value
for test of treatment effect should be printed.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau2.ci}{A logical value indicating whether to print
the confidence interval of \eqn{\tau^2}.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{print.tau.ci}{A logical value indicating whether to print the
confidence interval of \eqn{\tau}.}

\item{print.I2}{A logical specifying whether heterogeneity
statistic I\eqn{^2} should be printed.}

\item{print.I2.ci}{A logical specifying whether confidence interval for
heterogeneity statistic I\eqn{^2} should be printed.}

\item{print.prob}{A logical specifying whether to print probabilities
of clinically important benefit or harm.}

\item{text.tau2}{Text printed to identify between-study variance
\eqn{\tau^2}.}

\item{text.tau}{Text printed to identify \eqn{\tau}, the square
root of the between-study variance \eqn{\tau^2}.}

\item{text.I2}{Text printed to identify heterogeneity statistic
I\eqn{^2}.}

\item{details.methods}{A logical specifying whether details on
statistical methods should be printed.}

\item{\dots}{Additional arguments (ignored).}
}
\description{
Print results of a cumulative meta-analysis
}
\examples{
data(Fleiss1993bin)
ma <- metabin(d.asp, n.asp, d.plac, n.plac,
  data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
ma
metacum(ma)
metacum(ma, pooled = "random")
metacum(ma, pooled = "random", prediction = TRUE)

}
\seealso{
\code{\link{metacum}}, \code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caffeine.R
\docType{data}
\name{caffeine}
\alias{caffeine}
\title{Caffeine for daytime drowsiness}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{year}}\tab year of publication \cr
\bold{\emph{h.caf}}\tab Number of participants with headaches (caffeine group) \cr
\bold{\emph{n.caf}}\tab Number of participants (caffeine group) \cr
\bold{\emph{h.decaf}}\tab Number of participants with headaches (decaf group) \cr
\bold{\emph{n.decaf}}\tab Number of participants (decaf group) \cr
\bold{\emph{D1}}\tab Domain 1 of risk of bias 2 tool (RoB 2) \cr
\bold{\emph{D2}}\tab Domain 2 (RoB 2) \cr
\bold{\emph{D3}}\tab Domain 3 (RoB 2) \cr
\bold{\emph{D4}}\tab Domain 4 (RoB 2) \cr
\bold{\emph{D5}}\tab Domain 5 (RoB 2) \cr
\bold{\emph{rob}}\tab Overall RoB 2 assessment
}
}
\description{
Caffeine for daytime drowsiness (Cochrane Practice review)
}
\details{
Data come from the Cochrane Practice review on caffeine for daytime
drowsiness. Eight fictitous studies evaluate the risk of headaches
after drinking either caffeinated or decaffeinated coffee.
}
\examples{
oldset <- settings.meta("RevMan5")

data(caffeine)
head(caffeine)

ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
  data = caffeine, studlab = paste(study, year))

# Add risk of bias assessment to meta-analysis
ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")

# Print risk of bias assessment
rob(ma2)

# Forest plot with risk of bias assessment
forest(ma2)

settings.meta(oldset)
}
\references{
Higgins JPT, Savović J, Page MJ, Sterne JA on behalf of the RoB2
Development Group (2019):
Revised Cochrane risk-of-bias tool for randomized trials.
\url{https://www.riskofbias.info/welcome/rob-2-0-tool}
}
\seealso{
\code{\link{metabin}}, \code{\link{rob}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.metainf.R
\name{forest.metainf}
\alias{forest.metainf}
\title{Forest plot to display the result of a leave-one-out meta-analysis}
\usage{
\method{forest}{metainf}(
  x,
  prediction = x$prediction,
  overall = x$overall,
  just.addcols = "right",
  smlab = "Leave-One-Out Meta-Analysis",
  type = "square",
  layout = gs("layout"),
  lab.NA = ".",
  backtransf = x$backtransf,
  big.mark = gs("big.mark"),
  digits = gs("digits.forest"),
  digits.pval = gs("digits.pval"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.I2 = gs("digits.I2"),
  digits.cid = gs("digits.cid"),
  digits.percent = 1,
  col = gs("col.study"),
  col.bg = ifelse(type == "diamond", gs("col.diamond"), gs("col.square")),
  col.border = ifelse(type == "diamond", gs("col.diamond.lines"), gs("col.square.lines")),
  col.bg.predict = gs("col.predict"),
  col.border.predict = gs("col.predict.lines"),
  addrows.below.overall = 1L * details,
  details = gs("forest.details"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{metainf}}.}

\item{prediction}{A logical indicating whether prediction
intervals should be printed.}

\item{overall}{A logical indicating whether overall results should be
shown.}

\item{just.addcols}{Justification of text for additional columns
(possible values: "left", "right", "center").}

\item{smlab}{A label for the summary measure (printed at top of
figure).}

\item{type}{A character string or vector specifying how to
plot treatment effects and confidence intervals for cumulative
meta-analysis results.}

\item{layout}{A character string specifying the layout of the
forest plot (see \code{\link{forest.meta}}).}

\item{lab.NA}{A character string to label missing values.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{big.mark}{A character used as thousands separator.}

\item{digits}{Minimal number of significant digits for treatment
effects, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for
p-values.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance.}

\item{digits.I2}{Minimal number of significant digits for I-squared
statistic.}

\item{digits.cid}{Minimal number of significant digits for
CID / decision thresholds, see \code{print.default}.}

\item{digits.percent}{Minimal number of significant digits for
probabilities, printed as percentages, see \code{print.default}.}

\item{col}{The colour for cumulative meta-analysis results (only considered
if \code{type = "square"}).}

\item{col.bg}{The background colour for squares and diamonds of
cumulative meta-analysis results.}

\item{col.border}{The colour for the outer lines of squares and diamonds of
cumulative meta-analysis results.}

\item{col.bg.predict}{The background colour for prediction intervals of
cumulative meta-analysis results.}

\item{col.border.predict}{The colour for the outer lines of prediction
intervals of cumulative meta-analysis results.}

\item{addrows.below.overall}{A numeric value indicating how many
empty rows are printed between meta-analysis results and
meta-analysis details.}

\item{details}{A logical specifying whether details on statistical
methods should be printed.}

\item{\dots}{Additional graphical arguments (passed on to
\code{\link{forest.meta}}).}
}
\description{
Draws a forest plot in the active graphics window (using grid
graphics system).
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window. Internally, R function
\code{\link{forest.metacum}} is called to produce the forest plot.
}
\examples{
data(Fleiss1993bin)
ma <- metabin(d.asp, n.asp, d.plac, n.plac,
  data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
ma
metainf(ma)
metainf(ma, pooled = "random")

forest(metainf(ma))
forest(metainf(ma, pooled = "random"))
forest(metainf(ma, pooled = "random", prediction = TRUE))

}
\seealso{
\code{\link{forest.metacum}}, \code{\link{forest.meta}},
  \code{\link{metainf}}, \code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update.meta.R
\name{update.meta}
\alias{update.meta}
\title{Update a meta-analysis object}
\usage{
\method{update}{meta}(
  object,
  data = object$data,
  subset,
  studlab,
  exclude,
  cluster,
  rho = object$rho,
  cycles,
  weights = NULL,
  weights.common = object$weights.common,
  weights.random = object$weights.random,
  method,
  sm = object$sm,
  incr = object$incr,
  method.incr = object$method.incr,
  allstudies = object$allstudies,
  incr.e,
  incr.c,
  incr.event,
  MH.exact = object$MH.exact,
  RR.Cochrane = object$RR.Cochrane,
  Q.Cochrane = object$Q.Cochrane,
  model.glmm = object$model.glmm,
  level = object$level,
  level.ma = object$level.ma,
  common = object$common,
  random = object$random,
  overall = object$overall,
  overall.hetstat = object$overall.hetstat,
  method.common.ci = object$method.common.ci,
  method.random.ci = object$method.random.ci,
  adhoc.hakn.ci = object$adhoc.hakn.ci,
  method.predict = object$method.predict,
  adhoc.hakn.pi = object$adhoc.hakn.pi,
  seed.predict = object$seed.predict,
  method.tau = object$method.tau,
  method.tau.ci = object$method.tau.ci,
  level.hetstat = object$level.hetstat,
  tau.preset = object$tau.preset,
  TE.tau = object$TE.tau,
  tau.common = object$tau.common,
  detail.tau = object$detail.tau,
  method.I2 = object$method.I2,
  prediction = object$prediction,
  level.predict = object$level.predict,
  null.effect = object$null.effect,
  method.bias = object$method.bias,
  backtransf = object$backtransf,
  func.backtransf = object$func.backtransf,
  args.backtransf = object$args.backtransf,
  pscale = object$pscale,
  irscale = object$irscale,
  irunit = object$irunit,
  text.common = object$text.common,
  text.random = object$text.random,
  text.predict = object$text.predict,
  text.w.common = object$text.w.common,
  text.w.random = object$text.w.random,
  title = object$title,
  complab = object$complab,
  outclab = object$outclab,
  label.e = object$label.e,
  label.c = object$label.c,
  label.left = object$label.left,
  label.right = object$label.right,
  col.label.left = object$col.label.left,
  col.label.right = object$col.label.right,
  n.e = object$n.e,
  n.c = object$n.c,
  method.mean = object$method.mean,
  method.sd = object$method.sd,
  approx.mean.e = object$approx.mean.e,
  approx.mean.c = object$approx.mean.c,
  approx.sd.e = object$approx.sd.e,
  approx.sd.c = object$approx.sd.c,
  approx.mean = object$approx.mean,
  approx.sd = object$approx.sd,
  approx.TE = object$approx.TE,
  approx.seTE = object$approx.seTE,
  pooledvar = object$pooledvar,
  method.smd = object$method.smd,
  sd.glass = object$sd.glass,
  exact.smd = object$exact.smd,
  method.ci = object$method.ci,
  subgroup,
  subgroup.name = object$subgroup.name,
  print.subgroup.name = object$print.subgroup.name,
  sep.subgroup = object$sep.subgroup,
  test.subgroup = object$test.subgroup,
  prediction.subgroup = object$prediction.subgroup,
  seed.predict.subgroup = object$seed.predict.subgroup,
  byvar,
  id,
  print.CMH = object$print.CMH,
  keepdata = TRUE,
  left = object$left,
  ma.common = object$ma.common,
  type = object$type,
  n.iter.max = object$n.iter.max,
  warn = FALSE,
  warn.deprecated = gs("warn.deprecated"),
  verbose = FALSE,
  control = object$control,
  ...
)
}
\arguments{
\item{object}{An object of class \code{meta}.}

\item{data}{Dataset.}

\item{subset}{Subset.}

\item{studlab}{Study label.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{cycles}{A numeric vector with the number of cycles per patient / study
in n-of-1 trials (see \code{\link{metagen}}).}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{method}{A character string indicating which method is to be
used for pooling of studies (see \code{\link{metabin}},
\code{\link{metainc}}, \code{\link{metaprop}} and
\code{\link{metarate}}).}

\item{sm}{A character string indicating which summary measure is
used for pooling.}

\item{incr}{Information on increment added to cell frequencies of
studies with zero cell counts (see \code{\link{metabin}},
\code{\link{metainc}}, \code{\link{metaprop}} and
\code{\link{metarate}}).}

\item{method.incr}{A character string indicating which continuity
correction method should be used (see \code{\link{metabin}},
\code{\link{metainc}}, \code{\link{metaprop}} and
\code{\link{metarate}}).}

\item{allstudies}{A logical indicating if studies with zero or all
events in both groups are to be included in the meta-analysis
(applies only to \code{\link{metabin}} object with \code{sm}
equal to \code{"RR"} or \code{"OR"}).}

\item{incr.e}{Continuity correction in experimental group (see
\code{\link{metabin}} and \code{\link{metainc}}).}

\item{incr.c}{Continuity correction in control group (see
\code{\link{metabin}} and \code{\link{metainc}}).}

\item{incr.event}{Continuity correction (see
\code{\link{metaprop}} and \code{\link{metarate}}).}

\item{MH.exact}{A logical indicating if \code{incr} is not to be
added to all cell frequencies for studies with a zero cell count
to calculate the pooled estimate based on the Mantel-Haenszel
method (applies only to \code{\link{metabin}} object).}

\item{RR.Cochrane}{A logical indicating which method to use as
continuity correction for the risk ratio (see
\code{\link{metabin}}).}

\item{Q.Cochrane}{A logical indicating which method to use to
calculate the heterogeneity statistic Q (see
\code{\link{metabin}}).}

\item{model.glmm}{A character string indicating which GLMM model
should be used (see \code{\link{metabin}} and
\code{\link{metainc}}).}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}

\item{method.bias}{A character string indicating which test for
funnel plot asymmetry is to be used, can be abbreviated. See
function \code{\link{metabias}.}}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If \code{backtransf =
TRUE}, results for \code{sm = "OR"} are printed as odds ratios
rather than log odds ratios and results for \code{sm = "ZCOR"}
are printed as correlations rather than Fisher's z transformed
correlations, for example.}

\item{func.backtransf}{A function used to back-transform results
(see \code{\link{metagen}}).}

\item{args.backtransf}{An optional list to provide additional
arguments to \code{func.backtransf} (see \code{\link{metagen}}).}

\item{pscale}{A numeric giving scaling factor for printing of
single event probabilities or risk differences, i.e. if argument
\code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
\code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.}

\item{irscale}{A numeric defining a scaling factor for printing of
single incidence rates or incidence rate differences, i.e. if
argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
\code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.}

\item{irunit}{A character specifying the time unit used to
calculate rates, e.g. person-years.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.e}{Label for experimental group.}

\item{label.c}{Label for control group.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{n.e}{Number of observations in experimental group (only for
\code{\link{metagen}} or \code{\link{metainc}} object).}

\item{n.c}{Number of observations in control group (only for
\code{\link{metagen}} or \code{\link{metainc}} object).}

\item{method.mean}{A character string indicating which method to
use to approximate the mean from the median and other statistics
(see \code{\link{metacont}} and \code{\link{metamean}}).}

\item{method.sd}{A character string indicating which method to use
to approximate the standard deviation from sample size, median,
interquartile range and range (see \code{\link{metacont}} and
\code{\link{metamean}}).}

\item{approx.mean.e}{Approximation method to estimate means in
experimental group (see \code{\link{metacont}}).}

\item{approx.mean.c}{Approximation method to estimate means in
control group (see \code{\link{metacont}}).}

\item{approx.sd.e}{Approximation method to estimate standard
deviations in experimental group (see \code{\link{metacont}}).}

\item{approx.sd.c}{Approximation method to estimate standard
deviations in control group (see \code{\link{metacont}}).}

\item{approx.mean}{Approximation method to estimate means (see
\code{\link{metamean}}).}

\item{approx.sd}{Approximation method to estimate standard
deviations (see \code{\link{metamean}}).}

\item{approx.TE}{Approximation method to estimate treatment
estimate (see \code{\link{metagen}}).}

\item{approx.seTE}{Approximation method to estimate standard error
(see \code{\link{metagen}}).}

\item{pooledvar}{A logical indicating if a pooled variance should
be used for the mean difference or ratio of means (see
\code{\link{metacont}}).}

\item{method.smd}{A character string indicating which method is
used to estimate the standardised mean difference (see
\code{\link{metacont}}).}

\item{sd.glass}{A character string indicating which standard
deviation is used in the denominator for Glass' method to
estimate the standardised mean difference (only for metacont
object with \code{sm = "SMD"}). Either \code{"control"} using the
standard deviation in the control group (\code{sd.c}) or
\code{"experimental"} using the standard deviation in the
experimental group (\code{sd.e}), can be abbreviated.}

\item{exact.smd}{A logical indicating whether exact formulae should
be used in estimation of the standardised mean difference and its
standard error.}

\item{method.ci}{A character string indicating which method is used
to calculate confidence intervals for individual studies. Either
\code{"z"}, \code{"t"}, \code{"WS"}, \code{"WSCC"}, \code{"AC"},
\code{"SA"}, \code{"SACC"}, \code{"NAsm"}, or \code{"Poisson"},
can be abbreviated. See functions \code{\link{metacont}},
\code{\link{metaprop}} and \code{\link{metarate}}.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{id}{Deprecated argument (replaced by 'cluster').}

\item{print.CMH}{A logical indicating whether result of the
Cochran-Mantel-Haenszel test for overall effect should be
printed.}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{left}{A logical indicating whether studies are supposed to be
missing on the left or right side of the funnel plot. If NULL,
the linear regression test for funnel plot symmetry (i.e.,
function \code{metabias(..., method = "linreg")}) is used to
determine whether studies are missing on the left or right side.}

\item{ma.common}{A logical indicating whether a common effect or
random effects model is used to estimate the number of missing
studies.}

\item{type}{A character indicating which method is used to estimate
the number of missing studies. Either \code{"L"} or \code{"R"}.}

\item{n.iter.max}{Maximum number of iterations to estimate number
of missing studies.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if \code{incr} is added to studies with zero cell
frequencies).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{verbose}{A logical indicating whether to print information on
updates of older meta versions.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}} or
\code{\link[metafor]{rma.glmm}}, respectively.}

\item{\dots}{Additional arguments (ignored at the moment).}
}
\value{
An object of class \code{"meta"} and \code{"metabin"},
\code{"metacont"}, \code{"metacor"}, \code{"metainc"},
\code{"metagen"}, \code{"metamean"}, \code{"metaprop"}, or
\code{"metarate"} (see \code{\link{meta-object}}).
}
\description{
Update an existing meta-analysis object.
}
\details{
Wrapper function to update an existing meta-analysis object which
was created with R function \code{\link{metabin}},
\code{\link{metacont}}, \code{\link{metacor}},
\code{\link{metagen}}, \code{\link{metainc}},
\code{\link{metamean}}, \code{\link{metaprop}}, or
\code{\link{metarate}}. More details on function arguments are
available in help files of respective R functions.

This function can also be used for objects of class 'trimfill',
'metacum', and 'metainf'.
}
\examples{
data(Fleiss1993cont)
ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, studlab = paste(study, year), sm = "SMD")
ma1

# Change summary measure (from 'SMD' to 'MD')
#
update(ma1, sm = "MD")

# Restrict analysis to subset of studies
#
update(ma1, subset = 1:2)

# Use different levels for confidence intervals
#
ma2 <- update(ma1, level = 0.66, level.ma = 0.99)
print(ma2, digits = 2)
forest(ma2)

}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metacor}}, \code{\link{metagen}},
  \code{\link{metainc}}, \code{\link{metamean}},
  \code{\link{metaprop}}, \code{\link{metarate}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/smoking.R
\docType{data}
\name{smoking}
\alias{smoking}
\alias{lungcancer}
\title{Smoking example}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{participants}}\tab total number of participants \cr
\bold{\emph{d.smokers}}\tab number of deaths in smokers' group \cr
\bold{\emph{py.smokers}}\tab person years at risk in smokers' group
  \cr
\bold{\emph{d.nonsmokers}}\tab number of deaths in non-smokers'
  group \cr
\bold{\emph{py.nonsmokers}}\tab person years at risk in
  non-smokers' group
}
}
\source{
Bayne-Jones S et al. (1964):
Smoking and Health: Report of the Advisory Committee to the Surgeon
General of the United States.
U-23 Department of Health, Education, and Welfare.
Public Health Service Publication No. 1103.
}
\description{
Meta-analyses on the effect of smoking on mortality risk.
}
\details{
Data have been reconstructed based on the famous Smoking and Health
Report to the Surgeon General (Bayne-Jones S et al., 1964). Data
sets can be used to evaluate the risk of smoking on overall
mortality (dataset \code{smoking}) and lung-cancer deaths (dataset
\code{lungcancer}), respectively.

The person time is attributed such that the rate ratios are equal
to the reported mortality ratios implicitly assuming that the data
have arisen from a homogeneous age group; more detailed information
by age is not available from the report. Note, the group of
"non-smokers" actually consists of all participants except those
who are smokers of cigarettes only. Information on real non-smokers
is not available from the published Smoking and Health Report.
}
\examples{
data(smoking)

ma1 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
  data = smoking, studlab = study)
print(ma1, digits = 2)

data(lungcancer)

ma2 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
  data = lungcancer, studlab = study)
print(ma2, digits = 2)
}
\seealso{
\code{\link{metainc}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metacr.R
\name{metacr}
\alias{metacr}
\title{Meta-analysis of outcome data from Cochrane review}
\usage{
metacr(
  x,
  comp.no = 1,
  outcome.no = 1,
  method,
  sm,
  level = gs("level"),
  common,
  random,
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = "DL",
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.common = FALSE,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = "classic",
  method.random.ci = "classic",
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  Q.Cochrane,
  swap.events,
  logscale,
  backtransf = gs("backtransf"),
  test.subgroup,
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  rob = NULL,
  tool = NULL,
  categories = NULL,
  col = NULL,
  symbols = NULL,
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title,
  complab,
  outclab,
  label.left,
  label.right,
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  keepdata = gs("keepdata"),
  warn = FALSE,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{rm5} or \code{cdir} created by R
function \code{read.rm5} or \code{read.cdir} .}

\item{comp.no}{Comparison number.}

\item{outcome.no}{Outcome number.}

\item{method}{A character string indicating which method is to be
used for pooling of studies. One of \code{"Inverse"},
\code{"MH"}, or \code{"Peto"}, can be abbreviated.}

\item{sm}{A character string indicating which summary measure
(\code{"RR"}, \code{"OR"}, \code{"RD"}, \code{"ASD"},
\code{"HR"}, \code{"MD"}, or \code{"SMD"}, or \code{"ROM"}) is to
be used for pooling of studies.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{Q.Cochrane}{A logical indicating if the Mantel-Haenszel
estimate is used in the calculation of the heterogeneity
statistic Q which is implemented in RevMan 5.}

\item{swap.events}{A logical indicating whether events and
non-events should be interchanged.}

\item{logscale}{A logical indicating whether effect estimates are
entered on log-scale (ignored for \code{cdir} objects).}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If
\code{backtransf=TRUE} (default), results for \code{sm="OR"} are
printed as odds ratios rather than log odds ratios and results
for \code{sm="ZCOR"} are printed as correlations rather than
Fisher's z transformed correlations, for example.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{rob}{A logical indicating whether risk of bias (RoB)
assessment should be considered in meta-analysis (only for
\code{read.cdir} objects).}

\item{tool}{Risk of bias (RoB) tool (only for \code{read.cdir}
objects).}

\item{categories}{Possible RoB categories (only for
\code{read.cdir} objects).}

\item{col}{Colours for RoB categories (only for \code{read.cdir}
objects).}

\item{symbols}{Corresponding symbols for RoB categories (only for
\code{read.cdir} objects).}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if \code{incr} is added to studies with zero cell
frequencies).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{"meta"} and - depending on outcome type
utilised in Cochrane intervention review for selected outcome -
\code{"metabin"}, \code{"metacont"}, or \code{"metagen"} with
corresponding generic functions (see \code{\link{meta-object}}).
}
\description{
Wrapper function to perform meta-analysis for a single outcome of a
Cochrane Intervention review.
}
\details{
Cochrane intervention reviews are based on the comparison of two
interventions. Each Cochrane intervention review can have a
variable number of comparisons. For each comparison, a variable
number of outcomes can be define. For each outcome, a separate
meta-analysis is conducted. Review Manager 5 (RevMan 5) was the
software used for preparing and maintaining Cochrane Reviews.

This wrapper function can be used to perform meta-analysis for a
single outcome of a Cochrane intervention review. Internally, R
functions \code{\link{metabin}}, \code{\link{metacont}}, and
\code{\link{metagen}} are called - depending on the definition of
the outcome in RevMan 5.

Information on the risk of bias RoB) assessment can be provided
with arguments \code{tool}, \code{categories}, \code{col} and
\code{symbols}. This is not useful if an overall RoB assessment has
been done. In this case use \code{\link{rob}} to add the full
flexible RoB information to a \code{\link{metacr}} object.

Note, it is recommended to choose the RevMan 5 settings before
executing \code{metacr}, i.e., \code{settings.meta("revman5")}.
}
\examples{
# Locate export data file "Fleiss1993_CR.csv"
# in sub-directory of package "meta"
#
filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
#
Fleiss1993_CR <- read.rm5(filename)

# Choose RevMan 5 settings and store old settings
#
oldset <- settings.meta("revman5", quietly = FALSE)

# Same result as R command example(Fleiss1993bin)
#
metacr(Fleiss1993_CR)

# Same result as R command example(Fleiss1993cont)
#
metacr(Fleiss1993_CR, 1, 2)
forest(metacr(Fleiss1993_CR, 1, 2))

# Change summary measure to RR
#
ma <- metacr(Fleiss1993_CR)
update(ma, sm = "RR")

# Use old settings
#
settings.meta(oldset)

}
\references{
\emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
The Cochrane Collaboration, 2020
}
\seealso{
\code{\link{meta-package}}, \code{\link{rob}},
  \code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}, \code{\link{read.cdir}},
  \code{\link{read.rm5}}, \code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.metacum.R
\name{forest.metacum}
\alias{forest.metacum}
\title{Forest plot to display the result of a cumulative meta-analysis}
\usage{
\method{forest}{metacum}(
  x,
  leftcols = NULL,
  leftlabs = NULL,
  rightcols = NULL,
  rightlabs = NULL,
  prediction = x$prediction,
  overall = x$overall,
  just.addcols = "right",
  smlab = "Cumulative Meta-Analysis",
  type = "square",
  layout = gs("layout"),
  lab.NA = ".",
  backtransf = x$backtransf,
  big.mark = gs("big.mark"),
  digits = gs("digits.forest"),
  digits.pval = gs("digits.pval"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.I2 = gs("digits.I2"),
  digits.cid = gs("digits.cid"),
  digits.percent = 1,
  col = gs("col.study"),
  col.bg = ifelse(type == "diamond", gs("col.diamond"), gs("col.square")),
  col.border = ifelse(type == "diamond", gs("col.diamond.lines"), gs("col.square.lines")),
  col.bg.predict = gs("col.predict"),
  col.border.predict = gs("col.predict.lines"),
  addrows.below.overall = 1L * details,
  details = gs("forest.details"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{metacum}}.}

\item{leftcols}{A character vector specifying (additional) columns
to be plotted on the left side of the forest plot or a logical
value.}

\item{leftlabs}{A character vector specifying labels for
(additional) columns on left side of the forest plot.}

\item{rightcols}{A character vector specifying (additional) columns
to be plotted on the right side of the forest plot or a logical
value.}

\item{rightlabs}{A character vector specifying labels for
(additional) columns on right side of the forest plot.}

\item{prediction}{A logical indicating whether prediction
intervals should be printed.}

\item{overall}{A logical indicating whether overall results should be
shown.}

\item{just.addcols}{Justification of text for additional columns
(possible values: "left", "right", "center").}

\item{smlab}{A label for the summary measure (printed at top of
figure).}

\item{type}{A character string or vector specifying how to
plot treatment effects and confidence intervals for cumulative
meta-analysis results.}

\item{layout}{A character string specifying the layout of the
forest plot (see \code{\link{forest.meta}}).}

\item{lab.NA}{A character string to label missing values.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios, for example.}

\item{big.mark}{A character used as thousands separator.}

\item{digits}{Minimal number of significant digits for treatment
effects, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for
p-values.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance.}

\item{digits.I2}{Minimal number of significant digits for I-squared
statistic.}

\item{digits.cid}{Minimal number of significant digits for
CID / decision thresholds, see \code{print.default}.}

\item{digits.percent}{Minimal number of significant digits for
probabilities, printed as percentages, see \code{print.default}.}

\item{col}{The colour for cumulative meta-analysis results (only considered
if \code{type = "square"}).}

\item{col.bg}{The background colour for squares and diamonds of
cumulative meta-analysis results.}

\item{col.border}{The colour for the outer lines of squares and diamonds of
cumulative meta-analysis results.}

\item{col.bg.predict}{The background colour for prediction intervals of
cumulative meta-analysis results.}

\item{col.border.predict}{The colour for the outer lines of prediction
intervals of cumulative meta-analysis results.}

\item{addrows.below.overall}{A numeric value indicating how many
empty rows are printed between meta-analysis results and
meta-analysis details.}

\item{details}{A logical specifying whether details on statistical
methods should be printed.}

\item{\dots}{Additional graphical arguments (passed on to
\code{\link{forest.meta}}).}
}
\description{
Draws a forest plot in the active graphics window (using grid
graphics system).
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window. The forest functions in R package
\bold{meta} are based on the grid graphics system. In order to
print the forest plot, resize the graphics window and either use
\code{\link{dev.copy2eps}} or \code{\link{dev.copy2pdf}}. Another
possibility is to create a file using \code{\link{pdf}},
\code{\link{png}}, or \code{\link{svg}} and to specify the width
and height of the graphic (see \code{\link{forest.meta}} examples).

The arguments \code{leftcols} and \code{rightcols} can be used to
specify columns which are plotted on the left and right side of the
forest plot, respectively.

The arguments \code{leftlabs} and \code{rightlabs} can be used to
specify column headings which are plotted on left and right side of
the forest plot, respectively. For certain columns predefined
labels exist. For other columns, the column name will be used as a
label. It is possible to only provide labels for new columns (see
\code{\link{forest.meta}} examples). Otherwise the length of
\code{leftlabs} and \code{rightlabs} must be the same as the number
of printed columns, respectively. The value \code{NA} can be used
to specify columns which should use default labels.
}
\examples{
data(Fleiss1993bin)
ma <- metabin(d.asp, n.asp, d.plac, n.plac,
  data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
ma
metacum(ma)
metacum(ma, pooled = "random")

forest(metacum(ma))
forest(metacum(ma, pooled = "random"))
forest(metacum(ma, pooled = "random", prediction = TRUE))

}
\seealso{
\code{\link{forest.meta}}, \code{\link{metacum}},
  \code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gs.R
\name{gs}
\alias{gs}
\title{Get default for a meta-analysis setting.}
\usage{
gs(x = NULL, unname = NULL)
}
\arguments{
\item{x}{A character string or vector with setting name(s).}

\item{unname}{A logical whether to remove names from attributes.}
}
\description{
Get default for a meta-analysis setting in R package \bold{meta}.
}
\details{
This function can be used to get the default for a meta-analysis
setting defined using \code{\link{settings.meta}}.

This function is primarily used to define default settings in
meta-analysis functions, e.g., \code{\link{metabin}} or
\code{\link{metacont}}. A list of all arguments with current
settings is printed using the command
\code{settings.meta("print")}.
}
\examples{
# Get default setting for confidence interval of random effects
# model
#
gs("method.random.ci")

# Get default setting for summary measure in metabin()
#
gs("smbin")

}
\seealso{
\code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaprop.R
\name{metaprop}
\alias{metaprop}
\title{Meta-analysis of single proportions}
\usage{
metaprop(
  event,
  n,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  method,
  sm = gs("smprop"),
  incr = gs("incr"),
  method.incr = gs("method.incr"),
  method.ci = gs("method.ci.prop"),
  level = gs("level"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = ifelse(!is.na(charmatch(tolower(method), "glmm", nomatch = NA)), "ML",
    gs("method.tau")),
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  null.effect = NA,
  method.bias = gs("method.bias"),
  backtransf = gs("backtransf"),
  pscale = 1,
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  hakn,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{event}{Number of events.}

\item{n}{Number of observations.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information, i.e., event and n.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{method}{A character string indicating which method is to be
used for pooling of studies. One of \code{"Inverse"} and
\code{"GLMM"}, can be abbreviated.}

\item{sm}{A character string indicating which summary measure
(\code{"PLOGIT"}, \code{"PAS"}, \code{"PFT"}, \code{"PLN"}, or
\code{"PRAW"}) is to be used for pooling of studies, see Details.}

\item{incr}{A numeric which is added to event number and sample
size of studies with zero or all events, i.e., studies with an
event probability of either 0 or 1. Or a numeric vector with the
continuity correction for each study.}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, or \code{"all"}), see Details.}

\item{method.ci}{A character string indicating which method is used
to calculate confidence intervals for individual studies, see
Details.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, or
\code{"Thompson"}, can be abbreviated. See function
\code{\link{metabias}}.}

\item{backtransf}{A logical indicating whether results for
transformed proportions (argument \code{sm != "PRAW"}) should be
back transformed in printouts and plots. If TRUE (default),
results will be presented as proportions; otherwise transformed
proportions will be shown. See Details for presentation of
confidence intervals.}

\item{pscale}{A numeric defining a scaling factor for printing of
single event probabilities.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{hakn}{Deprecated argument (replaced by 'method.random.ci').}

\item{adhoc.hakn}{Deprecated argument (replaced by 'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if estimation problems exist in fitting a GLMM).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}} or
\code{\link[metafor]{rma.glmm}}, respectively.}

\item{\dots}{Additional arguments passed on to
\code{\link[metafor]{rma.glmm}} function and to catch deprecated
arguments.}
}
\value{
An object of class \code{c("metaprop", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of an overall proportion from studies reporting a
single proportion. Inverse variance method and generalised linear
mixed model (GLMM) are available for pooling. For GLMMs, the
\code{\link[metafor]{rma.glmm}} function from R package
\bold{metafor} (Viechtbauer 2010) is called internally.
}
\details{
This function provides methods for common effect and random effects
meta-analysis of single proportions to calculate an overall
proportion. Note, you should use R function \code{\link{metabin}}
to compare proportions of pairwise comparisons instead of using
\code{metaprop} for each treatment arm separately which will break
randomisation in randomised controlled trials.

The following transformations of proportions are
implemented to calculate an overall proportion:

\itemize{
\item Logit transformation (\code{sm = "PLOGIT"}, default)
\item Arcsine transformation (\code{sm = "PAS"})
\item Freeman-Tukey Double arcsine transformation (\code{sm = "PFT"})
\item Log transformation (\code{sm = "PLN"})
\item No transformation (\code{sm = "PRAW"})
}

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain the transformed proportions. In printouts and plots these
values are back transformed if argument \code{backtransf = TRUE}
(default).

A generalised linear mixed model (GLMM) - more specific, a random
intercept logistic regression model - can be utilised for the
meta-analysis of proportions (Stijnen et al., 2010). This is the
default method for the logit transformation (argument \code{sm =
"PLOGIT"}). Internally, the \code{\link[metafor]{rma.glmm}}
function from R package \bold{metafor} is called to fit a GLMM.

Classic meta-analysis (Borenstein et al., 2010) utilising the
(un)transformed proportions and corresponding standard errors in
the inverse variance method is conducted by calling the
\code{\link{metagen}} function internally. This is the only
available method for all transformations but the logit
transformation. The classic meta-analysis model with logit
transformed proportions is used by setting argument \code{method =
"Inverse"}.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

\subsection{Choice of transformation / meta-analysis method}{

Contradictory recommendations on the use of transformations of
proportions have been published in the literature. For example,
Barendregt et al. (2013) recommend the use of the Freeman-Tukey
double arcsine transformation instead of the logit transformation
whereas Warton & Hui (2011) strongly advise to use generalised
linear mixed models with the logit transformation instead of the
arcsine transformation.

Schwarzer et al. (2019) describe seriously misleading results in a
meta-analysis with very different sample sizes due to problems with
the back-transformation of the Freeman-Tukey transformation which
requires a single sample size (Miller, 1978). Accordingly,
Schwarzer et al. (2019) also recommend to use GLMMs for the
meta-analysis of single proportions, however, admit that individual
study weights are not available with this method. Meta-analysts
which require individual study weights should consider the inverse
variance method with the arcsine or logit transformation.

In order to prevent misleading conclusions for the Freeman-Tukey
double arcsine transformation, sensitivity analyses using other
transformations or using a range of sample sizes should be
conducted (Schwarzer et al., 2019).
}

\subsection{Continuity correction}{

Three approaches are available to apply a continuity correction:
\itemize{
\item Only studies with a zero cell count (\code{method.incr =
  "only0"})
\item All studies if at least one study has a zero cell count
  (\code{method.incr = "if0all"})
\item All studies irrespective of zero cell counts
  (\code{method.incr = "all"})
}

If the summary measure is equal to "PLOGIT", "PLN", or "PRAW", the
continuity correction is applied if a study has either zero or all
events, i.e., an event probability of either 0 or 1.

By default, 0.5 is used as continuity correction (argument
\code{incr}). This continuity correction is used both to calculate
individual study results with confidence limits and to conduct
meta-analysis based on the inverse variance method. For GLMMs no
continuity correction is used. Furthermore, the value of \code{incr} is
only considered in the calculation of confidence intervals for individual
studies if \code{method.ci = "NAsm"} (see next subsection).
}

\subsection{Confidence intervals for individual studies}{

Various methods are available to calculate confidence intervals for
individual study results (see Agresti & Coull 1998 and Newcombe
1988):
\itemize{
\item Clopper-Pearson interval also called 'exact' binomial
  interval (\code{method.ci = "CP"}, default)
\item Wilson Score interval (\code{method.ci = "WS"})
\item Wilson Score interval with continuity correction
  (\code{method.ci = "WSCC"})
\item Agresti-Coull interval (\code{method.ci = "AC"})
\item Simple approximation interval (\code{method.ci = "SA"})
\item Simple approximation interval with continuity correction
  (\code{method.ci = "SACC"})
\item Normal approximation interval based on summary measure,
  i.e. defined by argument \code{sm} (\code{method.ci = "NAsm"})
}

Note, with exception of the normal approximation based on the
summary measure, i.e. \code{method.ci = "NAsm"}, the same
confidence interval is calculated for individual studies for any
summary measure (argument \code{sm}) as only number of events and
observations are used in the calculation disregarding the chosen
transformation. Furthermore, the continuity correction 

Results will be presented for transformed proportions if argument
\code{backtransf = FALSE}. In this case, argument \code{method.ci =
"NAsm"} is used, i.e. confidence intervals based on the normal
approximation based on the summary measure.
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Specify the null hypothesis of test for an overall proportion}{

Argument \code{null.effect} can be used to specify the proportion
used under the null hypothesis in a test for an overall effect.

By default (\code{null.effect = NA}), no hypothesis test is
conducted as it is unclear which value is a sensible choice for the
data at hand.  An overall proportion of 50\%, for example, could be
tested by setting argument \code{null.effect = 0.5}.

Note, all tests for an overall effect are two-sided with the
alternative hypothesis that the effect is unequal to
\code{null.effect}.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments
\code{common} and \code{random}. Accordingly, the estimate
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"meta"} even if
argument \code{random = FALSE}. However, all functions in R
package \bold{meta} will adequately consider the values for
\code{common} and \code{random}. E.g. function
\code{\link{print.meta}} will not print results for the random
effects model if \code{random = FALSE}.

Argument \code{pscale} can be used to rescale proportions, e.g.
\code{pscale = 1000} means that proportions are expressed as events
per 1000 observations. This is useful in situations with (very) low
event probabilities.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\examples{
# Meta-analysis using generalised linear mixed model
#
metaprop(4:1, 10 * 1:4)

# Apply various classic meta-analysis methods to estimate
# proportions
#
ma1 <- metaprop(4:1, 10 * 1:4, method = "Inverse")
ma2 <- update(ma1, sm = "PAS")
ma3 <- update(ma1, sm = "PRAW")
ma4 <- update(ma1, sm = "PLN")
ma5 <- update(ma1, sm = "PFT")
#
ma1
ma2
ma3
ma4
ma5
#
forest(ma1)
\dontrun{
forest(ma2)
forest(ma3)
forest(ma3, pscale = 100)
forest(ma4)
forest(ma5)
}

# Do not back transform results, e.g. print logit transformed
# proportions if sm = "PLOGIT" and store old settings
#
oldset <- settings.meta(backtransf = FALSE)
#
ma6  <- metaprop(4:1, c(10, 20, 30, 40), method = "Inverse")
ma7  <- update(ma6, sm = "PAS")
ma8  <- update(ma6, sm = "PRAW")
ma9  <- update(ma6, sm = "PLN")
ma10 <- update(ma6, sm = "PFT")
#
forest(ma6)
\dontrun{
forest(ma7)
forest(ma8)
forest(ma8, pscale = 100)
forest(ma9)
forest(ma10)
}

# Use old settings
#
settings.meta(oldset)

# Examples with zero events
#
ma11 <- metaprop(c(0, 0, 10, 10), rep(100, 4), method = "Inverse")
ma12 <- metaprop(c(0, 0, 10, 10), rep(100, 4), incr = 0.1, method = "Inverse")
#
ma11
ma12
#
\dontrun{
forest(ma11)
forest(ma12)
}

# Example from Miller (1978):
#
death <- c(3, 6, 10, 1)
animals <- c(11, 17, 21, 6)
#
ma13 <- metaprop(death, animals, sm = "PFT")
forest(ma13)

# Data examples from Newcombe (1998)
# - apply various methods to estimate confidence intervals for
#   individual studies
#
event <- c(81, 15, 0, 1)
n <- c(263, 148, 20, 29)
#
ma14 <- metaprop(event, n, method.ci = "SA", method = "Inverse")
ma15 <- update(ma14, method.ci = "SACC")
ma16 <- update(ma14, method.ci = "WS")
ma17 <- update(ma14, method.ci = "WSCC")
ma18 <- update(ma14, method.ci = "CP")
#
lower <- round(logit2p(rbind(NA, ma14$lower, ma15$lower, NA, ma16$lower,
  ma17$lower, NA, ma18$lower)), 4)
upper <- round(logit2p(rbind(NA, ma14$upper, ma15$upper, NA, ma16$upper,
  ma17$upper, NA, ma18$upper)), 4)
#
tab1 <- data.frame(
  scen1 = meta:::formatCI(lower[, 1], upper[, 1]),
  scen2 = meta:::formatCI(lower[, 2], upper[, 2]),
  scen3 = meta:::formatCI(lower[, 3], upper[, 3]),
  scen4 = meta:::formatCI(lower[, 4], upper[, 4])
  )
names(tab1) <- c("r=81, n=263", "r=15, n=148",
  "r=0, n=20", "r=1, n=29")
row.names(tab1) <- c("Simple", "- SA", "- SACC",
  "Score", "- WS", "- WSCC", "Binomial", "- CP")
tab1[is.na(tab1)] <- ""
# Newcombe (1998), Table I, methods 1-5:
tab1

# Same confidence interval, i.e. unaffected by choice of summary
# measure
#
print(metaprop(event, n, method.ci = "WS", method = "Inverse"), ma = FALSE)
print(metaprop(event, n, sm = "PLN", method.ci = "WS"), ma = FALSE)
print(metaprop(event, n, sm = "PFT", method.ci = "WS"), ma = FALSE)
print(metaprop(event, n, sm = "PAS", method.ci = "WS"), ma = FALSE)
print(metaprop(event, n, sm = "PRAW", method.ci = "WS"), ma = FALSE)

# Different confidence intervals as argument sm = "NAsm"
#
print(metaprop(event, n, method.ci = "NAsm", method = "Inverse"), ma = FALSE)
print(metaprop(event, n, sm = "PLN", method.ci = "NAsm"), ma = FALSE)
print(metaprop(event, n, sm = "PFT", method.ci = "NAsm"), ma = FALSE)
print(metaprop(event, n, sm = "PAS", method.ci = "NAsm"), ma = FALSE)
print(metaprop(event, n, sm = "PRAW", method.ci = "NAsm"), ma = FALSE)

# Different confidence intervals as argument backtransf = FALSE.
# Accordingly, method.ci = "NAsm" used internally.
#
print(metaprop(event, n, method.ci = "WS", method = "Inverse"),
  ma = FALSE, backtransf = FALSE)
print(metaprop(event, n, sm = "PLN", method.ci = "WS"),
  ma = FALSE, backtransf = FALSE)
print(metaprop(event, n, sm = "PFT", method.ci = "WS"),
  ma = FALSE, backtransf = FALSE)
print(metaprop(event, n, sm = "PAS", method.ci = "WS"),
  ma = FALSE, backtransf = FALSE)
print(metaprop(event, n, sm = "PRAW", method.ci = "WS"),
  ma = FALSE, backtransf = FALSE)

# Same results (printed on original and log scale, respectively)
#
print(metaprop(event, n, sm = "PLN", method.ci = "NAsm"), ma = FALSE)
print(metaprop(event, n, sm = "PLN"), ma = FALSE, backtransf = FALSE)
# Results for first study (on log scale)
round(log(c(0.3079848, 0.2569522, 0.3691529)), 4)

# Print results as events per 1000 observations
#
print(metaprop(6:8, c(100, 1200, 1000), method = "Inverse"),
  pscale = 1000, digits = 1)

}
\references{
Agresti A & Coull BA (1998):
Approximate is better than "exact" for interval estimation of
binomial proportions.
\emph{The American Statistician},
\bold{52}, 119--26

Barendregt JJ, Doi SA, Lee YY, Norman RE, Vos T (2013):
Meta-analysis of prevalence.
\emph{Journal of Epidemiology and Community Health},
\bold{67}, 974--8

Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
A basic introduction to fixed-effect and random-effects models for
meta-analysis.
\emph{Research Synthesis Methods},
\bold{1}, 97--111

Freeman MF & Tukey JW (1950):
Transformations related to the angular and the square root.
\emph{Annals of Mathematical Statistics},
\bold{21}, 607--11

Miller JJ (1978):
The inverse of the Freeman-Tukey double arcsine transformation.
\emph{The American Statistician},
\bold{32}, 138

Newcombe RG (1998):
Two-sided confidence intervals for the single proportion:
comparison of seven methods.
\emph{Statistics in Medicine},
\bold{17}, 857--72

Pettigrew HM, Gart JJ, Thomas DG (1986):
The bias and higher cumulants of the logarithm of a binomial
variate.
\emph{Biometrika},
\bold{73}, 425--35

Schwarzer G, Chemaitelly H, Abu-Raddad LJ, Rücker G (2019):
Seriously misleading results using inverse of Freeman-Tukey double
arcsine transformation in meta-analysis of single proportions.
\emph{Research Synthesis Methods},
\bold{10}, 476--83

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Viechtbauer W (2010):
Conducting meta-analyses in R with the metafor package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48

Warton DI, Hui FKC (2011):
The arcsine is asinine: the analysis of proportions in ecology.
\emph{Ecology},
\bold{92}, 3--10
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{metacont}}, \code{\link{metagen}},
  \code{\link{print.meta}}, \code{\link{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Pagliaro1992.R
\docType{data}
\name{Pagliaro1992}
\alias{Pagliaro1992}
\title{Meta-analysis on Prevention of First Bleeding in Cirrhosis}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{id}}\tab study id \cr
\bold{\emph{treat.exp}}\tab treatment in experimental group \cr
\bold{\emph{logOR}}\tab log odds ratio
  \cr
\bold{\emph{selogOR}}\tab standard error of log odds ratio
  \cr
\bold{\emph{bleed.exp}}\tab number of bleedings in experimental group \cr
\bold{\emph{n.cont}}\tab number of observations in experimental group \cr
\bold{\emph{bleed.plac}}\tab number of bleedings in placebo group \cr
\bold{\emph{n.plac}}\tab number of observations in placebo group
}
}
\source{
Pagliaro L, D’Amico G et al. (1992):
Prevention of first bleeding in cirrhosis.
\emph{Annals in Internal Medicine},
\bold{117}, 59--70
}
\description{
Meta-analysis on Prevention of First Bleeding in Cirrhosis
comparing beta-blocker or sclerotherapy with placebo.
}
\examples{
data(Pagliaro1992)
sclero <- subset(Pagliaro1992, treat.exp == "Sclerotherapy")

m <- metagen(logOR, selogOR, data = sclero, sm = "OR")
m

# Thompson & Sharp (1999), Table IV, method (2)
metabias(m, method = "Egger")

# Thompson & Sharp (1999), Table IV, method (3a)
metabias(m, method = "Thompson")

# Thompson & Sharp (1999), Table IV, method (3b)
update(m, method.tau = "ML")
metabias(update(m, method.tau = "ML"), method = "Thompson")

}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.meta.R
\name{summary.meta}
\alias{summary.meta}
\title{Summary of meta-analysis results}
\usage{
\method{summary}{meta}(object, ...)
}
\arguments{
\item{object}{An object of class \code{meta}.}

\item{\dots}{Additional arguments (ignored).}
}
\value{
An object of classes \code{summary.meta} and \code{meta} (see
\code{\link{meta-object}}.
}
\description{
Summary method for objects of class \code{meta}.
}
\details{
Summary method for objects of class \code{meta}.
}
\examples{
data(Fleiss1993cont)
ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, studlab = paste(study, year), sm = "SMD")
summary(ma)

summary(update(ma, subgroup = c(1, 2, 1, 1, 2), subgroup.name = "group"))
forest(update(ma, subgroup = c(1, 2, 1, 1, 2), subgroup.name = "group"))

\dontrun{
# Use unicode characters to print tau^2, tau, and I^2
print(summary(ma),
  text.tau2 = "\u03c4\u00b2", text.tau = "\u03c4", text.I2 = "I\u00b2")
}

}
\references{
Cooper H & Hedges LV (1994):
\emph{The Handbook of Research Synthesis}.
Newbury Park, CA: Russell Sage Foundation

Crippa A, Khudyakov P, Wang M, Orsini N, Spiegelman D (2016):
A new measure of between-studies heterogeneity in meta-analysis.
\emph{Statistics in Medicine},
\bold{35}, 3661--75

Higgins JPT & Thompson SG (2002):
Quantifying heterogeneity in a meta-analysis.
\emph{Statistics in Medicine},
\bold{21}, 1539--58
}
\seealso{
\code{\link{print.summary.meta}}, \code{\link{metabin}},
  \code{\link{metacont}}, \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta-sm.R
\name{meta-sm}
\alias{meta-sm}
\title{Description of summary measures available in R package \bold{meta}}
\description{
Description of summary measures available in R package \bold{meta}
}
\details{
The following summary measures (argument \code{sm}) are recognised
in R package \bold{meta}.

\subsection{Meta-analysis of binary outcome data (\code{\link{metabin})}}{
\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "OR"} \tab Odds ratio (Fleiss, 1993)\cr
\code{sm = "RR"} \tab Risk ratio (Fleiss, 1993) \cr
\code{sm = "RD"} \tab Risk difference (Fleiss, 1993) \cr
\code{sm = "ASD"} \tab Arcsine difference (Rücker et al., 2009) \cr
\code{sm = "DOR"} \tab Diagnostic odds ratio (Moses et al., 1993)
  \cr
\code{sm = "VE"} \tab Vaccine efficacy or vaccine effectiveness
}

Note, mathematically, odds ratios and diagnostic odds ratios are
identical, however, the labels in printouts and figures
differ. Furthermore, log risk ratio (logRR) and log vaccine ratio
(logVR) are mathematical identical, however, back-transformed
results differ as vaccine efficacy or effectiveness is defined as
\code{VE = 100 * (1 - RR)}.

A continuity correction is used for some summary measures in the
case of a zero cell count (see \code{\link{metabin}}). 

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain transformed values, e.g., log odds ratios, log risk ratios
or log vaccine ratios.  In printouts and plots transformed values
are back transformed if argument \code{backtransf = TRUE}
(default), with exception of the arcsine difference where no
back-transformation exists. Auxiliary function
\code{\link{logVR2VE}} is used to back-transform log vaccine ratios
to vaccine efficacy or effectiveness while \code{\link[base]{exp}}
is used to back-transform log odds or risk ratios.
}

\subsection{Meta-analysis of continuous outcome data (\code{\link{metacont})}}{
\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "MD"} \tab Mean difference \cr
\code{sm = "SMD"} \tab Standardised mean difference \cr
\code{sm = "ROM"} \tab Ratio of means
}

Three variants to calculate the standardised mean difference are
available (see \code{\link{metacont}}).

For the ratio of means, list elements \code{TE}, \code{TE.common},
\code{TE.random}, etc., contain the log transformed ratio of
means. In printouts and plots these values are back transformed
using \code{\link[base]{exp}} if argument \code{backtransf = TRUE}
(default).
}

\subsection{Meta-analysis of correlations (\code{\link{metacor})}}{
\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "ZCOR"} \tab Fisher's z transformed correlation \cr
\code{sm = "COR"} \tab Untransformed correlations
}

For Fisher's z transformed correlations, list elements \code{TE},
\code{TE.common}, \code{TE.random}, etc., contain the transformed
correlations. In printouts and plots these values are back
transformed using auxiliary function \code{\link{z2cor}} if
argument \code{backtransf = TRUE} (default).
}

\subsection{Meta-analysis of incidence rates (\code{\link{metainc})}}{
\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "IRR"} \tab Incidence rate ratio \cr
\code{sm = "IRD"} \tab Incidence rate difference \cr
\code{sm = "IRSD"} \tab Square root transformed incidence rate
  difference \cr
\code{sm = "VE"} \tab Vaccine efficacy or vaccine effectiveness
}

Note, log incidence rate ratio (logIRR) and log vaccine ratio
(logVR) are mathematical identical, however, back-transformed
results differ as vaccine efficacy or effectiveness is defined as
\code{VE = 100 * (1 - IRR)}.

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain the transformed incidence rates. In printouts and plots
these values are back transformed if argument \code{backtransf =
TRUE} (default). For back-transformation, \code{\link[base]{exp}}
is used for the incidence rate ratio, power of 2 is used for square
root transformed rates and \code{\link{logVR2VE}} is used for
vaccine efficacy / effectiveness.
}

\subsection{Meta-analysis of single means (\code{\link{metamean})}}{
\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "MRAW"} \tab Raw, i.e. untransformed, means \cr
\code{sm = "MLN"} \tab Log transformed means
}

Calculations are conducted on the log scale if \code{sm =
"MLN"}. Accordingly, list elements \code{TE}, \code{TE.common}, and
\code{TE.random} contain the logarithm of means. In printouts and
plots these values are back transformed using
\code{\link[base]{exp}} if argument \code{backtransf = TRUE}.
}

\subsection{Meta-analysis of single proportions (\code{\link{metaprop})}}{

The following transformations of proportions are
implemented to calculate an overall proportion:

\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "PLOGIT"} \tab Logit transformation \cr
\code{sm = "PAS"} \tab Arcsine transformation \cr
\code{sm = "PFT"} \tab Freeman-Tukey Double arcsine transformation
  \cr
\code{sm = "PLN"} \tab Log transformation \cr
\code{sm = "PRAW"} \tab No transformation
}

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain the transformed proportions. In printouts and plots these
values are back transformed if argument \code{backtransf = TRUE}
(default). For back-transformation, \code{\link{logit2p}} is used
for logit transformed proportions, \code{\link{asin2p}} is used for
(Freeman-Tukey) arcsine transformed proportions and
\code{\link[base]{exp}} is used for log transformed proportions.
}

\subsection{Meta-analysis of single rates (\code{\link{metarate})}}{

The following transformations of incidence rates are implemented to
calculate an overall rate:

\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "IRLN"} \tab Log transformation \cr
\code{sm = "IRS"} \tab Square root transformation \cr
\code{sm = "IRFT"} \tab Freeman-Tukey Double arcsine transformation
  \cr
\code{sm = "IR"} \tab No transformation
}

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain the transformed incidence rates. In printouts and plots
these values are back transformed if argument \code{backtransf =
TRUE} (default). For back-transformation, \code{\link[base]{exp}}
is used for log transformed rates, power of 2 is used for square
root transformed rates and \code{\link{asin2ir}} is used for
Freeman-Tukey arcsine transformed rates.
}

\subsection{Generic inverse variance method (\code{\link{metagen})}}{

The following summary measures are recognised in addition to the
above mentioned summary measures:

\tabular{ll}{
\bold{Argument} \tab \bold{Summary measure} \cr
\code{sm = "HR"} \tab Hazard ratio \cr
\code{sm = "VE"} \tab Vaccine efficacy or vaccine effectiveness
}

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain transformed values, i.e., log hazard ratios and log vaccine
ratios. In printouts and plots these values are back transformed if
argument \code{backtransf = TRUE} (default); see also
\link{meta-transf}.
}
}
\references{
Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
A basic introduction to fixed-effect and random-effects models for
meta-analysis.
\emph{Research Synthesis Methods},
\bold{1}, 97--111

Fleiss JL (1993):
The statistical basis of meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{2}, 121--45

Moses LE, Shapiro D, Littenberg B (1993):
Combining Independent Studies of a Diagnostic Test into a Summary
Roc Curve: Data-Analytic Approaches and Some Additional
Considerations.
\emph{Statistics in Medicine},
\bold{12}, 1293--1316

Rücker G, Schwarzer G, Carpenter J, Olkin I (2009):
Why add anything to nothing? The arcsine difference as a measure of
treatment effect in meta-analysis with zero cells.
\emph{Statistics in Medicine},
\bold{28}, 721--38

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67
}
\seealso{
\code{\link{meta-package}}, \code{\link{meta-transf}},
  \code{\link{meta-object}}, \code{\link{print.meta}},
  \code{\link{summary.meta}}, \code{\link{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimfill.rm5.R
\name{trimfill.rm5}
\alias{trimfill.rm5}
\alias{trimfill.cdir}
\alias{print.trimfill.rm5}
\alias{print.trimfill.cdir}
\title{Cochrane review: trim-and-fill method}
\usage{
\method{trimfill}{rm5}(x, comp.no, outcome.no, ...)

\method{trimfill}{cdir}(x, comp.no, outcome.no, ...)

\method{print}{trimfill.rm5}(x, ...)

\method{print}{trimfill.cdir}(x, ...)
}
\arguments{
\item{x}{An object of class \code{rm5}, \code{trimfill.rm5},
\code{cdir} or \code{trimfill.cdir}.}

\item{comp.no}{Comparison number.}

\item{outcome.no}{Outcome number.}

\item{...}{Additional arguments (passed on to \code{metacr}).}
}
\description{
Conduct trim-and-fill analysis for all meta-analyses in a Cochrane
review.
}
\details{
This function can be used to conduct a trim-and-fill analysis for
all or selected meta-analyses in a Cochrane review of intervention
studies (Higgins et al., 2023).

The R function \code{\link{metacr}} is called internally.
}
\examples{
# Locate export data file "Fleiss1993_CR.csv"
# in sub-directory of package "meta"
#
filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
Fleiss1993_CR <- read.rm5(filename)

# Conduct trim-and-fill analysis
#
trimfill(Fleiss1993_CR)

# Conduct trim-and-fill analysis only for second outcome of first
# comparison
#
trimfill(Fleiss1993_CR, comp.no = 1, outcome.no = 2)

}
\references{
Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
VA (editors) (2023):
\emph{Cochrane Handbook for Systematic Reviews of Interventions
  Version 6.4 (updated August 2023)}.
Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}
}
\seealso{
\code{\link{summary.meta}}, \code{\link{metacr}},
  \code{\link{read.rm5}}, \code{\link{read.cdir}},
  \code{\link{metabias.rm5}}, \code{\link{metabias.cdir}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bracket.pairwise.R
\name{[.pairwise}
\alias{[.pairwise}
\alias{[,pairwise}
\title{Extract parts of pairwise object}
\usage{
\method{[}{pairwise}(x, ...)
}
\arguments{
\item{x}{An object of class \code{\link{pairwise}}.}

\item{\dots}{Additional arguments (passed on to [.data.frame).}
}
\description{
Auxiliary function to extract parts of \code{\link{pairwise}} object.
}
\examples{
# Transform data from arm-based format to contrast-based format
pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
  n = list(n1, n2, n3),
  mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
  data = Franchini2012, studlab = Study)

pw1[, 1:5]
pw1[!grepl("Lieberman", pw1$studlab), 1:5]

}
\seealso{
\code{\link{pairwise}}, \code{\link{subset.pairwise}},
  \code{\link[metabook]{Franchini2012}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabias.rm5.R
\name{metabias.rm5}
\alias{metabias.rm5}
\alias{metabias.cdir}
\title{Cochrane review: Test for funnel plot asymmetry}
\usage{
\method{metabias}{rm5}(
  x,
  comp.no,
  outcome.no,
  method.bias = "linreg",
  method.bias.binary = method.bias,
  method.bias.or = "score",
  k.min = 10,
  ...
)

\method{metabias}{cdir}(
  x,
  comp.no,
  outcome.no,
  method.bias = "linreg",
  method.bias.binary = method.bias,
  method.bias.or = "score",
  k.min = 10,
  ...
)
}
\arguments{
\item{x}{An object of class \code{rm5} or \code{cdir}.}

\item{comp.no}{Comparison number.}

\item{outcome.no}{Outcome number.}

\item{method.bias}{A character string indicating which test for
small-study effects is to be used for all outcomes. Either
\code{"rank"}, \code{"linreg"}, or \code{"mm"}, can be
abbreviated. See function \code{\link{metabias}}}

\item{method.bias.binary}{A character string indicating which test
is to be used for binary outcomes. Either \code{"rank"},
\code{"linreg"}, \code{"mm"}, \code{"count"}, \code{"score"}, or
\code{"peters"}, can be abbreviated. See function
\code{\link{metabias}}}

\item{method.bias.or}{A character string indicating which test is
to be used for binary outcomes with odds ratio as summary
measure. Either \code{"rank"}, \code{"linreg"}, \code{"mm"},
\code{"count"}, \code{"score"}, or \code{"peters"}, can be
abbreviated. See function \code{\link{metabias}}}

\item{k.min}{Minimum number of studies to perform test for
small-study effects.}

\item{...}{Additional arguments (ignored at the moment)}
}
\description{
Conduct a test for funnel plot asymmetry for all outcomes in a
Cochrane review of intervention studies
}
\details{
This function can be used to conduct a test for funnel plot
asymmetry for all or selected meta-analyses in a Cochrane review of
intervention studies (Higgins et al, 2023).

The R function \code{\link{metacr}} is called internally.
}
\examples{
# Locate export data file "Fleiss1993_CR.csv" in sub-directory of
# package "meta"
#
filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
Fleiss1993_CR <- read.rm5(filename)

# Print results for all tests of small-study effects
#
metabias(Fleiss1993_CR, k.min = 5)

# Print result of test of small-study effects for second outcome in
# first comparison
#
metabias(Fleiss1993_CR, comp.no = 1, outcome.no = 2, k.min = 5)

}
\references{
Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
VA (editors) (2023):
\emph{Cochrane Handbook for Systematic Reviews of Interventions
  Version 6.4 (updated August 2023)}.
Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}
}
\seealso{
\code{\link{metabias}}, \code{\link{metacr}},
  \code{\link{read.rm5}}, \code{\link{read.cdir}},
  \code{\link{summary.rm5}}, \code{\link{summary.cdir}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{htest}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rob.R
\name{rob}
\alias{rob}
\alias{print.rob}
\title{Risk of bias assessment}
\usage{
rob(
  item1,
  item2 = NULL,
  item3 = NULL,
  item4 = NULL,
  item5 = NULL,
  item6 = NULL,
  item7 = NULL,
  item8 = NULL,
  item9 = NULL,
  item10 = NULL,
  studlab = NULL,
  overall = NULL,
  weight = NULL,
  data = NULL,
  tool = gs("tool.rob"),
  domains = NULL,
  categories = NULL,
  cat1 = categories,
  cat2 = categories,
  cat3 = categories,
  cat4 = categories,
  cat5 = categories,
  cat6 = categories,
  cat7 = categories,
  cat8 = categories,
  cat9 = categories,
  cat10 = categories,
  cat.overall = categories,
  col = NULL,
  col1 = col,
  col2 = col,
  col3 = col,
  col4 = col,
  col5 = col,
  col6 = col,
  col7 = col,
  col8 = col,
  col9 = col,
  col10 = col,
  col.overall = col,
  symbols = NULL,
  symb1 = symbols,
  symb2 = symbols,
  symb3 = symbols,
  symb4 = symbols,
  symb5 = symbols,
  symb6 = symbols,
  symb7 = symbols,
  symb8 = symbols,
  symb9 = symbols,
  symb10 = symbols,
  symb.overall = symbols,
  legend = TRUE,
  overwrite = FALSE,
  warn = TRUE
)

\method{print}{rob}(x, legend = attr(x, "legend"), details = TRUE, ...)
}
\arguments{
\item{item1}{Risk of bias item 1 or a meta-analysis object of class
\code{meta} with information on risk of bias assessment.}

\item{item2}{Risk of bias item 2.}

\item{item3}{Risk of bias item 3.}

\item{item4}{Risk of bias item 4.}

\item{item5}{Risk of bias item 5.}

\item{item6}{Risk of bias item 6.}

\item{item7}{Risk of bias item 7.}

\item{item8}{Risk of bias item 8.}

\item{item9}{Risk of bias item 9.}

\item{item10}{Risk of bias item 10.}

\item{studlab}{Study labels.}

\item{overall}{Overall risk of bias assess.}

\item{weight}{Weight for each study.}

\item{data}{A data frame or a meta-analysis object of class
\code{meta}.}

\item{tool}{Risk of bias (RoB) tool.}

\item{domains}{A character vector with names of RoB domains.}

\item{categories}{Possible RoB categories.}

\item{cat1}{Possible categories for RoB item 1.}

\item{cat2}{Possible categories for RoB item 2.}

\item{cat3}{Possible categories for RoB item 3.}

\item{cat4}{Possible categories for RoB item 4.}

\item{cat5}{Possible categories for RoB item 5.}

\item{cat6}{Possible categories for RoB item 6.}

\item{cat7}{Possible categories for RoB item 7.}

\item{cat8}{Possible categories for RoB item 8.}

\item{cat9}{Possible categories for RoB item 9.}

\item{cat10}{Possible categories for RoB item 10.}

\item{cat.overall}{Possible categories for overall RoB.}

\item{col}{Colours for RoB categories.}

\item{col1}{Colours for categories for RoB item 1.}

\item{col2}{Colours for categories for RoB item 2.}

\item{col3}{Colours for categories for RoB item 3.}

\item{col4}{Colours for categories for RoB item 4.}

\item{col5}{Colours for categories for RoB item 5.}

\item{col6}{Colours for categories for RoB item 6.}

\item{col7}{Colours for categories for RoB item 7.}

\item{col8}{Colours for categories for RoB item 8.}

\item{col9}{Colours for categories for RoB item 9.}

\item{col10}{Colours for categories for RoB item 10.}

\item{col.overall}{Colours for categories for overall RoB.}

\item{symbols}{Corresponding symbols for RoB categories.}

\item{symb1}{Corresponding symbols for RoB item 1.}

\item{symb2}{Corresponding symbols for RoB item 2.}

\item{symb3}{Corresponding symbols for RoB item 3.}

\item{symb4}{Corresponding symbols for RoB item 4.}

\item{symb5}{Corresponding symbols for RoB item 5.}

\item{symb6}{Corresponding symbols for RoB item 6.}

\item{symb7}{Corresponding symbols for RoB item 7.}

\item{symb8}{Corresponding symbols for RoB item 8.}

\item{symb9}{Corresponding symbols for RoB item 9.}

\item{symb10}{Corresponding symbols for RoB item 10.}

\item{symb.overall}{Corresponding symbols for overall RoB.}

\item{legend}{A logical specifying whether legend with RoB domains
should be printed.}

\item{overwrite}{A logical indicating whether an existing risk of
bias table in a meta-analysis object should be overwritten.}

\item{warn}{A logical indicating whether warnings should be
printed.}

\item{x}{An object of class \code{rob}.}

\item{details}{A logical indicating whether to print details on
categories and colours.}

\item{\dots}{Additional printing arguments.}
}
\value{
A data frame with study labels and risk of bias items and
additional class "rob".
}
\description{
Create table with risk of bias assessment or add table to existing
meta-analysis
}
\details{
This function can be used to define a risk of bias (RoB) assessment
for a meta-analysis which can be shown in a forest plot
(\code{\link{forest.meta}}), summary weighted barplot
(\code{\link{barplot.rob}}) or traffic light plot
(\code{\link{traffic_light}}). It is also possible to extract the
risk of bias assessment from a meta-analysis with RoB information.

The risk of bias table contains

\itemize{
\item study labels;
\item variables for individual RoB domains (with variable names A,
  B, \dots);
\item an overall RoB assessment if argument \code{overall} is
  provided;
\item weights for individual studies used in summary weighted
  barplots.
}
Note, an overall RoB assessment is mandatory to create a summary
weighted barplot or a traffic light plot.

The RoB table is directly returned if argument \code{data} is a
data frame or argument \code{item1} is a meta-analysis with risk of
bias assessment. The RoB table is added as a new list element 'rob'
to a meta-analysis object if argument \code{data} is a
meta-analysis.

The user must either specify the categories and (optionally)
domains of the RoB tool (using the eponymous arguments) or one of
the following RoB tools.
\tabular{ll}{
\bold{Argument} \tab \bold{Risk of bias tool} \cr
\code{tool = "RoB1"} \tab RoB 1 tool for randomized studies
  (Higgins et al., 2011) \cr
\code{tool = "RoB2"} \tab RoB 2 tool for randomized studies
  (Sterne et al., 2019) \cr
\code{tool = "RoB2-cluster"} \tab RoB 2 tool for
  cluster-randomized trials \cr
\code{tool = "RoB2-crossover"} \tab RoB 2 tool for crossover
  trials \cr
\code{tool = "ROBINS-I"} \tab Risk Of Bias In Non-randomized
  Studies - of Interventions \cr
\tab (Sterne et al., 2016) \cr
\code{tool = "ROBINS-E"} \tab Risk Of Bias In Non-randomized
  Studies - of Exposures \cr
\tab (ROBINS-E Development Group, 2023)
}
These RoB tools are described on the website
\url{https://www.riskofbias.info/}.

\subsection{Risk of bias domains}{

By default, i.e., if argument \code{domains} is not provided by the
user, the following names are used for RoB domains.

\itemize{
\item RoB 1 tool for randomized studies (RoB1):

\enumerate{
 \item Random sequence generation (selection bias)
 \item Allocation concealment (selection bias)
 \item Blinding of participants and personnel (performance bias)
 \item Blinding of outcome assessment (detection bias)
 \item Incomplete outcome data (attrition bias)
 \item Selective reporting (reporting bias)
 \item Other bias
}

\item RoB 2 tool for randomized studies (RoB2):
\enumerate{
 \item Bias arising from the randomization process
 \item Bias due to deviations from intended intervention"
 \item Bias due to missing outcome data
 \item Bias in measurement of the outcome
 \item Bias in selection of the reported result
}

\item RoB 2 tool for cluster-randomized trials (RoB2-cluster):
\enumerate{
 \item Bias arising from the randomization process
 \item Bias arising from the identification or recruitment of
   participants into clusters
 \item Bias due to deviations from intended intervention
 \item Bias due to missing outcome data
 \item Bias in measurement of the outcome
 \item Bias in selection of the reported result
}

\item RoB 2 tool for crossover trials (RoB2-crossover)
\enumerate{
 \item Bias arising from the randomization process
 \item Bias arising from period and carryover effects
 \item Bias due to deviations from intended intervention
 \item Bias due to missing outcome data
 \item Bias in measurement of the outcome
 \item Bias in selection of the reported result
}

\item Risk Of Bias In Non-randomized Studies - of Intervention (ROBINS-I):
\enumerate{
 \item Risk of bias due to confounding
 \item Risk of bias in selection of participants into the study
 \item Risk of bias in classification of interventions
 \item Risk of bias due to deviations from intented interventions
 \item Risk of bias due to missing outcome data
 \item Risk of bias in measurement of the outcome
 \item Risk of bias in the selection of the reported results
}

\item Risk Of Bias In Non-randomized Studies - of Exposures (ROBINS-E):
\enumerate{
 \item Risk of bias due to confounding
 \item Risk of bias arising from measurement of the exposure into
   the study (or into the analysis)
 \item Risk of bias due to post-exposure interventions
 \item Risk of bias due to deviations from intented interventions
 \item Risk of bias due to missing outcome data
 \item Risk of bias in measurement of the outcome
 \item Risk of bias in the selection of the reported results
}

\item User-defined RoB assessment:
\enumerate{
 \item First item
 \item Second item
 \item \dots
}
}

It is possible to define additional bias domains for the available
RoB tools. In this case, only the names for new RoB domains have to
be provided in argument \code{domains}. If argument \code{domains}
is not used to specify new domains, the names "Additional item 1"
etc. will be used. It is also possible to modify the pre-defined
domain names using argument \code{domains}.

The maximum number of bias domains / items is ten (see arguments
\code{item1}, ..., \code{item10}).
}

\subsection{Risk of bias categories, colours and symbols}{

By default, the following settings are used.

RoB 1 tool:

\tabular{ll}{
\bold{Argument} \tab \bold{Values} \cr
\code{categories} \tab "Low risk of bias", "Unclear risk of bias",
  "High risk of bias" \cr
\code{col} \tab "green", "yellow", "red" \cr
\code{symbols} \tab "+", "?", "-"
}

RoB 2 tools:

\tabular{ll}{
\bold{Argument} \tab \bold{Values} \cr
\code{categories} \tab "Low risk of bias", "Some concerns",
  "High risk of bias" \cr
\code{col} \tab "green", "yellow", "red" \cr
\code{symbols} \tab "+", "?", "-"
}

ROBINS tools:

\tabular{ll}{
\bold{Argument} \tab \bold{Values} \cr
\code{categories} \tab "Low risk", "Some concerns", "High risk",
  "Very high risk", "NI" \cr
\code{col} \tab "green", "yellow", "red", "darkred", "darkgrey" \cr
\code{symbols} \tab none
}

User-defined RoB tools:

\tabular{ll}{
\bold{Argument} \tab \bold{Values} \cr
\code{categories} \tab Must be specified by the user \cr
\code{col} \tab 1, 2, ... \cr
\code{symbols} \tab none
}

If colours (\code{col}) and symbols (\code{symbols}) are provided,
they must be of the same length as the number of categories. 
}
}
\examples{
# Use RevMan 5 settings
oldset <- settings.meta("RevMan5", quietly = FALSE)

data(caffeine)

ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
  data = caffeine, studlab = paste(study, year))

# Add risk of bias assessment to meta-analysis
ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")

# Print risk of bias assessment
rob(ma2)

# Forest plot with risk of bias assessment
forest(ma2)

# Use previous settings
settings.meta(oldset)

}
\references{
Higgins JPT, Altman DG, Gøtzsche PC, Jüni P, Moher D, Oxman AD et
al. (2011):
The Cochrane Collaboration's tool for assessing risk of bias in
randomised trials.
\emph{British Medical Journal}, \bold{343}: d5928

ROBINS-E Development Group (Higgins J, Morgan R, Rooney A et al.) (2023):
Risk Of Bias In Non-randomized Studies - of Exposure (ROBINS-E)
Available from: \url{https://www.riskofbias.info/welcome/robins-e-tool}.

Sterne JA, Hernán MA, Reeves BC, Savović J, Berkman ND, Viswanathan
M, et al. (2016):
ROBINS-I: a tool for assessing risk of bias in non-randomised
studies of interventions.
\emph{British Medical Journal}, \bold{355}: i4919

Sterne JAC, Savović J, Page MJ, Elbers RG, Blencowe NS, Boutron I,
et al. (2019):
RoB 2: a revised tool for assessing risk of bias in randomised trials.
\emph{British Medical Journal}, \bold{366}: l4898.
}
\seealso{
\code{\link{forest.meta}}, \code{\link{barplot.rob}},
  \code{\link{traffic_light}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metarate.R
\name{metarate}
\alias{metarate}
\title{Meta-analysis of single incidence rates}
\usage{
metarate(
  event,
  time,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  n = NULL,
  method = "Inverse",
  sm = gs("smrate"),
  incr = gs("incr"),
  method.incr = gs("method.incr"),
  method.ci = gs("method.ci.rate"),
  level = gs("level"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau,
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  null.effect = NA,
  method.bias = gs("method.bias"),
  backtransf = gs("backtransf"),
  irscale = 1,
  irunit = "person-years",
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  hakn,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{event}{Number of events.}

\item{time}{Person time at risk.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information, i.e., event and time.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{n}{Number of observations.}

\item{method}{A character string indicating which method is to be
used for pooling of studies. One of \code{"Inverse"} and
\code{"GLMM"}, can be abbreviated.}

\item{sm}{A character string indicating which summary measure
(\code{"IR"}, \code{"IRLN"}, \code{"IRS"}, or \code{"IRFT"}) is
to be used for pooling of studies, see Details.}

\item{incr}{A numeric which is added to the event number of studies
with zero events, i.e., studies with an incidence rate of 0. Or a
numeric vector with the continuity correction for each study.}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, or \code{"all"}), see Details.}

\item{method.ci}{A character string indicating whether to use
approximate normal ("NAsm") or exact Poisson ("Poisson")
confidence limits.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, or
\code{"Thompson"}, can be abbreviated. See function
\code{\link{metabias}}.}

\item{backtransf}{A logical indicating whether results for
transformed rates (argument \code{sm != "IR"}) should be back
transformed in printouts and plots. If TRUE (default), results
will be presented as incidence rates; otherwise transformed rates
will be shown.}

\item{irscale}{A numeric defining a scaling factor for printing of
rates.}

\item{irunit}{A character string specifying the time unit used to
calculate rates, e.g. person-years.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{hakn}{Deprecated argument (replaced by 'method.random.ci').}

\item{adhoc.hakn}{Deprecated argument (replaced by 'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if \code{incr} is added to studies with zero cell
frequencies or if estimation problems exist in fitting a GLMM).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}} or
\code{\link[metafor]{rma.glmm}}, respectively.}

\item{\dots}{Additional arguments passed on to
\code{\link[metafor]{rma.glmm}} function and to catch deprecated
arguments.}
}
\value{
An object of class \code{c("metarate", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of an overall incidence rate from studies reporting a
single incidence rate. Inverse variance method and generalised
linear mixed model (GLMM) are available for pooling. For GLMMs, the
\code{\link[metafor]{rma.glmm}} function from R package
\bold{metafor} (Viechtbauer 2010) is called internally.
}
\details{
This function provides methods for common effect and random effects
meta-analysis of single incidence rates to calculate an overall
rate. Note, you should use R function \code{\link{metainc}} to
compare incidence rates of pairwise comparisons instead of using
\code{metarate} for each treatment arm separately which will break
randomisation in randomised controlled trials.

The following transformations of incidence rates are implemented to
calculate an overall rate:

\itemize{
\item Log transformation (\code{sm = "IRLN"}, default)
\item Square root transformation (\code{sm = "IRS"})
\item Freeman-Tukey Double arcsine transformation (\code{sm =
  "IRFT"})
\item No transformation (\code{sm = "IR"})
}

List elements \code{TE}, \code{TE.common}, \code{TE.random}, etc.,
contain the transformed incidence rates. In printouts and plots
these values are back transformed if argument \code{backtransf =
TRUE} (default).

By default (argument \code{method = "Inverse"}), the inverse
variance method (Borenstein et al., 2010) is used for pooling by
calling \code{\link{metagen}} internally. A random intercept
Poisson regression model (Stijnen et al., 2010) can be utilised
instead with argument \code{method = "GLMM"} which calls the
\code{\link[metafor]{rma.glmm}} function from R package
\bold{metafor}.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

\subsection{Continuity correction}{

Three approaches are available to apply a continuity correction:
\itemize{
\item Only studies with a zero cell count (\code{method.incr =
  "only0"})
\item All studies if at least one study has a zero cell count
  (\code{method.incr = "if0all"})
\item All studies irrespective of zero cell counts
  (\code{method.incr = "all"})
}

If the summary measure (argument \code{sm}) is equal to "IR" or
"IRLN", the continuity correction is applied if a study has zero
events, i.e., an incidence rate of 0.

By default, 0.5 is used as continuity correction (argument
\code{incr}). This continuity correction is used both to calculate
individual study results with confidence limits and to conduct
meta-analysis based on the inverse variance method.

For the Freeman-Tukey (Freeman & Tukey, 1950) and square root
transformation as well as GLMMs no continuity correction is used.
Furthermore, the value of \code{incr} is not considered for Poisson
confidence intervals for individual studies (\code{method.ci = "Poisson"}).
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Specify the null hypothesis of test for an overall effect}{

Argument \code{null.effect} can be used to specify the rate used
under the null hypothesis in a test for an overall effect.

By default (\code{null.effect = NA}), no hypothesis test is
conducted as it is unclear which value is a sensible choice for the
data at hand.  An overall rate of 2, for example, could be tested
by setting argument \code{null.effect = 2}.

Note, all tests for an overall effect are two-sided with the
alternative hypothesis that the effect is unequal to
\code{null.effect}.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments
\code{common} and \code{random}. Accordingly, the estimate
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"meta"} even if
argument \code{random = FALSE}. However, all functions in R
package \bold{meta} will adequately consider the values for
\code{common} and \code{random}. E.g. function
\code{\link{print.meta}} will not print results for the random
effects model if \code{random = FALSE}.

Argument \code{irscale} can be used to rescale rates, e.g.
\code{irscale = 1000} means that rates are expressed as events per
1000 time units, e.g. person-years. This is useful in situations
with (very) low rates. Argument \code{irunit} can be used to
specify the time unit used in individual studies (default:
"person-years"). This information is printed in summaries and
forest plots if argument \code{irscale} is not equal to 1.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\examples{
# Apply various meta-analysis methods to estimate incidence rates
#
ma1 <- metarate(4:1, c(10, 20, 30, 40))
ma2 <- update(ma1, sm = "IR")
ma3 <- update(ma1, sm = "IRS")
ma4 <- update(ma1, sm = "IRFT")
#
ma1
ma2
ma3
ma4
#
forest(ma1)
forest(ma1, irscale = 100)
forest(ma1, irscale = 100, irunit = "person-days")
forest(ma1, backtransf = FALSE)
\dontrun{
forest(ma2)
forest(ma3)
forest(ma4)
}

ma5 <- metarate(40:37, c(100, 200, 300, 400), sm = "IRFT")
ma5

}
\references{
Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
A basic introduction to fixed-effect and random-effects models for
meta-analysis.
\emph{Research Synthesis Methods},
\bold{1}, 97--111

Freeman MF & Tukey JW (1950):
Transformations related to the angular and the square root.
\emph{Annals of Mathematical Statistics},
\bold{21}, 607--11

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Viechtbauer W (2010):
Conducting Meta-Analyses in R with the Metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{metacont}}, \code{\link{metagen}},
  \code{\link{print.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimates.R
\name{estimates.meta}
\alias{estimates.meta}
\alias{estimates}
\alias{print.estimates.meta}
\title{Extract results from meta-analysis object}
\usage{
\method{estimates}{meta}(
  x,
  sortvar,
  study.results = TRUE,
  common = x$common,
  random = x$random,
  prediction = x$prediction,
  overall = x$overall,
  subgroup,
  prediction.subgroup = x$prediction.subgroup,
  se = FALSE,
  ci = TRUE,
  statistic = FALSE,
  pval = FALSE,
  n = TRUE,
  backtransf = x$backtransf,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  writexl = !missing(path),
  path = "estimates.xlsx",
  overwrite = FALSE,
  ...
)

estimates(x, ...)

\method{print}{estimates.meta}(
  x,
  digits.tau = gs("digits.tau"),
  text.tau2 = gs("text.tau2"),
  text.tau = gs("text.tau"),
  big.mark = gs("big.mark"),
  details = TRUE,
  ...
)
}
\arguments{
\item{x}{A meta-analysis object of class \code{meta}.}

\item{sortvar}{An optional vector used to sort the individual
studies (must be of same length as \code{x$TE}).}

\item{study.results}{A logical indicating whether study results
should be extracted.}

\item{common}{A logical indicating whether results of common effect
meta-analysis should be extracted.}

\item{random}{A logical indicating whether results of random
effects meta-analysis should be extracted.}

\item{prediction}{A logical indicating whether prediction interval
should be extracted.}

\item{overall}{A logical indicating whether overall summaries
should be extracted. This argument is useful in a meta-analysis
with subgroups if overall results should not be extracted.}

\item{subgroup}{A logical indicating whether subgroup results
should be extracted.}

\item{prediction.subgroup}{A single logical or logical vector
indicating whether / which prediction intervals should be
extracted for subgroups.}

\item{se}{A logical indicating whether standard errors should be
extracted.}

\item{ci}{A logical indicating whether confidence / prediction
interval should be extracted.}

\item{statistic}{A logical indicating whether to extract statistic
of test for overall effect.}

\item{pval}{A logical indicating whether to extract p-value of test
for overall effect.}

\item{n}{A logical indicating whether sample sizes should be
extracted (if available).}

\item{backtransf}{A logical indicating whether extracted results
should be back transformed.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors, see \code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-statistic for test of overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect, see \code{print.default}.}

\item{writexl}{A logical indicating whether an Excel file should be
created (R package \bold{writexl} must be available).}

\item{path}{A character string specifying the filename of the Excel
file.}

\item{overwrite}{A logical indicating whether an existing Excel
file should be overwritten.}

\item{\dots}{Additional arguments passed on to \code{prmatrix}.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance, see \code{print.default}.}

\item{text.tau2}{Text printed to identify between-study variance
\eqn{\tau^2}.}

\item{text.tau}{Text printed to identify \eqn{\tau}, the square
root of the between-study variance \eqn{\tau^2}.}

\item{big.mark}{A character used as thousands separator.}

\item{details}{A logical specifying whether details on statistical
methods should be printed.}
}
\value{
A data frame with additional class 'extract.meta' or an Excel file.
}
\description{
Extract study and meta-analysis results from meta-analysis object
which can be stored in an Excel file.
}
\details{
Extract study and meta-analysis results from meta-analysis object.
By default, a data frame with the results is
generated. Alternatively, an Excel file is created if argument
\code{writexl = TRUE}.

The following information is extracted.

\tabular{ll}{
\bold{Variable} \tab \bold{Content} \cr
studlab \tab Study label / descriptor of meta-analysis result \cr
estimate \tab (Back transformed) estimate for individual studies /
  meta-analysis \cr
se \tab Standard error \cr
lower \tab Lower (back transformed) confidence / prediction
  interval limit \cr
upper \tab Upper (back transformed) confidence / prediction
  interval limit \cr
k \tab Number of studies \cr
df \tab Degrees of freedom for confidence / prediction intervals \cr
statistic \tab Statistic for test of effect \cr
pval \tab P-value for test of effect \cr
n \tab Total sample size \cr
n.e \tab Sample size in first (experimental) group \cr
n.c \tab Sample size in second (control) group
}

Some variables are only extracted if the corresponding logical
argument \code{se}, \code{ci}, \code{statistic} or \code{n} is
\code{TRUE}. Furthermore, (group) sample sizes are only extracted
if available in the meta-analysis object.

The variables \code{estimate}, \code{lower} and \code{upper}
contain the back transformed effects and confidence interval
limits, e.g., odds ratio (argument \code{sm = "OR"} in
\code{\link{metabin}} or \code{\link{metagen}}) or correlation
(\code{sm = "ZCOR"} in \code{\link{metacor}} or
\code{\link{metagen}}), if argument \code{backtransf} is
\code{TRUE}. Otherwise, these variables contain the transformed
values, e.g., log odds ratio (\code{sm = "OR"}) or Fisher's Z
transformed correlations (\code{sm = "ZCOR"}). See
\code{\link{meta-sm}} for available summary measures and
\code{\link{meta-transf}} for the corresponding transformations and
back transformations.
}
\examples{
ma <- metacor(c(0.85, 0.7, 0.95), c(20, 40, 10))
summary(ma)
estimates(ma)
estimates(ma, backtransf = FALSE)
estimates(update(ma, common = FALSE, random = FALSE))
estimates(update(ma, prediction = TRUE))
estimates(update(ma, prediction = TRUE,
  level.ma = 0.99, level.predict = 0.9))

\dontrun{
# Create Excel file with extracted results
# (R package 'writexl' must be available)
if (requireNamespace("writexl", quietly = TRUE)) {
 fname1 <- tempfile(fileext = ".xlsx")
 estimates(ma, path = fname1)
 # An existing Excel file is not overwritten but a warning is printed
 estimates(ma, path = fname1)
 # Overwrite an existing Excel file
 estimates(ma, path = fname1, overwrite = TRUE)
 # Suppress message on file creation and overwrite existing file
 suppressMessages(estimates(ma, path = fname1, overwrite = TRUE))
}

# Save the extracted results in a text file
fname2 <- tempfile(fileext = ".csv")
fname2
write.csv(estimates(ma), file = fname2, row.names = FALSE)
}

}
\seealso{
\code{\link{meta-transf}}, \code{\link{meta-sm}},
  \code{\link{as.data.frame.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Fleiss1993cont.R
\docType{data}
\name{Fleiss1993cont}
\alias{Fleiss1993cont}
\alias{Fleiss93cont}
\title{Mental Health Treatment}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{year}}\tab year of publication \cr
\bold{\emph{n.psyc}}\tab number of observations in psychotherapy
  group \cr
\bold{\emph{mean.psyc}}\tab estimated mean in psychotherapy group
  \cr
\bold{\emph{sd.psyc}}\tab standard deviation in psychotherapy group
  \cr
\bold{\emph{n.cont}}\tab number of observations in control group
  \cr
\bold{\emph{mean.cont}}\tab estimated mean in control group \cr
\bold{\emph{sd.cont}}\tab standard deviation in control group
}
}
\source{
Fleiss JL (1993):
The statistical basis of meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{2}, 121--45
}
\description{
Meta-analysis on the Effect of Mental Health Treatment on Medical
Utilisation.

Data example in Fleiss (1993) for meta-analysis with continuous outcomes.
}
\examples{
data(Fleiss1993cont)
# Note, the following command uses the bias-corrected version of
# Hedges' g. Accordingly, results differ from Fleiss (1993), section 3,
# using the uncorrected version of Hedges' g.
metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, studlab = paste(study, year),
  random = FALSE, sm = "SMD")
}
\seealso{
\code{\link{Fleiss1993bin}}

\code{\link{metacont}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metaadd.R
\name{metaadd}
\alias{metaadd}
\title{Add pooled results from external analysis to meta-analysis}
\usage{
metaadd(
  x,
  type = NULL,
  TE = NA,
  lower = NA,
  upper = NA,
  statistic = NA,
  pval = NA,
  df = NA,
  se = NA,
  method = "",
  method.ci = "",
  text = "Added result",
  data = NULL,
  transf = gs("transf")
)
}
\arguments{
\item{x}{Meta-analysis object.}

\item{type}{A character string or vector indicating whether added
results are from common effect, random effects model or
prediction interval. Either \code{"common"}, \code{"random"},
\code{"prediction"}, or \code{"tau2"} can be abbreviated.}

\item{TE}{Pooled estimate(s) or between-study variance.}

\item{lower}{Lower limit(s) of confidence or prediction interval.}

\item{upper}{Upper limit(s) of confidence or prediction interval.}

\item{statistic}{Test statistic(s).}

\item{pval}{P-value(s).}

\item{df}{Degrees of freedom for confidence or prediction intervals.s}

\item{se}{Standard error(s).}

\item{method}{A character string or vector to describe the
method used to get the pooled estimate(s), prediction interval(s) or
between-study variance(s).}

\item{method.ci}{A character string or vector to describe the
method used to get the confidence or prediction interval.}

\item{text}{A character string or vector used in printouts and
forest plot to label the added results.}

\item{data}{An optional data frame containing the new results or an
object of class \code{meta}.}

\item{transf}{A logical indicating whether inputs for arguments
\code{TE}, \code{lower} and \code{upper} are already
appropriately transformed to conduct the meta-analysis or on the
original scale. If \code{transf = TRUE} (default), inputs are
expected to be log odds ratios instead of odds ratios for
\code{sm = "OR"} and Fisher's z transformed correlations instead
of correlations for \code{sm = "ZCOR"}, for example.}
}
\value{
An object of class \code{"meta"} with corresponding generic
functions (see \code{\link{meta-object}}).
}
\description{
Add pooled results from external analysis to an existing
meta-analysis object. This is useful, for example, to add results
from a Bayesian meta-analysis which is not implemented in R package
\bold{meta}.
}
\details{
In R package \bold{meta}, objects of class \code{"meta"} contain
results of both common effect and random effects
meta-analyses. This function enables the user to add the pooled
results of an additional analysis to an existing meta-analysis
object. This is useful, for example, to add the result of a
Bayesian meta-analysis.

If argument \code{data} is a meta-analysis object created with R
package \bold{meta}, arguments \code{TE}, \code{lower},
\code{upper}, \code{statistic} and \code{pval} are ignored as this
information is extracted from the meta-analysis.

Otherwise, arguments \code{TE}, \code{lower} and \code{upper} have
to be provided if \code{type = "common"} or \code{type =
"random"}. For \code{type = "prediction"}, only arguments
\code{lower} and \code{upper} are mandatory.

Note, R function \code{\link{metamerge}} can be used to add
meta-analysis results of another meta-analysis object (see
\code{\link{meta-object}}).
}
\examples{
data(Fleiss1993bin)

# Common effect and random effects meta-analysis
ma1 <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
  studlab = paste(study, year), sm = "OR")

# Naive pooling
ma2 <- metabin(sum(d.asp), sum(n.asp), sum(d.plac), sum(n.plac),
  data = Fleiss1993bin, sm = "OR", text.common = "Naive pooling")

# Add results of second meta-analysis from common effect model
ma12 <- metaadd(ma1, data = ma2)
ma12

forest(ma12)

}
\seealso{
\code{\link{metagen}}, \code{\link{metamerge}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.R
\name{forest.meta}
\alias{forest.meta}
\alias{forest}
\alias{plot.meta}
\alias{.forestArgs}
\title{Forest plot to display the result of a meta-analysis}
\usage{
\method{forest}{meta}(
  x,
  sortvar,
  studlab = TRUE,
  layout = gs("layout"),
  common = x$common,
  random = x$random,
  overall = x$overall,
  text.common = x$text.common,
  text.random = x$text.random,
  lty.common = gs("lty.common"),
  lty.random = gs("lty.random"),
  col.common = gs("col.common"),
  col.random = gs("col.random"),
  text.w.common = x$text.w.common,
  text.w.random = x$text.w.random,
  prediction = x$prediction,
  text.predict = x$text.predict,
  subgroup = TRUE,
  subgroup.hetstat = subgroup & (is.character(hetstat) || hetstat),
  print.subgroup.labels = TRUE,
  subgroup.name = x$subgroup.name,
  print.subgroup.name = x$print.subgroup.name,
  sep.subgroup = x$sep.subgroup,
  text.subgroup = NULL,
  text.common.w = text.common,
  text.random.w = text.random,
  text.predict.w = text.predict,
  sort.subgroup = gs("sort.subgroup"),
  pooled.totals = common | random,
  pooled.events = gs("pooled.events"),
  pooled.times = gs("pooled.times"),
  study.results = gs("study.results"),
  rob = x$rob,
  rob.text = "Risk of Bias",
  rob.xpos = 0,
  rob.legend = TRUE,
  rob.only = FALSE,
  xlab = "",
  xlab.pos,
  smlab = NULL,
  smlab.pos,
  xlim,
  allstudies = TRUE,
  weight.study = NULL,
  pscale = x$pscale,
  irscale = x$irscale,
  irunit = x$irunit,
  file = NULL,
  width = gs("width"),
  rows.gr = NULL,
  func.gr = NULL,
  args.gr = NULL,
  dev.off = NULL,
  ref,
  cid = gs("cid"),
  cid.below.null = gs("cid.below.null"),
  cid.above.null = gs("cid.above.null"),
  lty.cid = gs("lty.cid"),
  col.cid = gs("col.cid"),
  fill.cid = gs("fill.cid"),
  fill.cid.below.null = fill.cid,
  fill.cid.above.null = rev(fill.cid),
  cid.pooled.only = gs("cid.pooled.only"),
  fill = gs("fill"),
  fill.equi = gs("fill.equi"),
  fill.lower.equi = fill.equi,
  fill.upper.equi = rev(fill.equi),
  leftcols = gs("leftcols"),
  rightcols = gs("rightcols"),
  leftlabs = gs("leftlabs"),
  rightlabs = gs("rightlabs"),
  label.e = x$label.e,
  label.c = x$label.c,
  label.e.attach = gs("label.e.attach"),
  label.c.attach = gs("label.c.attach"),
  label.left = x$label.left,
  label.right = x$label.right,
  bottom.lr = gs("bottom.lr"),
  lab.NA = gs("lab.NA"),
  lab.NA.effect = gs("lab.NA.effect"),
  lab.NA.weight = gs("lab.NA.weight"),
  lwd = gs("lwd"),
  at = NULL,
  label = TRUE,
  col.label = gs("col.label"),
  type.study = gs("type.study"),
  type.common = gs("type.common"),
  type.random = type.common,
  type.subgroup = ifelse(study.results, "diamond", "square"),
  type.subgroup.common = type.subgroup,
  type.subgroup.random = type.subgroup,
  col.study = gs("col.study"),
  col.square = gs("col.square"),
  col.square.lines = gs("col.square.lines"),
  col.circle = gs("col.circle"),
  col.circle.lines = col.circle,
  col.inside = gs("col.inside"),
  col.inside.common = col.inside,
  col.inside.random = col.inside,
  col.diamond = gs("col.diamond"),
  col.diamond.common = col.diamond,
  col.diamond.random = col.diamond,
  col.diamond.lines = gs("col.diamond.lines"),
  col.diamond.lines.common = col.diamond.lines,
  col.diamond.lines.random = col.diamond.lines,
  col.predict = gs("col.predict"),
  col.predict.lines = gs("col.predict.lines"),
  col.subgroup = gs("col.subgroup"),
  col.label.left = x$col.label.left,
  col.label.right = x$col.label.right,
  hetstat = common | random | overall.hetstat,
  overall.hetstat = x$overall.hetstat & !inherits(x, "metamerge"),
  hetlab = gs("hetlab"),
  resid.hetstat = gs("resid.hetstat"),
  resid.hetlab = gs("resid.hetlab"),
  print.I2 = gs("forest.I2"),
  print.I2.ci = gs("forest.I2.ci"),
  print.tau2 = gs("forest.tau2"),
  print.tau2.ci = gs("forest.tau2.ci"),
  print.tau = gs("forest.tau"),
  print.tau.ci = gs("forest.tau.ci"),
  print.Q = gs("forest.Q"),
  print.pval.Q = gs("forest.pval.Q"),
  print.Rb = gs("forest.Rb"),
  print.Rb.ci = gs("forest.Rb.ci"),
  label.n = gs("label.n"),
  label.events = gs("label.events"),
  label.mean = gs("label.mean"),
  label.sd = gs("label.sd"),
  label.cor = gs("label.cor"),
  label.time = gs("label.time"),
  label.pval = gs("label.pval"),
  label.tau2 = gs("label.tau2"),
  label.tau = gs("label.tau"),
  label.I2 = gs("label.I2"),
  label.cluster = gs("label.cluster"),
  label.cycles = gs("label.cycles"),
  text.subgroup.nohet = gs("text.subgroup.nohet"),
  LRT = gs("LRT"),
  test.overall = gs("test.overall"),
  test.overall.common = common & overall & test.overall,
  test.overall.random = random & overall & test.overall,
  label.test.overall.common,
  label.test.overall.random,
  print.stat = gs("forest.stat"),
  test.subgroup = x$test.subgroup,
  test.subgroup.common = test.subgroup & common,
  test.subgroup.random = test.subgroup & random,
  common.subgroup = common,
  random.subgroup = random,
  prediction.subgroup = x$prediction.subgroup,
  print.tau2.ci.subgroup = gs("print.tau2.ci.subgroup"),
  print.tau.ci.subgroup = gs("print.tau.ci.subgroup"),
  print.Q.subgroup = gs("forest.Q.subgroup"),
  label.test.subgroup.common,
  label.test.subgroup.random,
  test.effect.subgroup = gs("test.effect.subgroup"),
  test.effect.subgroup.common,
  test.effect.subgroup.random,
  label.test.effect.subgroup.common,
  label.test.effect.subgroup.random,
  text.addline1,
  text.addline2,
  details = gs("forest.details"),
  col.lines = gs("col.lines"),
  header.line,
  col.header.line = col.lines,
  col.jama.line = col.subgroup,
  data.pooled = NULL,
  fontsize = gs("fontsize"),
  fontfamily = gs("fontfamily"),
  fs.heading = fontsize,
  fs.common = gs("fs.common"),
  fs.random = gs("fs.random"),
  fs.predict = gs("fs.predict"),
  fs.common.labels = gs("fs.common.labels"),
  fs.random.labels = gs("fs.random.labels"),
  fs.predict.labels = gs("fs.predict.labels"),
  fs.study = fontsize,
  fs.study.labels = fs.study,
  fs.hetstat = gs("fs.hetstat"),
  fs.test.overall = gs("fs.test.overall"),
  fs.test.subgroup = gs("fs.test.subgroup"),
  fs.test.effect.subgroup = gs("fs.test.effect.subgroup"),
  fs.addline = gs("fs.addline"),
  fs.axis = fontsize,
  fs.smlab = fontsize,
  fs.xlab = fontsize,
  fs.lr = fontsize,
  fs.rob = fontsize,
  fs.rob.symbols = fontsize,
  fs.details = fontsize,
  ff.heading = "bold",
  ff.common = gs("ff.common"),
  ff.random = gs("ff.random"),
  ff.predict = gs("ff.predict"),
  ff.common.labels = gs("ff.common.labels"),
  ff.random.labels = gs("ff.random.labels"),
  ff.predict.labels = gs("ff.predict.labels"),
  ff.study = "plain",
  ff.study.labels = ff.study,
  ff.hetstat = gs("ff.hetstat"),
  ff.test.overall = gs("ff.test.overall"),
  ff.test.subgroup = gs("ff.test.subgroup"),
  ff.test.effect.subgroup = gs("ff.test.effect.subgroup"),
  ff.addline = gs("ff.addline"),
  ff.axis = gs("ff.axis"),
  ff.smlab = gs("ff.smlab"),
  ff.xlab = gs("ff.xlab"),
  ff.lr = gs("ff.lr"),
  ff.rob = "plain",
  ff.rob.symbols = "bold",
  ff.details = "plain",
  squaresize = if (layout == "BMJ") 0.9/spacing else 0.8/spacing,
  lwd.square = gs("lwd.square"),
  lwd.diamond = gs("lwd.diamond"),
  arrow.type = gs("arrow.type"),
  arrow.length = gs("arrow.length"),
  plotwidth = if (layout \%in\% c("BMJ", "JAMA")) "8cm" else "6cm",
  colgap = gs("colgap"),
  colgap.left = colgap,
  colgap.right = colgap,
  colgap.studlab = colgap.left,
  colgap.forest = gs("colgap.forest"),
  colgap.forest.left = colgap.forest,
  colgap.forest.right = colgap.forest,
  colgap.rob = "1mm",
  colgap.rob.overall = "2mm",
  calcwidth.pooled = (common | random) & (overall | !is.null(x$subgroup)),
  calcwidth.common = calcwidth.pooled,
  calcwidth.random = calcwidth.pooled,
  calcwidth.predict = gs("calcwidth.predict"),
  calcwidth.hetstat = gs("calcwidth.hetstat"),
  calcwidth.tests = gs("calcwidth.tests"),
  calcwidth.subgroup = gs("calcwidth.subgroup"),
  calcwidth.addline = gs("calcwidth.addline"),
  just = if (layout == "JAMA") "left" else "right",
  just.studlab = gs("just.studlab"),
  just.addcols = gs("just.addcols"),
  just.addcols.left = just.addcols,
  just.addcols.right = just.addcols,
  just.label.e = just,
  just.label.c = just,
  bmj.text = NULL,
  bmj.xpos = 0,
  bmj.sep = " / ",
  spacing = gs("spacing"),
  addrow = gs("addrow"),
  addrow.overall = gs("addrow.overall"),
  addrow.subgroups = gs("addrow.subgroups"),
  addrows.below.overall = gs("addrows.below.overall"),
  new = TRUE,
  backtransf = x$backtransf,
  digits = gs("digits.forest"),
  digits.se = gs("digits.se"),
  digits.stat = gs("digits.stat"),
  digits.pval = gs("digits.pval"),
  digits.pval.Q = gs("digits.pval.Q"),
  digits.Q = gs("digits.Q"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.I2 = gs("digits.I2"),
  digits.weight = gs("digits.weight"),
  digits.mean = gs("digits.mean"),
  digits.sd = gs("digits.sd"),
  digits.cor = digits,
  digits.time = digits,
  digits.n = 0,
  digits.event = 0,
  digits.TE = gs("digits.TE.forest"),
  digits.addcols = digits,
  digits.addcols.right = digits.addcols,
  digits.addcols.left = digits.addcols,
  scientific.pval = gs("scientific.pval"),
  big.mark = gs("big.mark"),
  zero.pval = if (layout == "JAMA") FALSE else gs("zero.pval"),
  JAMA.pval = if (layout == "JAMA") TRUE else gs("JAMA.pval"),
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{plot}{meta}(x, ...)

.forestArgs()
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{sortvar}{An optional vector used to sort the individual
studies (must be of same length as \code{x$TE}).}

\item{studlab}{A logical indicating whether study labels should be
printed in the graph. A vector with study labels can also be
provided (must be of same length as the vector with estimates \code{x$TE}).}

\item{layout}{A character string specifying the layout of the
forest plot (see Details).}

\item{common}{A logical indicating whether common effect estimate
should be plotted.}

\item{random}{A logical indicating whether random effects estimate
should be plotted.}

\item{overall}{A logical indicating whether overall summaries
should be plotted. This argument is useful in a meta-analysis
with subgroups if summaries should only be plotted on group
level.}

\item{text.common}{A character string used in the plot to label the
pooled common effect estimate.}

\item{text.random}{A character string used in the plot to label the
pooled random effects estimate.}

\item{lty.common}{Line type of pooled common effect estimate.}

\item{lty.random}{Line type of pooled random effects estimate.}

\item{col.common}{Line colour of pooled common effect estimate.}

\item{col.random}{Line colour of pooled random effects estimate.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{text.predict}{A character string used in the plot to label
the prediction interval.}

\item{subgroup}{A single logical or logical vector indicating
whether / which subgroup results should be shown in forest
plot. This argument is useful in a meta-analysis with subgroups
if summaries should not be plotted for (some) subgroups.}

\item{subgroup.hetstat}{A single logical or logical vector
indicating whether / which information on heterogeneity in
subgroups should be shown in forest plot. This argument is useful
in a meta-analysis with subgroups if heterogeneity statistics
should not be printed for (some) subgroups.}

\item{print.subgroup.labels}{A logical indicating whether subgroup
label should be printed.}

\item{subgroup.name}{A character string with a label for the
grouping variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the grouping variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between label and levels of grouping variable.}

\item{text.subgroup}{A character vector or expression to label the subgroups.
The vector must be of the same length as the number of subgroups. Labels
are printed from top to bottom in the forest plot.}

\item{text.common.w}{A character string to label the pooled common
effect estimate within subgroups, or a character vector of same
length as number of subgroups with corresponging labels.}

\item{text.random.w}{A character string to label the pooled random
effect estimate within subgroups, or a character vector of same
length as number of subgroups with corresponging labels.}

\item{text.predict.w}{A character string to label the prediction
interval within subgroups, or a character vector of same length
as number of subgroups with corresponging labels.}

\item{sort.subgroup}{A logical indicating whether groups should be
ordered alphabetically.}

\item{pooled.totals}{A logical indicating whether total number of
observations should be given in the figure.}

\item{pooled.events}{A logical indicating whether total number of
events should be given in the figure.}

\item{pooled.times}{A logical indicating whether total person time
at risk should be given in the figure.}

\item{study.results}{A logical indicating whether results for
individual studies should be shown in the figure (useful to only
plot subgroup results).}

\item{rob}{Risk of bias (RoB) assessment.}

\item{rob.text}{Column heading for RoB table.}

\item{rob.xpos}{A numeric specifying the horizontal position of the
risk of bias label in RoB table heading. The value is a so called
normalised parent coordinate in the horizontal direction (see
\code{\link[grid]{unit}}).}

\item{rob.legend}{A logical specifying whether a legend with RoB
domains should be printed.}

\item{rob.only}{A logical indicating whether the risk of bias
assessment is the only information printed on the right side of
the forest plot.}

\item{xlab}{A label for the x-axis.}

\item{xlab.pos}{A numeric specifying the center of the label on the
x-axis.}

\item{smlab}{A label for the summary measure (printed at top of
figure).}

\item{smlab.pos}{A numeric specifying the center of the label for
the summary measure.}

\item{xlim}{The x limits (min,max) of the plot, or the character
string "symmetric" to produce symmetric forest plots.}

\item{allstudies}{A logical indicating whether studies with
inestimable treatment effects should be included in the forest
plot.}

\item{weight.study}{A character string indicating weighting used to
determine size of squares or diamonds (argument
\code{type.study}) to plot individual study results. One of
missing, \code{"same"}, \code{"common"}, or \code{"random"}, can
be abbreviated. Plot symbols have the same size for all studies
or represent study weights from common effect or random effects
model.}

\item{pscale}{A numeric giving scaling factor for printing of
single event probabilities or risk differences, i.e. if argument
\code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
\code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.}

\item{irscale}{A numeric defining a scaling factor for printing of
single incidence rates or incidence rate differences, i.e. if
argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
\code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.}

\item{irunit}{A character specifying the time unit used to
calculate rates, e.g., person-years.}

\item{file}{File name.}

\item{width}{Width of graphics file.}

\item{rows.gr}{Additional rows in forest plot to change height of
graphics file (e.g., in order to add a title at the top of the
forest plot).}

\item{func.gr}{Name of graphics function, e.g., \code{\link{pdf}}.}

\item{args.gr}{List with additional graphical parameters passed on
to graphics function (argument 'height' cannot be provided as the
height is calculated internally; use instead argument 'rows.gr').}

\item{dev.off}{A logical to specify whether current graphics device
should be shut down, i.e., whether file should be stored.}

\item{ref}{A numerical giving the reference value to be plotted as
a line in the forest plot. No reference line is plotted if
argument \code{ref} is equal to \code{NA}.}

\item{cid}{A numeric value or vector specifying clinically important
differences (CID) / decision thresholds used to calculate probabilities
of clinically important benefit or harm, or not important effects
(see Details).}

\item{cid.below.null}{A numeric value or vector specifying CID limits below
the null effect (see Details).}

\item{cid.above.null}{A numeric value or vector specifying CID limits above
the null effect (see Details).}

\item{lty.cid}{Line type for CID lines.}

\item{col.cid}{Line colour for CID lines.}

\item{fill.cid}{Colour(s) for regions below or above CID limits.}

\item{fill.cid.below.null}{Colour of CID regions below null effect /
reference value. Can be equal to the number of lower limits or
the number of limits plus 1 (in this case the region between
minimum and smallest limit is also filled).}

\item{fill.cid.above.null}{Colour of CID regions above null effect /
reference value. Can be equal to the number of upper limits or the
number of limits plus 1 (in this case the region between largest
limit and maximum is also filled).}

\item{cid.pooled.only}{A logical indicating whether CID regions should only
be visible for pooled estimates or also individual studies.}

\item{fill}{Colour for background of confidence interval plot (also used
as colour for region between CID limits if argument \code{fill.equi} was
not provided).}

\item{fill.equi}{Colour(s) for region between limits of equivalence defined
by arguments \code{cid}, \code{cid.lower} or \code{cid.upper}.}

\item{fill.lower.equi}{Colour of region between lower limit(s) and
reference value. Can be equal to the number of lower limits or
the number of limits plus 1 (in this case the the region between
minimum and smallest limit is also filled).}

\item{fill.upper.equi}{Colour of region between reference value and
upper limit(s). Can be equal to the number of upper limits or the
number of limits plus 1 (in this case the region between largest
limit and maximum is also filled).}

\item{leftcols}{A character vector specifying (additional) columns
to be printed on the left side of the forest plot or a logical
value (see Details).}

\item{rightcols}{A character vector specifying (additional) columns
to be printed on the right side of the forest plot or a logical
value (see Details).}

\item{leftlabs}{A character vector specifying labels for
(additional) columns on left side of the forest plot (see
Details).}

\item{rightlabs}{A character vector specifying labels for
(additional) columns on right side of the forest plot (see
Details).}

\item{label.e}{Label to be used for experimental group in table
heading.}

\item{label.c}{Label to be used for control group in table heading.}

\item{label.e.attach}{A character string or vector specifying the column
name(s) where label \code{label.e} should be attached to in table heading.}

\item{label.c.attach}{A character string or vector specifying the column
name(s) where label \code{label.c} should be attached to in table heading.}

\item{label.left}{Graph label on left side of null effect.}

\item{label.right}{Graph label on right side of null effect.}

\item{bottom.lr}{A logical indicating whether labels on right and
left side should be printed at bottom or top of forest plot.}

\item{lab.NA}{A character string to label missing values.}

\item{lab.NA.effect}{A character string to label missing values in
individual treatment estimates and confidence intervals.}

\item{lab.NA.weight}{A character string to label missing weights.}

\item{lwd}{The line width, see \code{\link{par}}.}

\item{at}{The points at which tick-marks are to be drawn, see
\code{grid.xaxis}.}

\item{label}{A logical value indicating whether to draw the labels
on the tick marks, or an expression or character vector which
specify the labels to use. See \code{\link{grid.xaxis}}.}

\item{col.label}{The colour of labels on the x-axis.}

\item{type.study}{A character string or vector specifying how to
plot treatment effects and confidence intervals for individual
studies (see Details).}

\item{type.common}{A character string specifying how to plot
treatment effect and confidence interval for common effect
meta-analysis (see Details).}

\item{type.random}{A character string specifying how to plot
treatment effect and confidence interval for random effects
meta-analysis (see Details).}

\item{type.subgroup}{A character string specifying how to plot
treatment effect and confidence interval for subgroup results
(see Details).}

\item{type.subgroup.common}{A character string specifying how to
plot treatment effect and confidence interval for subgroup
results (common effect model).}

\item{type.subgroup.random}{A character string specifying how to
plot treatment effect and confidence interval for subgroup
results (random effects model).}

\item{col.study}{The colour for individual study results and
confidence limits.}

\item{col.square}{The colour for squares reflecting study's weight
in the meta-analysis.}

\item{col.square.lines}{The colour for the outer lines of squares
reflecting study's weight in the meta-analysis.}

\item{col.circle}{The colour for circles reflecting study weights
in the meta-analysis.}

\item{col.circle.lines}{The colour for the outer lines of circles
reflecting study's weight in the meta-analysis.}

\item{col.inside}{The colour for individual study results and
confidence limits if confidence limits are completely within
squares.}

\item{col.inside.common}{The colour for result of common effect
meta-analysis if confidence limit lies completely within square.}

\item{col.inside.random}{The colour for result of random effects
meta-analysis if confidence limit lies completely within square.}

\item{col.diamond}{The colour of diamonds representing the results
for common effect and random effects models.}

\item{col.diamond.common}{The colour(s) of diamonds for common effect
estimates.}

\item{col.diamond.random}{The colour(s) of diamonds for random effects
estimates.}

\item{col.diamond.lines}{The colour of the outer lines of diamonds
representing the results for common effect and random effects
models.}

\item{col.diamond.lines.common}{The colour(s) of the outer lines of
diamond for common effect estimate.}

\item{col.diamond.lines.random}{The colour(s) of the outer lines of
diamond for random effects estimate.}

\item{col.predict}{Background colour(s) of prediction intervals.}

\item{col.predict.lines}{Colour(s) of outer lines of prediction
intervals.}

\item{col.subgroup}{The colour to print information on subgroups.}

\item{col.label.left}{The colour of the label on the left side of the null
effect.}

\item{col.label.right}{The colour of the label on the right side of the null
effect.}

\item{hetstat}{Either a logical value indicating whether to print
results for heterogeneity measures at all or a character string
(see Details).}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{hetlab}{Label printed in front of results for heterogeneity
measures.}

\item{resid.hetstat}{A logical value indicating whether to print
measures of residual heterogeneity in a meta-analysis with
subgroups.}

\item{resid.hetlab}{Label printed in front of results for residual
heterogeneity measures.}

\item{print.I2}{A logical value indicating whether to print the
value of the I-squared statistic.}

\item{print.I2.ci}{A logical value indicating whether to print the
confidence interval of the I-squared statistic.}

\item{print.tau2}{A logical value indicating whether to print the
value of the between-study variance \eqn{\tau^2}.}

\item{print.tau2.ci}{A logical value indicating whether to print
the confidence interval of \eqn{\tau^2}.}

\item{print.tau}{A logical value indicating whether to print
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{print.tau.ci}{A logical value indicating whether to print the
confidence interval of \eqn{\tau}.}

\item{print.Q}{A logical value indicating whether to print the
value of the heterogeneity statistic Q.}

\item{print.pval.Q}{A logical value indicating whether to print the
p-value of the heterogeneity statistic Q.}

\item{print.Rb}{A logical value indicating whether to print the
value of the I-squared statistic.}

\item{print.Rb.ci}{A logical value indicating whether to print the
confidence interval of the I-squared statistic.}

\item{label.n}{A character string specifying the column label for
the sample size.}

\item{label.events}{A character string specifying the column label for
the number of events.}

\item{label.mean}{A character string specifying the column label for
means.}

\item{label.sd}{A character string specifying the column label for
standard deviations.}

\item{label.cor}{A character string specifying the column label for
correlations.}

\item{label.time}{A character string specifying the column label for
times.}

\item{label.pval}{A character string specifying the column label for
p-values.}

\item{label.tau2}{A character string specifying the column label for
the between-study variance.}

\item{label.tau}{A character string specifying the column label for
the between-study standard deviation.}

\item{label.I2}{A character string specifying the column label for
the heterogeneity statistic I2.}

\item{label.cluster}{A character string specifying the column label for
the cluster variable.}

\item{label.cycles}{A character string specifying the column label for
the number of cycles.}

\item{text.subgroup.nohet}{A logical value or character string
which is printed to indicate subgroups with less than two studies
contributing to meta-analysis (and thus without
heterogeneity). If FALSE, heterogeneity statistics are printed
(with NAs).}

\item{LRT}{A logical value indicating whether to report
Likelihood-Ratio or Wald-type test of heterogeneity for
generalised linear mixed models.}

\item{test.overall}{A logical value indicating whether to print
results of test for overall effect.}

\item{test.overall.common}{A logical value indicating whether to
print results of test for overall effect (common effect model).}

\item{test.overall.random}{A logical value indicating whether to
print results of test for overall effect (random effects model).}

\item{label.test.overall.common}{Label printed in front of results
of test for overall effect (common effect model).}

\item{label.test.overall.random}{Label printed in front of results
of test for overall effect (random effects model).}

\item{print.stat}{A logical value indicating whether z- or t-value
for test of treatment effect should be printed.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{test.subgroup.common}{A logical value indicating whether to
print results of test for subgroup differences (common effect
model).}

\item{test.subgroup.random}{A logical value indicating whether to
print results of test for subgroup differences (random effects
model).}

\item{common.subgroup}{A single logical or logical vector
indicating whether / which common effect estimates should be
printed for subgroups.}

\item{random.subgroup}{A single logical or logical vector
indicating whether / which random effects estimates should be
printed for subgroups.}

\item{prediction.subgroup}{A single logical or logical vector
indicating whether / which prediction intervals should be printed
for subgroups.}

\item{print.tau2.ci.subgroup}{A single logical or logical vector
indicating whether / which confidence intervals for between-study
variances should be printed for subgroups.}

\item{print.tau.ci.subgroup}{A single logical or logical vector
indicating whether / which confidence intervals for between-study
standard deviations should be printed for subgroups.}

\item{print.Q.subgroup}{A logical value indicating whether to print
the value of the heterogeneity statistic Q (test for subgroup
differences).}

\item{label.test.subgroup.common}{Label printed in front of results
of test for subgroup differences (common effect model).}

\item{label.test.subgroup.random}{Label printed in front of results
of test for subgroup differences (random effects model).}

\item{test.effect.subgroup}{A single logical or logical vector
indicating whether / which tests for effect in subgroups should
be printed.}

\item{test.effect.subgroup.common}{A single logical or logical
vector indicating whether / which tests for effect in subgroups
should be printed (common effect model).}

\item{test.effect.subgroup.random}{A single logical or logical
vector indicating whether / which tests for effect in subgroups
should be printed (random effects model).}

\item{label.test.effect.subgroup.common}{Label printed in front of
results of test for effect in subgroups (common effect model).}

\item{label.test.effect.subgroup.random}{Label printed in front of
results of test for effect in subgroups (random effects model).}

\item{text.addline1}{Text for first additional line (below
meta-analysis results).}

\item{text.addline2}{Text for second additional line (below
meta-analysis results).}

\item{details}{A logical specifying whether details on statistical
methods should be printed.}

\item{col.lines}{The colour of lines.}

\item{header.line}{A logical value indicating whether to print a
header line or a character string ("both", "below", "").}

\item{col.header.line}{Colour of the header line(s).}

\item{col.jama.line}{Colour of the additional JAMA lines.}

\item{data.pooled}{Data set with information for line(s) with pooled
results (see Details).}

\item{fontsize}{The size of text (in points), see
\code{\link{gpar}}.}

\item{fontfamily}{The font family, see \code{\link{gpar}}.}

\item{fs.heading}{The size of text for column headings, see
\code{\link{gpar}}.}

\item{fs.common}{The size of text for results of common effect
model, see \code{\link{gpar}}.}

\item{fs.random}{The size of text for results of random effects
model, see \code{\link{gpar}}.}

\item{fs.predict}{The size of text for results of prediction
interval, see \code{\link{gpar}}.}

\item{fs.common.labels}{The size of text for label of common effect
model, see \code{\link{gpar}}.}

\item{fs.random.labels}{The size of text for label of random
effects model, see \code{\link{gpar}}.}

\item{fs.predict.labels}{The size of text for label of prediction
interval, see \code{\link{gpar}}.}

\item{fs.study}{The size of text for results of individual studies,
see \code{\link{gpar}}.}

\item{fs.study.labels}{The size of text for labels of individual
studies, see \code{\link{gpar}}.}

\item{fs.hetstat}{The size of text for heterogeneity measures, see
\code{\link{gpar}}.}

\item{fs.test.overall}{The size of text of test for overall effect,
see \code{\link{gpar}}.}

\item{fs.test.subgroup}{The size of text of test of subgroup
differences, see \code{\link{gpar}}.}

\item{fs.test.effect.subgroup}{The size of text of test of effect
in subgroups, see \code{\link{gpar}}.}

\item{fs.addline}{The size of text for additional lines, see
\code{\link{gpar}}.}

\item{fs.axis}{The size of text on x-axis, see \code{\link{gpar}}.}

\item{fs.smlab}{The size of text of label for summary measure, see
\code{\link{gpar}}.}

\item{fs.xlab}{The size of text of label on x-axis, see
\code{\link{gpar}}.}

\item{fs.lr}{The size of text of label on left and right side of
forest plot, see \code{\link{gpar}}.}

\item{fs.rob}{The size of text of risk of bias items in the legend,
see \code{\link{gpar}}.}

\item{fs.rob.symbols}{The size of risk of bias symbols, see
\code{\link{gpar}}.}

\item{fs.details}{The size of text for details on (meta-analysis)
methods, see \code{\link{gpar}}.}

\item{ff.heading}{The fontface for column headings, see
\code{\link{gpar}}.}

\item{ff.common}{The fontface of text for results of common effect
model, see \code{\link{gpar}}.}

\item{ff.random}{The fontface of text for results of random effects
model, see \code{\link{gpar}}.}

\item{ff.predict}{The fontface of text for results of prediction
interval, see \code{\link{gpar}}.}

\item{ff.common.labels}{The fontface of text for label of common
effect model, see \code{\link{gpar}}.}

\item{ff.random.labels}{The fontface of text for label of random
effects model, see \code{\link{gpar}}.}

\item{ff.predict.labels}{The fontface of text for label of
prediction interval, see \code{\link{gpar}}.}

\item{ff.study}{The fontface of text for results of individual
studies, see \code{\link{gpar}}.}

\item{ff.study.labels}{The fontface of text for labels of
individual studies, see \code{\link{gpar}}.}

\item{ff.hetstat}{The fontface of text for heterogeneity measures,
see \code{\link{gpar}}.}

\item{ff.test.overall}{The fontface of text of test for overall
effect, see \code{\link{gpar}}.}

\item{ff.test.subgroup}{The fontface of text for test of subgroup
differences, see \code{\link{gpar}}.}

\item{ff.test.effect.subgroup}{The fontface of text for test of
effect in subgroups, see \code{\link{gpar}}.}

\item{ff.addline}{The fontface of text for additional lines, see
\code{\link{gpar}}.}

\item{ff.axis}{The fontface of text on x-axis, see
\code{\link{gpar}}.}

\item{ff.smlab}{The fontface of text of label for summary measure,
see \code{\link{gpar}}.}

\item{ff.xlab}{The fontface of text of label on x-axis, see
\code{\link{gpar}}.}

\item{ff.lr}{The fontface of text of label on left and right side
of forest plot, see \code{\link{gpar}}.}

\item{ff.rob}{The fontface of text of risk of bias items, see
\code{\link{gpar}}.}

\item{ff.rob.symbols}{The fontface of risk of bias symbols, see
\code{\link{gpar}}.}

\item{ff.details}{The fontface for details on (meta-analysis)
methods, see \code{\link{gpar}}.}

\item{squaresize}{A numeric used to increase or decrease the size
of squares in the forest plot.}

\item{lwd.square}{The line width of the border around squares.}

\item{lwd.diamond}{The line width of the border around diamonds.}

\item{arrow.type}{A character string indicating whether arrows
printed for results outside the forest plot should be
\code{"open"}, or \code{"closed"}, can be abbreviated.}

\item{arrow.length}{The length of arrows in inches.}

\item{plotwidth}{Either a character string, e.g., "8cm", "60mm", or
"3inch", or a \code{\link[grid]{unit}} object specifying width of
the forest plot.}

\item{colgap}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between columns
printed on left and right side of forest plot.}

\item{colgap.left}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between columns
printed on left side of forest plot.}

\item{colgap.right}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between columns
printed on right side of forest plot.}

\item{colgap.studlab}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between column
with study labels and subsequent column.}

\item{colgap.forest}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between column
adjacent to forest plot and the forest plot.}

\item{colgap.forest.left}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between column on
the left side of forest plot and the forest plot.}

\item{colgap.forest.right}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between column on
the right side of forest plot and the forest plot.}

\item{colgap.rob}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap between risk of
bias columns.}

\item{colgap.rob.overall}{Either a character string or a
\code{\link[grid]{unit}} object specifying gap before column with
overall risk of bias assessment.}

\item{calcwidth.pooled}{A logical indicating whether text for
common effect and random effects model should be considered to
calculate width of the column with study labels.}

\item{calcwidth.common}{A logical indicating whether text given in
arguments \code{text.common} and \code{text.common.w} should be
considered to calculate width of the column with study labels.}

\item{calcwidth.random}{A logical indicating whether text given in
arguments \code{text.random} and \code{text.random.w} should be
considered to calculate width of the column with study labels.}

\item{calcwidth.predict}{A logical indicating whether text given in
argument \code{text.predict} and \code{text.predict.w} should be
considered to calculate width of the column with study labels.}

\item{calcwidth.hetstat}{A logical indicating whether text for
heterogeneity statistics should be considered to calculate width
of the column with study labels.}

\item{calcwidth.tests}{A logical indicating whether text for tests
of overall effect or subgroup differences should be considered to
calculate width of the column with study labels.}

\item{calcwidth.subgroup}{A logical indicating whether text with
subgroup labels should be considered to calculate width of the
column with study labels.}

\item{calcwidth.addline}{A logical indicating whether text for
additional lines should be considered to calculate width of the
column with study labels.}

\item{just}{Justification of text in all columns but columns with
study labels and additional variables (possible values: "left",
"right", "center").}

\item{just.studlab}{Justification of text for study labels
(possible values: "left", "right", "center").}

\item{just.addcols}{Justification of text for additional columns
(possible values: "left", "right", "center").}

\item{just.addcols.left}{Justification of text for additional
columns on left side of forest plot (possible values: "left",
"right", "center"). Can be of same length as number of additional
columns on left side of forest plot.}

\item{just.addcols.right}{Justification of text for additional
columns on right side of forest plot (possible values: "left",
"right", "center"). Can be of same length as number of additional
columns on right side of forest plot.}

\item{just.label.e}{Justification of text for experimental group in table
heading (possible values: "left", "right", "center").}

\item{just.label.c}{Justification of text for control group in table
heading (possible values: "left", "right", "center").}

\item{bmj.text}{A character string used in the plot with BMJ layout
to label the group specific information.}

\item{bmj.xpos}{A numeric specifying the horizontal position of the
BMJ label. The value is a so called normalised parent coordinate
in the horizontal direction (see \code{\link[grid]{unit}}).}

\item{bmj.sep}{A character string used to separate sample sizes
from number of events or means / standard deviations.}

\item{spacing}{A numeric determining line spacing in a forest plot.}

\item{addrow}{A logical value indicating whether an empty row is
printed above study results.}

\item{addrow.overall}{A logical value indicating whether an empty
row is printed above overall meta-analysis results.}

\item{addrow.subgroups}{A logical value indicating whether an empty
row is printed between results for subgroups.}

\item{addrows.below.overall}{A numeric value indicating how many
empty rows are printed between meta-analysis results and
heterogeneity statistics and test results.}

\item{new}{A logical value indicating whether a new figure should
be printed in an existing graphics window.}

\item{backtransf}{A logical indicating whether results should be
back transformed in forest plots. If \code{backtransf = TRUE},
results for \code{sm = "OR"} are presented as odds ratios rather
than log odds ratios and results for \code{sm = "ZCOR"} are
presented as correlations rather than Fisher's z transformed
correlations, for example.}

\item{digits}{Minimal number of significant digits for treatment
effects, see \code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-statistic for test of overall effect.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity test.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistic Q.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance.}

\item{digits.I2}{Minimal number of significant digits for I-squared
statistic.}

\item{digits.weight}{Minimal number of significant digits for
weights.}

\item{digits.mean}{Minimal number of significant digits for means;
only applies to \code{\link{metacont}} objects.}

\item{digits.sd}{Minimal number of significant digits for standard
deviations; only applies to \code{\link{metacont}} objects.}

\item{digits.cor}{Minimal number of significant digits for
correlations; only applies to \code{\link{metacor}} objects.}

\item{digits.time}{Minimal number of significant digits for times;
only applies to \code{\link{metainc}} and \code{\link{metarate}}
objects.}

\item{digits.n}{Minimal number of significant digits for sample
sizes.}

\item{digits.event}{Minimal number of significant digits for event
numbers.}

\item{digits.TE}{Minimal number of significant digits for list
element 'TE'.}

\item{digits.addcols}{A vector or scalar with minimal number of
significant digits for additional columns.}

\item{digits.addcols.right}{A vector or scalar with minimal number
of significant digits for additional columns on right side of
forest plot.}

\item{digits.addcols.left}{A vector or scalar with minimal number
of significant digits for additional columns on left side of
forest plot.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{big.mark}{A character used as thousands separator.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional graphical arguments.}
}
\description{
Draw a forest plot (using grid graphics system) in the active
graphics window or store the forest plot in a file.
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window. The forest functions in R package
\bold{meta} are based on the grid graphics system. Resize the
graphics windows if the forest plot is too large or too small for
the graphics window. Alternatively, save the forest plot in a file.

\subsection{Saving forest plots}{

A forest plot can be directly stored in a file using argument
\code{file} or specifying the R function for the graphics device
driver using argument \code{func.gr}, e.g., \code{\link{pdf}}. If
only the filename is provided, the extension is checked and matched
against the most common graphics device drivers.

\tabular{ll}{
\bold{Extension} \tab \bold{Graphics device} \cr
\code{.pdf} \tab \code{\link{pdf}} \cr
\code{.ps} \tab \code{\link{postscript}} \cr
\code{.svg} \tab \code{\link{svg}} \cr
\code{.bmp} \tab \code{\link{bmp}} \cr
\code{.jpg} / \code{.jpeg} \tab \code{\link{jpeg}} \cr
\code{.png} \tab \code{\link{png}} \cr
\code{.tif} / \code{.tiff} \tab \code{\link{tiff}}
}

The height of the graphics device is automatically determined if
the forest plot is saved to a file. Argument \code{rows.gr} can be
used to increase or decrease the number of rows shown in the forest
plot (either to show missing information or to remove
whitespace). The width of the graphics device can be specified with
argument \code{width}, see, for example, \code{\link{pdf}} or
\code{\link{jpeg}}. Other arguments of graphics device functions
can be provided as a list in argument \code{args.gr}.

Alternatively, the (resized) graphics window can be stored to a
file using either \code{\link{dev.copy2eps}} or
\code{\link{dev.copy2pdf}}. It is also possible to manually create
a file using, for example, \code{\link{pdf}}, \code{\link{png}}, or
\code{\link{svg}} and to specify the width and height of the
graphic (see Examples).
}

\subsection{Default layout for studies and pooled effects}{

By default, treatment estimates and confidence intervals are
plotted in the following way:
\itemize{
\item For an individual study, a square with treatment estimate in
  the center and confidence interval as line extending either side
  of the square (\code{type.study = "square"})
\item For meta-analysis results, a diamond with treatment estimate
  in the center and right and left side corresponding to lower and
  upper confidence limits (\code{type.common = "diamond"},
  \code{type.random = "diamond"}, and \code{type.subgroup = "diamond"})
}

In a forest plot, size of the squares typically reflects the precision of
individual treatment estimates based either on the common effect
(\code{weight.study = "common"}) or random effects meta-analysis
(\code{weight.study = "random"}). Information from meta-analysis object
\code{x} is utilised if argument \code{weight.study} is missing. Weights
from the common effect model are used if argument \code{x$common} is
\code{TRUE}; weights from the random effects model are used if argument
\code{x$random} is \code{TRUE} and \code{x$common} is \code{FALSE}.
The same square sizes are used if \code{weight.study = "same"}.

A prediction interval for treatment effect of a new study (Higgins
et al., 2009) is given in the forest plot if arguments
\code{prediction} and \code{random} are \code{TRUE}. For
graphical presentation of prediction intervals the approach by
Guddat et al. (2012) is used.
}

\subsection{Columns printed on left side of forest plot}{

Argument \code{leftcols} can be used to specify columns which are
printed on the left side of the forest plot. By default, i.e. if
argument \code{leftcols} is \code{NULL} and \code{layout = "meta"},
and depending on the class of the meta-analysis object (which is
defined by the R function used to generate the object) a different
set of \emph{\bold{columns}} is printed \emph{\bold{on the left
side of the forest plot}}:
\tabular{ll}{
\bold{Function} \tab \bold{Value of argument leftcols} \cr
\code{\link{metabin}} \tab \code{c("studlab", "event.e", "n.e",
  "event.c", "n.c")} \cr
\code{\link{metacont}} \tab \code{c("studlab", "n.e", "mean.e",
  "sd.e", "n.c", "mean.c", "sd.c")} \cr
\code{\link{metacor}} \tab \code{c("studlab", "n")} \cr
\code{\link{metagen}} \tab \code{c("studlab", "TE", "seTE")} \cr
\code{\link{metainc}} \tab \code{c("studlab", "event.e", "time.e",
  "event.c", "time.c")} \cr
\code{\link{metamean}} \tab \code{c("studlab", "n", "mean", "sd")}
  \cr
\code{\link{metaprop}} \tab \code{c("studlab", "event", "n")} \cr
\code{\link{metarate}} \tab \code{c("studlab", "event", "time", "n")}
}

For three-level models, the cluster variable is printed next to the
study labels (value \code{"cluster"} in argument \code{leftcols}).

By default, study labels and labels for pooled estimates and
heterogeneity statistics will be printed in the first column on the
left side of the forest plot. The character string \code{"studlab"}
is used to identify study labels as this is the name of the list
element of a meta-analysis object.

If the character string \code{"studlab"} is not provided in
\code{leftcols} and \code{rightcols}, the first \emph{additional}
variable specified by the user is used as study labels (and labels
for pooled estimates are printed in this column). Additional
variables are any variables not mentioned in the section on
predefined column names below. For example, \code{leftcols =
"studlab"} and \code{leftcols = "study"} would result in the same
forest plot if the variable \code{"study"} was used in the command
to conduct the meta-analysis. If no additional variable is provided
by the user, no study labels will be printed.
}

\subsection{Overlapping information on left side of forest plot}{

Depending on the number of columns printed on the left side of the
forest plot, information on heterogeneity measures or statistical
tests (see below) can be overlapping with the x-axis. Argument
\code{addrows.below.overall} can be used to specify the number of
empty rows that are printed between meta-analysis results and
information on heterogeneity measures and statistical tests. By
default, no additional rows are added to the forest plot. If
\code{addrows.below.overall = NULL}, the function tries to add a
sufficient number of empty rows to prevent overlapping
text. Another possibility is to manually increase the space between
the columns on the left side (argument \code{colgap.left}) or
between the columns on the left side and the forest plot (argument
\code{colgap.forest.left}).
}

\subsection{Columns printed on right side of forest plot}{

Argument \code{rightcols} can be used to
specify columns which are printed on the right side of the
forest plot. If argument \code{rightcols} is
\code{FALSE}, no columns will be printed on the right side. By
default, i.e. if argument \code{rightcols} is
\code{NULL} and \code{layout = "meta"}, the following
\emph{\bold{columns}} will be printed \emph{\bold{on the right side
of the forest plot}}:
\tabular{ll}{
\bold{Meta-analysis results} \tab \bold{Value of argument
  rightcols} \cr
No summary \tab \code{c("effect", "ci")} \cr
Only common effect model \tab \code{c("effect", "ci", "w.common")}
  \cr
Only random effects model \tab \code{c("effect", "ci", "w.random")}
  \cr
Both models \tab \code{c("effect", "ci", "w.common", "w.random")}
}

By default, estimated treatment effect and corresponding confidence
interval will be printed. Depending on arguments \code{common} and
\code{random}, weights of the common effect and/or random effects
model will be given too.
}

\subsection{Predefined columns and column labels}{

The arguments \code{leftlabs} and \code{rightlabs} can be used to
specify column headings which are printed on the left or right side of
the forest plot. For certain columns predefined labels exist which
are used by default, i.e., if arguments \code{leftlabs} and
\code{rightlabs} are \code{NULL}:
\tabular{rccc}{
Column: \tab \code{n.e}, \code{n.c}, \code{n} \tab
  \code{mean.e}, \code{mean.c}, \code{mean} \tab
  \code{sd.e}, \code{sd.c}, \code{sd} \cr 
Label: \tab \code{gs("label.n")} \tab
  \code{gs("label.mean")} \tab \code{gs("label.sd")} \cr
\cr
Column: \tab \code{event.e}, \code{event.c}, \code{event} \tab
  \code{time.e}, \code{time.c}, \code{time} \tab \code{cor} \cr
Label: \tab \code{gs("label.events")} \tab \code{gs("label.time")} \tab
  \code{gs("label.cor")} \cr
\cr
Column: \tab \code{effect} \tab \code{ci} \tab \code{effect.ci} \cr
Label: \tab \code{x$sm} \tab \code{x$level}"\%-CI" \tab \emph{effect+ci} \cr
\cr
Column: \tab \code{cluster} \tab \code{cycles} \tab \code{pval} \cr
Label: \tab \code{gs("label.cluster")} \tab \code{gs("label.cycles")} \tab
  \code{gs("label.pval")} \cr
\cr
Column: \tab \code{tau2} \tab \code{tau} \tab \code{I2} \cr
Label: \tab \code{gs("label.tau2")} \tab \code{gs("label.tau")} \tab
  \code{gs("label.I2")}
}

For the columns \code{studlab} (study labels), \code{TE}
(treatment estimates), \code{seTE} (standard errors of treatment estimates),
\code{w.common} (weights under the common effect model), and \code{w.random}
(weights under the random effects model), context-specific column labels
are used.

For other columns, the column name will be used as a label if no
column label is defined. It is possible to only provide labels for
new columns (see Examples). Otherwise the length of \code{leftlabs}
and \code{rightlabs} must be the same as the number of printed
columns. The value \code{NA} can be used to specify columns which
should use default labels (see Examples).

In pairwise meta-analysis comparing two groups (i.e.,
\code{\link{metabin}}, \code{\link{metacont}},
\code{\link{metainc}}, and \code{\link{metagen}} depending on the
outcome), arguments \code{label.e} and \code{label.c} are used to
label columns belonging to the two treatment groups. By default,
labels defined in the meta-analysis object are used. The columns
where treatment labels are attached can be changed using arguments
\code{label.e.attach} and \code{label.c.attach}.
}

\subsection{Risk of bias assessment}{

A risk of bias (RoB) assessment can be shown in the forest plot by
either using a meta-analysis object with an RoB assessment as main
input or providing a suitable object created with
\code{\link{rob}}. Argument \code{rob = FALSE} can be used to
suppress the print of the risk of bias information.

RoB assessments are shown as the only information on the right side
of the forest plot. Thus, arguments \code{rightcols} and
\code{rightlabs} should not be used. Predefined columns shown by
default on the right side of a forest plot will be moved to the
left side.
}

\subsection{Information on heterogeneity and statistical tests}{

Argument \code{hetstat} can be a character string to specify where
to print heterogeneity information:
\itemize{
\item row with results for common effect model (\code{hetstat =
"common"}),
\item row with results for random effects model (\code{hetstat =
"random"}).
}

Otherwise, information on heterogeneity measures is printed below
the meta-analysis results if argument \code{overall.hetstat = TRUE}
(default). The heterogeneity measures to print can be specified
(see list of arguments following \code{overall.hetstat}).

In addition, the following arguments can be used to print results
for various statistical tests:
\tabular{ll}{
\bold{Argument} \tab \bold{Statistical test} \cr
\code{test.overall.common} \tab Test for overall effect (common
  effect model) \cr
\code{test.overall.random} \tab Test for overall effect (random
  effects model) \cr
\code{test.effect.subgroup.common} \tab Test for effect in subgroup
  (CE model) \cr
\code{test.effect.subgroup.random} \tab Test for effect in subgroup
  (RE model) \cr
\code{test.subgroup.common} \tab Test for subgroup differences (CE
  model) \cr
\code{test.subgroup.random} \tab Test for subgroup differences (RE
  model)
}

By default, these arguments are \code{FALSE} with exception of
tests for subgroup differences which are \code{TRUE}. R function
\code{\link{settings.meta}} can be used to change this default for
the entire R session. For example, use the following command to
always print results of tests for an overall effect:
\code{settings.meta(test.overall = TRUE)}.
}

\subsection{Highlight regions corresponding to minimal clinically important
differences}{

Regions corresponding to minimal clinically important differences can be
added to the forest plot using either argument \code{cid} or
\code{cid.below.null} and \code{cid.above.null}. Input for the later arguments will
be ignored if argument \code{cid} was specified. In this case, the values
of \code{cid.below.null} and \code{cid.above.null} will be equal to
\itemize{
\item \code{cid} and \code{1 / cid} for ratio measures,
\item \code{cid} and \code{-cid} for difference measures.
}

Thresholds based on argument \code{cid} will always be symmetric. Asymmetric
thresholds can be defined using arguments \code{cid.below.null} and
\code{cid.above.null}.
}

\subsection{Flexible printing of subgroup results}{

Argument \code{subgroup} determines whether summary results are
printed for subgroups. A logical vector of length equal to the
number of subgroups can be provided to determine which subgroup
summaries are printed. By default, only subgroup results based on
at least two studies are printed which is identical to use argument
\code{subgroup = k.w > 1}. The order of the logical vector
corresponds to the order of subgroups in list element 'subgroup.levels' of a
meta-analysis object. Argument \code{subgroup = k.w >= 1} can be
used to show results for all subgroups (including those with a
single study).

The following arguments can be used in a similar way:

\itemize{
\item \code{subgroup.hetstat} (heterogeneity statistic in
  subgroups),
\item \code{common.subgroup} (common effect estimates in
  subgroups),
\item \code{random.subgroup} (random effects estimates in
  subgroups),
\item \code{prediction.subgroup} (prediction interval in
  subgroups),
\item \code{test.effect.subgroup} (test for effect in subgroups),
\item \code{test.effect.subgroup.common} (test for effect in
  subgroups, common effect model),
\item \code{test.effect.subgroup.random} (test for effect in
  subgroups, random effects model).
}
}

\subsection{Data set with information to print with overall results}{

Argument \code{data.pooled} can be used to provide information printed in
the line(s) with overall results, i.e., for the common effect or random
effects model. Input must be a data frame with variable names equal to those
provided in arguments \code{leftcols} or \code{rightcols}. Only variables
for additional variables are considered.

It is possible to provide a row in data set \code{data.pooled} for each
common effect or random effects estimate. The order in the data set
corresponds to the order of common effect and random effects estimates in
the forest plot, i.e., common effect followed by random effects estimates.
If the data set contains a single row, the value provided for a variable is
considered for all printed common effect and random effects estimates 

In meta-analyses with subgroups, a row must be provided in data set
\code{data.pooled} for each overall common effect or random effects estimate,
followed by common effect or random effects estimates within subgroups. The
order for subgroup results is the same as see in the forest plot.
}

\subsection{Additional general settings}{

Arguments \code{text.common}, \code{text.random}, and
\code{text.predict} can be used to change the label to identify
overall results (common effect and random effects model as well as
prediction interval). By default the following text is printed:
\itemize{
\item "Common effect model" (argument \code{text.common})
\item "Random effects model" (\code{text.random})
\item "Prediction interval" (\code{text.predict})
}

If confidence interval levels are different for individual studies,
meta-analysis, and prediction interval (arguments \code{level},
\code{level.ma}, \code{level.predict} in meta-analysis functions,
e.g., \code{\link{metabin}}), additional information is printed,
e.g., " (99\%-CI)" for a 99\% confidence interval in the
meta-analysis.

Argument \code{pscale} can be used to rescale single proportions or
risk differences, e.g., \code{pscale = 1000} means that proportions
are expressed as events per 1000 observations. This is useful in
situations with (very) low event probabilities.

Argument \code{irscale} can be used to rescale single rates or rate
differences, e.g., \code{irscale = 1000} means that rates are
expressed as events per 1000 time units, e.g., person-years. This is
useful in situations with (very) low rates. Argument \code{irunit}
can be used to specify the time unit used in individual studies
(default: "person-years"). This information is printed in summaries
and forest plots if argument \code{irscale} is not equal to 1.
}

\subsection{Forest plots in RevMan5 layout}{

If argument \code{layout = "RevMan5"} (and arguments \code{leftcols} and
\code{rightcols} are \code{NULL}), the layout for forest plots used for
Cochrane reviews (which were generated with Review Manager 5) is reproduced:
\enumerate{
\item All columns are printed on the left side of the forest plot
  (see arguments \code{leftcols} and \code{rightcols})
\item Tests for overall effect and subgroup differences are printed
  (\code{test.overall}, \code{test.effect.subgroup},
  \code{test.subgroup})
\item Diamonds representing meta-analysis results are printed in
  black (\code{diamond.common}, \code{diamond.random})
\item Colour of squares depends on the meta-analysis object
  (\code{col.square}, \code{col.square.lines})
\item Information on effect measure and meta-analysis method is
  printed above the forest plot (\code{smlab})
\item Label "Study or Subgroup" is printed for meta-analysis with
  subgroups (\code{leftlabs})
}
}

\subsection{Forest plots in JAMA layout}{

If argument \code{layout = "JAMA"} (and arguments \code{leftcols} and
\code{rightcols} are \code{NULL}), instructions for authors of the
\emph{Journal of the American Medical Association} are taken into account:
\enumerate{
\item Graph labels on right and left side are printed in bold font
  at top of forest plot (see arguments \code{bottom.lr} and
  \code{ff.lr})
\item Information on effect measure and level of confidence
  interval is printed at bottom of forest plot (\code{xlab})
\item Tests for overall effect are printed (\code{test.overall})
\item Diamonds representing meta-analysis results are printed in
  lightblue (\code{diamond.common}, \code{diamond.random})
\item Squares representing individual study results are printed in
  darkblue (\code{col.square}, \code{col.square.lines})
\item Between-study variance \eqn{\tau^2} is not printed
\item Empty rows are omitted (\code{addrow}, \code{addrow.overall},
  \code{addrow.subgroups})
\item Label "Source" is printed instead of "Study" (\code{leftlabs})
\item P-values are printed without leading zeros (\code{zero.pval})
\item P-values are rounded to three digits (for 0.001 < p \eqn{\le}
  0.01) or two digits (p > 0.01) (\code{JAMA.pval})
}
Study labels according to JAMA guidelines can be generated using
\code{\link{labels.meta}}.
}

\subsection{Forest plots showing results of subgroups}{

The following changes are conducted if argument
\code{layout = "subgroup"} (and arguments \code{leftcols} and
\code{rightcols} are \code{NULL}) and a subgroup analysis was
conducted:
\enumerate{
\item Individual study results are omitted (see argument
  \code{study.results})
\item Total number of observations is not printed
  (\code{pooled.totals})
\item Label "Subgroup" is printed instead of "Study"
  (\code{leftlabs})
}
}
}
\note{
R function \code{.forestArgs} generates a character vector with all
arguments of \code{forest.meta}.
}
\examples{
data(Olkin1995)
ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = c(41, 47, 51, 59),
  sm = "RR", method = "I",
  studlab = paste(author, year))


\dontrun{
# Do standard (symmetric) forest plot
#
forest(ma1)
}

# Layout of forest plot similar to Review Manager 5
#
# Furthermore, add labels on both sides of forest plot and
# prediction interval
#
forest(ma1, layout = "RevMan5", common = FALSE,
  label.left = "Favours experimental", col.label.left = "green",
  label.right = "Favours control", col.label.right = "red",
  prediction = TRUE)


\dontrun{
# Create PDF files with the forest plot
#
# - specify filename (R function pdf() is used due to extension .pdf)
# - height of the figure is automatically determined
# - width is set to 10 inches
forest(ma1, file = "forest-ma1-1.pdf", width = 10)
#
# - specify graphics device function
#   (filename "Rplots.pdf" used, see help page of R function pdf())
# - height of the figure is automatically determined
# - width is set to 10 inches
# - set title for PDF file
# - set background of forest plot
forest(ma1, func.gr = pdf, width = 10,
  args.gr = list(title = "My Forest Plot", bg = "green"))
#
# - manually specify the height of the figure
pdf("forest-ma1-2.pdf", width = 10, height = 3)
forest(ma1)
dev.off()

# Define equivalence limits: 0.75 and 1 / 0.75
#
forest(ma1, layout = "RevMan5", common = FALSE, cid = 0.75,
  fill = "lightgray",
  fill.cid = "white")

# Fill regions with beneficial and detrimental effects
#
forest(ma1, layout = "RevMan5", common = FALSE, cid = 0.75,
  fill = "lightgray",
  fill.cid.below.null = "green",
  fill.cid.above.null = "red")

# Define thresholds for small, moderate and large effects
# and use hcl.colors() to define colours to fill regions
#
thresholds <- c(0.25, 0.5, 0.75)
n.cols <- length(thresholds)
forest(ma1, layout = "RevMan5", common = FALSE,
  label.left = "Desirable effect", 
  label.right = "Undesirable effect", 
  lty.cid = 3, col.cid = "darkgray",
  cid.below.null = thresholds, cid.above.null = 1 / rev(thresholds),
  fill.cid.below.null =
    hcl.colors(n.cols, palette = "Blues 2", alpha = 0.6),
  fill.cid.above.null =
    hcl.colors(n.cols, palette = "Oranges", alpha = 0.6, rev = TRUE))

# Conduct subgroup meta-analysis
#
ma2 <- update(ma1,
  subgroup = ifelse(year < 1987, "Before 1987", "1987 and later"),
  print.subgroup.name = FALSE)

# Show summary results for subgroups with at least two studies
#
forest(ma2, sortvar = -TE, random = FALSE)

# Show results for all subgroups
#
forest(ma2, sortvar = -TE, random = FALSE, subgroup = k.w >= 1)

# Forest plot specifying argument xlim
#
forest(ma1, xlim = c(0.01, 10))

# Print results of test for overall effect
#
forest(ma1, test.overall.common = TRUE, test.overall.random = TRUE)

# Forest plot with 'classic' layout used in R package meta,
# version < 1.6-0
#
forest(ma1, col.square = "black", hetstat = FALSE)

# Change set of columns printed on left side of forest plot
# (resulting in overlapping text)
#
forest(ma1, random = FALSE, leftcols = "studlab")

# Use argument 'calcwidth.hetstat' to consider text for heterogeneity
# measures in width of column with study labels
#
forest(ma1, random = FALSE, leftcols = "studlab",
  calcwidth.hetstat = TRUE)

# Use argument 'addrows.below.overall' to manually add two empty
# rows
#
forest(ma1, random = FALSE, leftcols = "studlab", addrows = 2)

# Do not print columns on right side of forest plot
#
forest(ma1, rightcols = FALSE)

# Change study label to "Author"
#
forest(ma1, random = FALSE, leftlabs = c("Author", NA, NA, NA, NA))

# Just give effect estimate and 95\% confidence interval on right
# side of forest plot (in one column)
#
forest(ma1, rightcols = "effect.ci")

# Just give effect estimate and 95\% confidence interval on right
# side of forest plot
#
forest(ma1, rightcols = c("effect", "ci"))

# 1. Change order of columns on left side
# 2. Attach labels to columns 'event.e' and 'event.c' instead of
#    columns 'n.e' and 'n.c'
#
forest(ma1,
  leftcols = c("studlab", "n.e", "event.e", "n.c", "event.c"),
  label.e.attach = "event.e", label.c.attach = "event.c")

# Specify column labels only for variables 'year' and 'author'
# (and define digits for additional variables)
#
forest(ma1,
  leftcols = c("studlab", "event.e", "n.e", "event.c", "n.c", "author", "year"),
  leftlabs = c("Author", "Year of Publ"))

# Center text in all columns
#
forest(ma1,
  leftcols = c("studlab", "event.e", "n.e", "event.c", "n.c",
               "author", "year"),
  leftlabs = c("Author", "Year of Publ"), hetstat = FALSE,
  just = "center", just.addcols = "center", just.studlab = "center")

# Same result
#
forest(ma1,
  leftcols = c("studlab", "event.e", "n.e", "event.c", "n.c",
             "author", "year"),
  leftlabs = c("Author", "Year of Publ"), hetstat = FALSE,
  just = "c", just.addcols = "c", just.studlab = "c")

# Change some fontsizes and fontfaces
#
forest(ma1,
  fs.study = 10, ff.study = "italic",
  fs.study.label = 11, ff.study.label = "bold",
  fs.axis = 5, ff.axis = "italic",
  ff.smlab = "bold.italic",
  ff.common = "plain", ff.hetstat = "plain")

# Change some colours
#
forest(ma1,
  col.diamond = "green", col.diamond.lines = "red",
  col.study = c("green", "blue", "red", "orange"),
  col.square = "pink", col.square.lines = "black")

# Sort by weight in common effect model
#
forest(ma1, sortvar = w.common, random = FALSE)

# Sort by decreasing weight in common effect model
#
forest(ma1, sortvar = -w.common, random = FALSE)

# Sort by size of treatment effect
#
forest(ma1, sortvar = TE, random = FALSE)

# Sort by size of treatment effect
#
forest(ma1, sortvar = -TE, random = FALSE)

# Sort by decreasing year of publication
#
forest(ma1, sortvar = -year, random = FALSE)

# Print results of test for subgroup differences (random effects
# model)
#
forest(ma2, sortvar = -TE, common = FALSE)

# Print only subgroup results
#
forest(ma2, layout = "subgroup")

# Print only subgroup results (and consider text for tests of
# subgroup differences in width of subgroup column)
#
forest(ma2, layout = "subgroup", calcwidth.tests = TRUE)

# Print only subgroup results (and consider text for heterogeneity
# in width of subgroup column)
#
forest(ma2, layout = "subgroup", calcwidth.hetstat = TRUE)
}

}
\references{
Guddat C, Grouven U, Bender R, Skipka G (2012):
A note on the graphical presentation of prediction intervals in
random-effects meta-analyses.
\emph{Systematic Reviews},
\bold{1}, 34

Higgins JPT, Thompson SG, Spiegelhalter DJ (2009): 
A re-evaluation of random-effects meta-analysis.
\emph{Journal of the Royal Statistical Society: Series A},
\bold{172}, 137-59
}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}, \code{\link{forest.metabind}},
  \code{\link{settings.meta}}, \code{\link{labels.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.meta.R
\name{print.meta}
\alias{print.meta}
\alias{cilayout}
\title{Print meta-analysis results}
\usage{
\method{print}{meta}(
  x,
  common = x$common,
  random = x$random,
  prediction = x$prediction,
  overall = x$overall,
  overall.hetstat = x$overall.hetstat,
  test.subgroup = x$test.subgroup,
  test.subgroup.common = test.subgroup & common,
  test.subgroup.random = test.subgroup & random,
  prediction.subgroup = x$prediction.subgroup,
  backtransf = x$backtransf,
  pscale = x$pscale,
  irscale = x$irscale,
  irunit = x$irunit,
  subgroup.name = x$subgroup.name,
  print.subgroup.name = x$print.subgroup.name,
  sep.subgroup = x$sep.subgroup,
  nchar.subgroup = 35,
  sort.overall = NULL,
  sort.tau = NULL,
  sort.het = NULL,
  sort.Q = NULL,
  header = TRUE,
  print.CMH = x$print.CMH,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = max(gs("digits.pval"), 2),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.Q = gs("digits.Q"),
  digits.df = gs("digits.df"),
  digits.pval.Q = max(gs("digits.pval.Q"), 2),
  digits.H = gs("digits.H"),
  digits.I2 = gs("digits.I2"),
  big.mark = gs("big.mark"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  print.tau2 = gs("print.tau2"),
  print.tau2.ci = gs("print.tau2.ci"),
  print.tau = gs("print.tau"),
  print.tau.ci = gs("print.tau.ci"),
  print.Q = gs("print.Q"),
  print.I2 = gs("print.I2"),
  print.I2.ci = gs("print.I2.ci"),
  print.H = gs("print.H"),
  print.Rb = gs("print.Rb"),
  text.tau2 = gs("text.tau2"),
  text.tau = gs("text.tau"),
  text.I2 = gs("text.I2"),
  text.Rb = gs("text.Rb"),
  details.methods = gs("details"),
  warn.backtransf = FALSE,
  func.backtransf = x$func.backtransf,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

cilayout(
  bracket = gs("CIbracket"),
  separator = gs("CIseparator"),
  lower.blank = gs("CIlower.blank"),
  upper.blank = gs("CIupper.blank")
)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{common}{A logical indicating whether results for common
effect meta-analysis should be printed.}

\item{random}{A logical indicating whether results for random
effects meta-analysis should be printed.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{test.subgroup.common}{A logical value indicating whether to
print results of test for subgroup differences (based on common
effect model).}

\item{test.subgroup.random}{A logical value indicating whether to
print results of test for subgroup differences (based on random
effects model).}

\item{prediction.subgroup}{A single logical or logical vector
indicating whether / which prediction intervals should be printed
for subgroups.}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. If \code{backtransf=TRUE}, results
for \code{sm="OR"} are printed as odds ratios rather than log
odds ratios and results for \code{sm="ZCOR"} are printed as
correlations rather than Fisher's z transformed correlations, for
example.}

\item{pscale}{A numeric giving scaling factor for printing of
single event probabilities or risk differences, i.e. if argument
\code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
\code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.}

\item{irscale}{A numeric defining a scaling factor for printing of
single incidence rates or incidence rate differences, i.e. if
argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
\code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.}

\item{irunit}{A character specifying the time unit used to
calculate rates, e.g. person-years.}

\item{subgroup.name}{A character string with a name for the
grouping variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the grouping variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between label and levels of grouping variable.}

\item{nchar.subgroup}{A numeric specifying the number of characters
to print from subgroup labels.}

\item{sort.overall}{An optional vector used to sort meta-analysis
results.}

\item{sort.tau}{An optional vector used to sort estimators of the
between-study heterogeneity variance.}

\item{sort.het}{An optional vector used to sort heterogeneity
statistics.}

\item{sort.Q}{An optional vector used to sort test of
heterogeneity.}

\item{header}{A logical indicating whether information on title of
meta-analysis, comparison and outcome should be printed at the
beginning of the printout.}

\item{print.CMH}{A logical indicating whether result of the
Cochran-Mantel-Haenszel test for overall effect should be
printed.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-value of test for overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance \eqn{\tau^2}, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistic Q, see \code{print.default}.}

\item{digits.df}{Minimal number of significant digits for degrees
of freedom.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity test, see \code{print.default}.}

\item{digits.H}{Minimal number of significant digits for H
statistic, see \code{print.default}.}

\item{digits.I2}{Minimal number of significant digits for I-squared
and Rb statistic, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau2.ci}{A logical value indicating whether to print
the confidence interval of \eqn{\tau^2}.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{print.tau.ci}{A logical value indicating whether to print the
confidence interval of \eqn{\tau}.}

\item{print.Q}{A logical value indicating whether to print the
results of the test of heterogeneity.}

\item{print.I2}{A logical specifying whether heterogeneity
statistic I\eqn{^2} should be printed.}

\item{print.I2.ci}{A logical specifying whether confidence interval for
heterogeneity statistic I\eqn{^2} should be printed.}

\item{print.H}{A logical specifying whether heterogeneity statistic
H should be printed.}

\item{print.Rb}{A logical specifying whether heterogeneity
statistic R\eqn{_b} should be printed.}

\item{text.tau2}{Text printed to identify between-study variance
\eqn{\tau^2}.}

\item{text.tau}{Text printed to identify \eqn{\tau}, the square
root of the between-study variance \eqn{\tau^2}.}

\item{text.I2}{Text printed to identify heterogeneity statistic
I\eqn{^2}.}

\item{text.Rb}{Text printed to identify heterogeneity statistic
R\eqn{_b}.}

\item{details.methods}{A logical specifying whether details on
statistical methods should be printed.}

\item{warn.backtransf}{Deprecated argument (ignored).}

\item{func.backtransf}{A function used to back-transform results.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (passed on to \code{prmatrix}).}

\item{bracket}{A character with bracket symbol to print lower
confidence interval: "[", "(", "\{", "".}

\item{separator}{A character string with information on separator
between lower and upper confidence interval.}

\item{lower.blank}{A logical indicating whether blanks between left
bracket and lower confidence limit should be printed.}

\item{upper.blank}{A logical indicating whether blanks between
separator and upper confidence limit should be printed.}
}
\description{
Print method for objects of class \code{meta}.

R function cilayout can be utilised to change the layout to print
confidence intervals (both in printout from print.meta and
print.summary.meta function as well as in forest plots). The
default layout is "[lower; upper]". Another popular layout is
"(lower - upper)" which is used throughout an R session by using R
command \code{cilayout("(", " - ")}.

Argument \code{pscale} can be used to rescale single proportions or
risk differences, e.g. \code{pscale = 1000} means that proportions
are expressed as events per 1000 observations. This is useful in
situations with (very) low event probabilities.

Argument \code{irscale} can be used to rescale single rates or rate
differences, e.g. \code{irscale = 1000} means that rates are
expressed as events per 1000 time units, e.g. person-years. This is
useful in situations with (very) low rates. Argument \code{irunit}
can be used to specify the time unit used in individual studies
(default: "person-years"). This information is printed in summaries
and forest plots if argument \code{irscale} is not equal to 1.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metainf.R
\name{metainf.meta}
\alias{metainf.meta}
\alias{metainf}
\alias{metainf.default}
\title{Influence analysis in meta-analysis using leave-one-out method}
\usage{
\method{metainf}{meta}(
  x,
  pooled,
  sortvar,
  prediction,
  overall = x$overall,
  text.pooled,
  no = 1,
  cid = NULL,
  cid.below.null = NULL,
  cid.above.null = NULL,
  small.values = "desirable",
  ...
)

metainf(x, ...)

\method{metainf}{default}(x, ...)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{pooled}{A character string indicating whether a common effect
or random effects model is used for pooling. Either missing (see
Details), \code{"common"}, or \code{"random"}, can be abbreviated.}

\item{sortvar}{An optional vector used to sort the individual
studies (must be of same length as \code{x$TE}).}

\item{prediction}{A logical indicating whether to report prediction
intervals.}

\item{overall}{A logical indicating whether overall results should be
reported.}

\item{text.pooled}{A character string used in printouts and forest
plots to label the pooled effect estimate.}

\item{no}{A numeric specifying which meta-analysis results to
consider.}

\item{cid}{A numeric value or vector specifying clinically important
differences (CID) / decision thresholds used to calculate expected
proportions of clinically important benefit or harm
(see \code{\link{cidprop}}).}

\item{cid.below.null}{A single numeric defining the decision threshold below
the null effect to distinguish clinically important from not important
effects (see \code{\link{cidprop}}).}

\item{cid.above.null}{A single numeric defining the decision threshold above
the null effect to distinguish clinically important from not important
effects (see \code{\link{cidprop}}).}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated
(see \code{\link{cidprop}}).}

\item{\dots}{Additional arguments (ignored).}
}
\value{
An object of class \code{"metainf"} with dedicated print and forest
functions.

The following list elements provide results from meta-analyses, each
excluding one study at a time (see \code{\link{meta-object}} for more
information on these list elements):
\tabular{l}{
\cr
studlab, TE, seTE, df.random, lower, upper, statistic, pval, \cr
lower.predict, upper.predict, df.predict, w (sum of weights), \cr
tau2, se.tau2, lower.tau2, upper.tau2, tau, lower.tau, upper.tau, \cr
I2, lower.I2, upper.I2, Rb, n.harmonic.mean, t.harmonic.mean, \cr
k, k.study, k.all, k.TE, k.MH.
}

The following list elements contain results of the original meta-analysis:
\tabular{l}{
\cr
TE.pooled, seTE.pooled, df.random.pooled, \cr
lower.pooled, upper.pooled, statistic.pooled, pval.pooled, \cr
lower.predict.pooled, upper.predict.pooled, \cr
df.predict.pooled, w.pooled, \cr
tau2.pooled, se.tau2.pooled, lower.tau2.pooled, upper.tau2.pooled, \cr
tau.pooled, lower.tau.pooled, upper.tau.pooled, \cr
I2.pooled, lower.I2.pooled, upper.I2.pooled, Rb.pooled, \cr
n.harmonic.mean.pooled, t.harmonic.mean.pooled, \cr
k.pooled, k.study.pooled, k.all.pooled, k.TE.pooled, k.MH.pooled.
}
}
\description{
Performs an influence analysis. Pooled estimates are calculated
omitting one study at a time.
}
\details{
Performs a influence analysis; pooled estimates are calculated
omitting one study at a time. Studies are sorted according to
\code{sortvar}.

Information from object \code{x} is utilised if argument
\code{pooled} is missing. A common effect model is assumed
(\code{pooled = "common"}) if argument \code{x$common} is
\code{TRUE}; a random effects model is assumed (\code{pooled =
"random"}) if argument \code{x$random} is \code{TRUE} and
\code{x$common} is \code{FALSE}.
}
\examples{
data(Fleiss1993bin)
ma1 <- metabin(d.asp, n.asp, d.plac, n.plac,
  data = Fleiss1993bin, studlab = study, sm = "RR", method = "I")
ma1
metainf(ma1)
metainf(ma1, pooled = "random")

forest(metainf(ma1))
forest(metainf(ma1, pooled = "random"))

\dontrun{
# Create a PDF file with a forest plot containing UTF-8 characters
# for tau^2, tau, and I^2
forest(metainf(ma1, pooled = "random"),
  label.tau2 = "\u03c4\u00b2", label.tau = "\u03c4", label.I2 = "I\u00b2",
  file = "forest_with_unicode.pdf", width = 8.5, func.gr = "cairo_pdf")
}

metainf(ma1, sortvar = study)
metainf(ma1, sortvar = 7:1)

ma2 <- update(ma1, title = "Fleiss1993bin meta-analysis", backtransf = FALSE)
metainf(ma2)

data(Fleiss1993cont)
ma3 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")
metainf(ma3)

}
\references{
Cooper H & Hedges LV (1994):
\emph{The Handbook of Research Synthesis}.
Newbury Park, CA: Russell Sage Foundation
}
\seealso{
\code{\link{forest.metainf}}, \code{\link{print.metainf}},
  \code{\link{cidprop}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cisapride.R
\docType{data}
\name{cisapride}
\alias{cisapride}
\title{Cisapride in Non-Ulcer Dispepsia}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{event.cisa}}\tab number of events in cisapride group
  \cr
\bold{\emph{n.cisa}}\tab number of observations in cisapride group
  \cr
\bold{\emph{event.plac}}\tab number of events in placebo group \cr
\bold{\emph{n.plac}}\tab number of observations in placebo group
}
}
\source{
Hartung J & Knapp G (2001):
A refined method for the meta-analysis of controlled clinical
trials with binary outcome.
\emph{Statistics in Medicine},
\bold{20}, 3875--89
}
\description{
Meta-analysis on cisapride in non-ulcer dispepsia.

This meta-analysis is used as a data example in Hartung and Knapp
(2001).
}
\examples{
data(cisapride)

m.or <- metabin(event.cisa, n.cisa, event.plac, n.plac,
  data = cisapride, sm = "OR",
  method = "Inverse", method.tau = "DL",
  studlab = study, method.incr = "all")

m.or.hk <- update(m.or, method.random.ci = "HK")
m.rr <- update(m.or, sm = "RR")
m.rr.hk <- update(m.or, sm = "RR", method.random.ci = "HK")

vars.common <- c("TE.common", "lower.common", "upper.common")
vars.random <- c("TE.random", "lower.random", "upper.random")
#
res.common.or <- as.data.frame(m.or[vars.common])
names(res.common.or) <- vars.random
#
res.common.rr <- as.data.frame(m.rr[vars.common])
names(res.common.rr) <- vars.random

# Results for log risk ratio - see Table VII in Hartung and Knapp (2001) 
#
res.rr <- rbind(res.common.rr,
  as.data.frame(m.rr[vars.random]),
  as.data.frame(m.rr.hk[vars.random]))
#
row.names(res.rr) <- c("CE", "RE", "RE (HaKn)")
names(res.rr) <- c("Log risk ratio", "CI lower", "CI upper")
#
res.rr


# Results for log odds ratio (Table VII in Hartung and Knapp 2001) 
#
res.or <- rbind(res.common.or,
  as.data.frame(m.or[vars.random]),
  as.data.frame(m.or.hk[vars.random]))
#
row.names(res.or) <- c("CE", "RE", "RE (HaKn)")
names(res.or) <- c("Log odds ratio", "CI lower", "CI upper")
#
res.or
}
\seealso{
\code{\link{metabin}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabias.R
\name{metabias.meta}
\alias{metabias.meta}
\alias{metabias}
\alias{metabias.default}
\alias{print.metabias}
\title{Test for funnel plot asymmetry}
\usage{
\method{metabias}{meta}(
  x,
  method.bias = x$method.bias,
  plotit = FALSE,
  correct = FALSE,
  k.min = 10,
  ...
)

\method{print}{metabias}(
  x,
  digits = gs("digits"),
  digits.stat = gs("digits.stat"),
  digits.pval = max(gs("digits.pval"), 2),
  digits.se = gs("digits.se"),
  digits.tau2 = gs("digits.tau2"),
  scientific.pval = gs("scientific.pval"),
  big.mark = gs("big.mark"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  text.tau2 = gs("text.tau2"),
  details.methods = gs("details"),
  ...
)

metabias(x, ...)

\method{metabias}{default}(
  x,
  seTE,
  method.bias = "Egger",
  plotit = FALSE,
  correct = FALSE,
  k.min = 10,
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta} or estimated treatment
effect in individual studies.}

\item{method.bias}{A character string indicating which test is to
be used (see Details), can be abbreviated.}

\item{plotit}{A logical indicating whether a plot should be
produced (see Details).}

\item{correct}{A logical indicating whether a continuity corrected
statistic is used for rank correlation tests.}

\item{k.min}{Minimum number of studies to perform test for funnel
plot asymmetry.}

\item{\dots}{Additional arguments passed on to
\code{\link[metafor]{rma.uni}}.}

\item{digits}{Minimal number of significant digits for estimates,
see \code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-value of test for test of funnel plot asymmetry, see
\code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of test for test of funnel plot asymmetry, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
residual heterogeneity variance, see \code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{big.mark}{A character used as thousands separator.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{text.tau2}{Text printed to identify residual heterogeneity
variance \eqn{\tau^2}.}

\item{details.methods}{A logical specifying whether details on
statistical methods should be printed.}

\item{seTE}{Standard error of estimated treatment effect (mandatory
if \code{x} not of class \code{meta}).}
}
\value{
A list with class \code{metabias} containing the following
components if a test for funnel plot asymmetry is conducted:
\item{statistic}{Test statistic.}
\item{df}{The degrees of freedom of the test statistic in
  the case that it follows a t distribution.}
\item{pval}{The p-value for the test.}
\item{estimate}{Estimates used to calculate test statisic.}
\item{method}{A character string indicating what type of test was
  used.}
\item{title}{Title of Cochrane review.}
\item{complab}{Comparison label.}
\item{outclab}{Outcome label.}
\item{var.model}{A character string indicating whether none,
  multiplicative, or additive residual heterogeneity variance was
  assumed.}
\item{method.bias}{As defined above.}
\item{x}{Meta-analysis object.}
\item{version}{Version of R package \bold{meta} used to create
  object.}

Or a list with the following elements if test is not conducted due
to the number of studies:
\item{k}{Number of studies in meta-analysis.}
\item{k.min}{Minimum number of studies to perform test for funnel
  plot asymmetry.}
\item{version}{Version of R package \bold{meta} used to create
  object.}
}
\description{
Test for funnel plot asymmetry, based on rank correlation or linear
regression method.
}
\details{
Functions to conduct rank correlation or linear regression tests
for funnel plot asymmetry.

\subsection{Classic generic tests}{
The following tests are generic tests for funnel plot asymmetry
which only require estimates of the treatment effect and
corresponding standard errors. Accordingly, these are the only
tests provided by R function \code{metabias.default}.

If argument \code{method.bias} is \code{"Begg"}, the test statistic
is based on the rank correlation between standardised treatment
estimates and variance estimates of estimated treatment effects;
Kendall's tau is used as correlation measure (Begg & Mazumdar,
1994). The test statistic follows a standard normal
distribution. By default (if \code{correct} is FALSE), no
continuity correction is utilised (Kendall & Gibbons, 1990).

If argument \code{method.bias} is \code{"Egger"}, the test
statistic is based on a weighted linear regression of the treatment
effect on its standard error (Egger et al., 1997). The test
statistic follows a t distribution with \code{number of studies -
2} degrees of freedom.

If argument \code{method.bias} is \code{"Thompson"}, the test
statistic is based on a weighted linear regression of the treatment
effect on its standard error using an additive between-study
variance component denoted as methods (3a) - (3d) in Thompson &
Sharp (1999). The test statistic follows a t distribution with
\code{number of studies - 2} degrees of freedom.  }

\subsection{Tests for meta-analysis with binary outcomes}{
The following tests for funnel plot asymmetry are only available
for meta-analyses comparing two binary outcomes, i.e. meta-analyses
generated with the \code{metabin} function. The only exception is
the test by Peters et al. (2006) which can also be used in a
meta-analysis of single proportions generated with \code{metaprop}.

If argument \code{method.bias} is \code{"Harbord"}, the test
statistic is based on a weighted linear regression utilising
efficient score and score variance (Harbord et al., 2006,
2009). The test statistic follows a t distribution with
\code{number of studies - 2} degrees of freedom.

In order to calculate an arcsine test for funnel plot asymmetry
(Rücker et al., 2008), one has to use the \code{metabin} function
with argument \code{sm = "ASD"} as input to the \code{metabias}
command. The three arcsine tests described in Rücker et al. (2008)
can be calculated by setting \code{method.bias} to \code{"Begg"},
\code{"Egger"} and \code{"Thompson"}, respectively.

If argument \code{method.bias} is \code{"Macaskill"}, the test
statistic is based on a weighted linear regression of the treatment
effect on the total sample size with weights reciprocal to the
variance of the average event probability (Macaskill et al., 2001,
\emph{method FPV}). The test statistic follows a t distribution
with \code{number of studies - 2} degrees of freedom.

If argument \code{method.bias} is \code{"Peters"}, the test
statistic is based on a weighted linear regression of the treatment
effect on the inverse of the total sample size with weights
reciprocal to the variance of the average event probability (Peters
et al., 2006). The test statistic follows a t distribution with
\code{number of studies - 2} degrees of freedom. Note, this test is
a variant of Macaskill et al. (2001), \emph{method FPV}, using the
inverse sample size as covariate.

If argument \code{method.bias} is \code{"Schwarzer"}, the test
statistic is based on the rank correlation between a standardised
cell frequency and the inverse of the variance of the cell
frequency; Kendall's tau is used as correlation measure (Schwarzer
et al., 2007). The test statistic follows a standard normal
distribution. By default (if \code{correct} is FALSE), no
continuity correction is utilised (Kendall & Gibbons, 1990).

Finally, for meta-analysis of diagnostic test accuracy studies, if
argument \code{method.bias} is \code{"Deeks"}, the test statistic
is based on a weighted linear regression of the log diagnostic odds
ratio on the inverse of the squared effective sample size using the
effective sample size as weights (Deeks et al., 2005). The test
statistic follows a t distribution with \code{number of studies -
2} degrees of freedom.
}

\subsection{Test for the standardised mean difference}{
If argument \code{method.bias} is \code{"Pustejovsky"}, the test
statistic is based on a weighted linear regression of the treatment
effect on the square root of the sum of the inverse group sample
sizes using the treatment effect variance as weights (Pustejovsky &
Rodgers, 2019). The test statistic follows a t distribution with
\code{number of studies - 2} degrees of freedom.
}

\subsection{Recommendations and default settings}{
Following recommendations by Sterne et al. (2011), by default, a
test for funnel plot asymmetry is only conducted if the number of
studies is ten or larger (argument \code{k.min = 10}). This
behaviour can be changed by setting a smaller value for argument
\code{k.min}. Note, the minimum number of studies is three.

If argument \code{method.bias} is missing, the Harbord test
(\code{method.bias = "Harbord"}) is used in meta-analyses with a binary
outcome for the odds ratio and Deeks' test (\code{method.bias = "Deeks"})
for the diagnostic odds ratios. In all other settings, the Egger test
(\code{method.bias = "Egger"}) is used (Sterne et al., 2011).

No test for funnel plot asymmetry is conducted in meta-analyses
with subgroups.
}

If argument \code{plotit = TRUE}, a scatter plot is shown if
argument \code{method.bias} is equal to \code{"Begg"},
\code{"Egger"}, \code{"Thompson"}, \code{"Harbord"}, or
\code{"Deeks"}.
}
\examples{
data(Olkin1995)
ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = 1:10, sm = "RR", method = "I")

metabias(ma1)
metabias(ma1, plotit = TRUE)

metabias(ma1, method.bias = "Begg")
metabias(ma1, method.bias = "Begg", correct = TRUE)

metabias(ma1, method.bias = "Schwarzer")
metabias(ma1, method.bias = "Egger")$pval

# Arcsine test (based on linear regression)
#
ma1.as <- update(ma1, sm = "ASD")
metabias(ma1.as)
# Same result (using function metabias.default)
metabias(ma1.as$TE, ma1.as$seTE)

# No test for funnel plot asymmetry calculated
#
ma2 <- update(ma1, subset = 1:5)
metabias(ma2)

ma3 <- update(ma1, subset = 1:2)
metabias(ma3)

# Test for funnel plot asymmetry calculated (use of argument k.min)
#
metabias(ma2, k.min = 5)

}
\references{
Begg CB & Mazumdar M (1994):
Operating characteristics of a rank correlation test for
publication bias.
\emph{Biometrics},
\bold{50}, 1088--101

Deeks JJ, Macaskill P, Irwig L (2005):
The performance of tests of publication bias and other sample size
effects in systematic reviews of diagnostic test accuracy was
assessed.
\emph{Journal of Clinical Epidemiology},
\bold{58}:882--93

Egger M, Smith GD, Schneider M & Minder C (1997):
Bias in meta-analysis detected by a simple, graphical test.
\emph{British Medical Journal},
\bold{315}, 629--34

Harbord RM, Egger M & Sterne J (2006):
A modified test for small-study effects in meta-analyses of
controlled trials with binary endpoints.
\emph{Statistics in Medicine},
\bold{25}, 3443--57

Harbord RM, Harris RJ, Sterne JAC (2009):
Updated tests for small-study effects in meta–analyses.
\emph{The Stata Journal},
\bold{9}, 197--210

Kendall M & Gibbons JD (1990):
\emph{Rank Correlation Methods}.
London: Edward Arnold

Macaskill P, Walter SD, Irwig L (2001):
A comparison of methods to detect publication bias in
meta-analysis.
\emph{Statistics in Medicine},
\bold{20}, 641--54

Peters JL, Sutton AJ, Jones DR, Abrams KR & Rushton L (2006):
Comparison of two methods to detect publication bias in
meta-analysis.
\emph{Journal of the American Medical Association},
\bold{295}, 676--80

Pustejovsky JE, Rodgers MA (2019):
Testing for funnel plot asymmetry of standardized mean differences.
\emph{Research Synthesis Methods},
\bold{10}, 57--71

Rücker G, Schwarzer G, Carpenter JR (2008):
Arcsine test for publication bias in meta-analyses with binary
outcomes.
\emph{Statistics in Medicine},
\bold{27}, 746--63

Schwarzer G, Antes G & Schumacher M (2007):
A test for publication bias in meta-analysis with sparse binary
data.
\emph{Statistics in Medicine},
\bold{26}, 721--33

Sterne, JAC et al. (2011):
Recommendations for examining and interpreting funnel plot
asymmetry in meta-analyses of randomised controlled trials.
\emph{BMJ (Clinical research ed.)},
\bold{343}, 1

Thompson SG & Sharp, SJ (1999):
Explaining heterogeneity in meta-analysis: a comparison of methods,
\emph{Statistics in Medicine},
\bold{18}, 2693--708
}
\seealso{
\code{\link{funnel}}, \code{\link{funnel.meta}},
  \code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta-object.R
\name{meta-object}
\alias{meta-object}
\title{Description of R object of class "meta"}
\description{
Detailed description of R objects of class "meta".
}
\details{
The following R functions create an object of class \code{"meta"}:
\itemize{
\item \code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metacor}}, \code{\link{metagen}},
  \code{\link{metainc}}, \code{\link{metamean}},
  \code{\link{metaprop}}, \code{\link{metarate}},
  \code{\link{metacr}}, \code{\link{metamerge}},
  \code{\link{trimfill}}
}

The following generic functions are available for an object of
class \code{"meta"}:

\itemize{
\item \code{\link{as.data.frame.meta}}, \code{\link{labels.meta}},
  \code{\link{print.meta}}, \code{\link{print.summary.meta}},
  \code{\link{summary.meta}}, \code{\link{update.meta}},
  \code{\link{weights.meta}}
}

An object of class \code{"meta"} is a list containing the following
components.
\tabular{ll}{
\code{studlab} \tab Study labels \cr
\code{sm} \tab Effect measure \cr
\code{null.effect} \tab Effect under the null hypothesis \cr
\code{TE} \tab Effect estimates (individual studies) \cr
\code{seTE} \tab Standard error of effect estimates (individual studies) \cr
\code{statistic} \tab Statistics for test of effect (individual studies) \cr
\code{pval} \tab P-values for test of effect (individual studies) \cr
\code{df} \tab Degrees of freedom (individual studies) \cr
\code{level} \tab Level of confidence intervals for individual studies \cr
\code{lower} \tab Lower confidence limits (individual studies) \cr
\code{upper} \tab Upper confidence limits (individual studies) \cr
\code{three.level} \tab Indicator variable for three-level
  meta-analysis model \cr
\code{cluster} \tab Cluster variable (three-level meta-analysis
  model) \cr
\code{rho} \tab Within-cluster correlation (three-level
  meta-analysis model) \cr
\code{k} \tab Number of estimates combined in meta-analysis \cr
\code{k.study} \tab Number of studies combined in meta-analysis \cr
\code{k.all} \tab Number of all studies \cr
\code{k.TE} \tab Number of studies with estimable effects \cr
\code{overall} \tab Print meta-analysis results \cr
\code{overall.hetstat} \tab Print overall heterogeneity statistics \cr
\code{common} \tab Print results for common effect meta-analysis \cr
\code{random} \tab Print results for random effects meta-analysis \cr
\code{prediction} \tab Print prediction interval \cr
\code{backtransf} \tab Back transform results in printouts and plots \cr
\code{method} \tab Meta-analysis method (common effect model) \cr
\code{method.random} \tab Meta-analysis method (random effects model) \cr
\code{w.common} \tab Weights for common effect model (individual studies) \cr
\code{TE.common} \tab Estimated overall effect (common effect model) \cr
\code{seTE.common} \tab Standard error of overall effect (common effect model) \cr
\code{statistic.common} \tab Statistic for test of overall effect (common effect model) \cr
\code{pval.common} \tab P-value for test of overall effect (common effect model) \cr
\code{level.ma} \tab Level of confidence interval for meta-analysis
  estimates \cr
\code{lower.common} \tab Lower confidence limit (common effect model) \cr
\code{upper.common} \tab Upper confidence limit (common effect model) \cr
\code{w.random} \tab Weight for random effects model (individual studies) \cr
\code{TE.random} \tab Estimated overall effect (random effects model) \cr
\code{seTE.random} \tab Standard error of overall effect (random effects model) \cr
\code{statistic.random} \tab Statistic for test of overall effect (random effects model) \cr
\code{pval.random} \tab P-value for test of overall effect (random effects model) \cr
\code{method.random.ci} \tab Confidence interval method (random effects model) \cr
\code{df.random} \tab Degrees of freedom (random effects model) \cr
\code{lower.random} \tab Lower confidence limit (random effects model) \cr
\code{upper.random} \tab Upper confidence limit (random effects model) \cr
\code{seTE.classic} \tab Standard error (classic random effects method) \cr
\code{adhoc.hakn.ci} \tab \emph{Ad hoc} correction for
  Hartung-Knapp method (confidence interval) \cr
\code{df.hakn.ci} \tab Degrees of freedom for Hartung-Knapp method
  \cr \tab (if used in meta-analysis) \cr
\code{seTE.hakn.ci} \tab Standard error for Hartung-Knapp method
  \cr
\tab (not taking \emph{ad hoc} variance correction into account)
  \cr
\code{seTE.hakn.adhoc.ci} \tab Standard error for Hartung-Knapp
  method \cr
\tab (taking \emph{ad hoc} variance correction into account) \cr
\code{df.kero} \tab Degrees of freedom for Kenward-Roger method \cr
\tab  (if used in meta-analysis) \cr 
\code{seTE.kero} \tab Standard error for Kenward-Roger
  method \cr
\code{method.predict} \tab Method to calculate prediction interval \cr
\code{adhoc.hakn.pi} \tab \emph{Ad hoc} correction for
  Hartung-Knapp method (prediction interval) \cr
\code{df.hakn.ci} \tab Degrees of freedom for Hartung-Knapp method
  \cr \tab (prediction interval) \cr
\code{seTE.predict} \tab Standard error used to calculate
  prediction interval \cr
\code{df.predict} \tab Degrees of freedom for prediction interval \cr
\code{level.predict} \tab Level of prediction interval \cr
\code{lower.predict} \tab Lower limit of prediction interval \cr
\code{upper.predict} \tab Upper limit of prediction interval \cr
\code{seTE.hakn.pi} \tab Standard error for Hartung-Knapp method
  \cr
\tab (not taking \emph{ad hoc} variance correction into account)
  \cr
\code{seTE.hakn.adhoc.pi} \tab Standard error for Hartung-Knapp
  method \cr
\tab (taking \emph{ad hoc} variance correction into account) \cr
\code{Q} \tab Heterogeneity statistic \cr
\code{df.Q} \tab Degrees of freedom for heterogeneity statistic
  \code{Q} \cr
\code{pval.Q} \tab P-value of heterogeneity test \cr
\code{method.tau} \tab Method to estimate between-study variance \eqn{\tau^2} \cr
\code{control} \tab Additional arguments for iterative estimation
  of \eqn{\tau^2} \cr
\code{method.tau.ci} \tab Method for confidence interval of \eqn{\tau^2} \cr
\code{method.I2} \tab Method to estimate heterogeneity statistics I\eqn{^2}
  and H \cr
\code{level.hetstat} \tab Level of confidence intervals for heterogeneity
  statistics \cr
\code{tau2} \tab Between-study variance \eqn{\tau^2} \cr
\code{se.tau2} \tab Standard error of \eqn{\tau^2} \cr
\code{lower.tau2} \tab Lower confidence limit for \eqn{\tau^2} [1] \cr
\code{upper.tau2} \tab Upper confidence limit for \eqn{\tau^2} [1] \cr
\code{tau} \tab Square-root of between-study variance \eqn{\tau} \cr
\code{lower.tau} \tab Lower confidence limit for \eqn{\tau} [1] \cr
\code{upper.tau} \tab Upper confidence limit for \eqn{\tau} [1] \cr
\code{tau.preset} \tab Prespecified value for \eqn{\tau} \cr
\code{TE.tau} \tab Effect estimate used to estimate \eqn{\tau^2} \cr
\code{detail.tau} \tab Detail on between-study variance estimate \cr
\code{phi} \tab Multiplicative heterogeneity parameter \eqn{\phi} in
  penalised logistic regression \cr
\code{H} \tab Heterogeneity statistic H \cr
\code{lower.H} \tab Lower confidence limit for H [2] \cr
\code{upper.H} \tab Upper confidence limit for H [2] \cr
\code{I2} \tab Heterogeneity statistic I\eqn{^2} \cr
\code{lower.I2} \tab Lower confidence limit for I\eqn{^2} [2] \cr
\code{upper.I2} \tab Upper confidence limit for I\eqn{^2} [2] \cr
\code{Rb} \tab Heterogeneity statistic R\eqn{_b} \cr
\code{lower.Rb} \tab Lower confidence limit for R\eqn{_b} \cr
\code{upper.Rb} \tab Upper confidence limit for R\eqn{_b} \cr
\code{method.bias} \tab Method to test for funnel plot asymmetry \cr
\code{text.common} \tab Label for common effect model \cr
\code{text.random} \tab Label for random effects model \cr
\code{text.predict} \tab Label for prediction interval \cr
\code{text.w.common} \tab Label for weights (common effect model) \cr
\code{text.w.random} \tab Label for weights (random effects model) \cr
\code{title} \tab Title of meta-analysis / systematic review \cr
\code{complab} \tab Comparison label \cr
\code{outclab} \tab Outcome label \cr
\code{label.e} \tab Label for experimental group \cr
\code{label.c} \tab Label for control group \cr
\code{label.left} \tab Graph label on left side of forest plot \cr
\code{label.right} \tab Graph label on right side of forest plot \cr
\code{keepdata} \tab Keep original data \cr
\code{data} \tab Original data (set) used in function call (if
  \code{keepdata = TRUE}) \cr
\code{subset} \tab Information on subset of original data used in
  meta-analysis \cr
\tab (if \code{keepdata = TRUE}) \cr
\code{exclude} \tab Studies excluded from meta-analysis \cr
\code{warn} \tab Print warnings \cr
\code{call} \tab Function call \cr
\code{version} \tab Version of R package \bold{meta} used to create
  object
}
with
[1] not available for generalized linear mixed model;
[2] not available for generalized linear mixed model or multi-level model
  if calculation of I\eqn{^2} is based on \eqn{\tau^2}
   (argument \code{method.I2 = "tau2"}).

For subgroup analysis (argument \code{subgroup}), the following
additional components are added to the list.
\tabular{ll}{
\code{subgroup} \tab Subgroup information (for individual studies) \cr
\code{subgroup.name} \tab Name of subgroup variable \cr
\code{print.subgroup.name} \tab Print name of subgroup variable \cr
\code{sep.subgroup} \tab Separator between name of subgroup variable and value \cr
\code{test.subgroup} \tab Print test for subgroup differences \cr
\code{prediction.subgroup} \tab Print prediction interval for subgroup(s) \cr
\code{tau.common} \tab Assumption of common between-study variance in subgroups \cr
\code{subgroup.levels} \tab Levels of grouping variable \cr
\code{k.w} \tab Number of estimates combined in subgroups \cr
\code{k.study.w} \tab Number of studies combined in subgroups \cr
\code{k.all.w} \tab Number of studies in subgroups \cr
\code{k.TE.w} \tab Number of studies with estimable effects in subgroups \cr
\code{TE.common.w} \tab Estimated effect in subgroups (common
  effect model) \cr
\code{seTE.common.w} \tab Standard error in subgroups (common effect model) \cr
\code{statistic.common.w} \tab Statistic for test of effect in subgroups (common effect model) \cr
\code{pval.common.w} \tab P-value for test of effect in subgroups (common effect model) \cr
\code{lower.common.w} \tab Lower confidence limit in subgroups (common effect model) \cr
\code{upper.common.w} \tab Upper confidence limit in subgroups (common effect model) \cr
\code{w.common.w} \tab Total weight in subgroups (common effect model) \cr
\code{TE.random.w} \tab Estimated effect in subgroups (random
  effect model) \cr
\code{seTE.random.w} \tab Standard error in subgroups (random effects model) \cr
\code{statistic.random.w} \tab Statistic for test of effect in subgroups (random effects model) \cr
\code{pval.random.w} \tab P-value for test of effect in subgroups (random effects model) \cr
\code{df.random.w} \tab Degrees of freedom in subgroups (random effects model) \cr
\code{lower.random.w} \tab Lower confidence limit in subgroups (random effects model) \cr
\code{upper.random.w} \tab Upper confidence limit in subgroups (random effects model) \cr
\code{w.random.w} \tab Total weight in subgroups (random effects model) \cr
\code{seTE.classic.w} \tab Standard error (classic random effects method) \cr
\code{df.hakn.ci.w} \tab Degrees of freedom for Hartung-Knapp
  method in subgroups \cr
\code{seTE.hakn.ci.w} \tab Standard error for Hartung-Knapp method in subgroups
  \cr
\tab (not taking \emph{ad hoc} variance correction into account)
  \cr
\code{seTE.hakn.adhoc.ci.w} \tab Standard error for Hartung-Knapp
  method in subgroups \cr
\code{df.kero.w} \tab Degrees of freedom for Kenward-Roger method in subgroups \cr 
\code{seTE.kero.w} \tab Standard error for Kenward-Roger
  method in subgroups \cr
\code{seTE.predict.w} \tab Standard error for prediction interval in subgroups \cr
\code{df.predict.w} \tab Degrees of freedom for prediction interval in subgroups \cr
\code{lower.predict.w} \tab Lower limit of prediction interval in subgroups \cr
\code{upper.predict.w} \tab Upper limit of prediction interval in subgroups \cr
\code{seTE.hakn.pi.w} \tab Standard error for Hartung-Knapp method in
  subgroups (prediction intervals) \cr
\tab (not taking \emph{ad hoc} variance correction into account) \cr
\code{seTE.hakn.adhoc.pi.w} \tab Standard error for Hartung-Knapp
  method in subgroups (prediction intervals) \cr
\code{Q.w} \tab Heterogeneity statistic Q in subgroups \cr
\code{pval.Q.w} \tab P-value for test of heterogeneity in subgroups \cr
\code{tau2.w} \tab Between-study variance \eqn{\tau^2} in subgroups \cr
\code{lower.tau2.w} \tab Lower confidence limit for \eqn{\tau^2} in subgroups \cr
\code{upper.tau2.w} \tab Upper confidence limit for \eqn{\tau^2} in subgroups \cr
\code{tau.w} \tab Square-root of between-study variance \eqn{\tau} in subgroups \cr
\code{lower.tau.w} \tab Lower confidence limit for \eqn{\tau} in subgroups \cr
\code{upper.tau.w} \tab Upper confidence limit for \eqn{\tau} in subgroups \cr
\code{H.w} \tab Heterogeneity statistic H in subgroups \cr
\code{lower.H.w} \tab Lower confidence limit for H in subgroups \cr
\code{upper.H.w} \tab Upper confidence limit for H in subgroups \cr
\code{I2.w} \tab Heterogeneity statistic I\eqn{^2} in subgroups \cr
\code{lower.I2.w} \tab Lower confidence limit for I\eqn{^2} in subgroups \cr
\code{upper.I2.w} \tab Upper confidence limit for I\eqn{^2} in subgroups \cr
\code{Rb.w} \tab Heterogeneity statistic R\eqn{_b} in subgroups \cr
\code{lower.Rb.w} \tab Lower confidence limit for R\eqn{_b} in subgroups \cr
\code{upper.Rb.w} \tab Upper confidence limit for R\eqn{_b} in subgroups \cr
\code{Q.w.common} \tab Within-group heterogeneity statistic Q
  (common effect model) \cr
\code{Q.w.random} \tab Within-group heterogeneity statistic
  Q (random effects model) \cr
\tab (only calculated if argument \code{tau.common = TRUE}) \cr
\code{df.Q.w} \tab Degrees of freedom for \code{Q.w.common} and \code{Q.w.random} \cr
\code{pval.Q.w.common} \tab P-value of test for residual
  heterogeneity (common effect model) \cr
\code{pval.Q.w.random} \tab P-value of test for residual
  heterogeneity (random effects model) \cr
\code{Q.b.common} \tab Between-groups heterogeneity statistic
  Q (common effect model) \cr
\code{df.Q.b.common} \tab Degrees of freedom for \code{Q.b.common} \cr
\code{pval.Q.b.common} \tab P-value of test for subgroup
  differences (common effect model) \cr
\code{Q.b.random} \tab Between-groups heterogeneity statistic
  Q (random effects model) \cr
\code{df.Q.b.random} \tab Degrees of freedom for \code{Q.b.random} \cr
\code{pval.Q.b.random} \tab P-value of test for subgroup
  differences (random effects model)
}

For subgroup analysis assuming a common between-study variance in subgroups
(argument \code{tau.common = TRUE}), the following additional components are
added to the list.
\tabular{ll}{
\code{tau2.resid} \tab Residual between-study variance after accounting for
  \cr
\code{} \tab subgroup differences (equal to entries in list element
  \code{tau2.w}) \cr
\code{lower.tau2.resid} \tab Lower confidence limit for
  residual \eqn{\tau^2} [1] \cr
\code{upper.tau2.resid} \tab Upper confidence limit for
  residual \eqn{\tau^2} [1] \cr
\code{tau.resid} \tab Square-root of residual between-study variance \cr
\code{lower.tau.resid} \tab Lower confidence limit for
  residual \eqn{\tau} [1] \cr
\code{upper.tau.resid} \tab Upper confidence limit for
  residual \eqn{\tau} [1] \cr
\code{H.resid} \tab Heterogeneity statistic H after accounting for
  subgroup differences \cr
\code{lower.H.resid} \tab Lower confidence limit for residual H [2] \cr
\code{upper.H.resid} \tab Upper confidence limit for residual H [2] \cr
\code{I2.resid} \tab Heterogeneity statistic I\eqn{^2} after accounting for
  subgroup differences \cr
\code{lower.I2.resid} \tab Lower confidence limit for
  residual I\eqn{^2} [2] \cr
\code{upper.I2.resid} \tab Upper confidence limit for
  residual I\eqn{^2} [2] \cr
\code{Q.resid} \tab Heterogeneity statistic for residual heterogeneity \cr
\code{df.Q.resid} \tab Degrees of freedom for heterogeneity statistic
  \code{Q.resid} \cr
\code{pval.Q.resid} \tab P-value of test for residual heterogeneity \cr
}
with
[1] not available for generalized linear mixed model;
[2] not available for generalized linear mixed model or multi-level model
  if calculation of I\eqn{^2} is based on \eqn{\tau^2}
   (argument \code{method.I2 = "tau2"}).

An object created with \code{\link{metabin}} has the additional
class \code{"metabin"} and the following components.

\tabular{ll}{
\code{event.e} \tab Events in experimental group (individual
  studies) \cr
\code{n.e} \tab Sample size in experimental group (individual
  studies) \cr
\code{event.e} \tab Events in control group (individual studies)
  \cr
\code{n.e} \tab Sample size in control group (individual studies)
  \cr
\code{incr} \tab Increment added to zero cells \cr
\code{method.incr} \tab Continuity correction method \cr
\code{sparse} \tab Continuity correction applied \cr
\code{allstudies} \tab Include studies with double zeros \cr
\code{doublezeros} \tab Indicator for studies with double zeros \cr
\code{MH.exact} \tab Exact Mantel-Haenszel method \cr
\code{RR.Cochrane} \tab Cochrane method to calculate risk ratio \cr
\code{Q.Cochrane} \tab Cochrane method to calculate \eqn{\tau^2}
  \cr
\code{Q.CMH} \tab Cochran-Mantel-Haenszel statistic \cr
\code{df.Q.CMH} \tab Degrees of freedom for \code{Q.CMH} \cr
\code{pval.Q.CMH} \tab P-value of Cochran-Mantel-Haenszel test \cr
\code{print.CMH} \tab Print results for Cochran-Mantel-Haenszel
  statistic \cr
\code{incr.e} \tab Continuity correction in experimental group
  (individual studies) \cr
\code{incr.c} \tab Continuity correction in control group
  (individual studies) \cr
\code{k.MH} \tab Number of studies (Mantel-Haenszel method)
}

An object created with \code{\link{metacont}} has the additional
class \code{"metacont"} and the following components.

\tabular{ll}{
\code{n.e} \tab Sample size in experimental group (individual
  studies) \cr
\code{mean.e} \tab Estimated mean in experimental group (individual
  studies) \cr
\code{sd.e} \tab Standard deviation in experimental group
  (individual studies) \cr
\code{n.c} \tab Sample size in control group (individual studies)
  \cr
\code{mean.c} \tab Estimated mean in control group (individual
  studies) \cr
\code{sd.c} \tab Standard deviation in control group (individual
  studies) \cr
\code{pooledvar} \tab Use pooled variance for mean difference \cr
\code{method.smd} \tab Method for standardised mean difference
  (SMD) \cr
\code{sd.glass} \tab Denominator in Glass' method \cr
\code{exact.smd} \tab Use exact formulae for SMD \cr
\code{method.ci} \tab Method to calculate confidence limits \cr
\code{method.mean} \tab Method to approximate mean \cr
\code{method.sd} \tab Method to approximate standard deviation
}

An object created with \code{\link{metacor}} has the additional
class \code{"metacor"} and the following components.

\tabular{ll}{
\code{cor} \tab Correlation (individual studies) \cr
\code{n} \tab Sample size (individual studies)
}

An object created with \code{\link{metainc}} has the additional
class \code{"metainc"} and the following components.

\tabular{ll}{
\code{event.e} \tab Events in experimental group (individual
  studies) \cr
\code{time.e} \tab Person time in experimental group (individual
  studies) \cr
\code{n.e} \tab Sample size in experimental group (individual
  studies) \cr 
\code{event.c} \tab Events in control group (individual studies)
  \cr
\code{time.c} \tab Person time in control group (individual studies)
  \cr
\code{n.c} \tab Sample size in control group (individual studies)
  \cr
\code{incr} \tab Increment added to zero cells \cr
\code{method.incr} \tab Continuity correction method \cr
\code{sparse} \tab Continuity correction applied \cr
\code{incr.e} \tab Continuity correction in experimental group
  (individual studies) \cr
\code{incr.c} \tab Continuity correction in control group
  (individual studies) \cr
\code{k.MH} \tab Number of studies (Mantel-Haenszel method)
}

An object created with \code{\link{metamean}} has the additional
class \code{"metamean"} and the following components.

\tabular{ll}{
\code{n} \tab Sample size (individual studies) \cr
\code{mean} \tab Estimated mean (individual studies) \cr
\code{sd} \tab Standard deviation (individual studies) \cr
\code{method.ci} \tab Method to calculate confidence limits \cr
\code{method.mean} \tab Method to approximate mean \cr
\code{method.sd} \tab Method to approximate standard deviation \cr
}

An object created with \code{\link{metaprop}} has the additional
class \code{"metaprop"} and the following components.

\tabular{ll}{
\code{event} \tab Events (individual studies) \cr
\code{n} \tab Sample size (individual studies) \cr
\code{incr} \tab Increment added to zero cells \cr
\code{method.incr} \tab Continuity correction method \cr
\code{sparse} \tab Continuity correction applied \cr
\code{method.ci} \tab Method to calculate confidence limits \cr
\code{incr.event} \tab Continuity correction (individual studies)
}

An object created with \code{\link{metarate}} has the additional
class \code{"metarate"} and the following components.

\tabular{ll}{
\code{event} \tab Events (individual studies) \cr
\code{time} \tab Person time (individual studies) \cr
\code{n} \tab Sample size (individual studies) \cr
\code{incr} \tab Increment added to zero cells \cr
\code{method.incr} \tab Continuity correction method \cr
\code{sparse} \tab Continuity correction applied \cr
\code{method.ci} \tab Method to calculate confidence limits \cr
\code{incr.event} \tab Continuity correction (individual studies)
}

An object created with \code{\link{trimfill}} has the additional
classes \code{"trimfill"} and \code{"metagen"} and the following
components.

\tabular{ll}{
\code{k0} \tab Number of added studies \cr
\code{left} \tab Studies missing on left side \cr
\code{ma.common} \tab Use common effect or random effects model to
  estimate \cr
\tab number of missing studies \cr
\code{type} \tab Method to estimate missing studies \cr
\code{n.iter.max} \tab Maximum number of iterations \cr
\code{n.iter} \tab Number of iterations \cr
\code{trimfill} \tab Filled studies (individual studies) \cr
\code{class.x} \tab Primary class of meta-analysis object
}

An object created with \code{\link{metamerge}} has the additional
class \code{"metamerge"}. Furthermore, the following components
have a different meaning:

\tabular{ll}{
\code{k} \tab Vector with number of estimates \cr
\code{k.study} \tab Vector with number of studies \cr
\code{k.all} \tab Vector with total number of studies \cr
\code{k.TE} \tab Vector with number of studies with estimable
  effects \cr
\code{k.MH} \tab Vector with number of studies combined with
  Mantel-Haenszel method \cr
\code{TE.common} \tab Vector with common effect estimates \cr
\code{seTE.common} \tab Vector with standard errors of common effect
  estimates \cr
\code{lower.common} \tab Vector with lower confidence limits (common
  effect model) \cr
\code{upper.common} \tab Vector with upper confidence limits (common
  effect model) \cr
\code{statistic.common} \tab Vector with test statistics for test of
  overall effect (common effect model) \cr
\code{pval.common} \tab Vector with p-value of test for overall effect
  (common effect model) \cr
\code{TE.random} \tab Vector with random effects estimates \cr
\code{seTE.random} \tab Vector with standard errors of random effects
  estimates \cr
\code{lower.random} \tab Vector with lower confidence limits (random
  effects model) \cr
\code{upper.random} \tab Vector with upper confidence limits (random
  effects model) \cr
\code{statistic.random} \tab Vector with test statistics for test of
  overall effect (random effects model) \cr
\code{pval.random} \tab Vector with p-value of test for overall effect
  (random effects model) \cr
\code{w.common} \tab Vector or matrix with common effect weights \cr
\code{w.random} \tab Vector or matrix with random effects weights
}
}
\seealso{
\code{\link{meta-package}}, \code{\link{meta-sm}},
  \code{\link{print.meta}}, \code{\link{summary.meta}},
  \code{\link{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{list}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/drapery.R
\name{drapery}
\alias{drapery}
\title{Drapery plot}
\usage{
drapery(
  x,
  type = "zvalue",
  layout = "grayscale",
  study.results = TRUE,
  lty.study = 1,
  lwd.study = 1,
  col.study = "darkgray",
  labels,
  col.labels = "black",
  cex.labels = 0.7,
  subset.labels,
  srt.labels,
  common = x$common,
  random = x$random,
  lty.common = 1,
  lwd.common = max(3, lwd.study),
  col.common = "blue",
  lty.random = 1,
  lwd.random = lwd.common,
  col.random = "red",
  sign = NULL,
  lty.sign = 1,
  lwd.sign = 1,
  col.sign = "black",
  prediction = random,
  col.predict = "lightblue",
  alpha = if (type == "zvalue") c(0.001, 0.01, 0.05, 0.1) else c(0.01, 0.05, 0.1),
  lty.alpha = 2,
  lwd.alpha = 1,
  col.alpha = "black",
  cex.alpha = 0.7,
  col.null.effect = "black",
  legend = TRUE,
  pos.legend = "topleft",
  bg = "white",
  bty = "o",
  backtransf = x$backtransf,
  xlab,
  ylab,
  xlim,
  ylim,
  lwd.max = 2.5,
  lwd.study.weight = if (random) "random" else "common",
  at = NULL,
  n.grid = if (type == "zvalue") 10000 else 1000,
  mar = c(5.1, 4.1, 4.1, 4.1),
  plot = TRUE,
  warn.deprecated = gs("warn.deprecated"),
  fixed,
  lwd.fixed,
  lty.fixed,
  col.fixed,
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{type}{A character string indicating whether to plot test
statistics (\code{"zvalue"}) or p-values (\code{"pvalue"}), can
be abbreviated.}

\item{layout}{A character string for the line layout of individual
studies: \code{"grayscale"}, \code{"equal"}, or
\code{"linewidth"} (see Details), can be abbreviated.}

\item{study.results}{A logical indicating whether results for
individual studies should be shown in the figure.}

\item{lty.study}{Line type for individual studies.}

\item{lwd.study}{Line width for individual studies.}

\item{col.study}{Colour of lines for individual studies.}

\item{labels}{A logical or character string indicating whether
study labels should be shown at the top of the drapery plot;
either \code{FALSE}, \code{"id"}, or \code{"studlab"}; see
Details.}

\item{col.labels}{Colour of study labels.}

\item{cex.labels}{The magnification for study labels.}

\item{subset.labels}{A vector specifying which study labels should
be shown in the drapery plot.}

\item{srt.labels}{A numerical vector or single numeric (between 0
and 90) specifying the angle to rotate study labels; see Details.}

\item{common}{A logical indicating whether to show result for the
common effect model.}

\item{random}{A logical indicating whether to show result for the
random effects model.}

\item{lty.common}{Line type for common effect meta-analysis.}

\item{lwd.common}{Line width for common effect meta-analysis.}

\item{col.common}{Colour of lines for common effect meta-analysis.}

\item{lty.random}{Line type for random effects meta-analysis.}

\item{lwd.random}{Line width for random effects meta-analysis.}

\item{col.random}{Colour of lines for random effects meta-analysis.}

\item{sign}{Significance level used to highlight significant values
in curves.}

\item{lty.sign}{Line type for significant values.}

\item{lwd.sign}{Line width for significant values.}

\item{col.sign}{Line colour for significant values.}

\item{prediction}{A logical indicating whether to show prediction
region.}

\item{col.predict}{Colour of prediction region}

\item{alpha}{Horizonal lines are printed for the specified alpha
values.}

\item{lty.alpha}{Line type of horizonal lines for alpha values.}

\item{lwd.alpha}{Line width of horizonal lines for alpha values.}

\item{col.alpha}{Colour of horizonal lines for alpha values.}

\item{cex.alpha}{The magnification for the text of the alpha}

\item{col.null.effect}{Colour of vertical line indicating null
effect.}

\item{legend}{A logical indicating whether a legend should be
printed.}

\item{pos.legend}{A character string with position of legend (see
\code{\link{legend}}).}

\item{bg}{Background colour of legend (see \code{\link{legend}}).}

\item{bty}{Type of the box around the legend; either \code{"o"} or
\code{"n"} (see \code{\link{legend}}).}

\item{backtransf}{A logical indicating whether results should be
back transformed on the x-axis. For example, if \code{backtransf
= FALSE}, log odds ratios instead of odds ratios are shown on the
x-axis.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{xlim}{The x limits (min, max) of the plot.}

\item{ylim}{The y limits (min, max) of the plot (ignored if
\code{type = "pvalue"}).}

\item{lwd.max}{The maximum line width (only considered if argument
\code{layout} is equal to \code{"linewidth"}).}

\item{lwd.study.weight}{A character string indicating whether to
determine line width for individual studies using weights from
common effect (\code{"common"}) or random effects model
(\code{"random"}), can be abbreviated (only considered if
argument \code{layout} is equal to \code{"linewidth"}).}

\item{at}{Points at which tick-marks are to be drawn on the x-axis.}

\item{n.grid}{The number of grid points to calculate the p-value or
test statistic functions.}

\item{mar}{Physical plot margin, see \code{\link{par}}.}

\item{plot}{A logical indicating whether to generate a figure.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{fixed}{Deprecated argument (replaced by 'common').}

\item{lwd.fixed}{Deprecated argument (replaced by 'lwd.common').}

\item{lty.fixed}{Deprecated argument (replaced by 'lty.common').}

\item{col.fixed}{Deprecated argument (replaced by 'col.common').}

\item{\dots}{Graphical arguments as in \code{par} may also be
passed as arguments.}
}
\description{
Draw a drapery plot with (scaled) p-value curves for individual
studies and meta-analysis estimates.
}
\details{
The concept of a p-value function, also called confidence curve,
goes back to Birnbaum (1961). A drapery plot, showing p-value
functions (or a scaled version based on the corresponding test
statistics) for individual studies as well as meta-analysis
estimates, is drawn in the active graphics window. Furthermore, a
prediction region for a single future study is shown as a shaded
area. In contrast to a forest plot, a drapery plot does not provide
information for a single confidence level however for any
confidence level.

Argument \code{type} can be used to either show p-value functions
(Birnbaum, 1961) or a scaled version (Infanger, 2019) with test
statistics (default).

Argument \code{layout} determines how curves for individual studies
are presented:
\itemize{
\item darker gray tones with increasing precision (\code{layout =
  "grayscale"})
\item thicker lines with increasing precision (\code{layout =
  "linewidth"})
\item equal lines (\code{layout = "equal"})
}

Argument \code{labels} determines how curves of individual studies
are labelled:
\itemize{
\item number of the study in the (unsorted) forest plot / printout
  of a meta-analysis (\code{labels = "id"})
\item study labels provided by argument \code{studlab} in
  meta-analysis functions (\code{labels = "studlab"})
\item no study labels (\code{labels = FALSE})
}
By default, study labels are used (\code{labels = "studlab"}) if no
label has more than three characters; otherwise IDs are used
(\code{labels = "id"}). The connection between IDs and study labels
(among other information) is part of a data frame which is
invisibly returned (if argument \code{ study.results = TRUE}).

Argument \code{srt.labels} can be used to change the rotation of
IDs or study labels. By default, study labels are rotated by +/- 45
degrees if at least one study label has more than three characters;
otherwise labels are not rotated.

If \code{labels = "studlab"}, labels are rotated by -45 degrees for
studies with a treatment estimate below the common effect estimate
and otherwise by 45 degrees.
}
\examples{
data("lungcancer")
ma1 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
  data = lungcancer, studlab = study)

# Drapery plot
#
drapery(ma1, xlim = c(0.5, 50))

\dontrun{
data(Fleiss1993bin)
ma2 <- metabin(d.asp, n.asp, d.plac, n.plac,
  data = Fleiss1993bin, studlab = paste(study, year),
  sm = "OR", random = FALSE)

# Produce drapery plot and print data frame with connection between
# IDs and study labels
#
(drapery(ma2))

# For studies with a significant effect (p < 0.05), show
# study labels and print labels and lines in red
#
drapery(ma2,
  labels = "studlab", subset.labels = pval < 0.05,
  srt.labels = 0, col.labels = "red",
  col.study = ifelse(pval < 0.05, "red", "darkgray"))
}

}
\references{
Birnbaum A (1961):
Confidence Curves: An Omnibus Technique for Estimation and Testing
Statistical Hypotheses.
\emph{Journal of the American Statistical Association},
\bold{56}, 246--9

Infanger D and Schmidt-Trucksäss A (2019):
P value functions: An underused method to present research results
and to promote quantitative reasoning
\emph{Statistics in Medicine},
\bold{38}, 4189--97
}
\seealso{
\code{\link{forest}}, \code{\link{radial}}
}
\author{
Gerta Rücker \email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/barplot.rob.R
\name{barplot.rob}
\alias{barplot.rob}
\title{Produce weighted bar plot of risk of bias assessment}
\usage{
\method{barplot}{rob}(
  height,
  overall = FALSE,
  weighted = TRUE,
  colour = "cochrane",
  quiet = FALSE,
  ...
)
}
\arguments{
\item{height}{An object of class \code{rob}.}

\item{overall}{A logical indicating whether to include a bar for
overall risk of bias in the figure.}

\item{weighted}{A logical indicating whether weights should be used
in the bar plot.}

\item{colour}{Specify colour scheme for the bar plot; see
\code{\link[robvis]{rob_summary}}.}

\item{quiet}{A logical to suppress the display of the bar plot.}

\item{\dots}{Additional arguments (ignored)}
}
\description{
Produce weighted bar plot of risk of bias assessment
}
\details{
This is a wrapper function for \code{\link[robvis]{rob_summary}} of
R package \bold{robvis} to produce a weighted bar plot of risk of
bias assessment.
}
\examples{
# Use RevMan 5 settings
oldset <- settings.meta("RevMan5")

data(caffeine)

ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
  data = caffeine, studlab = paste(study, year))

# Add risk of bias assessment to meta-analysis
ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")

# Print risk of bias assessment
rob(ma2)

\dontrun{
# Weighted bar plot (R package 'robvis' must be available)
if (requireNamespace("robvis", quietly = TRUE))
 barplot(rob(ma2))
}

# Use previous settings
settings.meta(oldset)

}
\seealso{
\code{\link{rob}}, \code{\link{traffic_light}},
  \code{\link[robvis]{rob_summary}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.longarm.R
\name{subset.longarm}
\alias{subset.longarm}
\title{Return subset of longarm object}
\usage{
\method{subset}{longarm}(x, subset, ...)
}
\arguments{
\item{x}{An object of class \code{longarm}.}

\item{subset}{A logical expression indicating elements or rows to
keep: missing values are taken as false.}

\item{\dots}{Additional arguments.}
}
\value{
A \code{\link{longarm}} object is returned.
}
\description{
The \code{subset} method returns a subset of a longarm object.
}
\examples{
# Transform data from wide arm-based format to contrast-based format
pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
  n = list(n1, n2, n3),
  mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
  data = Franchini2012, studlab = Study)

# Transform data from contrast-based to long arm-based format
# and only keep the main variables
la1 <- longarm(pw1, append = FALSE)
head(la1)

# Subset without Lieberman studies
subset(la1, grepl("Lieberman", studlab))

}
\seealso{
\code{\link{longarm}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weights.meta.R
\name{weights.meta}
\alias{weights.meta}
\title{Calculate absolute and percentage weights for meta-analysis}
\usage{
\method{weights}{meta}(
  object,
  common = object$common,
  random = object$random,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{object}{An object of class \code{meta}.}

\item{common}{A logical indicating whether absolute and
percentage weights from the common effect model should be
calculated.}

\item{random}{A logical indicating whether absolute and
percentage weights from the random effects model should be
calculated.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
A data frame with the following variables is returned:
\tabular{lll}{
\bold{Variable} \tab \bold{Definition} \tab \bold{Condition} \cr
w.common \tab absolute weights in common effect model \tab (if
  \code{common = TRUE}) \cr
p.common \tab percentage weights in common effect model \tab (if
  \code{common = TRUE}) \cr
w.random \tab absolute weights in random effects model \tab (if
  \code{random = TRUE}) \cr
p.random \tab percentage weights in random effects model \tab (if
  \code{random = TRUE})
}
}
\description{
This function returns a data frame containing information on
absolute and percentage weights of individual studies contributing
to common effect and random effects meta-analysis.
}
\examples{
data(Fleiss1993cont)
# Do meta-analysis (common effect and random effects model)
#
meta1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, studlab = paste(study, year), sm = "SMD")

# Print weights for common effect and random effects meta-analysis
#
weights(meta1)

# Do meta-analysis (only random effects model)
#
meta2 <- update(meta1, common = FALSE)

# Print weights for random effects meta-analysis
#
weights(meta2)

# Print weights for common effect and random effects meta-analysis
#
weights(meta2, common = TRUE)

}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/baujat.R
\name{baujat.meta}
\alias{baujat.meta}
\alias{baujat}
\title{Baujat plot to explore heterogeneity in meta-analysis}
\usage{
\method{baujat}{meta}(
  x,
  yscale = 1,
  xlim,
  ylim,
  xlab = "Contribution to overall heterogeneity",
  ylab = "Influence on overall result",
  pch = 21,
  cex = 1,
  col = "black",
  bg = "darkgray",
  studlab = TRUE,
  cex.studlab = 0.8,
  pos.studlab,
  offset = 0.5,
  xmin = 0,
  ymin = 0,
  grid = TRUE,
  col.grid = "lightgray",
  lty.grid = "dotted",
  lwd.grid = par("lwd"),
  pty = "s",
  pooled,
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{yscale}{Scaling factor for values on y-axis.}

\item{xlim}{The x limits (min,max) of the plot.}

\item{ylim}{The y limits (min,max) of the plot.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{pch}{The plotting symbol used for individual studies.}

\item{cex}{The magnification to be used for plotting symbol.}

\item{col}{A vector with colour of plotting symbols.}

\item{bg}{A vector with background colour of plotting symbols (only
used if \code{pch} in \code{21:25}).}

\item{studlab}{A logical indicating whether study labels should be
printed in the graph. A vector with study labels can also be
provided (must be of same length as \code{x$TE} then).}

\item{cex.studlab}{The magnification for study labels.}

\item{pos.studlab}{Position of study labels, see argument
\code{pos} in \code{\link{text}}.}

\item{offset}{Offset for study labels (see \code{\link{text}}).}

\item{xmin}{A numeric specifying minimal value to print study
labels (on x-axis).}

\item{ymin}{A numeric specifying minimal value to print study
labels (on y-axis).}

\item{grid}{A logical indicating whether a grid is printed in the
plot.}

\item{col.grid}{Colour for grid lines.}

\item{lty.grid}{The line type for grid lines.}

\item{lwd.grid}{The line width for grid lines.}

\item{pty}{A character specifying type of plot region (see
\code{\link{par}}).}

\item{pooled}{A character string indicating whether a common effect
or random effects model is used for pooling. Either missing (see
Details), \code{"common"} or \code{"random"}, can be abbreviated.}

\item{\dots}{Graphical arguments as in \code{par} may also be
passed as arguments.}
}
\value{
A data.frame with the following variables:
\item{x}{Coordinate on x-axis (contribution to heterogeneity
  statistic)}
\item{y}{Coordinate on y-axis (influence on overall treatment
  effect)}
}
\description{
Draw a Baujat plot to explore heterogeneity in meta-analysis.
}
\details{
Baujat et al. (2002) introduced a scatter plot to explore
heterogeneity in meta-analysis. On the x-axis the contribution of
each study to the overall heterogeneity statistic (see list object
\code{Q} of the meta-analysis object \code{x}) is plotted. On the
y-axis the standardised difference of the overall treatment effect
with and without each study is plotted; this quantity describes the
influence of each study on the overal treatment effect.

Information from object \code{x} is utilised if argument
\code{pooled} is missing. A common effect model is assumed
(\code{pooled="common"}) if argument \code{x$common} is
\code{TRUE}; a random effects model is assumed
(\code{pooled="random"}) if argument \code{x$random} is
\code{TRUE} and \code{x$common} is \code{FALSE}.

Internally, the \code{\link{metainf}} function is used to calculate
the values on the y-axis.
}
\examples{
data(Olkin1995)

# Only consider first ten studies
ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, sm = "OR", method = "I", studlab = paste(author, year),
  subset = 1:10)

# Generate Baujat plot
baujat(ma1)

\dontrun{
ma2 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, sm = "OR", method = "I", studlab = paste(author, year))

# Do not print study labels if the x-value is smaller than 4 and
# the y-value is smaller than 1
baujat(ma2, yscale = 10, xmin = 4, ymin = 1)

# Change position of study labels
baujat(ma2, yscale = 10, xmin = 4, ymin = 1,
  pos = 1, xlim = c(0, 6.5))

# Generate Baujat plot and assign x- and y- coordinates to R object
# ba2
ba2 <- baujat(ma2)

# Calculate overall heterogeneity statistic
sum(ba2$x)
ma2$Q
}

}
\references{
Baujat B, Mahé C, Pignon JP, Hill C (2002):
A graphical method for exploring heterogeneity in meta-analyses:
Application to a meta-analysis of 65 trials.
\emph{Statistics in Medicine},
\bold{30}, 2641--52
}
\seealso{
\code{\link{metagen}}, \code{\link{metainf}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.mtv.R
\name{read.mtv}
\alias{read.mtv}
\title{Import RevMan 4 data files (.mtv)}
\usage{
read.mtv(file)
}
\arguments{
\item{file}{The name of a file to read data values from.}
}
\value{
A data frame containing the following components:
\item{comp.no}{Comparison number.}
\item{outcome.no}{Outcome number.}
\item{group.no}{Group number.}
\item{studlab}{Study label.}
\item{year}{Year of publication.}
\item{event.e}{Number of events in experimental group.}
\item{n.e}{Number of observations in experimental group.}
\item{event.c}{Number of events in control group.}
\item{n.c}{Number of observations in control group.}
\item{mean.e}{Estimated mean in experimental group.}
\item{sd.e}{Standard deviation in experimental group.}
\item{mean.c}{Estimated mean in control group.}
\item{sd.c}{Standard deviation in control group.}
\item{O.E}{Observed minus expected (IPD analysis).}
\item{V}{Variance of \code{O.E} (IPD analysis).}
\item{order}{Ordering of studies.}
\item{conceal}{Concealment of treatment allocation.}
\item{grplab}{Group label.}
\item{type}{Type of outcome. D = dichotomous, C = continuous, P =
  IPD.}
\item{outclab}{Outcome label.}
\item{graph.exp}{Graph label for experimental group.}
\item{graph.cont}{Graph label for control group.}
\item{label.exp}{Label for experimental group.}
\item{label.cont}{Label for control group.}
\item{complab}{Comparison label.}
}
\description{
Reads a file created with RevMan 4 and creates a data frame from
it.
}
\details{
Reads a file created with RevMan 4 (Menu: "File" - "Export" -
"Analysis data file...") and creates a data frame from it.
}
\examples{
# Locate MTV-data file "FLEISS1993.MTV" in sub-directory of R package
# meta
#
filename <- system.file("extdata/FLEISS1993.MTV", package = "meta")
fleiss1933.cc <- read.mtv(filename)

# Same result as R Command example(Fleiss1993bin):
#
metabin(event.e, n.e, event.c, n.c,
  data = fleiss1933.cc, subset = type == "D",
  studlab = paste(studlab, year))

# Same result: example(Fleiss1993cont)
#
metacont(n.e, mean.e, sd.e, n.c, mean.c, sd.c,
  data = fleiss1933.cc, subset = type == "C",
  studlab = paste(studlab, year))

}
\references{
\emph{Review Manager (RevMan)} [Computer program]. Version 4.2.
Copenhagen: The Nordic Cochrane Centre, The Cochrane Collaboration, 2003
}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metainc.R
\name{metainc}
\alias{metainc}
\title{Meta-analysis of incidence rates}
\usage{
metainc(
  event.e,
  time.e,
  event.c,
  time.c,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  method = if (sm == "IRSD") "Inverse" else "MH",
  sm = gs("sminc"),
  incr = gs("incr"),
  method.incr = gs("method.incr"),
  incr.e = if (length(incr) > 1) incr else NULL,
  incr.c = if (length(incr) > 1) incr else NULL,
  model.glmm = "UM.FS",
  level = gs("level"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = ifelse(!is.na(charmatch(tolower(method), "glmm", nomatch = NA)), "ML",
    gs("method.tau")),
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  method.bias = gs("method.bias"),
  n.e = NULL,
  n.c = NULL,
  backtransf = if (sm == "IRSD") FALSE else gs("backtransf"),
  irscale = 1,
  irunit = "person-years",
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.e = gs("label.e"),
  label.c = gs("label.c"),
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  hakn,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{event.e}{Number of events in experimental group or an R object
created with \code{\link{pairwise}}.}

\item{time.e}{Person time at risk in experimental group.}

\item{event.c}{Number of events in control group.}

\item{time.c}{Person time at risk in control group.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information, i.e., event.e, time.e, event.c, and time.c.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{method}{A character string indicating which method is to be
used for pooling of studies. One of \code{"MH"},
\code{"Inverse"}, \code{"Cochran"}, or \code{"GLMM"} can be
abbreviated.}

\item{sm}{A character string indicating which summary measure
(\code{"IRR"}, \code{"IRD"}, \code{"IRSD"}, or \code{"VE"}) is to
be used for pooling of studies, see Details.}

\item{incr}{A numerical value which is added to cell frequencies
for studies with a zero cell count or a numeric vector with the continuity
correction for each study, see Details.}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, \code{"all"}, or \code{"user"}), see Details.}

\item{incr.e}{Continuity correction in experimental group, see Details.}

\item{incr.c}{Continuity correction in control group, see Details.}

\item{model.glmm}{A character string indicating which GLMM should
be used. One of \code{"UM.FS"}, \code{"UM.RS"}, and
\code{"CM.EL"}, see Details.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, or
\code{"Thompson"}, can be abbreviated. See function
\code{\link{metabias}}.}

\item{n.e}{Number of observations in experimental group (optional).}

\item{n.c}{Number of observations in control group (optional).}

\item{backtransf}{A logical indicating whether results for
incidence rate ratio (\code{sm = "IRR"}) and vaccine efficacy or
vaccine effectiveness (\code{sm = "VE"}) should be back
transformed in printouts and plots. If TRUE (default), results
will be presented as incidence rate ratios or vaccine efficacy /
effectiveness; otherwise log incidence rate ratios or log vaccine
rate ratios will be shown.}

\item{irscale}{A numeric defining a scaling factor for printing of
incidence rate differences.}

\item{irunit}{A character string specifying the time unit used to
calculate rates, e.g. person-years.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.e}{Label for experimental group.}

\item{label.c}{Label for control group.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{hakn}{Deprecated argument (replaced by 'method.random.ci').}

\item{adhoc.hakn}{Deprecated argument (replaced by
'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if \code{incr} is added to studies with zero cell
frequencies or if estimation problems exist in fitting a GLMM).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}} or
\code{\link[metafor]{rma.glmm}}, respectively.}

\item{\dots}{Additional arguments passed on to
\code{\link[metafor]{rma.glmm}} function and to catch deprecated
arguments.}
}
\value{
An object of class \code{c("metainc", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of common effect and random effects estimates (incidence
rate ratio or incidence rate difference) for meta-analyses with
event counts.  Mantel-Haenszel, Cochran, inverse variance method,
and generalised linear mixed model (GLMM) are available for
pooling. For GLMMs, the \code{\link[metafor]{rma.glmm}} function
from R package \bold{metafor} (Viechtbauer 2010) is called
internally.
}
\details{
Calculation of common and random effects estimates for meta-analyses
comparing two incidence rates.

The following measures of treatment effect are available:
\itemize{
\item Incidence Rate Ratio (\code{sm = "IRR"})
\item Incidence Rate Difference (\code{sm = "IRD"})
\item Square root transformed Incidence Rate Difference (\code{sm =
  "IRSD"})
\item Vaccine efficacy or vaccine effectiveness (\code{sm = "VE"})
}

Note, log incidence rate ratio (logIRR) and log vaccine ratio
(logVR) are mathematical identical, however, back-transformed
results differ as vaccine efficacy or effectiveness is defined as
\code{VE = 100 * (1 - IRR)}.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

\subsection{Meta-analysis method}{

By default, both common effect and random effects models are
considered (see arguments \code{common} and \code{random}). If
\code{method} is \code{"MH"} (default), the Mantel-Haenszel method
is used to calculate the common effect estimate (Greenland &
Robbins, 1985); if \code{method} is \code{"Inverse"}, inverse
variance weighting is used for pooling; if \code{method} is
\code{"Cochran"}, the Cochran method is used for pooling
(Bayne-Jones, 1964, Chapter 8). For these three methods, the random
effects estimate is always based on the inverse variance method.

A distinctive and frequently overlooked advantage of incidence
rates is that individual patient data (IPD) can be extracted from
count data. Accordingly, statistical methods for IPD, i.e.,
generalised linear mixed models, can be utilised in a meta-analysis
of incidence rate ratios (Stijnen et al., 2010). These methods are
available (argument \code{method = "GLMM"}) for the common effect
and random effects model by calling the
\code{\link[metafor]{rma.glmm}} function from R package
\bold{metafor} internally.

Three different GLMMs are available for meta-analysis of incidence
rate ratios using argument \code{model.glmm} (which corresponds to
argument \code{model} in the \code{\link[metafor]{rma.glmm}}
function):
\tabular{cl}{
1. \tab Poisson regression model with fixed study effects (default)
 \cr
 \tab (\code{model.glmm = "UM.FS"}, i.e., \bold{U}nconditional
 \bold{M}odel - \bold{F}ixed \bold{S}tudy effects) \cr
2. \tab Mixed-effects Poisson regression model with random study
 effects \cr
 \tab (\code{model.glmm = "UM.RS"}, i.e., \bold{U}nconditional
 \bold{M}odel - \bold{R}andom \bold{S}tudy effects) \cr
3. \tab Generalised linear mixed model (conditional Poisson-Normal)
 \cr
 \tab (\code{model.glmm = "CM.EL"}, i.e., \bold{C}onditional
  \bold{M}odel - \bold{E}xact \bold{L}ikelihood)
}

Details on these three GLMMs as well as additional arguments which
can be provided using argument '\code{\dots}' in \code{metainc}
are described in \code{\link[metafor]{rma.glmm}} where you can also
find information on the iterative algorithms used for estimation.
Note, regardless of which value is used for argument
\code{model.glmm}, results for two different GLMMs are calculated:
common effect model (with fixed treatment effect) and random effects
model (with random treatment effects).
}

\subsection{Continuity correction}{

Four approaches are available to apply a continuity correction:
\itemize{
\item Only studies with a zero cell count (\code{method.incr =
  "only0", default})
\item All studies if at least one study has a zero cell count
  (\code{method.incr = "if0all"})
\item All studies irrespective of zero cell counts
  (\code{method.incr = "all"})
\item Use values provided in arguments \code{incr.e} and \code{incr.c}
  (\code{method.incr = "user"})
}

For studies with a zero cell count, by default, 0.5 is added to all
cell frequencies of these studies (argument \code{incr}). This
continuity correction is used both to calculate individual study
results with confidence limits and to conduct meta-analysis based
on the inverse variance method. For Mantel-Haenszel method, Cochran
method, and GLMMs, nothing is added to zero cell counts.
Accordingly, estimates for these methods are not defined if the
number of events is zero in all studies either in the experimental
or control group.
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments
\code{common} and \code{random}. Accordingly, the estimate
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"meta"} even if
argument \code{random = FALSE}. However, all functions in R
package \bold{meta} will adequately consider the values for
\code{common} and \code{random}. E.g. function
\code{\link{print.meta}} will not print results for the random
effects model if \code{random = FALSE}.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\examples{
data(smoking)
ma1 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
  data = smoking, studlab = study)
print(ma1, digits = 2)

ma2 <- update(ma1, method = "Cochran")
print(ma2, digits = 2)

data(lungcancer)
ma3 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
  data = lungcancer, studlab = study)
print(ma3, digits = 2)

# Redo Cochran meta-analysis with inflated standard errors
#
# All cause mortality
#
TEa <- log((smoking$d.smokers/smoking$py.smokers) /
  (smoking$d.nonsmokers/smoking$py.nonsmokers))
seTEa <- sqrt(1 / smoking$d.smokers + 1 / smoking$d.nonsmokers +
  2.5 / smoking$d.nonsmokers)
metagen(TEa, seTEa, sm = "IRR", studlab = smoking$study)

# Lung cancer mortality
#
TEl <- log((lungcancer$d.smokers/lungcancer$py.smokers) /
  (lungcancer$d.nonsmokers/lungcancer$py.nonsmokers))
seTEl <- sqrt(1 / lungcancer$d.smokers + 1 / lungcancer$d.nonsmokers +
  2.25 / lungcancer$d.nonsmokers)
metagen(TEl, seTEl, sm = "IRR", studlab = lungcancer$study)

\dontrun{
# Meta-analysis using generalised linear mixed models
# (only if R packages 'metafor' and 'lme4' are available)

# Poisson regression model (fixed study effects)
#
ma4 <- metainc(d.smokers, py.smokers, d.nonsmokers, py.nonsmokers,
  data = smoking, studlab = study, method = "GLMM")
ma4

# Mixed-effects Poisson regression model (random study effects)
#
update(ma4, model.glmm = "UM.RS", nAGQ = 1)
#
# Generalised linear mixed model (conditional Poisson-Normal)
#
update(ma4, model.glmm = "CM.EL")
}

}
\references{
Bayne-Jones S et al. (1964):
Smoking and Health: Report of the Advisory Committee to the Surgeon
General of the United States.
U-23 Department of Health, Education, and Welfare.
Public Health Service Publication No. 1103.

Greenland S & Robins JM (1985):
Estimation of a common effect parameter from sparse follow-up data.
\emph{Biometrics},
\bold{41}, 55--68

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Viechtbauer W (2010):
Conducting Meta-Analyses in R with the Metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48
}
\seealso{
\code{\link{meta-package}}, \code{\link{metabin}},
  \code{\link{update.meta}}, \code{\link{print.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabind.R
\name{metabind}
\alias{metabind}
\title{Combine and summarize meta-analysis objects}
\usage{
metabind(
  ...,
  subgroup = NULL,
  name = NULL,
  common = NULL,
  random = NULL,
  prediction = NULL,
  backtransf = NULL,
  outclab = NULL,
  pooled = NULL,
  warn.deprecated = gs("warn.deprecated")
)
}
\arguments{
\item{...}{Any number of meta-analysis objects or a single list
with meta-analyses.}

\item{subgroup}{An optional variable to generate a forest plot with
subgroups.}

\item{name}{An optional character vector providing descriptive
names for the meta-analysis objects.}

\item{common}{A logical vector indicating whether results of common
effect model should be considered.}

\item{random}{A logical vector indicating whether results of random
effects model should be considered.}

\item{prediction}{A logical vector indicating whether results of
prediction intervals should be considered.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If
\code{backtransf=TRUE} (default), results for \code{sm="OR"} are
printed as odds ratios rather than log odds ratios, for example.}

\item{outclab}{Outcome label for all meta-analyis objects.}

\item{pooled}{Deprecated argument (replaced by \code{common} and
\code{random}.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}
}
\value{
An object of class \code{c("metabind", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
This function can be used to combine meta-analysis objects and is,
for example, useful to summarize results of various meta-analysis
methods or to generate a forest plot with results of several
subgroup analyses.
}
\details{
This function can be used to combine any number of meta-analysis
objects which is useful, for example, to summarize results of
various meta-analysis methods or to generate a forest plot with
results of several subgroup analyses (see Examples).

Individual study results are not retained with \code{metabind} as
the function allows to combine meta-analyses from different data
sets (e.g., with randomised or observational studies). Individual
study results are retained with R function \code{\link{metamerge}}
which can be used to combine results of meta-analyses of the
same dataset.
}
\examples{
data(Fleiss1993cont)

# Add some (fictitious) grouping variables:
#
Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")

ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")

# Conduct two subgroup analyses
#
ma1 <- update(ma, subgroup = age, subgroup.name = "Age group")
ma2 <- update(ma, subgroup = region, subgroup.name = "Region")

# Combine random effects subgroup meta-analyses and show forest
# plot with subgroup results
#
mb <- metabind(ma1, ma2, common = FALSE)
mb
forest(mb)

# Use various estimation methods for between-study heterogeneity
# variance
#
ma.pm <- update(ma, method.tau = "PM")
ma.dl <- update(ma, method.tau = "DL")
ma.ml <- update(ma, method.tau = "ML")
ma.hs <- update(ma, method.tau = "HS")
ma.sj <- update(ma, method.tau = "SJ")
ma.he <- update(ma, method.tau = "HE")
ma.eb <- update(ma, method.tau = "EB")

# Combine meta-analyses and show results
#
taus <- c("Restricted maximum-likelihood estimator",
  "Paule-Mandel estimator",
  "DerSimonian-Laird estimator",
  "Maximum-likelihood estimator",
  "Hunter-Schmidt estimator",
  "Sidik-Jonkman estimator",
  "Hedges estimator",
  "Empirical Bayes estimator")
#
ma.taus <- metabind(ma, ma.pm, ma.dl, ma.ml, ma.hs, ma.sj, ma.he, ma.eb,
  name = taus, common = FALSE)
ma.taus
forest(ma.taus)

}
\seealso{
\code{\link{metagen}}, \code{\link{forest.metabind}},
  \code{\link{metamerge}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cidprop.R
\name{cidprop.meta}
\alias{cidprop.meta}
\alias{cidprop}
\alias{print.cidprop}
\title{Calculate expected proportion of comparable studies with clinically important
benefit or harm}
\usage{
\method{cidprop}{meta}(
  x,
  cid = NULL,
  cid.below.null = NULL,
  cid.above.null = NULL,
  label.cid = "",
  label.cid.below.null = NULL,
  label.cid.above.null = NULL,
  small.values = "desirable",
  ...
)

cidprop(x, ...)

\method{print}{cidprop}(
  x,
  digits.cid = gs("digits.cid"),
  digits.percent = 1,
  big.mark = gs("big.mark"),
  details.methods = gs("details"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{cid}{A numeric value or vector specifying clinically important
differences (CID) / decision thresholds used to calculate expected
proportions of clinically important benefit or harm (see Details).}

\item{cid.below.null}{A numeric value or vector specifying CID limits below
the null effect (see Details).}

\item{cid.above.null}{A numeric value or vector specifying CID limits above
the null effect (see Details).}

\item{label.cid}{A character string or vector specifying labels for
clinically important differences. Must be of same length as argument
\code{cid}.}

\item{label.cid.below.null}{A character string or vector specifying labels
for clinically important differences below the null effect. Must be of
same length as argument \code{cid.below.null} (or \code{cid}).}

\item{label.cid.above.null}{A character string or vector specifying labels
for clinically important differences above the null effect. Must be of
same length as argument \code{cid.above.null} (or \code{cid}).}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}), can be abbreviated.}

\item{\dots}{Additional arguments (ignored)}

\item{digits.cid}{Minimal number of significant digits for
CIDs / decision thresholds, see \code{print.default}.}

\item{digits.percent}{Minimal number of significant digits for
expected proportions, printed as percentages, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}

\item{details.methods}{A logical specifying whether details on
statistical methods should be printed.}
}
\value{
A list with elements
\item{prop.cid.below.null}{Expected proportion of comparable studies below
  lower CID(s)}
\item{prop.cid.above.null}{Expected proportion of comparable studies above
  upper CID(s)}
\item{prop.within.cid}{Expected proportion of comparable studies between
  lower and upper CID(s)}
\item{cid, cid.below.null, cid.above.null, small.values, x}{As defined
  above}
\item{label.cid, label.cid.below.null, label.cid.above.null}{As defined
  above}
}
\description{
Calculate expected proportion of comparable studies with clinically important
benefit or harm which is derived from the prediction interval.
}
\details{
Expected proportions of comparable studies with clinically important
benefit or harm are derived from the prediction interval in
the meta-analysis (Siemens et al., 2025).

Clinically important benefit or harm can be defined using either argument
\code{cid} or \code{cid.below.null} and \code{cid.above.null}.
Input for the later arguments will be ignored if argument \code{cid} was
specified. In this case, the values of \code{cid.below.null} and
\code{cid.above.null} will be equal to
\itemize{
\item \code{cid} and \code{1 / cid} for ratio measures,
\item \code{cid} and \code{-cid} for difference measures.
}

By default, i.e., if no input is provided for argument \code{cid}, the
null effect is used as threshold.

Thresholds based on argument \code{cid} will always be symmetric. Asymmetric
thresholds can be defined using arguments \code{cid.below.null} and
\code{cid.above.null}.
}
\examples{
oldset <- settings.meta(digits.cid = 0)

m <- metagen(1:10 - 3, 1:10, sm = "MD")
#
pp1 <- cidprop(m, cid = 2)
pp1
#
pp2 <- cidprop(m, cid.below = 0.5, cid.above = 2)
pp2
#
pp3 <- cidprop(m, cid.below = 0.5, cid.above = 2, small.values = "u")
pp3

pp4 <- cidprop(m, cid = 1:2, label.cid = c("moderate", "large"))
pp4
#
pp5 <- cidprop(m, cid.below = -1.5, cid.above = 1:2,
  label.cid.below = "large", label.cid.above = c("moderate", "large"))
pp5

settings.meta(oldset)

}
\references{
Siemens W, Borenstein M, Evrenoglou T, Meerpohl JJ, Schwarzer G (2025):
Beyond prediction intervals in meta-analysis: reporting the expected
proportion of comparable studies with clinically relevant benefit or harm.
\emph{BMC Medical Research Methodology},
\bold{25}, 275
}
\seealso{
\code{\link{plot.cidprop}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Olkin1995.R
\docType{data}
\name{Olkin1995}
\alias{Olkin1995}
\alias{Olkin95}
\title{Thrombolytic Therapy after Acute Myocardial Infarction}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{author}}\tab first author \cr
\bold{\emph{year}}\tab year of publication \cr
\bold{\emph{ev.exp}}\tab number of events in experimental group
  \cr
\bold{\emph{n.exp}}\tab number of observations in experimental group
  \cr
\bold{\emph{ev.cont}}\tab number of events in control group \cr
\bold{\emph{n.cont}}\tab number of observations in control group
}
}
\source{
Olkin I (1995):
Statistical and theoretical considerations in meta-analysis.
\emph{Journal of Clinical Epidemiology},
\bold{48}, 133--46
}
\description{
Meta-analysis on Thrombolytic Therapy after Acute Myocardial Infarction
}
\examples{
data(Olkin1995)
metabin(ev.exp, n.exp, ev.cont, n.cont, data = Olkin1995)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bracket.longarm.R
\name{[.longarm}
\alias{[.longarm}
\alias{[,longarm}
\title{Extract parts of longarm object}
\usage{
\method{[}{longarm}(x, ...)
}
\arguments{
\item{x}{An object of class \code{\link{longarm}}.}

\item{\dots}{Additional arguments (passed on to [.data.frame).}
}
\description{
Auxiliary function to extract parts of \code{\link{longarm}} object.
}
\examples{
# Transform data from wide arm-based format to contrast-based format
pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
  n = list(n1, n2, n3),
  mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
  data = Franchini2012, studlab = Study)

# Transform data from contrast-based to long arm-based format
# and only keep the main variables
la1 <- longarm(pw1, append = FALSE)
head(la1)

la1[la1$studlab == "Lieberman 1998", ]

}
\seealso{
\code{\link{longarm}}, \code{\link{subset.longarm}},
  \code{\link[metabook]{Franchini2012}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JAMAlabels.R
\name{JAMAlabels}
\alias{JAMAlabels}
\title{Create study labels in JAMA layout (deprecated function)}
\usage{
JAMAlabels(author, year, citation, data = NULL)
}
\arguments{
\item{author}{A vector providing study authors.}

\item{year}{A vector providing year of publication.}

\item{citation}{A vector providing citation numbers.}

\item{data}{An optional data frame containing the study
information.}
}
\description{
Deprecated function to create study labels in JAMA layout (for
forest plot). Replaced by \code{\link{labels.meta}}.
}
\details{
This auxiliary function can be used to create study labels in JAMA
layout which can be added to a forest plot using argument
'studlab'.
}
\examples{
data(Fleiss1993bin)

refs <- 20 + 1:7

Fleiss1993bin$mylabs <-
  JAMAlabels(study, year, refs, data = Fleiss1993bin)

m <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
  studlab = paste(study, year),
  sm = "OR", random = FALSE)

forest(m, studlab = mylabs, layout = "JAMA",
  fontfamily = "Times", fontsize = 10)

}
\seealso{
\code{\link{labels.meta}}, \code{\link{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metareg.R
\name{metareg.meta}
\alias{metareg.meta}
\alias{metareg}
\alias{metareg.default}
\title{Meta-regression}
\usage{
\method{metareg}{meta}(
  x,
  formula,
  method.tau = x$method.tau,
  hakn = x$method.random.ci == "HK",
  level.ma = x$level.ma,
  intercept = TRUE,
  ...
)

metareg(x, ...)

\method{metareg}{default}(x, ...)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{formula}{Either a character string or a formula object.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance tau-squared. Either
\code{"FE"}, \code{"DL"}, \code{"REML"}, \code{"ML"},
\code{"HS"}, \code{"SJ"}, \code{"HE"}, or \code{"EB"}, can be
abbreviated.}

\item{hakn}{A logical indicating whether the method by Hartung and
Knapp should be used to adjust test statistics and confidence
intervals.}

\item{level.ma}{The level used to calculate confidence intervals
for parameter estimates in the meta-regression model.}

\item{intercept}{A logical indicating whether an intercept should
be included in the meta-regression model.}

\item{\dots}{Additional arguments passed to R function
\code{\link[metafor]{rma.uni}}.}
}
\value{
An object of class \code{c("metareg", "rma.uni", "rma")}. Please
look at the help page of R function \code{\link[metafor]{rma.uni}}
for more details on the output from this function.

In addition, a list \code{.meta} is added to the output containing
the following components:
\item{x, formula, method.tau, hakn, level.ma, intercept}{As
  defined above.}
\item{dots}{Information provided in argument '\dots{}'.}
\item{call}{Function call.}
\item{version}{Version of R package \bold{meta} used to create
  object.}
\item{version.metafor}{Version of R package \bold{metafor} used to
  create object.}
}
\description{
Meta-regression for objects of class \code{meta}. This is a wrapper
function for the R function \code{\link[metafor]{rma.uni}} in the R
package \bold{metafor} (Viechtbauer 2010).
}
\details{
This R function is a wrapper function for R function
\code{\link[metafor]{rma.uni}} in the R package \bold{metafor}
(Viechtbauer 2010).

Note, results are not back-transformed in printouts of
meta-analyses using summary measures with transformations, e.g.,
log risk ratios are printed instead of the risk ratio if argument
\code{sm = "RR"} and logit transformed proportions are printed if
argument \code{sm = "PLOGIT"}.

Argument '\dots{}' can be used to pass additional arguments to R
function \code{\link[metafor]{rma.uni}}. For example, argument
\code{control} to provide a list of control values for the
iterative estimation algorithm. See help page of R function
\code{\link[metafor]{rma.uni}} for more details.
}
\examples{
data(Fleiss1993cont)
# Add some (fictitious) grouping variables:
Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")

ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")
\dontrun{
# Warning as no information on covariate is available
#
metareg(ma1)
}

# Do meta-regression for covariate region
#
ma2 <- update(ma1, subgroup = region, tau.common = TRUE, common = FALSE)
metareg(ma2)

# Same result for
# - tau-squared
# - test of heterogeneity
# - test for subgroup differences
# (as argument 'tau.common' was used to create ma2)
#
ma2
metareg(ma2, intercept = FALSE)
metareg(ma1, region)

# Different result for
# - tau-squared
# - test of heterogeneity
# - test for subgroup differences
# (as argument 'tau.common' is - by default - FALSE)
#
ma3 <- update(ma1, subgroup = region)
ma3

# Generate bubble plot
#
bubble(metareg(ma2))

# Do meta-regression with two covariates
#
metareg(ma3, region + age)

# Do same meta-regressions using formula notation
#
metareg(ma1, ~ region)
metareg(ma3, ~ region + age)

# Print intermediate results for iterative estimation algorithm;
# furthermore print results with three digits.
#
metareg(ma3, region, control = list(verbose = TRUE), digits = 3)

}
\references{
Viechtbauer W (2010):
Conducting Meta-Analyses in R with the Metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48
}
\seealso{
\code{\link{bubble}}, \code{\link{summary.meta}},
  \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{models}
\keyword{regression}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ci.R
\name{ci}
\alias{ci}
\title{Calculation of confidence intervals (based on normal approximation
or t-distribution)}
\usage{
ci(TE, seTE, level = 0.95, df = NULL, null.effect = 0)
}
\arguments{
\item{TE}{Estimated treatment effect.}

\item{seTE}{Standard error of treatment estimate.}

\item{level}{The confidence level required.}

\item{df}{Degrees of freedom (for confidence intervals based on
t-distribution).}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}
}
\value{
List with components
\item{TE}{Estimated treatment effect}
\item{seTE}{Standard error of treatment estimate}
\item{lower}{Lower confidence limits}
\item{upper}{Upper confidence limits}
\item{statistic}{Test statistic (either z-score or t-score)}
\item{p}{P-value of text.with null hypothesis \code{TE=0}}
\item{level}{The confidence level required}
\item{df}{Degrees of freedom (t-distribution)}
}
\description{
Calculation of confidence intervals; based on normal approximation
or t-distribution.
}
\note{
This function is primarily called from other functions of the
library \code{meta}, e.g. \code{forest.meta}, \code{summary.meta}.
}
\examples{
data.frame(ci(170, 10))
data.frame(ci(170, 10, 0.99))
data.frame(ci(1.959964, 1))
data.frame(ci(2.2621571628, 1, df = 9))

}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/woodyplants.R
\docType{data}
\name{woodyplants}
\alias{woodyplants}
\title{Elevated CO_2 and total biomass of woody plants}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{obsno}}\tab observation number \cr
\bold{\emph{papno}}\tab database paper number \cr
\bold{\emph{treat}}\tab treatment code \cr
\bold{\emph{level}}\tab treatment level \cr
\bold{\emph{n.elev}}\tab number of observations in experimental
  group (elevated CO_2-level) \cr
\bold{\emph{mean.elev}}\tab estimated mean in experimental group
  \cr
\bold{\emph{sd.elev}}\tab standard deviation in experimental group
  \cr
\bold{\emph{n.amb}}\tab number of observations in control group
  (ambient CO_2-level) \cr
\bold{\emph{mean.amb}}\tab estimated mean in control group \cr
\bold{\emph{sd.amb}}\tab standard deviation in control group
}
}
\description{
Meta-analysis on effects of elevated CO_2 on total biomass of woody
plants

This dataset has been used as an example in Hedges et al. (1999) to
describe methods for the meta-analysis of response ratios. The
complete dataset with 102 observations and 26 variables is
available online as a supplement. Here only a subset of 10
variables is provided and used in the examples.
}
\examples{
data(woodyplants)

# Meta-analysis of response ratios (Hedges et al., 1999)
#
ma1 <- metacont(n.elev, mean.elev, sd.elev, n.amb, mean.amb, sd.amb,
  data = woodyplants, sm = "ROM", studlab = paste(obsno, papno, sep = " / "))
print(ma1, prediction = TRUE)

# Meta-analysis for plants grown with low soil fertility treatment
#
ma2 <- update(ma1, subset = (treat == "fert" & level == "low"))
print(ma2, prediction = TRUE)

# Meta-analysis for plants grown under low light conditions
#
ma3 <- update(ma1, subset = (treat == "light" & level == "low"))
print(ma3, prediction = TRUE)
}
\references{
Hedges LV, Gurevitch J, Curtis PS (1999):
The meta-analysis of response ratios in experimental ecology.
\emph{Ecology},
\bold{80}, 1150--6
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/labbe.R
\name{labbe.metabin}
\alias{labbe.metabin}
\alias{labbe}
\alias{labbe.default}
\title{L'Abbé plot for meta-analysis with binary outcomes}
\usage{
\method{labbe}{metabin}(
  x,
  xlim,
  ylim,
  xlab = NULL,
  ylab = NULL,
  TE.common = x$TE.common,
  TE.random = x$TE.random,
  common = x$common,
  random = x$random,
  backtransf = x$backtransf,
  axes = TRUE,
  pch = 21,
  text = NULL,
  cex = 1,
  col = "black",
  bg = "lightgray",
  lwd = 1,
  lwd.common = lwd,
  lwd.random = lwd,
  lty.common = 2,
  lty.random = 9,
  col.common = col,
  col.random = col,
  nulleffect = TRUE,
  lwd.nulleffect = lwd,
  col.nulleffect = "lightgray",
  sm = x$sm,
  weight,
  studlab = FALSE,
  cex.studlab = 0.8,
  pos.studlab = 2,
  label.e = x$label.e,
  label.c = x$label.c,
  warn.deprecated = gs("warn.deprecated"),
  TE.fixed,
  fixed,
  lwd.fixed,
  lty.fixed,
  col.fixed,
  ...
)

\method{labbe}{default}(
  x,
  y,
  xlim,
  ylim,
  xlab = NULL,
  ylab = NULL,
  TE.common = NULL,
  TE.random = NULL,
  common = !is.null(TE.common),
  random = !is.null(TE.random),
  backtransf = TRUE,
  axes = TRUE,
  pch = 21,
  text = NULL,
  cex = 1,
  col = "black",
  bg = "lightgray",
  lwd = 1,
  lwd.common = lwd,
  lwd.random = lwd,
  lty.common = 2,
  lty.random = 9,
  col.common = col,
  col.random = col,
  nulleffect = TRUE,
  lwd.nulleffect = lwd,
  col.nulleffect = "lightgray",
  sm = "",
  weight,
  studlab = FALSE,
  cex.studlab = 0.8,
  pos.studlab = 2,
  label.e = NULL,
  label.c = NULL,
  warn.deprecated = gs("warn.deprecated"),
  TE.fixed,
  fixed,
  lwd.fixed,
  lty.fixed,
  col.fixed,
  ...
)
}
\arguments{
\item{x}{An object of class \code{metabin}. Alternatively, the x
coordinates of points of the L'Abbé plot.}

\item{xlim}{The x limits (min, max) of the plot.}

\item{ylim}{The y limits (min, max) of the plot.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{TE.common}{A numeric or vector specifying combined common
effect estimate(s).}

\item{TE.random}{A numeric or vector specifying combined random
effects estimate(s).}

\item{common}{A logical indicating whether the common effect
estimate should be plotted.}

\item{random}{A logical indicating whether the random effects
estimate should be plotted.}

\item{backtransf}{A logical indicating which values should be
printed on x- and y-axis (see Details).}

\item{axes}{A logical indicating whether axes should be drawn on
the plot.}

\item{pch}{The plotting symbol used for individual studies.}

\item{text}{A character vector specifying the text to be used
instead of plotting symbol.}

\item{cex}{The magnification to be used for plotting symbol.}

\item{col}{A vector with colour of plotting symbols.}

\item{bg}{A vector with background colour of plotting symbols (only
used if \code{pch} in \code{21:25}).}

\item{lwd}{The line width.}

\item{lwd.common}{The line width(s) for common effect estimate(s)
(if \code{common} is not \code{NULL} or \code{FALSE}).}

\item{lwd.random}{The line width(s) for random effects estimate(s)
(if \code{random} is not \code{NULL} or \code{FALSE}).}

\item{lty.common}{Line type(s) for common effect estimate(s).}

\item{lty.random}{Line type(s) for random effects estimate(s).}

\item{col.common}{Colour of line(s) for common effect estimate(s).}

\item{col.random}{Colour of line(s) for random effects estimate(s).}

\item{nulleffect}{A logical indicating whether line for null effect
should be added to the plot..}

\item{lwd.nulleffect}{Width of line for null effect.}

\item{col.nulleffect}{Colour of line for null effect.}

\item{sm}{A character string indicating underlying summary measure,
i.e., \code{"RD"}, \code{"RR"}, \code{"OR"}, or \code{"ASD"}.}

\item{weight}{Either a numeric vector specifying relative sizes of
plotting symbols or a character string indicating which type of
plotting symbols is to be used for individual treatment
estimates. One of missing (see Details), \code{"same"},
\code{"common"}, or \code{"random"}, can be abbreviated. Plot
symbols have the same size for all studies or represent study
weights from common effect or random effects model.}

\item{studlab}{A logical indicating whether study labels should be
printed in the graph. A vector with study labels can also be
provided (must be of same length as \code{x$event.e} then).}

\item{cex.studlab}{Size of study labels.}

\item{pos.studlab}{Position of study labels, see argument
\code{pos} in \code{\link{text}}.}

\item{label.e}{Label for experimental group.}

\item{label.c}{Label for control group.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{TE.fixed}{Deprecated argument (replaced by 'TE.common').}

\item{fixed}{Deprecated argument (replaced by 'common').}

\item{lwd.fixed}{Deprecated argument (replaced by 'lwd.common').}

\item{lty.fixed}{Deprecated argument (replaced by 'lty.common').}

\item{col.fixed}{Deprecated argument (replaced by 'col.common').}

\item{\dots}{Graphical arguments as in \code{par} may also be
passed as arguments.}

\item{y}{The y coordinates of the L'Abbé plot, if argument \code{x}
is not an object of class \code{metabin}.}
}
\description{
Draw a L'Abbé plot for meta-analysis with binary outcomes.
}
\details{
A L'Abbé plot is a scatter plot with the risk in the control group
on the x-axis and the risk in the experimental group on the y-axis
(L'Abbé et al., 1987). It can be used to evaluate heterogeneity in
meta-analysis.  Furthermore, this plot can aid to choose a summary
measure (odds ratio, risk ratio, risk difference) that will result
in more consistent results (Jiménez et al., 1997; Deeks, 2002).

If argument \code{backtransf} is TRUE (default), event
probabilities will be printed on x- and y-axis. Otherwise,
transformed event probabilities will be printed as defined by the
summary measure, i.e., log odds of probabilities for odds ratio as
summary measure (\code{sm = "OR"}), log probabilities for \code{sm
= "RR"}, and arcsine-transformed probabilities for \code{sm =
"ASD"}.

If \code{common} is TRUE, the estimate of the common effct model is
plotted as a line. If \code{random} is TRUE, the estimate of the
random effects model is plotted as a line.

Information from object \code{x} is utilised if argument
\code{weight} is missing. Weights from the common effect model are
used (\code{weight = "common"}) if argument \code{x$common} is
\code{TRUE}; weights from the random effects model are used
(\code{weight = "random"}) if argument \code{x$random} is
\code{TRUE} and \code{x$common} is \code{FALSE}.
}
\examples{
data(Olkin1995)
ma <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, studlab = paste(author, year),
  sm = "RR", method = "I")

# L'Abbe plot for risk ratio
#
labbe(ma)

# L'Abbe plot for odds ratio
#
labbe(ma, sm = "OR")
# same plot
labbe(update(ma, sm = "OR"))

# L'Abbe plot for risk difference
#
labbe(ma, sm = "RD")

# L'Abbe plot on log odds scale
#
labbe(ma, sm = "OR", backtransf = FALSE)

# L'Abbe plot for odds ratio with coloured lines for various
# treatment effects (defined as log odds ratios)
#
mycols <- c("blue", "yellow", "green", "red", "green", "yellow", "blue")
labbe(ma, sm = "OR", random = FALSE,
  TE.common = log(c(1 / 10, 1 / 5, 1 / 2, 1, 2, 5, 10)),
  col.common = mycols, lwd.common = 2)

# L'Abbe plot on log odds scale with coloured lines for various
# treatment effects (defined as log odds ratios)
#
labbe(ma, sm = "OR", random = FALSE, backtransf = FALSE,
  TE.common = log(c(1 / 10, 1 / 5, 1 / 2, 1, 2, 5, 10)),
  col.common = mycols, lwd.common = 2)

}
\references{
Deeks JJ (2002):
Issues in the selection of a summary statistic for meta-analysis of
clinical trials with binary outcomes.
\emph{Statistics in Medicine},
\bold{21}, 1575--600

Jiménez FJ, Guallar E, Martín-Moreno JM (1997):
A graphical display useful for meta-analysis.
\emph{European Journal of Public Health},
\bold{1}, 101--5

L'Abbé KA, Detsky AS, O'Rourke K (1987):
Meta-analysis in clinical research.
\emph{Annals of Internal Medicine},
\bold{107}, 224--33
}
\seealso{
\code{\link{metabin}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metagen.R
\name{metagen}
\alias{metagen}
\title{Generic inverse variance meta-analysis}
\usage{
metagen(
  TE,
  seTE,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  cycles = NULL,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  sm = "",
  method.ci = if (missing(df)) "z" else "t",
  level = gs("level"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = gs("method.tau"),
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  null.effect = 0,
  method.bias = gs("method.bias"),
  n.e = NULL,
  n.c = NULL,
  pval,
  df,
  lower,
  upper,
  level.ci = 0.95,
  median,
  q1,
  q3,
  min,
  max,
  method.mean = "Luo",
  method.sd = "Shi",
  approx.TE,
  approx.seTE,
  transf = gs("transf") & missing(func.transf),
  backtransf = gs("backtransf") | !missing(func.backtransf),
  func.transf,
  func.backtransf,
  args.transf,
  args.backtransf,
  pscale = 1,
  irscale = 1,
  irunit = "person-years",
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.e = gs("label.e"),
  label.c = gs("label.c"),
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  id,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  keeprma = gs("keeprma"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{TE}{Estimate of treatment effect, e.g., log hazard ratio or
risk difference or an R object created with \code{\link{pairwise}}.}

\item{seTE}{Standard error of treatment estimate or standard deviation of
n-of-1 trials.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information.}

\item{subset}{An optional vector specifying a subset of studies to
be used (see Details).}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots (see Details).}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{cycles}{A numeric vector with the number of cycles per patient / study
in n-of-1 trials.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{sm}{A character string indicating underlying summary measure,
e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
\code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.}

\item{method.ci}{A character string indicating which method is used
to calculate confidence intervals for individual studies, see
Details.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, or
\code{"Thompson"}, can be abbreviated. See function
\code{\link{metabias}}.}

\item{n.e}{Number of observations in experimental group (or total
sample size in study).}

\item{n.c}{Number of observations in control group.}

\item{pval}{P-value (used to estimate the standard error).}

\item{df}{Degrees of freedom (used in test or to construct
confidence intervals).}

\item{lower}{Lower limit of confidence interval (used to estimate
the standard error).}

\item{upper}{Upper limit of confidence interval (used to estimate
the standard error).}

\item{level.ci}{Level of confidence interval.}

\item{median}{Median (used to estimate the treatment effect and
standard error).}

\item{q1}{First quartile (used to estimate the treatment effect and
standard error).}

\item{q3}{Third quartile (used to estimate the treatment effect and
standard error).}

\item{min}{Minimum (used to estimate the treatment effect and
standard error).}

\item{max}{Maximum (used to estimate the treatment effect and
standard error).}

\item{method.mean}{A character string indicating which method to
use to approximate the mean from the median and other statistics
(see Details).}

\item{method.sd}{A character string indicating which method to use
to approximate the standard deviation from sample size, median,
interquartile range and range (see Details).}

\item{approx.TE}{Approximation method to estimate treatment
estimate (see Details).}

\item{approx.seTE}{Approximation method to estimate standard error
(see Details).}

\item{transf}{A logical indicating whether inputs for arguments
\code{TE}, \code{lower} and \code{upper} are already
appropriately transformed to conduct the meta-analysis or on the
original scale. If \code{transf = TRUE} (default), inputs are
expected to be log odds ratios instead of odds ratios for
\code{sm = "OR"} and Fisher's z transformed correlations instead
of correlations for \code{sm = "ZCOR"}, for example.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If \code{backtransf =
TRUE} (default), results for \code{sm = "OR"} are printed as odds
ratios rather than log odds ratios and results for \code{sm =
"ZCOR"} are printed as correlations rather than Fisher's z
transformed correlations, for example.}

\item{func.transf}{A function used to transform inputs for
arguments \code{TE}, \code{lower} and \code{upper}.}

\item{func.backtransf}{A function used to back-transform results.}

\item{args.transf}{An optional list to provide additional arguments
to \code{func.transf}.}

\item{args.backtransf}{An optional list to provide additional
arguments to \code{func.backtransf}.}

\item{pscale}{A numeric giving scaling factor for printing of
single event probabilities or risk differences, i.e. if argument
\code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
\code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.}

\item{irscale}{A numeric defining a scaling factor for printing of
single incidence rates or incidence rate differences, i.e. if
argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
\code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.}

\item{irunit}{A character specifying the time unit used to
calculate rates, e.g. person-years.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.e}{Label for experimental group.}

\item{label.c}{Label for control group.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{id}{Deprecated argument (replaced by 'cluster').}

\item{adhoc.hakn}{Deprecated argument (replaced by
'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{keeprma}{A logical indicating whether \code{\link[metafor]{rma.mv}}
object from three-level meta-analysis should be stored.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded from meta-analysis due to zero
standard errors).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}} or
\code{\link[metafor]{rma.mv}}.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{c("metagen", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Common effect and random effects meta-analysis based on estimates
(e.g. log hazard ratios) and their standard errors. The inverse
variance method is used for pooling.

Three-level random effects meta-analysis (Van den Noortgate et al., 2013) is
available by internally calling \code{\link[metafor]{rma.mv}} function from
R package \bold{metafor} (Viechtbauer, 2010).
}
\details{
This function provides the \emph{generic inverse variance method}
for meta-analysis which requires treatment estimates and their
standard errors (Borenstein et al., 2010). The method is useful,
e.g., for pooling of survival data (using log hazard ratio and
standard errors as input). Arguments \code{TE} and \code{seTE} can
be used to provide treatment estimates and standard errors
directly. However, it is possible to derive these quantities from
other information.

Argument \code{cycles} can be used to conduct a meta-analysis of n-of-1
trials according to Senn (2024). In this case, argument \code{seTE} does
not contain the standard error but standard deviation for individual
trials / patients. Trial-specific standard errors are calculated from an
average standard deviation multiplied by the number of cycles minus 1, i.e.,
the degrees of freedom. Details of the meta-analysis method are provided in
Senn (2024). Note, arguments used in the approximation of means or
standard errors, like \code{lower} and \code{upper}, or \code{df}, are
ignored for the meta-analysis of n-of-1 trials.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

\subsection{Approximate treatment estimates}{

Missing treatment estimates can be derived from
\enumerate{
\item confidence limits provided by arguments \code{lower} and
  \code{upper};
\item median, interquartile range and range (arguments
  \code{median}, \code{q1}, \code{q3}, \code{min}, and \code{max});
\item median and interquartile range (arguments \code{median},
  \code{q1} and \code{q3});
\item median and range (arguments \code{median}, \code{min} and
  \code{max}).
}
For confidence limits, the treatment estimate is defined as the
center of the confidence interval (on the log scale for relative
effect measures like the odds ratio or hazard ratio).

If the treatment effect is a mean it can be approximated from
sample size, median, interquartile range and range.

By default, methods described in Luo et al. (2018) are utilised
(argument \code{method.mean = "Luo"}):
\itemize{
\item equation (7) if sample size, median and range are available,
\item equation (11) if sample size, median and interquartile range
  are available,
\item equation (15) if sample size, median, range and interquartile
  range are available.
}

Instead the methods described in Wan et al. (2014) are used if
argument \code{method.mean = "Wan"}:
\itemize{
\item equation (2) if sample size, median and range are available,
\item equation (14) if sample size, median and interquartile range
  are available,
\item equation (10) if sample size, median, range and interquartile
  range are available.
}

The following methods are also available to estimate means from
quantiles or ranges if R package \bold{estmeansd} is installed:
\itemize{
\item Method for Unknown Non-Normal Distributions (MLN) approach
  (Cai et al. (2021), argument \code{method.mean = "Cai"}),
\item Quantile Estimation (QE) method (McGrath et al. (2020),
  argument \code{method.mean = "QE-McGrath"})),
\item Box-Cox (BC) method (McGrath et al. (2020),
  argument \code{method.mean = "BC-McGrath"})).
}

By default, missing treatment estimates are replaced successively
using these method, i.e., confidence limits are utilised before
interquartile ranges. Argument \code{approx.TE} can be used to
overwrite this default for each individual study:
\itemize{
\item Use treatment estimate directly (entry \code{""} in argument
  \code{approx.TE});
\item confidence limits (\code{"ci"} in argument \code{approx.TE});
\item median, interquartile range and range (\code{"iqr.range"});
\item median and interquartile range (\code{"iqr"});
\item median and range (\code{"range"}).
}
}

\subsection{Approximate standard errors}{

Missing standard errors can be derived from
\enumerate{
\item p-value provided by arguments \code{pval} and (optional)
  \code{df};
\item confidence limits (arguments \code{lower}, \code{upper}, and
  (optional) \code{df});
\item sample size, median, interquartile range and range (arguments
  \code{n.e} and / or \code{n.c}, \code{median}, \code{q1},
  \code{q3}, \code{min}, and \code{max});
\item sample size, median and interquartile range (arguments
  \code{n.e} and / or \code{n.c}, \code{median}, \code{q1} and
  \code{q3});
\item sample size, median and range (arguments \code{n.e} and / or
  \code{n.c}, \code{median}, \code{min} and \code{max}).
}
For p-values and confidence limits, calculations are either based
on the standard normal or \emph{t}-distribution if argument
\code{df} is provided. Furthermore, argument \code{level.ci} can be
used to provide the level of the confidence interval.

Wan et al. (2014) describe methods to estimate the standard
deviation (and thus the standard error by deviding the standard
deviation with the square root of the sample size) from the sample
size, median and additional statistics. Shi et al. (2020) provide
an improved estimate of the standard deviation if the interquartile
range and range are available in addition to the sample size and
median. Accordingly, equation (11) in Shi et al. (2020) is the
default (argument \code{method.sd = "Shi"}), if the median,
interquartile range and range are provided (arguments
\code{median}, \code{q1}, \code{q3}, \code{min} and
\code{max}). The method by Wan et al. (2014) is used if argument
\code{method.sd = "Wan"} and, depending on the sample size, either
equation (12) or (13) is used. If only the interquartile range or
range is available, equations (15) / (16) and (7) / (9) in Wan et
al. (2014) are used, respectively. The sample size of individual
studies must be provided with arguments \code{n.e} and / or
\code{n.c}. The total sample size is calculated as \code{n.e} +
\code{n.c} if both arguments are provided.

The following methods are also available to estimate standard
deviations from quantiles or ranges if R package \bold{estmeansd}
is installed:
\itemize{
\item Method for Unknown Non-Normal Distributions (MLN) approach
  (Cai et al. (2021), argument \code{method.mean = "Cai"}),
\item Quantile Estimation (QE) method (McGrath et al. (2020),
  argument \code{method.mean = "QE-McGrath"})),
\item Box-Cox (BC) method (McGrath et al. (2020),
  argument \code{method.mean = "BC-McGrath"})).
}

By default, missing standard errors are replaced successively using
these method, e.g., p-value before confidence limits before
interquartile range and range. Argument \code{approx.seTE} can be
used to overwrite this default for each individual study:

\itemize{
\item Use standard error directly (entry \code{""} in argument
  \code{approx.seTE});
\item p-value (\code{"pval"} in argument \code{approx.seTE});
\item confidence limits (\code{"ci"});
\item median, interquartile range and range (\code{"iqr.range"});
\item median and interquartile range (\code{"iqr"});
\item median and range (\code{"range"}).
}
}

\subsection{Confidence intervals for individual studies}{

For the mean difference (argument \code{sm = "MD"}), the confidence
interval for individual studies can be based on the
\itemize{
\item standard normal distribution (\code{method.ci = "z"}), or
\item \emph{t}-distribution (\code{method.ci = "t"}).
}

By default, the first method is used if argument \code{df} is
missing and the second method otherwise.

Note, this choice does not affect the results of the common effect
and random effects meta-analysis.
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Specify the null hypothesis of test for an overall effect}{

Argument \code{null.effect} can be used to specify the (treatment)
effect under the null hypothesis in a test for an overall
effect.

By default (\code{null.effect = 0}), the null hypothesis
corresponds to "no difference" (which is obvious for absolute
effect measures like the mean difference (\code{sm = "MD"}) or
standardised mean difference (\code{sm = "SMD"})). For relative
effect measures, e.g., risk ratio (\code{sm = "RR"}) or odds ratio
(\code{sm = "OR"}), the null effect is defined on the log scale,
i.e., \emph{log}(RR) = 0 or \emph{log}(OR) = 0 which is equivalent
to testing RR = 1 or OR = 1.

Use of argument \code{null.effect} is especially useful for summary
measures without a "natural" null effect, i.e., in situations
without a second (treatment) group. For example, an overall
proportion of 50\% could be tested in the meta-analysis of single
proportions with argument \code{null.effect = 0.5}.

Note, all tests for an overall effect are two-sided with the
alternative hypothesis that the effect is unequal to
\code{null.effect}.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments \code{common}
and \code{random}. Accordingly, the estimate for the random effects
model can be extracted from component \code{TE.random} of an object
of class \code{"meta"} even if argument \code{random =
FALSE}. However, all functions in R package \bold{meta} will
adequately consider the values for \code{common} and
\code{random}. For example, functions \code{\link{print.meta}} and
\code{\link{forest.meta}} will not show results for the random
effects model if \code{random = FALSE}.

A prediction interval will only be shown if \code{prediction =
TRUE}.

Argument \code{pscale} can be used to rescale single proportions or
risk differences, e.g. \code{pscale = 1000} means that proportions
are expressed as events per 1000 observations. This is useful in
situations with (very) low event probabilities.

Argument \code{irscale} can be used to rescale single rates or rate
differences, e.g. \code{irscale = 1000} means that rates are
expressed as events per 1000 time units, e.g. person-years. This is
useful in situations with (very) low rates. Argument \code{irunit}
can be used to specify the time unit used in individual studies
(default: "person-years"). This information is printed in summaries
and forest plots if argument \code{irscale} is not equal to 1.

Default settings for \code{common}, \code{random},
\code{pscale}, \code{irscale}, \code{irunit} and several other
arguments can be set for the whole R session using
\code{\link{settings.meta}}.
}
}
\note{
R function \code{\link[metafor]{rma.uni}} from R package
\pkg{metafor} (Viechtbauer 2010) is called internally to estimate
the between-study variance \eqn{\tau^2}.
}
\examples{
data(Fleiss1993bin)
ma <- metabin(d.asp, n.asp, d.plac, n.plac, study,
  data = Fleiss1993bin, sm = "RR", method = "I")
ma

# Identical results using the generic inverse variance method with
# log risk ratio and its standard error:
# Note, argument 'n.e' in metagen() is used to provide the total
# sample size which is calculated from the group sample sizes n.e
# and n.c in meta-analysis ma.
ma.gen <- metagen(TE, seTE, studlab, n.e = n.e + n.c, data = ma, sm = "RR")
ma.gen
forest(ma.gen, leftcols = c("studlab", "n.e", "TE", "seTE"))

# Meta-analysis with prespecified between-study variance
#
metagen(ma$TE, ma$seTE, sm = "RR", tau.preset = sqrt(0.1))

# Meta-analysis of survival data:
#
logHR <- log(c(0.95, 1.5))
selogHR <- c(0.25, 0.35)
metagen(logHR, selogHR, sm = "HR")

# Paule-Mandel method to estimate between-study variance for data
# from Paule & Mandel (1982)
#
average <- c(27.044, 26.022, 26.340, 26.787, 26.796)
variance <- c(0.003, 0.076, 0.464, 0.003, 0.014)
#
metagen(average, sqrt(variance), sm = "MD", method.tau = "PM")

# Conduct meta-analysis using hazard ratios and 95\% confidence intervals
#
# Data from Steurer et al. (2006), Analysis 1.1 Overall survival
# https://doi.org/10.1002/14651858.CD004270.pub2
#
study <- c("FCG on CLL 1996", "Leporrier 2001", "Rai 2000", "Robak 2000")
HR <- c(0.55, 0.92, 0.79, 1.18)
lower.HR <- c(0.28, 0.79, 0.59, 0.64)
upper.HR <- c(1.09, 1.08, 1.05, 2.17)
#
# Hazard ratios and confidence intervals as input
#
summary(metagen(HR, lower = lower.HR, upper = upper.HR,
  studlab = study, sm = "HR", transf = FALSE))
#
# Same result with log hazard ratios as input
#
summary(metagen(log(HR), lower = log(lower.HR), upper = log(upper.HR),
  studlab = study, sm = "HR"))
#
# Again, same result using an unknown summary measure and
# arguments 'func.transf' and 'func.backtransf'
#
summary(metagen(HR, lower = lower.HR, upper = upper.HR,
  studlab = study, sm = "Hazard ratio",
  func.transf = log, func.backtransf = exp))
#
# Finally, same result only providing argument 'func.transf' as the
# back-transformation for the logarithm is known
#
summary(metagen(HR, lower = lower.HR, upper = upper.HR,
  studlab = study, sm = "Hazard ratio",
  func.transf = log))

# Exclude MRC-1 and MRC-2 studies from meta-analysis, however,
# show them in printouts and forest plots
#
metabin(d.asp, n.asp, d.plac, n.plac, study,
  data = Fleiss1993bin, sm = "RR", method = "I",
  exclude = study \%in\% c("MRC-1", "MRC-2"))
#
# Exclude MRC-1 and MRC-2 studies completely from meta-analysis
#
metabin(d.asp, n.asp, d.plac, n.plac, study,
  data = Fleiss1993bin, sm = "RR", method = "I",
  subset = !(study \%in\% c("MRC-1", "MRC-2")))

# Exclude studies with total sample size above 1500
#
metabin(d.asp, n.asp, d.plac, n.plac, study,
  data = Fleiss1993bin, sm = "RR", method = "I",
  exclude = (n.asp + n.plac) > 1500)

# Exclude studies containing "MRC" in study name
#
metabin(d.asp, n.asp, d.plac, n.plac, study,
  data = Fleiss1993bin, sm = "RR", method = "I",
  exclude = grep("MRC", study))

# Use both arguments 'subset' and 'exclude'
#
metabin(d.asp, n.asp, d.plac, n.plac, study,
  data = Fleiss1993bin, sm = "RR", method = "I",
  subset = (n.asp + n.plac) > 1500,
  exclude = grep("MRC", study))

\dontrun{
if (requireNamespace("metadat", quietly = TRUE)) {
# Three-level model: effects of modified school calendars on
# student achievement
data(dat.konstantopoulos2011, package = "metadat")
metagen(yi, sqrt(vi), studlab = study, data = dat.konstantopoulos2011,
  sm = "SMD",
  cluster = district, detail.tau = c("district", "district/school"))
}
}

}
\references{
Borenstein M, Hedges LV, Higgins JP, Rothstein HR (2010):
A basic introduction to fixed-effect and random-effects models for
meta-analysis.
\emph{Research Synthesis Methods},
\bold{1}, 97--111

Cai S, Zhou J, Pan J (2021):
Estimating the sample mean and standard deviation from order
statistics and sample size in meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{30}, 2701--2719

Luo D, Wan X, Liu J, Tong T (2018):
Optimally estimating the sample mean from the sample size, median,
mid-range, and/or mid-quartile range.
\emph{Statistical Methods in Medical Research},
\bold{27}, 1785--805

McGrath S, Zhao X, Steele R, et al. and the DEPRESsion Screening
Data (DEPRESSD) Collaboration (2020):
Estimating the sample mean and standard deviation from commonly
reported quantiles in meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{29}, 2520--2537

Senn S (2024):
The analysis of continuous data from n-of-1 trials using paired cycles:
a simple tutorial.
\emph{Trials},
\bold{25}.

Shi J, Luo D, Weng H, Zeng X-T, Lin L, Chu H, et al. (2020):
Optimally estimating the sample standard deviation from the
five-number summary.
\emph{Research Synthesis Methods}.

Viechtbauer W (2010):
Conducting Meta-Analyses in R with the metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Wan X, Wang W, Liu J, Tong T (2014):
Estimating the sample mean and standard deviation from the sample
size, median, range and/or interquartile range.
\emph{BMC Medical Research Methodology},
\bold{14}, 135
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{metabin}}, \code{\link{metacont}}, \code{\link{pairwise}},
  \code{\link{print.meta}}, \code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/forest.metabind.R
\name{forest.metabind}
\alias{forest.metabind}
\title{Forest plot to display the result of a meta-analysis}
\usage{
\method{forest}{metabind}(
  x,
  leftcols,
  leftlabs,
  rightcols = c("effect", "ci"),
  rightlabs,
  common = x$common,
  random = x$random,
  overall = x$overall,
  subgroup = FALSE,
  hetstat = FALSE,
  overall.hetstat = x$overall.hetstat,
  prediction = x$prediction,
  lab.NA = "",
  col.square = gs("col.square"),
  col.square.lines = col.square,
  col.circle = gs("col.circle"),
  col.circle.lines = col.circle,
  col.diamond = gs("col.diamond"),
  col.diamond.common = col.diamond,
  col.diamond.random = col.diamond,
  col.diamond.lines = gs("col.diamond.lines"),
  col.diamond.lines.common = col.diamond.lines,
  col.diamond.lines.random = col.diamond.lines,
  col.predict = gs("col.predict"),
  col.predict.lines = gs("col.predict.lines"),
  type = NULL,
  type.common = NULL,
  type.random = NULL,
  type.predict = NULL,
  digits = gs("digits.forest"),
  digits.se = gs("digits.se"),
  digits.stat = gs("digits.stat"),
  digits.pval = max(gs("digits.pval") - 2, 2),
  digits.pval.Q = max(gs("digits.pval.Q") - 2, 2),
  digits.Q = gs("digits.Q"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.I2 = max(gs("digits.I2") - 1, 0),
  scientific.pval = gs("scientific.pval"),
  big.mark = gs("big.mark"),
  print.subgroup.labels = x$with.subgroups,
  addrow.subgroups = print.subgroup.labels,
  smlab,
  calcwidth.pooled = overall,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{\link{metabind}}.}

\item{leftcols}{A character vector specifying (additional) columns
to be plotted on the left side of the forest plot or a logical
value (see Details).}

\item{leftlabs}{A character vector specifying labels for
(additional) columns on left side of the forest plot (see
Details).}

\item{rightcols}{A character vector specifying (additional) columns
to be plotted on the right side of the forest plot or a logical
value (see Details).}

\item{rightlabs}{A character vector specifying labels for
(additional) columns on right side of the forest plot (see
Details).}

\item{common}{A logical indicating whether common effect estimates
should be plotted.}

\item{random}{A logical indicating whether random effects estimates
should be plotted.}

\item{overall}{A logical indicating whether overall summaries
should be plotted. This argument is useful in a meta-analysis
with subgroups if summaries should only be plotted on group
level.}

\item{subgroup}{A logical indicating whether subgroup results
should be shown in forest plot. This argument is useful in a
meta-analysis with subgroups if summaries should not be plotted
on group level.}

\item{hetstat}{Either a logical value indicating whether to print
results for heterogeneity measures at all or a character string
(see Details).}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether prediction
interval(s) should be printed.}

\item{lab.NA}{A character string to label missing values.}

\item{col.square}{The colour for squares reflecting study's weight
in the meta-analysis.}

\item{col.square.lines}{The colour for the outer lines of squares
reflecting study's weight in the meta-analysis.}

\item{col.circle}{The colour for circles reflecting study weights
in the meta-analysis.}

\item{col.circle.lines}{The colour for the outer lines of circles
reflecting study's weight in the meta-analysis.}

\item{col.diamond}{The colour of diamonds representing the results
for common effect and random effects models.}

\item{col.diamond.common}{The colour of diamonds for common effect
estimates.}

\item{col.diamond.random}{The colour of diamonds for random effects
estimates.}

\item{col.diamond.lines}{The colour of the outer lines of diamonds
representing the results for common effect and random effects
models.}

\item{col.diamond.lines.common}{The colour of the outer lines of
diamond for common effect estimates.}

\item{col.diamond.lines.random}{The colour of the outer lines of
diamond for random effects estimates.}

\item{col.predict}{Background colour of prediction intervals.}

\item{col.predict.lines}{Colour of outer lines of prediction
intervals.}

\item{type}{A character string or vector specifying how to
plot estimates.}

\item{type.common}{A single character string specifying how to
plot common effect estimates.}

\item{type.random}{A single character string specifying how to
plot random effects estimates.}

\item{type.predict}{A single character string specifying how to
plot prediction intervals.}

\item{digits}{Minimal number of significant digits for treatment
effects, see \code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
errors, see \code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-statistic for test of overall effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of overall treatment effect, see \code{print.default}.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity test, see \code{print.default}.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistic Q, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance, see \code{print.default}.}

\item{digits.I2}{Minimal number of significant digits for I-squared
statistic, see \code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{big.mark}{A character used as thousands separator.}

\item{print.subgroup.labels}{A logical indicating whether subgroup
label should be printed.}

\item{addrow.subgroups}{A logical value indicating whether an empty
row is printed between results for subgroups.}

\item{smlab}{A label for the summary measurex (printed at top of
figure).}

\item{calcwidth.pooled}{A logical indicating whether text for
common effect and random effects model should be considered to
calculate width of the column with study labels.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional graphical arguments (passed on to
\code{\link{forest.meta}}).}
}
\description{
Draws a forest plot in the active graphics window (using grid
graphics system).
}
\details{
A forest plot, also called confidence interval plot, is drawn in
the active graphics window. The forest functions in R package
\bold{meta} are based on the grid graphics system. In order to
print the forest plot, resize the graphics window and either use
\code{\link{dev.copy2eps}} or \code{\link{dev.copy2pdf}}. Another
possibility is to create a file using \code{\link{pdf}},
\code{\link{png}}, or \code{\link{svg}} and to specify the width
and height of the graphic (see \code{\link{forest.meta}} examples).

The arguments \code{leftcols} and \code{rightcols} can be used to
specify columns which are plotted on the left and right side of the
forest plot, respectively.

The arguments \code{leftlabs} and \code{rightlabs} can be used to
specify column headings which are plotted on left and right side of
the forest plot, respectively. For certain columns predefined
labels exist. For other columns, the column name will be used as a
label. It is possible to only provide labels for new columns (see
\code{\link{forest.meta}} examples). Otherwise the length of
\code{leftlabs} and \code{rightlabs} must be the same as the number
of printed columns, respectively. The value \code{NA} can be used
to specify columns which should use default labels.

Argument \code{hetstat} can be a character string to specify where
to print heterogeneity information:
\itemize{
\item row with results for common effect model (\code{hetstat =
"common"}),
\item row with results for random effects model (\code{hetstat =
"random"}),
\item rows with 'study' information (\code{hetstat = "study"}).
}
Otherwise, information on heterogeneity is printed in dedicated rows.
}
\examples{
data(Fleiss1993cont)

# Add some (fictitious) grouping variables:
#
Fleiss1993cont$age <- c(55, 65, 55, 65, 55)
Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")

ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")

# Conduct two subgroup analyses
#
ma1 <- update(ma, subgroup = age, subgroup.name = "Age group")
ma2 <- update(ma, subgroup = region, subgroup.name = "Region")

# Combine subgroup meta-analyses and show forest plot with subgroup
# results
#
mb <- metabind(ma1, ma2)
mb
forest(mb)

}
\seealso{
\code{\link{forest.meta}}, \code{\link{metabin}},
  \code{\link{metacont}}, \code{\link{metagen}},
  \code{\link{metabind}}, \code{\link{settings.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/blup.meta.R
\name{blup.meta}
\alias{blup.meta}
\alias{blup}
\alias{print.blup.meta}
\alias{estimates.blup.meta}
\alias{print.estimates.blup.meta}
\title{Calculate best linear unbiased predictor for \code{meta} object}
\usage{
\method{blup}{meta}(x, level = x$level, backtransf = x$backtransf, ...)

\method{print}{blup.meta}(
  x,
  backtransf = attr(x, "x")$backtransf,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  big.mark = gs("big.mark"),
  se = FALSE,
  print.tau2 = gs("print.tau2"),
  print.tau = gs("print.tau"),
  details = gs("details"),
  ...
)

\method{estimates}{blup.meta}(
  x,
  se = FALSE,
  backtransf = attr(x, "x")$backtransf,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  writexl = !missing(path),
  path = "estimates_blup.xlsx",
  overwrite = FALSE,
  ...
)

\method{print}{estimates.blup.meta}(x, big.mark = gs("big.mark"), details = gs("details"), ...)
}
\arguments{
\item{x}{An object of class \code{meta}, \code{blup.meta}, or
\code{estimates.blup.meta}.}

\item{level}{The level used to calculate prediction intervals for BLUPs.}

\item{backtransf}{A logical indicating whether BLUPs should be
back transformed. If \code{backtransf = TRUE}, results for
\code{sm = "OR"} will be odds ratios rather than log odds ratios,
 for example.}

\item{\dots}{Additional arguments (passed on to \code{\link{prmatrix}}).}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard errors.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance \eqn{\tau^2}, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for
\eqn{\tau}, the square root of the between-study variance
\eqn{\tau^2}.}

\item{big.mark}{A character used as thousands separator.}

\item{se}{A logical indicating whether standard errors should be
printed / extracted.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{details}{A logical specifying whether details on
statistical methods should be printed.}

\item{writexl}{A logical indicating whether an Excel file should be
created (R package \bold{writexl} must be available).}

\item{path}{A character string specifying the filename of the Excel
file.}

\item{overwrite}{A logical indicating whether an existing Excel
file should be overwritten.}
}
\value{
Data frame with variables
\item{studlab}{Study label}
\item{blup}{estimated best linear unbiased predictor}
\item{se.blup}{standard error (only if argument \code{backtransf = FALSE})}
\item{lower}{lower prediction limits}
\item{upper}{upper prediction limits}
}
\description{
Calculate best linear unbiased predictors (BLUPs) for meta-analysis object
created with R package \bold{meta}.
}
\examples{
\dontrun{
if (requireNamespace("metadat", quietly = TRUE)) {
data(dat.bcg, package = "metadat")
ma <- metabin(tpos, tpos + tneg, cpos, cpos + cneg,
  data = dat.bcg, studlab = paste(author, year), method = "Inverse")
summary(ma)
blup(ma)

# Save estimates in Excel file (R package 'writexl' must be available)
if (requireNamespace("writexl", quietly = TRUE))
 estimates(blup(ma), path = "blup_ma.xlsx")
}
}

}
\seealso{
\code{\link{metabin}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairwise.R
\name{pairwise}
\alias{pairwise}
\title{Transform meta-analysis data from two arm-based formats into contrast-based
format}
\usage{
pairwise(
  treat,
  event,
  n,
  mean,
  sd,
  TE,
  seTE,
  time,
  agent,
  dose,
  data = NULL,
  studlab,
  method = "Inverse",
  sm = NULL,
  incr = gs("incr"),
  method.incr = gs("method.incr"),
  allstudies = gs("allstudies"),
  relative.effects = FALSE,
  reference.group,
  keep.all.comparisons,
  sep.ag = "*",
  varnames = c("TE", "seTE"),
  append = !is.null(data),
  addincr = gs("addincr"),
  allincr = gs("allincr"),
  warn = FALSE,
  warn.deprecated = gs("warn.deprecated"),
  ...
)
}
\arguments{
\item{treat}{A list or vector with treatment information for
individual treatment arms (see Details).}

\item{event}{A list or vector with information on number of events
for individual treatment arms (see Details).}

\item{n}{A list or vector with information on number of
observations for individual treatment arms (see Details).}

\item{mean}{A list or vector with estimated means for individual
treatment arms (see Details).}

\item{sd}{A list or vector with information on the standard
deviation for individual treatment arms (see Details).}

\item{TE}{A list or vector with estimated treatment effects for
individual treatment arms (see Details).}

\item{seTE}{A list or vector with standard errors of estimated
treatment effect for individual treatment arms (see Details).}

\item{time}{A list or vector with information on person time at
risk for individual treatment arms (see Details).}

\item{agent}{A list or vector with agent information for
individual treatment arms (see Details).}

\item{dose}{A list or vector with dose information for
individual treatment arms (see Details).}

\item{data}{An optional data frame containing the study
information.}

\item{studlab}{A vector with study labels (optional).}

\item{method}{A character string indicating which method is to be
used to calculate treatment estimates (see Details).}

\item{sm}{A character string indicating which summary measure is to be
used to calculate treatment estimates (see Details).}

\item{incr}{A numerical value which is added to cell frequencies
for studies with a zero cell count, see Details.}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, or \code{"all"}), see \code{\link{metabin}}.}

\item{allstudies}{A logical indicating if studies with zero or all
events in two treatment arms are to be included in the
meta-analysis (applies only if \code{sm} is equal to \code{"RR"}
or \code{"OR"}).}

\item{relative.effects}{A logical indicating whether treatment estimates
and standard errors provided in \code{TE} and \code{seTE} are relative
treatment effects.}

\item{reference.group}{Reference treatment (first treatment is used
if argument is missing).}

\item{keep.all.comparisons}{A logical indicating whether all
pairwise comparisons or only comparisons with the study-specific
reference group should be kept ('basic parameters').}

\item{sep.ag}{A character used as separator between agent and dose to
create treatment labels.}

\item{varnames}{Character vector of length 2 with the variable names for the
treatment estimate and its standard error; by default, "TE" and "seTE".}

\item{append}{Either a logical indicating whether variables from the dataset
provided in argument \code{data} are appended to the dataset with
pairwise comparisons or a character vector with variable names to append to
the dataset.}

\item{addincr}{Deprecated argument (replaced by 'method.incr');
see \code{\link{metabin}}.}

\item{allincr}{Deprecated argument (replaced by 'method.incr');
see \code{\link{metabin}}.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded due to only providing a single
treatment arm).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments passed-through to the functions
to calculate effects.}
}
\value{
A data frame with the following columns:
\item{TE}{Treatment estimate comparing treatment 'treat1' and
  'treat2'.}
\item{seTE}{Standard error of treatment estimate.}
\item{studlab}{Study labels.}
\item{treat1}{First treatment in comparison.}
\item{treat2}{Second treatment in comparison.}
\item{event1}{Number of events for first treatment arm (for metabin
  and metainc).}
\item{event2}{Number of events for second treatment arm (for
  metabin and metainc).}
\item{n1}{Number of observations for first treatment arm (for
  metabin and metacont).}
\item{n2}{Number of observations for second treatment arm (for
  metabin and metacont).}
\item{mean1}{Estimated mean for first treatment arm (for
  metacont).}
\item{mean2}{Estimated mean for second treatment arm (for
  metacont).}
\item{sd1}{Standard deviation for first treatment arm (for
  metacont).}
\item{sd2}{Standard deviation for second treatment arm (for
  metacont).}
\item{TE1}{Estimated treatment effect for first treatment arm (for
  metagen).}
\item{TE2}{Estimated treatment effect for second treatment arm (for
  metagen).}
\item{seTE1}{Standard error of estimated treatment effect for first
  treatment arm (for metagen).}
\item{seTE2}{Standard error of estimated treatment effect for
  second treatment arm (for metagen).}
\item{time1}{Person time at risk for first treatment arm (for
  metainc).}
\item{time2}{Person time at risk for second treatment arm (for
  metainc).}
\item{agent1}{First agent in comparison.}
\item{agent2}{Second agent in comparison.}
\item{dose1}{Dose of first agent in comparison.}
\item{dose2}{Dose of second agent in comparison.}

All variables from the original dataset are also part of the output
dataset; see Details.
}
\description{
This function transforms data that are given in wide or long arm-based
format (e.g. input format for WinBUGS) to a contrast-based format that is
needed as input to R functions \code{\link{metabin}}, \code{\link{metacont}},
\code{\link{metainc}}, \code{\link{metagen}}, or
\code{\link[netmeta]{netmeta}} from R package \bold{netmeta}. The function
can transform data with binary, continuous, or generic outcomes as well as
incidence rates from arm-based to contrast-based format.
}
\details{
The pairwise function transforms data given in (wide or long)
arm-based format into the contrast-based format which consists of
\emph{pairwise} comparisons and which is needed as input to R functions
\code{\link{metabin}}, \code{\link{metacont}}, \code{\link{metainc}},
\code{\link{metagen}}, or \code{\link[netmeta]{netmeta}} from R package
\bold{netmeta}.

The pairwise function can transform data with binary outcomes
continuous outcomes (\code{\link{metacont}} function), incidence
rates (\code{\link{metainc}} function), and generic outcomes
(\code{\link{metagen}} function). Depending on the outcome, the
following arguments are mandatory:
\itemize{
\item treat, event, n (see \code{\link{metabin}});
\item treat, n, mean, sd (see \code{\link{metacont}});
\item treat, event, time (see \code{\link{metainc}});
\item treat, TE, seTE (see \code{\link{metagen}}).
}

Admissible values for arguments \code{method} and \code{sm} are outcome
specific; see help pages of R functions \code{\link{metabin}},
\code{\link{metacont}}, \code{\link{metainc}}, and \code{\link{metagen}}.

It is also possible to provide data in the common BUGS format, i.e.,
relative treatment effects in comparison to a reference treatment. In this
case, the user has to specify argument \code{relative.effects = TRUE} and
the data set must provide the relative treatment estimates and standard
errors. The relative treatment effect for the study-specific reference
treatment must be equal to \code{NA} (see Examples).

Argument \code{treat} is mandatory to identify the individual
treatments. The other arguments contain outcome specific
data. These arguments must be either lists (wide arm-based format,
i.e., one row per study) or vectors (long arm-based format,
i.e., multiple rows per study) of the same length.

For the wide arm-based format, each list consists of as many
vectors of the same length as the multi-arm study with the largest
number of treatments. If a single multi-arm study has five arms,
five vectors have to be provided for each lists. Two-arm studies
have entries with \code{NA} for the third and subsequent
vectors. Each list entry is a vector with information for each
individual study; i.e., the length of this vector corresponds to
the total number of studies incorporated in the network
meta-analysis. Typically, list elements are part of a data frame
(argument \code{data}, optional); see Examples. An optional vector
with study labels can be provided which can be part of the data
frame.

In the long arm-based format, argument \code{studlab} is mandatory
to identify rows contributing to individual studies.

Additional arguments for meta-analysis functions can be provided
using argument '\dots'; see help pages of R functions
\code{\link{metabin}}, \code{\link{metacont}},
\code{\link{metainc}}, and \code{\link{metagen}}.

For standardised mean differences (argument \code{sm = "SMD"}),
equations (4) and (5) in Crippa & Orsini (2016) are used to
calculated SMDs and standard errors. These equations guarantee
consistent SMDs and standard errors for multi-arm studies. Note,
the summary measure is actually Cohen's d as Hedges' g is not
consistent in multi-arm studies.

For binary outcomes, 0.5 is added to all cell frequencies (odds
ratio) or only the number of events (risk ratio) for studies with a
zero cell count. For odds ratio and risk ratio, treatment estimates
and standard errors are only calculated for studies with zero or
all events in both groups if \code{allstudies} is \code{TRUE}. This
continuity correction is used both to calculate individual study
results with confidence limits and to conduct meta-analysis based
on the inverse variance method. For the risk difference, 0.5 is
only added to all cell frequencies to calculate the standard error.

For incidence rates, 0.5 is added to all cell frequencies for the
incidence rate ratio as summary measure. For the incidence risk
difference, 0.5 is only added to all cell frequencies to calculate
the standard error.

The value of pairwise is a data frame with as many rows as there
are pairwise comparisons. For each study with \emph{p} treatments,
\emph{p*(p-1) / 2} contrasts are generated. Each row contains the
treatment effect (\code{TE}), its standard error (\code{seTE}), the
treatments compared ((\code{treat1}), (\code{treat2})) and the
study label ((\code{studlab})). Further columns are added according
to type of data.

All variables from the original dataset are also part of the output
dataset if argument \code{append = TRUE}. If data are provided in the long
arm-based format, the value of a variable can differ between treatment arms;
for example, the mean age or percentage of women in the treatment arm. In
this situation, two variables instead of one variable will be included
in the output dataset. The values "1" and "2" are added to the
names for these variables, e.g. "mean.age1" and "mean.age2" for the
mean age.

In general, any variable names in the original dataset that are
identical to the main variable names (i.e., "TE", "seTE", ...) will
be renamed to variable names with ending ".orig".

A reduced dataset with basic comparisons (Rücker & Schwarzer,
2014) can be generated using argument \code{keep.all.comparisons =
FALSE}. Furthermore, the reference group for the basic comparisons
can be specified with argument \code{reference.group}.

\subsection{Use in network meta-analysis}{
R function \code{\link[netmeta]{netmeta}} expects data in a
\bold{contrast-based format}, where each row corresponds to a
comparison of two treatments and contains a measure of the
treatment effect comparing two treatments with standard error,
labels for the two treatments and an optional study label.  In
contrast-based format, a three-arm study contributes three rows
with treatment comparison and corresponding standard error for
pairwise comparison \emph{A} vs \emph{B}, \emph{A} vs \emph{C}, and
\emph{B} vs \emph{C} whereas a four-arm study contributes six rows
/ pairwise comparisons: \emph{A} vs \emph{B}, \emph{A} vs \emph{C},
\dots{}, \emph{C} vs \emph{D}.

Other programs for network meta-analysis in WinBUGS and Stata
require data in an \emph{arm-based} format, i.e. treatment estimate
for each treatment arm instead of a difference of two treatments. A
common \bold{(wide) arm-based format} consists of one data row per
study, containing treatment and other necessary information for all
study arms. For example, a four-arm study contributes one row with
four treatment estimates and corresponding standard errors for
treatments \emph{A}, \emph{B}, \emph{C}, and \emph{D}.  Another
possible arm-based format is a long format where each row
corresponds to a single study arm. Accordingly, in the \bold{long
arm-based format} a study contributes as many rows as treatments
considered in the study.
}
}
\note{
This function must not be confused with \code{\link[netmeta]{netpairwise}}
which can be used to conduct pairwise meta-analyses for all
comparisons with direct evidence in a network meta-analysis.
}
\examples{
pw0 <- pairwise(studlab = study, treat = treatment,
  n = n, mean = mean, sd = sd, data = Senn2013,
  varnames = c("MD", "seMD"), append = c("study", "type"))
head(pw0)
# Meta-analysis of studies comparing metformin to placebo
metagen(pw0, subset = treat1 == "Metformin" & treat2 == "Placebo")

\dontrun{
# Use pairwise() to run network meta-analyses
# (R package 'netmeta' must be available)
if (requireNamespace("netmeta", quietly = TRUE)) {
 # Example with continuous outcome - transform data from wide arm-based to
 # contrast-based format (internal call of function metacont)
 pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
   n = list(n1, n2, n3),
   mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
   data = Franchini2012, studlab = Study)
 pw1

 # Conduct network meta-analysis
 library("netmeta")
 #
 nma1 <- netmeta(pw1)
 nma1

 # Draw network graphs
 #
 netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.5,
   thickness = "se.common")
 netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.5,
   plastic = TRUE, thickness = "se.common",
   iterate = TRUE)
 netgraph(nma1, points = TRUE, cex.points = 3, cex = 1.5,
   plastic = TRUE, thickness = "se.common",
   iterate = TRUE, start = "eigen")

 # Example using generic outcome (internal call of function
 # metagen)
 #
 # Calculate standard error for means y1, y2, y3
 Franchini2012$se1 <- with(Franchini2012, sqrt(sd1^2 / n1))
 Franchini2012$se2 <- with(Franchini2012, sqrt(sd2^2 / n2))
 Franchini2012$se3 <- with(Franchini2012, sqrt(sd3^2 / n3))
 # Transform data from arm-based format to contrast-based format
 # using means and standard errors (note, argument 'sm' has to be
 # used to specify that argument 'TE' is a mean difference)
 pw2 <- pairwise(list(Treatment1, Treatment2, Treatment3),
   TE = list(y1, y2, y3), seTE = list(se1, se2, se3),
   n = list(n1, n2, n3),
   data = Franchini2012, studlab = Study,
   sm = "MD")
 pw2

 # Compare pairwise objects pw1 (based on continuous outcomes) and pw2
 # (based on generic outcomes)
 #
 all.equal(
   pw1[, c("TE", "seTE", "studlab", "treat1", "treat2")],
   pw2[, c("TE", "seTE", "studlab", "treat1", "treat2")])
 # Only difference:
 attributes(pw1)$type
 attributes(pw2)$type

 # Same result as network meta-analysis based on continuous outcomes
 # (object nma1)
 nma2 <- netmeta(pw2)
 nma2

 # Example with binary data
 #
 data(smokingcessation)
 # Transform data from arm-based format to contrast-based format
 # (internal call of metabin function). Argument 'sm' has to be used
 # for odds ratio as risk ratio (sm = "RR") is default of metabin
 # function.
 #
 pw3 <- pairwise(list(treat1, treat2, treat3),
   list(event1, event2, event3), list(n1, n2, n3),
   data = smokingcessation,
   sm = "OR")
 pw3

 # Conduct network meta-analysis
 #
 nma3 <- netmeta(pw3)
 nma3

 # Example with incidence rates
 #
 data(dietaryfat)

 # Transform data from arm-based format to contrast-based format
 #
 pw4 <- pairwise(list(treat1, treat2, treat3),
   list(d1, d2, d3), time = list(years1, years2, years3),
   studlab = ID,
   data = dietaryfat)
 pw4

 # Conduct network meta-analysis using incidence rate ratios (sm =
 # "IRR"). Note, the argument 'sm' is not necessary as this is the
 # default in R function metainc called internally.
 #
 nma4 <- netmeta(pw4, sm = "IRR")
 summary(nma4)

 # Example with long data format
 #
 # Transform data from long arm-based format to contrast-based
 # format Argument 'sm' has to be used for odds ratio as summary
 # measure; by default the risk ratio is used in the metabin
 # function called internally.
 #
 pw5 <- pairwise(treatment, event = r, n = N,
   studlab = author, data = Woods2010, sm = "OR")
 pw5

 # Conduct network meta-analysis
 nma5 <- netmeta(pw5)
 nma5
 
 # Relative treatment effects as input, i.e., common BUGS format
 #
 if (requireNamespace("gemtc", quietly = TRUE) ) {
 library(gemtc)
 
 dat1 <- parkinson_diff$data
 dat1
 
 pw6 <- pairwise(treatment, TE = diff, seTE = std.err, studlab = study,
   data = dat1, relative.effects = TRUE, sm = "MD")
 pw6
 #
 nma6 <- netmeta(pw6, common = FALSE, method.tau = "REML",
   small.values = "undesirable")
 nma6

 datE <- dat1[nrow(dat1), ]
 datE$treatment <- "E"
 dat2 <- rbind(dat1, datE)
 #
 pw7 <- pairwise(treatment, TE = diff, seTE = std.err, studlab = study,
   data = dat2, relative.effects = TRUE, sm = "MD")
 nma7 <- netmeta(pw7, common = FALSE, method.tau = "REML",
   small.values = "undesirable")
 nma7
 #
 # Number of pairwise comparisons
 #
 nma6$m
 nma7$m
 #
 detach("package:gemtc")
}
}
}

}
\references{
Crippa A, Orsini N (2016):
Dose-response meta-analysis of differences in means.
\emph{BMC Medical Research Methodology},
\bold{16}:91.
}
\seealso{
\code{\link{longarm}}, \code{\link{metabin}},
  \code{\link{metacont}}, \code{\link{metagen}},
  \code{\link{metainc}}, \code{\link[netmeta]{netmeta}},
  \code{\link[netmeta]{netgraph.netmeta}},
  \code{\link[metabook]{Senn2013}},
  \code{\link[metabook]{Franchini2012}}
}
\author{
Gerta Rücker\email{gerta.ruecker@uniklinik-freiburg.de}, Guido
  Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de},
  Gert van Valkenhoef \email{gvanvalkenhoef@cochrane.org}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metamean.R
\name{metamean}
\alias{metamean}
\title{Meta-analysis of single means}
\usage{
metamean(
  n,
  mean,
  sd,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  median,
  q1,
  q3,
  min,
  max,
  method.mean = "Luo",
  method.sd = "Shi",
  approx.mean,
  approx.sd,
  sm = gs("smmean"),
  method.ci = gs("method.ci.cont"),
  level = gs("level"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = gs("method.tau"),
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  null.effect = NA,
  method.bias = gs("method.bias"),
  backtransf = gs("backtransf"),
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{n}{Number of observations.}

\item{mean}{Estimated mean.}

\item{sd}{Standard deviation.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{median}{Median (used to estimate the mean and standard
deviation).}

\item{q1}{First quartile (used to estimate the mean and standard
deviation).}

\item{q3}{Third quartile (used to estimate the mean and standard
deviation).}

\item{min}{Minimum (used to estimate the mean and standard
deviation).}

\item{max}{Maximum (used to estimate the mean and standard
deviation).}

\item{method.mean}{A character string indicating which method to
use to approximate the mean from the median and other statistics
(see Details).}

\item{method.sd}{A character string indicating which method to use
to approximate the standard deviation from sample size, median,
interquartile range and range (see Details).}

\item{approx.mean}{Approximation method to estimate means (see
Details).}

\item{approx.sd}{Approximation method to estimate standard
deviations (see Details).}

\item{sm}{A character string indicating which summary measure
(\code{"MRAW"} or \code{"MLN"}) is to be used for pooling of
studies.}

\item{method.ci}{A character string indicating which method is used
to calculate confidence intervals for individual studies, see
Details.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{null.effect}{A numeric value specifying the effect under the
null hypothesis.}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, or
\code{"Thompson"}, can be abbreviated. See function
\code{\link{metabias}}.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots for \code{sm = "MLN"}. If
TRUE (default), results will be presented as means; otherwise
logarithm of means will be shown.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{adhoc.hakn}{Deprecated argument (replaced by 'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded from meta-analysis due to zero
standard deviations).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}}.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{c("metamean", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of an overall mean from studies reporting a single mean
using the inverse variance method for pooling; inverse variance
weighting is used for pooling.
}
\details{
Common effect and random effects meta-analysis of single means to
calculate an overall mean; inverse variance weighting is used for
pooling. Note, you should use R function \code{\link{metacont}} to
compare means of pairwise comparisons instead of using
\code{metamean} for each treatment arm separately which will break
randomisation in randomised controlled trials.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

The following transformations of means are implemented to
calculate an overall mean:
\itemize{
\item Raw, i.e. untransformed, means (\code{sm = "MRAW"}, default)
\item Log transformed means (\code{sm = "MLN"})
}

Calculations are conducted on the log scale if \code{sm =
"MLN"}. Accordingly, list elements \code{TE}, \code{TE.common}, and
\code{TE.random} contain the logarithm of means. In printouts and
plots these values are back transformed if argument
\code{backtransf = TRUE} (default).

\subsection{Approximate means from sample sizes, medians and other statistics}{

Missing means can be derived from
\enumerate{
\item sample size, median, interquartile range and range (arguments
  \code{n}, \code{median}, \code{q1}, \code{q3}, \code{min}, and
  \code{max}),
\item sample size, median and interquartile range (arguments
  \code{n}, \code{median}, \code{q1}, and \code{q3}), or
\item sample size, median and range (arguments \code{n},
  \code{median}, \code{min}, and \code{max}).
}

By default, methods described in Luo et al. (2018) are utilised
(argument \code{method.mean = "Luo"}):
\itemize{
\item equation (15) if sample size, median, interquartile range and 
  range are available,
\item equation (11) if sample size, median and interquartile range
  are available,
\item equation (7) if sample size, median and range are available.
}

Instead the methods described in Wan et al. (2014) are used if
argument \code{method.mean = "Wan"}:
\itemize{
\item equation (10) if sample size, median, interquartile range and 
  range are available,
\item equation (14) if sample size, median and interquartile range
  are available,
\item equation (2) if sample size, median and range are available.
}

The following methods are also available to estimate means from
quantiles or ranges if R package \bold{estmeansd} is installed:
\itemize{
\item Method for Unknown Non-Normal Distributions (MLN) approach
  (Cai et al. (2021), argument \code{method.mean = "Cai"}),
\item Quantile Estimation (QE) method (McGrath et al. (2020),
  argument \code{method.mean = "QE-McGrath"})),
\item Box-Cox (BC) method (McGrath et al. (2020),
  argument \code{method.mean = "BC-McGrath"})).
}

By default, missing means are replaced successively using
interquartile ranges and ranges (if available), interquartile
ranges (if available) and finally ranges. Argument
\code{approx.mean} can be used to overwrite this behaviour for each
individual study and treatment arm:
\itemize{
\item use means directly (entry \code{""} in argument
  \code{approx.mean});
\item median, interquartile range and range (\code{"iqr.range"});
\item median and interquartile range (\code{"iqr"});
\item median and range (\code{"range"}).
}
}

\subsection{Approximate standard deviations from sample sizes, medians and other statistics}{

Missing standard deviations can be derived from
\enumerate{
\item sample size, median, interquartile range and range (arguments
  \code{n}, \code{median}, \code{q1}, \code{q3}, \code{min}, and
  \code{max}),
\item sample size, median and interquartile range (arguments
  \code{n}, \code{median}, \code{q1} and \code{q3}), or
\item sample size, median and range (arguments \code{n},
  \code{median}, \code{min} and \code{max}).
}

Wan et al. (2014) describe methods to estimate the standard
deviation from the sample size, median and additional
statistics. Shi et al. (2020) provide an improved estimate of the
standard deviation if the interquartile range and range are
available in addition to the sample size and median. Accordingly,
equation (11) in Shi et al. (2020) is the default (argument
\code{method.sd = "Shi"}), if the median, interquartile range and
range are provided. The method by Wan et al. (2014) is used if
argument \code{method.sd = "Wan"} and, depending on the sample
size, either equation (12) or (13) is used. If only the
interquartile range or range is available, equations (15) / (16)
and (7) / (9) in Wan et al. (2014) are used, respectively.

The following methods are also available to estimate standard
deviations from quantiles or ranges if R package \bold{estmeansd}
is installed:
\itemize{
\item Method for Unknown Non-Normal Distributions (MLN) approach
  (Cai et al. (2021), argument \code{method.mean = "Cai"}),
\item Quantile Estimation (QE) method (McGrath et al. (2020),
  argument \code{method.mean = "QE-McGrath"})),
\item Box-Cox (BC) method (McGrath et al. (2020),
  argument \code{method.mean = "BC-McGrath"})).
}

By default, missing standard deviations are replaced successively
using these method, i.e., interquartile ranges and ranges are used
before interquartile ranges before ranges. Argument
\code{approx.sd} can be used to overwrite this default for each
individual study and treatment arms:
\itemize{
\item sample size, median, interquartile range and range
  (\code{"iqr.range"});
\item sample size, median and interquartile range (\code{"iqr"});
\item sample size, median and range (\code{"range"}).
}
}

\subsection{Confidence intervals for individual studies}{

For untransformed means (argument \code{sm = "MRAW"}), the
confidence interval for individual studies can be based on the
\itemize{
\item standard normal distribution (\code{method.ci = "z"}, default), or
\item t-distribution (\code{method.ci = "t"}).
}
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments
\code{common} and \code{random}. Accordingly, the estimate
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"meta"} even if
argument \code{random = FALSE}. However, all functions in R
package \bold{meta} will adequately consider the values for
\code{common} and \code{random}. E.g. functions
\code{\link{print.meta}} and \code{\link{forest.meta}} will not
print results for the random effects model if \code{random =
FALSE}.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\note{
The function \code{\link{metagen}} is called internally to
calculate individual and overall treatment estimates and standard
errors.
}
\examples{
ma1 <- metamean(rep(100, 3), 1:3, rep(1, 3))
ma1

ma2 <- update(ma1, sm = "MLN")
ma2

# With test for overall mean equal to 2
#
update(ma1, null.effect = 2)
update(ma2, null.effect = 2)

# Print results without back-transformation
#
update(ma1, backtransf = FALSE)
update(ma2, backtransf = FALSE)
update(ma1, null.effect = 2, backtransf = FALSE)
update(ma2, null.effect = 2, backtransf = FALSE)

}
\references{
Cai S, Zhou J, Pan J (2021):
Estimating the sample mean and standard deviation from order
statistics and sample size in meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{30}, 2701--2719

Luo D, Wan X, Liu J, Tong T (2018):
Optimally estimating the sample mean from the sample size, median,
mid-range, and/or mid-quartile range.
\emph{Statistical Methods in Medical Research},
\bold{27}, 1785--805

McGrath S, Zhao X, Steele R, et al. and the DEPRESsion Screening
Data (DEPRESSD) Collaboration (2020):
Estimating the sample mean and standard deviation from commonly
reported quantiles in meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{29}, 2520--2537

Shi J, Luo D, Weng H, Zeng X-T, Lin L, Chu H, et al. (2020):
Optimally estimating the sample standard deviation from the
five-number summary.
\emph{Research Synthesis Methods}.

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca
J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Wan X, Wang W, Liu J, Tong T (2014):
Estimating the sample mean and standard deviation from the sample
size, median, range and/or interquartile range.
\emph{BMC Medical Research Methodology},
\bold{14}, 135
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{metamean}}, \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.cidprop.R
\name{plot.cidprop}
\alias{plot.cidprop}
\title{Plot density of prediction distribution highlighting areas of clinically
important benefit or harm}
\usage{
\method{plot}{cidprop}(
  x,
  cid = NULL,
  cid.below.null = x$cid.below.null,
  cid.above.null = x$cid.above.null,
  label.cid = "",
  label.cid.below.null = x$label.cid.below.null,
  label.cid.above.null = x$label.cid.above.null,
  small.values = x$small.values,
  fill.cid.below.null = NULL,
  fill.cid.above.null = NULL,
  fill = "white",
  legend = FALSE,
  studies = TRUE,
  random = TRUE,
  col.diamond = gs("col.diamond"),
  col.diamond.lines = gs("col.diamond.lines"),
  prediction = TRUE,
  col.predict = gs("col.predict"),
  col.predict.lines = gs("col.predict.lines"),
  big.mark = gs("big.mark"),
  digits.cid = gs("digits.cid"),
  digits.percent = 1,
  digits.xaxis = gs("digits.forest"),
  xlab = NULL,
  ylab = NULL,
  xlim = NULL,
  ylim = NULL,
  labels.x = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{cidprop}.}

\item{cid}{A numeric value or vector specifying clinically important
differences (CID) / decision thresholds used to calculate expected
proportions of clinically important benefit or harm (see Details).}

\item{cid.below.null}{A numeric value or vector specifying CID limits below
the null effect (see Details).}

\item{cid.above.null}{A numeric value or vector specifying CID limits above
the null effect (see Details).}

\item{label.cid}{A character string or vector specifying labels for
clinically important differences. Must be of same length as argument
\code{cid}.}

\item{label.cid.below.null}{A character string or vector specifying labels
for clinically important differences below the null effect. Must be of
same length as argument \code{cid.below.null} (or \code{cid}).}

\item{label.cid.above.null}{A character string or vector specifying labels
for clinically important differences above the null effect. Must be of
same length as argument \code{cid.above.null} (or \code{cid}).}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}), can be abbreviated.}

\item{fill.cid.below.null}{Background colour(s) for CID areas below null
effect.}

\item{fill.cid.above.null}{Background colour(s) for CID areas above null
effect.}

\item{fill}{Background colour for area between decision thresholds.}

\item{legend}{A logical indicating whether to print a legend with
expected proportions of beneficial, harmful, or not important effects.}

\item{studies}{A logical indicating whether to print estimates of individual
studies.}

\item{random}{A logical indicating whether to show diamond of the random
effects meta-analysis.}

\item{col.diamond}{The colour of the diamond representing the results
for the random effects model.}

\item{col.diamond.lines}{The colour of the outer lines of the diamond
representing the results of the random effects model.}

\item{prediction}{A logical indicating whether to show the prediction
interval.}

\item{col.predict}{The colour of the prediction interval.}

\item{col.predict.lines}{The colour of the outer lines of the prediction
interval.}

\item{big.mark}{A character used as thousands separator.}

\item{digits.cid}{Minimal number of significant digits for
decision thresholds, see \code{\link{print.default}}.}

\item{digits.percent}{Minimal number of significant digits for
expected proportions, printed as percentages, see
\code{\link{print.default}}.}

\item{digits.xaxis}{Minimal number of significant digits for
labels on x-axis, see \code{\link{print.default}}.}

\item{xlab}{Label on x-axis.}

\item{ylab}{Label on y-axis.}

\item{xlim}{Limits for x-axis.}

\item{ylim}{Limits for y-axis.}

\item{labels.x}{Predefined labels for tick marks on x-axis.}

\item{\dots}{Additional arguments (ignored)}
}
\value{
A ggplot object with additional class 'plot.cidprop'.
}
\description{
Plot density of prediction distribution highlighting areas of clinically
important benefit or harm
}
\details{
This function plots the density of the prediction distribution highlighting
areas of clinically important benefit or harm (Siemens et al., 2025).

Arguments \code{cid}, \code{cid.below.null}, \code{cid.above.null},
\code{label.cid}, \code{label.cid.below.null}, \code{label.cid.above.null},
and \code{small.values} are identical to the main arguments of R function
\code{\link{cidprop}} which is called internally if any of these values has
been provided by the user.

R packages \bold{ggpubr} and \bold{gridExtra} must be installed in order to
add a legend to the plot with the CIDs, expected proportions of clinically
benefit or harm, and the area colours (due to using R functions
\code{ggarrange} and \code{tableGrob}). The data and colours shown in the
legend are stored in the attribute 'data.cid' of the returned ggplot object
(see Examples).

UTF-8 code for the less than or equal and greater than or equal signs are
used in the legend. Accordingly, graphic devices with full UTF-8 support are
required to save graphics, for example, \code{\link[grDevices]{cairo_pdf}}
instead of \code{\link[grDevices]{pdf}} from R package \bold{grDevices}.
}
\examples{
oldset <- settings.meta(digits.cid = 0)

m <- metagen(1:10 - 3, 1:10, sm = "MD")

pp1 <- cidprop(m, cid = 2)
pp1
plot(pp1, xlim = c(-4, 4))
\donttest{
pp2 <- cidprop(m, cid.below.null = 0.5, cid.above.null = 2)
pp2
plot(pp2, xlim = c(-4, 4))

pp3 <- cidprop(m, cid.below.null = 0.5, cid.above.null = 2,
  small.values = "u")
pp3
plot(pp3, xlim = c(-4, 4))

pp4 <- cidprop(m, cid = 1:2, label.cid = c("moderate", "large"))
pp4
plot(pp4, xlim = c(-4, 4))

pp5 <- cidprop(m, cid.below.null = -1.5, cid.above.null = 1:2,
  label.cid.below.null = "large",
  label.cid.above.null = c("moderate", "large"))
pp5
plpp5 <- plot(pp5, xlim = c(-4, 4))
plpp5
# Information on CIDs and colours
attr(plpp5, "data.cid")
}
\dontrun{
# R packages 'ggpubr' and 'gridExtra' must be available
if (requireNamespace("ggpubr", quietly = TRUE) &
    requireNamespace("gridExtra", quietly = TRUE)) {
  plot(pp1, xlim = c(-4, 4), legend = TRUE)
}
}

settings.meta(oldset)

}
\references{
Siemens W, Borenstein M, Evrenoglou T, Meerpohl JJ, Schwarzer G (2025):
Beyond prediction intervals in meta-analysis: reporting the expected
proportion of comparable studies with clinically relevant benefit or harm.
\emph{BMC Medical Research Methodology},
\bold{25}, 275
}
\seealso{
\code{\link{cidprop}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimfill.R
\name{trimfill.meta}
\alias{trimfill.meta}
\alias{trimfill}
\alias{trimfill.default}
\title{Trim-and-fill method to adjust for bias in meta-analysis}
\usage{
\method{trimfill}{meta}(
  x,
  left = NULL,
  ma.common = TRUE,
  type = "L",
  n.iter.max = 50,
  common = FALSE,
  random = TRUE,
  prediction = x$prediction,
  backtransf = x$backtransf,
  pscale = x$pscale,
  irscale = x$irscale,
  irunit = x$irunit,
  silent = TRUE,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

\method{trimfill}{default}(
  x,
  seTE,
  left = NULL,
  ma.common = TRUE,
  type = "L",
  n.iter.max = 50,
  sm = "",
  studlab = NULL,
  level = 0.95,
  level.ma = level,
  common = FALSE,
  random = TRUE,
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  method.tau = gs("method.tau"),
  method.tau.ci = if (method.tau == "DL") "J" else "QP",
  level.hetstat = gs("level.hetstat"),
  prediction = FALSE,
  level.predict = level,
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  backtransf = TRUE,
  pscale = 1,
  irscale = 1,
  irunit = "person-years",
  silent = TRUE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}, or estimated treatment
effect in individual studies.}

\item{left}{A logical indicating whether studies are supposed to be
missing on the left or right side of the funnel plot. If NULL,
the linear regression test for funnel plot symmetry (i.e.,
function \code{metabias(..., method="Egger")}) is used to
determine whether studies are missing on the left or right side.}

\item{ma.common}{A logical indicating whether a common effect or
random effects model is used to estimate the number of missing
studies.}

\item{type}{A character indicating which method is used to estimate
the number of missing studies. Either \code{"L"} or \code{"R"}.}

\item{n.iter.max}{Maximum number of iterations to estimate number
of missing studies.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If
\code{backtransf=TRUE}, results for \code{sm="OR"} are printed as
odds ratios rather than log odds ratios and results for
\code{sm="ZCOR"} are printed as correlations rather than Fisher's
z transformed correlations, for example.}

\item{pscale}{A numeric giving scaling factor for printing of
single event probabilities or risk differences, i.e. if argument
\code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
\code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.}

\item{irscale}{A numeric defining a scaling factor for printing of
single incidence rates or incidence rate differences, i.e. if
argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
\code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.}

\item{irunit}{A character specifying the time unit used to
calculate rates, e.g. person-years.}

\item{silent}{A logical indicating whether basic information on
iterations shown.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}

\item{seTE}{Standard error of estimated treatment effect.}

\item{sm}{An optional character string indicating underlying
summary measure, e.g., \code{"RD"}, \code{"RR"}, \code{"OR"},
\code{"ASD"}, \code{"HR"}, \code{"MD"}, \code{"SMD"}, or
\code{"ROM"}; ignored if \code{x} is of class \code{meta}.}

\item{studlab}{An optional vector with study labels; ignored if
\code{x} is of class \code{meta}.}

\item{level}{The level used to calculate confidence intervals for
individual studies. If existing, \code{x$level} is used as value
for \code{level}; otherwise 0.95 is used.}

\item{level.ma}{The level used to calculate confidence interval for
the pooled estimate. If existing, \code{x$level.ma} is used as
value for \code{level.ma}; otherwise 0.95 is used.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for the
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}
}
\value{
An object of class \code{c("trimfill", "metagen", "meta")} with
corresponding generic functions (see \code{\link{meta-object}}).
}
\description{
Trim-and-fill method for estimating and adjusting for the number
and outcomes of missing studies in a meta-analysis.
}
\details{
The trim-and-fill method (Duval, Tweedie 2000a, 2000b) can be used
for estimating and adjusting for the number and outcomes of missing
studies in a meta-analysis. The method relies on scrutiny of one
side of a funnel plot for asymmetry assumed due to publication
bias.

Three different methods have been proposed originally to estimate
the number of missing studies. Two of these methods (L- and
R-estimator) have been shown to perform better in simulations, and
are available in this R function (argument \code{type}).

A common effect or random effects model can be used to estimate the
number of missing studies (argument \code{ma.common}). Furthermore,
a common effect and/or random effects model can be used to
summaries study results (arguments \code{common} and
\code{random}). Simulation results (Peters et al. 2007) indicate
that the common-random model, i.e. using a common effect model to
estimate the number of missing studies and a random effects model
to summaries results, (i) performs better than the common-common
model, and (ii) performs no worse than and marginally better in
certain situations than the random-random model. Accordingly, the
common-random model is the default.

An empirical comparison of the trim-and-fill method and the Copas
selection model (Schwarzer et al. 2010) indicates that the
trim-and-fill method leads to excessively conservative inference in
practice. The Copas selection model is available in R package
\bold{metasens}.

The function \code{\link{metagen}} is called internally.
}
\examples{
data(Fleiss1993bin)
ma <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin, sm = "OR")
tf <- trimfill(ma)
tf
funnel(tf)
funnel(tf, pch = ifelse(tf$trimfill, 1, 16), level = 0.9, random = FALSE)
#
# Use log odds ratios on x-axis
#
funnel(tf, backtransf = FALSE)
funnel(tf, pch = ifelse(tf$trimfill, 1, 16), level = 0.9, random = FALSE,
  backtransf = FALSE)

trimfill(ma$TE, ma$seTE, sm = ma$sm)

}
\references{
Duval S & Tweedie R (2000a):
A nonparametric "Trim and Fill" method of accounting for
publication bias in meta-analysis.
\emph{Journal of the American Statistical Association},
\bold{95}, 89--98

Duval S & Tweedie R (2000b):
Trim and Fill: A simple funnel-plot-based method of testing and
adjusting for publication bias in meta-analysis.
\emph{Biometrics},
\bold{56}, 455--63

Peters JL, Sutton AJ, Jones DR, Abrams KR, Rushton L (2007):
Performance of the trim and fill method in the presence of
publication bias and between-study heterogeneity.
\emph{Statisics in Medicine},
\bold{10}, 4544--62

Schwarzer G, Carpenter J, Rücker G (2010):
Empirical evaluation suggests Copas selection model preferable to
trim-and-fill method for selection bias in meta-analysis
\emph{Journal of Clinical Epidemiology},
\bold{63}, 282--8
}
\seealso{
\code{\link{metagen}}, \code{\link{metabias}},
  \code{\link{funnel}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset.pairwise.R
\name{subset.pairwise}
\alias{subset.pairwise}
\title{Return subset of pairwise object}
\usage{
\method{subset}{pairwise}(x, subset, ...)
}
\arguments{
\item{x}{An object of class \code{pairwise}.}

\item{subset}{A logical expression indicating rows to keep; missing values
are taken as false.}

\item{\dots}{Additional arguments.}
}
\value{
A pairwise object is returned.
}
\description{
The \code{subset} method returns a subset of a pairwise object.
}
\examples{
# Transform data from arm-based format to contrast-based format
pw1 <- pairwise(list(Treatment1, Treatment2, Treatment3),
  n = list(n1, n2, n3),
  mean = list(y1, y2, y3), sd = list(sd1, sd2, sd3),
  data = Franchini2012, studlab = Study)
# Subset without Lieberman studies
subset(pw1, !grepl("Lieberman", studlab))[, 1:5]

}
\seealso{
\code{\link{pairwise}}, \code{\link[metabook]{Franchini2012}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/traffic_light.R
\name{traffic_light}
\alias{traffic_light}
\title{Produce traffic light plot of risk of bias assessment}
\usage{
traffic_light(object, colour = "cochrane", psize = 15, quiet = FALSE)
}
\arguments{
\item{object}{An object of class \code{rob}.}

\item{colour}{Specify colour scheme for the traffic light plot; see
\code{\link[robvis]{rob_summary}}.}

\item{psize}{Size of the traffic lights.}

\item{quiet}{A logical to suppress the display of the traffic light
plot.}
}
\description{
Produce traffic light plot of risk of bias assessment
}
\details{
This is a wrapper function for
\code{\link[robvis]{rob_traffic_light}} of R package \bold{robvis}
to produce a traffic light plot of risk of bias assessment.
}
\examples{
# Use RevMan 5 settings
oldset <- settings.meta("RevMan5")

data(caffeine)

ma1 <- metabin(h.caf, n.caf, h.decaf, n.decaf, sm = "OR",
  data = caffeine, studlab = paste(study, year))

# Add risk of bias assessment to meta-analysis
ma2 <- rob(D1, D2, D3, D4, D5, overall = rob, data = ma1, tool = "rob2")

# Print risk of bias assessment
rob(ma2)

\dontrun{
# Traffic light plot (R package 'robvis' must be available)
if (requireNamespace("robvis", quietly = TRUE))
 traffic_light(rob(ma2))
}

# Use previous settings
settings.meta(oldset)

}
\seealso{
\code{\link{rob}}, \code{\link{barplot.rob}},
  \code{\link[robvis]{rob_traffic_light}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.cdir.R
\name{read.cdir}
\alias{read.cdir}
\alias{print.cdir}
\title{Import data of Cochrane intervention review}
\usage{
read.cdir(
  file,
  title = "Cochrane Review of Interventions",
  exdir = tempdir(),
  numbers.in.labels = TRUE,
  rob = !missing(tool) | !missing(categories) | !missing(col) | !missing(symbols),
  tool = NULL,
  categories = NULL,
  col = NULL,
  symbols = NULL,
  keep.orig = FALSE,
  ...
)

\method{print}{cdir}(x, ...)
}
\arguments{
\item{file}{The name of a file to read data values from.}

\item{title}{Title of Cochrane review.}

\item{exdir}{The directory to extract files to (the equivalent of
‘unzip -d’). It will be created if necessary.}

\item{numbers.in.labels}{A logical indicating whether comparison
number and outcome number should be printed at the beginning of
the comparison (argument \code{complab}) and outcome label
(argument \code{outclab}); this is the default in RevMan Web.}

\item{rob}{A logical indicating whether risk of bias (RoB)
assessment should be considered in meta-analyses.}

\item{tool}{Risk of bias (RoB) tool.}

\item{categories}{Possible RoB categories.}

\item{col}{Colours for RoB categories.}

\item{symbols}{Corresponding symbols for RoB categories.}

\item{keep.orig}{A logical indicating whether to return the
original data files.}

\item{\dots}{Additional arguments (passed on to
\code{\link[utils]{unzip}})}

\item{x}{An object of class "cdir".}
}
\value{
A list consisting of a data frame 'data' with the study data and
(if available) a data frame 'rob' with information on the risk of
bias assessment. If \code{keep.orig = TRUE}, an additional list
'orig' is returned containing elements 'settings', 'datarows',
'subgroup' and 'rob' (if available).

The data frame 'data' contains the following variables:

\item{comp.no}{Comparison number.}
\item{outcome.no}{Outcome number.}
\item{group.no}{Group number.}
\item{studlab}{Study label.}
\item{year}{Year of publication.}
\item{event.e}{Number of events in experimental group.}
\item{n.e}{Number of observations in experimental group.}
\item{event.c}{Number of events in control group.}
\item{n.c}{Number of observations in control group.}
\item{mean.e}{Estimated mean in experimental group.}
\item{sd.e}{Standard deviation in experimental group.}
\item{mean.c}{Estimated mean in control group.}
\item{sd.c}{Standard deviation in control group.}
\item{O.E}{Observed minus expected (IPD analysis).}
\item{V}{Variance of \code{O.E} (IPD analysis).}
\item{TE, seTE}{Estimated treatment effect and standard error of
  individual studies.}
\item{lower, upper}{Lower and upper limit of 95\% confidence
  interval for treatment effect in individual studies.}
\item{weight}{Weight of individual studies (according to
  meta-analytical method used in respective meta-analysis - see
  details).}
\item{order}{Ordering of studies.}
\item{grplab}{Group label.}
\item{type}{Type of outcome. D = dichotomous, C = continuous, P =
  IPD.}
\item{method}{A character string indicating which method has been
  used for pooling of studies. One of \code{"Inverse"},
  \code{"MH"}, or \code{"Peto"}.}
\item{sm}{A character string indicating which summary measure has
  been used for pooling of studies.}
\item{model}{A character string indicating which meta-analytical
  model has been used (either \code{"Fixed"} or \code{"Random"}).}
\item{common}{A logical indicating whether common effect
  meta-analysis has been used in respective meta-analysis (see
  details).}
\item{random}{A logical indicating whether random effects
  meta-analysis has been used in respective meta-analysis (see
  details).}
\item{title}{Title of Cochrane review.}
\item{complab}{Comparison label.}
\item{outclab}{Outcome label.}
\item{label.e}{Label for experimental group.}
\item{label.c}{Label for control group.}
\item{label.left}{Graph label on left side of forest plot.}
\item{label.right}{Graph label on right side of forest plot.}

The data frame 'rob' contains the following variables:

\item{studlab}{Study label.}
\item{D1, D2, \dots}{Risk of bias domain 1, 2, \dots}
\item{D1.details, D2.details, \dots}{Details on risk of bias domain
  1, 2, \dots}
}
\description{
Reads Cochrane data package (version 1) of a Cochrane intervention
review and creates a data frame from it.
}
\details{
RevMan Web is the current software used for preparing and maintaining
Cochrane reviews. RevMan Web includes the ability to write systematic
reviews of interventions or diagnostic test accuracy reviews.

This function provides the ability to read the Cochrane data
package from a Cochrane intervention review created with RevMan
Web. The ZIP-file is extracted with \code{\link[utils]{unzip}}.

Argument \code{title} can be used to overwrite the title of the
Cochrane review.

Information on the risk of bias (RoB) assessment can be provided
with arguments \code{tool}, \code{categories}, \code{col} and
\code{symbols}. This is only useful if (i) all outcomes are based
on the same RoB categories and (ii) an overall RoB assessment has
not been done. If no overall RoB assessment was conducted, R
function \code{\link{metacr}} can be used to provide the RoB
information for a single outcome. R function \code{\link{rob}} is
the most flexible way to add RoB information to a meta-analysis
object.

\subsection{Creation of Cochrane data package}{

Two possible ways exist to create the ZIP-file.

In RevMan Web, press the "Export" button at the bottom of the
\emph{Default view} website. After a couple of seconds, the data package will
be shown at the bottom of the \emph{Default view} website under "Downloads".

In the Cochrane Library, press on "Download statistical data" in the
Contents menu to download an rm5-file. This file can be converted to a data
package in RevMan Web using \emph{Help} - \emph{Convert a RevMan 5 file}.
}
}
\examples{
# Locate file "Fleiss1993.zip" with Cochrane data package in
# sub-directory of R package meta
#
filename <- system.file("extdata/Fleiss1993.zip", package = "meta")
Fleiss1993_CR <- read.cdir(filename)
Fleiss1993_CR

# Same result as R Command example(Fleiss1993bin):
#
metacr(Fleiss1993_CR)

# Same result as R Command example(Fleiss1993cont):
#
metacr(Fleiss1993_CR, 1, 2)

}
\references{
\url{https://documentation.cochrane.org/revman-kb/data-package-user-guide-243761660.html}

\url{https://documentation.cochrane.org/revman-kb/data-package-specification-249561249.html}
}
\seealso{
\code{\link{metacr}}, \code{\link{rob}},
  \code{\link{read.rm5}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metamerge.R
\name{metamerge}
\alias{metamerge}
\title{Merge results of two meta-analyses on the same data set}
\usage{
metamerge(
  meta1,
  meta2,
  common1 = meta1$common,
  random1 = meta1$random,
  prediction1 = meta1$prediction,
  common2 = meta2$common,
  random2 = meta2$random,
  prediction2 = meta2$prediction,
  label1 = NULL,
  label2 = NULL,
  label1.common = label1,
  label2.common = label2,
  label1.random = label1,
  label2.random = label2,
  label1.predict = label1,
  label2.predict = label2,
  label1.subgroup = label1,
  label2.subgroup = label2,
  hetlabel1 = label1.random,
  hetlabel2 = label2.random,
  taulabel1 = label1.random,
  taulabel2 = label2.random,
  text.pooled1 = NULL,
  text.pooled2 = NULL,
  text.w.pooled1 = NULL,
  text.w.pooled2 = NULL,
  text.common1 = text.pooled1,
  text.common2 = text.pooled2,
  text.random1 = text.pooled1,
  text.random2 = text.pooled2,
  text.predict1 = text.pooled1,
  text.predict2 = text.pooled2,
  text.w.common1 = text.w.pooled1,
  text.w.common2 = text.w.pooled2,
  text.w.random1 = text.w.pooled1,
  text.w.random2 = text.w.pooled2,
  keep = FALSE,
  keep.Q = keep,
  keep.I2 = keep.Q,
  keep.w = keep,
  common = common1 | common2,
  random = random1 | random2,
  overall = common | random,
  overall.hetstat = common | random,
  prediction = prediction1 | prediction2,
  backtransf,
  warn.deprecated = gs("warn.deprecated"),
  pooled1,
  pooled2
)
}
\arguments{
\item{meta1}{First meta-analysis object (see Details).}

\item{meta2}{Second meta-analysis object (see Details).}

\item{common1}{A logical indicating whether results of common
effect model should be considered for first meta-analysis.}

\item{random1}{A logical indicating whether results of random
effects model should be considered for first meta-analysis.}

\item{prediction1}{A logical indicating whether prediction interval
should be considered for first meta-analysis.}

\item{common2}{A logical indicating whether results of common
effect model should be considered for second meta-analysis.}

\item{random2}{A logical indicating whether results of random
effects model should be considered for second meta-analysis.}

\item{prediction2}{A logical indicating whether prediction interval
should be considered for second meta-analysis.}

\item{label1}{Default setting for arguments 'label1.common',
'label1.random', 'label1.predict' and 'label1.subgroup'.}

\item{label2}{Default setting for arguments 'label2.common',
'label2.random', 'label2.predict' and 'label2.subgroup'.}

\item{label1.common}{A character string to label the common effect estimate
from the first meta-analysis.}

\item{label2.common}{A character string to label the common effect estimate
from the second meta-analysis.}

\item{label1.random}{A character string to label the random effects estimate
from the first meta-analysis (default label for arguments 'hetlabel1' and
'taulabel1').}

\item{label2.random}{A character string to label the random effects estimate
from the second meta-analysis (default label for arguments 'hetlabel2' and
'taulabel2').}

\item{label1.predict}{A character string to label the prediction interval
from the first meta-analysis.}

\item{label2.predict}{A character string to label the prediction interval
from the second meta-analysis.}

\item{label1.subgroup}{A character string to label the subgroup results
from the first meta-analysis.}

\item{label2.subgroup}{A character string to label the subgroup results
from the second meta-analysis.}

\item{hetlabel1}{A character string used to label heterogeneity
statistics of the first meta-analysis.}

\item{hetlabel2}{A character string used to label heterogeneity
statistics of the second meta-analysis.}

\item{taulabel1}{A character string used to label estimate of
between-study variance of the first meta-analysis.}

\item{taulabel2}{A character string used to label estimate of
between-study variance of the second meta-analysis.}

\item{text.pooled1}{A character string used in printouts and forest
plot to label the results from the first meta-analysis.}

\item{text.pooled2}{A character string used in printouts and forest
plot to label the results from the second meta-analysis.}

\item{text.w.pooled1}{A character string used to label weights of
the first meta-analysis; can be of same length as the number of
pooled estimates requested in argument \code{pooled1}.}

\item{text.w.pooled2}{A character string used to label weights of
the second meta-analysis; can be of same length as the number of
pooled estimates requested in argument \code{pooled1}.}

\item{text.common1}{A character string used in printouts and forest
plot to label results for common effect models from the first
meta-analysis.}

\item{text.common2}{A character string used in printouts and forest
plot to label results for common effect models from the second
meta-analysis.}

\item{text.random1}{A character string used in printouts and forest
plot to label results for random effects models from the first
meta-analysis.}

\item{text.random2}{A character string used in printouts and forest
plot to label results for random effects models from the second
meta-analysis.}

\item{text.predict1}{A character string used in printouts and
forest plot to label prediction interval from the first
meta-analysis.}

\item{text.predict2}{A character string used in printouts and
forest plot to label prediction interval from the second
meta-analysis.}

\item{text.w.common1}{A character string used to label common
effect weights of the first meta-analysis; can be of same length
as the number of common effect estimates.}

\item{text.w.common2}{A character string used to label common
effect weights of the second meta-analysis; can be of same length
as the number of common effect estimates.}

\item{text.w.random1}{A character string used to label random
effects weights of the first meta-analysis; can be of same length
as the number of random effects estimates.}

\item{text.w.random2}{A character string used to label random
effects weights of the second meta-analysis; can be of same
length as the number of random effects estimates.}

\item{keep}{A logical indicating whether to keep additional
information from second meta-analysis.}

\item{keep.Q}{A logical indicating whether heterogeneity statistic
Q of second meta-analysis should be kept or ignored.}

\item{keep.I2}{A logical indicating whether heterogeneity statistic
I2 of second meta-analysis should be kept or ignored.}

\item{keep.w}{A logical indicating whether weights of the second
meta-analysis should be kept or ignored.}

\item{common}{A logical indicating whether results of common effect
meta-analyses should be reported.}

\item{random}{A logical indicating whether results of random
effects meta-analyses should be reported.}

\item{overall}{A logical indicating whether overall summaries
should be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons.}

\item{prediction}{A logical indicating whether prediction intervals
should be reported.}

\item{backtransf}{A logical indicating whether results should be
back transformed in printouts and plots. If
\code{backtransf=TRUE} (default), results for \code{sm="OR"} are
printed as odds ratios rather than log odds ratios, for example.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{pooled1}{Deprecated argument (replaced by 'common1',
'random1', 'prediction1'). A character string indicating whether
results of common effect or random effects model should be
considered for first meta-analysis. Either \code{"both"},
\code{"common"} or \code{"random"}, can be abbreviated.}

\item{pooled2}{Deprecated argument (replaced by 'common2',
'random2', 'prediction2'). A character string indicating whether
results of common effect or random effects model should be
considered for second meta-analysis. Either \code{"both"},
\code{"common"} or \code{"random"}, can be abbreviated.}
}
\value{
An object of class \code{"meta"} and \code{"metamerge"} with
corresponding generic functions (see \code{\link{meta-object}}).

The following list elements have a different meaning:
\item{TE, seTE, studlab}{Treatment estimate, standard error, and
  study labels (first meta-analysis).}
\item{lower, upper}{Lower and upper confidence interval limits for
  individual studies (first meta-analysis).}
\item{statistic, pval}{Statistic and p-value for test of treatment
  effect for individual studies (first meta-analysis.}
\item{w.common}{Vector or matrix with common effect weights.}
\item{w.random}{Vector or matrix with random effects weights.}
\item{k}{Vector with number of estimates (same length as number of
  common effect and random effects estimates).}
\item{k.study}{Vector with number of studies (same length as
  number of common effect and random effects estimates).}
\item{k.all}{Vector with total number of studies (same length as
  number of common effect and random effects estimates).}
\item{k.TE}{Vector with number of studies with estimable effects
  (same length as number of common effect and random effects
  estimates).}
\item{k.MH}{Vector with number of studies combined with
  Mantel-Haenszel method (same length as number of common effect
  and random effects estimates).}
\item{TE.common}{Vector with common effect estimates.}
\item{seTE.common}{Vector with standard errors of common effect
  estimates.}
\item{lower.common}{Vector with lower confidence limits (common
  effect model).}
\item{upper.common}{Vector with upper confidence limits (common
  effect model).}
\item{statistic.common}{Vector with test statistics for test of
  overall effect (common effect model).}
\item{pval.common}{Vector with p-value of test for overall effect
  (common effect model).}
\item{TE.random}{Vector with random effects estimates.}
\item{seTE.random}{Vector with standard errors of random effects
  estimates.}
\item{lower.random}{Vector with lower confidence limits (random
  effects model).}
\item{upper.random}{Vector with upper confidence limits (random
  effects model).}
\item{statistic.random}{Vector with test statistics for test of
  overall effect (random effects model).}
\item{pval.random}{Vector with p-value of test for overall effect
  (random effects model).}

Furthermore, meta-analysis results of common effect or random
effects model are taken from first meta-analysis if only random
effects or common effects models are selected from both
meta-analyses (arguments \code{pooled1} and \code{pooled2}).
}
\description{
This function can be used to merge results of two meta-analyses
into a single meta-analysis object if they are based on the same
data set. This is, for example, useful to produce a forest plot of
a random-effects meta-analysis with different estimates of the
between-study variance \eqn{\tau^2}.
}
\details{
In R package \bold{meta}, objects of class \code{"meta"} contain
results of both common effect and random effects
meta-analyses. This function enables the user to merge the results
of two meta-analysis object if they are based on the same data set.

Applications of this function include printing and plotting results
of the common effect or random effects meta-analysis and the
\itemize{
\item trim-and-fill method (\code{\link{trimfill}}),
\item limit meta-analyis (\code{\link[metasens]{limitmeta}} from R
  package \bold{metasens}),
\item Copas selection model (\code{\link[metasens]{copas}} from R
  package \bold{metasens}),
\item robust variance meta-analysis model
  (\code{\link[robumeta]{robu}} from R package \bold{robumeta}).
}

The first argument (\code{meta1}) must be an object created by a
meta-analysis function (see \code{\link{meta-object}}). If an
object created with \code{\link[metasens]{limitmeta}} or
\code{\link[metasens]{copas}} is provided as the first argument,
this object will be returned, i.e., argument \code{meta2} will be
ignored.

The second meta-analysis could be an object created by a
meta-analysis function or with \code{\link{trimfill}},
\code{\link[metasens]{limitmeta}}, \code{\link[metasens]{copas}},
or \code{\link[robumeta]{robu}}.

The created meta-analysis object only contains the study results,
i.e., estimated effects and confidence intervals, from the first
meta-analysis which are shown in printouts and forest plots. This
only makes a difference for meta-analysis methods where individual
study results differ, e.g., Mantel-Haenszel and Peto method for
binary outcomes (see \code{\link{metabin}}).

R function \code{\link{metaadd}} can be used to add pooled results
from any (external) meta-analysis.

R function \code{\link{metabind}} can be used to print and plot the
results of several meta-analyses without the restriction that the
same data set has to be used. Accordingly, individual study results
are ignored.
}
\examples{
# Print results with more significant digits and do not show confidence
# intervals for tau^2 and tau
oldset <- settings.meta(digits = 6, digits.stat = 4, digits.pval = 6,
  digits.Q = 6, digits.I2 = 4, digits.H = 4,
  print.tau2.ci = FALSE, print.tau.ci = FALSE)
oldopts <- options(width = 120)

data(Fleiss1993bin)

# Mantel-Haenszel method
ma1 <- metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
  studlab = paste(study, year), sm = "OR")
# Peto method
ma2 <- update(ma1, method = "Peto")
# Inverse variance method (only common effect model)
ma3 <- update(ma2, method = "Inverse", random = FALSE)

# Merge results from MH and Peto method
# - show individual results for MH method
#   (as this is the first meta-analysis)
# - keep all additional information from Peto meta-analysis (i.e.,
#   weights, Q statistic and I2 statistic)
ma12 <- metamerge(ma1, ma2,
  label1 = "REML", label2 = "REML-Peto",
  label1.common = "MH", label2.common = "Peto", 
  text.common1 = "Mantel-Haenszel method",
  text.common2 = "Peto method",
  text.w.random1 = "REML", text.w.random2 = "REML-Peto",
  hetlabel1 = "MH/IV", hetlabel2 = "Peto",
  keep = TRUE)

# Add common effect results from inverse variance method
# - keep weights from IV meta-analysis
# - Q and I2 statistic are identical for sm = "MH" and sm = "Inverse"
#   as inverse variance method is used for sm = "MH" under random
#   effects model
ma123 <- metamerge(ma12, ma3,
  label2 = "IV",
  text.common2 = "Inverse variance method",
  keep.w = TRUE)
summary(ma123)
\dontrun{
forest(ma123, digits = 6)

# Merge results (show individual results for Peto method)
ma21 <- metamerge(ma2, ma1,
  label1 = "REML-Peto", label2 = "REML",
  label1.common = "Peto", label2.common = "MH", 
  hetlabel1 = "Peto", hetlabel2 = "MH/IV",
  text.common1 = "Peto method",
  text.common2 = "Mantel-Haenszel method",
  keep = TRUE)

# Add results from inverse variance method
# - keep weights from IV meta-analysis
# - Q and I2 statistic are identical for sm = "MH" and sm = "Inverse"
#   as inverse variance method is used for sm = "MH" under random
#   effects model
ma213 <- metamerge(ma21, ma3,
  label2 = "IV",
  text.common2 = "Inverse variance method",
  keep.w = TRUE)
summary(ma213)

# Random effects method using ML estimator for between-study variance tau2
ma4 <- update(ma1, common = FALSE, method.tau = "ML")

# Use DerSimonian-Laird estimator for tau2
ma5 <- update(ma4, method.tau = "DL")

# Use Paule-Mandel estimator for tau2
ma6 <- update(ma4, method.tau = "PM")

# Merge random effects results for ML and DL estimators
# - keep weights for DL estimator (which are different from ML)
ma45 <- metamerge(ma4, ma5, label1 = "ML", label2 = "DL",
  text.w.random1 = "RE-ML", text.w.random2 = "RE-DL", keep.w = TRUE)
summary(ma45)

# Add results for PM estimator
# - keep weights
ma456 <- metamerge(ma45, ma6, label2 = "PM",
  text.w.random2 = "RE-PM", keep.w = TRUE)
summary(ma456)

ma123456 <- metamerge(ma123, ma456)
ma123456

# Use Hartung-Knapp confidence intervals
# - do not keep information on Q, I2 and weights
ma7 <- update(ma4, method.random.ci = "HK",
  text.random = "Hartung-Knapp method")
ma8 <- update(ma5, method.random.ci = "HK",
  text.random = "Hartung-Knapp method")
ma9 <- update(ma6, method.random.ci = "HK",
  text.random = "Hartung-Knapp method")

# Merge results for Hartung-Knapp method (with REML and DL estimator)
# - RE weights for REML estimator are shown
ma78 <- metamerge(ma7, ma8, label1 = "ML", label2 = "DL")
summary(ma78)

ma789 <- metamerge(ma78, ma9, label2 = "PM")
summary(ma789)

# Merge everything
ma1to9 <- metamerge(metamerge(ma123, ma456, keep.w = TRUE), ma789)
summary(ma1to9)

ma10 <- update(ma1, method = "GLMM")

ma.all <- metamerge(ma1to9, ma10, keep.Q = TRUE,
  label2 = "GLMM", taulabel2 = "ML-GLMM")
summary(ma.all)

forest(ma.all, layout = "JAMA")
forest(ma.all, details = TRUE)
}

settings.meta(oldset)
options(oldopts)

}
\seealso{
\code{\link{metagen}}, \code{\link{metabind}},
  \code{\link{metaadd}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta-transf.R
\name{meta-transf}
\alias{meta-transf}
\alias{transf}
\alias{cor2z}
\alias{p2asin}
\alias{p2logit}
\alias{VE2logVR}
\alias{backtransf}
\alias{asin2ir}
\alias{asin2p}
\alias{logit2p}
\alias{logVR2VE}
\alias{z2cor}
\title{Auxiliary functions for (back) transformations}
\usage{
transf(x, sm, func = NULL, args = NULL)

cor2z(x)

p2asin(x)

p2logit(x)

VE2logVR(x)

backtransf(x, sm, n, time, func = NULL, args = NULL)

asin2ir(x, time = NULL)

asin2p(x, n = NULL)

logit2p(x)

logVR2VE(x)

z2cor(x)
}
\arguments{
\item{x}{Numerical vector with effect estimates, lower or upper
confidence / prediction interval limit(s).}

\item{sm}{Summary measure.}

\item{func}{User-specified function for (back) transformation.}

\item{args}{Function arguments for user-specified function.}

\item{n}{Sample size(s) to back transform Freeman-Tukey transformed
proportions.}

\item{time}{Time(s) to back transform Freeman-Tukey transformed
incidence rates.}
}
\description{
Auxiliary functions to (back) transform effect estimates or
confidence / prediction interval limit(s).
}
\details{
Often in a meta-analysis, effect estimates are transformed before
calculating a weighted average. For example, the log odds ratio and
its standard error is used instead of the odds ratio in R function
\code{\link{metagen}}. To report the results of a meta-analysis,
effect estimates are typically back transformed to the original
scale. R package \bold{meta} provides some auxiliary functions for
(back) transformations.

\subsection{Transformations}{
The following auxiliary functions are provided by R package
\bold{meta} to transform effect estimates or confidence /
prediction interval limits.

\tabular{ll}{
\bold{Function} \tab \bold{Transformation} \cr
\code{cor2z} \tab Correlations to Fisher's Z transformed correlations \cr
\code{p2logit} \tab Proportions to logit transformed proportions \cr
\code{p2asin} \tab Proportions to arcsine transformed proportions \cr
\code{VE2logVR} \tab Vaccine efficacy / effectiveness to log vaccine ratio
}

Note, no function for the Freeman-Tukey arcsine transformation is
provided as this transformation is based on the number of events
and sample sizes instead of the effect estimates.

R function \code{transf} is a wrapper function for the above and
additional transformations, e.g., the log transformation using
\code{\link[base]{log}} for odds or risk ratios. Argument \code{sm}
is mandatory to specify the requested transformation. It is also
possible to specify a different function with arguments \code{func}
and \code{args}.
}

\subsection{Back transformations}{
The following auxiliary functions are available to back transform
effect estimates or confidence / prediction interval limits.

\tabular{ll}{
\bold{Function} \tab \bold{Transformation} \cr
\code{asin2ir} \tab Freeman-Tukey arcsine transformed rates to rates \cr
\code{asin2p} \tab (Freeman-Tukey) arcsine transformed proportions to proportions \cr
\code{logit2p} \tab Logit transformed proportions to proportions \cr
\code{logVR2VE} \tab Log vaccine ratio to vaccine efficacy / effectiveness \cr
\code{z2cor} \tab Fisher's Z transformed correlations  to correlations
}

Argument \code{time} is mandatory in R function \code{asin2ir}.

If argument \code{n} is provided in R function \code{asin2p},
Freeman-Tukey arcsine transformed proportions are
back transformed. Otherwise, arcsine transformed proportions are
back transformed.

R function \code{backtransf} is a wrapper function for the above
and additional transformations, e.g., the exponential
transformation using \code{\link[base]{exp}} for log odds or log
risk ratios. Argument \code{sm} is mandatory to specify the
requested transformation. For the Freeman-Tukey transformations,
argument \code{n} or \code{time} is mandatory.

It is also possible to specify a different function with arguments
\code{func} and \code{args}.
}
}
\examples{
logit2p(p2logit(0.5))

}
\seealso{
\code{\link{meta-sm}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bubble.R
\name{bubble.metareg}
\alias{bubble.metareg}
\alias{bubble}
\title{Bubble plot to display the result of a meta-regression}
\usage{
\method{bubble}{metareg}(
  x,
  xlim,
  ylim,
  xlab,
  ylab,
  cex,
  min.cex = 0.5,
  max.cex = 5,
  pch = 21,
  col = "black",
  bg = "darkgray",
  lty = 1,
  lwd = 1,
  col.line = "black",
  studlab = FALSE,
  cex.studlab = 0.8,
  pos.studlab = 2,
  offset = 0.5,
  regline = TRUE,
  backtransf = x$.meta$x$backtransf,
  ref,
  col.ref = "lightgray",
  lty.ref = 1,
  lwd.ref = 1,
  pscale = x$.meta$x$pscale,
  irscale = x$.meta$x$irscale,
  axes = TRUE,
  box = TRUE,
  ...
)

bubble(x, ...)
}
\arguments{
\item{x}{An object of class \code{metareg}.}

\item{xlim}{The x limits (min,max) of the plot.}

\item{ylim}{The y limits (min,max) of the plot.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{cex}{The magnification to be used for plotting symbols.}

\item{min.cex}{Minimal magnification for plotting symbols.}

\item{max.cex}{Maximal magnification for plotting symbols.}

\item{pch}{The plotting symbol(s) used for individual studies.}

\item{col}{A vector with colour of plotting symbols.}

\item{bg}{A vector with background colour of plotting symbols (only
used if \code{pch} in \code{21:25}).}

\item{lty}{The line type for the meta-regression line.}

\item{lwd}{The line width for the meta-regression line.}

\item{col.line}{Colour for the meta-regression line.}

\item{studlab}{A logical indicating whether study labels should be
printed in the graph. A vector with study labels can also be
provided (must be of same length as the numer of studies in the
meta-analysis then).}

\item{cex.studlab}{The magnification for study labels.}

\item{pos.studlab}{Position of study labels, see argument
\code{pos} in \code{\link{text}}.}

\item{offset}{Offset for study labels (see \code{\link{text}}).}

\item{regline}{A logical indicating whether a regression line
should be added to the bubble plot.}

\item{backtransf}{A logical indicating whether results for relative
summary measures (argument \code{sm} equal to \code{"OR"},
\code{"RR"}, \code{"HR"}, or \code{"IRR"}) should be back
transformed. If \code{backtransf=TRUE}, results for
\code{sm="OR"} are printed as odds ratios rather than log odds
ratios, for example.}

\item{ref}{A numerical giving the reference value to be plotted as
a line in the bubble plot. No reference line is plotted if
argument \code{ref} is equal to \code{NA}.}

\item{col.ref}{Colour of the reference line.}

\item{lty.ref}{The line type for the reference line.}

\item{lwd.ref}{The line width for the reference line.}

\item{pscale}{A numeric giving scaling factor for printing of
probabilities.}

\item{irscale}{A numeric defining a scaling factor for printing of
incidence rates.}

\item{axes}{Either a logical or a character string equal to \code{"x"},
\code{"y"} or \code{"xy"} indicating whether x- and y-axis should be
printed.}

\item{box}{A logical indicating whether a box should be printed.}

\item{\dots}{Graphical arguments as in \code{par} may also be
passed as arguments.}
}
\description{
Draw a bubble plot to display the result of a meta-regression.
}
\details{
A bubble plot can be used to display the result of a
meta-regression. It is a scatter plot with the treatment effect for
each study on the y-axis and the covariate used in the
meta-regression on the x-axis. Typically, the size of the plotting
symbol is inversely proportional to the variance of the estimated
treatment effect (Thompson & Higgins, 2002).

Argument \code{cex} specifies the plotting size for each individual
study. If this argument is missing the weights from the
meta-regression model will be used (which typically is a random
effects model). Use \code{cex="common"} in order to utilise weights
from a common effect model to define the size of the plotted
symbols (even for a random effects meta-regression). If a vector
with individual study weights is provided, the length of this
vector must be of the same length as the number of studies.

Arguments \code{min.cex} and \code{max.cex} can be used to define
the size of the smallest and largest plotting symbol. The plotting
size of the most precise study is set to \code{max.cex} whereas the
plotting size of all studies with a plotting size smaller than
\code{min.cex} will be set to \code{min.cex}.

For a meta-regression with more than one covariate. Only a scatter plot of
the first covariate in the regression model is shown. In this case the
effect of the first covariate adjusted for other covariates in the
meta-regression model is shown.

For a factor or categorial covariate separate bubble plots for each
group compared to the baseline group are plotted.
}
\examples{
data(Fleiss1993cont)

# Add some (fictitious) grouping variables:
Fleiss1993cont$age <- c(55, 65, 52, 65, 58)
Fleiss1993cont$region <- c("Europe", "Europe", "Asia", "Asia", "Europe")

ma <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")

mr1 <- metareg(ma, region)
mr1

bubble(mr1)
bubble(mr1, lwd = 2, col.line = "blue")

mr2 <- metareg(ma, age)
mr2

bubble(mr2, lwd = 2, col.line = "blue", xlim = c(50, 70))
bubble(mr2, lwd = 2, col.line = "blue", xlim = c(50, 70), cex = "common")

# Do not print regression line
#
bubble(mr2, lwd = 2, col.line = "blue", xlim = c(50, 70), regline = FALSE)

}
\references{
Thompson SG, Higgins JP (2002):
How should meta-regression analyses be undertaken and interpreted?
\emph{Statistics in Medicine},
\bold{21}, 1559--73
}
\seealso{
\code{\link{metagen}}, \code{\link{metainf}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.data.frame.meta.R
\name{as.data.frame.meta}
\alias{as.data.frame.meta}
\title{Coerce to a data frame}
\usage{
\method{as.data.frame}{meta}(x, row.names = NULL, optional = FALSE, ...)
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{row.names}{\code{NULL} or a character vector giving the row
names for the data frame.}

\item{optional}{logical. If \code{TRUE}, setting row names and
converting column names (to syntactic names) is optional.}

\item{\dots}{other arguments}
}
\value{
A data frame is returned by the function \code{as.data.frame}.
}
\description{
The \code{as.data.frame} method returns a data frame containing
information on individual studies, e.g., estimated treatment effect
and its standard error.
}
\examples{
data(Fleiss1993cont)
#
# Generate additional variable with grouping information
#
Fleiss1993cont$group <- c(1, 2, 1, 1, 2)
#
# Do meta-analysis without grouping information
#
ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD", studlab = paste(study, year))
#
# Update meta-analysis object and do subgroup analyses
#
update(ma1, subgroup = group)

# Same result using metacont function directly
#
ma2 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD", studlab = paste(study, year),
  subgroup = group)
ma2

# Compare printout of the following two commands
#
as.data.frame(ma1)
ma1$data

}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}, \code{\link{forest.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.summary.meta.R
\name{print.summary.meta}
\alias{print.summary.meta}
\title{Print detailed meta-analysis results}
\usage{
\method{print}{summary.meta}(
  x,
  sortvar,
  common = x$x$common,
  random = x$x$random,
  details = FALSE,
  ma = TRUE,
  overall = x$overall & ma,
  backtransf = x$backtransf,
  pscale = x$pscale,
  irscale = x$irscale,
  irunit = x$irunit,
  digits = gs("digits"),
  digits.se = gs("digits.se"),
  digits.stat = gs("digits.stat"),
  digits.pval = max(gs("digits.pval"), 2),
  digits.tau2 = gs("digits.tau2"),
  digits.tau = gs("digits.tau"),
  digits.Q = gs("digits.Q"),
  digits.df = gs("digits.df"),
  digits.pval.Q = max(gs("digits.pval.Q"), 2),
  digits.H = gs("digits.H"),
  digits.I2 = gs("digits.I2"),
  digits.prop = gs("digits.prop"),
  digits.weight = gs("digits.weight"),
  scientific.pval = gs("scientific.pval"),
  zero.pval = gs("zero.pval"),
  JAMA.pval = gs("JAMA.pval"),
  big.mark = gs("big.mark"),
  print.tau2 = gs("print.tau2"),
  print.tau2.ci = gs("print.tau2.ci"),
  print.tau = gs("print.tau"),
  print.tau.ci = gs("print.tau.ci"),
  print.Q = gs("print.Q"),
  print.I2 = gs("print.I2"),
  print.I2.ci = gs("print.I2.ci"),
  print.H = gs("print.H"),
  print.Rb = gs("print.Rb"),
  text.tau2 = gs("text.tau2"),
  text.tau = gs("text.tau"),
  text.I2 = gs("text.I2"),
  text.Rb = gs("text.Rb"),
  truncate,
  text.truncate = "*** Output truncated ***",
  details.methods = TRUE,
  warn.backtransf = FALSE,
  ...
)
}
\arguments{
\item{x}{An object of class \code{summary.meta}}

\item{sortvar}{An optional vector used to sort the individual
studies (must be of same length as \code{x$TE}).}

\item{common}{A logical indicating whether results of common effect
meta-analysis should be printed.}

\item{random}{A logical indicating whether results of random
effects meta-analysis should be printed.}

\item{details}{A logical indicating whether further details of
individual studies should be printed.}

\item{ma}{A logical indicating whether the summary results of the
meta-analysis should be printed.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{backtransf}{A logical indicating whether printed results
should be back transformed. If \code{backtransf = TRUE}, results
for \code{sm = "OR"} are printed as odds ratios rather than log
odds ratios and results for \code{sm = "ZCOR"} are printed as
correlations rather than Fisher's z transformed correlations, for
example.}

\item{pscale}{A numeric giving scaling factor for printing of
single event probabilities or risk differences, i.e. if argument
\code{sm} is equal to \code{"PLOGIT"}, \code{"PLN"},
\code{"PRAW"}, \code{"PAS"}, \code{"PFT"}, or \code{"RD"}.}

\item{irscale}{A numeric defining a scaling factor for printing of
single incidence rates or incidence rate differences, i.e. if
argument \code{sm} is equal to \code{"IR"}, \code{"IRLN"},
\code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.}

\item{irunit}{A character specifying the time unit used to
calculate rates, e.g. person-years.}

\item{digits}{Minimal number of significant digits, see
\code{print.default}.}

\item{digits.se}{Minimal number of significant digits for standard
deviations and standard errors, see \code{print.default}.}

\item{digits.stat}{Minimal number of significant digits for z- or
t-value of test for effect, see \code{print.default}.}

\item{digits.pval}{Minimal number of significant digits for p-value
of test of treatment effect, see \code{print.default}.}

\item{digits.tau2}{Minimal number of significant digits for
between-study variance, see \code{print.default}.}

\item{digits.tau}{Minimal number of significant digits for square
root of between-study variance, see \code{print.default}.}

\item{digits.Q}{Minimal number of significant digits for
heterogeneity statistic Q, see \code{print.default}.}

\item{digits.df}{Minimal number of significant digits for degrees
of freedom.}

\item{digits.pval.Q}{Minimal number of significant digits for
p-value of heterogeneity test, see \code{print.default}.}

\item{digits.H}{Minimal number of significant digits for H
statistic, see \code{print.default}.}

\item{digits.I2}{Minimal number of significant digits for I-squared
and Rb statistic, see \code{print.default}.}

\item{digits.prop}{Minimal number of significant digits for
proportions, see \code{print.default}.}

\item{digits.weight}{Minimal number of significant digits for
weights, see \code{print.default}.}

\item{scientific.pval}{A logical specifying whether p-values should
be printed in scientific notation, e.g., 1.2345e-01 instead of
0.12345.}

\item{zero.pval}{A logical specifying whether p-values should be
printed with a leading zero.}

\item{JAMA.pval}{A logical specifying whether p-values for test of
overall effect should be printed according to JAMA reporting
standards.}

\item{big.mark}{A character used as thousands separator.}

\item{print.tau2}{A logical specifying whether between-study
variance \eqn{\tau^2} should be printed.}

\item{print.tau2.ci}{A logical value indicating whether to print
the confidence interval of \eqn{\tau^2}.}

\item{print.tau}{A logical specifying whether \eqn{\tau}, the
square root of the between-study variance \eqn{\tau^2}, should be
printed.}

\item{print.tau.ci}{A logical value indicating whether to print the
confidence interval of \eqn{\tau}.}

\item{print.Q}{A logical value indicating whether to print the
results of the test of heterogeneity.}

\item{print.I2}{A logical specifying whether heterogeneity
statistic I\eqn{^2} should be printed.}

\item{print.I2.ci}{A logical specifying whether confidence interval for
heterogeneity statistic I\eqn{^2} should be printed.}

\item{print.H}{A logical specifying whether heterogeneity statistic
H should be printed.}

\item{print.Rb}{A logical specifying whether heterogeneity
statistic R\eqn{_b} should be printed.}

\item{text.tau2}{Text printed to identify between-study variance
\eqn{\tau^2}.}

\item{text.tau}{Text printed to identify \eqn{\tau}, the square
root of the between-study variance \eqn{\tau^2}.}

\item{text.I2}{Text printed to identify heterogeneity statistic
I\eqn{^2}.}

\item{text.Rb}{Text printed to identify heterogeneity statistic
R\eqn{_b}.}

\item{truncate}{An optional vector used to truncate the printout of
results for individual studies (must be a logical vector of same
length as \code{x$TE} or contain numerical values).}

\item{text.truncate}{A character string printed if study results
were truncated from the printout.}

\item{details.methods}{A logical specifying whether details on
statistical methods should be printed.}

\item{warn.backtransf}{Deprecated argument (ignored).}

\item{\dots}{Additional arguments (passed on to
\code{\link{print.meta}} called internally).}
}
\description{
Print method for objects of class \code{summary.meta}.
}
\details{
Print method for objects of class \code{summary.meta} giving
detailed information on the meta-analysis.

Argument \code{pscale} can be used to rescale single proportions or
risk differences, e.g. \code{pscale = 1000} means that proportions
are expressed as events per 1000 observations. This is useful in
situations with (very) low event probabilities.

Argument \code{irscale} can be used to rescale single rates or rate
differences, e.g. \code{irscale = 1000} means that rates are
expressed as events per 1000 time units, e.g. person-years. This is
useful in situations with (very) low rates. Argument \code{irunit}
can be used to specify the time unit used in individual studies
(default: "person-years"). This information is printed in summaries
and forest plots if argument \code{irscale} is not equal to 1.
}
\examples{
data(Fleiss1993cont)
ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD", studlab = paste(study, year))
sma1 <- summary(ma1)
sma1

print(sma1, digits = 2)

\dontrun{
# Use unicode characters to print tau^2, tau, and I^2 
print(sma1,
  text.tau2 = "\u03c4\u00b2",
  text.tau = "\u03c4", text.I2 = "I\u00b2")
}

}
\references{
Cooper H & Hedges LV (1994), \emph{The Handbook of
  Research Synthesis}.  Newbury Park, CA: Russell Sage Foundation.

Crippa A, Khudyakov P, Wang M, Orsini N, Spiegelman D (2016), A new measure
of between-studies heterogeneity in meta-analysis.  \emph{Statistics in
Medicine}, \bold{35}, 3661--75.

Higgins JPT & Thompson SG (2002), Quantifying heterogeneity in a
meta-analysis.  \emph{Statistics in Medicine}, \bold{21}, 1539--58.
}
\seealso{
\code{\link{summary.meta}}, \code{\link{update.meta}},
  \code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/longarm.R
\name{longarm}
\alias{longarm}
\title{Transform data from pairwise comparisons to long arm-based format}
\usage{
longarm(
  treat1,
  treat2,
  event1,
  n1,
  event2,
  n2,
  mean1,
  sd1,
  mean2,
  sd2,
  time1,
  time2,
  agent1,
  agent2,
  dose1,
  dose2,
  sep.ag = "*",
  data = NULL,
  studlab,
  id1 = NULL,
  id2 = NULL,
  append = TRUE,
  keep.duplicated = FALSE,
  keep.internal = FALSE
)
}
\arguments{
\item{treat1}{Either label for first treatment or a meta-analysis
or pairwise object (see Details).}

\item{treat2}{Label for second treatment.}

\item{event1}{Number of events (first treatment).}

\item{n1}{Number of observations (first treatment).}

\item{event2}{Number of events (second treatment).}

\item{n2}{Number of observations (second treatment).}

\item{mean1}{Estimated mean (first treatment).}

\item{sd1}{Standard deviation (first treatment).}

\item{mean2}{Estimated mean (second treatment).}

\item{sd2}{Standard deviation (second treatment).}

\item{time1}{Person time at risk (first treatment).}

\item{time2}{Person time at risk (second treatment).}

\item{agent1}{Agent (first treatment).}

\item{agent2}{Agent (second treatment).}

\item{dose1}{Dose (first treatment).}

\item{dose2}{Dose (second treatment).}

\item{sep.ag}{A character used as separator between agent and dose to
create treatment labels.}

\item{data}{An optional data frame containing the study
information.}

\item{studlab}{A vector with study labels (optional).}

\item{id1}{Last character(s) of variable names for additional
variables with group specific information for first treatment.}

\item{id2}{Last character(s) of variable names for additional
variables with group specific information for second treatment.}

\item{append}{A logical indicating if data frame provided in
argument 'data' should be returned.}

\item{keep.duplicated}{A logical indicating if duplicated rows
should be returned (see Details).}

\item{keep.internal}{A logical indicating if variables generated
internally should be returned (typically only relevant for data
checking).}
}
\value{
A data frame in long arm-based format.
}
\description{
This function transforms data from pairwise comparisons to a long
arm-based format, i.e., two rows for a pairwise comparison.
}
\details{
This function transforms data given as one pairwise comparison per
row to a long arm-based format with one row per treatment arm. The
long arm-based format is, for example, the required input format
for WinBUGS.

The function can be used to transform data with a binary,
continuous or count outcome. The corresponding meta-analysis
functions are \code{\link{metabin}}, \code{\link{metacont}} and
\code{\link{metainc}}. Accordingly, a meta-analysis object created
with one of these functions can be provided as argument
\code{treat1}. It is also possible to use the longarm function with
an R objected created with \code{\link{pairwise}}.

Otherwise, arguments \code{treat1} and \code{treat2} are mandatory
to identify the individual treatments and, depending on the
outcome, the following additional arguments are mandatory:

\itemize{
\item event1, n1, event2, n2 (binary outcome);
\item n1, mean1, sd1, n2, mean2, sd2 (continuous outcome);
\item time1, n1, time2, n2 (count outcome).
}

Argument \code{studlab} must be provided if several pairwise
comparisons come from a single study with more than two treatments.

The following variables will be returned:

\tabular{rl}{
\bold{\emph{studlab}}\tab study label \cr
\bold{\emph{treat}}\tab treatment label \cr
\bold{\emph{n}}\tab group sample size (count outcome only if provided) \cr
\bold{\emph{events}}\tab number of events (binary or count outcome) \cr
\bold{\emph{nonevents}}\tab number of non-events (binary outcome) \cr
\bold{\emph{mean}}\tab estimated mean (continuous outcome) \cr
\bold{\emph{sd}}\tab standard deviation (continuous outcome) \cr
\bold{\emph{time}}\tab person time at risk (count outcome) \cr
}

In addition, the data set provided in argument \code{data} will be
returned if argument \code{append = TRUE} (default).

Argument \code{keep.duplicated} can be used to keep duplicated rows
from the data set. Duplicated rows can occur, for example, in a
three-arm study comparing treatments A and B with placebo. In this
situation, the placebo arm will be returned twice in the data set
in long arm-based format if \code{keep.duplicated = TRUE}. By
default, duplicated rows with not be kept in the data set.
}
\note{
R function \code{\link[metafor]{to.long}} from R package
\bold{metafor} is called internally.
}
\examples{
# Artificial example with three studies
m <- metabin(1:3, 100:102, 4:6, 200:202, studlab = LETTERS[1:3])
# Transform data to long arm-based format
longarm(m)
# Keep internal variables
longarm(m, keep.internal = TRUE)

}
\seealso{
\code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metainc}}, \code{\link{pairwise}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metabin.R
\name{metabin}
\alias{metabin}
\title{Meta-analysis of binary outcome data}
\usage{
metabin(
  event.e,
  n.e,
  event.c,
  n.c,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  method = ifelse(tau.common, "Inverse", gs("method")),
  sm = ifelse(!is.na(charmatch(tolower(method), c("peto", "glmm", "lrp", "ssw"), nomatch
    = NA)), "OR", gs("smbin")),
  incr = gs("incr"),
  method.incr = gs("method.incr"),
  allstudies = gs("allstudies"),
  incr.e = if (length(incr) > 1) incr else NULL,
  incr.c = if (length(incr) > 1) incr else NULL,
  level = gs("level"),
  MH.exact = gs("MH.exact"),
  RR.Cochrane = gs("RR.Cochrane"),
  Q.Cochrane = gs("Q.Cochrane") & method == "MH" & method.tau == "DL",
  model.glmm = gs("model.glmm"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau,
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  method.bias = ifelse(sm == "OR", "Harbord", ifelse(sm == "DOR", "Deeks",
    gs("method.bias"))),
  backtransf = gs("backtransf"),
  pscale = 1,
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.e = gs("label.e"),
  label.c = gs("label.c"),
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  hakn,
  adhoc.hakn,
  print.CMH = gs("print.CMH"),
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{event.e}{Number of events in experimental group, or true
positives in diagnostic study, or an R object
created with \code{\link{pairwise}}.}

\item{n.e}{Number of observations in experimental group or number
of ill participants in diagnostic study.}

\item{event.c}{Number of events in control group or false positives
in diagnostic study.}

\item{n.c}{Number of observations in control group or number of
healthy participants in diagnostic study.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information, i.e., event.e, n.e, event.c, and n.c.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{method}{A character string indicating which method is to be
used for pooling of studies. One of \code{"Inverse"},
\code{"MH"}, \code{"Peto"}, \code{"GLMM"}, \code{"LRP"}, or \code{"SSW"},
can be abbreviated.}

\item{sm}{A character string indicating which summary measure
(\code{"RR"}, \code{"OR"}, \code{"RD"}, \code{"ASD"},
\code{"DOR"}, or \code{"VE"}) is to be used for pooling of
studies, see Details.}

\item{incr}{Could be either a numerical value which is added to
cell frequencies for studies with a zero cell count, the
character string \code{"TACC"} which stands for treatment arm
continuity correction, or a numeric vector with the continuity
correction for each study, see Details.}

\item{method.incr}{A character string indicating which continuity
correction method should be used (\code{"only0"},
\code{"if0all"}, \code{"all"}, or \code{"user"}), see Details.}

\item{allstudies}{A logical indicating if studies with zero or all
events in both groups are to be included in the meta-analysis
(applies only if \code{sm} is equal to \code{"RR"}, \code{"OR"},
or \code{"DOR"}).}

\item{incr.e}{Continuity correction in experimental group, see Details.}

\item{incr.c}{Continuity correction in control group, see Details.}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{MH.exact}{A logical indicating if \code{incr} is not to be
added to cell frequencies for studies with a zero cell count to
calculate the pooled estimate based on the Mantel-Haenszel
method.}

\item{RR.Cochrane}{A logical indicating if 2*\code{incr} instead of
1*\code{incr} is to be added to \code{n.e} and \code{n.c} in the
calculation of the risk ratio (i.e., \code{sm="RR"}) for studies
with a zero cell. This is used in RevMan 5, the program for
preparing and maintaining Cochrane reviews.}

\item{Q.Cochrane}{A logical indicating if the Mantel-Haenszel
estimate is used in the calculation of the heterogeneity
statistic Q which is implemented in RevMan 5, the program for
preparing and maintaining Cochrane reviews.}

\item{model.glmm}{A character string indicating which GLMM should
be used.  One of \code{"UM.FS"}, \code{"UM.RS"}, \code{"CM.EL"},
and \code{"CM.AL"}, see Details.}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{method.bias}{A character string indicating which test for
funnel plot asymmetry is to be used. Either \code{"Begg"},
\code{"Egger"}, \code{"Thompson"}, \code{"Schwarzer"},
\code{"Harbord"}, \code{"Peters"}, or \code{"Deeks"}, can be
abbreviated. See function \code{\link{metabias}.}}

\item{backtransf}{A logical indicating whether results for odds
ratio (\code{sm="OR"}), risk ratio (\code{sm="RR"}), or
diagnostic odds ratio (\code{sm="DOR"}) should be back
transformed in printouts and plots. If TRUE (default), results
will be presented as odds ratios and risk ratios; otherwise log
odds ratios and log risk ratios will be shown.}

\item{pscale}{A numeric defining a scaling factor for printing of
risk differences.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.e}{Label for experimental group.}

\item{label.c}{Label for control group.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{hakn}{Deprecated argument (replaced by 'method.random.ci').}

\item{adhoc.hakn}{Deprecated argument (replaced by
'adhoc.hakn.ci').}

\item{print.CMH}{A logical indicating whether result of the
Cochran-Mantel-Haenszel test for overall effect should be
printed.}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if \code{incr} is added to studies with zero cell
frequencies or if estimation problems exist in fitting a GLMM).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}} or
\code{\link[metafor]{rma.glmm}}.}

\item{\dots}{Additional arguments passed on to
\code{\link[metafor]{rma.glmm}} function and to catch deprecated
arguments.}
}
\value{
An object of class \code{c("metabin", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of common effect and random effects estimates (risk
ratio, odds ratio, risk difference, arcsine difference, or
diagnostic odds ratio) for meta-analyses with binary outcome
data. Mantel-Haenszel, inverse variance, Peto method, generalised
linear mixed model (GLMM), logistic regression with penalised likelihood
and sample size method are available for pooling. For GLMMs,
the \code{\link[metafor]{rma.glmm}} function from R package \bold{metafor}
(Viechtbauer, 2010) is called internally. For penalised logistic regression,
R package \bold{brglma2} must be available.
}
\details{
Calculation of common and random effects estimates for meta-analyses
with binary outcome data.

The following measures of treatment effect are available (Rücker et
al., 2009):

\itemize{
\item Risk ratio (\code{sm = "RR"})
\item Odds ratio (\code{sm = "OR"})
\item Risk difference (\code{sm = "RD"})
\item Arcsine difference (\code{sm = "ASD"})
\item Diagnostic Odds ratio (\code{sm = "DOR"})
\item Vaccine efficacy or vaccine effectiveness (\code{sm = "VE"})
}

Note, mathematically, odds ratios and diagnostic odds ratios are
identical, however, the labels in printouts and figures
differ. Furthermore, log risk ratio (logRR) and log vaccine ratio
(logVR) are mathematical identical, however, back-transformed
results differ as vaccine efficacy or effectiveness is defined as
\code{VE = 100 * (1 - RR)}.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

\subsection{Meta-analysis method}{

By default, both common effect (also called common effect) and
random effects models are considered (see arguments \code{common}
and \code{random}). If \code{method} is \code{"MH"} (default), the
Mantel-Haenszel method (Greenland & Robins, 1985; Robins et al.,
1986) is used to calculate the common effect estimate; if
\code{method} is \code{"Inverse"}, inverse variance weighting is
used for pooling (Fleiss, 1993); if \code{method} is \code{"Peto"},
the Peto method is used for pooling (Yusuf et al., 1985); if
\code{method} is \code{"SSW"}, the sample size method is used for
pooling (Bakbergenuly et al., 2020).

While the Mantel-Haenszel and Peto method are defined under the
common effect model, random effects variants based on these methods
are also implemented in \code{metabin}. Following RevMan 5, the
Mantel-Haenszel estimator is used in the calculation of the
between-study heterogeneity statistic Q which is used in the
DerSimonian-Laird estimator (DerSimonian and Laird,
1986). Accordingly, the results for the random effects
meta-analysis using the Mantel-Haenszel or inverse variance method
are typically very similar. For the Peto method, Peto's log odds
ratio, i.e. \code{(O-E) / V} and its standard error \code{sqrt(1 /
V)} with \code{O-E} and \code{V} denoting "Observed minus Expected"
and its variance, are utilised in the random effects
model. Accordingly, results of a random effects model using
\code{sm = "Peto"} can be different to results from a random
effects model using \code{sm = "MH"} or \code{sm =
"Inverse"}. Note, the random effects estimate is based on the
inverse variance method for all methods discussed so far.

A distinctive and frequently overlooked advantage of binary
endpoints is that individual patient data (IPD) can be extracted
from a two-by-two table.  Accordingly, statistical methods for IPD,
i.e., logistic regression and generalised linear mixed models, can
be utilised in a meta-analysis of binary outcomes (Stijnen et al.,
2010; Simmonds et al., 2016).

R package \bold{brglma2} must be available to fit a one-stage logistic
regression model with penalised likelihood (Evrenoglou et al., 2022).
The estimation of the summary odds ratio relies on the maximisation of the 
likelihood function, penalised using a Firth-type correction. This
penalisation aims to reduce bias in cases with rare events and a small
number of available studies. However, this method is not restricted 
to only such cases and can be applied more generally to binary data. Note,
with this type of penalisation, all studies can be included in the analysis,
regardless of the total number of observed events. This allows both single
and double zero studies to be included without any continuity correction.
The random effects model uses a multiplicative heterogeneity parameter
\eqn{\phi}, added to the model as an \emph{ad hoc} term. The estimation of
this parameter relies on a modified expression of Pearson's statistic, which
accounts for sparse data. An estimate of \eqn{\phi} equal to 1 indicates the
absence of heterogeneity.

Generalised linear mixed models are available
(argument \code{method = "GLMM"}) for the odds ratio as summary measure for
the common effect and random effects model by calling the
\code{\link[metafor]{rma.glmm}} function from R package
\bold{metafor} internally. 

Four different GLMMs are available for
meta-analysis with binary outcomes using argument \code{model.glmm}
(which corresponds to argument \code{model} in the
\code{\link[metafor]{rma.glmm}} function):
\tabular{cl}{
1. \tab Logistic regression model with common study effects
 (default) \cr
 \tab (\code{model.glmm = "UM.FS"}, i.e., \bold{U}nconditional
 \bold{M}odel - \bold{F}ixed \bold{S}tudy effects) \cr
2. \tab Mixed-effects logistic regression model with random study
 effects \cr
 \tab (\code{model.glmm = "UM.RS"}, i.e., \bold{U}nconditional
 \bold{M}odel - \bold{R}andom \bold{S}tudy effects) \cr
3. \tab Generalised linear mixed model (conditional
 Hypergeometric-Normal) \cr
 \tab (\code{model.glmm = "CM.EL"}, i.e., \bold{C}onditional
 \bold{M}odel - \bold{E}xact \bold{L}ikelihood) \cr
4. \tab Generalised linear mixed model (conditional
  Binomial-Normal) \cr
 \tab (\code{model.glmm = "CM.AL"}, i.e., \bold{C}onditional
  \bold{M}odel - \bold{A}pproximate \bold{L}ikelihood)
}

Details on these four GLMMs as well as additional arguments which
can be provided using argument '\code{\dots}' in \code{metabin} are
described in \code{\link[metafor]{rma.glmm}} where you can also
find information on the iterative algorithms used for estimation.
Note, regardless of which value is used for argument
\code{model.glmm}, results for two different GLMMs are calculated:
common effect model (with fixed treatment effect) and random
effects model (with random treatment effects).
}

\subsection{Continuity correction}{

Four approaches are available to apply a continuity correction:
\itemize{
\item Only studies with a zero cell count (\code{method.incr =
  "only0"})
\item All studies if at least one study has a zero cell count
  (\code{method.incr = "if0all"})
\item All studies irrespective of zero cell counts
  (\code{method.incr = "all"})
\item Use values provided in arguments \code{incr.e} and \code{incr.c}
  (\code{method.incr = "user"})
}

By default, a continuity correction is only applied to studies with
a zero cell count (\code{method.incr = "only0"}). This method
showed the best performance for the odds ratio in a simulation
study under the random effects model (Weber et al., 2020).

The continuity correction method is used both to calculate
individual study results with confidence limits and to conduct
meta-analysis based on the inverse variance method. For the risk
difference, the method is only considered to calculate standard
errors and confidence limits.  For Peto method and GLMMs no
continuity correction is used in the meta-analysis. Furthermore,
the continuity correction is ignored for individual studies for the
Peto method.

For studies with a zero cell count, by default, 0.5 (argument
\code{incr}) is added to all cell frequencies for the odds ratio or
only the number of events for the risk ratio (argument
\code{RR.Cochrane = FALSE}, default). The increment is added to all
cell frequencies for the risk ratio if argument \code{RR.Cochrane =
TRUE}. For the risk difference, \code{incr} is only added to all
cell frequencies to calculate the standard error. Finally, a
treatment arm continuity correction is used if \code{incr = "TACC"}
(Sweeting et al., 2004; Diamond et al., 2007).

For odds ratio and risk ratio, treatment estimates and standard
errors are only calculated for studies with zero or all events in
both groups if \code{allstudies = TRUE}.

For the Mantel-Haenszel method, by default (if \code{MH.exact} is
FALSE), \code{incr} is added to cell frequencies of a study with a
zero cell count in the calculation of the pooled risk ratio or odds
ratio as well as the estimation of the variance of the pooled risk
difference, risk ratio or odds ratio. This approach is also used in
other software, e.g. RevMan 5 and the Stata procedure
metan. According to Fleiss (in Cooper & Hedges, 1994), there is no
need to add 0.5 to a cell frequency of zero to calculate the
Mantel-Haenszel estimate and he advocates the exact method
(\code{MH.exact} = TRUE). Note, estimates based on exact
Mantel-Haenszel method or GLMM are not defined if the number of
events is zero in all studies either in the experimental or control
group.
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments
\code{common} and \code{random}. Accordingly, the estimate
for the random effects model can be extracted from component
\code{TE.random} of an object of class \code{"meta"} even if
argument \code{random = FALSE}. However, all functions in R
package \bold{meta} will adequately consider the values for
\code{common} and \code{random}. E.g. function
\code{\link{print.meta}} will not print results for the random
effects model if \code{random = FALSE}.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\examples{
# Calculate odds ratio and confidence interval for a single study
#
metabin(10, 20, 15, 20, sm = "OR")

# Different results (due to handling of studies with double zeros)
#
metabin(0, 10, 0, 10, sm = "OR")
metabin(0, 10, 0, 10, sm = "OR", allstudies = TRUE)

# Use subset of Olkin (1995) to conduct meta-analysis based on
# inverse variance method (with risk ratio as summary measure)
#
data(Olkin1995)
ma1 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = c(41, 47, 51, 59),
  studlab = paste(author, year),
  method = "Inverse")
ma1
# Show results for individual studies
summary(ma1)

# Use different subset of Olkin (1995)
#
ma2 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = year < 1970,
  studlab = paste(author, year),
  method = "Inverse")
ma2
forest(ma2)

# Meta-analysis with odds ratio as summary measure
#
ma3 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = year < 1970,
  studlab = paste(author, year),
  sm = "OR", method = "Inverse")
# Same meta-analysis result using 'update.meta' function
ma3 <- update(ma2, sm = "OR")
ma3

# Meta-analysis based on Mantel-Haenszel method (with odds ratio as
# summary measure)
#
ma4 <- update(ma3, method = "MH")
ma4

# Meta-analysis based on Peto method (only available for odds ratio
# as summary measure)
#
ma5 <- update(ma3, method = "Peto")
ma5

\dontrun{
# Meta-analyses using generalised linear mixed models (GLMM)

# Logistic regression model with (k = 4) fixed study effects
# (default: model.glmm = "UM.FS")
ma6 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  studlab = paste(author, year),
  data = Olkin1995, subset = year < 1970, method = "GLMM")
# Same results:
ma6 <- update(ma2, method = "GLMM")
ma6

# Mixed-effects logistic regression model with random study effects
ma7 <- update(ma6, model.glmm = "UM.RS")
#
# Use additional argument 'nAGQ' for internal call of 'rma.glmm'
# function
#
ma7 <- update(ma6, model.glmm = "UM.RS", nAGQ = 1)
ma7

# Generalised linear mixed model (conditional Hypergeometric-Normal)
# (R package 'BiasedUrn' must be available)
if (requireNamespace("BiasedUrn", quietly = TRUE)) {
 ma8 <- update(ma6, model.glmm = "CM.EL")
 ma8
}

# Generalised linear mixed model (conditional Binomial-Normal)
ma9 <- update(ma6, model.glmm = "CM.AL")
ma9

# Logistic regression model with (k = 70) fixed study effects
ma10 <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  studlab = paste(author, year),
  data = Olkin1995, method = "GLMM")
ma10

# Mixed-effects logistic regression model with random study effects
update(ma10, model.glmm = "UM.RS")

# Conditional Hypergeometric-Normal GLMM (with long computation time)
system.time(ma11 <- update(ma10, model.glmm = "CM.EL"))
ma11

# Generalised linear mixed model (conditional Binomial-Normal)
update(ma10, model.glmm = "CM.AL")
}

}
\references{
Bakbergenuly I, Hoaglin DC, Kulinskaya E (2020):
Methods for estimating between-study variance and overall
effect in meta-analysis of odds-ratios.
\emph{Research Synthesis Methods},
\bold{11}, 426--42

Cooper H & Hedges LV (1994):
\emph{The Handbook of Research Synthesis}.
Newbury Park, CA: Russell Sage Foundation

Diamond GA, Bax L, Kaul S (2007):
Uncertain Effects of Rosiglitazone on the Risk for Myocardial
Infarction and Cardiovascular Death.
\emph{Annals of Internal Medicine},
\bold{147}, 578--81

DerSimonian R & Laird N (1986):
Meta-analysis in clinical trials.
\emph{Controlled Clinical Trials},
\bold{7}, 177--88

Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A. (2022):
Network meta-analysis of rare events using penalized likelihood regression.
\emph{Statistics in Medicine},
\bold{41}, 5203--19

Fleiss JL (1993):
The statistical basis of meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{2}, 121--45

Greenland S & Robins JM (1985):
Estimation of a common effect parameter from sparse follow-up data.
\emph{Biometrics},
\bold{41}, 55--68

\emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
The Cochrane Collaboration, 2020

Robins J, Breslow N, Greenland S (1986):
Estimators of the Mantel-Haenszel Variance Consistent in Both
Sparse Data and Large-Strata Limiting Models.
\emph{Biometrics},
\bold{42}, 311--23

Rücker G, Schwarzer G, Carpenter J, Olkin I (2009):
Why add anything to nothing? The arcsine difference as a measure of
treatment effect in meta-analysis with zero cells.
\emph{Statistics in Medicine},
\bold{28}, 721--38

Simmonds MC, Higgins JP (2016):
A general framework for the use of logistic regression models in
meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{25}, 2858--77

StataCorp. 2011.
\emph{Stata Statistical Software: Release 12}.
College Station, TX: StataCorp LP.

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67

Sweeting MJ, Sutton AJ, Lambert PC (2004):
What to add to nothing? Use and avoidance of continuity corrections
in meta-analysis of sparse data.
\emph{Statistics in Medicine},
\bold{23}, 1351--75

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Viechtbauer W (2010):
Conducting meta-analyses in R with the metafor package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48

Weber F, Knapp G, Ickstadt K, Kundt G, Glass Ä (2020):
Zero-cell corrections in random-effects meta-analyses.
\emph{Research Synthesis Methods},
\bold{11}, 913--9

Yusuf S, Peto R, Lewis J, Collins R, Sleight P (1985):
Beta blockade during and after myocardial infarction: An overview
of the randomized trials.
\emph{Progress in Cardiovascular Diseases},
\bold{27}, 335--71
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{forest}}, \code{\link{funnel}},
  \code{\link{metabias}}, \code{\link{metacont}},
  \code{\link{metagen}}, \code{\link{metareg}},
  \code{\link{print.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metacont.R
\name{metacont}
\alias{metacont}
\title{Meta-analysis of continuous outcome data}
\usage{
metacont(
  n.e,
  mean.e,
  sd.e,
  n.c,
  mean.c,
  sd.c,
  studlab,
  data = NULL,
  subset = NULL,
  exclude = NULL,
  cluster = NULL,
  rho = 0,
  weights = NULL,
  weights.common = weights,
  weights.random = weights,
  median.e,
  q1.e,
  q3.e,
  min.e,
  max.e,
  median.c,
  q1.c,
  q3.c,
  min.c,
  max.c,
  method.mean = "Luo",
  method.sd = "Shi",
  approx.mean.e,
  approx.mean.c = approx.mean.e,
  approx.sd.e,
  approx.sd.c = approx.sd.e,
  sm = gs("smcont"),
  method.ci = gs("method.ci.cont"),
  level = gs("level"),
  pooledvar = gs("pooledvar"),
  method.smd = gs("method.smd"),
  sd.glass = gs("sd.glass"),
  exact.smd = gs("exact.smd"),
  common = gs("common"),
  random = gs("random") | !is.null(tau.preset),
  overall = common | random,
  overall.hetstat = if (is.null(gs("overall.hetstat"))) common | random else
    gs("overall.hetstat"),
  prediction = gs("prediction") | !missing(method.predict),
  method.tau = gs("method.tau"),
  method.tau.ci = gs("method.tau.ci"),
  level.hetstat = gs("level.hetstat"),
  tau.preset = NULL,
  TE.tau = NULL,
  tau.common = gs("tau.common"),
  detail.tau = NULL,
  method.I2 = gs("method.I2"),
  level.ma = gs("level.ma"),
  method.common.ci = gs("method.common.ci"),
  method.random.ci = gs("method.random.ci"),
  adhoc.hakn.ci = gs("adhoc.hakn.ci"),
  level.predict = gs("level.predict"),
  method.predict = gs("method.predict"),
  adhoc.hakn.pi = gs("adhoc.hakn.pi"),
  seed.predict = NULL,
  method.bias = gs("method.bias"),
  backtransf = gs("backtransf"),
  text.common = gs("text.common"),
  text.random = gs("text.random"),
  text.predict = gs("text.predict"),
  text.w.common = gs("text.w.common"),
  text.w.random = gs("text.w.random"),
  title = gs("title"),
  complab = gs("complab"),
  outclab = "",
  label.e = gs("label.e"),
  label.c = gs("label.c"),
  label.left = gs("label.left"),
  label.right = gs("label.right"),
  col.label.left = gs("col.label.left"),
  col.label.right = gs("col.label.right"),
  subgroup,
  subgroup.name = NULL,
  print.subgroup.name = gs("print.subgroup.name"),
  sep.subgroup = gs("sep.subgroup"),
  test.subgroup = gs("test.subgroup"),
  prediction.subgroup = gs("prediction.subgroup"),
  seed.predict.subgroup = NULL,
  byvar,
  id,
  adhoc.hakn,
  keepdata = gs("keepdata"),
  warn = gs("warn"),
  warn.deprecated = gs("warn.deprecated"),
  control = NULL,
  ...
)
}
\arguments{
\item{n.e}{Number of observations in experimental group or an R object
created with \code{\link{pairwise}}.}

\item{mean.e}{Estimated mean in experimental group.}

\item{sd.e}{Standard deviation in experimental group.}

\item{n.c}{Number of observations in control group.}

\item{mean.c}{Estimated mean in control group.}

\item{sd.c}{Standard deviation in control group.}

\item{studlab}{An optional vector with study labels.}

\item{data}{An optional data frame containing the study
information.}

\item{subset}{An optional vector specifying a subset of studies to
be used.}

\item{exclude}{An optional vector specifying studies to exclude
from meta-analysis, however, to include in printouts and forest
plots.}

\item{cluster}{An optional vector specifying which estimates come
from the same cluster resulting in the use of a three-level
meta-analysis model.}

\item{rho}{Assumed correlation of estimates within a cluster.}

\item{weights}{A single numeric or vector with user-specified weights.}

\item{weights.common}{User-specified weights (common effect model).}

\item{weights.random}{User-specified weights (random effects model).}

\item{median.e}{Median in experimental group (used to estimate the
mean and standard deviation).}

\item{q1.e}{First quartile in experimental group (used to estimate
the mean and standard deviation).}

\item{q3.e}{Third quartile in experimental group (used to estimate
the mean and standard deviation).}

\item{min.e}{Minimum in experimental group (used to estimate the
mean and standard deviation).}

\item{max.e}{Maximum in experimental group (used to estimate the
mean and standard deviation).}

\item{median.c}{Median in control group (used to estimate the mean
and standard deviation).}

\item{q1.c}{First quartile in control group (used to estimate the
mean and standard deviation).}

\item{q3.c}{Third quartile in control group (used to estimate the
mean and standard deviation).}

\item{min.c}{Minimum in control group (used to estimate the mean
and standard deviation).}

\item{max.c}{Maximum in control group (used to estimate the mean
and standard deviation).}

\item{method.mean}{A character string indicating which method to
use to approximate the mean from the median and other statistics
(see Details).}

\item{method.sd}{A character string indicating which method to use
to approximate the standard deviation from sample size, median,
interquartile range and range (see Details).}

\item{approx.mean.e}{Approximation method to estimate means in
experimental group (see Details).}

\item{approx.mean.c}{Approximation method to estimate means in
control group (see Details).}

\item{approx.sd.e}{Approximation method to estimate standard
deviations in experimental group (see Details).}

\item{approx.sd.c}{Approximation method to estimate standard
deviations in control group (see Details).}

\item{sm}{A character string indicating which summary measure
(\code{"MD"}, \code{"SMD"}, or \code{"ROM"}) is to be used for
pooling of studies.}

\item{method.ci}{A character string indicating which method is used
to calculate confidence intervals for individual studies (see
Details).}

\item{level}{The level used to calculate confidence intervals for
individual studies.}

\item{pooledvar}{A logical indicating if a pooled variance should
be used for the mean difference (\code{sm="MD"}) or ratio of
means (\code{sm="ROM"}).}

\item{method.smd}{A character string indicating which method is
used to estimate the standardised mean difference
(\code{sm="SMD"}). Either \code{"Hedges"} for Hedges' g
(default), \code{"Cohen"} for Cohen's d, or \code{"Glass"} for
Glass' delta, can be abbreviated.}

\item{sd.glass}{A character string indicating which standard
deviation is used in the denominator for Glass' method to
estimate the standardised mean difference. Either
\code{"control"} using the standard deviation in the control
group (\code{sd.c}) or \code{"experimental"} using the standard
deviation in the experimental group (\code{sd.e}), can be
abbreviated.}

\item{exact.smd}{A logical indicating whether exact formulae should
be used in estimation of the standardised mean difference and its
standard error (see Details).}

\item{common}{A logical indicating whether a common effect
meta-analysis should be conducted.}

\item{random}{A logical indicating whether a random effects
meta-analysis should be conducted.}

\item{overall}{A logical indicating whether overall summaries
should be reported. This argument is useful in a meta-analysis
with subgroups if overall results should not be reported.}

\item{overall.hetstat}{A logical value indicating whether to print
heterogeneity measures for overall treatment comparisons. This
argument is useful in a meta-analysis with subgroups if
heterogeneity statistics should only be printed on subgroup
level.}

\item{prediction}{A logical indicating whether a prediction
interval should be printed.}

\item{method.tau}{A character string indicating which method is
used to estimate the between-study variance \eqn{\tau^2} and its
square root \eqn{\tau} (see \code{\link{meta-package}}).}

\item{method.tau.ci}{A character string indicating which method is
used to estimate the confidence interval of \eqn{\tau^2} and
\eqn{\tau} (see \code{\link{meta-package}}).}

\item{level.hetstat}{The level used to calculate confidence intervals
for heterogeneity statistics.}

\item{tau.preset}{Prespecified value for the square root of the
between-study variance \eqn{\tau^2}.}

\item{TE.tau}{Overall treatment effect used to estimate the
between-study variance tau-squared.}

\item{tau.common}{A logical indicating whether tau-squared should
be the same across subgroups.}

\item{detail.tau}{Detail on between-study variance estimate.}

\item{method.I2}{A character string indicating which method is
used to estimate the heterogeneity statistic I\eqn{^2}. Either
\code{"Q"} or \code{"tau2"}, can be abbreviated
(see \code{\link{meta-package}}).}

\item{level.ma}{The level used to calculate confidence intervals
for meta-analysis estimates.}

\item{method.common.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
common effect estimate (see \code{\link{meta-package}}).}

\item{method.random.ci}{A character string indicating which method
is used to calculate confidence interval and test statistic for
random effects estimate (see \code{\link{meta-package}}).}

\item{adhoc.hakn.ci}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied in the case
of an arbitrarily small Hartung-Knapp variance estimate (see
\code{\link{meta-package}}).}

\item{level.predict}{The level used to calculate prediction
interval for a new study.}

\item{method.predict}{A character string indicating which method is
used to calculate a prediction interval (see
\code{\link{meta-package}}).}

\item{adhoc.hakn.pi}{A character string indicating whether an
\emph{ad hoc} variance correction should be applied for
prediction interval (see \code{\link{meta-package}}).}

\item{seed.predict}{A numeric value used as seed to calculate
bootstrap prediction interval (see \code{\link{meta-package}}).}

\item{method.bias}{A character string indicating which test is to
be used. Either \code{"Begg"}, \code{"Egger"}, \code{"Thompson"},
or \code{"Pustejovsky"} (see \code{\link{metabias}}), can be
abbreviated.}

\item{backtransf}{A logical indicating whether results for ratio of
means (\code{sm="ROM"}) should be back transformed in printouts
and plots. If TRUE (default), results will be presented as ratio
of means; otherwise log ratio of means will be shown.}

\item{text.common}{A character string used in printouts and forest
plot to label the pooled common effect estimate.}

\item{text.random}{A character string used in printouts and forest
plot to label the pooled random effects estimate.}

\item{text.predict}{A character string used in printouts and forest
plot to label the prediction interval.}

\item{text.w.common}{A character string used to label weights of
common effect model.}

\item{text.w.random}{A character string used to label weights of
random effects model.}

\item{title}{Title of meta-analysis / systematic review.}

\item{complab}{Comparison label.}

\item{outclab}{Outcome label.}

\item{label.e}{Label for experimental group.}

\item{label.c}{Label for control group.}

\item{label.left}{Graph label on left side of null effect in forest plot.}

\item{label.right}{Graph label on right side of null effect in forest plot.}

\item{col.label.left}{The colour of the graph label on the left side of
the null effect.}

\item{col.label.right}{The colour of the graph label on the right side of
the null effect.}

\item{subgroup}{An optional vector to conduct a meta-analysis with
subgroups.}

\item{subgroup.name}{A character string with a name for the
subgroup variable.}

\item{print.subgroup.name}{A logical indicating whether the name of
the subgroup variable should be printed in front of the group
labels.}

\item{sep.subgroup}{A character string defining the separator
between name of subgroup variable and subgroup label.}

\item{test.subgroup}{A logical value indicating whether to print
results of test for subgroup differences.}

\item{prediction.subgroup}{A logical indicating whether prediction
intervals should be printed for subgroups.}

\item{seed.predict.subgroup}{A numeric vector providing seeds to
calculate bootstrap prediction intervals within subgroups. Must
be of same length as the number of subgroups.}

\item{byvar}{Deprecated argument (replaced by 'subgroup').}

\item{id}{Deprecated argument (replaced by 'cluster').}

\item{adhoc.hakn}{Deprecated argument (replaced by
'adhoc.hakn.ci').}

\item{keepdata}{A logical indicating whether original data (set)
should be kept in meta object.}

\item{warn}{A logical indicating whether warnings should be printed
(e.g., if studies are excluded from meta-analysis due to zero
standard deviations).}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{control}{An optional list to control the iterative process to
estimate the between-study variance \eqn{\tau^2}. This argument
is passed on to \code{\link[metafor]{rma.uni}}.}

\item{\dots}{Additional arguments (to catch deprecated arguments).}
}
\value{
An object of class \code{c("metacont", "meta")} with corresponding
generic functions (see \code{\link{meta-object}}).
}
\description{
Calculation of common and random effects estimates for meta-analyses
with continuous outcome data; inverse variance weighting is used
for pooling.
}
\details{
Calculation of common and random effects estimates for meta-analyses
with continuous outcome data; inverse variance weighting is used
for pooling.

A three-level random effects meta-analysis model (Van den Noortgate
et al., 2013) is utilised if argument \code{cluster} is used and at
least one cluster provides more than one estimate. Internally,
\code{\link[metafor]{rma.mv}} is called to conduct the analysis and
\code{\link[metafor]{weights.rma.mv}} with argument \code{type =
"rowsum"} is used to calculate random effects weights.

Default settings are utilised for several arguments (assignments
using \code{\link{gs}} function). These defaults can be changed for
the current R session using the \code{\link{settings.meta}}
function.

Furthermore, R function \code{\link{update.meta}} can be used to
rerun a meta-analysis with different settings.

Three different types of summary measures are available for continuous
outcomes:
\itemize{
\item mean difference (argument \code{sm = "MD"})
\item standardised mean difference (\code{sm = "SMD"})
\item ratio of means (\code{sm = "ROM"})
}

\subsection{Standardised mean difference}{

For the standardised mean difference three methods are implemented:
\itemize{
\item Hedges' g (default, \code{method.smd = "Hedges"}) - see
  Hedges (1981)
\item Cohen's d (\code{method.smd = "Cohen"}) - see Cohen (1988)
\item Glass' delta (\code{method.smd = "Glass"}) - see Glass (1976)
}

Hedges (1981) calculated the exact bias in Cohen's d which is a
ratio of gamma distributions with the degrees of freedom,
i.e. total sample size minus two, as parameter. In the 1980s and 1990s, the
gamma distribution was not easily accessible in statistical software.
Accordingly, an accurate approximation of the exact bias, also provided in
Hedges (1981), was utilised for Hedges' g and its standard error in software
like RevMan 5. White and Thomas (2005) argued that approximations are
unnecessary with modern software and accordingly promoted to use the exact
formulae which is the default in R function metacont
(argument \code{exact.smd = TRUE}). For Hedges' g, the exact formulae are
used to calculate the standardised mean difference as well as the standard
error; for Cohen's d the exact formula is only used to calculate the standard
error. In typical applications (with sample sizes above 10), the differences
between using the exact formulae (\code{exact.smd = TRUE}) and the
approximation (\code{exact.smd = FALSE}) will be minimal.

For Glass' delta, by default (argument \code{sd.glass =
"control"}), the standard deviation in the control group
(\code{sd.c}) is used in the denominator of the standard mean
difference. The standard deviation in the experimental group
(\code{sd.e}) can be used by specifying \code{sd.glass =
"experimental"}.
}

\subsection{Ratio of means}{

Meta-analysis of ratio of means -- also called response ratios --
is described in Hedges et al. (1999) and Friedrich et al. (2008).
Calculations are conducted on the log scale and list elements
\code{TE}, \code{TE.common}, and \code{TE.random} contain the
logarithm of the ratio of means. In printouts and plots these
values are back transformed if argument \code{backtransf = TRUE}.
}

\subsection{Approximate means from sample sizes, medians and other statistics}{

Missing means in the experimental group (analogously for the
control group) can be derived from
\enumerate{
\item sample size, median, interquartile range and range (arguments
  \code{n.e}, \code{median.e}, \code{q1.e}, \code{q3.e},
  \code{min.e}, and \code{max.e}),
\item sample size, median and interquartile range (arguments
  \code{n.e}, \code{median.e}, \code{q1.e}, and \code{q3.e}), or
\item sample size, median and range (arguments \code{n.e},
  \code{median.e}, \code{min.e}, and \code{max.e}).
}

By default, methods described in Luo et al. (2018) are utilised
(argument \code{method.mean = "Luo"}):
\itemize{
\item equation (15) if sample size, median, interquartile range and 
  range are available,
\item equation (11) if sample size, median and interquartile range
  are available,
\item equation (7) if sample size, median and range are available.
}

Instead the methods described in Wan et al. (2014) are used if
argument \code{method.mean = "Wan"}:
\itemize{
\item equation (10) if sample size, median, interquartile range and 
  range are available,
\item equation (14) if sample size, median and interquartile range
  are available,
\item equation (2) if sample size, median and range are available.
}

The following methods are also available to estimate means from
quantiles or ranges if R package \bold{estmeansd} is installed:
\itemize{
\item Method for Unknown Non-Normal Distributions (MLN) approach
  (Cai et al. (2021), argument \code{method.mean = "Cai"}),
\item Quantile Estimation (QE) method (McGrath et al. (2020),
  argument \code{method.mean = "QE-McGrath"})),
\item Box-Cox (BC) method (McGrath et al. (2020),
  argument \code{method.mean = "BC-McGrath"})).
}

By default, missing means are replaced successively using
interquartile ranges and ranges (if available), interquartile
ranges (if available) and finally ranges. Arguments
\code{approx.mean.e} and \code{approx.mean.c} can be used to
overwrite this behaviour for each individual study and treatment
arm:
\itemize{
\item use means directly (entry \code{""} in argument
  \code{approx.mean.e} or \code{approx.mean.c});
\item median, interquartile range and range (\code{"iqr.range"});
\item median and interquartile range (\code{"iqr"});
\item median and range (\code{"range"}).
}
}

\subsection{Approximate standard deviations from sample sizes,
medians and other statistics}{

Missing standard deviations in the experimental group (analogously
for the control group) can be derived from
\enumerate{
\item sample size, median, interquartile range and range (arguments
  \code{n.e}, \code{median.e}, \code{q1.e}, \code{q3.e},
  \code{min.e}, and \code{max.e}),
\item sample size, median and interquartile range (arguments
  \code{n.e}, \code{median.e}, \code{q1.e} and \code{q3.e}), or
\item sample size, median and range (arguments \code{n.e},
  \code{median.e}, \code{min.e} and \code{max.e}).
}

Wan et al. (2014) describe methods to estimate the standard
deviation from the sample size, median and additional
statistics. Shi et al. (2020) provide an improved estimate of the
standard deviation if the interquartile range and range are
available in addition to the sample size and median. Accordingly,
equation (11) in Shi et al. (2020) is the default (argument
\code{method.sd = "Shi"}), if the median, interquartile range and
range are provided. The method by Wan et al. (2014) is used if
argument \code{method.sd = "Wan"} and, depending on the sample
size, either equation (12) or (13) is used. If only the
interquartile range or range is available, equations (15) / (16)
and (7) / (9) in Wan et al. (2014) are used, respectively.

The following methods are also available to estimate standard
deviations from quantiles or ranges if R package \bold{estmeansd}
is installed:
\itemize{
\item Method for Unknown Non-Normal Distributions (MLN) approach
  (Cai et al. (2021), argument \code{method.mean = "Cai"}),
\item Quantile Estimation (QE) method (McGrath et al. (2020),
  argument \code{method.mean = "QE-McGrath"})),
\item Box-Cox (BC) method (McGrath et al. (2020),
  argument \code{method.mean = "BC-McGrath"})).
}

By default, missing standard deviations are replaced successively
using these method, i.e., interquartile ranges and ranges are used
before interquartile ranges before ranges. Arguments
\code{approx.sd.e} and \code{approx.sd.c} can be used to overwrite
this default for each individual study and treatment arms:
\itemize{
\item sample size, median, interquartile range and range
  (\code{"iqr.range"});
\item sample size, median and interquartile range (\code{"iqr"});
\item sample size, median and range (\code{"range"}).
}
}

\subsection{Confidence intervals for individual studies}{

For the mean difference (argument \code{sm = "MD"}), the confidence
interval for individual studies can be based on the
\itemize{
\item standard normal distribution (\code{method.ci = "z"}, default), or
\item t-distribution (\code{method.ci = "t"}).
}

Note, this choice does not affect the results of the common effect
and random effects meta-analysis.
}

\subsection{Subgroup analysis}{

Argument \code{subgroup} can be used to conduct subgroup analysis for
a categorical covariate. The \code{\link{metareg}} function can be
used instead for more than one categorical covariate or continuous
covariates.
}

\subsection{Exclusion of studies from meta-analysis}{

Arguments \code{subset} and \code{exclude} can be used to exclude
studies from the meta-analysis. Studies are removed completely from
the meta-analysis using argument \code{subset}, while excluded
studies are shown in printouts and forest plots using argument
\code{exclude} (see Examples in \code{\link{metagen}}).
Meta-analysis results are the same for both arguments.
}

\subsection{Presentation of meta-analysis results}{

Internally, both common effect and random effects models are
calculated regardless of values choosen for arguments \code{common}
and \code{random}. Accordingly, the estimate for the random effects
model can be extracted from component \code{TE.random} of an object
of class \code{"meta"} even if argument \code{random =
FALSE}. However, all functions in R package \bold{meta} will
adequately consider the values for \code{common} and
\code{random}. E.g. function \code{\link{print.meta}} will not
print results for the random effects model if \code{random =
FALSE}.

A prediction interval will only be shown if \code{prediction =
TRUE}.
}
}
\note{
The function \code{\link{metagen}} is called internally to
calculate individual and overall treatment estimates and standard
errors.
}
\examples{
data(Fleiss1993cont)

# Meta-analysis with Hedges' g as effect measure
#
ma1 <- metacont(n.psyc, mean.psyc, sd.psyc, n.cont, mean.cont, sd.cont,
  data = Fleiss1993cont, sm = "SMD")
ma1
forest(ma1)

# Use Cohen's d instead of Hedges' g as effect measure
#
update(ma1, method.smd = "Cohen")

# Use Glass' delta instead of Hedges' g as effect measure
#
update(ma1, method.smd = "Glass")

# Use Glass' delta based on the standard deviation in the experimental group
#
update(ma1, method.smd = "Glass", sd.glass = "experimental")

# Calculate Hedges' g based on exact formulae
#
update(ma1, exact.smd = TRUE)

data(amlodipine)
ma2 <- metacont(n.amlo, mean.amlo, sqrt(var.amlo),
  n.plac, mean.plac, sqrt(var.plac),
  data = amlodipine, studlab = study)
ma2

# Use pooled variance
#
update(ma2, pooledvar = TRUE)

# Meta-analysis of response ratios (Hedges et al., 1999)
#
data(woodyplants)
ma3 <- metacont(n.elev, mean.elev, sd.elev, n.amb, mean.amb, sd.amb,
  data = woodyplants, sm = "ROM")
ma3
print(ma3, backtransf = FALSE)

}
\references{
Borenstein M, Hedges LV, Higgins JPT, Rothstein HR (2009):
\emph{Introduction to Meta-Analysis}.
Chichester: Wiley

Cai S, Zhou J, Pan J (2021):
Estimating the sample mean and standard deviation from order
statistics and sample size in meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{30}, 2701--2719

Cohen J (1988):
\emph{Statistical Power Analysis for the Behavioral Sciences
  (second ed.)}.
Lawrence Erlbaum Associates

Friedrich JO, Adhikari NK, Beyene J (2008):
The ratio of means method as an alternative to mean differences for
analyzing continuous outcome variables in meta-analysis: A
simulation study.
\emph{BMC Medical Research Methodology},
\bold{8}, 32

Glass G (1976):
Primary, secondary, and meta-analysis of research.
\emph{Educational Researcher},
\bold{5}, 3--8

Hedges LV (1981):
Distribution theory for Glass's estimator of effect size and
related estimators.
\emph{Journal of Educational and Behavioral Statistics},
\bold{6}, 107--28

Hedges LV, Gurevitch J, Curtis PS (1999):
The meta-analysis of response ratios in experimental ecology.
\emph{Ecology},
\bold{80}, 1150--6

Luo D, Wan X, Liu J, Tong T (2018):
Optimally estimating the sample mean from the sample size, median,
mid-range, and/or mid-quartile range.
\emph{Statistical Methods in Medical Research},
\bold{27}, 1785--805

McGrath S, Zhao X, Steele R, et al. and the DEPRESsion Screening
Data (DEPRESSD) Collaboration (2020):
Estimating the sample mean and standard deviation from commonly
reported quantiles in meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{29}, 2520--2537

\emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
The Cochrane Collaboration, 2020

Shi J, Luo D, Weng H, Zeng XT, Lin L, Chu H, Tong T (2020):
Optimally estimating the sample standard deviation from the
five-number summary.
\emph{Research Synthesis Methods},
\bold{11}, 641--54

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Wan X, Wang W, Liu J, Tong T (2014):
Estimating the sample mean and standard deviation from the sample
size, median, range and/or interquartile range.
\emph{BMC Medical Research Methodology},
\bold{14}, 135

White IR, Thomas J (2005):
Standardized mean differences in individually-randomized and
cluster-randomized trials, with applications to meta-analysis.
\emph{Clinical Trials},
\bold{2}, 141--51
}
\seealso{
\code{\link{meta-package}}, \code{\link{update.meta}},
  \code{\link{metabin}}, \code{\link{metagen}}, \code{\link{pairwise}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/amlodipine.R
\docType{data}
\name{amlodipine}
\alias{amlodipine}
\title{Amlodipine for Work Capacity}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{n.amlo}}\tab number of observations in amlodipine group
  \cr
\bold{\emph{mean.amlo}}\tab estimated mean in amlodipine group \cr
\bold{\emph{var.amlo}}\tab variance in amlodipine group \cr
\bold{\emph{n.plac}}\tab number of observations in placebo group
  \cr
\bold{\emph{mean.plac}}\tab estimated mean in placebo group \cr
\bold{\emph{var.plac}}\tab variance in placebo group
}
}
\source{
Hartung J & Knapp G (2001):
On tests of the overall treatment effect in meta-analysis with
normally distributed responses.
\emph{Statistics in Medicine},
\bold{20}, 1771--82
}
\description{
Meta-analysis on the effect of amlodipine on work capacity.

This meta-analysis is used as a data example in Hartung and Knapp (2001).
}
\examples{
data(amlodipine)

ma <- metacont(n.amlo, mean.amlo, sqrt(var.amlo),
  n.plac, mean.plac, sqrt(var.plac),
  data = amlodipine, studlab = study,
  method.tau = "DL")
ma.hk <- update(ma, method.random.ci = "HK")

# Same results for mean difference as in Table III in Hartung and
# Knapp (2001)
#
vars.common <- c("TE.common", "lower.common", "upper.common")
vars.random <- c("TE.random", "lower.random", "upper.random")
#
res.common <- as.data.frame(ma[vars.common])
names(res.common) <- vars.random
#
res.md <- rbind(res.common,
  as.data.frame(ma[vars.random]),
  as.data.frame(ma.hk[vars.random]))
#
res.md <- round(res.md, 5)
#
row.names(res.md) <- c("CE", "RE", "RE (HaKn)")
names(res.md) <- c("Absolute difference", "CI lower", "CI upper")
#
res.md
}
\seealso{
\code{\link{metacont}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read.rm5.R
\name{read.rm5}
\alias{read.rm5}
\alias{Fleiss1993_CR}
\title{Import RevMan 5 analysis data}
\usage{
read.rm5(
  file,
  sep = ",",
  quote = "\\"",
  title,
  numbers.in.labels = TRUE,
  debug = 0
)
}
\arguments{
\item{file}{The name of a file to read data values from.}

\item{sep}{The field separator character (only considered for
CSV-files). Values on each line of the file are separated by this
character. The comma is the default field separator character in
RevMan 5.}

\item{quote}{The set of quoting characters (only considered for
CSV-files). In RevMan 5 a "\"" is the default quoting character.}

\item{title}{Title of Cochrane review.}

\item{numbers.in.labels}{A logical indicating whether comparison
number and outcome number should be printed at the beginning of
the comparison (argument \code{complab}) and outcome label
(argument \code{outclab}); this is the default in RevMan 5.}

\item{debug}{An integer between 0 and 3 indicating whether to print
debug messages (only considered for RM5-files).}
}
\value{
A data frame containing the following components:
\item{comp.no}{Comparison number.}
\item{outcome.no}{Outcome number.}
\item{group.no}{Group number.}
\item{studlab}{Study label.}
\item{year}{Year of publication.}
\item{event.e}{Number of events in experimental group.}
\item{n.e}{Number of observations in experimental group.}
\item{event.c}{Number of events in control group.}
\item{n.c}{Number of observations in control group.}
\item{mean.e}{Estimated mean in experimental group.}
\item{sd.e}{Standard deviation in experimental group.}
\item{mean.c}{Estimated mean in control group.}
\item{sd.c}{Standard deviation in control group.}
\item{O.E}{Observed minus expected (IPD analysis).}
\item{V}{Variance of \code{O.E} (IPD analysis).}
\item{TE, seTE}{Estimated treatment effect and standard error of
  individual studies.}
\item{lower, upper}{Lower and upper limit of 95\% confidence
  interval for treatment effect in individual studies.}
\item{weight}{Weight of individual studies (according to
  meta-analytical method used in respective meta-analysis - see
  details).}
\item{order}{Ordering of studies.}
\item{grplab}{Group label.}
\item{type}{Type of outcome. D = dichotomous, C = continuous, P =
  IPD.}
\item{method}{A character string indicating which method has been
  used for pooling of studies. One of \code{"Inverse"},
  \code{"MH"}, or \code{"Peto"}.}
\item{sm}{A character string indicating which summary measure has
  been used for pooling of studies.}
\item{model}{A character string indicating which meta-analytical
  model has been used (either \code{"Fixed"} or \code{"Random"}).}
\item{common}{A logical indicating whether common effect
  meta-analysis has been used in respective meta-analysis (see
  details).}
\item{random}{A logical indicating whether random effects
  meta-analysis has been used in respective meta-analysis (see
  details).}
\item{outclab}{Outcome label.}
\item{k}{Total number of studies combined in respective
  meta-analysis).}
\item{event.e.pooled}{Number of events in experimental group in
  respective meta-analysis (see details).}
\item{n.e.pooled}{Number of observations in experimental group in
  respective meta-analysis (see details).}
\item{event.c.pooled}{Number of events in control group in
  respective meta-analysis (see details).}
\item{n.c.pooled}{Number of observations in control group in
  respective meta-analysis (see details).}
\item{TE.pooled}{Estimated treatment effect in respective
  meta-analysis (see details).}
\item{lower, upper}{Lower and upper limit of 95\% confidence
  interval for treatment effect in respective meta-analysis (see
  details).}
\item{weight.pooled}{Total weight in respective meta-analysis (see
  details).}
\item{Z.pooled}{Z-score for test of overall treatment effect in
  respective meta-analysis (see details).}
\item{pval.pooled}{P-value for test of overall treatment effect in
  respective meta-analysis (see details).}
\item{Q}{Heterogeneity statistic Q in respective meta-analysis (see
  details).}
\item{pval.Q}{P-value of heterogeneity statistic Q in respective
  meta-analysis (see details).}
\item{I2}{Heterogeneity statistic I\eqn{^2} in respective meta-analysis
  (see details).}
\item{tau2}{Between-study variance (moment estimator of
  DerSimonian-Laird) in respective meta-analysis.}
\item{Q.w}{Heterogeneity statistic Q within groups in respective
  meta-analysis (see details).}
\item{pval.Q.w}{P-value of heterogeneity statistic Q within groups
  in respective meta-analysis (see details).}
\item{I2.w}{Heterogeneity statistic I\eqn{^2} within groups in respective
  meta-analysis (see details).}
\item{label.e}{Label for experimental group.}
\item{label.c}{Label for control group.}
\item{label.left}{Graph label on left side of forest plot.}
\item{label.right}{Graph label on right side of forest plot.}
\item{complab}{Comparison label.}
}
\description{
Reads analysis data from Cochrane intervention review created with
RevMan 5 and creates a data frame from it.
}
\details{
Review Manager 5 (RevMan 5) was the software used for preparing and
maintaining Cochrane reviews. RevMan 5 includes the ability to write
systematic reviews of interventions, diagnostic test accuracy reviews,
methodology reviews and overviews of reviews.

This function provides the ability to read the analysis data from a
Cochrane intervention review created with RevMan 5; a data frame is
created from it. Cochrane intervention reviews are based on
comparisons of two interventions.

By default in RevMan 5, the name of the exported CSV data file is
the title of the Cochrane review. Furthermore, the title is part of
the RM5-file. Argument \code{title} can be used to overwrite the
title of the Cochrane review.

\subsection{Import RM5-file}{

A RM5-file (which is in a specific XML format) can be used directly
to import the analysis dataset.

If the import fails, use argument \code{debug = 3} for more details.
}

\subsection{Import CSV-file}{

In the past, the following (rather complicated) procedure based on
a CSV-file generated within RevMan 5 was necessary - which is only
described here for backward compatibility.

In order to generate a data analysis file in RevMan 5 use the
following Menu points: \code{"File"} - \code{"Export"} -
\code{"Data and analyses"}. It is mandatory to include the
following fields in the exported data file by selecting them with
the mouse cursor in the Export Analysis Data Wizard: (i) Comparison
Number, (ii) Outcome Number, (iii) Subgroup Number. When these
fields are not selected a corresponding error message will be
printed in R.  It is recommended to include all fields in the
exported data file except for the last field
"Risk of bias tables". For example, in order to redo the
meta-analysis in R for the RevMan 5 data type
\code{"O-E and Variance"} the fields \code{"O-E"} and
\code{"Variance"} have to be selected in the Export Analysis Data
Wizard. If the last field "Risk of bias tables" is selected the
import in R fails with an error message
"line X did not have Y elements".
}
}
\examples{
# Locate export data file "Fleiss1993_CR.csv"
# in sub-directory of package "meta"
#
filename <- system.file("extdata", "Fleiss1993_CR.csv", package = "meta")
Fleiss1993_CR <- read.rm5(filename)

# Same result as R command example(Fleiss1993bin):
#
metacr(Fleiss1993_CR)

# Same result as R command example(Fleiss1993cont):
#
metacr(Fleiss1993_CR, 1, 2)

\dontrun{
# Locate file "Fleiss1993.rm5" in sub-directory of R package meta
#
filename <- system.file("extdata/Fleiss1993.rm5", package = "meta")
Fleiss1993_CR <- read.cdir(filename)
Fleiss1993_CR

# Same result as R Command example(Fleiss1993bin):
#
metacr(Fleiss1993_CR)
}

}
\references{
\emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
The Cochrane Collaboration, 2020
}
\seealso{
\code{\link{summary.rm5}}, \code{\link{metabias.rm5}},
  \code{\link{metabin}}, \code{\link{metacont}},
  \code{\link{metagen}}, \code{\link{metacr}},
  \code{\link{print.rm5}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{datagen}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radial.R
\name{radial.meta}
\alias{radial.meta}
\alias{radial}
\alias{radial.default}
\title{Radial plot}
\usage{
\method{radial}{meta}(
  x,
  xlim = NULL,
  ylim = NULL,
  xlab = "Inverse of standard error",
  ylab = "Standardised treatment effect (z-score)",
  common = TRUE,
  axes = TRUE,
  pch = 1,
  text = NULL,
  cex = 1,
  col = NULL,
  level = NULL,
  warn.deprecated = gs("warn.deprecated"),
  fixed,
  ...
)

\method{radial}{default}(
  x,
  y,
  xlim = NULL,
  ylim = NULL,
  xlab = "Inverse of standard error",
  ylab = "Standardised treatment effect (z-score)",
  common = TRUE,
  axes = TRUE,
  pch = 1,
  text = NULL,
  cex = 1,
  col = NULL,
  level = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}, or estimated treatment
effect in individual studies.}

\item{xlim}{The x limits (min, max) of the plot.}

\item{ylim}{The y limits (min, max) of the plot.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{common}{A logical indicating whether the pooled common effect
estimate should be plotted.}

\item{axes}{A logical indicating whether axes should be drawn on
the plot.}

\item{pch}{The plotting symbol used for individual studies.}

\item{text}{A character vector specifying the text to be used
instead of plotting symbol.}

\item{cex}{The magnification to be used for plotting symbol.}

\item{col}{A vector with colour of plotting symbols.}

\item{level}{The confidence level utilised in the plot.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{fixed}{Deprecated argument (replaced by 'common').}

\item{\dots}{Graphical arguments as in \code{par} may also be
passed as arguments.}

\item{y}{Standard error of estimated treatment effect.}
}
\description{
Draw a radial plot (also called Galbraith plot) which can be used
to assess bias in meta-analysis.
}
\details{
A radial plot (Galbraith 1988a,b), also called Galbraith plot, is
drawn in the active graphics window. If \code{common} is TRUE, the
pooled estimate of the common effect model is plotted. If
\code{level} is not NULL, the corresponding confidence limits are
drawn.
}
\examples{
data(Olkin1995)
ma <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = c(41, 47, 51, 59),
  studlab = paste(author, year), sm = "RR", method = "I")

# Radial plot
#
radial(ma, level = 0.95)

}
\references{
Galbraith RF (1988a):
Graphical display of estimates having differing standard errors.
\emph{Technometrics},
\bold{30}, 271--81

Galbraith RF (1988b):
A note on graphical presentation of estimated odds ratios from
several clinical trials.
\emph{Statistics in Medicine},
\bold{7}, 889--94
}
\seealso{
\code{\link{metabias}}, \code{\link{metabin}},
  \code{\link{metagen}}, \code{\link{funnel}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Fleiss1993bin.R
\docType{data}
\name{Fleiss1993bin}
\alias{Fleiss1993bin}
\alias{Fleiss93}
\title{Aspirin after Myocardial Infarction}
\format{
A data frame with the following columns:
\tabular{rl}{
\bold{\emph{study}}\tab study label \cr
\bold{\emph{year}}\tab year of publication \cr
\bold{\emph{d.asp}}\tab number of deaths in aspirin group \cr
\bold{\emph{n.asp}}\tab number of observations in aspirin group \cr
\bold{\emph{d.plac}}\tab number of deaths in placebo group \cr
\bold{\emph{n.plac}}\tab number of observations in placebo group
}
}
\source{
Fleiss JL (1993):
The statistical basis of meta-analysis.
\emph{Statistical Methods in Medical Research},
\bold{2}, 121--45
}
\description{
Meta-analysis on aspirin in preventing death after myocardial
infarction.

Data example in Fleiss (1993) for meta-analysis with binary
outcomes.
}
\examples{
data(Fleiss1993bin)
metabin(d.asp, n.asp, d.plac, n.plac, data = Fleiss1993bin,
  studlab = paste(study, year), sm = "OR", random = FALSE)
}
\seealso{
\code{\link{metabin}}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nnt.R
\name{nnt}
\alias{nnt}
\alias{nnt.default}
\alias{nnt.meta}
\alias{print.nnt.meta}
\alias{print.nnt.default}
\title{Calculate the number needed to treat (NNT)}
\usage{
nnt(x, ...)

\method{nnt}{meta}(
  x,
  p.c,
  common = x$common,
  random = x$random,
  small.values = "desirable",
  ...
)

\method{nnt}{default}(x, p.c, sm, lower, upper, small.values = "desirable", transf = FALSE, ...)

\method{print}{nnt.meta}(
  x,
  common = x$common,
  random = x$random,
  digits = gs("digits"),
  digits.prop = gs("digits.prop"),
  big.mark = gs("big.mark"),
  ...
)

\method{print}{nnt.default}(
  x,
  digits = gs("digits"),
  digits.prop = gs("digits.prop"),
  big.mark = gs("big.mark"),
  ...
)
}
\arguments{
\item{x}{An object of class \code{meta}, or estimated treatment
effect(s), i.e., risk difference(s), risk ratio(s), odds
ratio(s), or hazard ratio(s).}

\item{\dots}{Additional arguments (to catch deprecated arguments).}

\item{p.c}{Baseline probability, i.e., control group event probability
for binary outcomes or survival probability in control group for
hazard ratios.}

\item{common}{A logical indicating whether NNTs should be
calculated based on common effect estimate.}

\item{random}{A logical indicating whether NNTs should be
calculated based on random effects estimate.}

\item{small.values}{A character string specifying whether small
treatment effects indicate a beneficial (\code{"desirable"}) or
harmful (\code{"undesirable"}) effect, can be abbreviated.}

\item{sm}{Summary measure.}

\item{lower}{Lower confidence interval limit.}

\item{upper}{Upper confidence interval limit.}

\item{transf}{A logical indicating whether treatment estimates and
confidence limits are transformed or on the original scale.
If \code{transf = TRUE}, inputs are expected to be log odds ratios instead
of odds ratios for \code{sm = "OR"}, for example.}

\item{digits}{Minimal number of significant digits to print NNT and its
confidence interval, see \code{print.default}.}

\item{digits.prop}{Minimal number of significant digits for
proportions, see \code{print.default}.}

\item{big.mark}{A character used as thousands separator.}
}
\description{
Calculate the number needed to treat (NNT) from the estimated risk
difference, risk ratio, odds ratio, or hazard ratio, and a baseline
probability, i.e., control group event probability for binary outcomes or
survival probability for hazard ratios.
}
\details{
The number needed to treat (NNT) is the estimated number of
patients who need to be treated with a new treatment instead of a
standard for one additional patient to benefit (Laupacis et al.,
1988; Cook & Sackett, 1995). This definition of the NNT implies
that the new treatment is more beneficial than the standard. If the
new treatment is indeed less beneficial than the standard, the NNT
gives the number of patients treated with the new treatment to
observe an additional harmful event. Accordingly, the abbreviations
NNTB and NNTH can be used to distinguish between beneficial and
harmful NNTs (Altman, 1998).

NNTs can be easily computed from an estimated risk difference (RD),
risk ratio (RR), or odds ratio (OR) and a given baseline probability
(Higgins et al., 2023, section 15.4.4). It is also possible to
calculate NNTs from hazard ratios (HR) (Altman & Andersen,
1999). Accordingly, NNTs can be calculated for meta-analyses
generated with \code{\link{metabin}} or \code{\link{metagen}} if
argument \code{sm} was equal to \code{"RD"}, \code{"RR"},
\code{"OR"}, or \code{"HR"}. It is also possible to provide only
estimated treatment effects and baseline probabilities (see Examples).

The baseline probability can be specified using argument \code{p.c}. If
this argument is missing, the minimum, mean, and maximum of the
control event probabilities in the meta-analysis are used for
\code{\link{metabin}} and control event probabilities of 0.1, 0.2,
\dots, 0.9 are used for \code{\link{metagen}}. Note, the survival instead of
mortality probability must be provided for hazard ratios.

Argument \code{small.values} can be used to specify whether small
treatment effects indicate a beneficial (\code{"desirable"}) or harmful
(\code{"undesirable"}) effect. For \code{small.values = "desirable"}, odds,
risk and hazard ratios below 1 and risk differences below 0 indicate that the
new treatment is beneficial. For \code{small.values = "undesirable"}, odds,
risk and hazard ratios above 1 and risk differences above 0 indicate that
the new treatment is beneficial.

\subsection{Interpretation of (positive and negative) NNTs}{
A positive value for the estimated NNT indicates that the new
treatment is beneficial, i.e., the NNT is actually an NNTB. On the
other hand, a negative value for the estimated NNT indicates that
the new treatment is harmful, i.e., the NNT is actually an NNTH.

The minimal value for the NNTB is 1. In this extreme case the new
treatment is 100\% effective and the standard treatment is 0\%
effective, i.e., only one patient has to be treated with the new
treatment for one additional patient to benefit. The NNTB increases
with decreasing difference between the two risks. If both risks are
equal, the NNTB is infinite.

The other extreme is an NNT of -1 if the new treatment is 0\%
effective and the standard is 100\% effective. Here, one additional
harmful event is observed for each patient treated with the new
treatment. The NNT approaches minus infinity if the difference
between the two risks decreases to 0. Finally, an NNT of -1
translates to an NNTH of 1 with possible values from 1 to infinity.
}

\subsection{Confidence interval for the NNT}{
Confidence limits for an NNT are derived from the lower and upper
confidence limits of the summary measure using the same formulae as
for the NNT (Higgins et al., 2023, section 15.4.4).

A peculiar problem arises if the confidence interval for the
summary measure includes the null effect (i.e., RR = 1, OR = 1, HR
= 1, or RD = 0). In this case the confidence interval for the NNT
contains both NNTB and NNTH values and it seemingly does not
include the estimated NNT.

As described above, a positive NNT value corresponds to an NNTB and
the absolute value of a negative NNT is equal to an
NNTH. Accordingly, a confidence interval for the NNT from 20 to -5
translates to NNTB values between 20 and infinity and NNTH values
between 5 and infinity (Altman, 1998).
}
}
\examples{
# Calculate NNTs for risk differences of -0.12 and -0.22
# (Cochrane Handbook, version 6.3, subsection 15.4.4.1)
nnt(c(-0.12, -0.22), sm = "RD")

# Calculate NNT for risk ratio of 0.92 and baseline risk of 0.3
# (Cochrane Handbook, version 6.3, subsection 15.4.4.2)
nnt(0.92, p.c = 0.3, sm = "RR")

# Calculate NNT for odds ratio of 0.73 and baseline risk of 0.3
# (Cochrane Handbook, version 6.3, subsection 15.4.4.3)
nnt(0.73, p.c = 0.3, sm = "OR")

# Calculate NNTs for Mantel-Haenszel odds ratio
data(Olkin1995)
ma <-
  metabin(ev.exp, n.exp, ev.cont, n.cont, data = Olkin1995, random = FALSE)
nnt(ma)

# Calculate NNTs from hazard ratio at two and four years (example from
# Altman & Andersen, 1999). Note, argument 'p.c' must provide survival
# probabilities instead of mortality rates for the control group.
nnt(0.72, lower = 0.55, upper = 0.92, sm = "HR", p.c = 1 - c(0.33, 0.49))

}
\references{
Altman DG (1998):
Confidence intervals for the number needed to treat.
\emph{British Medical Journal},
\bold{317}, 1309--12

Altman DG, Andersen PK (1999):
Calculating the number needed to treat for trials where the outcome
is time to an event.
\emph{British Medical Journal},
\bold{319}, 1492--95

Cook RJ, Sackett DL (1995):
The number needed to treat: a clinically useful measure of
treatment effect.
\emph{British Medical Journal},
\bold{310}, 452--54

Higgins JPT, Thomas J, Chandler J, Cumpston M, Li T, Page MJ, Welch
VA (editors) (2023):
\emph{Cochrane Handbook for Systematic Reviews of Interventions
  Version 6.4 (updated August 2023)}.
Available from \url{https://www.cochrane.org/authors/handbooks-and-manuals/handbook}

Laupacis A, Sackett DL, Roberts RS (1988):
An assessment of clinically useful measures of the consequences of
treatment.
\emph{New England Journal of Medicine},
\bold{318}, 1728--33
}
\seealso{
\code{\link{metabin}}, \code{\link{metagen}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta-package.R
\docType{package}
\name{meta-package}
\alias{meta-package}
\alias{meta}
\title{meta: Brief overview of methods and general hints}
\description{
R package \bold{meta} is a user-friendly general package providing
standard methods for meta-analysis and supporting Schwarzer et
al. (2015),
\url{https://link.springer.com/book/10.1007/978-3-319-21416-0}.
}
\details{
R package \bold{meta} (Schwarzer, 2007; Balduzzi et al., 2019)
provides the following statistical methods for meta-analysis.
\enumerate{
\item Common effect (also called fixed effect) and random effects model:
\itemize{
 \item Meta-analysis of continuous outcome data (\code{\link{metacont}})
 \item Meta-analysis of binary outcome data (\code{\link{metabin}})
 \item Meta-analysis of incidence rates (\code{\link{metainc}})
 \item Generic inverse variance meta-analysis (\code{\link{metagen}})
 \item Meta-analysis of single correlations (\code{\link{metacor}})
 \item Meta-analysis of single means (\code{\link{metamean}})
 \item Meta-analysis of single proportions (\code{\link{metaprop}})
 \item Meta-analysis of single incidence rates (\code{\link{metarate}})
}
\item Several plots for meta-analysis:
\itemize{
 \item Forest plot (\code{\link{forest.meta}}, \code{\link{forest.metabind}})
 \item Funnel plot (\code{\link{funnel.meta}})
 \item Galbraith plot / radial plot (\code{\link{radial.meta}})
 \item L'Abbe plot for meta-analysis with binary outcome data
  (\code{\link{labbe.metabin}}, \code{\link{labbe.default}})
 \item Baujat plot to explore heterogeneity in meta-analysis
  (\code{\link{baujat.meta}})
 \item Bubble plot to display the result of a meta-regression
  (\code{\link{bubble.metareg}})
}
\item Three-level meta-analysis model (Van den Noortgate et al.,
  2013)
\item Generalised linear mixed models (GLMMs) for binary and count
  data (Stijnen et al., 2010) (\code{\link{metabin}},
  \code{\link{metainc}}, \code{\link{metaprop}}, and
  \code{\link{metarate}})
\item Logistic regression with penalised likelihood for rare binary data
  (Evrenoglou et al., 2022) (\code{\link{metabin}},
\item Various estimators for the between-study variance
 \eqn{\tau^2} in a random effects model (Veroniki et al., 2016);
 see description of argument \code{method.tau} below
\item Two methods to estimate the I-squared statistic
  (Higgins and Thompson, 2002); see description of argument
  \code{method.I2} below
\item Hartung-Knapp method for random effects meta-analysis
 (Hartung & Knapp, 2001a,b), see description of arguments
 \code{method.random.ci} and \code{adhoc.hakn.ci} below
\item Kenward-Roger method for random effects meta-analysis
 (Partlett and Riley, 2017), see description of arguments
 \code{method.random.ci} and \code{method.predict} below
\item Inverse variance heterogeneity method (Doi et al., 2015),
 see description of argument \code{method.common.ci} below
\item Prediction interval for the treatment effect of a new study
 (Veroniki et al., 2019; Higgins et al., 2009; Partlett and Riley, 2017;
 Nagashima et al., 2019), see description of argument \code{method.predict}
 below
\item Expected proportion of comparable studies with clinically important
 benefit or harm derived from the prediction distribution
 (Siemens et al., 2025)
\item Statistical tests for funnel plot asymmetry
 (\code{\link{metabias.meta}}, \code{\link{metabias.rm5}}) and
 trim-and-fill method (\code{\link{trimfill.meta}},
 \code{\link{trimfill.default}}) to evaluate bias in meta-analysis
\item Meta-regression (\code{\link{metareg}})
\item Cumulative meta-analysis (\code{\link{metacum}}) and
  leave-one-out meta-analysis (\code{\link{metainf}})
\item Import data from RevMan Web (\code{\link{read.cdir}}), RevMan
  5 (\code{\link{read.rm5}}), see also \code{\link{metacr}} to
  conduct meta-analysis for a single comparison and outcome from a
  Cochrane review
}

R package \bold{meta} provides two vignettes:
\itemize{
 \item \code{vignette("meta-workflow")} with an overview of main
   functions,
 \item \code{vignette("meta-tutorial")} with up-to-date commands for
   Balduzzi et al. (2019).
}

Additional statistical meta-analysis methods are provided by add-on
R packages:
\itemize{
\item Frequentist methods for network meta-analysis (R package
  \bold{netmeta})
\item Statistical methods for sensitivity analysis in meta-analysis
  (R package \bold{metasens})
\item Statistical methods for meta-analysis of diagnostic accuracy
  studies with several cutpoints (R package \bold{diagmeta})
}

In the following, more details on available and default statistical
meta-analysis methods are provided and R function
\code{\link{settings.meta}} is briefly described which can be used
to change the default settings. Additional information on
meta-analysis objects and available summary measures can be found
on the help pages \code{\link{meta-object}} and
\code{\link{meta-sm}}.

\subsection{Estimation of between-study variance}{

The following methods are available in all meta-analysis functions
to estimate the between-study variance \eqn{\tau^2}.
\tabular{ll}{
\bold{Argument} \tab \bold{Method} \cr 
\code{method.tau = "REML"}
 \tab Restricted maximum-likelihood estimator (Viechtbauer, 2005) \cr
\tab (default) \cr
\code{method.tau = "PM"}
 \tab Paule-Mandel estimator (Paule and Mandel, 1982) \cr
\code{method.tau = "DL"}
 \tab DerSimonian-Laird estimator (DerSimonian and Laird, 1986) \cr
\code{method.tau = "ML"}
 \tab Maximum-likelihood estimator (Viechtbauer, 2005) \cr
\code{method.tau = "HS"}
 \tab Hunter-Schmidt estimator (Hunter and Schmidt, 2015) \cr
\code{method.tau = "SJ"}
 \tab Sidik-Jonkman estimator (Sidik and Jonkman, 2005) \cr
\code{method.tau = "HE"}
 \tab Hedges estimator (Hedges and Olkin, 1985) \cr
\code{method.tau = "EB"}
 \tab Empirical Bayes estimator (Morris, 1983)
}

For GLMMs, only the maximum-likelihood method is available.

Historically, the DerSimonian-Laird method was the de facto
standard to estimate the between-study variance \eqn{\tau^2} and is
the default in some software packages including Review Manager 5
(RevMan 5) and R package \bold{meta}, version 4 and below. However,
its role has been challenged and especially the REML and
Paule-Mandel estimators have been recommended (Veroniki et al.,
2016; Langan et al., 2019). Accordingly, the currenct default in R
package \bold{meta} is the REML estimator.

The following R command could be used to employ the Paule-Mandel
instead of the REML estimator in all meta-analyses of the current R
session:
\itemize{
\item \code{settings.meta(method.tau = "PM")}
}

Other estimators for \eqn{\tau^2} could be selected in a similar
way.

Note, for binary outcomes, two variants of the DerSimonian-Laird
estimator are available if the Mantel-Haenszel method is used for
pooling. If argument \code{Q.Cochrane = TRUE} (default), the
heterogeneity statistic Q is based on the Mantel-Haenszel instead
of the inverse variance estimator under the common effect
model. This is the estimator for \eqn{\tau^2} implemented in RevMan
5.
}


\subsection{Estimation of I-squared statistic}{

The following methods are available in all meta-analysis functions
to estimate the I-squared statistic (Higgins and Thompson, 2002).
\tabular{ll}{
\bold{Argument} \tab \bold{Method} \cr 
\code{method.I2 = "Q"}
 \tab Based on heterogeneity statistic Q (default) \cr
\code{method.I2 = "tau2"}
 \tab Based on between-study variance \eqn{\tau^2}
}

Using \code{method.I2 = "Q"} (Higgins and Thompson, 2002, section 3.3), the
value of I\eqn{^2} does not change if the estimate of \eqn{\tau^2} changes.
Furthermore, the value of I\eqn{^2} and the test of heterogeneity based on
the Q statistic are in agreement. R package \bold{metafor} uses the second
method (\code{method.I2 = "tau2"}) which is described in Higgins and Thompson
(2002), section 3.2. This method is more general in the way that the value
of I\eqn{^2} changes with the estimate of \eqn{\tau^2}.
}

\subsection{Confidence interval for common effect estimate}{

The following methods are available in all meta-analysis functions
to calculate a confidence interval for the common effect estimate.
\tabular{ll}{
\bold{Argument} \tab \bold{Method} \cr
\code{method.common.ci = "classic"} \tab Based on standard normal
  quantile \cr
\code{method.common.ci = "IVhet"} \tab Method by Doi et al. (2015) going
  back to Henmi and Copas (2010) \cr
}

The inverse variance heterogeneity method (IVhet) uses the weights under the
random effects model to calculate the confidence interval. This method is
only available if argument \code{method = "Inverse"}.
}

\subsection{Confidence interval for random effects estimate}{

The following methods are available in all meta-analysis functions
to calculate a confidence interval for the random effects estimate.
\tabular{ll}{
\bold{Argument} \tab \bold{Method} \cr
\code{method.random.ci = "classic"} \tab Based on standard normal
  quantile \cr
\tab (DerSimonian and Laird, 1986) (default) \cr
\code{method.random.ci = "HK"} \tab Method by Hartung and Knapp
  (2001a/b) \cr
\code{method.random.ci = "KR"} \tab Kenward-Roger method (Partlett and
  Riley, 2017)
}

DerSimonian and Laird (1986) introduced the classic random effects
model using a quantile of the standard normal distribution to
calculate a confidence interval for the random effects
estimate. This method implicitly assumes that the weights in the
random effects meta-analysis are not estimated but
given. Particularly, the uncertainty in the estimation of the
between-study variance \eqn{\tau^2} is ignored.

Hartung and Knapp (2001a,b) proposed an alternative method for
random effects meta-analysis based on a refined variance estimator
for the treatment estimate and a quantile of a
\emph{t}-distribution with \emph{k-1} degrees of freedom where
\emph{k} corresponds to the number of studies in the
meta-analysis.

The Kenward-Roger method is only available for the REML estimator
(\code{method.tau = "REML"}) of the between-study variance
\eqn{\tau^2} (Partlett and Riley, 2017). This method is based on an
adjusted variance estimate for the random effects
estimate. Furthermore, a quantile of a \emph{t}-distribution with
adequately modified degrees of freedom is used to calculate the
confidence interval.

For GLMMs and three-level models, the Kenward-Roger method is not
available, but a method similar to Knapp and Hartung (2003) is used
if \code{method.random.ci = "HK"}. For this method, the variance
estimator is not modified, however, a quantile of a
\emph{t}-distribution with \emph{k-1} degrees of freedom is used;
see description of argument \code{test} in
\code{\link[metafor]{rma.glmm}} and \code{\link[metafor]{rma.mv}}.

Simulation studies (Hartung and Knapp, 2001a,b; IntHout et al.,
2014; Langan et al., 2019) show improved coverage probabilities of
the Hartung-Knapp method compared to the classic random effects
method. However, in rare settings with very homogeneous treatment
estimates, the Hartung-Knapp variance estimate can be arbitrarily
small resulting in a very narrow confidence interval (Knapp and
Hartung, 2003; Wiksten et al., 2016). In such cases, an \emph{ad
hoc} variance correction has been proposed by utilising the
variance estimate from the classic random effects model with the
Hartung-Knapp method (Knapp and Hartung, 2003; IQWiQ, 2022). An
alternative \emph{ad hoc} approach is to use the confidence
interval of the classic common or random effects meta-analysis if
it is wider than the interval from the Hartung-Knapp method
(Wiksten et al., 2016; Jackson et al., 2017).

Argument \code{adhoc.hakn.ci} can be used to choose the \emph{ad
hoc} correction for the Hartung-Knapp (HK) method:
\tabular{ll}{
\bold{Argument} \tab \bold{\emph{Ad hoc} method} \cr
\code{adhoc.hakn.ci = ""} \tab no \emph{ad hoc} correction (default)
  \cr
\code{adhoc.hakn.ci = "se"} \tab use variance correction if HK standard
 error is smaller \cr
 \tab than standard error from classic random effects
 \cr
 \tab meta-analysis (Knapp and Hartung, 2003) \cr
\code{adhoc.hakn.ci = "IQWiG6"} \tab use variance correction if HK
 confidence interval \cr
 \tab is narrower than CI from classic random effects model \cr
 \tab with DerSimonian-Laird estimator (IQWiG, 2022) \cr
\code{adhoc.hakn.ci = "ci"} \tab use wider confidence interval of
 classic random effects \cr
 \tab and HK meta-analysis \cr
 \tab (Hybrid method 2 in Jackson et al., 2017)
}

For GLMMs and three-level models, the \emph{ad hoc} variance
corrections are not available.
}

\subsection{Prediction interval}{

The following methods are available in all meta-analysis functions
to calculate a prediction interval for the treatment effect in a
single new study.
\tabular{ll}{
\bold{Argument} \tab \bold{Method} \cr
\code{method.predict = "V"} \tab Based on \emph{t}-distribution
  with \emph{k-1} degrees of freedom \cr
  \tab (Veroniki et al., 2019) (default) \cr
\code{method.predict = "HTS"} \tab Based on \emph{t}-distribution
  with \emph{k-2} degrees of freedom \cr
  \tab (Higgins et al., 2009) \cr
\code{method.predict = "HK"} \tab Based on Hartung-Knapp standard error
  and \cr
  \tab \emph{t}-distribution with \emph{k-1} degrees of freedom \cr
\code{method.predict = "HK-PR"} \tab Based on Hartung-Knapp standard error
  and \cr
  \tab \emph{t}-distribution with \emph{k-2} degrees of freedom \cr
  \tab (Partlett and Riley, 2017) \cr
\code{method.predict = "KR"} \tab Based on Kenward-Roger standard error
  and \cr
  \tab \emph{t}-distribution with approximate Kenward-Roger \cr
  \tab degrees of freedom \cr
\code{method.predict = "KR-PR"} \tab Based on Kenward-Roger standard error
  and \cr
  \tab \emph{t}-distribution with approximate Kenward-Roger \cr
  \tab degrees of freedom minus 1 (Partlett and Riley, 2017) \cr
\code{method.predict = "NNF"} \tab Bootstrap approach (Nagashima et
  al., 2019) \cr
\code{method.predict = "S"} \tab Based on standard normal quantile
  (Skipka, 2006)
}

By default (\code{method.predict = "V"}), the prediction interval
is based on a \emph{t}-distribution with \emph{k-1} degrees of
freedom where \emph{k} corresponds to the number of studies in the
meta-analysis (Veroniki et al., 2019). The method by Higgins et al., (2009),
which is based on a \emph{t}-distribution with \emph{k-2} degrees of freedom,
has been the default in R package \bold{meta}, version 7.0-0 or lower.

The Hartung-Knapp prediction intervals are also based on a
\emph{t}-distribution, however, use a different standard error.

The Kenward-Roger method is only available for the REML estimator
(\code{method.tau = "REML"}) of the between-study variance
\eqn{\tau^2} (Partlett and Riley, 2017). This method is based on an
adjusted variance estimate for the random effects
estimate. Furthermore, a quantile of a \emph{t}-distribution with
adequately modified degrees of freedom is used to calculate the prediction
interval.

The bootstrap approach is only available if R package \bold{pimeta}
is installed (Nagashima et al., 2019). Internally, the
\code{\link[pimeta]{pima}} function is called with argument
\code{method = "boot"}. Argument \code{seed.predict} can be used to
get a reproducible bootstrap prediction interval and argument
\code{seed.predict.subgroup} for reproducible bootstrap prediction
intervals in subgroups.

The method of Skipka (2006) ignores the uncertainty in the
estimation of the between-study variance \eqn{\tau^2} and thus has
too narrow limits for meta-analyses with a small number of studies.

For GLMMs and three-level models, only the methods by Veroniki et al.
(2019), Higgins et al. (2009) and Skipka (2006) are available. Argument
\code{method.predict = "V"} in R package \bold{meta} gives the same
prediction intervals as R functions \code{\link[metafor]{rma.glmm}} or
\code{\link[metafor]{rma.mv}} with argument \code{test = "t"}.

Note, in R package \bold{meta}, version 7.0-0 or lower, the methods
\code{method.predict = "HK-PR"} and \code{method.predict = "KR-PR"} have
been available as \code{method.predict = "HK"} and
\code{method.predict = "KR"}.

Argument \code{adhoc.hakn.pi} can be used to choose the \emph{ad
hoc} correction for the Hartung-Knapp method:

\tabular{ll}{
\bold{Argument} \tab \bold{\emph{Ad hoc} method} \cr
\code{adhoc.hakn.pi = ""} \tab no \emph{ad hoc} correction (default)
  \cr
\code{adhoc.hakn.pi = "se"} \tab use variance correction if HK
 standard error is smaller
}
}

\subsection{Confidence interval for the between-study variance}{

The following methods are available in all meta-analysis functions
to calculate a confidence interval for \eqn{\tau^2} and \eqn{\tau}.
\tabular{ll}{
\bold{Argument} \tab \bold{Method} \cr 
\code{method.tau.ci = "J"} \tab Method by Jackson (2013) \cr
\code{method.tau.ci = "BJ"} \tab Method by Biggerstaff and Jackson (2008) \cr
\code{method.tau.ci = "QP"} \tab Q-Profile method (Viechtbauer, 2007) \cr
\code{method.tau.ci = "PL"} \tab Profile-Likelihood method for three-level \cr
 \tab meta-analysis model (Van den Noortgate et al., 2013) \cr
\code{method.tau.ci = ""} \tab No confidence interval
}
The first three methods have been recommended by Veroniki et
al. (2016). By default, the Jackson method is used for the
DerSimonian-Laird estimator of \eqn{\tau^2} and the Q-profile
method for all other estimators of \eqn{\tau^2}.

The Profile-Likelihood method is the only method available for the
three-level meta-analysis model.

For GLMMs, no confidence intervals for \eqn{\tau^2} and \eqn{\tau}
are calculated.
}

\subsection{Change default settings for R session}{

R function \code{\link{settings.meta}} can be used to change the
previously described and several other default settings for the
current R session.

Some pre-defined general settings are available:
\itemize{
\item \code{settings.meta("RevMan5")}
\item \code{settings.meta("JAMA")}
\item \code{settings.meta("BMJ")}
\item \code{settings.meta("IQWiG5")}
\item \code{settings.meta("IQWiG6")}
\item \code{settings.meta("geneexpr")}
}

The first command can be used to reproduce meta-analyses from
Cochrane reviews conducted with \emph{Review Manager 5} (RevMan 5)
and specifies to use a RevMan 5 layout in forest plots.

The second command can be used to generate forest plots following
instructions for authors of the \emph{Journal of the American
Medical Association}. Study labels according to JAMA guidelines can be
generated using \code{\link{labels.meta}}.

The third command can be used to generate forest plots in the current layout
of the \emph{British Medical Journal}.

The next two commands implement the recommendations of the
Institute for Quality and Efficiency in Health Care (IQWiG),
Germany accordinging to General Methods 5 and 6, respectively
(\url{https://www.iqwig.de/en/about-us/methods/methods-paper/}).

The last setting can be used to print p-values in scientific
notation and to suppress the calculation of confidence intervals
for the between-study variance.

See \code{\link{settings.meta}} for more details on these
pre-defined general settings.

In addition, \code{\link{settings.meta}} can be used to define
individual settings for the current R session. For example, the
following R command specifies the use of Hartung-Knapp and
Paule-Mandel method, and the printing of prediction intervals for
any meta-analysis generated after execution of this command:
\itemize{
\item \code{settings.meta(method.random.ci = "HK", method.tau =
  "PM", prediction = TRUE)}
}
}

\subsection{Data sets}{
The following data sets are available in R package \bold{meta}.

\tabular{ll}{
\bold{Data set} \tab \bold{Description} \cr
\code{\link{Fleiss1993bin}} \tab Aspirin after myocardial infarction \cr
\code{\link{Fleiss1993cont}} \tab Mental health treatment on medical utilisation\cr
\code{\link{Olkin1995}} \tab Thrombolytic therapy after acute myocardial infarction \cr
\code{\link{Pagliaro1992}} \tab Prevention of first bleeding in cirrhosis \cr
\code{\link{amlodipine}} \tab Amlodipine for work capacity \cr
\code{\link{caffeine}} \tab Caffeine for daytime drowsiness (Cochrane Practice review) \cr
\code{\link{cisapride}} \tab Cisapride in non-ulcer dispepsia \cr
\code{\link{lungcancer}} \tab Smoking example \cr
\code{\link{smoking}} \tab Smoking example \cr
\code{\link{woodyplants}} \tab Elevated CO$_2$ and total biomass of woody plants
}

\bold{R} packages \bold{metabook} and \bold{metadat} have a large collection
 of meta-analysis data sets.
}
}
\note{
Balduzzi et al. (2019) is the preferred citation in publications
for \bold{meta}. Type \code{citation("meta")} for a BibTeX entry of
this publication.

Type \code{help(package = "meta")} for a listing of all R functions
and datasets available in \bold{meta}. For example, results of
several meta-analyses can be combined with \code{\link{metabind}}
which is useful to generate a forest plot with results of several
subgroup analyses.

R package \bold{meta} imports R functions from \bold{metafor}
(Viechtbauer, 2010) to
\itemize{
\item estimate the between-study variance \eqn{\tau^2},
\item conduct meta-regression,
\item estimate three-level models,
\item estimate generalised linear mixed models.
}

To report problems and bugs
\itemize{
\item type \code{bug.report(package = "meta")} if you do not use
  RStudio,
\item send an email to Guido Schwarzer
  \email{guido.schwarzer@uniklinik-freiburg.de} if you use RStudio.
}

The development version of \bold{meta} is available on GitHub
\url{https://github.com/guido-s/meta/}.
}
\references{
Balduzzi S, Rücker G, Schwarzer G (2019):
How to perform a meta-analysis with R: a practical tutorial.
\emph{Evidence-Based Mental Health},
\bold{22}, 153--160

Biggerstaff BJ, Jackson D (2008):
The exact distribution of Cochran’s heterogeneity statistic in
one-way random effects meta-analysis.
\emph{Statistics in Medicine},
\bold{27}, 6093--110

DerSimonian R & Laird N (1986):
Meta-analysis in clinical trials.
\emph{Controlled Clinical Trials},
\bold{7}, 177--88

Doi SAR, Barendregt JJ, Khan S, Thalib L, Williams GM (2015):
Advances in the meta-analysis of heterogeneous clinical trials I:
The inverse variance heterogeneity model.
\emph{Contemporary Clinical Trials},
\bold{45}, 130--8

Evrenoglou T, White IR, Afach S, Mavridis D, Chaimani A. (2022):
Network meta-analysis of rare events using penalized likelihood regression.
\emph{Statistics in Medicine},
\bold{41}, 5203--19

Hartung J, Knapp G (2001a):
On tests of the overall treatment effect in meta-analysis with
normally distributed responses.
\emph{Statistics in Medicine},
\bold{20}, 1771--82

Hartung J, Knapp G (2001b):
A refined method for the meta-analysis of controlled clinical
trials with binary outcome.
\emph{Statistics in Medicine},
\bold{20}, 3875--89

Hedges LV & Olkin I (1985):
\emph{Statistical methods for meta-analysis}.
San Diego, CA: Academic Press

Henmi M & Copas JB (2010):
Confidence intervals for random effects meta-analysis and
robustness to publication bias.
\emph{Statistics in Medicine},
\bold{29}, 2969--83

Higgins JPT & Thompson SG (2002):
Quantifying heterogeneity in a meta-analysis.
\emph{Statistics in Medicine},
\bold{21}, 1539--58

Higgins JPT, Thompson SG, Spiegelhalter DJ (2009):
A re-evaluation of random-effects meta-analysis.
\emph{Journal of the Royal Statistical Society: Series A},
\bold{172}, 137--59

Hunter JE & Schmidt FL (2015):
\emph{Methods of Meta-Analysis: Correcting Error and Bias in
Research Findings} (Third edition).
Thousand Oaks, CA: Sage

IntHout J, Ioannidis JPA, Borm GF (2014):
The Hartung-Knapp-Sidik-Jonkman method for random effects
meta-analysis is straightforward and considerably outperforms the
standard DerSimonian-Laird method.
\emph{BMC Medical Research Methodology},
\bold{14}, 25

IQWiG (2022):
General Methods: Version 6.1.
\url{https://www.iqwig.de/en/about-us/methods/methods-paper/}

Jackson D (2013):
Confidence intervals for the between-study variance in random
effects meta-analysis using generalised Cochran heterogeneity
statistics.
\emph{Research Synthesis Methods},
\bold{4}, 220--229

Jackson D, Law M, Rücker G, Schwarzer G (2017): 
The Hartung-Knapp modification for random-effects meta-analysis: A
useful refinement but are there any residual concerns?
\emph{Statistics in Medicine},
\bold{36}, 3923--34

Knapp G & Hartung J (2003):
Improved tests for a random effects meta-regression with a single
covariate.
\emph{Statistics in Medicine},
\bold{22}, 2693--710

Langan D, Higgins JPT, Jackson D, Bowden J, Veroniki AA,
Kontopantelis E, et al. (2019):
A comparison of heterogeneity variance estimators in simulated
random-effects meta-analyses.
\emph{Research Synthesis Methods},
\bold{10}, 83--98

Schwarzer G (2007):
meta: An R package for meta-analysis.
\emph{R News},
\bold{7}, 40--5

Schwarzer G, Carpenter JR and Rücker G (2015):
\emph{Meta-Analysis with R (Use-R!)}.
Springer International Publishing, Switzerland

Siemens W, Borenstein M, Evrenoglou T, Meerpohl JJ, Schwarzer G (2025):
Beyond prediction intervals in meta-analysis: reporting the expected
proportion of comparable studies with clinically relevant benefit or harm.
\emph{BMC Medical Research Methodology},
\bold{25}, 275

Skipka G (2006):
The inclusion of the estimated inter-study variation into forest
plots for random effects meta-analysis - a suggestion for a
graphical representation [abstract].
\emph{XIV Cochrane Colloquium, Dublin}, 23-26.

Stijnen T, Hamza TH, Ozdemir P (2010):
Random effects meta-analysis of event outcome in the framework of
the generalized linear mixed model with applications in sparse
data.
\emph{Statistics in Medicine},
\bold{29}, 3046--67

Veroniki AA, Jackson D, Viechtbauer W, Bender R, Bowden J, Knapp G,
et al. (2016):
Methods to estimate the between-study variance and its uncertainty
in meta-analysis.
\emph{Research Synthesis Methods},
\bold{7}, 55--79 

Veroniki AA, Jackson D, Bender R, Kuss O, Higgins JPT, Knapp G, Salanti G
(2019):
Methods to calculate uncertainty in the estimated overall effect size from a
random-effects meta-analysis.
\emph{Research Synthesis Methods},
\bold{10}, 23--43

Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
Three-level meta-analysis of dependent effect sizes.
\emph{Behavior Research Methods},
\bold{45}, 576--94

Viechtbauer W (2005):
Bias and efficiency of meta-analytic variance estimators in the
random-effects model.
\emph{Journal of Educational and Behavioral Statistics},
\bold{30}, 261--93

Viechtbauer W (2007):
Confidence intervals for the amount of heterogeneity in
meta-analysis.
\emph{Statistics in Medicine},
\bold{26}, 37--52

Viechtbauer W (2010):
Conducting Meta-Analyses in R with the metafor Package.
\emph{Journal of Statistical Software},
\bold{36}, 1--48

Wiksten A, Rücker G, Schwarzer G (2016):
Hartung-Knapp method is not always conservative compared with
fixed-effect meta-analysis.
\emph{Statistics in Medicine},
\bold{35}, 2503--15
}
\seealso{
\code{\link{meta-object}}, \code{\link{meta-sm}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/settings.meta.R
\name{settings.meta}
\alias{settings.meta}
\title{Print and change default meta-analysis settings in R package \bold{meta}}
\usage{
settings.meta(..., quietly = TRUE)
}
\arguments{
\item{...}{Arguments to change default settings.}

\item{quietly}{A logical indicating whether information on settings
should be printed.}
}
\description{
Print and change default settings to conduct and print or plot
meta-analyses in R package \bold{meta}. The following general
settings are available: \emph{Review Manager 5}, \emph{Journal of
the American Medical Association}.
}
\details{
This function can be used to define defaults for several arguments
(i.e., assignments using \code{\link{gs}}) of the following R
functions: \code{\link{metabin}}, \code{\link{metacont}},
\code{\link{metacor}}, \code{\link{metacr}}, \code{\link{metagen}},
\code{\link{metainc}}, \code{\link{metaprop}},
\code{\link{metarate}}

Furthermore, some of these settings are considered to print
meta-analysis results and to produce forest plots.

The function can be used to either change individual settings (see
Examples) or use one of the following general settings:
\itemize{
\item \code{settings.meta("RevMan5")}
\item \code{settings.meta("BMJ")}
\item \code{settings.meta("JAMA")}
\item \code{settings.meta("IQWiG5")}
\item \code{settings.meta("IQWiG6")}
\item \code{settings.meta("geneexpr")}
\item \code{settings.meta("IVhet")}
\item \code{settings.meta("meta4")}
\item \code{settings.meta("meta7")}
}

The first command can be used to reproduce meta-analyses from
Cochrane reviews conducted with \emph{Review Manager 5} (RevMan 5)
and specifies to use a RevMan 5 layout in forest plots.

The second command can be used to generate forest plots in BMJ layout.

The third command can be used to generate forest plots following
instructions for authors of the \emph{Journal of the American
Medical Association}. Study labels according to JAMA guidelines can be
generated using \code{\link{labels.meta}}.

The next commands implement the recommendations of the Institute
for Quality and Efficiency in Health Care, Germany (IQWiG)
accordinging to General Methods 5 and 6, respectively
(\url{https://www.iqwig.de/en/about-us/methods/methods-paper/}).

The setting \code{"geneexpr"} can be used to print p-values in
scientific notation and to suppress the calculation of confidence
intervals for the between-study variance.

The setting \code{"IVhet"} can be used for the inverse variance
heterogeneity model (Henmi & Copas, 2010; Doi et al., 2015).

The last settings use the default settings of R package
\bold{meta}, version 4 and 7.0-0, respectively, or below.

RevMan 5 settings, in detail:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{method.random.ci} \tab "classic" \tab only available method
  in RevMan 5 \cr
\code{method.tau} \tab "DL" \tab only available method in RevMan 5
  \cr
\code{method.I2} \tab "Q" \tab only available method in RevMan 5 \cr
\code{tau.common} \tab FALSE \tab common between-study variance in
  subgroups \cr
\code{MH.exact} \tab FALSE \tab exact Mantel-Haenszel method \cr
\code{RR.Cochrane} \tab TRUE \tab calculation of risk ratios \cr
\code{Q.Cochrane} \tab TRUE \tab calculation of heterogeneity
  statistic \cr
\code{exact.smd} \tab FALSE \tab exact formulae for Hedges' g and
  Cohen's d \cr
\code{layout} \tab "RevMan5" \tab layout for forest plots \cr
\code{prediction} \tab FALSE \tab no prediction interval \cr
\code{test.overall} \tab TRUE \tab print information on test of
  overall effect \cr
\code{test.subgroup} \tab TRUE \tab print information on test for
  subgroup differences \cr
\code{test.effect.subgroup} \tab TRUE \tab print information on
  test for effect in subgroups \cr
\code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
  forest plots \cr
\code{forest.tau2} \tab TRUE \tab show between-study heterogeneity \cr
  \tab \tab variance in forest plots \cr
\code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity \cr
  \tab \tab standard deviation in forest plots \cr
\code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
  forest plots
  \cr
\code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
  in forest plots
  \cr
\code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
  forest plots
  \cr
\code{digits.tau2} \tab 3 \tab number of digits for tau-squared \cr
\code{digits.tau} \tab 4 \tab number of digits for square root of
  tau-squared \cr
\code{digits.I2} \tab 0 \tab number of digits for I-squared measure
  \cr
\code{CIbracket}, \tab "[" \tab \cr
\code{CIseparator} \tab ", " \tab print confidence intervals as
  "\code{[., .]}" \cr
\code{header.line}, \tab TRUE \tab print header line
}

BMJ settings:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{layout} \tab "BMJ" \tab layout for forest plots \cr
\code{test.overall} \tab TRUE \tab print information on test of
  overall effect \cr
\code{test.subgroup} \tab FALSE \tab print information on test for
  subgroup differences \cr
\code{test.effect.subgroup} \tab FALSE \tab print information on
  test for effect in subgroups \cr
\code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
  forest plots
  \cr
\code{forest.tau2} \tab TRUE \tab show between-study heterogeneity \cr
  \tab \tab variance in forest plots \cr
\code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity \cr
  \tab \tab standard deviation in forest plots \cr
\code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
  forest plots
  \cr
\code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
  in forest plots
  \cr
\code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
  forest plots
  \cr
\code{digits.I2} \tab 0 \tab number of digits for I-squared measure
  \cr
\code{digits.pval} \tab 2 \tab number of digits for p-values \cr
\code{CIbracket}, \tab "(" \tab \cr
\code{CIseparator} \tab " to " \tab print confidence intervals as
  "\code{(. to .)}" \cr
\code{hetlab}, \tab \tab "Test for heterogeneity: " \cr
\code{header.line}, \tab TRUE \tab print header line
}

JAMA settings:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{layout} \tab "JAMA" \tab layout for forest plots \cr
\code{test.overall} \tab TRUE \tab print information on test of
  overall effect \cr
\code{test.subgroup} \tab FALSE \tab print information on test for
  subgroup differences \cr
\code{test.effect.subgroup} \tab FALSE \tab print information on
  test for effect in subgroups \cr
\code{forest.I2} \tab TRUE \tab show heterogeneity statistic I2 in
  forest plots
  \cr
\code{forest.tau2} \tab FALSE \tab do not show between-study heterogeneity \cr
  \tab \tab variance in forest plots \cr
\code{forest.tau} \tab FALSE \tab do not show between-study heterogeneity \cr
  \tab \tab standard deviation in forest plots \cr
\code{forest.Q} \tab TRUE \tab show heterogeneity statistic Q in
  forest plots
  \cr
\code{forest.pval.Q} \tab TRUE \tab show p-value of test for heterogeneity
  in forest plots
  \cr
\code{forest.Rb} \tab FALSE \tab do not show heterogeneity statistic Rb in
  forest plots
  \cr
\code{digits.I2} \tab 0 \tab number of digits for I-squared measure
  \cr
\code{digits.pval} \tab 3 \tab number of digits for p-values \cr
\code{CIbracket}, \tab "(" \tab \cr
\code{CIseparator} \tab "-" \tab print confidence intervals as
  "\code{(.-.)}" \cr
\code{zero.pval}, \tab FALSE \tab print p-values with leading zero
\cr
\code{JAMA.pval}, \tab TRUE \tab round p-values to three digits
  (for 0.001 < p \eqn{\le} 0.01) \cr
  \tab \tab or two digits (p > 0.01) \cr
\code{header.line}, \tab TRUE \tab print header line
}

IQWiG, General Methods 5 settings:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{method.random.ci} \tab "HK" \tab Hartung-Knapp method \cr
\code{prediction} \tab TRUE \tab Prediction interval \cr
}

IQWiG, General Methods 6 settings:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{method.random.ci} \tab "HK" \tab Hartung-Knapp method \cr
\code{adhoc.hakn.ci} \tab "IQWiG6" \tab \emph{ad hoc} variance correction \cr
\code{method.tau} \tab "PM" \tab Paule-Mandel estimator for
  between-study variance \cr
\code{prediction} \tab TRUE \tab Prediction interval \cr
}

Settings for gene expression data:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{scientific.pval} \tab TRUE \tab Scientific notation for p-values \cr
\code{method.tau.ci} \tab FALSE \tab
  no confidence interval for between-study \cr
 \tab \tab heterogeneity variance \cr
}

IVhet settings:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{method.common.ci} \tab "IVhet" \tab inverse variance heterogeneity \cr
\code{text.common} \tab "IVhet model" \tab  \cr
\code{text.w.common} \tab "IVhet" \tab  \cr
}

Settings for \bold{meta}, version 4 or below:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{method.tau} \tab "DL" \tab DerSimonian-Laird estimator \cr
\code{method.I2} \tab "Q" \tab Use Q to calculate I-squared \cr
\code{method.predict} \tab "HTS" \tab prediction interval with \emph{k-2}
  degrees \cr
  \tab \tab of freedom \cr
\code{exact.smd} \tab FALSE \tab Use exact formula for standardised mean \cr
  \tab \tab difference (White and Thomas, 2005) \cr
\code{text.common} \tab "Fixed effect model" \tab \cr
\code{text.w.common} \tab "fixed" \tab \cr
\code{warn.deprecated} \tab FALSE \tab Do not print warnings for deprecated
  \cr
  \tab \tab arguments
}

Settings for \bold{meta}, version 7.0-0 or below:
\tabular{lll}{
\bold{Argument} \tab \bold{Value} \tab \bold{Comment} \cr
\code{method.tau} \tab "REML" \tab REML estimator \cr
\code{method.I2} \tab "Q" \tab Use Q to calculate I-squared \cr
\code{method.predict} \tab "HTS" \tab prediction interval with \emph{k-2}
  degrees \cr
  \tab \tab of freedom \cr
\code{exact.smd} \tab TRUE \tab Use exact formula for standardised mean \cr
  \tab \tab difference (White and Thomas, 2005) \cr
\code{text.common} \tab "Common effect model" \tab \cr
\code{text.w.common} \tab "common" \tab \cr
\code{warn.deprecated} \tab FALSE \tab Do not print warnings for deprecated
  \cr
  \tab \tab arguments
}

A list of all arguments with current settings is printed using the
command \code{settings.meta()}.

In order to reset all settings of R package \bold{meta} the command
\code{settings.meta(reset = TRUE)} can be used.
}
\examples{
# Get listing of current settings
#
settings.meta()

# Meta-analyses using default settings
#
metabin(10, 20, 15, 20)
metaprop(4, 20)
metabin(10, 20, 15, 20, sm = "RD")
metaprop(4, 20, sm = "PLN")

# Change summary measure for R functions metabin and metaprop
# and store old settings
#
oldset <- settings.meta(smbin = "RD", smprop = "PLN")
#
metabin(10, 20, 15, 20)
metaprop(4, 20)

# Use old settings
#
settings.meta(oldset)

# Change level used to calculate confidence intervals
# (99\%-CI for studies, 99.9\%-CI for pooled effects)
#
metagen(1:3, 2:4 / 10, sm = "MD")
settings.meta(level = 0.99, level.ma = 0.999)
metagen(1:3, 2:4 / 10, sm = "MD")

# Always print a prediction interval
#
settings.meta(prediction = TRUE)
metagen(1:3, 2:4 / 10, sm = "MD")
metagen(4:6, 4:2 / 10, sm = "MD")

# Try to set unknown argument results in a warning
#
try(settings.meta(unknownarg = TRUE))

# Reset to default settings of R package meta
#
settings.meta("reset")
metabin(10, 20, 15, 20)
metaprop(4, 20)
metagen(1:3, 2:4 / 10, sm = "MD")

# Do not back transform results (e.g. print log odds ratios instead
# of odds ratios, print transformed correlations / proportions
# instead of correlations / proportions)
#
settings.meta(backtransf = FALSE)
metabin(10, 20, 15, 20)
metaprop(4, 20)
metacor(c(0.85, 0.7, 0.95), c(20, 40, 10))

# Forest plot using RevMan 5 style
#
settings.meta("RevMan5")
forest(metagen(1:3, 2:4 / 10, sm = "MD", common = FALSE),
  label.left = "Favours A", label.right = "Favours B",
  colgap.studlab = "2cm", colgap.forest.left = "0.2cm")

# Forest plot using JAMA style
#
settings.meta("JAMA")
forest(metagen(1:3, 2:4 / 10, sm = "MD", common = FALSE),
  label.left = "Favours A", label.right = "Favours B",
  colgap.studlab = "2cm", colgap.forest.left = "0.2cm")

# Use slightly different layout for confidence intervals
# (especially useful if upper confidence limit can be negative)
#
settings.meta(CIseparator = " - ")
forest(metagen(-(1:3), 2:4 / 10, sm = "MD", common = FALSE),
  label.left = "Favours A", label.right = "Favours B",
  colgap.studlab = "2cm", colgap.forest.left = "0.2cm")

# Use old settings
#
settings.meta(oldset)

}
\references{
Doi SAR, Barendregt JJ, Khan S, Thalib L, Williams GM (2015):
Advances in the meta-analysis of heterogeneous clinical trials I:
The inverse variance heterogeneity model.
\emph{Contemporary Clinical Trials},
\bold{45}, 130--8

Henmi M & Copas JB (2010):
Confidence intervals for random effects meta-analysis and
robustness to publication bias.
\emph{Statistics in Medicine},
\bold{29}, 2969--83

White IR, Thomas J (2005):
Standardized mean differences in individually-randomized and
cluster-randomized trials, with applications to meta-analysis.
\emph{Clinical Trials},
\bold{2}, 141--51
}
\seealso{
\code{\link{gs}}, \code{\link{forest.meta}},
  \code{\link{print.meta}}, \code{\link{labels.meta}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/funnel.R
\name{funnel.meta}
\alias{funnel.meta}
\alias{funnel}
\alias{setvals}
\title{Funnel plot}
\usage{
\method{funnel}{meta}(
  x,
  type = "standard",
  xlim = NULL,
  ylim = NULL,
  xlab = NULL,
  ylab = NULL,
  common = x$common,
  random = x$random,
  axes = TRUE,
  pch = if (!inherits(x, "trimfill")) 21 else ifelse(x$trimfill, 1, 21),
  text = NULL,
  cex = 1,
  lty.common = 2,
  lty.random = 9,
  lwd = 1,
  lwd.common = lwd,
  lwd.random = lwd,
  col = "black",
  bg = "darkgray",
  col.common = "black",
  col.random = "black",
  log,
  yaxis,
  contour.levels = if (type == "contour") c(0.9, 0.95, 0.99) else NULL,
  col.contour = if (type == "contour") c("gray80", "gray70", "gray60") else NULL,
  ref = ifelse(is_relative_effect(x$sm), 1, 0),
  level = if (common | random) x$level else NULL,
  studlab = FALSE,
  cex.studlab = 0.8,
  pos.studlab = 2,
  ref.triangle = FALSE,
  lty.ref = 1,
  lwd.ref = lwd,
  col.ref = "black",
  lty.ref.triangle = 5,
  backtransf = x$backtransf,
  warn.deprecated = gs("warn.deprecated"),
  ...
)

setvals(x, vals = seq_along(unique(x)))
}
\arguments{
\item{x}{An object of class \code{meta}.}

\item{type}{A character string indicating type of funnel plot. Either
\code{"standard"} or \code{"contour"}, can be abbreviated.}

\item{xlim}{The x limits (min,max) of the plot.}

\item{ylim}{The y limits (min,max) of the plot.}

\item{xlab}{A label for the x-axis.}

\item{ylab}{A label for the y-axis.}

\item{common}{A logical indicating whether the common effect
estimate should be plotted.}

\item{random}{A logical indicating whether the random effects
estimate should be plotted.}

\item{axes}{A logical indicating whether axes should be drawn on
the plot.}

\item{pch}{The plotting symbol(s) used for individual studies.}

\item{text}{A character vector specifying the text to be used
instead of plotting symbol.}

\item{cex}{The magnification to be used for plotting symbols.}

\item{lty.common}{Line type (common effect estimate).}

\item{lty.random}{Line type (random effects estimate).}

\item{lwd}{The line width for confidence intervals (if \code{level}
is not \code{NULL}).}

\item{lwd.common}{The line width for common effect estimate (if
\code{common} is not \code{NULL}).}

\item{lwd.random}{The line width for random effects estimate (if
\code{random} is not \code{NULL}).}

\item{col}{A vector with colour of plotting symbols.}

\item{bg}{A vector with background colour of plotting symbols (only
used if \code{pch} in \code{21:25}).}

\item{col.common}{Colour of line representing common effect estimate.}

\item{col.random}{Colour of line representing random effects
estimate.}

\item{log}{A character string which contains \code{"x"} if the
x-axis is to be logarithmic, \code{"y"} if the y-axis is to be
logarithmic and \code{"xy"} or \code{"yx"} if both axes are to be
logarithmic.}

\item{yaxis}{A character string indicating which type of weights
are to be used. Either \code{"se"}, \code{"invvar"},
\code{"invse"}, \code{"size"}, \code{"invsqrtsize"}, or
\code{"ess"}.}

\item{contour.levels}{A numeric vector specifying contour levels to
produce contour-enhanced funnel plot.}

\item{col.contour}{Colour of contours.}

\item{ref}{Reference value (null effect) used to produce
contour-enhanced funnel plot.}

\item{level}{The confidence level utilised in the plot. For the
funnel plot, confidence limits are not drawn if
\code{yaxis="size"} or \code{yaxis="invsqrtsize"}.}

\item{studlab}{A logical indicating whether study labels should be
printed in the graph. A vector with study labels can also be
provided (must be of same length as \code{x$TE} then).}

\item{cex.studlab}{Size(s) of study labels, see argument \code{cex} in
\code{\link{text}}.}

\item{pos.studlab}{Position of study labels, see argument
\code{pos} in \code{\link{text}}.}

\item{ref.triangle}{A logical indicating whether approximate
confidence limits should be printed around reference value (null
effect).}

\item{lty.ref}{Line type (reference value).}

\item{lwd.ref}{The line width for the reference value and
corresponding confidence intervals (if \code{ref.triangle} is
TRUE and \code{level} is not \code{NULL}).}

\item{col.ref}{Colour of line representing reference value.}

\item{lty.ref.triangle}{Line type (confidence intervals of
reference value).}

\item{backtransf}{A logical indicating whether results for relative
summary measures (argument \code{sm} equal to \code{"OR"},
\code{"RR"}, \code{"HR"}, or \code{"IRR"}) should be back
transformed in funnel plots. If \code{backtransf=TRUE}, results
for \code{sm="OR"} are printed as odds ratios rather than log
odds ratios, for example.}

\item{warn.deprecated}{A logical indicating whether warnings should
be printed if deprecated arguments are used.}

\item{\dots}{Additional arguments (passed on to plot.default).}

\item{vals}{Vector with values used in \code{setvals} (see Examples).}
}
\description{
Draw a funnel plot which can be used to assess small study effects
in meta-analysis. A contour-enhanced funnel plot can also be
produced to assess causes of funnel plot asymmetry.
}
\details{
A funnel plot (Light & Pillemer, 1984) is drawn in the active
graphics window. If \code{common} is TRUE, the estimate of the
common effect model is plotted as a vertical line. Similarly, if
\code{random} is TRUE, the estimate of the random effects model is
plotted. If \code{level} is not NULL, the corresponding approximate
confidence limits are drawn around the common effect estimate (if
\code{common} is TRUE) or the random effects estimate (if
\code{random} is TRUE and \code{common} is FALSE).

In the funnel plot, the standard error of the treatment estimates
is plotted on the y-axis by default (\code{yaxis = "se"}) which is
likely to be the best choice (Sterne & Egger, 2001). Only exception
is meta-analysis of diagnostic test accuracy studies (Deeks et al.,
2005) where the inverse of the square root of the \emph{effective
study size} is used (\code{yaxis = "ess"}). Other possible choices
for \code{yaxis} are \code{"invvar"} (inverse of the variance),
\code{"invse"} (inverse of the standard error), \code{"size"}
(study size), and \code{"invsqrtsize"} (1 / sqrt(study size)).

If argument \code{yaxis} is not equal to \code{"size"},
\code{"invsqrtsize"} or \code{"ess"}, contour-enhanced funnel plots
can be produced (Peters et al., 2008) by specifying the contour
levels (argument \code{contour.levels}). By default (argument
\code{col.contour} missing), suitable gray levels will be used to
distinguish the contours. Different colours can be chosen by
argument \code{col.contour}.
}
\note{
R function \code{setvals} can be used to easily define the input for the
arguments \code{pch}, \code{text}, \code{cex}, \code{col}, \code{bg},
and \code{cex.studlab}.
}
\examples{
data(Olkin1995)
ma <- metabin(ev.exp, n.exp, ev.cont, n.cont,
  data = Olkin1995, subset = c(41, 47, 51, 59),
  studlab = paste(author, year),
  sm = "RR", method = "I")

# Standard funnel plot
#
funnel(ma)

# Funnel plot with confidence intervals, common effect estimate and
# contours
#
fun <- funnel(ma, common = TRUE, level = 0.95, type = "contour")
legend("topleft", fun$text.contour, fill = fun$col.contour, bg = "white")

# Contour-enhanced funnel plot with user-chosen colours
#
funnel(ma, common = TRUE,
  level = 0.95, contour = c(0.9, 0.95, 0.99),
  col.contour = c("darkgreen", "green", "lightgreen"),
  lwd = 2, cex = 2, pch = 16, studlab = TRUE, cex.studlab = 1.25)
legend(0.05, 0.05,
  c("0.1 > p > 0.05", "0.05 > p > 0.01", "< 0.01"),
  fill = c("darkgreen", "green", "lightgreen"))

fun <- funnel(ma, common = TRUE,
  level = 0.95, contour = c(0.9, 0.95, 0.99),
  col.contour = c("darkgreen", "green", "lightgreen"),
  lwd = 2, cex = 2, pch = 16, studlab = TRUE, cex.studlab = 1.25)
legend(0.05, 0.05, fun$text.contour, fill = fun$col.contour)

# Use different colours for log risk ratios below and above 0
#
funnel(ma, bg = setvals(TE < 0, c("green", "red")))

}
\references{
Deeks JJ, Macaskill P, Irwig L (2005):
The performance of tests of publication bias and other sample size
effects in systematic reviews of diagnostic test accuracy was
assessed.
\emph{Journal of Clinical Epidemiology},
\bold{58}:882--93

Light RJ & Pillemer DB (1984):
\emph{Summing Up. The Science of Reviewing Research}.
Cambridge: Harvard University Press

Peters JL, Sutton AJ, Jones DR, Abrams KR, Rushton L (2008):
Contour-enhanced meta-analysis funnel plots help distinguish
publication bias from other causes of asymmetry.
\emph{Journal of Clinical Epidemiology},
\bold{61}, 991--6

Sterne JAC & Egger M (2001):
Funnel plots for detecting bias in meta-analysis: Guidelines on
choice of axis.
\emph{Journal of Clinical Epidemiology},
\bold{54}, 1046--55
}
\seealso{
\code{\link{metabias}}, \code{\link{metabin}},
  \code{\link{metagen}}, \code{\link{radial}}
}
\author{
Guido Schwarzer \email{guido.schwarzer@uniklinik-freiburg.de}, Petra
  Graham \email{pgraham@efs.mq.edu.au}
}
\keyword{hplot}
